$wnd.rstudio.runAsyncCallback9("function SYj(a){this.a=a}\nfunction KXj(a){a.J.BI();N8e(a.j.o)}\nfunction IXj(a){OVi(a.j);C$f(a.a,'Load History','loadhistory')}\nfunction JXj(a){OVi(a.j);H$f(a.a,'Save History As','.Rhistory',false,'savehistory')}\nfunction OYj(a){try{if(a.c<0)return;if(!a.a.a){a.a.a=vZj(a.a.b.a);rQi(a.a,a.a.a)}switch(a.c){case 1:a.a.a.J.zI();break;case 2:a.a.a.J.AI();break;case 3:IXj(a.a.a);break;case 4:JXj(a.a.a);break;case 5:KXj(a.a.a);break;case 6:a.a.a.J.CI();}}finally{a.c<=0&&a.b[0]!=null&&a.b[0].gn()}}\nE6c(514,1,gio);_.en=function RYj(){OYj((new SYj(this)).a)};E6c(6458,1,ISn,SYj);_.gn=function TYj(){OYj(this.a)};var Zbc=nQd(6458);wJn(hm)(9);\n//# sourceURL=rstudio-9.js\n")
