$wnd.rstudio.runAsyncCallback7("function Gnk(a){this.a=a}\nfunction Qmk(a,b){this.a=a;this.b=b}\nfunction Mmk(a,b){b==null?Vfe(a,pdp):Vfe(a,b)}\nfunction _mk(a,b){yi(a.d,new Aqe(false));Smk(a.d,b,0)}\nfunction X1h(a,b){z3h(a,Ypo,'markers_tab_closed',new PH,b)}\nfunction Nmk(a,b){var c,d,e,f,g,h,i;d=a.q;wAd(d.o);for(g=b,h=0,i=g.length;h<i;++h){f=g[h];c=new Qmk(a,f);e=new g8c(e8c(new f8c,f).a.a);imf(d,new DBd(e,c))}}\nfunction Smk(a,b,c){var d;fBg(a.b);Mmk(a.c,null);Nmk(a.c,GJ(AJ(tY,1),gKn,2,6,[]));if(b.markers){d=b.markers;jBg(a.b,null,d.base_path,d.markers,c,false);Nmk(a.c,Yxe(b.names));Mmk(a.c,d.name)}}\nfunction $mk(a,b){var c,d,e,f,g;g=b.a.markers_state;if(g.markers){yi(a.d,new Aqe(true));Smk(a.d,b.a.markers_state,b.a.auto_select);e=g.markers;if(e){d=e.markers;if(d.length>0){f=null;c=b.a.auto_select;c==1?(f=d[0]):c==2&&(f=TAg(d));!!f&&Tgg(a.b,(yse(),Qse(f.path,false,-1,0)),S7d(f.line,f.column),false)}}}else{yi(a.d,new tqe)}}\nfunction Cnk(a){try{if(a.c<0)return;if(!a.a.a){a.a.a=Pnk(a.a.b.a);rQi(a.a,a.a.a)}switch(a.c){case 1:Zmk(a.a.a);break;case 2:a.a.a.J.zI();break;case 3:a.a.a.J.AI();break;case 4:X1h(a.a.a.c,new QBf);break;case 5:$mk(a.a.a,a.b[0]);break;case 6:a.a.a.J.BI();break;case 7:a.a.a.J.CI();break;case 8:_mk(a.a.a,a.b[0]);}}finally{a.c<=0&&a.b[0]!=null&&a.b[0].gn()}}\nE6c(7733,1,XLn,Qmk);_.gn=function Rmk(){vE(this.a,this.b)};var qic=nQd(7733);E6c(434,1,gio);_.en=function Fnk(){Cnk((new Gnk(this)).a)};E6c(6339,1,ISn,Gnk);_.gn=function Hnk(){Cnk(this.a)};var Dic=nQd(6339);wJn(hm)(7);\n//# sourceURL=rstudio-7.js\n")
