<% if (title || logo) { %>
  <a class="navbar-brand" href="/index.html">
  <% if (logo) { %>
    <img src="<%- logo %>" alt="" />
  <% } %>
  <% if (title) { %>
    <span class="navbar-title"><%- title %></span>
  <% } %>
  </a>
<% } %>
