$wnd.rstudio.runAsyncCallback17("function xLk(a){this.a=a}\nfunction rUk(a){this.a=a}\nfunction tUk(a,b){this.a=a;this.b=b}\nfunction LVl(a,b){this.a=a;this.b=b}\nfunction fWl(a,b){this.a=a;this.b=b}\nfunction lVm(a,b){kVm();this.b=a;this.a=b}\nfunction $Gm(a,b){kZg(a.u,b)}\nfunction SHl(a,b){$Gm(a.ub,b)}\nfunction fKk(a){W$f(a.n,a.c,null)}\nfunction gKk(a){W$f(a.n,a.g,null)}\nfunction tKk(a){KKk(a.o,a.g,u1o,null)}\nfunction yKk(a){KKk(a.o,a.g,'next',null)}\nfunction AKk(a){KKk(a.o,a.g,'prev',null)}\nfunction BKk(a){KKk(a.o,a.g,'refresh',null)}\nfunction hKk(a){W$f(a.n,a.g+'?print-pdf',null)}\nfunction frg(a){return {items:a}}\nfunction NPk(a,b){var c;c=a;ZHl(c,b.a)}\nfunction MPk(a,b){gn((_m(),$m),new tUk(b,a))}\nfunction LHl(a,b){gn((_m(),$m),new fWl(a,b))}\nfunction ZHl(a,b){bZm(a.w,Hno,false)?hGl(a,new LVl(a,b)):t$l(a._,b)}\nfunction _Jk(a,b){ZRd(b.substr(0,4),YNn)||(b=v_h(a.p,b));return b}\nfunction kZg(a,b){a.d.navigateToPresentationEditorLocation(b)}\nfunction QGf(a){$wnd.desktop.setPresentationUrl(a)}\nfunction eKk(a){!!a.b&&$If(a.k,new lVm((yse(),Qse(a.b.source_file,false,-1,0)),_qg(a.d,a.a,a.e)),false)}\nfunction arg(a,b){var c,d,e,f;b==-1&&(b=a.auto_slide_level);f=$qg(a,b,true);for(c=0;c<f.length;c++){e=f[c];for(d=0;d<e.items.length;d++){if(ZRd(e.items[d].type,bdo))return c}}return 0}\nfunction _qg(a,b,c){var d,e,f,g,h;c==-1&&(c=b.auto_slide_level);h=$qg(b,c,false);if(a<h.length){g=h[a];zl(g.items,crg(g.items[0].row))}e=[];for(d=0;d<h.length;d++){g=h[d];for(f=0;f<g.items.length;f++){xl(e,g.items[f])}}return brg(e,b.auto_slide_level)}\nfunction t$l(a,b){var c,d,e,f,g,h,i;e=s$l(a).items;d=[];for(g=0;g<e.length;g++){h=e[g];ZRd(h.type,bdo)||(d[d.length]=h,undefined)}c=-1;i=b.items;for(f=0;f<i.length;f++){if(ZRd(i[f].type,bdo)){c=f;break}}if(c>=0){h=d[c];!!h&&hnl(a.a,h_m(h.row,0),true)}}\nfunction $qg(a,b,c){var d,e,f,g,h,i;i=[];g=a.items;for(d=0;d<g.length;d++){f=g[d];e=i.length==0||ZRd(f.type,uLn)||ZRd(f.type,'hr')||ZRd(f.type,TKn)&&f.level<=b;if(e){h=frg([]);xl(h.items,f);i[i.length]=h}else (c||!ZRd(f.type,bdo))&&xl(i[i.length-1].items,f)}return i}\nfunction iKk(a,b){var c,d;c=b.a;d=_Jk(a,c.url);a.o=arg(b.a.editor_state,b.a.slide_level);a.o==0&&a.d!=0&&(a.o=a.d);OVi(a.j);if(!ZRd(d,a.g)||!a.j.o){a.g=d;a.c=d;a.b=c.quarto_navigation;a.a=c.editor_state;a.e=c.slide_level;(RFf()||!!$wnd.remoteDesktop)&&QGf(a.g);xKk(a.j,a.g,a.b)}else{BKk(a.j)}}\nfunction xKk(a,b,c){var d;(RFf()||!!$wnd.remoteDesktop)&&(Sce(b)?(Oh(a.d).removeAttribute(Sco),undefined):(Oh(a.d).setAttribute(Sco,Cnp),undefined));a.b=b;a.o=null;a.g=null;d=Qh(a.d);d>0&&yi(a,new nqe(AK(d*0.6666666666666666)+((a.K?Ph(a.K):0)+(a.O?Ph(a.O):0))));lxd(a.d,a.b);c.is_website?oXh(a.i):nXh(a.i,c.source_file,c.output_file);Nnf(a.p)}\nfunction tLk(a){try{if(a.c<0)return;if(!a.a.a){a.a.a=YLk(a.a.b.a);rQi(a.a,a.a.a)}switch(a.c){case 1:a.a.a.J.zI();break;case 2:a.a.a.J.AI();break;case 3:eKk(a.a.a);break;case 4:tKk(a.a.a.j);break;case 5:yKk(a.a.a.j);break;case 6:fKk(a.a.a);break;case 7:gKk(a.a.a);break;case 8:AKk(a.a.a.j);break;case 9:hKk(a.a.a);break;case 10:iKk(a.a.a,a.b[0]);break;case 11:BKk(a.a.a.j);break;case 12:a.a.a.J.BI();break;case 13:a.a.a.J.CI();}}finally{a.c<=0&&a.b[0]!=null&&a.b[0].gn()}}\nE6c(302,1,gio);_.en=function wLk(){tLk((new xLk(this)).a)};E6c(6542,1,ISn,xLk);_.gn=function yLk(){tLk(this.a)};var foc=nQd(6542);E6c(3154,1,DSn,rUk);_.du=function sUk(a){MPk(this.a,a)};var Grc=nQd(3154);E6c(3153,1,XLn,tUk);_.gn=function uUk(){NPk(this.a,this.b)};var Hrc=nQd(3153);E6c(4405,1,ISn,LVl);_.gn=function MVl(){LHl(this.a,this.b)};var mGc=nQd(4405);E6c(4404,1,XLn,fWl);_.gn=function gWl(){SHl(this.a,this.b)};var xGc=nQd(4404);E6c(3197,ENn,{},lVm);_.Vn=function mVm(a){var b,c;b=(c=Zgg(a.k,this.b),!!c&&tK(c,80)?c:(Sgg(),Ngg));kYk(a.b,this.b,b,new rUk(this))};_.Xn=function oVm(){return jVm};_.Wn=function nVm(){return jVm};var gVc=nQd(3197);wJn(hm)(17);\n//# sourceURL=rstudio-17.js\n")
