$wnd.rstudio.runAsyncCallback23('function Pok(a){this.a=a}\nfunction gok(a,b){this.a=a;this.b=b}\nfunction Unk(a,b){a.n?f9d(4,\'Stop R Markdown Rendering\',"The rendering of \'"+a.i+"\' is in progress. Do you want to terminate and close the tab?",false,new gok(a,b),null,null,Gfo,bTn,true):b.gn()}\nfunction Lok(a){try{if(a.c<0)return;if(!a.a.a){a.a.a=Rok(a.a.b.a);rQi(a.a,a.a.a)}switch(a.c){case 1:D$i(a.a.a,a.b[0]);break;case 2:Unk(a.a.a,a.b[0]);break;case 3:Vnk(a.a.a);break;case 4:a.a.a.J.zI();break;case 5:a.a.a.J.AI();break;case 6:Wnk(a.a.a,a.b[0]);break;case 7:Xnk(a.a.a,a.b[0]);break;case 8:Ynk(a.a.a,a.b[0]);break;case 9:Znk(a.a.a,a.b[0]);break;case 10:a.a.a.J.BI();break;case 11:a.a.a.J.CI();}}finally{a.c<=0&&a.b[0]!=null&&a.b[0].gn()}}\nE6c(7113,1,hTn,gok);_.gn=function hok(){$nk(this.a);this.b.gn()};var Mic=nQd(7113);E6c(318,1,gio);_.en=function Ook(){Lok((new Pok(this)).a)};E6c(6577,1,ISn,Pok);_.gn=function Qok(){Lok(this.a)};var Vic=nQd(6577);wJn(hm)(23);\n//# sourceURL=rstudio-23.js\n')
