var __defProp = Object.defineProperty;
var __markAsModule = (target) => __defProp(target, "__esModule", { value: true });
var __export = (target, all) => {
  __markAsModule(target);
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};

// standalone-schema-validators.js
__export(exports, {
  default: () => standalone_schema_validators_default
});
var h0 = (l, o) => () => (o || l((o = { exports: {} }).exports, o), o.exports);
var pq = h0((B) => {
  "use strict";
  B["math-methods"] = gc;
  var Nd = { enum: ["plain", "webtex", "gladtex", "mathml", "mathjax", "katex"], description: "be one of: `plain`, `webtex`, `gladtex`, `mathml`, `mathjax`, `katex`", $id: "math-methods" };
  function gc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Nd.enum;
    if (!(l === "plain" || l === "webtex" || l === "gladtex" || l === "mathml" || l === "mathjax" || l === "katex")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Nd.enum, schema: Nd }, message: "must be equal to one of the allowed values", schema: Nd.enum, parentSchema: Nd, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return gc.errors = e, r === 0;
  }
  B["pandoc-format-request-headers"] = Cd;
  var ac = { type: "array", description: "be an array of values, where each element should be an array of values, where each element should be a string", items: { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }, $id: "pandoc-format-request-headers" };
  function Cd(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let u = true, y = l.length;
      for (let g = 0; g < y; g++) {
        let x = l[g], D = r, $ = r;
        if (Array.isArray(x)) {
          let m = true, R = x.length;
          for (let j = 0; j < R; j++) {
            let V = x[j], se = r;
            if (typeof V != "string") {
              let oe = { instancePath: o + "/" + g + "/" + j, schemaPath: "#/items/items/type", keyword: "type", params: { type: "string", schema: ac.items.items }, message: "must be string", schema: ac.items.items.type, parentSchema: ac.items.items, data: V };
              e === null ? e = [oe] : e.push(oe), r++;
            }
            let P = r;
            var c = se === r;
            c || (m = false);
          }
        } else {
          let m = { instancePath: o + "/" + g, schemaPath: "#/items/type", keyword: "type", params: { type: "array", schema: ac.items }, message: "must be array", schema: ac.items.type, parentSchema: ac.items, data: x };
          e === null ? e = [m] : e.push(m), r++;
        }
        var d = D === r;
        d || (u = false);
      }
    } else {
      let u = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: ac }, message: "must be array", schema: ac.type, parentSchema: ac, data: l };
      e === null ? e = [u] : e.push(u), r++;
    }
    return Cd.errors = e, r === 0;
  }
  B["pandoc-format-output-file"] = Id;
  var ll = { oneOf: [{ type: "string", description: "be a string" }, { enum: [null], description: "be 'null'" }], description: "be exactly one of: a string, 'null'", $id: "pandoc-format-output-file" };
  function Id(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: ll.oneOf[0] }, message: "must be string", schema: ll.oneOf[0].type, parentSchema: ll.oneOf[0], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = ll.oneOf[1].enum;
    if (l !== null) {
      let $ = { instancePath: o, schemaPath: "#/oneOf/1/enum", keyword: "enum", params: { allowedValues: ll.oneOf[1].enum, schema: ll.oneOf[1] }, message: "must be equal to one of the allowed values", schema: ll.oneOf[1].enum, parentSchema: ll.oneOf[1], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: ll }, message: "must match exactly one schema in oneOf", schema: ll.oneOf, parentSchema: ll, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return Id.errors = e, r === 0;
  }
  B["pandoc-format-filters"] = Hd;
  var nn = { type: "array", description: "be an array of values, where each element should be exactly one of: a string, an object, an object", items: { oneOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: { type: { type: "string", description: "be a string" }, path: { type: "string", description: "be a string" } }, patternProperties: {}, required: ["type", "path"], additionalProperties: false }, { type: "object", description: "be an object", properties: { type: { enum: ["citeproc"], description: "be 'citeproc'" } }, patternProperties: {}, required: ["type"], additionalProperties: false }], description: "be exactly one of: a string, an object, an object" }, $id: "pandoc-format-filters" };
  function Hd(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let g = true, x = l.length;
      for (let D = 0; D < x; D++) {
        let $ = l[D], m = r, R = r, j = false, V = null, se = r;
        if (typeof $ != "string") {
          let f = { instancePath: o + "/" + D, schemaPath: "#/items/oneOf/0/type", keyword: "type", params: { type: "string", schema: nn.items.oneOf[0] }, message: "must be string", schema: nn.items.oneOf[0].type, parentSchema: nn.items.oneOf[0], data: $ };
          e === null ? e = [f] : e.push(f), r++;
        }
        let P = r;
        var c = se === r;
        c && (j = true, V = 0);
        let oe = r, h = r;
        if ($ && typeof $ == "object" && !Array.isArray($)) {
          if ($.type === void 0) {
            let ft = { instancePath: o + "/" + D, schemaPath: "#/items/oneOf/1/required", keyword: "required", params: { missingProperty: "type", schema: nn.items.oneOf[1] }, message: "must have required property 'type'", schema: nn.items.oneOf[1].required, parentSchema: nn.items.oneOf[1], data: $ };
            e === null ? e = [ft] : e.push(ft), r++;
          }
          if ($.path === void 0) {
            let ft = { instancePath: o + "/" + D, schemaPath: "#/items/oneOf/1/required", keyword: "required", params: { missingProperty: "path", schema: nn.items.oneOf[1] }, message: "must have required property 'path'", schema: nn.items.oneOf[1].required, parentSchema: nn.items.oneOf[1], data: $ };
            e === null ? e = [ft] : e.push(ft), r++;
          }
          let f = r;
          for (let ft in $)
            if (!(ft === "type" || ft === "path")) {
              let Cr = { instancePath: o + "/" + D, schemaPath: "#/items/oneOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: ft, schema: nn.items.oneOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: nn.items.oneOf[1], data: $ };
              e === null ? e = [Cr] : e.push(Cr), r++;
            }
          if ($.type !== void 0) {
            let ft = $.type, Cr = r;
            if (typeof ft != "string") {
              let wr = { instancePath: o + "/" + D + "/type", schemaPath: "#/items/oneOf/1/properties/type/type", keyword: "type", params: { type: "string", schema: nn.items.oneOf[1].properties.type }, message: "must be string", schema: nn.items.oneOf[1].properties.type.type, parentSchema: nn.items.oneOf[1].properties.type, data: ft };
              e === null ? e = [wr] : e.push(wr), r++;
            }
            let Dr = r;
            var d = Cr === r;
          }
          if ($.path !== void 0) {
            let ft = $.path, Cr = r;
            if (typeof ft != "string") {
              let ta = { instancePath: o + "/" + D + "/path", schemaPath: "#/items/oneOf/1/properties/path/type", keyword: "type", params: { type: "string", schema: nn.items.oneOf[1].properties.path }, message: "must be string", schema: nn.items.oneOf[1].properties.path.type, parentSchema: nn.items.oneOf[1].properties.path, data: ft };
              e === null ? e = [ta] : e.push(ta), r++;
            }
            let Dr = r;
            var d = Cr === r;
          }
        } else {
          let f = { instancePath: o + "/" + D, schemaPath: "#/items/oneOf/1/type", keyword: "type", params: { type: "object", schema: nn.items.oneOf[1] }, message: "must be object", schema: nn.items.oneOf[1].type, parentSchema: nn.items.oneOf[1], data: $ };
          e === null ? e = [f] : e.push(f), r++;
        }
        var c = oe === r;
        if (c && j)
          j = false, V = [V, 1];
        else {
          c && (j = true, V = 1);
          let f = r, ft = r;
          if ($ && typeof $ == "object" && !Array.isArray($)) {
            if ($.type === void 0) {
              let wr = { instancePath: o + "/" + D, schemaPath: "#/items/oneOf/2/required", keyword: "required", params: { missingProperty: "type", schema: nn.items.oneOf[2] }, message: "must have required property 'type'", schema: nn.items.oneOf[2].required, parentSchema: nn.items.oneOf[2], data: $ };
              e === null ? e = [wr] : e.push(wr), r++;
            }
            let Dr = r;
            for (let wr in $)
              if (wr !== "type") {
                let ta = { instancePath: o + "/" + D, schemaPath: "#/items/oneOf/2/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: wr, schema: nn.items.oneOf[2] }, message: "must NOT have additional properties", schema: false, parentSchema: nn.items.oneOf[2], data: $ };
                e === null ? e = [ta] : e.push(ta), r++;
              }
            if ($.type !== void 0) {
              let wr = $.type, ta = r, jt = nn.items.oneOf[2].properties.type.enum;
              if (wr !== "citeproc") {
                let ee = { instancePath: o + "/" + D + "/type", schemaPath: "#/items/oneOf/2/properties/type/enum", keyword: "enum", params: { allowedValues: nn.items.oneOf[2].properties.type.enum, schema: nn.items.oneOf[2].properties.type }, message: "must be equal to one of the allowed values", schema: nn.items.oneOf[2].properties.type.enum, parentSchema: nn.items.oneOf[2].properties.type, data: wr };
                e === null ? e = [ee] : e.push(ee), r++;
              }
              var u = ta === r;
            }
          } else {
            let Dr = { instancePath: o + "/" + D, schemaPath: "#/items/oneOf/2/type", keyword: "type", params: { type: "object", schema: nn.items.oneOf[2] }, message: "must be object", schema: nn.items.oneOf[2].type, parentSchema: nn.items.oneOf[2], data: $ };
            e === null ? e = [Dr] : e.push(Dr), r++;
          }
          var c = f === r;
          c && j ? (j = false, V = [V, 2]) : c && (j = true, V = 2);
        }
        if (j)
          r = R, e !== null && (R ? e.length = R : e = null);
        else {
          let f = { instancePath: o + "/" + D, schemaPath: "#/items/oneOf", keyword: "oneOf", params: { passingSchemas: V, schema: nn.items }, message: "must match exactly one schema in oneOf", schema: nn.items.oneOf, parentSchema: nn.items, data: $ };
          e === null ? e = [f] : e.push(f), r++;
        }
        var y = m === r;
        y || (g = false);
      }
    } else {
      let g = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: nn }, message: "must be array", schema: nn.type, parentSchema: nn, data: l };
      e === null ? e = [g] : e.push(g), r++;
    }
    return Hd.errors = e, r === 0;
  }
  B["page-column"] = cl;
  var Ld = { enum: ["body", "body-outset", "body-outset-left", "body-outset-right", "page", "page-left", "page-right", "page-inset-left", "page-inset-right", "screen", "screen-left", "screen-right", "screen-rightcolumn", "screen-inset", "screen-inset-shaded", "screen-inset-left", "screen-inset-right", "margin"], description: "be one of: `body`, `body-outset`, `body-outset-left`, `body-outset-right`, `page`, `page-left`, `page-right`, `page-inset-left`, `page-inset-right`, `screen`, `screen-left`, `screen-right`, `screen-rightcolumn`, `screen-inset`, `screen-inset-shaded`, `screen-inset-left`, `screen-inset-right`, `margin`", $id: "page-column" };
  function cl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Ld.enum;
    if (!(l === "body" || l === "body-outset" || l === "body-outset-left" || l === "body-outset-right" || l === "page" || l === "page-left" || l === "page-right" || l === "page-inset-left" || l === "page-inset-right" || l === "screen" || l === "screen-left" || l === "screen-right" || l === "screen-rightcolumn" || l === "screen-inset" || l === "screen-inset-shaded" || l === "screen-inset-left" || l === "screen-inset-right" || l === "margin")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Ld.enum, schema: Ld }, message: "must be equal to one of the allowed values", schema: Ld.enum, parentSchema: Ld, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return cl.errors = e, r === 0;
  }
  B["navigation-item"] = wa;
  var on = { anyOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: { href: { type: "string", description: "be a string" }, url: { type: "string", description: "be a string" }, file: { type: "string", description: "be a string" }, text: { type: "string", description: "be a string" }, icon: { type: "string", description: "be a string" }, "aria-label": { type: "string", description: "be a string" }, menu: { type: "array", description: "be an array of values, where each element should be navigation-item", items: { $ref: "navigation-item", description: "be navigation-item" } } }, patternProperties: {}, additionalProperties: false, propertyNames: { type: "string", pattern: "^(?!(aria_label|ariaLabel))" } }], description: "be at least one of: a string, an object", $id: "navigation-item" }, y0 = new RegExp("^(?!(aria_label|ariaLabel))", "u"), eg = { validate: wa };
  function wa(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = r;
    if (typeof l != "string") {
      let $ = { instancePath: o, schemaPath: "#/anyOf/0/type", keyword: "type", params: { type: "string", schema: on.anyOf[0] }, message: "must be string", schema: on.anyOf[0].type, parentSchema: on.anyOf[0], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let u = r;
    var y = d === r;
    if (c = c || y, !c) {
      let $ = r, m = r;
      if (l && typeof l == "object" && !Array.isArray(l)) {
        for (let V in l) {
          let se = r, P = r;
          if (typeof V == "string") {
            if (!y0.test(V)) {
              let oe = { instancePath: o, schemaPath: "#/anyOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(aria_label|ariaLabel))", schema: on.anyOf[1].propertyNames }, message: 'must match pattern "^(?!(aria_label|ariaLabel))"', schema: "^(?!(aria_label|ariaLabel))", parentSchema: on.anyOf[1].propertyNames, data: V, propertyName: V };
              e === null ? e = [oe] : e.push(oe), r++;
            }
          } else {
            let oe = { instancePath: o, schemaPath: "#/anyOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: on.anyOf[1].propertyNames }, message: "must be string", schema: on.anyOf[1].propertyNames.type, parentSchema: on.anyOf[1].propertyNames, data: V, propertyName: V };
            e === null ? e = [oe] : e.push(oe), r++;
          }
          var g = se === r;
          if (!g) {
            let oe = { instancePath: o, schemaPath: "#/anyOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: V, schema: on.anyOf[1] }, message: "property name must be valid", schema: on.anyOf[1].propertyNames, parentSchema: on.anyOf[1], data: l };
            e === null ? e = [oe] : e.push(oe), r++;
          }
        }
        let j = r;
        for (let V in l)
          if (!(V === "href" || V === "url" || V === "file" || V === "text" || V === "icon" || V === "aria-label" || V === "menu")) {
            let se = { instancePath: o, schemaPath: "#/anyOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: V, schema: on.anyOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: on.anyOf[1], data: l };
            e === null ? e = [se] : e.push(se), r++;
          }
        if (l.href !== void 0) {
          let V = l.href, se = r;
          if (typeof V != "string") {
            let oe = { instancePath: o + "/href", schemaPath: "#/anyOf/1/properties/href/type", keyword: "type", params: { type: "string", schema: on.anyOf[1].properties.href }, message: "must be string", schema: on.anyOf[1].properties.href.type, parentSchema: on.anyOf[1].properties.href, data: V };
            e === null ? e = [oe] : e.push(oe), r++;
          }
          let P = r;
          var x = se === r;
        }
        if (l.url !== void 0) {
          let V = l.url, se = r;
          if (typeof V != "string") {
            let h = { instancePath: o + "/url", schemaPath: "#/anyOf/1/properties/url/type", keyword: "type", params: { type: "string", schema: on.anyOf[1].properties.url }, message: "must be string", schema: on.anyOf[1].properties.url.type, parentSchema: on.anyOf[1].properties.url, data: V };
            e === null ? e = [h] : e.push(h), r++;
          }
          let P = r;
          var x = se === r;
        }
        if (l.file !== void 0) {
          let V = l.file, se = r;
          if (typeof V != "string") {
            let h = { instancePath: o + "/file", schemaPath: "#/anyOf/1/properties/file/type", keyword: "type", params: { type: "string", schema: on.anyOf[1].properties.file }, message: "must be string", schema: on.anyOf[1].properties.file.type, parentSchema: on.anyOf[1].properties.file, data: V };
            e === null ? e = [h] : e.push(h), r++;
          }
          let P = r;
          var x = se === r;
        }
        if (l.text !== void 0) {
          let V = l.text, se = r;
          if (typeof V != "string") {
            let h = { instancePath: o + "/text", schemaPath: "#/anyOf/1/properties/text/type", keyword: "type", params: { type: "string", schema: on.anyOf[1].properties.text }, message: "must be string", schema: on.anyOf[1].properties.text.type, parentSchema: on.anyOf[1].properties.text, data: V };
            e === null ? e = [h] : e.push(h), r++;
          }
          let P = r;
          var x = se === r;
        }
        if (l.icon !== void 0) {
          let V = l.icon, se = r;
          if (typeof V != "string") {
            let h = { instancePath: o + "/icon", schemaPath: "#/anyOf/1/properties/icon/type", keyword: "type", params: { type: "string", schema: on.anyOf[1].properties.icon }, message: "must be string", schema: on.anyOf[1].properties.icon.type, parentSchema: on.anyOf[1].properties.icon, data: V };
            e === null ? e = [h] : e.push(h), r++;
          }
          let P = r;
          var x = se === r;
        }
        if (l["aria-label"] !== void 0) {
          let V = l["aria-label"], se = r;
          if (typeof V != "string") {
            let h = { instancePath: o + "/aria-label", schemaPath: "#/anyOf/1/properties/aria-label/type", keyword: "type", params: { type: "string", schema: on.anyOf[1].properties["aria-label"] }, message: "must be string", schema: on.anyOf[1].properties["aria-label"].type, parentSchema: on.anyOf[1].properties["aria-label"], data: V };
            e === null ? e = [h] : e.push(h), r++;
          }
          let P = r;
          var x = se === r;
        }
        if (l.menu !== void 0) {
          let V = l.menu, se = r, P = r;
          if (Array.isArray(V)) {
            let h = true, f = V.length;
            for (let ft = 0; ft < f; ft++) {
              let Cr = V[ft], Dr = r;
              eg.validate(Cr, { instancePath: o + "/menu/" + ft, parentData: V, parentDataProperty: ft, rootData: s }) || (e = e === null ? eg.validate.errors : e.concat(eg.validate.errors), r = e.length);
              var D = Dr === r;
              D || (h = false);
            }
          } else {
            let h = { instancePath: o + "/menu", schemaPath: "#/anyOf/1/properties/menu/type", keyword: "type", params: { type: "array", schema: on.anyOf[1].properties.menu }, message: "must be array", schema: on.anyOf[1].properties.menu.type, parentSchema: on.anyOf[1].properties.menu, data: V };
            e === null ? e = [h] : e.push(h), r++;
          }
          var x = se === r;
        }
      } else {
        let j = { instancePath: o, schemaPath: "#/anyOf/1/type", keyword: "type", params: { type: "object", schema: on.anyOf[1] }, message: "must be object", schema: on.anyOf[1].type, parentSchema: on.anyOf[1], data: l };
        e === null ? e = [j] : e.push(j), r++;
      }
      var y = $ === r;
      c = c || y;
    }
    if (c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: on.anyOf, parentSchema: on, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return wa.errors = e, r === 0;
  }
  B["tool-item"] = co;
  var pi = { type: "object", description: "be an object", properties: { icon: { type: "string", description: "be a string" }, href: { type: "string", description: "be a string" }, menu: { type: "array", description: "be an array of values, where each element should be navigation-item", items: { $ref: "navigation-item", description: "be navigation-item" } } }, patternProperties: {}, $id: "tool-item" };
  function co(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      if (l.icon !== void 0) {
        let u = l.icon, y = r;
        if (typeof u != "string") {
          let x = { instancePath: o + "/icon", schemaPath: "#/properties/icon/type", keyword: "type", params: { type: "string", schema: pi.properties.icon }, message: "must be string", schema: pi.properties.icon.type, parentSchema: pi.properties.icon, data: u };
          e === null ? e = [x] : e.push(x), r++;
        }
        let g = r;
        var c = y === r;
      }
      if (l.href !== void 0) {
        let u = l.href, y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/href", schemaPath: "#/properties/href/type", keyword: "type", params: { type: "string", schema: pi.properties.href }, message: "must be string", schema: pi.properties.href.type, parentSchema: pi.properties.href, data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var c = y === r;
      }
      if (l.menu !== void 0) {
        let u = l.menu, y = r, g = r;
        if (Array.isArray(u)) {
          let D = true, $ = u.length;
          for (let m = 0; m < $; m++) {
            let R = u[m], j = r;
            wa(R, { instancePath: o + "/menu/" + m, parentData: u, parentDataProperty: m, rootData: s }) || (e = e === null ? wa.errors : e.concat(wa.errors), r = e.length);
            var d = j === r;
            d || (D = false);
          }
        } else {
          let D = { instancePath: o + "/menu", schemaPath: "#/properties/menu/type", keyword: "type", params: { type: "array", schema: pi.properties.menu }, message: "must be array", schema: pi.properties.menu.type, parentSchema: pi.properties.menu, data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        var c = y === r;
      }
    } else {
      let u = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: pi }, message: "must be object", schema: pi.type, parentSchema: pi, data: l };
      e === null ? e = [u] : e.push(u), r++;
    }
    return co.errors = e, r === 0;
  }
  B.comments = Uo;
  var ve = { type: "object", description: "be an object", properties: { utterances: { type: "object", description: "be an object", properties: { repo: { type: "string", description: "be a string" }, label: { type: "string", description: "be a string" }, theme: { type: "string", description: "be a string" } }, patternProperties: {}, required: ["repo"] }, hypothesis: { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { openSidebar: { type: "boolean", description: "be `true` or `false`" }, showHighlights: { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { enum: ["always", "never"], description: "be one of: `always`, `never`" }], description: "be exactly one of: `true` or `false`, one of: `always`, `never`" }, theme: { enum: ["classic", "clean"], description: "be one of: `classic`, `clean`" }, enableExperimentalNewNoteButton: { type: "boolean", description: "be `true` or `false`" }, usernameUrl: { type: "string", description: "be a string" }, services: { type: "array", description: "be an array of values, where each element should be an object", items: { type: "object", description: "be an object", properties: { apiUrl: { type: "string", description: "be a string" }, authority: { type: "string", description: "be a string" }, grantToken: { type: "string", description: "be a string" }, allowLeavingGroups: { type: "boolean", description: "be `true` or `false`" }, enableShareLinks: { type: "boolean", description: "be `true` or `false`" }, groups: { oneOf: [{ enum: ["$rpc:requestGroups"], description: "be '$rpc:requestGroups'" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: '$rpc:requestGroups', an array of values, where each element should be a string" }, icon: { type: "string", description: "be a string" } }, patternProperties: {}, required: ["apiUrl", "authority", "grantToken"], propertyNames: { type: "string", pattern: "^(?!(api_url|api-url|grant_token|grant-token|allow_leaving_groups|allow-leaving-groups|enable_share_links|enable-share-links))" } } }, branding: { type: "object", description: "be an object", properties: { accentColor: { type: "string", description: "be a string" }, appBackgroundColor: { type: "string", description: "be a string" }, ctaBackgroundColor: { type: "string", description: "be a string" }, selectionFontFamily: { type: "string", description: "be a string" }, annotationFontFamily: { type: "string", description: "be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(accent_color|accent-color|app_background_color|app-background-color|cta_background_color|cta-background-color|selection_font_family|selection-font-family|annotation_font_family|annotation-font-family))" } }, externalContainerSelector: { type: "string", description: "be a string" }, focus: { type: "object", description: "be an object", properties: { user: { type: "object", description: "be an object", properties: { username: { type: "string", description: "be a string" }, userid: { type: "string", description: "be a string" }, displayName: { type: "string", description: "be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(display_name|display-name))" } } }, patternProperties: {}, required: ["user"] }, requestConfigFromFrame: { type: "object", description: "be an object", properties: { origin: { type: "string", description: "be a string" }, ancestorLevel: { type: "number", description: "be a number" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(ancestor_level|ancestor-level))" } }, assetRoot: { type: "string", description: "be a string" }, sidebarAppUrl: { type: "string", description: "be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(open_sidebar|open-sidebar|show_highlights|show-highlights|enable_experimental_new_note_button|enable-experimental-new-note-button|username_url|username-url|external_container_selector|external-container-selector|request_config_from_frame|request-config-from-frame|asset_root|asset-root|sidebar_app_url|sidebar-app-url))" } }], description: "be exactly one of: `true` or `false`, an object" } }, patternProperties: {}, additionalProperties: false, $id: "comments" }, b0 = new RegExp("^(?!(open_sidebar|open-sidebar|show_highlights|show-highlights|enable_experimental_new_note_button|enable-experimental-new-note-button|username_url|username-url|external_container_selector|external-container-selector|request_config_from_frame|request-config-from-frame|asset_root|asset-root|sidebar_app_url|sidebar-app-url))", "u"), _0 = new RegExp("^(?!(api_url|api-url|grant_token|grant-token|allow_leaving_groups|allow-leaving-groups|enable_share_links|enable-share-links))", "u"), v0 = new RegExp("^(?!(accent_color|accent-color|app_background_color|app-background-color|cta_background_color|cta-background-color|selection_font_family|selection-font-family|annotation_font_family|annotation-font-family))", "u"), q0 = new RegExp("^(?!(display_name|display-name))", "u"), O0 = new RegExp("^(?!(ancestor_level|ancestor-level))", "u");
  function Uo(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      let Cr = r;
      for (let Dr in l)
        if (!(Dr === "utterances" || Dr === "hypothesis")) {
          let wr = { instancePath: o, schemaPath: "#/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: Dr, schema: ve }, message: "must NOT have additional properties", schema: false, parentSchema: ve, data: l };
          e === null ? e = [wr] : e.push(wr), r++;
        }
      if (l.utterances !== void 0) {
        let Dr = l.utterances, wr = r, ta = r;
        if (Dr && typeof Dr == "object" && !Array.isArray(Dr)) {
          if (Dr.repo === void 0) {
            let jt = { instancePath: o + "/utterances", schemaPath: "#/properties/utterances/required", keyword: "required", params: { missingProperty: "repo", schema: ve.properties.utterances }, message: "must have required property 'repo'", schema: ve.properties.utterances.required, parentSchema: ve.properties.utterances, data: Dr };
            e === null ? e = [jt] : e.push(jt), r++;
          }
          if (Dr.repo !== void 0) {
            let jt = Dr.repo, ee = r;
            if (typeof jt != "string") {
              let sa = { instancePath: o + "/utterances/repo", schemaPath: "#/properties/utterances/properties/repo/type", keyword: "type", params: { type: "string", schema: ve.properties.utterances.properties.repo }, message: "must be string", schema: ve.properties.utterances.properties.repo.type, parentSchema: ve.properties.utterances.properties.repo, data: jt };
              e === null ? e = [sa] : e.push(sa), r++;
            }
            let za = r;
            var c = ee === r;
          }
          if (Dr.label !== void 0) {
            let jt = Dr.label, ee = r;
            if (typeof jt != "string") {
              let _e = { instancePath: o + "/utterances/label", schemaPath: "#/properties/utterances/properties/label/type", keyword: "type", params: { type: "string", schema: ve.properties.utterances.properties.label }, message: "must be string", schema: ve.properties.utterances.properties.label.type, parentSchema: ve.properties.utterances.properties.label, data: jt };
              e === null ? e = [_e] : e.push(_e), r++;
            }
            let za = r;
            var c = ee === r;
          }
          if (Dr.theme !== void 0) {
            let jt = Dr.theme, ee = r;
            if (typeof jt != "string") {
              let _e = { instancePath: o + "/utterances/theme", schemaPath: "#/properties/utterances/properties/theme/type", keyword: "type", params: { type: "string", schema: ve.properties.utterances.properties.theme }, message: "must be string", schema: ve.properties.utterances.properties.theme.type, parentSchema: ve.properties.utterances.properties.theme, data: jt };
              e === null ? e = [_e] : e.push(_e), r++;
            }
            let za = r;
            var c = ee === r;
          }
        } else {
          let jt = { instancePath: o + "/utterances", schemaPath: "#/properties/utterances/type", keyword: "type", params: { type: "object", schema: ve.properties.utterances }, message: "must be object", schema: ve.properties.utterances.type, parentSchema: ve.properties.utterances, data: Dr };
          e === null ? e = [jt] : e.push(jt), r++;
        }
        var d = wr === r;
      }
      if (l.hypothesis !== void 0) {
        let Dr = l.hypothesis, wr = r, ta = r, jt = false, ee = null, za = r;
        if (typeof Dr != "boolean") {
          let ie = { instancePath: o + "/hypothesis", schemaPath: "#/properties/hypothesis/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: ve.properties.hypothesis.oneOf[0] }, message: "must be boolean", schema: ve.properties.hypothesis.oneOf[0].type, parentSchema: ve.properties.hypothesis.oneOf[0], data: Dr };
          e === null ? e = [ie] : e.push(ie), r++;
        }
        let sa = r;
        var u = za === r;
        u && (jt = true, ee = 0);
        let _e = r, Ma = r;
        if (Dr && typeof Dr == "object" && !Array.isArray(Dr)) {
          for (let ie in Dr) {
            let ka = r, hn = r;
            if (typeof ie == "string") {
              if (!b0.test(ie)) {
                let et = { instancePath: o + "/hypothesis", schemaPath: "#/properties/hypothesis/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(open_sidebar|open-sidebar|show_highlights|show-highlights|enable_experimental_new_note_button|enable-experimental-new-note-button|username_url|username-url|external_container_selector|external-container-selector|request_config_from_frame|request-config-from-frame|asset_root|asset-root|sidebar_app_url|sidebar-app-url))", schema: ve.properties.hypothesis.oneOf[1].propertyNames }, message: 'must match pattern "^(?!(open_sidebar|open-sidebar|show_highlights|show-highlights|enable_experimental_new_note_button|enable-experimental-new-note-button|username_url|username-url|external_container_selector|external-container-selector|request_config_from_frame|request-config-from-frame|asset_root|asset-root|sidebar_app_url|sidebar-app-url))"', schema: "^(?!(open_sidebar|open-sidebar|show_highlights|show-highlights|enable_experimental_new_note_button|enable-experimental-new-note-button|username_url|username-url|external_container_selector|external-container-selector|request_config_from_frame|request-config-from-frame|asset_root|asset-root|sidebar_app_url|sidebar-app-url))", parentSchema: ve.properties.hypothesis.oneOf[1].propertyNames, data: ie, propertyName: ie };
                e === null ? e = [et] : e.push(et), r++;
              }
            } else {
              let et = { instancePath: o + "/hypothesis", schemaPath: "#/properties/hypothesis/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].propertyNames }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].propertyNames.type, parentSchema: ve.properties.hypothesis.oneOf[1].propertyNames, data: ie, propertyName: ie };
              e === null ? e = [et] : e.push(et), r++;
            }
            var y = ka === r;
            if (!y) {
              let et = { instancePath: o + "/hypothesis", schemaPath: "#/properties/hypothesis/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: ie, schema: ve.properties.hypothesis.oneOf[1] }, message: "property name must be valid", schema: ve.properties.hypothesis.oneOf[1].propertyNames, parentSchema: ve.properties.hypothesis.oneOf[1], data: Dr };
              e === null ? e = [et] : e.push(et), r++;
            }
          }
          if (Dr.openSidebar !== void 0) {
            let ie = Dr.openSidebar, ka = r;
            if (typeof ie != "boolean") {
              let et = { instancePath: o + "/hypothesis/openSidebar", schemaPath: "#/properties/hypothesis/oneOf/1/properties/openSidebar/type", keyword: "type", params: { type: "boolean", schema: ve.properties.hypothesis.oneOf[1].properties.openSidebar }, message: "must be boolean", schema: ve.properties.hypothesis.oneOf[1].properties.openSidebar.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.openSidebar, data: ie };
              e === null ? e = [et] : e.push(et), r++;
            }
            let hn = r;
            var g = ka === r;
          }
          if (Dr.showHighlights !== void 0) {
            let ie = Dr.showHighlights, ka = r, hn = r, et = false, yt = null, _n = r;
            if (typeof ie != "boolean") {
              let Tt = { instancePath: o + "/hypothesis/showHighlights", schemaPath: "#/properties/hypothesis/oneOf/1/properties/showHighlights/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: ve.properties.hypothesis.oneOf[1].properties.showHighlights.oneOf[0] }, message: "must be boolean", schema: ve.properties.hypothesis.oneOf[1].properties.showHighlights.oneOf[0].type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.showHighlights.oneOf[0], data: ie };
              e === null ? e = [Tt] : e.push(Tt), r++;
            }
            let st = r;
            var x = _n === r;
            x && (et = true, yt = 0);
            let Et = r, ca = ve.properties.hypothesis.oneOf[1].properties.showHighlights.oneOf[1].enum;
            if (!(ie === "always" || ie === "never")) {
              let Tt = { instancePath: o + "/hypothesis/showHighlights", schemaPath: "#/properties/hypothesis/oneOf/1/properties/showHighlights/oneOf/1/enum", keyword: "enum", params: { allowedValues: ve.properties.hypothesis.oneOf[1].properties.showHighlights.oneOf[1].enum, schema: ve.properties.hypothesis.oneOf[1].properties.showHighlights.oneOf[1] }, message: "must be equal to one of the allowed values", schema: ve.properties.hypothesis.oneOf[1].properties.showHighlights.oneOf[1].enum, parentSchema: ve.properties.hypothesis.oneOf[1].properties.showHighlights.oneOf[1], data: ie };
              e === null ? e = [Tt] : e.push(Tt), r++;
            }
            var x = Et === r;
            if (x && et ? (et = false, yt = [yt, 1]) : x && (et = true, yt = 1), et)
              r = hn, e !== null && (hn ? e.length = hn : e = null);
            else {
              let Tt = { instancePath: o + "/hypothesis/showHighlights", schemaPath: "#/properties/hypothesis/oneOf/1/properties/showHighlights/oneOf", keyword: "oneOf", params: { passingSchemas: yt, schema: ve.properties.hypothesis.oneOf[1].properties.showHighlights }, message: "must match exactly one schema in oneOf", schema: ve.properties.hypothesis.oneOf[1].properties.showHighlights.oneOf, parentSchema: ve.properties.hypothesis.oneOf[1].properties.showHighlights, data: ie };
              e === null ? e = [Tt] : e.push(Tt), r++;
            }
            var g = ka === r;
          }
          if (Dr.theme !== void 0) {
            let ie = Dr.theme, ka = r, hn = ve.properties.hypothesis.oneOf[1].properties.theme.enum;
            if (!(ie === "classic" || ie === "clean")) {
              let yt = { instancePath: o + "/hypothesis/theme", schemaPath: "#/properties/hypothesis/oneOf/1/properties/theme/enum", keyword: "enum", params: { allowedValues: ve.properties.hypothesis.oneOf[1].properties.theme.enum, schema: ve.properties.hypothesis.oneOf[1].properties.theme }, message: "must be equal to one of the allowed values", schema: ve.properties.hypothesis.oneOf[1].properties.theme.enum, parentSchema: ve.properties.hypothesis.oneOf[1].properties.theme, data: ie };
              e === null ? e = [yt] : e.push(yt), r++;
            }
            var g = ka === r;
          }
          if (Dr.enableExperimentalNewNoteButton !== void 0) {
            let ie = Dr.enableExperimentalNewNoteButton, ka = r;
            if (typeof ie != "boolean") {
              let yt = { instancePath: o + "/hypothesis/enableExperimentalNewNoteButton", schemaPath: "#/properties/hypothesis/oneOf/1/properties/enableExperimentalNewNoteButton/type", keyword: "type", params: { type: "boolean", schema: ve.properties.hypothesis.oneOf[1].properties.enableExperimentalNewNoteButton }, message: "must be boolean", schema: ve.properties.hypothesis.oneOf[1].properties.enableExperimentalNewNoteButton.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.enableExperimentalNewNoteButton, data: ie };
              e === null ? e = [yt] : e.push(yt), r++;
            }
            let hn = r;
            var g = ka === r;
          }
          if (Dr.usernameUrl !== void 0) {
            let ie = Dr.usernameUrl, ka = r;
            if (typeof ie != "string") {
              let yt = { instancePath: o + "/hypothesis/usernameUrl", schemaPath: "#/properties/hypothesis/oneOf/1/properties/usernameUrl/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.usernameUrl }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.usernameUrl.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.usernameUrl, data: ie };
              e === null ? e = [yt] : e.push(yt), r++;
            }
            let hn = r;
            var g = ka === r;
          }
          if (Dr.services !== void 0) {
            let ie = Dr.services, ka = r, hn = r;
            if (Array.isArray(ie)) {
              let yt = true, _n = ie.length;
              for (let st = 0; st < _n; st++) {
                let Et = ie[st], ca = r, Gr = r;
                if (Et && typeof Et == "object" && !Array.isArray(Et)) {
                  if (Et.apiUrl === void 0) {
                    let Tt = { instancePath: o + "/hypothesis/services/" + st, schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/required", keyword: "required", params: { missingProperty: "apiUrl", schema: ve.properties.hypothesis.oneOf[1].properties.services.items }, message: "must have required property 'apiUrl'", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.required, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items, data: Et };
                    e === null ? e = [Tt] : e.push(Tt), r++;
                  }
                  if (Et.authority === void 0) {
                    let Tt = { instancePath: o + "/hypothesis/services/" + st, schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/required", keyword: "required", params: { missingProperty: "authority", schema: ve.properties.hypothesis.oneOf[1].properties.services.items }, message: "must have required property 'authority'", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.required, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items, data: Et };
                    e === null ? e = [Tt] : e.push(Tt), r++;
                  }
                  if (Et.grantToken === void 0) {
                    let Tt = { instancePath: o + "/hypothesis/services/" + st, schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/required", keyword: "required", params: { missingProperty: "grantToken", schema: ve.properties.hypothesis.oneOf[1].properties.services.items }, message: "must have required property 'grantToken'", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.required, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items, data: Et };
                    e === null ? e = [Tt] : e.push(Tt), r++;
                  }
                  for (let Tt in Et) {
                    let bn = r, vt = r;
                    if (typeof Tt == "string") {
                      if (!_0.test(Tt)) {
                        let yn = { instancePath: o + "/hypothesis/services/" + st, schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(api_url|api-url|grant_token|grant-token|allow_leaving_groups|allow-leaving-groups|enable_share_links|enable-share-links))", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.propertyNames }, message: 'must match pattern "^(?!(api_url|api-url|grant_token|grant-token|allow_leaving_groups|allow-leaving-groups|enable_share_links|enable-share-links))"', schema: "^(?!(api_url|api-url|grant_token|grant-token|allow_leaving_groups|allow-leaving-groups|enable_share_links|enable-share-links))", parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items.propertyNames, data: Tt, propertyName: Tt };
                        e === null ? e = [yn] : e.push(yn), r++;
                      }
                    } else {
                      let yn = { instancePath: o + "/hypothesis/services/" + st, schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/propertyNames/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.propertyNames }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.propertyNames.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items.propertyNames, data: Tt, propertyName: Tt };
                      e === null ? e = [yn] : e.push(yn), r++;
                    }
                    var D = bn === r;
                    if (!D) {
                      let yn = { instancePath: o + "/hypothesis/services/" + st, schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/propertyNames", keyword: "propertyNames", params: { propertyName: Tt, schema: ve.properties.hypothesis.oneOf[1].properties.services.items }, message: "property name must be valid", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.propertyNames, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items, data: Et };
                      e === null ? e = [yn] : e.push(yn), r++;
                    }
                  }
                  if (Et.apiUrl !== void 0) {
                    let Tt = Et.apiUrl, bn = r;
                    if (typeof Tt != "string") {
                      let yn = { instancePath: o + "/hypothesis/services/" + st + "/apiUrl", schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/properties/apiUrl/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.apiUrl }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.apiUrl.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.apiUrl, data: Tt };
                      e === null ? e = [yn] : e.push(yn), r++;
                    }
                    let vt = r;
                    var $ = bn === r;
                  }
                  if (Et.authority !== void 0) {
                    let Tt = Et.authority, bn = r;
                    if (typeof Tt != "string") {
                      let Va = { instancePath: o + "/hypothesis/services/" + st + "/authority", schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/properties/authority/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.authority }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.authority.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.authority, data: Tt };
                      e === null ? e = [Va] : e.push(Va), r++;
                    }
                    let vt = r;
                    var $ = bn === r;
                  }
                  if (Et.grantToken !== void 0) {
                    let Tt = Et.grantToken, bn = r;
                    if (typeof Tt != "string") {
                      let Va = { instancePath: o + "/hypothesis/services/" + st + "/grantToken", schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/properties/grantToken/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.grantToken }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.grantToken.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.grantToken, data: Tt };
                      e === null ? e = [Va] : e.push(Va), r++;
                    }
                    let vt = r;
                    var $ = bn === r;
                  }
                  if (Et.allowLeavingGroups !== void 0) {
                    let Tt = Et.allowLeavingGroups, bn = r;
                    if (typeof Tt != "boolean") {
                      let Va = { instancePath: o + "/hypothesis/services/" + st + "/allowLeavingGroups", schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/properties/allowLeavingGroups/type", keyword: "type", params: { type: "boolean", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.allowLeavingGroups }, message: "must be boolean", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.allowLeavingGroups.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.allowLeavingGroups, data: Tt };
                      e === null ? e = [Va] : e.push(Va), r++;
                    }
                    let vt = r;
                    var $ = bn === r;
                  }
                  if (Et.enableShareLinks !== void 0) {
                    let Tt = Et.enableShareLinks, bn = r;
                    if (typeof Tt != "boolean") {
                      let Va = { instancePath: o + "/hypothesis/services/" + st + "/enableShareLinks", schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/properties/enableShareLinks/type", keyword: "type", params: { type: "boolean", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.enableShareLinks }, message: "must be boolean", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.enableShareLinks.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.enableShareLinks, data: Tt };
                      e === null ? e = [Va] : e.push(Va), r++;
                    }
                    let vt = r;
                    var $ = bn === r;
                  }
                  if (Et.groups !== void 0) {
                    let Tt = Et.groups, bn = r, vt = r, yn = false, Va = null, Dt = r, nl = ve.properties.hypothesis.oneOf[1].properties.services.items.properties.groups.oneOf[0].enum;
                    if (Tt !== "$rpc:requestGroups") {
                      let Jn = { instancePath: o + "/hypothesis/services/" + st + "/groups", schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/properties/groups/oneOf/0/enum", keyword: "enum", params: { allowedValues: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.groups.oneOf[0].enum, schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.groups.oneOf[0] }, message: "must be equal to one of the allowed values", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.groups.oneOf[0].enum, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.groups.oneOf[0], data: Tt };
                      e === null ? e = [Jn] : e.push(Jn), r++;
                    }
                    var m = Dt === r;
                    m && (yn = true, Va = 0);
                    let ol = r, wt = r;
                    if (Array.isArray(Tt)) {
                      let Jn = true, Bt = Tt.length;
                      for (let oo = 0; oo < Bt; oo++) {
                        let qo = Tt[oo], Ot = r;
                        if (typeof qo != "string") {
                          let li = { instancePath: o + "/hypothesis/services/" + st + "/groups/" + oo, schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/properties/groups/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.groups.oneOf[1].items }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.groups.oneOf[1].items.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.groups.oneOf[1].items, data: qo };
                          e === null ? e = [li] : e.push(li), r++;
                        }
                        let xo = r;
                        var R = Ot === r;
                        R || (Jn = false);
                      }
                    } else {
                      let Jn = { instancePath: o + "/hypothesis/services/" + st + "/groups", schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/properties/groups/oneOf/1/type", keyword: "type", params: { type: "array", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.groups.oneOf[1] }, message: "must be array", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.groups.oneOf[1].type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.groups.oneOf[1], data: Tt };
                      e === null ? e = [Jn] : e.push(Jn), r++;
                    }
                    var m = ol === r;
                    if (m && yn ? (yn = false, Va = [Va, 1]) : m && (yn = true, Va = 1), yn)
                      r = vt, e !== null && (vt ? e.length = vt : e = null);
                    else {
                      let Jn = { instancePath: o + "/hypothesis/services/" + st + "/groups", schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/properties/groups/oneOf", keyword: "oneOf", params: { passingSchemas: Va, schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.groups }, message: "must match exactly one schema in oneOf", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.groups.oneOf, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.groups, data: Tt };
                      e === null ? e = [Jn] : e.push(Jn), r++;
                    }
                    var $ = bn === r;
                  }
                  if (Et.icon !== void 0) {
                    let Tt = Et.icon, bn = r;
                    if (typeof Tt != "string") {
                      let Va = { instancePath: o + "/hypothesis/services/" + st + "/icon", schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/properties/icon/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.icon }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.icon.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items.properties.icon, data: Tt };
                      e === null ? e = [Va] : e.push(Va), r++;
                    }
                    let vt = r;
                    var $ = bn === r;
                  }
                } else {
                  let Tt = { instancePath: o + "/hypothesis/services/" + st, schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/items/type", keyword: "type", params: { type: "object", schema: ve.properties.hypothesis.oneOf[1].properties.services.items }, message: "must be object", schema: ve.properties.hypothesis.oneOf[1].properties.services.items.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services.items, data: Et };
                  e === null ? e = [Tt] : e.push(Tt), r++;
                }
                var j = ca === r;
                j || (yt = false);
              }
            } else {
              let yt = { instancePath: o + "/hypothesis/services", schemaPath: "#/properties/hypothesis/oneOf/1/properties/services/type", keyword: "type", params: { type: "array", schema: ve.properties.hypothesis.oneOf[1].properties.services }, message: "must be array", schema: ve.properties.hypothesis.oneOf[1].properties.services.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.services, data: ie };
              e === null ? e = [yt] : e.push(yt), r++;
            }
            var g = ka === r;
          }
          if (Dr.branding !== void 0) {
            let ie = Dr.branding, ka = r, hn = r;
            if (ie && typeof ie == "object" && !Array.isArray(ie)) {
              for (let yt in ie) {
                let _n = r, st = r;
                if (typeof yt == "string") {
                  if (!v0.test(yt)) {
                    let Et = { instancePath: o + "/hypothesis/branding", schemaPath: "#/properties/hypothesis/oneOf/1/properties/branding/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(accent_color|accent-color|app_background_color|app-background-color|cta_background_color|cta-background-color|selection_font_family|selection-font-family|annotation_font_family|annotation-font-family))", schema: ve.properties.hypothesis.oneOf[1].properties.branding.propertyNames }, message: 'must match pattern "^(?!(accent_color|accent-color|app_background_color|app-background-color|cta_background_color|cta-background-color|selection_font_family|selection-font-family|annotation_font_family|annotation-font-family))"', schema: "^(?!(accent_color|accent-color|app_background_color|app-background-color|cta_background_color|cta-background-color|selection_font_family|selection-font-family|annotation_font_family|annotation-font-family))", parentSchema: ve.properties.hypothesis.oneOf[1].properties.branding.propertyNames, data: yt, propertyName: yt };
                    e === null ? e = [Et] : e.push(Et), r++;
                  }
                } else {
                  let Et = { instancePath: o + "/hypothesis/branding", schemaPath: "#/properties/hypothesis/oneOf/1/properties/branding/propertyNames/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.branding.propertyNames }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.branding.propertyNames.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.branding.propertyNames, data: yt, propertyName: yt };
                  e === null ? e = [Et] : e.push(Et), r++;
                }
                var V = _n === r;
                if (!V) {
                  let Et = { instancePath: o + "/hypothesis/branding", schemaPath: "#/properties/hypothesis/oneOf/1/properties/branding/propertyNames", keyword: "propertyNames", params: { propertyName: yt, schema: ve.properties.hypothesis.oneOf[1].properties.branding }, message: "property name must be valid", schema: ve.properties.hypothesis.oneOf[1].properties.branding.propertyNames, parentSchema: ve.properties.hypothesis.oneOf[1].properties.branding, data: ie };
                  e === null ? e = [Et] : e.push(Et), r++;
                }
              }
              if (ie.accentColor !== void 0) {
                let yt = ie.accentColor, _n = r;
                if (typeof yt != "string") {
                  let Et = { instancePath: o + "/hypothesis/branding/accentColor", schemaPath: "#/properties/hypothesis/oneOf/1/properties/branding/properties/accentColor/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.branding.properties.accentColor }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.branding.properties.accentColor.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.branding.properties.accentColor, data: yt };
                  e === null ? e = [Et] : e.push(Et), r++;
                }
                let st = r;
                var se = _n === r;
              }
              if (ie.appBackgroundColor !== void 0) {
                let yt = ie.appBackgroundColor, _n = r;
                if (typeof yt != "string") {
                  let ca = { instancePath: o + "/hypothesis/branding/appBackgroundColor", schemaPath: "#/properties/hypothesis/oneOf/1/properties/branding/properties/appBackgroundColor/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.branding.properties.appBackgroundColor }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.branding.properties.appBackgroundColor.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.branding.properties.appBackgroundColor, data: yt };
                  e === null ? e = [ca] : e.push(ca), r++;
                }
                let st = r;
                var se = _n === r;
              }
              if (ie.ctaBackgroundColor !== void 0) {
                let yt = ie.ctaBackgroundColor, _n = r;
                if (typeof yt != "string") {
                  let ca = { instancePath: o + "/hypothesis/branding/ctaBackgroundColor", schemaPath: "#/properties/hypothesis/oneOf/1/properties/branding/properties/ctaBackgroundColor/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.branding.properties.ctaBackgroundColor }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.branding.properties.ctaBackgroundColor.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.branding.properties.ctaBackgroundColor, data: yt };
                  e === null ? e = [ca] : e.push(ca), r++;
                }
                let st = r;
                var se = _n === r;
              }
              if (ie.selectionFontFamily !== void 0) {
                let yt = ie.selectionFontFamily, _n = r;
                if (typeof yt != "string") {
                  let ca = { instancePath: o + "/hypothesis/branding/selectionFontFamily", schemaPath: "#/properties/hypothesis/oneOf/1/properties/branding/properties/selectionFontFamily/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.branding.properties.selectionFontFamily }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.branding.properties.selectionFontFamily.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.branding.properties.selectionFontFamily, data: yt };
                  e === null ? e = [ca] : e.push(ca), r++;
                }
                let st = r;
                var se = _n === r;
              }
              if (ie.annotationFontFamily !== void 0) {
                let yt = ie.annotationFontFamily, _n = r;
                if (typeof yt != "string") {
                  let ca = { instancePath: o + "/hypothesis/branding/annotationFontFamily", schemaPath: "#/properties/hypothesis/oneOf/1/properties/branding/properties/annotationFontFamily/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.branding.properties.annotationFontFamily }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.branding.properties.annotationFontFamily.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.branding.properties.annotationFontFamily, data: yt };
                  e === null ? e = [ca] : e.push(ca), r++;
                }
                let st = r;
                var se = _n === r;
              }
            } else {
              let yt = { instancePath: o + "/hypothesis/branding", schemaPath: "#/properties/hypothesis/oneOf/1/properties/branding/type", keyword: "type", params: { type: "object", schema: ve.properties.hypothesis.oneOf[1].properties.branding }, message: "must be object", schema: ve.properties.hypothesis.oneOf[1].properties.branding.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.branding, data: ie };
              e === null ? e = [yt] : e.push(yt), r++;
            }
            var g = ka === r;
          }
          if (Dr.externalContainerSelector !== void 0) {
            let ie = Dr.externalContainerSelector, ka = r;
            if (typeof ie != "string") {
              let yt = { instancePath: o + "/hypothesis/externalContainerSelector", schemaPath: "#/properties/hypothesis/oneOf/1/properties/externalContainerSelector/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.externalContainerSelector }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.externalContainerSelector.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.externalContainerSelector, data: ie };
              e === null ? e = [yt] : e.push(yt), r++;
            }
            let hn = r;
            var g = ka === r;
          }
          if (Dr.focus !== void 0) {
            let ie = Dr.focus, ka = r, hn = r;
            if (ie && typeof ie == "object" && !Array.isArray(ie)) {
              if (ie.user === void 0) {
                let yt = { instancePath: o + "/hypothesis/focus", schemaPath: "#/properties/hypothesis/oneOf/1/properties/focus/required", keyword: "required", params: { missingProperty: "user", schema: ve.properties.hypothesis.oneOf[1].properties.focus }, message: "must have required property 'user'", schema: ve.properties.hypothesis.oneOf[1].properties.focus.required, parentSchema: ve.properties.hypothesis.oneOf[1].properties.focus, data: ie };
                e === null ? e = [yt] : e.push(yt), r++;
              }
              if (ie.user !== void 0) {
                let yt = ie.user, _n = r, st = r;
                if (yt && typeof yt == "object" && !Array.isArray(yt)) {
                  for (let Et in yt) {
                    let ca = r, Gr = r;
                    if (typeof Et == "string") {
                      if (!q0.test(Et)) {
                        let Tt = { instancePath: o + "/hypothesis/focus/user", schemaPath: "#/properties/hypothesis/oneOf/1/properties/focus/properties/user/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(display_name|display-name))", schema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user.propertyNames }, message: 'must match pattern "^(?!(display_name|display-name))"', schema: "^(?!(display_name|display-name))", parentSchema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user.propertyNames, data: Et, propertyName: Et };
                        e === null ? e = [Tt] : e.push(Tt), r++;
                      }
                    } else {
                      let Tt = { instancePath: o + "/hypothesis/focus/user", schemaPath: "#/properties/hypothesis/oneOf/1/properties/focus/properties/user/propertyNames/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user.propertyNames }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user.propertyNames.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user.propertyNames, data: Et, propertyName: Et };
                      e === null ? e = [Tt] : e.push(Tt), r++;
                    }
                    var P = ca === r;
                    if (!P) {
                      let Tt = { instancePath: o + "/hypothesis/focus/user", schemaPath: "#/properties/hypothesis/oneOf/1/properties/focus/properties/user/propertyNames", keyword: "propertyNames", params: { propertyName: Et, schema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user }, message: "property name must be valid", schema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user.propertyNames, parentSchema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user, data: yt };
                      e === null ? e = [Tt] : e.push(Tt), r++;
                    }
                  }
                  if (yt.username !== void 0) {
                    let Et = yt.username, ca = r;
                    if (typeof Et != "string") {
                      let Tt = { instancePath: o + "/hypothesis/focus/user/username", schemaPath: "#/properties/hypothesis/oneOf/1/properties/focus/properties/user/properties/username/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user.properties.username }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user.properties.username.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user.properties.username, data: Et };
                      e === null ? e = [Tt] : e.push(Tt), r++;
                    }
                    let Gr = r;
                    var oe = ca === r;
                  }
                  if (yt.userid !== void 0) {
                    let Et = yt.userid, ca = r;
                    if (typeof Et != "string") {
                      let bn = { instancePath: o + "/hypothesis/focus/user/userid", schemaPath: "#/properties/hypothesis/oneOf/1/properties/focus/properties/user/properties/userid/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user.properties.userid }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user.properties.userid.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user.properties.userid, data: Et };
                      e === null ? e = [bn] : e.push(bn), r++;
                    }
                    let Gr = r;
                    var oe = ca === r;
                  }
                  if (yt.displayName !== void 0) {
                    let Et = yt.displayName, ca = r;
                    if (typeof Et != "string") {
                      let bn = { instancePath: o + "/hypothesis/focus/user/displayName", schemaPath: "#/properties/hypothesis/oneOf/1/properties/focus/properties/user/properties/displayName/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user.properties.displayName }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user.properties.displayName.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user.properties.displayName, data: Et };
                      e === null ? e = [bn] : e.push(bn), r++;
                    }
                    let Gr = r;
                    var oe = ca === r;
                  }
                } else {
                  let Et = { instancePath: o + "/hypothesis/focus/user", schemaPath: "#/properties/hypothesis/oneOf/1/properties/focus/properties/user/type", keyword: "type", params: { type: "object", schema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user }, message: "must be object", schema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.focus.properties.user, data: yt };
                  e === null ? e = [Et] : e.push(Et), r++;
                }
                var h = _n === r;
              }
            } else {
              let yt = { instancePath: o + "/hypothesis/focus", schemaPath: "#/properties/hypothesis/oneOf/1/properties/focus/type", keyword: "type", params: { type: "object", schema: ve.properties.hypothesis.oneOf[1].properties.focus }, message: "must be object", schema: ve.properties.hypothesis.oneOf[1].properties.focus.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.focus, data: ie };
              e === null ? e = [yt] : e.push(yt), r++;
            }
            var g = ka === r;
          }
          if (Dr.requestConfigFromFrame !== void 0) {
            let ie = Dr.requestConfigFromFrame, ka = r, hn = r;
            if (ie && typeof ie == "object" && !Array.isArray(ie)) {
              for (let yt in ie) {
                let _n = r, st = r;
                if (typeof yt == "string") {
                  if (!O0.test(yt)) {
                    let Et = { instancePath: o + "/hypothesis/requestConfigFromFrame", schemaPath: "#/properties/hypothesis/oneOf/1/properties/requestConfigFromFrame/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(ancestor_level|ancestor-level))", schema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame.propertyNames }, message: 'must match pattern "^(?!(ancestor_level|ancestor-level))"', schema: "^(?!(ancestor_level|ancestor-level))", parentSchema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame.propertyNames, data: yt, propertyName: yt };
                    e === null ? e = [Et] : e.push(Et), r++;
                  }
                } else {
                  let Et = { instancePath: o + "/hypothesis/requestConfigFromFrame", schemaPath: "#/properties/hypothesis/oneOf/1/properties/requestConfigFromFrame/propertyNames/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame.propertyNames }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame.propertyNames.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame.propertyNames, data: yt, propertyName: yt };
                  e === null ? e = [Et] : e.push(Et), r++;
                }
                var f = _n === r;
                if (!f) {
                  let Et = { instancePath: o + "/hypothesis/requestConfigFromFrame", schemaPath: "#/properties/hypothesis/oneOf/1/properties/requestConfigFromFrame/propertyNames", keyword: "propertyNames", params: { propertyName: yt, schema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame }, message: "property name must be valid", schema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame.propertyNames, parentSchema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame, data: ie };
                  e === null ? e = [Et] : e.push(Et), r++;
                }
              }
              if (ie.origin !== void 0) {
                let yt = ie.origin, _n = r;
                if (typeof yt != "string") {
                  let Et = { instancePath: o + "/hypothesis/requestConfigFromFrame/origin", schemaPath: "#/properties/hypothesis/oneOf/1/properties/requestConfigFromFrame/properties/origin/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame.properties.origin }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame.properties.origin.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame.properties.origin, data: yt };
                  e === null ? e = [Et] : e.push(Et), r++;
                }
                let st = r;
                var ft = _n === r;
              }
              if (ie.ancestorLevel !== void 0) {
                let yt = ie.ancestorLevel, _n = r;
                if (!(typeof yt == "number" && isFinite(yt))) {
                  let ca = { instancePath: o + "/hypothesis/requestConfigFromFrame/ancestorLevel", schemaPath: "#/properties/hypothesis/oneOf/1/properties/requestConfigFromFrame/properties/ancestorLevel/type", keyword: "type", params: { type: "number", schema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame.properties.ancestorLevel }, message: "must be number", schema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame.properties.ancestorLevel.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame.properties.ancestorLevel, data: yt };
                  e === null ? e = [ca] : e.push(ca), r++;
                }
                let st = r;
                var ft = _n === r;
              }
            } else {
              let yt = { instancePath: o + "/hypothesis/requestConfigFromFrame", schemaPath: "#/properties/hypothesis/oneOf/1/properties/requestConfigFromFrame/type", keyword: "type", params: { type: "object", schema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame }, message: "must be object", schema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.requestConfigFromFrame, data: ie };
              e === null ? e = [yt] : e.push(yt), r++;
            }
            var g = ka === r;
          }
          if (Dr.assetRoot !== void 0) {
            let ie = Dr.assetRoot, ka = r;
            if (typeof ie != "string") {
              let yt = { instancePath: o + "/hypothesis/assetRoot", schemaPath: "#/properties/hypothesis/oneOf/1/properties/assetRoot/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.assetRoot }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.assetRoot.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.assetRoot, data: ie };
              e === null ? e = [yt] : e.push(yt), r++;
            }
            let hn = r;
            var g = ka === r;
          }
          if (Dr.sidebarAppUrl !== void 0) {
            let ie = Dr.sidebarAppUrl, ka = r;
            if (typeof ie != "string") {
              let yt = { instancePath: o + "/hypothesis/sidebarAppUrl", schemaPath: "#/properties/hypothesis/oneOf/1/properties/sidebarAppUrl/type", keyword: "type", params: { type: "string", schema: ve.properties.hypothesis.oneOf[1].properties.sidebarAppUrl }, message: "must be string", schema: ve.properties.hypothesis.oneOf[1].properties.sidebarAppUrl.type, parentSchema: ve.properties.hypothesis.oneOf[1].properties.sidebarAppUrl, data: ie };
              e === null ? e = [yt] : e.push(yt), r++;
            }
            let hn = r;
            var g = ka === r;
          }
        } else {
          let ie = { instancePath: o + "/hypothesis", schemaPath: "#/properties/hypothesis/oneOf/1/type", keyword: "type", params: { type: "object", schema: ve.properties.hypothesis.oneOf[1] }, message: "must be object", schema: ve.properties.hypothesis.oneOf[1].type, parentSchema: ve.properties.hypothesis.oneOf[1], data: Dr };
          e === null ? e = [ie] : e.push(ie), r++;
        }
        var u = _e === r;
        if (u && jt ? (jt = false, ee = [ee, 1]) : u && (jt = true, ee = 1), jt)
          r = ta, e !== null && (ta ? e.length = ta : e = null);
        else {
          let ie = { instancePath: o + "/hypothesis", schemaPath: "#/properties/hypothesis/oneOf", keyword: "oneOf", params: { passingSchemas: ee, schema: ve.properties.hypothesis }, message: "must match exactly one schema in oneOf", schema: ve.properties.hypothesis.oneOf, parentSchema: ve.properties.hypothesis, data: Dr };
          e === null ? e = [ie] : e.push(ie), r++;
        }
        var d = wr === r;
      }
    } else {
      let Cr = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: ve }, message: "must be object", schema: ve.type, parentSchema: ve, data: l };
      e === null ? e = [Cr] : e.push(Cr), r++;
    }
    return Uo.errors = e, r === 0;
  }
  B["social-metadata"] = D1;
  var zn = { type: "object", description: "be an object", properties: { title: { type: "string", description: "be a string" }, description: { type: "string", description: "be a string" }, image: { type: "string", description: "be a string" }, "image-width": { type: "number", description: "be a number" }, "image-height": { type: "number", description: "be a number" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(image_width|imageWidth|image_height|imageHeight))" }, $id: "social-metadata" }, P0 = new RegExp("^(?!(image_width|imageWidth|image_height|imageHeight))", "u");
  function D1(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let u in l) {
        let y = r, g = r;
        if (typeof u == "string") {
          if (!P0.test(u)) {
            let x = { instancePath: o, schemaPath: "#/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(image_width|imageWidth|image_height|imageHeight))", schema: zn.propertyNames }, message: 'must match pattern "^(?!(image_width|imageWidth|image_height|imageHeight))"', schema: "^(?!(image_width|imageWidth|image_height|imageHeight))", parentSchema: zn.propertyNames, data: u, propertyName: u };
            e === null ? e = [x] : e.push(x), r++;
          }
        } else {
          let x = { instancePath: o, schemaPath: "#/propertyNames/type", keyword: "type", params: { type: "string", schema: zn.propertyNames }, message: "must be string", schema: zn.propertyNames.type, parentSchema: zn.propertyNames, data: u, propertyName: u };
          e === null ? e = [x] : e.push(x), r++;
        }
        var c = y === r;
        if (!c) {
          let x = { instancePath: o, schemaPath: "#/propertyNames", keyword: "propertyNames", params: { propertyName: u, schema: zn }, message: "property name must be valid", schema: zn.propertyNames, parentSchema: zn, data: l };
          e === null ? e = [x] : e.push(x), r++;
        }
      }
      if (l.title !== void 0) {
        let u = l.title, y = r;
        if (typeof u != "string") {
          let x = { instancePath: o + "/title", schemaPath: "#/properties/title/type", keyword: "type", params: { type: "string", schema: zn.properties.title }, message: "must be string", schema: zn.properties.title.type, parentSchema: zn.properties.title, data: u };
          e === null ? e = [x] : e.push(x), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l.description !== void 0) {
        let u = l.description, y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/description", schemaPath: "#/properties/description/type", keyword: "type", params: { type: "string", schema: zn.properties.description }, message: "must be string", schema: zn.properties.description.type, parentSchema: zn.properties.description, data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l.image !== void 0) {
        let u = l.image, y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/image", schemaPath: "#/properties/image/type", keyword: "type", params: { type: "string", schema: zn.properties.image }, message: "must be string", schema: zn.properties.image.type, parentSchema: zn.properties.image, data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["image-width"] !== void 0) {
        let u = l["image-width"], y = r;
        if (!(typeof u == "number" && isFinite(u))) {
          let D = { instancePath: o + "/image-width", schemaPath: "#/properties/image-width/type", keyword: "type", params: { type: "number", schema: zn.properties["image-width"] }, message: "must be number", schema: zn.properties["image-width"].type, parentSchema: zn.properties["image-width"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["image-height"] !== void 0) {
        let u = l["image-height"], y = r;
        if (!(typeof u == "number" && isFinite(u))) {
          let D = { instancePath: o + "/image-height", schemaPath: "#/properties/image-height/type", keyword: "type", params: { type: "number", schema: zn.properties["image-height"] }, message: "must be number", schema: zn.properties["image-height"].type, parentSchema: zn.properties["image-height"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
    } else {
      let u = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: zn }, message: "must be object", schema: zn.type, parentSchema: zn, data: l };
      e === null ? e = [u] : e.push(u), r++;
    }
    return D1.errors = e, r === 0;
  }
  B["page-footer-region"] = qn;
  var nc = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be navigation-item", items: { $ref: "navigation-item", description: "be navigation-item" } }], description: "be exactly one of: a string, an array of values, where each element should be navigation-item", $id: "page-footer-region" };
  function qn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: nc.oneOf[0] }, message: "must be string", schema: nc.oneOf[0].type, parentSchema: nc.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        wa(V, { instancePath: o + "/" + j, parentData: l, parentDataProperty: j, rootData: s }) || (e = e === null ? wa.errors : e.concat(wa.errors), r = e.length);
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: nc.oneOf[1] }, message: "must be array", schema: nc.oneOf[1].type, parentSchema: nc.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: nc }, message: "must match exactly one schema in oneOf", schema: nc.oneOf, parentSchema: nc, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return qn.errors = e, r === 0;
  }
  B["sidebar-contents"] = so;
  var Ko = { type: "array", description: "be an array of values, where each element should be at least one of: navigation-item, a string, an object", items: { anyOf: [{ $ref: "navigation-item", description: "be navigation-item" }, { type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: { section: { type: "string", description: "be a string" }, contents: { $ref: "sidebar-contents", description: "be sidebar-contents" } }, patternProperties: {} }], description: "be at least one of: navigation-item, a string, an object" }, $id: "sidebar-contents" }, rg = { validate: so };
  function so(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let y = true, g = l.length;
      for (let x = 0; x < g; x++) {
        let D = l[x], $ = r, m = r, R = false, j = r;
        wa(D, { instancePath: o + "/" + x, parentData: l, parentDataProperty: x, rootData: s }) || (e = e === null ? wa.errors : e.concat(wa.errors), r = e.length);
        var c = j === r;
        if (R = R || c, !R) {
          let V = r;
          if (typeof D != "string") {
            let oe = { instancePath: o + "/" + x, schemaPath: "#/items/anyOf/1/type", keyword: "type", params: { type: "string", schema: Ko.items.anyOf[1] }, message: "must be string", schema: Ko.items.anyOf[1].type, parentSchema: Ko.items.anyOf[1], data: D };
            e === null ? e = [oe] : e.push(oe), r++;
          }
          let se = r;
          var c = V === r;
          if (R = R || c, !R) {
            let oe = r, h = r;
            if (D && typeof D == "object" && !Array.isArray(D)) {
              if (D.section !== void 0) {
                let ft = D.section, Cr = r;
                if (typeof ft != "string") {
                  let wr = { instancePath: o + "/" + x + "/section", schemaPath: "#/items/anyOf/2/properties/section/type", keyword: "type", params: { type: "string", schema: Ko.items.anyOf[2].properties.section }, message: "must be string", schema: Ko.items.anyOf[2].properties.section.type, parentSchema: Ko.items.anyOf[2].properties.section, data: ft };
                  e === null ? e = [wr] : e.push(wr), r++;
                }
                let Dr = r;
                var d = Cr === r;
              }
              if (D.contents !== void 0) {
                let ft = D.contents, Cr = r;
                rg.validate(ft, { instancePath: o + "/" + x + "/contents", parentData: D, parentDataProperty: "contents", rootData: s }) || (e = e === null ? rg.validate.errors : e.concat(rg.validate.errors), r = e.length);
                var d = Cr === r;
              }
            } else {
              let ft = { instancePath: o + "/" + x, schemaPath: "#/items/anyOf/2/type", keyword: "type", params: { type: "object", schema: Ko.items.anyOf[2] }, message: "must be object", schema: Ko.items.anyOf[2].type, parentSchema: Ko.items.anyOf[2], data: D };
              e === null ? e = [ft] : e.push(ft), r++;
            }
            var c = oe === r;
            R = R || c;
          }
        }
        if (R)
          r = m, e !== null && (m ? e.length = m : e = null);
        else {
          let V = { instancePath: o + "/" + x, schemaPath: "#/items/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: Ko.items.anyOf, parentSchema: Ko.items, data: D };
          e === null ? e = [V] : e.push(V), r++;
        }
        var u = $ === r;
        u || (y = false);
      }
    } else {
      let y = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: Ko }, message: "must be array", schema: Ko.type, parentSchema: Ko, data: l };
      e === null ? e = [y] : e.push(y), r++;
    }
    return so.errors = e, r === 0;
  }
  B["base-website"] = hc;
  var O = { type: "object", description: "be an object", properties: { title: { type: "string", description: "be a string" }, description: { type: "string", description: "be a string" }, "fav-icon": { type: "string", description: "be a string" }, "site-url": { type: "string", description: "be a string" }, "site-path": { type: "string", description: "be a string" }, "repo-url": { type: "string", description: "be a string" }, "repo-branch": { type: "string", description: "be a string" }, "repo-actions": { oneOf: [{ enum: ["none", "edit", "source", "issue"], description: "be one of: `none`, `edit`, `source`, `issue`" }, { type: "array", description: "be an array of values, where each element should be one of: `none`, `edit`, `source`, `issue`", items: { enum: ["none", "edit", "source", "issue"], description: "be one of: `none`, `edit`, `source`, `issue`" } }], description: "be exactly one of: one of: `none`, `edit`, `source`, `issue`, an array of values, where each element should be one of: `none`, `edit`, `source`, `issue`" }, "google-analytics": { anyOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: { "tracking-id": { type: "string", description: "be a string" }, storage: { enum: ["cookies", "none"], description: "be one of: `cookies`, `none`" }, "anonymize-ip": { type: "boolean", description: "be `true` or `false`" }, version: { enum: [3, 4], description: "be one of: `3`, `4`" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tracking_id|trackingId|anonymize_ip|anonymizeIp))" } }], description: "be at least one of: a string, an object" }, "cookie-consent": { anyOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { type: { enum: ["implied", "express"], description: "be one of: `implied`, `express`" }, style: { enum: ["simple", "headline", "interstitial", "standalone"], description: "be one of: `simple`, `headline`, `interstitial`, `standalone`" }, palette: { enum: ["light", "dark"], description: "be one of: `light`, `dark`" }, "policy-url": { type: "string", description: "be a string" }, "prefs-text": { type: "string", description: "be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(policy_url|policyUrl|prefs_text|prefsText))" } }], description: "be at least one of: `true` or `false`, an object" }, search: { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { location: { enum: ["navbar", "sidebar"], description: "be one of: `navbar`, `sidebar`" }, type: { enum: ["overlay", "textbox"], description: "be one of: `overlay`, `textbox`" }, limit: { type: "number", description: "be a number" }, "collapse-after": { type: "number", description: "be a number" }, "copy-button": { type: "boolean", description: "be `true` or `false`" }, algolia: { type: "object", description: "be an object", properties: { "index-name": { type: "string", description: "be a string" }, "application-id": { type: "string", description: "be a string" }, "search-only-api-key": { type: "string", description: "be a string" }, "analytics-events": { type: "boolean", description: "be `true` or `false`" }, "show-logo": { type: "boolean", description: "be `true` or `false`" }, "index-fields": { type: "object", description: "be an object", properties: { href: { type: "string", description: "be a string" }, title: { type: "string", description: "be a string" }, text: { type: "string", description: "be a string" }, section: { type: "string", description: "be a string" } }, patternProperties: {}, additionalProperties: false }, params: { type: "object", description: "be an object", properties: {}, patternProperties: {} } }, patternProperties: {}, additionalProperties: false, propertyNames: { type: "string", pattern: "^(?!(index_name|indexName|application_id|applicationId|search_only_api_key|searchOnlyApiKey|analytics_events|analyticsEvents|show_logo|showLogo|index_fields|indexFields))" } } }, patternProperties: {}, additionalProperties: false, propertyNames: { type: "string", pattern: "^(?!(collapse_after|collapseAfter|copy_button|copyButton))" } }], description: "be exactly one of: `true` or `false`, an object" }, navbar: { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { title: { oneOf: [{ type: "string", description: "be a string" }, { type: "boolean", description: "be `true` or `false`" }], description: "be exactly one of: a string, `true` or `false`" }, logo: { type: "string", description: "be a string" }, background: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, foreground: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, search: { type: "boolean", description: "be `true` or `false`" }, pinned: { type: "boolean", description: "be `true` or `false`" }, collapse: { type: "boolean", description: "be `true` or `false`" }, "collapse-below": { enum: ["sm", "md", "lg", "xl", "xxl"], description: "be one of: `sm`, `md`, `lg`, `xl`, `xxl`" }, left: { type: "array", description: "be an array of values, where each element should be navigation-item", items: { $ref: "navigation-item", description: "be navigation-item" } }, right: { type: "array", description: "be an array of values, where each element should be navigation-item", items: { $ref: "navigation-item", description: "be navigation-item" } } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(collapse_below|collapseBelow))" } }], description: "be exactly one of: `true` or `false`, an object" }, sidebar: { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { oneOf: [{ type: "object", description: "be an object", properties: { id: { type: "string", description: "be a string" }, title: { oneOf: [{ type: "string", description: "be a string" }, { type: "boolean", description: "be `true` or `false`" }], description: "be exactly one of: a string, `true` or `false`" }, subtitle: { type: "string", description: "be a string" }, logo: { type: "string", description: "be a string" }, search: { type: "boolean", description: "be `true` or `false`" }, tools: { type: "array", description: "be an array of values, where each element should be tool-item", items: { $ref: "tool-item", description: "be tool-item" } }, contents: { $ref: "sidebar-contents", description: "be sidebar-contents" }, style: { enum: ["docked", "floating"], description: "be one of: `docked`, `floating`" }, background: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, foreground: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, border: { type: "boolean", description: "be `true` or `false`" }, alignment: { enum: ["left", "right", "center"], description: "be one of: `left`, `right`, `center`" }, "collapse-level": { type: "number", description: "be a number" }, pinned: { type: "boolean", description: "be `true` or `false`" }, header: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, footer: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(collapse_level|collapseLevel))" } }, { type: "array", description: "be an array of values, where each element should be an object", items: { type: "object", description: "be an object", properties: { id: { type: "string", description: "be a string" }, title: { oneOf: [{ type: "string", description: "be a string" }, { type: "boolean", description: "be `true` or `false`" }], description: "be exactly one of: a string, `true` or `false`" }, subtitle: { type: "string", description: "be a string" }, logo: { type: "string", description: "be a string" }, search: { type: "boolean", description: "be `true` or `false`" }, tools: { type: "array", description: "be an array of values, where each element should be tool-item", items: { $ref: "tool-item", description: "be tool-item" } }, contents: { $ref: "sidebar-contents", description: "be sidebar-contents" }, style: { enum: ["docked", "floating"], description: "be one of: `docked`, `floating`" }, background: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, foreground: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, border: { type: "boolean", description: "be `true` or `false`" }, alignment: { enum: ["left", "right", "center"], description: "be one of: `left`, `right`, `center`" }, "collapse-level": { type: "number", description: "be a number" }, pinned: { type: "boolean", description: "be `true` or `false`" }, header: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, footer: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(collapse_level|collapseLevel))" } } }], description: "be exactly one of: an object, an array of values, where each element should be an object" }], description: "be exactly one of: `true` or `false`, exactly one of: an object, an array of values, where each element should be an object" }, "body-header": { type: "string", description: "be a string" }, "body-footer": { type: "string", description: "be a string" }, "margin-header": { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, "margin-footer": { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, "page-navigation": { type: "boolean", description: "be `true` or `false`" }, "page-footer": { oneOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: { left: { $ref: "page-footer-region", description: "be page-footer-region" }, right: { $ref: "page-footer-region", description: "be page-footer-region" }, center: { $ref: "page-footer-region", description: "be page-footer-region" }, border: { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "string", description: "be a string" }], description: "be exactly one of: `true` or `false`, a string" }, background: { type: "string", description: "be a string" }, foreground: { type: "string", description: "be a string" } }, patternProperties: {}, additionalProperties: false }], description: "be exactly one of: a string, an object" }, image: { type: "string", description: "be a string" }, comments: { $ref: "comments", description: "be comments" }, "open-graph": { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { title: { type: "string", description: "be a string" }, description: { type: "string", description: "be a string" }, image: { type: "string", description: "be a string" }, "image-width": { type: "number", description: "be a number" }, "image-height": { type: "number", description: "be a number" }, locale: { type: "string", description: "be a string" }, "site-name": { type: "string", description: "be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(^(?!^(?!(site_name|siteName)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))" }, required: [], additionalProperties: false }], description: "be exactly one of: `true` or `false`, an object" }, "twitter-card": { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { title: { type: "string", description: "be a string" }, description: { type: "string", description: "be a string" }, image: { type: "string", description: "be a string" }, "image-width": { type: "number", description: "be a number" }, "image-height": { type: "number", description: "be a number" }, "card-style": { enum: ["summary", "summary_large_image"], description: "be one of: `summary`, `summary_large_image`" }, creator: { type: "string", description: "be a string" }, site: { type: "string", description: "be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(^(?!^(?!(card_style|cardStyle)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))" }, required: [], additionalProperties: false }], description: "be exactly one of: `true` or `false`, an object" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(fav_icon|favIcon|site_url|siteUrl|site_path|sitePath|repo_url|repoUrl|repo_branch|repoBranch|repo_actions|repoActions|google_analytics|googleAnalytics|cookie_consent|cookieConsent|body_header|bodyHeader|body_footer|bodyFooter|margin_header|marginHeader|margin_footer|marginFooter|page_navigation|pageNavigation|page_footer|pageFooter|open_graph|openGraph|twitter_card|twitterCard))" }, $id: "base-website" }, x0 = new RegExp("^(?!(fav_icon|favIcon|site_url|siteUrl|site_path|sitePath|repo_url|repoUrl|repo_branch|repoBranch|repo_actions|repoActions|google_analytics|googleAnalytics|cookie_consent|cookieConsent|body_header|bodyHeader|body_footer|bodyFooter|margin_header|marginHeader|margin_footer|marginFooter|page_navigation|pageNavigation|page_footer|pageFooter|open_graph|openGraph|twitter_card|twitterCard))", "u"), tg = new RegExp("^(?!(tracking_id|trackingId|anonymize_ip|anonymizeIp))", "u"), ag = new RegExp("^(?!(policy_url|policyUrl|prefs_text|prefsText))", "u"), ng = new RegExp("^(?!(collapse_after|collapseAfter|copy_button|copyButton))", "u"), og = new RegExp("^(?!(index_name|indexName|application_id|applicationId|search_only_api_key|searchOnlyApiKey|analytics_events|analyticsEvents|show_logo|showLogo|index_fields|indexFields))", "u"), sg = new RegExp("^(?!(collapse_below|collapseBelow))", "u"), vd = new RegExp("^(?!(collapse_level|collapseLevel))", "u"), ig = new RegExp("^(?!(^(?!^(?!(site_name|siteName)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))", "u"), lg = new RegExp("^(?!(^(?!^(?!(card_style|cardStyle)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))", "u");
  function hc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let Q in l) {
        let ba = r, $r = r;
        if (typeof Q == "string") {
          if (!x0.test(Q)) {
            let kt = { instancePath: o, schemaPath: "#/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(fav_icon|favIcon|site_url|siteUrl|site_path|sitePath|repo_url|repoUrl|repo_branch|repoBranch|repo_actions|repoActions|google_analytics|googleAnalytics|cookie_consent|cookieConsent|body_header|bodyHeader|body_footer|bodyFooter|margin_header|marginHeader|margin_footer|marginFooter|page_navigation|pageNavigation|page_footer|pageFooter|open_graph|openGraph|twitter_card|twitterCard))", schema: O.propertyNames }, message: 'must match pattern "^(?!(fav_icon|favIcon|site_url|siteUrl|site_path|sitePath|repo_url|repoUrl|repo_branch|repoBranch|repo_actions|repoActions|google_analytics|googleAnalytics|cookie_consent|cookieConsent|body_header|bodyHeader|body_footer|bodyFooter|margin_header|marginHeader|margin_footer|marginFooter|page_navigation|pageNavigation|page_footer|pageFooter|open_graph|openGraph|twitter_card|twitterCard))"', schema: "^(?!(fav_icon|favIcon|site_url|siteUrl|site_path|sitePath|repo_url|repoUrl|repo_branch|repoBranch|repo_actions|repoActions|google_analytics|googleAnalytics|cookie_consent|cookieConsent|body_header|bodyHeader|body_footer|bodyFooter|margin_header|marginHeader|margin_footer|marginFooter|page_navigation|pageNavigation|page_footer|pageFooter|open_graph|openGraph|twitter_card|twitterCard))", parentSchema: O.propertyNames, data: Q, propertyName: Q };
            e === null ? e = [kt] : e.push(kt), r++;
          }
        } else {
          let kt = { instancePath: o, schemaPath: "#/propertyNames/type", keyword: "type", params: { type: "string", schema: O.propertyNames }, message: "must be string", schema: O.propertyNames.type, parentSchema: O.propertyNames, data: Q, propertyName: Q };
          e === null ? e = [kt] : e.push(kt), r++;
        }
        var c = ba === r;
        if (!c) {
          let kt = { instancePath: o, schemaPath: "#/propertyNames", keyword: "propertyNames", params: { propertyName: Q, schema: O }, message: "property name must be valid", schema: O.propertyNames, parentSchema: O, data: l };
          e === null ? e = [kt] : e.push(kt), r++;
        }
      }
      if (l.title !== void 0) {
        let Q = l.title, ba = r;
        if (typeof Q != "string") {
          let kt = { instancePath: o + "/title", schemaPath: "#/properties/title/type", keyword: "type", params: { type: "string", schema: O.properties.title }, message: "must be string", schema: O.properties.title.type, parentSchema: O.properties.title, data: Q };
          e === null ? e = [kt] : e.push(kt), r++;
        }
        let $r = r;
        var d = ba === r;
      }
      if (l.description !== void 0) {
        let Q = l.description, ba = r;
        if (typeof Q != "string") {
          let $t = { instancePath: o + "/description", schemaPath: "#/properties/description/type", keyword: "type", params: { type: "string", schema: O.properties.description }, message: "must be string", schema: O.properties.description.type, parentSchema: O.properties.description, data: Q };
          e === null ? e = [$t] : e.push($t), r++;
        }
        let $r = r;
        var d = ba === r;
      }
      if (l["fav-icon"] !== void 0) {
        let Q = l["fav-icon"], ba = r;
        if (typeof Q != "string") {
          let $t = { instancePath: o + "/fav-icon", schemaPath: "#/properties/fav-icon/type", keyword: "type", params: { type: "string", schema: O.properties["fav-icon"] }, message: "must be string", schema: O.properties["fav-icon"].type, parentSchema: O.properties["fav-icon"], data: Q };
          e === null ? e = [$t] : e.push($t), r++;
        }
        let $r = r;
        var d = ba === r;
      }
      if (l["site-url"] !== void 0) {
        let Q = l["site-url"], ba = r;
        if (typeof Q != "string") {
          let $t = { instancePath: o + "/site-url", schemaPath: "#/properties/site-url/type", keyword: "type", params: { type: "string", schema: O.properties["site-url"] }, message: "must be string", schema: O.properties["site-url"].type, parentSchema: O.properties["site-url"], data: Q };
          e === null ? e = [$t] : e.push($t), r++;
        }
        let $r = r;
        var d = ba === r;
      }
      if (l["site-path"] !== void 0) {
        let Q = l["site-path"], ba = r;
        if (typeof Q != "string") {
          let $t = { instancePath: o + "/site-path", schemaPath: "#/properties/site-path/type", keyword: "type", params: { type: "string", schema: O.properties["site-path"] }, message: "must be string", schema: O.properties["site-path"].type, parentSchema: O.properties["site-path"], data: Q };
          e === null ? e = [$t] : e.push($t), r++;
        }
        let $r = r;
        var d = ba === r;
      }
      if (l["repo-url"] !== void 0) {
        let Q = l["repo-url"], ba = r;
        if (typeof Q != "string") {
          let $t = { instancePath: o + "/repo-url", schemaPath: "#/properties/repo-url/type", keyword: "type", params: { type: "string", schema: O.properties["repo-url"] }, message: "must be string", schema: O.properties["repo-url"].type, parentSchema: O.properties["repo-url"], data: Q };
          e === null ? e = [$t] : e.push($t), r++;
        }
        let $r = r;
        var d = ba === r;
      }
      if (l["repo-branch"] !== void 0) {
        let Q = l["repo-branch"], ba = r;
        if (typeof Q != "string") {
          let $t = { instancePath: o + "/repo-branch", schemaPath: "#/properties/repo-branch/type", keyword: "type", params: { type: "string", schema: O.properties["repo-branch"] }, message: "must be string", schema: O.properties["repo-branch"].type, parentSchema: O.properties["repo-branch"], data: Q };
          e === null ? e = [$t] : e.push($t), r++;
        }
        let $r = r;
        var d = ba === r;
      }
      if (l["repo-actions"] !== void 0) {
        let Q = l["repo-actions"], ba = r, $r = r, kt = false, $t = null, Jr = r, io = O.properties["repo-actions"].oneOf[0].enum;
        if (!(Q === "none" || Q === "edit" || Q === "source" || Q === "issue")) {
          let Y = { instancePath: o + "/repo-actions", schemaPath: "#/properties/repo-actions/oneOf/0/enum", keyword: "enum", params: { allowedValues: O.properties["repo-actions"].oneOf[0].enum, schema: O.properties["repo-actions"].oneOf[0] }, message: "must be equal to one of the allowed values", schema: O.properties["repo-actions"].oneOf[0].enum, parentSchema: O.properties["repo-actions"].oneOf[0], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var u = Jr === r;
        u && (kt = true, $t = 0);
        let an = r, w = r;
        if (Array.isArray(Q)) {
          let Y = true, N = Q.length;
          for (let ae = 0; ae < N; ae++) {
            let nt = Q[ae], J = r, ot = O.properties["repo-actions"].oneOf[1].items.enum;
            if (!(nt === "none" || nt === "edit" || nt === "source" || nt === "issue")) {
              let le = { instancePath: o + "/repo-actions/" + ae, schemaPath: "#/properties/repo-actions/oneOf/1/items/enum", keyword: "enum", params: { allowedValues: O.properties["repo-actions"].oneOf[1].items.enum, schema: O.properties["repo-actions"].oneOf[1].items }, message: "must be equal to one of the allowed values", schema: O.properties["repo-actions"].oneOf[1].items.enum, parentSchema: O.properties["repo-actions"].oneOf[1].items, data: nt };
              e === null ? e = [le] : e.push(le), r++;
            }
            var y = J === r;
            y || (Y = false);
          }
        } else {
          let Y = { instancePath: o + "/repo-actions", schemaPath: "#/properties/repo-actions/oneOf/1/type", keyword: "type", params: { type: "array", schema: O.properties["repo-actions"].oneOf[1] }, message: "must be array", schema: O.properties["repo-actions"].oneOf[1].type, parentSchema: O.properties["repo-actions"].oneOf[1], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var u = an === r;
        if (u && kt ? (kt = false, $t = [$t, 1]) : u && (kt = true, $t = 1), kt)
          r = $r, e !== null && ($r ? e.length = $r : e = null);
        else {
          let Y = { instancePath: o + "/repo-actions", schemaPath: "#/properties/repo-actions/oneOf", keyword: "oneOf", params: { passingSchemas: $t, schema: O.properties["repo-actions"] }, message: "must match exactly one schema in oneOf", schema: O.properties["repo-actions"].oneOf, parentSchema: O.properties["repo-actions"], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var d = ba === r;
      }
      if (l["google-analytics"] !== void 0) {
        let Q = l["google-analytics"], ba = r, $r = r, kt = false, $t = r;
        if (typeof Q != "string") {
          let an = { instancePath: o + "/google-analytics", schemaPath: "#/properties/google-analytics/anyOf/0/type", keyword: "type", params: { type: "string", schema: O.properties["google-analytics"].anyOf[0] }, message: "must be string", schema: O.properties["google-analytics"].anyOf[0].type, parentSchema: O.properties["google-analytics"].anyOf[0], data: Q };
          e === null ? e = [an] : e.push(an), r++;
        }
        let Jr = r;
        var g = $t === r;
        if (kt = kt || g, !kt) {
          let an = r, w = r;
          if (Q && typeof Q == "object" && !Array.isArray(Q)) {
            for (let Y in Q) {
              let N = r, ae = r;
              if (typeof Y == "string") {
                if (!tg.test(Y)) {
                  let nt = { instancePath: o + "/google-analytics", schemaPath: "#/properties/google-analytics/anyOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tracking_id|trackingId|anonymize_ip|anonymizeIp))", schema: O.properties["google-analytics"].anyOf[1].propertyNames }, message: 'must match pattern "^(?!(tracking_id|trackingId|anonymize_ip|anonymizeIp))"', schema: "^(?!(tracking_id|trackingId|anonymize_ip|anonymizeIp))", parentSchema: O.properties["google-analytics"].anyOf[1].propertyNames, data: Y, propertyName: Y };
                  e === null ? e = [nt] : e.push(nt), r++;
                }
              } else {
                let nt = { instancePath: o + "/google-analytics", schemaPath: "#/properties/google-analytics/anyOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: O.properties["google-analytics"].anyOf[1].propertyNames }, message: "must be string", schema: O.properties["google-analytics"].anyOf[1].propertyNames.type, parentSchema: O.properties["google-analytics"].anyOf[1].propertyNames, data: Y, propertyName: Y };
                e === null ? e = [nt] : e.push(nt), r++;
              }
              var x = N === r;
              if (!x) {
                let nt = { instancePath: o + "/google-analytics", schemaPath: "#/properties/google-analytics/anyOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: Y, schema: O.properties["google-analytics"].anyOf[1] }, message: "property name must be valid", schema: O.properties["google-analytics"].anyOf[1].propertyNames, parentSchema: O.properties["google-analytics"].anyOf[1], data: Q };
                e === null ? e = [nt] : e.push(nt), r++;
              }
            }
            if (Q["tracking-id"] !== void 0) {
              let Y = Q["tracking-id"], N = r;
              if (typeof Y != "string") {
                let nt = { instancePath: o + "/google-analytics/tracking-id", schemaPath: "#/properties/google-analytics/anyOf/1/properties/tracking-id/type", keyword: "type", params: { type: "string", schema: O.properties["google-analytics"].anyOf[1].properties["tracking-id"] }, message: "must be string", schema: O.properties["google-analytics"].anyOf[1].properties["tracking-id"].type, parentSchema: O.properties["google-analytics"].anyOf[1].properties["tracking-id"], data: Y };
                e === null ? e = [nt] : e.push(nt), r++;
              }
              let ae = r;
              var D = N === r;
            }
            if (Q.storage !== void 0) {
              let Y = Q.storage, N = r, ae = O.properties["google-analytics"].anyOf[1].properties.storage.enum;
              if (!(Y === "cookies" || Y === "none")) {
                let J = { instancePath: o + "/google-analytics/storage", schemaPath: "#/properties/google-analytics/anyOf/1/properties/storage/enum", keyword: "enum", params: { allowedValues: O.properties["google-analytics"].anyOf[1].properties.storage.enum, schema: O.properties["google-analytics"].anyOf[1].properties.storage }, message: "must be equal to one of the allowed values", schema: O.properties["google-analytics"].anyOf[1].properties.storage.enum, parentSchema: O.properties["google-analytics"].anyOf[1].properties.storage, data: Y };
                e === null ? e = [J] : e.push(J), r++;
              }
              var D = N === r;
            }
            if (Q["anonymize-ip"] !== void 0) {
              let Y = Q["anonymize-ip"], N = r;
              if (typeof Y != "boolean") {
                let J = { instancePath: o + "/google-analytics/anonymize-ip", schemaPath: "#/properties/google-analytics/anyOf/1/properties/anonymize-ip/type", keyword: "type", params: { type: "boolean", schema: O.properties["google-analytics"].anyOf[1].properties["anonymize-ip"] }, message: "must be boolean", schema: O.properties["google-analytics"].anyOf[1].properties["anonymize-ip"].type, parentSchema: O.properties["google-analytics"].anyOf[1].properties["anonymize-ip"], data: Y };
                e === null ? e = [J] : e.push(J), r++;
              }
              let ae = r;
              var D = N === r;
            }
            if (Q.version !== void 0) {
              let Y = Q.version, N = r, ae = O.properties["google-analytics"].anyOf[1].properties.version.enum;
              if (!(Y === 3 || Y === 4)) {
                let J = { instancePath: o + "/google-analytics/version", schemaPath: "#/properties/google-analytics/anyOf/1/properties/version/enum", keyword: "enum", params: { allowedValues: O.properties["google-analytics"].anyOf[1].properties.version.enum, schema: O.properties["google-analytics"].anyOf[1].properties.version }, message: "must be equal to one of the allowed values", schema: O.properties["google-analytics"].anyOf[1].properties.version.enum, parentSchema: O.properties["google-analytics"].anyOf[1].properties.version, data: Y };
                e === null ? e = [J] : e.push(J), r++;
              }
              var D = N === r;
            }
          } else {
            let Y = { instancePath: o + "/google-analytics", schemaPath: "#/properties/google-analytics/anyOf/1/type", keyword: "type", params: { type: "object", schema: O.properties["google-analytics"].anyOf[1] }, message: "must be object", schema: O.properties["google-analytics"].anyOf[1].type, parentSchema: O.properties["google-analytics"].anyOf[1], data: Q };
            e === null ? e = [Y] : e.push(Y), r++;
          }
          var g = an === r;
          kt = kt || g;
        }
        if (kt)
          r = $r, e !== null && ($r ? e.length = $r : e = null);
        else {
          let an = { instancePath: o + "/google-analytics", schemaPath: "#/properties/google-analytics/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: O.properties["google-analytics"].anyOf, parentSchema: O.properties["google-analytics"], data: Q };
          e === null ? e = [an] : e.push(an), r++;
        }
        var d = ba === r;
      }
      if (l["cookie-consent"] !== void 0) {
        let Q = l["cookie-consent"], ba = r, $r = r, kt = false, $t = r;
        if (typeof Q != "boolean") {
          let an = { instancePath: o + "/cookie-consent", schemaPath: "#/properties/cookie-consent/anyOf/0/type", keyword: "type", params: { type: "boolean", schema: O.properties["cookie-consent"].anyOf[0] }, message: "must be boolean", schema: O.properties["cookie-consent"].anyOf[0].type, parentSchema: O.properties["cookie-consent"].anyOf[0], data: Q };
          e === null ? e = [an] : e.push(an), r++;
        }
        let Jr = r;
        var $ = $t === r;
        if (kt = kt || $, !kt) {
          let an = r, w = r;
          if (Q && typeof Q == "object" && !Array.isArray(Q)) {
            for (let Y in Q) {
              let N = r, ae = r;
              if (typeof Y == "string") {
                if (!ag.test(Y)) {
                  let nt = { instancePath: o + "/cookie-consent", schemaPath: "#/properties/cookie-consent/anyOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(policy_url|policyUrl|prefs_text|prefsText))", schema: O.properties["cookie-consent"].anyOf[1].propertyNames }, message: 'must match pattern "^(?!(policy_url|policyUrl|prefs_text|prefsText))"', schema: "^(?!(policy_url|policyUrl|prefs_text|prefsText))", parentSchema: O.properties["cookie-consent"].anyOf[1].propertyNames, data: Y, propertyName: Y };
                  e === null ? e = [nt] : e.push(nt), r++;
                }
              } else {
                let nt = { instancePath: o + "/cookie-consent", schemaPath: "#/properties/cookie-consent/anyOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: O.properties["cookie-consent"].anyOf[1].propertyNames }, message: "must be string", schema: O.properties["cookie-consent"].anyOf[1].propertyNames.type, parentSchema: O.properties["cookie-consent"].anyOf[1].propertyNames, data: Y, propertyName: Y };
                e === null ? e = [nt] : e.push(nt), r++;
              }
              var m = N === r;
              if (!m) {
                let nt = { instancePath: o + "/cookie-consent", schemaPath: "#/properties/cookie-consent/anyOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: Y, schema: O.properties["cookie-consent"].anyOf[1] }, message: "property name must be valid", schema: O.properties["cookie-consent"].anyOf[1].propertyNames, parentSchema: O.properties["cookie-consent"].anyOf[1], data: Q };
                e === null ? e = [nt] : e.push(nt), r++;
              }
            }
            if (Q.type !== void 0) {
              let Y = Q.type, N = r, ae = O.properties["cookie-consent"].anyOf[1].properties.type.enum;
              if (!(Y === "implied" || Y === "express")) {
                let nt = { instancePath: o + "/cookie-consent/type", schemaPath: "#/properties/cookie-consent/anyOf/1/properties/type/enum", keyword: "enum", params: { allowedValues: O.properties["cookie-consent"].anyOf[1].properties.type.enum, schema: O.properties["cookie-consent"].anyOf[1].properties.type }, message: "must be equal to one of the allowed values", schema: O.properties["cookie-consent"].anyOf[1].properties.type.enum, parentSchema: O.properties["cookie-consent"].anyOf[1].properties.type, data: Y };
                e === null ? e = [nt] : e.push(nt), r++;
              }
              var R = N === r;
            }
            if (Q.style !== void 0) {
              let Y = Q.style, N = r, ae = O.properties["cookie-consent"].anyOf[1].properties.style.enum;
              if (!(Y === "simple" || Y === "headline" || Y === "interstitial" || Y === "standalone")) {
                let J = { instancePath: o + "/cookie-consent/style", schemaPath: "#/properties/cookie-consent/anyOf/1/properties/style/enum", keyword: "enum", params: { allowedValues: O.properties["cookie-consent"].anyOf[1].properties.style.enum, schema: O.properties["cookie-consent"].anyOf[1].properties.style }, message: "must be equal to one of the allowed values", schema: O.properties["cookie-consent"].anyOf[1].properties.style.enum, parentSchema: O.properties["cookie-consent"].anyOf[1].properties.style, data: Y };
                e === null ? e = [J] : e.push(J), r++;
              }
              var R = N === r;
            }
            if (Q.palette !== void 0) {
              let Y = Q.palette, N = r, ae = O.properties["cookie-consent"].anyOf[1].properties.palette.enum;
              if (!(Y === "light" || Y === "dark")) {
                let J = { instancePath: o + "/cookie-consent/palette", schemaPath: "#/properties/cookie-consent/anyOf/1/properties/palette/enum", keyword: "enum", params: { allowedValues: O.properties["cookie-consent"].anyOf[1].properties.palette.enum, schema: O.properties["cookie-consent"].anyOf[1].properties.palette }, message: "must be equal to one of the allowed values", schema: O.properties["cookie-consent"].anyOf[1].properties.palette.enum, parentSchema: O.properties["cookie-consent"].anyOf[1].properties.palette, data: Y };
                e === null ? e = [J] : e.push(J), r++;
              }
              var R = N === r;
            }
            if (Q["policy-url"] !== void 0) {
              let Y = Q["policy-url"], N = r;
              if (typeof Y != "string") {
                let J = { instancePath: o + "/cookie-consent/policy-url", schemaPath: "#/properties/cookie-consent/anyOf/1/properties/policy-url/type", keyword: "type", params: { type: "string", schema: O.properties["cookie-consent"].anyOf[1].properties["policy-url"] }, message: "must be string", schema: O.properties["cookie-consent"].anyOf[1].properties["policy-url"].type, parentSchema: O.properties["cookie-consent"].anyOf[1].properties["policy-url"], data: Y };
                e === null ? e = [J] : e.push(J), r++;
              }
              let ae = r;
              var R = N === r;
            }
            if (Q["prefs-text"] !== void 0) {
              let Y = Q["prefs-text"], N = r;
              if (typeof Y != "string") {
                let J = { instancePath: o + "/cookie-consent/prefs-text", schemaPath: "#/properties/cookie-consent/anyOf/1/properties/prefs-text/type", keyword: "type", params: { type: "string", schema: O.properties["cookie-consent"].anyOf[1].properties["prefs-text"] }, message: "must be string", schema: O.properties["cookie-consent"].anyOf[1].properties["prefs-text"].type, parentSchema: O.properties["cookie-consent"].anyOf[1].properties["prefs-text"], data: Y };
                e === null ? e = [J] : e.push(J), r++;
              }
              let ae = r;
              var R = N === r;
            }
          } else {
            let Y = { instancePath: o + "/cookie-consent", schemaPath: "#/properties/cookie-consent/anyOf/1/type", keyword: "type", params: { type: "object", schema: O.properties["cookie-consent"].anyOf[1] }, message: "must be object", schema: O.properties["cookie-consent"].anyOf[1].type, parentSchema: O.properties["cookie-consent"].anyOf[1], data: Q };
            e === null ? e = [Y] : e.push(Y), r++;
          }
          var $ = an === r;
          kt = kt || $;
        }
        if (kt)
          r = $r, e !== null && ($r ? e.length = $r : e = null);
        else {
          let an = { instancePath: o + "/cookie-consent", schemaPath: "#/properties/cookie-consent/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: O.properties["cookie-consent"].anyOf, parentSchema: O.properties["cookie-consent"], data: Q };
          e === null ? e = [an] : e.push(an), r++;
        }
        var d = ba === r;
      }
      if (l.search !== void 0) {
        let Q = l.search, ba = r, $r = r, kt = false, $t = null, Jr = r;
        if (typeof Q != "boolean") {
          let Y = { instancePath: o + "/search", schemaPath: "#/properties/search/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: O.properties.search.oneOf[0] }, message: "must be boolean", schema: O.properties.search.oneOf[0].type, parentSchema: O.properties.search.oneOf[0], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        let io = r;
        var j = Jr === r;
        j && (kt = true, $t = 0);
        let an = r, w = r;
        if (Q && typeof Q == "object" && !Array.isArray(Q)) {
          for (let N in Q) {
            let ae = r, nt = r;
            if (typeof N == "string") {
              if (!ng.test(N)) {
                let J = { instancePath: o + "/search", schemaPath: "#/properties/search/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(collapse_after|collapseAfter|copy_button|copyButton))", schema: O.properties.search.oneOf[1].propertyNames }, message: 'must match pattern "^(?!(collapse_after|collapseAfter|copy_button|copyButton))"', schema: "^(?!(collapse_after|collapseAfter|copy_button|copyButton))", parentSchema: O.properties.search.oneOf[1].propertyNames, data: N, propertyName: N };
                e === null ? e = [J] : e.push(J), r++;
              }
            } else {
              let J = { instancePath: o + "/search", schemaPath: "#/properties/search/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: O.properties.search.oneOf[1].propertyNames }, message: "must be string", schema: O.properties.search.oneOf[1].propertyNames.type, parentSchema: O.properties.search.oneOf[1].propertyNames, data: N, propertyName: N };
              e === null ? e = [J] : e.push(J), r++;
            }
            var V = ae === r;
            if (!V) {
              let J = { instancePath: o + "/search", schemaPath: "#/properties/search/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: N, schema: O.properties.search.oneOf[1] }, message: "property name must be valid", schema: O.properties.search.oneOf[1].propertyNames, parentSchema: O.properties.search.oneOf[1], data: Q };
              e === null ? e = [J] : e.push(J), r++;
            }
          }
          let Y = r;
          for (let N in Q)
            if (!(N === "location" || N === "type" || N === "limit" || N === "collapse-after" || N === "copy-button" || N === "algolia")) {
              let ae = { instancePath: o + "/search", schemaPath: "#/properties/search/oneOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: N, schema: O.properties.search.oneOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: O.properties.search.oneOf[1], data: Q };
              e === null ? e = [ae] : e.push(ae), r++;
            }
          if (Q.location !== void 0) {
            let N = Q.location, ae = r, nt = O.properties.search.oneOf[1].properties.location.enum;
            if (!(N === "navbar" || N === "sidebar")) {
              let J = { instancePath: o + "/search/location", schemaPath: "#/properties/search/oneOf/1/properties/location/enum", keyword: "enum", params: { allowedValues: O.properties.search.oneOf[1].properties.location.enum, schema: O.properties.search.oneOf[1].properties.location }, message: "must be equal to one of the allowed values", schema: O.properties.search.oneOf[1].properties.location.enum, parentSchema: O.properties.search.oneOf[1].properties.location, data: N };
              e === null ? e = [J] : e.push(J), r++;
            }
            var se = ae === r;
          }
          if (Q.type !== void 0) {
            let N = Q.type, ae = r, nt = O.properties.search.oneOf[1].properties.type.enum;
            if (!(N === "overlay" || N === "textbox")) {
              let ot = { instancePath: o + "/search/type", schemaPath: "#/properties/search/oneOf/1/properties/type/enum", keyword: "enum", params: { allowedValues: O.properties.search.oneOf[1].properties.type.enum, schema: O.properties.search.oneOf[1].properties.type }, message: "must be equal to one of the allowed values", schema: O.properties.search.oneOf[1].properties.type.enum, parentSchema: O.properties.search.oneOf[1].properties.type, data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            var se = ae === r;
          }
          if (Q.limit !== void 0) {
            let N = Q.limit, ae = r;
            if (!(typeof N == "number" && isFinite(N))) {
              let ot = { instancePath: o + "/search/limit", schemaPath: "#/properties/search/oneOf/1/properties/limit/type", keyword: "type", params: { type: "number", schema: O.properties.search.oneOf[1].properties.limit }, message: "must be number", schema: O.properties.search.oneOf[1].properties.limit.type, parentSchema: O.properties.search.oneOf[1].properties.limit, data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var se = ae === r;
          }
          if (Q["collapse-after"] !== void 0) {
            let N = Q["collapse-after"], ae = r;
            if (!(typeof N == "number" && isFinite(N))) {
              let ot = { instancePath: o + "/search/collapse-after", schemaPath: "#/properties/search/oneOf/1/properties/collapse-after/type", keyword: "type", params: { type: "number", schema: O.properties.search.oneOf[1].properties["collapse-after"] }, message: "must be number", schema: O.properties.search.oneOf[1].properties["collapse-after"].type, parentSchema: O.properties.search.oneOf[1].properties["collapse-after"], data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var se = ae === r;
          }
          if (Q["copy-button"] !== void 0) {
            let N = Q["copy-button"], ae = r;
            if (typeof N != "boolean") {
              let ot = { instancePath: o + "/search/copy-button", schemaPath: "#/properties/search/oneOf/1/properties/copy-button/type", keyword: "type", params: { type: "boolean", schema: O.properties.search.oneOf[1].properties["copy-button"] }, message: "must be boolean", schema: O.properties.search.oneOf[1].properties["copy-button"].type, parentSchema: O.properties.search.oneOf[1].properties["copy-button"], data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var se = ae === r;
          }
          if (Q.algolia !== void 0) {
            let N = Q.algolia, ae = r, nt = r;
            if (N && typeof N == "object" && !Array.isArray(N)) {
              for (let le in N) {
                let pe = r, v = r;
                if (typeof le == "string") {
                  if (!og.test(le)) {
                    let E = { instancePath: o + "/search/algolia", schemaPath: "#/properties/search/oneOf/1/properties/algolia/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(index_name|indexName|application_id|applicationId|search_only_api_key|searchOnlyApiKey|analytics_events|analyticsEvents|show_logo|showLogo|index_fields|indexFields))", schema: O.properties.search.oneOf[1].properties.algolia.propertyNames }, message: 'must match pattern "^(?!(index_name|indexName|application_id|applicationId|search_only_api_key|searchOnlyApiKey|analytics_events|analyticsEvents|show_logo|showLogo|index_fields|indexFields))"', schema: "^(?!(index_name|indexName|application_id|applicationId|search_only_api_key|searchOnlyApiKey|analytics_events|analyticsEvents|show_logo|showLogo|index_fields|indexFields))", parentSchema: O.properties.search.oneOf[1].properties.algolia.propertyNames, data: le, propertyName: le };
                    e === null ? e = [E] : e.push(E), r++;
                  }
                } else {
                  let E = { instancePath: o + "/search/algolia", schemaPath: "#/properties/search/oneOf/1/properties/algolia/propertyNames/type", keyword: "type", params: { type: "string", schema: O.properties.search.oneOf[1].properties.algolia.propertyNames }, message: "must be string", schema: O.properties.search.oneOf[1].properties.algolia.propertyNames.type, parentSchema: O.properties.search.oneOf[1].properties.algolia.propertyNames, data: le, propertyName: le };
                  e === null ? e = [E] : e.push(E), r++;
                }
                var P = pe === r;
                if (!P) {
                  let E = { instancePath: o + "/search/algolia", schemaPath: "#/properties/search/oneOf/1/properties/algolia/propertyNames", keyword: "propertyNames", params: { propertyName: le, schema: O.properties.search.oneOf[1].properties.algolia }, message: "property name must be valid", schema: O.properties.search.oneOf[1].properties.algolia.propertyNames, parentSchema: O.properties.search.oneOf[1].properties.algolia, data: N };
                  e === null ? e = [E] : e.push(E), r++;
                }
              }
              let ot = r;
              for (let le in N)
                if (!(le === "index-name" || le === "application-id" || le === "search-only-api-key" || le === "analytics-events" || le === "show-logo" || le === "index-fields" || le === "params")) {
                  let pe = { instancePath: o + "/search/algolia", schemaPath: "#/properties/search/oneOf/1/properties/algolia/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: le, schema: O.properties.search.oneOf[1].properties.algolia }, message: "must NOT have additional properties", schema: false, parentSchema: O.properties.search.oneOf[1].properties.algolia, data: N };
                  e === null ? e = [pe] : e.push(pe), r++;
                }
              if (N["index-name"] !== void 0) {
                let le = N["index-name"], pe = r;
                if (typeof le != "string") {
                  let E = { instancePath: o + "/search/algolia/index-name", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/index-name/type", keyword: "type", params: { type: "string", schema: O.properties.search.oneOf[1].properties.algolia.properties["index-name"] }, message: "must be string", schema: O.properties.search.oneOf[1].properties.algolia.properties["index-name"].type, parentSchema: O.properties.search.oneOf[1].properties.algolia.properties["index-name"], data: le };
                  e === null ? e = [E] : e.push(E), r++;
                }
                let v = r;
                var oe = pe === r;
              }
              if (N["application-id"] !== void 0) {
                let le = N["application-id"], pe = r;
                if (typeof le != "string") {
                  let A = { instancePath: o + "/search/algolia/application-id", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/application-id/type", keyword: "type", params: { type: "string", schema: O.properties.search.oneOf[1].properties.algolia.properties["application-id"] }, message: "must be string", schema: O.properties.search.oneOf[1].properties.algolia.properties["application-id"].type, parentSchema: O.properties.search.oneOf[1].properties.algolia.properties["application-id"], data: le };
                  e === null ? e = [A] : e.push(A), r++;
                }
                let v = r;
                var oe = pe === r;
              }
              if (N["search-only-api-key"] !== void 0) {
                let le = N["search-only-api-key"], pe = r;
                if (typeof le != "string") {
                  let A = { instancePath: o + "/search/algolia/search-only-api-key", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/search-only-api-key/type", keyword: "type", params: { type: "string", schema: O.properties.search.oneOf[1].properties.algolia.properties["search-only-api-key"] }, message: "must be string", schema: O.properties.search.oneOf[1].properties.algolia.properties["search-only-api-key"].type, parentSchema: O.properties.search.oneOf[1].properties.algolia.properties["search-only-api-key"], data: le };
                  e === null ? e = [A] : e.push(A), r++;
                }
                let v = r;
                var oe = pe === r;
              }
              if (N["analytics-events"] !== void 0) {
                let le = N["analytics-events"], pe = r;
                if (typeof le != "boolean") {
                  let A = { instancePath: o + "/search/algolia/analytics-events", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/analytics-events/type", keyword: "type", params: { type: "boolean", schema: O.properties.search.oneOf[1].properties.algolia.properties["analytics-events"] }, message: "must be boolean", schema: O.properties.search.oneOf[1].properties.algolia.properties["analytics-events"].type, parentSchema: O.properties.search.oneOf[1].properties.algolia.properties["analytics-events"], data: le };
                  e === null ? e = [A] : e.push(A), r++;
                }
                let v = r;
                var oe = pe === r;
              }
              if (N["show-logo"] !== void 0) {
                let le = N["show-logo"], pe = r;
                if (typeof le != "boolean") {
                  let A = { instancePath: o + "/search/algolia/show-logo", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/show-logo/type", keyword: "type", params: { type: "boolean", schema: O.properties.search.oneOf[1].properties.algolia.properties["show-logo"] }, message: "must be boolean", schema: O.properties.search.oneOf[1].properties.algolia.properties["show-logo"].type, parentSchema: O.properties.search.oneOf[1].properties.algolia.properties["show-logo"], data: le };
                  e === null ? e = [A] : e.push(A), r++;
                }
                let v = r;
                var oe = pe === r;
              }
              if (N["index-fields"] !== void 0) {
                let le = N["index-fields"], pe = r, v = r;
                if (le && typeof le == "object" && !Array.isArray(le)) {
                  let A = r;
                  for (let Z in le)
                    if (!(Z === "href" || Z === "title" || Z === "text" || Z === "section")) {
                      let T = { instancePath: o + "/search/algolia/index-fields", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/index-fields/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: Z, schema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"] }, message: "must NOT have additional properties", schema: false, parentSchema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"], data: le };
                      e === null ? e = [T] : e.push(T), r++;
                    }
                  if (le.href !== void 0) {
                    let Z = le.href, T = r;
                    if (typeof Z != "string") {
                      let re = { instancePath: o + "/search/algolia/index-fields/href", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/index-fields/properties/href/type", keyword: "type", params: { type: "string", schema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.href }, message: "must be string", schema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.href.type, parentSchema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.href, data: Z };
                      e === null ? e = [re] : e.push(re), r++;
                    }
                    let U = r;
                    var h = T === r;
                  }
                  if (le.title !== void 0) {
                    let Z = le.title, T = r;
                    if (typeof Z != "string") {
                      let Fr = { instancePath: o + "/search/algolia/index-fields/title", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/index-fields/properties/title/type", keyword: "type", params: { type: "string", schema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.title }, message: "must be string", schema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.title.type, parentSchema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.title, data: Z };
                      e === null ? e = [Fr] : e.push(Fr), r++;
                    }
                    let U = r;
                    var h = T === r;
                  }
                  if (le.text !== void 0) {
                    let Z = le.text, T = r;
                    if (typeof Z != "string") {
                      let Fr = { instancePath: o + "/search/algolia/index-fields/text", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/index-fields/properties/text/type", keyword: "type", params: { type: "string", schema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.text }, message: "must be string", schema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.text.type, parentSchema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.text, data: Z };
                      e === null ? e = [Fr] : e.push(Fr), r++;
                    }
                    let U = r;
                    var h = T === r;
                  }
                  if (le.section !== void 0) {
                    let Z = le.section, T = r;
                    if (typeof Z != "string") {
                      let Fr = { instancePath: o + "/search/algolia/index-fields/section", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/index-fields/properties/section/type", keyword: "type", params: { type: "string", schema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.section }, message: "must be string", schema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.section.type, parentSchema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.section, data: Z };
                      e === null ? e = [Fr] : e.push(Fr), r++;
                    }
                    let U = r;
                    var h = T === r;
                  }
                } else {
                  let A = { instancePath: o + "/search/algolia/index-fields", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/index-fields/type", keyword: "type", params: { type: "object", schema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"] }, message: "must be object", schema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"].type, parentSchema: O.properties.search.oneOf[1].properties.algolia.properties["index-fields"], data: le };
                  e === null ? e = [A] : e.push(A), r++;
                }
                var oe = pe === r;
              }
              if (N.params !== void 0) {
                let le = N.params, pe = r, v = r;
                if (!(le && typeof le == "object" && !Array.isArray(le))) {
                  let A = { instancePath: o + "/search/algolia/params", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/params/type", keyword: "type", params: { type: "object", schema: O.properties.search.oneOf[1].properties.algolia.properties.params }, message: "must be object", schema: O.properties.search.oneOf[1].properties.algolia.properties.params.type, parentSchema: O.properties.search.oneOf[1].properties.algolia.properties.params, data: le };
                  e === null ? e = [A] : e.push(A), r++;
                }
                var oe = pe === r;
              }
            } else {
              let ot = { instancePath: o + "/search/algolia", schemaPath: "#/properties/search/oneOf/1/properties/algolia/type", keyword: "type", params: { type: "object", schema: O.properties.search.oneOf[1].properties.algolia }, message: "must be object", schema: O.properties.search.oneOf[1].properties.algolia.type, parentSchema: O.properties.search.oneOf[1].properties.algolia, data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            var se = ae === r;
          }
        } else {
          let Y = { instancePath: o + "/search", schemaPath: "#/properties/search/oneOf/1/type", keyword: "type", params: { type: "object", schema: O.properties.search.oneOf[1] }, message: "must be object", schema: O.properties.search.oneOf[1].type, parentSchema: O.properties.search.oneOf[1], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var j = an === r;
        if (j && kt ? (kt = false, $t = [$t, 1]) : j && (kt = true, $t = 1), kt)
          r = $r, e !== null && ($r ? e.length = $r : e = null);
        else {
          let Y = { instancePath: o + "/search", schemaPath: "#/properties/search/oneOf", keyword: "oneOf", params: { passingSchemas: $t, schema: O.properties.search }, message: "must match exactly one schema in oneOf", schema: O.properties.search.oneOf, parentSchema: O.properties.search, data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var d = ba === r;
      }
      if (l.navbar !== void 0) {
        let Q = l.navbar, ba = r, $r = r, kt = false, $t = null, Jr = r;
        if (typeof Q != "boolean") {
          let Y = { instancePath: o + "/navbar", schemaPath: "#/properties/navbar/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: O.properties.navbar.oneOf[0] }, message: "must be boolean", schema: O.properties.navbar.oneOf[0].type, parentSchema: O.properties.navbar.oneOf[0], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        let io = r;
        var f = Jr === r;
        f && (kt = true, $t = 0);
        let an = r, w = r;
        if (Q && typeof Q == "object" && !Array.isArray(Q)) {
          for (let Y in Q) {
            let N = r, ae = r;
            if (typeof Y == "string") {
              if (!sg.test(Y)) {
                let nt = { instancePath: o + "/navbar", schemaPath: "#/properties/navbar/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(collapse_below|collapseBelow))", schema: O.properties.navbar.oneOf[1].propertyNames }, message: 'must match pattern "^(?!(collapse_below|collapseBelow))"', schema: "^(?!(collapse_below|collapseBelow))", parentSchema: O.properties.navbar.oneOf[1].propertyNames, data: Y, propertyName: Y };
                e === null ? e = [nt] : e.push(nt), r++;
              }
            } else {
              let nt = { instancePath: o + "/navbar", schemaPath: "#/properties/navbar/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: O.properties.navbar.oneOf[1].propertyNames }, message: "must be string", schema: O.properties.navbar.oneOf[1].propertyNames.type, parentSchema: O.properties.navbar.oneOf[1].propertyNames, data: Y, propertyName: Y };
              e === null ? e = [nt] : e.push(nt), r++;
            }
            var ft = N === r;
            if (!ft) {
              let nt = { instancePath: o + "/navbar", schemaPath: "#/properties/navbar/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: Y, schema: O.properties.navbar.oneOf[1] }, message: "property name must be valid", schema: O.properties.navbar.oneOf[1].propertyNames, parentSchema: O.properties.navbar.oneOf[1], data: Q };
              e === null ? e = [nt] : e.push(nt), r++;
            }
          }
          if (Q.title !== void 0) {
            let Y = Q.title, N = r, ae = r, nt = false, J = null, ot = r;
            if (typeof Y != "string") {
              let E = { instancePath: o + "/navbar/title", schemaPath: "#/properties/navbar/oneOf/1/properties/title/oneOf/0/type", keyword: "type", params: { type: "string", schema: O.properties.navbar.oneOf[1].properties.title.oneOf[0] }, message: "must be string", schema: O.properties.navbar.oneOf[1].properties.title.oneOf[0].type, parentSchema: O.properties.navbar.oneOf[1].properties.title.oneOf[0], data: Y };
              e === null ? e = [E] : e.push(E), r++;
            }
            let le = r;
            var Cr = ot === r;
            Cr && (nt = true, J = 0);
            let pe = r;
            if (typeof Y != "boolean") {
              let E = { instancePath: o + "/navbar/title", schemaPath: "#/properties/navbar/oneOf/1/properties/title/oneOf/1/type", keyword: "type", params: { type: "boolean", schema: O.properties.navbar.oneOf[1].properties.title.oneOf[1] }, message: "must be boolean", schema: O.properties.navbar.oneOf[1].properties.title.oneOf[1].type, parentSchema: O.properties.navbar.oneOf[1].properties.title.oneOf[1], data: Y };
              e === null ? e = [E] : e.push(E), r++;
            }
            let v = r;
            var Cr = pe === r;
            if (Cr && nt ? (nt = false, J = [J, 1]) : Cr && (nt = true, J = 1), nt)
              r = ae, e !== null && (ae ? e.length = ae : e = null);
            else {
              let E = { instancePath: o + "/navbar/title", schemaPath: "#/properties/navbar/oneOf/1/properties/title/oneOf", keyword: "oneOf", params: { passingSchemas: J, schema: O.properties.navbar.oneOf[1].properties.title }, message: "must match exactly one schema in oneOf", schema: O.properties.navbar.oneOf[1].properties.title.oneOf, parentSchema: O.properties.navbar.oneOf[1].properties.title, data: Y };
              e === null ? e = [E] : e.push(E), r++;
            }
            var Dr = N === r;
          }
          if (Q.logo !== void 0) {
            let Y = Q.logo, N = r;
            if (typeof Y != "string") {
              let J = { instancePath: o + "/navbar/logo", schemaPath: "#/properties/navbar/oneOf/1/properties/logo/type", keyword: "type", params: { type: "string", schema: O.properties.navbar.oneOf[1].properties.logo }, message: "must be string", schema: O.properties.navbar.oneOf[1].properties.logo.type, parentSchema: O.properties.navbar.oneOf[1].properties.logo, data: Y };
              e === null ? e = [J] : e.push(J), r++;
            }
            let ae = r;
            var Dr = N === r;
          }
          if (Q.background !== void 0) {
            let Y = Q.background, N = r, ae = r, nt = false, J = r, ot = O.properties.navbar.oneOf[1].properties.background.anyOf[0].enum;
            if (!(Y === "primary" || Y === "secondary" || Y === "success" || Y === "danger" || Y === "warning" || Y === "info" || Y === "light" || Y === "dark")) {
              let pe = { instancePath: o + "/navbar/background", schemaPath: "#/properties/navbar/oneOf/1/properties/background/anyOf/0/enum", keyword: "enum", params: { allowedValues: O.properties.navbar.oneOf[1].properties.background.anyOf[0].enum, schema: O.properties.navbar.oneOf[1].properties.background.anyOf[0] }, message: "must be equal to one of the allowed values", schema: O.properties.navbar.oneOf[1].properties.background.anyOf[0].enum, parentSchema: O.properties.navbar.oneOf[1].properties.background.anyOf[0], data: Y };
              e === null ? e = [pe] : e.push(pe), r++;
            }
            var wr = J === r;
            if (nt = nt || wr, !nt) {
              let pe = r;
              if (typeof Y != "string") {
                let A = { instancePath: o + "/navbar/background", schemaPath: "#/properties/navbar/oneOf/1/properties/background/anyOf/1/type", keyword: "type", params: { type: "string", schema: O.properties.navbar.oneOf[1].properties.background.anyOf[1] }, message: "must be string", schema: O.properties.navbar.oneOf[1].properties.background.anyOf[1].type, parentSchema: O.properties.navbar.oneOf[1].properties.background.anyOf[1], data: Y };
                e === null ? e = [A] : e.push(A), r++;
              }
              let v = r;
              var wr = pe === r;
              nt = nt || wr;
            }
            if (nt)
              r = ae, e !== null && (ae ? e.length = ae : e = null);
            else {
              let pe = { instancePath: o + "/navbar/background", schemaPath: "#/properties/navbar/oneOf/1/properties/background/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: O.properties.navbar.oneOf[1].properties.background.anyOf, parentSchema: O.properties.navbar.oneOf[1].properties.background, data: Y };
              e === null ? e = [pe] : e.push(pe), r++;
            }
            var Dr = N === r;
          }
          if (Q.foreground !== void 0) {
            let Y = Q.foreground, N = r, ae = r, nt = false, J = r, ot = O.properties.navbar.oneOf[1].properties.foreground.anyOf[0].enum;
            if (!(Y === "primary" || Y === "secondary" || Y === "success" || Y === "danger" || Y === "warning" || Y === "info" || Y === "light" || Y === "dark")) {
              let pe = { instancePath: o + "/navbar/foreground", schemaPath: "#/properties/navbar/oneOf/1/properties/foreground/anyOf/0/enum", keyword: "enum", params: { allowedValues: O.properties.navbar.oneOf[1].properties.foreground.anyOf[0].enum, schema: O.properties.navbar.oneOf[1].properties.foreground.anyOf[0] }, message: "must be equal to one of the allowed values", schema: O.properties.navbar.oneOf[1].properties.foreground.anyOf[0].enum, parentSchema: O.properties.navbar.oneOf[1].properties.foreground.anyOf[0], data: Y };
              e === null ? e = [pe] : e.push(pe), r++;
            }
            var ta = J === r;
            if (nt = nt || ta, !nt) {
              let pe = r;
              if (typeof Y != "string") {
                let A = { instancePath: o + "/navbar/foreground", schemaPath: "#/properties/navbar/oneOf/1/properties/foreground/anyOf/1/type", keyword: "type", params: { type: "string", schema: O.properties.navbar.oneOf[1].properties.foreground.anyOf[1] }, message: "must be string", schema: O.properties.navbar.oneOf[1].properties.foreground.anyOf[1].type, parentSchema: O.properties.navbar.oneOf[1].properties.foreground.anyOf[1], data: Y };
                e === null ? e = [A] : e.push(A), r++;
              }
              let v = r;
              var ta = pe === r;
              nt = nt || ta;
            }
            if (nt)
              r = ae, e !== null && (ae ? e.length = ae : e = null);
            else {
              let pe = { instancePath: o + "/navbar/foreground", schemaPath: "#/properties/navbar/oneOf/1/properties/foreground/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: O.properties.navbar.oneOf[1].properties.foreground.anyOf, parentSchema: O.properties.navbar.oneOf[1].properties.foreground, data: Y };
              e === null ? e = [pe] : e.push(pe), r++;
            }
            var Dr = N === r;
          }
          if (Q.search !== void 0) {
            let Y = Q.search, N = r;
            if (typeof Y != "boolean") {
              let J = { instancePath: o + "/navbar/search", schemaPath: "#/properties/navbar/oneOf/1/properties/search/type", keyword: "type", params: { type: "boolean", schema: O.properties.navbar.oneOf[1].properties.search }, message: "must be boolean", schema: O.properties.navbar.oneOf[1].properties.search.type, parentSchema: O.properties.navbar.oneOf[1].properties.search, data: Y };
              e === null ? e = [J] : e.push(J), r++;
            }
            let ae = r;
            var Dr = N === r;
          }
          if (Q.pinned !== void 0) {
            let Y = Q.pinned, N = r;
            if (typeof Y != "boolean") {
              let J = { instancePath: o + "/navbar/pinned", schemaPath: "#/properties/navbar/oneOf/1/properties/pinned/type", keyword: "type", params: { type: "boolean", schema: O.properties.navbar.oneOf[1].properties.pinned }, message: "must be boolean", schema: O.properties.navbar.oneOf[1].properties.pinned.type, parentSchema: O.properties.navbar.oneOf[1].properties.pinned, data: Y };
              e === null ? e = [J] : e.push(J), r++;
            }
            let ae = r;
            var Dr = N === r;
          }
          if (Q.collapse !== void 0) {
            let Y = Q.collapse, N = r;
            if (typeof Y != "boolean") {
              let J = { instancePath: o + "/navbar/collapse", schemaPath: "#/properties/navbar/oneOf/1/properties/collapse/type", keyword: "type", params: { type: "boolean", schema: O.properties.navbar.oneOf[1].properties.collapse }, message: "must be boolean", schema: O.properties.navbar.oneOf[1].properties.collapse.type, parentSchema: O.properties.navbar.oneOf[1].properties.collapse, data: Y };
              e === null ? e = [J] : e.push(J), r++;
            }
            let ae = r;
            var Dr = N === r;
          }
          if (Q["collapse-below"] !== void 0) {
            let Y = Q["collapse-below"], N = r, ae = O.properties.navbar.oneOf[1].properties["collapse-below"].enum;
            if (!(Y === "sm" || Y === "md" || Y === "lg" || Y === "xl" || Y === "xxl")) {
              let J = { instancePath: o + "/navbar/collapse-below", schemaPath: "#/properties/navbar/oneOf/1/properties/collapse-below/enum", keyword: "enum", params: { allowedValues: O.properties.navbar.oneOf[1].properties["collapse-below"].enum, schema: O.properties.navbar.oneOf[1].properties["collapse-below"] }, message: "must be equal to one of the allowed values", schema: O.properties.navbar.oneOf[1].properties["collapse-below"].enum, parentSchema: O.properties.navbar.oneOf[1].properties["collapse-below"], data: Y };
              e === null ? e = [J] : e.push(J), r++;
            }
            var Dr = N === r;
          }
          if (Q.left !== void 0) {
            let Y = Q.left, N = r, ae = r;
            if (Array.isArray(Y)) {
              let J = true, ot = Y.length;
              for (let le = 0; le < ot; le++) {
                let pe = Y[le], v = r;
                wa(pe, { instancePath: o + "/navbar/left/" + le, parentData: Y, parentDataProperty: le, rootData: s }) || (e = e === null ? wa.errors : e.concat(wa.errors), r = e.length);
                var jt = v === r;
                jt || (J = false);
              }
            } else {
              let J = { instancePath: o + "/navbar/left", schemaPath: "#/properties/navbar/oneOf/1/properties/left/type", keyword: "type", params: { type: "array", schema: O.properties.navbar.oneOf[1].properties.left }, message: "must be array", schema: O.properties.navbar.oneOf[1].properties.left.type, parentSchema: O.properties.navbar.oneOf[1].properties.left, data: Y };
              e === null ? e = [J] : e.push(J), r++;
            }
            var Dr = N === r;
          }
          if (Q.right !== void 0) {
            let Y = Q.right, N = r, ae = r;
            if (Array.isArray(Y)) {
              let J = true, ot = Y.length;
              for (let le = 0; le < ot; le++) {
                let pe = Y[le], v = r;
                wa(pe, { instancePath: o + "/navbar/right/" + le, parentData: Y, parentDataProperty: le, rootData: s }) || (e = e === null ? wa.errors : e.concat(wa.errors), r = e.length);
                var ee = v === r;
                ee || (J = false);
              }
            } else {
              let J = { instancePath: o + "/navbar/right", schemaPath: "#/properties/navbar/oneOf/1/properties/right/type", keyword: "type", params: { type: "array", schema: O.properties.navbar.oneOf[1].properties.right }, message: "must be array", schema: O.properties.navbar.oneOf[1].properties.right.type, parentSchema: O.properties.navbar.oneOf[1].properties.right, data: Y };
              e === null ? e = [J] : e.push(J), r++;
            }
            var Dr = N === r;
          }
        } else {
          let Y = { instancePath: o + "/navbar", schemaPath: "#/properties/navbar/oneOf/1/type", keyword: "type", params: { type: "object", schema: O.properties.navbar.oneOf[1] }, message: "must be object", schema: O.properties.navbar.oneOf[1].type, parentSchema: O.properties.navbar.oneOf[1], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var f = an === r;
        if (f && kt ? (kt = false, $t = [$t, 1]) : f && (kt = true, $t = 1), kt)
          r = $r, e !== null && ($r ? e.length = $r : e = null);
        else {
          let Y = { instancePath: o + "/navbar", schemaPath: "#/properties/navbar/oneOf", keyword: "oneOf", params: { passingSchemas: $t, schema: O.properties.navbar }, message: "must match exactly one schema in oneOf", schema: O.properties.navbar.oneOf, parentSchema: O.properties.navbar, data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var d = ba === r;
      }
      if (l.sidebar !== void 0) {
        let Q = l.sidebar, ba = r, $r = r, kt = false, $t = null, Jr = r;
        if (typeof Q != "boolean") {
          let le = { instancePath: o + "/sidebar", schemaPath: "#/properties/sidebar/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: O.properties.sidebar.oneOf[0] }, message: "must be boolean", schema: O.properties.sidebar.oneOf[0].type, parentSchema: O.properties.sidebar.oneOf[0], data: Q };
          e === null ? e = [le] : e.push(le), r++;
        }
        let io = r;
        var za = Jr === r;
        za && (kt = true, $t = 0);
        let an = r, w = r, Nt = false, Y = null, N = r, ae = r;
        if (Q && typeof Q == "object" && !Array.isArray(Q)) {
          for (let le in Q) {
            let pe = r, v = r;
            if (typeof le == "string") {
              if (!vd.test(le)) {
                let E = { instancePath: o + "/sidebar", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(collapse_level|collapseLevel))", schema: O.properties.sidebar.oneOf[1].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(collapse_level|collapseLevel))"', schema: "^(?!(collapse_level|collapseLevel))", parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].propertyNames, data: le, propertyName: le };
                e === null ? e = [E] : e.push(E), r++;
              }
            } else {
              let E = { instancePath: o + "/sidebar", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[0].propertyNames }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[0].propertyNames.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].propertyNames, data: le, propertyName: le };
              e === null ? e = [E] : e.push(E), r++;
            }
            var sa = pe === r;
            if (!sa) {
              let E = { instancePath: o + "/sidebar", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: le, schema: O.properties.sidebar.oneOf[1].oneOf[0] }, message: "property name must be valid", schema: O.properties.sidebar.oneOf[1].oneOf[0].propertyNames, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0], data: Q };
              e === null ? e = [E] : e.push(E), r++;
            }
          }
          if (Q.id !== void 0) {
            let le = Q.id, pe = r;
            if (typeof le != "string") {
              let E = { instancePath: o + "/sidebar/id", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/id/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.id }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.id.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.id, data: le };
              e === null ? e = [E] : e.push(E), r++;
            }
            let v = r;
            var _e = pe === r;
          }
          if (Q.title !== void 0) {
            let le = Q.title, pe = r, v = r, E = false, A = null, Z = r;
            if (typeof le != "string") {
              let z = { instancePath: o + "/sidebar/title", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/title/oneOf/0/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[0] }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[0].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[0], data: le };
              e === null ? e = [z] : e.push(z), r++;
            }
            let T = r;
            var Ma = Z === r;
            Ma && (E = true, A = 0);
            let U = r;
            if (typeof le != "boolean") {
              let z = { instancePath: o + "/sidebar/title", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/title/oneOf/1/type", keyword: "type", params: { type: "boolean", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[1] }, message: "must be boolean", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[1].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[1], data: le };
              e === null ? e = [z] : e.push(z), r++;
            }
            let re = r;
            var Ma = U === r;
            if (Ma && E ? (E = false, A = [A, 1]) : Ma && (E = true, A = 1), E)
              r = v, e !== null && (v ? e.length = v : e = null);
            else {
              let z = { instancePath: o + "/sidebar/title", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/title/oneOf", keyword: "oneOf", params: { passingSchemas: A, schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.title }, message: "must match exactly one schema in oneOf", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.title, data: le };
              e === null ? e = [z] : e.push(z), r++;
            }
            var _e = pe === r;
          }
          if (Q.subtitle !== void 0) {
            let le = Q.subtitle, pe = r;
            if (typeof le != "string") {
              let A = { instancePath: o + "/sidebar/subtitle", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/subtitle/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.subtitle }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.subtitle.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.subtitle, data: le };
              e === null ? e = [A] : e.push(A), r++;
            }
            let v = r;
            var _e = pe === r;
          }
          if (Q.logo !== void 0) {
            let le = Q.logo, pe = r;
            if (typeof le != "string") {
              let A = { instancePath: o + "/sidebar/logo", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/logo/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.logo }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.logo.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.logo, data: le };
              e === null ? e = [A] : e.push(A), r++;
            }
            let v = r;
            var _e = pe === r;
          }
          if (Q.search !== void 0) {
            let le = Q.search, pe = r;
            if (typeof le != "boolean") {
              let A = { instancePath: o + "/sidebar/search", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/search/type", keyword: "type", params: { type: "boolean", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.search }, message: "must be boolean", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.search.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.search, data: le };
              e === null ? e = [A] : e.push(A), r++;
            }
            let v = r;
            var _e = pe === r;
          }
          if (Q.tools !== void 0) {
            let le = Q.tools, pe = r, v = r;
            if (Array.isArray(le)) {
              let A = true, Z = le.length;
              for (let T = 0; T < Z; T++) {
                let U = le[T], re = r;
                co(U, { instancePath: o + "/sidebar/tools/" + T, parentData: le, parentDataProperty: T, rootData: s }) || (e = e === null ? co.errors : e.concat(co.errors), r = e.length);
                var fa = re === r;
                fa || (A = false);
              }
            } else {
              let A = { instancePath: o + "/sidebar/tools", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/tools/type", keyword: "type", params: { type: "array", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.tools }, message: "must be array", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.tools.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.tools, data: le };
              e === null ? e = [A] : e.push(A), r++;
            }
            var _e = pe === r;
          }
          if (Q.contents !== void 0) {
            let le = Q.contents, pe = r;
            so(le, { instancePath: o + "/sidebar/contents", parentData: Q, parentDataProperty: "contents", rootData: s }) || (e = e === null ? so.errors : e.concat(so.errors), r = e.length);
            var _e = pe === r;
          }
          if (Q.style !== void 0) {
            let le = Q.style, pe = r, v = O.properties.sidebar.oneOf[1].oneOf[0].properties.style.enum;
            if (!(le === "docked" || le === "floating")) {
              let A = { instancePath: o + "/sidebar/style", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/style/enum", keyword: "enum", params: { allowedValues: O.properties.sidebar.oneOf[1].oneOf[0].properties.style.enum, schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.style }, message: "must be equal to one of the allowed values", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.style.enum, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.style, data: le };
              e === null ? e = [A] : e.push(A), r++;
            }
            var _e = pe === r;
          }
          if (Q.background !== void 0) {
            let le = Q.background, pe = r, v = r, E = false, A = r, Z = O.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[0].enum;
            if (!(le === "primary" || le === "secondary" || le === "success" || le === "danger" || le === "warning" || le === "info" || le === "light" || le === "dark")) {
              let U = { instancePath: o + "/sidebar/background", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/background/anyOf/0/enum", keyword: "enum", params: { allowedValues: O.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[0].enum, schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[0] }, message: "must be equal to one of the allowed values", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[0].enum, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[0], data: le };
              e === null ? e = [U] : e.push(U), r++;
            }
            var ie = A === r;
            if (E = E || ie, !E) {
              let U = r;
              if (typeof le != "string") {
                let z = { instancePath: o + "/sidebar/background", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/background/anyOf/1/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[1] }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[1].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[1], data: le };
                e === null ? e = [z] : e.push(z), r++;
              }
              let re = r;
              var ie = U === r;
              E = E || ie;
            }
            if (E)
              r = v, e !== null && (v ? e.length = v : e = null);
            else {
              let U = { instancePath: o + "/sidebar/background", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/background/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.background, data: le };
              e === null ? e = [U] : e.push(U), r++;
            }
            var _e = pe === r;
          }
          if (Q.foreground !== void 0) {
            let le = Q.foreground, pe = r, v = r, E = false, A = r, Z = O.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[0].enum;
            if (!(le === "primary" || le === "secondary" || le === "success" || le === "danger" || le === "warning" || le === "info" || le === "light" || le === "dark")) {
              let U = { instancePath: o + "/sidebar/foreground", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/foreground/anyOf/0/enum", keyword: "enum", params: { allowedValues: O.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[0].enum, schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[0] }, message: "must be equal to one of the allowed values", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[0].enum, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[0], data: le };
              e === null ? e = [U] : e.push(U), r++;
            }
            var ka = A === r;
            if (E = E || ka, !E) {
              let U = r;
              if (typeof le != "string") {
                let z = { instancePath: o + "/sidebar/foreground", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/foreground/anyOf/1/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[1] }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[1].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[1], data: le };
                e === null ? e = [z] : e.push(z), r++;
              }
              let re = r;
              var ka = U === r;
              E = E || ka;
            }
            if (E)
              r = v, e !== null && (v ? e.length = v : e = null);
            else {
              let U = { instancePath: o + "/sidebar/foreground", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/foreground/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.foreground, data: le };
              e === null ? e = [U] : e.push(U), r++;
            }
            var _e = pe === r;
          }
          if (Q.border !== void 0) {
            let le = Q.border, pe = r;
            if (typeof le != "boolean") {
              let A = { instancePath: o + "/sidebar/border", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/border/type", keyword: "type", params: { type: "boolean", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.border }, message: "must be boolean", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.border.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.border, data: le };
              e === null ? e = [A] : e.push(A), r++;
            }
            let v = r;
            var _e = pe === r;
          }
          if (Q.alignment !== void 0) {
            let le = Q.alignment, pe = r, v = O.properties.sidebar.oneOf[1].oneOf[0].properties.alignment.enum;
            if (!(le === "left" || le === "right" || le === "center")) {
              let A = { instancePath: o + "/sidebar/alignment", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/alignment/enum", keyword: "enum", params: { allowedValues: O.properties.sidebar.oneOf[1].oneOf[0].properties.alignment.enum, schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.alignment }, message: "must be equal to one of the allowed values", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.alignment.enum, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.alignment, data: le };
              e === null ? e = [A] : e.push(A), r++;
            }
            var _e = pe === r;
          }
          if (Q["collapse-level"] !== void 0) {
            let le = Q["collapse-level"], pe = r;
            if (!(typeof le == "number" && isFinite(le))) {
              let A = { instancePath: o + "/sidebar/collapse-level", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/collapse-level/type", keyword: "type", params: { type: "number", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties["collapse-level"] }, message: "must be number", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties["collapse-level"].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties["collapse-level"], data: le };
              e === null ? e = [A] : e.push(A), r++;
            }
            let v = r;
            var _e = pe === r;
          }
          if (Q.pinned !== void 0) {
            let le = Q.pinned, pe = r;
            if (typeof le != "boolean") {
              let A = { instancePath: o + "/sidebar/pinned", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/pinned/type", keyword: "type", params: { type: "boolean", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.pinned }, message: "must be boolean", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.pinned.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.pinned, data: le };
              e === null ? e = [A] : e.push(A), r++;
            }
            let v = r;
            var _e = pe === r;
          }
          if (Q.header !== void 0) {
            let le = Q.header, pe = r, v = r, E = false, A = null, Z = r;
            if (typeof le != "string") {
              let z = { instancePath: o + "/sidebar/header", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/header/oneOf/0/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[0] }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[0].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[0], data: le };
              e === null ? e = [z] : e.push(z), r++;
            }
            let T = r;
            var hn = Z === r;
            hn && (E = true, A = 0);
            let U = r, re = r;
            if (Array.isArray(le)) {
              let z = true, Xr = le.length;
              for (let W = 0; W < Xr; W++) {
                let C = le[W], te = r;
                if (typeof C != "string") {
                  let X = { instancePath: o + "/sidebar/header/" + W, schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/header/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1].items }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1].items.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1].items, data: C };
                  e === null ? e = [X] : e.push(X), r++;
                }
                let Br = r;
                var et = te === r;
                et || (z = false);
              }
            } else {
              let z = { instancePath: o + "/sidebar/header", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/header/oneOf/1/type", keyword: "type", params: { type: "array", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1] }, message: "must be array", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1], data: le };
              e === null ? e = [z] : e.push(z), r++;
            }
            var hn = U === r;
            if (hn && E ? (E = false, A = [A, 1]) : hn && (E = true, A = 1), E)
              r = v, e !== null && (v ? e.length = v : e = null);
            else {
              let z = { instancePath: o + "/sidebar/header", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/header/oneOf", keyword: "oneOf", params: { passingSchemas: A, schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.header }, message: "must match exactly one schema in oneOf", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.header, data: le };
              e === null ? e = [z] : e.push(z), r++;
            }
            var _e = pe === r;
          }
          if (Q.footer !== void 0) {
            let le = Q.footer, pe = r, v = r, E = false, A = null, Z = r;
            if (typeof le != "string") {
              let z = { instancePath: o + "/sidebar/footer", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/footer/oneOf/0/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[0] }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[0].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[0], data: le };
              e === null ? e = [z] : e.push(z), r++;
            }
            let T = r;
            var yt = Z === r;
            yt && (E = true, A = 0);
            let U = r, re = r;
            if (Array.isArray(le)) {
              let z = true, Xr = le.length;
              for (let W = 0; W < Xr; W++) {
                let C = le[W], te = r;
                if (typeof C != "string") {
                  let X = { instancePath: o + "/sidebar/footer/" + W, schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/footer/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1].items }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1].items.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1].items, data: C };
                  e === null ? e = [X] : e.push(X), r++;
                }
                let Br = r;
                var _n = te === r;
                _n || (z = false);
              }
            } else {
              let z = { instancePath: o + "/sidebar/footer", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/footer/oneOf/1/type", keyword: "type", params: { type: "array", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1] }, message: "must be array", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1], data: le };
              e === null ? e = [z] : e.push(z), r++;
            }
            var yt = U === r;
            if (yt && E ? (E = false, A = [A, 1]) : yt && (E = true, A = 1), E)
              r = v, e !== null && (v ? e.length = v : e = null);
            else {
              let z = { instancePath: o + "/sidebar/footer", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/footer/oneOf", keyword: "oneOf", params: { passingSchemas: A, schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.footer }, message: "must match exactly one schema in oneOf", schema: O.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0].properties.footer, data: le };
              e === null ? e = [z] : e.push(z), r++;
            }
            var _e = pe === r;
          }
        } else {
          let le = { instancePath: o + "/sidebar", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/type", keyword: "type", params: { type: "object", schema: O.properties.sidebar.oneOf[1].oneOf[0] }, message: "must be object", schema: O.properties.sidebar.oneOf[1].oneOf[0].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[0], data: Q };
          e === null ? e = [le] : e.push(le), r++;
        }
        var st = N === r;
        st && (Nt = true, Y = 0);
        let nt = r, J = r;
        if (Array.isArray(Q)) {
          let le = true, pe = Q.length;
          for (let v = 0; v < pe; v++) {
            let E = Q[v], A = r, Z = r;
            if (E && typeof E == "object" && !Array.isArray(E)) {
              for (let T in E) {
                let U = r, re = r;
                if (typeof T == "string") {
                  if (!vd.test(T)) {
                    let Fr = { instancePath: o + "/sidebar/" + v, schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(collapse_level|collapseLevel))", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames }, message: 'must match pattern "^(?!(collapse_level|collapseLevel))"', schema: "^(?!(collapse_level|collapseLevel))", parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames, data: T, propertyName: T };
                    e === null ? e = [Fr] : e.push(Fr), r++;
                  }
                } else {
                  let Fr = { instancePath: o + "/sidebar/" + v, schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/propertyNames/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames, data: T, propertyName: T };
                  e === null ? e = [Fr] : e.push(Fr), r++;
                }
                var Et = U === r;
                if (!Et) {
                  let Fr = { instancePath: o + "/sidebar/" + v, schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/propertyNames", keyword: "propertyNames", params: { propertyName: T, schema: O.properties.sidebar.oneOf[1].oneOf[1].items }, message: "property name must be valid", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items, data: E };
                  e === null ? e = [Fr] : e.push(Fr), r++;
                }
              }
              if (E.id !== void 0) {
                let T = E.id, U = r;
                if (typeof T != "string") {
                  let Fr = { instancePath: o + "/sidebar/" + v + "/id", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/id/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.id }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.id.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.id, data: T };
                  e === null ? e = [Fr] : e.push(Fr), r++;
                }
                let re = r;
                var ca = U === r;
              }
              if (E.title !== void 0) {
                let T = E.title, U = r, re = r, Fr = false, z = null, Xr = r;
                if (typeof T != "string") {
                  let X = { instancePath: o + "/sidebar/" + v + "/title", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/title/oneOf/0/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[0] }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[0].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[0], data: T };
                  e === null ? e = [X] : e.push(X), r++;
                }
                let W = r;
                var Gr = Xr === r;
                Gr && (Fr = true, z = 0);
                let C = r;
                if (typeof T != "boolean") {
                  let X = { instancePath: o + "/sidebar/" + v + "/title", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/title/oneOf/1/type", keyword: "type", params: { type: "boolean", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[1] }, message: "must be boolean", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[1].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[1], data: T };
                  e === null ? e = [X] : e.push(X), r++;
                }
                let te = r;
                var Gr = C === r;
                if (Gr && Fr ? (Fr = false, z = [z, 1]) : Gr && (Fr = true, z = 1), Fr)
                  r = re, e !== null && (re ? e.length = re : e = null);
                else {
                  let X = { instancePath: o + "/sidebar/" + v + "/title", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/title/oneOf", keyword: "oneOf", params: { passingSchemas: z, schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.title }, message: "must match exactly one schema in oneOf", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.title, data: T };
                  e === null ? e = [X] : e.push(X), r++;
                }
                var ca = U === r;
              }
              if (E.subtitle !== void 0) {
                let T = E.subtitle, U = r;
                if (typeof T != "string") {
                  let z = { instancePath: o + "/sidebar/" + v + "/subtitle", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/subtitle/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.subtitle }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.subtitle.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.subtitle, data: T };
                  e === null ? e = [z] : e.push(z), r++;
                }
                let re = r;
                var ca = U === r;
              }
              if (E.logo !== void 0) {
                let T = E.logo, U = r;
                if (typeof T != "string") {
                  let z = { instancePath: o + "/sidebar/" + v + "/logo", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/logo/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.logo }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.logo.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.logo, data: T };
                  e === null ? e = [z] : e.push(z), r++;
                }
                let re = r;
                var ca = U === r;
              }
              if (E.search !== void 0) {
                let T = E.search, U = r;
                if (typeof T != "boolean") {
                  let z = { instancePath: o + "/sidebar/" + v + "/search", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/search/type", keyword: "type", params: { type: "boolean", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.search }, message: "must be boolean", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.search.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.search, data: T };
                  e === null ? e = [z] : e.push(z), r++;
                }
                let re = r;
                var ca = U === r;
              }
              if (E.tools !== void 0) {
                let T = E.tools, U = r, re = r;
                if (Array.isArray(T)) {
                  let z = true, Xr = T.length;
                  for (let W = 0; W < Xr; W++) {
                    let C = T[W], te = r;
                    co(C, { instancePath: o + "/sidebar/" + v + "/tools/" + W, parentData: T, parentDataProperty: W, rootData: s }) || (e = e === null ? co.errors : e.concat(co.errors), r = e.length);
                    var Tt = te === r;
                    Tt || (z = false);
                  }
                } else {
                  let z = { instancePath: o + "/sidebar/" + v + "/tools", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/tools/type", keyword: "type", params: { type: "array", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.tools }, message: "must be array", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.tools.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.tools, data: T };
                  e === null ? e = [z] : e.push(z), r++;
                }
                var ca = U === r;
              }
              if (E.contents !== void 0) {
                let T = E.contents, U = r;
                so(T, { instancePath: o + "/sidebar/" + v + "/contents", parentData: E, parentDataProperty: "contents", rootData: s }) || (e = e === null ? so.errors : e.concat(so.errors), r = e.length);
                var ca = U === r;
              }
              if (E.style !== void 0) {
                let T = E.style, U = r, re = O.properties.sidebar.oneOf[1].oneOf[1].items.properties.style.enum;
                if (!(T === "docked" || T === "floating")) {
                  let z = { instancePath: o + "/sidebar/" + v + "/style", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/style/enum", keyword: "enum", params: { allowedValues: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.style.enum, schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.style }, message: "must be equal to one of the allowed values", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.style.enum, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.style, data: T };
                  e === null ? e = [z] : e.push(z), r++;
                }
                var ca = U === r;
              }
              if (E.background !== void 0) {
                let T = E.background, U = r, re = r, Fr = false, z = r, Xr = O.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[0].enum;
                if (!(T === "primary" || T === "secondary" || T === "success" || T === "danger" || T === "warning" || T === "info" || T === "light" || T === "dark")) {
                  let C = { instancePath: o + "/sidebar/" + v + "/background", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/background/anyOf/0/enum", keyword: "enum", params: { allowedValues: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[0].enum, schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[0] }, message: "must be equal to one of the allowed values", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[0].enum, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[0], data: T };
                  e === null ? e = [C] : e.push(C), r++;
                }
                var bn = z === r;
                if (Fr = Fr || bn, !Fr) {
                  let C = r;
                  if (typeof T != "string") {
                    let X = { instancePath: o + "/sidebar/" + v + "/background", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/background/anyOf/1/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[1] }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[1].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[1], data: T };
                    e === null ? e = [X] : e.push(X), r++;
                  }
                  let te = r;
                  var bn = C === r;
                  Fr = Fr || bn;
                }
                if (Fr)
                  r = re, e !== null && (re ? e.length = re : e = null);
                else {
                  let C = { instancePath: o + "/sidebar/" + v + "/background", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/background/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.background, data: T };
                  e === null ? e = [C] : e.push(C), r++;
                }
                var ca = U === r;
              }
              if (E.foreground !== void 0) {
                let T = E.foreground, U = r, re = r, Fr = false, z = r, Xr = O.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[0].enum;
                if (!(T === "primary" || T === "secondary" || T === "success" || T === "danger" || T === "warning" || T === "info" || T === "light" || T === "dark")) {
                  let C = { instancePath: o + "/sidebar/" + v + "/foreground", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/foreground/anyOf/0/enum", keyword: "enum", params: { allowedValues: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[0].enum, schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[0] }, message: "must be equal to one of the allowed values", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[0].enum, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[0], data: T };
                  e === null ? e = [C] : e.push(C), r++;
                }
                var vt = z === r;
                if (Fr = Fr || vt, !Fr) {
                  let C = r;
                  if (typeof T != "string") {
                    let X = { instancePath: o + "/sidebar/" + v + "/foreground", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/foreground/anyOf/1/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[1] }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[1].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[1], data: T };
                    e === null ? e = [X] : e.push(X), r++;
                  }
                  let te = r;
                  var vt = C === r;
                  Fr = Fr || vt;
                }
                if (Fr)
                  r = re, e !== null && (re ? e.length = re : e = null);
                else {
                  let C = { instancePath: o + "/sidebar/" + v + "/foreground", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/foreground/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground, data: T };
                  e === null ? e = [C] : e.push(C), r++;
                }
                var ca = U === r;
              }
              if (E.border !== void 0) {
                let T = E.border, U = r;
                if (typeof T != "boolean") {
                  let z = { instancePath: o + "/sidebar/" + v + "/border", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/border/type", keyword: "type", params: { type: "boolean", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.border }, message: "must be boolean", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.border.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.border, data: T };
                  e === null ? e = [z] : e.push(z), r++;
                }
                let re = r;
                var ca = U === r;
              }
              if (E.alignment !== void 0) {
                let T = E.alignment, U = r, re = O.properties.sidebar.oneOf[1].oneOf[1].items.properties.alignment.enum;
                if (!(T === "left" || T === "right" || T === "center")) {
                  let z = { instancePath: o + "/sidebar/" + v + "/alignment", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/alignment/enum", keyword: "enum", params: { allowedValues: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.alignment.enum, schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.alignment }, message: "must be equal to one of the allowed values", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.alignment.enum, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.alignment, data: T };
                  e === null ? e = [z] : e.push(z), r++;
                }
                var ca = U === r;
              }
              if (E["collapse-level"] !== void 0) {
                let T = E["collapse-level"], U = r;
                if (!(typeof T == "number" && isFinite(T))) {
                  let z = { instancePath: o + "/sidebar/" + v + "/collapse-level", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/collapse-level/type", keyword: "type", params: { type: "number", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties["collapse-level"] }, message: "must be number", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties["collapse-level"].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties["collapse-level"], data: T };
                  e === null ? e = [z] : e.push(z), r++;
                }
                let re = r;
                var ca = U === r;
              }
              if (E.pinned !== void 0) {
                let T = E.pinned, U = r;
                if (typeof T != "boolean") {
                  let z = { instancePath: o + "/sidebar/" + v + "/pinned", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/pinned/type", keyword: "type", params: { type: "boolean", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.pinned }, message: "must be boolean", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.pinned.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.pinned, data: T };
                  e === null ? e = [z] : e.push(z), r++;
                }
                let re = r;
                var ca = U === r;
              }
              if (E.header !== void 0) {
                let T = E.header, U = r, re = r, Fr = false, z = null, Xr = r;
                if (typeof T != "string") {
                  let X = { instancePath: o + "/sidebar/" + v + "/header", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/header/oneOf/0/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[0] }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[0].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[0], data: T };
                  e === null ? e = [X] : e.push(X), r++;
                }
                let W = r;
                var yn = Xr === r;
                yn && (Fr = true, z = 0);
                let C = r, te = r;
                if (Array.isArray(T)) {
                  let X = true, Yr = T.length;
                  for (let G = 0; G < Yr; G++) {
                    let Ar = T[G], _t = r;
                    if (typeof Ar != "string") {
                      let Er = { instancePath: o + "/sidebar/" + v + "/header/" + G, schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/header/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1].items }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1].items.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1].items, data: Ar };
                      e === null ? e = [Er] : e.push(Er), r++;
                    }
                    let Zr = r;
                    var Va = _t === r;
                    Va || (X = false);
                  }
                } else {
                  let X = { instancePath: o + "/sidebar/" + v + "/header", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/header/oneOf/1/type", keyword: "type", params: { type: "array", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1] }, message: "must be array", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1], data: T };
                  e === null ? e = [X] : e.push(X), r++;
                }
                var yn = C === r;
                if (yn && Fr ? (Fr = false, z = [z, 1]) : yn && (Fr = true, z = 1), Fr)
                  r = re, e !== null && (re ? e.length = re : e = null);
                else {
                  let X = { instancePath: o + "/sidebar/" + v + "/header", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/header/oneOf", keyword: "oneOf", params: { passingSchemas: z, schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.header }, message: "must match exactly one schema in oneOf", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.header, data: T };
                  e === null ? e = [X] : e.push(X), r++;
                }
                var ca = U === r;
              }
              if (E.footer !== void 0) {
                let T = E.footer, U = r, re = r, Fr = false, z = null, Xr = r;
                if (typeof T != "string") {
                  let X = { instancePath: o + "/sidebar/" + v + "/footer", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/footer/oneOf/0/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[0] }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[0].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[0], data: T };
                  e === null ? e = [X] : e.push(X), r++;
                }
                let W = r;
                var Dt = Xr === r;
                Dt && (Fr = true, z = 0);
                let C = r, te = r;
                if (Array.isArray(T)) {
                  let X = true, Yr = T.length;
                  for (let G = 0; G < Yr; G++) {
                    let Ar = T[G], _t = r;
                    if (typeof Ar != "string") {
                      let Er = { instancePath: o + "/sidebar/" + v + "/footer/" + G, schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/footer/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1].items }, message: "must be string", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1].items.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1].items, data: Ar };
                      e === null ? e = [Er] : e.push(Er), r++;
                    }
                    let Zr = r;
                    var nl = _t === r;
                    nl || (X = false);
                  }
                } else {
                  let X = { instancePath: o + "/sidebar/" + v + "/footer", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/footer/oneOf/1/type", keyword: "type", params: { type: "array", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1] }, message: "must be array", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1], data: T };
                  e === null ? e = [X] : e.push(X), r++;
                }
                var Dt = C === r;
                if (Dt && Fr ? (Fr = false, z = [z, 1]) : Dt && (Fr = true, z = 1), Fr)
                  r = re, e !== null && (re ? e.length = re : e = null);
                else {
                  let X = { instancePath: o + "/sidebar/" + v + "/footer", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/footer/oneOf", keyword: "oneOf", params: { passingSchemas: z, schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer }, message: "must match exactly one schema in oneOf", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer, data: T };
                  e === null ? e = [X] : e.push(X), r++;
                }
                var ca = U === r;
              }
            } else {
              let T = { instancePath: o + "/sidebar/" + v, schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/type", keyword: "type", params: { type: "object", schema: O.properties.sidebar.oneOf[1].oneOf[1].items }, message: "must be object", schema: O.properties.sidebar.oneOf[1].oneOf[1].items.type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1].items, data: E };
              e === null ? e = [T] : e.push(T), r++;
            }
            var ol = A === r;
            ol || (le = false);
          }
        } else {
          let le = { instancePath: o + "/sidebar", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/type", keyword: "type", params: { type: "array", schema: O.properties.sidebar.oneOf[1].oneOf[1] }, message: "must be array", schema: O.properties.sidebar.oneOf[1].oneOf[1].type, parentSchema: O.properties.sidebar.oneOf[1].oneOf[1], data: Q };
          e === null ? e = [le] : e.push(le), r++;
        }
        var st = nt === r;
        if (st && Nt ? (Nt = false, Y = [Y, 1]) : st && (Nt = true, Y = 1), Nt)
          r = w, e !== null && (w ? e.length = w : e = null);
        else {
          let le = { instancePath: o + "/sidebar", schemaPath: "#/properties/sidebar/oneOf/1/oneOf", keyword: "oneOf", params: { passingSchemas: Y, schema: O.properties.sidebar.oneOf[1] }, message: "must match exactly one schema in oneOf", schema: O.properties.sidebar.oneOf[1].oneOf, parentSchema: O.properties.sidebar.oneOf[1], data: Q };
          e === null ? e = [le] : e.push(le), r++;
        }
        var za = an === r;
        if (za && kt ? (kt = false, $t = [$t, 1]) : za && (kt = true, $t = 1), kt)
          r = $r, e !== null && ($r ? e.length = $r : e = null);
        else {
          let le = { instancePath: o + "/sidebar", schemaPath: "#/properties/sidebar/oneOf", keyword: "oneOf", params: { passingSchemas: $t, schema: O.properties.sidebar }, message: "must match exactly one schema in oneOf", schema: O.properties.sidebar.oneOf, parentSchema: O.properties.sidebar, data: Q };
          e === null ? e = [le] : e.push(le), r++;
        }
        var d = ba === r;
      }
      if (l["body-header"] !== void 0) {
        let Q = l["body-header"], ba = r;
        if (typeof Q != "string") {
          let $t = { instancePath: o + "/body-header", schemaPath: "#/properties/body-header/type", keyword: "type", params: { type: "string", schema: O.properties["body-header"] }, message: "must be string", schema: O.properties["body-header"].type, parentSchema: O.properties["body-header"], data: Q };
          e === null ? e = [$t] : e.push($t), r++;
        }
        let $r = r;
        var d = ba === r;
      }
      if (l["body-footer"] !== void 0) {
        let Q = l["body-footer"], ba = r;
        if (typeof Q != "string") {
          let $t = { instancePath: o + "/body-footer", schemaPath: "#/properties/body-footer/type", keyword: "type", params: { type: "string", schema: O.properties["body-footer"] }, message: "must be string", schema: O.properties["body-footer"].type, parentSchema: O.properties["body-footer"], data: Q };
          e === null ? e = [$t] : e.push($t), r++;
        }
        let $r = r;
        var d = ba === r;
      }
      if (l["margin-header"] !== void 0) {
        let Q = l["margin-header"], ba = r, $r = r, kt = false, $t = null, Jr = r;
        if (typeof Q != "string") {
          let Y = { instancePath: o + "/margin-header", schemaPath: "#/properties/margin-header/oneOf/0/type", keyword: "type", params: { type: "string", schema: O.properties["margin-header"].oneOf[0] }, message: "must be string", schema: O.properties["margin-header"].oneOf[0].type, parentSchema: O.properties["margin-header"].oneOf[0], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        let io = r;
        var wt = Jr === r;
        wt && (kt = true, $t = 0);
        let an = r, w = r;
        if (Array.isArray(Q)) {
          let Y = true, N = Q.length;
          for (let ae = 0; ae < N; ae++) {
            let nt = Q[ae], J = r;
            if (typeof nt != "string") {
              let le = { instancePath: o + "/margin-header/" + ae, schemaPath: "#/properties/margin-header/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: O.properties["margin-header"].oneOf[1].items }, message: "must be string", schema: O.properties["margin-header"].oneOf[1].items.type, parentSchema: O.properties["margin-header"].oneOf[1].items, data: nt };
              e === null ? e = [le] : e.push(le), r++;
            }
            let ot = r;
            var ii = J === r;
            ii || (Y = false);
          }
        } else {
          let Y = { instancePath: o + "/margin-header", schemaPath: "#/properties/margin-header/oneOf/1/type", keyword: "type", params: { type: "array", schema: O.properties["margin-header"].oneOf[1] }, message: "must be array", schema: O.properties["margin-header"].oneOf[1].type, parentSchema: O.properties["margin-header"].oneOf[1], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var wt = an === r;
        if (wt && kt ? (kt = false, $t = [$t, 1]) : wt && (kt = true, $t = 1), kt)
          r = $r, e !== null && ($r ? e.length = $r : e = null);
        else {
          let Y = { instancePath: o + "/margin-header", schemaPath: "#/properties/margin-header/oneOf", keyword: "oneOf", params: { passingSchemas: $t, schema: O.properties["margin-header"] }, message: "must match exactly one schema in oneOf", schema: O.properties["margin-header"].oneOf, parentSchema: O.properties["margin-header"], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var d = ba === r;
      }
      if (l["margin-footer"] !== void 0) {
        let Q = l["margin-footer"], ba = r, $r = r, kt = false, $t = null, Jr = r;
        if (typeof Q != "string") {
          let Y = { instancePath: o + "/margin-footer", schemaPath: "#/properties/margin-footer/oneOf/0/type", keyword: "type", params: { type: "string", schema: O.properties["margin-footer"].oneOf[0] }, message: "must be string", schema: O.properties["margin-footer"].oneOf[0].type, parentSchema: O.properties["margin-footer"].oneOf[0], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        let io = r;
        var Jn = Jr === r;
        Jn && (kt = true, $t = 0);
        let an = r, w = r;
        if (Array.isArray(Q)) {
          let Y = true, N = Q.length;
          for (let ae = 0; ae < N; ae++) {
            let nt = Q[ae], J = r;
            if (typeof nt != "string") {
              let le = { instancePath: o + "/margin-footer/" + ae, schemaPath: "#/properties/margin-footer/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: O.properties["margin-footer"].oneOf[1].items }, message: "must be string", schema: O.properties["margin-footer"].oneOf[1].items.type, parentSchema: O.properties["margin-footer"].oneOf[1].items, data: nt };
              e === null ? e = [le] : e.push(le), r++;
            }
            let ot = r;
            var Bt = J === r;
            Bt || (Y = false);
          }
        } else {
          let Y = { instancePath: o + "/margin-footer", schemaPath: "#/properties/margin-footer/oneOf/1/type", keyword: "type", params: { type: "array", schema: O.properties["margin-footer"].oneOf[1] }, message: "must be array", schema: O.properties["margin-footer"].oneOf[1].type, parentSchema: O.properties["margin-footer"].oneOf[1], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var Jn = an === r;
        if (Jn && kt ? (kt = false, $t = [$t, 1]) : Jn && (kt = true, $t = 1), kt)
          r = $r, e !== null && ($r ? e.length = $r : e = null);
        else {
          let Y = { instancePath: o + "/margin-footer", schemaPath: "#/properties/margin-footer/oneOf", keyword: "oneOf", params: { passingSchemas: $t, schema: O.properties["margin-footer"] }, message: "must match exactly one schema in oneOf", schema: O.properties["margin-footer"].oneOf, parentSchema: O.properties["margin-footer"], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var d = ba === r;
      }
      if (l["page-navigation"] !== void 0) {
        let Q = l["page-navigation"], ba = r;
        if (typeof Q != "boolean") {
          let $t = { instancePath: o + "/page-navigation", schemaPath: "#/properties/page-navigation/type", keyword: "type", params: { type: "boolean", schema: O.properties["page-navigation"] }, message: "must be boolean", schema: O.properties["page-navigation"].type, parentSchema: O.properties["page-navigation"], data: Q };
          e === null ? e = [$t] : e.push($t), r++;
        }
        let $r = r;
        var d = ba === r;
      }
      if (l["page-footer"] !== void 0) {
        let Q = l["page-footer"], ba = r, $r = r, kt = false, $t = null, Jr = r;
        if (typeof Q != "string") {
          let Y = { instancePath: o + "/page-footer", schemaPath: "#/properties/page-footer/oneOf/0/type", keyword: "type", params: { type: "string", schema: O.properties["page-footer"].oneOf[0] }, message: "must be string", schema: O.properties["page-footer"].oneOf[0].type, parentSchema: O.properties["page-footer"].oneOf[0], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        let io = r;
        var oo = Jr === r;
        oo && (kt = true, $t = 0);
        let an = r, w = r;
        if (Q && typeof Q == "object" && !Array.isArray(Q)) {
          let Y = r;
          for (let N in Q)
            if (!(N === "left" || N === "right" || N === "center" || N === "border" || N === "background" || N === "foreground")) {
              let ae = { instancePath: o + "/page-footer", schemaPath: "#/properties/page-footer/oneOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: N, schema: O.properties["page-footer"].oneOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: O.properties["page-footer"].oneOf[1], data: Q };
              e === null ? e = [ae] : e.push(ae), r++;
            }
          if (Q.left !== void 0) {
            let N = Q.left, ae = r;
            qn(N, { instancePath: o + "/page-footer/left", parentData: Q, parentDataProperty: "left", rootData: s }) || (e = e === null ? qn.errors : e.concat(qn.errors), r = e.length);
            var qo = ae === r;
          }
          if (Q.right !== void 0) {
            let N = Q.right, ae = r;
            qn(N, { instancePath: o + "/page-footer/right", parentData: Q, parentDataProperty: "right", rootData: s }) || (e = e === null ? qn.errors : e.concat(qn.errors), r = e.length);
            var qo = ae === r;
          }
          if (Q.center !== void 0) {
            let N = Q.center, ae = r;
            qn(N, { instancePath: o + "/page-footer/center", parentData: Q, parentDataProperty: "center", rootData: s }) || (e = e === null ? qn.errors : e.concat(qn.errors), r = e.length);
            var qo = ae === r;
          }
          if (Q.border !== void 0) {
            let N = Q.border, ae = r, nt = r, J = false, ot = null, le = r;
            if (typeof N != "boolean") {
              let Z = { instancePath: o + "/page-footer/border", schemaPath: "#/properties/page-footer/oneOf/1/properties/border/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: O.properties["page-footer"].oneOf[1].properties.border.oneOf[0] }, message: "must be boolean", schema: O.properties["page-footer"].oneOf[1].properties.border.oneOf[0].type, parentSchema: O.properties["page-footer"].oneOf[1].properties.border.oneOf[0], data: N };
              e === null ? e = [Z] : e.push(Z), r++;
            }
            let pe = r;
            var Ot = le === r;
            Ot && (J = true, ot = 0);
            let v = r;
            if (typeof N != "string") {
              let Z = { instancePath: o + "/page-footer/border", schemaPath: "#/properties/page-footer/oneOf/1/properties/border/oneOf/1/type", keyword: "type", params: { type: "string", schema: O.properties["page-footer"].oneOf[1].properties.border.oneOf[1] }, message: "must be string", schema: O.properties["page-footer"].oneOf[1].properties.border.oneOf[1].type, parentSchema: O.properties["page-footer"].oneOf[1].properties.border.oneOf[1], data: N };
              e === null ? e = [Z] : e.push(Z), r++;
            }
            let E = r;
            var Ot = v === r;
            if (Ot && J ? (J = false, ot = [ot, 1]) : Ot && (J = true, ot = 1), J)
              r = nt, e !== null && (nt ? e.length = nt : e = null);
            else {
              let Z = { instancePath: o + "/page-footer/border", schemaPath: "#/properties/page-footer/oneOf/1/properties/border/oneOf", keyword: "oneOf", params: { passingSchemas: ot, schema: O.properties["page-footer"].oneOf[1].properties.border }, message: "must match exactly one schema in oneOf", schema: O.properties["page-footer"].oneOf[1].properties.border.oneOf, parentSchema: O.properties["page-footer"].oneOf[1].properties.border, data: N };
              e === null ? e = [Z] : e.push(Z), r++;
            }
            var qo = ae === r;
          }
          if (Q.background !== void 0) {
            let N = Q.background, ae = r;
            if (typeof N != "string") {
              let ot = { instancePath: o + "/page-footer/background", schemaPath: "#/properties/page-footer/oneOf/1/properties/background/type", keyword: "type", params: { type: "string", schema: O.properties["page-footer"].oneOf[1].properties.background }, message: "must be string", schema: O.properties["page-footer"].oneOf[1].properties.background.type, parentSchema: O.properties["page-footer"].oneOf[1].properties.background, data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var qo = ae === r;
          }
          if (Q.foreground !== void 0) {
            let N = Q.foreground, ae = r;
            if (typeof N != "string") {
              let ot = { instancePath: o + "/page-footer/foreground", schemaPath: "#/properties/page-footer/oneOf/1/properties/foreground/type", keyword: "type", params: { type: "string", schema: O.properties["page-footer"].oneOf[1].properties.foreground }, message: "must be string", schema: O.properties["page-footer"].oneOf[1].properties.foreground.type, parentSchema: O.properties["page-footer"].oneOf[1].properties.foreground, data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var qo = ae === r;
          }
        } else {
          let Y = { instancePath: o + "/page-footer", schemaPath: "#/properties/page-footer/oneOf/1/type", keyword: "type", params: { type: "object", schema: O.properties["page-footer"].oneOf[1] }, message: "must be object", schema: O.properties["page-footer"].oneOf[1].type, parentSchema: O.properties["page-footer"].oneOf[1], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var oo = an === r;
        if (oo && kt ? (kt = false, $t = [$t, 1]) : oo && (kt = true, $t = 1), kt)
          r = $r, e !== null && ($r ? e.length = $r : e = null);
        else {
          let Y = { instancePath: o + "/page-footer", schemaPath: "#/properties/page-footer/oneOf", keyword: "oneOf", params: { passingSchemas: $t, schema: O.properties["page-footer"] }, message: "must match exactly one schema in oneOf", schema: O.properties["page-footer"].oneOf, parentSchema: O.properties["page-footer"], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var d = ba === r;
      }
      if (l.image !== void 0) {
        let Q = l.image, ba = r;
        if (typeof Q != "string") {
          let $t = { instancePath: o + "/image", schemaPath: "#/properties/image/type", keyword: "type", params: { type: "string", schema: O.properties.image }, message: "must be string", schema: O.properties.image.type, parentSchema: O.properties.image, data: Q };
          e === null ? e = [$t] : e.push($t), r++;
        }
        let $r = r;
        var d = ba === r;
      }
      if (l.comments !== void 0) {
        let Q = l.comments, ba = r;
        Uo(Q, { instancePath: o + "/comments", parentData: l, parentDataProperty: "comments", rootData: s }) || (e = e === null ? Uo.errors : e.concat(Uo.errors), r = e.length);
        var d = ba === r;
      }
      if (l["open-graph"] !== void 0) {
        let Q = l["open-graph"], ba = r, $r = r, kt = false, $t = null, Jr = r;
        if (typeof Q != "boolean") {
          let Y = { instancePath: o + "/open-graph", schemaPath: "#/properties/open-graph/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: O.properties["open-graph"].oneOf[0] }, message: "must be boolean", schema: O.properties["open-graph"].oneOf[0].type, parentSchema: O.properties["open-graph"].oneOf[0], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        let io = r;
        var xo = Jr === r;
        xo && (kt = true, $t = 0);
        let an = r, w = r;
        if (Q && typeof Q == "object" && !Array.isArray(Q)) {
          for (let N in Q) {
            let ae = r, nt = r;
            if (typeof N == "string") {
              if (!ig.test(N)) {
                let J = { instancePath: o + "/open-graph", schemaPath: "#/properties/open-graph/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(^(?!^(?!(site_name|siteName)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))", schema: O.properties["open-graph"].oneOf[1].propertyNames }, message: 'must match pattern "^(?!(^(?!^(?!(site_name|siteName)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))"', schema: "^(?!(^(?!^(?!(site_name|siteName)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))", parentSchema: O.properties["open-graph"].oneOf[1].propertyNames, data: N, propertyName: N };
                e === null ? e = [J] : e.push(J), r++;
              }
            } else {
              let J = { instancePath: o + "/open-graph", schemaPath: "#/properties/open-graph/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: O.properties["open-graph"].oneOf[1].propertyNames }, message: "must be string", schema: O.properties["open-graph"].oneOf[1].propertyNames.type, parentSchema: O.properties["open-graph"].oneOf[1].propertyNames, data: N, propertyName: N };
              e === null ? e = [J] : e.push(J), r++;
            }
            var li = ae === r;
            if (!li) {
              let J = { instancePath: o + "/open-graph", schemaPath: "#/properties/open-graph/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: N, schema: O.properties["open-graph"].oneOf[1] }, message: "property name must be valid", schema: O.properties["open-graph"].oneOf[1].propertyNames, parentSchema: O.properties["open-graph"].oneOf[1], data: Q };
              e === null ? e = [J] : e.push(J), r++;
            }
          }
          let Y = r;
          for (let N in Q)
            if (!(N === "title" || N === "description" || N === "image" || N === "image-width" || N === "image-height" || N === "locale" || N === "site-name")) {
              let ae = { instancePath: o + "/open-graph", schemaPath: "#/properties/open-graph/oneOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: N, schema: O.properties["open-graph"].oneOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: O.properties["open-graph"].oneOf[1], data: Q };
              e === null ? e = [ae] : e.push(ae), r++;
            }
          if (Q.title !== void 0) {
            let N = Q.title, ae = r;
            if (typeof N != "string") {
              let J = { instancePath: o + "/open-graph/title", schemaPath: "#/properties/open-graph/oneOf/1/properties/title/type", keyword: "type", params: { type: "string", schema: O.properties["open-graph"].oneOf[1].properties.title }, message: "must be string", schema: O.properties["open-graph"].oneOf[1].properties.title.type, parentSchema: O.properties["open-graph"].oneOf[1].properties.title, data: N };
              e === null ? e = [J] : e.push(J), r++;
            }
            let nt = r;
            var Pt = ae === r;
          }
          if (Q.description !== void 0) {
            let N = Q.description, ae = r;
            if (typeof N != "string") {
              let ot = { instancePath: o + "/open-graph/description", schemaPath: "#/properties/open-graph/oneOf/1/properties/description/type", keyword: "type", params: { type: "string", schema: O.properties["open-graph"].oneOf[1].properties.description }, message: "must be string", schema: O.properties["open-graph"].oneOf[1].properties.description.type, parentSchema: O.properties["open-graph"].oneOf[1].properties.description, data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var Pt = ae === r;
          }
          if (Q.image !== void 0) {
            let N = Q.image, ae = r;
            if (typeof N != "string") {
              let ot = { instancePath: o + "/open-graph/image", schemaPath: "#/properties/open-graph/oneOf/1/properties/image/type", keyword: "type", params: { type: "string", schema: O.properties["open-graph"].oneOf[1].properties.image }, message: "must be string", schema: O.properties["open-graph"].oneOf[1].properties.image.type, parentSchema: O.properties["open-graph"].oneOf[1].properties.image, data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var Pt = ae === r;
          }
          if (Q["image-width"] !== void 0) {
            let N = Q["image-width"], ae = r;
            if (!(typeof N == "number" && isFinite(N))) {
              let ot = { instancePath: o + "/open-graph/image-width", schemaPath: "#/properties/open-graph/oneOf/1/properties/image-width/type", keyword: "type", params: { type: "number", schema: O.properties["open-graph"].oneOf[1].properties["image-width"] }, message: "must be number", schema: O.properties["open-graph"].oneOf[1].properties["image-width"].type, parentSchema: O.properties["open-graph"].oneOf[1].properties["image-width"], data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var Pt = ae === r;
          }
          if (Q["image-height"] !== void 0) {
            let N = Q["image-height"], ae = r;
            if (!(typeof N == "number" && isFinite(N))) {
              let ot = { instancePath: o + "/open-graph/image-height", schemaPath: "#/properties/open-graph/oneOf/1/properties/image-height/type", keyword: "type", params: { type: "number", schema: O.properties["open-graph"].oneOf[1].properties["image-height"] }, message: "must be number", schema: O.properties["open-graph"].oneOf[1].properties["image-height"].type, parentSchema: O.properties["open-graph"].oneOf[1].properties["image-height"], data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var Pt = ae === r;
          }
          if (Q.locale !== void 0) {
            let N = Q.locale, ae = r;
            if (typeof N != "string") {
              let ot = { instancePath: o + "/open-graph/locale", schemaPath: "#/properties/open-graph/oneOf/1/properties/locale/type", keyword: "type", params: { type: "string", schema: O.properties["open-graph"].oneOf[1].properties.locale }, message: "must be string", schema: O.properties["open-graph"].oneOf[1].properties.locale.type, parentSchema: O.properties["open-graph"].oneOf[1].properties.locale, data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var Pt = ae === r;
          }
          if (Q["site-name"] !== void 0) {
            let N = Q["site-name"], ae = r;
            if (typeof N != "string") {
              let ot = { instancePath: o + "/open-graph/site-name", schemaPath: "#/properties/open-graph/oneOf/1/properties/site-name/type", keyword: "type", params: { type: "string", schema: O.properties["open-graph"].oneOf[1].properties["site-name"] }, message: "must be string", schema: O.properties["open-graph"].oneOf[1].properties["site-name"].type, parentSchema: O.properties["open-graph"].oneOf[1].properties["site-name"], data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var Pt = ae === r;
          }
        } else {
          let Y = { instancePath: o + "/open-graph", schemaPath: "#/properties/open-graph/oneOf/1/type", keyword: "type", params: { type: "object", schema: O.properties["open-graph"].oneOf[1] }, message: "must be object", schema: O.properties["open-graph"].oneOf[1].type, parentSchema: O.properties["open-graph"].oneOf[1], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var xo = an === r;
        if (xo && kt ? (kt = false, $t = [$t, 1]) : xo && (kt = true, $t = 1), kt)
          r = $r, e !== null && ($r ? e.length = $r : e = null);
        else {
          let Y = { instancePath: o + "/open-graph", schemaPath: "#/properties/open-graph/oneOf", keyword: "oneOf", params: { passingSchemas: $t, schema: O.properties["open-graph"] }, message: "must match exactly one schema in oneOf", schema: O.properties["open-graph"].oneOf, parentSchema: O.properties["open-graph"], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var d = ba === r;
      }
      if (l["twitter-card"] !== void 0) {
        let Q = l["twitter-card"], ba = r, $r = r, kt = false, $t = null, Jr = r;
        if (typeof Q != "boolean") {
          let Y = { instancePath: o + "/twitter-card", schemaPath: "#/properties/twitter-card/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: O.properties["twitter-card"].oneOf[0] }, message: "must be boolean", schema: O.properties["twitter-card"].oneOf[0].type, parentSchema: O.properties["twitter-card"].oneOf[0], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        let io = r;
        var bo = Jr === r;
        bo && (kt = true, $t = 0);
        let an = r, w = r;
        if (Q && typeof Q == "object" && !Array.isArray(Q)) {
          for (let N in Q) {
            let ae = r, nt = r;
            if (typeof N == "string") {
              if (!lg.test(N)) {
                let J = { instancePath: o + "/twitter-card", schemaPath: "#/properties/twitter-card/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(^(?!^(?!(card_style|cardStyle)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))", schema: O.properties["twitter-card"].oneOf[1].propertyNames }, message: 'must match pattern "^(?!(^(?!^(?!(card_style|cardStyle)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))"', schema: "^(?!(^(?!^(?!(card_style|cardStyle)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))", parentSchema: O.properties["twitter-card"].oneOf[1].propertyNames, data: N, propertyName: N };
                e === null ? e = [J] : e.push(J), r++;
              }
            } else {
              let J = { instancePath: o + "/twitter-card", schemaPath: "#/properties/twitter-card/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: O.properties["twitter-card"].oneOf[1].propertyNames }, message: "must be string", schema: O.properties["twitter-card"].oneOf[1].propertyNames.type, parentSchema: O.properties["twitter-card"].oneOf[1].propertyNames, data: N, propertyName: N };
              e === null ? e = [J] : e.push(J), r++;
            }
            var sl = ae === r;
            if (!sl) {
              let J = { instancePath: o + "/twitter-card", schemaPath: "#/properties/twitter-card/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: N, schema: O.properties["twitter-card"].oneOf[1] }, message: "property name must be valid", schema: O.properties["twitter-card"].oneOf[1].propertyNames, parentSchema: O.properties["twitter-card"].oneOf[1], data: Q };
              e === null ? e = [J] : e.push(J), r++;
            }
          }
          let Y = r;
          for (let N in Q)
            if (!(N === "title" || N === "description" || N === "image" || N === "image-width" || N === "image-height" || N === "card-style" || N === "creator" || N === "site")) {
              let ae = { instancePath: o + "/twitter-card", schemaPath: "#/properties/twitter-card/oneOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: N, schema: O.properties["twitter-card"].oneOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: O.properties["twitter-card"].oneOf[1], data: Q };
              e === null ? e = [ae] : e.push(ae), r++;
            }
          if (Q.title !== void 0) {
            let N = Q.title, ae = r;
            if (typeof N != "string") {
              let J = { instancePath: o + "/twitter-card/title", schemaPath: "#/properties/twitter-card/oneOf/1/properties/title/type", keyword: "type", params: { type: "string", schema: O.properties["twitter-card"].oneOf[1].properties.title }, message: "must be string", schema: O.properties["twitter-card"].oneOf[1].properties.title.type, parentSchema: O.properties["twitter-card"].oneOf[1].properties.title, data: N };
              e === null ? e = [J] : e.push(J), r++;
            }
            let nt = r;
            var Ur = ae === r;
          }
          if (Q.description !== void 0) {
            let N = Q.description, ae = r;
            if (typeof N != "string") {
              let ot = { instancePath: o + "/twitter-card/description", schemaPath: "#/properties/twitter-card/oneOf/1/properties/description/type", keyword: "type", params: { type: "string", schema: O.properties["twitter-card"].oneOf[1].properties.description }, message: "must be string", schema: O.properties["twitter-card"].oneOf[1].properties.description.type, parentSchema: O.properties["twitter-card"].oneOf[1].properties.description, data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var Ur = ae === r;
          }
          if (Q.image !== void 0) {
            let N = Q.image, ae = r;
            if (typeof N != "string") {
              let ot = { instancePath: o + "/twitter-card/image", schemaPath: "#/properties/twitter-card/oneOf/1/properties/image/type", keyword: "type", params: { type: "string", schema: O.properties["twitter-card"].oneOf[1].properties.image }, message: "must be string", schema: O.properties["twitter-card"].oneOf[1].properties.image.type, parentSchema: O.properties["twitter-card"].oneOf[1].properties.image, data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var Ur = ae === r;
          }
          if (Q["image-width"] !== void 0) {
            let N = Q["image-width"], ae = r;
            if (!(typeof N == "number" && isFinite(N))) {
              let ot = { instancePath: o + "/twitter-card/image-width", schemaPath: "#/properties/twitter-card/oneOf/1/properties/image-width/type", keyword: "type", params: { type: "number", schema: O.properties["twitter-card"].oneOf[1].properties["image-width"] }, message: "must be number", schema: O.properties["twitter-card"].oneOf[1].properties["image-width"].type, parentSchema: O.properties["twitter-card"].oneOf[1].properties["image-width"], data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var Ur = ae === r;
          }
          if (Q["image-height"] !== void 0) {
            let N = Q["image-height"], ae = r;
            if (!(typeof N == "number" && isFinite(N))) {
              let ot = { instancePath: o + "/twitter-card/image-height", schemaPath: "#/properties/twitter-card/oneOf/1/properties/image-height/type", keyword: "type", params: { type: "number", schema: O.properties["twitter-card"].oneOf[1].properties["image-height"] }, message: "must be number", schema: O.properties["twitter-card"].oneOf[1].properties["image-height"].type, parentSchema: O.properties["twitter-card"].oneOf[1].properties["image-height"], data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var Ur = ae === r;
          }
          if (Q["card-style"] !== void 0) {
            let N = Q["card-style"], ae = r, nt = O.properties["twitter-card"].oneOf[1].properties["card-style"].enum;
            if (!(N === "summary" || N === "summary_large_image")) {
              let ot = { instancePath: o + "/twitter-card/card-style", schemaPath: "#/properties/twitter-card/oneOf/1/properties/card-style/enum", keyword: "enum", params: { allowedValues: O.properties["twitter-card"].oneOf[1].properties["card-style"].enum, schema: O.properties["twitter-card"].oneOf[1].properties["card-style"] }, message: "must be equal to one of the allowed values", schema: O.properties["twitter-card"].oneOf[1].properties["card-style"].enum, parentSchema: O.properties["twitter-card"].oneOf[1].properties["card-style"], data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            var Ur = ae === r;
          }
          if (Q.creator !== void 0) {
            let N = Q.creator, ae = r;
            if (typeof N != "string") {
              let ot = { instancePath: o + "/twitter-card/creator", schemaPath: "#/properties/twitter-card/oneOf/1/properties/creator/type", keyword: "type", params: { type: "string", schema: O.properties["twitter-card"].oneOf[1].properties.creator }, message: "must be string", schema: O.properties["twitter-card"].oneOf[1].properties.creator.type, parentSchema: O.properties["twitter-card"].oneOf[1].properties.creator, data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var Ur = ae === r;
          }
          if (Q.site !== void 0) {
            let N = Q.site, ae = r;
            if (typeof N != "string") {
              let ot = { instancePath: o + "/twitter-card/site", schemaPath: "#/properties/twitter-card/oneOf/1/properties/site/type", keyword: "type", params: { type: "string", schema: O.properties["twitter-card"].oneOf[1].properties.site }, message: "must be string", schema: O.properties["twitter-card"].oneOf[1].properties.site.type, parentSchema: O.properties["twitter-card"].oneOf[1].properties.site, data: N };
              e === null ? e = [ot] : e.push(ot), r++;
            }
            let nt = r;
            var Ur = ae === r;
          }
        } else {
          let Y = { instancePath: o + "/twitter-card", schemaPath: "#/properties/twitter-card/oneOf/1/type", keyword: "type", params: { type: "object", schema: O.properties["twitter-card"].oneOf[1] }, message: "must be object", schema: O.properties["twitter-card"].oneOf[1].type, parentSchema: O.properties["twitter-card"].oneOf[1], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var bo = an === r;
        if (bo && kt ? (kt = false, $t = [$t, 1]) : bo && (kt = true, $t = 1), kt)
          r = $r, e !== null && ($r ? e.length = $r : e = null);
        else {
          let Y = { instancePath: o + "/twitter-card", schemaPath: "#/properties/twitter-card/oneOf", keyword: "oneOf", params: { passingSchemas: $t, schema: O.properties["twitter-card"] }, message: "must match exactly one schema in oneOf", schema: O.properties["twitter-card"].oneOf, parentSchema: O.properties["twitter-card"], data: Q };
          e === null ? e = [Y] : e.push(Y), r++;
        }
        var d = ba === r;
      }
    } else {
      let Q = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: O }, message: "must be object", schema: O.type, parentSchema: O, data: l };
      e === null ? e = [Q] : e.push(Q), r++;
    }
    return hc.errors = e, r === 0;
  }
  B["chapter-item"] = yc;
  var _o = { anyOf: [{ $ref: "navigation-item", description: "be navigation-item" }, { type: "object", description: "be an object", properties: { part: { type: "string", description: "be a string" }, chapters: { type: "array", description: "be an array of values, where each element should be navigation-item", items: { $ref: "navigation-item", description: "be navigation-item" } } }, patternProperties: {}, required: ["part", "chapters"], additionalProperties: false }], description: "be at least one of: navigation-item, an object", $id: "chapter-item" };
  function yc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = r;
    wa(l, { instancePath: o, parentData: S, parentDataProperty: F, rootData: s }) || (e = e === null ? wa.errors : e.concat(wa.errors), r = e.length);
    var u = d === r;
    if (c = c || u, !c) {
      let x = r, D = r;
      if (l && typeof l == "object" && !Array.isArray(l)) {
        if (l.part === void 0) {
          let R = { instancePath: o, schemaPath: "#/anyOf/1/required", keyword: "required", params: { missingProperty: "part", schema: _o.anyOf[1] }, message: "must have required property 'part'", schema: _o.anyOf[1].required, parentSchema: _o.anyOf[1], data: l };
          e === null ? e = [R] : e.push(R), r++;
        }
        if (l.chapters === void 0) {
          let R = { instancePath: o, schemaPath: "#/anyOf/1/required", keyword: "required", params: { missingProperty: "chapters", schema: _o.anyOf[1] }, message: "must have required property 'chapters'", schema: _o.anyOf[1].required, parentSchema: _o.anyOf[1], data: l };
          e === null ? e = [R] : e.push(R), r++;
        }
        let m = r;
        for (let R in l)
          if (!(R === "part" || R === "chapters")) {
            let j = { instancePath: o, schemaPath: "#/anyOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: R, schema: _o.anyOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: _o.anyOf[1], data: l };
            e === null ? e = [j] : e.push(j), r++;
          }
        if (l.part !== void 0) {
          let R = l.part, j = r;
          if (typeof R != "string") {
            let se = { instancePath: o + "/part", schemaPath: "#/anyOf/1/properties/part/type", keyword: "type", params: { type: "string", schema: _o.anyOf[1].properties.part }, message: "must be string", schema: _o.anyOf[1].properties.part.type, parentSchema: _o.anyOf[1].properties.part, data: R };
            e === null ? e = [se] : e.push(se), r++;
          }
          let V = r;
          var y = j === r;
        }
        if (l.chapters !== void 0) {
          let R = l.chapters, j = r, V = r;
          if (Array.isArray(R)) {
            let P = true, oe = R.length;
            for (let h = 0; h < oe; h++) {
              let f = R[h], ft = r;
              wa(f, { instancePath: o + "/chapters/" + h, parentData: R, parentDataProperty: h, rootData: s }) || (e = e === null ? wa.errors : e.concat(wa.errors), r = e.length);
              var g = ft === r;
              g || (P = false);
            }
          } else {
            let P = { instancePath: o + "/chapters", schemaPath: "#/anyOf/1/properties/chapters/type", keyword: "type", params: { type: "array", schema: _o.anyOf[1].properties.chapters }, message: "must be array", schema: _o.anyOf[1].properties.chapters.type, parentSchema: _o.anyOf[1].properties.chapters, data: R };
            e === null ? e = [P] : e.push(P), r++;
          }
          var y = j === r;
        }
      } else {
        let m = { instancePath: o, schemaPath: "#/anyOf/1/type", keyword: "type", params: { type: "object", schema: _o.anyOf[1] }, message: "must be object", schema: _o.anyOf[1].type, parentSchema: _o.anyOf[1], data: l };
        e === null ? e = [m] : e.push(m), r++;
      }
      var u = x === r;
      c = c || u;
    }
    if (c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let x = { instancePath: o, schemaPath: "#/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: _o.anyOf, parentSchema: _o, data: l };
      e === null ? e = [x] : e.push(x), r++;
    }
    return yc.errors = e, r === 0;
  }
  B["chapter-list"] = bc;
  var ao = { type: "array", description: "be an array of values, where each element should be at least one of: navigation-item, an object", items: { anyOf: [{ $ref: "navigation-item", description: "be navigation-item" }, { type: "object", description: "be an object", properties: { part: { type: "string", description: "be a string" }, chapters: { type: "array", description: "be an array of values, where each element should be navigation-item", items: { $ref: "navigation-item", description: "be navigation-item" } } }, patternProperties: {}, required: ["part", "chapters"], additionalProperties: false }], description: "be at least one of: navigation-item, an object" }, $id: "chapter-list" };
  function bc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let g = true, x = l.length;
      for (let D = 0; D < x; D++) {
        let $ = l[D], m = r, R = r, j = false, V = r;
        wa($, { instancePath: o + "/" + D, parentData: l, parentDataProperty: D, rootData: s }) || (e = e === null ? wa.errors : e.concat(wa.errors), r = e.length);
        var c = V === r;
        if (j = j || c, !j) {
          let se = r, P = r;
          if ($ && typeof $ == "object" && !Array.isArray($)) {
            if ($.part === void 0) {
              let f = { instancePath: o + "/" + D, schemaPath: "#/items/anyOf/1/required", keyword: "required", params: { missingProperty: "part", schema: ao.items.anyOf[1] }, message: "must have required property 'part'", schema: ao.items.anyOf[1].required, parentSchema: ao.items.anyOf[1], data: $ };
              e === null ? e = [f] : e.push(f), r++;
            }
            if ($.chapters === void 0) {
              let f = { instancePath: o + "/" + D, schemaPath: "#/items/anyOf/1/required", keyword: "required", params: { missingProperty: "chapters", schema: ao.items.anyOf[1] }, message: "must have required property 'chapters'", schema: ao.items.anyOf[1].required, parentSchema: ao.items.anyOf[1], data: $ };
              e === null ? e = [f] : e.push(f), r++;
            }
            let h = r;
            for (let f in $)
              if (!(f === "part" || f === "chapters")) {
                let ft = { instancePath: o + "/" + D, schemaPath: "#/items/anyOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: f, schema: ao.items.anyOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: ao.items.anyOf[1], data: $ };
                e === null ? e = [ft] : e.push(ft), r++;
              }
            if ($.part !== void 0) {
              let f = $.part, ft = r;
              if (typeof f != "string") {
                let Dr = { instancePath: o + "/" + D + "/part", schemaPath: "#/items/anyOf/1/properties/part/type", keyword: "type", params: { type: "string", schema: ao.items.anyOf[1].properties.part }, message: "must be string", schema: ao.items.anyOf[1].properties.part.type, parentSchema: ao.items.anyOf[1].properties.part, data: f };
                e === null ? e = [Dr] : e.push(Dr), r++;
              }
              let Cr = r;
              var d = ft === r;
            }
            if ($.chapters !== void 0) {
              let f = $.chapters, ft = r, Cr = r;
              if (Array.isArray(f)) {
                let wr = true, ta = f.length;
                for (let jt = 0; jt < ta; jt++) {
                  let ee = f[jt], za = r;
                  wa(ee, { instancePath: o + "/" + D + "/chapters/" + jt, parentData: f, parentDataProperty: jt, rootData: s }) || (e = e === null ? wa.errors : e.concat(wa.errors), r = e.length);
                  var u = za === r;
                  u || (wr = false);
                }
              } else {
                let wr = { instancePath: o + "/" + D + "/chapters", schemaPath: "#/items/anyOf/1/properties/chapters/type", keyword: "type", params: { type: "array", schema: ao.items.anyOf[1].properties.chapters }, message: "must be array", schema: ao.items.anyOf[1].properties.chapters.type, parentSchema: ao.items.anyOf[1].properties.chapters, data: f };
                e === null ? e = [wr] : e.push(wr), r++;
              }
              var d = ft === r;
            }
          } else {
            let h = { instancePath: o + "/" + D, schemaPath: "#/items/anyOf/1/type", keyword: "type", params: { type: "object", schema: ao.items.anyOf[1] }, message: "must be object", schema: ao.items.anyOf[1].type, parentSchema: ao.items.anyOf[1], data: $ };
            e === null ? e = [h] : e.push(h), r++;
          }
          var c = se === r;
          j = j || c;
        }
        if (j)
          r = R, e !== null && (R ? e.length = R : e = null);
        else {
          let se = { instancePath: o + "/" + D, schemaPath: "#/items/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: ao.items.anyOf, parentSchema: ao.items, data: $ };
          e === null ? e = [se] : e.push(se), r++;
        }
        var y = m === r;
        y || (g = false);
      }
    } else {
      let g = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: ao }, message: "must be array", schema: ao.type, parentSchema: ao, data: l };
      e === null ? e = [g] : e.push(g), r++;
    }
    return bc.errors = e, r === 0;
  }
  B["crossref-labels-schema"] = Sa;
  var cg = { type: "string", description: "be a string", $id: "crossref-labels-schema" };
  function Sa(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: cg }, message: "must be string", schema: cg.type, parentSchema: cg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Sa.errors = e, r === 0;
  }
  B["epub-contributor"] = _c;
  var xa = { oneOf: [{ type: "string", description: "be a string" }, { oneOf: [{ type: "object", description: "be an object", properties: { role: { type: "string", description: "be a string" }, "file-as": { type: "string", description: "be a string" }, text: { type: "string", description: "be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(file_as|fileAs))" } }, { type: "array", description: "be an array of values, where each element should be an object", items: { type: "object", description: "be an object", properties: { role: { type: "string", description: "be a string" }, "file-as": { type: "string", description: "be a string" }, text: { type: "string", description: "be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(file_as|fileAs))" } } }], description: "be exactly one of: an object, an array of values, where each element should be an object" }], description: "be exactly one of: a string, exactly one of: an object, an array of values, where each element should be an object", $id: "epub-contributor" }, w1 = new RegExp("^(?!(file_as|fileAs))", "u");
  function _c(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let Dr = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: xa.oneOf[0] }, message: "must be string", schema: xa.oneOf[0].type, parentSchema: xa.oneOf[0], data: l };
      e === null ? e = [Dr] : e.push(Dr), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r, $ = false, m = null, R = r, j = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let Dr in l) {
        let wr = r, ta = r;
        if (typeof Dr == "string") {
          if (!w1.test(Dr)) {
            let jt = { instancePath: o, schemaPath: "#/oneOf/1/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(file_as|fileAs))", schema: xa.oneOf[1].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(file_as|fileAs))"', schema: "^(?!(file_as|fileAs))", parentSchema: xa.oneOf[1].oneOf[0].propertyNames, data: Dr, propertyName: Dr };
            e === null ? e = [jt] : e.push(jt), r++;
          }
        } else {
          let jt = { instancePath: o, schemaPath: "#/oneOf/1/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: xa.oneOf[1].oneOf[0].propertyNames }, message: "must be string", schema: xa.oneOf[1].oneOf[0].propertyNames.type, parentSchema: xa.oneOf[1].oneOf[0].propertyNames, data: Dr, propertyName: Dr };
          e === null ? e = [jt] : e.push(jt), r++;
        }
        var V = wr === r;
        if (!V) {
          let jt = { instancePath: o, schemaPath: "#/oneOf/1/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: Dr, schema: xa.oneOf[1].oneOf[0] }, message: "property name must be valid", schema: xa.oneOf[1].oneOf[0].propertyNames, parentSchema: xa.oneOf[1].oneOf[0], data: l };
          e === null ? e = [jt] : e.push(jt), r++;
        }
      }
      if (l.role !== void 0) {
        let Dr = l.role, wr = r;
        if (typeof Dr != "string") {
          let jt = { instancePath: o + "/role", schemaPath: "#/oneOf/1/oneOf/0/properties/role/type", keyword: "type", params: { type: "string", schema: xa.oneOf[1].oneOf[0].properties.role }, message: "must be string", schema: xa.oneOf[1].oneOf[0].properties.role.type, parentSchema: xa.oneOf[1].oneOf[0].properties.role, data: Dr };
          e === null ? e = [jt] : e.push(jt), r++;
        }
        let ta = r;
        var se = wr === r;
      }
      if (l["file-as"] !== void 0) {
        let Dr = l["file-as"], wr = r;
        if (typeof Dr != "string") {
          let ee = { instancePath: o + "/file-as", schemaPath: "#/oneOf/1/oneOf/0/properties/file-as/type", keyword: "type", params: { type: "string", schema: xa.oneOf[1].oneOf[0].properties["file-as"] }, message: "must be string", schema: xa.oneOf[1].oneOf[0].properties["file-as"].type, parentSchema: xa.oneOf[1].oneOf[0].properties["file-as"], data: Dr };
          e === null ? e = [ee] : e.push(ee), r++;
        }
        let ta = r;
        var se = wr === r;
      }
      if (l.text !== void 0) {
        let Dr = l.text, wr = r;
        if (typeof Dr != "string") {
          let ee = { instancePath: o + "/text", schemaPath: "#/oneOf/1/oneOf/0/properties/text/type", keyword: "type", params: { type: "string", schema: xa.oneOf[1].oneOf[0].properties.text }, message: "must be string", schema: xa.oneOf[1].oneOf[0].properties.text.type, parentSchema: xa.oneOf[1].oneOf[0].properties.text, data: Dr };
          e === null ? e = [ee] : e.push(ee), r++;
        }
        let ta = r;
        var se = wr === r;
      }
    } else {
      let Dr = { instancePath: o, schemaPath: "#/oneOf/1/oneOf/0/type", keyword: "type", params: { type: "object", schema: xa.oneOf[1].oneOf[0] }, message: "must be object", schema: xa.oneOf[1].oneOf[0].type, parentSchema: xa.oneOf[1].oneOf[0], data: l };
      e === null ? e = [Dr] : e.push(Dr), r++;
    }
    var P = R === r;
    P && ($ = true, m = 0);
    let oe = r, h = r;
    if (Array.isArray(l)) {
      let Dr = true, wr = l.length;
      for (let ta = 0; ta < wr; ta++) {
        let jt = l[ta], ee = r, za = r;
        if (jt && typeof jt == "object" && !Array.isArray(jt)) {
          for (let sa in jt) {
            let _e = r, Ma = r;
            if (typeof sa == "string") {
              if (!w1.test(sa)) {
                let fa = { instancePath: o + "/" + ta, schemaPath: "#/oneOf/1/oneOf/1/items/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(file_as|fileAs))", schema: xa.oneOf[1].oneOf[1].items.propertyNames }, message: 'must match pattern "^(?!(file_as|fileAs))"', schema: "^(?!(file_as|fileAs))", parentSchema: xa.oneOf[1].oneOf[1].items.propertyNames, data: sa, propertyName: sa };
                e === null ? e = [fa] : e.push(fa), r++;
              }
            } else {
              let fa = { instancePath: o + "/" + ta, schemaPath: "#/oneOf/1/oneOf/1/items/propertyNames/type", keyword: "type", params: { type: "string", schema: xa.oneOf[1].oneOf[1].items.propertyNames }, message: "must be string", schema: xa.oneOf[1].oneOf[1].items.propertyNames.type, parentSchema: xa.oneOf[1].oneOf[1].items.propertyNames, data: sa, propertyName: sa };
              e === null ? e = [fa] : e.push(fa), r++;
            }
            var f = _e === r;
            if (!f) {
              let fa = { instancePath: o + "/" + ta, schemaPath: "#/oneOf/1/oneOf/1/items/propertyNames", keyword: "propertyNames", params: { propertyName: sa, schema: xa.oneOf[1].oneOf[1].items }, message: "property name must be valid", schema: xa.oneOf[1].oneOf[1].items.propertyNames, parentSchema: xa.oneOf[1].oneOf[1].items, data: jt };
              e === null ? e = [fa] : e.push(fa), r++;
            }
          }
          if (jt.role !== void 0) {
            let sa = jt.role, _e = r;
            if (typeof sa != "string") {
              let fa = { instancePath: o + "/" + ta + "/role", schemaPath: "#/oneOf/1/oneOf/1/items/properties/role/type", keyword: "type", params: { type: "string", schema: xa.oneOf[1].oneOf[1].items.properties.role }, message: "must be string", schema: xa.oneOf[1].oneOf[1].items.properties.role.type, parentSchema: xa.oneOf[1].oneOf[1].items.properties.role, data: sa };
              e === null ? e = [fa] : e.push(fa), r++;
            }
            let Ma = r;
            var ft = _e === r;
          }
          if (jt["file-as"] !== void 0) {
            let sa = jt["file-as"], _e = r;
            if (typeof sa != "string") {
              let ie = { instancePath: o + "/" + ta + "/file-as", schemaPath: "#/oneOf/1/oneOf/1/items/properties/file-as/type", keyword: "type", params: { type: "string", schema: xa.oneOf[1].oneOf[1].items.properties["file-as"] }, message: "must be string", schema: xa.oneOf[1].oneOf[1].items.properties["file-as"].type, parentSchema: xa.oneOf[1].oneOf[1].items.properties["file-as"], data: sa };
              e === null ? e = [ie] : e.push(ie), r++;
            }
            let Ma = r;
            var ft = _e === r;
          }
          if (jt.text !== void 0) {
            let sa = jt.text, _e = r;
            if (typeof sa != "string") {
              let ie = { instancePath: o + "/" + ta + "/text", schemaPath: "#/oneOf/1/oneOf/1/items/properties/text/type", keyword: "type", params: { type: "string", schema: xa.oneOf[1].oneOf[1].items.properties.text }, message: "must be string", schema: xa.oneOf[1].oneOf[1].items.properties.text.type, parentSchema: xa.oneOf[1].oneOf[1].items.properties.text, data: sa };
              e === null ? e = [ie] : e.push(ie), r++;
            }
            let Ma = r;
            var ft = _e === r;
          }
        } else {
          let sa = { instancePath: o + "/" + ta, schemaPath: "#/oneOf/1/oneOf/1/items/type", keyword: "type", params: { type: "object", schema: xa.oneOf[1].oneOf[1].items }, message: "must be object", schema: xa.oneOf[1].oneOf[1].items.type, parentSchema: xa.oneOf[1].oneOf[1].items, data: jt };
          e === null ? e = [sa] : e.push(sa), r++;
        }
        var Cr = ee === r;
        Cr || (Dr = false);
      }
    } else {
      let Dr = { instancePath: o, schemaPath: "#/oneOf/1/oneOf/1/type", keyword: "type", params: { type: "array", schema: xa.oneOf[1].oneOf[1] }, message: "must be array", schema: xa.oneOf[1].oneOf[1].type, parentSchema: xa.oneOf[1].oneOf[1], data: l };
      e === null ? e = [Dr] : e.push(Dr), r++;
    }
    var P = oe === r;
    if (P && $ ? ($ = false, m = [m, 1]) : P && ($ = true, m = 1), $)
      r = D, e !== null && (D ? e.length = D : e = null);
    else {
      let Dr = { instancePath: o, schemaPath: "#/oneOf/1/oneOf", keyword: "oneOf", params: { passingSchemas: m, schema: xa.oneOf[1] }, message: "must match exactly one schema in oneOf", schema: xa.oneOf[1].oneOf, parentSchema: xa.oneOf[1], data: l };
      e === null ? e = [Dr] : e.push(Dr), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let Dr = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: xa }, message: "must match exactly one schema in oneOf", schema: xa.oneOf, parentSchema: xa, data: l };
      e === null ? e = [Dr] : e.push(Dr), r++;
    }
    return _c.errors = e, r === 0;
  }
  B["format-language"] = $1;
  var jr = { type: "object", description: "be a format language description object", properties: { "toc-title-document": { type: "string", description: "be a string" }, "toc-title-website": { type: "string", description: "be a string" }, "callout-tip-caption": { type: "string", description: "be a string" }, "callout-note-caption": { type: "string", description: "be a string" }, "callout-warning-caption": { type: "string", description: "be a string" }, "callout-important-caption": { type: "string", description: "be a string" }, "callout-danger-caption": { type: "string", description: "be a string" }, "section-title-abstract": { type: "string", description: "be a string" }, "section-title-footnotes": { type: "string", description: "be a string" }, "section-title-appendices": { type: "string", description: "be a string" }, "code-summary": { type: "string", description: "be a string" }, "code-tools-menu-caption": { type: "string", description: "be a string" }, "code-tools-show-all-code": { type: "string", description: "be a string" }, "code-tools-hide-all-code": { type: "string", description: "be a string" }, "code-tools-view-source": { type: "string", description: "be a string" }, "code-tools-source-code": { type: "string", description: "be a string" }, "search-no-results-text": { type: "string", description: "be a string" }, "copy-button-tooltip": { type: "string", description: "be a string" }, "repo-action-links-edit": { type: "string", description: "be a string" }, "repo-action-links-source": { type: "string", description: "be a string" }, "repo-action-links-issue": { type: "string", description: "be a string" }, "search-matching-documents-text": { type: "string", description: "be a string" }, "search-copy-link-title": { type: "string", description: "be a string" }, "search-hide-matches-text": { type: "string", description: "be a string" }, "search-more-match-text": { type: "string", description: "be a string" }, "search-more-matches-text": { type: "string", description: "be a string" }, "search-clear-button-title": { type: "string", description: "be a string" }, "search-detached-cancel-button-title": { type: "string", description: "be a string" }, "search-submit-button-title": { type: "string", description: "be a string" }, "crossref-fig-title": { type: "string", description: "be a string" }, "crossref-tbl-title": { type: "string", description: "be a string" }, "crossref-lst-title": { type: "string", description: "be a string" }, "crossref-thm-title": { type: "string", description: "be a string" }, "crossref-lem-title": { type: "string", description: "be a string" }, "crossref-cor-title": { type: "string", description: "be a string" }, "crossref-prp-title": { type: "string", description: "be a string" }, "crossref-cnj-title": { type: "string", description: "be a string" }, "crossref-def-title": { type: "string", description: "be a string" }, "crossref-exm-title": { type: "string", description: "be a string" }, "crossref-exr-title": { type: "string", description: "be a string" }, "crossref-fig-prefix": { type: "string", description: "be a string" }, "crossref-tbl-prefix": { type: "string", description: "be a string" }, "crossref-lst-prefix": { type: "string", description: "be a string" }, "crossref-sec-prefix": { type: "string", description: "be a string" }, "crossref-eq-prefix": { type: "string", description: "be a string" }, "crossref-thm-prefix": { type: "string", description: "be a string" }, "crossref-lem-prefix": { type: "string", description: "be a string" }, "crossref-cor-prefix": { type: "string", description: "be a string" }, "crossref-prp-prefix": { type: "string", description: "be a string" }, "crossref-cnj-prefix": { type: "string", description: "be a string" }, "crossref-def-prefix": { type: "string", description: "be a string" }, "crossref-exm-prefix": { type: "string", description: "be a string" }, "crossref-exr-prefix": { type: "string", description: "be a string" }, "crossref-lof-title": { type: "string", description: "be a string" }, "crossref-lot-title": { type: "string", description: "be a string" }, "crossref-lol-title": { type: "string", description: "be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(toc_title_document|tocTitleDocument|toc_title_website|tocTitleWebsite|callout_tip_caption|calloutTipCaption|callout_note_caption|calloutNoteCaption|callout_warning_caption|calloutWarningCaption|callout_important_caption|calloutImportantCaption|callout_danger_caption|calloutDangerCaption|section_title_abstract|sectionTitleAbstract|section_title_footnotes|sectionTitleFootnotes|section_title_appendices|sectionTitleAppendices|code_summary|codeSummary|code_tools_menu_caption|codeToolsMenuCaption|code_tools_show_all_code|codeToolsShowAllCode|code_tools_hide_all_code|codeToolsHideAllCode|code_tools_view_source|codeToolsViewSource|code_tools_source_code|codeToolsSourceCode|search_no_results_text|searchNoResultsText|copy_button_tooltip|copyButtonTooltip|repo_action_links_edit|repoActionLinksEdit|repo_action_links_source|repoActionLinksSource|repo_action_links_issue|repoActionLinksIssue|search_matching_documents_text|searchMatchingDocumentsText|search_copy_link_title|searchCopyLinkTitle|search_hide_matches_text|searchHideMatchesText|search_more_match_text|searchMoreMatchText|search_more_matches_text|searchMoreMatchesText|search_clear_button_title|searchClearButtonTitle|search_detached_cancel_button_title|searchDetachedCancelButtonTitle|search_submit_button_title|searchSubmitButtonTitle|crossref_fig_title|crossrefFigTitle|crossref_tbl_title|crossrefTblTitle|crossref_lst_title|crossrefLstTitle|crossref_thm_title|crossrefThmTitle|crossref_lem_title|crossrefLemTitle|crossref_cor_title|crossrefCorTitle|crossref_prp_title|crossrefPrpTitle|crossref_cnj_title|crossrefCnjTitle|crossref_def_title|crossrefDefTitle|crossref_exm_title|crossrefExmTitle|crossref_exr_title|crossrefExrTitle|crossref_fig_prefix|crossrefFigPrefix|crossref_tbl_prefix|crossrefTblPrefix|crossref_lst_prefix|crossrefLstPrefix|crossref_sec_prefix|crossrefSecPrefix|crossref_eq_prefix|crossrefEqPrefix|crossref_thm_prefix|crossrefThmPrefix|crossref_lem_prefix|crossrefLemPrefix|crossref_cor_prefix|crossrefCorPrefix|crossref_prp_prefix|crossrefPrpPrefix|crossref_cnj_prefix|crossrefCnjPrefix|crossref_def_prefix|crossrefDefPrefix|crossref_exm_prefix|crossrefExmPrefix|crossref_exr_prefix|crossrefExrPrefix|crossref_lof_title|crossrefLofTitle|crossref_lot_title|crossrefLotTitle|crossref_lol_title|crossrefLolTitle))" }, $id: "format-language" }, k0 = new RegExp("^(?!(toc_title_document|tocTitleDocument|toc_title_website|tocTitleWebsite|callout_tip_caption|calloutTipCaption|callout_note_caption|calloutNoteCaption|callout_warning_caption|calloutWarningCaption|callout_important_caption|calloutImportantCaption|callout_danger_caption|calloutDangerCaption|section_title_abstract|sectionTitleAbstract|section_title_footnotes|sectionTitleFootnotes|section_title_appendices|sectionTitleAppendices|code_summary|codeSummary|code_tools_menu_caption|codeToolsMenuCaption|code_tools_show_all_code|codeToolsShowAllCode|code_tools_hide_all_code|codeToolsHideAllCode|code_tools_view_source|codeToolsViewSource|code_tools_source_code|codeToolsSourceCode|search_no_results_text|searchNoResultsText|copy_button_tooltip|copyButtonTooltip|repo_action_links_edit|repoActionLinksEdit|repo_action_links_source|repoActionLinksSource|repo_action_links_issue|repoActionLinksIssue|search_matching_documents_text|searchMatchingDocumentsText|search_copy_link_title|searchCopyLinkTitle|search_hide_matches_text|searchHideMatchesText|search_more_match_text|searchMoreMatchText|search_more_matches_text|searchMoreMatchesText|search_clear_button_title|searchClearButtonTitle|search_detached_cancel_button_title|searchDetachedCancelButtonTitle|search_submit_button_title|searchSubmitButtonTitle|crossref_fig_title|crossrefFigTitle|crossref_tbl_title|crossrefTblTitle|crossref_lst_title|crossrefLstTitle|crossref_thm_title|crossrefThmTitle|crossref_lem_title|crossrefLemTitle|crossref_cor_title|crossrefCorTitle|crossref_prp_title|crossrefPrpTitle|crossref_cnj_title|crossrefCnjTitle|crossref_def_title|crossrefDefTitle|crossref_exm_title|crossrefExmTitle|crossref_exr_title|crossrefExrTitle|crossref_fig_prefix|crossrefFigPrefix|crossref_tbl_prefix|crossrefTblPrefix|crossref_lst_prefix|crossrefLstPrefix|crossref_sec_prefix|crossrefSecPrefix|crossref_eq_prefix|crossrefEqPrefix|crossref_thm_prefix|crossrefThmPrefix|crossref_lem_prefix|crossrefLemPrefix|crossref_cor_prefix|crossrefCorPrefix|crossref_prp_prefix|crossrefPrpPrefix|crossref_cnj_prefix|crossrefCnjPrefix|crossref_def_prefix|crossrefDefPrefix|crossref_exm_prefix|crossrefExmPrefix|crossref_exr_prefix|crossrefExrPrefix|crossref_lof_title|crossrefLofTitle|crossref_lot_title|crossrefLotTitle|crossref_lol_title|crossrefLolTitle))", "u");
  function $1(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let u in l) {
        let y = r, g = r;
        if (typeof u == "string") {
          if (!k0.test(u)) {
            let x = { instancePath: o, schemaPath: "#/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(toc_title_document|tocTitleDocument|toc_title_website|tocTitleWebsite|callout_tip_caption|calloutTipCaption|callout_note_caption|calloutNoteCaption|callout_warning_caption|calloutWarningCaption|callout_important_caption|calloutImportantCaption|callout_danger_caption|calloutDangerCaption|section_title_abstract|sectionTitleAbstract|section_title_footnotes|sectionTitleFootnotes|section_title_appendices|sectionTitleAppendices|code_summary|codeSummary|code_tools_menu_caption|codeToolsMenuCaption|code_tools_show_all_code|codeToolsShowAllCode|code_tools_hide_all_code|codeToolsHideAllCode|code_tools_view_source|codeToolsViewSource|code_tools_source_code|codeToolsSourceCode|search_no_results_text|searchNoResultsText|copy_button_tooltip|copyButtonTooltip|repo_action_links_edit|repoActionLinksEdit|repo_action_links_source|repoActionLinksSource|repo_action_links_issue|repoActionLinksIssue|search_matching_documents_text|searchMatchingDocumentsText|search_copy_link_title|searchCopyLinkTitle|search_hide_matches_text|searchHideMatchesText|search_more_match_text|searchMoreMatchText|search_more_matches_text|searchMoreMatchesText|search_clear_button_title|searchClearButtonTitle|search_detached_cancel_button_title|searchDetachedCancelButtonTitle|search_submit_button_title|searchSubmitButtonTitle|crossref_fig_title|crossrefFigTitle|crossref_tbl_title|crossrefTblTitle|crossref_lst_title|crossrefLstTitle|crossref_thm_title|crossrefThmTitle|crossref_lem_title|crossrefLemTitle|crossref_cor_title|crossrefCorTitle|crossref_prp_title|crossrefPrpTitle|crossref_cnj_title|crossrefCnjTitle|crossref_def_title|crossrefDefTitle|crossref_exm_title|crossrefExmTitle|crossref_exr_title|crossrefExrTitle|crossref_fig_prefix|crossrefFigPrefix|crossref_tbl_prefix|crossrefTblPrefix|crossref_lst_prefix|crossrefLstPrefix|crossref_sec_prefix|crossrefSecPrefix|crossref_eq_prefix|crossrefEqPrefix|crossref_thm_prefix|crossrefThmPrefix|crossref_lem_prefix|crossrefLemPrefix|crossref_cor_prefix|crossrefCorPrefix|crossref_prp_prefix|crossrefPrpPrefix|crossref_cnj_prefix|crossrefCnjPrefix|crossref_def_prefix|crossrefDefPrefix|crossref_exm_prefix|crossrefExmPrefix|crossref_exr_prefix|crossrefExrPrefix|crossref_lof_title|crossrefLofTitle|crossref_lot_title|crossrefLotTitle|crossref_lol_title|crossrefLolTitle))", schema: jr.propertyNames }, message: 'must match pattern "^(?!(toc_title_document|tocTitleDocument|toc_title_website|tocTitleWebsite|callout_tip_caption|calloutTipCaption|callout_note_caption|calloutNoteCaption|callout_warning_caption|calloutWarningCaption|callout_important_caption|calloutImportantCaption|callout_danger_caption|calloutDangerCaption|section_title_abstract|sectionTitleAbstract|section_title_footnotes|sectionTitleFootnotes|section_title_appendices|sectionTitleAppendices|code_summary|codeSummary|code_tools_menu_caption|codeToolsMenuCaption|code_tools_show_all_code|codeToolsShowAllCode|code_tools_hide_all_code|codeToolsHideAllCode|code_tools_view_source|codeToolsViewSource|code_tools_source_code|codeToolsSourceCode|search_no_results_text|searchNoResultsText|copy_button_tooltip|copyButtonTooltip|repo_action_links_edit|repoActionLinksEdit|repo_action_links_source|repoActionLinksSource|repo_action_links_issue|repoActionLinksIssue|search_matching_documents_text|searchMatchingDocumentsText|search_copy_link_title|searchCopyLinkTitle|search_hide_matches_text|searchHideMatchesText|search_more_match_text|searchMoreMatchText|search_more_matches_text|searchMoreMatchesText|search_clear_button_title|searchClearButtonTitle|search_detached_cancel_button_title|searchDetachedCancelButtonTitle|search_submit_button_title|searchSubmitButtonTitle|crossref_fig_title|crossrefFigTitle|crossref_tbl_title|crossrefTblTitle|crossref_lst_title|crossrefLstTitle|crossref_thm_title|crossrefThmTitle|crossref_lem_title|crossrefLemTitle|crossref_cor_title|crossrefCorTitle|crossref_prp_title|crossrefPrpTitle|crossref_cnj_title|crossrefCnjTitle|crossref_def_title|crossrefDefTitle|crossref_exm_title|crossrefExmTitle|crossref_exr_title|crossrefExrTitle|crossref_fig_prefix|crossrefFigPrefix|crossref_tbl_prefix|crossrefTblPrefix|crossref_lst_prefix|crossrefLstPrefix|crossref_sec_prefix|crossrefSecPrefix|crossref_eq_prefix|crossrefEqPrefix|crossref_thm_prefix|crossrefThmPrefix|crossref_lem_prefix|crossrefLemPrefix|crossref_cor_prefix|crossrefCorPrefix|crossref_prp_prefix|crossrefPrpPrefix|crossref_cnj_prefix|crossrefCnjPrefix|crossref_def_prefix|crossrefDefPrefix|crossref_exm_prefix|crossrefExmPrefix|crossref_exr_prefix|crossrefExrPrefix|crossref_lof_title|crossrefLofTitle|crossref_lot_title|crossrefLotTitle|crossref_lol_title|crossrefLolTitle))"', schema: "^(?!(toc_title_document|tocTitleDocument|toc_title_website|tocTitleWebsite|callout_tip_caption|calloutTipCaption|callout_note_caption|calloutNoteCaption|callout_warning_caption|calloutWarningCaption|callout_important_caption|calloutImportantCaption|callout_danger_caption|calloutDangerCaption|section_title_abstract|sectionTitleAbstract|section_title_footnotes|sectionTitleFootnotes|section_title_appendices|sectionTitleAppendices|code_summary|codeSummary|code_tools_menu_caption|codeToolsMenuCaption|code_tools_show_all_code|codeToolsShowAllCode|code_tools_hide_all_code|codeToolsHideAllCode|code_tools_view_source|codeToolsViewSource|code_tools_source_code|codeToolsSourceCode|search_no_results_text|searchNoResultsText|copy_button_tooltip|copyButtonTooltip|repo_action_links_edit|repoActionLinksEdit|repo_action_links_source|repoActionLinksSource|repo_action_links_issue|repoActionLinksIssue|search_matching_documents_text|searchMatchingDocumentsText|search_copy_link_title|searchCopyLinkTitle|search_hide_matches_text|searchHideMatchesText|search_more_match_text|searchMoreMatchText|search_more_matches_text|searchMoreMatchesText|search_clear_button_title|searchClearButtonTitle|search_detached_cancel_button_title|searchDetachedCancelButtonTitle|search_submit_button_title|searchSubmitButtonTitle|crossref_fig_title|crossrefFigTitle|crossref_tbl_title|crossrefTblTitle|crossref_lst_title|crossrefLstTitle|crossref_thm_title|crossrefThmTitle|crossref_lem_title|crossrefLemTitle|crossref_cor_title|crossrefCorTitle|crossref_prp_title|crossrefPrpTitle|crossref_cnj_title|crossrefCnjTitle|crossref_def_title|crossrefDefTitle|crossref_exm_title|crossrefExmTitle|crossref_exr_title|crossrefExrTitle|crossref_fig_prefix|crossrefFigPrefix|crossref_tbl_prefix|crossrefTblPrefix|crossref_lst_prefix|crossrefLstPrefix|crossref_sec_prefix|crossrefSecPrefix|crossref_eq_prefix|crossrefEqPrefix|crossref_thm_prefix|crossrefThmPrefix|crossref_lem_prefix|crossrefLemPrefix|crossref_cor_prefix|crossrefCorPrefix|crossref_prp_prefix|crossrefPrpPrefix|crossref_cnj_prefix|crossrefCnjPrefix|crossref_def_prefix|crossrefDefPrefix|crossref_exm_prefix|crossrefExmPrefix|crossref_exr_prefix|crossrefExrPrefix|crossref_lof_title|crossrefLofTitle|crossref_lot_title|crossrefLotTitle|crossref_lol_title|crossrefLolTitle))", parentSchema: jr.propertyNames, data: u, propertyName: u };
            e === null ? e = [x] : e.push(x), r++;
          }
        } else {
          let x = { instancePath: o, schemaPath: "#/propertyNames/type", keyword: "type", params: { type: "string", schema: jr.propertyNames }, message: "must be string", schema: jr.propertyNames.type, parentSchema: jr.propertyNames, data: u, propertyName: u };
          e === null ? e = [x] : e.push(x), r++;
        }
        var c = y === r;
        if (!c) {
          let x = { instancePath: o, schemaPath: "#/propertyNames", keyword: "propertyNames", params: { propertyName: u, schema: jr }, message: "property name must be valid", schema: jr.propertyNames, parentSchema: jr, data: l };
          e === null ? e = [x] : e.push(x), r++;
        }
      }
      if (l["toc-title-document"] !== void 0) {
        let u = l["toc-title-document"], y = r;
        if (typeof u != "string") {
          let x = { instancePath: o + "/toc-title-document", schemaPath: "#/properties/toc-title-document/type", keyword: "type", params: { type: "string", schema: jr.properties["toc-title-document"] }, message: "must be string", schema: jr.properties["toc-title-document"].type, parentSchema: jr.properties["toc-title-document"], data: u };
          e === null ? e = [x] : e.push(x), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["toc-title-website"] !== void 0) {
        let u = l["toc-title-website"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/toc-title-website", schemaPath: "#/properties/toc-title-website/type", keyword: "type", params: { type: "string", schema: jr.properties["toc-title-website"] }, message: "must be string", schema: jr.properties["toc-title-website"].type, parentSchema: jr.properties["toc-title-website"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["callout-tip-caption"] !== void 0) {
        let u = l["callout-tip-caption"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/callout-tip-caption", schemaPath: "#/properties/callout-tip-caption/type", keyword: "type", params: { type: "string", schema: jr.properties["callout-tip-caption"] }, message: "must be string", schema: jr.properties["callout-tip-caption"].type, parentSchema: jr.properties["callout-tip-caption"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["callout-note-caption"] !== void 0) {
        let u = l["callout-note-caption"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/callout-note-caption", schemaPath: "#/properties/callout-note-caption/type", keyword: "type", params: { type: "string", schema: jr.properties["callout-note-caption"] }, message: "must be string", schema: jr.properties["callout-note-caption"].type, parentSchema: jr.properties["callout-note-caption"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["callout-warning-caption"] !== void 0) {
        let u = l["callout-warning-caption"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/callout-warning-caption", schemaPath: "#/properties/callout-warning-caption/type", keyword: "type", params: { type: "string", schema: jr.properties["callout-warning-caption"] }, message: "must be string", schema: jr.properties["callout-warning-caption"].type, parentSchema: jr.properties["callout-warning-caption"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["callout-important-caption"] !== void 0) {
        let u = l["callout-important-caption"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/callout-important-caption", schemaPath: "#/properties/callout-important-caption/type", keyword: "type", params: { type: "string", schema: jr.properties["callout-important-caption"] }, message: "must be string", schema: jr.properties["callout-important-caption"].type, parentSchema: jr.properties["callout-important-caption"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["callout-danger-caption"] !== void 0) {
        let u = l["callout-danger-caption"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/callout-danger-caption", schemaPath: "#/properties/callout-danger-caption/type", keyword: "type", params: { type: "string", schema: jr.properties["callout-danger-caption"] }, message: "must be string", schema: jr.properties["callout-danger-caption"].type, parentSchema: jr.properties["callout-danger-caption"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["section-title-abstract"] !== void 0) {
        let u = l["section-title-abstract"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/section-title-abstract", schemaPath: "#/properties/section-title-abstract/type", keyword: "type", params: { type: "string", schema: jr.properties["section-title-abstract"] }, message: "must be string", schema: jr.properties["section-title-abstract"].type, parentSchema: jr.properties["section-title-abstract"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["section-title-footnotes"] !== void 0) {
        let u = l["section-title-footnotes"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/section-title-footnotes", schemaPath: "#/properties/section-title-footnotes/type", keyword: "type", params: { type: "string", schema: jr.properties["section-title-footnotes"] }, message: "must be string", schema: jr.properties["section-title-footnotes"].type, parentSchema: jr.properties["section-title-footnotes"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["section-title-appendices"] !== void 0) {
        let u = l["section-title-appendices"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/section-title-appendices", schemaPath: "#/properties/section-title-appendices/type", keyword: "type", params: { type: "string", schema: jr.properties["section-title-appendices"] }, message: "must be string", schema: jr.properties["section-title-appendices"].type, parentSchema: jr.properties["section-title-appendices"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["code-summary"] !== void 0) {
        let u = l["code-summary"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/code-summary", schemaPath: "#/properties/code-summary/type", keyword: "type", params: { type: "string", schema: jr.properties["code-summary"] }, message: "must be string", schema: jr.properties["code-summary"].type, parentSchema: jr.properties["code-summary"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["code-tools-menu-caption"] !== void 0) {
        let u = l["code-tools-menu-caption"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/code-tools-menu-caption", schemaPath: "#/properties/code-tools-menu-caption/type", keyword: "type", params: { type: "string", schema: jr.properties["code-tools-menu-caption"] }, message: "must be string", schema: jr.properties["code-tools-menu-caption"].type, parentSchema: jr.properties["code-tools-menu-caption"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["code-tools-show-all-code"] !== void 0) {
        let u = l["code-tools-show-all-code"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/code-tools-show-all-code", schemaPath: "#/properties/code-tools-show-all-code/type", keyword: "type", params: { type: "string", schema: jr.properties["code-tools-show-all-code"] }, message: "must be string", schema: jr.properties["code-tools-show-all-code"].type, parentSchema: jr.properties["code-tools-show-all-code"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["code-tools-hide-all-code"] !== void 0) {
        let u = l["code-tools-hide-all-code"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/code-tools-hide-all-code", schemaPath: "#/properties/code-tools-hide-all-code/type", keyword: "type", params: { type: "string", schema: jr.properties["code-tools-hide-all-code"] }, message: "must be string", schema: jr.properties["code-tools-hide-all-code"].type, parentSchema: jr.properties["code-tools-hide-all-code"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["code-tools-view-source"] !== void 0) {
        let u = l["code-tools-view-source"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/code-tools-view-source", schemaPath: "#/properties/code-tools-view-source/type", keyword: "type", params: { type: "string", schema: jr.properties["code-tools-view-source"] }, message: "must be string", schema: jr.properties["code-tools-view-source"].type, parentSchema: jr.properties["code-tools-view-source"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["code-tools-source-code"] !== void 0) {
        let u = l["code-tools-source-code"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/code-tools-source-code", schemaPath: "#/properties/code-tools-source-code/type", keyword: "type", params: { type: "string", schema: jr.properties["code-tools-source-code"] }, message: "must be string", schema: jr.properties["code-tools-source-code"].type, parentSchema: jr.properties["code-tools-source-code"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["search-no-results-text"] !== void 0) {
        let u = l["search-no-results-text"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/search-no-results-text", schemaPath: "#/properties/search-no-results-text/type", keyword: "type", params: { type: "string", schema: jr.properties["search-no-results-text"] }, message: "must be string", schema: jr.properties["search-no-results-text"].type, parentSchema: jr.properties["search-no-results-text"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["copy-button-tooltip"] !== void 0) {
        let u = l["copy-button-tooltip"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/copy-button-tooltip", schemaPath: "#/properties/copy-button-tooltip/type", keyword: "type", params: { type: "string", schema: jr.properties["copy-button-tooltip"] }, message: "must be string", schema: jr.properties["copy-button-tooltip"].type, parentSchema: jr.properties["copy-button-tooltip"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["repo-action-links-edit"] !== void 0) {
        let u = l["repo-action-links-edit"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/repo-action-links-edit", schemaPath: "#/properties/repo-action-links-edit/type", keyword: "type", params: { type: "string", schema: jr.properties["repo-action-links-edit"] }, message: "must be string", schema: jr.properties["repo-action-links-edit"].type, parentSchema: jr.properties["repo-action-links-edit"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["repo-action-links-source"] !== void 0) {
        let u = l["repo-action-links-source"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/repo-action-links-source", schemaPath: "#/properties/repo-action-links-source/type", keyword: "type", params: { type: "string", schema: jr.properties["repo-action-links-source"] }, message: "must be string", schema: jr.properties["repo-action-links-source"].type, parentSchema: jr.properties["repo-action-links-source"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["repo-action-links-issue"] !== void 0) {
        let u = l["repo-action-links-issue"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/repo-action-links-issue", schemaPath: "#/properties/repo-action-links-issue/type", keyword: "type", params: { type: "string", schema: jr.properties["repo-action-links-issue"] }, message: "must be string", schema: jr.properties["repo-action-links-issue"].type, parentSchema: jr.properties["repo-action-links-issue"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["search-matching-documents-text"] !== void 0) {
        let u = l["search-matching-documents-text"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/search-matching-documents-text", schemaPath: "#/properties/search-matching-documents-text/type", keyword: "type", params: { type: "string", schema: jr.properties["search-matching-documents-text"] }, message: "must be string", schema: jr.properties["search-matching-documents-text"].type, parentSchema: jr.properties["search-matching-documents-text"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["search-copy-link-title"] !== void 0) {
        let u = l["search-copy-link-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/search-copy-link-title", schemaPath: "#/properties/search-copy-link-title/type", keyword: "type", params: { type: "string", schema: jr.properties["search-copy-link-title"] }, message: "must be string", schema: jr.properties["search-copy-link-title"].type, parentSchema: jr.properties["search-copy-link-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["search-hide-matches-text"] !== void 0) {
        let u = l["search-hide-matches-text"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/search-hide-matches-text", schemaPath: "#/properties/search-hide-matches-text/type", keyword: "type", params: { type: "string", schema: jr.properties["search-hide-matches-text"] }, message: "must be string", schema: jr.properties["search-hide-matches-text"].type, parentSchema: jr.properties["search-hide-matches-text"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["search-more-match-text"] !== void 0) {
        let u = l["search-more-match-text"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/search-more-match-text", schemaPath: "#/properties/search-more-match-text/type", keyword: "type", params: { type: "string", schema: jr.properties["search-more-match-text"] }, message: "must be string", schema: jr.properties["search-more-match-text"].type, parentSchema: jr.properties["search-more-match-text"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["search-more-matches-text"] !== void 0) {
        let u = l["search-more-matches-text"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/search-more-matches-text", schemaPath: "#/properties/search-more-matches-text/type", keyword: "type", params: { type: "string", schema: jr.properties["search-more-matches-text"] }, message: "must be string", schema: jr.properties["search-more-matches-text"].type, parentSchema: jr.properties["search-more-matches-text"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["search-clear-button-title"] !== void 0) {
        let u = l["search-clear-button-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/search-clear-button-title", schemaPath: "#/properties/search-clear-button-title/type", keyword: "type", params: { type: "string", schema: jr.properties["search-clear-button-title"] }, message: "must be string", schema: jr.properties["search-clear-button-title"].type, parentSchema: jr.properties["search-clear-button-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["search-detached-cancel-button-title"] !== void 0) {
        let u = l["search-detached-cancel-button-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/search-detached-cancel-button-title", schemaPath: "#/properties/search-detached-cancel-button-title/type", keyword: "type", params: { type: "string", schema: jr.properties["search-detached-cancel-button-title"] }, message: "must be string", schema: jr.properties["search-detached-cancel-button-title"].type, parentSchema: jr.properties["search-detached-cancel-button-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["search-submit-button-title"] !== void 0) {
        let u = l["search-submit-button-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/search-submit-button-title", schemaPath: "#/properties/search-submit-button-title/type", keyword: "type", params: { type: "string", schema: jr.properties["search-submit-button-title"] }, message: "must be string", schema: jr.properties["search-submit-button-title"].type, parentSchema: jr.properties["search-submit-button-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-fig-title"] !== void 0) {
        let u = l["crossref-fig-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-fig-title", schemaPath: "#/properties/crossref-fig-title/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-fig-title"] }, message: "must be string", schema: jr.properties["crossref-fig-title"].type, parentSchema: jr.properties["crossref-fig-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-tbl-title"] !== void 0) {
        let u = l["crossref-tbl-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-tbl-title", schemaPath: "#/properties/crossref-tbl-title/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-tbl-title"] }, message: "must be string", schema: jr.properties["crossref-tbl-title"].type, parentSchema: jr.properties["crossref-tbl-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-lst-title"] !== void 0) {
        let u = l["crossref-lst-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-lst-title", schemaPath: "#/properties/crossref-lst-title/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-lst-title"] }, message: "must be string", schema: jr.properties["crossref-lst-title"].type, parentSchema: jr.properties["crossref-lst-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-thm-title"] !== void 0) {
        let u = l["crossref-thm-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-thm-title", schemaPath: "#/properties/crossref-thm-title/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-thm-title"] }, message: "must be string", schema: jr.properties["crossref-thm-title"].type, parentSchema: jr.properties["crossref-thm-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-lem-title"] !== void 0) {
        let u = l["crossref-lem-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-lem-title", schemaPath: "#/properties/crossref-lem-title/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-lem-title"] }, message: "must be string", schema: jr.properties["crossref-lem-title"].type, parentSchema: jr.properties["crossref-lem-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-cor-title"] !== void 0) {
        let u = l["crossref-cor-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-cor-title", schemaPath: "#/properties/crossref-cor-title/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-cor-title"] }, message: "must be string", schema: jr.properties["crossref-cor-title"].type, parentSchema: jr.properties["crossref-cor-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-prp-title"] !== void 0) {
        let u = l["crossref-prp-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-prp-title", schemaPath: "#/properties/crossref-prp-title/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-prp-title"] }, message: "must be string", schema: jr.properties["crossref-prp-title"].type, parentSchema: jr.properties["crossref-prp-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-cnj-title"] !== void 0) {
        let u = l["crossref-cnj-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-cnj-title", schemaPath: "#/properties/crossref-cnj-title/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-cnj-title"] }, message: "must be string", schema: jr.properties["crossref-cnj-title"].type, parentSchema: jr.properties["crossref-cnj-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-def-title"] !== void 0) {
        let u = l["crossref-def-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-def-title", schemaPath: "#/properties/crossref-def-title/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-def-title"] }, message: "must be string", schema: jr.properties["crossref-def-title"].type, parentSchema: jr.properties["crossref-def-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-exm-title"] !== void 0) {
        let u = l["crossref-exm-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-exm-title", schemaPath: "#/properties/crossref-exm-title/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-exm-title"] }, message: "must be string", schema: jr.properties["crossref-exm-title"].type, parentSchema: jr.properties["crossref-exm-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-exr-title"] !== void 0) {
        let u = l["crossref-exr-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-exr-title", schemaPath: "#/properties/crossref-exr-title/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-exr-title"] }, message: "must be string", schema: jr.properties["crossref-exr-title"].type, parentSchema: jr.properties["crossref-exr-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-fig-prefix"] !== void 0) {
        let u = l["crossref-fig-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-fig-prefix", schemaPath: "#/properties/crossref-fig-prefix/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-fig-prefix"] }, message: "must be string", schema: jr.properties["crossref-fig-prefix"].type, parentSchema: jr.properties["crossref-fig-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-tbl-prefix"] !== void 0) {
        let u = l["crossref-tbl-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-tbl-prefix", schemaPath: "#/properties/crossref-tbl-prefix/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-tbl-prefix"] }, message: "must be string", schema: jr.properties["crossref-tbl-prefix"].type, parentSchema: jr.properties["crossref-tbl-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-lst-prefix"] !== void 0) {
        let u = l["crossref-lst-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-lst-prefix", schemaPath: "#/properties/crossref-lst-prefix/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-lst-prefix"] }, message: "must be string", schema: jr.properties["crossref-lst-prefix"].type, parentSchema: jr.properties["crossref-lst-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-sec-prefix"] !== void 0) {
        let u = l["crossref-sec-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-sec-prefix", schemaPath: "#/properties/crossref-sec-prefix/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-sec-prefix"] }, message: "must be string", schema: jr.properties["crossref-sec-prefix"].type, parentSchema: jr.properties["crossref-sec-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-eq-prefix"] !== void 0) {
        let u = l["crossref-eq-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-eq-prefix", schemaPath: "#/properties/crossref-eq-prefix/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-eq-prefix"] }, message: "must be string", schema: jr.properties["crossref-eq-prefix"].type, parentSchema: jr.properties["crossref-eq-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-thm-prefix"] !== void 0) {
        let u = l["crossref-thm-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-thm-prefix", schemaPath: "#/properties/crossref-thm-prefix/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-thm-prefix"] }, message: "must be string", schema: jr.properties["crossref-thm-prefix"].type, parentSchema: jr.properties["crossref-thm-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-lem-prefix"] !== void 0) {
        let u = l["crossref-lem-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-lem-prefix", schemaPath: "#/properties/crossref-lem-prefix/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-lem-prefix"] }, message: "must be string", schema: jr.properties["crossref-lem-prefix"].type, parentSchema: jr.properties["crossref-lem-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-cor-prefix"] !== void 0) {
        let u = l["crossref-cor-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-cor-prefix", schemaPath: "#/properties/crossref-cor-prefix/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-cor-prefix"] }, message: "must be string", schema: jr.properties["crossref-cor-prefix"].type, parentSchema: jr.properties["crossref-cor-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-prp-prefix"] !== void 0) {
        let u = l["crossref-prp-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-prp-prefix", schemaPath: "#/properties/crossref-prp-prefix/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-prp-prefix"] }, message: "must be string", schema: jr.properties["crossref-prp-prefix"].type, parentSchema: jr.properties["crossref-prp-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-cnj-prefix"] !== void 0) {
        let u = l["crossref-cnj-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-cnj-prefix", schemaPath: "#/properties/crossref-cnj-prefix/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-cnj-prefix"] }, message: "must be string", schema: jr.properties["crossref-cnj-prefix"].type, parentSchema: jr.properties["crossref-cnj-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-def-prefix"] !== void 0) {
        let u = l["crossref-def-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-def-prefix", schemaPath: "#/properties/crossref-def-prefix/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-def-prefix"] }, message: "must be string", schema: jr.properties["crossref-def-prefix"].type, parentSchema: jr.properties["crossref-def-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-exm-prefix"] !== void 0) {
        let u = l["crossref-exm-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-exm-prefix", schemaPath: "#/properties/crossref-exm-prefix/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-exm-prefix"] }, message: "must be string", schema: jr.properties["crossref-exm-prefix"].type, parentSchema: jr.properties["crossref-exm-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-exr-prefix"] !== void 0) {
        let u = l["crossref-exr-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-exr-prefix", schemaPath: "#/properties/crossref-exr-prefix/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-exr-prefix"] }, message: "must be string", schema: jr.properties["crossref-exr-prefix"].type, parentSchema: jr.properties["crossref-exr-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-lof-title"] !== void 0) {
        let u = l["crossref-lof-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-lof-title", schemaPath: "#/properties/crossref-lof-title/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-lof-title"] }, message: "must be string", schema: jr.properties["crossref-lof-title"].type, parentSchema: jr.properties["crossref-lof-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-lot-title"] !== void 0) {
        let u = l["crossref-lot-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-lot-title", schemaPath: "#/properties/crossref-lot-title/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-lot-title"] }, message: "must be string", schema: jr.properties["crossref-lot-title"].type, parentSchema: jr.properties["crossref-lot-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["crossref-lol-title"] !== void 0) {
        let u = l["crossref-lol-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/crossref-lol-title", schemaPath: "#/properties/crossref-lol-title/type", keyword: "type", params: { type: "string", schema: jr.properties["crossref-lol-title"] }, message: "must be string", schema: jr.properties["crossref-lol-title"].type, parentSchema: jr.properties["crossref-lol-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
    } else {
      let u = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: jr }, message: "must be object", schema: jr.type, parentSchema: jr, data: l };
      e === null ? e = [u] : e.push(u), r++;
    }
    return $1.errors = e, r === 0;
  }
  B["quarto-resource-cell-attributes-label"] = pl;
  var pg = { type: "string", description: "be a string", $id: "quarto-resource-cell-attributes-label" };
  function pl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: pg }, message: "must be string", schema: pg.type, parentSchema: pg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return pl.errors = e, r === 0;
  }
  B["quarto-resource-cell-attributes-classes"] = ul;
  var ug = { type: "string", description: "be a string", $id: "quarto-resource-cell-attributes-classes" };
  function ul(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: ug }, message: "must be string", schema: ug.type, parentSchema: ug, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ul.errors = e, r === 0;
  }
  B["quarto-resource-cell-attributes-tags"] = Md;
  var qd = { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" }, $id: "quarto-resource-cell-attributes-tags" };
  function Md(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let d = true, u = l.length;
      for (let y = 0; y < u; y++) {
        let g = l[y], x = r;
        if (typeof g != "string") {
          let $ = { instancePath: o + "/" + y, schemaPath: "#/items/type", keyword: "type", params: { type: "string", schema: qd.items }, message: "must be string", schema: qd.items.type, parentSchema: qd.items, data: g };
          e === null ? e = [$] : e.push($), r++;
        }
        let D = r;
        var c = x === r;
        c || (d = false);
      }
    } else {
      let d = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: qd }, message: "must be array", schema: qd.type, parentSchema: qd, data: l };
      e === null ? e = [d] : e.push(d), r++;
    }
    return Md.errors = e, r === 0;
  }
  B["quarto-resource-cell-attributes-id"] = Td;
  var dg = { type: "string", description: "be a string", $id: "quarto-resource-cell-attributes-id" };
  function Td(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: dg }, message: "must be string", schema: dg.type, parentSchema: dg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Td.errors = e, r === 0;
  }
  B["quarto-resource-cell-cache-cache"] = zd;
  var fg = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-cache-cache" };
  function zd(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: fg }, message: "must be boolean", schema: fg.type, parentSchema: fg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return zd.errors = e, r === 0;
  }
  B["quarto-resource-cell-cache-cache-path"] = Wd;
  var mg = { type: "string", description: "be a string", $id: "quarto-resource-cell-cache-cache-path" };
  function Wd(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: mg }, message: "must be string", schema: mg.type, parentSchema: mg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Wd.errors = e, r === 0;
  }
  B["quarto-resource-cell-cache-cache-vars"] = Rd;
  var ui = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-cache-cache-vars" };
  function Rd(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: ui.oneOf[0] }, message: "must be string", schema: ui.oneOf[0].type, parentSchema: ui.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: ui.oneOf[1].items }, message: "must be string", schema: ui.oneOf[1].items.type, parentSchema: ui.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: ui.oneOf[1] }, message: "must be array", schema: ui.oneOf[1].type, parentSchema: ui.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: ui }, message: "must match exactly one schema in oneOf", schema: ui.oneOf, parentSchema: ui, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Rd.errors = e, r === 0;
  }
  B["quarto-resource-cell-cache-cache-globals"] = Vd;
  var gg = { type: "string", description: "be a string", $id: "quarto-resource-cell-cache-cache-globals" };
  function Vd(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: gg }, message: "must be string", schema: gg.type, parentSchema: gg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Vd.errors = e, r === 0;
  }
  B["quarto-resource-cell-cache-cache-lazy"] = Ud;
  var hg = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-cache-cache-lazy" };
  function Ud(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: hg }, message: "must be boolean", schema: hg.type, parentSchema: hg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ud.errors = e, r === 0;
  }
  B["quarto-resource-cell-cache-cache-rebuild"] = Kd;
  var yg = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-cache-cache-rebuild" };
  function Kd(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: yg }, message: "must be boolean", schema: yg.type, parentSchema: yg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Kd.errors = e, r === 0;
  }
  B["quarto-resource-cell-cache-cache-comments"] = Jd;
  var bg = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-cache-cache-comments" };
  function Jd(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: bg }, message: "must be boolean", schema: bg.type, parentSchema: bg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Jd.errors = e, r === 0;
  }
  B["quarto-resource-cell-cache-dependson"] = Qd;
  var Mn = { oneOf: [{ oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, { oneOf: [{ type: "number", description: "be a number" }, { type: "array", description: "be an array of values, where each element should be a number", items: { type: "number", description: "be a number" } }], description: "be exactly one of: a number, an array of values, where each element should be a number" }], description: "be exactly one of: exactly one of: a string, an array of values, where each element should be a string, exactly one of: a number, an array of values, where each element should be a number", $id: "quarto-resource-cell-cache-dependson" };
  function Qd(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r, y = r, g = false, x = null, D = r;
    if (typeof l != "string") {
      let ee = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/0/type", keyword: "type", params: { type: "string", schema: Mn.oneOf[0].oneOf[0] }, message: "must be string", schema: Mn.oneOf[0].oneOf[0].type, parentSchema: Mn.oneOf[0].oneOf[0], data: l };
      e === null ? e = [ee] : e.push(ee), r++;
    }
    let $ = r;
    var m = D === r;
    m && (g = true, x = 0);
    let R = r, j = r;
    if (Array.isArray(l)) {
      let ee = true, za = l.length;
      for (let sa = 0; sa < za; sa++) {
        let _e = l[sa], Ma = r;
        if (typeof _e != "string") {
          let ie = { instancePath: o + "/" + sa, schemaPath: "#/oneOf/0/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Mn.oneOf[0].oneOf[1].items }, message: "must be string", schema: Mn.oneOf[0].oneOf[1].items.type, parentSchema: Mn.oneOf[0].oneOf[1].items, data: _e };
          e === null ? e = [ie] : e.push(ie), r++;
        }
        let fa = r;
        var V = Ma === r;
        V || (ee = false);
      }
    } else {
      let ee = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/1/type", keyword: "type", params: { type: "array", schema: Mn.oneOf[0].oneOf[1] }, message: "must be array", schema: Mn.oneOf[0].oneOf[1].type, parentSchema: Mn.oneOf[0].oneOf[1], data: l };
      e === null ? e = [ee] : e.push(ee), r++;
    }
    var m = R === r;
    if (m && g ? (g = false, x = [x, 1]) : m && (g = true, x = 1), g)
      r = y, e !== null && (y ? e.length = y : e = null);
    else {
      let ee = { instancePath: o, schemaPath: "#/oneOf/0/oneOf", keyword: "oneOf", params: { passingSchemas: x, schema: Mn.oneOf[0] }, message: "must match exactly one schema in oneOf", schema: Mn.oneOf[0].oneOf, parentSchema: Mn.oneOf[0], data: l };
      e === null ? e = [ee] : e.push(ee), r++;
    }
    var se = u === r;
    se && (c = true, d = 0);
    let P = r, oe = r, h = false, f = null, ft = r;
    if (!(typeof l == "number" && isFinite(l))) {
      let ee = { instancePath: o, schemaPath: "#/oneOf/1/oneOf/0/type", keyword: "type", params: { type: "number", schema: Mn.oneOf[1].oneOf[0] }, message: "must be number", schema: Mn.oneOf[1].oneOf[0].type, parentSchema: Mn.oneOf[1].oneOf[0], data: l };
      e === null ? e = [ee] : e.push(ee), r++;
    }
    let Cr = r;
    var Dr = ft === r;
    Dr && (h = true, f = 0);
    let wr = r, ta = r;
    if (Array.isArray(l)) {
      let ee = true, za = l.length;
      for (let sa = 0; sa < za; sa++) {
        let _e = l[sa], Ma = r;
        if (!(typeof _e == "number" && isFinite(_e))) {
          let ie = { instancePath: o + "/" + sa, schemaPath: "#/oneOf/1/oneOf/1/items/type", keyword: "type", params: { type: "number", schema: Mn.oneOf[1].oneOf[1].items }, message: "must be number", schema: Mn.oneOf[1].oneOf[1].items.type, parentSchema: Mn.oneOf[1].oneOf[1].items, data: _e };
          e === null ? e = [ie] : e.push(ie), r++;
        }
        let fa = r;
        var jt = Ma === r;
        jt || (ee = false);
      }
    } else {
      let ee = { instancePath: o, schemaPath: "#/oneOf/1/oneOf/1/type", keyword: "type", params: { type: "array", schema: Mn.oneOf[1].oneOf[1] }, message: "must be array", schema: Mn.oneOf[1].oneOf[1].type, parentSchema: Mn.oneOf[1].oneOf[1], data: l };
      e === null ? e = [ee] : e.push(ee), r++;
    }
    var Dr = wr === r;
    if (Dr && h ? (h = false, f = [f, 1]) : Dr && (h = true, f = 1), h)
      r = oe, e !== null && (oe ? e.length = oe : e = null);
    else {
      let ee = { instancePath: o, schemaPath: "#/oneOf/1/oneOf", keyword: "oneOf", params: { passingSchemas: f, schema: Mn.oneOf[1] }, message: "must match exactly one schema in oneOf", schema: Mn.oneOf[1].oneOf, parentSchema: Mn.oneOf[1], data: l };
      e === null ? e = [ee] : e.push(ee), r++;
    }
    var se = P === r;
    if (se && c ? (c = false, d = [d, 1]) : se && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let ee = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Mn }, message: "must match exactly one schema in oneOf", schema: Mn.oneOf, parentSchema: Mn, data: l };
      e === null ? e = [ee] : e.push(ee), r++;
    }
    return Qd.errors = e, r === 0;
  }
  B["quarto-resource-cell-cache-autodep"] = Gd;
  var _g = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-cache-autodep" };
  function Gd(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: _g }, message: "must be boolean", schema: _g.type, parentSchema: _g, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Gd.errors = e, r === 0;
  }
  B["quarto-resource-cell-codeoutput-eval"] = ue;
  var vg = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-codeoutput-eval" };
  function ue(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: vg }, message: "must be boolean", schema: vg.type, parentSchema: vg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ue.errors = e, r === 0;
  }
  B["quarto-resource-cell-codeoutput-echo"] = de;
  var Xl = { anyOf: [{ type: "boolean", description: "be `true` or `false`" }, { enum: ["fenced"], description: "be 'fenced'" }], description: "be `true`, `false`, or `fenced`", $id: "quarto-resource-cell-codeoutput-echo" };
  function de(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = r;
    if (typeof l != "boolean") {
      let g = { instancePath: o, schemaPath: "#/anyOf/0/type", keyword: "type", params: { type: "boolean", schema: Xl.anyOf[0] }, message: "must be boolean", schema: Xl.anyOf[0].type, parentSchema: Xl.anyOf[0], data: l };
      e === null ? e = [g] : e.push(g), r++;
    }
    let u = r;
    var y = d === r;
    if (c = c || y, !c) {
      let g = r, x = Xl.anyOf[1].enum;
      if (l !== "fenced") {
        let $ = { instancePath: o, schemaPath: "#/anyOf/1/enum", keyword: "enum", params: { allowedValues: Xl.anyOf[1].enum, schema: Xl.anyOf[1] }, message: "must be equal to one of the allowed values", schema: Xl.anyOf[1].enum, parentSchema: Xl.anyOf[1], data: l };
        e === null ? e = [$] : e.push($), r++;
      }
      var y = g === r;
      c = c || y;
    }
    if (c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let g = { instancePath: o, schemaPath: "#/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: Xl.anyOf, parentSchema: Xl, data: l };
      e === null ? e = [g] : e.push(g), r++;
    }
    return de.errors = e, r === 0;
  }
  B["quarto-resource-cell-codeoutput-code-fold"] = Fa;
  var dl = { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { enum: ["show"], description: "be 'show'" }], description: "be exactly one of: `true` or `false`, 'show'", $id: "quarto-resource-cell-codeoutput-code-fold" };
  function Fa(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "boolean") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: dl.oneOf[0] }, message: "must be boolean", schema: dl.oneOf[0].type, parentSchema: dl.oneOf[0], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = dl.oneOf[1].enum;
    if (l !== "show") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/1/enum", keyword: "enum", params: { allowedValues: dl.oneOf[1].enum, schema: dl.oneOf[1] }, message: "must be equal to one of the allowed values", schema: dl.oneOf[1].enum, parentSchema: dl.oneOf[1], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: dl }, message: "must match exactly one schema in oneOf", schema: dl.oneOf, parentSchema: dl, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return Fa.errors = e, r === 0;
  }
  B["quarto-resource-cell-codeoutput-code-summary"] = Aa;
  var qg = { type: "string", description: "be a string", $id: "quarto-resource-cell-codeoutput-code-summary" };
  function Aa(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: qg }, message: "must be string", schema: qg.type, parentSchema: qg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Aa.errors = e, r === 0;
  }
  B["quarto-resource-cell-codeoutput-code-overflow"] = Ba;
  var Yd = { enum: ["scroll", "wrap"], description: "be one of: `scroll`, `wrap`", $id: "quarto-resource-cell-codeoutput-code-overflow" };
  function Ba(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Yd.enum;
    if (!(l === "scroll" || l === "wrap")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Yd.enum, schema: Yd }, message: "must be equal to one of the allowed values", schema: Yd.enum, parentSchema: Yd, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Ba.errors = e, r === 0;
  }
  B["quarto-resource-cell-codeoutput-code-line-numbers"] = ua;
  var oc = { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "string", description: "be a string" }], description: "be exactly one of: `true` or `false`, a string", $id: "quarto-resource-cell-codeoutput-code-line-numbers" };
  function ua(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "boolean") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: oc.oneOf[0] }, message: "must be boolean", schema: oc.oneOf[0].type, parentSchema: oc.oneOf[0], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r;
    if (typeof l != "string") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "string", schema: oc.oneOf[1] }, message: "must be string", schema: oc.oneOf[1].type, parentSchema: oc.oneOf[1], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let D = r;
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: oc }, message: "must match exactly one schema in oneOf", schema: oc.oneOf, parentSchema: oc, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return ua.errors = e, r === 0;
  }
  B["quarto-resource-cell-codeoutput-lst-label"] = fl;
  var Og = { type: "string", description: "be a string", $id: "quarto-resource-cell-codeoutput-lst-label" };
  function fl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Og }, message: "must be string", schema: Og.type, parentSchema: Og, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return fl.errors = e, r === 0;
  }
  B["quarto-resource-cell-codeoutput-lst-cap"] = ml;
  var Pg = { type: "string", description: "be a string", $id: "quarto-resource-cell-codeoutput-lst-cap" };
  function ml(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Pg }, message: "must be string", schema: Pg.type, parentSchema: Pg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ml.errors = e, r === 0;
  }
  B["quarto-resource-cell-codeoutput-tidy"] = Xd;
  var xg = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-codeoutput-tidy" };
  function Xd(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: xg }, message: "must be boolean", schema: xg.type, parentSchema: xg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Xd.errors = e, r === 0;
  }
  B["quarto-resource-cell-codeoutput-tidy-opts"] = Zd;
  var Od = { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" }, $id: "quarto-resource-cell-codeoutput-tidy-opts" };
  function Zd(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let d = true, u = l.length;
      for (let y = 0; y < u; y++) {
        let g = l[y], x = r;
        if (typeof g != "string") {
          let $ = { instancePath: o + "/" + y, schemaPath: "#/items/type", keyword: "type", params: { type: "string", schema: Od.items }, message: "must be string", schema: Od.items.type, parentSchema: Od.items, data: g };
          e === null ? e = [$] : e.push($), r++;
        }
        let D = r;
        var c = x === r;
        c || (d = false);
      }
    } else {
      let d = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: Od }, message: "must be array", schema: Od.type, parentSchema: Od, data: l };
      e === null ? e = [d] : e.push(d), r++;
    }
    return Zd.errors = e, r === 0;
  }
  B["quarto-resource-cell-codeoutput-collapse"] = ef;
  var kg = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-codeoutput-collapse" };
  function ef(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: kg }, message: "must be boolean", schema: kg.type, parentSchema: kg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ef.errors = e, r === 0;
  }
  B["quarto-resource-cell-codeoutput-prompt"] = rf;
  var Dg = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-codeoutput-prompt" };
  function rf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Dg }, message: "must be boolean", schema: Dg.type, parentSchema: Dg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return rf.errors = e, r === 0;
  }
  B["quarto-resource-cell-codeoutput-highlight"] = tf;
  var wg = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-codeoutput-highlight" };
  function tf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: wg }, message: "must be boolean", schema: wg.type, parentSchema: wg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return tf.errors = e, r === 0;
  }
  B["quarto-resource-cell-codeoutput-class-source"] = af;
  var di = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-codeoutput-class-source" };
  function af(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: di.oneOf[0] }, message: "must be string", schema: di.oneOf[0].type, parentSchema: di.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: di.oneOf[1].items }, message: "must be string", schema: di.oneOf[1].items.type, parentSchema: di.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: di.oneOf[1] }, message: "must be array", schema: di.oneOf[1].type, parentSchema: di.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: di }, message: "must match exactly one schema in oneOf", schema: di.oneOf, parentSchema: di, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return af.errors = e, r === 0;
  }
  B["quarto-resource-cell-codeoutput-attr-source"] = nf;
  var fi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-codeoutput-attr-source" };
  function nf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: fi.oneOf[0] }, message: "must be string", schema: fi.oneOf[0].type, parentSchema: fi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: fi.oneOf[1].items }, message: "must be string", schema: fi.oneOf[1].items.type, parentSchema: fi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: fi.oneOf[1] }, message: "must be array", schema: fi.oneOf[1].type, parentSchema: fi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: fi }, message: "must match exactly one schema in oneOf", schema: fi.oneOf, parentSchema: fi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return nf.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-fig-width"] = of;
  var $g = { type: "number", description: "be a number", $id: "quarto-resource-cell-figure-fig-width" };
  function of(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: $g }, message: "must be number", schema: $g.type, parentSchema: $g, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return of.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-fig-height"] = sf;
  var Sg = { type: "number", description: "be a number", $id: "quarto-resource-cell-figure-fig-height" };
  function sf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: Sg }, message: "must be number", schema: Sg.type, parentSchema: Sg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return sf.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-fig-cap"] = gl;
  var Fg = { type: "string", description: "be a string", $id: "quarto-resource-cell-figure-fig-cap" };
  function gl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Fg }, message: "must be string", schema: Fg.type, parentSchema: Fg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return gl.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-fig-subcap"] = hl;
  var po = { oneOf: [{ enum: [true], description: "be true" }, { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }], description: "be exactly one of: true, exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-figure-fig-subcap" };
  function hl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r, y = po.oneOf[0].enum;
    if (l !== true) {
      let h = { instancePath: o, schemaPath: "#/oneOf/0/enum", keyword: "enum", params: { allowedValues: po.oneOf[0].enum, schema: po.oneOf[0] }, message: "must be equal to one of the allowed values", schema: po.oneOf[0].enum, parentSchema: po.oneOf[0], data: l };
      e === null ? e = [h] : e.push(h), r++;
    }
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r, $ = false, m = null, R = r;
    if (typeof l != "string") {
      let h = { instancePath: o, schemaPath: "#/oneOf/1/oneOf/0/type", keyword: "type", params: { type: "string", schema: po.oneOf[1].oneOf[0] }, message: "must be string", schema: po.oneOf[1].oneOf[0].type, parentSchema: po.oneOf[1].oneOf[0], data: l };
      e === null ? e = [h] : e.push(h), r++;
    }
    let j = r;
    var V = R === r;
    V && ($ = true, m = 0);
    let se = r, P = r;
    if (Array.isArray(l)) {
      let h = true, f = l.length;
      for (let ft = 0; ft < f; ft++) {
        let Cr = l[ft], Dr = r;
        if (typeof Cr != "string") {
          let ta = { instancePath: o + "/" + ft, schemaPath: "#/oneOf/1/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: po.oneOf[1].oneOf[1].items }, message: "must be string", schema: po.oneOf[1].oneOf[1].items.type, parentSchema: po.oneOf[1].oneOf[1].items, data: Cr };
          e === null ? e = [ta] : e.push(ta), r++;
        }
        let wr = r;
        var oe = Dr === r;
        oe || (h = false);
      }
    } else {
      let h = { instancePath: o, schemaPath: "#/oneOf/1/oneOf/1/type", keyword: "type", params: { type: "array", schema: po.oneOf[1].oneOf[1] }, message: "must be array", schema: po.oneOf[1].oneOf[1].type, parentSchema: po.oneOf[1].oneOf[1], data: l };
      e === null ? e = [h] : e.push(h), r++;
    }
    var V = se === r;
    if (V && $ ? ($ = false, m = [m, 1]) : V && ($ = true, m = 1), $)
      r = D, e !== null && (D ? e.length = D : e = null);
    else {
      let h = { instancePath: o, schemaPath: "#/oneOf/1/oneOf", keyword: "oneOf", params: { passingSchemas: m, schema: po.oneOf[1] }, message: "must match exactly one schema in oneOf", schema: po.oneOf[1].oneOf, parentSchema: po.oneOf[1], data: l };
      e === null ? e = [h] : e.push(h), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let h = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: po }, message: "must match exactly one schema in oneOf", schema: po.oneOf, parentSchema: po, data: l };
      e === null ? e = [h] : e.push(h), r++;
    }
    return hl.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-fig-link"] = yl;
  var mi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-figure-fig-link" };
  function yl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: mi.oneOf[0] }, message: "must be string", schema: mi.oneOf[0].type, parentSchema: mi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: mi.oneOf[1].items }, message: "must be string", schema: mi.oneOf[1].items.type, parentSchema: mi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: mi.oneOf[1] }, message: "must be array", schema: mi.oneOf[1].type, parentSchema: mi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: mi }, message: "must match exactly one schema in oneOf", schema: mi.oneOf, parentSchema: mi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return yl.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-fig-align"] = la;
  var To = { oneOf: [{ enum: ["default", "left", "right", "center"], description: "be one of: `default`, `left`, `right`, `center`" }, { type: "array", description: "be an array of values, where each element should be one of: `default`, `left`, `right`, `center`", items: { enum: ["default", "left", "right", "center"], description: "be one of: `default`, `left`, `right`, `center`" } }], description: "be exactly one of: one of: `default`, `left`, `right`, `center`, an array of values, where each element should be one of: `default`, `left`, `right`, `center`", $id: "quarto-resource-cell-figure-fig-align" };
  function la(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r, y = To.oneOf[0].enum;
    if (!(l === "default" || l === "left" || l === "right" || l === "center")) {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/enum", keyword: "enum", params: { allowedValues: To.oneOf[0].enum, schema: To.oneOf[0] }, message: "must be equal to one of the allowed values", schema: To.oneOf[0].enum, parentSchema: To.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r, P = To.oneOf[1].items.enum;
        if (!(V === "default" || V === "left" || V === "right" || V === "center")) {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/enum", keyword: "enum", params: { allowedValues: To.oneOf[1].items.enum, schema: To.oneOf[1].items }, message: "must be equal to one of the allowed values", schema: To.oneOf[1].items.enum, parentSchema: To.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: To.oneOf[1] }, message: "must be array", schema: To.oneOf[1].type, parentSchema: To.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: To }, message: "must match exactly one schema in oneOf", schema: To.oneOf, parentSchema: To, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return la.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-fig-alt"] = bl;
  var gi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-figure-fig-alt" };
  function bl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: gi.oneOf[0] }, message: "must be string", schema: gi.oneOf[0].type, parentSchema: gi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: gi.oneOf[1].items }, message: "must be string", schema: gi.oneOf[1].items.type, parentSchema: gi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: gi.oneOf[1] }, message: "must be array", schema: gi.oneOf[1].type, parentSchema: gi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: gi }, message: "must match exactly one schema in oneOf", schema: gi.oneOf, parentSchema: gi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return bl.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-fig-env"] = Zn;
  var hi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-figure-fig-env" };
  function Zn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: hi.oneOf[0] }, message: "must be string", schema: hi.oneOf[0].type, parentSchema: hi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: hi.oneOf[1].items }, message: "must be string", schema: hi.oneOf[1].items.type, parentSchema: hi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: hi.oneOf[1] }, message: "must be array", schema: hi.oneOf[1].type, parentSchema: hi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: hi }, message: "must match exactly one schema in oneOf", schema: hi.oneOf, parentSchema: hi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Zn.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-fig-pos"] = eo;
  var yi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-figure-fig-pos" };
  function eo(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: yi.oneOf[0] }, message: "must be string", schema: yi.oneOf[0].type, parentSchema: yi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: yi.oneOf[1].items }, message: "must be string", schema: yi.oneOf[1].items.type, parentSchema: yi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: yi.oneOf[1] }, message: "must be array", schema: yi.oneOf[1].type, parentSchema: yi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: yi }, message: "must match exactly one schema in oneOf", schema: yi.oneOf, parentSchema: yi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return eo.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-fig-scap"] = _l;
  var bi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-figure-fig-scap" };
  function _l(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: bi.oneOf[0] }, message: "must be string", schema: bi.oneOf[0].type, parentSchema: bi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: bi.oneOf[1].items }, message: "must be string", schema: bi.oneOf[1].items.type, parentSchema: bi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: bi.oneOf[1] }, message: "must be array", schema: bi.oneOf[1].type, parentSchema: bi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: bi }, message: "must match exactly one schema in oneOf", schema: bi.oneOf, parentSchema: bi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return _l.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-fig-format"] = cf;
  var lf = { enum: ["retina", "png", "jpeg", "svg", "pdf"], description: "be one of: `retina`, `png`, `jpeg`, `svg`, `pdf`", $id: "quarto-resource-cell-figure-fig-format" };
  function cf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = lf.enum;
    if (!(l === "retina" || l === "png" || l === "jpeg" || l === "svg" || l === "pdf")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: lf.enum, schema: lf }, message: "must be equal to one of the allowed values", schema: lf.enum, parentSchema: lf, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return cf.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-fig-dpi"] = pf;
  var Ag = { type: "number", description: "be a number", $id: "quarto-resource-cell-figure-fig-dpi" };
  function pf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: Ag }, message: "must be number", schema: Ag.type, parentSchema: Ag, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return pf.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-out-width"] = uf;
  var Bg = { type: "string", description: "be a string", $id: "quarto-resource-cell-figure-out-width" };
  function uf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Bg }, message: "must be string", schema: Bg.type, parentSchema: Bg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return uf.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-out-height"] = df;
  var jg = { type: "string", description: "be a string", $id: "quarto-resource-cell-figure-out-height" };
  function df(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: jg }, message: "must be string", schema: jg.type, parentSchema: jg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return df.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-fig-keep"] = ff;
  var uo = { oneOf: [{ enum: ["high", "none", "all", "first", "last"], description: "be one of: `high`, `none`, `all`, `first`, `last`" }, { oneOf: [{ type: "number", description: "be a number" }, { type: "array", description: "be an array of values, where each element should be a number", items: { type: "number", description: "be a number" } }], description: "be exactly one of: a number, an array of values, where each element should be a number" }], description: "be exactly one of: one of: `high`, `none`, `all`, `first`, `last`, exactly one of: a number, an array of values, where each element should be a number", $id: "quarto-resource-cell-figure-fig-keep" };
  function ff(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r, y = uo.oneOf[0].enum;
    if (!(l === "high" || l === "none" || l === "all" || l === "first" || l === "last")) {
      let h = { instancePath: o, schemaPath: "#/oneOf/0/enum", keyword: "enum", params: { allowedValues: uo.oneOf[0].enum, schema: uo.oneOf[0] }, message: "must be equal to one of the allowed values", schema: uo.oneOf[0].enum, parentSchema: uo.oneOf[0], data: l };
      e === null ? e = [h] : e.push(h), r++;
    }
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r, $ = false, m = null, R = r;
    if (!(typeof l == "number" && isFinite(l))) {
      let h = { instancePath: o, schemaPath: "#/oneOf/1/oneOf/0/type", keyword: "type", params: { type: "number", schema: uo.oneOf[1].oneOf[0] }, message: "must be number", schema: uo.oneOf[1].oneOf[0].type, parentSchema: uo.oneOf[1].oneOf[0], data: l };
      e === null ? e = [h] : e.push(h), r++;
    }
    let j = r;
    var V = R === r;
    V && ($ = true, m = 0);
    let se = r, P = r;
    if (Array.isArray(l)) {
      let h = true, f = l.length;
      for (let ft = 0; ft < f; ft++) {
        let Cr = l[ft], Dr = r;
        if (!(typeof Cr == "number" && isFinite(Cr))) {
          let ta = { instancePath: o + "/" + ft, schemaPath: "#/oneOf/1/oneOf/1/items/type", keyword: "type", params: { type: "number", schema: uo.oneOf[1].oneOf[1].items }, message: "must be number", schema: uo.oneOf[1].oneOf[1].items.type, parentSchema: uo.oneOf[1].oneOf[1].items, data: Cr };
          e === null ? e = [ta] : e.push(ta), r++;
        }
        let wr = r;
        var oe = Dr === r;
        oe || (h = false);
      }
    } else {
      let h = { instancePath: o, schemaPath: "#/oneOf/1/oneOf/1/type", keyword: "type", params: { type: "array", schema: uo.oneOf[1].oneOf[1] }, message: "must be array", schema: uo.oneOf[1].oneOf[1].type, parentSchema: uo.oneOf[1].oneOf[1], data: l };
      e === null ? e = [h] : e.push(h), r++;
    }
    var V = se === r;
    if (V && $ ? ($ = false, m = [m, 1]) : V && ($ = true, m = 1), $)
      r = D, e !== null && (D ? e.length = D : e = null);
    else {
      let h = { instancePath: o, schemaPath: "#/oneOf/1/oneOf", keyword: "oneOf", params: { passingSchemas: m, schema: uo.oneOf[1] }, message: "must match exactly one schema in oneOf", schema: uo.oneOf[1].oneOf, parentSchema: uo.oneOf[1], data: l };
      e === null ? e = [h] : e.push(h), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let h = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: uo }, message: "must match exactly one schema in oneOf", schema: uo.oneOf, parentSchema: uo, data: l };
      e === null ? e = [h] : e.push(h), r++;
    }
    return ff.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-fig-show"] = gf;
  var mf = { enum: ["asis", "hold", "animate", "hide"], description: "be one of: `asis`, `hold`, `animate`, `hide`", $id: "quarto-resource-cell-figure-fig-show" };
  function gf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = mf.enum;
    if (!(l === "asis" || l === "hold" || l === "animate" || l === "hide")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: mf.enum, schema: mf }, message: "must be equal to one of the allowed values", schema: mf.enum, parentSchema: mf, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return gf.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-out-extra"] = hf;
  var Eg = { type: "string", description: "be a string", $id: "quarto-resource-cell-figure-out-extra" };
  function hf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Eg }, message: "must be string", schema: Eg.type, parentSchema: Eg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return hf.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-external"] = yf;
  var Ng = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-figure-external" };
  function yf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Ng }, message: "must be boolean", schema: Ng.type, parentSchema: Ng, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return yf.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-sanitize"] = bf;
  var Cg = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-figure-sanitize" };
  function bf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Cg }, message: "must be boolean", schema: Cg.type, parentSchema: Cg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return bf.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-interval"] = _f;
  var Ig = { type: "number", description: "be a number", $id: "quarto-resource-cell-figure-interval" };
  function _f(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: Ig }, message: "must be number", schema: Ig.type, parentSchema: Ig, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return _f.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-aniopts"] = vf;
  var Hg = { type: "string", description: "be a string", $id: "quarto-resource-cell-figure-aniopts" };
  function vf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Hg }, message: "must be string", schema: Hg.type, parentSchema: Hg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return vf.errors = e, r === 0;
  }
  B["quarto-resource-cell-figure-animation-hook"] = qf;
  var Lg = { type: "string", description: "be a string", $id: "quarto-resource-cell-figure-animation-hook" };
  function qf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Lg }, message: "must be string", schema: Lg.type, parentSchema: Lg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return qf.errors = e, r === 0;
  }
  B["quarto-resource-cell-include-child"] = Of;
  var _i = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-include-child" };
  function Of(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: _i.oneOf[0] }, message: "must be string", schema: _i.oneOf[0].type, parentSchema: _i.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: _i.oneOf[1].items }, message: "must be string", schema: _i.oneOf[1].items.type, parentSchema: _i.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: _i.oneOf[1] }, message: "must be array", schema: _i.oneOf[1].type, parentSchema: _i.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: _i }, message: "must match exactly one schema in oneOf", schema: _i.oneOf, parentSchema: _i, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Of.errors = e, r === 0;
  }
  B["quarto-resource-cell-include-file"] = Pf;
  var Mg = { type: "string", description: "be a string", $id: "quarto-resource-cell-include-file" };
  function Pf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Mg }, message: "must be string", schema: Mg.type, parentSchema: Mg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Pf.errors = e, r === 0;
  }
  B["quarto-resource-cell-include-code"] = xf;
  var Tg = { type: "string", description: "be a string", $id: "quarto-resource-cell-include-code" };
  function xf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Tg }, message: "must be string", schema: Tg.type, parentSchema: Tg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return xf.errors = e, r === 0;
  }
  B["quarto-resource-cell-include-purl"] = kf;
  var zg = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-include-purl" };
  function kf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: zg }, message: "must be boolean", schema: zg.type, parentSchema: zg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return kf.errors = e, r === 0;
  }
  B["quarto-resource-cell-layout-layout"] = vl;
  var Vo = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be an array of values, where each element should be a number", items: { type: "array", description: "be an array of values, where each element should be a number", items: { type: "number", description: "be a number" } } }], description: "be exactly one of: a string, an array of values, where each element should be an array of values, where each element should be a number", $id: "quarto-resource-cell-layout-layout" };
  function vl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let R = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Vo.oneOf[0] }, message: "must be string", schema: Vo.oneOf[0].type, parentSchema: Vo.oneOf[0], data: l };
      e === null ? e = [R] : e.push(R), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let R = true, j = l.length;
      for (let V = 0; V < j; V++) {
        let se = l[V], P = r, oe = r;
        if (Array.isArray(se)) {
          let h = true, f = se.length;
          for (let ft = 0; ft < f; ft++) {
            let Cr = se[ft], Dr = r;
            if (!(typeof Cr == "number" && isFinite(Cr))) {
              let ta = { instancePath: o + "/" + V + "/" + ft, schemaPath: "#/oneOf/1/items/items/type", keyword: "type", params: { type: "number", schema: Vo.oneOf[1].items.items }, message: "must be number", schema: Vo.oneOf[1].items.items.type, parentSchema: Vo.oneOf[1].items.items, data: Cr };
              e === null ? e = [ta] : e.push(ta), r++;
            }
            let wr = r;
            var $ = Dr === r;
            $ || (h = false);
          }
        } else {
          let h = { instancePath: o + "/" + V, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "array", schema: Vo.oneOf[1].items }, message: "must be array", schema: Vo.oneOf[1].items.type, parentSchema: Vo.oneOf[1].items, data: se };
          e === null ? e = [h] : e.push(h), r++;
        }
        var m = P === r;
        m || (R = false);
      }
    } else {
      let R = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Vo.oneOf[1] }, message: "must be array", schema: Vo.oneOf[1].type, parentSchema: Vo.oneOf[1], data: l };
      e === null ? e = [R] : e.push(R), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let R = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Vo }, message: "must match exactly one schema in oneOf", schema: Vo.oneOf, parentSchema: Vo, data: l };
      e === null ? e = [R] : e.push(R), r++;
    }
    return vl.errors = e, r === 0;
  }
  B["quarto-resource-cell-layout-layout-ncol"] = ql;
  var Wg = { type: "number", description: "be a number", $id: "quarto-resource-cell-layout-layout-ncol" };
  function ql(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: Wg }, message: "must be number", schema: Wg.type, parentSchema: Wg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ql.errors = e, r === 0;
  }
  B["quarto-resource-cell-layout-layout-nrow"] = Ol;
  var Rg = { type: "number", description: "be a number", $id: "quarto-resource-cell-layout-layout-nrow" };
  function Ol(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: Rg }, message: "must be number", schema: Rg.type, parentSchema: Rg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ol.errors = e, r === 0;
  }
  B["quarto-resource-cell-layout-layout-align"] = Pl;
  var Df = { enum: ["default", "left", "center", "right"], description: "be one of: `default`, `left`, `center`, `right`", $id: "quarto-resource-cell-layout-layout-align" };
  function Pl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Df.enum;
    if (!(l === "default" || l === "left" || l === "center" || l === "right")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Df.enum, schema: Df }, message: "must be equal to one of the allowed values", schema: Df.enum, parentSchema: Df, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Pl.errors = e, r === 0;
  }
  B["quarto-resource-cell-layout-layout-valign"] = xl;
  var wf = { enum: ["default", "top", "center", "bottom"], description: "be one of: `default`, `top`, `center`, `bottom`", $id: "quarto-resource-cell-layout-layout-valign" };
  function xl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = wf.enum;
    if (!(l === "default" || l === "top" || l === "center" || l === "bottom")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: wf.enum, schema: wf }, message: "must be equal to one of the allowed values", schema: wf.enum, parentSchema: wf, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return xl.errors = e, r === 0;
  }
  B["quarto-resource-cell-pagelayout-column"] = kl;
  function kl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    return cl(l, { instancePath: o, parentData: S, parentDataProperty: F, rootData: s }) || (e = e === null ? cl.errors : e.concat(cl.errors), r = e.length), kl.errors = e, r === 0;
  }
  B["quarto-resource-cell-pagelayout-fig-column"] = Dl;
  function Dl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    return cl(l, { instancePath: o, parentData: S, parentDataProperty: F, rootData: s }) || (e = e === null ? cl.errors : e.concat(cl.errors), r = e.length), Dl.errors = e, r === 0;
  }
  B["quarto-resource-cell-pagelayout-tbl-column"] = wl;
  function wl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    return cl(l, { instancePath: o, parentData: S, parentDataProperty: F, rootData: s }) || (e = e === null ? cl.errors : e.concat(cl.errors), r = e.length), wl.errors = e, r === 0;
  }
  B["quarto-resource-cell-pagelayout-cap-location"] = ja;
  var $f = { enum: ["top", "bottom", "margin"], description: "be one of: `top`, `bottom`, `margin`", $id: "quarto-resource-cell-pagelayout-cap-location" };
  function ja(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = $f.enum;
    if (!(l === "top" || l === "bottom" || l === "margin")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: $f.enum, schema: $f }, message: "must be equal to one of the allowed values", schema: $f.enum, parentSchema: $f, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return ja.errors = e, r === 0;
  }
  B["quarto-resource-cell-pagelayout-fig-cap-location"] = Ea;
  var Sf = { enum: ["top", "bottom", "margin"], description: "be one of: `top`, `bottom`, `margin`", $id: "quarto-resource-cell-pagelayout-fig-cap-location" };
  function Ea(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Sf.enum;
    if (!(l === "top" || l === "bottom" || l === "margin")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Sf.enum, schema: Sf }, message: "must be equal to one of the allowed values", schema: Sf.enum, parentSchema: Sf, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Ea.errors = e, r === 0;
  }
  B["quarto-resource-cell-pagelayout-tbl-cap-location"] = Na;
  var Ff = { enum: ["top", "bottom", "margin"], description: "be one of: `top`, `bottom`, `margin`", $id: "quarto-resource-cell-pagelayout-tbl-cap-location" };
  function Na(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Ff.enum;
    if (!(l === "top" || l === "bottom" || l === "margin")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Ff.enum, schema: Ff }, message: "must be equal to one of the allowed values", schema: Ff.enum, parentSchema: Ff, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Na.errors = e, r === 0;
  }
  B["quarto-resource-cell-table-tbl-cap"] = $l;
  var Vg = { type: "string", description: "be a string", $id: "quarto-resource-cell-table-tbl-cap" };
  function $l(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Vg }, message: "must be string", schema: Vg.type, parentSchema: Vg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return $l.errors = e, r === 0;
  }
  B["quarto-resource-cell-table-tbl-subcap"] = Sl;
  var vo = { anyOf: [{ enum: [true], description: "be true" }, { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }], description: "be at least one of: true, exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-table-tbl-subcap" };
  function Sl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = r, u = vo.anyOf[0].enum;
    if (l !== true) {
      let D = { instancePath: o, schemaPath: "#/anyOf/0/enum", keyword: "enum", params: { allowedValues: vo.anyOf[0].enum, schema: vo.anyOf[0] }, message: "must be equal to one of the allowed values", schema: vo.anyOf[0].enum, parentSchema: vo.anyOf[0], data: l };
      e === null ? e = [D] : e.push(D), r++;
    }
    var y = d === r;
    if (c = c || y, !c) {
      let D = r, $ = r, m = false, R = null, j = r;
      if (typeof l != "string") {
        let h = { instancePath: o, schemaPath: "#/anyOf/1/oneOf/0/type", keyword: "type", params: { type: "string", schema: vo.anyOf[1].oneOf[0] }, message: "must be string", schema: vo.anyOf[1].oneOf[0].type, parentSchema: vo.anyOf[1].oneOf[0], data: l };
        e === null ? e = [h] : e.push(h), r++;
      }
      let V = r;
      var g = j === r;
      g && (m = true, R = 0);
      let se = r, P = r;
      if (Array.isArray(l)) {
        let h = true, f = l.length;
        for (let ft = 0; ft < f; ft++) {
          let Cr = l[ft], Dr = r;
          if (typeof Cr != "string") {
            let ta = { instancePath: o + "/" + ft, schemaPath: "#/anyOf/1/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: vo.anyOf[1].oneOf[1].items }, message: "must be string", schema: vo.anyOf[1].oneOf[1].items.type, parentSchema: vo.anyOf[1].oneOf[1].items, data: Cr };
            e === null ? e = [ta] : e.push(ta), r++;
          }
          let wr = r;
          var x = Dr === r;
          x || (h = false);
        }
      } else {
        let h = { instancePath: o, schemaPath: "#/anyOf/1/oneOf/1/type", keyword: "type", params: { type: "array", schema: vo.anyOf[1].oneOf[1] }, message: "must be array", schema: vo.anyOf[1].oneOf[1].type, parentSchema: vo.anyOf[1].oneOf[1], data: l };
        e === null ? e = [h] : e.push(h), r++;
      }
      var g = se === r;
      if (g && m ? (m = false, R = [R, 1]) : g && (m = true, R = 1), m)
        r = $, e !== null && ($ ? e.length = $ : e = null);
      else {
        let h = { instancePath: o, schemaPath: "#/anyOf/1/oneOf", keyword: "oneOf", params: { passingSchemas: R, schema: vo.anyOf[1] }, message: "must match exactly one schema in oneOf", schema: vo.anyOf[1].oneOf, parentSchema: vo.anyOf[1], data: l };
        e === null ? e = [h] : e.push(h), r++;
      }
      var y = D === r;
      c = c || y;
    }
    if (c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let D = { instancePath: o, schemaPath: "#/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: vo.anyOf, parentSchema: vo, data: l };
      e === null ? e = [D] : e.push(D), r++;
    }
    return Sl.errors = e, r === 0;
  }
  B["quarto-resource-cell-table-tbl-colwidths"] = be;
  var zo = { anyOf: [{ type: "boolean", description: "be `true` or `false`" }, { enum: ["auto"], description: "be 'auto'" }, { type: "array", description: "be an array of values, where each element should be a number", items: { type: "number", description: "be a number" } }], description: "be at least one of: `true` or `false`, 'auto', an array of values, where each element should be a number", $id: "quarto-resource-cell-table-tbl-colwidths" };
  function be(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = r;
    if (typeof l != "boolean") {
      let x = { instancePath: o, schemaPath: "#/anyOf/0/type", keyword: "type", params: { type: "boolean", schema: zo.anyOf[0] }, message: "must be boolean", schema: zo.anyOf[0].type, parentSchema: zo.anyOf[0], data: l };
      e === null ? e = [x] : e.push(x), r++;
    }
    let u = r;
    var y = d === r;
    if (c = c || y, !c) {
      let x = r, D = zo.anyOf[1].enum;
      if (l !== "auto") {
        let m = { instancePath: o, schemaPath: "#/anyOf/1/enum", keyword: "enum", params: { allowedValues: zo.anyOf[1].enum, schema: zo.anyOf[1] }, message: "must be equal to one of the allowed values", schema: zo.anyOf[1].enum, parentSchema: zo.anyOf[1], data: l };
        e === null ? e = [m] : e.push(m), r++;
      }
      var y = x === r;
      if (c = c || y, !c) {
        let m = r, R = r;
        if (Array.isArray(l)) {
          let V = true, se = l.length;
          for (let P = 0; P < se; P++) {
            let oe = l[P], h = r;
            if (!(typeof oe == "number" && isFinite(oe))) {
              let ft = { instancePath: o + "/" + P, schemaPath: "#/anyOf/2/items/type", keyword: "type", params: { type: "number", schema: zo.anyOf[2].items }, message: "must be number", schema: zo.anyOf[2].items.type, parentSchema: zo.anyOf[2].items, data: oe };
              e === null ? e = [ft] : e.push(ft), r++;
            }
            let f = r;
            var g = h === r;
            g || (V = false);
          }
        } else {
          let V = { instancePath: o, schemaPath: "#/anyOf/2/type", keyword: "type", params: { type: "array", schema: zo.anyOf[2] }, message: "must be array", schema: zo.anyOf[2].type, parentSchema: zo.anyOf[2], data: l };
          e === null ? e = [V] : e.push(V), r++;
        }
        var y = m === r;
        c = c || y;
      }
    }
    if (c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let x = { instancePath: o, schemaPath: "#/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: zo.anyOf, parentSchema: zo, data: l };
      e === null ? e = [x] : e.push(x), r++;
    }
    return be.errors = e, r === 0;
  }
  B["quarto-resource-cell-textoutput-output"] = fe;
  var Wo = { anyOf: [{ type: "boolean", description: "be `true` or `false`" }, { enum: ["asis"], description: "be 'asis'" }, { type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: {}, patternProperties: {} }], description: "be at least one of: `true` or `false`, 'asis', a string, an object", $id: "quarto-resource-cell-textoutput-output" };
  function fe(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = r;
    if (typeof l != "boolean") {
      let g = { instancePath: o, schemaPath: "#/anyOf/0/type", keyword: "type", params: { type: "boolean", schema: Wo.anyOf[0] }, message: "must be boolean", schema: Wo.anyOf[0].type, parentSchema: Wo.anyOf[0], data: l };
      e === null ? e = [g] : e.push(g), r++;
    }
    let u = r;
    var y = d === r;
    if (c = c || y, !c) {
      let g = r, x = Wo.anyOf[1].enum;
      if (l !== "asis") {
        let $ = { instancePath: o, schemaPath: "#/anyOf/1/enum", keyword: "enum", params: { allowedValues: Wo.anyOf[1].enum, schema: Wo.anyOf[1] }, message: "must be equal to one of the allowed values", schema: Wo.anyOf[1].enum, parentSchema: Wo.anyOf[1], data: l };
        e === null ? e = [$] : e.push($), r++;
      }
      var y = g === r;
      if (c = c || y, !c) {
        let $ = r;
        if (typeof l != "string") {
          let j = { instancePath: o, schemaPath: "#/anyOf/2/type", keyword: "type", params: { type: "string", schema: Wo.anyOf[2] }, message: "must be string", schema: Wo.anyOf[2].type, parentSchema: Wo.anyOf[2], data: l };
          e === null ? e = [j] : e.push(j), r++;
        }
        let m = r;
        var y = $ === r;
        if (c = c || y, !c) {
          let j = r, V = r;
          if (!(l && typeof l == "object" && !Array.isArray(l))) {
            let P = { instancePath: o, schemaPath: "#/anyOf/3/type", keyword: "type", params: { type: "object", schema: Wo.anyOf[3] }, message: "must be object", schema: Wo.anyOf[3].type, parentSchema: Wo.anyOf[3], data: l };
            e === null ? e = [P] : e.push(P), r++;
          }
          var y = j === r;
          c = c || y;
        }
      }
    }
    if (c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let g = { instancePath: o, schemaPath: "#/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: Wo.anyOf, parentSchema: Wo, data: l };
      e === null ? e = [g] : e.push(g), r++;
    }
    return fe.errors = e, r === 0;
  }
  B["quarto-resource-cell-textoutput-warning"] = me;
  var Ug = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-textoutput-warning" };
  function me(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Ug }, message: "must be boolean", schema: Ug.type, parentSchema: Ug, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return me.errors = e, r === 0;
  }
  B["quarto-resource-cell-textoutput-error"] = ge;
  var Kg = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-textoutput-error" };
  function ge(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Kg }, message: "must be boolean", schema: Kg.type, parentSchema: Kg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ge.errors = e, r === 0;
  }
  B["quarto-resource-cell-textoutput-include"] = he;
  var Jg = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-textoutput-include" };
  function he(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Jg }, message: "must be boolean", schema: Jg.type, parentSchema: Jg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return he.errors = e, r === 0;
  }
  B["quarto-resource-cell-textoutput-panel"] = Fl;
  var Af = { enum: ["tabset", "input", "sidebar", "fill", "center"], description: "be one of: `tabset`, `input`, `sidebar`, `fill`, `center`", $id: "quarto-resource-cell-textoutput-panel" };
  function Fl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Af.enum;
    if (!(l === "tabset" || l === "input" || l === "sidebar" || l === "fill" || l === "center")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Af.enum, schema: Af }, message: "must be equal to one of the allowed values", schema: Af.enum, parentSchema: Af, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Fl.errors = e, r === 0;
  }
  B["quarto-resource-cell-textoutput-message"] = Bf;
  var Qg = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-cell-textoutput-message" };
  function Bf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Qg }, message: "must be boolean", schema: Qg.type, parentSchema: Qg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Bf.errors = e, r === 0;
  }
  B["quarto-resource-cell-textoutput-results"] = jf;
  var ci = { oneOf: [{ enum: ["markup", "asis", "hold", "hide"], description: "be one of: `markup`, `asis`, `hold`, `hide`" }, { enum: [false], description: "be false" }], description: "be exactly one of: one of: `markup`, `asis`, `hold`, `hide`, false", $id: "quarto-resource-cell-textoutput-results" };
  function jf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r, y = ci.oneOf[0].enum;
    if (!(l === "markup" || l === "asis" || l === "hold" || l === "hide")) {
      let $ = { instancePath: o, schemaPath: "#/oneOf/0/enum", keyword: "enum", params: { allowedValues: ci.oneOf[0].enum, schema: ci.oneOf[0] }, message: "must be equal to one of the allowed values", schema: ci.oneOf[0].enum, parentSchema: ci.oneOf[0], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = ci.oneOf[1].enum;
    if (l !== false) {
      let $ = { instancePath: o, schemaPath: "#/oneOf/1/enum", keyword: "enum", params: { allowedValues: ci.oneOf[1].enum, schema: ci.oneOf[1] }, message: "must be equal to one of the allowed values", schema: ci.oneOf[1].enum, parentSchema: ci.oneOf[1], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: ci }, message: "must match exactly one schema in oneOf", schema: ci.oneOf, parentSchema: ci, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return jf.errors = e, r === 0;
  }
  B["quarto-resource-cell-textoutput-comment"] = Ef;
  var Gg = { type: "string", description: "be a string", $id: "quarto-resource-cell-textoutput-comment" };
  function Ef(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Gg }, message: "must be string", schema: Gg.type, parentSchema: Gg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ef.errors = e, r === 0;
  }
  B["quarto-resource-cell-textoutput-class-output"] = Nf;
  var vi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-textoutput-class-output" };
  function Nf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: vi.oneOf[0] }, message: "must be string", schema: vi.oneOf[0].type, parentSchema: vi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: vi.oneOf[1].items }, message: "must be string", schema: vi.oneOf[1].items.type, parentSchema: vi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: vi.oneOf[1] }, message: "must be array", schema: vi.oneOf[1].type, parentSchema: vi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: vi }, message: "must match exactly one schema in oneOf", schema: vi.oneOf, parentSchema: vi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Nf.errors = e, r === 0;
  }
  B["quarto-resource-cell-textoutput-attr-output"] = Cf;
  var qi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-textoutput-attr-output" };
  function Cf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: qi.oneOf[0] }, message: "must be string", schema: qi.oneOf[0].type, parentSchema: qi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: qi.oneOf[1].items }, message: "must be string", schema: qi.oneOf[1].items.type, parentSchema: qi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: qi.oneOf[1] }, message: "must be array", schema: qi.oneOf[1].type, parentSchema: qi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: qi }, message: "must match exactly one schema in oneOf", schema: qi.oneOf, parentSchema: qi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Cf.errors = e, r === 0;
  }
  B["quarto-resource-cell-textoutput-class-warning"] = If;
  var Oi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-textoutput-class-warning" };
  function If(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Oi.oneOf[0] }, message: "must be string", schema: Oi.oneOf[0].type, parentSchema: Oi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Oi.oneOf[1].items }, message: "must be string", schema: Oi.oneOf[1].items.type, parentSchema: Oi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Oi.oneOf[1] }, message: "must be array", schema: Oi.oneOf[1].type, parentSchema: Oi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Oi }, message: "must match exactly one schema in oneOf", schema: Oi.oneOf, parentSchema: Oi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return If.errors = e, r === 0;
  }
  B["quarto-resource-cell-textoutput-attr-warning"] = Hf;
  var Pi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-textoutput-attr-warning" };
  function Hf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Pi.oneOf[0] }, message: "must be string", schema: Pi.oneOf[0].type, parentSchema: Pi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Pi.oneOf[1].items }, message: "must be string", schema: Pi.oneOf[1].items.type, parentSchema: Pi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Pi.oneOf[1] }, message: "must be array", schema: Pi.oneOf[1].type, parentSchema: Pi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Pi }, message: "must match exactly one schema in oneOf", schema: Pi.oneOf, parentSchema: Pi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Hf.errors = e, r === 0;
  }
  B["quarto-resource-cell-textoutput-class-message"] = Lf;
  var xi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-textoutput-class-message" };
  function Lf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: xi.oneOf[0] }, message: "must be string", schema: xi.oneOf[0].type, parentSchema: xi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: xi.oneOf[1].items }, message: "must be string", schema: xi.oneOf[1].items.type, parentSchema: xi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: xi.oneOf[1] }, message: "must be array", schema: xi.oneOf[1].type, parentSchema: xi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: xi }, message: "must match exactly one schema in oneOf", schema: xi.oneOf, parentSchema: xi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Lf.errors = e, r === 0;
  }
  B["quarto-resource-cell-textoutput-attr-message"] = Mf;
  var ki = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-textoutput-attr-message" };
  function Mf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: ki.oneOf[0] }, message: "must be string", schema: ki.oneOf[0].type, parentSchema: ki.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: ki.oneOf[1].items }, message: "must be string", schema: ki.oneOf[1].items.type, parentSchema: ki.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: ki.oneOf[1] }, message: "must be array", schema: ki.oneOf[1].type, parentSchema: ki.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: ki }, message: "must match exactly one schema in oneOf", schema: ki.oneOf, parentSchema: ki, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Mf.errors = e, r === 0;
  }
  B["quarto-resource-cell-textoutput-class-error"] = Tf;
  var Di = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-textoutput-class-error" };
  function Tf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Di.oneOf[0] }, message: "must be string", schema: Di.oneOf[0].type, parentSchema: Di.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Di.oneOf[1].items }, message: "must be string", schema: Di.oneOf[1].items.type, parentSchema: Di.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Di.oneOf[1] }, message: "must be array", schema: Di.oneOf[1].type, parentSchema: Di.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Di }, message: "must match exactly one schema in oneOf", schema: Di.oneOf, parentSchema: Di, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Tf.errors = e, r === 0;
  }
  B["quarto-resource-cell-textoutput-attr-error"] = zf;
  var wi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-cell-textoutput-attr-error" };
  function zf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: wi.oneOf[0] }, message: "must be string", schema: wi.oneOf[0].type, parentSchema: wi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: wi.oneOf[1].items }, message: "must be string", schema: wi.oneOf[1].items.type, parentSchema: wi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: wi.oneOf[1] }, message: "must be array", schema: wi.oneOf[1].type, parentSchema: wi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: wi }, message: "must match exactly one schema in oneOf", schema: wi.oneOf, parentSchema: wi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return zf.errors = e, r === 0;
  }
  B["quarto-resource-document-attributes-title"] = Ee;
  var Yg = { type: "string", description: "be a string", $id: "quarto-resource-document-attributes-title" };
  function Ee(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Yg }, message: "must be string", schema: Yg.type, parentSchema: Yg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ee.errors = e, r === 0;
  }
  B["quarto-resource-document-attributes-subtitle"] = ha;
  var Xg = { type: "string", description: "be a string", $id: "quarto-resource-document-attributes-subtitle" };
  function ha(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Xg }, message: "must be string", schema: Xg.type, parentSchema: Xg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ha.errors = e, r === 0;
  }
  B["quarto-resource-document-attributes-date"] = Ne;
  var Zg = { type: "string", description: "be a string", $id: "quarto-resource-document-attributes-date" };
  function Ne(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Zg }, message: "must be string", schema: Zg.type, parentSchema: Zg, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ne.errors = e, r === 0;
  }
  B["quarto-resource-document-attributes-author"] = Ce;
  var Qn = { oneOf: [{ oneOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: {}, patternProperties: {} }], description: "be exactly one of: a string, an object" }, { type: "array", description: "be an array of values, where each element should be exactly one of: a string, an object", items: { oneOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: {}, patternProperties: {} }], description: "be exactly one of: a string, an object" } }], description: "be exactly one of: exactly one of: a string, an object, an array of values, where each element should be exactly one of: a string, an object", $id: "quarto-resource-document-attributes-author" };
  function Ce(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r, y = r, g = false, x = null, D = r;
    if (typeof l != "string") {
      let f = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/0/type", keyword: "type", params: { type: "string", schema: Qn.oneOf[0].oneOf[0] }, message: "must be string", schema: Qn.oneOf[0].oneOf[0].type, parentSchema: Qn.oneOf[0].oneOf[0], data: l };
      e === null ? e = [f] : e.push(f), r++;
    }
    let $ = r;
    var m = D === r;
    m && (g = true, x = 0);
    let R = r, j = r;
    if (!(l && typeof l == "object" && !Array.isArray(l))) {
      let f = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/1/type", keyword: "type", params: { type: "object", schema: Qn.oneOf[0].oneOf[1] }, message: "must be object", schema: Qn.oneOf[0].oneOf[1].type, parentSchema: Qn.oneOf[0].oneOf[1], data: l };
      e === null ? e = [f] : e.push(f), r++;
    }
    var m = R === r;
    if (m && g ? (g = false, x = [x, 1]) : m && (g = true, x = 1), g)
      r = y, e !== null && (y ? e.length = y : e = null);
    else {
      let f = { instancePath: o, schemaPath: "#/oneOf/0/oneOf", keyword: "oneOf", params: { passingSchemas: x, schema: Qn.oneOf[0] }, message: "must match exactly one schema in oneOf", schema: Qn.oneOf[0].oneOf, parentSchema: Qn.oneOf[0], data: l };
      e === null ? e = [f] : e.push(f), r++;
    }
    var V = u === r;
    V && (c = true, d = 0);
    let se = r, P = r;
    if (Array.isArray(l)) {
      let f = true, ft = l.length;
      for (let Cr = 0; Cr < ft; Cr++) {
        let Dr = l[Cr], wr = r, ta = r, jt = false, ee = null, za = r;
        if (typeof Dr != "string") {
          let fa = { instancePath: o + "/" + Cr, schemaPath: "#/oneOf/1/items/oneOf/0/type", keyword: "type", params: { type: "string", schema: Qn.oneOf[1].items.oneOf[0] }, message: "must be string", schema: Qn.oneOf[1].items.oneOf[0].type, parentSchema: Qn.oneOf[1].items.oneOf[0], data: Dr };
          e === null ? e = [fa] : e.push(fa), r++;
        }
        let sa = r;
        var oe = za === r;
        oe && (jt = true, ee = 0);
        let _e = r, Ma = r;
        if (!(Dr && typeof Dr == "object" && !Array.isArray(Dr))) {
          let fa = { instancePath: o + "/" + Cr, schemaPath: "#/oneOf/1/items/oneOf/1/type", keyword: "type", params: { type: "object", schema: Qn.oneOf[1].items.oneOf[1] }, message: "must be object", schema: Qn.oneOf[1].items.oneOf[1].type, parentSchema: Qn.oneOf[1].items.oneOf[1], data: Dr };
          e === null ? e = [fa] : e.push(fa), r++;
        }
        var oe = _e === r;
        if (oe && jt ? (jt = false, ee = [ee, 1]) : oe && (jt = true, ee = 1), jt)
          r = ta, e !== null && (ta ? e.length = ta : e = null);
        else {
          let fa = { instancePath: o + "/" + Cr, schemaPath: "#/oneOf/1/items/oneOf", keyword: "oneOf", params: { passingSchemas: ee, schema: Qn.oneOf[1].items }, message: "must match exactly one schema in oneOf", schema: Qn.oneOf[1].items.oneOf, parentSchema: Qn.oneOf[1].items, data: Dr };
          e === null ? e = [fa] : e.push(fa), r++;
        }
        var h = wr === r;
        h || (f = false);
      }
    } else {
      let f = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Qn.oneOf[1] }, message: "must be array", schema: Qn.oneOf[1].type, parentSchema: Qn.oneOf[1], data: l };
      e === null ? e = [f] : e.push(f), r++;
    }
    var V = se === r;
    if (V && c ? (c = false, d = [d, 1]) : V && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let f = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Qn }, message: "must match exactly one schema in oneOf", schema: Qn.oneOf, parentSchema: Qn, data: l };
      e === null ? e = [f] : e.push(f), r++;
    }
    return Ce.errors = e, r === 0;
  }
  B["quarto-resource-document-attributes-affiliation"] = ko;
  var Gn = { oneOf: [{ oneOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: {}, patternProperties: {} }], description: "be exactly one of: a string, an object" }, { type: "array", description: "be an array of values, where each element should be exactly one of: a string, an object", items: { oneOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: {}, patternProperties: {} }], description: "be exactly one of: a string, an object" } }], description: "be exactly one of: exactly one of: a string, an object, an array of values, where each element should be exactly one of: a string, an object", $id: "quarto-resource-document-attributes-affiliation" };
  function ko(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r, y = r, g = false, x = null, D = r;
    if (typeof l != "string") {
      let f = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/0/type", keyword: "type", params: { type: "string", schema: Gn.oneOf[0].oneOf[0] }, message: "must be string", schema: Gn.oneOf[0].oneOf[0].type, parentSchema: Gn.oneOf[0].oneOf[0], data: l };
      e === null ? e = [f] : e.push(f), r++;
    }
    let $ = r;
    var m = D === r;
    m && (g = true, x = 0);
    let R = r, j = r;
    if (!(l && typeof l == "object" && !Array.isArray(l))) {
      let f = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/1/type", keyword: "type", params: { type: "object", schema: Gn.oneOf[0].oneOf[1] }, message: "must be object", schema: Gn.oneOf[0].oneOf[1].type, parentSchema: Gn.oneOf[0].oneOf[1], data: l };
      e === null ? e = [f] : e.push(f), r++;
    }
    var m = R === r;
    if (m && g ? (g = false, x = [x, 1]) : m && (g = true, x = 1), g)
      r = y, e !== null && (y ? e.length = y : e = null);
    else {
      let f = { instancePath: o, schemaPath: "#/oneOf/0/oneOf", keyword: "oneOf", params: { passingSchemas: x, schema: Gn.oneOf[0] }, message: "must match exactly one schema in oneOf", schema: Gn.oneOf[0].oneOf, parentSchema: Gn.oneOf[0], data: l };
      e === null ? e = [f] : e.push(f), r++;
    }
    var V = u === r;
    V && (c = true, d = 0);
    let se = r, P = r;
    if (Array.isArray(l)) {
      let f = true, ft = l.length;
      for (let Cr = 0; Cr < ft; Cr++) {
        let Dr = l[Cr], wr = r, ta = r, jt = false, ee = null, za = r;
        if (typeof Dr != "string") {
          let fa = { instancePath: o + "/" + Cr, schemaPath: "#/oneOf/1/items/oneOf/0/type", keyword: "type", params: { type: "string", schema: Gn.oneOf[1].items.oneOf[0] }, message: "must be string", schema: Gn.oneOf[1].items.oneOf[0].type, parentSchema: Gn.oneOf[1].items.oneOf[0], data: Dr };
          e === null ? e = [fa] : e.push(fa), r++;
        }
        let sa = r;
        var oe = za === r;
        oe && (jt = true, ee = 0);
        let _e = r, Ma = r;
        if (!(Dr && typeof Dr == "object" && !Array.isArray(Dr))) {
          let fa = { instancePath: o + "/" + Cr, schemaPath: "#/oneOf/1/items/oneOf/1/type", keyword: "type", params: { type: "object", schema: Gn.oneOf[1].items.oneOf[1] }, message: "must be object", schema: Gn.oneOf[1].items.oneOf[1].type, parentSchema: Gn.oneOf[1].items.oneOf[1], data: Dr };
          e === null ? e = [fa] : e.push(fa), r++;
        }
        var oe = _e === r;
        if (oe && jt ? (jt = false, ee = [ee, 1]) : oe && (jt = true, ee = 1), jt)
          r = ta, e !== null && (ta ? e.length = ta : e = null);
        else {
          let fa = { instancePath: o + "/" + Cr, schemaPath: "#/oneOf/1/items/oneOf", keyword: "oneOf", params: { passingSchemas: ee, schema: Gn.oneOf[1].items }, message: "must match exactly one schema in oneOf", schema: Gn.oneOf[1].items.oneOf, parentSchema: Gn.oneOf[1].items, data: Dr };
          e === null ? e = [fa] : e.push(fa), r++;
        }
        var h = wr === r;
        h || (f = false);
      }
    } else {
      let f = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Gn.oneOf[1] }, message: "must be array", schema: Gn.oneOf[1].type, parentSchema: Gn.oneOf[1], data: l };
      e === null ? e = [f] : e.push(f), r++;
    }
    var V = se === r;
    if (V && c ? (c = false, d = [d, 1]) : V && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let f = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Gn }, message: "must match exactly one schema in oneOf", schema: Gn.oneOf, parentSchema: Gn, data: l };
      e === null ? e = [f] : e.push(f), r++;
    }
    return ko.errors = e, r === 0;
  }
  B["quarto-resource-document-attributes-copyright"] = Do;
  var eh = { type: "object", description: "be an object", properties: {}, patternProperties: {}, $id: "quarto-resource-document-attributes-copyright" };
  function Do(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (!(l && typeof l == "object" && !Array.isArray(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: eh }, message: "must be object", schema: eh.type, parentSchema: eh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Do.errors = e, r === 0;
  }
  B["quarto-resource-document-attributes-article"] = wo;
  var rh = { type: "object", description: "be an object", properties: {}, patternProperties: {}, $id: "quarto-resource-document-attributes-article" };
  function wo(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (!(l && typeof l == "object" && !Array.isArray(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: rh }, message: "must be object", schema: rh.type, parentSchema: rh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return wo.errors = e, r === 0;
  }
  B["quarto-resource-document-attributes-journal"] = $o;
  var th = { type: "object", description: "be an object", properties: {}, patternProperties: {}, $id: "quarto-resource-document-attributes-journal" };
  function $o(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (!(l && typeof l == "object" && !Array.isArray(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: th }, message: "must be object", schema: th.type, parentSchema: th, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return $o.errors = e, r === 0;
  }
  B["quarto-resource-document-attributes-institute"] = ro;
  var $i = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-attributes-institute" };
  function ro(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: $i.oneOf[0] }, message: "must be string", schema: $i.oneOf[0].type, parentSchema: $i.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: $i.oneOf[1].items }, message: "must be string", schema: $i.oneOf[1].items.type, parentSchema: $i.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: $i.oneOf[1] }, message: "must be array", schema: $i.oneOf[1].type, parentSchema: $i.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: $i }, message: "must match exactly one schema in oneOf", schema: $i.oneOf, parentSchema: $i, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return ro.errors = e, r === 0;
  }
  B["quarto-resource-document-attributes-abstract"] = ga;
  var ah = { type: "string", description: "be a string", $id: "quarto-resource-document-attributes-abstract" };
  function ga(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: ah }, message: "must be string", schema: ah.type, parentSchema: ah, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ga.errors = e, r === 0;
  }
  B["quarto-resource-document-attributes-abstract-title"] = to;
  var nh = { type: "string", description: "be a string", $id: "quarto-resource-document-attributes-abstract-title" };
  function to(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: nh }, message: "must be string", schema: nh.type, parentSchema: nh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return to.errors = e, r === 0;
  }
  B["quarto-resource-document-attributes-notes"] = So;
  var oh = { type: "string", description: "be a string", $id: "quarto-resource-document-attributes-notes" };
  function So(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: oh }, message: "must be string", schema: oh.type, parentSchema: oh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return So.errors = e, r === 0;
  }
  B["quarto-resource-document-attributes-tags"] = Fo;
  var Pd = { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" }, $id: "quarto-resource-document-attributes-tags" };
  function Fo(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let d = true, u = l.length;
      for (let y = 0; y < u; y++) {
        let g = l[y], x = r;
        if (typeof g != "string") {
          let $ = { instancePath: o + "/" + y, schemaPath: "#/items/type", keyword: "type", params: { type: "string", schema: Pd.items }, message: "must be string", schema: Pd.items.type, parentSchema: Pd.items, data: g };
          e === null ? e = [$] : e.push($), r++;
        }
        let D = r;
        var c = x === r;
        c || (d = false);
      }
    } else {
      let d = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: Pd }, message: "must be array", schema: Pd.type, parentSchema: Pd, data: l };
      e === null ? e = [d] : e.push(d), r++;
    }
    return Fo.errors = e, r === 0;
  }
  B["quarto-resource-document-attributes-doi"] = Jo;
  var sh = { type: "string", description: "be a string", $id: "quarto-resource-document-attributes-doi" };
  function Jo(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: sh }, message: "must be string", schema: sh.type, parentSchema: sh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Jo.errors = e, r === 0;
  }
  B["quarto-resource-document-attributes-thanks"] = Qo;
  var ih = { type: "string", description: "be a string", $id: "quarto-resource-document-attributes-thanks" };
  function Qo(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: ih }, message: "must be string", schema: ih.type, parentSchema: ih, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Qo.errors = e, r === 0;
  }
  B["quarto-resource-document-code-code-copy"] = ln;
  var Al = { oneOf: [{ enum: ["hover"], description: "be 'hover'" }, { type: "boolean", description: "be `true` or `false`" }], description: "be exactly one of: 'hover', `true` or `false`", $id: "quarto-resource-document-code-code-copy" };
  function ln(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r, y = Al.oneOf[0].enum;
    if (l !== "hover") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/0/enum", keyword: "enum", params: { allowedValues: Al.oneOf[0].enum, schema: Al.oneOf[0] }, message: "must be equal to one of the allowed values", schema: Al.oneOf[0].enum, parentSchema: Al.oneOf[0], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    var g = u === r;
    g && (c = true, d = 0);
    let x = r;
    if (typeof l != "boolean") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "boolean", schema: Al.oneOf[1] }, message: "must be boolean", schema: Al.oneOf[1].type, parentSchema: Al.oneOf[1], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let D = r;
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Al }, message: "must match exactly one schema in oneOf", schema: Al.oneOf, parentSchema: Al, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return ln.errors = e, r === 0;
  }
  B["quarto-resource-document-code-code-link"] = On;
  var lh = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-code-code-link" };
  function On(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: lh }, message: "must be boolean", schema: lh.type, parentSchema: lh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return On.errors = e, r === 0;
  }
  B["quarto-resource-document-code-code-tools"] = Go;
  var Yn = { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { source: { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "string", description: "be a string" }], description: "be exactly one of: `true` or `false`, a string" }, toggle: { type: "boolean", description: "be `true` or `false`" }, caption: { type: "string", description: "be a string" } }, patternProperties: {} }], description: "be exactly one of: `true` or `false`, an object", $id: "quarto-resource-document-code-code-tools" };
  function Go(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "boolean") {
      let R = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: Yn.oneOf[0] }, message: "must be boolean", schema: Yn.oneOf[0].type, parentSchema: Yn.oneOf[0], data: l };
      e === null ? e = [R] : e.push(R), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      if (l.source !== void 0) {
        let R = l.source, j = r, V = r, se = false, P = null, oe = r;
        if (typeof R != "boolean") {
          let Cr = { instancePath: o + "/source", schemaPath: "#/oneOf/1/properties/source/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: Yn.oneOf[1].properties.source.oneOf[0] }, message: "must be boolean", schema: Yn.oneOf[1].properties.source.oneOf[0].type, parentSchema: Yn.oneOf[1].properties.source.oneOf[0], data: R };
          e === null ? e = [Cr] : e.push(Cr), r++;
        }
        let h = r;
        var $ = oe === r;
        $ && (se = true, P = 0);
        let f = r;
        if (typeof R != "string") {
          let Cr = { instancePath: o + "/source", schemaPath: "#/oneOf/1/properties/source/oneOf/1/type", keyword: "type", params: { type: "string", schema: Yn.oneOf[1].properties.source.oneOf[1] }, message: "must be string", schema: Yn.oneOf[1].properties.source.oneOf[1].type, parentSchema: Yn.oneOf[1].properties.source.oneOf[1], data: R };
          e === null ? e = [Cr] : e.push(Cr), r++;
        }
        let ft = r;
        var $ = f === r;
        if ($ && se ? (se = false, P = [P, 1]) : $ && (se = true, P = 1), se)
          r = V, e !== null && (V ? e.length = V : e = null);
        else {
          let Cr = { instancePath: o + "/source", schemaPath: "#/oneOf/1/properties/source/oneOf", keyword: "oneOf", params: { passingSchemas: P, schema: Yn.oneOf[1].properties.source }, message: "must match exactly one schema in oneOf", schema: Yn.oneOf[1].properties.source.oneOf, parentSchema: Yn.oneOf[1].properties.source, data: R };
          e === null ? e = [Cr] : e.push(Cr), r++;
        }
        var m = j === r;
      }
      if (l.toggle !== void 0) {
        let R = l.toggle, j = r;
        if (typeof R != "boolean") {
          let P = { instancePath: o + "/toggle", schemaPath: "#/oneOf/1/properties/toggle/type", keyword: "type", params: { type: "boolean", schema: Yn.oneOf[1].properties.toggle }, message: "must be boolean", schema: Yn.oneOf[1].properties.toggle.type, parentSchema: Yn.oneOf[1].properties.toggle, data: R };
          e === null ? e = [P] : e.push(P), r++;
        }
        let V = r;
        var m = j === r;
      }
      if (l.caption !== void 0) {
        let R = l.caption, j = r;
        if (typeof R != "string") {
          let P = { instancePath: o + "/caption", schemaPath: "#/oneOf/1/properties/caption/type", keyword: "type", params: { type: "string", schema: Yn.oneOf[1].properties.caption }, message: "must be string", schema: Yn.oneOf[1].properties.caption.type, parentSchema: Yn.oneOf[1].properties.caption, data: R };
          e === null ? e = [P] : e.push(P), r++;
        }
        let V = r;
        var m = j === r;
      }
    } else {
      let R = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "object", schema: Yn.oneOf[1] }, message: "must be object", schema: Yn.oneOf[1].type, parentSchema: Yn.oneOf[1], data: l };
      e === null ? e = [R] : e.push(R), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let R = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Yn }, message: "must match exactly one schema in oneOf", schema: Yn.oneOf, parentSchema: Yn, data: l };
      e === null ? e = [R] : e.push(R), r++;
    }
    return Go.errors = e, r === 0;
  }
  B["quarto-resource-document-code-highlight-style"] = _a;
  var ch = { type: "string", description: "be a string", $id: "quarto-resource-document-code-highlight-style" };
  function _a(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: ch }, message: "must be string", schema: ch.type, parentSchema: ch, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return _a.errors = e, r === 0;
  }
  B["quarto-resource-document-code-syntax-definition"] = va;
  var ph = { type: "string", description: "be a string", $id: "quarto-resource-document-code-syntax-definition" };
  function va(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: ph }, message: "must be string", schema: ph.type, parentSchema: ph, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return va.errors = e, r === 0;
  }
  B["quarto-resource-document-code-syntax-definitions"] = qa;
  var xd = { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" }, $id: "quarto-resource-document-code-syntax-definitions" };
  function qa(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let d = true, u = l.length;
      for (let y = 0; y < u; y++) {
        let g = l[y], x = r;
        if (typeof g != "string") {
          let $ = { instancePath: o + "/" + y, schemaPath: "#/items/type", keyword: "type", params: { type: "string", schema: xd.items }, message: "must be string", schema: xd.items.type, parentSchema: xd.items, data: g };
          e === null ? e = [$] : e.push($), r++;
        }
        let D = r;
        var c = x === r;
        c || (d = false);
      }
    } else {
      let d = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: xd }, message: "must be array", schema: xd.type, parentSchema: xd, data: l };
      e === null ? e = [d] : e.push(d), r++;
    }
    return qa.errors = e, r === 0;
  }
  B["quarto-resource-document-code-listings"] = Yo;
  var uh = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-code-listings" };
  function Yo(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: uh }, message: "must be boolean", schema: uh.type, parentSchema: uh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Yo.errors = e, r === 0;
  }
  B["quarto-resource-document-code-indented-code-classes"] = Oa;
  var kd = { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" }, $id: "quarto-resource-document-code-indented-code-classes" };
  function Oa(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let d = true, u = l.length;
      for (let y = 0; y < u; y++) {
        let g = l[y], x = r;
        if (typeof g != "string") {
          let $ = { instancePath: o + "/" + y, schemaPath: "#/items/type", keyword: "type", params: { type: "string", schema: kd.items }, message: "must be string", schema: kd.items.type, parentSchema: kd.items, data: g };
          e === null ? e = [$] : e.push($), r++;
        }
        let D = r;
        var c = x === r;
        c || (d = false);
      }
    } else {
      let d = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: kd }, message: "must be array", schema: kd.type, parentSchema: kd, data: l };
      e === null ? e = [d] : e.push(d), r++;
    }
    return Oa.errors = e, r === 0;
  }
  B["quarto-resource-document-colors-fontcolor"] = Pn;
  var dh = { type: "string", description: "be a string", $id: "quarto-resource-document-colors-fontcolor" };
  function Pn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: dh }, message: "must be string", schema: dh.type, parentSchema: dh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Pn.errors = e, r === 0;
  }
  B["quarto-resource-document-colors-linkcolor"] = Ua;
  var fh = { type: "string", description: "be a string", $id: "quarto-resource-document-colors-linkcolor" };
  function Ua(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: fh }, message: "must be string", schema: fh.type, parentSchema: fh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ua.errors = e, r === 0;
  }
  B["quarto-resource-document-colors-monobackgroundcolor"] = xn;
  var mh = { type: "string", description: "be a string", $id: "quarto-resource-document-colors-monobackgroundcolor" };
  function xn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: mh }, message: "must be string", schema: mh.type, parentSchema: mh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return xn.errors = e, r === 0;
  }
  B["quarto-resource-document-colors-backgroundcolor"] = kn;
  var gh = { type: "string", description: "be a string", $id: "quarto-resource-document-colors-backgroundcolor" };
  function kn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: gh }, message: "must be string", schema: gh.type, parentSchema: gh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return kn.errors = e, r === 0;
  }
  B["quarto-resource-document-colors-filecolor"] = Xo;
  var hh = { type: "string", description: "be a string", $id: "quarto-resource-document-colors-filecolor" };
  function Xo(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: hh }, message: "must be string", schema: hh.type, parentSchema: hh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Xo.errors = e, r === 0;
  }
  B["quarto-resource-document-colors-citecolor"] = Zo;
  var yh = { type: "string", description: "be a string", $id: "quarto-resource-document-colors-citecolor" };
  function Zo(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: yh }, message: "must be string", schema: yh.type, parentSchema: yh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Zo.errors = e, r === 0;
  }
  B["quarto-resource-document-colors-urlcolor"] = es;
  var bh = { type: "string", description: "be a string", $id: "quarto-resource-document-colors-urlcolor" };
  function es(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: bh }, message: "must be string", schema: bh.type, parentSchema: bh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return es.errors = e, r === 0;
  }
  B["quarto-resource-document-colors-toccolor"] = rs;
  var _h = { type: "string", description: "be a string", $id: "quarto-resource-document-colors-toccolor" };
  function rs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: _h }, message: "must be string", schema: _h.type, parentSchema: _h, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return rs.errors = e, r === 0;
  }
  B["quarto-resource-document-colors-colorlinks"] = ts;
  var vh = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-colors-colorlinks" };
  function ts(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: vh }, message: "must be boolean", schema: vh.type, parentSchema: vh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ts.errors = e, r === 0;
  }
  B["quarto-resource-document-colors-contrastcolor"] = vc;
  var qh = { type: "string", description: "be a string", $id: "quarto-resource-document-colors-contrastcolor" };
  function vc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: qh }, message: "must be string", schema: qh.type, parentSchema: qh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return vc.errors = e, r === 0;
  }
  B["quarto-resource-document-comments-comments"] = Dn;
  function Dn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    return Uo(l, { instancePath: o, parentData: S, parentDataProperty: F, rootData: s }) || (e = e === null ? Uo.errors : e.concat(Uo.errors), r = e.length), Dn.errors = e, r === 0;
  }
  B["quarto-resource-document-crossref-crossref"] = Ie;
  var qt = { type: "object", description: "be an object", properties: { chapters: { type: "boolean", description: "be `true` or `false`" }, "title-delim": { type: "string", description: "be a string" }, "fig-title": { type: "string", description: "be a string" }, "tbl-title": { type: "string", description: "be a string" }, "eq-title": { type: "string", description: "be a string" }, "lst-title": { type: "string", description: "be a string" }, "thm-title": { type: "string", description: "be a string" }, "lem-title": { type: "string", description: "be a string" }, "cor-title": { type: "string", description: "be a string" }, "prp-title": { type: "string", description: "be a string" }, "cnj-title": { type: "string", description: "be a string" }, "def-title": { type: "string", description: "be a string" }, "exm-title": { type: "string", description: "be a string" }, "exr-title": { type: "string", description: "be a string" }, "fig-prefix": { type: "string", description: "be a string" }, "tbl-prefix": { type: "string", description: "be a string" }, "eq-prefix": { type: "string", description: "be a string" }, "sec-prefix": { type: "string", description: "be a string" }, "lst-prefix": { type: "string", description: "be a string" }, "thm-prefix": { type: "string", description: "be a string" }, "lem-prefix": { type: "string", description: "be a string" }, "cor-prefix": { type: "string", description: "be a string" }, "prp-prefix": { type: "string", description: "be a string" }, "cnj-prefix": { type: "string", description: "be a string" }, "def-prefix": { type: "string", description: "be a string" }, "exm-prefix": { type: "string", description: "be a string" }, "exr-prefix": { type: "string", description: "be a string" }, "fig-labels": { $ref: "crossref-labels-schema", description: "be crossref-labels-schema" }, "tbl-labels": { $ref: "crossref-labels-schema", description: "be crossref-labels-schema" }, "eq-labels": { $ref: "crossref-labels-schema", description: "be crossref-labels-schema" }, "sec-labels": { $ref: "crossref-labels-schema", description: "be crossref-labels-schema" }, "lst-labels": { $ref: "crossref-labels-schema", description: "be crossref-labels-schema" }, "thm-labels": { $ref: "crossref-labels-schema", description: "be crossref-labels-schema" }, "lem-labels": { $ref: "crossref-labels-schema", description: "be crossref-labels-schema" }, "cor-labels": { $ref: "crossref-labels-schema", description: "be crossref-labels-schema" }, "prp-labels": { $ref: "crossref-labels-schema", description: "be crossref-labels-schema" }, "cnj-labels": { $ref: "crossref-labels-schema", description: "be crossref-labels-schema" }, "def-labels": { $ref: "crossref-labels-schema", description: "be crossref-labels-schema" }, "exm-labels": { $ref: "crossref-labels-schema", description: "be crossref-labels-schema" }, "exr-labels": { $ref: "crossref-labels-schema", description: "be crossref-labels-schema" }, "lof-title": { type: "string", description: "be a string" }, "lot-title": { type: "string", description: "be a string" }, "lol-title": { type: "string", description: "be a string" }, labels: { $ref: "crossref-labels-schema", description: "be crossref-labels-schema" }, "subref-labels": { $ref: "crossref-labels-schema", description: "be crossref-labels-schema" }, "ref-hyperlink": { type: "boolean", description: "be `true` or `false`" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(title_delim|titleDelim|fig_title|figTitle|tbl_title|tblTitle|eq_title|eqTitle|lst_title|lstTitle|thm_title|thmTitle|lem_title|lemTitle|cor_title|corTitle|prp_title|prpTitle|cnj_title|cnjTitle|def_title|defTitle|exm_title|exmTitle|exr_title|exrTitle|fig_prefix|figPrefix|tbl_prefix|tblPrefix|eq_prefix|eqPrefix|sec_prefix|secPrefix|lst_prefix|lstPrefix|thm_prefix|thmPrefix|lem_prefix|lemPrefix|cor_prefix|corPrefix|prp_prefix|prpPrefix|cnj_prefix|cnjPrefix|def_prefix|defPrefix|exm_prefix|exmPrefix|exr_prefix|exrPrefix|fig_labels|figLabels|tbl_labels|tblLabels|eq_labels|eqLabels|sec_labels|secLabels|lst_labels|lstLabels|thm_labels|thmLabels|lem_labels|lemLabels|cor_labels|corLabels|prp_labels|prpLabels|cnj_labels|cnjLabels|def_labels|defLabels|exm_labels|exmLabels|exr_labels|exrLabels|lof_title|lofTitle|lot_title|lotTitle|lol_title|lolTitle|subref_labels|subrefLabels|ref_hyperlink|refHyperlink))" }, $id: "quarto-resource-document-crossref-crossref" }, D0 = new RegExp("^(?!(title_delim|titleDelim|fig_title|figTitle|tbl_title|tblTitle|eq_title|eqTitle|lst_title|lstTitle|thm_title|thmTitle|lem_title|lemTitle|cor_title|corTitle|prp_title|prpTitle|cnj_title|cnjTitle|def_title|defTitle|exm_title|exmTitle|exr_title|exrTitle|fig_prefix|figPrefix|tbl_prefix|tblPrefix|eq_prefix|eqPrefix|sec_prefix|secPrefix|lst_prefix|lstPrefix|thm_prefix|thmPrefix|lem_prefix|lemPrefix|cor_prefix|corPrefix|prp_prefix|prpPrefix|cnj_prefix|cnjPrefix|def_prefix|defPrefix|exm_prefix|exmPrefix|exr_prefix|exrPrefix|fig_labels|figLabels|tbl_labels|tblLabels|eq_labels|eqLabels|sec_labels|secLabels|lst_labels|lstLabels|thm_labels|thmLabels|lem_labels|lemLabels|cor_labels|corLabels|prp_labels|prpLabels|cnj_labels|cnjLabels|def_labels|defLabels|exm_labels|exmLabels|exr_labels|exrLabels|lof_title|lofTitle|lot_title|lotTitle|lol_title|lolTitle|subref_labels|subrefLabels|ref_hyperlink|refHyperlink))", "u");
  function Ie(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let u in l) {
        let y = r, g = r;
        if (typeof u == "string") {
          if (!D0.test(u)) {
            let x = { instancePath: o, schemaPath: "#/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(title_delim|titleDelim|fig_title|figTitle|tbl_title|tblTitle|eq_title|eqTitle|lst_title|lstTitle|thm_title|thmTitle|lem_title|lemTitle|cor_title|corTitle|prp_title|prpTitle|cnj_title|cnjTitle|def_title|defTitle|exm_title|exmTitle|exr_title|exrTitle|fig_prefix|figPrefix|tbl_prefix|tblPrefix|eq_prefix|eqPrefix|sec_prefix|secPrefix|lst_prefix|lstPrefix|thm_prefix|thmPrefix|lem_prefix|lemPrefix|cor_prefix|corPrefix|prp_prefix|prpPrefix|cnj_prefix|cnjPrefix|def_prefix|defPrefix|exm_prefix|exmPrefix|exr_prefix|exrPrefix|fig_labels|figLabels|tbl_labels|tblLabels|eq_labels|eqLabels|sec_labels|secLabels|lst_labels|lstLabels|thm_labels|thmLabels|lem_labels|lemLabels|cor_labels|corLabels|prp_labels|prpLabels|cnj_labels|cnjLabels|def_labels|defLabels|exm_labels|exmLabels|exr_labels|exrLabels|lof_title|lofTitle|lot_title|lotTitle|lol_title|lolTitle|subref_labels|subrefLabels|ref_hyperlink|refHyperlink))", schema: qt.propertyNames }, message: 'must match pattern "^(?!(title_delim|titleDelim|fig_title|figTitle|tbl_title|tblTitle|eq_title|eqTitle|lst_title|lstTitle|thm_title|thmTitle|lem_title|lemTitle|cor_title|corTitle|prp_title|prpTitle|cnj_title|cnjTitle|def_title|defTitle|exm_title|exmTitle|exr_title|exrTitle|fig_prefix|figPrefix|tbl_prefix|tblPrefix|eq_prefix|eqPrefix|sec_prefix|secPrefix|lst_prefix|lstPrefix|thm_prefix|thmPrefix|lem_prefix|lemPrefix|cor_prefix|corPrefix|prp_prefix|prpPrefix|cnj_prefix|cnjPrefix|def_prefix|defPrefix|exm_prefix|exmPrefix|exr_prefix|exrPrefix|fig_labels|figLabels|tbl_labels|tblLabels|eq_labels|eqLabels|sec_labels|secLabels|lst_labels|lstLabels|thm_labels|thmLabels|lem_labels|lemLabels|cor_labels|corLabels|prp_labels|prpLabels|cnj_labels|cnjLabels|def_labels|defLabels|exm_labels|exmLabels|exr_labels|exrLabels|lof_title|lofTitle|lot_title|lotTitle|lol_title|lolTitle|subref_labels|subrefLabels|ref_hyperlink|refHyperlink))"', schema: "^(?!(title_delim|titleDelim|fig_title|figTitle|tbl_title|tblTitle|eq_title|eqTitle|lst_title|lstTitle|thm_title|thmTitle|lem_title|lemTitle|cor_title|corTitle|prp_title|prpTitle|cnj_title|cnjTitle|def_title|defTitle|exm_title|exmTitle|exr_title|exrTitle|fig_prefix|figPrefix|tbl_prefix|tblPrefix|eq_prefix|eqPrefix|sec_prefix|secPrefix|lst_prefix|lstPrefix|thm_prefix|thmPrefix|lem_prefix|lemPrefix|cor_prefix|corPrefix|prp_prefix|prpPrefix|cnj_prefix|cnjPrefix|def_prefix|defPrefix|exm_prefix|exmPrefix|exr_prefix|exrPrefix|fig_labels|figLabels|tbl_labels|tblLabels|eq_labels|eqLabels|sec_labels|secLabels|lst_labels|lstLabels|thm_labels|thmLabels|lem_labels|lemLabels|cor_labels|corLabels|prp_labels|prpLabels|cnj_labels|cnjLabels|def_labels|defLabels|exm_labels|exmLabels|exr_labels|exrLabels|lof_title|lofTitle|lot_title|lotTitle|lol_title|lolTitle|subref_labels|subrefLabels|ref_hyperlink|refHyperlink))", parentSchema: qt.propertyNames, data: u, propertyName: u };
            e === null ? e = [x] : e.push(x), r++;
          }
        } else {
          let x = { instancePath: o, schemaPath: "#/propertyNames/type", keyword: "type", params: { type: "string", schema: qt.propertyNames }, message: "must be string", schema: qt.propertyNames.type, parentSchema: qt.propertyNames, data: u, propertyName: u };
          e === null ? e = [x] : e.push(x), r++;
        }
        var c = y === r;
        if (!c) {
          let x = { instancePath: o, schemaPath: "#/propertyNames", keyword: "propertyNames", params: { propertyName: u, schema: qt }, message: "property name must be valid", schema: qt.propertyNames, parentSchema: qt, data: l };
          e === null ? e = [x] : e.push(x), r++;
        }
      }
      if (l.chapters !== void 0) {
        let u = l.chapters, y = r;
        if (typeof u != "boolean") {
          let x = { instancePath: o + "/chapters", schemaPath: "#/properties/chapters/type", keyword: "type", params: { type: "boolean", schema: qt.properties.chapters }, message: "must be boolean", schema: qt.properties.chapters.type, parentSchema: qt.properties.chapters, data: u };
          e === null ? e = [x] : e.push(x), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["title-delim"] !== void 0) {
        let u = l["title-delim"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/title-delim", schemaPath: "#/properties/title-delim/type", keyword: "type", params: { type: "string", schema: qt.properties["title-delim"] }, message: "must be string", schema: qt.properties["title-delim"].type, parentSchema: qt.properties["title-delim"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["fig-title"] !== void 0) {
        let u = l["fig-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/fig-title", schemaPath: "#/properties/fig-title/type", keyword: "type", params: { type: "string", schema: qt.properties["fig-title"] }, message: "must be string", schema: qt.properties["fig-title"].type, parentSchema: qt.properties["fig-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["tbl-title"] !== void 0) {
        let u = l["tbl-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/tbl-title", schemaPath: "#/properties/tbl-title/type", keyword: "type", params: { type: "string", schema: qt.properties["tbl-title"] }, message: "must be string", schema: qt.properties["tbl-title"].type, parentSchema: qt.properties["tbl-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["eq-title"] !== void 0) {
        let u = l["eq-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/eq-title", schemaPath: "#/properties/eq-title/type", keyword: "type", params: { type: "string", schema: qt.properties["eq-title"] }, message: "must be string", schema: qt.properties["eq-title"].type, parentSchema: qt.properties["eq-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["lst-title"] !== void 0) {
        let u = l["lst-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/lst-title", schemaPath: "#/properties/lst-title/type", keyword: "type", params: { type: "string", schema: qt.properties["lst-title"] }, message: "must be string", schema: qt.properties["lst-title"].type, parentSchema: qt.properties["lst-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["thm-title"] !== void 0) {
        let u = l["thm-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/thm-title", schemaPath: "#/properties/thm-title/type", keyword: "type", params: { type: "string", schema: qt.properties["thm-title"] }, message: "must be string", schema: qt.properties["thm-title"].type, parentSchema: qt.properties["thm-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["lem-title"] !== void 0) {
        let u = l["lem-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/lem-title", schemaPath: "#/properties/lem-title/type", keyword: "type", params: { type: "string", schema: qt.properties["lem-title"] }, message: "must be string", schema: qt.properties["lem-title"].type, parentSchema: qt.properties["lem-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["cor-title"] !== void 0) {
        let u = l["cor-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/cor-title", schemaPath: "#/properties/cor-title/type", keyword: "type", params: { type: "string", schema: qt.properties["cor-title"] }, message: "must be string", schema: qt.properties["cor-title"].type, parentSchema: qt.properties["cor-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["prp-title"] !== void 0) {
        let u = l["prp-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/prp-title", schemaPath: "#/properties/prp-title/type", keyword: "type", params: { type: "string", schema: qt.properties["prp-title"] }, message: "must be string", schema: qt.properties["prp-title"].type, parentSchema: qt.properties["prp-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["cnj-title"] !== void 0) {
        let u = l["cnj-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/cnj-title", schemaPath: "#/properties/cnj-title/type", keyword: "type", params: { type: "string", schema: qt.properties["cnj-title"] }, message: "must be string", schema: qt.properties["cnj-title"].type, parentSchema: qt.properties["cnj-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["def-title"] !== void 0) {
        let u = l["def-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/def-title", schemaPath: "#/properties/def-title/type", keyword: "type", params: { type: "string", schema: qt.properties["def-title"] }, message: "must be string", schema: qt.properties["def-title"].type, parentSchema: qt.properties["def-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["exm-title"] !== void 0) {
        let u = l["exm-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/exm-title", schemaPath: "#/properties/exm-title/type", keyword: "type", params: { type: "string", schema: qt.properties["exm-title"] }, message: "must be string", schema: qt.properties["exm-title"].type, parentSchema: qt.properties["exm-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["exr-title"] !== void 0) {
        let u = l["exr-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/exr-title", schemaPath: "#/properties/exr-title/type", keyword: "type", params: { type: "string", schema: qt.properties["exr-title"] }, message: "must be string", schema: qt.properties["exr-title"].type, parentSchema: qt.properties["exr-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["fig-prefix"] !== void 0) {
        let u = l["fig-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/fig-prefix", schemaPath: "#/properties/fig-prefix/type", keyword: "type", params: { type: "string", schema: qt.properties["fig-prefix"] }, message: "must be string", schema: qt.properties["fig-prefix"].type, parentSchema: qt.properties["fig-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["tbl-prefix"] !== void 0) {
        let u = l["tbl-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/tbl-prefix", schemaPath: "#/properties/tbl-prefix/type", keyword: "type", params: { type: "string", schema: qt.properties["tbl-prefix"] }, message: "must be string", schema: qt.properties["tbl-prefix"].type, parentSchema: qt.properties["tbl-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["eq-prefix"] !== void 0) {
        let u = l["eq-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/eq-prefix", schemaPath: "#/properties/eq-prefix/type", keyword: "type", params: { type: "string", schema: qt.properties["eq-prefix"] }, message: "must be string", schema: qt.properties["eq-prefix"].type, parentSchema: qt.properties["eq-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["sec-prefix"] !== void 0) {
        let u = l["sec-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/sec-prefix", schemaPath: "#/properties/sec-prefix/type", keyword: "type", params: { type: "string", schema: qt.properties["sec-prefix"] }, message: "must be string", schema: qt.properties["sec-prefix"].type, parentSchema: qt.properties["sec-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["lst-prefix"] !== void 0) {
        let u = l["lst-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/lst-prefix", schemaPath: "#/properties/lst-prefix/type", keyword: "type", params: { type: "string", schema: qt.properties["lst-prefix"] }, message: "must be string", schema: qt.properties["lst-prefix"].type, parentSchema: qt.properties["lst-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["thm-prefix"] !== void 0) {
        let u = l["thm-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/thm-prefix", schemaPath: "#/properties/thm-prefix/type", keyword: "type", params: { type: "string", schema: qt.properties["thm-prefix"] }, message: "must be string", schema: qt.properties["thm-prefix"].type, parentSchema: qt.properties["thm-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["lem-prefix"] !== void 0) {
        let u = l["lem-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/lem-prefix", schemaPath: "#/properties/lem-prefix/type", keyword: "type", params: { type: "string", schema: qt.properties["lem-prefix"] }, message: "must be string", schema: qt.properties["lem-prefix"].type, parentSchema: qt.properties["lem-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["cor-prefix"] !== void 0) {
        let u = l["cor-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/cor-prefix", schemaPath: "#/properties/cor-prefix/type", keyword: "type", params: { type: "string", schema: qt.properties["cor-prefix"] }, message: "must be string", schema: qt.properties["cor-prefix"].type, parentSchema: qt.properties["cor-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["prp-prefix"] !== void 0) {
        let u = l["prp-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/prp-prefix", schemaPath: "#/properties/prp-prefix/type", keyword: "type", params: { type: "string", schema: qt.properties["prp-prefix"] }, message: "must be string", schema: qt.properties["prp-prefix"].type, parentSchema: qt.properties["prp-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["cnj-prefix"] !== void 0) {
        let u = l["cnj-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/cnj-prefix", schemaPath: "#/properties/cnj-prefix/type", keyword: "type", params: { type: "string", schema: qt.properties["cnj-prefix"] }, message: "must be string", schema: qt.properties["cnj-prefix"].type, parentSchema: qt.properties["cnj-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["def-prefix"] !== void 0) {
        let u = l["def-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/def-prefix", schemaPath: "#/properties/def-prefix/type", keyword: "type", params: { type: "string", schema: qt.properties["def-prefix"] }, message: "must be string", schema: qt.properties["def-prefix"].type, parentSchema: qt.properties["def-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["exm-prefix"] !== void 0) {
        let u = l["exm-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/exm-prefix", schemaPath: "#/properties/exm-prefix/type", keyword: "type", params: { type: "string", schema: qt.properties["exm-prefix"] }, message: "must be string", schema: qt.properties["exm-prefix"].type, parentSchema: qt.properties["exm-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["exr-prefix"] !== void 0) {
        let u = l["exr-prefix"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/exr-prefix", schemaPath: "#/properties/exr-prefix/type", keyword: "type", params: { type: "string", schema: qt.properties["exr-prefix"] }, message: "must be string", schema: qt.properties["exr-prefix"].type, parentSchema: qt.properties["exr-prefix"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["fig-labels"] !== void 0) {
        let u = l["fig-labels"], y = r;
        Sa(u, { instancePath: o + "/fig-labels", parentData: l, parentDataProperty: "fig-labels", rootData: s }) || (e = e === null ? Sa.errors : e.concat(Sa.errors), r = e.length);
        var d = y === r;
      }
      if (l["tbl-labels"] !== void 0) {
        let u = l["tbl-labels"], y = r;
        Sa(u, { instancePath: o + "/tbl-labels", parentData: l, parentDataProperty: "tbl-labels", rootData: s }) || (e = e === null ? Sa.errors : e.concat(Sa.errors), r = e.length);
        var d = y === r;
      }
      if (l["eq-labels"] !== void 0) {
        let u = l["eq-labels"], y = r;
        Sa(u, { instancePath: o + "/eq-labels", parentData: l, parentDataProperty: "eq-labels", rootData: s }) || (e = e === null ? Sa.errors : e.concat(Sa.errors), r = e.length);
        var d = y === r;
      }
      if (l["sec-labels"] !== void 0) {
        let u = l["sec-labels"], y = r;
        Sa(u, { instancePath: o + "/sec-labels", parentData: l, parentDataProperty: "sec-labels", rootData: s }) || (e = e === null ? Sa.errors : e.concat(Sa.errors), r = e.length);
        var d = y === r;
      }
      if (l["lst-labels"] !== void 0) {
        let u = l["lst-labels"], y = r;
        Sa(u, { instancePath: o + "/lst-labels", parentData: l, parentDataProperty: "lst-labels", rootData: s }) || (e = e === null ? Sa.errors : e.concat(Sa.errors), r = e.length);
        var d = y === r;
      }
      if (l["thm-labels"] !== void 0) {
        let u = l["thm-labels"], y = r;
        Sa(u, { instancePath: o + "/thm-labels", parentData: l, parentDataProperty: "thm-labels", rootData: s }) || (e = e === null ? Sa.errors : e.concat(Sa.errors), r = e.length);
        var d = y === r;
      }
      if (l["lem-labels"] !== void 0) {
        let u = l["lem-labels"], y = r;
        Sa(u, { instancePath: o + "/lem-labels", parentData: l, parentDataProperty: "lem-labels", rootData: s }) || (e = e === null ? Sa.errors : e.concat(Sa.errors), r = e.length);
        var d = y === r;
      }
      if (l["cor-labels"] !== void 0) {
        let u = l["cor-labels"], y = r;
        Sa(u, { instancePath: o + "/cor-labels", parentData: l, parentDataProperty: "cor-labels", rootData: s }) || (e = e === null ? Sa.errors : e.concat(Sa.errors), r = e.length);
        var d = y === r;
      }
      if (l["prp-labels"] !== void 0) {
        let u = l["prp-labels"], y = r;
        Sa(u, { instancePath: o + "/prp-labels", parentData: l, parentDataProperty: "prp-labels", rootData: s }) || (e = e === null ? Sa.errors : e.concat(Sa.errors), r = e.length);
        var d = y === r;
      }
      if (l["cnj-labels"] !== void 0) {
        let u = l["cnj-labels"], y = r;
        Sa(u, { instancePath: o + "/cnj-labels", parentData: l, parentDataProperty: "cnj-labels", rootData: s }) || (e = e === null ? Sa.errors : e.concat(Sa.errors), r = e.length);
        var d = y === r;
      }
      if (l["def-labels"] !== void 0) {
        let u = l["def-labels"], y = r;
        Sa(u, { instancePath: o + "/def-labels", parentData: l, parentDataProperty: "def-labels", rootData: s }) || (e = e === null ? Sa.errors : e.concat(Sa.errors), r = e.length);
        var d = y === r;
      }
      if (l["exm-labels"] !== void 0) {
        let u = l["exm-labels"], y = r;
        Sa(u, { instancePath: o + "/exm-labels", parentData: l, parentDataProperty: "exm-labels", rootData: s }) || (e = e === null ? Sa.errors : e.concat(Sa.errors), r = e.length);
        var d = y === r;
      }
      if (l["exr-labels"] !== void 0) {
        let u = l["exr-labels"], y = r;
        Sa(u, { instancePath: o + "/exr-labels", parentData: l, parentDataProperty: "exr-labels", rootData: s }) || (e = e === null ? Sa.errors : e.concat(Sa.errors), r = e.length);
        var d = y === r;
      }
      if (l["lof-title"] !== void 0) {
        let u = l["lof-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/lof-title", schemaPath: "#/properties/lof-title/type", keyword: "type", params: { type: "string", schema: qt.properties["lof-title"] }, message: "must be string", schema: qt.properties["lof-title"].type, parentSchema: qt.properties["lof-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["lot-title"] !== void 0) {
        let u = l["lot-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/lot-title", schemaPath: "#/properties/lot-title/type", keyword: "type", params: { type: "string", schema: qt.properties["lot-title"] }, message: "must be string", schema: qt.properties["lot-title"].type, parentSchema: qt.properties["lot-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["lol-title"] !== void 0) {
        let u = l["lol-title"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/lol-title", schemaPath: "#/properties/lol-title/type", keyword: "type", params: { type: "string", schema: qt.properties["lol-title"] }, message: "must be string", schema: qt.properties["lol-title"].type, parentSchema: qt.properties["lol-title"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l.labels !== void 0) {
        let u = l.labels, y = r;
        Sa(u, { instancePath: o + "/labels", parentData: l, parentDataProperty: "labels", rootData: s }) || (e = e === null ? Sa.errors : e.concat(Sa.errors), r = e.length);
        var d = y === r;
      }
      if (l["subref-labels"] !== void 0) {
        let u = l["subref-labels"], y = r;
        Sa(u, { instancePath: o + "/subref-labels", parentData: l, parentDataProperty: "subref-labels", rootData: s }) || (e = e === null ? Sa.errors : e.concat(Sa.errors), r = e.length);
        var d = y === r;
      }
      if (l["ref-hyperlink"] !== void 0) {
        let u = l["ref-hyperlink"], y = r;
        if (typeof u != "boolean") {
          let D = { instancePath: o + "/ref-hyperlink", schemaPath: "#/properties/ref-hyperlink/type", keyword: "type", params: { type: "boolean", schema: qt.properties["ref-hyperlink"] }, message: "must be boolean", schema: qt.properties["ref-hyperlink"].type, parentSchema: qt.properties["ref-hyperlink"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
    } else {
      let u = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: qt }, message: "must be object", schema: qt.type, parentSchema: qt, data: l };
      e === null ? e = [u] : e.push(u), r++;
    }
    return Ie.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-identifier"] = as;
  var Ao = { oneOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: { text: { type: "string", description: "be a string" }, schema: { enum: ["ISBN-10", "GTIN-13", "UPC", "ISMN-10", "DOI", "LCCN", "GTIN-14", "ISBN-13", "Legal deposit number", "URN", "OCLC", "ISMN-13", "ISBN-A", "JP", "OLCC"], description: "be one of: `ISBN-10`, `GTIN-13`, `UPC`, `ISMN-10`, `DOI`, `LCCN`, `GTIN-14`, `ISBN-13`, `Legal deposit number`, `URN`, `OCLC`, `ISMN-13`, `ISBN-A`, `JP`, `OLCC`" } }, patternProperties: {} }], description: "be exactly one of: a string, an object", $id: "quarto-resource-document-epub-identifier" };
  function as(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Ao.oneOf[0] }, message: "must be string", schema: Ao.oneOf[0].type, parentSchema: Ao.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      if (l.text !== void 0) {
        let m = l.text, R = r;
        if (typeof m != "string") {
          let V = { instancePath: o + "/text", schemaPath: "#/oneOf/1/properties/text/type", keyword: "type", params: { type: "string", schema: Ao.oneOf[1].properties.text }, message: "must be string", schema: Ao.oneOf[1].properties.text.type, parentSchema: Ao.oneOf[1].properties.text, data: m };
          e === null ? e = [V] : e.push(V), r++;
        }
        let j = r;
        var $ = R === r;
      }
      if (l.schema !== void 0) {
        let m = l.schema, R = r, j = Ao.oneOf[1].properties.schema.enum;
        if (!(m === "ISBN-10" || m === "GTIN-13" || m === "UPC" || m === "ISMN-10" || m === "DOI" || m === "LCCN" || m === "GTIN-14" || m === "ISBN-13" || m === "Legal deposit number" || m === "URN" || m === "OCLC" || m === "ISMN-13" || m === "ISBN-A" || m === "JP" || m === "OLCC")) {
          let se = { instancePath: o + "/schema", schemaPath: "#/oneOf/1/properties/schema/enum", keyword: "enum", params: { allowedValues: Ao.oneOf[1].properties.schema.enum, schema: Ao.oneOf[1].properties.schema }, message: "must be equal to one of the allowed values", schema: Ao.oneOf[1].properties.schema.enum, parentSchema: Ao.oneOf[1].properties.schema, data: m };
          e === null ? e = [se] : e.push(se), r++;
        }
        var $ = R === r;
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "object", schema: Ao.oneOf[1] }, message: "must be object", schema: Ao.oneOf[1].type, parentSchema: Ao.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Ao }, message: "must match exactly one schema in oneOf", schema: Ao.oneOf, parentSchema: Ao, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return as.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-creator"] = ns;
  function ns(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    return _c(l, { instancePath: o, parentData: S, parentDataProperty: F, rootData: s }) || (e = e === null ? _c.errors : e.concat(_c.errors), r = e.length), ns.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-contributor"] = os;
  function os(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    return _c(l, { instancePath: o, parentData: S, parentDataProperty: F, rootData: s }) || (e = e === null ? _c.errors : e.concat(_c.errors), r = e.length), os.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-subject"] = Bl;
  var Oo = { oneOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: { text: { type: "string", description: "be a string" }, authority: { type: "string", description: "be a string" }, term: { type: "string", description: "be a string" } }, patternProperties: {} }], description: "be exactly one of: a string, an object", $id: "quarto-resource-document-epub-subject" };
  function Bl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Oo.oneOf[0] }, message: "must be string", schema: Oo.oneOf[0].type, parentSchema: Oo.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      if (l.text !== void 0) {
        let m = l.text, R = r;
        if (typeof m != "string") {
          let V = { instancePath: o + "/text", schemaPath: "#/oneOf/1/properties/text/type", keyword: "type", params: { type: "string", schema: Oo.oneOf[1].properties.text }, message: "must be string", schema: Oo.oneOf[1].properties.text.type, parentSchema: Oo.oneOf[1].properties.text, data: m };
          e === null ? e = [V] : e.push(V), r++;
        }
        let j = r;
        var $ = R === r;
      }
      if (l.authority !== void 0) {
        let m = l.authority, R = r;
        if (typeof m != "string") {
          let se = { instancePath: o + "/authority", schemaPath: "#/oneOf/1/properties/authority/type", keyword: "type", params: { type: "string", schema: Oo.oneOf[1].properties.authority }, message: "must be string", schema: Oo.oneOf[1].properties.authority.type, parentSchema: Oo.oneOf[1].properties.authority, data: m };
          e === null ? e = [se] : e.push(se), r++;
        }
        let j = r;
        var $ = R === r;
      }
      if (l.term !== void 0) {
        let m = l.term, R = r;
        if (typeof m != "string") {
          let se = { instancePath: o + "/term", schemaPath: "#/oneOf/1/properties/term/type", keyword: "type", params: { type: "string", schema: Oo.oneOf[1].properties.term }, message: "must be string", schema: Oo.oneOf[1].properties.term.type, parentSchema: Oo.oneOf[1].properties.term, data: m };
          e === null ? e = [se] : e.push(se), r++;
        }
        let j = r;
        var $ = R === r;
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "object", schema: Oo.oneOf[1] }, message: "must be object", schema: Oo.oneOf[1].type, parentSchema: Oo.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Oo }, message: "must match exactly one schema in oneOf", schema: Oo.oneOf, parentSchema: Oo, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Bl.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-type"] = ss;
  var Oh = { type: "string", description: "be a string", $id: "quarto-resource-document-epub-type" };
  function ss(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Oh }, message: "must be string", schema: Oh.type, parentSchema: Oh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ss.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-format"] = jl;
  var Ph = { type: "string", description: "be a string", $id: "quarto-resource-document-epub-format" };
  function jl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Ph }, message: "must be string", schema: Ph.type, parentSchema: Ph, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return jl.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-relation"] = is;
  var xh = { type: "string", description: "be a string", $id: "quarto-resource-document-epub-relation" };
  function is(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: xh }, message: "must be string", schema: xh.type, parentSchema: xh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return is.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-coverage"] = ls;
  var kh = { type: "string", description: "be a string", $id: "quarto-resource-document-epub-coverage" };
  function ls(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: kh }, message: "must be string", schema: kh.type, parentSchema: kh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ls.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-rights"] = cs;
  var Dh = { type: "string", description: "be a string", $id: "quarto-resource-document-epub-rights" };
  function cs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Dh }, message: "must be string", schema: Dh.type, parentSchema: Dh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return cs.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-belongs-to-collection"] = ps;
  var wh = { type: "string", description: "be a string", $id: "quarto-resource-document-epub-belongs-to-collection" };
  function ps(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: wh }, message: "must be string", schema: wh.type, parentSchema: wh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ps.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-group-position"] = us;
  var $h = { type: "number", description: "be a number", $id: "quarto-resource-document-epub-group-position" };
  function us(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: $h }, message: "must be number", schema: $h.type, parentSchema: $h, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return us.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-page-progression-direction"] = ds;
  var Wf = { enum: ["ltr", "rtl"], description: "be one of: `ltr`, `rtl`", $id: "quarto-resource-document-epub-page-progression-direction" };
  function ds(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Wf.enum;
    if (!(l === "ltr" || l === "rtl")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Wf.enum, schema: Wf }, message: "must be equal to one of the allowed values", schema: Wf.enum, parentSchema: Wf, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return ds.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-ibooks"] = fs;
  var no = { type: "object", description: "be an object", properties: { version: { type: "string", description: "be a string" }, "specified-fonts": { type: "boolean", description: "be `true` or `false`" }, "scroll-axis": { enum: ["vertical", "horizontal", "default"], description: "be one of: `vertical`, `horizontal`, `default`" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(specified_fonts|specifiedFonts|scroll_axis|scrollAxis))" }, $id: "quarto-resource-document-epub-ibooks" }, w0 = new RegExp("^(?!(specified_fonts|specifiedFonts|scroll_axis|scrollAxis))", "u");
  function fs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let u in l) {
        let y = r, g = r;
        if (typeof u == "string") {
          if (!w0.test(u)) {
            let x = { instancePath: o, schemaPath: "#/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(specified_fonts|specifiedFonts|scroll_axis|scrollAxis))", schema: no.propertyNames }, message: 'must match pattern "^(?!(specified_fonts|specifiedFonts|scroll_axis|scrollAxis))"', schema: "^(?!(specified_fonts|specifiedFonts|scroll_axis|scrollAxis))", parentSchema: no.propertyNames, data: u, propertyName: u };
            e === null ? e = [x] : e.push(x), r++;
          }
        } else {
          let x = { instancePath: o, schemaPath: "#/propertyNames/type", keyword: "type", params: { type: "string", schema: no.propertyNames }, message: "must be string", schema: no.propertyNames.type, parentSchema: no.propertyNames, data: u, propertyName: u };
          e === null ? e = [x] : e.push(x), r++;
        }
        var c = y === r;
        if (!c) {
          let x = { instancePath: o, schemaPath: "#/propertyNames", keyword: "propertyNames", params: { propertyName: u, schema: no }, message: "property name must be valid", schema: no.propertyNames, parentSchema: no, data: l };
          e === null ? e = [x] : e.push(x), r++;
        }
      }
      if (l.version !== void 0) {
        let u = l.version, y = r;
        if (typeof u != "string") {
          let x = { instancePath: o + "/version", schemaPath: "#/properties/version/type", keyword: "type", params: { type: "string", schema: no.properties.version }, message: "must be string", schema: no.properties.version.type, parentSchema: no.properties.version, data: u };
          e === null ? e = [x] : e.push(x), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["specified-fonts"] !== void 0) {
        let u = l["specified-fonts"], y = r;
        if (typeof u != "boolean") {
          let D = { instancePath: o + "/specified-fonts", schemaPath: "#/properties/specified-fonts/type", keyword: "type", params: { type: "boolean", schema: no.properties["specified-fonts"] }, message: "must be boolean", schema: no.properties["specified-fonts"].type, parentSchema: no.properties["specified-fonts"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["scroll-axis"] !== void 0) {
        let u = l["scroll-axis"], y = r, g = no.properties["scroll-axis"].enum;
        if (!(u === "vertical" || u === "horizontal" || u === "default")) {
          let D = { instancePath: o + "/scroll-axis", schemaPath: "#/properties/scroll-axis/enum", keyword: "enum", params: { allowedValues: no.properties["scroll-axis"].enum, schema: no.properties["scroll-axis"] }, message: "must be equal to one of the allowed values", schema: no.properties["scroll-axis"].enum, parentSchema: no.properties["scroll-axis"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        var d = y === r;
      }
    } else {
      let u = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: no }, message: "must be object", schema: no.type, parentSchema: no, data: l };
      e === null ? e = [u] : e.push(u), r++;
    }
    return fs.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-epub-metadata"] = ms;
  var Sh = { type: "string", description: "be a string", $id: "quarto-resource-document-epub-epub-metadata" };
  function ms(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Sh }, message: "must be string", schema: Sh.type, parentSchema: Sh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ms.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-epub-subdirectory"] = gs;
  var sc = { oneOf: [{ type: "string", description: "be a string" }, { type: "null", description: "be the null value" }], description: "be exactly one of: a string, the null value", $id: "quarto-resource-document-epub-epub-subdirectory" };
  function gs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: sc.oneOf[0] }, message: "must be string", schema: sc.oneOf[0].type, parentSchema: sc.oneOf[0], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r;
    if (l !== null) {
      let $ = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "null", schema: sc.oneOf[1] }, message: "must be null", schema: sc.oneOf[1].type, parentSchema: sc.oneOf[1], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let D = r;
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: sc }, message: "must match exactly one schema in oneOf", schema: sc.oneOf, parentSchema: sc, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return gs.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-epub-fonts"] = hs;
  var Dd = { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" }, $id: "quarto-resource-document-epub-epub-fonts" };
  function hs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let d = true, u = l.length;
      for (let y = 0; y < u; y++) {
        let g = l[y], x = r;
        if (typeof g != "string") {
          let $ = { instancePath: o + "/" + y, schemaPath: "#/items/type", keyword: "type", params: { type: "string", schema: Dd.items }, message: "must be string", schema: Dd.items.type, parentSchema: Dd.items, data: g };
          e === null ? e = [$] : e.push($), r++;
        }
        let D = r;
        var c = x === r;
        c || (d = false);
      }
    } else {
      let d = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: Dd }, message: "must be array", schema: Dd.type, parentSchema: Dd, data: l };
      e === null ? e = [d] : e.push(d), r++;
    }
    return hs.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-epub-chapter-level"] = ys;
  var Fh = { type: "number", description: "be a number", $id: "quarto-resource-document-epub-epub-chapter-level" };
  function ys(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: Fh }, message: "must be number", schema: Fh.type, parentSchema: Fh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ys.errors = e, r === 0;
  }
  B["quarto-resource-document-epub-epub-cover-image"] = bs;
  var Ah = { type: "string", description: "be a string", $id: "quarto-resource-document-epub-epub-cover-image" };
  function bs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Ah }, message: "must be string", schema: Ah.type, parentSchema: Ah, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return bs.errors = e, r === 0;
  }
  B["quarto-resource-document-execute-engine"] = qe;
  var Bh = { type: "string", description: "be a string", $id: "quarto-resource-document-execute-engine" };
  function qe(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Bh }, message: "must be string", schema: Bh.type, parentSchema: Bh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return qe.errors = e, r === 0;
  }
  B["quarto-resource-document-execute-jupyter"] = Oe;
  var Ka = { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: { kernelspec: { type: "object", description: "be an object", properties: { display_name: { type: "string", description: "be a string" }, language: { type: "string", description: "be a string" }, name: { type: "string", description: "be a string" } }, patternProperties: {}, required: ["display_name", "language", "name"], propertyNames: { type: "string", pattern: "^(?!(display-name|displayName))" } } }, patternProperties: {} }], description: "be exactly one of: `true` or `false`, a string, an object", $id: "quarto-resource-document-execute-jupyter" }, $0 = new RegExp("^(?!(display-name|displayName))", "u");
  function Oe(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "boolean") {
      let j = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: Ka.oneOf[0] }, message: "must be boolean", schema: Ka.oneOf[0].type, parentSchema: Ka.oneOf[0], data: l };
      e === null ? e = [j] : e.push(j), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r;
    if (typeof l != "string") {
      let j = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "string", schema: Ka.oneOf[1] }, message: "must be string", schema: Ka.oneOf[1].type, parentSchema: Ka.oneOf[1], data: l };
      e === null ? e = [j] : e.push(j), r++;
    }
    let D = r;
    var g = x === r;
    if (g && c)
      c = false, d = [d, 1];
    else {
      g && (c = true, d = 1);
      let j = r, V = r;
      if (l && typeof l == "object" && !Array.isArray(l)) {
        if (l.kernelspec !== void 0) {
          let P = l.kernelspec, oe = r, h = r;
          if (P && typeof P == "object" && !Array.isArray(P)) {
            if (P.display_name === void 0) {
              let f = { instancePath: o + "/kernelspec", schemaPath: "#/oneOf/2/properties/kernelspec/required", keyword: "required", params: { missingProperty: "display_name", schema: Ka.oneOf[2].properties.kernelspec }, message: "must have required property 'display_name'", schema: Ka.oneOf[2].properties.kernelspec.required, parentSchema: Ka.oneOf[2].properties.kernelspec, data: P };
              e === null ? e = [f] : e.push(f), r++;
            }
            if (P.language === void 0) {
              let f = { instancePath: o + "/kernelspec", schemaPath: "#/oneOf/2/properties/kernelspec/required", keyword: "required", params: { missingProperty: "language", schema: Ka.oneOf[2].properties.kernelspec }, message: "must have required property 'language'", schema: Ka.oneOf[2].properties.kernelspec.required, parentSchema: Ka.oneOf[2].properties.kernelspec, data: P };
              e === null ? e = [f] : e.push(f), r++;
            }
            if (P.name === void 0) {
              let f = { instancePath: o + "/kernelspec", schemaPath: "#/oneOf/2/properties/kernelspec/required", keyword: "required", params: { missingProperty: "name", schema: Ka.oneOf[2].properties.kernelspec }, message: "must have required property 'name'", schema: Ka.oneOf[2].properties.kernelspec.required, parentSchema: Ka.oneOf[2].properties.kernelspec, data: P };
              e === null ? e = [f] : e.push(f), r++;
            }
            for (let f in P) {
              let ft = r, Cr = r;
              if (typeof f == "string") {
                if (!$0.test(f)) {
                  let Dr = { instancePath: o + "/kernelspec", schemaPath: "#/oneOf/2/properties/kernelspec/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(display-name|displayName))", schema: Ka.oneOf[2].properties.kernelspec.propertyNames }, message: 'must match pattern "^(?!(display-name|displayName))"', schema: "^(?!(display-name|displayName))", parentSchema: Ka.oneOf[2].properties.kernelspec.propertyNames, data: f, propertyName: f };
                  e === null ? e = [Dr] : e.push(Dr), r++;
                }
              } else {
                let Dr = { instancePath: o + "/kernelspec", schemaPath: "#/oneOf/2/properties/kernelspec/propertyNames/type", keyword: "type", params: { type: "string", schema: Ka.oneOf[2].properties.kernelspec.propertyNames }, message: "must be string", schema: Ka.oneOf[2].properties.kernelspec.propertyNames.type, parentSchema: Ka.oneOf[2].properties.kernelspec.propertyNames, data: f, propertyName: f };
                e === null ? e = [Dr] : e.push(Dr), r++;
              }
              var $ = ft === r;
              if (!$) {
                let Dr = { instancePath: o + "/kernelspec", schemaPath: "#/oneOf/2/properties/kernelspec/propertyNames", keyword: "propertyNames", params: { propertyName: f, schema: Ka.oneOf[2].properties.kernelspec }, message: "property name must be valid", schema: Ka.oneOf[2].properties.kernelspec.propertyNames, parentSchema: Ka.oneOf[2].properties.kernelspec, data: P };
                e === null ? e = [Dr] : e.push(Dr), r++;
              }
            }
            if (P.display_name !== void 0) {
              let f = P.display_name, ft = r;
              if (typeof f != "string") {
                let Dr = { instancePath: o + "/kernelspec/display_name", schemaPath: "#/oneOf/2/properties/kernelspec/properties/display_name/type", keyword: "type", params: { type: "string", schema: Ka.oneOf[2].properties.kernelspec.properties.display_name }, message: "must be string", schema: Ka.oneOf[2].properties.kernelspec.properties.display_name.type, parentSchema: Ka.oneOf[2].properties.kernelspec.properties.display_name, data: f };
                e === null ? e = [Dr] : e.push(Dr), r++;
              }
              let Cr = r;
              var m = ft === r;
            }
            if (P.language !== void 0) {
              let f = P.language, ft = r;
              if (typeof f != "string") {
                let wr = { instancePath: o + "/kernelspec/language", schemaPath: "#/oneOf/2/properties/kernelspec/properties/language/type", keyword: "type", params: { type: "string", schema: Ka.oneOf[2].properties.kernelspec.properties.language }, message: "must be string", schema: Ka.oneOf[2].properties.kernelspec.properties.language.type, parentSchema: Ka.oneOf[2].properties.kernelspec.properties.language, data: f };
                e === null ? e = [wr] : e.push(wr), r++;
              }
              let Cr = r;
              var m = ft === r;
            }
            if (P.name !== void 0) {
              let f = P.name, ft = r;
              if (typeof f != "string") {
                let wr = { instancePath: o + "/kernelspec/name", schemaPath: "#/oneOf/2/properties/kernelspec/properties/name/type", keyword: "type", params: { type: "string", schema: Ka.oneOf[2].properties.kernelspec.properties.name }, message: "must be string", schema: Ka.oneOf[2].properties.kernelspec.properties.name.type, parentSchema: Ka.oneOf[2].properties.kernelspec.properties.name, data: f };
                e === null ? e = [wr] : e.push(wr), r++;
              }
              let Cr = r;
              var m = ft === r;
            }
          } else {
            let f = { instancePath: o + "/kernelspec", schemaPath: "#/oneOf/2/properties/kernelspec/type", keyword: "type", params: { type: "object", schema: Ka.oneOf[2].properties.kernelspec }, message: "must be object", schema: Ka.oneOf[2].properties.kernelspec.type, parentSchema: Ka.oneOf[2].properties.kernelspec, data: P };
            e === null ? e = [f] : e.push(f), r++;
          }
          var R = oe === r;
        }
      } else {
        let P = { instancePath: o, schemaPath: "#/oneOf/2/type", keyword: "type", params: { type: "object", schema: Ka.oneOf[2] }, message: "must be object", schema: Ka.oneOf[2].type, parentSchema: Ka.oneOf[2], data: l };
        e === null ? e = [P] : e.push(P), r++;
      }
      var g = j === r;
      g && c ? (c = false, d = [d, 2]) : g && (c = true, d = 2);
    }
    if (c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let j = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Ka }, message: "must match exactly one schema in oneOf", schema: Ka.oneOf, parentSchema: Ka, data: l };
      e === null ? e = [j] : e.push(j), r++;
    }
    return Oe.errors = e, r === 0;
  }
  B["quarto-resource-document-execute-knitr"] = Pe;
  var Kn = { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { opts_knit: { type: "object", description: "be an object", properties: {}, patternProperties: {} }, opts_chunk: { type: "object", description: "be an object", properties: {}, patternProperties: {} } }, patternProperties: {}, additionalProperties: false, propertyNames: { type: "string", pattern: "^(?!(opts-knit|optsKnit|opts-chunk|optsChunk))" } }], description: "be exactly one of: `true` or `false`, an object", $id: "quarto-resource-document-execute-knitr" }, S0 = new RegExp("^(?!(opts-knit|optsKnit|opts-chunk|optsChunk))", "u");
  function Pe(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "boolean") {
      let R = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: Kn.oneOf[0] }, message: "must be boolean", schema: Kn.oneOf[0].type, parentSchema: Kn.oneOf[0], data: l };
      e === null ? e = [R] : e.push(R), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let j in l) {
        let V = r, se = r;
        if (typeof j == "string") {
          if (!S0.test(j)) {
            let P = { instancePath: o, schemaPath: "#/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(opts-knit|optsKnit|opts-chunk|optsChunk))", schema: Kn.oneOf[1].propertyNames }, message: 'must match pattern "^(?!(opts-knit|optsKnit|opts-chunk|optsChunk))"', schema: "^(?!(opts-knit|optsKnit|opts-chunk|optsChunk))", parentSchema: Kn.oneOf[1].propertyNames, data: j, propertyName: j };
            e === null ? e = [P] : e.push(P), r++;
          }
        } else {
          let P = { instancePath: o, schemaPath: "#/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: Kn.oneOf[1].propertyNames }, message: "must be string", schema: Kn.oneOf[1].propertyNames.type, parentSchema: Kn.oneOf[1].propertyNames, data: j, propertyName: j };
          e === null ? e = [P] : e.push(P), r++;
        }
        var $ = V === r;
        if (!$) {
          let P = { instancePath: o, schemaPath: "#/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: j, schema: Kn.oneOf[1] }, message: "property name must be valid", schema: Kn.oneOf[1].propertyNames, parentSchema: Kn.oneOf[1], data: l };
          e === null ? e = [P] : e.push(P), r++;
        }
      }
      let R = r;
      for (let j in l)
        if (!(j === "opts_knit" || j === "opts_chunk")) {
          let V = { instancePath: o, schemaPath: "#/oneOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: j, schema: Kn.oneOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: Kn.oneOf[1], data: l };
          e === null ? e = [V] : e.push(V), r++;
        }
      if (l.opts_knit !== void 0) {
        let j = l.opts_knit, V = r, se = r;
        if (!(j && typeof j == "object" && !Array.isArray(j))) {
          let P = { instancePath: o + "/opts_knit", schemaPath: "#/oneOf/1/properties/opts_knit/type", keyword: "type", params: { type: "object", schema: Kn.oneOf[1].properties.opts_knit }, message: "must be object", schema: Kn.oneOf[1].properties.opts_knit.type, parentSchema: Kn.oneOf[1].properties.opts_knit, data: j };
          e === null ? e = [P] : e.push(P), r++;
        }
        var m = V === r;
      }
      if (l.opts_chunk !== void 0) {
        let j = l.opts_chunk, V = r, se = r;
        if (!(j && typeof j == "object" && !Array.isArray(j))) {
          let oe = { instancePath: o + "/opts_chunk", schemaPath: "#/oneOf/1/properties/opts_chunk/type", keyword: "type", params: { type: "object", schema: Kn.oneOf[1].properties.opts_chunk }, message: "must be object", schema: Kn.oneOf[1].properties.opts_chunk.type, parentSchema: Kn.oneOf[1].properties.opts_chunk, data: j };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        var m = V === r;
      }
    } else {
      let R = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "object", schema: Kn.oneOf[1] }, message: "must be object", schema: Kn.oneOf[1].type, parentSchema: Kn.oneOf[1], data: l };
      e === null ? e = [R] : e.push(R), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let R = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Kn }, message: "must match exactly one schema in oneOf", schema: Kn.oneOf, parentSchema: Kn, data: l };
      e === null ? e = [R] : e.push(R), r++;
    }
    return Pe.errors = e, r === 0;
  }
  B["quarto-resource-document-execute-cache"] = xe;
  var Zl = { anyOf: [{ type: "boolean", description: "be `true` or `false`" }, { enum: ["refresh"], description: "be 'refresh'" }], description: "be at least one of: `true` or `false`, 'refresh'", $id: "quarto-resource-document-execute-cache" };
  function xe(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = r;
    if (typeof l != "boolean") {
      let g = { instancePath: o, schemaPath: "#/anyOf/0/type", keyword: "type", params: { type: "boolean", schema: Zl.anyOf[0] }, message: "must be boolean", schema: Zl.anyOf[0].type, parentSchema: Zl.anyOf[0], data: l };
      e === null ? e = [g] : e.push(g), r++;
    }
    let u = r;
    var y = d === r;
    if (c = c || y, !c) {
      let g = r, x = Zl.anyOf[1].enum;
      if (l !== "refresh") {
        let $ = { instancePath: o, schemaPath: "#/anyOf/1/enum", keyword: "enum", params: { allowedValues: Zl.anyOf[1].enum, schema: Zl.anyOf[1] }, message: "must be equal to one of the allowed values", schema: Zl.anyOf[1].enum, parentSchema: Zl.anyOf[1], data: l };
        e === null ? e = [$] : e.push($), r++;
      }
      var y = g === r;
      c = c || y;
    }
    if (c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let g = { instancePath: o, schemaPath: "#/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: Zl.anyOf, parentSchema: Zl, data: l };
      e === null ? e = [g] : e.push(g), r++;
    }
    return xe.errors = e, r === 0;
  }
  B["quarto-resource-document-execute-freeze"] = ke;
  var ec = { anyOf: [{ type: "boolean", description: "be `true` or `false`" }, { enum: ["auto"], description: "be 'auto'" }], description: "be at least one of: `true` or `false`, 'auto'", $id: "quarto-resource-document-execute-freeze" };
  function ke(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = r;
    if (typeof l != "boolean") {
      let g = { instancePath: o, schemaPath: "#/anyOf/0/type", keyword: "type", params: { type: "boolean", schema: ec.anyOf[0] }, message: "must be boolean", schema: ec.anyOf[0].type, parentSchema: ec.anyOf[0], data: l };
      e === null ? e = [g] : e.push(g), r++;
    }
    let u = r;
    var y = d === r;
    if (c = c || y, !c) {
      let g = r, x = ec.anyOf[1].enum;
      if (l !== "auto") {
        let $ = { instancePath: o, schemaPath: "#/anyOf/1/enum", keyword: "enum", params: { allowedValues: ec.anyOf[1].enum, schema: ec.anyOf[1] }, message: "must be equal to one of the allowed values", schema: ec.anyOf[1].enum, parentSchema: ec.anyOf[1], data: l };
        e === null ? e = [$] : e.push($), r++;
      }
      var y = g === r;
      c = c || y;
    }
    if (c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let g = { instancePath: o, schemaPath: "#/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: ec.anyOf, parentSchema: ec, data: l };
      e === null ? e = [g] : e.push(g), r++;
    }
    return ke.errors = e, r === 0;
  }
  B["quarto-resource-document-execute-server"] = De;
  var $a = { oneOf: [{ enum: ["shiny"], description: "be 'shiny'" }, { type: "object", description: "be an object", properties: { type: { enum: ["shiny"], description: "be 'shiny'" }, "ojs-export": { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, "ojs-import": { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(ojs_export|ojsExport|ojs_import|ojsImport))" } }], description: "be exactly one of: 'shiny', an object", $id: "quarto-resource-document-execute-server" }, F0 = new RegExp("^(?!(ojs_export|ojsExport|ojs_import|ojsImport))", "u");
  function De(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r, y = $a.oneOf[0].enum;
    if (l !== "shiny") {
      let P = { instancePath: o, schemaPath: "#/oneOf/0/enum", keyword: "enum", params: { allowedValues: $a.oneOf[0].enum, schema: $a.oneOf[0] }, message: "must be equal to one of the allowed values", schema: $a.oneOf[0].enum, parentSchema: $a.oneOf[0], data: l };
      e === null ? e = [P] : e.push(P), r++;
    }
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let P in l) {
        let oe = r, h = r;
        if (typeof P == "string") {
          if (!F0.test(P)) {
            let f = { instancePath: o, schemaPath: "#/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(ojs_export|ojsExport|ojs_import|ojsImport))", schema: $a.oneOf[1].propertyNames }, message: 'must match pattern "^(?!(ojs_export|ojsExport|ojs_import|ojsImport))"', schema: "^(?!(ojs_export|ojsExport|ojs_import|ojsImport))", parentSchema: $a.oneOf[1].propertyNames, data: P, propertyName: P };
            e === null ? e = [f] : e.push(f), r++;
          }
        } else {
          let f = { instancePath: o, schemaPath: "#/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: $a.oneOf[1].propertyNames }, message: "must be string", schema: $a.oneOf[1].propertyNames.type, parentSchema: $a.oneOf[1].propertyNames, data: P, propertyName: P };
          e === null ? e = [f] : e.push(f), r++;
        }
        var $ = oe === r;
        if (!$) {
          let f = { instancePath: o, schemaPath: "#/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: P, schema: $a.oneOf[1] }, message: "property name must be valid", schema: $a.oneOf[1].propertyNames, parentSchema: $a.oneOf[1], data: l };
          e === null ? e = [f] : e.push(f), r++;
        }
      }
      if (l.type !== void 0) {
        let P = l.type, oe = r, h = $a.oneOf[1].properties.type.enum;
        if (P !== "shiny") {
          let f = { instancePath: o + "/type", schemaPath: "#/oneOf/1/properties/type/enum", keyword: "enum", params: { allowedValues: $a.oneOf[1].properties.type.enum, schema: $a.oneOf[1].properties.type }, message: "must be equal to one of the allowed values", schema: $a.oneOf[1].properties.type.enum, parentSchema: $a.oneOf[1].properties.type, data: P };
          e === null ? e = [f] : e.push(f), r++;
        }
        var m = oe === r;
      }
      if (l["ojs-export"] !== void 0) {
        let P = l["ojs-export"], oe = r, h = r, f = false, ft = null, Cr = r;
        if (typeof P != "string") {
          let ee = { instancePath: o + "/ojs-export", schemaPath: "#/oneOf/1/properties/ojs-export/oneOf/0/type", keyword: "type", params: { type: "string", schema: $a.oneOf[1].properties["ojs-export"].oneOf[0] }, message: "must be string", schema: $a.oneOf[1].properties["ojs-export"].oneOf[0].type, parentSchema: $a.oneOf[1].properties["ojs-export"].oneOf[0], data: P };
          e === null ? e = [ee] : e.push(ee), r++;
        }
        let Dr = r;
        var R = Cr === r;
        R && (f = true, ft = 0);
        let wr = r, ta = r;
        if (Array.isArray(P)) {
          let ee = true, za = P.length;
          for (let sa = 0; sa < za; sa++) {
            let _e = P[sa], Ma = r;
            if (typeof _e != "string") {
              let ie = { instancePath: o + "/ojs-export/" + sa, schemaPath: "#/oneOf/1/properties/ojs-export/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: $a.oneOf[1].properties["ojs-export"].oneOf[1].items }, message: "must be string", schema: $a.oneOf[1].properties["ojs-export"].oneOf[1].items.type, parentSchema: $a.oneOf[1].properties["ojs-export"].oneOf[1].items, data: _e };
              e === null ? e = [ie] : e.push(ie), r++;
            }
            let fa = r;
            var j = Ma === r;
            j || (ee = false);
          }
        } else {
          let ee = { instancePath: o + "/ojs-export", schemaPath: "#/oneOf/1/properties/ojs-export/oneOf/1/type", keyword: "type", params: { type: "array", schema: $a.oneOf[1].properties["ojs-export"].oneOf[1] }, message: "must be array", schema: $a.oneOf[1].properties["ojs-export"].oneOf[1].type, parentSchema: $a.oneOf[1].properties["ojs-export"].oneOf[1], data: P };
          e === null ? e = [ee] : e.push(ee), r++;
        }
        var R = wr === r;
        if (R && f ? (f = false, ft = [ft, 1]) : R && (f = true, ft = 1), f)
          r = h, e !== null && (h ? e.length = h : e = null);
        else {
          let ee = { instancePath: o + "/ojs-export", schemaPath: "#/oneOf/1/properties/ojs-export/oneOf", keyword: "oneOf", params: { passingSchemas: ft, schema: $a.oneOf[1].properties["ojs-export"] }, message: "must match exactly one schema in oneOf", schema: $a.oneOf[1].properties["ojs-export"].oneOf, parentSchema: $a.oneOf[1].properties["ojs-export"], data: P };
          e === null ? e = [ee] : e.push(ee), r++;
        }
        var m = oe === r;
      }
      if (l["ojs-import"] !== void 0) {
        let P = l["ojs-import"], oe = r, h = r, f = false, ft = null, Cr = r;
        if (typeof P != "string") {
          let ee = { instancePath: o + "/ojs-import", schemaPath: "#/oneOf/1/properties/ojs-import/oneOf/0/type", keyword: "type", params: { type: "string", schema: $a.oneOf[1].properties["ojs-import"].oneOf[0] }, message: "must be string", schema: $a.oneOf[1].properties["ojs-import"].oneOf[0].type, parentSchema: $a.oneOf[1].properties["ojs-import"].oneOf[0], data: P };
          e === null ? e = [ee] : e.push(ee), r++;
        }
        let Dr = r;
        var V = Cr === r;
        V && (f = true, ft = 0);
        let wr = r, ta = r;
        if (Array.isArray(P)) {
          let ee = true, za = P.length;
          for (let sa = 0; sa < za; sa++) {
            let _e = P[sa], Ma = r;
            if (typeof _e != "string") {
              let ie = { instancePath: o + "/ojs-import/" + sa, schemaPath: "#/oneOf/1/properties/ojs-import/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: $a.oneOf[1].properties["ojs-import"].oneOf[1].items }, message: "must be string", schema: $a.oneOf[1].properties["ojs-import"].oneOf[1].items.type, parentSchema: $a.oneOf[1].properties["ojs-import"].oneOf[1].items, data: _e };
              e === null ? e = [ie] : e.push(ie), r++;
            }
            let fa = r;
            var se = Ma === r;
            se || (ee = false);
          }
        } else {
          let ee = { instancePath: o + "/ojs-import", schemaPath: "#/oneOf/1/properties/ojs-import/oneOf/1/type", keyword: "type", params: { type: "array", schema: $a.oneOf[1].properties["ojs-import"].oneOf[1] }, message: "must be array", schema: $a.oneOf[1].properties["ojs-import"].oneOf[1].type, parentSchema: $a.oneOf[1].properties["ojs-import"].oneOf[1], data: P };
          e === null ? e = [ee] : e.push(ee), r++;
        }
        var V = wr === r;
        if (V && f ? (f = false, ft = [ft, 1]) : V && (f = true, ft = 1), f)
          r = h, e !== null && (h ? e.length = h : e = null);
        else {
          let ee = { instancePath: o + "/ojs-import", schemaPath: "#/oneOf/1/properties/ojs-import/oneOf", keyword: "oneOf", params: { passingSchemas: ft, schema: $a.oneOf[1].properties["ojs-import"] }, message: "must match exactly one schema in oneOf", schema: $a.oneOf[1].properties["ojs-import"].oneOf, parentSchema: $a.oneOf[1].properties["ojs-import"], data: P };
          e === null ? e = [ee] : e.push(ee), r++;
        }
        var m = oe === r;
      }
    } else {
      let P = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "object", schema: $a.oneOf[1] }, message: "must be object", schema: $a.oneOf[1].type, parentSchema: $a.oneOf[1], data: l };
      e === null ? e = [P] : e.push(P), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let P = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: $a }, message: "must match exactly one schema in oneOf", schema: $a.oneOf, parentSchema: $a, data: l };
      e === null ? e = [P] : e.push(P), r++;
    }
    return De.errors = e, r === 0;
  }
  B["quarto-resource-document-execute-daemon"] = we;
  var ic = { oneOf: [{ type: "number", description: "be a number" }, { type: "boolean", description: "be `true` or `false`" }], description: "be exactly one of: a number, `true` or `false`", $id: "quarto-resource-document-execute-daemon" };
  function we(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (!(typeof l == "number" && isFinite(l))) {
      let $ = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "number", schema: ic.oneOf[0] }, message: "must be number", schema: ic.oneOf[0].type, parentSchema: ic.oneOf[0], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r;
    if (typeof l != "boolean") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "boolean", schema: ic.oneOf[1] }, message: "must be boolean", schema: ic.oneOf[1].type, parentSchema: ic.oneOf[1], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let D = r;
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: ic }, message: "must match exactly one schema in oneOf", schema: ic.oneOf, parentSchema: ic, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return we.errors = e, r === 0;
  }
  B["quarto-resource-document-execute-daemon-restart"] = $e;
  var jh = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-execute-daemon-restart" };
  function $e(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: jh }, message: "must be boolean", schema: jh.type, parentSchema: jh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return $e.errors = e, r === 0;
  }
  B["quarto-resource-document-execute-enabled"] = Se;
  var Eh = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-execute-enabled" };
  function Se(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Eh }, message: "must be boolean", schema: Eh.type, parentSchema: Eh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Se.errors = e, r === 0;
  }
  B["quarto-resource-document-execute-ipynb"] = Fe;
  var Nh = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-execute-ipynb" };
  function Fe(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Nh }, message: "must be boolean", schema: Nh.type, parentSchema: Nh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Fe.errors = e, r === 0;
  }
  B["quarto-resource-document-execute-debug"] = Ae;
  var Ch = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-execute-debug" };
  function Ae(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Ch }, message: "must be boolean", schema: Ch.type, parentSchema: Ch, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ae.errors = e, r === 0;
  }
  B["quarto-resource-document-figures-fig-width"] = He;
  var Ih = { type: "number", description: "be a number", $id: "quarto-resource-document-figures-fig-width" };
  function He(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: Ih }, message: "must be number", schema: Ih.type, parentSchema: Ih, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return He.errors = e, r === 0;
  }
  B["quarto-resource-document-figures-fig-height"] = Le;
  var Hh = { type: "number", description: "be a number", $id: "quarto-resource-document-figures-fig-height" };
  function Le(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: Hh }, message: "must be number", schema: Hh.type, parentSchema: Hh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Le.errors = e, r === 0;
  }
  B["quarto-resource-document-figures-fig-format"] = Me;
  var Rf = { enum: ["retina", "png", "jpeg", "svg", "pdf"], description: "be one of: `retina`, `png`, `jpeg`, `svg`, `pdf`", $id: "quarto-resource-document-figures-fig-format" };
  function Me(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Rf.enum;
    if (!(l === "retina" || l === "png" || l === "jpeg" || l === "svg" || l === "pdf")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Rf.enum, schema: Rf }, message: "must be equal to one of the allowed values", schema: Rf.enum, parentSchema: Rf, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Me.errors = e, r === 0;
  }
  B["quarto-resource-document-figures-fig-dpi"] = Te;
  var Lh = { type: "number", description: "be a number", $id: "quarto-resource-document-figures-fig-dpi" };
  function Te(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: Lh }, message: "must be number", schema: Lh.type, parentSchema: Lh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Te.errors = e, r === 0;
  }
  B["quarto-resource-document-figures-fig-responsive"] = cn;
  var Mh = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-figures-fig-responsive" };
  function cn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Mh }, message: "must be boolean", schema: Mh.type, parentSchema: Mh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return cn.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-mainfont"] = Ja;
  var Th = { type: "string", description: "be a string", $id: "quarto-resource-document-fonts-mainfont" };
  function Ja(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Th }, message: "must be string", schema: Th.type, parentSchema: Th, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ja.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-monofont"] = Qa;
  var zh = { type: "string", description: "be a string", $id: "quarto-resource-document-fonts-monofont" };
  function Qa(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: zh }, message: "must be string", schema: zh.type, parentSchema: zh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Qa.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-fontsize"] = Ga;
  var Wh = { type: "string", description: "be a string", $id: "quarto-resource-document-fonts-fontsize" };
  function Ga(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Wh }, message: "must be string", schema: Wh.type, parentSchema: Wh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ga.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-fontenc"] = _s;
  var Rh = { type: "string", description: "be a string", $id: "quarto-resource-document-fonts-fontenc" };
  function _s(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Rh }, message: "must be string", schema: Rh.type, parentSchema: Rh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return _s.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-fontfamily"] = Bo;
  var Vh = { type: "string", description: "be a string", $id: "quarto-resource-document-fonts-fontfamily" };
  function Bo(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Vh }, message: "must be string", schema: Vh.type, parentSchema: Vh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Bo.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-fontfamilyoptions"] = vs;
  var Si = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-fonts-fontfamilyoptions" };
  function vs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Si.oneOf[0] }, message: "must be string", schema: Si.oneOf[0].type, parentSchema: Si.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Si.oneOf[1].items }, message: "must be string", schema: Si.oneOf[1].items.type, parentSchema: Si.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Si.oneOf[1] }, message: "must be array", schema: Si.oneOf[1].type, parentSchema: Si.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Si }, message: "must match exactly one schema in oneOf", schema: Si.oneOf, parentSchema: Si, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return vs.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-sansfont"] = qs;
  var Uh = { type: "string", description: "be a string", $id: "quarto-resource-document-fonts-sansfont" };
  function qs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Uh }, message: "must be string", schema: Uh.type, parentSchema: Uh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return qs.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-mathfont"] = Os;
  var Kh = { type: "string", description: "be a string", $id: "quarto-resource-document-fonts-mathfont" };
  function Os(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Kh }, message: "must be string", schema: Kh.type, parentSchema: Kh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Os.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-CJKmainfont"] = Ps;
  var Jh = { type: "string", description: "be a string", $id: "quarto-resource-document-fonts-CJKmainfont" };
  function Ps(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Jh }, message: "must be string", schema: Jh.type, parentSchema: Jh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ps.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-mainfontoptions"] = xs;
  var Fi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-fonts-mainfontoptions" };
  function xs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Fi.oneOf[0] }, message: "must be string", schema: Fi.oneOf[0].type, parentSchema: Fi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Fi.oneOf[1].items }, message: "must be string", schema: Fi.oneOf[1].items.type, parentSchema: Fi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Fi.oneOf[1] }, message: "must be array", schema: Fi.oneOf[1].type, parentSchema: Fi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Fi }, message: "must match exactly one schema in oneOf", schema: Fi.oneOf, parentSchema: Fi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return xs.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-sansfontoptions"] = ks;
  var Ai = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-fonts-sansfontoptions" };
  function ks(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Ai.oneOf[0] }, message: "must be string", schema: Ai.oneOf[0].type, parentSchema: Ai.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Ai.oneOf[1].items }, message: "must be string", schema: Ai.oneOf[1].items.type, parentSchema: Ai.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Ai.oneOf[1] }, message: "must be array", schema: Ai.oneOf[1].type, parentSchema: Ai.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Ai }, message: "must match exactly one schema in oneOf", schema: Ai.oneOf, parentSchema: Ai, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return ks.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-monofontoptions"] = Ds;
  var Bi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-fonts-monofontoptions" };
  function Ds(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Bi.oneOf[0] }, message: "must be string", schema: Bi.oneOf[0].type, parentSchema: Bi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Bi.oneOf[1].items }, message: "must be string", schema: Bi.oneOf[1].items.type, parentSchema: Bi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Bi.oneOf[1] }, message: "must be array", schema: Bi.oneOf[1].type, parentSchema: Bi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Bi }, message: "must match exactly one schema in oneOf", schema: Bi.oneOf, parentSchema: Bi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Ds.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-mathfontoptions"] = ws;
  var ji = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-fonts-mathfontoptions" };
  function ws(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: ji.oneOf[0] }, message: "must be string", schema: ji.oneOf[0].type, parentSchema: ji.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: ji.oneOf[1].items }, message: "must be string", schema: ji.oneOf[1].items.type, parentSchema: ji.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: ji.oneOf[1] }, message: "must be array", schema: ji.oneOf[1].type, parentSchema: ji.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: ji }, message: "must match exactly one schema in oneOf", schema: ji.oneOf, parentSchema: ji, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return ws.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-CJKoptions"] = $s;
  var Ei = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-fonts-CJKoptions" };
  function $s(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Ei.oneOf[0] }, message: "must be string", schema: Ei.oneOf[0].type, parentSchema: Ei.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Ei.oneOf[1].items }, message: "must be string", schema: Ei.oneOf[1].items.type, parentSchema: Ei.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Ei.oneOf[1] }, message: "must be array", schema: Ei.oneOf[1].type, parentSchema: Ei.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Ei }, message: "must match exactly one schema in oneOf", schema: Ei.oneOf, parentSchema: Ei, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return $s.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-microtypeoptions"] = Ss;
  var Ni = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-fonts-microtypeoptions" };
  function Ss(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Ni.oneOf[0] }, message: "must be string", schema: Ni.oneOf[0].type, parentSchema: Ni.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Ni.oneOf[1].items }, message: "must be string", schema: Ni.oneOf[1].items.type, parentSchema: Ni.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Ni.oneOf[1] }, message: "must be array", schema: Ni.oneOf[1].type, parentSchema: Ni.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Ni }, message: "must match exactly one schema in oneOf", schema: Ni.oneOf, parentSchema: Ni, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Ss.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-pointsize"] = qc;
  var Qh = { type: "string", description: "be a string", $id: "quarto-resource-document-fonts-pointsize" };
  function qc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Qh }, message: "must be string", schema: Qh.type, parentSchema: Qh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return qc.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-lineheight"] = Oc;
  var Gh = { type: "string", description: "be a string", $id: "quarto-resource-document-fonts-lineheight" };
  function Oc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Gh }, message: "must be string", schema: Gh.type, parentSchema: Gh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Oc.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-linestretch"] = Ya;
  var Yh = { type: "string", description: "be a string", $id: "quarto-resource-document-fonts-linestretch" };
  function Ya(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Yh }, message: "must be string", schema: Yh.type, parentSchema: Yh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ya.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-interlinespace"] = Pc;
  var Ci = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-fonts-interlinespace" };
  function Pc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Ci.oneOf[0] }, message: "must be string", schema: Ci.oneOf[0].type, parentSchema: Ci.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Ci.oneOf[1].items }, message: "must be string", schema: Ci.oneOf[1].items.type, parentSchema: Ci.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Ci.oneOf[1] }, message: "must be array", schema: Ci.oneOf[1].type, parentSchema: Ci.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Ci }, message: "must match exactly one schema in oneOf", schema: Ci.oneOf, parentSchema: Ci, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Pc.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-linkstyle"] = xc;
  var Xh = { type: "string", description: "be a string", $id: "quarto-resource-document-fonts-linkstyle" };
  function xc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Xh }, message: "must be string", schema: Xh.type, parentSchema: Xh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return xc.errors = e, r === 0;
  }
  B["quarto-resource-document-fonts-whitespace"] = kc;
  var Zh = { type: "string", description: "be a string", $id: "quarto-resource-document-fonts-whitespace" };
  function kc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Zh }, message: "must be string", schema: Zh.type, parentSchema: Zh, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return kc.errors = e, r === 0;
  }
  B["quarto-resource-document-footnotes-footnotes-hover"] = wn;
  var ey = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-footnotes-footnotes-hover" };
  function wn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: ey }, message: "must be boolean", schema: ey.type, parentSchema: ey, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return wn.errors = e, r === 0;
  }
  B["quarto-resource-document-footnotes-links-as-notes"] = Fs;
  var ry = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-footnotes-links-as-notes" };
  function Fs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: ry }, message: "must be boolean", schema: ry.type, parentSchema: ry, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Fs.errors = e, r === 0;
  }
  B["quarto-resource-document-footnotes-reference-location"] = Ca;
  var Vf = { enum: ["block", "section", "margin", "document"], description: "be one of: `block`, `section`, `margin`, `document`", $id: "quarto-resource-document-footnotes-reference-location" };
  function Ca(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Vf.enum;
    if (!(l === "block" || l === "section" || l === "margin" || l === "document")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Vf.enum, schema: Vf }, message: "must be equal to one of the allowed values", schema: Vf.enum, parentSchema: Vf, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Ca.errors = e, r === 0;
  }
  B["quarto-resource-document-formatting-indenting"] = Dc;
  var Ii = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-formatting-indenting" };
  function Dc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Ii.oneOf[0] }, message: "must be string", schema: Ii.oneOf[0].type, parentSchema: Ii.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Ii.oneOf[1].items }, message: "must be string", schema: Ii.oneOf[1].items.type, parentSchema: Ii.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Ii.oneOf[1] }, message: "must be array", schema: Ii.oneOf[1].type, parentSchema: Ii.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Ii }, message: "must match exactly one schema in oneOf", schema: Ii.oneOf, parentSchema: Ii, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Dc.errors = e, r === 0;
  }
  B["quarto-resource-document-formatting-adjusting"] = wc;
  var Uf = { enum: ["l", "r", "c", "b"], description: "be one of: `l`, `r`, `c`, `b`", $id: "quarto-resource-document-formatting-adjusting" };
  function wc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Uf.enum;
    if (!(l === "l" || l === "r" || l === "c" || l === "b")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Uf.enum, schema: Uf }, message: "must be equal to one of the allowed values", schema: Uf.enum, parentSchema: Uf, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return wc.errors = e, r === 0;
  }
  B["quarto-resource-document-formatting-hyphenate"] = $c;
  var ty = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-formatting-hyphenate" };
  function $c(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: ty }, message: "must be boolean", schema: ty.type, parentSchema: ty, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return $c.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-to"] = I;
  var ay = { type: "string", description: "be a string", $id: "quarto-resource-document-hidden-to" };
  function I(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: ay }, message: "must be string", schema: ay.type, parentSchema: ay, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return I.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-writer"] = S1;
  var ny = { type: "string", description: "be a string", $id: "quarto-resource-document-hidden-writer" };
  function S1(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: ny }, message: "must be string", schema: ny.type, parentSchema: ny, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return S1.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-input-file"] = ze;
  var oy = { type: "string", description: "be a string", $id: "quarto-resource-document-hidden-input-file" };
  function ze(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: oy }, message: "must be string", schema: oy.type, parentSchema: oy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ze.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-input-files"] = We;
  var wd = { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" }, $id: "quarto-resource-document-hidden-input-files" };
  function We(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let d = true, u = l.length;
      for (let y = 0; y < u; y++) {
        let g = l[y], x = r;
        if (typeof g != "string") {
          let $ = { instancePath: o + "/" + y, schemaPath: "#/items/type", keyword: "type", params: { type: "string", schema: wd.items }, message: "must be string", schema: wd.items.type, parentSchema: wd.items, data: g };
          e === null ? e = [$] : e.push($), r++;
        }
        let D = r;
        var c = x === r;
        c || (d = false);
      }
    } else {
      let d = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: wd }, message: "must be array", schema: wd.type, parentSchema: wd, data: l };
      e === null ? e = [d] : e.push(d), r++;
    }
    return We.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-defaults"] = Re;
  var $d = { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" }, $id: "quarto-resource-document-hidden-defaults" };
  function Re(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let d = true, u = l.length;
      for (let y = 0; y < u; y++) {
        let g = l[y], x = r;
        if (typeof g != "string") {
          let $ = { instancePath: o + "/" + y, schemaPath: "#/items/type", keyword: "type", params: { type: "string", schema: $d.items }, message: "must be string", schema: $d.items.type, parentSchema: $d.items, data: g };
          e === null ? e = [$] : e.push($), r++;
        }
        let D = r;
        var c = x === r;
        c || (d = false);
      }
    } else {
      let d = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: $d }, message: "must be array", schema: $d.type, parentSchema: $d, data: l };
      e === null ? e = [d] : e.push(d), r++;
    }
    return Re.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-variables"] = Ve;
  var sy = { type: "object", description: "be an object", properties: {}, patternProperties: {}, $id: "quarto-resource-document-hidden-variables" };
  function Ve(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (!(l && typeof l == "object" && !Array.isArray(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: sy }, message: "must be object", schema: sy.type, parentSchema: sy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Ve.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-metadata"] = Ue;
  var iy = { type: "object", description: "be an object", properties: {}, patternProperties: {}, $id: "quarto-resource-document-hidden-metadata" };
  function Ue(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (!(l && typeof l == "object" && !Array.isArray(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: iy }, message: "must be object", schema: iy.type, parentSchema: iy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Ue.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-request-headers"] = Ke;
  function Ke(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    return Cd(l, { instancePath: o, parentData: S, parentDataProperty: F, rootData: s }) || (e = e === null ? Cd.errors : e.concat(Cd.errors), r = e.length), Ke.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-trace"] = Je;
  var ly = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-hidden-trace" };
  function Je(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: ly }, message: "must be boolean", schema: ly.type, parentSchema: ly, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Je.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-fail-if-warnings"] = Qe;
  var cy = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-hidden-fail-if-warnings" };
  function Qe(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: cy }, message: "must be boolean", schema: cy.type, parentSchema: cy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Qe.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-dump-args"] = Ge;
  var py = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-hidden-dump-args" };
  function Ge(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: py }, message: "must be boolean", schema: py.type, parentSchema: py, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ge.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-ignore-args"] = Ye;
  var uy = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-hidden-ignore-args" };
  function Ye(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: uy }, message: "must be boolean", schema: uy.type, parentSchema: uy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ye.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-file-scope"] = Xe;
  var dy = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-hidden-file-scope" };
  function Xe(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: dy }, message: "must be boolean", schema: dy.type, parentSchema: dy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Xe.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-data-dir"] = Ze;
  var fy = { type: "string", description: "be a string", $id: "quarto-resource-document-hidden-data-dir" };
  function Ze(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: fy }, message: "must be string", schema: fy.type, parentSchema: fy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ze.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-verbosity"] = er;
  var Kf = { enum: ["ERROR", "WARNING", "INFO"], description: "be one of: `ERROR`, `WARNING`, `INFO`", $id: "quarto-resource-document-hidden-verbosity" };
  function er(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Kf.enum;
    if (!(l === "ERROR" || l === "WARNING" || l === "INFO")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Kf.enum, schema: Kf }, message: "must be equal to one of the allowed values", schema: Kf.enum, parentSchema: Kf, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return er.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-log-file"] = rr;
  var my = { type: "string", description: "be a string", $id: "quarto-resource-document-hidden-log-file" };
  function rr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: my }, message: "must be string", schema: my.type, parentSchema: my, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return rr.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-track-changes"] = Sc;
  var Jf = { enum: ["accept", "reject", "all"], description: "be one of: `accept`, `reject`, `all`", $id: "quarto-resource-document-hidden-track-changes" };
  function Sc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Jf.enum;
    if (!(l === "accept" || l === "reject" || l === "all")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Jf.enum, schema: Jf }, message: "must be equal to one of the allowed values", schema: Jf.enum, parentSchema: Jf, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Sc.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-strip-empty-paragraphs"] = tr;
  var gy = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-hidden-strip-empty-paragraphs" };
  function tr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: gy }, message: "must be boolean", schema: gy.type, parentSchema: gy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return tr.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-keep-source"] = As;
  var hy = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-hidden-keep-source" };
  function As(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: hy }, message: "must be boolean", schema: hy.type, parentSchema: hy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return As.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-keep-hidden"] = Bs;
  var yy = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-hidden-keep-hidden" };
  function Bs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: yy }, message: "must be boolean", schema: yy.type, parentSchema: yy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Bs.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-prefer-html"] = jo;
  var by = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-hidden-prefer-html" };
  function jo(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: by }, message: "must be boolean", schema: by.type, parentSchema: by, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return jo.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-output-divs"] = ar;
  var _y = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-hidden-output-divs" };
  function ar(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: _y }, message: "must be boolean", schema: _y.type, parentSchema: _y, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ar.errors = e, r === 0;
  }
  B["quarto-resource-document-hidden-merge-includes"] = nr;
  var vy = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-hidden-merge-includes" };
  function nr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: vy }, message: "must be boolean", schema: vy.type, parentSchema: vy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return nr.errors = e, r === 0;
  }
  B["quarto-resource-document-includes-header-includes"] = Ir;
  var qy = { type: "string", description: "be a string", $id: "quarto-resource-document-includes-header-includes" };
  function Ir(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: qy }, message: "must be string", schema: qy.type, parentSchema: qy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ir.errors = e, r === 0;
  }
  B["quarto-resource-document-includes-include-before"] = Hr;
  var Oy = { type: "string", description: "be a string", $id: "quarto-resource-document-includes-include-before" };
  function Hr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Oy }, message: "must be string", schema: Oy.type, parentSchema: Oy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Hr.errors = e, r === 0;
  }
  B["quarto-resource-document-includes-include-after"] = Lr;
  var Py = { type: "string", description: "be a string", $id: "quarto-resource-document-includes-include-after" };
  function Lr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Py }, message: "must be string", schema: Py.type, parentSchema: Py, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Lr.errors = e, r === 0;
  }
  B["quarto-resource-document-includes-include-before-body"] = Mr;
  var Hi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-includes-include-before-body" };
  function Mr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Hi.oneOf[0] }, message: "must be string", schema: Hi.oneOf[0].type, parentSchema: Hi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Hi.oneOf[1].items }, message: "must be string", schema: Hi.oneOf[1].items.type, parentSchema: Hi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Hi.oneOf[1] }, message: "must be array", schema: Hi.oneOf[1].type, parentSchema: Hi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Hi }, message: "must match exactly one schema in oneOf", schema: Hi.oneOf, parentSchema: Hi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Mr.errors = e, r === 0;
  }
  B["quarto-resource-document-includes-include-after-body"] = Tr;
  var Li = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-includes-include-after-body" };
  function Tr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Li.oneOf[0] }, message: "must be string", schema: Li.oneOf[0].type, parentSchema: Li.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Li.oneOf[1].items }, message: "must be string", schema: Li.oneOf[1].items.type, parentSchema: Li.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Li.oneOf[1] }, message: "must be array", schema: Li.oneOf[1].type, parentSchema: Li.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Li }, message: "must match exactly one schema in oneOf", schema: Li.oneOf, parentSchema: Li, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Tr.errors = e, r === 0;
  }
  B["quarto-resource-document-includes-include-in-header"] = zr;
  var Mi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-includes-include-in-header" };
  function zr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Mi.oneOf[0] }, message: "must be string", schema: Mi.oneOf[0].type, parentSchema: Mi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Mi.oneOf[1].items }, message: "must be string", schema: Mi.oneOf[1].items.type, parentSchema: Mi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Mi.oneOf[1] }, message: "must be array", schema: Mi.oneOf[1].type, parentSchema: Mi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Mi }, message: "must match exactly one schema in oneOf", schema: Mi.oneOf, parentSchema: Mi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return zr.errors = e, r === 0;
  }
  B["quarto-resource-document-includes-resources"] = pn;
  var Ti = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-includes-resources" };
  function pn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Ti.oneOf[0] }, message: "must be string", schema: Ti.oneOf[0].type, parentSchema: Ti.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Ti.oneOf[1].items }, message: "must be string", schema: Ti.oneOf[1].items.type, parentSchema: Ti.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Ti.oneOf[1] }, message: "must be array", schema: Ti.oneOf[1].type, parentSchema: Ti.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Ti }, message: "must match exactly one schema in oneOf", schema: Ti.oneOf, parentSchema: Ti, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return pn.errors = e, r === 0;
  }
  B["quarto-resource-document-includes-headertext"] = Fc;
  var zi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-includes-headertext" };
  function Fc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: zi.oneOf[0] }, message: "must be string", schema: zi.oneOf[0].type, parentSchema: zi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: zi.oneOf[1].items }, message: "must be string", schema: zi.oneOf[1].items.type, parentSchema: zi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: zi.oneOf[1] }, message: "must be array", schema: zi.oneOf[1].type, parentSchema: zi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: zi }, message: "must match exactly one schema in oneOf", schema: zi.oneOf, parentSchema: zi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Fc.errors = e, r === 0;
  }
  B["quarto-resource-document-includes-footertext"] = Ac;
  var Wi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-includes-footertext" };
  function Ac(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Wi.oneOf[0] }, message: "must be string", schema: Wi.oneOf[0].type, parentSchema: Wi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Wi.oneOf[1].items }, message: "must be string", schema: Wi.oneOf[1].items.type, parentSchema: Wi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Wi.oneOf[1] }, message: "must be array", schema: Wi.oneOf[1].type, parentSchema: Wi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Wi }, message: "must match exactly one schema in oneOf", schema: Wi.oneOf, parentSchema: Wi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Ac.errors = e, r === 0;
  }
  B["quarto-resource-document-includes-includesource"] = Bc;
  var xy = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-includes-includesource" };
  function Bc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: xy }, message: "must be boolean", schema: xy.type, parentSchema: xy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Bc.errors = e, r === 0;
  }
  B["quarto-resource-document-includes-footer"] = Qf;
  var ky = { type: "string", description: "be a string", $id: "quarto-resource-document-includes-footer" };
  function Qf(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: ky }, message: "must be string", schema: ky.type, parentSchema: ky, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Qf.errors = e, r === 0;
  }
  B["quarto-resource-document-includes-header"] = jc;
  var Dy = { type: "string", description: "be a string", $id: "quarto-resource-document-includes-header" };
  function jc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Dy }, message: "must be string", schema: Dy.type, parentSchema: Dy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return jc.errors = e, r === 0;
  }
  B["quarto-resource-document-includes-metadata-file"] = or;
  var wy = { type: "string", description: "be a string", $id: "quarto-resource-document-includes-metadata-file" };
  function or(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: wy }, message: "must be string", schema: wy.type, parentSchema: wy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return or.errors = e, r === 0;
  }
  B["quarto-resource-document-includes-metadata-files"] = sr;
  var Sd = { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" }, $id: "quarto-resource-document-includes-metadata-files" };
  function sr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let d = true, u = l.length;
      for (let y = 0; y < u; y++) {
        let g = l[y], x = r;
        if (typeof g != "string") {
          let $ = { instancePath: o + "/" + y, schemaPath: "#/items/type", keyword: "type", params: { type: "string", schema: Sd.items }, message: "must be string", schema: Sd.items.type, parentSchema: Sd.items, data: g };
          e === null ? e = [$] : e.push($), r++;
        }
        let D = r;
        var c = x === r;
        c || (d = false);
      }
    } else {
      let d = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: Sd }, message: "must be array", schema: Sd.type, parentSchema: Sd, data: l };
      e === null ? e = [d] : e.push(d), r++;
    }
    return sr.errors = e, r === 0;
  }
  B["quarto-resource-document-language-lang"] = ir;
  var $y = { type: "string", description: "be a string", $id: "quarto-resource-document-language-lang" };
  function ir(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: $y }, message: "must be string", schema: $y.type, parentSchema: $y, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ir.errors = e, r === 0;
  }
  B["quarto-resource-document-language-language"] = lr;
  var lc = { oneOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: {}, patternProperties: {} }], description: "be exactly one of: a string, an object", $id: "quarto-resource-document-language-language" };
  function lr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: lc.oneOf[0] }, message: "must be string", schema: lc.oneOf[0].type, parentSchema: lc.oneOf[0], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (!(l && typeof l == "object" && !Array.isArray(l))) {
      let $ = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "object", schema: lc.oneOf[1] }, message: "must be object", schema: lc.oneOf[1].type, parentSchema: lc.oneOf[1], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: lc }, message: "must match exactly one schema in oneOf", schema: lc.oneOf, parentSchema: lc, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return lr.errors = e, r === 0;
  }
  B["quarto-resource-document-language-dir"] = cr;
  var Gf = { enum: ["rtl", "ltr"], description: "be one of: `rtl`, `ltr`", $id: "quarto-resource-document-language-dir" };
  function cr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Gf.enum;
    if (!(l === "rtl" || l === "ltr")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Gf.enum, schema: Gf }, message: "must be equal to one of the allowed values", schema: Gf.enum, parentSchema: Gf, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return cr.errors = e, r === 0;
  }
  B["quarto-resource-document-latexmk-latex-auto-mk"] = El;
  var Sy = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-latexmk-latex-auto-mk" };
  function El(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Sy }, message: "must be boolean", schema: Sy.type, parentSchema: Sy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return El.errors = e, r === 0;
  }
  B["quarto-resource-document-latexmk-latex-auto-install"] = Nl;
  var Fy = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-latexmk-latex-auto-install" };
  function Nl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Fy }, message: "must be boolean", schema: Fy.type, parentSchema: Fy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Nl.errors = e, r === 0;
  }
  B["quarto-resource-document-latexmk-latex-min-runs"] = Cl;
  var Ay = { type: "number", description: "be a number", $id: "quarto-resource-document-latexmk-latex-min-runs" };
  function Cl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: Ay }, message: "must be number", schema: Ay.type, parentSchema: Ay, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Cl.errors = e, r === 0;
  }
  B["quarto-resource-document-latexmk-latex-max-runs"] = Il;
  var By = { type: "number", description: "be a number", $id: "quarto-resource-document-latexmk-latex-max-runs" };
  function Il(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: By }, message: "must be number", schema: By.type, parentSchema: By, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Il.errors = e, r === 0;
  }
  B["quarto-resource-document-latexmk-latex-clean"] = Hl;
  var jy = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-latexmk-latex-clean" };
  function Hl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: jy }, message: "must be boolean", schema: jy.type, parentSchema: jy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Hl.errors = e, r === 0;
  }
  B["quarto-resource-document-latexmk-latex-makeindex"] = Ll;
  var Ey = { type: "string", description: "be a string", $id: "quarto-resource-document-latexmk-latex-makeindex" };
  function Ll(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Ey }, message: "must be string", schema: Ey.type, parentSchema: Ey, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ll.errors = e, r === 0;
  }
  B["quarto-resource-document-latexmk-latex-makeindex-opts"] = Ml;
  var Fd = { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" }, $id: "quarto-resource-document-latexmk-latex-makeindex-opts" };
  function Ml(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let d = true, u = l.length;
      for (let y = 0; y < u; y++) {
        let g = l[y], x = r;
        if (typeof g != "string") {
          let $ = { instancePath: o + "/" + y, schemaPath: "#/items/type", keyword: "type", params: { type: "string", schema: Fd.items }, message: "must be string", schema: Fd.items.type, parentSchema: Fd.items, data: g };
          e === null ? e = [$] : e.push($), r++;
        }
        let D = r;
        var c = x === r;
        c || (d = false);
      }
    } else {
      let d = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: Fd }, message: "must be array", schema: Fd.type, parentSchema: Fd, data: l };
      e === null ? e = [d] : e.push(d), r++;
    }
    return Ml.errors = e, r === 0;
  }
  B["quarto-resource-document-latexmk-latex-tlmgr-opts"] = Tl;
  var Ad = { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" }, $id: "quarto-resource-document-latexmk-latex-tlmgr-opts" };
  function Tl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let d = true, u = l.length;
      for (let y = 0; y < u; y++) {
        let g = l[y], x = r;
        if (typeof g != "string") {
          let $ = { instancePath: o + "/" + y, schemaPath: "#/items/type", keyword: "type", params: { type: "string", schema: Ad.items }, message: "must be string", schema: Ad.items.type, parentSchema: Ad.items, data: g };
          e === null ? e = [$] : e.push($), r++;
        }
        let D = r;
        var c = x === r;
        c || (d = false);
      }
    } else {
      let d = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: Ad }, message: "must be array", schema: Ad.type, parentSchema: Ad, data: l };
      e === null ? e = [d] : e.push(d), r++;
    }
    return Tl.errors = e, r === 0;
  }
  B["quarto-resource-document-latexmk-latex-output-dir"] = zl;
  var Ny = { type: "string", description: "be a string", $id: "quarto-resource-document-latexmk-latex-output-dir" };
  function zl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Ny }, message: "must be string", schema: Ny.type, parentSchema: Ny, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return zl.errors = e, r === 0;
  }
  B["quarto-resource-document-layout-documentclass"] = js;
  var Cy = { type: "string", description: "be a string", $id: "quarto-resource-document-layout-documentclass" };
  function js(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Cy }, message: "must be string", schema: Cy.type, parentSchema: Cy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return js.errors = e, r === 0;
  }
  B["quarto-resource-document-layout-classoption"] = un;
  var Ri = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-layout-classoption" };
  function un(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Ri.oneOf[0] }, message: "must be string", schema: Ri.oneOf[0].type, parentSchema: Ri.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Ri.oneOf[1].items }, message: "must be string", schema: Ri.oneOf[1].items.type, parentSchema: Ri.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Ri.oneOf[1] }, message: "must be array", schema: Ri.oneOf[1].type, parentSchema: Ri.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Ri }, message: "must match exactly one schema in oneOf", schema: Ri.oneOf, parentSchema: Ri, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return un.errors = e, r === 0;
  }
  B["quarto-resource-document-layout-pagestyle"] = Es;
  var Iy = { type: "string", description: "be a string", $id: "quarto-resource-document-layout-pagestyle" };
  function Es(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Iy }, message: "must be string", schema: Iy.type, parentSchema: Iy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Es.errors = e, r === 0;
  }
  B["quarto-resource-document-layout-papersize"] = Ns;
  var Hy = { type: "string", description: "be a string", $id: "quarto-resource-document-layout-papersize" };
  function Ns(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Hy }, message: "must be string", schema: Hy.type, parentSchema: Hy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ns.errors = e, r === 0;
  }
  B["quarto-resource-document-layout-layout"] = Ec;
  var Vi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-layout-layout" };
  function Ec(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Vi.oneOf[0] }, message: "must be string", schema: Vi.oneOf[0].type, parentSchema: Vi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Vi.oneOf[1].items }, message: "must be string", schema: Vi.oneOf[1].items.type, parentSchema: Vi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Vi.oneOf[1] }, message: "must be array", schema: Vi.oneOf[1].type, parentSchema: Vi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Vi }, message: "must match exactly one schema in oneOf", schema: Vi.oneOf, parentSchema: Vi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Ec.errors = e, r === 0;
  }
  B["quarto-resource-document-layout-page-layout"] = Cs;
  var Yf = { enum: ["article", "full", "custom"], description: "be one of: `article`, `full`, `custom`", $id: "quarto-resource-document-layout-page-layout" };
  function Cs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Yf.enum;
    if (!(l === "article" || l === "full" || l === "custom")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Yf.enum, schema: Yf }, message: "must be equal to one of the allowed values", schema: Yf.enum, parentSchema: Yf, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Cs.errors = e, r === 0;
  }
  B["quarto-resource-document-layout-page-width"] = Is;
  var Ly = { type: "number", description: "be a number", $id: "quarto-resource-document-layout-page-width" };
  function Is(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: Ly }, message: "must be number", schema: Ly.type, parentSchema: Ly, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Is.errors = e, r === 0;
  }
  B["quarto-resource-document-layout-max-width"] = $n;
  var My = { type: "string", description: "be a string", $id: "quarto-resource-document-layout-max-width" };
  function $n(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: My }, message: "must be string", schema: My.type, parentSchema: My, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return $n.errors = e, r === 0;
  }
  B["quarto-resource-document-layout-margin-left"] = Xa;
  var Ty = { type: "string", description: "be a string", $id: "quarto-resource-document-layout-margin-left" };
  function Xa(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Ty }, message: "must be string", schema: Ty.type, parentSchema: Ty, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Xa.errors = e, r === 0;
  }
  B["quarto-resource-document-layout-margin-right"] = Za;
  var zy = { type: "string", description: "be a string", $id: "quarto-resource-document-layout-margin-right" };
  function Za(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: zy }, message: "must be string", schema: zy.type, parentSchema: zy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Za.errors = e, r === 0;
  }
  B["quarto-resource-document-layout-margin-top"] = en;
  var Wy = { type: "string", description: "be a string", $id: "quarto-resource-document-layout-margin-top" };
  function en(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Wy }, message: "must be string", schema: Wy.type, parentSchema: Wy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return en.errors = e, r === 0;
  }
  B["quarto-resource-document-layout-margin-bottom"] = rn;
  var Ry = { type: "string", description: "be a string", $id: "quarto-resource-document-layout-margin-bottom" };
  function rn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Ry }, message: "must be string", schema: Ry.type, parentSchema: Ry, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return rn.errors = e, r === 0;
  }
  B["quarto-resource-document-layout-geometry"] = Hs;
  var Ui = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-layout-geometry" };
  function Hs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Ui.oneOf[0] }, message: "must be string", schema: Ui.oneOf[0].type, parentSchema: Ui.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Ui.oneOf[1].items }, message: "must be string", schema: Ui.oneOf[1].items.type, parentSchema: Ui.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Ui.oneOf[1] }, message: "must be array", schema: Ui.oneOf[1].type, parentSchema: Ui.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Ui }, message: "must match exactly one schema in oneOf", schema: Ui.oneOf, parentSchema: Ui, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Hs.errors = e, r === 0;
  }
  B["quarto-resource-document-layout-hyperrefoptions"] = Ls;
  var Ki = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-layout-hyperrefoptions" };
  function Ls(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Ki.oneOf[0] }, message: "must be string", schema: Ki.oneOf[0].type, parentSchema: Ki.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Ki.oneOf[1].items }, message: "must be string", schema: Ki.oneOf[1].items.type, parentSchema: Ki.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Ki.oneOf[1] }, message: "must be array", schema: Ki.oneOf[1].type, parentSchema: Ki.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Ki }, message: "must match exactly one schema in oneOf", schema: Ki.oneOf, parentSchema: Ki, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Ls.errors = e, r === 0;
  }
  B["quarto-resource-document-layout-indent"] = Eo;
  var cc = { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "string", description: "be a string" }], description: "be exactly one of: `true` or `false`, a string", $id: "quarto-resource-document-layout-indent" };
  function Eo(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "boolean") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: cc.oneOf[0] }, message: "must be boolean", schema: cc.oneOf[0].type, parentSchema: cc.oneOf[0], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r;
    if (typeof l != "string") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "string", schema: cc.oneOf[1] }, message: "must be string", schema: cc.oneOf[1].type, parentSchema: cc.oneOf[1], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let D = r;
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: cc }, message: "must match exactly one schema in oneOf", schema: cc.oneOf, parentSchema: cc, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return Eo.errors = e, r === 0;
  }
  B["quarto-resource-document-layout-block-headings"] = Ms;
  var Vy = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-layout-block-headings" };
  function Ms(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Vy }, message: "must be boolean", schema: Vy.type, parentSchema: Vy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ms.errors = e, r === 0;
  }
  B["quarto-resource-document-library-reveal-js-url"] = Nc;
  var Uy = { type: "string", description: "be a string", $id: "quarto-resource-document-library-reveal-js-url" };
  function Nc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Uy }, message: "must be string", schema: Uy.type, parentSchema: Uy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Nc.errors = e, r === 0;
  }
  B["quarto-resource-document-library-s5-url"] = Cc;
  var Ky = { type: "string", description: "be a string", $id: "quarto-resource-document-library-s5-url" };
  function Cc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Ky }, message: "must be string", schema: Ky.type, parentSchema: Ky, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Cc.errors = e, r === 0;
  }
  B["quarto-resource-document-library-slidy-url"] = Ic;
  var Jy = { type: "string", description: "be a string", $id: "quarto-resource-document-library-slidy-url" };
  function Ic(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Jy }, message: "must be string", schema: Jy.type, parentSchema: Jy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ic.errors = e, r === 0;
  }
  B["quarto-resource-document-library-slideous-url"] = Hc;
  var Qy = { type: "string", description: "be a string", $id: "quarto-resource-document-library-slideous-url" };
  function Hc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Qy }, message: "must be string", schema: Qy.type, parentSchema: Qy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Hc.errors = e, r === 0;
  }
  B["quarto-resource-document-links-link-external-icon"] = No;
  var Gy = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-links-link-external-icon" };
  function No(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Gy }, message: "must be boolean", schema: Gy.type, parentSchema: Gy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return No.errors = e, r === 0;
  }
  B["quarto-resource-document-links-link-external-newwindow"] = Co;
  var Yy = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-links-link-external-newwindow" };
  function Co(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Yy }, message: "must be boolean", schema: Yy.type, parentSchema: Yy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Co.errors = e, r === 0;
  }
  B["quarto-resource-document-links-link-external-filter"] = Io;
  var Xy = { type: "string", description: "be a string", $id: "quarto-resource-document-links-link-external-filter" };
  function Io(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Xy }, message: "must be string", schema: Xy.type, parentSchema: Xy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Io.errors = e, r === 0;
  }
  B["quarto-resource-document-metadata-keywords"] = Ia;
  var Ji = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-metadata-keywords" };
  function Ia(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Ji.oneOf[0] }, message: "must be string", schema: Ji.oneOf[0].type, parentSchema: Ji.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Ji.oneOf[1].items }, message: "must be string", schema: Ji.oneOf[1].items.type, parentSchema: Ji.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Ji.oneOf[1] }, message: "must be array", schema: Ji.oneOf[1].type, parentSchema: Ji.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Ji }, message: "must match exactly one schema in oneOf", schema: Ji.oneOf, parentSchema: Ji, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Ia.errors = e, r === 0;
  }
  B["quarto-resource-document-metadata-subject"] = fo;
  var Zy = { type: "string", description: "be a string", $id: "quarto-resource-document-metadata-subject" };
  function fo(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Zy }, message: "must be string", schema: Zy.type, parentSchema: Zy, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return fo.errors = e, r === 0;
  }
  B["quarto-resource-document-metadata-description"] = Wl;
  var eb = { type: "string", description: "be a string", $id: "quarto-resource-document-metadata-description" };
  function Wl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: eb }, message: "must be string", schema: eb.type, parentSchema: eb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Wl.errors = e, r === 0;
  }
  B["quarto-resource-document-metadata-category"] = Rl;
  var rb = { type: "string", description: "be a string", $id: "quarto-resource-document-metadata-category" };
  function Rl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: rb }, message: "must be string", schema: rb.type, parentSchema: rb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Rl.errors = e, r === 0;
  }
  B["quarto-resource-document-metadata-title-meta"] = Ts;
  var tb = { type: "string", description: "be a string", $id: "quarto-resource-document-metadata-title-meta" };
  function Ts(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: tb }, message: "must be string", schema: tb.type, parentSchema: tb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ts.errors = e, r === 0;
  }
  B["quarto-resource-document-metadata-pagetitle"] = Sn;
  var ab = { type: "string", description: "be a string", $id: "quarto-resource-document-metadata-pagetitle" };
  function Sn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: ab }, message: "must be string", schema: ab.type, parentSchema: ab, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Sn.errors = e, r === 0;
  }
  B["quarto-resource-document-metadata-title-prefix"] = Fn;
  var nb = { type: "string", description: "be a string", $id: "quarto-resource-document-metadata-title-prefix" };
  function Fn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: nb }, message: "must be string", schema: nb.type, parentSchema: nb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Fn.errors = e, r === 0;
  }
  B["quarto-resource-document-metadata-description-meta"] = An;
  var ob = { type: "string", description: "be a string", $id: "quarto-resource-document-metadata-description-meta" };
  function An(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: ob }, message: "must be string", schema: ob.type, parentSchema: ob, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return An.errors = e, r === 0;
  }
  B["quarto-resource-document-metadata-author-meta"] = dn;
  var sb = { type: "string", description: "be a string", $id: "quarto-resource-document-metadata-author-meta" };
  function dn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: sb }, message: "must be string", schema: sb.type, parentSchema: sb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return dn.errors = e, r === 0;
  }
  B["quarto-resource-document-metadata-date-meta"] = Ha;
  var ib = { type: "string", description: "be a string", $id: "quarto-resource-document-metadata-date-meta" };
  function Ha(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: ib }, message: "must be string", schema: ib.type, parentSchema: ib, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ha.errors = e, r === 0;
  }
  B["quarto-resource-document-numbering-number-sections"] = pr;
  var lb = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-numbering-number-sections" };
  function pr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: lb }, message: "must be boolean", schema: lb.type, parentSchema: lb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return pr.errors = e, r === 0;
  }
  B["quarto-resource-document-numbering-number-depth"] = Da;
  var cb = { type: "number", description: "be a number", $id: "quarto-resource-document-numbering-number-depth" };
  function Da(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: cb }, message: "must be number", schema: cb.type, parentSchema: cb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Da.errors = e, r === 0;
  }
  B["quarto-resource-document-numbering-secnumdepth"] = zs;
  var pb = { type: "number", description: "be a number", $id: "quarto-resource-document-numbering-secnumdepth" };
  function zs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: pb }, message: "must be number", schema: pb.type, parentSchema: pb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return zs.errors = e, r === 0;
  }
  B["quarto-resource-document-numbering-number-offset"] = ur;
  var Bd = { type: "array", description: "be an array of values, where each element should be a number", items: { type: "number", description: "be a number" }, $id: "quarto-resource-document-numbering-number-offset" };
  function ur(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let d = true, u = l.length;
      for (let y = 0; y < u; y++) {
        let g = l[y], x = r;
        if (!(typeof g == "number" && isFinite(g))) {
          let $ = { instancePath: o + "/" + y, schemaPath: "#/items/type", keyword: "type", params: { type: "number", schema: Bd.items }, message: "must be number", schema: Bd.items.type, parentSchema: Bd.items, data: g };
          e === null ? e = [$] : e.push($), r++;
        }
        let D = r;
        var c = x === r;
        c || (d = false);
      }
    } else {
      let d = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: Bd }, message: "must be array", schema: Bd.type, parentSchema: Bd, data: l };
      e === null ? e = [d] : e.push(d), r++;
    }
    return ur.errors = e, r === 0;
  }
  B["quarto-resource-document-numbering-shift-heading-level-by"] = dr;
  var ub = { type: "number", description: "be a number", $id: "quarto-resource-document-numbering-shift-heading-level-by" };
  function dr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: ub }, message: "must be number", schema: ub.type, parentSchema: ub, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return dr.errors = e, r === 0;
  }
  B["quarto-resource-document-numbering-pagenumbering"] = Lc;
  var Qi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-numbering-pagenumbering" };
  function Lc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Qi.oneOf[0] }, message: "must be string", schema: Qi.oneOf[0].type, parentSchema: Qi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Qi.oneOf[1].items }, message: "must be string", schema: Qi.oneOf[1].items.type, parentSchema: Qi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Qi.oneOf[1] }, message: "must be array", schema: Qi.oneOf[1].type, parentSchema: Qi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Qi }, message: "must match exactly one schema in oneOf", schema: Qi.oneOf, parentSchema: Qi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Lc.errors = e, r === 0;
  }
  B["quarto-resource-document-numbering-top-level-division"] = Bn;
  var Xf = { enum: ["default", "section", "chapter", "part"], description: "be one of: `default`, `section`, `chapter`, `part`", $id: "quarto-resource-document-numbering-top-level-division" };
  function Bn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Xf.enum;
    if (!(l === "default" || l === "section" || l === "chapter" || l === "part")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Xf.enum, schema: Xf }, message: "must be equal to one of the allowed values", schema: Xf.enum, parentSchema: Xf, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Bn.errors = e, r === 0;
  }
  B["quarto-resource-document-options-reference-doc"] = Ws;
  var db = { type: "string", description: "be a string", $id: "quarto-resource-document-options-reference-doc" };
  function Ws(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: db }, message: "must be string", schema: db.type, parentSchema: db, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ws.errors = e, r === 0;
  }
  B["quarto-resource-document-options-theme"] = Ho;
  var sn = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }, { type: "object", description: "be an object", properties: { light: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, dark: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" } }, patternProperties: {} }], description: "be exactly one of: a string, an array of values, where each element should be a string, an object", $id: "quarto-resource-document-options-theme" };
  function Ho(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let P = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: sn.oneOf[0] }, message: "must be string", schema: sn.oneOf[0].type, parentSchema: sn.oneOf[0], data: l };
      e === null ? e = [P] : e.push(P), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let P = true, oe = l.length;
      for (let h = 0; h < oe; h++) {
        let f = l[h], ft = r;
        if (typeof f != "string") {
          let Dr = { instancePath: o + "/" + h, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: sn.oneOf[1].items }, message: "must be string", schema: sn.oneOf[1].items.type, parentSchema: sn.oneOf[1].items, data: f };
          e === null ? e = [Dr] : e.push(Dr), r++;
        }
        let Cr = r;
        var $ = ft === r;
        $ || (P = false);
      }
    } else {
      let P = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: sn.oneOf[1] }, message: "must be array", schema: sn.oneOf[1].type, parentSchema: sn.oneOf[1], data: l };
      e === null ? e = [P] : e.push(P), r++;
    }
    var g = x === r;
    if (g && c)
      c = false, d = [d, 1];
    else {
      g && (c = true, d = 1);
      let P = r, oe = r;
      if (l && typeof l == "object" && !Array.isArray(l)) {
        if (l.light !== void 0) {
          let f = l.light, ft = r, Cr = r, Dr = false, wr = null, ta = r;
          if (typeof f != "string") {
            let sa = { instancePath: o + "/light", schemaPath: "#/oneOf/2/properties/light/oneOf/0/type", keyword: "type", params: { type: "string", schema: sn.oneOf[2].properties.light.oneOf[0] }, message: "must be string", schema: sn.oneOf[2].properties.light.oneOf[0].type, parentSchema: sn.oneOf[2].properties.light.oneOf[0], data: f };
            e === null ? e = [sa] : e.push(sa), r++;
          }
          let jt = r;
          var m = ta === r;
          m && (Dr = true, wr = 0);
          let ee = r, za = r;
          if (Array.isArray(f)) {
            let sa = true, _e = f.length;
            for (let Ma = 0; Ma < _e; Ma++) {
              let fa = f[Ma], ie = r;
              if (typeof fa != "string") {
                let hn = { instancePath: o + "/light/" + Ma, schemaPath: "#/oneOf/2/properties/light/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: sn.oneOf[2].properties.light.oneOf[1].items }, message: "must be string", schema: sn.oneOf[2].properties.light.oneOf[1].items.type, parentSchema: sn.oneOf[2].properties.light.oneOf[1].items, data: fa };
                e === null ? e = [hn] : e.push(hn), r++;
              }
              let ka = r;
              var R = ie === r;
              R || (sa = false);
            }
          } else {
            let sa = { instancePath: o + "/light", schemaPath: "#/oneOf/2/properties/light/oneOf/1/type", keyword: "type", params: { type: "array", schema: sn.oneOf[2].properties.light.oneOf[1] }, message: "must be array", schema: sn.oneOf[2].properties.light.oneOf[1].type, parentSchema: sn.oneOf[2].properties.light.oneOf[1], data: f };
            e === null ? e = [sa] : e.push(sa), r++;
          }
          var m = ee === r;
          if (m && Dr ? (Dr = false, wr = [wr, 1]) : m && (Dr = true, wr = 1), Dr)
            r = Cr, e !== null && (Cr ? e.length = Cr : e = null);
          else {
            let sa = { instancePath: o + "/light", schemaPath: "#/oneOf/2/properties/light/oneOf", keyword: "oneOf", params: { passingSchemas: wr, schema: sn.oneOf[2].properties.light }, message: "must match exactly one schema in oneOf", schema: sn.oneOf[2].properties.light.oneOf, parentSchema: sn.oneOf[2].properties.light, data: f };
            e === null ? e = [sa] : e.push(sa), r++;
          }
          var j = ft === r;
        }
        if (l.dark !== void 0) {
          let f = l.dark, ft = r, Cr = r, Dr = false, wr = null, ta = r;
          if (typeof f != "string") {
            let _e = { instancePath: o + "/dark", schemaPath: "#/oneOf/2/properties/dark/oneOf/0/type", keyword: "type", params: { type: "string", schema: sn.oneOf[2].properties.dark.oneOf[0] }, message: "must be string", schema: sn.oneOf[2].properties.dark.oneOf[0].type, parentSchema: sn.oneOf[2].properties.dark.oneOf[0], data: f };
            e === null ? e = [_e] : e.push(_e), r++;
          }
          let jt = r;
          var V = ta === r;
          V && (Dr = true, wr = 0);
          let ee = r, za = r;
          if (Array.isArray(f)) {
            let _e = true, Ma = f.length;
            for (let fa = 0; fa < Ma; fa++) {
              let ie = f[fa], ka = r;
              if (typeof ie != "string") {
                let et = { instancePath: o + "/dark/" + fa, schemaPath: "#/oneOf/2/properties/dark/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: sn.oneOf[2].properties.dark.oneOf[1].items }, message: "must be string", schema: sn.oneOf[2].properties.dark.oneOf[1].items.type, parentSchema: sn.oneOf[2].properties.dark.oneOf[1].items, data: ie };
                e === null ? e = [et] : e.push(et), r++;
              }
              let hn = r;
              var se = ka === r;
              se || (_e = false);
            }
          } else {
            let _e = { instancePath: o + "/dark", schemaPath: "#/oneOf/2/properties/dark/oneOf/1/type", keyword: "type", params: { type: "array", schema: sn.oneOf[2].properties.dark.oneOf[1] }, message: "must be array", schema: sn.oneOf[2].properties.dark.oneOf[1].type, parentSchema: sn.oneOf[2].properties.dark.oneOf[1], data: f };
            e === null ? e = [_e] : e.push(_e), r++;
          }
          var V = ee === r;
          if (V && Dr ? (Dr = false, wr = [wr, 1]) : V && (Dr = true, wr = 1), Dr)
            r = Cr, e !== null && (Cr ? e.length = Cr : e = null);
          else {
            let _e = { instancePath: o + "/dark", schemaPath: "#/oneOf/2/properties/dark/oneOf", keyword: "oneOf", params: { passingSchemas: wr, schema: sn.oneOf[2].properties.dark }, message: "must match exactly one schema in oneOf", schema: sn.oneOf[2].properties.dark.oneOf, parentSchema: sn.oneOf[2].properties.dark, data: f };
            e === null ? e = [_e] : e.push(_e), r++;
          }
          var j = ft === r;
        }
      } else {
        let f = { instancePath: o, schemaPath: "#/oneOf/2/type", keyword: "type", params: { type: "object", schema: sn.oneOf[2] }, message: "must be object", schema: sn.oneOf[2].type, parentSchema: sn.oneOf[2], data: l };
        e === null ? e = [f] : e.push(f), r++;
      }
      var g = P === r;
      g && c ? (c = false, d = [d, 2]) : g && (c = true, d = 2);
    }
    if (c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let P = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: sn }, message: "must match exactly one schema in oneOf", schema: sn.oneOf, parentSchema: sn, data: l };
      e === null ? e = [P] : e.push(P), r++;
    }
    return Ho.errors = e, r === 0;
  }
  B["quarto-resource-document-options-minimal"] = Rs;
  var fb = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-options-minimal" };
  function Rs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: fb }, message: "must be boolean", schema: fb.type, parentSchema: fb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Rs.errors = e, r === 0;
  }
  B["quarto-resource-document-options-document-css"] = jn;
  var mb = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-options-document-css" };
  function jn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: mb }, message: "must be boolean", schema: mb.type, parentSchema: mb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return jn.errors = e, r === 0;
  }
  B["quarto-resource-document-options-css"] = fn;
  var Gi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-options-css" };
  function fn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Gi.oneOf[0] }, message: "must be string", schema: Gi.oneOf[0].type, parentSchema: Gi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Gi.oneOf[1].items }, message: "must be string", schema: Gi.oneOf[1].items.type, parentSchema: Gi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Gi.oneOf[1] }, message: "must be array", schema: Gi.oneOf[1].type, parentSchema: Gi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Gi }, message: "must match exactly one schema in oneOf", schema: Gi.oneOf, parentSchema: Gi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return fn.errors = e, r === 0;
  }
  B["quarto-resource-document-options-anchor-sections"] = Vs;
  var gb = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-options-anchor-sections" };
  function Vs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: gb }, message: "must be boolean", schema: gb.type, parentSchema: gb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Vs.errors = e, r === 0;
  }
  B["quarto-resource-document-options-html-math-method"] = Wn;
  var Yi = { oneOf: [{ $ref: "math-methods", description: "be math-methods" }, { type: "object", description: "be an object", properties: { method: { $ref: "math-methods", description: "be math-methods" }, url: { type: "string", description: "be a string" } }, patternProperties: {}, required: ["method"] }], description: "be exactly one of: math-methods, an object", $id: "quarto-resource-document-options-html-math-method" };
  function Wn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    gc(l, { instancePath: o, parentData: S, parentDataProperty: F, rootData: s }) || (e = e === null ? gc.errors : e.concat(gc.errors), r = e.length);
    var y = u === r;
    y && (c = true, d = 0);
    let g = r, x = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      if (l.method === void 0) {
        let $ = { instancePath: o, schemaPath: "#/oneOf/1/required", keyword: "required", params: { missingProperty: "method", schema: Yi.oneOf[1] }, message: "must have required property 'method'", schema: Yi.oneOf[1].required, parentSchema: Yi.oneOf[1], data: l };
        e === null ? e = [$] : e.push($), r++;
      }
      if (l.method !== void 0) {
        let $ = l.method, m = r;
        gc($, { instancePath: o + "/method", parentData: l, parentDataProperty: "method", rootData: s }) || (e = e === null ? gc.errors : e.concat(gc.errors), r = e.length);
        var D = m === r;
      }
      if (l.url !== void 0) {
        let $ = l.url, m = r;
        if (typeof $ != "string") {
          let V = { instancePath: o + "/url", schemaPath: "#/oneOf/1/properties/url/type", keyword: "type", params: { type: "string", schema: Yi.oneOf[1].properties.url }, message: "must be string", schema: Yi.oneOf[1].properties.url.type, parentSchema: Yi.oneOf[1].properties.url, data: $ };
          e === null ? e = [V] : e.push(V), r++;
        }
        let R = r;
        var D = m === r;
      }
    } else {
      let $ = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "object", schema: Yi.oneOf[1] }, message: "must be object", schema: Yi.oneOf[1].type, parentSchema: Yi.oneOf[1], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    var y = g === r;
    if (y && c ? (c = false, d = [d, 1]) : y && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Yi }, message: "must match exactly one schema in oneOf", schema: Yi.oneOf, parentSchema: Yi, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return Wn.errors = e, r === 0;
  }
  B["quarto-resource-document-options-section-divs"] = En;
  var hb = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-options-section-divs" };
  function En(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: hb }, message: "must be boolean", schema: hb.type, parentSchema: hb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return En.errors = e, r === 0;
  }
  B["quarto-resource-document-options-identifier-prefix"] = Pa;
  var yb = { type: "string", description: "be a string", $id: "quarto-resource-document-options-identifier-prefix" };
  function Pa(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: yb }, message: "must be string", schema: yb.type, parentSchema: yb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Pa.errors = e, r === 0;
  }
  B["quarto-resource-document-options-email-obfuscation"] = Nn;
  var Zf = { enum: ["none", "references", "javascript"], description: "be one of: `none`, `references`, `javascript`", $id: "quarto-resource-document-options-email-obfuscation" };
  function Nn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = Zf.enum;
    if (!(l === "none" || l === "references" || l === "javascript")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: Zf.enum, schema: Zf }, message: "must be equal to one of the allowed values", schema: Zf.enum, parentSchema: Zf, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Nn.errors = e, r === 0;
  }
  B["quarto-resource-document-options-html-q-tags"] = mn;
  var bb = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-options-html-q-tags" };
  function mn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: bb }, message: "must be boolean", schema: bb.type, parentSchema: bb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return mn.errors = e, r === 0;
  }
  B["quarto-resource-document-options-pdf-engine"] = mo;
  var _b = { type: "string", description: "be a string", $id: "quarto-resource-document-options-pdf-engine" };
  function mo(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: _b }, message: "must be string", schema: _b.type, parentSchema: _b, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return mo.errors = e, r === 0;
  }
  B["quarto-resource-document-options-pdf-engine-opt"] = go;
  var vb = { type: "string", description: "be a string", $id: "quarto-resource-document-options-pdf-engine-opt" };
  function go(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: vb }, message: "must be string", schema: vb.type, parentSchema: vb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return go.errors = e, r === 0;
  }
  B["quarto-resource-document-options-beamerarticle"] = Mc;
  var qb = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-options-beamerarticle" };
  function Mc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: qb }, message: "must be boolean", schema: qb.type, parentSchema: qb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Mc.errors = e, r === 0;
  }
  B["quarto-resource-document-options-beameroption"] = Tc;
  var Xi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-options-beameroption" };
  function Tc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Xi.oneOf[0] }, message: "must be string", schema: Xi.oneOf[0].type, parentSchema: Xi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Xi.oneOf[1].items }, message: "must be string", schema: Xi.oneOf[1].items.type, parentSchema: Xi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Xi.oneOf[1] }, message: "must be array", schema: Xi.oneOf[1].type, parentSchema: Xi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Xi }, message: "must match exactly one schema in oneOf", schema: Xi.oneOf, parentSchema: Xi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Tc.errors = e, r === 0;
  }
  B["quarto-resource-document-options-aspectratio"] = zc;
  var gn = { oneOf: [{ enum: [43], description: "be 43" }, { enum: [169], description: "be 169" }, { enum: [1610], description: "be 1610" }, { enum: [149], description: "be 149" }, { enum: [141], description: "be 141" }, { enum: [54], description: "be 54" }, { enum: [32], description: "be 32" }], description: "be exactly one of: 43, 169, 1610, 149, 141, 54, 32", $id: "quarto-resource-document-options-aspectratio" };
  function zc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r, y = gn.oneOf[0].enum;
    if (l !== 43) {
      let $ = { instancePath: o, schemaPath: "#/oneOf/0/enum", keyword: "enum", params: { allowedValues: gn.oneOf[0].enum, schema: gn.oneOf[0] }, message: "must be equal to one of the allowed values", schema: gn.oneOf[0].enum, parentSchema: gn.oneOf[0], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = gn.oneOf[1].enum;
    if (l !== 169) {
      let $ = { instancePath: o, schemaPath: "#/oneOf/1/enum", keyword: "enum", params: { allowedValues: gn.oneOf[1].enum, schema: gn.oneOf[1] }, message: "must be equal to one of the allowed values", schema: gn.oneOf[1].enum, parentSchema: gn.oneOf[1], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    var g = x === r;
    if (g && c)
      c = false, d = [d, 1];
    else {
      g && (c = true, d = 1);
      let $ = r, m = gn.oneOf[2].enum;
      if (l !== 1610) {
        let j = { instancePath: o, schemaPath: "#/oneOf/2/enum", keyword: "enum", params: { allowedValues: gn.oneOf[2].enum, schema: gn.oneOf[2] }, message: "must be equal to one of the allowed values", schema: gn.oneOf[2].enum, parentSchema: gn.oneOf[2], data: l };
        e === null ? e = [j] : e.push(j), r++;
      }
      var g = $ === r;
      if (g && c)
        c = false, d = [d, 2];
      else {
        g && (c = true, d = 2);
        let j = r, V = gn.oneOf[3].enum;
        if (l !== 149) {
          let P = { instancePath: o, schemaPath: "#/oneOf/3/enum", keyword: "enum", params: { allowedValues: gn.oneOf[3].enum, schema: gn.oneOf[3] }, message: "must be equal to one of the allowed values", schema: gn.oneOf[3].enum, parentSchema: gn.oneOf[3], data: l };
          e === null ? e = [P] : e.push(P), r++;
        }
        var g = j === r;
        if (g && c)
          c = false, d = [d, 3];
        else {
          g && (c = true, d = 3);
          let P = r, oe = gn.oneOf[4].enum;
          if (l !== 141) {
            let f = { instancePath: o, schemaPath: "#/oneOf/4/enum", keyword: "enum", params: { allowedValues: gn.oneOf[4].enum, schema: gn.oneOf[4] }, message: "must be equal to one of the allowed values", schema: gn.oneOf[4].enum, parentSchema: gn.oneOf[4], data: l };
            e === null ? e = [f] : e.push(f), r++;
          }
          var g = P === r;
          if (g && c)
            c = false, d = [d, 4];
          else {
            g && (c = true, d = 4);
            let f = r, ft = gn.oneOf[5].enum;
            if (l !== 54) {
              let Dr = { instancePath: o, schemaPath: "#/oneOf/5/enum", keyword: "enum", params: { allowedValues: gn.oneOf[5].enum, schema: gn.oneOf[5] }, message: "must be equal to one of the allowed values", schema: gn.oneOf[5].enum, parentSchema: gn.oneOf[5], data: l };
              e === null ? e = [Dr] : e.push(Dr), r++;
            }
            var g = f === r;
            if (g && c)
              c = false, d = [d, 5];
            else {
              g && (c = true, d = 5);
              let Dr = r, wr = gn.oneOf[6].enum;
              if (l !== 32) {
                let jt = { instancePath: o, schemaPath: "#/oneOf/6/enum", keyword: "enum", params: { allowedValues: gn.oneOf[6].enum, schema: gn.oneOf[6] }, message: "must be equal to one of the allowed values", schema: gn.oneOf[6].enum, parentSchema: gn.oneOf[6], data: l };
                e === null ? e = [jt] : e.push(jt), r++;
              }
              var g = Dr === r;
              g && c ? (c = false, d = [d, 6]) : g && (c = true, d = 6);
            }
          }
        }
      }
    }
    if (c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: gn }, message: "must match exactly one schema in oneOf", schema: gn.oneOf, parentSchema: gn, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return zc.errors = e, r === 0;
  }
  B["quarto-resource-document-options-logo"] = em;
  var Ob = { type: "string", description: "be a string", $id: "quarto-resource-document-options-logo" };
  function em(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Ob }, message: "must be string", schema: Ob.type, parentSchema: Ob, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return em.errors = e, r === 0;
  }
  B["quarto-resource-document-options-titlegraphic"] = Wc;
  var Pb = { type: "string", description: "be a string", $id: "quarto-resource-document-options-titlegraphic" };
  function Wc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Pb }, message: "must be string", schema: Pb.type, parentSchema: Pb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Wc.errors = e, r === 0;
  }
  B["quarto-resource-document-options-navigation"] = Rc;
  var rm = { enum: ["empty", "frame", "vertical", "horizontal"], description: "be one of: `empty`, `frame`, `vertical`, `horizontal`", $id: "quarto-resource-document-options-navigation" };
  function Rc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = rm.enum;
    if (!(l === "empty" || l === "frame" || l === "vertical" || l === "horizontal")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: rm.enum, schema: rm }, message: "must be equal to one of the allowed values", schema: rm.enum, parentSchema: rm, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Rc.errors = e, r === 0;
  }
  B["quarto-resource-document-options-section-titles"] = Vc;
  var xb = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-options-section-titles" };
  function Vc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: xb }, message: "must be boolean", schema: xb.type, parentSchema: xb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Vc.errors = e, r === 0;
  }
  B["quarto-resource-document-options-colortheme"] = Uc;
  var kb = { type: "string", description: "be a string", $id: "quarto-resource-document-options-colortheme" };
  function Uc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: kb }, message: "must be string", schema: kb.type, parentSchema: kb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Uc.errors = e, r === 0;
  }
  B["quarto-resource-document-options-fonttheme"] = Kc;
  var Db = { type: "string", description: "be a string", $id: "quarto-resource-document-options-fonttheme" };
  function Kc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Db }, message: "must be string", schema: Db.type, parentSchema: Db, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Kc.errors = e, r === 0;
  }
  B["quarto-resource-document-options-innertheme"] = Jc;
  var wb = { type: "string", description: "be a string", $id: "quarto-resource-document-options-innertheme" };
  function Jc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: wb }, message: "must be string", schema: wb.type, parentSchema: wb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Jc.errors = e, r === 0;
  }
  B["quarto-resource-document-options-outertheme"] = Qc;
  var $b = { type: "string", description: "be a string", $id: "quarto-resource-document-options-outertheme" };
  function Qc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: $b }, message: "must be string", schema: $b.type, parentSchema: $b, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Qc.errors = e, r === 0;
  }
  B["quarto-resource-document-options-themeoptions"] = Gc;
  var Zi = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-options-themeoptions" };
  function Gc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: Zi.oneOf[0] }, message: "must be string", schema: Zi.oneOf[0].type, parentSchema: Zi.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Zi.oneOf[1].items }, message: "must be string", schema: Zi.oneOf[1].items.type, parentSchema: Zi.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: Zi.oneOf[1] }, message: "must be array", schema: Zi.oneOf[1].type, parentSchema: Zi.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Zi }, message: "must match exactly one schema in oneOf", schema: Zi.oneOf, parentSchema: Zi, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Gc.errors = e, r === 0;
  }
  B["quarto-resource-document-options-section"] = Yc;
  var Sb = { type: "number", description: "be a number", $id: "quarto-resource-document-options-section" };
  function Yc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: Sb }, message: "must be number", schema: Sb.type, parentSchema: Sb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Yc.errors = e, r === 0;
  }
  B["quarto-resource-document-options-variant"] = Lo;
  var Fb = { type: "string", description: "be a string", $id: "quarto-resource-document-options-variant" };
  function Lo(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Fb }, message: "must be string", schema: Fb.type, parentSchema: Fb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Lo.errors = e, r === 0;
  }
  B["quarto-resource-document-options-markdown-headings"] = ho;
  var tm = { enum: ["setext", "atx"], description: "be one of: `setext`, `atx`", $id: "quarto-resource-document-options-markdown-headings" };
  function ho(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = tm.enum;
    if (!(l === "setext" || l === "atx")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: tm.enum, schema: tm }, message: "must be equal to one of the allowed values", schema: tm.enum, parentSchema: tm, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return ho.errors = e, r === 0;
  }
  B["quarto-resource-document-options-keep-yaml"] = Mo;
  var Ab = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-options-keep-yaml" };
  function Mo(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Ab }, message: "must be boolean", schema: Ab.type, parentSchema: Ab, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Mo.errors = e, r === 0;
  }
  B["quarto-resource-document-options-ipynb-output"] = Xc;
  var am = { enum: ["none", "all", "best"], description: "be one of: `none`, `all`, `best`", $id: "quarto-resource-document-options-ipynb-output" };
  function Xc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = am.enum;
    if (!(l === "none" || l === "all" || l === "best")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: am.enum, schema: am }, message: "must be equal to one of the allowed values", schema: am.enum, parentSchema: am, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Xc.errors = e, r === 0;
  }
  B["quarto-resource-document-pdfa-pdfa"] = Zc;
  var pc = { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "string", description: "be a string" }], description: "be exactly one of: `true` or `false`, a string", $id: "quarto-resource-document-pdfa-pdfa" };
  function Zc(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "boolean") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: pc.oneOf[0] }, message: "must be boolean", schema: pc.oneOf[0].type, parentSchema: pc.oneOf[0], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r;
    if (typeof l != "string") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "string", schema: pc.oneOf[1] }, message: "must be string", schema: pc.oneOf[1].type, parentSchema: pc.oneOf[1], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let D = r;
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: pc }, message: "must match exactly one schema in oneOf", schema: pc.oneOf, parentSchema: pc, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return Zc.errors = e, r === 0;
  }
  B["quarto-resource-document-pdfa-pdfaiccprofile"] = ep;
  var el = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-pdfa-pdfaiccprofile" };
  function ep(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: el.oneOf[0] }, message: "must be string", schema: el.oneOf[0].type, parentSchema: el.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: el.oneOf[1].items }, message: "must be string", schema: el.oneOf[1].items.type, parentSchema: el.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: el.oneOf[1] }, message: "must be array", schema: el.oneOf[1].type, parentSchema: el.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: el }, message: "must match exactly one schema in oneOf", schema: el.oneOf, parentSchema: el, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return ep.errors = e, r === 0;
  }
  B["quarto-resource-document-pdfa-pdfaintent"] = rp;
  var Bb = { type: "string", description: "be a string", $id: "quarto-resource-document-pdfa-pdfaintent" };
  function rp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Bb }, message: "must be string", schema: Bb.type, parentSchema: Bb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return rp.errors = e, r === 0;
  }
  B["quarto-resource-document-references-bibliography"] = fr;
  var rl = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-references-bibliography" };
  function fr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: rl.oneOf[0] }, message: "must be string", schema: rl.oneOf[0].type, parentSchema: rl.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: rl.oneOf[1].items }, message: "must be string", schema: rl.oneOf[1].items.type, parentSchema: rl.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: rl.oneOf[1] }, message: "must be array", schema: rl.oneOf[1].type, parentSchema: rl.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: rl }, message: "must match exactly one schema in oneOf", schema: rl.oneOf, parentSchema: rl, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return fr.errors = e, r === 0;
  }
  B["quarto-resource-document-references-csl"] = mr;
  var jb = { type: "string", description: "be a string", $id: "quarto-resource-document-references-csl" };
  function mr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: jb }, message: "must be string", schema: jb.type, parentSchema: jb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return mr.errors = e, r === 0;
  }
  B["quarto-resource-document-references-citations-hover"] = Cn;
  var Eb = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-references-citations-hover" };
  function Cn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Eb }, message: "must be boolean", schema: Eb.type, parentSchema: Eb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Cn.errors = e, r === 0;
  }
  B["quarto-resource-document-references-citation-location"] = Us;
  var nm = { enum: ["document", "margin"], description: "be one of: `document`, `margin`", $id: "quarto-resource-document-references-citation-location" };
  function Us(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = nm.enum;
    if (!(l === "document" || l === "margin")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: nm.enum, schema: nm }, message: "must be equal to one of the allowed values", schema: nm.enum, parentSchema: nm, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Us.errors = e, r === 0;
  }
  B["quarto-resource-document-references-cite-method"] = Ks;
  var om = { enum: ["citeproc", "natbib", "biblatex"], description: "be one of: `citeproc`, `natbib`, `biblatex`", $id: "quarto-resource-document-references-cite-method" };
  function Ks(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = om.enum;
    if (!(l === "citeproc" || l === "natbib" || l === "biblatex")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: om.enum, schema: om }, message: "must be equal to one of the allowed values", schema: om.enum, parentSchema: om, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Ks.errors = e, r === 0;
  }
  B["quarto-resource-document-references-citeproc"] = gr;
  var Nb = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-references-citeproc" };
  function gr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Nb }, message: "must be boolean", schema: Nb.type, parentSchema: Nb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return gr.errors = e, r === 0;
  }
  B["quarto-resource-document-references-biblatexoptions"] = Js;
  var tl = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-references-biblatexoptions" };
  function Js(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: tl.oneOf[0] }, message: "must be string", schema: tl.oneOf[0].type, parentSchema: tl.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: tl.oneOf[1].items }, message: "must be string", schema: tl.oneOf[1].items.type, parentSchema: tl.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: tl.oneOf[1] }, message: "must be array", schema: tl.oneOf[1].type, parentSchema: tl.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: tl }, message: "must match exactly one schema in oneOf", schema: tl.oneOf, parentSchema: tl, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Js.errors = e, r === 0;
  }
  B["quarto-resource-document-references-natbiboptions"] = Qs;
  var al = { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string", $id: "quarto-resource-document-references-natbiboptions" };
  function Qs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "string") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "string", schema: al.oneOf[0] }, message: "must be string", schema: al.oneOf[0].type, parentSchema: al.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (Array.isArray(l)) {
      let m = true, R = l.length;
      for (let j = 0; j < R; j++) {
        let V = l[j], se = r;
        if (typeof V != "string") {
          let oe = { instancePath: o + "/" + j, schemaPath: "#/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: al.oneOf[1].items }, message: "must be string", schema: al.oneOf[1].items.type, parentSchema: al.oneOf[1].items, data: V };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let P = r;
        var $ = se === r;
        $ || (m = false);
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "array", schema: al.oneOf[1] }, message: "must be array", schema: al.oneOf[1].type, parentSchema: al.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: al }, message: "must match exactly one schema in oneOf", schema: al.oneOf, parentSchema: al, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return Qs.errors = e, r === 0;
  }
  B["quarto-resource-document-references-biblio-style"] = Gs;
  var Cb = { type: "string", description: "be a string", $id: "quarto-resource-document-references-biblio-style" };
  function Gs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Cb }, message: "must be string", schema: Cb.type, parentSchema: Cb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Gs.errors = e, r === 0;
  }
  B["quarto-resource-document-references-biblio-title"] = Ys;
  var Ib = { type: "string", description: "be a string", $id: "quarto-resource-document-references-biblio-title" };
  function Ys(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Ib }, message: "must be string", schema: Ib.type, parentSchema: Ib, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ys.errors = e, r === 0;
  }
  B["quarto-resource-document-references-citation-abbreviations"] = hr;
  var Hb = { type: "string", description: "be a string", $id: "quarto-resource-document-references-citation-abbreviations" };
  function hr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Hb }, message: "must be string", schema: Hb.type, parentSchema: Hb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return hr.errors = e, r === 0;
  }
  B["quarto-resource-document-render-from"] = H;
  var Lb = { type: "string", description: "be a string", $id: "quarto-resource-document-render-from" };
  function H(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Lb }, message: "must be string", schema: Lb.type, parentSchema: Lb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return H.errors = e, r === 0;
  }
  B["quarto-resource-document-render-reader"] = F1;
  var Mb = { type: "string", description: "be a string", $id: "quarto-resource-document-render-reader" };
  function F1(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Mb }, message: "must be string", schema: Mb.type, parentSchema: Mb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return F1.errors = e, r === 0;
  }
  B["quarto-resource-document-render-output-file"] = yr;
  function yr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    return Id(l, { instancePath: o, parentData: S, parentDataProperty: F, rootData: s }) || (e = e === null ? Id.errors : e.concat(Id.errors), r = e.length), yr.errors = e, r === 0;
  }
  B["quarto-resource-document-render-output-ext"] = br;
  var Tb = { type: "string", description: "be a string", $id: "quarto-resource-document-render-output-ext" };
  function br(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Tb }, message: "must be string", schema: Tb.type, parentSchema: Tb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return br.errors = e, r === 0;
  }
  B["quarto-resource-document-render-template"] = Sr;
  var zb = { type: "string", description: "be a string", $id: "quarto-resource-document-render-template" };
  function Sr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: zb }, message: "must be string", schema: zb.type, parentSchema: zb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Sr.errors = e, r === 0;
  }
  B["quarto-resource-document-render-standalone"] = _r;
  var Wb = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-render-standalone" };
  function _r(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Wb }, message: "must be boolean", schema: Wb.type, parentSchema: Wb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return _r.errors = e, r === 0;
  }
  B["quarto-resource-document-render-self-contained"] = In;
  var Rb = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-render-self-contained" };
  function In(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Rb }, message: "must be boolean", schema: Rb.type, parentSchema: Rb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return In.errors = e, r === 0;
  }
  B["quarto-resource-document-render-self-contained-math"] = Hn;
  var Vb = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-render-self-contained-math" };
  function Hn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Vb }, message: "must be boolean", schema: Vb.type, parentSchema: Vb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Hn.errors = e, r === 0;
  }
  B["quarto-resource-document-render-filters"] = vr;
  function vr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    return Hd(l, { instancePath: o, parentData: S, parentDataProperty: F, rootData: s }) || (e = e === null ? Hd.errors : e.concat(Hd.errors), r = e.length), vr.errors = e, r === 0;
  }
  B["quarto-resource-document-render-keep-md"] = Be;
  var Ub = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-render-keep-md" };
  function Be(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Ub }, message: "must be boolean", schema: Ub.type, parentSchema: Ub, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Be.errors = e, r === 0;
  }
  B["quarto-resource-document-render-keep-ipynb"] = je;
  var Kb = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-render-keep-ipynb" };
  function je(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Kb }, message: "must be boolean", schema: Kb.type, parentSchema: Kb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return je.errors = e, r === 0;
  }
  B["quarto-resource-document-render-keep-tex"] = Vl;
  var Jb = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-render-keep-tex" };
  function Vl(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Jb }, message: "must be boolean", schema: Jb.type, parentSchema: Jb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Vl.errors = e, r === 0;
  }
  B["quarto-resource-document-render-extract-media"] = qr;
  var Qb = { type: "string", description: "be a string", $id: "quarto-resource-document-render-extract-media" };
  function qr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Qb }, message: "must be string", schema: Qb.type, parentSchema: Qb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return qr.errors = e, r === 0;
  }
  B["quarto-resource-document-render-resource-path"] = Or;
  var Gb = { type: "string", description: "be a string", $id: "quarto-resource-document-render-resource-path" };
  function Or(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Gb }, message: "must be string", schema: Gb.type, parentSchema: Gb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Or.errors = e, r === 0;
  }
  B["quarto-resource-document-render-default-image-extension"] = Pr;
  var Yb = { type: "string", description: "be a string", $id: "quarto-resource-document-render-default-image-extension" };
  function Pr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Yb }, message: "must be string", schema: Yb.type, parentSchema: Yb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Pr.errors = e, r === 0;
  }
  B["quarto-resource-document-render-abbreviations"] = xr;
  var Xb = { type: "string", description: "be a string", $id: "quarto-resource-document-render-abbreviations" };
  function xr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: Xb }, message: "must be string", schema: Xb.type, parentSchema: Xb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return xr.errors = e, r === 0;
  }
  B["quarto-resource-document-render-dpi"] = kr;
  var Zb = { type: "number", description: "be a number", $id: "quarto-resource-document-render-dpi" };
  function kr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: Zb }, message: "must be number", schema: Zb.type, parentSchema: Zb, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return kr.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-content-logo"] = tp;
  var e_ = { type: "string", description: "be a string", $id: "quarto-resource-document-reveal-content-logo" };
  function tp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: e_ }, message: "must be string", schema: e_.type, parentSchema: e_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return tp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-content-footer"] = ap;
  var r_ = { type: "string", description: "be a string", $id: "quarto-resource-document-reveal-content-footer" };
  function ap(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: r_ }, message: "must be string", schema: r_.type, parentSchema: r_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ap.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-content-scrollable"] = np;
  var t_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-content-scrollable" };
  function np(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: t_ }, message: "must be boolean", schema: t_.type, parentSchema: t_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return np.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-content-smaller"] = op;
  var a_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-content-smaller" };
  function op(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: a_ }, message: "must be boolean", schema: a_.type, parentSchema: a_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return op.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-hidden-embedded"] = sp;
  var n_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-hidden-embedded" };
  function sp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: n_ }, message: "must be boolean", schema: n_.type, parentSchema: n_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return sp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-hidden-display"] = ip;
  var o_ = { type: "string", description: "be a string", $id: "quarto-resource-document-reveal-hidden-display" };
  function ip(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: o_ }, message: "must be string", schema: o_.type, parentSchema: o_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ip.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-layout-auto-stretch"] = lp;
  var s_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-layout-auto-stretch" };
  function lp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: s_ }, message: "must be boolean", schema: s_.type, parentSchema: s_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return lp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-layout-width"] = cp;
  var uc = { oneOf: [{ type: "number", description: "be a number" }, { type: "string", description: "be a string" }], description: "be exactly one of: a number, a string", $id: "quarto-resource-document-reveal-layout-width" };
  function cp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (!(typeof l == "number" && isFinite(l))) {
      let $ = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "number", schema: uc.oneOf[0] }, message: "must be number", schema: uc.oneOf[0].type, parentSchema: uc.oneOf[0], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r;
    if (typeof l != "string") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "string", schema: uc.oneOf[1] }, message: "must be string", schema: uc.oneOf[1].type, parentSchema: uc.oneOf[1], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let D = r;
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: uc }, message: "must match exactly one schema in oneOf", schema: uc.oneOf, parentSchema: uc, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return cp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-layout-height"] = pp;
  var dc = { oneOf: [{ type: "number", description: "be a number" }, { type: "string", description: "be a string" }], description: "be exactly one of: a number, a string", $id: "quarto-resource-document-reveal-layout-height" };
  function pp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (!(typeof l == "number" && isFinite(l))) {
      let $ = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "number", schema: dc.oneOf[0] }, message: "must be number", schema: dc.oneOf[0].type, parentSchema: dc.oneOf[0], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r;
    if (typeof l != "string") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "string", schema: dc.oneOf[1] }, message: "must be string", schema: dc.oneOf[1].type, parentSchema: dc.oneOf[1], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let D = r;
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: dc }, message: "must match exactly one schema in oneOf", schema: dc.oneOf, parentSchema: dc, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return pp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-layout-margin"] = up;
  var i_ = { type: "number", description: "be a number", $id: "quarto-resource-document-reveal-layout-margin" };
  function up(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: i_ }, message: "must be number", schema: i_.type, parentSchema: i_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return up.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-layout-min-scale"] = dp;
  var l_ = { type: "number", description: "be a number", $id: "quarto-resource-document-reveal-layout-min-scale" };
  function dp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: l_ }, message: "must be number", schema: l_.type, parentSchema: l_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return dp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-layout-max-scale"] = fp;
  var c_ = { type: "number", description: "be a number", $id: "quarto-resource-document-reveal-layout-max-scale" };
  function fp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: c_ }, message: "must be number", schema: c_.type, parentSchema: c_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return fp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-layout-center"] = mp;
  var p_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-layout-center" };
  function mp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: p_ }, message: "must be boolean", schema: p_.type, parentSchema: p_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return mp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-layout-disable-layout"] = gp;
  var u_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-layout-disable-layout" };
  function gp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: u_ }, message: "must be boolean", schema: u_.type, parentSchema: u_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return gp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-media-preview-links"] = hp;
  var rc = { anyOf: [{ enum: ["auto"], description: "be 'auto'" }, { type: "boolean", description: "be `true` or `false`" }], description: "be at least one of: 'auto', `true` or `false`", $id: "quarto-resource-document-reveal-media-preview-links" };
  function hp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = r, u = rc.anyOf[0].enum;
    if (l !== "auto") {
      let g = { instancePath: o, schemaPath: "#/anyOf/0/enum", keyword: "enum", params: { allowedValues: rc.anyOf[0].enum, schema: rc.anyOf[0] }, message: "must be equal to one of the allowed values", schema: rc.anyOf[0].enum, parentSchema: rc.anyOf[0], data: l };
      e === null ? e = [g] : e.push(g), r++;
    }
    var y = d === r;
    if (c = c || y, !c) {
      let g = r;
      if (typeof l != "boolean") {
        let $ = { instancePath: o, schemaPath: "#/anyOf/1/type", keyword: "type", params: { type: "boolean", schema: rc.anyOf[1] }, message: "must be boolean", schema: rc.anyOf[1].type, parentSchema: rc.anyOf[1], data: l };
        e === null ? e = [$] : e.push($), r++;
      }
      let x = r;
      var y = g === r;
      c = c || y;
    }
    if (c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let g = { instancePath: o, schemaPath: "#/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: rc.anyOf, parentSchema: rc, data: l };
      e === null ? e = [g] : e.push(g), r++;
    }
    return hp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-media-auto-play-media"] = yp;
  var sm = { enum: [null, true, false], description: "be one of: `null`, `true`, `false`", $id: "quarto-resource-document-reveal-media-auto-play-media" };
  function yp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = sm.enum;
    if (!(l === null || l === true || l === false)) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: sm.enum, schema: sm }, message: "must be equal to one of the allowed values", schema: sm.enum, parentSchema: sm, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return yp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-media-preload-iframes"] = bp;
  var im = { enum: [null, true, false], description: "be one of: `null`, `true`, `false`", $id: "quarto-resource-document-reveal-media-preload-iframes" };
  function bp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = im.enum;
    if (!(l === null || l === true || l === false)) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: im.enum, schema: im }, message: "must be equal to one of the allowed values", schema: im.enum, parentSchema: im, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return bp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-media-view-distance"] = _p;
  var d_ = { type: "number", description: "be a number", $id: "quarto-resource-document-reveal-media-view-distance" };
  function _p(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: d_ }, message: "must be number", schema: d_.type, parentSchema: d_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return _p.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-media-mobile-view-distance"] = vp;
  var f_ = { type: "number", description: "be a number", $id: "quarto-resource-document-reveal-media-mobile-view-distance" };
  function vp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: f_ }, message: "must be number", schema: f_.type, parentSchema: f_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return vp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-media-parallax-background-image"] = qp;
  var m_ = { type: "string", description: "be a string", $id: "quarto-resource-document-reveal-media-parallax-background-image" };
  function qp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: m_ }, message: "must be string", schema: m_.type, parentSchema: m_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return qp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-media-parallax-background-size"] = Op;
  var g_ = { type: "string", description: "be a string", $id: "quarto-resource-document-reveal-media-parallax-background-size" };
  function Op(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: g_ }, message: "must be string", schema: g_.type, parentSchema: g_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Op.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-media-parallax-background-horizontal"] = Pp;
  var h_ = { type: "number", description: "be a number", $id: "quarto-resource-document-reveal-media-parallax-background-horizontal" };
  function Pp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: h_ }, message: "must be number", schema: h_.type, parentSchema: h_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Pp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-media-parallax-background-vertical"] = xp;
  var y_ = { type: "number", description: "be a number", $id: "quarto-resource-document-reveal-media-parallax-background-vertical" };
  function xp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: y_ }, message: "must be number", schema: y_.type, parentSchema: y_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return xp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-progress"] = kp;
  var b_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-progress" };
  function kp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: b_ }, message: "must be boolean", schema: b_.type, parentSchema: b_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return kp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-history"] = Dp;
  var __ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-history" };
  function Dp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: __ }, message: "must be boolean", schema: __.type, parentSchema: __, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Dp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-navigation-mode"] = wp;
  var lm = { enum: ["linear", "vertical", "grid"], description: "be one of: `linear`, `vertical`, `grid`", $id: "quarto-resource-document-reveal-navigation-navigation-mode" };
  function wp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = lm.enum;
    if (!(l === "linear" || l === "vertical" || l === "grid")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: lm.enum, schema: lm }, message: "must be equal to one of the allowed values", schema: lm.enum, parentSchema: lm, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return wp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-touch"] = $p;
  var v_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-touch" };
  function $p(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: v_ }, message: "must be boolean", schema: v_.type, parentSchema: v_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return $p.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-keyboard"] = Sp;
  var q_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-keyboard" };
  function Sp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: q_ }, message: "must be boolean", schema: q_.type, parentSchema: q_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Sp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-mouse-wheel"] = Fp;
  var O_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-mouse-wheel" };
  function Fp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: O_ }, message: "must be boolean", schema: O_.type, parentSchema: O_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Fp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-hide-inactive-cursor"] = Ap;
  var P_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-hide-inactive-cursor" };
  function Ap(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: P_ }, message: "must be boolean", schema: P_.type, parentSchema: P_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ap.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-hide-cursor-time"] = Bp;
  var x_ = { type: "number", description: "be a number", $id: "quarto-resource-document-reveal-navigation-hide-cursor-time" };
  function Bp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: x_ }, message: "must be number", schema: x_.type, parentSchema: x_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Bp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-loop"] = jp;
  var k_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-loop" };
  function jp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: k_ }, message: "must be boolean", schema: k_.type, parentSchema: k_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return jp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-shuffle"] = Ep;
  var D_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-shuffle" };
  function Ep(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: D_ }, message: "must be boolean", schema: D_.type, parentSchema: D_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ep.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-controls"] = Np;
  var tc = { anyOf: [{ type: "boolean", description: "be `true` or `false`" }, { enum: ["auto"], description: "be 'auto'" }], description: "be at least one of: `true` or `false`, 'auto'", $id: "quarto-resource-document-reveal-navigation-controls" };
  function Np(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = r;
    if (typeof l != "boolean") {
      let g = { instancePath: o, schemaPath: "#/anyOf/0/type", keyword: "type", params: { type: "boolean", schema: tc.anyOf[0] }, message: "must be boolean", schema: tc.anyOf[0].type, parentSchema: tc.anyOf[0], data: l };
      e === null ? e = [g] : e.push(g), r++;
    }
    let u = r;
    var y = d === r;
    if (c = c || y, !c) {
      let g = r, x = tc.anyOf[1].enum;
      if (l !== "auto") {
        let $ = { instancePath: o, schemaPath: "#/anyOf/1/enum", keyword: "enum", params: { allowedValues: tc.anyOf[1].enum, schema: tc.anyOf[1] }, message: "must be equal to one of the allowed values", schema: tc.anyOf[1].enum, parentSchema: tc.anyOf[1], data: l };
        e === null ? e = [$] : e.push($), r++;
      }
      var y = g === r;
      c = c || y;
    }
    if (c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let g = { instancePath: o, schemaPath: "#/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: tc.anyOf, parentSchema: tc, data: l };
      e === null ? e = [g] : e.push(g), r++;
    }
    return Np.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-controls-layout"] = Cp;
  var cm = { enum: ["edges", "bottom-right"], description: "be one of: `edges`, `bottom-right`", $id: "quarto-resource-document-reveal-navigation-controls-layout" };
  function Cp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = cm.enum;
    if (!(l === "edges" || l === "bottom-right")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: cm.enum, schema: cm }, message: "must be equal to one of the allowed values", schema: cm.enum, parentSchema: cm, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Cp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-controls-tutorial"] = Ip;
  var w_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-controls-tutorial" };
  function Ip(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: w_ }, message: "must be boolean", schema: w_.type, parentSchema: w_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ip.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-controls-back-arrows"] = Hp;
  var pm = { enum: ["faded", "hidden", "visible"], description: "be one of: `faded`, `hidden`, `visible`", $id: "quarto-resource-document-reveal-navigation-controls-back-arrows" };
  function Hp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = pm.enum;
    if (!(l === "faded" || l === "hidden" || l === "visible")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: pm.enum, schema: pm }, message: "must be equal to one of the allowed values", schema: pm.enum, parentSchema: pm, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Hp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-auto-slide"] = Lp;
  var Ul = { oneOf: [{ type: "number", description: "be a number" }, { enum: [false], description: "be false" }], description: "be exactly one of: a number, false", $id: "quarto-resource-document-reveal-navigation-auto-slide" };
  function Lp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (!(typeof l == "number" && isFinite(l))) {
      let $ = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "number", schema: Ul.oneOf[0] }, message: "must be number", schema: Ul.oneOf[0].type, parentSchema: Ul.oneOf[0], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = Ul.oneOf[1].enum;
    if (l !== false) {
      let $ = { instancePath: o, schemaPath: "#/oneOf/1/enum", keyword: "enum", params: { allowedValues: Ul.oneOf[1].enum, schema: Ul.oneOf[1] }, message: "must be equal to one of the allowed values", schema: Ul.oneOf[1].enum, parentSchema: Ul.oneOf[1], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Ul }, message: "must match exactly one schema in oneOf", schema: Ul.oneOf, parentSchema: Ul, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return Lp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-auto-slide-stoppable"] = Mp;
  var $_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-auto-slide-stoppable" };
  function Mp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: $_ }, message: "must be boolean", schema: $_.type, parentSchema: $_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Mp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-auto-slide-method"] = Tp;
  var S_ = { type: "string", description: "be a string", $id: "quarto-resource-document-reveal-navigation-auto-slide-method" };
  function Tp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: S_ }, message: "must be string", schema: S_.type, parentSchema: S_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Tp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-default-timing"] = zp;
  var F_ = { type: "number", description: "be a number", $id: "quarto-resource-document-reveal-navigation-default-timing" };
  function zp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: F_ }, message: "must be number", schema: F_.type, parentSchema: F_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return zp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-pause"] = Wp;
  var A_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-pause" };
  function Wp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: A_ }, message: "must be boolean", schema: A_.type, parentSchema: A_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Wp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-help"] = Rp;
  var B_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-help" };
  function Rp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: B_ }, message: "must be boolean", schema: B_.type, parentSchema: B_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Rp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-hash"] = Vp;
  var j_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-hash" };
  function Vp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: j_ }, message: "must be boolean", schema: j_.type, parentSchema: j_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Vp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-hash-type"] = Up;
  var um = { enum: ["number", "title"], description: "be one of: `number`, `title`", $id: "quarto-resource-document-reveal-navigation-hash-type" };
  function Up(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = um.enum;
    if (!(l === "number" || l === "title")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: um.enum, schema: um }, message: "must be equal to one of the allowed values", schema: um.enum, parentSchema: um, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Up.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-hash-one-based-index"] = Kp;
  var E_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-hash-one-based-index" };
  function Kp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: E_ }, message: "must be boolean", schema: E_.type, parentSchema: E_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Kp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-respond-to-hash-changes"] = Jp;
  var N_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-respond-to-hash-changes" };
  function Jp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: N_ }, message: "must be boolean", schema: N_.type, parentSchema: N_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Jp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-fragment-in-url"] = Qp;
  var C_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-fragment-in-url" };
  function Qp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: C_ }, message: "must be boolean", schema: C_.type, parentSchema: C_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Qp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-navigation-slide-tone"] = Gp;
  var I_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-navigation-slide-tone" };
  function Gp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: I_ }, message: "must be boolean", schema: I_.type, parentSchema: I_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Gp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-print-pdf-separate-fragments"] = Yp;
  var H_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-print-pdf-separate-fragments" };
  function Yp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: H_ }, message: "must be boolean", schema: H_.type, parentSchema: H_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Yp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-print-pdf-page-height-offset"] = Xp;
  var L_ = { type: "number", description: "be a number", $id: "quarto-resource-document-reveal-print-pdf-page-height-offset" };
  function Xp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: L_ }, message: "must be number", schema: L_.type, parentSchema: L_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Xp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-tools-overview"] = Zp;
  var M_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-tools-overview" };
  function Zp(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: M_ }, message: "must be boolean", schema: M_.type, parentSchema: M_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Zp.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-tools-menu"] = eu;
  var vn = { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { side: { enum: ["left", "right"], description: "be one of: `left`, `right`" }, width: { type: "string", description: "be a string" }, numbers: { type: "boolean", description: "be `true` or `false`" }, "use-text-content-for-missing-titles": { type: "boolean", description: "be `true` or `false`" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(use_text_content_for_missing_titles|useTextContentForMissingTitles))" } }], description: "be exactly one of: `true` or `false`, an object", $id: "quarto-resource-document-reveal-tools-menu" }, A0 = new RegExp("^(?!(use_text_content_for_missing_titles|useTextContentForMissingTitles))", "u");
  function eu(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "boolean") {
      let R = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: vn.oneOf[0] }, message: "must be boolean", schema: vn.oneOf[0].type, parentSchema: vn.oneOf[0], data: l };
      e === null ? e = [R] : e.push(R), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let R in l) {
        let j = r, V = r;
        if (typeof R == "string") {
          if (!A0.test(R)) {
            let se = { instancePath: o, schemaPath: "#/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(use_text_content_for_missing_titles|useTextContentForMissingTitles))", schema: vn.oneOf[1].propertyNames }, message: 'must match pattern "^(?!(use_text_content_for_missing_titles|useTextContentForMissingTitles))"', schema: "^(?!(use_text_content_for_missing_titles|useTextContentForMissingTitles))", parentSchema: vn.oneOf[1].propertyNames, data: R, propertyName: R };
            e === null ? e = [se] : e.push(se), r++;
          }
        } else {
          let se = { instancePath: o, schemaPath: "#/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: vn.oneOf[1].propertyNames }, message: "must be string", schema: vn.oneOf[1].propertyNames.type, parentSchema: vn.oneOf[1].propertyNames, data: R, propertyName: R };
          e === null ? e = [se] : e.push(se), r++;
        }
        var $ = j === r;
        if (!$) {
          let se = { instancePath: o, schemaPath: "#/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: R, schema: vn.oneOf[1] }, message: "property name must be valid", schema: vn.oneOf[1].propertyNames, parentSchema: vn.oneOf[1], data: l };
          e === null ? e = [se] : e.push(se), r++;
        }
      }
      if (l.side !== void 0) {
        let R = l.side, j = r, V = vn.oneOf[1].properties.side.enum;
        if (!(R === "left" || R === "right")) {
          let se = { instancePath: o + "/side", schemaPath: "#/oneOf/1/properties/side/enum", keyword: "enum", params: { allowedValues: vn.oneOf[1].properties.side.enum, schema: vn.oneOf[1].properties.side }, message: "must be equal to one of the allowed values", schema: vn.oneOf[1].properties.side.enum, parentSchema: vn.oneOf[1].properties.side, data: R };
          e === null ? e = [se] : e.push(se), r++;
        }
        var m = j === r;
      }
      if (l.width !== void 0) {
        let R = l.width, j = r;
        if (typeof R != "string") {
          let P = { instancePath: o + "/width", schemaPath: "#/oneOf/1/properties/width/type", keyword: "type", params: { type: "string", schema: vn.oneOf[1].properties.width }, message: "must be string", schema: vn.oneOf[1].properties.width.type, parentSchema: vn.oneOf[1].properties.width, data: R };
          e === null ? e = [P] : e.push(P), r++;
        }
        let V = r;
        var m = j === r;
      }
      if (l.numbers !== void 0) {
        let R = l.numbers, j = r;
        if (typeof R != "boolean") {
          let P = { instancePath: o + "/numbers", schemaPath: "#/oneOf/1/properties/numbers/type", keyword: "type", params: { type: "boolean", schema: vn.oneOf[1].properties.numbers }, message: "must be boolean", schema: vn.oneOf[1].properties.numbers.type, parentSchema: vn.oneOf[1].properties.numbers, data: R };
          e === null ? e = [P] : e.push(P), r++;
        }
        let V = r;
        var m = j === r;
      }
      if (l["use-text-content-for-missing-titles"] !== void 0) {
        let R = l["use-text-content-for-missing-titles"], j = r;
        if (typeof R != "boolean") {
          let P = { instancePath: o + "/use-text-content-for-missing-titles", schemaPath: "#/oneOf/1/properties/use-text-content-for-missing-titles/type", keyword: "type", params: { type: "boolean", schema: vn.oneOf[1].properties["use-text-content-for-missing-titles"] }, message: "must be boolean", schema: vn.oneOf[1].properties["use-text-content-for-missing-titles"].type, parentSchema: vn.oneOf[1].properties["use-text-content-for-missing-titles"], data: R };
          e === null ? e = [P] : e.push(P), r++;
        }
        let V = r;
        var m = j === r;
      }
    } else {
      let R = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "object", schema: vn.oneOf[1] }, message: "must be object", schema: vn.oneOf[1].type, parentSchema: vn.oneOf[1], data: l };
      e === null ? e = [R] : e.push(R), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let R = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: vn }, message: "must match exactly one schema in oneOf", schema: vn.oneOf, parentSchema: vn, data: l };
      e === null ? e = [R] : e.push(R), r++;
    }
    return eu.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-tools-chalkboard"] = ru;
  var tn = { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { theme: { enum: ["chalkboard", "whiteboard"], description: "be one of: `chalkboard`, `whiteboard`" }, "boardmarker-width": { type: "number", description: "be a number" }, "chalk-width": { type: "number", description: "be a number" }, src: { type: "string", description: "be a string" }, "read-only": { type: "boolean", description: "be `true` or `false`" }, buttons: { type: "boolean", description: "be `true` or `false`" }, transition: { type: "number", description: "be a number" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(boardmarker_width|boardmarkerWidth|chalk_width|chalkWidth|read_only|readOnly))" } }], description: "be exactly one of: `true` or `false`, an object", $id: "quarto-resource-document-reveal-tools-chalkboard" }, B0 = new RegExp("^(?!(boardmarker_width|boardmarkerWidth|chalk_width|chalkWidth|read_only|readOnly))", "u");
  function ru(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "boolean") {
      let R = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: tn.oneOf[0] }, message: "must be boolean", schema: tn.oneOf[0].type, parentSchema: tn.oneOf[0], data: l };
      e === null ? e = [R] : e.push(R), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let R in l) {
        let j = r, V = r;
        if (typeof R == "string") {
          if (!B0.test(R)) {
            let se = { instancePath: o, schemaPath: "#/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(boardmarker_width|boardmarkerWidth|chalk_width|chalkWidth|read_only|readOnly))", schema: tn.oneOf[1].propertyNames }, message: 'must match pattern "^(?!(boardmarker_width|boardmarkerWidth|chalk_width|chalkWidth|read_only|readOnly))"', schema: "^(?!(boardmarker_width|boardmarkerWidth|chalk_width|chalkWidth|read_only|readOnly))", parentSchema: tn.oneOf[1].propertyNames, data: R, propertyName: R };
            e === null ? e = [se] : e.push(se), r++;
          }
        } else {
          let se = { instancePath: o, schemaPath: "#/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: tn.oneOf[1].propertyNames }, message: "must be string", schema: tn.oneOf[1].propertyNames.type, parentSchema: tn.oneOf[1].propertyNames, data: R, propertyName: R };
          e === null ? e = [se] : e.push(se), r++;
        }
        var $ = j === r;
        if (!$) {
          let se = { instancePath: o, schemaPath: "#/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: R, schema: tn.oneOf[1] }, message: "property name must be valid", schema: tn.oneOf[1].propertyNames, parentSchema: tn.oneOf[1], data: l };
          e === null ? e = [se] : e.push(se), r++;
        }
      }
      if (l.theme !== void 0) {
        let R = l.theme, j = r, V = tn.oneOf[1].properties.theme.enum;
        if (!(R === "chalkboard" || R === "whiteboard")) {
          let se = { instancePath: o + "/theme", schemaPath: "#/oneOf/1/properties/theme/enum", keyword: "enum", params: { allowedValues: tn.oneOf[1].properties.theme.enum, schema: tn.oneOf[1].properties.theme }, message: "must be equal to one of the allowed values", schema: tn.oneOf[1].properties.theme.enum, parentSchema: tn.oneOf[1].properties.theme, data: R };
          e === null ? e = [se] : e.push(se), r++;
        }
        var m = j === r;
      }
      if (l["boardmarker-width"] !== void 0) {
        let R = l["boardmarker-width"], j = r;
        if (!(typeof R == "number" && isFinite(R))) {
          let P = { instancePath: o + "/boardmarker-width", schemaPath: "#/oneOf/1/properties/boardmarker-width/type", keyword: "type", params: { type: "number", schema: tn.oneOf[1].properties["boardmarker-width"] }, message: "must be number", schema: tn.oneOf[1].properties["boardmarker-width"].type, parentSchema: tn.oneOf[1].properties["boardmarker-width"], data: R };
          e === null ? e = [P] : e.push(P), r++;
        }
        let V = r;
        var m = j === r;
      }
      if (l["chalk-width"] !== void 0) {
        let R = l["chalk-width"], j = r;
        if (!(typeof R == "number" && isFinite(R))) {
          let P = { instancePath: o + "/chalk-width", schemaPath: "#/oneOf/1/properties/chalk-width/type", keyword: "type", params: { type: "number", schema: tn.oneOf[1].properties["chalk-width"] }, message: "must be number", schema: tn.oneOf[1].properties["chalk-width"].type, parentSchema: tn.oneOf[1].properties["chalk-width"], data: R };
          e === null ? e = [P] : e.push(P), r++;
        }
        let V = r;
        var m = j === r;
      }
      if (l.src !== void 0) {
        let R = l.src, j = r;
        if (typeof R != "string") {
          let P = { instancePath: o + "/src", schemaPath: "#/oneOf/1/properties/src/type", keyword: "type", params: { type: "string", schema: tn.oneOf[1].properties.src }, message: "must be string", schema: tn.oneOf[1].properties.src.type, parentSchema: tn.oneOf[1].properties.src, data: R };
          e === null ? e = [P] : e.push(P), r++;
        }
        let V = r;
        var m = j === r;
      }
      if (l["read-only"] !== void 0) {
        let R = l["read-only"], j = r;
        if (typeof R != "boolean") {
          let P = { instancePath: o + "/read-only", schemaPath: "#/oneOf/1/properties/read-only/type", keyword: "type", params: { type: "boolean", schema: tn.oneOf[1].properties["read-only"] }, message: "must be boolean", schema: tn.oneOf[1].properties["read-only"].type, parentSchema: tn.oneOf[1].properties["read-only"], data: R };
          e === null ? e = [P] : e.push(P), r++;
        }
        let V = r;
        var m = j === r;
      }
      if (l.buttons !== void 0) {
        let R = l.buttons, j = r;
        if (typeof R != "boolean") {
          let P = { instancePath: o + "/buttons", schemaPath: "#/oneOf/1/properties/buttons/type", keyword: "type", params: { type: "boolean", schema: tn.oneOf[1].properties.buttons }, message: "must be boolean", schema: tn.oneOf[1].properties.buttons.type, parentSchema: tn.oneOf[1].properties.buttons, data: R };
          e === null ? e = [P] : e.push(P), r++;
        }
        let V = r;
        var m = j === r;
      }
      if (l.transition !== void 0) {
        let R = l.transition, j = r;
        if (!(typeof R == "number" && isFinite(R))) {
          let P = { instancePath: o + "/transition", schemaPath: "#/oneOf/1/properties/transition/type", keyword: "type", params: { type: "number", schema: tn.oneOf[1].properties.transition }, message: "must be number", schema: tn.oneOf[1].properties.transition.type, parentSchema: tn.oneOf[1].properties.transition, data: R };
          e === null ? e = [P] : e.push(P), r++;
        }
        let V = r;
        var m = j === r;
      }
    } else {
      let R = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "object", schema: tn.oneOf[1] }, message: "must be object", schema: tn.oneOf[1].type, parentSchema: tn.oneOf[1], data: l };
      e === null ? e = [R] : e.push(R), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let R = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: tn }, message: "must match exactly one schema in oneOf", schema: tn.oneOf, parentSchema: tn, data: l };
      e === null ? e = [R] : e.push(R), r++;
    }
    return ru.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-tools-multiplex"] = tu;
  var Po = { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { url: { type: "string", description: "be a string" }, id: { type: "string", description: "be a string" }, secret: { type: "string", description: "be a string" } }, patternProperties: {} }], description: "be exactly one of: `true` or `false`, an object", $id: "quarto-resource-document-reveal-tools-multiplex" };
  function tu(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "boolean") {
      let m = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: Po.oneOf[0] }, message: "must be boolean", schema: Po.oneOf[0].type, parentSchema: Po.oneOf[0], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      if (l.url !== void 0) {
        let m = l.url, R = r;
        if (typeof m != "string") {
          let V = { instancePath: o + "/url", schemaPath: "#/oneOf/1/properties/url/type", keyword: "type", params: { type: "string", schema: Po.oneOf[1].properties.url }, message: "must be string", schema: Po.oneOf[1].properties.url.type, parentSchema: Po.oneOf[1].properties.url, data: m };
          e === null ? e = [V] : e.push(V), r++;
        }
        let j = r;
        var $ = R === r;
      }
      if (l.id !== void 0) {
        let m = l.id, R = r;
        if (typeof m != "string") {
          let se = { instancePath: o + "/id", schemaPath: "#/oneOf/1/properties/id/type", keyword: "type", params: { type: "string", schema: Po.oneOf[1].properties.id }, message: "must be string", schema: Po.oneOf[1].properties.id.type, parentSchema: Po.oneOf[1].properties.id, data: m };
          e === null ? e = [se] : e.push(se), r++;
        }
        let j = r;
        var $ = R === r;
      }
      if (l.secret !== void 0) {
        let m = l.secret, R = r;
        if (typeof m != "string") {
          let se = { instancePath: o + "/secret", schemaPath: "#/oneOf/1/properties/secret/type", keyword: "type", params: { type: "string", schema: Po.oneOf[1].properties.secret }, message: "must be string", schema: Po.oneOf[1].properties.secret.type, parentSchema: Po.oneOf[1].properties.secret, data: m };
          e === null ? e = [se] : e.push(se), r++;
        }
        let j = r;
        var $ = R === r;
      }
    } else {
      let m = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "object", schema: Po.oneOf[1] }, message: "must be object", schema: Po.oneOf[1].type, parentSchema: Po.oneOf[1], data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let m = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Po }, message: "must match exactly one schema in oneOf", schema: Po.oneOf, parentSchema: Po, data: l };
      e === null ? e = [m] : e.push(m), r++;
    }
    return tu.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-transitions-transition"] = au;
  var dm = { enum: ["none", "fade", "slide", "convex", "concave", "zoom"], description: "be one of: `none`, `fade`, `slide`, `convex`, `concave`, `zoom`", $id: "quarto-resource-document-reveal-transitions-transition" };
  function au(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = dm.enum;
    if (!(l === "none" || l === "fade" || l === "slide" || l === "convex" || l === "concave" || l === "zoom")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: dm.enum, schema: dm }, message: "must be equal to one of the allowed values", schema: dm.enum, parentSchema: dm, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return au.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-transitions-transition-speed"] = nu;
  var fm = { enum: ["default", "fast", "slow"], description: "be one of: `default`, `fast`, `slow`", $id: "quarto-resource-document-reveal-transitions-transition-speed" };
  function nu(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = fm.enum;
    if (!(l === "default" || l === "fast" || l === "slow")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: fm.enum, schema: fm }, message: "must be equal to one of the allowed values", schema: fm.enum, parentSchema: fm, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return nu.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-transitions-background-transition"] = ou;
  var mm = { enum: ["none", "fade", "slide", "convex", "concave", "zoom"], description: "be one of: `none`, `fade`, `slide`, `convex`, `concave`, `zoom`", $id: "quarto-resource-document-reveal-transitions-background-transition" };
  function ou(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = mm.enum;
    if (!(l === "none" || l === "fade" || l === "slide" || l === "convex" || l === "concave" || l === "zoom")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: mm.enum, schema: mm }, message: "must be equal to one of the allowed values", schema: mm.enum, parentSchema: mm, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return ou.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-transitions-fragments"] = su;
  var T_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-transitions-fragments" };
  function su(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: T_ }, message: "must be boolean", schema: T_.type, parentSchema: T_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return su.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-transitions-auto-animate"] = iu;
  var z_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-transitions-auto-animate" };
  function iu(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: z_ }, message: "must be boolean", schema: z_.type, parentSchema: z_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return iu.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-transitions-auto-animate-easing"] = lu;
  var W_ = { type: "string", description: "be a string", $id: "quarto-resource-document-reveal-transitions-auto-animate-easing" };
  function lu(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: W_ }, message: "must be string", schema: W_.type, parentSchema: W_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return lu.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-transitions-auto-animate-duration"] = cu;
  var R_ = { type: "number", description: "be a number", $id: "quarto-resource-document-reveal-transitions-auto-animate-duration" };
  function cu(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: R_ }, message: "must be number", schema: R_.type, parentSchema: R_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return cu.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-transitions-auto-animate-unmatched"] = pu;
  var V_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-reveal-transitions-auto-animate-unmatched" };
  function pu(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: V_ }, message: "must be boolean", schema: V_.type, parentSchema: V_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return pu.errors = e, r === 0;
  }
  B["quarto-resource-document-reveal-transitions-auto-animate-styles"] = du;
  var uu = { type: "array", description: "be an array of values, where each element should be one of: `opacity`, `color`, `background-color`, `padding`, `font-size`, `line-height`, `letter-spacing`, `border-width`, `border-color`, `border-radius`, `outline`, `outline-offset`", items: { enum: ["opacity", "color", "background-color", "padding", "font-size", "line-height", "letter-spacing", "border-width", "border-color", "border-radius", "outline", "outline-offset"], description: "be one of: `opacity`, `color`, `background-color`, `padding`, `font-size`, `line-height`, `letter-spacing`, `border-width`, `border-color`, `border-radius`, `outline`, `outline-offset`" }, $id: "quarto-resource-document-reveal-transitions-auto-animate-styles" };
  function du(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let d = true, u = l.length;
      for (let y = 0; y < u; y++) {
        let g = l[y], x = r, D = uu.items.enum;
        if (!(g === "opacity" || g === "color" || g === "background-color" || g === "padding" || g === "font-size" || g === "line-height" || g === "letter-spacing" || g === "border-width" || g === "border-color" || g === "border-radius" || g === "outline" || g === "outline-offset")) {
          let $ = { instancePath: o + "/" + y, schemaPath: "#/items/enum", keyword: "enum", params: { allowedValues: uu.items.enum, schema: uu.items }, message: "must be equal to one of the allowed values", schema: uu.items.enum, parentSchema: uu.items, data: g };
          e === null ? e = [$] : e.push($), r++;
        }
        var c = x === r;
        c || (d = false);
      }
    } else {
      let d = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: uu }, message: "must be array", schema: uu.type, parentSchema: uu, data: l };
      e === null ? e = [d] : e.push(d), r++;
    }
    return du.errors = e, r === 0;
  }
  B["quarto-resource-document-slides-incremental"] = Rn;
  var U_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-slides-incremental" };
  function Rn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: U_ }, message: "must be boolean", schema: U_.type, parentSchema: U_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Rn.errors = e, r === 0;
  }
  B["quarto-resource-document-slides-slide-level"] = Vn;
  var K_ = { type: "number", description: "be a number", $id: "quarto-resource-document-slides-slide-level" };
  function Vn(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: K_ }, message: "must be number", schema: K_.type, parentSchema: K_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Vn.errors = e, r === 0;
  }
  B["quarto-resource-document-slides-slide-number"] = fu;
  var Kl = { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { enum: ["h.v", "h/v", "c", "c/t"], description: "be one of: `h.v`, `h/v`, `c`, `c/t`" }], description: "be exactly one of: `true` or `false`, one of: `h.v`, `h/v`, `c`, `c/t`", $id: "quarto-resource-document-slides-slide-number" };
  function fu(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (typeof l != "boolean") {
      let $ = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: Kl.oneOf[0] }, message: "must be boolean", schema: Kl.oneOf[0].type, parentSchema: Kl.oneOf[0], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = Kl.oneOf[1].enum;
    if (!(l === "h.v" || l === "h/v" || l === "c" || l === "c/t")) {
      let $ = { instancePath: o, schemaPath: "#/oneOf/1/enum", keyword: "enum", params: { allowedValues: Kl.oneOf[1].enum, schema: Kl.oneOf[1] }, message: "must be equal to one of the allowed values", schema: Kl.oneOf[1].enum, parentSchema: Kl.oneOf[1], data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    var g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let $ = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: Kl }, message: "must match exactly one schema in oneOf", schema: Kl.oneOf, parentSchema: Kl, data: l };
      e === null ? e = [$] : e.push($), r++;
    }
    return fu.errors = e, r === 0;
  }
  B["quarto-resource-document-slides-show-slide-number"] = mu;
  var gm = { enum: ["all", "print", "speaker"], description: "be one of: `all`, `print`, `speaker`", $id: "quarto-resource-document-slides-show-slide-number" };
  function mu(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = gm.enum;
    if (!(l === "all" || l === "print" || l === "speaker")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: gm.enum, schema: gm }, message: "must be equal to one of the allowed values", schema: gm.enum, parentSchema: gm, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return mu.errors = e, r === 0;
  }
  B["quarto-resource-document-slides-title-slide-attributes"] = gu;
  var Un = { type: "object", description: "be an object", properties: { "data-background-image": { type: "string", description: "be a string" }, "data-background-size": { type: "string", description: "be a string" }, "data-background-position": { type: "string", description: "be a string" }, "data-background-repeat": { type: "string", description: "be a string" }, "data-background-opacity": { type: "string", description: "be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(data_background_image|dataBackgroundImage|data_background_size|dataBackgroundSize|data_background_position|dataBackgroundPosition|data_background_repeat|dataBackgroundRepeat|data_background_opacity|dataBackgroundOpacity))" }, $id: "quarto-resource-document-slides-title-slide-attributes" }, j0 = new RegExp("^(?!(data_background_image|dataBackgroundImage|data_background_size|dataBackgroundSize|data_background_position|dataBackgroundPosition|data_background_repeat|dataBackgroundRepeat|data_background_opacity|dataBackgroundOpacity))", "u");
  function gu(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let u in l) {
        let y = r, g = r;
        if (typeof u == "string") {
          if (!j0.test(u)) {
            let x = { instancePath: o, schemaPath: "#/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(data_background_image|dataBackgroundImage|data_background_size|dataBackgroundSize|data_background_position|dataBackgroundPosition|data_background_repeat|dataBackgroundRepeat|data_background_opacity|dataBackgroundOpacity))", schema: Un.propertyNames }, message: 'must match pattern "^(?!(data_background_image|dataBackgroundImage|data_background_size|dataBackgroundSize|data_background_position|dataBackgroundPosition|data_background_repeat|dataBackgroundRepeat|data_background_opacity|dataBackgroundOpacity))"', schema: "^(?!(data_background_image|dataBackgroundImage|data_background_size|dataBackgroundSize|data_background_position|dataBackgroundPosition|data_background_repeat|dataBackgroundRepeat|data_background_opacity|dataBackgroundOpacity))", parentSchema: Un.propertyNames, data: u, propertyName: u };
            e === null ? e = [x] : e.push(x), r++;
          }
        } else {
          let x = { instancePath: o, schemaPath: "#/propertyNames/type", keyword: "type", params: { type: "string", schema: Un.propertyNames }, message: "must be string", schema: Un.propertyNames.type, parentSchema: Un.propertyNames, data: u, propertyName: u };
          e === null ? e = [x] : e.push(x), r++;
        }
        var c = y === r;
        if (!c) {
          let x = { instancePath: o, schemaPath: "#/propertyNames", keyword: "propertyNames", params: { propertyName: u, schema: Un }, message: "property name must be valid", schema: Un.propertyNames, parentSchema: Un, data: l };
          e === null ? e = [x] : e.push(x), r++;
        }
      }
      if (l["data-background-image"] !== void 0) {
        let u = l["data-background-image"], y = r;
        if (typeof u != "string") {
          let x = { instancePath: o + "/data-background-image", schemaPath: "#/properties/data-background-image/type", keyword: "type", params: { type: "string", schema: Un.properties["data-background-image"] }, message: "must be string", schema: Un.properties["data-background-image"].type, parentSchema: Un.properties["data-background-image"], data: u };
          e === null ? e = [x] : e.push(x), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["data-background-size"] !== void 0) {
        let u = l["data-background-size"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/data-background-size", schemaPath: "#/properties/data-background-size/type", keyword: "type", params: { type: "string", schema: Un.properties["data-background-size"] }, message: "must be string", schema: Un.properties["data-background-size"].type, parentSchema: Un.properties["data-background-size"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["data-background-position"] !== void 0) {
        let u = l["data-background-position"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/data-background-position", schemaPath: "#/properties/data-background-position/type", keyword: "type", params: { type: "string", schema: Un.properties["data-background-position"] }, message: "must be string", schema: Un.properties["data-background-position"].type, parentSchema: Un.properties["data-background-position"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["data-background-repeat"] !== void 0) {
        let u = l["data-background-repeat"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/data-background-repeat", schemaPath: "#/properties/data-background-repeat/type", keyword: "type", params: { type: "string", schema: Un.properties["data-background-repeat"] }, message: "must be string", schema: Un.properties["data-background-repeat"].type, parentSchema: Un.properties["data-background-repeat"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
      if (l["data-background-opacity"] !== void 0) {
        let u = l["data-background-opacity"], y = r;
        if (typeof u != "string") {
          let D = { instancePath: o + "/data-background-opacity", schemaPath: "#/properties/data-background-opacity/type", keyword: "type", params: { type: "string", schema: Un.properties["data-background-opacity"] }, message: "must be string", schema: Un.properties["data-background-opacity"].type, parentSchema: Un.properties["data-background-opacity"], data: u };
          e === null ? e = [D] : e.push(D), r++;
        }
        let g = r;
        var d = y === r;
      }
    } else {
      let u = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: Un }, message: "must be object", schema: Un.type, parentSchema: Un, data: l };
      e === null ? e = [u] : e.push(u), r++;
    }
    return gu.errors = e, r === 0;
  }
  B["quarto-resource-document-slides-center-title-slide"] = hu;
  var J_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-slides-center-title-slide" };
  function hu(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: J_ }, message: "must be boolean", schema: J_.type, parentSchema: J_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return hu.errors = e, r === 0;
  }
  B["quarto-resource-document-slides-show-notes"] = yu;
  var Q_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-slides-show-notes" };
  function yu(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Q_ }, message: "must be boolean", schema: Q_.type, parentSchema: Q_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return yu.errors = e, r === 0;
  }
  B["quarto-resource-document-slides-rtl"] = bu;
  var G_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-slides-rtl" };
  function bu(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: G_ }, message: "must be boolean", schema: G_.type, parentSchema: G_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return bu.errors = e, r === 0;
  }
  B["quarto-resource-document-text-wrap"] = it;
  var hm = { enum: ["auto", "none", "preserve"], description: "be one of: `auto`, `none`, `preserve`", $id: "quarto-resource-document-text-wrap" };
  function it(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = hm.enum;
    if (!(l === "auto" || l === "none" || l === "preserve")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: hm.enum, schema: hm }, message: "must be equal to one of the allowed values", schema: hm.enum, parentSchema: hm, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return it.errors = e, r === 0;
  }
  B["quarto-resource-document-text-columns"] = lt;
  var Y_ = { type: "number", description: "be a number", $id: "quarto-resource-document-text-columns" };
  function lt(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: Y_ }, message: "must be number", schema: Y_.type, parentSchema: Y_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return lt.errors = e, r === 0;
  }
  B["quarto-resource-document-text-tab-stop"] = ct;
  var X_ = { type: "number", description: "be a number", $id: "quarto-resource-document-text-tab-stop" };
  function ct(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: X_ }, message: "must be number", schema: X_.type, parentSchema: X_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ct.errors = e, r === 0;
  }
  B["quarto-resource-document-text-preserve-tabs"] = pt;
  var Z_ = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-text-preserve-tabs" };
  function pt(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: Z_ }, message: "must be boolean", schema: Z_.type, parentSchema: Z_, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return pt.errors = e, r === 0;
  }
  B["quarto-resource-document-text-eol"] = ut;
  var ym = { enum: ["lf", "crlf", "native"], description: "be one of: `lf`, `crlf`, `native`", $id: "quarto-resource-document-text-eol" };
  function ut(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = ym.enum;
    if (!(l === "lf" || l === "crlf" || l === "native")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: ym.enum, schema: ym }, message: "must be equal to one of the allowed values", schema: ym.enum, parentSchema: ym, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return ut.errors = e, r === 0;
  }
  B["quarto-resource-document-text-strip-comments"] = Ra;
  var ev = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-text-strip-comments" };
  function Ra(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: ev }, message: "must be boolean", schema: ev.type, parentSchema: ev, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Ra.errors = e, r === 0;
  }
  B["quarto-resource-document-text-ascii"] = da;
  var rv = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-text-ascii" };
  function da(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: rv }, message: "must be boolean", schema: rv.type, parentSchema: rv, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return da.errors = e, r === 0;
  }
  B["quarto-resource-document-toc-toc"] = K;
  var tv = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-toc-toc" };
  function K(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: tv }, message: "must be boolean", schema: tv.type, parentSchema: tv, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return K.errors = e, r === 0;
  }
  B["quarto-resource-document-toc-table-of-contents"] = A1;
  var av = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-toc-table-of-contents" };
  function A1(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: av }, message: "must be boolean", schema: av.type, parentSchema: av, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return A1.errors = e, r === 0;
  }
  B["quarto-resource-document-toc-toc-depth"] = Wr;
  var nv = { type: "number", description: "be a number", $id: "quarto-resource-document-toc-toc-depth" };
  function Wr(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (!(typeof l == "number" && isFinite(l))) {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "number", schema: nv }, message: "must be number", schema: nv.type, parentSchema: nv, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Wr.errors = e, r === 0;
  }
  B["quarto-resource-document-toc-toc-location"] = Xs;
  var bm = { enum: ["body", "left", "right"], description: "be one of: `body`, `left`, `right`", $id: "quarto-resource-document-toc-toc-location" };
  function Xs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = bm.enum;
    if (!(l === "body" || l === "left" || l === "right")) {
      let c = { instancePath: o, schemaPath: "#/enum", keyword: "enum", params: { allowedValues: bm.enum, schema: bm }, message: "must be equal to one of the allowed values", schema: bm.enum, parentSchema: bm, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    return Xs.errors = e, r === 0;
  }
  B["quarto-resource-document-toc-toc-title"] = La;
  var ov = { type: "string", description: "be a string", $id: "quarto-resource-document-toc-toc-title" };
  function La(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: ov }, message: "must be string", schema: ov.type, parentSchema: ov, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return La.errors = e, r === 0;
  }
  B["quarto-resource-document-toc-lof"] = Zs;
  var sv = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-toc-lof" };
  function Zs(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: sv }, message: "must be boolean", schema: sv.type, parentSchema: sv, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return Zs.errors = e, r === 0;
  }
  B["quarto-resource-document-toc-lot"] = ei;
  var iv = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-toc-lot" };
  function ei(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: iv }, message: "must be boolean", schema: iv.type, parentSchema: iv, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ei.errors = e, r === 0;
  }
  B["quarto-resource-document-website-search"] = ri;
  var lv = { type: "boolean", description: "be `true` or `false`", $id: "quarto-resource-document-website-search" };
  function ri(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "boolean") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "boolean", schema: lv }, message: "must be boolean", schema: lv.type, parentSchema: lv, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ri.errors = e, r === 0;
  }
  B["quarto-resource-document-website-aliases"] = ti;
  var jd = { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" }, $id: "quarto-resource-document-website-aliases" };
  function ti(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (Array.isArray(l)) {
      let d = true, u = l.length;
      for (let y = 0; y < u; y++) {
        let g = l[y], x = r;
        if (typeof g != "string") {
          let $ = { instancePath: o + "/" + y, schemaPath: "#/items/type", keyword: "type", params: { type: "string", schema: jd.items }, message: "must be string", schema: jd.items.type, parentSchema: jd.items, data: g };
          e === null ? e = [$] : e.push($), r++;
        }
        let D = r;
        var c = x === r;
        c || (d = false);
      }
    } else {
      let d = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "array", schema: jd }, message: "must be array", schema: jd.type, parentSchema: jd, data: l };
      e === null ? e = [d] : e.push(d), r++;
    }
    return ti.errors = e, r === 0;
  }
  B["quarto-resource-document-website-image"] = ai;
  var cv = { type: "string", description: "be a string", $id: "quarto-resource-document-website-image" };
  function ai(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: cv }, message: "must be string", schema: cv.type, parentSchema: cv, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ai.errors = e, r === 0;
  }
  B["quarto-resource-document-website-image-height"] = ni;
  var pv = { type: "string", description: "be a string", $id: "quarto-resource-document-website-image-height" };
  function ni(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: pv }, message: "must be string", schema: pv.type, parentSchema: pv, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return ni.errors = e, r === 0;
  }
  B["quarto-resource-document-website-image-width"] = oi;
  var uv = { type: "string", description: "be a string", $id: "quarto-resource-document-website-image-width" };
  function oi(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    if (typeof l != "string") {
      let c = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "string", schema: uv }, message: "must be string", schema: uv.type, parentSchema: uv, data: l };
      e === null ? e = [c] : e.push(c), r++;
    }
    let _ = r;
    return oi.errors = e, r === 0;
  }
  B["quarto-resource-project-project"] = j1;
  var Ta = { type: "object", description: "be an object", properties: { type: { enum: ["website", "book", "site"], description: "be one of: `website`, `book`, `site`" }, render: { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }, "execute-dir": { enum: ["file", "project"], description: "be one of: `file`, `project`" }, "output-dir": { type: "string", description: "be a string" }, "lib-dir": { type: "string", description: "be a string" }, resources: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(execute_dir|executeDir|output_dir|outputDir|lib_dir|libDir))" }, $id: "quarto-resource-project-project" }, B1 = new RegExp("^(?!(execute_dir|executeDir|output_dir|outputDir|lib_dir|libDir))", "u");
  function j1(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let x in l) {
        let D = r, $ = r;
        if (typeof x == "string") {
          if (!B1.test(x)) {
            let m = { instancePath: o, schemaPath: "#/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(execute_dir|executeDir|output_dir|outputDir|lib_dir|libDir))", schema: Ta.propertyNames }, message: 'must match pattern "^(?!(execute_dir|executeDir|output_dir|outputDir|lib_dir|libDir))"', schema: "^(?!(execute_dir|executeDir|output_dir|outputDir|lib_dir|libDir))", parentSchema: Ta.propertyNames, data: x, propertyName: x };
            e === null ? e = [m] : e.push(m), r++;
          }
        } else {
          let m = { instancePath: o, schemaPath: "#/propertyNames/type", keyword: "type", params: { type: "string", schema: Ta.propertyNames }, message: "must be string", schema: Ta.propertyNames.type, parentSchema: Ta.propertyNames, data: x, propertyName: x };
          e === null ? e = [m] : e.push(m), r++;
        }
        var c = D === r;
        if (!c) {
          let m = { instancePath: o, schemaPath: "#/propertyNames", keyword: "propertyNames", params: { propertyName: x, schema: Ta }, message: "property name must be valid", schema: Ta.propertyNames, parentSchema: Ta, data: l };
          e === null ? e = [m] : e.push(m), r++;
        }
      }
      if (l.type !== void 0) {
        let x = l.type, D = r, $ = Ta.properties.type.enum;
        if (!(x === "website" || x === "book" || x === "site")) {
          let m = { instancePath: o + "/type", schemaPath: "#/properties/type/enum", keyword: "enum", params: { allowedValues: Ta.properties.type.enum, schema: Ta.properties.type }, message: "must be equal to one of the allowed values", schema: Ta.properties.type.enum, parentSchema: Ta.properties.type, data: x };
          e === null ? e = [m] : e.push(m), r++;
        }
        var d = D === r;
      }
      if (l.render !== void 0) {
        let x = l.render, D = r, $ = r;
        if (Array.isArray(x)) {
          let R = true, j = x.length;
          for (let V = 0; V < j; V++) {
            let se = x[V], P = r;
            if (typeof se != "string") {
              let h = { instancePath: o + "/render/" + V, schemaPath: "#/properties/render/items/type", keyword: "type", params: { type: "string", schema: Ta.properties.render.items }, message: "must be string", schema: Ta.properties.render.items.type, parentSchema: Ta.properties.render.items, data: se };
              e === null ? e = [h] : e.push(h), r++;
            }
            let oe = r;
            var u = P === r;
            u || (R = false);
          }
        } else {
          let R = { instancePath: o + "/render", schemaPath: "#/properties/render/type", keyword: "type", params: { type: "array", schema: Ta.properties.render }, message: "must be array", schema: Ta.properties.render.type, parentSchema: Ta.properties.render, data: x };
          e === null ? e = [R] : e.push(R), r++;
        }
        var d = D === r;
      }
      if (l["execute-dir"] !== void 0) {
        let x = l["execute-dir"], D = r, $ = Ta.properties["execute-dir"].enum;
        if (!(x === "file" || x === "project")) {
          let R = { instancePath: o + "/execute-dir", schemaPath: "#/properties/execute-dir/enum", keyword: "enum", params: { allowedValues: Ta.properties["execute-dir"].enum, schema: Ta.properties["execute-dir"] }, message: "must be equal to one of the allowed values", schema: Ta.properties["execute-dir"].enum, parentSchema: Ta.properties["execute-dir"], data: x };
          e === null ? e = [R] : e.push(R), r++;
        }
        var d = D === r;
      }
      if (l["output-dir"] !== void 0) {
        let x = l["output-dir"], D = r;
        if (typeof x != "string") {
          let R = { instancePath: o + "/output-dir", schemaPath: "#/properties/output-dir/type", keyword: "type", params: { type: "string", schema: Ta.properties["output-dir"] }, message: "must be string", schema: Ta.properties["output-dir"].type, parentSchema: Ta.properties["output-dir"], data: x };
          e === null ? e = [R] : e.push(R), r++;
        }
        let $ = r;
        var d = D === r;
      }
      if (l["lib-dir"] !== void 0) {
        let x = l["lib-dir"], D = r;
        if (typeof x != "string") {
          let R = { instancePath: o + "/lib-dir", schemaPath: "#/properties/lib-dir/type", keyword: "type", params: { type: "string", schema: Ta.properties["lib-dir"] }, message: "must be string", schema: Ta.properties["lib-dir"].type, parentSchema: Ta.properties["lib-dir"], data: x };
          e === null ? e = [R] : e.push(R), r++;
        }
        let $ = r;
        var d = D === r;
      }
      if (l.resources !== void 0) {
        let x = l.resources, D = r, $ = r, m = false, R = null, j = r;
        if (typeof x != "string") {
          let h = { instancePath: o + "/resources", schemaPath: "#/properties/resources/oneOf/0/type", keyword: "type", params: { type: "string", schema: Ta.properties.resources.oneOf[0] }, message: "must be string", schema: Ta.properties.resources.oneOf[0].type, parentSchema: Ta.properties.resources.oneOf[0], data: x };
          e === null ? e = [h] : e.push(h), r++;
        }
        let V = r;
        var y = j === r;
        y && (m = true, R = 0);
        let se = r, P = r;
        if (Array.isArray(x)) {
          let h = true, f = x.length;
          for (let ft = 0; ft < f; ft++) {
            let Cr = x[ft], Dr = r;
            if (typeof Cr != "string") {
              let ta = { instancePath: o + "/resources/" + ft, schemaPath: "#/properties/resources/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: Ta.properties.resources.oneOf[1].items }, message: "must be string", schema: Ta.properties.resources.oneOf[1].items.type, parentSchema: Ta.properties.resources.oneOf[1].items, data: Cr };
              e === null ? e = [ta] : e.push(ta), r++;
            }
            let wr = r;
            var g = Dr === r;
            g || (h = false);
          }
        } else {
          let h = { instancePath: o + "/resources", schemaPath: "#/properties/resources/oneOf/1/type", keyword: "type", params: { type: "array", schema: Ta.properties.resources.oneOf[1] }, message: "must be array", schema: Ta.properties.resources.oneOf[1].type, parentSchema: Ta.properties.resources.oneOf[1], data: x };
          e === null ? e = [h] : e.push(h), r++;
        }
        var y = se === r;
        if (y && m ? (m = false, R = [R, 1]) : y && (m = true, R = 1), m)
          r = $, e !== null && ($ ? e.length = $ : e = null);
        else {
          let h = { instancePath: o + "/resources", schemaPath: "#/properties/resources/oneOf", keyword: "oneOf", params: { passingSchemas: R, schema: Ta.properties.resources }, message: "must match exactly one schema in oneOf", schema: Ta.properties.resources.oneOf, parentSchema: Ta.properties.resources, data: x };
          e === null ? e = [h] : e.push(h), r++;
        }
        var d = D === r;
      }
    } else {
      let x = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: Ta }, message: "must be object", schema: Ta.type, parentSchema: Ta, data: l };
      e === null ? e = [x] : e.push(x), r++;
    }
    return j1.errors = e, r === 0;
  }
  B["quarto-resource-project-website"] = E1;
  function E1(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0;
    return hc(l, { instancePath: o, parentData: S, parentDataProperty: F, rootData: s }) || (e = e === null ? hc.errors : e.concat(hc.errors), r = e.length), E1.errors = e, r === 0;
  }
  B["quarto-resource-project-book"] = C1;
  var q = { type: "object", description: "be an object", properties: { title: { type: "string", description: "be a string" }, description: { type: "string", description: "be a string" }, "fav-icon": { type: "string", description: "be a string" }, "site-url": { type: "string", description: "be a string" }, "site-path": { type: "string", description: "be a string" }, "repo-url": { type: "string", description: "be a string" }, "repo-branch": { type: "string", description: "be a string" }, "repo-actions": { oneOf: [{ enum: ["none", "edit", "source", "issue"], description: "be one of: `none`, `edit`, `source`, `issue`" }, { type: "array", description: "be an array of values, where each element should be one of: `none`, `edit`, `source`, `issue`", items: { enum: ["none", "edit", "source", "issue"], description: "be one of: `none`, `edit`, `source`, `issue`" } }], description: "be exactly one of: one of: `none`, `edit`, `source`, `issue`, an array of values, where each element should be one of: `none`, `edit`, `source`, `issue`" }, "google-analytics": { anyOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: { "tracking-id": { type: "string", description: "be a string" }, storage: { enum: ["cookies", "none"], description: "be one of: `cookies`, `none`" }, "anonymize-ip": { type: "boolean", description: "be `true` or `false`" }, version: { enum: [3, 4], description: "be one of: `3`, `4`" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tracking_id|trackingId|anonymize_ip|anonymizeIp))" } }], description: "be at least one of: a string, an object" }, "cookie-consent": { anyOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { type: { enum: ["implied", "express"], description: "be one of: `implied`, `express`" }, style: { enum: ["simple", "headline", "interstitial", "standalone"], description: "be one of: `simple`, `headline`, `interstitial`, `standalone`" }, palette: { enum: ["light", "dark"], description: "be one of: `light`, `dark`" }, "policy-url": { type: "string", description: "be a string" }, "prefs-text": { type: "string", description: "be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(policy_url|policyUrl|prefs_text|prefsText))" } }], description: "be at least one of: `true` or `false`, an object" }, search: { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { location: { enum: ["navbar", "sidebar"], description: "be one of: `navbar`, `sidebar`" }, type: { enum: ["overlay", "textbox"], description: "be one of: `overlay`, `textbox`" }, limit: { type: "number", description: "be a number" }, "collapse-after": { type: "number", description: "be a number" }, "copy-button": { type: "boolean", description: "be `true` or `false`" }, algolia: { type: "object", description: "be an object", properties: { "index-name": { type: "string", description: "be a string" }, "application-id": { type: "string", description: "be a string" }, "search-only-api-key": { type: "string", description: "be a string" }, "analytics-events": { type: "boolean", description: "be `true` or `false`" }, "show-logo": { type: "boolean", description: "be `true` or `false`" }, "index-fields": { type: "object", description: "be an object", properties: { href: { type: "string", description: "be a string" }, title: { type: "string", description: "be a string" }, text: { type: "string", description: "be a string" }, section: { type: "string", description: "be a string" } }, patternProperties: {}, additionalProperties: false }, params: { type: "object", description: "be an object", properties: {}, patternProperties: {} } }, patternProperties: {}, additionalProperties: false, propertyNames: { type: "string", pattern: "^(?!(index_name|indexName|application_id|applicationId|search_only_api_key|searchOnlyApiKey|analytics_events|analyticsEvents|show_logo|showLogo|index_fields|indexFields))" } } }, patternProperties: {}, additionalProperties: false, propertyNames: { type: "string", pattern: "^(?!(collapse_after|collapseAfter|copy_button|copyButton))" } }], description: "be exactly one of: `true` or `false`, an object" }, navbar: { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { title: { oneOf: [{ type: "string", description: "be a string" }, { type: "boolean", description: "be `true` or `false`" }], description: "be exactly one of: a string, `true` or `false`" }, logo: { type: "string", description: "be a string" }, background: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, foreground: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, search: { type: "boolean", description: "be `true` or `false`" }, pinned: { type: "boolean", description: "be `true` or `false`" }, collapse: { type: "boolean", description: "be `true` or `false`" }, "collapse-below": { enum: ["sm", "md", "lg", "xl", "xxl"], description: "be one of: `sm`, `md`, `lg`, `xl`, `xxl`" }, left: { type: "array", description: "be an array of values, where each element should be navigation-item", items: { $ref: "navigation-item", description: "be navigation-item" } }, right: { type: "array", description: "be an array of values, where each element should be navigation-item", items: { $ref: "navigation-item", description: "be navigation-item" } } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(collapse_below|collapseBelow))" } }], description: "be exactly one of: `true` or `false`, an object" }, sidebar: { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { oneOf: [{ type: "object", description: "be an object", properties: { id: { type: "string", description: "be a string" }, title: { oneOf: [{ type: "string", description: "be a string" }, { type: "boolean", description: "be `true` or `false`" }], description: "be exactly one of: a string, `true` or `false`" }, subtitle: { type: "string", description: "be a string" }, logo: { type: "string", description: "be a string" }, search: { type: "boolean", description: "be `true` or `false`" }, tools: { type: "array", description: "be an array of values, where each element should be tool-item", items: { $ref: "tool-item", description: "be tool-item" } }, contents: { $ref: "sidebar-contents", description: "be sidebar-contents" }, style: { enum: ["docked", "floating"], description: "be one of: `docked`, `floating`" }, background: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, foreground: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, border: { type: "boolean", description: "be `true` or `false`" }, alignment: { enum: ["left", "right", "center"], description: "be one of: `left`, `right`, `center`" }, "collapse-level": { type: "number", description: "be a number" }, pinned: { type: "boolean", description: "be `true` or `false`" }, header: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, footer: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(collapse_level|collapseLevel))" } }, { type: "array", description: "be an array of values, where each element should be an object", items: { type: "object", description: "be an object", properties: { id: { type: "string", description: "be a string" }, title: { oneOf: [{ type: "string", description: "be a string" }, { type: "boolean", description: "be `true` or `false`" }], description: "be exactly one of: a string, `true` or `false`" }, subtitle: { type: "string", description: "be a string" }, logo: { type: "string", description: "be a string" }, search: { type: "boolean", description: "be `true` or `false`" }, tools: { type: "array", description: "be an array of values, where each element should be tool-item", items: { $ref: "tool-item", description: "be tool-item" } }, contents: { $ref: "sidebar-contents", description: "be sidebar-contents" }, style: { enum: ["docked", "floating"], description: "be one of: `docked`, `floating`" }, background: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, foreground: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, border: { type: "boolean", description: "be `true` or `false`" }, alignment: { enum: ["left", "right", "center"], description: "be one of: `left`, `right`, `center`" }, "collapse-level": { type: "number", description: "be a number" }, pinned: { type: "boolean", description: "be `true` or `false`" }, header: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, footer: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(collapse_level|collapseLevel))" } } }], description: "be exactly one of: an object, an array of values, where each element should be an object" }], description: "be exactly one of: `true` or `false`, exactly one of: an object, an array of values, where each element should be an object" }, "body-header": { type: "string", description: "be a string" }, "body-footer": { type: "string", description: "be a string" }, "margin-header": { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, "margin-footer": { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, "page-navigation": { type: "boolean", description: "be `true` or `false`" }, "page-footer": { oneOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: { left: { $ref: "page-footer-region", description: "be page-footer-region" }, right: { $ref: "page-footer-region", description: "be page-footer-region" }, center: { $ref: "page-footer-region", description: "be page-footer-region" }, border: { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "string", description: "be a string" }], description: "be exactly one of: `true` or `false`, a string" }, background: { type: "string", description: "be a string" }, foreground: { type: "string", description: "be a string" } }, patternProperties: {}, additionalProperties: false }], description: "be exactly one of: a string, an object" }, image: { type: "string", description: "be a string" }, comments: { $ref: "comments", description: "be comments" }, "open-graph": { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { title: { type: "string", description: "be a string" }, description: { type: "string", description: "be a string" }, image: { type: "string", description: "be a string" }, "image-width": { type: "number", description: "be a number" }, "image-height": { type: "number", description: "be a number" }, locale: { type: "string", description: "be a string" }, "site-name": { type: "string", description: "be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(^(?!^(?!(site_name|siteName)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))" }, required: [], additionalProperties: false }], description: "be exactly one of: `true` or `false`, an object" }, "twitter-card": { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { title: { type: "string", description: "be a string" }, description: { type: "string", description: "be a string" }, image: { type: "string", description: "be a string" }, "image-width": { type: "number", description: "be a number" }, "image-height": { type: "number", description: "be a number" }, "card-style": { enum: ["summary", "summary_large_image"], description: "be one of: `summary`, `summary_large_image`" }, creator: { type: "string", description: "be a string" }, site: { type: "string", description: "be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(^(?!^(?!(card_style|cardStyle)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))" }, required: [], additionalProperties: false }], description: "be exactly one of: `true` or `false`, an object" }, subtitle: { type: "string", description: "be a string" }, author: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, date: { type: "string", description: "be a string" }, abstract: { type: "string", description: "be a string" }, chapters: { $ref: "chapter-list", description: "be chapter-list" }, references: { type: "string", description: "be a string" }, appendices: { type: "array", description: "be an array of values, where each element should be chapter-item", items: { $ref: "chapter-item", description: "be chapter-item" } }, "output-file": { type: "string", description: "be a string" }, "cover-image": { type: "string", description: "be a string" }, sharing: { oneOf: [{ enum: ["twitter", "facebook", "linkedin"], description: "be one of: `twitter`, `facebook`, `linkedin`" }, { type: "array", description: "be an array of values, where each element should be one of: `twitter`, `facebook`, `linkedin`", items: { enum: ["twitter", "facebook", "linkedin"], description: "be one of: `twitter`, `facebook`, `linkedin`" } }], description: "be exactly one of: one of: `twitter`, `facebook`, `linkedin`, an array of values, where each element should be one of: `twitter`, `facebook`, `linkedin`" }, downloads: { oneOf: [{ enum: ["pdf", "epub", "docx"], description: "be one of: `pdf`, `epub`, `docx`" }, { type: "array", description: "be an array of values, where each element should be one of: `pdf`, `epub`, `docx`", items: { enum: ["pdf", "epub", "docx"], description: "be one of: `pdf`, `epub`, `docx`" } }], description: "be exactly one of: one of: `pdf`, `epub`, `docx`, an array of values, where each element should be one of: `pdf`, `epub`, `docx`" }, tools: { type: "array", description: "be an array of values, where each element should be navigation-item", items: { $ref: "navigation-item", description: "be navigation-item" } } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(^(?!^(?!(output_file|outputFile|cover_image|coverImage)))|^(?!^(?!(fav_icon|favIcon|site_url|siteUrl|site_path|sitePath|repo_url|repoUrl|repo_branch|repoBranch|repo_actions|repoActions|google_analytics|googleAnalytics|cookie_consent|cookieConsent|body_header|bodyHeader|body_footer|bodyFooter|margin_header|marginHeader|margin_footer|marginFooter|page_navigation|pageNavigation|page_footer|pageFooter|open_graph|openGraph|twitter_card|twitterCard)))))" }, required: [], $id: "quarto-resource-project-book" }, N1 = new RegExp("^(?!(^(?!^(?!(output_file|outputFile|cover_image|coverImage)))|^(?!^(?!(fav_icon|favIcon|site_url|siteUrl|site_path|sitePath|repo_url|repoUrl|repo_branch|repoBranch|repo_actions|repoActions|google_analytics|googleAnalytics|cookie_consent|cookieConsent|body_header|bodyHeader|body_footer|bodyFooter|margin_header|marginHeader|margin_footer|marginFooter|page_navigation|pageNavigation|page_footer|pageFooter|open_graph|openGraph|twitter_card|twitterCard)))))", "u");
  function C1(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let w in l) {
        let Nt = r, Y = r;
        if (typeof w == "string") {
          if (!N1.test(w)) {
            let N = { instancePath: o, schemaPath: "#/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(^(?!^(?!(output_file|outputFile|cover_image|coverImage)))|^(?!^(?!(fav_icon|favIcon|site_url|siteUrl|site_path|sitePath|repo_url|repoUrl|repo_branch|repoBranch|repo_actions|repoActions|google_analytics|googleAnalytics|cookie_consent|cookieConsent|body_header|bodyHeader|body_footer|bodyFooter|margin_header|marginHeader|margin_footer|marginFooter|page_navigation|pageNavigation|page_footer|pageFooter|open_graph|openGraph|twitter_card|twitterCard)))))", schema: q.propertyNames }, message: 'must match pattern "^(?!(^(?!^(?!(output_file|outputFile|cover_image|coverImage)))|^(?!^(?!(fav_icon|favIcon|site_url|siteUrl|site_path|sitePath|repo_url|repoUrl|repo_branch|repoBranch|repo_actions|repoActions|google_analytics|googleAnalytics|cookie_consent|cookieConsent|body_header|bodyHeader|body_footer|bodyFooter|margin_header|marginHeader|margin_footer|marginFooter|page_navigation|pageNavigation|page_footer|pageFooter|open_graph|openGraph|twitter_card|twitterCard)))))"', schema: "^(?!(^(?!^(?!(output_file|outputFile|cover_image|coverImage)))|^(?!^(?!(fav_icon|favIcon|site_url|siteUrl|site_path|sitePath|repo_url|repoUrl|repo_branch|repoBranch|repo_actions|repoActions|google_analytics|googleAnalytics|cookie_consent|cookieConsent|body_header|bodyHeader|body_footer|bodyFooter|margin_header|marginHeader|margin_footer|marginFooter|page_navigation|pageNavigation|page_footer|pageFooter|open_graph|openGraph|twitter_card|twitterCard)))))", parentSchema: q.propertyNames, data: w, propertyName: w };
            e === null ? e = [N] : e.push(N), r++;
          }
        } else {
          let N = { instancePath: o, schemaPath: "#/propertyNames/type", keyword: "type", params: { type: "string", schema: q.propertyNames }, message: "must be string", schema: q.propertyNames.type, parentSchema: q.propertyNames, data: w, propertyName: w };
          e === null ? e = [N] : e.push(N), r++;
        }
        var c = Nt === r;
        if (!c) {
          let N = { instancePath: o, schemaPath: "#/propertyNames", keyword: "propertyNames", params: { propertyName: w, schema: q }, message: "property name must be valid", schema: q.propertyNames, parentSchema: q, data: l };
          e === null ? e = [N] : e.push(N), r++;
        }
      }
      if (l.title !== void 0) {
        let w = l.title, Nt = r;
        if (typeof w != "string") {
          let N = { instancePath: o + "/title", schemaPath: "#/properties/title/type", keyword: "type", params: { type: "string", schema: q.properties.title }, message: "must be string", schema: q.properties.title.type, parentSchema: q.properties.title, data: w };
          e === null ? e = [N] : e.push(N), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l.description !== void 0) {
        let w = l.description, Nt = r;
        if (typeof w != "string") {
          let ae = { instancePath: o + "/description", schemaPath: "#/properties/description/type", keyword: "type", params: { type: "string", schema: q.properties.description }, message: "must be string", schema: q.properties.description.type, parentSchema: q.properties.description, data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l["fav-icon"] !== void 0) {
        let w = l["fav-icon"], Nt = r;
        if (typeof w != "string") {
          let ae = { instancePath: o + "/fav-icon", schemaPath: "#/properties/fav-icon/type", keyword: "type", params: { type: "string", schema: q.properties["fav-icon"] }, message: "must be string", schema: q.properties["fav-icon"].type, parentSchema: q.properties["fav-icon"], data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l["site-url"] !== void 0) {
        let w = l["site-url"], Nt = r;
        if (typeof w != "string") {
          let ae = { instancePath: o + "/site-url", schemaPath: "#/properties/site-url/type", keyword: "type", params: { type: "string", schema: q.properties["site-url"] }, message: "must be string", schema: q.properties["site-url"].type, parentSchema: q.properties["site-url"], data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l["site-path"] !== void 0) {
        let w = l["site-path"], Nt = r;
        if (typeof w != "string") {
          let ae = { instancePath: o + "/site-path", schemaPath: "#/properties/site-path/type", keyword: "type", params: { type: "string", schema: q.properties["site-path"] }, message: "must be string", schema: q.properties["site-path"].type, parentSchema: q.properties["site-path"], data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l["repo-url"] !== void 0) {
        let w = l["repo-url"], Nt = r;
        if (typeof w != "string") {
          let ae = { instancePath: o + "/repo-url", schemaPath: "#/properties/repo-url/type", keyword: "type", params: { type: "string", schema: q.properties["repo-url"] }, message: "must be string", schema: q.properties["repo-url"].type, parentSchema: q.properties["repo-url"], data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l["repo-branch"] !== void 0) {
        let w = l["repo-branch"], Nt = r;
        if (typeof w != "string") {
          let ae = { instancePath: o + "/repo-branch", schemaPath: "#/properties/repo-branch/type", keyword: "type", params: { type: "string", schema: q.properties["repo-branch"] }, message: "must be string", schema: q.properties["repo-branch"].type, parentSchema: q.properties["repo-branch"], data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l["repo-actions"] !== void 0) {
        let w = l["repo-actions"], Nt = r, Y = r, N = false, ae = null, nt = r, J = q.properties["repo-actions"].oneOf[0].enum;
        if (!(w === "none" || w === "edit" || w === "source" || w === "issue")) {
          let v = { instancePath: o + "/repo-actions", schemaPath: "#/properties/repo-actions/oneOf/0/enum", keyword: "enum", params: { allowedValues: q.properties["repo-actions"].oneOf[0].enum, schema: q.properties["repo-actions"].oneOf[0] }, message: "must be equal to one of the allowed values", schema: q.properties["repo-actions"].oneOf[0].enum, parentSchema: q.properties["repo-actions"].oneOf[0], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var u = nt === r;
        u && (N = true, ae = 0);
        let ot = r, le = r;
        if (Array.isArray(w)) {
          let v = true, E = w.length;
          for (let A = 0; A < E; A++) {
            let Z = w[A], T = r, U = q.properties["repo-actions"].oneOf[1].items.enum;
            if (!(Z === "none" || Z === "edit" || Z === "source" || Z === "issue")) {
              let re = { instancePath: o + "/repo-actions/" + A, schemaPath: "#/properties/repo-actions/oneOf/1/items/enum", keyword: "enum", params: { allowedValues: q.properties["repo-actions"].oneOf[1].items.enum, schema: q.properties["repo-actions"].oneOf[1].items }, message: "must be equal to one of the allowed values", schema: q.properties["repo-actions"].oneOf[1].items.enum, parentSchema: q.properties["repo-actions"].oneOf[1].items, data: Z };
              e === null ? e = [re] : e.push(re), r++;
            }
            var y = T === r;
            y || (v = false);
          }
        } else {
          let v = { instancePath: o + "/repo-actions", schemaPath: "#/properties/repo-actions/oneOf/1/type", keyword: "type", params: { type: "array", schema: q.properties["repo-actions"].oneOf[1] }, message: "must be array", schema: q.properties["repo-actions"].oneOf[1].type, parentSchema: q.properties["repo-actions"].oneOf[1], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var u = ot === r;
        if (u && N ? (N = false, ae = [ae, 1]) : u && (N = true, ae = 1), N)
          r = Y, e !== null && (Y ? e.length = Y : e = null);
        else {
          let v = { instancePath: o + "/repo-actions", schemaPath: "#/properties/repo-actions/oneOf", keyword: "oneOf", params: { passingSchemas: ae, schema: q.properties["repo-actions"] }, message: "must match exactly one schema in oneOf", schema: q.properties["repo-actions"].oneOf, parentSchema: q.properties["repo-actions"], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var d = Nt === r;
      }
      if (l["google-analytics"] !== void 0) {
        let w = l["google-analytics"], Nt = r, Y = r, N = false, ae = r;
        if (typeof w != "string") {
          let ot = { instancePath: o + "/google-analytics", schemaPath: "#/properties/google-analytics/anyOf/0/type", keyword: "type", params: { type: "string", schema: q.properties["google-analytics"].anyOf[0] }, message: "must be string", schema: q.properties["google-analytics"].anyOf[0].type, parentSchema: q.properties["google-analytics"].anyOf[0], data: w };
          e === null ? e = [ot] : e.push(ot), r++;
        }
        let nt = r;
        var g = ae === r;
        if (N = N || g, !N) {
          let ot = r, le = r;
          if (w && typeof w == "object" && !Array.isArray(w)) {
            for (let v in w) {
              let E = r, A = r;
              if (typeof v == "string") {
                if (!tg.test(v)) {
                  let Z = { instancePath: o + "/google-analytics", schemaPath: "#/properties/google-analytics/anyOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tracking_id|trackingId|anonymize_ip|anonymizeIp))", schema: q.properties["google-analytics"].anyOf[1].propertyNames }, message: 'must match pattern "^(?!(tracking_id|trackingId|anonymize_ip|anonymizeIp))"', schema: "^(?!(tracking_id|trackingId|anonymize_ip|anonymizeIp))", parentSchema: q.properties["google-analytics"].anyOf[1].propertyNames, data: v, propertyName: v };
                  e === null ? e = [Z] : e.push(Z), r++;
                }
              } else {
                let Z = { instancePath: o + "/google-analytics", schemaPath: "#/properties/google-analytics/anyOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: q.properties["google-analytics"].anyOf[1].propertyNames }, message: "must be string", schema: q.properties["google-analytics"].anyOf[1].propertyNames.type, parentSchema: q.properties["google-analytics"].anyOf[1].propertyNames, data: v, propertyName: v };
                e === null ? e = [Z] : e.push(Z), r++;
              }
              var x = E === r;
              if (!x) {
                let Z = { instancePath: o + "/google-analytics", schemaPath: "#/properties/google-analytics/anyOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: v, schema: q.properties["google-analytics"].anyOf[1] }, message: "property name must be valid", schema: q.properties["google-analytics"].anyOf[1].propertyNames, parentSchema: q.properties["google-analytics"].anyOf[1], data: w };
                e === null ? e = [Z] : e.push(Z), r++;
              }
            }
            if (w["tracking-id"] !== void 0) {
              let v = w["tracking-id"], E = r;
              if (typeof v != "string") {
                let Z = { instancePath: o + "/google-analytics/tracking-id", schemaPath: "#/properties/google-analytics/anyOf/1/properties/tracking-id/type", keyword: "type", params: { type: "string", schema: q.properties["google-analytics"].anyOf[1].properties["tracking-id"] }, message: "must be string", schema: q.properties["google-analytics"].anyOf[1].properties["tracking-id"].type, parentSchema: q.properties["google-analytics"].anyOf[1].properties["tracking-id"], data: v };
                e === null ? e = [Z] : e.push(Z), r++;
              }
              let A = r;
              var D = E === r;
            }
            if (w.storage !== void 0) {
              let v = w.storage, E = r, A = q.properties["google-analytics"].anyOf[1].properties.storage.enum;
              if (!(v === "cookies" || v === "none")) {
                let T = { instancePath: o + "/google-analytics/storage", schemaPath: "#/properties/google-analytics/anyOf/1/properties/storage/enum", keyword: "enum", params: { allowedValues: q.properties["google-analytics"].anyOf[1].properties.storage.enum, schema: q.properties["google-analytics"].anyOf[1].properties.storage }, message: "must be equal to one of the allowed values", schema: q.properties["google-analytics"].anyOf[1].properties.storage.enum, parentSchema: q.properties["google-analytics"].anyOf[1].properties.storage, data: v };
                e === null ? e = [T] : e.push(T), r++;
              }
              var D = E === r;
            }
            if (w["anonymize-ip"] !== void 0) {
              let v = w["anonymize-ip"], E = r;
              if (typeof v != "boolean") {
                let T = { instancePath: o + "/google-analytics/anonymize-ip", schemaPath: "#/properties/google-analytics/anyOf/1/properties/anonymize-ip/type", keyword: "type", params: { type: "boolean", schema: q.properties["google-analytics"].anyOf[1].properties["anonymize-ip"] }, message: "must be boolean", schema: q.properties["google-analytics"].anyOf[1].properties["anonymize-ip"].type, parentSchema: q.properties["google-analytics"].anyOf[1].properties["anonymize-ip"], data: v };
                e === null ? e = [T] : e.push(T), r++;
              }
              let A = r;
              var D = E === r;
            }
            if (w.version !== void 0) {
              let v = w.version, E = r, A = q.properties["google-analytics"].anyOf[1].properties.version.enum;
              if (!(v === 3 || v === 4)) {
                let T = { instancePath: o + "/google-analytics/version", schemaPath: "#/properties/google-analytics/anyOf/1/properties/version/enum", keyword: "enum", params: { allowedValues: q.properties["google-analytics"].anyOf[1].properties.version.enum, schema: q.properties["google-analytics"].anyOf[1].properties.version }, message: "must be equal to one of the allowed values", schema: q.properties["google-analytics"].anyOf[1].properties.version.enum, parentSchema: q.properties["google-analytics"].anyOf[1].properties.version, data: v };
                e === null ? e = [T] : e.push(T), r++;
              }
              var D = E === r;
            }
          } else {
            let v = { instancePath: o + "/google-analytics", schemaPath: "#/properties/google-analytics/anyOf/1/type", keyword: "type", params: { type: "object", schema: q.properties["google-analytics"].anyOf[1] }, message: "must be object", schema: q.properties["google-analytics"].anyOf[1].type, parentSchema: q.properties["google-analytics"].anyOf[1], data: w };
            e === null ? e = [v] : e.push(v), r++;
          }
          var g = ot === r;
          N = N || g;
        }
        if (N)
          r = Y, e !== null && (Y ? e.length = Y : e = null);
        else {
          let ot = { instancePath: o + "/google-analytics", schemaPath: "#/properties/google-analytics/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: q.properties["google-analytics"].anyOf, parentSchema: q.properties["google-analytics"], data: w };
          e === null ? e = [ot] : e.push(ot), r++;
        }
        var d = Nt === r;
      }
      if (l["cookie-consent"] !== void 0) {
        let w = l["cookie-consent"], Nt = r, Y = r, N = false, ae = r;
        if (typeof w != "boolean") {
          let ot = { instancePath: o + "/cookie-consent", schemaPath: "#/properties/cookie-consent/anyOf/0/type", keyword: "type", params: { type: "boolean", schema: q.properties["cookie-consent"].anyOf[0] }, message: "must be boolean", schema: q.properties["cookie-consent"].anyOf[0].type, parentSchema: q.properties["cookie-consent"].anyOf[0], data: w };
          e === null ? e = [ot] : e.push(ot), r++;
        }
        let nt = r;
        var $ = ae === r;
        if (N = N || $, !N) {
          let ot = r, le = r;
          if (w && typeof w == "object" && !Array.isArray(w)) {
            for (let v in w) {
              let E = r, A = r;
              if (typeof v == "string") {
                if (!ag.test(v)) {
                  let Z = { instancePath: o + "/cookie-consent", schemaPath: "#/properties/cookie-consent/anyOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(policy_url|policyUrl|prefs_text|prefsText))", schema: q.properties["cookie-consent"].anyOf[1].propertyNames }, message: 'must match pattern "^(?!(policy_url|policyUrl|prefs_text|prefsText))"', schema: "^(?!(policy_url|policyUrl|prefs_text|prefsText))", parentSchema: q.properties["cookie-consent"].anyOf[1].propertyNames, data: v, propertyName: v };
                  e === null ? e = [Z] : e.push(Z), r++;
                }
              } else {
                let Z = { instancePath: o + "/cookie-consent", schemaPath: "#/properties/cookie-consent/anyOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: q.properties["cookie-consent"].anyOf[1].propertyNames }, message: "must be string", schema: q.properties["cookie-consent"].anyOf[1].propertyNames.type, parentSchema: q.properties["cookie-consent"].anyOf[1].propertyNames, data: v, propertyName: v };
                e === null ? e = [Z] : e.push(Z), r++;
              }
              var m = E === r;
              if (!m) {
                let Z = { instancePath: o + "/cookie-consent", schemaPath: "#/properties/cookie-consent/anyOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: v, schema: q.properties["cookie-consent"].anyOf[1] }, message: "property name must be valid", schema: q.properties["cookie-consent"].anyOf[1].propertyNames, parentSchema: q.properties["cookie-consent"].anyOf[1], data: w };
                e === null ? e = [Z] : e.push(Z), r++;
              }
            }
            if (w.type !== void 0) {
              let v = w.type, E = r, A = q.properties["cookie-consent"].anyOf[1].properties.type.enum;
              if (!(v === "implied" || v === "express")) {
                let Z = { instancePath: o + "/cookie-consent/type", schemaPath: "#/properties/cookie-consent/anyOf/1/properties/type/enum", keyword: "enum", params: { allowedValues: q.properties["cookie-consent"].anyOf[1].properties.type.enum, schema: q.properties["cookie-consent"].anyOf[1].properties.type }, message: "must be equal to one of the allowed values", schema: q.properties["cookie-consent"].anyOf[1].properties.type.enum, parentSchema: q.properties["cookie-consent"].anyOf[1].properties.type, data: v };
                e === null ? e = [Z] : e.push(Z), r++;
              }
              var R = E === r;
            }
            if (w.style !== void 0) {
              let v = w.style, E = r, A = q.properties["cookie-consent"].anyOf[1].properties.style.enum;
              if (!(v === "simple" || v === "headline" || v === "interstitial" || v === "standalone")) {
                let T = { instancePath: o + "/cookie-consent/style", schemaPath: "#/properties/cookie-consent/anyOf/1/properties/style/enum", keyword: "enum", params: { allowedValues: q.properties["cookie-consent"].anyOf[1].properties.style.enum, schema: q.properties["cookie-consent"].anyOf[1].properties.style }, message: "must be equal to one of the allowed values", schema: q.properties["cookie-consent"].anyOf[1].properties.style.enum, parentSchema: q.properties["cookie-consent"].anyOf[1].properties.style, data: v };
                e === null ? e = [T] : e.push(T), r++;
              }
              var R = E === r;
            }
            if (w.palette !== void 0) {
              let v = w.palette, E = r, A = q.properties["cookie-consent"].anyOf[1].properties.palette.enum;
              if (!(v === "light" || v === "dark")) {
                let T = { instancePath: o + "/cookie-consent/palette", schemaPath: "#/properties/cookie-consent/anyOf/1/properties/palette/enum", keyword: "enum", params: { allowedValues: q.properties["cookie-consent"].anyOf[1].properties.palette.enum, schema: q.properties["cookie-consent"].anyOf[1].properties.palette }, message: "must be equal to one of the allowed values", schema: q.properties["cookie-consent"].anyOf[1].properties.palette.enum, parentSchema: q.properties["cookie-consent"].anyOf[1].properties.palette, data: v };
                e === null ? e = [T] : e.push(T), r++;
              }
              var R = E === r;
            }
            if (w["policy-url"] !== void 0) {
              let v = w["policy-url"], E = r;
              if (typeof v != "string") {
                let T = { instancePath: o + "/cookie-consent/policy-url", schemaPath: "#/properties/cookie-consent/anyOf/1/properties/policy-url/type", keyword: "type", params: { type: "string", schema: q.properties["cookie-consent"].anyOf[1].properties["policy-url"] }, message: "must be string", schema: q.properties["cookie-consent"].anyOf[1].properties["policy-url"].type, parentSchema: q.properties["cookie-consent"].anyOf[1].properties["policy-url"], data: v };
                e === null ? e = [T] : e.push(T), r++;
              }
              let A = r;
              var R = E === r;
            }
            if (w["prefs-text"] !== void 0) {
              let v = w["prefs-text"], E = r;
              if (typeof v != "string") {
                let T = { instancePath: o + "/cookie-consent/prefs-text", schemaPath: "#/properties/cookie-consent/anyOf/1/properties/prefs-text/type", keyword: "type", params: { type: "string", schema: q.properties["cookie-consent"].anyOf[1].properties["prefs-text"] }, message: "must be string", schema: q.properties["cookie-consent"].anyOf[1].properties["prefs-text"].type, parentSchema: q.properties["cookie-consent"].anyOf[1].properties["prefs-text"], data: v };
                e === null ? e = [T] : e.push(T), r++;
              }
              let A = r;
              var R = E === r;
            }
          } else {
            let v = { instancePath: o + "/cookie-consent", schemaPath: "#/properties/cookie-consent/anyOf/1/type", keyword: "type", params: { type: "object", schema: q.properties["cookie-consent"].anyOf[1] }, message: "must be object", schema: q.properties["cookie-consent"].anyOf[1].type, parentSchema: q.properties["cookie-consent"].anyOf[1], data: w };
            e === null ? e = [v] : e.push(v), r++;
          }
          var $ = ot === r;
          N = N || $;
        }
        if (N)
          r = Y, e !== null && (Y ? e.length = Y : e = null);
        else {
          let ot = { instancePath: o + "/cookie-consent", schemaPath: "#/properties/cookie-consent/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: q.properties["cookie-consent"].anyOf, parentSchema: q.properties["cookie-consent"], data: w };
          e === null ? e = [ot] : e.push(ot), r++;
        }
        var d = Nt === r;
      }
      if (l.search !== void 0) {
        let w = l.search, Nt = r, Y = r, N = false, ae = null, nt = r;
        if (typeof w != "boolean") {
          let v = { instancePath: o + "/search", schemaPath: "#/properties/search/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: q.properties.search.oneOf[0] }, message: "must be boolean", schema: q.properties.search.oneOf[0].type, parentSchema: q.properties.search.oneOf[0], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        let J = r;
        var j = nt === r;
        j && (N = true, ae = 0);
        let ot = r, le = r;
        if (w && typeof w == "object" && !Array.isArray(w)) {
          for (let E in w) {
            let A = r, Z = r;
            if (typeof E == "string") {
              if (!ng.test(E)) {
                let T = { instancePath: o + "/search", schemaPath: "#/properties/search/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(collapse_after|collapseAfter|copy_button|copyButton))", schema: q.properties.search.oneOf[1].propertyNames }, message: 'must match pattern "^(?!(collapse_after|collapseAfter|copy_button|copyButton))"', schema: "^(?!(collapse_after|collapseAfter|copy_button|copyButton))", parentSchema: q.properties.search.oneOf[1].propertyNames, data: E, propertyName: E };
                e === null ? e = [T] : e.push(T), r++;
              }
            } else {
              let T = { instancePath: o + "/search", schemaPath: "#/properties/search/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: q.properties.search.oneOf[1].propertyNames }, message: "must be string", schema: q.properties.search.oneOf[1].propertyNames.type, parentSchema: q.properties.search.oneOf[1].propertyNames, data: E, propertyName: E };
              e === null ? e = [T] : e.push(T), r++;
            }
            var V = A === r;
            if (!V) {
              let T = { instancePath: o + "/search", schemaPath: "#/properties/search/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: E, schema: q.properties.search.oneOf[1] }, message: "property name must be valid", schema: q.properties.search.oneOf[1].propertyNames, parentSchema: q.properties.search.oneOf[1], data: w };
              e === null ? e = [T] : e.push(T), r++;
            }
          }
          let v = r;
          for (let E in w)
            if (!(E === "location" || E === "type" || E === "limit" || E === "collapse-after" || E === "copy-button" || E === "algolia")) {
              let A = { instancePath: o + "/search", schemaPath: "#/properties/search/oneOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: E, schema: q.properties.search.oneOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: q.properties.search.oneOf[1], data: w };
              e === null ? e = [A] : e.push(A), r++;
            }
          if (w.location !== void 0) {
            let E = w.location, A = r, Z = q.properties.search.oneOf[1].properties.location.enum;
            if (!(E === "navbar" || E === "sidebar")) {
              let T = { instancePath: o + "/search/location", schemaPath: "#/properties/search/oneOf/1/properties/location/enum", keyword: "enum", params: { allowedValues: q.properties.search.oneOf[1].properties.location.enum, schema: q.properties.search.oneOf[1].properties.location }, message: "must be equal to one of the allowed values", schema: q.properties.search.oneOf[1].properties.location.enum, parentSchema: q.properties.search.oneOf[1].properties.location, data: E };
              e === null ? e = [T] : e.push(T), r++;
            }
            var se = A === r;
          }
          if (w.type !== void 0) {
            let E = w.type, A = r, Z = q.properties.search.oneOf[1].properties.type.enum;
            if (!(E === "overlay" || E === "textbox")) {
              let U = { instancePath: o + "/search/type", schemaPath: "#/properties/search/oneOf/1/properties/type/enum", keyword: "enum", params: { allowedValues: q.properties.search.oneOf[1].properties.type.enum, schema: q.properties.search.oneOf[1].properties.type }, message: "must be equal to one of the allowed values", schema: q.properties.search.oneOf[1].properties.type.enum, parentSchema: q.properties.search.oneOf[1].properties.type, data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            var se = A === r;
          }
          if (w.limit !== void 0) {
            let E = w.limit, A = r;
            if (!(typeof E == "number" && isFinite(E))) {
              let U = { instancePath: o + "/search/limit", schemaPath: "#/properties/search/oneOf/1/properties/limit/type", keyword: "type", params: { type: "number", schema: q.properties.search.oneOf[1].properties.limit }, message: "must be number", schema: q.properties.search.oneOf[1].properties.limit.type, parentSchema: q.properties.search.oneOf[1].properties.limit, data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var se = A === r;
          }
          if (w["collapse-after"] !== void 0) {
            let E = w["collapse-after"], A = r;
            if (!(typeof E == "number" && isFinite(E))) {
              let U = { instancePath: o + "/search/collapse-after", schemaPath: "#/properties/search/oneOf/1/properties/collapse-after/type", keyword: "type", params: { type: "number", schema: q.properties.search.oneOf[1].properties["collapse-after"] }, message: "must be number", schema: q.properties.search.oneOf[1].properties["collapse-after"].type, parentSchema: q.properties.search.oneOf[1].properties["collapse-after"], data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var se = A === r;
          }
          if (w["copy-button"] !== void 0) {
            let E = w["copy-button"], A = r;
            if (typeof E != "boolean") {
              let U = { instancePath: o + "/search/copy-button", schemaPath: "#/properties/search/oneOf/1/properties/copy-button/type", keyword: "type", params: { type: "boolean", schema: q.properties.search.oneOf[1].properties["copy-button"] }, message: "must be boolean", schema: q.properties.search.oneOf[1].properties["copy-button"].type, parentSchema: q.properties.search.oneOf[1].properties["copy-button"], data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var se = A === r;
          }
          if (w.algolia !== void 0) {
            let E = w.algolia, A = r, Z = r;
            if (E && typeof E == "object" && !Array.isArray(E)) {
              for (let re in E) {
                let Fr = r, z = r;
                if (typeof re == "string") {
                  if (!og.test(re)) {
                    let Xr = { instancePath: o + "/search/algolia", schemaPath: "#/properties/search/oneOf/1/properties/algolia/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(index_name|indexName|application_id|applicationId|search_only_api_key|searchOnlyApiKey|analytics_events|analyticsEvents|show_logo|showLogo|index_fields|indexFields))", schema: q.properties.search.oneOf[1].properties.algolia.propertyNames }, message: 'must match pattern "^(?!(index_name|indexName|application_id|applicationId|search_only_api_key|searchOnlyApiKey|analytics_events|analyticsEvents|show_logo|showLogo|index_fields|indexFields))"', schema: "^(?!(index_name|indexName|application_id|applicationId|search_only_api_key|searchOnlyApiKey|analytics_events|analyticsEvents|show_logo|showLogo|index_fields|indexFields))", parentSchema: q.properties.search.oneOf[1].properties.algolia.propertyNames, data: re, propertyName: re };
                    e === null ? e = [Xr] : e.push(Xr), r++;
                  }
                } else {
                  let Xr = { instancePath: o + "/search/algolia", schemaPath: "#/properties/search/oneOf/1/properties/algolia/propertyNames/type", keyword: "type", params: { type: "string", schema: q.properties.search.oneOf[1].properties.algolia.propertyNames }, message: "must be string", schema: q.properties.search.oneOf[1].properties.algolia.propertyNames.type, parentSchema: q.properties.search.oneOf[1].properties.algolia.propertyNames, data: re, propertyName: re };
                  e === null ? e = [Xr] : e.push(Xr), r++;
                }
                var P = Fr === r;
                if (!P) {
                  let Xr = { instancePath: o + "/search/algolia", schemaPath: "#/properties/search/oneOf/1/properties/algolia/propertyNames", keyword: "propertyNames", params: { propertyName: re, schema: q.properties.search.oneOf[1].properties.algolia }, message: "property name must be valid", schema: q.properties.search.oneOf[1].properties.algolia.propertyNames, parentSchema: q.properties.search.oneOf[1].properties.algolia, data: E };
                  e === null ? e = [Xr] : e.push(Xr), r++;
                }
              }
              let U = r;
              for (let re in E)
                if (!(re === "index-name" || re === "application-id" || re === "search-only-api-key" || re === "analytics-events" || re === "show-logo" || re === "index-fields" || re === "params")) {
                  let Fr = { instancePath: o + "/search/algolia", schemaPath: "#/properties/search/oneOf/1/properties/algolia/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: re, schema: q.properties.search.oneOf[1].properties.algolia }, message: "must NOT have additional properties", schema: false, parentSchema: q.properties.search.oneOf[1].properties.algolia, data: E };
                  e === null ? e = [Fr] : e.push(Fr), r++;
                }
              if (E["index-name"] !== void 0) {
                let re = E["index-name"], Fr = r;
                if (typeof re != "string") {
                  let Xr = { instancePath: o + "/search/algolia/index-name", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/index-name/type", keyword: "type", params: { type: "string", schema: q.properties.search.oneOf[1].properties.algolia.properties["index-name"] }, message: "must be string", schema: q.properties.search.oneOf[1].properties.algolia.properties["index-name"].type, parentSchema: q.properties.search.oneOf[1].properties.algolia.properties["index-name"], data: re };
                  e === null ? e = [Xr] : e.push(Xr), r++;
                }
                let z = r;
                var oe = Fr === r;
              }
              if (E["application-id"] !== void 0) {
                let re = E["application-id"], Fr = r;
                if (typeof re != "string") {
                  let W = { instancePath: o + "/search/algolia/application-id", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/application-id/type", keyword: "type", params: { type: "string", schema: q.properties.search.oneOf[1].properties.algolia.properties["application-id"] }, message: "must be string", schema: q.properties.search.oneOf[1].properties.algolia.properties["application-id"].type, parentSchema: q.properties.search.oneOf[1].properties.algolia.properties["application-id"], data: re };
                  e === null ? e = [W] : e.push(W), r++;
                }
                let z = r;
                var oe = Fr === r;
              }
              if (E["search-only-api-key"] !== void 0) {
                let re = E["search-only-api-key"], Fr = r;
                if (typeof re != "string") {
                  let W = { instancePath: o + "/search/algolia/search-only-api-key", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/search-only-api-key/type", keyword: "type", params: { type: "string", schema: q.properties.search.oneOf[1].properties.algolia.properties["search-only-api-key"] }, message: "must be string", schema: q.properties.search.oneOf[1].properties.algolia.properties["search-only-api-key"].type, parentSchema: q.properties.search.oneOf[1].properties.algolia.properties["search-only-api-key"], data: re };
                  e === null ? e = [W] : e.push(W), r++;
                }
                let z = r;
                var oe = Fr === r;
              }
              if (E["analytics-events"] !== void 0) {
                let re = E["analytics-events"], Fr = r;
                if (typeof re != "boolean") {
                  let W = { instancePath: o + "/search/algolia/analytics-events", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/analytics-events/type", keyword: "type", params: { type: "boolean", schema: q.properties.search.oneOf[1].properties.algolia.properties["analytics-events"] }, message: "must be boolean", schema: q.properties.search.oneOf[1].properties.algolia.properties["analytics-events"].type, parentSchema: q.properties.search.oneOf[1].properties.algolia.properties["analytics-events"], data: re };
                  e === null ? e = [W] : e.push(W), r++;
                }
                let z = r;
                var oe = Fr === r;
              }
              if (E["show-logo"] !== void 0) {
                let re = E["show-logo"], Fr = r;
                if (typeof re != "boolean") {
                  let W = { instancePath: o + "/search/algolia/show-logo", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/show-logo/type", keyword: "type", params: { type: "boolean", schema: q.properties.search.oneOf[1].properties.algolia.properties["show-logo"] }, message: "must be boolean", schema: q.properties.search.oneOf[1].properties.algolia.properties["show-logo"].type, parentSchema: q.properties.search.oneOf[1].properties.algolia.properties["show-logo"], data: re };
                  e === null ? e = [W] : e.push(W), r++;
                }
                let z = r;
                var oe = Fr === r;
              }
              if (E["index-fields"] !== void 0) {
                let re = E["index-fields"], Fr = r, z = r;
                if (re && typeof re == "object" && !Array.isArray(re)) {
                  let W = r;
                  for (let C in re)
                    if (!(C === "href" || C === "title" || C === "text" || C === "section")) {
                      let te = { instancePath: o + "/search/algolia/index-fields", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/index-fields/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: C, schema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"] }, message: "must NOT have additional properties", schema: false, parentSchema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"], data: re };
                      e === null ? e = [te] : e.push(te), r++;
                    }
                  if (re.href !== void 0) {
                    let C = re.href, te = r;
                    if (typeof C != "string") {
                      let X = { instancePath: o + "/search/algolia/index-fields/href", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/index-fields/properties/href/type", keyword: "type", params: { type: "string", schema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.href }, message: "must be string", schema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.href.type, parentSchema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.href, data: C };
                      e === null ? e = [X] : e.push(X), r++;
                    }
                    let Br = r;
                    var h = te === r;
                  }
                  if (re.title !== void 0) {
                    let C = re.title, te = r;
                    if (typeof C != "string") {
                      let Yr = { instancePath: o + "/search/algolia/index-fields/title", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/index-fields/properties/title/type", keyword: "type", params: { type: "string", schema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.title }, message: "must be string", schema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.title.type, parentSchema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.title, data: C };
                      e === null ? e = [Yr] : e.push(Yr), r++;
                    }
                    let Br = r;
                    var h = te === r;
                  }
                  if (re.text !== void 0) {
                    let C = re.text, te = r;
                    if (typeof C != "string") {
                      let Yr = { instancePath: o + "/search/algolia/index-fields/text", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/index-fields/properties/text/type", keyword: "type", params: { type: "string", schema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.text }, message: "must be string", schema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.text.type, parentSchema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.text, data: C };
                      e === null ? e = [Yr] : e.push(Yr), r++;
                    }
                    let Br = r;
                    var h = te === r;
                  }
                  if (re.section !== void 0) {
                    let C = re.section, te = r;
                    if (typeof C != "string") {
                      let Yr = { instancePath: o + "/search/algolia/index-fields/section", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/index-fields/properties/section/type", keyword: "type", params: { type: "string", schema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.section }, message: "must be string", schema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.section.type, parentSchema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.section, data: C };
                      e === null ? e = [Yr] : e.push(Yr), r++;
                    }
                    let Br = r;
                    var h = te === r;
                  }
                } else {
                  let W = { instancePath: o + "/search/algolia/index-fields", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/index-fields/type", keyword: "type", params: { type: "object", schema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"] }, message: "must be object", schema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"].type, parentSchema: q.properties.search.oneOf[1].properties.algolia.properties["index-fields"], data: re };
                  e === null ? e = [W] : e.push(W), r++;
                }
                var oe = Fr === r;
              }
              if (E.params !== void 0) {
                let re = E.params, Fr = r, z = r;
                if (!(re && typeof re == "object" && !Array.isArray(re))) {
                  let W = { instancePath: o + "/search/algolia/params", schemaPath: "#/properties/search/oneOf/1/properties/algolia/properties/params/type", keyword: "type", params: { type: "object", schema: q.properties.search.oneOf[1].properties.algolia.properties.params }, message: "must be object", schema: q.properties.search.oneOf[1].properties.algolia.properties.params.type, parentSchema: q.properties.search.oneOf[1].properties.algolia.properties.params, data: re };
                  e === null ? e = [W] : e.push(W), r++;
                }
                var oe = Fr === r;
              }
            } else {
              let U = { instancePath: o + "/search/algolia", schemaPath: "#/properties/search/oneOf/1/properties/algolia/type", keyword: "type", params: { type: "object", schema: q.properties.search.oneOf[1].properties.algolia }, message: "must be object", schema: q.properties.search.oneOf[1].properties.algolia.type, parentSchema: q.properties.search.oneOf[1].properties.algolia, data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            var se = A === r;
          }
        } else {
          let v = { instancePath: o + "/search", schemaPath: "#/properties/search/oneOf/1/type", keyword: "type", params: { type: "object", schema: q.properties.search.oneOf[1] }, message: "must be object", schema: q.properties.search.oneOf[1].type, parentSchema: q.properties.search.oneOf[1], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var j = ot === r;
        if (j && N ? (N = false, ae = [ae, 1]) : j && (N = true, ae = 1), N)
          r = Y, e !== null && (Y ? e.length = Y : e = null);
        else {
          let v = { instancePath: o + "/search", schemaPath: "#/properties/search/oneOf", keyword: "oneOf", params: { passingSchemas: ae, schema: q.properties.search }, message: "must match exactly one schema in oneOf", schema: q.properties.search.oneOf, parentSchema: q.properties.search, data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var d = Nt === r;
      }
      if (l.navbar !== void 0) {
        let w = l.navbar, Nt = r, Y = r, N = false, ae = null, nt = r;
        if (typeof w != "boolean") {
          let v = { instancePath: o + "/navbar", schemaPath: "#/properties/navbar/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: q.properties.navbar.oneOf[0] }, message: "must be boolean", schema: q.properties.navbar.oneOf[0].type, parentSchema: q.properties.navbar.oneOf[0], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        let J = r;
        var f = nt === r;
        f && (N = true, ae = 0);
        let ot = r, le = r;
        if (w && typeof w == "object" && !Array.isArray(w)) {
          for (let v in w) {
            let E = r, A = r;
            if (typeof v == "string") {
              if (!sg.test(v)) {
                let Z = { instancePath: o + "/navbar", schemaPath: "#/properties/navbar/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(collapse_below|collapseBelow))", schema: q.properties.navbar.oneOf[1].propertyNames }, message: 'must match pattern "^(?!(collapse_below|collapseBelow))"', schema: "^(?!(collapse_below|collapseBelow))", parentSchema: q.properties.navbar.oneOf[1].propertyNames, data: v, propertyName: v };
                e === null ? e = [Z] : e.push(Z), r++;
              }
            } else {
              let Z = { instancePath: o + "/navbar", schemaPath: "#/properties/navbar/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: q.properties.navbar.oneOf[1].propertyNames }, message: "must be string", schema: q.properties.navbar.oneOf[1].propertyNames.type, parentSchema: q.properties.navbar.oneOf[1].propertyNames, data: v, propertyName: v };
              e === null ? e = [Z] : e.push(Z), r++;
            }
            var ft = E === r;
            if (!ft) {
              let Z = { instancePath: o + "/navbar", schemaPath: "#/properties/navbar/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: v, schema: q.properties.navbar.oneOf[1] }, message: "property name must be valid", schema: q.properties.navbar.oneOf[1].propertyNames, parentSchema: q.properties.navbar.oneOf[1], data: w };
              e === null ? e = [Z] : e.push(Z), r++;
            }
          }
          if (w.title !== void 0) {
            let v = w.title, E = r, A = r, Z = false, T = null, U = r;
            if (typeof v != "string") {
              let Xr = { instancePath: o + "/navbar/title", schemaPath: "#/properties/navbar/oneOf/1/properties/title/oneOf/0/type", keyword: "type", params: { type: "string", schema: q.properties.navbar.oneOf[1].properties.title.oneOf[0] }, message: "must be string", schema: q.properties.navbar.oneOf[1].properties.title.oneOf[0].type, parentSchema: q.properties.navbar.oneOf[1].properties.title.oneOf[0], data: v };
              e === null ? e = [Xr] : e.push(Xr), r++;
            }
            let re = r;
            var Cr = U === r;
            Cr && (Z = true, T = 0);
            let Fr = r;
            if (typeof v != "boolean") {
              let Xr = { instancePath: o + "/navbar/title", schemaPath: "#/properties/navbar/oneOf/1/properties/title/oneOf/1/type", keyword: "type", params: { type: "boolean", schema: q.properties.navbar.oneOf[1].properties.title.oneOf[1] }, message: "must be boolean", schema: q.properties.navbar.oneOf[1].properties.title.oneOf[1].type, parentSchema: q.properties.navbar.oneOf[1].properties.title.oneOf[1], data: v };
              e === null ? e = [Xr] : e.push(Xr), r++;
            }
            let z = r;
            var Cr = Fr === r;
            if (Cr && Z ? (Z = false, T = [T, 1]) : Cr && (Z = true, T = 1), Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let Xr = { instancePath: o + "/navbar/title", schemaPath: "#/properties/navbar/oneOf/1/properties/title/oneOf", keyword: "oneOf", params: { passingSchemas: T, schema: q.properties.navbar.oneOf[1].properties.title }, message: "must match exactly one schema in oneOf", schema: q.properties.navbar.oneOf[1].properties.title.oneOf, parentSchema: q.properties.navbar.oneOf[1].properties.title, data: v };
              e === null ? e = [Xr] : e.push(Xr), r++;
            }
            var Dr = E === r;
          }
          if (w.logo !== void 0) {
            let v = w.logo, E = r;
            if (typeof v != "string") {
              let T = { instancePath: o + "/navbar/logo", schemaPath: "#/properties/navbar/oneOf/1/properties/logo/type", keyword: "type", params: { type: "string", schema: q.properties.navbar.oneOf[1].properties.logo }, message: "must be string", schema: q.properties.navbar.oneOf[1].properties.logo.type, parentSchema: q.properties.navbar.oneOf[1].properties.logo, data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var Dr = E === r;
          }
          if (w.background !== void 0) {
            let v = w.background, E = r, A = r, Z = false, T = r, U = q.properties.navbar.oneOf[1].properties.background.anyOf[0].enum;
            if (!(v === "primary" || v === "secondary" || v === "success" || v === "danger" || v === "warning" || v === "info" || v === "light" || v === "dark")) {
              let Fr = { instancePath: o + "/navbar/background", schemaPath: "#/properties/navbar/oneOf/1/properties/background/anyOf/0/enum", keyword: "enum", params: { allowedValues: q.properties.navbar.oneOf[1].properties.background.anyOf[0].enum, schema: q.properties.navbar.oneOf[1].properties.background.anyOf[0] }, message: "must be equal to one of the allowed values", schema: q.properties.navbar.oneOf[1].properties.background.anyOf[0].enum, parentSchema: q.properties.navbar.oneOf[1].properties.background.anyOf[0], data: v };
              e === null ? e = [Fr] : e.push(Fr), r++;
            }
            var wr = T === r;
            if (Z = Z || wr, !Z) {
              let Fr = r;
              if (typeof v != "string") {
                let W = { instancePath: o + "/navbar/background", schemaPath: "#/properties/navbar/oneOf/1/properties/background/anyOf/1/type", keyword: "type", params: { type: "string", schema: q.properties.navbar.oneOf[1].properties.background.anyOf[1] }, message: "must be string", schema: q.properties.navbar.oneOf[1].properties.background.anyOf[1].type, parentSchema: q.properties.navbar.oneOf[1].properties.background.anyOf[1], data: v };
                e === null ? e = [W] : e.push(W), r++;
              }
              let z = r;
              var wr = Fr === r;
              Z = Z || wr;
            }
            if (Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let Fr = { instancePath: o + "/navbar/background", schemaPath: "#/properties/navbar/oneOf/1/properties/background/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: q.properties.navbar.oneOf[1].properties.background.anyOf, parentSchema: q.properties.navbar.oneOf[1].properties.background, data: v };
              e === null ? e = [Fr] : e.push(Fr), r++;
            }
            var Dr = E === r;
          }
          if (w.foreground !== void 0) {
            let v = w.foreground, E = r, A = r, Z = false, T = r, U = q.properties.navbar.oneOf[1].properties.foreground.anyOf[0].enum;
            if (!(v === "primary" || v === "secondary" || v === "success" || v === "danger" || v === "warning" || v === "info" || v === "light" || v === "dark")) {
              let Fr = { instancePath: o + "/navbar/foreground", schemaPath: "#/properties/navbar/oneOf/1/properties/foreground/anyOf/0/enum", keyword: "enum", params: { allowedValues: q.properties.navbar.oneOf[1].properties.foreground.anyOf[0].enum, schema: q.properties.navbar.oneOf[1].properties.foreground.anyOf[0] }, message: "must be equal to one of the allowed values", schema: q.properties.navbar.oneOf[1].properties.foreground.anyOf[0].enum, parentSchema: q.properties.navbar.oneOf[1].properties.foreground.anyOf[0], data: v };
              e === null ? e = [Fr] : e.push(Fr), r++;
            }
            var ta = T === r;
            if (Z = Z || ta, !Z) {
              let Fr = r;
              if (typeof v != "string") {
                let W = { instancePath: o + "/navbar/foreground", schemaPath: "#/properties/navbar/oneOf/1/properties/foreground/anyOf/1/type", keyword: "type", params: { type: "string", schema: q.properties.navbar.oneOf[1].properties.foreground.anyOf[1] }, message: "must be string", schema: q.properties.navbar.oneOf[1].properties.foreground.anyOf[1].type, parentSchema: q.properties.navbar.oneOf[1].properties.foreground.anyOf[1], data: v };
                e === null ? e = [W] : e.push(W), r++;
              }
              let z = r;
              var ta = Fr === r;
              Z = Z || ta;
            }
            if (Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let Fr = { instancePath: o + "/navbar/foreground", schemaPath: "#/properties/navbar/oneOf/1/properties/foreground/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: q.properties.navbar.oneOf[1].properties.foreground.anyOf, parentSchema: q.properties.navbar.oneOf[1].properties.foreground, data: v };
              e === null ? e = [Fr] : e.push(Fr), r++;
            }
            var Dr = E === r;
          }
          if (w.search !== void 0) {
            let v = w.search, E = r;
            if (typeof v != "boolean") {
              let T = { instancePath: o + "/navbar/search", schemaPath: "#/properties/navbar/oneOf/1/properties/search/type", keyword: "type", params: { type: "boolean", schema: q.properties.navbar.oneOf[1].properties.search }, message: "must be boolean", schema: q.properties.navbar.oneOf[1].properties.search.type, parentSchema: q.properties.navbar.oneOf[1].properties.search, data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var Dr = E === r;
          }
          if (w.pinned !== void 0) {
            let v = w.pinned, E = r;
            if (typeof v != "boolean") {
              let T = { instancePath: o + "/navbar/pinned", schemaPath: "#/properties/navbar/oneOf/1/properties/pinned/type", keyword: "type", params: { type: "boolean", schema: q.properties.navbar.oneOf[1].properties.pinned }, message: "must be boolean", schema: q.properties.navbar.oneOf[1].properties.pinned.type, parentSchema: q.properties.navbar.oneOf[1].properties.pinned, data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var Dr = E === r;
          }
          if (w.collapse !== void 0) {
            let v = w.collapse, E = r;
            if (typeof v != "boolean") {
              let T = { instancePath: o + "/navbar/collapse", schemaPath: "#/properties/navbar/oneOf/1/properties/collapse/type", keyword: "type", params: { type: "boolean", schema: q.properties.navbar.oneOf[1].properties.collapse }, message: "must be boolean", schema: q.properties.navbar.oneOf[1].properties.collapse.type, parentSchema: q.properties.navbar.oneOf[1].properties.collapse, data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var Dr = E === r;
          }
          if (w["collapse-below"] !== void 0) {
            let v = w["collapse-below"], E = r, A = q.properties.navbar.oneOf[1].properties["collapse-below"].enum;
            if (!(v === "sm" || v === "md" || v === "lg" || v === "xl" || v === "xxl")) {
              let T = { instancePath: o + "/navbar/collapse-below", schemaPath: "#/properties/navbar/oneOf/1/properties/collapse-below/enum", keyword: "enum", params: { allowedValues: q.properties.navbar.oneOf[1].properties["collapse-below"].enum, schema: q.properties.navbar.oneOf[1].properties["collapse-below"] }, message: "must be equal to one of the allowed values", schema: q.properties.navbar.oneOf[1].properties["collapse-below"].enum, parentSchema: q.properties.navbar.oneOf[1].properties["collapse-below"], data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            var Dr = E === r;
          }
          if (w.left !== void 0) {
            let v = w.left, E = r, A = r;
            if (Array.isArray(v)) {
              let T = true, U = v.length;
              for (let re = 0; re < U; re++) {
                let Fr = v[re], z = r;
                wa(Fr, { instancePath: o + "/navbar/left/" + re, parentData: v, parentDataProperty: re, rootData: s }) || (e = e === null ? wa.errors : e.concat(wa.errors), r = e.length);
                var jt = z === r;
                jt || (T = false);
              }
            } else {
              let T = { instancePath: o + "/navbar/left", schemaPath: "#/properties/navbar/oneOf/1/properties/left/type", keyword: "type", params: { type: "array", schema: q.properties.navbar.oneOf[1].properties.left }, message: "must be array", schema: q.properties.navbar.oneOf[1].properties.left.type, parentSchema: q.properties.navbar.oneOf[1].properties.left, data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            var Dr = E === r;
          }
          if (w.right !== void 0) {
            let v = w.right, E = r, A = r;
            if (Array.isArray(v)) {
              let T = true, U = v.length;
              for (let re = 0; re < U; re++) {
                let Fr = v[re], z = r;
                wa(Fr, { instancePath: o + "/navbar/right/" + re, parentData: v, parentDataProperty: re, rootData: s }) || (e = e === null ? wa.errors : e.concat(wa.errors), r = e.length);
                var ee = z === r;
                ee || (T = false);
              }
            } else {
              let T = { instancePath: o + "/navbar/right", schemaPath: "#/properties/navbar/oneOf/1/properties/right/type", keyword: "type", params: { type: "array", schema: q.properties.navbar.oneOf[1].properties.right }, message: "must be array", schema: q.properties.navbar.oneOf[1].properties.right.type, parentSchema: q.properties.navbar.oneOf[1].properties.right, data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            var Dr = E === r;
          }
        } else {
          let v = { instancePath: o + "/navbar", schemaPath: "#/properties/navbar/oneOf/1/type", keyword: "type", params: { type: "object", schema: q.properties.navbar.oneOf[1] }, message: "must be object", schema: q.properties.navbar.oneOf[1].type, parentSchema: q.properties.navbar.oneOf[1], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var f = ot === r;
        if (f && N ? (N = false, ae = [ae, 1]) : f && (N = true, ae = 1), N)
          r = Y, e !== null && (Y ? e.length = Y : e = null);
        else {
          let v = { instancePath: o + "/navbar", schemaPath: "#/properties/navbar/oneOf", keyword: "oneOf", params: { passingSchemas: ae, schema: q.properties.navbar }, message: "must match exactly one schema in oneOf", schema: q.properties.navbar.oneOf, parentSchema: q.properties.navbar, data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var d = Nt === r;
      }
      if (l.sidebar !== void 0) {
        let w = l.sidebar, Nt = r, Y = r, N = false, ae = null, nt = r;
        if (typeof w != "boolean") {
          let re = { instancePath: o + "/sidebar", schemaPath: "#/properties/sidebar/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: q.properties.sidebar.oneOf[0] }, message: "must be boolean", schema: q.properties.sidebar.oneOf[0].type, parentSchema: q.properties.sidebar.oneOf[0], data: w };
          e === null ? e = [re] : e.push(re), r++;
        }
        let J = r;
        var za = nt === r;
        za && (N = true, ae = 0);
        let ot = r, le = r, pe = false, v = null, E = r, A = r;
        if (w && typeof w == "object" && !Array.isArray(w)) {
          for (let re in w) {
            let Fr = r, z = r;
            if (typeof re == "string") {
              if (!vd.test(re)) {
                let Xr = { instancePath: o + "/sidebar", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(collapse_level|collapseLevel))", schema: q.properties.sidebar.oneOf[1].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(collapse_level|collapseLevel))"', schema: "^(?!(collapse_level|collapseLevel))", parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].propertyNames, data: re, propertyName: re };
                e === null ? e = [Xr] : e.push(Xr), r++;
              }
            } else {
              let Xr = { instancePath: o + "/sidebar", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[0].propertyNames }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[0].propertyNames.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].propertyNames, data: re, propertyName: re };
              e === null ? e = [Xr] : e.push(Xr), r++;
            }
            var sa = Fr === r;
            if (!sa) {
              let Xr = { instancePath: o + "/sidebar", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: re, schema: q.properties.sidebar.oneOf[1].oneOf[0] }, message: "property name must be valid", schema: q.properties.sidebar.oneOf[1].oneOf[0].propertyNames, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0], data: w };
              e === null ? e = [Xr] : e.push(Xr), r++;
            }
          }
          if (w.id !== void 0) {
            let re = w.id, Fr = r;
            if (typeof re != "string") {
              let Xr = { instancePath: o + "/sidebar/id", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/id/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.id }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.id.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.id, data: re };
              e === null ? e = [Xr] : e.push(Xr), r++;
            }
            let z = r;
            var _e = Fr === r;
          }
          if (w.title !== void 0) {
            let re = w.title, Fr = r, z = r, Xr = false, W = null, C = r;
            if (typeof re != "string") {
              let G = { instancePath: o + "/sidebar/title", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/title/oneOf/0/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[0] }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[0].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[0], data: re };
              e === null ? e = [G] : e.push(G), r++;
            }
            let te = r;
            var Ma = C === r;
            Ma && (Xr = true, W = 0);
            let Br = r;
            if (typeof re != "boolean") {
              let G = { instancePath: o + "/sidebar/title", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/title/oneOf/1/type", keyword: "type", params: { type: "boolean", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[1] }, message: "must be boolean", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[1].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[1], data: re };
              e === null ? e = [G] : e.push(G), r++;
            }
            let X = r;
            var Ma = Br === r;
            if (Ma && Xr ? (Xr = false, W = [W, 1]) : Ma && (Xr = true, W = 1), Xr)
              r = z, e !== null && (z ? e.length = z : e = null);
            else {
              let G = { instancePath: o + "/sidebar/title", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/title/oneOf", keyword: "oneOf", params: { passingSchemas: W, schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.title }, message: "must match exactly one schema in oneOf", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.title, data: re };
              e === null ? e = [G] : e.push(G), r++;
            }
            var _e = Fr === r;
          }
          if (w.subtitle !== void 0) {
            let re = w.subtitle, Fr = r;
            if (typeof re != "string") {
              let W = { instancePath: o + "/sidebar/subtitle", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/subtitle/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.subtitle }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.subtitle.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.subtitle, data: re };
              e === null ? e = [W] : e.push(W), r++;
            }
            let z = r;
            var _e = Fr === r;
          }
          if (w.logo !== void 0) {
            let re = w.logo, Fr = r;
            if (typeof re != "string") {
              let W = { instancePath: o + "/sidebar/logo", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/logo/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.logo }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.logo.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.logo, data: re };
              e === null ? e = [W] : e.push(W), r++;
            }
            let z = r;
            var _e = Fr === r;
          }
          if (w.search !== void 0) {
            let re = w.search, Fr = r;
            if (typeof re != "boolean") {
              let W = { instancePath: o + "/sidebar/search", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/search/type", keyword: "type", params: { type: "boolean", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.search }, message: "must be boolean", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.search.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.search, data: re };
              e === null ? e = [W] : e.push(W), r++;
            }
            let z = r;
            var _e = Fr === r;
          }
          if (w.tools !== void 0) {
            let re = w.tools, Fr = r, z = r;
            if (Array.isArray(re)) {
              let W = true, C = re.length;
              for (let te = 0; te < C; te++) {
                let Br = re[te], X = r;
                co(Br, { instancePath: o + "/sidebar/tools/" + te, parentData: re, parentDataProperty: te, rootData: s }) || (e = e === null ? co.errors : e.concat(co.errors), r = e.length);
                var fa = X === r;
                fa || (W = false);
              }
            } else {
              let W = { instancePath: o + "/sidebar/tools", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/tools/type", keyword: "type", params: { type: "array", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.tools }, message: "must be array", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.tools.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.tools, data: re };
              e === null ? e = [W] : e.push(W), r++;
            }
            var _e = Fr === r;
          }
          if (w.contents !== void 0) {
            let re = w.contents, Fr = r;
            so(re, { instancePath: o + "/sidebar/contents", parentData: w, parentDataProperty: "contents", rootData: s }) || (e = e === null ? so.errors : e.concat(so.errors), r = e.length);
            var _e = Fr === r;
          }
          if (w.style !== void 0) {
            let re = w.style, Fr = r, z = q.properties.sidebar.oneOf[1].oneOf[0].properties.style.enum;
            if (!(re === "docked" || re === "floating")) {
              let W = { instancePath: o + "/sidebar/style", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/style/enum", keyword: "enum", params: { allowedValues: q.properties.sidebar.oneOf[1].oneOf[0].properties.style.enum, schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.style }, message: "must be equal to one of the allowed values", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.style.enum, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.style, data: re };
              e === null ? e = [W] : e.push(W), r++;
            }
            var _e = Fr === r;
          }
          if (w.background !== void 0) {
            let re = w.background, Fr = r, z = r, Xr = false, W = r, C = q.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[0].enum;
            if (!(re === "primary" || re === "secondary" || re === "success" || re === "danger" || re === "warning" || re === "info" || re === "light" || re === "dark")) {
              let Br = { instancePath: o + "/sidebar/background", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/background/anyOf/0/enum", keyword: "enum", params: { allowedValues: q.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[0].enum, schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[0] }, message: "must be equal to one of the allowed values", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[0].enum, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[0], data: re };
              e === null ? e = [Br] : e.push(Br), r++;
            }
            var ie = W === r;
            if (Xr = Xr || ie, !Xr) {
              let Br = r;
              if (typeof re != "string") {
                let G = { instancePath: o + "/sidebar/background", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/background/anyOf/1/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[1] }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[1].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[1], data: re };
                e === null ? e = [G] : e.push(G), r++;
              }
              let X = r;
              var ie = Br === r;
              Xr = Xr || ie;
            }
            if (Xr)
              r = z, e !== null && (z ? e.length = z : e = null);
            else {
              let Br = { instancePath: o + "/sidebar/background", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/background/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.background, data: re };
              e === null ? e = [Br] : e.push(Br), r++;
            }
            var _e = Fr === r;
          }
          if (w.foreground !== void 0) {
            let re = w.foreground, Fr = r, z = r, Xr = false, W = r, C = q.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[0].enum;
            if (!(re === "primary" || re === "secondary" || re === "success" || re === "danger" || re === "warning" || re === "info" || re === "light" || re === "dark")) {
              let Br = { instancePath: o + "/sidebar/foreground", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/foreground/anyOf/0/enum", keyword: "enum", params: { allowedValues: q.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[0].enum, schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[0] }, message: "must be equal to one of the allowed values", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[0].enum, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[0], data: re };
              e === null ? e = [Br] : e.push(Br), r++;
            }
            var ka = W === r;
            if (Xr = Xr || ka, !Xr) {
              let Br = r;
              if (typeof re != "string") {
                let G = { instancePath: o + "/sidebar/foreground", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/foreground/anyOf/1/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[1] }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[1].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[1], data: re };
                e === null ? e = [G] : e.push(G), r++;
              }
              let X = r;
              var ka = Br === r;
              Xr = Xr || ka;
            }
            if (Xr)
              r = z, e !== null && (z ? e.length = z : e = null);
            else {
              let Br = { instancePath: o + "/sidebar/foreground", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/foreground/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.foreground, data: re };
              e === null ? e = [Br] : e.push(Br), r++;
            }
            var _e = Fr === r;
          }
          if (w.border !== void 0) {
            let re = w.border, Fr = r;
            if (typeof re != "boolean") {
              let W = { instancePath: o + "/sidebar/border", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/border/type", keyword: "type", params: { type: "boolean", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.border }, message: "must be boolean", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.border.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.border, data: re };
              e === null ? e = [W] : e.push(W), r++;
            }
            let z = r;
            var _e = Fr === r;
          }
          if (w.alignment !== void 0) {
            let re = w.alignment, Fr = r, z = q.properties.sidebar.oneOf[1].oneOf[0].properties.alignment.enum;
            if (!(re === "left" || re === "right" || re === "center")) {
              let W = { instancePath: o + "/sidebar/alignment", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/alignment/enum", keyword: "enum", params: { allowedValues: q.properties.sidebar.oneOf[1].oneOf[0].properties.alignment.enum, schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.alignment }, message: "must be equal to one of the allowed values", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.alignment.enum, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.alignment, data: re };
              e === null ? e = [W] : e.push(W), r++;
            }
            var _e = Fr === r;
          }
          if (w["collapse-level"] !== void 0) {
            let re = w["collapse-level"], Fr = r;
            if (!(typeof re == "number" && isFinite(re))) {
              let W = { instancePath: o + "/sidebar/collapse-level", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/collapse-level/type", keyword: "type", params: { type: "number", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties["collapse-level"] }, message: "must be number", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties["collapse-level"].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties["collapse-level"], data: re };
              e === null ? e = [W] : e.push(W), r++;
            }
            let z = r;
            var _e = Fr === r;
          }
          if (w.pinned !== void 0) {
            let re = w.pinned, Fr = r;
            if (typeof re != "boolean") {
              let W = { instancePath: o + "/sidebar/pinned", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/pinned/type", keyword: "type", params: { type: "boolean", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.pinned }, message: "must be boolean", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.pinned.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.pinned, data: re };
              e === null ? e = [W] : e.push(W), r++;
            }
            let z = r;
            var _e = Fr === r;
          }
          if (w.header !== void 0) {
            let re = w.header, Fr = r, z = r, Xr = false, W = null, C = r;
            if (typeof re != "string") {
              let G = { instancePath: o + "/sidebar/header", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/header/oneOf/0/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[0] }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[0].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[0], data: re };
              e === null ? e = [G] : e.push(G), r++;
            }
            let te = r;
            var hn = C === r;
            hn && (Xr = true, W = 0);
            let Br = r, X = r;
            if (Array.isArray(re)) {
              let G = true, Ar = re.length;
              for (let _t = 0; _t < Ar; _t++) {
                let Zr = re[_t], Er = r;
                if (typeof Zr != "string") {
                  let Rr = { instancePath: o + "/sidebar/header/" + _t, schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/header/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1].items }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1].items.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1].items, data: Zr };
                  e === null ? e = [Rr] : e.push(Rr), r++;
                }
                let Wa = r;
                var et = Er === r;
                et || (G = false);
              }
            } else {
              let G = { instancePath: o + "/sidebar/header", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/header/oneOf/1/type", keyword: "type", params: { type: "array", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1] }, message: "must be array", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1], data: re };
              e === null ? e = [G] : e.push(G), r++;
            }
            var hn = Br === r;
            if (hn && Xr ? (Xr = false, W = [W, 1]) : hn && (Xr = true, W = 1), Xr)
              r = z, e !== null && (z ? e.length = z : e = null);
            else {
              let G = { instancePath: o + "/sidebar/header", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/header/oneOf", keyword: "oneOf", params: { passingSchemas: W, schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.header }, message: "must match exactly one schema in oneOf", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.header, data: re };
              e === null ? e = [G] : e.push(G), r++;
            }
            var _e = Fr === r;
          }
          if (w.footer !== void 0) {
            let re = w.footer, Fr = r, z = r, Xr = false, W = null, C = r;
            if (typeof re != "string") {
              let G = { instancePath: o + "/sidebar/footer", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/footer/oneOf/0/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[0] }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[0].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[0], data: re };
              e === null ? e = [G] : e.push(G), r++;
            }
            let te = r;
            var yt = C === r;
            yt && (Xr = true, W = 0);
            let Br = r, X = r;
            if (Array.isArray(re)) {
              let G = true, Ar = re.length;
              for (let _t = 0; _t < Ar; _t++) {
                let Zr = re[_t], Er = r;
                if (typeof Zr != "string") {
                  let Rr = { instancePath: o + "/sidebar/footer/" + _t, schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/footer/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1].items }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1].items.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1].items, data: Zr };
                  e === null ? e = [Rr] : e.push(Rr), r++;
                }
                let Wa = r;
                var _n = Er === r;
                _n || (G = false);
              }
            } else {
              let G = { instancePath: o + "/sidebar/footer", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/footer/oneOf/1/type", keyword: "type", params: { type: "array", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1] }, message: "must be array", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1], data: re };
              e === null ? e = [G] : e.push(G), r++;
            }
            var yt = Br === r;
            if (yt && Xr ? (Xr = false, W = [W, 1]) : yt && (Xr = true, W = 1), Xr)
              r = z, e !== null && (z ? e.length = z : e = null);
            else {
              let G = { instancePath: o + "/sidebar/footer", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/properties/footer/oneOf", keyword: "oneOf", params: { passingSchemas: W, schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.footer }, message: "must match exactly one schema in oneOf", schema: q.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0].properties.footer, data: re };
              e === null ? e = [G] : e.push(G), r++;
            }
            var _e = Fr === r;
          }
        } else {
          let re = { instancePath: o + "/sidebar", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/0/type", keyword: "type", params: { type: "object", schema: q.properties.sidebar.oneOf[1].oneOf[0] }, message: "must be object", schema: q.properties.sidebar.oneOf[1].oneOf[0].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[0], data: w };
          e === null ? e = [re] : e.push(re), r++;
        }
        var st = E === r;
        st && (pe = true, v = 0);
        let Z = r, T = r;
        if (Array.isArray(w)) {
          let re = true, Fr = w.length;
          for (let z = 0; z < Fr; z++) {
            let Xr = w[z], W = r, C = r;
            if (Xr && typeof Xr == "object" && !Array.isArray(Xr)) {
              for (let te in Xr) {
                let Br = r, X = r;
                if (typeof te == "string") {
                  if (!vd.test(te)) {
                    let Yr = { instancePath: o + "/sidebar/" + z, schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(collapse_level|collapseLevel))", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames }, message: 'must match pattern "^(?!(collapse_level|collapseLevel))"', schema: "^(?!(collapse_level|collapseLevel))", parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames, data: te, propertyName: te };
                    e === null ? e = [Yr] : e.push(Yr), r++;
                  }
                } else {
                  let Yr = { instancePath: o + "/sidebar/" + z, schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/propertyNames/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames, data: te, propertyName: te };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                var Et = Br === r;
                if (!Et) {
                  let Yr = { instancePath: o + "/sidebar/" + z, schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/propertyNames", keyword: "propertyNames", params: { propertyName: te, schema: q.properties.sidebar.oneOf[1].oneOf[1].items }, message: "property name must be valid", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items, data: Xr };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
              }
              if (Xr.id !== void 0) {
                let te = Xr.id, Br = r;
                if (typeof te != "string") {
                  let Yr = { instancePath: o + "/sidebar/" + z + "/id", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/id/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.id }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.id.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.id, data: te };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let X = r;
                var ca = Br === r;
              }
              if (Xr.title !== void 0) {
                let te = Xr.title, Br = r, X = r, Yr = false, G = null, Ar = r;
                if (typeof te != "string") {
                  let Rr = { instancePath: o + "/sidebar/" + z + "/title", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/title/oneOf/0/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[0] }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[0].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[0], data: te };
                  e === null ? e = [Rr] : e.push(Rr), r++;
                }
                let _t = r;
                var Gr = Ar === r;
                Gr && (Yr = true, G = 0);
                let Zr = r;
                if (typeof te != "boolean") {
                  let Rr = { instancePath: o + "/sidebar/" + z + "/title", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/title/oneOf/1/type", keyword: "type", params: { type: "boolean", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[1] }, message: "must be boolean", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[1].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[1], data: te };
                  e === null ? e = [Rr] : e.push(Rr), r++;
                }
                let Er = r;
                var Gr = Zr === r;
                if (Gr && Yr ? (Yr = false, G = [G, 1]) : Gr && (Yr = true, G = 1), Yr)
                  r = X, e !== null && (X ? e.length = X : e = null);
                else {
                  let Rr = { instancePath: o + "/sidebar/" + z + "/title", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/title/oneOf", keyword: "oneOf", params: { passingSchemas: G, schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.title }, message: "must match exactly one schema in oneOf", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.title, data: te };
                  e === null ? e = [Rr] : e.push(Rr), r++;
                }
                var ca = Br === r;
              }
              if (Xr.subtitle !== void 0) {
                let te = Xr.subtitle, Br = r;
                if (typeof te != "string") {
                  let G = { instancePath: o + "/sidebar/" + z + "/subtitle", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/subtitle/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.subtitle }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.subtitle.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.subtitle, data: te };
                  e === null ? e = [G] : e.push(G), r++;
                }
                let X = r;
                var ca = Br === r;
              }
              if (Xr.logo !== void 0) {
                let te = Xr.logo, Br = r;
                if (typeof te != "string") {
                  let G = { instancePath: o + "/sidebar/" + z + "/logo", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/logo/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.logo }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.logo.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.logo, data: te };
                  e === null ? e = [G] : e.push(G), r++;
                }
                let X = r;
                var ca = Br === r;
              }
              if (Xr.search !== void 0) {
                let te = Xr.search, Br = r;
                if (typeof te != "boolean") {
                  let G = { instancePath: o + "/sidebar/" + z + "/search", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/search/type", keyword: "type", params: { type: "boolean", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.search }, message: "must be boolean", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.search.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.search, data: te };
                  e === null ? e = [G] : e.push(G), r++;
                }
                let X = r;
                var ca = Br === r;
              }
              if (Xr.tools !== void 0) {
                let te = Xr.tools, Br = r, X = r;
                if (Array.isArray(te)) {
                  let G = true, Ar = te.length;
                  for (let _t = 0; _t < Ar; _t++) {
                    let Zr = te[_t], Er = r;
                    co(Zr, { instancePath: o + "/sidebar/" + z + "/tools/" + _t, parentData: te, parentDataProperty: _t, rootData: s }) || (e = e === null ? co.errors : e.concat(co.errors), r = e.length);
                    var Tt = Er === r;
                    Tt || (G = false);
                  }
                } else {
                  let G = { instancePath: o + "/sidebar/" + z + "/tools", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/tools/type", keyword: "type", params: { type: "array", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.tools }, message: "must be array", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.tools.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.tools, data: te };
                  e === null ? e = [G] : e.push(G), r++;
                }
                var ca = Br === r;
              }
              if (Xr.contents !== void 0) {
                let te = Xr.contents, Br = r;
                so(te, { instancePath: o + "/sidebar/" + z + "/contents", parentData: Xr, parentDataProperty: "contents", rootData: s }) || (e = e === null ? so.errors : e.concat(so.errors), r = e.length);
                var ca = Br === r;
              }
              if (Xr.style !== void 0) {
                let te = Xr.style, Br = r, X = q.properties.sidebar.oneOf[1].oneOf[1].items.properties.style.enum;
                if (!(te === "docked" || te === "floating")) {
                  let G = { instancePath: o + "/sidebar/" + z + "/style", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/style/enum", keyword: "enum", params: { allowedValues: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.style.enum, schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.style }, message: "must be equal to one of the allowed values", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.style.enum, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.style, data: te };
                  e === null ? e = [G] : e.push(G), r++;
                }
                var ca = Br === r;
              }
              if (Xr.background !== void 0) {
                let te = Xr.background, Br = r, X = r, Yr = false, G = r, Ar = q.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[0].enum;
                if (!(te === "primary" || te === "secondary" || te === "success" || te === "danger" || te === "warning" || te === "info" || te === "light" || te === "dark")) {
                  let Zr = { instancePath: o + "/sidebar/" + z + "/background", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/background/anyOf/0/enum", keyword: "enum", params: { allowedValues: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[0].enum, schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[0] }, message: "must be equal to one of the allowed values", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[0].enum, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[0], data: te };
                  e === null ? e = [Zr] : e.push(Zr), r++;
                }
                var bn = G === r;
                if (Yr = Yr || bn, !Yr) {
                  let Zr = r;
                  if (typeof te != "string") {
                    let Rr = { instancePath: o + "/sidebar/" + z + "/background", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/background/anyOf/1/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[1] }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[1].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[1], data: te };
                    e === null ? e = [Rr] : e.push(Rr), r++;
                  }
                  let Er = r;
                  var bn = Zr === r;
                  Yr = Yr || bn;
                }
                if (Yr)
                  r = X, e !== null && (X ? e.length = X : e = null);
                else {
                  let Zr = { instancePath: o + "/sidebar/" + z + "/background", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/background/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.background, data: te };
                  e === null ? e = [Zr] : e.push(Zr), r++;
                }
                var ca = Br === r;
              }
              if (Xr.foreground !== void 0) {
                let te = Xr.foreground, Br = r, X = r, Yr = false, G = r, Ar = q.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[0].enum;
                if (!(te === "primary" || te === "secondary" || te === "success" || te === "danger" || te === "warning" || te === "info" || te === "light" || te === "dark")) {
                  let Zr = { instancePath: o + "/sidebar/" + z + "/foreground", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/foreground/anyOf/0/enum", keyword: "enum", params: { allowedValues: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[0].enum, schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[0] }, message: "must be equal to one of the allowed values", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[0].enum, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[0], data: te };
                  e === null ? e = [Zr] : e.push(Zr), r++;
                }
                var vt = G === r;
                if (Yr = Yr || vt, !Yr) {
                  let Zr = r;
                  if (typeof te != "string") {
                    let Rr = { instancePath: o + "/sidebar/" + z + "/foreground", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/foreground/anyOf/1/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[1] }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[1].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[1], data: te };
                    e === null ? e = [Rr] : e.push(Rr), r++;
                  }
                  let Er = r;
                  var vt = Zr === r;
                  Yr = Yr || vt;
                }
                if (Yr)
                  r = X, e !== null && (X ? e.length = X : e = null);
                else {
                  let Zr = { instancePath: o + "/sidebar/" + z + "/foreground", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/foreground/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground, data: te };
                  e === null ? e = [Zr] : e.push(Zr), r++;
                }
                var ca = Br === r;
              }
              if (Xr.border !== void 0) {
                let te = Xr.border, Br = r;
                if (typeof te != "boolean") {
                  let G = { instancePath: o + "/sidebar/" + z + "/border", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/border/type", keyword: "type", params: { type: "boolean", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.border }, message: "must be boolean", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.border.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.border, data: te };
                  e === null ? e = [G] : e.push(G), r++;
                }
                let X = r;
                var ca = Br === r;
              }
              if (Xr.alignment !== void 0) {
                let te = Xr.alignment, Br = r, X = q.properties.sidebar.oneOf[1].oneOf[1].items.properties.alignment.enum;
                if (!(te === "left" || te === "right" || te === "center")) {
                  let G = { instancePath: o + "/sidebar/" + z + "/alignment", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/alignment/enum", keyword: "enum", params: { allowedValues: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.alignment.enum, schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.alignment }, message: "must be equal to one of the allowed values", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.alignment.enum, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.alignment, data: te };
                  e === null ? e = [G] : e.push(G), r++;
                }
                var ca = Br === r;
              }
              if (Xr["collapse-level"] !== void 0) {
                let te = Xr["collapse-level"], Br = r;
                if (!(typeof te == "number" && isFinite(te))) {
                  let G = { instancePath: o + "/sidebar/" + z + "/collapse-level", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/collapse-level/type", keyword: "type", params: { type: "number", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties["collapse-level"] }, message: "must be number", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties["collapse-level"].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties["collapse-level"], data: te };
                  e === null ? e = [G] : e.push(G), r++;
                }
                let X = r;
                var ca = Br === r;
              }
              if (Xr.pinned !== void 0) {
                let te = Xr.pinned, Br = r;
                if (typeof te != "boolean") {
                  let G = { instancePath: o + "/sidebar/" + z + "/pinned", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/pinned/type", keyword: "type", params: { type: "boolean", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.pinned }, message: "must be boolean", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.pinned.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.pinned, data: te };
                  e === null ? e = [G] : e.push(G), r++;
                }
                let X = r;
                var ca = Br === r;
              }
              if (Xr.header !== void 0) {
                let te = Xr.header, Br = r, X = r, Yr = false, G = null, Ar = r;
                if (typeof te != "string") {
                  let Rr = { instancePath: o + "/sidebar/" + z + "/header", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/header/oneOf/0/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[0] }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[0].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[0], data: te };
                  e === null ? e = [Rr] : e.push(Rr), r++;
                }
                let _t = r;
                var yn = Ar === r;
                yn && (Yr = true, G = 0);
                let Zr = r, Er = r;
                if (Array.isArray(te)) {
                  let Rr = true, Qr = te.length;
                  for (let ya = 0; ya < Qr; ya++) {
                    let pa = te[ya], Nr = r;
                    if (typeof pa != "string") {
                      let Tn = { instancePath: o + "/sidebar/" + z + "/header/" + ya, schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/header/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1].items }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1].items.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1].items, data: pa };
                      e === null ? e = [Tn] : e.push(Tn), r++;
                    }
                    let lo = r;
                    var Va = Nr === r;
                    Va || (Rr = false);
                  }
                } else {
                  let Rr = { instancePath: o + "/sidebar/" + z + "/header", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/header/oneOf/1/type", keyword: "type", params: { type: "array", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1] }, message: "must be array", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1], data: te };
                  e === null ? e = [Rr] : e.push(Rr), r++;
                }
                var yn = Zr === r;
                if (yn && Yr ? (Yr = false, G = [G, 1]) : yn && (Yr = true, G = 1), Yr)
                  r = X, e !== null && (X ? e.length = X : e = null);
                else {
                  let Rr = { instancePath: o + "/sidebar/" + z + "/header", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/header/oneOf", keyword: "oneOf", params: { passingSchemas: G, schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.header }, message: "must match exactly one schema in oneOf", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.header, data: te };
                  e === null ? e = [Rr] : e.push(Rr), r++;
                }
                var ca = Br === r;
              }
              if (Xr.footer !== void 0) {
                let te = Xr.footer, Br = r, X = r, Yr = false, G = null, Ar = r;
                if (typeof te != "string") {
                  let Rr = { instancePath: o + "/sidebar/" + z + "/footer", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/footer/oneOf/0/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[0] }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[0].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[0], data: te };
                  e === null ? e = [Rr] : e.push(Rr), r++;
                }
                let _t = r;
                var Dt = Ar === r;
                Dt && (Yr = true, G = 0);
                let Zr = r, Er = r;
                if (Array.isArray(te)) {
                  let Rr = true, Qr = te.length;
                  for (let ya = 0; ya < Qr; ya++) {
                    let pa = te[ya], Nr = r;
                    if (typeof pa != "string") {
                      let Tn = { instancePath: o + "/sidebar/" + z + "/footer/" + ya, schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/footer/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1].items }, message: "must be string", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1].items.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1].items, data: pa };
                      e === null ? e = [Tn] : e.push(Tn), r++;
                    }
                    let lo = r;
                    var nl = Nr === r;
                    nl || (Rr = false);
                  }
                } else {
                  let Rr = { instancePath: o + "/sidebar/" + z + "/footer", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/footer/oneOf/1/type", keyword: "type", params: { type: "array", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1] }, message: "must be array", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1], data: te };
                  e === null ? e = [Rr] : e.push(Rr), r++;
                }
                var Dt = Zr === r;
                if (Dt && Yr ? (Yr = false, G = [G, 1]) : Dt && (Yr = true, G = 1), Yr)
                  r = X, e !== null && (X ? e.length = X : e = null);
                else {
                  let Rr = { instancePath: o + "/sidebar/" + z + "/footer", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/properties/footer/oneOf", keyword: "oneOf", params: { passingSchemas: G, schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer }, message: "must match exactly one schema in oneOf", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer, data: te };
                  e === null ? e = [Rr] : e.push(Rr), r++;
                }
                var ca = Br === r;
              }
            } else {
              let te = { instancePath: o + "/sidebar/" + z, schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/items/type", keyword: "type", params: { type: "object", schema: q.properties.sidebar.oneOf[1].oneOf[1].items }, message: "must be object", schema: q.properties.sidebar.oneOf[1].oneOf[1].items.type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1].items, data: Xr };
              e === null ? e = [te] : e.push(te), r++;
            }
            var ol = W === r;
            ol || (re = false);
          }
        } else {
          let re = { instancePath: o + "/sidebar", schemaPath: "#/properties/sidebar/oneOf/1/oneOf/1/type", keyword: "type", params: { type: "array", schema: q.properties.sidebar.oneOf[1].oneOf[1] }, message: "must be array", schema: q.properties.sidebar.oneOf[1].oneOf[1].type, parentSchema: q.properties.sidebar.oneOf[1].oneOf[1], data: w };
          e === null ? e = [re] : e.push(re), r++;
        }
        var st = Z === r;
        if (st && pe ? (pe = false, v = [v, 1]) : st && (pe = true, v = 1), pe)
          r = le, e !== null && (le ? e.length = le : e = null);
        else {
          let re = { instancePath: o + "/sidebar", schemaPath: "#/properties/sidebar/oneOf/1/oneOf", keyword: "oneOf", params: { passingSchemas: v, schema: q.properties.sidebar.oneOf[1] }, message: "must match exactly one schema in oneOf", schema: q.properties.sidebar.oneOf[1].oneOf, parentSchema: q.properties.sidebar.oneOf[1], data: w };
          e === null ? e = [re] : e.push(re), r++;
        }
        var za = ot === r;
        if (za && N ? (N = false, ae = [ae, 1]) : za && (N = true, ae = 1), N)
          r = Y, e !== null && (Y ? e.length = Y : e = null);
        else {
          let re = { instancePath: o + "/sidebar", schemaPath: "#/properties/sidebar/oneOf", keyword: "oneOf", params: { passingSchemas: ae, schema: q.properties.sidebar }, message: "must match exactly one schema in oneOf", schema: q.properties.sidebar.oneOf, parentSchema: q.properties.sidebar, data: w };
          e === null ? e = [re] : e.push(re), r++;
        }
        var d = Nt === r;
      }
      if (l["body-header"] !== void 0) {
        let w = l["body-header"], Nt = r;
        if (typeof w != "string") {
          let ae = { instancePath: o + "/body-header", schemaPath: "#/properties/body-header/type", keyword: "type", params: { type: "string", schema: q.properties["body-header"] }, message: "must be string", schema: q.properties["body-header"].type, parentSchema: q.properties["body-header"], data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l["body-footer"] !== void 0) {
        let w = l["body-footer"], Nt = r;
        if (typeof w != "string") {
          let ae = { instancePath: o + "/body-footer", schemaPath: "#/properties/body-footer/type", keyword: "type", params: { type: "string", schema: q.properties["body-footer"] }, message: "must be string", schema: q.properties["body-footer"].type, parentSchema: q.properties["body-footer"], data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l["margin-header"] !== void 0) {
        let w = l["margin-header"], Nt = r, Y = r, N = false, ae = null, nt = r;
        if (typeof w != "string") {
          let v = { instancePath: o + "/margin-header", schemaPath: "#/properties/margin-header/oneOf/0/type", keyword: "type", params: { type: "string", schema: q.properties["margin-header"].oneOf[0] }, message: "must be string", schema: q.properties["margin-header"].oneOf[0].type, parentSchema: q.properties["margin-header"].oneOf[0], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        let J = r;
        var wt = nt === r;
        wt && (N = true, ae = 0);
        let ot = r, le = r;
        if (Array.isArray(w)) {
          let v = true, E = w.length;
          for (let A = 0; A < E; A++) {
            let Z = w[A], T = r;
            if (typeof Z != "string") {
              let re = { instancePath: o + "/margin-header/" + A, schemaPath: "#/properties/margin-header/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: q.properties["margin-header"].oneOf[1].items }, message: "must be string", schema: q.properties["margin-header"].oneOf[1].items.type, parentSchema: q.properties["margin-header"].oneOf[1].items, data: Z };
              e === null ? e = [re] : e.push(re), r++;
            }
            let U = r;
            var ii = T === r;
            ii || (v = false);
          }
        } else {
          let v = { instancePath: o + "/margin-header", schemaPath: "#/properties/margin-header/oneOf/1/type", keyword: "type", params: { type: "array", schema: q.properties["margin-header"].oneOf[1] }, message: "must be array", schema: q.properties["margin-header"].oneOf[1].type, parentSchema: q.properties["margin-header"].oneOf[1], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var wt = ot === r;
        if (wt && N ? (N = false, ae = [ae, 1]) : wt && (N = true, ae = 1), N)
          r = Y, e !== null && (Y ? e.length = Y : e = null);
        else {
          let v = { instancePath: o + "/margin-header", schemaPath: "#/properties/margin-header/oneOf", keyword: "oneOf", params: { passingSchemas: ae, schema: q.properties["margin-header"] }, message: "must match exactly one schema in oneOf", schema: q.properties["margin-header"].oneOf, parentSchema: q.properties["margin-header"], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var d = Nt === r;
      }
      if (l["margin-footer"] !== void 0) {
        let w = l["margin-footer"], Nt = r, Y = r, N = false, ae = null, nt = r;
        if (typeof w != "string") {
          let v = { instancePath: o + "/margin-footer", schemaPath: "#/properties/margin-footer/oneOf/0/type", keyword: "type", params: { type: "string", schema: q.properties["margin-footer"].oneOf[0] }, message: "must be string", schema: q.properties["margin-footer"].oneOf[0].type, parentSchema: q.properties["margin-footer"].oneOf[0], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        let J = r;
        var Jn = nt === r;
        Jn && (N = true, ae = 0);
        let ot = r, le = r;
        if (Array.isArray(w)) {
          let v = true, E = w.length;
          for (let A = 0; A < E; A++) {
            let Z = w[A], T = r;
            if (typeof Z != "string") {
              let re = { instancePath: o + "/margin-footer/" + A, schemaPath: "#/properties/margin-footer/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: q.properties["margin-footer"].oneOf[1].items }, message: "must be string", schema: q.properties["margin-footer"].oneOf[1].items.type, parentSchema: q.properties["margin-footer"].oneOf[1].items, data: Z };
              e === null ? e = [re] : e.push(re), r++;
            }
            let U = r;
            var Bt = T === r;
            Bt || (v = false);
          }
        } else {
          let v = { instancePath: o + "/margin-footer", schemaPath: "#/properties/margin-footer/oneOf/1/type", keyword: "type", params: { type: "array", schema: q.properties["margin-footer"].oneOf[1] }, message: "must be array", schema: q.properties["margin-footer"].oneOf[1].type, parentSchema: q.properties["margin-footer"].oneOf[1], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var Jn = ot === r;
        if (Jn && N ? (N = false, ae = [ae, 1]) : Jn && (N = true, ae = 1), N)
          r = Y, e !== null && (Y ? e.length = Y : e = null);
        else {
          let v = { instancePath: o + "/margin-footer", schemaPath: "#/properties/margin-footer/oneOf", keyword: "oneOf", params: { passingSchemas: ae, schema: q.properties["margin-footer"] }, message: "must match exactly one schema in oneOf", schema: q.properties["margin-footer"].oneOf, parentSchema: q.properties["margin-footer"], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var d = Nt === r;
      }
      if (l["page-navigation"] !== void 0) {
        let w = l["page-navigation"], Nt = r;
        if (typeof w != "boolean") {
          let ae = { instancePath: o + "/page-navigation", schemaPath: "#/properties/page-navigation/type", keyword: "type", params: { type: "boolean", schema: q.properties["page-navigation"] }, message: "must be boolean", schema: q.properties["page-navigation"].type, parentSchema: q.properties["page-navigation"], data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l["page-footer"] !== void 0) {
        let w = l["page-footer"], Nt = r, Y = r, N = false, ae = null, nt = r;
        if (typeof w != "string") {
          let v = { instancePath: o + "/page-footer", schemaPath: "#/properties/page-footer/oneOf/0/type", keyword: "type", params: { type: "string", schema: q.properties["page-footer"].oneOf[0] }, message: "must be string", schema: q.properties["page-footer"].oneOf[0].type, parentSchema: q.properties["page-footer"].oneOf[0], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        let J = r;
        var oo = nt === r;
        oo && (N = true, ae = 0);
        let ot = r, le = r;
        if (w && typeof w == "object" && !Array.isArray(w)) {
          let v = r;
          for (let E in w)
            if (!(E === "left" || E === "right" || E === "center" || E === "border" || E === "background" || E === "foreground")) {
              let A = { instancePath: o + "/page-footer", schemaPath: "#/properties/page-footer/oneOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: E, schema: q.properties["page-footer"].oneOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: q.properties["page-footer"].oneOf[1], data: w };
              e === null ? e = [A] : e.push(A), r++;
            }
          if (w.left !== void 0) {
            let E = w.left, A = r;
            qn(E, { instancePath: o + "/page-footer/left", parentData: w, parentDataProperty: "left", rootData: s }) || (e = e === null ? qn.errors : e.concat(qn.errors), r = e.length);
            var qo = A === r;
          }
          if (w.right !== void 0) {
            let E = w.right, A = r;
            qn(E, { instancePath: o + "/page-footer/right", parentData: w, parentDataProperty: "right", rootData: s }) || (e = e === null ? qn.errors : e.concat(qn.errors), r = e.length);
            var qo = A === r;
          }
          if (w.center !== void 0) {
            let E = w.center, A = r;
            qn(E, { instancePath: o + "/page-footer/center", parentData: w, parentDataProperty: "center", rootData: s }) || (e = e === null ? qn.errors : e.concat(qn.errors), r = e.length);
            var qo = A === r;
          }
          if (w.border !== void 0) {
            let E = w.border, A = r, Z = r, T = false, U = null, re = r;
            if (typeof E != "boolean") {
              let C = { instancePath: o + "/page-footer/border", schemaPath: "#/properties/page-footer/oneOf/1/properties/border/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: q.properties["page-footer"].oneOf[1].properties.border.oneOf[0] }, message: "must be boolean", schema: q.properties["page-footer"].oneOf[1].properties.border.oneOf[0].type, parentSchema: q.properties["page-footer"].oneOf[1].properties.border.oneOf[0], data: E };
              e === null ? e = [C] : e.push(C), r++;
            }
            let Fr = r;
            var Ot = re === r;
            Ot && (T = true, U = 0);
            let z = r;
            if (typeof E != "string") {
              let C = { instancePath: o + "/page-footer/border", schemaPath: "#/properties/page-footer/oneOf/1/properties/border/oneOf/1/type", keyword: "type", params: { type: "string", schema: q.properties["page-footer"].oneOf[1].properties.border.oneOf[1] }, message: "must be string", schema: q.properties["page-footer"].oneOf[1].properties.border.oneOf[1].type, parentSchema: q.properties["page-footer"].oneOf[1].properties.border.oneOf[1], data: E };
              e === null ? e = [C] : e.push(C), r++;
            }
            let Xr = r;
            var Ot = z === r;
            if (Ot && T ? (T = false, U = [U, 1]) : Ot && (T = true, U = 1), T)
              r = Z, e !== null && (Z ? e.length = Z : e = null);
            else {
              let C = { instancePath: o + "/page-footer/border", schemaPath: "#/properties/page-footer/oneOf/1/properties/border/oneOf", keyword: "oneOf", params: { passingSchemas: U, schema: q.properties["page-footer"].oneOf[1].properties.border }, message: "must match exactly one schema in oneOf", schema: q.properties["page-footer"].oneOf[1].properties.border.oneOf, parentSchema: q.properties["page-footer"].oneOf[1].properties.border, data: E };
              e === null ? e = [C] : e.push(C), r++;
            }
            var qo = A === r;
          }
          if (w.background !== void 0) {
            let E = w.background, A = r;
            if (typeof E != "string") {
              let U = { instancePath: o + "/page-footer/background", schemaPath: "#/properties/page-footer/oneOf/1/properties/background/type", keyword: "type", params: { type: "string", schema: q.properties["page-footer"].oneOf[1].properties.background }, message: "must be string", schema: q.properties["page-footer"].oneOf[1].properties.background.type, parentSchema: q.properties["page-footer"].oneOf[1].properties.background, data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var qo = A === r;
          }
          if (w.foreground !== void 0) {
            let E = w.foreground, A = r;
            if (typeof E != "string") {
              let U = { instancePath: o + "/page-footer/foreground", schemaPath: "#/properties/page-footer/oneOf/1/properties/foreground/type", keyword: "type", params: { type: "string", schema: q.properties["page-footer"].oneOf[1].properties.foreground }, message: "must be string", schema: q.properties["page-footer"].oneOf[1].properties.foreground.type, parentSchema: q.properties["page-footer"].oneOf[1].properties.foreground, data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var qo = A === r;
          }
        } else {
          let v = { instancePath: o + "/page-footer", schemaPath: "#/properties/page-footer/oneOf/1/type", keyword: "type", params: { type: "object", schema: q.properties["page-footer"].oneOf[1] }, message: "must be object", schema: q.properties["page-footer"].oneOf[1].type, parentSchema: q.properties["page-footer"].oneOf[1], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var oo = ot === r;
        if (oo && N ? (N = false, ae = [ae, 1]) : oo && (N = true, ae = 1), N)
          r = Y, e !== null && (Y ? e.length = Y : e = null);
        else {
          let v = { instancePath: o + "/page-footer", schemaPath: "#/properties/page-footer/oneOf", keyword: "oneOf", params: { passingSchemas: ae, schema: q.properties["page-footer"] }, message: "must match exactly one schema in oneOf", schema: q.properties["page-footer"].oneOf, parentSchema: q.properties["page-footer"], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var d = Nt === r;
      }
      if (l.image !== void 0) {
        let w = l.image, Nt = r;
        if (typeof w != "string") {
          let ae = { instancePath: o + "/image", schemaPath: "#/properties/image/type", keyword: "type", params: { type: "string", schema: q.properties.image }, message: "must be string", schema: q.properties.image.type, parentSchema: q.properties.image, data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l.comments !== void 0) {
        let w = l.comments, Nt = r;
        Uo(w, { instancePath: o + "/comments", parentData: l, parentDataProperty: "comments", rootData: s }) || (e = e === null ? Uo.errors : e.concat(Uo.errors), r = e.length);
        var d = Nt === r;
      }
      if (l["open-graph"] !== void 0) {
        let w = l["open-graph"], Nt = r, Y = r, N = false, ae = null, nt = r;
        if (typeof w != "boolean") {
          let v = { instancePath: o + "/open-graph", schemaPath: "#/properties/open-graph/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: q.properties["open-graph"].oneOf[0] }, message: "must be boolean", schema: q.properties["open-graph"].oneOf[0].type, parentSchema: q.properties["open-graph"].oneOf[0], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        let J = r;
        var xo = nt === r;
        xo && (N = true, ae = 0);
        let ot = r, le = r;
        if (w && typeof w == "object" && !Array.isArray(w)) {
          for (let E in w) {
            let A = r, Z = r;
            if (typeof E == "string") {
              if (!ig.test(E)) {
                let T = { instancePath: o + "/open-graph", schemaPath: "#/properties/open-graph/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(^(?!^(?!(site_name|siteName)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))", schema: q.properties["open-graph"].oneOf[1].propertyNames }, message: 'must match pattern "^(?!(^(?!^(?!(site_name|siteName)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))"', schema: "^(?!(^(?!^(?!(site_name|siteName)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))", parentSchema: q.properties["open-graph"].oneOf[1].propertyNames, data: E, propertyName: E };
                e === null ? e = [T] : e.push(T), r++;
              }
            } else {
              let T = { instancePath: o + "/open-graph", schemaPath: "#/properties/open-graph/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: q.properties["open-graph"].oneOf[1].propertyNames }, message: "must be string", schema: q.properties["open-graph"].oneOf[1].propertyNames.type, parentSchema: q.properties["open-graph"].oneOf[1].propertyNames, data: E, propertyName: E };
              e === null ? e = [T] : e.push(T), r++;
            }
            var li = A === r;
            if (!li) {
              let T = { instancePath: o + "/open-graph", schemaPath: "#/properties/open-graph/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: E, schema: q.properties["open-graph"].oneOf[1] }, message: "property name must be valid", schema: q.properties["open-graph"].oneOf[1].propertyNames, parentSchema: q.properties["open-graph"].oneOf[1], data: w };
              e === null ? e = [T] : e.push(T), r++;
            }
          }
          let v = r;
          for (let E in w)
            if (!(E === "title" || E === "description" || E === "image" || E === "image-width" || E === "image-height" || E === "locale" || E === "site-name")) {
              let A = { instancePath: o + "/open-graph", schemaPath: "#/properties/open-graph/oneOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: E, schema: q.properties["open-graph"].oneOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: q.properties["open-graph"].oneOf[1], data: w };
              e === null ? e = [A] : e.push(A), r++;
            }
          if (w.title !== void 0) {
            let E = w.title, A = r;
            if (typeof E != "string") {
              let T = { instancePath: o + "/open-graph/title", schemaPath: "#/properties/open-graph/oneOf/1/properties/title/type", keyword: "type", params: { type: "string", schema: q.properties["open-graph"].oneOf[1].properties.title }, message: "must be string", schema: q.properties["open-graph"].oneOf[1].properties.title.type, parentSchema: q.properties["open-graph"].oneOf[1].properties.title, data: E };
              e === null ? e = [T] : e.push(T), r++;
            }
            let Z = r;
            var Pt = A === r;
          }
          if (w.description !== void 0) {
            let E = w.description, A = r;
            if (typeof E != "string") {
              let U = { instancePath: o + "/open-graph/description", schemaPath: "#/properties/open-graph/oneOf/1/properties/description/type", keyword: "type", params: { type: "string", schema: q.properties["open-graph"].oneOf[1].properties.description }, message: "must be string", schema: q.properties["open-graph"].oneOf[1].properties.description.type, parentSchema: q.properties["open-graph"].oneOf[1].properties.description, data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var Pt = A === r;
          }
          if (w.image !== void 0) {
            let E = w.image, A = r;
            if (typeof E != "string") {
              let U = { instancePath: o + "/open-graph/image", schemaPath: "#/properties/open-graph/oneOf/1/properties/image/type", keyword: "type", params: { type: "string", schema: q.properties["open-graph"].oneOf[1].properties.image }, message: "must be string", schema: q.properties["open-graph"].oneOf[1].properties.image.type, parentSchema: q.properties["open-graph"].oneOf[1].properties.image, data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var Pt = A === r;
          }
          if (w["image-width"] !== void 0) {
            let E = w["image-width"], A = r;
            if (!(typeof E == "number" && isFinite(E))) {
              let U = { instancePath: o + "/open-graph/image-width", schemaPath: "#/properties/open-graph/oneOf/1/properties/image-width/type", keyword: "type", params: { type: "number", schema: q.properties["open-graph"].oneOf[1].properties["image-width"] }, message: "must be number", schema: q.properties["open-graph"].oneOf[1].properties["image-width"].type, parentSchema: q.properties["open-graph"].oneOf[1].properties["image-width"], data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var Pt = A === r;
          }
          if (w["image-height"] !== void 0) {
            let E = w["image-height"], A = r;
            if (!(typeof E == "number" && isFinite(E))) {
              let U = { instancePath: o + "/open-graph/image-height", schemaPath: "#/properties/open-graph/oneOf/1/properties/image-height/type", keyword: "type", params: { type: "number", schema: q.properties["open-graph"].oneOf[1].properties["image-height"] }, message: "must be number", schema: q.properties["open-graph"].oneOf[1].properties["image-height"].type, parentSchema: q.properties["open-graph"].oneOf[1].properties["image-height"], data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var Pt = A === r;
          }
          if (w.locale !== void 0) {
            let E = w.locale, A = r;
            if (typeof E != "string") {
              let U = { instancePath: o + "/open-graph/locale", schemaPath: "#/properties/open-graph/oneOf/1/properties/locale/type", keyword: "type", params: { type: "string", schema: q.properties["open-graph"].oneOf[1].properties.locale }, message: "must be string", schema: q.properties["open-graph"].oneOf[1].properties.locale.type, parentSchema: q.properties["open-graph"].oneOf[1].properties.locale, data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var Pt = A === r;
          }
          if (w["site-name"] !== void 0) {
            let E = w["site-name"], A = r;
            if (typeof E != "string") {
              let U = { instancePath: o + "/open-graph/site-name", schemaPath: "#/properties/open-graph/oneOf/1/properties/site-name/type", keyword: "type", params: { type: "string", schema: q.properties["open-graph"].oneOf[1].properties["site-name"] }, message: "must be string", schema: q.properties["open-graph"].oneOf[1].properties["site-name"].type, parentSchema: q.properties["open-graph"].oneOf[1].properties["site-name"], data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var Pt = A === r;
          }
        } else {
          let v = { instancePath: o + "/open-graph", schemaPath: "#/properties/open-graph/oneOf/1/type", keyword: "type", params: { type: "object", schema: q.properties["open-graph"].oneOf[1] }, message: "must be object", schema: q.properties["open-graph"].oneOf[1].type, parentSchema: q.properties["open-graph"].oneOf[1], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var xo = ot === r;
        if (xo && N ? (N = false, ae = [ae, 1]) : xo && (N = true, ae = 1), N)
          r = Y, e !== null && (Y ? e.length = Y : e = null);
        else {
          let v = { instancePath: o + "/open-graph", schemaPath: "#/properties/open-graph/oneOf", keyword: "oneOf", params: { passingSchemas: ae, schema: q.properties["open-graph"] }, message: "must match exactly one schema in oneOf", schema: q.properties["open-graph"].oneOf, parentSchema: q.properties["open-graph"], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var d = Nt === r;
      }
      if (l["twitter-card"] !== void 0) {
        let w = l["twitter-card"], Nt = r, Y = r, N = false, ae = null, nt = r;
        if (typeof w != "boolean") {
          let v = { instancePath: o + "/twitter-card", schemaPath: "#/properties/twitter-card/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: q.properties["twitter-card"].oneOf[0] }, message: "must be boolean", schema: q.properties["twitter-card"].oneOf[0].type, parentSchema: q.properties["twitter-card"].oneOf[0], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        let J = r;
        var bo = nt === r;
        bo && (N = true, ae = 0);
        let ot = r, le = r;
        if (w && typeof w == "object" && !Array.isArray(w)) {
          for (let E in w) {
            let A = r, Z = r;
            if (typeof E == "string") {
              if (!lg.test(E)) {
                let T = { instancePath: o + "/twitter-card", schemaPath: "#/properties/twitter-card/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(^(?!^(?!(card_style|cardStyle)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))", schema: q.properties["twitter-card"].oneOf[1].propertyNames }, message: 'must match pattern "^(?!(^(?!^(?!(card_style|cardStyle)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))"', schema: "^(?!(^(?!^(?!(card_style|cardStyle)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))", parentSchema: q.properties["twitter-card"].oneOf[1].propertyNames, data: E, propertyName: E };
                e === null ? e = [T] : e.push(T), r++;
              }
            } else {
              let T = { instancePath: o + "/twitter-card", schemaPath: "#/properties/twitter-card/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: q.properties["twitter-card"].oneOf[1].propertyNames }, message: "must be string", schema: q.properties["twitter-card"].oneOf[1].propertyNames.type, parentSchema: q.properties["twitter-card"].oneOf[1].propertyNames, data: E, propertyName: E };
              e === null ? e = [T] : e.push(T), r++;
            }
            var sl = A === r;
            if (!sl) {
              let T = { instancePath: o + "/twitter-card", schemaPath: "#/properties/twitter-card/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: E, schema: q.properties["twitter-card"].oneOf[1] }, message: "property name must be valid", schema: q.properties["twitter-card"].oneOf[1].propertyNames, parentSchema: q.properties["twitter-card"].oneOf[1], data: w };
              e === null ? e = [T] : e.push(T), r++;
            }
          }
          let v = r;
          for (let E in w)
            if (!(E === "title" || E === "description" || E === "image" || E === "image-width" || E === "image-height" || E === "card-style" || E === "creator" || E === "site")) {
              let A = { instancePath: o + "/twitter-card", schemaPath: "#/properties/twitter-card/oneOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: E, schema: q.properties["twitter-card"].oneOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: q.properties["twitter-card"].oneOf[1], data: w };
              e === null ? e = [A] : e.push(A), r++;
            }
          if (w.title !== void 0) {
            let E = w.title, A = r;
            if (typeof E != "string") {
              let T = { instancePath: o + "/twitter-card/title", schemaPath: "#/properties/twitter-card/oneOf/1/properties/title/type", keyword: "type", params: { type: "string", schema: q.properties["twitter-card"].oneOf[1].properties.title }, message: "must be string", schema: q.properties["twitter-card"].oneOf[1].properties.title.type, parentSchema: q.properties["twitter-card"].oneOf[1].properties.title, data: E };
              e === null ? e = [T] : e.push(T), r++;
            }
            let Z = r;
            var Ur = A === r;
          }
          if (w.description !== void 0) {
            let E = w.description, A = r;
            if (typeof E != "string") {
              let U = { instancePath: o + "/twitter-card/description", schemaPath: "#/properties/twitter-card/oneOf/1/properties/description/type", keyword: "type", params: { type: "string", schema: q.properties["twitter-card"].oneOf[1].properties.description }, message: "must be string", schema: q.properties["twitter-card"].oneOf[1].properties.description.type, parentSchema: q.properties["twitter-card"].oneOf[1].properties.description, data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var Ur = A === r;
          }
          if (w.image !== void 0) {
            let E = w.image, A = r;
            if (typeof E != "string") {
              let U = { instancePath: o + "/twitter-card/image", schemaPath: "#/properties/twitter-card/oneOf/1/properties/image/type", keyword: "type", params: { type: "string", schema: q.properties["twitter-card"].oneOf[1].properties.image }, message: "must be string", schema: q.properties["twitter-card"].oneOf[1].properties.image.type, parentSchema: q.properties["twitter-card"].oneOf[1].properties.image, data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var Ur = A === r;
          }
          if (w["image-width"] !== void 0) {
            let E = w["image-width"], A = r;
            if (!(typeof E == "number" && isFinite(E))) {
              let U = { instancePath: o + "/twitter-card/image-width", schemaPath: "#/properties/twitter-card/oneOf/1/properties/image-width/type", keyword: "type", params: { type: "number", schema: q.properties["twitter-card"].oneOf[1].properties["image-width"] }, message: "must be number", schema: q.properties["twitter-card"].oneOf[1].properties["image-width"].type, parentSchema: q.properties["twitter-card"].oneOf[1].properties["image-width"], data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var Ur = A === r;
          }
          if (w["image-height"] !== void 0) {
            let E = w["image-height"], A = r;
            if (!(typeof E == "number" && isFinite(E))) {
              let U = { instancePath: o + "/twitter-card/image-height", schemaPath: "#/properties/twitter-card/oneOf/1/properties/image-height/type", keyword: "type", params: { type: "number", schema: q.properties["twitter-card"].oneOf[1].properties["image-height"] }, message: "must be number", schema: q.properties["twitter-card"].oneOf[1].properties["image-height"].type, parentSchema: q.properties["twitter-card"].oneOf[1].properties["image-height"], data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var Ur = A === r;
          }
          if (w["card-style"] !== void 0) {
            let E = w["card-style"], A = r, Z = q.properties["twitter-card"].oneOf[1].properties["card-style"].enum;
            if (!(E === "summary" || E === "summary_large_image")) {
              let U = { instancePath: o + "/twitter-card/card-style", schemaPath: "#/properties/twitter-card/oneOf/1/properties/card-style/enum", keyword: "enum", params: { allowedValues: q.properties["twitter-card"].oneOf[1].properties["card-style"].enum, schema: q.properties["twitter-card"].oneOf[1].properties["card-style"] }, message: "must be equal to one of the allowed values", schema: q.properties["twitter-card"].oneOf[1].properties["card-style"].enum, parentSchema: q.properties["twitter-card"].oneOf[1].properties["card-style"], data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            var Ur = A === r;
          }
          if (w.creator !== void 0) {
            let E = w.creator, A = r;
            if (typeof E != "string") {
              let U = { instancePath: o + "/twitter-card/creator", schemaPath: "#/properties/twitter-card/oneOf/1/properties/creator/type", keyword: "type", params: { type: "string", schema: q.properties["twitter-card"].oneOf[1].properties.creator }, message: "must be string", schema: q.properties["twitter-card"].oneOf[1].properties.creator.type, parentSchema: q.properties["twitter-card"].oneOf[1].properties.creator, data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var Ur = A === r;
          }
          if (w.site !== void 0) {
            let E = w.site, A = r;
            if (typeof E != "string") {
              let U = { instancePath: o + "/twitter-card/site", schemaPath: "#/properties/twitter-card/oneOf/1/properties/site/type", keyword: "type", params: { type: "string", schema: q.properties["twitter-card"].oneOf[1].properties.site }, message: "must be string", schema: q.properties["twitter-card"].oneOf[1].properties.site.type, parentSchema: q.properties["twitter-card"].oneOf[1].properties.site, data: E };
              e === null ? e = [U] : e.push(U), r++;
            }
            let Z = r;
            var Ur = A === r;
          }
        } else {
          let v = { instancePath: o + "/twitter-card", schemaPath: "#/properties/twitter-card/oneOf/1/type", keyword: "type", params: { type: "object", schema: q.properties["twitter-card"].oneOf[1] }, message: "must be object", schema: q.properties["twitter-card"].oneOf[1].type, parentSchema: q.properties["twitter-card"].oneOf[1], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var bo = ot === r;
        if (bo && N ? (N = false, ae = [ae, 1]) : bo && (N = true, ae = 1), N)
          r = Y, e !== null && (Y ? e.length = Y : e = null);
        else {
          let v = { instancePath: o + "/twitter-card", schemaPath: "#/properties/twitter-card/oneOf", keyword: "oneOf", params: { passingSchemas: ae, schema: q.properties["twitter-card"] }, message: "must match exactly one schema in oneOf", schema: q.properties["twitter-card"].oneOf, parentSchema: q.properties["twitter-card"], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var d = Nt === r;
      }
      if (l.subtitle !== void 0) {
        let w = l.subtitle, Nt = r;
        if (typeof w != "string") {
          let ae = { instancePath: o + "/subtitle", schemaPath: "#/properties/subtitle/type", keyword: "type", params: { type: "string", schema: q.properties.subtitle }, message: "must be string", schema: q.properties.subtitle.type, parentSchema: q.properties.subtitle, data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l.author !== void 0) {
        let w = l.author, Nt = r, Y = r, N = false, ae = null, nt = r;
        if (typeof w != "string") {
          let v = { instancePath: o + "/author", schemaPath: "#/properties/author/oneOf/0/type", keyword: "type", params: { type: "string", schema: q.properties.author.oneOf[0] }, message: "must be string", schema: q.properties.author.oneOf[0].type, parentSchema: q.properties.author.oneOf[0], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        let J = r;
        var Q = nt === r;
        Q && (N = true, ae = 0);
        let ot = r, le = r;
        if (Array.isArray(w)) {
          let v = true, E = w.length;
          for (let A = 0; A < E; A++) {
            let Z = w[A], T = r;
            if (typeof Z != "string") {
              let re = { instancePath: o + "/author/" + A, schemaPath: "#/properties/author/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: q.properties.author.oneOf[1].items }, message: "must be string", schema: q.properties.author.oneOf[1].items.type, parentSchema: q.properties.author.oneOf[1].items, data: Z };
              e === null ? e = [re] : e.push(re), r++;
            }
            let U = r;
            var ba = T === r;
            ba || (v = false);
          }
        } else {
          let v = { instancePath: o + "/author", schemaPath: "#/properties/author/oneOf/1/type", keyword: "type", params: { type: "array", schema: q.properties.author.oneOf[1] }, message: "must be array", schema: q.properties.author.oneOf[1].type, parentSchema: q.properties.author.oneOf[1], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var Q = ot === r;
        if (Q && N ? (N = false, ae = [ae, 1]) : Q && (N = true, ae = 1), N)
          r = Y, e !== null && (Y ? e.length = Y : e = null);
        else {
          let v = { instancePath: o + "/author", schemaPath: "#/properties/author/oneOf", keyword: "oneOf", params: { passingSchemas: ae, schema: q.properties.author }, message: "must match exactly one schema in oneOf", schema: q.properties.author.oneOf, parentSchema: q.properties.author, data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var d = Nt === r;
      }
      if (l.date !== void 0) {
        let w = l.date, Nt = r;
        if (typeof w != "string") {
          let ae = { instancePath: o + "/date", schemaPath: "#/properties/date/type", keyword: "type", params: { type: "string", schema: q.properties.date }, message: "must be string", schema: q.properties.date.type, parentSchema: q.properties.date, data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l.abstract !== void 0) {
        let w = l.abstract, Nt = r;
        if (typeof w != "string") {
          let ae = { instancePath: o + "/abstract", schemaPath: "#/properties/abstract/type", keyword: "type", params: { type: "string", schema: q.properties.abstract }, message: "must be string", schema: q.properties.abstract.type, parentSchema: q.properties.abstract, data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l.chapters !== void 0) {
        let w = l.chapters, Nt = r;
        bc(w, { instancePath: o + "/chapters", parentData: l, parentDataProperty: "chapters", rootData: s }) || (e = e === null ? bc.errors : e.concat(bc.errors), r = e.length);
        var d = Nt === r;
      }
      if (l.references !== void 0) {
        let w = l.references, Nt = r;
        if (typeof w != "string") {
          let ae = { instancePath: o + "/references", schemaPath: "#/properties/references/type", keyword: "type", params: { type: "string", schema: q.properties.references }, message: "must be string", schema: q.properties.references.type, parentSchema: q.properties.references, data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l.appendices !== void 0) {
        let w = l.appendices, Nt = r, Y = r;
        if (Array.isArray(w)) {
          let ae = true, nt = w.length;
          for (let J = 0; J < nt; J++) {
            let ot = w[J], le = r;
            yc(ot, { instancePath: o + "/appendices/" + J, parentData: w, parentDataProperty: J, rootData: s }) || (e = e === null ? yc.errors : e.concat(yc.errors), r = e.length);
            var $r = le === r;
            $r || (ae = false);
          }
        } else {
          let ae = { instancePath: o + "/appendices", schemaPath: "#/properties/appendices/type", keyword: "type", params: { type: "array", schema: q.properties.appendices }, message: "must be array", schema: q.properties.appendices.type, parentSchema: q.properties.appendices, data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        var d = Nt === r;
      }
      if (l["output-file"] !== void 0) {
        let w = l["output-file"], Nt = r;
        if (typeof w != "string") {
          let ae = { instancePath: o + "/output-file", schemaPath: "#/properties/output-file/type", keyword: "type", params: { type: "string", schema: q.properties["output-file"] }, message: "must be string", schema: q.properties["output-file"].type, parentSchema: q.properties["output-file"], data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l["cover-image"] !== void 0) {
        let w = l["cover-image"], Nt = r;
        if (typeof w != "string") {
          let ae = { instancePath: o + "/cover-image", schemaPath: "#/properties/cover-image/type", keyword: "type", params: { type: "string", schema: q.properties["cover-image"] }, message: "must be string", schema: q.properties["cover-image"].type, parentSchema: q.properties["cover-image"], data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        let Y = r;
        var d = Nt === r;
      }
      if (l.sharing !== void 0) {
        let w = l.sharing, Nt = r, Y = r, N = false, ae = null, nt = r, J = q.properties.sharing.oneOf[0].enum;
        if (!(w === "twitter" || w === "facebook" || w === "linkedin")) {
          let v = { instancePath: o + "/sharing", schemaPath: "#/properties/sharing/oneOf/0/enum", keyword: "enum", params: { allowedValues: q.properties.sharing.oneOf[0].enum, schema: q.properties.sharing.oneOf[0] }, message: "must be equal to one of the allowed values", schema: q.properties.sharing.oneOf[0].enum, parentSchema: q.properties.sharing.oneOf[0], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var kt = nt === r;
        kt && (N = true, ae = 0);
        let ot = r, le = r;
        if (Array.isArray(w)) {
          let v = true, E = w.length;
          for (let A = 0; A < E; A++) {
            let Z = w[A], T = r, U = q.properties.sharing.oneOf[1].items.enum;
            if (!(Z === "twitter" || Z === "facebook" || Z === "linkedin")) {
              let re = { instancePath: o + "/sharing/" + A, schemaPath: "#/properties/sharing/oneOf/1/items/enum", keyword: "enum", params: { allowedValues: q.properties.sharing.oneOf[1].items.enum, schema: q.properties.sharing.oneOf[1].items }, message: "must be equal to one of the allowed values", schema: q.properties.sharing.oneOf[1].items.enum, parentSchema: q.properties.sharing.oneOf[1].items, data: Z };
              e === null ? e = [re] : e.push(re), r++;
            }
            var $t = T === r;
            $t || (v = false);
          }
        } else {
          let v = { instancePath: o + "/sharing", schemaPath: "#/properties/sharing/oneOf/1/type", keyword: "type", params: { type: "array", schema: q.properties.sharing.oneOf[1] }, message: "must be array", schema: q.properties.sharing.oneOf[1].type, parentSchema: q.properties.sharing.oneOf[1], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var kt = ot === r;
        if (kt && N ? (N = false, ae = [ae, 1]) : kt && (N = true, ae = 1), N)
          r = Y, e !== null && (Y ? e.length = Y : e = null);
        else {
          let v = { instancePath: o + "/sharing", schemaPath: "#/properties/sharing/oneOf", keyword: "oneOf", params: { passingSchemas: ae, schema: q.properties.sharing }, message: "must match exactly one schema in oneOf", schema: q.properties.sharing.oneOf, parentSchema: q.properties.sharing, data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var d = Nt === r;
      }
      if (l.downloads !== void 0) {
        let w = l.downloads, Nt = r, Y = r, N = false, ae = null, nt = r, J = q.properties.downloads.oneOf[0].enum;
        if (!(w === "pdf" || w === "epub" || w === "docx")) {
          let v = { instancePath: o + "/downloads", schemaPath: "#/properties/downloads/oneOf/0/enum", keyword: "enum", params: { allowedValues: q.properties.downloads.oneOf[0].enum, schema: q.properties.downloads.oneOf[0] }, message: "must be equal to one of the allowed values", schema: q.properties.downloads.oneOf[0].enum, parentSchema: q.properties.downloads.oneOf[0], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var Jr = nt === r;
        Jr && (N = true, ae = 0);
        let ot = r, le = r;
        if (Array.isArray(w)) {
          let v = true, E = w.length;
          for (let A = 0; A < E; A++) {
            let Z = w[A], T = r, U = q.properties.downloads.oneOf[1].items.enum;
            if (!(Z === "pdf" || Z === "epub" || Z === "docx")) {
              let re = { instancePath: o + "/downloads/" + A, schemaPath: "#/properties/downloads/oneOf/1/items/enum", keyword: "enum", params: { allowedValues: q.properties.downloads.oneOf[1].items.enum, schema: q.properties.downloads.oneOf[1].items }, message: "must be equal to one of the allowed values", schema: q.properties.downloads.oneOf[1].items.enum, parentSchema: q.properties.downloads.oneOf[1].items, data: Z };
              e === null ? e = [re] : e.push(re), r++;
            }
            var io = T === r;
            io || (v = false);
          }
        } else {
          let v = { instancePath: o + "/downloads", schemaPath: "#/properties/downloads/oneOf/1/type", keyword: "type", params: { type: "array", schema: q.properties.downloads.oneOf[1] }, message: "must be array", schema: q.properties.downloads.oneOf[1].type, parentSchema: q.properties.downloads.oneOf[1], data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var Jr = ot === r;
        if (Jr && N ? (N = false, ae = [ae, 1]) : Jr && (N = true, ae = 1), N)
          r = Y, e !== null && (Y ? e.length = Y : e = null);
        else {
          let v = { instancePath: o + "/downloads", schemaPath: "#/properties/downloads/oneOf", keyword: "oneOf", params: { passingSchemas: ae, schema: q.properties.downloads }, message: "must match exactly one schema in oneOf", schema: q.properties.downloads.oneOf, parentSchema: q.properties.downloads, data: w };
          e === null ? e = [v] : e.push(v), r++;
        }
        var d = Nt === r;
      }
      if (l.tools !== void 0) {
        let w = l.tools, Nt = r, Y = r;
        if (Array.isArray(w)) {
          let ae = true, nt = w.length;
          for (let J = 0; J < nt; J++) {
            let ot = w[J], le = r;
            wa(ot, { instancePath: o + "/tools/" + J, parentData: w, parentDataProperty: J, rootData: s }) || (e = e === null ? wa.errors : e.concat(wa.errors), r = e.length);
            var an = le === r;
            an || (ae = false);
          }
        } else {
          let ae = { instancePath: o + "/tools", schemaPath: "#/properties/tools/type", keyword: "type", params: { type: "array", schema: q.properties.tools }, message: "must be array", schema: q.properties.tools.type, parentSchema: q.properties.tools, data: w };
          e === null ? e = [ae] : e.push(ae), r++;
        }
        var d = Nt === r;
      }
    } else {
      let w = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: q }, message: "must be object", schema: q.type, parentSchema: q, data: l };
      e === null ? e = [w] : e.push(w), r++;
    }
    return C1.errors = e, r === 0;
  }
  B["front-matter-execute"] = si;
  var Jl = { type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(daemon_restart|daemonRestart|keep_md|keepMd|keep_ipynb|keepIpynb))" }, $id: "front-matter-execute" }, E0 = new RegExp("^(?!(daemon_restart|daemonRestart|keep_md|keepMd|keep_ipynb|keepIpynb))", "u");
  function si(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let u in l) {
        let y = r, g = r;
        if (typeof u == "string") {
          if (!E0.test(u)) {
            let x = { instancePath: o, schemaPath: "#/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(daemon_restart|daemonRestart|keep_md|keepMd|keep_ipynb|keepIpynb))", schema: Jl.propertyNames }, message: 'must match pattern "^(?!(daemon_restart|daemonRestart|keep_md|keepMd|keep_ipynb|keepIpynb))"', schema: "^(?!(daemon_restart|daemonRestart|keep_md|keepMd|keep_ipynb|keepIpynb))", parentSchema: Jl.propertyNames, data: u, propertyName: u };
            e === null ? e = [x] : e.push(x), r++;
          }
        } else {
          let x = { instancePath: o, schemaPath: "#/propertyNames/type", keyword: "type", params: { type: "string", schema: Jl.propertyNames }, message: "must be string", schema: Jl.propertyNames.type, parentSchema: Jl.propertyNames, data: u, propertyName: u };
          e === null ? e = [x] : e.push(x), r++;
        }
        var c = y === r;
        if (!c) {
          let x = { instancePath: o, schemaPath: "#/propertyNames", keyword: "propertyNames", params: { propertyName: u, schema: Jl }, message: "property name must be valid", schema: Jl.propertyNames, parentSchema: Jl, data: l };
          e === null ? e = [x] : e.push(x), r++;
        }
      }
      if (l.eval !== void 0) {
        let u = l.eval, y = r;
        ue(u, { instancePath: o + "/eval", parentData: l, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
        var d = y === r;
      }
      if (l.echo !== void 0) {
        let u = l.echo, y = r;
        de(u, { instancePath: o + "/echo", parentData: l, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
        var d = y === r;
      }
      if (l.output !== void 0) {
        let u = l.output, y = r;
        fe(u, { instancePath: o + "/output", parentData: l, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
        var d = y === r;
      }
      if (l.warning !== void 0) {
        let u = l.warning, y = r;
        me(u, { instancePath: o + "/warning", parentData: l, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
        var d = y === r;
      }
      if (l.error !== void 0) {
        let u = l.error, y = r;
        ge(u, { instancePath: o + "/error", parentData: l, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
        var d = y === r;
      }
      if (l.include !== void 0) {
        let u = l.include, y = r;
        he(u, { instancePath: o + "/include", parentData: l, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
        var d = y === r;
      }
      if (l.engine !== void 0) {
        let u = l.engine, y = r;
        qe(u, { instancePath: o + "/engine", parentData: l, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
        var d = y === r;
      }
      if (l.jupyter !== void 0) {
        let u = l.jupyter, y = r;
        Oe(u, { instancePath: o + "/jupyter", parentData: l, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
        var d = y === r;
      }
      if (l.knitr !== void 0) {
        let u = l.knitr, y = r;
        Pe(u, { instancePath: o + "/knitr", parentData: l, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
        var d = y === r;
      }
      if (l.cache !== void 0) {
        let u = l.cache, y = r;
        xe(u, { instancePath: o + "/cache", parentData: l, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
        var d = y === r;
      }
      if (l.freeze !== void 0) {
        let u = l.freeze, y = r;
        ke(u, { instancePath: o + "/freeze", parentData: l, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
        var d = y === r;
      }
      if (l.server !== void 0) {
        let u = l.server, y = r;
        De(u, { instancePath: o + "/server", parentData: l, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
        var d = y === r;
      }
      if (l.daemon !== void 0) {
        let u = l.daemon, y = r;
        we(u, { instancePath: o + "/daemon", parentData: l, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
        var d = y === r;
      }
      if (l["daemon-restart"] !== void 0) {
        let u = l["daemon-restart"], y = r;
        $e(u, { instancePath: o + "/daemon-restart", parentData: l, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
        var d = y === r;
      }
      if (l.enabled !== void 0) {
        let u = l.enabled, y = r;
        Se(u, { instancePath: o + "/enabled", parentData: l, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
        var d = y === r;
      }
      if (l.ipynb !== void 0) {
        let u = l.ipynb, y = r;
        Fe(u, { instancePath: o + "/ipynb", parentData: l, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
        var d = y === r;
      }
      if (l.debug !== void 0) {
        let u = l.debug, y = r;
        Ae(u, { instancePath: o + "/debug", parentData: l, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
        var d = y === r;
      }
      if (l["keep-md"] !== void 0) {
        let u = l["keep-md"], y = r;
        Be(u, { instancePath: o + "/keep-md", parentData: l, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
        var d = y === r;
      }
      if (l["keep-ipynb"] !== void 0) {
        let u = l["keep-ipynb"], y = r;
        je(u, { instancePath: o + "/keep-ipynb", parentData: l, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
        var d = y === r;
      }
    } else {
      let u = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: Jl }, message: "must be object", schema: Jl.type, parentSchema: Jl, data: l };
      e === null ? e = [u] : e.push(u), r++;
    }
    return si.errors = e, r === 0;
  }
  B["front-matter-format"] = _u;
  var p = { oneOf: [{ oneOf: [{ type: "string", pattern: "^asciidoc(\\+.+)?$", description: "be 'asciidoc'" }, { type: "string", pattern: "^asciidoctor(\\+.+)?$", description: "be 'asciidoctor'" }, { type: "string", pattern: "^beamer(\\+.+)?$", description: "be 'beamer'" }, { type: "string", pattern: "^biblatex(\\+.+)?$", description: "be 'biblatex'" }, { type: "string", pattern: "^bibtex(\\+.+)?$", description: "be 'bibtex'" }, { type: "string", pattern: "^commonmark(\\+.+)?$", description: "be 'commonmark'" }, { type: "string", pattern: "^commonmark_x(\\+.+)?$", description: "be 'commonmark_x'" }, { type: "string", pattern: "^context(\\+.+)?$", description: "be 'context'" }, { type: "string", pattern: "^csljson(\\+.+)?$", description: "be 'csljson'" }, { type: "string", pattern: "^docbook(\\+.+)?$", description: "be 'docbook'" }, { type: "string", pattern: "^docbook4(\\+.+)?$", description: "be 'docbook4'" }, { type: "string", pattern: "^docbook5(\\+.+)?$", description: "be 'docbook5'" }, { type: "string", pattern: "^docx(\\+.+)?$", description: "be 'docx'" }, { type: "string", pattern: "^dokuwiki(\\+.+)?$", description: "be 'dokuwiki'" }, { type: "string", pattern: "^dzslides(\\+.+)?$", description: "be 'dzslides'" }, { type: "string", pattern: "^epub(\\+.+)?$", description: "be 'epub'" }, { type: "string", pattern: "^epub2(\\+.+)?$", description: "be 'epub2'" }, { type: "string", pattern: "^epub3(\\+.+)?$", description: "be 'epub3'" }, { type: "string", pattern: "^fb2(\\+.+)?$", description: "be 'fb2'" }, { type: "string", pattern: "^gfm(\\+.+)?$", description: "be 'gfm'" }, { type: "string", pattern: "^haddock(\\+.+)?$", description: "be 'haddock'" }, { type: "string", pattern: "^html(\\+.+)?$", description: "be 'html'" }, { type: "string", pattern: "^html4(\\+.+)?$", description: "be 'html4'" }, { type: "string", pattern: "^html5(\\+.+)?$", description: "be 'html5'" }, { type: "string", pattern: "^icml(\\+.+)?$", description: "be 'icml'" }, { type: "string", pattern: "^ipynb(\\+.+)?$", description: "be 'ipynb'" }, { type: "string", pattern: "^jats(\\+.+)?$", description: "be 'jats'" }, { type: "string", pattern: "^jats_archiving(\\+.+)?$", description: "be 'jats_archiving'" }, { type: "string", pattern: "^jats_articleauthoring(\\+.+)?$", description: "be 'jats_articleauthoring'" }, { type: "string", pattern: "^jats_publishing(\\+.+)?$", description: "be 'jats_publishing'" }, { type: "string", pattern: "^jira(\\+.+)?$", description: "be 'jira'" }, { type: "string", pattern: "^json(\\+.+)?$", description: "be 'json'" }, { type: "string", pattern: "^latex(\\+.+)?$", description: "be 'latex'" }, { type: "string", pattern: "^man(\\+.+)?$", description: "be 'man'" }, { type: "string", pattern: "^markdown(\\+.+)?$", description: "be 'markdown'" }, { type: "string", pattern: "^markdown_github(\\+.+)?$", description: "be 'markdown_github'" }, { type: "string", pattern: "^markdown_mmd(\\+.+)?$", description: "be 'markdown_mmd'" }, { type: "string", pattern: "^markdown_phpextra(\\+.+)?$", description: "be 'markdown_phpextra'" }, { type: "string", pattern: "^markdown_strict(\\+.+)?$", description: "be 'markdown_strict'" }, { type: "string", pattern: "^markua(\\+.+)?$", description: "be 'markua'" }, { type: "string", pattern: "^mediawiki(\\+.+)?$", description: "be 'mediawiki'" }, { type: "string", pattern: "^ms(\\+.+)?$", description: "be 'ms'" }, { type: "string", pattern: "^muse(\\+.+)?$", description: "be 'muse'" }, { type: "string", pattern: "^native(\\+.+)?$", description: "be 'native'" }, { type: "string", pattern: "^odt(\\+.+)?$", description: "be 'odt'" }, { type: "string", pattern: "^opendocument(\\+.+)?$", description: "be 'opendocument'" }, { type: "string", pattern: "^opml(\\+.+)?$", description: "be 'opml'" }, { type: "string", pattern: "^org(\\+.+)?$", description: "be 'org'" }, { type: "string", pattern: "^pdf(\\+.+)?$", description: "be 'pdf'" }, { type: "string", pattern: "^plain(\\+.+)?$", description: "be 'plain'" }, { type: "string", pattern: "^pptx(\\+.+)?$", description: "be 'pptx'" }, { type: "string", pattern: "^revealjs(\\+.+)?$", description: "be 'revealjs'" }, { type: "string", pattern: "^rst(\\+.+)?$", description: "be 'rst'" }, { type: "string", pattern: "^rtf(\\+.+)?$", description: "be 'rtf'" }, { type: "string", pattern: "^s5(\\+.+)?$", description: "be 's5'" }, { type: "string", pattern: "^slideous(\\+.+)?$", description: "be 'slideous'" }, { type: "string", pattern: "^slidy(\\+.+)?$", description: "be 'slidy'" }, { type: "string", pattern: "^tei(\\+.+)?$", description: "be 'tei'" }, { type: "string", pattern: "^texinfo(\\+.+)?$", description: "be 'texinfo'" }, { type: "string", pattern: "^textile(\\+.+)?$", description: "be 'textile'" }, { type: "string", pattern: "^xwiki(\\+.+)?$", description: "be 'xwiki'" }, { type: "string", pattern: "^zimwiki(\\+.+)?$", description: "be 'zimwiki'" }], description: "be the name of a pandoc-supported output format" }, { type: "string", pattern: "^hugo(\\+.+)?$", description: "be 'hugo'" }, { allOf: [{ type: "object", description: "be an object", properties: {}, patternProperties: { "^asciidoc(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, keywords: { $ref: "quarto-resource-document-metadata-keywords", description: "quarto-resource-document-metadata-keywords" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^asciidoctor(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, keywords: { $ref: "quarto-resource-document-metadata-keywords", description: "quarto-resource-document-metadata-keywords" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^beamer(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "fig-env": { $ref: "quarto-resource-cell-figure-fig-env", description: "quarto-resource-cell-figure-fig-env" }, "fig-pos": { $ref: "quarto-resource-cell-figure-fig-pos", description: "quarto-resource-cell-figure-fig-pos" }, "cap-location": { $ref: "quarto-resource-cell-pagelayout-cap-location", description: "quarto-resource-cell-pagelayout-cap-location" }, "fig-cap-location": { $ref: "quarto-resource-cell-pagelayout-fig-cap-location", description: "quarto-resource-cell-pagelayout-fig-cap-location" }, "tbl-cap-location": { $ref: "quarto-resource-cell-pagelayout-tbl-cap-location", description: "quarto-resource-cell-pagelayout-tbl-cap-location" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, institute: { $ref: "quarto-resource-document-attributes-institute", description: "quarto-resource-document-attributes-institute" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, thanks: { $ref: "quarto-resource-document-attributes-thanks", description: "quarto-resource-document-attributes-thanks" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, listings: { $ref: "quarto-resource-document-code-listings", description: "quarto-resource-document-code-listings" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, linkcolor: { $ref: "quarto-resource-document-colors-linkcolor", description: "quarto-resource-document-colors-linkcolor" }, filecolor: { $ref: "quarto-resource-document-colors-filecolor", description: "quarto-resource-document-colors-filecolor" }, citecolor: { $ref: "quarto-resource-document-colors-citecolor", description: "quarto-resource-document-colors-citecolor" }, urlcolor: { $ref: "quarto-resource-document-colors-urlcolor", description: "quarto-resource-document-colors-urlcolor" }, toccolor: { $ref: "quarto-resource-document-colors-toccolor", description: "quarto-resource-document-colors-toccolor" }, colorlinks: { $ref: "quarto-resource-document-colors-colorlinks", description: "quarto-resource-document-colors-colorlinks" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, mainfont: { $ref: "quarto-resource-document-fonts-mainfont", description: "quarto-resource-document-fonts-mainfont" }, monofont: { $ref: "quarto-resource-document-fonts-monofont", description: "quarto-resource-document-fonts-monofont" }, fontsize: { $ref: "quarto-resource-document-fonts-fontsize", description: "quarto-resource-document-fonts-fontsize" }, fontenc: { $ref: "quarto-resource-document-fonts-fontenc", description: "quarto-resource-document-fonts-fontenc" }, fontfamily: { $ref: "quarto-resource-document-fonts-fontfamily", description: "quarto-resource-document-fonts-fontfamily" }, fontfamilyoptions: { $ref: "quarto-resource-document-fonts-fontfamilyoptions", description: "quarto-resource-document-fonts-fontfamilyoptions" }, sansfont: { $ref: "quarto-resource-document-fonts-sansfont", description: "quarto-resource-document-fonts-sansfont" }, mathfont: { $ref: "quarto-resource-document-fonts-mathfont", description: "quarto-resource-document-fonts-mathfont" }, CJKmainfont: { $ref: "quarto-resource-document-fonts-CJKmainfont", description: "quarto-resource-document-fonts-CJKmainfont" }, mainfontoptions: { $ref: "quarto-resource-document-fonts-mainfontoptions", description: "quarto-resource-document-fonts-mainfontoptions" }, sansfontoptions: { $ref: "quarto-resource-document-fonts-sansfontoptions", description: "quarto-resource-document-fonts-sansfontoptions" }, monofontoptions: { $ref: "quarto-resource-document-fonts-monofontoptions", description: "quarto-resource-document-fonts-monofontoptions" }, mathfontoptions: { $ref: "quarto-resource-document-fonts-mathfontoptions", description: "quarto-resource-document-fonts-mathfontoptions" }, CJKoptions: { $ref: "quarto-resource-document-fonts-CJKoptions", description: "quarto-resource-document-fonts-CJKoptions" }, microtypeoptions: { $ref: "quarto-resource-document-fonts-microtypeoptions", description: "quarto-resource-document-fonts-microtypeoptions" }, linestretch: { $ref: "quarto-resource-document-fonts-linestretch", description: "quarto-resource-document-fonts-linestretch" }, "links-as-notes": { $ref: "quarto-resource-document-footnotes-links-as-notes", description: "quarto-resource-document-footnotes-links-as-notes" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "latex-auto-mk": { $ref: "quarto-resource-document-latexmk-latex-auto-mk", description: "quarto-resource-document-latexmk-latex-auto-mk" }, "latex-auto-install": { $ref: "quarto-resource-document-latexmk-latex-auto-install", description: "quarto-resource-document-latexmk-latex-auto-install" }, "latex-min-runs": { $ref: "quarto-resource-document-latexmk-latex-min-runs", description: "quarto-resource-document-latexmk-latex-min-runs" }, "latex-max-runs": { $ref: "quarto-resource-document-latexmk-latex-max-runs", description: "quarto-resource-document-latexmk-latex-max-runs" }, "latex-clean": { $ref: "quarto-resource-document-latexmk-latex-clean", description: "quarto-resource-document-latexmk-latex-clean" }, "latex-makeindex": { $ref: "quarto-resource-document-latexmk-latex-makeindex", description: "quarto-resource-document-latexmk-latex-makeindex" }, "latex-makeindex-opts": { $ref: "quarto-resource-document-latexmk-latex-makeindex-opts", description: "quarto-resource-document-latexmk-latex-makeindex-opts" }, "latex-tlmgr-opts": { $ref: "quarto-resource-document-latexmk-latex-tlmgr-opts", description: "quarto-resource-document-latexmk-latex-tlmgr-opts" }, "latex-output-dir": { $ref: "quarto-resource-document-latexmk-latex-output-dir", description: "quarto-resource-document-latexmk-latex-output-dir" }, documentclass: { $ref: "quarto-resource-document-layout-documentclass", description: "quarto-resource-document-layout-documentclass" }, classoption: { $ref: "quarto-resource-document-layout-classoption", description: "quarto-resource-document-layout-classoption" }, pagestyle: { $ref: "quarto-resource-document-layout-pagestyle", description: "quarto-resource-document-layout-pagestyle" }, papersize: { $ref: "quarto-resource-document-layout-papersize", description: "quarto-resource-document-layout-papersize" }, "margin-left": { $ref: "quarto-resource-document-layout-margin-left", description: "quarto-resource-document-layout-margin-left" }, "margin-right": { $ref: "quarto-resource-document-layout-margin-right", description: "quarto-resource-document-layout-margin-right" }, "margin-top": { $ref: "quarto-resource-document-layout-margin-top", description: "quarto-resource-document-layout-margin-top" }, "margin-bottom": { $ref: "quarto-resource-document-layout-margin-bottom", description: "quarto-resource-document-layout-margin-bottom" }, geometry: { $ref: "quarto-resource-document-layout-geometry", description: "quarto-resource-document-layout-geometry" }, hyperrefoptions: { $ref: "quarto-resource-document-layout-hyperrefoptions", description: "quarto-resource-document-layout-hyperrefoptions" }, indent: { $ref: "quarto-resource-document-layout-indent", description: "quarto-resource-document-layout-indent" }, "block-headings": { $ref: "quarto-resource-document-layout-block-headings", description: "quarto-resource-document-layout-block-headings" }, keywords: { $ref: "quarto-resource-document-metadata-keywords", description: "quarto-resource-document-metadata-keywords" }, subject: { $ref: "quarto-resource-document-metadata-subject", description: "quarto-resource-document-metadata-subject" }, "title-meta": { $ref: "quarto-resource-document-metadata-title-meta", description: "quarto-resource-document-metadata-title-meta" }, "author-meta": { $ref: "quarto-resource-document-metadata-author-meta", description: "quarto-resource-document-metadata-author-meta" }, "date-meta": { $ref: "quarto-resource-document-metadata-date-meta", description: "quarto-resource-document-metadata-date-meta" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-depth": { $ref: "quarto-resource-document-numbering-number-depth", description: "quarto-resource-document-numbering-number-depth" }, secnumdepth: { $ref: "quarto-resource-document-numbering-secnumdepth", description: "quarto-resource-document-numbering-secnumdepth" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "top-level-division": { $ref: "quarto-resource-document-numbering-top-level-division", description: "quarto-resource-document-numbering-top-level-division" }, "pdf-engine": { $ref: "quarto-resource-document-options-pdf-engine", description: "quarto-resource-document-options-pdf-engine" }, "pdf-engine-opt": { $ref: "quarto-resource-document-options-pdf-engine-opt", description: "quarto-resource-document-options-pdf-engine-opt" }, beamerarticle: { $ref: "quarto-resource-document-options-beamerarticle", description: "quarto-resource-document-options-beamerarticle" }, beameroption: { $ref: "quarto-resource-document-options-beameroption", description: "quarto-resource-document-options-beameroption" }, aspectratio: { $ref: "quarto-resource-document-options-aspectratio", description: "quarto-resource-document-options-aspectratio" }, logo: { $ref: "quarto-resource-document-options-logo", description: "quarto-resource-document-options-logo" }, titlegraphic: { $ref: "quarto-resource-document-options-titlegraphic", description: "quarto-resource-document-options-titlegraphic" }, navigation: { $ref: "quarto-resource-document-options-navigation", description: "quarto-resource-document-options-navigation" }, "section-titles": { $ref: "quarto-resource-document-options-section-titles", description: "quarto-resource-document-options-section-titles" }, colortheme: { $ref: "quarto-resource-document-options-colortheme", description: "quarto-resource-document-options-colortheme" }, fonttheme: { $ref: "quarto-resource-document-options-fonttheme", description: "quarto-resource-document-options-fonttheme" }, innertheme: { $ref: "quarto-resource-document-options-innertheme", description: "quarto-resource-document-options-innertheme" }, outertheme: { $ref: "quarto-resource-document-options-outertheme", description: "quarto-resource-document-options-outertheme" }, themeoptions: { $ref: "quarto-resource-document-options-themeoptions", description: "quarto-resource-document-options-themeoptions" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, "cite-method": { $ref: "quarto-resource-document-references-cite-method", description: "quarto-resource-document-references-cite-method" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, biblatexoptions: { $ref: "quarto-resource-document-references-biblatexoptions", description: "quarto-resource-document-references-biblatexoptions" }, natbiboptions: { $ref: "quarto-resource-document-references-natbiboptions", description: "quarto-resource-document-references-natbiboptions" }, "biblio-style": { $ref: "quarto-resource-document-references-biblio-style", description: "quarto-resource-document-references-biblio-style" }, "biblio-title": { $ref: "quarto-resource-document-references-biblio-title", description: "quarto-resource-document-references-biblio-title" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "keep-tex": { $ref: "quarto-resource-document-render-keep-tex", description: "quarto-resource-document-render-keep-tex" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, incremental: { $ref: "quarto-resource-document-slides-incremental", description: "quarto-resource-document-slides-incremental" }, "slide-level": { $ref: "quarto-resource-document-slides-slide-level", description: "quarto-resource-document-slides-slide-level" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" }, "toc-title": { $ref: "quarto-resource-document-toc-toc-title", description: "quarto-resource-document-toc-toc-title" }, lof: { $ref: "quarto-resource-document-toc-lof", description: "quarto-resource-document-toc-lof" }, lot: { $ref: "quarto-resource-document-toc-lot", description: "quarto-resource-document-toc-lot" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|links_as_notes|linksAsNotes|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|latex_auto_mk|latexAutoMk|latex_auto_install|latexAutoInstall|latex_min_runs|latexMinRuns|latex_max_runs|latexMaxRuns|latex_clean|latexClean|latex_makeindex|latexMakeindex|latex_makeindex_opts|latexMakeindexOpts|latex_tlmgr_opts|latexTlmgrOpts|latex_output_dir|latexOutputDir|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|title_meta|titleMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|section_titles|sectionTitles|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|keep_tex|keepTex|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^biblatex(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^bibtex(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^commonmark(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "reference-location": { $ref: "quarto-resource-document-footnotes-reference-location", description: "quarto-resource-document-footnotes-reference-location" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "prefer-html": { $ref: "quarto-resource-document-hidden-prefer-html", description: "quarto-resource-document-hidden-prefer-html" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, variant: { $ref: "quarto-resource-document-options-variant", description: "quarto-resource-document-options-variant" }, "markdown-headings": { $ref: "quarto-resource-document-options-markdown-headings", description: "quarto-resource-document-options-markdown-headings" }, "keep-yaml": { $ref: "quarto-resource-document-options-keep-yaml", description: "quarto-resource-document-options-keep-yaml" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, "strip-comments": { $ref: "quarto-resource-document-text-strip-comments", description: "quarto-resource-document-text-strip-comments" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^commonmark_x(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^context(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, linkcolor: { $ref: "quarto-resource-document-colors-linkcolor", description: "quarto-resource-document-colors-linkcolor" }, contrastcolor: { $ref: "quarto-resource-document-colors-contrastcolor", description: "quarto-resource-document-colors-contrastcolor" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, mainfont: { $ref: "quarto-resource-document-fonts-mainfont", description: "quarto-resource-document-fonts-mainfont" }, monofont: { $ref: "quarto-resource-document-fonts-monofont", description: "quarto-resource-document-fonts-monofont" }, fontsize: { $ref: "quarto-resource-document-fonts-fontsize", description: "quarto-resource-document-fonts-fontsize" }, linestretch: { $ref: "quarto-resource-document-fonts-linestretch", description: "quarto-resource-document-fonts-linestretch" }, interlinespace: { $ref: "quarto-resource-document-fonts-interlinespace", description: "quarto-resource-document-fonts-interlinespace" }, linkstyle: { $ref: "quarto-resource-document-fonts-linkstyle", description: "quarto-resource-document-fonts-linkstyle" }, whitespace: { $ref: "quarto-resource-document-fonts-whitespace", description: "quarto-resource-document-fonts-whitespace" }, indenting: { $ref: "quarto-resource-document-formatting-indenting", description: "quarto-resource-document-formatting-indenting" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, headertext: { $ref: "quarto-resource-document-includes-headertext", description: "quarto-resource-document-includes-headertext" }, footertext: { $ref: "quarto-resource-document-includes-footertext", description: "quarto-resource-document-includes-footertext" }, includesource: { $ref: "quarto-resource-document-includes-includesource", description: "quarto-resource-document-includes-includesource" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, layout: { $ref: "quarto-resource-document-layout-layout", description: "quarto-resource-document-layout-layout" }, "margin-left": { $ref: "quarto-resource-document-layout-margin-left", description: "quarto-resource-document-layout-margin-left" }, "margin-right": { $ref: "quarto-resource-document-layout-margin-right", description: "quarto-resource-document-layout-margin-right" }, "margin-top": { $ref: "quarto-resource-document-layout-margin-top", description: "quarto-resource-document-layout-margin-top" }, "margin-bottom": { $ref: "quarto-resource-document-layout-margin-bottom", description: "quarto-resource-document-layout-margin-bottom" }, keywords: { $ref: "quarto-resource-document-metadata-keywords", description: "quarto-resource-document-metadata-keywords" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, pagenumbering: { $ref: "quarto-resource-document-numbering-pagenumbering", description: "quarto-resource-document-numbering-pagenumbering" }, "top-level-division": { $ref: "quarto-resource-document-numbering-top-level-division", description: "quarto-resource-document-numbering-top-level-division" }, "pdf-engine": { $ref: "quarto-resource-document-options-pdf-engine", description: "quarto-resource-document-options-pdf-engine" }, "pdf-engine-opt": { $ref: "quarto-resource-document-options-pdf-engine-opt", description: "quarto-resource-document-options-pdf-engine-opt" }, pdfa: { $ref: "quarto-resource-document-pdfa-pdfa", description: "quarto-resource-document-pdfa-pdfa" }, pdfaiccprofile: { $ref: "quarto-resource-document-pdfa-pdfaiccprofile", description: "quarto-resource-document-pdfa-pdfaiccprofile" }, pdfaintent: { $ref: "quarto-resource-document-pdfa-pdfaintent", description: "quarto-resource-document-pdfa-pdfaintent" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^csljson(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^docbook(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "top-level-division": { $ref: "quarto-resource-document-numbering-top-level-division", description: "quarto-resource-document-numbering-top-level-division" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^docbook4(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "top-level-division": { $ref: "quarto-resource-document-numbering-top-level-division", description: "quarto-resource-document-numbering-top-level-division" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^docbook5(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "top-level-division": { $ref: "quarto-resource-document-numbering-top-level-division", description: "quarto-resource-document-numbering-top-level-division" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^docx(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "track-changes": { $ref: "quarto-resource-document-hidden-track-changes", description: "quarto-resource-document-hidden-track-changes" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "page-width": { $ref: "quarto-resource-document-layout-page-width", description: "quarto-resource-document-layout-page-width" }, subject: { $ref: "quarto-resource-document-metadata-subject", description: "quarto-resource-document-metadata-subject" }, description: { $ref: "quarto-resource-document-metadata-description", description: "quarto-resource-document-metadata-description" }, category: { $ref: "quarto-resource-document-metadata-category", description: "quarto-resource-document-metadata-category" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-depth": { $ref: "quarto-resource-document-numbering-number-depth", description: "quarto-resource-document-numbering-number-depth" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "reference-doc": { $ref: "quarto-resource-document-options-reference-doc", description: "quarto-resource-document-options-reference-doc" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" }, "toc-title": { $ref: "quarto-resource-document-toc-toc-title", description: "quarto-resource-document-toc-toc-title" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|track_changes|trackChanges|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|page_width|pageWidth|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|reference_doc|referenceDoc|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^dokuwiki(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^dzslides(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-fold": { $ref: "quarto-resource-cell-codeoutput-code-fold", description: "quarto-resource-cell-codeoutput-code-fold" }, "code-summary": { $ref: "quarto-resource-cell-codeoutput-code-summary", description: "quarto-resource-cell-codeoutput-code-summary" }, "code-overflow": { $ref: "quarto-resource-cell-codeoutput-code-overflow", description: "quarto-resource-cell-codeoutput-code-overflow" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "cap-location": { $ref: "quarto-resource-cell-pagelayout-cap-location", description: "quarto-resource-cell-pagelayout-cap-location" }, "fig-cap-location": { $ref: "quarto-resource-cell-pagelayout-fig-cap-location", description: "quarto-resource-cell-pagelayout-fig-cap-location" }, "tbl-cap-location": { $ref: "quarto-resource-cell-pagelayout-tbl-cap-location", description: "quarto-resource-cell-pagelayout-tbl-cap-location" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, institute: { $ref: "quarto-resource-document-attributes-institute", description: "quarto-resource-document-attributes-institute" }, "code-copy": { $ref: "quarto-resource-document-code-code-copy", description: "quarto-resource-document-code-code-copy" }, "code-link": { $ref: "quarto-resource-document-code-code-link", description: "quarto-resource-document-code-code-link" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, fontcolor: { $ref: "quarto-resource-document-colors-fontcolor", description: "quarto-resource-document-colors-fontcolor" }, linkcolor: { $ref: "quarto-resource-document-colors-linkcolor", description: "quarto-resource-document-colors-linkcolor" }, monobackgroundcolor: { $ref: "quarto-resource-document-colors-monobackgroundcolor", description: "quarto-resource-document-colors-monobackgroundcolor" }, backgroundcolor: { $ref: "quarto-resource-document-colors-backgroundcolor", description: "quarto-resource-document-colors-backgroundcolor" }, comments: { $ref: "quarto-resource-document-comments-comments", description: "quarto-resource-document-comments-comments" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "fig-responsive": { $ref: "quarto-resource-document-figures-fig-responsive", description: "quarto-resource-document-figures-fig-responsive" }, mainfont: { $ref: "quarto-resource-document-fonts-mainfont", description: "quarto-resource-document-fonts-mainfont" }, monofont: { $ref: "quarto-resource-document-fonts-monofont", description: "quarto-resource-document-fonts-monofont" }, fontsize: { $ref: "quarto-resource-document-fonts-fontsize", description: "quarto-resource-document-fonts-fontsize" }, linestretch: { $ref: "quarto-resource-document-fonts-linestretch", description: "quarto-resource-document-fonts-linestretch" }, "footnotes-hover": { $ref: "quarto-resource-document-footnotes-footnotes-hover", description: "quarto-resource-document-footnotes-footnotes-hover" }, "reference-location": { $ref: "quarto-resource-document-footnotes-reference-location", description: "quarto-resource-document-footnotes-reference-location" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, resources: { $ref: "quarto-resource-document-includes-resources", description: "quarto-resource-document-includes-resources" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, classoption: { $ref: "quarto-resource-document-layout-classoption", description: "quarto-resource-document-layout-classoption" }, "max-width": { $ref: "quarto-resource-document-layout-max-width", description: "quarto-resource-document-layout-max-width" }, "margin-left": { $ref: "quarto-resource-document-layout-margin-left", description: "quarto-resource-document-layout-margin-left" }, "margin-right": { $ref: "quarto-resource-document-layout-margin-right", description: "quarto-resource-document-layout-margin-right" }, "margin-top": { $ref: "quarto-resource-document-layout-margin-top", description: "quarto-resource-document-layout-margin-top" }, "margin-bottom": { $ref: "quarto-resource-document-layout-margin-bottom", description: "quarto-resource-document-layout-margin-bottom" }, keywords: { $ref: "quarto-resource-document-metadata-keywords", description: "quarto-resource-document-metadata-keywords" }, pagetitle: { $ref: "quarto-resource-document-metadata-pagetitle", description: "quarto-resource-document-metadata-pagetitle" }, "title-prefix": { $ref: "quarto-resource-document-metadata-title-prefix", description: "quarto-resource-document-metadata-title-prefix" }, "description-meta": { $ref: "quarto-resource-document-metadata-description-meta", description: "quarto-resource-document-metadata-description-meta" }, "author-meta": { $ref: "quarto-resource-document-metadata-author-meta", description: "quarto-resource-document-metadata-author-meta" }, "date-meta": { $ref: "quarto-resource-document-metadata-date-meta", description: "quarto-resource-document-metadata-date-meta" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-depth": { $ref: "quarto-resource-document-numbering-number-depth", description: "quarto-resource-document-numbering-number-depth" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "document-css": { $ref: "quarto-resource-document-options-document-css", description: "quarto-resource-document-options-document-css" }, css: { $ref: "quarto-resource-document-options-css", description: "quarto-resource-document-options-css" }, "section-divs": { $ref: "quarto-resource-document-options-section-divs", description: "quarto-resource-document-options-section-divs" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, "email-obfuscation": { $ref: "quarto-resource-document-options-email-obfuscation", description: "quarto-resource-document-options-email-obfuscation" }, "html-q-tags": { $ref: "quarto-resource-document-options-html-q-tags", description: "quarto-resource-document-options-html-q-tags" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, "citations-hover": { $ref: "quarto-resource-document-references-citations-hover", description: "quarto-resource-document-references-citations-hover" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, "self-contained": { $ref: "quarto-resource-document-render-self-contained", description: "quarto-resource-document-render-self-contained" }, "self-contained-math": { $ref: "quarto-resource-document-render-self-contained-math", description: "quarto-resource-document-render-self-contained-math" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, incremental: { $ref: "quarto-resource-document-slides-incremental", description: "quarto-resource-document-slides-incremental" }, "slide-level": { $ref: "quarto-resource-document-slides-slide-level", description: "quarto-resource-document-slides-slide-level" }, "strip-comments": { $ref: "quarto-resource-document-text-strip-comments", description: "quarto-resource-document-text-strip-comments" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^epub(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-fold": { $ref: "quarto-resource-cell-codeoutput-code-fold", description: "quarto-resource-cell-codeoutput-code-fold" }, "code-summary": { $ref: "quarto-resource-cell-codeoutput-code-summary", description: "quarto-resource-cell-codeoutput-code-summary" }, "code-overflow": { $ref: "quarto-resource-cell-codeoutput-code-overflow", description: "quarto-resource-cell-codeoutput-code-overflow" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, "abstract-title": { $ref: "quarto-resource-document-attributes-abstract-title", description: "quarto-resource-document-attributes-abstract-title" }, "code-copy": { $ref: "quarto-resource-document-code-code-copy", description: "quarto-resource-document-code-code-copy" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, identifier: { $ref: "quarto-resource-document-epub-identifier", description: "quarto-resource-document-epub-identifier" }, creator: { $ref: "quarto-resource-document-epub-creator", description: "quarto-resource-document-epub-creator" }, contributor: { $ref: "quarto-resource-document-epub-contributor", description: "quarto-resource-document-epub-contributor" }, subject: { $ref: "quarto-resource-document-epub-subject", description: "quarto-resource-document-epub-subject" }, type: { $ref: "quarto-resource-document-epub-type", description: "quarto-resource-document-epub-type" }, format: { $ref: "quarto-resource-document-epub-format", description: "quarto-resource-document-epub-format" }, relation: { $ref: "quarto-resource-document-epub-relation", description: "quarto-resource-document-epub-relation" }, coverage: { $ref: "quarto-resource-document-epub-coverage", description: "quarto-resource-document-epub-coverage" }, rights: { $ref: "quarto-resource-document-epub-rights", description: "quarto-resource-document-epub-rights" }, "belongs-to-collection": { $ref: "quarto-resource-document-epub-belongs-to-collection", description: "quarto-resource-document-epub-belongs-to-collection" }, "group-position": { $ref: "quarto-resource-document-epub-group-position", description: "quarto-resource-document-epub-group-position" }, "page-progression-direction": { $ref: "quarto-resource-document-epub-page-progression-direction", description: "quarto-resource-document-epub-page-progression-direction" }, ibooks: { $ref: "quarto-resource-document-epub-ibooks", description: "quarto-resource-document-epub-ibooks" }, "epub-metadata": { $ref: "quarto-resource-document-epub-epub-metadata", description: "quarto-resource-document-epub-epub-metadata" }, "epub-subdirectory": { $ref: "quarto-resource-document-epub-epub-subdirectory", description: "quarto-resource-document-epub-epub-subdirectory" }, "epub-fonts": { $ref: "quarto-resource-document-epub-epub-fonts", description: "quarto-resource-document-epub-epub-fonts" }, "epub-chapter-level": { $ref: "quarto-resource-document-epub-epub-chapter-level", description: "quarto-resource-document-epub-epub-chapter-level" }, "epub-cover-image": { $ref: "quarto-resource-document-epub-epub-cover-image", description: "quarto-resource-document-epub-epub-cover-image" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "fig-responsive": { $ref: "quarto-resource-document-figures-fig-responsive", description: "quarto-resource-document-figures-fig-responsive" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, resources: { $ref: "quarto-resource-document-includes-resources", description: "quarto-resource-document-includes-resources" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "date-meta": { $ref: "quarto-resource-document-metadata-date-meta", description: "quarto-resource-document-metadata-date-meta" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-depth": { $ref: "quarto-resource-document-numbering-number-depth", description: "quarto-resource-document-numbering-number-depth" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, css: { $ref: "quarto-resource-document-options-css", description: "quarto-resource-document-options-css" }, "html-math-method": { $ref: "quarto-resource-document-options-html-math-method", description: "quarto-resource-document-options-html-math-method" }, "html-q-tags": { $ref: "quarto-resource-document-options-html-q-tags", description: "quarto-resource-document-options-html-q-tags" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" }, "toc-title": { $ref: "quarto-resource-document-toc-toc-title", description: "quarto-resource-document-toc-toc-title" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|html_q_tags|htmlQTags|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^epub2(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-fold": { $ref: "quarto-resource-cell-codeoutput-code-fold", description: "quarto-resource-cell-codeoutput-code-fold" }, "code-summary": { $ref: "quarto-resource-cell-codeoutput-code-summary", description: "quarto-resource-cell-codeoutput-code-summary" }, "code-overflow": { $ref: "quarto-resource-cell-codeoutput-code-overflow", description: "quarto-resource-cell-codeoutput-code-overflow" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, "abstract-title": { $ref: "quarto-resource-document-attributes-abstract-title", description: "quarto-resource-document-attributes-abstract-title" }, "code-copy": { $ref: "quarto-resource-document-code-code-copy", description: "quarto-resource-document-code-code-copy" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, identifier: { $ref: "quarto-resource-document-epub-identifier", description: "quarto-resource-document-epub-identifier" }, creator: { $ref: "quarto-resource-document-epub-creator", description: "quarto-resource-document-epub-creator" }, contributor: { $ref: "quarto-resource-document-epub-contributor", description: "quarto-resource-document-epub-contributor" }, subject: { $ref: "quarto-resource-document-epub-subject", description: "quarto-resource-document-epub-subject" }, type: { $ref: "quarto-resource-document-epub-type", description: "quarto-resource-document-epub-type" }, format: { $ref: "quarto-resource-document-epub-format", description: "quarto-resource-document-epub-format" }, relation: { $ref: "quarto-resource-document-epub-relation", description: "quarto-resource-document-epub-relation" }, coverage: { $ref: "quarto-resource-document-epub-coverage", description: "quarto-resource-document-epub-coverage" }, rights: { $ref: "quarto-resource-document-epub-rights", description: "quarto-resource-document-epub-rights" }, "belongs-to-collection": { $ref: "quarto-resource-document-epub-belongs-to-collection", description: "quarto-resource-document-epub-belongs-to-collection" }, "group-position": { $ref: "quarto-resource-document-epub-group-position", description: "quarto-resource-document-epub-group-position" }, "page-progression-direction": { $ref: "quarto-resource-document-epub-page-progression-direction", description: "quarto-resource-document-epub-page-progression-direction" }, ibooks: { $ref: "quarto-resource-document-epub-ibooks", description: "quarto-resource-document-epub-ibooks" }, "epub-metadata": { $ref: "quarto-resource-document-epub-epub-metadata", description: "quarto-resource-document-epub-epub-metadata" }, "epub-subdirectory": { $ref: "quarto-resource-document-epub-epub-subdirectory", description: "quarto-resource-document-epub-epub-subdirectory" }, "epub-fonts": { $ref: "quarto-resource-document-epub-epub-fonts", description: "quarto-resource-document-epub-epub-fonts" }, "epub-chapter-level": { $ref: "quarto-resource-document-epub-epub-chapter-level", description: "quarto-resource-document-epub-epub-chapter-level" }, "epub-cover-image": { $ref: "quarto-resource-document-epub-epub-cover-image", description: "quarto-resource-document-epub-epub-cover-image" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "fig-responsive": { $ref: "quarto-resource-document-figures-fig-responsive", description: "quarto-resource-document-figures-fig-responsive" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, resources: { $ref: "quarto-resource-document-includes-resources", description: "quarto-resource-document-includes-resources" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "date-meta": { $ref: "quarto-resource-document-metadata-date-meta", description: "quarto-resource-document-metadata-date-meta" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-depth": { $ref: "quarto-resource-document-numbering-number-depth", description: "quarto-resource-document-numbering-number-depth" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, css: { $ref: "quarto-resource-document-options-css", description: "quarto-resource-document-options-css" }, "html-math-method": { $ref: "quarto-resource-document-options-html-math-method", description: "quarto-resource-document-options-html-math-method" }, "html-q-tags": { $ref: "quarto-resource-document-options-html-q-tags", description: "quarto-resource-document-options-html-q-tags" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" }, "toc-title": { $ref: "quarto-resource-document-toc-toc-title", description: "quarto-resource-document-toc-toc-title" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|html_q_tags|htmlQTags|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^epub3(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-fold": { $ref: "quarto-resource-cell-codeoutput-code-fold", description: "quarto-resource-cell-codeoutput-code-fold" }, "code-summary": { $ref: "quarto-resource-cell-codeoutput-code-summary", description: "quarto-resource-cell-codeoutput-code-summary" }, "code-overflow": { $ref: "quarto-resource-cell-codeoutput-code-overflow", description: "quarto-resource-cell-codeoutput-code-overflow" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, "abstract-title": { $ref: "quarto-resource-document-attributes-abstract-title", description: "quarto-resource-document-attributes-abstract-title" }, "code-copy": { $ref: "quarto-resource-document-code-code-copy", description: "quarto-resource-document-code-code-copy" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, identifier: { $ref: "quarto-resource-document-epub-identifier", description: "quarto-resource-document-epub-identifier" }, creator: { $ref: "quarto-resource-document-epub-creator", description: "quarto-resource-document-epub-creator" }, contributor: { $ref: "quarto-resource-document-epub-contributor", description: "quarto-resource-document-epub-contributor" }, subject: { $ref: "quarto-resource-document-epub-subject", description: "quarto-resource-document-epub-subject" }, type: { $ref: "quarto-resource-document-epub-type", description: "quarto-resource-document-epub-type" }, format: { $ref: "quarto-resource-document-epub-format", description: "quarto-resource-document-epub-format" }, relation: { $ref: "quarto-resource-document-epub-relation", description: "quarto-resource-document-epub-relation" }, coverage: { $ref: "quarto-resource-document-epub-coverage", description: "quarto-resource-document-epub-coverage" }, rights: { $ref: "quarto-resource-document-epub-rights", description: "quarto-resource-document-epub-rights" }, "belongs-to-collection": { $ref: "quarto-resource-document-epub-belongs-to-collection", description: "quarto-resource-document-epub-belongs-to-collection" }, "group-position": { $ref: "quarto-resource-document-epub-group-position", description: "quarto-resource-document-epub-group-position" }, "page-progression-direction": { $ref: "quarto-resource-document-epub-page-progression-direction", description: "quarto-resource-document-epub-page-progression-direction" }, ibooks: { $ref: "quarto-resource-document-epub-ibooks", description: "quarto-resource-document-epub-ibooks" }, "epub-metadata": { $ref: "quarto-resource-document-epub-epub-metadata", description: "quarto-resource-document-epub-epub-metadata" }, "epub-subdirectory": { $ref: "quarto-resource-document-epub-epub-subdirectory", description: "quarto-resource-document-epub-epub-subdirectory" }, "epub-fonts": { $ref: "quarto-resource-document-epub-epub-fonts", description: "quarto-resource-document-epub-epub-fonts" }, "epub-chapter-level": { $ref: "quarto-resource-document-epub-epub-chapter-level", description: "quarto-resource-document-epub-epub-chapter-level" }, "epub-cover-image": { $ref: "quarto-resource-document-epub-epub-cover-image", description: "quarto-resource-document-epub-epub-cover-image" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "fig-responsive": { $ref: "quarto-resource-document-figures-fig-responsive", description: "quarto-resource-document-figures-fig-responsive" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, resources: { $ref: "quarto-resource-document-includes-resources", description: "quarto-resource-document-includes-resources" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "date-meta": { $ref: "quarto-resource-document-metadata-date-meta", description: "quarto-resource-document-metadata-date-meta" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-depth": { $ref: "quarto-resource-document-numbering-number-depth", description: "quarto-resource-document-numbering-number-depth" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, css: { $ref: "quarto-resource-document-options-css", description: "quarto-resource-document-options-css" }, "html-math-method": { $ref: "quarto-resource-document-options-html-math-method", description: "quarto-resource-document-options-html-math-method" }, "html-q-tags": { $ref: "quarto-resource-document-options-html-q-tags", description: "quarto-resource-document-options-html-q-tags" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" }, "toc-title": { $ref: "quarto-resource-document-toc-toc-title", description: "quarto-resource-document-toc-toc-title" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|html_q_tags|htmlQTags|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^fb2(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^gfm(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "reference-location": { $ref: "quarto-resource-document-footnotes-reference-location", description: "quarto-resource-document-footnotes-reference-location" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "prefer-html": { $ref: "quarto-resource-document-hidden-prefer-html", description: "quarto-resource-document-hidden-prefer-html" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "html-math-method": { $ref: "quarto-resource-document-options-html-math-method", description: "quarto-resource-document-options-html-math-method" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, variant: { $ref: "quarto-resource-document-options-variant", description: "quarto-resource-document-options-variant" }, "markdown-headings": { $ref: "quarto-resource-document-options-markdown-headings", description: "quarto-resource-document-options-markdown-headings" }, "keep-yaml": { $ref: "quarto-resource-document-options-keep-yaml", description: "quarto-resource-document-options-keep-yaml" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, "strip-comments": { $ref: "quarto-resource-document-text-strip-comments", description: "quarto-resource-document-text-strip-comments" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^haddock(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^html(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-fold": { $ref: "quarto-resource-cell-codeoutput-code-fold", description: "quarto-resource-cell-codeoutput-code-fold" }, "code-summary": { $ref: "quarto-resource-cell-codeoutput-code-summary", description: "quarto-resource-cell-codeoutput-code-summary" }, "code-overflow": { $ref: "quarto-resource-cell-codeoutput-code-overflow", description: "quarto-resource-cell-codeoutput-code-overflow" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "cap-location": { $ref: "quarto-resource-cell-pagelayout-cap-location", description: "quarto-resource-cell-pagelayout-cap-location" }, "fig-cap-location": { $ref: "quarto-resource-cell-pagelayout-fig-cap-location", description: "quarto-resource-cell-pagelayout-fig-cap-location" }, "tbl-cap-location": { $ref: "quarto-resource-cell-pagelayout-tbl-cap-location", description: "quarto-resource-cell-pagelayout-tbl-cap-location" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, "abstract-title": { $ref: "quarto-resource-document-attributes-abstract-title", description: "quarto-resource-document-attributes-abstract-title" }, doi: { $ref: "quarto-resource-document-attributes-doi", description: "quarto-resource-document-attributes-doi" }, "code-copy": { $ref: "quarto-resource-document-code-code-copy", description: "quarto-resource-document-code-code-copy" }, "code-link": { $ref: "quarto-resource-document-code-code-link", description: "quarto-resource-document-code-code-link" }, "code-tools": { $ref: "quarto-resource-document-code-code-tools", description: "quarto-resource-document-code-code-tools" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, fontcolor: { $ref: "quarto-resource-document-colors-fontcolor", description: "quarto-resource-document-colors-fontcolor" }, linkcolor: { $ref: "quarto-resource-document-colors-linkcolor", description: "quarto-resource-document-colors-linkcolor" }, monobackgroundcolor: { $ref: "quarto-resource-document-colors-monobackgroundcolor", description: "quarto-resource-document-colors-monobackgroundcolor" }, backgroundcolor: { $ref: "quarto-resource-document-colors-backgroundcolor", description: "quarto-resource-document-colors-backgroundcolor" }, comments: { $ref: "quarto-resource-document-comments-comments", description: "quarto-resource-document-comments-comments" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "fig-responsive": { $ref: "quarto-resource-document-figures-fig-responsive", description: "quarto-resource-document-figures-fig-responsive" }, mainfont: { $ref: "quarto-resource-document-fonts-mainfont", description: "quarto-resource-document-fonts-mainfont" }, monofont: { $ref: "quarto-resource-document-fonts-monofont", description: "quarto-resource-document-fonts-monofont" }, fontsize: { $ref: "quarto-resource-document-fonts-fontsize", description: "quarto-resource-document-fonts-fontsize" }, linestretch: { $ref: "quarto-resource-document-fonts-linestretch", description: "quarto-resource-document-fonts-linestretch" }, "footnotes-hover": { $ref: "quarto-resource-document-footnotes-footnotes-hover", description: "quarto-resource-document-footnotes-footnotes-hover" }, "reference-location": { $ref: "quarto-resource-document-footnotes-reference-location", description: "quarto-resource-document-footnotes-reference-location" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "keep-source": { $ref: "quarto-resource-document-hidden-keep-source", description: "quarto-resource-document-hidden-keep-source" }, "keep-hidden": { $ref: "quarto-resource-document-hidden-keep-hidden", description: "quarto-resource-document-hidden-keep-hidden" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, resources: { $ref: "quarto-resource-document-includes-resources", description: "quarto-resource-document-includes-resources" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, classoption: { $ref: "quarto-resource-document-layout-classoption", description: "quarto-resource-document-layout-classoption" }, "page-layout": { $ref: "quarto-resource-document-layout-page-layout", description: "quarto-resource-document-layout-page-layout" }, "max-width": { $ref: "quarto-resource-document-layout-max-width", description: "quarto-resource-document-layout-max-width" }, "margin-left": { $ref: "quarto-resource-document-layout-margin-left", description: "quarto-resource-document-layout-margin-left" }, "margin-right": { $ref: "quarto-resource-document-layout-margin-right", description: "quarto-resource-document-layout-margin-right" }, "margin-top": { $ref: "quarto-resource-document-layout-margin-top", description: "quarto-resource-document-layout-margin-top" }, "margin-bottom": { $ref: "quarto-resource-document-layout-margin-bottom", description: "quarto-resource-document-layout-margin-bottom" }, "link-external-icon": { $ref: "quarto-resource-document-links-link-external-icon", description: "quarto-resource-document-links-link-external-icon" }, "link-external-newwindow": { $ref: "quarto-resource-document-links-link-external-newwindow", description: "quarto-resource-document-links-link-external-newwindow" }, "link-external-filter": { $ref: "quarto-resource-document-links-link-external-filter", description: "quarto-resource-document-links-link-external-filter" }, keywords: { $ref: "quarto-resource-document-metadata-keywords", description: "quarto-resource-document-metadata-keywords" }, pagetitle: { $ref: "quarto-resource-document-metadata-pagetitle", description: "quarto-resource-document-metadata-pagetitle" }, "title-prefix": { $ref: "quarto-resource-document-metadata-title-prefix", description: "quarto-resource-document-metadata-title-prefix" }, "description-meta": { $ref: "quarto-resource-document-metadata-description-meta", description: "quarto-resource-document-metadata-description-meta" }, "author-meta": { $ref: "quarto-resource-document-metadata-author-meta", description: "quarto-resource-document-metadata-author-meta" }, "date-meta": { $ref: "quarto-resource-document-metadata-date-meta", description: "quarto-resource-document-metadata-date-meta" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-depth": { $ref: "quarto-resource-document-numbering-number-depth", description: "quarto-resource-document-numbering-number-depth" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, theme: { $ref: "quarto-resource-document-options-theme", description: "quarto-resource-document-options-theme" }, minimal: { $ref: "quarto-resource-document-options-minimal", description: "quarto-resource-document-options-minimal" }, "document-css": { $ref: "quarto-resource-document-options-document-css", description: "quarto-resource-document-options-document-css" }, css: { $ref: "quarto-resource-document-options-css", description: "quarto-resource-document-options-css" }, "anchor-sections": { $ref: "quarto-resource-document-options-anchor-sections", description: "quarto-resource-document-options-anchor-sections" }, "html-math-method": { $ref: "quarto-resource-document-options-html-math-method", description: "quarto-resource-document-options-html-math-method" }, "section-divs": { $ref: "quarto-resource-document-options-section-divs", description: "quarto-resource-document-options-section-divs" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, "email-obfuscation": { $ref: "quarto-resource-document-options-email-obfuscation", description: "quarto-resource-document-options-email-obfuscation" }, "html-q-tags": { $ref: "quarto-resource-document-options-html-q-tags", description: "quarto-resource-document-options-html-q-tags" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, "citations-hover": { $ref: "quarto-resource-document-references-citations-hover", description: "quarto-resource-document-references-citations-hover" }, "citation-location": { $ref: "quarto-resource-document-references-citation-location", description: "quarto-resource-document-references-citation-location" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, "self-contained": { $ref: "quarto-resource-document-render-self-contained", description: "quarto-resource-document-render-self-contained" }, "self-contained-math": { $ref: "quarto-resource-document-render-self-contained-math", description: "quarto-resource-document-render-self-contained-math" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, "strip-comments": { $ref: "quarto-resource-document-text-strip-comments", description: "quarto-resource-document-text-strip-comments" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" }, "toc-location": { $ref: "quarto-resource-document-toc-toc-location", description: "quarto-resource-document-toc-toc-location" }, "toc-title": { $ref: "quarto-resource-document-toc-toc-title", description: "quarto-resource-document-toc-toc-title" }, search: { $ref: "quarto-resource-document-website-search", description: "quarto-resource-document-website-search" }, aliases: { $ref: "quarto-resource-document-website-aliases", description: "quarto-resource-document-website-aliases" }, image: { $ref: "quarto-resource-document-website-image", description: "quarto-resource-document-website-image" }, "image-height": { $ref: "quarto-resource-document-website-image-height", description: "quarto-resource-document-website-image-height" }, "image-width": { $ref: "quarto-resource-document-website-image-width", description: "quarto-resource-document-website-image-width" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_layout|pageLayout|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_location|citationLocation|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^html4(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-fold": { $ref: "quarto-resource-cell-codeoutput-code-fold", description: "quarto-resource-cell-codeoutput-code-fold" }, "code-summary": { $ref: "quarto-resource-cell-codeoutput-code-summary", description: "quarto-resource-cell-codeoutput-code-summary" }, "code-overflow": { $ref: "quarto-resource-cell-codeoutput-code-overflow", description: "quarto-resource-cell-codeoutput-code-overflow" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "cap-location": { $ref: "quarto-resource-cell-pagelayout-cap-location", description: "quarto-resource-cell-pagelayout-cap-location" }, "fig-cap-location": { $ref: "quarto-resource-cell-pagelayout-fig-cap-location", description: "quarto-resource-cell-pagelayout-fig-cap-location" }, "tbl-cap-location": { $ref: "quarto-resource-cell-pagelayout-tbl-cap-location", description: "quarto-resource-cell-pagelayout-tbl-cap-location" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, "abstract-title": { $ref: "quarto-resource-document-attributes-abstract-title", description: "quarto-resource-document-attributes-abstract-title" }, doi: { $ref: "quarto-resource-document-attributes-doi", description: "quarto-resource-document-attributes-doi" }, "code-copy": { $ref: "quarto-resource-document-code-code-copy", description: "quarto-resource-document-code-code-copy" }, "code-link": { $ref: "quarto-resource-document-code-code-link", description: "quarto-resource-document-code-code-link" }, "code-tools": { $ref: "quarto-resource-document-code-code-tools", description: "quarto-resource-document-code-code-tools" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, fontcolor: { $ref: "quarto-resource-document-colors-fontcolor", description: "quarto-resource-document-colors-fontcolor" }, linkcolor: { $ref: "quarto-resource-document-colors-linkcolor", description: "quarto-resource-document-colors-linkcolor" }, monobackgroundcolor: { $ref: "quarto-resource-document-colors-monobackgroundcolor", description: "quarto-resource-document-colors-monobackgroundcolor" }, backgroundcolor: { $ref: "quarto-resource-document-colors-backgroundcolor", description: "quarto-resource-document-colors-backgroundcolor" }, comments: { $ref: "quarto-resource-document-comments-comments", description: "quarto-resource-document-comments-comments" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "fig-responsive": { $ref: "quarto-resource-document-figures-fig-responsive", description: "quarto-resource-document-figures-fig-responsive" }, mainfont: { $ref: "quarto-resource-document-fonts-mainfont", description: "quarto-resource-document-fonts-mainfont" }, monofont: { $ref: "quarto-resource-document-fonts-monofont", description: "quarto-resource-document-fonts-monofont" }, fontsize: { $ref: "quarto-resource-document-fonts-fontsize", description: "quarto-resource-document-fonts-fontsize" }, linestretch: { $ref: "quarto-resource-document-fonts-linestretch", description: "quarto-resource-document-fonts-linestretch" }, "footnotes-hover": { $ref: "quarto-resource-document-footnotes-footnotes-hover", description: "quarto-resource-document-footnotes-footnotes-hover" }, "reference-location": { $ref: "quarto-resource-document-footnotes-reference-location", description: "quarto-resource-document-footnotes-reference-location" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "keep-source": { $ref: "quarto-resource-document-hidden-keep-source", description: "quarto-resource-document-hidden-keep-source" }, "keep-hidden": { $ref: "quarto-resource-document-hidden-keep-hidden", description: "quarto-resource-document-hidden-keep-hidden" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, resources: { $ref: "quarto-resource-document-includes-resources", description: "quarto-resource-document-includes-resources" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, classoption: { $ref: "quarto-resource-document-layout-classoption", description: "quarto-resource-document-layout-classoption" }, "page-layout": { $ref: "quarto-resource-document-layout-page-layout", description: "quarto-resource-document-layout-page-layout" }, "max-width": { $ref: "quarto-resource-document-layout-max-width", description: "quarto-resource-document-layout-max-width" }, "margin-left": { $ref: "quarto-resource-document-layout-margin-left", description: "quarto-resource-document-layout-margin-left" }, "margin-right": { $ref: "quarto-resource-document-layout-margin-right", description: "quarto-resource-document-layout-margin-right" }, "margin-top": { $ref: "quarto-resource-document-layout-margin-top", description: "quarto-resource-document-layout-margin-top" }, "margin-bottom": { $ref: "quarto-resource-document-layout-margin-bottom", description: "quarto-resource-document-layout-margin-bottom" }, "link-external-icon": { $ref: "quarto-resource-document-links-link-external-icon", description: "quarto-resource-document-links-link-external-icon" }, "link-external-newwindow": { $ref: "quarto-resource-document-links-link-external-newwindow", description: "quarto-resource-document-links-link-external-newwindow" }, "link-external-filter": { $ref: "quarto-resource-document-links-link-external-filter", description: "quarto-resource-document-links-link-external-filter" }, keywords: { $ref: "quarto-resource-document-metadata-keywords", description: "quarto-resource-document-metadata-keywords" }, pagetitle: { $ref: "quarto-resource-document-metadata-pagetitle", description: "quarto-resource-document-metadata-pagetitle" }, "title-prefix": { $ref: "quarto-resource-document-metadata-title-prefix", description: "quarto-resource-document-metadata-title-prefix" }, "description-meta": { $ref: "quarto-resource-document-metadata-description-meta", description: "quarto-resource-document-metadata-description-meta" }, "author-meta": { $ref: "quarto-resource-document-metadata-author-meta", description: "quarto-resource-document-metadata-author-meta" }, "date-meta": { $ref: "quarto-resource-document-metadata-date-meta", description: "quarto-resource-document-metadata-date-meta" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-depth": { $ref: "quarto-resource-document-numbering-number-depth", description: "quarto-resource-document-numbering-number-depth" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, theme: { $ref: "quarto-resource-document-options-theme", description: "quarto-resource-document-options-theme" }, minimal: { $ref: "quarto-resource-document-options-minimal", description: "quarto-resource-document-options-minimal" }, "document-css": { $ref: "quarto-resource-document-options-document-css", description: "quarto-resource-document-options-document-css" }, css: { $ref: "quarto-resource-document-options-css", description: "quarto-resource-document-options-css" }, "anchor-sections": { $ref: "quarto-resource-document-options-anchor-sections", description: "quarto-resource-document-options-anchor-sections" }, "html-math-method": { $ref: "quarto-resource-document-options-html-math-method", description: "quarto-resource-document-options-html-math-method" }, "section-divs": { $ref: "quarto-resource-document-options-section-divs", description: "quarto-resource-document-options-section-divs" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, "email-obfuscation": { $ref: "quarto-resource-document-options-email-obfuscation", description: "quarto-resource-document-options-email-obfuscation" }, "html-q-tags": { $ref: "quarto-resource-document-options-html-q-tags", description: "quarto-resource-document-options-html-q-tags" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, "citations-hover": { $ref: "quarto-resource-document-references-citations-hover", description: "quarto-resource-document-references-citations-hover" }, "citation-location": { $ref: "quarto-resource-document-references-citation-location", description: "quarto-resource-document-references-citation-location" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, "self-contained": { $ref: "quarto-resource-document-render-self-contained", description: "quarto-resource-document-render-self-contained" }, "self-contained-math": { $ref: "quarto-resource-document-render-self-contained-math", description: "quarto-resource-document-render-self-contained-math" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, "strip-comments": { $ref: "quarto-resource-document-text-strip-comments", description: "quarto-resource-document-text-strip-comments" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" }, "toc-location": { $ref: "quarto-resource-document-toc-toc-location", description: "quarto-resource-document-toc-toc-location" }, "toc-title": { $ref: "quarto-resource-document-toc-toc-title", description: "quarto-resource-document-toc-toc-title" }, search: { $ref: "quarto-resource-document-website-search", description: "quarto-resource-document-website-search" }, aliases: { $ref: "quarto-resource-document-website-aliases", description: "quarto-resource-document-website-aliases" }, image: { $ref: "quarto-resource-document-website-image", description: "quarto-resource-document-website-image" }, "image-height": { $ref: "quarto-resource-document-website-image-height", description: "quarto-resource-document-website-image-height" }, "image-width": { $ref: "quarto-resource-document-website-image-width", description: "quarto-resource-document-website-image-width" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_layout|pageLayout|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_location|citationLocation|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^html5(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-fold": { $ref: "quarto-resource-cell-codeoutput-code-fold", description: "quarto-resource-cell-codeoutput-code-fold" }, "code-summary": { $ref: "quarto-resource-cell-codeoutput-code-summary", description: "quarto-resource-cell-codeoutput-code-summary" }, "code-overflow": { $ref: "quarto-resource-cell-codeoutput-code-overflow", description: "quarto-resource-cell-codeoutput-code-overflow" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "cap-location": { $ref: "quarto-resource-cell-pagelayout-cap-location", description: "quarto-resource-cell-pagelayout-cap-location" }, "fig-cap-location": { $ref: "quarto-resource-cell-pagelayout-fig-cap-location", description: "quarto-resource-cell-pagelayout-fig-cap-location" }, "tbl-cap-location": { $ref: "quarto-resource-cell-pagelayout-tbl-cap-location", description: "quarto-resource-cell-pagelayout-tbl-cap-location" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, "abstract-title": { $ref: "quarto-resource-document-attributes-abstract-title", description: "quarto-resource-document-attributes-abstract-title" }, doi: { $ref: "quarto-resource-document-attributes-doi", description: "quarto-resource-document-attributes-doi" }, "code-copy": { $ref: "quarto-resource-document-code-code-copy", description: "quarto-resource-document-code-code-copy" }, "code-link": { $ref: "quarto-resource-document-code-code-link", description: "quarto-resource-document-code-code-link" }, "code-tools": { $ref: "quarto-resource-document-code-code-tools", description: "quarto-resource-document-code-code-tools" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, fontcolor: { $ref: "quarto-resource-document-colors-fontcolor", description: "quarto-resource-document-colors-fontcolor" }, linkcolor: { $ref: "quarto-resource-document-colors-linkcolor", description: "quarto-resource-document-colors-linkcolor" }, monobackgroundcolor: { $ref: "quarto-resource-document-colors-monobackgroundcolor", description: "quarto-resource-document-colors-monobackgroundcolor" }, backgroundcolor: { $ref: "quarto-resource-document-colors-backgroundcolor", description: "quarto-resource-document-colors-backgroundcolor" }, comments: { $ref: "quarto-resource-document-comments-comments", description: "quarto-resource-document-comments-comments" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "fig-responsive": { $ref: "quarto-resource-document-figures-fig-responsive", description: "quarto-resource-document-figures-fig-responsive" }, mainfont: { $ref: "quarto-resource-document-fonts-mainfont", description: "quarto-resource-document-fonts-mainfont" }, monofont: { $ref: "quarto-resource-document-fonts-monofont", description: "quarto-resource-document-fonts-monofont" }, fontsize: { $ref: "quarto-resource-document-fonts-fontsize", description: "quarto-resource-document-fonts-fontsize" }, linestretch: { $ref: "quarto-resource-document-fonts-linestretch", description: "quarto-resource-document-fonts-linestretch" }, "footnotes-hover": { $ref: "quarto-resource-document-footnotes-footnotes-hover", description: "quarto-resource-document-footnotes-footnotes-hover" }, "reference-location": { $ref: "quarto-resource-document-footnotes-reference-location", description: "quarto-resource-document-footnotes-reference-location" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "keep-source": { $ref: "quarto-resource-document-hidden-keep-source", description: "quarto-resource-document-hidden-keep-source" }, "keep-hidden": { $ref: "quarto-resource-document-hidden-keep-hidden", description: "quarto-resource-document-hidden-keep-hidden" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, resources: { $ref: "quarto-resource-document-includes-resources", description: "quarto-resource-document-includes-resources" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, classoption: { $ref: "quarto-resource-document-layout-classoption", description: "quarto-resource-document-layout-classoption" }, "page-layout": { $ref: "quarto-resource-document-layout-page-layout", description: "quarto-resource-document-layout-page-layout" }, "max-width": { $ref: "quarto-resource-document-layout-max-width", description: "quarto-resource-document-layout-max-width" }, "margin-left": { $ref: "quarto-resource-document-layout-margin-left", description: "quarto-resource-document-layout-margin-left" }, "margin-right": { $ref: "quarto-resource-document-layout-margin-right", description: "quarto-resource-document-layout-margin-right" }, "margin-top": { $ref: "quarto-resource-document-layout-margin-top", description: "quarto-resource-document-layout-margin-top" }, "margin-bottom": { $ref: "quarto-resource-document-layout-margin-bottom", description: "quarto-resource-document-layout-margin-bottom" }, "link-external-icon": { $ref: "quarto-resource-document-links-link-external-icon", description: "quarto-resource-document-links-link-external-icon" }, "link-external-newwindow": { $ref: "quarto-resource-document-links-link-external-newwindow", description: "quarto-resource-document-links-link-external-newwindow" }, "link-external-filter": { $ref: "quarto-resource-document-links-link-external-filter", description: "quarto-resource-document-links-link-external-filter" }, keywords: { $ref: "quarto-resource-document-metadata-keywords", description: "quarto-resource-document-metadata-keywords" }, pagetitle: { $ref: "quarto-resource-document-metadata-pagetitle", description: "quarto-resource-document-metadata-pagetitle" }, "title-prefix": { $ref: "quarto-resource-document-metadata-title-prefix", description: "quarto-resource-document-metadata-title-prefix" }, "description-meta": { $ref: "quarto-resource-document-metadata-description-meta", description: "quarto-resource-document-metadata-description-meta" }, "author-meta": { $ref: "quarto-resource-document-metadata-author-meta", description: "quarto-resource-document-metadata-author-meta" }, "date-meta": { $ref: "quarto-resource-document-metadata-date-meta", description: "quarto-resource-document-metadata-date-meta" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-depth": { $ref: "quarto-resource-document-numbering-number-depth", description: "quarto-resource-document-numbering-number-depth" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, theme: { $ref: "quarto-resource-document-options-theme", description: "quarto-resource-document-options-theme" }, minimal: { $ref: "quarto-resource-document-options-minimal", description: "quarto-resource-document-options-minimal" }, "document-css": { $ref: "quarto-resource-document-options-document-css", description: "quarto-resource-document-options-document-css" }, css: { $ref: "quarto-resource-document-options-css", description: "quarto-resource-document-options-css" }, "anchor-sections": { $ref: "quarto-resource-document-options-anchor-sections", description: "quarto-resource-document-options-anchor-sections" }, "html-math-method": { $ref: "quarto-resource-document-options-html-math-method", description: "quarto-resource-document-options-html-math-method" }, "section-divs": { $ref: "quarto-resource-document-options-section-divs", description: "quarto-resource-document-options-section-divs" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, "email-obfuscation": { $ref: "quarto-resource-document-options-email-obfuscation", description: "quarto-resource-document-options-email-obfuscation" }, "html-q-tags": { $ref: "quarto-resource-document-options-html-q-tags", description: "quarto-resource-document-options-html-q-tags" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, "citations-hover": { $ref: "quarto-resource-document-references-citations-hover", description: "quarto-resource-document-references-citations-hover" }, "citation-location": { $ref: "quarto-resource-document-references-citation-location", description: "quarto-resource-document-references-citation-location" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, "self-contained": { $ref: "quarto-resource-document-render-self-contained", description: "quarto-resource-document-render-self-contained" }, "self-contained-math": { $ref: "quarto-resource-document-render-self-contained-math", description: "quarto-resource-document-render-self-contained-math" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, "strip-comments": { $ref: "quarto-resource-document-text-strip-comments", description: "quarto-resource-document-text-strip-comments" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" }, "toc-location": { $ref: "quarto-resource-document-toc-toc-location", description: "quarto-resource-document-toc-toc-location" }, "toc-title": { $ref: "quarto-resource-document-toc-toc-title", description: "quarto-resource-document-toc-toc-title" }, search: { $ref: "quarto-resource-document-website-search", description: "quarto-resource-document-website-search" }, aliases: { $ref: "quarto-resource-document-website-aliases", description: "quarto-resource-document-website-aliases" }, image: { $ref: "quarto-resource-document-website-image", description: "quarto-resource-document-website-image" }, "image-height": { $ref: "quarto-resource-document-website-image-height", description: "quarto-resource-document-website-image-height" }, "image-width": { $ref: "quarto-resource-document-website-image-width", description: "quarto-resource-document-website-image-width" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_layout|pageLayout|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_location|citationLocation|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^icml(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^ipynb(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "markdown-headings": { $ref: "quarto-resource-document-options-markdown-headings", description: "quarto-resource-document-options-markdown-headings" }, "ipynb-output": { $ref: "quarto-resource-document-options-ipynb-output", description: "quarto-resource-document-options-ipynb-output" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|markdown_headings|markdownHeadings|ipynb_output|ipynbOutput|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^jats(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, affiliation: { $ref: "quarto-resource-document-attributes-affiliation", description: "quarto-resource-document-attributes-affiliation" }, copyright: { $ref: "quarto-resource-document-attributes-copyright", description: "quarto-resource-document-attributes-copyright" }, article: { $ref: "quarto-resource-document-attributes-article", description: "quarto-resource-document-attributes-article" }, journal: { $ref: "quarto-resource-document-attributes-journal", description: "quarto-resource-document-attributes-journal" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, notes: { $ref: "quarto-resource-document-attributes-notes", description: "quarto-resource-document-attributes-notes" }, tags: { $ref: "quarto-resource-document-attributes-tags", description: "quarto-resource-document-attributes-tags" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^jats_archiving(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, affiliation: { $ref: "quarto-resource-document-attributes-affiliation", description: "quarto-resource-document-attributes-affiliation" }, copyright: { $ref: "quarto-resource-document-attributes-copyright", description: "quarto-resource-document-attributes-copyright" }, article: { $ref: "quarto-resource-document-attributes-article", description: "quarto-resource-document-attributes-article" }, journal: { $ref: "quarto-resource-document-attributes-journal", description: "quarto-resource-document-attributes-journal" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, notes: { $ref: "quarto-resource-document-attributes-notes", description: "quarto-resource-document-attributes-notes" }, tags: { $ref: "quarto-resource-document-attributes-tags", description: "quarto-resource-document-attributes-tags" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^jats_articleauthoring(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, affiliation: { $ref: "quarto-resource-document-attributes-affiliation", description: "quarto-resource-document-attributes-affiliation" }, copyright: { $ref: "quarto-resource-document-attributes-copyright", description: "quarto-resource-document-attributes-copyright" }, article: { $ref: "quarto-resource-document-attributes-article", description: "quarto-resource-document-attributes-article" }, journal: { $ref: "quarto-resource-document-attributes-journal", description: "quarto-resource-document-attributes-journal" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, notes: { $ref: "quarto-resource-document-attributes-notes", description: "quarto-resource-document-attributes-notes" }, tags: { $ref: "quarto-resource-document-attributes-tags", description: "quarto-resource-document-attributes-tags" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^jats_publishing(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, affiliation: { $ref: "quarto-resource-document-attributes-affiliation", description: "quarto-resource-document-attributes-affiliation" }, copyright: { $ref: "quarto-resource-document-attributes-copyright", description: "quarto-resource-document-attributes-copyright" }, article: { $ref: "quarto-resource-document-attributes-article", description: "quarto-resource-document-attributes-article" }, journal: { $ref: "quarto-resource-document-attributes-journal", description: "quarto-resource-document-attributes-journal" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, notes: { $ref: "quarto-resource-document-attributes-notes", description: "quarto-resource-document-attributes-notes" }, tags: { $ref: "quarto-resource-document-attributes-tags", description: "quarto-resource-document-attributes-tags" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^jira(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^json(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^latex(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "fig-env": { $ref: "quarto-resource-cell-figure-fig-env", description: "quarto-resource-cell-figure-fig-env" }, "fig-pos": { $ref: "quarto-resource-cell-figure-fig-pos", description: "quarto-resource-cell-figure-fig-pos" }, "cap-location": { $ref: "quarto-resource-cell-pagelayout-cap-location", description: "quarto-resource-cell-pagelayout-cap-location" }, "fig-cap-location": { $ref: "quarto-resource-cell-pagelayout-fig-cap-location", description: "quarto-resource-cell-pagelayout-fig-cap-location" }, "tbl-cap-location": { $ref: "quarto-resource-cell-pagelayout-tbl-cap-location", description: "quarto-resource-cell-pagelayout-tbl-cap-location" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, thanks: { $ref: "quarto-resource-document-attributes-thanks", description: "quarto-resource-document-attributes-thanks" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, listings: { $ref: "quarto-resource-document-code-listings", description: "quarto-resource-document-code-listings" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, linkcolor: { $ref: "quarto-resource-document-colors-linkcolor", description: "quarto-resource-document-colors-linkcolor" }, filecolor: { $ref: "quarto-resource-document-colors-filecolor", description: "quarto-resource-document-colors-filecolor" }, citecolor: { $ref: "quarto-resource-document-colors-citecolor", description: "quarto-resource-document-colors-citecolor" }, urlcolor: { $ref: "quarto-resource-document-colors-urlcolor", description: "quarto-resource-document-colors-urlcolor" }, toccolor: { $ref: "quarto-resource-document-colors-toccolor", description: "quarto-resource-document-colors-toccolor" }, colorlinks: { $ref: "quarto-resource-document-colors-colorlinks", description: "quarto-resource-document-colors-colorlinks" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, mainfont: { $ref: "quarto-resource-document-fonts-mainfont", description: "quarto-resource-document-fonts-mainfont" }, monofont: { $ref: "quarto-resource-document-fonts-monofont", description: "quarto-resource-document-fonts-monofont" }, fontsize: { $ref: "quarto-resource-document-fonts-fontsize", description: "quarto-resource-document-fonts-fontsize" }, fontenc: { $ref: "quarto-resource-document-fonts-fontenc", description: "quarto-resource-document-fonts-fontenc" }, fontfamily: { $ref: "quarto-resource-document-fonts-fontfamily", description: "quarto-resource-document-fonts-fontfamily" }, fontfamilyoptions: { $ref: "quarto-resource-document-fonts-fontfamilyoptions", description: "quarto-resource-document-fonts-fontfamilyoptions" }, sansfont: { $ref: "quarto-resource-document-fonts-sansfont", description: "quarto-resource-document-fonts-sansfont" }, mathfont: { $ref: "quarto-resource-document-fonts-mathfont", description: "quarto-resource-document-fonts-mathfont" }, CJKmainfont: { $ref: "quarto-resource-document-fonts-CJKmainfont", description: "quarto-resource-document-fonts-CJKmainfont" }, mainfontoptions: { $ref: "quarto-resource-document-fonts-mainfontoptions", description: "quarto-resource-document-fonts-mainfontoptions" }, sansfontoptions: { $ref: "quarto-resource-document-fonts-sansfontoptions", description: "quarto-resource-document-fonts-sansfontoptions" }, monofontoptions: { $ref: "quarto-resource-document-fonts-monofontoptions", description: "quarto-resource-document-fonts-monofontoptions" }, mathfontoptions: { $ref: "quarto-resource-document-fonts-mathfontoptions", description: "quarto-resource-document-fonts-mathfontoptions" }, CJKoptions: { $ref: "quarto-resource-document-fonts-CJKoptions", description: "quarto-resource-document-fonts-CJKoptions" }, microtypeoptions: { $ref: "quarto-resource-document-fonts-microtypeoptions", description: "quarto-resource-document-fonts-microtypeoptions" }, linestretch: { $ref: "quarto-resource-document-fonts-linestretch", description: "quarto-resource-document-fonts-linestretch" }, "links-as-notes": { $ref: "quarto-resource-document-footnotes-links-as-notes", description: "quarto-resource-document-footnotes-links-as-notes" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, documentclass: { $ref: "quarto-resource-document-layout-documentclass", description: "quarto-resource-document-layout-documentclass" }, classoption: { $ref: "quarto-resource-document-layout-classoption", description: "quarto-resource-document-layout-classoption" }, pagestyle: { $ref: "quarto-resource-document-layout-pagestyle", description: "quarto-resource-document-layout-pagestyle" }, papersize: { $ref: "quarto-resource-document-layout-papersize", description: "quarto-resource-document-layout-papersize" }, "margin-left": { $ref: "quarto-resource-document-layout-margin-left", description: "quarto-resource-document-layout-margin-left" }, "margin-right": { $ref: "quarto-resource-document-layout-margin-right", description: "quarto-resource-document-layout-margin-right" }, "margin-top": { $ref: "quarto-resource-document-layout-margin-top", description: "quarto-resource-document-layout-margin-top" }, "margin-bottom": { $ref: "quarto-resource-document-layout-margin-bottom", description: "quarto-resource-document-layout-margin-bottom" }, geometry: { $ref: "quarto-resource-document-layout-geometry", description: "quarto-resource-document-layout-geometry" }, hyperrefoptions: { $ref: "quarto-resource-document-layout-hyperrefoptions", description: "quarto-resource-document-layout-hyperrefoptions" }, indent: { $ref: "quarto-resource-document-layout-indent", description: "quarto-resource-document-layout-indent" }, "block-headings": { $ref: "quarto-resource-document-layout-block-headings", description: "quarto-resource-document-layout-block-headings" }, keywords: { $ref: "quarto-resource-document-metadata-keywords", description: "quarto-resource-document-metadata-keywords" }, subject: { $ref: "quarto-resource-document-metadata-subject", description: "quarto-resource-document-metadata-subject" }, "title-meta": { $ref: "quarto-resource-document-metadata-title-meta", description: "quarto-resource-document-metadata-title-meta" }, "author-meta": { $ref: "quarto-resource-document-metadata-author-meta", description: "quarto-resource-document-metadata-author-meta" }, "date-meta": { $ref: "quarto-resource-document-metadata-date-meta", description: "quarto-resource-document-metadata-date-meta" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-depth": { $ref: "quarto-resource-document-numbering-number-depth", description: "quarto-resource-document-numbering-number-depth" }, secnumdepth: { $ref: "quarto-resource-document-numbering-secnumdepth", description: "quarto-resource-document-numbering-secnumdepth" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "top-level-division": { $ref: "quarto-resource-document-numbering-top-level-division", description: "quarto-resource-document-numbering-top-level-division" }, "pdf-engine": { $ref: "quarto-resource-document-options-pdf-engine", description: "quarto-resource-document-options-pdf-engine" }, "pdf-engine-opt": { $ref: "quarto-resource-document-options-pdf-engine-opt", description: "quarto-resource-document-options-pdf-engine-opt" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, "cite-method": { $ref: "quarto-resource-document-references-cite-method", description: "quarto-resource-document-references-cite-method" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, biblatexoptions: { $ref: "quarto-resource-document-references-biblatexoptions", description: "quarto-resource-document-references-biblatexoptions" }, natbiboptions: { $ref: "quarto-resource-document-references-natbiboptions", description: "quarto-resource-document-references-natbiboptions" }, "biblio-style": { $ref: "quarto-resource-document-references-biblio-style", description: "quarto-resource-document-references-biblio-style" }, "biblio-title": { $ref: "quarto-resource-document-references-biblio-title", description: "quarto-resource-document-references-biblio-title" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" }, "toc-title": { $ref: "quarto-resource-document-toc-toc-title", description: "quarto-resource-document-toc-toc-title" }, lof: { $ref: "quarto-resource-document-toc-lof", description: "quarto-resource-document-toc-lof" }, lot: { $ref: "quarto-resource-document-toc-lot", description: "quarto-resource-document-toc-lot" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|links_as_notes|linksAsNotes|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|title_meta|titleMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^man(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, adjusting: { $ref: "quarto-resource-document-formatting-adjusting", description: "quarto-resource-document-formatting-adjusting" }, hyphenate: { $ref: "quarto-resource-document-formatting-hyphenate", description: "quarto-resource-document-formatting-hyphenate" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, footer: { $ref: "quarto-resource-document-includes-footer", description: "quarto-resource-document-includes-footer" }, header: { $ref: "quarto-resource-document-includes-header", description: "quarto-resource-document-includes-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, section: { $ref: "quarto-resource-document-options-section", description: "quarto-resource-document-options-section" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^markdown(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "reference-location": { $ref: "quarto-resource-document-footnotes-reference-location", description: "quarto-resource-document-footnotes-reference-location" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "prefer-html": { $ref: "quarto-resource-document-hidden-prefer-html", description: "quarto-resource-document-hidden-prefer-html" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, variant: { $ref: "quarto-resource-document-options-variant", description: "quarto-resource-document-options-variant" }, "markdown-headings": { $ref: "quarto-resource-document-options-markdown-headings", description: "quarto-resource-document-options-markdown-headings" }, "keep-yaml": { $ref: "quarto-resource-document-options-keep-yaml", description: "quarto-resource-document-options-keep-yaml" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, "strip-comments": { $ref: "quarto-resource-document-text-strip-comments", description: "quarto-resource-document-text-strip-comments" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^markdown_github(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^markdown_mmd(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^markdown_phpextra(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^markdown_strict(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^markua(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "reference-location": { $ref: "quarto-resource-document-footnotes-reference-location", description: "quarto-resource-document-footnotes-reference-location" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "prefer-html": { $ref: "quarto-resource-document-hidden-prefer-html", description: "quarto-resource-document-hidden-prefer-html" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, variant: { $ref: "quarto-resource-document-options-variant", description: "quarto-resource-document-options-variant" }, "markdown-headings": { $ref: "quarto-resource-document-options-markdown-headings", description: "quarto-resource-document-options-markdown-headings" }, "keep-yaml": { $ref: "quarto-resource-document-options-keep-yaml", description: "quarto-resource-document-options-keep-yaml" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, "strip-comments": { $ref: "quarto-resource-document-text-strip-comments", description: "quarto-resource-document-text-strip-comments" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^mediawiki(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^ms(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, fontfamily: { $ref: "quarto-resource-document-fonts-fontfamily", description: "quarto-resource-document-fonts-fontfamily" }, pointsize: { $ref: "quarto-resource-document-fonts-pointsize", description: "quarto-resource-document-fonts-pointsize" }, lineheight: { $ref: "quarto-resource-document-fonts-lineheight", description: "quarto-resource-document-fonts-lineheight" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, indent: { $ref: "quarto-resource-document-layout-indent", description: "quarto-resource-document-layout-indent" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "pdf-engine": { $ref: "quarto-resource-document-options-pdf-engine", description: "quarto-resource-document-options-pdf-engine" }, "pdf-engine-opt": { $ref: "quarto-resource-document-options-pdf-engine-opt", description: "quarto-resource-document-options-pdf-engine-opt" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_line_numbers|codeLineNumbers|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^muse(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "reference-location": { $ref: "quarto-resource-document-footnotes-reference-location", description: "quarto-resource-document-footnotes-reference-location" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^native(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^odt(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "page-width": { $ref: "quarto-resource-document-layout-page-width", description: "quarto-resource-document-layout-page-width" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "reference-doc": { $ref: "quarto-resource-document-options-reference-doc", description: "quarto-resource-document-options-reference-doc" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" }, "toc-title": { $ref: "quarto-resource-document-toc-toc-title", description: "quarto-resource-document-toc-toc-title" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(fig_align|figAlign|tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_width|pageWidth|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|reference_doc|referenceDoc|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^opendocument(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "page-width": { $ref: "quarto-resource-document-layout-page-width", description: "quarto-resource-document-layout-page-width" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" }, "toc-title": { $ref: "quarto-resource-document-toc-toc-title", description: "quarto-resource-document-toc-toc-title" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(fig_align|figAlign|tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_width|pageWidth|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^opml(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^org(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^pdf(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "fig-env": { $ref: "quarto-resource-cell-figure-fig-env", description: "quarto-resource-cell-figure-fig-env" }, "fig-pos": { $ref: "quarto-resource-cell-figure-fig-pos", description: "quarto-resource-cell-figure-fig-pos" }, "cap-location": { $ref: "quarto-resource-cell-pagelayout-cap-location", description: "quarto-resource-cell-pagelayout-cap-location" }, "fig-cap-location": { $ref: "quarto-resource-cell-pagelayout-fig-cap-location", description: "quarto-resource-cell-pagelayout-fig-cap-location" }, "tbl-cap-location": { $ref: "quarto-resource-cell-pagelayout-tbl-cap-location", description: "quarto-resource-cell-pagelayout-tbl-cap-location" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, thanks: { $ref: "quarto-resource-document-attributes-thanks", description: "quarto-resource-document-attributes-thanks" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, listings: { $ref: "quarto-resource-document-code-listings", description: "quarto-resource-document-code-listings" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, linkcolor: { $ref: "quarto-resource-document-colors-linkcolor", description: "quarto-resource-document-colors-linkcolor" }, filecolor: { $ref: "quarto-resource-document-colors-filecolor", description: "quarto-resource-document-colors-filecolor" }, citecolor: { $ref: "quarto-resource-document-colors-citecolor", description: "quarto-resource-document-colors-citecolor" }, urlcolor: { $ref: "quarto-resource-document-colors-urlcolor", description: "quarto-resource-document-colors-urlcolor" }, toccolor: { $ref: "quarto-resource-document-colors-toccolor", description: "quarto-resource-document-colors-toccolor" }, colorlinks: { $ref: "quarto-resource-document-colors-colorlinks", description: "quarto-resource-document-colors-colorlinks" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, mainfont: { $ref: "quarto-resource-document-fonts-mainfont", description: "quarto-resource-document-fonts-mainfont" }, monofont: { $ref: "quarto-resource-document-fonts-monofont", description: "quarto-resource-document-fonts-monofont" }, fontsize: { $ref: "quarto-resource-document-fonts-fontsize", description: "quarto-resource-document-fonts-fontsize" }, fontenc: { $ref: "quarto-resource-document-fonts-fontenc", description: "quarto-resource-document-fonts-fontenc" }, fontfamily: { $ref: "quarto-resource-document-fonts-fontfamily", description: "quarto-resource-document-fonts-fontfamily" }, fontfamilyoptions: { $ref: "quarto-resource-document-fonts-fontfamilyoptions", description: "quarto-resource-document-fonts-fontfamilyoptions" }, sansfont: { $ref: "quarto-resource-document-fonts-sansfont", description: "quarto-resource-document-fonts-sansfont" }, mathfont: { $ref: "quarto-resource-document-fonts-mathfont", description: "quarto-resource-document-fonts-mathfont" }, CJKmainfont: { $ref: "quarto-resource-document-fonts-CJKmainfont", description: "quarto-resource-document-fonts-CJKmainfont" }, mainfontoptions: { $ref: "quarto-resource-document-fonts-mainfontoptions", description: "quarto-resource-document-fonts-mainfontoptions" }, sansfontoptions: { $ref: "quarto-resource-document-fonts-sansfontoptions", description: "quarto-resource-document-fonts-sansfontoptions" }, monofontoptions: { $ref: "quarto-resource-document-fonts-monofontoptions", description: "quarto-resource-document-fonts-monofontoptions" }, mathfontoptions: { $ref: "quarto-resource-document-fonts-mathfontoptions", description: "quarto-resource-document-fonts-mathfontoptions" }, CJKoptions: { $ref: "quarto-resource-document-fonts-CJKoptions", description: "quarto-resource-document-fonts-CJKoptions" }, microtypeoptions: { $ref: "quarto-resource-document-fonts-microtypeoptions", description: "quarto-resource-document-fonts-microtypeoptions" }, linestretch: { $ref: "quarto-resource-document-fonts-linestretch", description: "quarto-resource-document-fonts-linestretch" }, "links-as-notes": { $ref: "quarto-resource-document-footnotes-links-as-notes", description: "quarto-resource-document-footnotes-links-as-notes" }, "reference-location": { $ref: "quarto-resource-document-footnotes-reference-location", description: "quarto-resource-document-footnotes-reference-location" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "latex-auto-mk": { $ref: "quarto-resource-document-latexmk-latex-auto-mk", description: "quarto-resource-document-latexmk-latex-auto-mk" }, "latex-auto-install": { $ref: "quarto-resource-document-latexmk-latex-auto-install", description: "quarto-resource-document-latexmk-latex-auto-install" }, "latex-min-runs": { $ref: "quarto-resource-document-latexmk-latex-min-runs", description: "quarto-resource-document-latexmk-latex-min-runs" }, "latex-max-runs": { $ref: "quarto-resource-document-latexmk-latex-max-runs", description: "quarto-resource-document-latexmk-latex-max-runs" }, "latex-clean": { $ref: "quarto-resource-document-latexmk-latex-clean", description: "quarto-resource-document-latexmk-latex-clean" }, "latex-makeindex": { $ref: "quarto-resource-document-latexmk-latex-makeindex", description: "quarto-resource-document-latexmk-latex-makeindex" }, "latex-makeindex-opts": { $ref: "quarto-resource-document-latexmk-latex-makeindex-opts", description: "quarto-resource-document-latexmk-latex-makeindex-opts" }, "latex-tlmgr-opts": { $ref: "quarto-resource-document-latexmk-latex-tlmgr-opts", description: "quarto-resource-document-latexmk-latex-tlmgr-opts" }, "latex-output-dir": { $ref: "quarto-resource-document-latexmk-latex-output-dir", description: "quarto-resource-document-latexmk-latex-output-dir" }, documentclass: { $ref: "quarto-resource-document-layout-documentclass", description: "quarto-resource-document-layout-documentclass" }, classoption: { $ref: "quarto-resource-document-layout-classoption", description: "quarto-resource-document-layout-classoption" }, pagestyle: { $ref: "quarto-resource-document-layout-pagestyle", description: "quarto-resource-document-layout-pagestyle" }, papersize: { $ref: "quarto-resource-document-layout-papersize", description: "quarto-resource-document-layout-papersize" }, "margin-left": { $ref: "quarto-resource-document-layout-margin-left", description: "quarto-resource-document-layout-margin-left" }, "margin-right": { $ref: "quarto-resource-document-layout-margin-right", description: "quarto-resource-document-layout-margin-right" }, "margin-top": { $ref: "quarto-resource-document-layout-margin-top", description: "quarto-resource-document-layout-margin-top" }, "margin-bottom": { $ref: "quarto-resource-document-layout-margin-bottom", description: "quarto-resource-document-layout-margin-bottom" }, geometry: { $ref: "quarto-resource-document-layout-geometry", description: "quarto-resource-document-layout-geometry" }, hyperrefoptions: { $ref: "quarto-resource-document-layout-hyperrefoptions", description: "quarto-resource-document-layout-hyperrefoptions" }, indent: { $ref: "quarto-resource-document-layout-indent", description: "quarto-resource-document-layout-indent" }, "block-headings": { $ref: "quarto-resource-document-layout-block-headings", description: "quarto-resource-document-layout-block-headings" }, keywords: { $ref: "quarto-resource-document-metadata-keywords", description: "quarto-resource-document-metadata-keywords" }, subject: { $ref: "quarto-resource-document-metadata-subject", description: "quarto-resource-document-metadata-subject" }, "title-meta": { $ref: "quarto-resource-document-metadata-title-meta", description: "quarto-resource-document-metadata-title-meta" }, "author-meta": { $ref: "quarto-resource-document-metadata-author-meta", description: "quarto-resource-document-metadata-author-meta" }, "date-meta": { $ref: "quarto-resource-document-metadata-date-meta", description: "quarto-resource-document-metadata-date-meta" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-depth": { $ref: "quarto-resource-document-numbering-number-depth", description: "quarto-resource-document-numbering-number-depth" }, secnumdepth: { $ref: "quarto-resource-document-numbering-secnumdepth", description: "quarto-resource-document-numbering-secnumdepth" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "top-level-division": { $ref: "quarto-resource-document-numbering-top-level-division", description: "quarto-resource-document-numbering-top-level-division" }, "pdf-engine": { $ref: "quarto-resource-document-options-pdf-engine", description: "quarto-resource-document-options-pdf-engine" }, "pdf-engine-opt": { $ref: "quarto-resource-document-options-pdf-engine-opt", description: "quarto-resource-document-options-pdf-engine-opt" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, "cite-method": { $ref: "quarto-resource-document-references-cite-method", description: "quarto-resource-document-references-cite-method" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, biblatexoptions: { $ref: "quarto-resource-document-references-biblatexoptions", description: "quarto-resource-document-references-biblatexoptions" }, natbiboptions: { $ref: "quarto-resource-document-references-natbiboptions", description: "quarto-resource-document-references-natbiboptions" }, "biblio-style": { $ref: "quarto-resource-document-references-biblio-style", description: "quarto-resource-document-references-biblio-style" }, "biblio-title": { $ref: "quarto-resource-document-references-biblio-title", description: "quarto-resource-document-references-biblio-title" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "keep-tex": { $ref: "quarto-resource-document-render-keep-tex", description: "quarto-resource-document-render-keep-tex" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" }, "toc-title": { $ref: "quarto-resource-document-toc-toc-title", description: "quarto-resource-document-toc-toc-title" }, lof: { $ref: "quarto-resource-document-toc-lof", description: "quarto-resource-document-toc-lof" }, lot: { $ref: "quarto-resource-document-toc-lot", description: "quarto-resource-document-toc-lot" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|links_as_notes|linksAsNotes|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|latex_auto_mk|latexAutoMk|latex_auto_install|latexAutoInstall|latex_min_runs|latexMinRuns|latex_max_runs|latexMaxRuns|latex_clean|latexClean|latex_makeindex|latexMakeindex|latex_makeindex_opts|latexMakeindexOpts|latex_tlmgr_opts|latexTlmgrOpts|latex_output_dir|latexOutputDir|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|title_meta|titleMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|keep_tex|keepTex|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^plain(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^pptx(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, subject: { $ref: "quarto-resource-document-metadata-subject", description: "quarto-resource-document-metadata-subject" }, description: { $ref: "quarto-resource-document-metadata-description", description: "quarto-resource-document-metadata-description" }, category: { $ref: "quarto-resource-document-metadata-category", description: "quarto-resource-document-metadata-category" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "reference-doc": { $ref: "quarto-resource-document-options-reference-doc", description: "quarto-resource-document-options-reference-doc" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, incremental: { $ref: "quarto-resource-document-slides-incremental", description: "quarto-resource-document-slides-incremental" }, "slide-level": { $ref: "quarto-resource-document-slides-slide-level", description: "quarto-resource-document-slides-slide-level" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" }, "toc-title": { $ref: "quarto-resource-document-toc-toc-title", description: "quarto-resource-document-toc-toc-title" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|reference_doc|referenceDoc|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^revealjs(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-fold": { $ref: "quarto-resource-cell-codeoutput-code-fold", description: "quarto-resource-cell-codeoutput-code-fold" }, "code-summary": { $ref: "quarto-resource-cell-codeoutput-code-summary", description: "quarto-resource-cell-codeoutput-code-summary" }, "code-overflow": { $ref: "quarto-resource-cell-codeoutput-code-overflow", description: "quarto-resource-cell-codeoutput-code-overflow" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "cap-location": { $ref: "quarto-resource-cell-pagelayout-cap-location", description: "quarto-resource-cell-pagelayout-cap-location" }, "fig-cap-location": { $ref: "quarto-resource-cell-pagelayout-fig-cap-location", description: "quarto-resource-cell-pagelayout-fig-cap-location" }, "tbl-cap-location": { $ref: "quarto-resource-cell-pagelayout-tbl-cap-location", description: "quarto-resource-cell-pagelayout-tbl-cap-location" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, institute: { $ref: "quarto-resource-document-attributes-institute", description: "quarto-resource-document-attributes-institute" }, "code-copy": { $ref: "quarto-resource-document-code-code-copy", description: "quarto-resource-document-code-code-copy" }, "code-link": { $ref: "quarto-resource-document-code-code-link", description: "quarto-resource-document-code-code-link" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, fontcolor: { $ref: "quarto-resource-document-colors-fontcolor", description: "quarto-resource-document-colors-fontcolor" }, linkcolor: { $ref: "quarto-resource-document-colors-linkcolor", description: "quarto-resource-document-colors-linkcolor" }, monobackgroundcolor: { $ref: "quarto-resource-document-colors-monobackgroundcolor", description: "quarto-resource-document-colors-monobackgroundcolor" }, backgroundcolor: { $ref: "quarto-resource-document-colors-backgroundcolor", description: "quarto-resource-document-colors-backgroundcolor" }, comments: { $ref: "quarto-resource-document-comments-comments", description: "quarto-resource-document-comments-comments" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "fig-responsive": { $ref: "quarto-resource-document-figures-fig-responsive", description: "quarto-resource-document-figures-fig-responsive" }, mainfont: { $ref: "quarto-resource-document-fonts-mainfont", description: "quarto-resource-document-fonts-mainfont" }, monofont: { $ref: "quarto-resource-document-fonts-monofont", description: "quarto-resource-document-fonts-monofont" }, fontsize: { $ref: "quarto-resource-document-fonts-fontsize", description: "quarto-resource-document-fonts-fontsize" }, linestretch: { $ref: "quarto-resource-document-fonts-linestretch", description: "quarto-resource-document-fonts-linestretch" }, "footnotes-hover": { $ref: "quarto-resource-document-footnotes-footnotes-hover", description: "quarto-resource-document-footnotes-footnotes-hover" }, "reference-location": { $ref: "quarto-resource-document-footnotes-reference-location", description: "quarto-resource-document-footnotes-reference-location" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, resources: { $ref: "quarto-resource-document-includes-resources", description: "quarto-resource-document-includes-resources" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, classoption: { $ref: "quarto-resource-document-layout-classoption", description: "quarto-resource-document-layout-classoption" }, "max-width": { $ref: "quarto-resource-document-layout-max-width", description: "quarto-resource-document-layout-max-width" }, "margin-left": { $ref: "quarto-resource-document-layout-margin-left", description: "quarto-resource-document-layout-margin-left" }, "margin-right": { $ref: "quarto-resource-document-layout-margin-right", description: "quarto-resource-document-layout-margin-right" }, "margin-top": { $ref: "quarto-resource-document-layout-margin-top", description: "quarto-resource-document-layout-margin-top" }, "margin-bottom": { $ref: "quarto-resource-document-layout-margin-bottom", description: "quarto-resource-document-layout-margin-bottom" }, "reveal-js-url": { $ref: "quarto-resource-document-library-reveal-js-url", description: "quarto-resource-document-library-reveal-js-url" }, "link-external-icon": { $ref: "quarto-resource-document-links-link-external-icon", description: "quarto-resource-document-links-link-external-icon" }, "link-external-newwindow": { $ref: "quarto-resource-document-links-link-external-newwindow", description: "quarto-resource-document-links-link-external-newwindow" }, "link-external-filter": { $ref: "quarto-resource-document-links-link-external-filter", description: "quarto-resource-document-links-link-external-filter" }, keywords: { $ref: "quarto-resource-document-metadata-keywords", description: "quarto-resource-document-metadata-keywords" }, pagetitle: { $ref: "quarto-resource-document-metadata-pagetitle", description: "quarto-resource-document-metadata-pagetitle" }, "title-prefix": { $ref: "quarto-resource-document-metadata-title-prefix", description: "quarto-resource-document-metadata-title-prefix" }, "description-meta": { $ref: "quarto-resource-document-metadata-description-meta", description: "quarto-resource-document-metadata-description-meta" }, "author-meta": { $ref: "quarto-resource-document-metadata-author-meta", description: "quarto-resource-document-metadata-author-meta" }, "date-meta": { $ref: "quarto-resource-document-metadata-date-meta", description: "quarto-resource-document-metadata-date-meta" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-depth": { $ref: "quarto-resource-document-numbering-number-depth", description: "quarto-resource-document-numbering-number-depth" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, theme: { $ref: "quarto-resource-document-options-theme", description: "quarto-resource-document-options-theme" }, "document-css": { $ref: "quarto-resource-document-options-document-css", description: "quarto-resource-document-options-document-css" }, css: { $ref: "quarto-resource-document-options-css", description: "quarto-resource-document-options-css" }, "section-divs": { $ref: "quarto-resource-document-options-section-divs", description: "quarto-resource-document-options-section-divs" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, "email-obfuscation": { $ref: "quarto-resource-document-options-email-obfuscation", description: "quarto-resource-document-options-email-obfuscation" }, "html-q-tags": { $ref: "quarto-resource-document-options-html-q-tags", description: "quarto-resource-document-options-html-q-tags" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, "citations-hover": { $ref: "quarto-resource-document-references-citations-hover", description: "quarto-resource-document-references-citations-hover" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, "self-contained": { $ref: "quarto-resource-document-render-self-contained", description: "quarto-resource-document-render-self-contained" }, "self-contained-math": { $ref: "quarto-resource-document-render-self-contained-math", description: "quarto-resource-document-render-self-contained-math" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, logo: { $ref: "quarto-resource-document-reveal-content-logo", description: "quarto-resource-document-reveal-content-logo" }, footer: { $ref: "quarto-resource-document-reveal-content-footer", description: "quarto-resource-document-reveal-content-footer" }, scrollable: { $ref: "quarto-resource-document-reveal-content-scrollable", description: "quarto-resource-document-reveal-content-scrollable" }, smaller: { $ref: "quarto-resource-document-reveal-content-smaller", description: "quarto-resource-document-reveal-content-smaller" }, embedded: { $ref: "quarto-resource-document-reveal-hidden-embedded", description: "quarto-resource-document-reveal-hidden-embedded" }, display: { $ref: "quarto-resource-document-reveal-hidden-display", description: "quarto-resource-document-reveal-hidden-display" }, "auto-stretch": { $ref: "quarto-resource-document-reveal-layout-auto-stretch", description: "quarto-resource-document-reveal-layout-auto-stretch" }, width: { $ref: "quarto-resource-document-reveal-layout-width", description: "quarto-resource-document-reveal-layout-width" }, height: { $ref: "quarto-resource-document-reveal-layout-height", description: "quarto-resource-document-reveal-layout-height" }, margin: { $ref: "quarto-resource-document-reveal-layout-margin", description: "quarto-resource-document-reveal-layout-margin" }, "min-scale": { $ref: "quarto-resource-document-reveal-layout-min-scale", description: "quarto-resource-document-reveal-layout-min-scale" }, "max-scale": { $ref: "quarto-resource-document-reveal-layout-max-scale", description: "quarto-resource-document-reveal-layout-max-scale" }, center: { $ref: "quarto-resource-document-reveal-layout-center", description: "quarto-resource-document-reveal-layout-center" }, "disable-layout": { $ref: "quarto-resource-document-reveal-layout-disable-layout", description: "quarto-resource-document-reveal-layout-disable-layout" }, "preview-links": { $ref: "quarto-resource-document-reveal-media-preview-links", description: "quarto-resource-document-reveal-media-preview-links" }, "auto-play-media": { $ref: "quarto-resource-document-reveal-media-auto-play-media", description: "quarto-resource-document-reveal-media-auto-play-media" }, "preload-iframes": { $ref: "quarto-resource-document-reveal-media-preload-iframes", description: "quarto-resource-document-reveal-media-preload-iframes" }, "view-distance": { $ref: "quarto-resource-document-reveal-media-view-distance", description: "quarto-resource-document-reveal-media-view-distance" }, "mobile-view-distance": { $ref: "quarto-resource-document-reveal-media-mobile-view-distance", description: "quarto-resource-document-reveal-media-mobile-view-distance" }, "parallax-background-image": { $ref: "quarto-resource-document-reveal-media-parallax-background-image", description: "quarto-resource-document-reveal-media-parallax-background-image" }, "parallax-background-size": { $ref: "quarto-resource-document-reveal-media-parallax-background-size", description: "quarto-resource-document-reveal-media-parallax-background-size" }, "parallax-background-horizontal": { $ref: "quarto-resource-document-reveal-media-parallax-background-horizontal", description: "quarto-resource-document-reveal-media-parallax-background-horizontal" }, "parallax-background-vertical": { $ref: "quarto-resource-document-reveal-media-parallax-background-vertical", description: "quarto-resource-document-reveal-media-parallax-background-vertical" }, progress: { $ref: "quarto-resource-document-reveal-navigation-progress", description: "quarto-resource-document-reveal-navigation-progress" }, history: { $ref: "quarto-resource-document-reveal-navigation-history", description: "quarto-resource-document-reveal-navigation-history" }, "navigation-mode": { $ref: "quarto-resource-document-reveal-navigation-navigation-mode", description: "quarto-resource-document-reveal-navigation-navigation-mode" }, touch: { $ref: "quarto-resource-document-reveal-navigation-touch", description: "quarto-resource-document-reveal-navigation-touch" }, keyboard: { $ref: "quarto-resource-document-reveal-navigation-keyboard", description: "quarto-resource-document-reveal-navigation-keyboard" }, "mouse-wheel": { $ref: "quarto-resource-document-reveal-navigation-mouse-wheel", description: "quarto-resource-document-reveal-navigation-mouse-wheel" }, "hide-inactive-cursor": { $ref: "quarto-resource-document-reveal-navigation-hide-inactive-cursor", description: "quarto-resource-document-reveal-navigation-hide-inactive-cursor" }, "hide-cursor-time": { $ref: "quarto-resource-document-reveal-navigation-hide-cursor-time", description: "quarto-resource-document-reveal-navigation-hide-cursor-time" }, loop: { $ref: "quarto-resource-document-reveal-navigation-loop", description: "quarto-resource-document-reveal-navigation-loop" }, shuffle: { $ref: "quarto-resource-document-reveal-navigation-shuffle", description: "quarto-resource-document-reveal-navigation-shuffle" }, controls: { $ref: "quarto-resource-document-reveal-navigation-controls", description: "quarto-resource-document-reveal-navigation-controls" }, "controls-layout": { $ref: "quarto-resource-document-reveal-navigation-controls-layout", description: "quarto-resource-document-reveal-navigation-controls-layout" }, "controls-tutorial": { $ref: "quarto-resource-document-reveal-navigation-controls-tutorial", description: "quarto-resource-document-reveal-navigation-controls-tutorial" }, "controls-back-arrows": { $ref: "quarto-resource-document-reveal-navigation-controls-back-arrows", description: "quarto-resource-document-reveal-navigation-controls-back-arrows" }, "auto-slide": { $ref: "quarto-resource-document-reveal-navigation-auto-slide", description: "quarto-resource-document-reveal-navigation-auto-slide" }, "auto-slide-stoppable": { $ref: "quarto-resource-document-reveal-navigation-auto-slide-stoppable", description: "quarto-resource-document-reveal-navigation-auto-slide-stoppable" }, "auto-slide-method": { $ref: "quarto-resource-document-reveal-navigation-auto-slide-method", description: "quarto-resource-document-reveal-navigation-auto-slide-method" }, "default-timing": { $ref: "quarto-resource-document-reveal-navigation-default-timing", description: "quarto-resource-document-reveal-navigation-default-timing" }, pause: { $ref: "quarto-resource-document-reveal-navigation-pause", description: "quarto-resource-document-reveal-navigation-pause" }, help: { $ref: "quarto-resource-document-reveal-navigation-help", description: "quarto-resource-document-reveal-navigation-help" }, hash: { $ref: "quarto-resource-document-reveal-navigation-hash", description: "quarto-resource-document-reveal-navigation-hash" }, "hash-type": { $ref: "quarto-resource-document-reveal-navigation-hash-type", description: "quarto-resource-document-reveal-navigation-hash-type" }, "hash-one-based-index": { $ref: "quarto-resource-document-reveal-navigation-hash-one-based-index", description: "quarto-resource-document-reveal-navigation-hash-one-based-index" }, "respond-to-hash-changes": { $ref: "quarto-resource-document-reveal-navigation-respond-to-hash-changes", description: "quarto-resource-document-reveal-navigation-respond-to-hash-changes" }, "fragment-in-url": { $ref: "quarto-resource-document-reveal-navigation-fragment-in-url", description: "quarto-resource-document-reveal-navigation-fragment-in-url" }, "slide-tone": { $ref: "quarto-resource-document-reveal-navigation-slide-tone", description: "quarto-resource-document-reveal-navigation-slide-tone" }, "pdf-separate-fragments": { $ref: "quarto-resource-document-reveal-print-pdf-separate-fragments", description: "quarto-resource-document-reveal-print-pdf-separate-fragments" }, "pdf-page-height-offset": { $ref: "quarto-resource-document-reveal-print-pdf-page-height-offset", description: "quarto-resource-document-reveal-print-pdf-page-height-offset" }, overview: { $ref: "quarto-resource-document-reveal-tools-overview", description: "quarto-resource-document-reveal-tools-overview" }, menu: { $ref: "quarto-resource-document-reveal-tools-menu", description: "quarto-resource-document-reveal-tools-menu" }, chalkboard: { $ref: "quarto-resource-document-reveal-tools-chalkboard", description: "quarto-resource-document-reveal-tools-chalkboard" }, multiplex: { $ref: "quarto-resource-document-reveal-tools-multiplex", description: "quarto-resource-document-reveal-tools-multiplex" }, transition: { $ref: "quarto-resource-document-reveal-transitions-transition", description: "quarto-resource-document-reveal-transitions-transition" }, "transition-speed": { $ref: "quarto-resource-document-reveal-transitions-transition-speed", description: "quarto-resource-document-reveal-transitions-transition-speed" }, "background-transition": { $ref: "quarto-resource-document-reveal-transitions-background-transition", description: "quarto-resource-document-reveal-transitions-background-transition" }, fragments: { $ref: "quarto-resource-document-reveal-transitions-fragments", description: "quarto-resource-document-reveal-transitions-fragments" }, "auto-animate": { $ref: "quarto-resource-document-reveal-transitions-auto-animate", description: "quarto-resource-document-reveal-transitions-auto-animate" }, "auto-animate-easing": { $ref: "quarto-resource-document-reveal-transitions-auto-animate-easing", description: "quarto-resource-document-reveal-transitions-auto-animate-easing" }, "auto-animate-duration": { $ref: "quarto-resource-document-reveal-transitions-auto-animate-duration", description: "quarto-resource-document-reveal-transitions-auto-animate-duration" }, "auto-animate-unmatched": { $ref: "quarto-resource-document-reveal-transitions-auto-animate-unmatched", description: "quarto-resource-document-reveal-transitions-auto-animate-unmatched" }, "auto-animate-styles": { $ref: "quarto-resource-document-reveal-transitions-auto-animate-styles", description: "quarto-resource-document-reveal-transitions-auto-animate-styles" }, incremental: { $ref: "quarto-resource-document-slides-incremental", description: "quarto-resource-document-slides-incremental" }, "slide-level": { $ref: "quarto-resource-document-slides-slide-level", description: "quarto-resource-document-slides-slide-level" }, "slide-number": { $ref: "quarto-resource-document-slides-slide-number", description: "quarto-resource-document-slides-slide-number" }, "show-slide-number": { $ref: "quarto-resource-document-slides-show-slide-number", description: "quarto-resource-document-slides-show-slide-number" }, "title-slide-attributes": { $ref: "quarto-resource-document-slides-title-slide-attributes", description: "quarto-resource-document-slides-title-slide-attributes" }, "center-title-slide": { $ref: "quarto-resource-document-slides-center-title-slide", description: "quarto-resource-document-slides-center-title-slide" }, "show-notes": { $ref: "quarto-resource-document-slides-show-notes", description: "quarto-resource-document-slides-show-notes" }, rtl: { $ref: "quarto-resource-document-slides-rtl", description: "quarto-resource-document-slides-rtl" }, "strip-comments": { $ref: "quarto-resource-document-text-strip-comments", description: "quarto-resource-document-text-strip-comments" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" }, "toc-title": { $ref: "quarto-resource-document-toc-toc-title", description: "quarto-resource-document-toc-toc-title" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|reveal_js_url|revealJsUrl|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|auto_stretch|autoStretch|min_scale|minScale|max_scale|maxScale|disable_layout|disableLayout|preview_links|previewLinks|auto_play_media|autoPlayMedia|preload_iframes|preloadIframes|view_distance|viewDistance|mobile_view_distance|mobileViewDistance|parallax_background_image|parallaxBackgroundImage|parallax_background_size|parallaxBackgroundSize|parallax_background_horizontal|parallaxBackgroundHorizontal|parallax_background_vertical|parallaxBackgroundVertical|navigation_mode|navigationMode|mouse_wheel|mouseWheel|hide_inactive_cursor|hideInactiveCursor|hide_cursor_time|hideCursorTime|controls_layout|controlsLayout|controls_tutorial|controlsTutorial|controls_back_arrows|controlsBackArrows|auto_slide|autoSlide|auto_slide_stoppable|autoSlideStoppable|auto_slide_method|autoSlideMethod|default_timing|defaultTiming|hash_type|hashType|hash_one_based_index|hashOneBasedIndex|respond_to_hash_changes|respondToHashChanges|fragment_in_url|fragmentInUrl|slide_tone|slideTone|pdf_separate_fragments|pdfSeparateFragments|pdf_page_height_offset|pdfPageHeightOffset|transition_speed|transitionSpeed|background_transition|backgroundTransition|auto_animate|autoAnimate|auto_animate_easing|autoAnimateEasing|auto_animate_duration|autoAnimateDuration|auto_animate_unmatched|autoAnimateUnmatched|auto_animate_styles|autoAnimateStyles|slide_level|slideLevel|slide_number|slideNumber|show_slide_number|showSlideNumber|title_slide_attributes|titleSlideAttributes|center_title_slide|centerTitleSlide|show_notes|showNotes|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^rst(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^rtf(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(fig_align|figAlign|tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^s5(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-fold": { $ref: "quarto-resource-cell-codeoutput-code-fold", description: "quarto-resource-cell-codeoutput-code-fold" }, "code-summary": { $ref: "quarto-resource-cell-codeoutput-code-summary", description: "quarto-resource-cell-codeoutput-code-summary" }, "code-overflow": { $ref: "quarto-resource-cell-codeoutput-code-overflow", description: "quarto-resource-cell-codeoutput-code-overflow" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "cap-location": { $ref: "quarto-resource-cell-pagelayout-cap-location", description: "quarto-resource-cell-pagelayout-cap-location" }, "fig-cap-location": { $ref: "quarto-resource-cell-pagelayout-fig-cap-location", description: "quarto-resource-cell-pagelayout-fig-cap-location" }, "tbl-cap-location": { $ref: "quarto-resource-cell-pagelayout-tbl-cap-location", description: "quarto-resource-cell-pagelayout-tbl-cap-location" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, institute: { $ref: "quarto-resource-document-attributes-institute", description: "quarto-resource-document-attributes-institute" }, "code-copy": { $ref: "quarto-resource-document-code-code-copy", description: "quarto-resource-document-code-code-copy" }, "code-link": { $ref: "quarto-resource-document-code-code-link", description: "quarto-resource-document-code-code-link" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, fontcolor: { $ref: "quarto-resource-document-colors-fontcolor", description: "quarto-resource-document-colors-fontcolor" }, linkcolor: { $ref: "quarto-resource-document-colors-linkcolor", description: "quarto-resource-document-colors-linkcolor" }, monobackgroundcolor: { $ref: "quarto-resource-document-colors-monobackgroundcolor", description: "quarto-resource-document-colors-monobackgroundcolor" }, backgroundcolor: { $ref: "quarto-resource-document-colors-backgroundcolor", description: "quarto-resource-document-colors-backgroundcolor" }, comments: { $ref: "quarto-resource-document-comments-comments", description: "quarto-resource-document-comments-comments" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "fig-responsive": { $ref: "quarto-resource-document-figures-fig-responsive", description: "quarto-resource-document-figures-fig-responsive" }, mainfont: { $ref: "quarto-resource-document-fonts-mainfont", description: "quarto-resource-document-fonts-mainfont" }, monofont: { $ref: "quarto-resource-document-fonts-monofont", description: "quarto-resource-document-fonts-monofont" }, fontsize: { $ref: "quarto-resource-document-fonts-fontsize", description: "quarto-resource-document-fonts-fontsize" }, linestretch: { $ref: "quarto-resource-document-fonts-linestretch", description: "quarto-resource-document-fonts-linestretch" }, "footnotes-hover": { $ref: "quarto-resource-document-footnotes-footnotes-hover", description: "quarto-resource-document-footnotes-footnotes-hover" }, "reference-location": { $ref: "quarto-resource-document-footnotes-reference-location", description: "quarto-resource-document-footnotes-reference-location" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, resources: { $ref: "quarto-resource-document-includes-resources", description: "quarto-resource-document-includes-resources" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, classoption: { $ref: "quarto-resource-document-layout-classoption", description: "quarto-resource-document-layout-classoption" }, "max-width": { $ref: "quarto-resource-document-layout-max-width", description: "quarto-resource-document-layout-max-width" }, "margin-left": { $ref: "quarto-resource-document-layout-margin-left", description: "quarto-resource-document-layout-margin-left" }, "margin-right": { $ref: "quarto-resource-document-layout-margin-right", description: "quarto-resource-document-layout-margin-right" }, "margin-top": { $ref: "quarto-resource-document-layout-margin-top", description: "quarto-resource-document-layout-margin-top" }, "margin-bottom": { $ref: "quarto-resource-document-layout-margin-bottom", description: "quarto-resource-document-layout-margin-bottom" }, "s5-url": { $ref: "quarto-resource-document-library-s5-url", description: "quarto-resource-document-library-s5-url" }, keywords: { $ref: "quarto-resource-document-metadata-keywords", description: "quarto-resource-document-metadata-keywords" }, pagetitle: { $ref: "quarto-resource-document-metadata-pagetitle", description: "quarto-resource-document-metadata-pagetitle" }, "title-prefix": { $ref: "quarto-resource-document-metadata-title-prefix", description: "quarto-resource-document-metadata-title-prefix" }, "description-meta": { $ref: "quarto-resource-document-metadata-description-meta", description: "quarto-resource-document-metadata-description-meta" }, "author-meta": { $ref: "quarto-resource-document-metadata-author-meta", description: "quarto-resource-document-metadata-author-meta" }, "date-meta": { $ref: "quarto-resource-document-metadata-date-meta", description: "quarto-resource-document-metadata-date-meta" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-depth": { $ref: "quarto-resource-document-numbering-number-depth", description: "quarto-resource-document-numbering-number-depth" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "document-css": { $ref: "quarto-resource-document-options-document-css", description: "quarto-resource-document-options-document-css" }, css: { $ref: "quarto-resource-document-options-css", description: "quarto-resource-document-options-css" }, "section-divs": { $ref: "quarto-resource-document-options-section-divs", description: "quarto-resource-document-options-section-divs" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, "email-obfuscation": { $ref: "quarto-resource-document-options-email-obfuscation", description: "quarto-resource-document-options-email-obfuscation" }, "html-q-tags": { $ref: "quarto-resource-document-options-html-q-tags", description: "quarto-resource-document-options-html-q-tags" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, "citations-hover": { $ref: "quarto-resource-document-references-citations-hover", description: "quarto-resource-document-references-citations-hover" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, "self-contained": { $ref: "quarto-resource-document-render-self-contained", description: "quarto-resource-document-render-self-contained" }, "self-contained-math": { $ref: "quarto-resource-document-render-self-contained-math", description: "quarto-resource-document-render-self-contained-math" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, incremental: { $ref: "quarto-resource-document-slides-incremental", description: "quarto-resource-document-slides-incremental" }, "slide-level": { $ref: "quarto-resource-document-slides-slide-level", description: "quarto-resource-document-slides-slide-level" }, "strip-comments": { $ref: "quarto-resource-document-text-strip-comments", description: "quarto-resource-document-text-strip-comments" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|s5_url|s5Url|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^slideous(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-fold": { $ref: "quarto-resource-cell-codeoutput-code-fold", description: "quarto-resource-cell-codeoutput-code-fold" }, "code-summary": { $ref: "quarto-resource-cell-codeoutput-code-summary", description: "quarto-resource-cell-codeoutput-code-summary" }, "code-overflow": { $ref: "quarto-resource-cell-codeoutput-code-overflow", description: "quarto-resource-cell-codeoutput-code-overflow" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "cap-location": { $ref: "quarto-resource-cell-pagelayout-cap-location", description: "quarto-resource-cell-pagelayout-cap-location" }, "fig-cap-location": { $ref: "quarto-resource-cell-pagelayout-fig-cap-location", description: "quarto-resource-cell-pagelayout-fig-cap-location" }, "tbl-cap-location": { $ref: "quarto-resource-cell-pagelayout-tbl-cap-location", description: "quarto-resource-cell-pagelayout-tbl-cap-location" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, institute: { $ref: "quarto-resource-document-attributes-institute", description: "quarto-resource-document-attributes-institute" }, "code-copy": { $ref: "quarto-resource-document-code-code-copy", description: "quarto-resource-document-code-code-copy" }, "code-link": { $ref: "quarto-resource-document-code-code-link", description: "quarto-resource-document-code-code-link" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, fontcolor: { $ref: "quarto-resource-document-colors-fontcolor", description: "quarto-resource-document-colors-fontcolor" }, linkcolor: { $ref: "quarto-resource-document-colors-linkcolor", description: "quarto-resource-document-colors-linkcolor" }, monobackgroundcolor: { $ref: "quarto-resource-document-colors-monobackgroundcolor", description: "quarto-resource-document-colors-monobackgroundcolor" }, backgroundcolor: { $ref: "quarto-resource-document-colors-backgroundcolor", description: "quarto-resource-document-colors-backgroundcolor" }, comments: { $ref: "quarto-resource-document-comments-comments", description: "quarto-resource-document-comments-comments" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "fig-responsive": { $ref: "quarto-resource-document-figures-fig-responsive", description: "quarto-resource-document-figures-fig-responsive" }, mainfont: { $ref: "quarto-resource-document-fonts-mainfont", description: "quarto-resource-document-fonts-mainfont" }, monofont: { $ref: "quarto-resource-document-fonts-monofont", description: "quarto-resource-document-fonts-monofont" }, fontsize: { $ref: "quarto-resource-document-fonts-fontsize", description: "quarto-resource-document-fonts-fontsize" }, linestretch: { $ref: "quarto-resource-document-fonts-linestretch", description: "quarto-resource-document-fonts-linestretch" }, "footnotes-hover": { $ref: "quarto-resource-document-footnotes-footnotes-hover", description: "quarto-resource-document-footnotes-footnotes-hover" }, "reference-location": { $ref: "quarto-resource-document-footnotes-reference-location", description: "quarto-resource-document-footnotes-reference-location" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, resources: { $ref: "quarto-resource-document-includes-resources", description: "quarto-resource-document-includes-resources" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, classoption: { $ref: "quarto-resource-document-layout-classoption", description: "quarto-resource-document-layout-classoption" }, "max-width": { $ref: "quarto-resource-document-layout-max-width", description: "quarto-resource-document-layout-max-width" }, "margin-left": { $ref: "quarto-resource-document-layout-margin-left", description: "quarto-resource-document-layout-margin-left" }, "margin-right": { $ref: "quarto-resource-document-layout-margin-right", description: "quarto-resource-document-layout-margin-right" }, "margin-top": { $ref: "quarto-resource-document-layout-margin-top", description: "quarto-resource-document-layout-margin-top" }, "margin-bottom": { $ref: "quarto-resource-document-layout-margin-bottom", description: "quarto-resource-document-layout-margin-bottom" }, "slideous-url": { $ref: "quarto-resource-document-library-slideous-url", description: "quarto-resource-document-library-slideous-url" }, keywords: { $ref: "quarto-resource-document-metadata-keywords", description: "quarto-resource-document-metadata-keywords" }, pagetitle: { $ref: "quarto-resource-document-metadata-pagetitle", description: "quarto-resource-document-metadata-pagetitle" }, "title-prefix": { $ref: "quarto-resource-document-metadata-title-prefix", description: "quarto-resource-document-metadata-title-prefix" }, "description-meta": { $ref: "quarto-resource-document-metadata-description-meta", description: "quarto-resource-document-metadata-description-meta" }, "author-meta": { $ref: "quarto-resource-document-metadata-author-meta", description: "quarto-resource-document-metadata-author-meta" }, "date-meta": { $ref: "quarto-resource-document-metadata-date-meta", description: "quarto-resource-document-metadata-date-meta" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-depth": { $ref: "quarto-resource-document-numbering-number-depth", description: "quarto-resource-document-numbering-number-depth" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "document-css": { $ref: "quarto-resource-document-options-document-css", description: "quarto-resource-document-options-document-css" }, css: { $ref: "quarto-resource-document-options-css", description: "quarto-resource-document-options-css" }, "section-divs": { $ref: "quarto-resource-document-options-section-divs", description: "quarto-resource-document-options-section-divs" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, "email-obfuscation": { $ref: "quarto-resource-document-options-email-obfuscation", description: "quarto-resource-document-options-email-obfuscation" }, "html-q-tags": { $ref: "quarto-resource-document-options-html-q-tags", description: "quarto-resource-document-options-html-q-tags" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, "citations-hover": { $ref: "quarto-resource-document-references-citations-hover", description: "quarto-resource-document-references-citations-hover" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, "self-contained": { $ref: "quarto-resource-document-render-self-contained", description: "quarto-resource-document-render-self-contained" }, "self-contained-math": { $ref: "quarto-resource-document-render-self-contained-math", description: "quarto-resource-document-render-self-contained-math" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, incremental: { $ref: "quarto-resource-document-slides-incremental", description: "quarto-resource-document-slides-incremental" }, "slide-level": { $ref: "quarto-resource-document-slides-slide-level", description: "quarto-resource-document-slides-slide-level" }, "strip-comments": { $ref: "quarto-resource-document-text-strip-comments", description: "quarto-resource-document-text-strip-comments" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|slideous_url|slideousUrl|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^slidy(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-fold": { $ref: "quarto-resource-cell-codeoutput-code-fold", description: "quarto-resource-cell-codeoutput-code-fold" }, "code-summary": { $ref: "quarto-resource-cell-codeoutput-code-summary", description: "quarto-resource-cell-codeoutput-code-summary" }, "code-overflow": { $ref: "quarto-resource-cell-codeoutput-code-overflow", description: "quarto-resource-cell-codeoutput-code-overflow" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "cap-location": { $ref: "quarto-resource-cell-pagelayout-cap-location", description: "quarto-resource-cell-pagelayout-cap-location" }, "fig-cap-location": { $ref: "quarto-resource-cell-pagelayout-fig-cap-location", description: "quarto-resource-cell-pagelayout-fig-cap-location" }, "tbl-cap-location": { $ref: "quarto-resource-cell-pagelayout-tbl-cap-location", description: "quarto-resource-cell-pagelayout-tbl-cap-location" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, institute: { $ref: "quarto-resource-document-attributes-institute", description: "quarto-resource-document-attributes-institute" }, "code-copy": { $ref: "quarto-resource-document-code-code-copy", description: "quarto-resource-document-code-code-copy" }, "code-link": { $ref: "quarto-resource-document-code-code-link", description: "quarto-resource-document-code-code-link" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, fontcolor: { $ref: "quarto-resource-document-colors-fontcolor", description: "quarto-resource-document-colors-fontcolor" }, linkcolor: { $ref: "quarto-resource-document-colors-linkcolor", description: "quarto-resource-document-colors-linkcolor" }, monobackgroundcolor: { $ref: "quarto-resource-document-colors-monobackgroundcolor", description: "quarto-resource-document-colors-monobackgroundcolor" }, backgroundcolor: { $ref: "quarto-resource-document-colors-backgroundcolor", description: "quarto-resource-document-colors-backgroundcolor" }, comments: { $ref: "quarto-resource-document-comments-comments", description: "quarto-resource-document-comments-comments" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "fig-responsive": { $ref: "quarto-resource-document-figures-fig-responsive", description: "quarto-resource-document-figures-fig-responsive" }, mainfont: { $ref: "quarto-resource-document-fonts-mainfont", description: "quarto-resource-document-fonts-mainfont" }, monofont: { $ref: "quarto-resource-document-fonts-monofont", description: "quarto-resource-document-fonts-monofont" }, fontsize: { $ref: "quarto-resource-document-fonts-fontsize", description: "quarto-resource-document-fonts-fontsize" }, linestretch: { $ref: "quarto-resource-document-fonts-linestretch", description: "quarto-resource-document-fonts-linestretch" }, "footnotes-hover": { $ref: "quarto-resource-document-footnotes-footnotes-hover", description: "quarto-resource-document-footnotes-footnotes-hover" }, "reference-location": { $ref: "quarto-resource-document-footnotes-reference-location", description: "quarto-resource-document-footnotes-reference-location" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, resources: { $ref: "quarto-resource-document-includes-resources", description: "quarto-resource-document-includes-resources" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, classoption: { $ref: "quarto-resource-document-layout-classoption", description: "quarto-resource-document-layout-classoption" }, "max-width": { $ref: "quarto-resource-document-layout-max-width", description: "quarto-resource-document-layout-max-width" }, "margin-left": { $ref: "quarto-resource-document-layout-margin-left", description: "quarto-resource-document-layout-margin-left" }, "margin-right": { $ref: "quarto-resource-document-layout-margin-right", description: "quarto-resource-document-layout-margin-right" }, "margin-top": { $ref: "quarto-resource-document-layout-margin-top", description: "quarto-resource-document-layout-margin-top" }, "margin-bottom": { $ref: "quarto-resource-document-layout-margin-bottom", description: "quarto-resource-document-layout-margin-bottom" }, "slidy-url": { $ref: "quarto-resource-document-library-slidy-url", description: "quarto-resource-document-library-slidy-url" }, keywords: { $ref: "quarto-resource-document-metadata-keywords", description: "quarto-resource-document-metadata-keywords" }, pagetitle: { $ref: "quarto-resource-document-metadata-pagetitle", description: "quarto-resource-document-metadata-pagetitle" }, "title-prefix": { $ref: "quarto-resource-document-metadata-title-prefix", description: "quarto-resource-document-metadata-title-prefix" }, "description-meta": { $ref: "quarto-resource-document-metadata-description-meta", description: "quarto-resource-document-metadata-description-meta" }, "author-meta": { $ref: "quarto-resource-document-metadata-author-meta", description: "quarto-resource-document-metadata-author-meta" }, "date-meta": { $ref: "quarto-resource-document-metadata-date-meta", description: "quarto-resource-document-metadata-date-meta" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-depth": { $ref: "quarto-resource-document-numbering-number-depth", description: "quarto-resource-document-numbering-number-depth" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "document-css": { $ref: "quarto-resource-document-options-document-css", description: "quarto-resource-document-options-document-css" }, css: { $ref: "quarto-resource-document-options-css", description: "quarto-resource-document-options-css" }, "section-divs": { $ref: "quarto-resource-document-options-section-divs", description: "quarto-resource-document-options-section-divs" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, "email-obfuscation": { $ref: "quarto-resource-document-options-email-obfuscation", description: "quarto-resource-document-options-email-obfuscation" }, "html-q-tags": { $ref: "quarto-resource-document-options-html-q-tags", description: "quarto-resource-document-options-html-q-tags" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, "citations-hover": { $ref: "quarto-resource-document-references-citations-hover", description: "quarto-resource-document-references-citations-hover" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, "self-contained": { $ref: "quarto-resource-document-render-self-contained", description: "quarto-resource-document-render-self-contained" }, "self-contained-math": { $ref: "quarto-resource-document-render-self-contained-math", description: "quarto-resource-document-render-self-contained-math" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, incremental: { $ref: "quarto-resource-document-slides-incremental", description: "quarto-resource-document-slides-incremental" }, "slide-level": { $ref: "quarto-resource-document-slides-slide-level", description: "quarto-resource-document-slides-slide-level" }, "strip-comments": { $ref: "quarto-resource-document-text-strip-comments", description: "quarto-resource-document-text-strip-comments" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|slidy_url|slidyUrl|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^tei(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, "top-level-division": { $ref: "quarto-resource-document-numbering-top-level-division", description: "quarto-resource-document-numbering-top-level-division" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^texinfo(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^textile(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, "strip-comments": { $ref: "quarto-resource-document-text-strip-comments", description: "quarto-resource-document-text-strip-comments" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^xwiki(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" }, "^zimwiki(\\+.+)?$": { oneOf: [{ type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))" } }, { enum: ["default"], description: "be 'default'" }], description: "be exactly one of: an object, 'default'" } }, additionalProperties: false, propertyNames: { type: "string", pattern: "^(?!(commonmark-x|commonmarkX|jats-archiving|jatsArchiving|jats-articleauthoring|jatsArticleauthoring|jats-publishing|jatsPublishing|markdown-github|markdownGithub|markdown-mmd|markdownMmd|markdown-phpextra|markdownPhpextra|markdown-strict|markdownStrict))" } }], description: "be all of: an object" }], description: "be exactly one of: the name of a pandoc-supported output format, 'hugo', all of: an object", $id: "front-matter-format" }, dv = new RegExp("^asciidoc(\\+.+)?$", "u"), fv = new RegExp("^asciidoctor(\\+.+)?$", "u"), mv = new RegExp("^beamer(\\+.+)?$", "u"), gv = new RegExp("^biblatex(\\+.+)?$", "u"), hv = new RegExp("^bibtex(\\+.+)?$", "u"), yv = new RegExp("^commonmark(\\+.+)?$", "u"), bv = new RegExp("^commonmark_x(\\+.+)?$", "u"), _v = new RegExp("^context(\\+.+)?$", "u"), vv = new RegExp("^csljson(\\+.+)?$", "u"), qv = new RegExp("^docbook(\\+.+)?$", "u"), Ov = new RegExp("^docbook4(\\+.+)?$", "u"), Pv = new RegExp("^docbook5(\\+.+)?$", "u"), xv = new RegExp("^docx(\\+.+)?$", "u"), kv = new RegExp("^dokuwiki(\\+.+)?$", "u"), Dv = new RegExp("^dzslides(\\+.+)?$", "u"), wv = new RegExp("^epub(\\+.+)?$", "u"), $v = new RegExp("^epub2(\\+.+)?$", "u"), Sv = new RegExp("^epub3(\\+.+)?$", "u"), Fv = new RegExp("^fb2(\\+.+)?$", "u"), Av = new RegExp("^gfm(\\+.+)?$", "u"), Bv = new RegExp("^haddock(\\+.+)?$", "u"), jv = new RegExp("^html(\\+.+)?$", "u"), Ev = new RegExp("^html4(\\+.+)?$", "u"), Nv = new RegExp("^html5(\\+.+)?$", "u"), Cv = new RegExp("^icml(\\+.+)?$", "u"), Iv = new RegExp("^ipynb(\\+.+)?$", "u"), Hv = new RegExp("^jats(\\+.+)?$", "u"), Lv = new RegExp("^jats_archiving(\\+.+)?$", "u"), Mv = new RegExp("^jats_articleauthoring(\\+.+)?$", "u"), Tv = new RegExp("^jats_publishing(\\+.+)?$", "u"), zv = new RegExp("^jira(\\+.+)?$", "u"), Wv = new RegExp("^json(\\+.+)?$", "u"), Rv = new RegExp("^latex(\\+.+)?$", "u"), Vv = new RegExp("^man(\\+.+)?$", "u"), Uv = new RegExp("^markdown(\\+.+)?$", "u"), Kv = new RegExp("^markdown_github(\\+.+)?$", "u"), Jv = new RegExp("^markdown_mmd(\\+.+)?$", "u"), Qv = new RegExp("^markdown_phpextra(\\+.+)?$", "u"), Gv = new RegExp("^markdown_strict(\\+.+)?$", "u"), Yv = new RegExp("^markua(\\+.+)?$", "u"), Xv = new RegExp("^mediawiki(\\+.+)?$", "u"), Zv = new RegExp("^ms(\\+.+)?$", "u"), e1 = new RegExp("^muse(\\+.+)?$", "u"), r1 = new RegExp("^native(\\+.+)?$", "u"), t1 = new RegExp("^odt(\\+.+)?$", "u"), a1 = new RegExp("^opendocument(\\+.+)?$", "u"), n1 = new RegExp("^opml(\\+.+)?$", "u"), o1 = new RegExp("^org(\\+.+)?$", "u"), s1 = new RegExp("^pdf(\\+.+)?$", "u"), i1 = new RegExp("^plain(\\+.+)?$", "u"), l1 = new RegExp("^pptx(\\+.+)?$", "u"), c1 = new RegExp("^revealjs(\\+.+)?$", "u"), p1 = new RegExp("^rst(\\+.+)?$", "u"), u1 = new RegExp("^rtf(\\+.+)?$", "u"), d1 = new RegExp("^s5(\\+.+)?$", "u"), f1 = new RegExp("^slideous(\\+.+)?$", "u"), m1 = new RegExp("^slidy(\\+.+)?$", "u"), g1 = new RegExp("^tei(\\+.+)?$", "u"), h1 = new RegExp("^texinfo(\\+.+)?$", "u"), y1 = new RegExp("^textile(\\+.+)?$", "u"), b1 = new RegExp("^xwiki(\\+.+)?$", "u"), _1 = new RegExp("^zimwiki(\\+.+)?$", "u"), N0 = new RegExp("^hugo(\\+.+)?$", "u"), C0 = new RegExp("^(?!(commonmark-x|commonmarkX|jats-archiving|jatsArchiving|jats-articleauthoring|jatsArticleauthoring|jats-publishing|jatsPublishing|markdown-github|markdownGithub|markdown-mmd|markdownMmd|markdown-phpextra|markdownPhpextra|markdown-strict|markdownStrict))", "u"), Xn = new RegExp("^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", "u"), I0 = new RegExp("^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|links_as_notes|linksAsNotes|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|latex_auto_mk|latexAutoMk|latex_auto_install|latexAutoInstall|latex_min_runs|latexMinRuns|latex_max_runs|latexMaxRuns|latex_clean|latexClean|latex_makeindex|latexMakeindex|latex_makeindex_opts|latexMakeindexOpts|latex_tlmgr_opts|latexTlmgrOpts|latex_output_dir|latexOutputDir|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|title_meta|titleMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|section_titles|sectionTitles|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|keep_tex|keepTex|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", "u"), v1 = new RegExp("^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", "u"), H0 = new RegExp("^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", "u"), q1 = new RegExp("^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension))", "u"), L0 = new RegExp("^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|track_changes|trackChanges|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|page_width|pageWidth|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|reference_doc|referenceDoc|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", "u"), M0 = new RegExp("^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", "u"), O1 = new RegExp("^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|html_q_tags|htmlQTags|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", "u"), T0 = new RegExp("^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", "u"), z0 = new RegExp("^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", "u"), P1 = new RegExp("^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_layout|pageLayout|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_location|citationLocation|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))", "u"), W0 = new RegExp("^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|markdown_headings|markdownHeadings|ipynb_output|ipynbOutput|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", "u"), Ym = new RegExp("^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))", "u"), R0 = new RegExp("^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|links_as_notes|linksAsNotes|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|title_meta|titleMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", "u"), V0 = new RegExp("^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))", "u"), U0 = new RegExp("^(?!(code_line_numbers|codeLineNumbers|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", "u"), K0 = new RegExp("^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", "u"), J0 = new RegExp("^(?!(fig_align|figAlign|tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_width|pageWidth|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|reference_doc|referenceDoc|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", "u"), Q0 = new RegExp("^(?!(fig_align|figAlign|tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_width|pageWidth|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", "u"), G0 = new RegExp("^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|links_as_notes|linksAsNotes|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|latex_auto_mk|latexAutoMk|latex_auto_install|latexAutoInstall|latex_min_runs|latexMinRuns|latex_max_runs|latexMaxRuns|latex_clean|latexClean|latex_makeindex|latexMakeindex|latex_makeindex_opts|latexMakeindexOpts|latex_tlmgr_opts|latexTlmgrOpts|latex_output_dir|latexOutputDir|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|title_meta|titleMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|keep_tex|keepTex|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", "u"), Y0 = new RegExp("^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|reference_doc|referenceDoc|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", "u"), X0 = new RegExp("^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|reveal_js_url|revealJsUrl|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|auto_stretch|autoStretch|min_scale|minScale|max_scale|maxScale|disable_layout|disableLayout|preview_links|previewLinks|auto_play_media|autoPlayMedia|preload_iframes|preloadIframes|view_distance|viewDistance|mobile_view_distance|mobileViewDistance|parallax_background_image|parallaxBackgroundImage|parallax_background_size|parallaxBackgroundSize|parallax_background_horizontal|parallaxBackgroundHorizontal|parallax_background_vertical|parallaxBackgroundVertical|navigation_mode|navigationMode|mouse_wheel|mouseWheel|hide_inactive_cursor|hideInactiveCursor|hide_cursor_time|hideCursorTime|controls_layout|controlsLayout|controls_tutorial|controlsTutorial|controls_back_arrows|controlsBackArrows|auto_slide|autoSlide|auto_slide_stoppable|autoSlideStoppable|auto_slide_method|autoSlideMethod|default_timing|defaultTiming|hash_type|hashType|hash_one_based_index|hashOneBasedIndex|respond_to_hash_changes|respondToHashChanges|fragment_in_url|fragmentInUrl|slide_tone|slideTone|pdf_separate_fragments|pdfSeparateFragments|pdf_page_height_offset|pdfPageHeightOffset|transition_speed|transitionSpeed|background_transition|backgroundTransition|auto_animate|autoAnimate|auto_animate_easing|autoAnimateEasing|auto_animate_duration|autoAnimateDuration|auto_animate_unmatched|autoAnimateUnmatched|auto_animate_styles|autoAnimateStyles|slide_level|slideLevel|slide_number|slideNumber|show_slide_number|showSlideNumber|title_slide_attributes|titleSlideAttributes|center_title_slide|centerTitleSlide|show_notes|showNotes|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", "u"), Z0 = new RegExp("^(?!(fig_align|figAlign|tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", "u"), eq = new RegExp("^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|s5_url|s5Url|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", "u"), rq = new RegExp("^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|slideous_url|slideousUrl|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", "u"), tq = new RegExp("^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|slidy_url|slidyUrl|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", "u"), aq = new RegExp("^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", "u"), nq = new RegExp("^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", "u");
  function _u(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r, y = r, g = false, x = null, D = r, $ = r;
    if (typeof l == "string") {
      if (!dv.test(l)) {
        let Ln = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/0/pattern", keyword: "pattern", params: { pattern: "^asciidoc(\\+.+)?$", schema: p.oneOf[0].oneOf[0] }, message: 'must match pattern "^asciidoc(\\+.+)?$"', schema: "^asciidoc(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[0], data: l };
        e === null ? e = [Ln] : e.push(Ln), r++;
      }
    } else {
      let Ln = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/0/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[0] }, message: "must be string", schema: p.oneOf[0].oneOf[0].type, parentSchema: p.oneOf[0].oneOf[0], data: l };
      e === null ? e = [Ln] : e.push(Ln), r++;
    }
    var m = D === r;
    m && (g = true, x = 0);
    let R = r, j = r;
    if (typeof l == "string") {
      if (!fv.test(l)) {
        let Ln = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/1/pattern", keyword: "pattern", params: { pattern: "^asciidoctor(\\+.+)?$", schema: p.oneOf[0].oneOf[1] }, message: 'must match pattern "^asciidoctor(\\+.+)?$"', schema: "^asciidoctor(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[1], data: l };
        e === null ? e = [Ln] : e.push(Ln), r++;
      }
    } else {
      let Ln = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/1/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[1] }, message: "must be string", schema: p.oneOf[0].oneOf[1].type, parentSchema: p.oneOf[0].oneOf[1], data: l };
      e === null ? e = [Ln] : e.push(Ln), r++;
    }
    var m = R === r;
    if (m && g)
      g = false, x = [x, 1];
    else {
      m && (g = true, x = 1);
      let Ln = r, k1 = r;
      if (typeof l == "string") {
        if (!mv.test(l)) {
          let vu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/2/pattern", keyword: "pattern", params: { pattern: "^beamer(\\+.+)?$", schema: p.oneOf[0].oneOf[2] }, message: 'must match pattern "^beamer(\\+.+)?$"', schema: "^beamer(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[2], data: l };
          e === null ? e = [vu] : e.push(vu), r++;
        }
      } else {
        let vu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/2/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[2] }, message: "must be string", schema: p.oneOf[0].oneOf[2].type, parentSchema: p.oneOf[0].oneOf[2], data: l };
        e === null ? e = [vu] : e.push(vu), r++;
      }
      var m = Ln === r;
      if (m && g)
        g = false, x = [x, 2];
      else {
        m && (g = true, x = 2);
        let vu = r, Gm = r;
        if (typeof l == "string") {
          if (!gv.test(l)) {
            let t = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/3/pattern", keyword: "pattern", params: { pattern: "^biblatex(\\+.+)?$", schema: p.oneOf[0].oneOf[3] }, message: 'must match pattern "^biblatex(\\+.+)?$"', schema: "^biblatex(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[3], data: l };
            e === null ? e = [t] : e.push(t), r++;
          }
        } else {
          let t = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/3/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[3] }, message: "must be string", schema: p.oneOf[0].oneOf[3].type, parentSchema: p.oneOf[0].oneOf[3], data: l };
          e === null ? e = [t] : e.push(t), r++;
        }
        var m = vu === r;
        if (m && g)
          g = false, x = [x, 3];
        else {
          m && (g = true, x = 3);
          let t = r, mt = r;
          if (typeof l == "string") {
            if (!hv.test(l)) {
              let L = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/4/pattern", keyword: "pattern", params: { pattern: "^bibtex(\\+.+)?$", schema: p.oneOf[0].oneOf[4] }, message: 'must match pattern "^bibtex(\\+.+)?$"', schema: "^bibtex(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[4], data: l };
              e === null ? e = [L] : e.push(L), r++;
            }
          } else {
            let L = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/4/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[4] }, message: "must be string", schema: p.oneOf[0].oneOf[4].type, parentSchema: p.oneOf[0].oneOf[4], data: l };
            e === null ? e = [L] : e.push(L), r++;
          }
          var m = t === r;
          if (m && g)
            g = false, x = [x, 4];
          else {
            m && (g = true, x = 4);
            let L = r, M = r;
            if (typeof l == "string") {
              if (!yv.test(l)) {
                let ia = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/5/pattern", keyword: "pattern", params: { pattern: "^commonmark(\\+.+)?$", schema: p.oneOf[0].oneOf[5] }, message: 'must match pattern "^commonmark(\\+.+)?$"', schema: "^commonmark(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[5], data: l };
                e === null ? e = [ia] : e.push(ia), r++;
              }
            } else {
              let ia = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/5/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[5] }, message: "must be string", schema: p.oneOf[0].oneOf[5].type, parentSchema: p.oneOf[0].oneOf[5], data: l };
              e === null ? e = [ia] : e.push(ia), r++;
            }
            var m = L === r;
            if (m && g)
              g = false, x = [x, 5];
            else {
              m && (g = true, x = 5);
              let ia = r, gt = r;
              if (typeof l == "string") {
                if (!bv.test(l)) {
                  let ye = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/6/pattern", keyword: "pattern", params: { pattern: "^commonmark_x(\\+.+)?$", schema: p.oneOf[0].oneOf[6] }, message: 'must match pattern "^commonmark_x(\\+.+)?$"', schema: "^commonmark_x(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[6], data: l };
                  e === null ? e = [ye] : e.push(ye), r++;
                }
              } else {
                let ye = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/6/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[6] }, message: "must be string", schema: p.oneOf[0].oneOf[6].type, parentSchema: p.oneOf[0].oneOf[6], data: l };
                e === null ? e = [ye] : e.push(ye), r++;
              }
              var m = ia === r;
              if (m && g)
                g = false, x = [x, 6];
              else {
                m && (g = true, x = 6);
                let ye = r, a = r;
                if (typeof l == "string") {
                  if (!_v.test(l)) {
                    let oa = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/7/pattern", keyword: "pattern", params: { pattern: "^context(\\+.+)?$", schema: p.oneOf[0].oneOf[7] }, message: 'must match pattern "^context(\\+.+)?$"', schema: "^context(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[7], data: l };
                    e === null ? e = [oa] : e.push(oa), r++;
                  }
                } else {
                  let oa = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/7/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[7] }, message: "must be string", schema: p.oneOf[0].oneOf[7].type, parentSchema: p.oneOf[0].oneOf[7], data: l };
                  e === null ? e = [oa] : e.push(oa), r++;
                }
                var m = ye === r;
                if (m && g)
                  g = false, x = [x, 7];
                else {
                  m && (g = true, x = 7);
                  let oa = r, k = r;
                  if (typeof l == "string") {
                    if (!vv.test(l)) {
                      let qu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/8/pattern", keyword: "pattern", params: { pattern: "^csljson(\\+.+)?$", schema: p.oneOf[0].oneOf[8] }, message: 'must match pattern "^csljson(\\+.+)?$"', schema: "^csljson(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[8], data: l };
                      e === null ? e = [qu] : e.push(qu), r++;
                    }
                  } else {
                    let qu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/8/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[8] }, message: "must be string", schema: p.oneOf[0].oneOf[8].type, parentSchema: p.oneOf[0].oneOf[8], data: l };
                    e === null ? e = [qu] : e.push(qu), r++;
                  }
                  var m = oa === r;
                  if (m && g)
                    g = false, x = [x, 8];
                  else {
                    m && (g = true, x = 8);
                    let qu = r, fq = r;
                    if (typeof l == "string") {
                      if (!qv.test(l)) {
                        let Ou = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/9/pattern", keyword: "pattern", params: { pattern: "^docbook(\\+.+)?$", schema: p.oneOf[0].oneOf[9] }, message: 'must match pattern "^docbook(\\+.+)?$"', schema: "^docbook(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[9], data: l };
                        e === null ? e = [Ou] : e.push(Ou), r++;
                      }
                    } else {
                      let Ou = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/9/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[9] }, message: "must be string", schema: p.oneOf[0].oneOf[9].type, parentSchema: p.oneOf[0].oneOf[9], data: l };
                      e === null ? e = [Ou] : e.push(Ou), r++;
                    }
                    var m = qu === r;
                    if (m && g)
                      g = false, x = [x, 9];
                    else {
                      m && (g = true, x = 9);
                      let Ou = r, mq = r;
                      if (typeof l == "string") {
                        if (!Ov.test(l)) {
                          let Pu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/10/pattern", keyword: "pattern", params: { pattern: "^docbook4(\\+.+)?$", schema: p.oneOf[0].oneOf[10] }, message: 'must match pattern "^docbook4(\\+.+)?$"', schema: "^docbook4(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[10], data: l };
                          e === null ? e = [Pu] : e.push(Pu), r++;
                        }
                      } else {
                        let Pu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/10/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[10] }, message: "must be string", schema: p.oneOf[0].oneOf[10].type, parentSchema: p.oneOf[0].oneOf[10], data: l };
                        e === null ? e = [Pu] : e.push(Pu), r++;
                      }
                      var m = Ou === r;
                      if (m && g)
                        g = false, x = [x, 10];
                      else {
                        m && (g = true, x = 10);
                        let Pu = r, gq = r;
                        if (typeof l == "string") {
                          if (!Pv.test(l)) {
                            let xu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/11/pattern", keyword: "pattern", params: { pattern: "^docbook5(\\+.+)?$", schema: p.oneOf[0].oneOf[11] }, message: 'must match pattern "^docbook5(\\+.+)?$"', schema: "^docbook5(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[11], data: l };
                            e === null ? e = [xu] : e.push(xu), r++;
                          }
                        } else {
                          let xu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/11/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[11] }, message: "must be string", schema: p.oneOf[0].oneOf[11].type, parentSchema: p.oneOf[0].oneOf[11], data: l };
                          e === null ? e = [xu] : e.push(xu), r++;
                        }
                        var m = Pu === r;
                        if (m && g)
                          g = false, x = [x, 11];
                        else {
                          m && (g = true, x = 11);
                          let xu = r, hq = r;
                          if (typeof l == "string") {
                            if (!xv.test(l)) {
                              let ku = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/12/pattern", keyword: "pattern", params: { pattern: "^docx(\\+.+)?$", schema: p.oneOf[0].oneOf[12] }, message: 'must match pattern "^docx(\\+.+)?$"', schema: "^docx(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[12], data: l };
                              e === null ? e = [ku] : e.push(ku), r++;
                            }
                          } else {
                            let ku = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/12/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[12] }, message: "must be string", schema: p.oneOf[0].oneOf[12].type, parentSchema: p.oneOf[0].oneOf[12], data: l };
                            e === null ? e = [ku] : e.push(ku), r++;
                          }
                          var m = xu === r;
                          if (m && g)
                            g = false, x = [x, 12];
                          else {
                            m && (g = true, x = 12);
                            let ku = r, yq = r;
                            if (typeof l == "string") {
                              if (!kv.test(l)) {
                                let Du = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/13/pattern", keyword: "pattern", params: { pattern: "^dokuwiki(\\+.+)?$", schema: p.oneOf[0].oneOf[13] }, message: 'must match pattern "^dokuwiki(\\+.+)?$"', schema: "^dokuwiki(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[13], data: l };
                                e === null ? e = [Du] : e.push(Du), r++;
                              }
                            } else {
                              let Du = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/13/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[13] }, message: "must be string", schema: p.oneOf[0].oneOf[13].type, parentSchema: p.oneOf[0].oneOf[13], data: l };
                              e === null ? e = [Du] : e.push(Du), r++;
                            }
                            var m = ku === r;
                            if (m && g)
                              g = false, x = [x, 13];
                            else {
                              m && (g = true, x = 13);
                              let Du = r, bq = r;
                              if (typeof l == "string") {
                                if (!Dv.test(l)) {
                                  let wu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/14/pattern", keyword: "pattern", params: { pattern: "^dzslides(\\+.+)?$", schema: p.oneOf[0].oneOf[14] }, message: 'must match pattern "^dzslides(\\+.+)?$"', schema: "^dzslides(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[14], data: l };
                                  e === null ? e = [wu] : e.push(wu), r++;
                                }
                              } else {
                                let wu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/14/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[14] }, message: "must be string", schema: p.oneOf[0].oneOf[14].type, parentSchema: p.oneOf[0].oneOf[14], data: l };
                                e === null ? e = [wu] : e.push(wu), r++;
                              }
                              var m = Du === r;
                              if (m && g)
                                g = false, x = [x, 14];
                              else {
                                m && (g = true, x = 14);
                                let wu = r, _q = r;
                                if (typeof l == "string") {
                                  if (!wv.test(l)) {
                                    let $u = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/15/pattern", keyword: "pattern", params: { pattern: "^epub(\\+.+)?$", schema: p.oneOf[0].oneOf[15] }, message: 'must match pattern "^epub(\\+.+)?$"', schema: "^epub(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[15], data: l };
                                    e === null ? e = [$u] : e.push($u), r++;
                                  }
                                } else {
                                  let $u = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/15/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[15] }, message: "must be string", schema: p.oneOf[0].oneOf[15].type, parentSchema: p.oneOf[0].oneOf[15], data: l };
                                  e === null ? e = [$u] : e.push($u), r++;
                                }
                                var m = wu === r;
                                if (m && g)
                                  g = false, x = [x, 15];
                                else {
                                  m && (g = true, x = 15);
                                  let $u = r, vq = r;
                                  if (typeof l == "string") {
                                    if (!$v.test(l)) {
                                      let Su = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/16/pattern", keyword: "pattern", params: { pattern: "^epub2(\\+.+)?$", schema: p.oneOf[0].oneOf[16] }, message: 'must match pattern "^epub2(\\+.+)?$"', schema: "^epub2(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[16], data: l };
                                      e === null ? e = [Su] : e.push(Su), r++;
                                    }
                                  } else {
                                    let Su = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/16/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[16] }, message: "must be string", schema: p.oneOf[0].oneOf[16].type, parentSchema: p.oneOf[0].oneOf[16], data: l };
                                    e === null ? e = [Su] : e.push(Su), r++;
                                  }
                                  var m = $u === r;
                                  if (m && g)
                                    g = false, x = [x, 16];
                                  else {
                                    m && (g = true, x = 16);
                                    let Su = r, qq = r;
                                    if (typeof l == "string") {
                                      if (!Sv.test(l)) {
                                        let Fu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/17/pattern", keyword: "pattern", params: { pattern: "^epub3(\\+.+)?$", schema: p.oneOf[0].oneOf[17] }, message: 'must match pattern "^epub3(\\+.+)?$"', schema: "^epub3(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[17], data: l };
                                        e === null ? e = [Fu] : e.push(Fu), r++;
                                      }
                                    } else {
                                      let Fu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/17/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[17] }, message: "must be string", schema: p.oneOf[0].oneOf[17].type, parentSchema: p.oneOf[0].oneOf[17], data: l };
                                      e === null ? e = [Fu] : e.push(Fu), r++;
                                    }
                                    var m = Su === r;
                                    if (m && g)
                                      g = false, x = [x, 17];
                                    else {
                                      m && (g = true, x = 17);
                                      let Fu = r, Oq = r;
                                      if (typeof l == "string") {
                                        if (!Fv.test(l)) {
                                          let Au = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/18/pattern", keyword: "pattern", params: { pattern: "^fb2(\\+.+)?$", schema: p.oneOf[0].oneOf[18] }, message: 'must match pattern "^fb2(\\+.+)?$"', schema: "^fb2(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[18], data: l };
                                          e === null ? e = [Au] : e.push(Au), r++;
                                        }
                                      } else {
                                        let Au = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/18/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[18] }, message: "must be string", schema: p.oneOf[0].oneOf[18].type, parentSchema: p.oneOf[0].oneOf[18], data: l };
                                        e === null ? e = [Au] : e.push(Au), r++;
                                      }
                                      var m = Fu === r;
                                      if (m && g)
                                        g = false, x = [x, 18];
                                      else {
                                        m && (g = true, x = 18);
                                        let Au = r, Pq = r;
                                        if (typeof l == "string") {
                                          if (!Av.test(l)) {
                                            let Bu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/19/pattern", keyword: "pattern", params: { pattern: "^gfm(\\+.+)?$", schema: p.oneOf[0].oneOf[19] }, message: 'must match pattern "^gfm(\\+.+)?$"', schema: "^gfm(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[19], data: l };
                                            e === null ? e = [Bu] : e.push(Bu), r++;
                                          }
                                        } else {
                                          let Bu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/19/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[19] }, message: "must be string", schema: p.oneOf[0].oneOf[19].type, parentSchema: p.oneOf[0].oneOf[19], data: l };
                                          e === null ? e = [Bu] : e.push(Bu), r++;
                                        }
                                        var m = Au === r;
                                        if (m && g)
                                          g = false, x = [x, 19];
                                        else {
                                          m && (g = true, x = 19);
                                          let Bu = r, xq = r;
                                          if (typeof l == "string") {
                                            if (!Bv.test(l)) {
                                              let ju = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/20/pattern", keyword: "pattern", params: { pattern: "^haddock(\\+.+)?$", schema: p.oneOf[0].oneOf[20] }, message: 'must match pattern "^haddock(\\+.+)?$"', schema: "^haddock(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[20], data: l };
                                              e === null ? e = [ju] : e.push(ju), r++;
                                            }
                                          } else {
                                            let ju = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/20/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[20] }, message: "must be string", schema: p.oneOf[0].oneOf[20].type, parentSchema: p.oneOf[0].oneOf[20], data: l };
                                            e === null ? e = [ju] : e.push(ju), r++;
                                          }
                                          var m = Bu === r;
                                          if (m && g)
                                            g = false, x = [x, 20];
                                          else {
                                            m && (g = true, x = 20);
                                            let ju = r, kq = r;
                                            if (typeof l == "string") {
                                              if (!jv.test(l)) {
                                                let Eu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/21/pattern", keyword: "pattern", params: { pattern: "^html(\\+.+)?$", schema: p.oneOf[0].oneOf[21] }, message: 'must match pattern "^html(\\+.+)?$"', schema: "^html(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[21], data: l };
                                                e === null ? e = [Eu] : e.push(Eu), r++;
                                              }
                                            } else {
                                              let Eu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/21/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[21] }, message: "must be string", schema: p.oneOf[0].oneOf[21].type, parentSchema: p.oneOf[0].oneOf[21], data: l };
                                              e === null ? e = [Eu] : e.push(Eu), r++;
                                            }
                                            var m = ju === r;
                                            if (m && g)
                                              g = false, x = [x, 21];
                                            else {
                                              m && (g = true, x = 21);
                                              let Eu = r, Dq = r;
                                              if (typeof l == "string") {
                                                if (!Ev.test(l)) {
                                                  let Nu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/22/pattern", keyword: "pattern", params: { pattern: "^html4(\\+.+)?$", schema: p.oneOf[0].oneOf[22] }, message: 'must match pattern "^html4(\\+.+)?$"', schema: "^html4(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[22], data: l };
                                                  e === null ? e = [Nu] : e.push(Nu), r++;
                                                }
                                              } else {
                                                let Nu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/22/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[22] }, message: "must be string", schema: p.oneOf[0].oneOf[22].type, parentSchema: p.oneOf[0].oneOf[22], data: l };
                                                e === null ? e = [Nu] : e.push(Nu), r++;
                                              }
                                              var m = Eu === r;
                                              if (m && g)
                                                g = false, x = [x, 22];
                                              else {
                                                m && (g = true, x = 22);
                                                let Nu = r, wq = r;
                                                if (typeof l == "string") {
                                                  if (!Nv.test(l)) {
                                                    let Cu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/23/pattern", keyword: "pattern", params: { pattern: "^html5(\\+.+)?$", schema: p.oneOf[0].oneOf[23] }, message: 'must match pattern "^html5(\\+.+)?$"', schema: "^html5(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[23], data: l };
                                                    e === null ? e = [Cu] : e.push(Cu), r++;
                                                  }
                                                } else {
                                                  let Cu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/23/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[23] }, message: "must be string", schema: p.oneOf[0].oneOf[23].type, parentSchema: p.oneOf[0].oneOf[23], data: l };
                                                  e === null ? e = [Cu] : e.push(Cu), r++;
                                                }
                                                var m = Nu === r;
                                                if (m && g)
                                                  g = false, x = [x, 23];
                                                else {
                                                  m && (g = true, x = 23);
                                                  let Cu = r, $q = r;
                                                  if (typeof l == "string") {
                                                    if (!Cv.test(l)) {
                                                      let Iu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/24/pattern", keyword: "pattern", params: { pattern: "^icml(\\+.+)?$", schema: p.oneOf[0].oneOf[24] }, message: 'must match pattern "^icml(\\+.+)?$"', schema: "^icml(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[24], data: l };
                                                      e === null ? e = [Iu] : e.push(Iu), r++;
                                                    }
                                                  } else {
                                                    let Iu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/24/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[24] }, message: "must be string", schema: p.oneOf[0].oneOf[24].type, parentSchema: p.oneOf[0].oneOf[24], data: l };
                                                    e === null ? e = [Iu] : e.push(Iu), r++;
                                                  }
                                                  var m = Cu === r;
                                                  if (m && g)
                                                    g = false, x = [x, 24];
                                                  else {
                                                    m && (g = true, x = 24);
                                                    let Iu = r, Sq = r;
                                                    if (typeof l == "string") {
                                                      if (!Iv.test(l)) {
                                                        let Hu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/25/pattern", keyword: "pattern", params: { pattern: "^ipynb(\\+.+)?$", schema: p.oneOf[0].oneOf[25] }, message: 'must match pattern "^ipynb(\\+.+)?$"', schema: "^ipynb(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[25], data: l };
                                                        e === null ? e = [Hu] : e.push(Hu), r++;
                                                      }
                                                    } else {
                                                      let Hu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/25/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[25] }, message: "must be string", schema: p.oneOf[0].oneOf[25].type, parentSchema: p.oneOf[0].oneOf[25], data: l };
                                                      e === null ? e = [Hu] : e.push(Hu), r++;
                                                    }
                                                    var m = Iu === r;
                                                    if (m && g)
                                                      g = false, x = [x, 25];
                                                    else {
                                                      m && (g = true, x = 25);
                                                      let Hu = r, Fq = r;
                                                      if (typeof l == "string") {
                                                        if (!Hv.test(l)) {
                                                          let Lu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/26/pattern", keyword: "pattern", params: { pattern: "^jats(\\+.+)?$", schema: p.oneOf[0].oneOf[26] }, message: 'must match pattern "^jats(\\+.+)?$"', schema: "^jats(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[26], data: l };
                                                          e === null ? e = [Lu] : e.push(Lu), r++;
                                                        }
                                                      } else {
                                                        let Lu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/26/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[26] }, message: "must be string", schema: p.oneOf[0].oneOf[26].type, parentSchema: p.oneOf[0].oneOf[26], data: l };
                                                        e === null ? e = [Lu] : e.push(Lu), r++;
                                                      }
                                                      var m = Hu === r;
                                                      if (m && g)
                                                        g = false, x = [x, 26];
                                                      else {
                                                        m && (g = true, x = 26);
                                                        let Lu = r, Aq = r;
                                                        if (typeof l == "string") {
                                                          if (!Lv.test(l)) {
                                                            let Mu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/27/pattern", keyword: "pattern", params: { pattern: "^jats_archiving(\\+.+)?$", schema: p.oneOf[0].oneOf[27] }, message: 'must match pattern "^jats_archiving(\\+.+)?$"', schema: "^jats_archiving(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[27], data: l };
                                                            e === null ? e = [Mu] : e.push(Mu), r++;
                                                          }
                                                        } else {
                                                          let Mu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/27/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[27] }, message: "must be string", schema: p.oneOf[0].oneOf[27].type, parentSchema: p.oneOf[0].oneOf[27], data: l };
                                                          e === null ? e = [Mu] : e.push(Mu), r++;
                                                        }
                                                        var m = Lu === r;
                                                        if (m && g)
                                                          g = false, x = [x, 27];
                                                        else {
                                                          m && (g = true, x = 27);
                                                          let Mu = r, Bq = r;
                                                          if (typeof l == "string") {
                                                            if (!Mv.test(l)) {
                                                              let Tu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/28/pattern", keyword: "pattern", params: { pattern: "^jats_articleauthoring(\\+.+)?$", schema: p.oneOf[0].oneOf[28] }, message: 'must match pattern "^jats_articleauthoring(\\+.+)?$"', schema: "^jats_articleauthoring(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[28], data: l };
                                                              e === null ? e = [Tu] : e.push(Tu), r++;
                                                            }
                                                          } else {
                                                            let Tu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/28/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[28] }, message: "must be string", schema: p.oneOf[0].oneOf[28].type, parentSchema: p.oneOf[0].oneOf[28], data: l };
                                                            e === null ? e = [Tu] : e.push(Tu), r++;
                                                          }
                                                          var m = Mu === r;
                                                          if (m && g)
                                                            g = false, x = [x, 28];
                                                          else {
                                                            m && (g = true, x = 28);
                                                            let Tu = r, jq = r;
                                                            if (typeof l == "string") {
                                                              if (!Tv.test(l)) {
                                                                let zu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/29/pattern", keyword: "pattern", params: { pattern: "^jats_publishing(\\+.+)?$", schema: p.oneOf[0].oneOf[29] }, message: 'must match pattern "^jats_publishing(\\+.+)?$"', schema: "^jats_publishing(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[29], data: l };
                                                                e === null ? e = [zu] : e.push(zu), r++;
                                                              }
                                                            } else {
                                                              let zu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/29/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[29] }, message: "must be string", schema: p.oneOf[0].oneOf[29].type, parentSchema: p.oneOf[0].oneOf[29], data: l };
                                                              e === null ? e = [zu] : e.push(zu), r++;
                                                            }
                                                            var m = Tu === r;
                                                            if (m && g)
                                                              g = false, x = [x, 29];
                                                            else {
                                                              m && (g = true, x = 29);
                                                              let zu = r, Eq = r;
                                                              if (typeof l == "string") {
                                                                if (!zv.test(l)) {
                                                                  let Wu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/30/pattern", keyword: "pattern", params: { pattern: "^jira(\\+.+)?$", schema: p.oneOf[0].oneOf[30] }, message: 'must match pattern "^jira(\\+.+)?$"', schema: "^jira(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[30], data: l };
                                                                  e === null ? e = [Wu] : e.push(Wu), r++;
                                                                }
                                                              } else {
                                                                let Wu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/30/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[30] }, message: "must be string", schema: p.oneOf[0].oneOf[30].type, parentSchema: p.oneOf[0].oneOf[30], data: l };
                                                                e === null ? e = [Wu] : e.push(Wu), r++;
                                                              }
                                                              var m = zu === r;
                                                              if (m && g)
                                                                g = false, x = [x, 30];
                                                              else {
                                                                m && (g = true, x = 30);
                                                                let Wu = r, Nq = r;
                                                                if (typeof l == "string") {
                                                                  if (!Wv.test(l)) {
                                                                    let Ru = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/31/pattern", keyword: "pattern", params: { pattern: "^json(\\+.+)?$", schema: p.oneOf[0].oneOf[31] }, message: 'must match pattern "^json(\\+.+)?$"', schema: "^json(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[31], data: l };
                                                                    e === null ? e = [Ru] : e.push(Ru), r++;
                                                                  }
                                                                } else {
                                                                  let Ru = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/31/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[31] }, message: "must be string", schema: p.oneOf[0].oneOf[31].type, parentSchema: p.oneOf[0].oneOf[31], data: l };
                                                                  e === null ? e = [Ru] : e.push(Ru), r++;
                                                                }
                                                                var m = Wu === r;
                                                                if (m && g)
                                                                  g = false, x = [x, 31];
                                                                else {
                                                                  m && (g = true, x = 31);
                                                                  let Ru = r, Cq = r;
                                                                  if (typeof l == "string") {
                                                                    if (!Rv.test(l)) {
                                                                      let Vu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/32/pattern", keyword: "pattern", params: { pattern: "^latex(\\+.+)?$", schema: p.oneOf[0].oneOf[32] }, message: 'must match pattern "^latex(\\+.+)?$"', schema: "^latex(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[32], data: l };
                                                                      e === null ? e = [Vu] : e.push(Vu), r++;
                                                                    }
                                                                  } else {
                                                                    let Vu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/32/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[32] }, message: "must be string", schema: p.oneOf[0].oneOf[32].type, parentSchema: p.oneOf[0].oneOf[32], data: l };
                                                                    e === null ? e = [Vu] : e.push(Vu), r++;
                                                                  }
                                                                  var m = Ru === r;
                                                                  if (m && g)
                                                                    g = false, x = [x, 32];
                                                                  else {
                                                                    m && (g = true, x = 32);
                                                                    let Vu = r, Iq = r;
                                                                    if (typeof l == "string") {
                                                                      if (!Vv.test(l)) {
                                                                        let Uu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/33/pattern", keyword: "pattern", params: { pattern: "^man(\\+.+)?$", schema: p.oneOf[0].oneOf[33] }, message: 'must match pattern "^man(\\+.+)?$"', schema: "^man(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[33], data: l };
                                                                        e === null ? e = [Uu] : e.push(Uu), r++;
                                                                      }
                                                                    } else {
                                                                      let Uu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/33/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[33] }, message: "must be string", schema: p.oneOf[0].oneOf[33].type, parentSchema: p.oneOf[0].oneOf[33], data: l };
                                                                      e === null ? e = [Uu] : e.push(Uu), r++;
                                                                    }
                                                                    var m = Vu === r;
                                                                    if (m && g)
                                                                      g = false, x = [x, 33];
                                                                    else {
                                                                      m && (g = true, x = 33);
                                                                      let Uu = r, Hq = r;
                                                                      if (typeof l == "string") {
                                                                        if (!Uv.test(l)) {
                                                                          let Ku = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/34/pattern", keyword: "pattern", params: { pattern: "^markdown(\\+.+)?$", schema: p.oneOf[0].oneOf[34] }, message: 'must match pattern "^markdown(\\+.+)?$"', schema: "^markdown(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[34], data: l };
                                                                          e === null ? e = [Ku] : e.push(Ku), r++;
                                                                        }
                                                                      } else {
                                                                        let Ku = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/34/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[34] }, message: "must be string", schema: p.oneOf[0].oneOf[34].type, parentSchema: p.oneOf[0].oneOf[34], data: l };
                                                                        e === null ? e = [Ku] : e.push(Ku), r++;
                                                                      }
                                                                      var m = Uu === r;
                                                                      if (m && g)
                                                                        g = false, x = [x, 34];
                                                                      else {
                                                                        m && (g = true, x = 34);
                                                                        let Ku = r, Lq = r;
                                                                        if (typeof l == "string") {
                                                                          if (!Kv.test(l)) {
                                                                            let Ju = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/35/pattern", keyword: "pattern", params: { pattern: "^markdown_github(\\+.+)?$", schema: p.oneOf[0].oneOf[35] }, message: 'must match pattern "^markdown_github(\\+.+)?$"', schema: "^markdown_github(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[35], data: l };
                                                                            e === null ? e = [Ju] : e.push(Ju), r++;
                                                                          }
                                                                        } else {
                                                                          let Ju = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/35/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[35] }, message: "must be string", schema: p.oneOf[0].oneOf[35].type, parentSchema: p.oneOf[0].oneOf[35], data: l };
                                                                          e === null ? e = [Ju] : e.push(Ju), r++;
                                                                        }
                                                                        var m = Ku === r;
                                                                        if (m && g)
                                                                          g = false, x = [x, 35];
                                                                        else {
                                                                          m && (g = true, x = 35);
                                                                          let Ju = r, Mq = r;
                                                                          if (typeof l == "string") {
                                                                            if (!Jv.test(l)) {
                                                                              let Qu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/36/pattern", keyword: "pattern", params: { pattern: "^markdown_mmd(\\+.+)?$", schema: p.oneOf[0].oneOf[36] }, message: 'must match pattern "^markdown_mmd(\\+.+)?$"', schema: "^markdown_mmd(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[36], data: l };
                                                                              e === null ? e = [Qu] : e.push(Qu), r++;
                                                                            }
                                                                          } else {
                                                                            let Qu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/36/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[36] }, message: "must be string", schema: p.oneOf[0].oneOf[36].type, parentSchema: p.oneOf[0].oneOf[36], data: l };
                                                                            e === null ? e = [Qu] : e.push(Qu), r++;
                                                                          }
                                                                          var m = Ju === r;
                                                                          if (m && g)
                                                                            g = false, x = [x, 36];
                                                                          else {
                                                                            m && (g = true, x = 36);
                                                                            let Qu = r, Tq = r;
                                                                            if (typeof l == "string") {
                                                                              if (!Qv.test(l)) {
                                                                                let Gu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/37/pattern", keyword: "pattern", params: { pattern: "^markdown_phpextra(\\+.+)?$", schema: p.oneOf[0].oneOf[37] }, message: 'must match pattern "^markdown_phpextra(\\+.+)?$"', schema: "^markdown_phpextra(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[37], data: l };
                                                                                e === null ? e = [Gu] : e.push(Gu), r++;
                                                                              }
                                                                            } else {
                                                                              let Gu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/37/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[37] }, message: "must be string", schema: p.oneOf[0].oneOf[37].type, parentSchema: p.oneOf[0].oneOf[37], data: l };
                                                                              e === null ? e = [Gu] : e.push(Gu), r++;
                                                                            }
                                                                            var m = Qu === r;
                                                                            if (m && g)
                                                                              g = false, x = [x, 37];
                                                                            else {
                                                                              m && (g = true, x = 37);
                                                                              let Gu = r, zq = r;
                                                                              if (typeof l == "string") {
                                                                                if (!Gv.test(l)) {
                                                                                  let Yu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/38/pattern", keyword: "pattern", params: { pattern: "^markdown_strict(\\+.+)?$", schema: p.oneOf[0].oneOf[38] }, message: 'must match pattern "^markdown_strict(\\+.+)?$"', schema: "^markdown_strict(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[38], data: l };
                                                                                  e === null ? e = [Yu] : e.push(Yu), r++;
                                                                                }
                                                                              } else {
                                                                                let Yu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/38/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[38] }, message: "must be string", schema: p.oneOf[0].oneOf[38].type, parentSchema: p.oneOf[0].oneOf[38], data: l };
                                                                                e === null ? e = [Yu] : e.push(Yu), r++;
                                                                              }
                                                                              var m = Gu === r;
                                                                              if (m && g)
                                                                                g = false, x = [x, 38];
                                                                              else {
                                                                                m && (g = true, x = 38);
                                                                                let Yu = r, Wq = r;
                                                                                if (typeof l == "string") {
                                                                                  if (!Yv.test(l)) {
                                                                                    let Xu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/39/pattern", keyword: "pattern", params: { pattern: "^markua(\\+.+)?$", schema: p.oneOf[0].oneOf[39] }, message: 'must match pattern "^markua(\\+.+)?$"', schema: "^markua(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[39], data: l };
                                                                                    e === null ? e = [Xu] : e.push(Xu), r++;
                                                                                  }
                                                                                } else {
                                                                                  let Xu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/39/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[39] }, message: "must be string", schema: p.oneOf[0].oneOf[39].type, parentSchema: p.oneOf[0].oneOf[39], data: l };
                                                                                  e === null ? e = [Xu] : e.push(Xu), r++;
                                                                                }
                                                                                var m = Yu === r;
                                                                                if (m && g)
                                                                                  g = false, x = [x, 39];
                                                                                else {
                                                                                  m && (g = true, x = 39);
                                                                                  let Xu = r, Rq = r;
                                                                                  if (typeof l == "string") {
                                                                                    if (!Xv.test(l)) {
                                                                                      let Zu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/40/pattern", keyword: "pattern", params: { pattern: "^mediawiki(\\+.+)?$", schema: p.oneOf[0].oneOf[40] }, message: 'must match pattern "^mediawiki(\\+.+)?$"', schema: "^mediawiki(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[40], data: l };
                                                                                      e === null ? e = [Zu] : e.push(Zu), r++;
                                                                                    }
                                                                                  } else {
                                                                                    let Zu = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/40/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[40] }, message: "must be string", schema: p.oneOf[0].oneOf[40].type, parentSchema: p.oneOf[0].oneOf[40], data: l };
                                                                                    e === null ? e = [Zu] : e.push(Zu), r++;
                                                                                  }
                                                                                  var m = Xu === r;
                                                                                  if (m && g)
                                                                                    g = false, x = [x, 40];
                                                                                  else {
                                                                                    m && (g = true, x = 40);
                                                                                    let Zu = r, Vq = r;
                                                                                    if (typeof l == "string") {
                                                                                      if (!Zv.test(l)) {
                                                                                        let ed = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/41/pattern", keyword: "pattern", params: { pattern: "^ms(\\+.+)?$", schema: p.oneOf[0].oneOf[41] }, message: 'must match pattern "^ms(\\+.+)?$"', schema: "^ms(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[41], data: l };
                                                                                        e === null ? e = [ed] : e.push(ed), r++;
                                                                                      }
                                                                                    } else {
                                                                                      let ed = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/41/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[41] }, message: "must be string", schema: p.oneOf[0].oneOf[41].type, parentSchema: p.oneOf[0].oneOf[41], data: l };
                                                                                      e === null ? e = [ed] : e.push(ed), r++;
                                                                                    }
                                                                                    var m = Zu === r;
                                                                                    if (m && g)
                                                                                      g = false, x = [x, 41];
                                                                                    else {
                                                                                      m && (g = true, x = 41);
                                                                                      let ed = r, Uq = r;
                                                                                      if (typeof l == "string") {
                                                                                        if (!e1.test(l)) {
                                                                                          let rd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/42/pattern", keyword: "pattern", params: { pattern: "^muse(\\+.+)?$", schema: p.oneOf[0].oneOf[42] }, message: 'must match pattern "^muse(\\+.+)?$"', schema: "^muse(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[42], data: l };
                                                                                          e === null ? e = [rd] : e.push(rd), r++;
                                                                                        }
                                                                                      } else {
                                                                                        let rd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/42/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[42] }, message: "must be string", schema: p.oneOf[0].oneOf[42].type, parentSchema: p.oneOf[0].oneOf[42], data: l };
                                                                                        e === null ? e = [rd] : e.push(rd), r++;
                                                                                      }
                                                                                      var m = ed === r;
                                                                                      if (m && g)
                                                                                        g = false, x = [x, 42];
                                                                                      else {
                                                                                        m && (g = true, x = 42);
                                                                                        let rd = r, Kq = r;
                                                                                        if (typeof l == "string") {
                                                                                          if (!r1.test(l)) {
                                                                                            let td = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/43/pattern", keyword: "pattern", params: { pattern: "^native(\\+.+)?$", schema: p.oneOf[0].oneOf[43] }, message: 'must match pattern "^native(\\+.+)?$"', schema: "^native(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[43], data: l };
                                                                                            e === null ? e = [td] : e.push(td), r++;
                                                                                          }
                                                                                        } else {
                                                                                          let td = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/43/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[43] }, message: "must be string", schema: p.oneOf[0].oneOf[43].type, parentSchema: p.oneOf[0].oneOf[43], data: l };
                                                                                          e === null ? e = [td] : e.push(td), r++;
                                                                                        }
                                                                                        var m = rd === r;
                                                                                        if (m && g)
                                                                                          g = false, x = [x, 43];
                                                                                        else {
                                                                                          m && (g = true, x = 43);
                                                                                          let td = r, Jq = r;
                                                                                          if (typeof l == "string") {
                                                                                            if (!t1.test(l)) {
                                                                                              let ad = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/44/pattern", keyword: "pattern", params: { pattern: "^odt(\\+.+)?$", schema: p.oneOf[0].oneOf[44] }, message: 'must match pattern "^odt(\\+.+)?$"', schema: "^odt(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[44], data: l };
                                                                                              e === null ? e = [ad] : e.push(ad), r++;
                                                                                            }
                                                                                          } else {
                                                                                            let ad = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/44/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[44] }, message: "must be string", schema: p.oneOf[0].oneOf[44].type, parentSchema: p.oneOf[0].oneOf[44], data: l };
                                                                                            e === null ? e = [ad] : e.push(ad), r++;
                                                                                          }
                                                                                          var m = td === r;
                                                                                          if (m && g)
                                                                                            g = false, x = [x, 44];
                                                                                          else {
                                                                                            m && (g = true, x = 44);
                                                                                            let ad = r, Qq = r;
                                                                                            if (typeof l == "string") {
                                                                                              if (!a1.test(l)) {
                                                                                                let nd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/45/pattern", keyword: "pattern", params: { pattern: "^opendocument(\\+.+)?$", schema: p.oneOf[0].oneOf[45] }, message: 'must match pattern "^opendocument(\\+.+)?$"', schema: "^opendocument(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[45], data: l };
                                                                                                e === null ? e = [nd] : e.push(nd), r++;
                                                                                              }
                                                                                            } else {
                                                                                              let nd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/45/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[45] }, message: "must be string", schema: p.oneOf[0].oneOf[45].type, parentSchema: p.oneOf[0].oneOf[45], data: l };
                                                                                              e === null ? e = [nd] : e.push(nd), r++;
                                                                                            }
                                                                                            var m = ad === r;
                                                                                            if (m && g)
                                                                                              g = false, x = [x, 45];
                                                                                            else {
                                                                                              m && (g = true, x = 45);
                                                                                              let nd = r, Gq = r;
                                                                                              if (typeof l == "string") {
                                                                                                if (!n1.test(l)) {
                                                                                                  let od = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/46/pattern", keyword: "pattern", params: { pattern: "^opml(\\+.+)?$", schema: p.oneOf[0].oneOf[46] }, message: 'must match pattern "^opml(\\+.+)?$"', schema: "^opml(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[46], data: l };
                                                                                                  e === null ? e = [od] : e.push(od), r++;
                                                                                                }
                                                                                              } else {
                                                                                                let od = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/46/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[46] }, message: "must be string", schema: p.oneOf[0].oneOf[46].type, parentSchema: p.oneOf[0].oneOf[46], data: l };
                                                                                                e === null ? e = [od] : e.push(od), r++;
                                                                                              }
                                                                                              var m = nd === r;
                                                                                              if (m && g)
                                                                                                g = false, x = [x, 46];
                                                                                              else {
                                                                                                m && (g = true, x = 46);
                                                                                                let od = r, Yq = r;
                                                                                                if (typeof l == "string") {
                                                                                                  if (!o1.test(l)) {
                                                                                                    let sd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/47/pattern", keyword: "pattern", params: { pattern: "^org(\\+.+)?$", schema: p.oneOf[0].oneOf[47] }, message: 'must match pattern "^org(\\+.+)?$"', schema: "^org(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[47], data: l };
                                                                                                    e === null ? e = [sd] : e.push(sd), r++;
                                                                                                  }
                                                                                                } else {
                                                                                                  let sd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/47/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[47] }, message: "must be string", schema: p.oneOf[0].oneOf[47].type, parentSchema: p.oneOf[0].oneOf[47], data: l };
                                                                                                  e === null ? e = [sd] : e.push(sd), r++;
                                                                                                }
                                                                                                var m = od === r;
                                                                                                if (m && g)
                                                                                                  g = false, x = [x, 47];
                                                                                                else {
                                                                                                  m && (g = true, x = 47);
                                                                                                  let sd = r, Xq = r;
                                                                                                  if (typeof l == "string") {
                                                                                                    if (!s1.test(l)) {
                                                                                                      let id = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/48/pattern", keyword: "pattern", params: { pattern: "^pdf(\\+.+)?$", schema: p.oneOf[0].oneOf[48] }, message: 'must match pattern "^pdf(\\+.+)?$"', schema: "^pdf(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[48], data: l };
                                                                                                      e === null ? e = [id] : e.push(id), r++;
                                                                                                    }
                                                                                                  } else {
                                                                                                    let id = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/48/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[48] }, message: "must be string", schema: p.oneOf[0].oneOf[48].type, parentSchema: p.oneOf[0].oneOf[48], data: l };
                                                                                                    e === null ? e = [id] : e.push(id), r++;
                                                                                                  }
                                                                                                  var m = sd === r;
                                                                                                  if (m && g)
                                                                                                    g = false, x = [x, 48];
                                                                                                  else {
                                                                                                    m && (g = true, x = 48);
                                                                                                    let id = r, Zq = r;
                                                                                                    if (typeof l == "string") {
                                                                                                      if (!i1.test(l)) {
                                                                                                        let ld = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/49/pattern", keyword: "pattern", params: { pattern: "^plain(\\+.+)?$", schema: p.oneOf[0].oneOf[49] }, message: 'must match pattern "^plain(\\+.+)?$"', schema: "^plain(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[49], data: l };
                                                                                                        e === null ? e = [ld] : e.push(ld), r++;
                                                                                                      }
                                                                                                    } else {
                                                                                                      let ld = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/49/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[49] }, message: "must be string", schema: p.oneOf[0].oneOf[49].type, parentSchema: p.oneOf[0].oneOf[49], data: l };
                                                                                                      e === null ? e = [ld] : e.push(ld), r++;
                                                                                                    }
                                                                                                    var m = id === r;
                                                                                                    if (m && g)
                                                                                                      g = false, x = [x, 49];
                                                                                                    else {
                                                                                                      m && (g = true, x = 49);
                                                                                                      let ld = r, eO = r;
                                                                                                      if (typeof l == "string") {
                                                                                                        if (!l1.test(l)) {
                                                                                                          let cd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/50/pattern", keyword: "pattern", params: { pattern: "^pptx(\\+.+)?$", schema: p.oneOf[0].oneOf[50] }, message: 'must match pattern "^pptx(\\+.+)?$"', schema: "^pptx(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[50], data: l };
                                                                                                          e === null ? e = [cd] : e.push(cd), r++;
                                                                                                        }
                                                                                                      } else {
                                                                                                        let cd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/50/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[50] }, message: "must be string", schema: p.oneOf[0].oneOf[50].type, parentSchema: p.oneOf[0].oneOf[50], data: l };
                                                                                                        e === null ? e = [cd] : e.push(cd), r++;
                                                                                                      }
                                                                                                      var m = ld === r;
                                                                                                      if (m && g)
                                                                                                        g = false, x = [x, 50];
                                                                                                      else {
                                                                                                        m && (g = true, x = 50);
                                                                                                        let cd = r, rO = r;
                                                                                                        if (typeof l == "string") {
                                                                                                          if (!c1.test(l)) {
                                                                                                            let pd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/51/pattern", keyword: "pattern", params: { pattern: "^revealjs(\\+.+)?$", schema: p.oneOf[0].oneOf[51] }, message: 'must match pattern "^revealjs(\\+.+)?$"', schema: "^revealjs(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[51], data: l };
                                                                                                            e === null ? e = [pd] : e.push(pd), r++;
                                                                                                          }
                                                                                                        } else {
                                                                                                          let pd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/51/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[51] }, message: "must be string", schema: p.oneOf[0].oneOf[51].type, parentSchema: p.oneOf[0].oneOf[51], data: l };
                                                                                                          e === null ? e = [pd] : e.push(pd), r++;
                                                                                                        }
                                                                                                        var m = cd === r;
                                                                                                        if (m && g)
                                                                                                          g = false, x = [x, 51];
                                                                                                        else {
                                                                                                          m && (g = true, x = 51);
                                                                                                          let pd = r, tO = r;
                                                                                                          if (typeof l == "string") {
                                                                                                            if (!p1.test(l)) {
                                                                                                              let ud = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/52/pattern", keyword: "pattern", params: { pattern: "^rst(\\+.+)?$", schema: p.oneOf[0].oneOf[52] }, message: 'must match pattern "^rst(\\+.+)?$"', schema: "^rst(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[52], data: l };
                                                                                                              e === null ? e = [ud] : e.push(ud), r++;
                                                                                                            }
                                                                                                          } else {
                                                                                                            let ud = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/52/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[52] }, message: "must be string", schema: p.oneOf[0].oneOf[52].type, parentSchema: p.oneOf[0].oneOf[52], data: l };
                                                                                                            e === null ? e = [ud] : e.push(ud), r++;
                                                                                                          }
                                                                                                          var m = pd === r;
                                                                                                          if (m && g)
                                                                                                            g = false, x = [x, 52];
                                                                                                          else {
                                                                                                            m && (g = true, x = 52);
                                                                                                            let ud = r, aO = r;
                                                                                                            if (typeof l == "string") {
                                                                                                              if (!u1.test(l)) {
                                                                                                                let dd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/53/pattern", keyword: "pattern", params: { pattern: "^rtf(\\+.+)?$", schema: p.oneOf[0].oneOf[53] }, message: 'must match pattern "^rtf(\\+.+)?$"', schema: "^rtf(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[53], data: l };
                                                                                                                e === null ? e = [dd] : e.push(dd), r++;
                                                                                                              }
                                                                                                            } else {
                                                                                                              let dd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/53/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[53] }, message: "must be string", schema: p.oneOf[0].oneOf[53].type, parentSchema: p.oneOf[0].oneOf[53], data: l };
                                                                                                              e === null ? e = [dd] : e.push(dd), r++;
                                                                                                            }
                                                                                                            var m = ud === r;
                                                                                                            if (m && g)
                                                                                                              g = false, x = [x, 53];
                                                                                                            else {
                                                                                                              m && (g = true, x = 53);
                                                                                                              let dd = r, nO = r;
                                                                                                              if (typeof l == "string") {
                                                                                                                if (!d1.test(l)) {
                                                                                                                  let fd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/54/pattern", keyword: "pattern", params: { pattern: "^s5(\\+.+)?$", schema: p.oneOf[0].oneOf[54] }, message: 'must match pattern "^s5(\\+.+)?$"', schema: "^s5(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[54], data: l };
                                                                                                                  e === null ? e = [fd] : e.push(fd), r++;
                                                                                                                }
                                                                                                              } else {
                                                                                                                let fd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/54/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[54] }, message: "must be string", schema: p.oneOf[0].oneOf[54].type, parentSchema: p.oneOf[0].oneOf[54], data: l };
                                                                                                                e === null ? e = [fd] : e.push(fd), r++;
                                                                                                              }
                                                                                                              var m = dd === r;
                                                                                                              if (m && g)
                                                                                                                g = false, x = [x, 54];
                                                                                                              else {
                                                                                                                m && (g = true, x = 54);
                                                                                                                let fd = r, oO = r;
                                                                                                                if (typeof l == "string") {
                                                                                                                  if (!f1.test(l)) {
                                                                                                                    let md = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/55/pattern", keyword: "pattern", params: { pattern: "^slideous(\\+.+)?$", schema: p.oneOf[0].oneOf[55] }, message: 'must match pattern "^slideous(\\+.+)?$"', schema: "^slideous(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[55], data: l };
                                                                                                                    e === null ? e = [md] : e.push(md), r++;
                                                                                                                  }
                                                                                                                } else {
                                                                                                                  let md = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/55/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[55] }, message: "must be string", schema: p.oneOf[0].oneOf[55].type, parentSchema: p.oneOf[0].oneOf[55], data: l };
                                                                                                                  e === null ? e = [md] : e.push(md), r++;
                                                                                                                }
                                                                                                                var m = fd === r;
                                                                                                                if (m && g)
                                                                                                                  g = false, x = [x, 55];
                                                                                                                else {
                                                                                                                  m && (g = true, x = 55);
                                                                                                                  let md = r, sO = r;
                                                                                                                  if (typeof l == "string") {
                                                                                                                    if (!m1.test(l)) {
                                                                                                                      let gd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/56/pattern", keyword: "pattern", params: { pattern: "^slidy(\\+.+)?$", schema: p.oneOf[0].oneOf[56] }, message: 'must match pattern "^slidy(\\+.+)?$"', schema: "^slidy(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[56], data: l };
                                                                                                                      e === null ? e = [gd] : e.push(gd), r++;
                                                                                                                    }
                                                                                                                  } else {
                                                                                                                    let gd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/56/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[56] }, message: "must be string", schema: p.oneOf[0].oneOf[56].type, parentSchema: p.oneOf[0].oneOf[56], data: l };
                                                                                                                    e === null ? e = [gd] : e.push(gd), r++;
                                                                                                                  }
                                                                                                                  var m = md === r;
                                                                                                                  if (m && g)
                                                                                                                    g = false, x = [x, 56];
                                                                                                                  else {
                                                                                                                    m && (g = true, x = 56);
                                                                                                                    let gd = r, iO = r;
                                                                                                                    if (typeof l == "string") {
                                                                                                                      if (!g1.test(l)) {
                                                                                                                        let hd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/57/pattern", keyword: "pattern", params: { pattern: "^tei(\\+.+)?$", schema: p.oneOf[0].oneOf[57] }, message: 'must match pattern "^tei(\\+.+)?$"', schema: "^tei(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[57], data: l };
                                                                                                                        e === null ? e = [hd] : e.push(hd), r++;
                                                                                                                      }
                                                                                                                    } else {
                                                                                                                      let hd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/57/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[57] }, message: "must be string", schema: p.oneOf[0].oneOf[57].type, parentSchema: p.oneOf[0].oneOf[57], data: l };
                                                                                                                      e === null ? e = [hd] : e.push(hd), r++;
                                                                                                                    }
                                                                                                                    var m = gd === r;
                                                                                                                    if (m && g)
                                                                                                                      g = false, x = [x, 57];
                                                                                                                    else {
                                                                                                                      m && (g = true, x = 57);
                                                                                                                      let hd = r, lO = r;
                                                                                                                      if (typeof l == "string") {
                                                                                                                        if (!h1.test(l)) {
                                                                                                                          let yd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/58/pattern", keyword: "pattern", params: { pattern: "^texinfo(\\+.+)?$", schema: p.oneOf[0].oneOf[58] }, message: 'must match pattern "^texinfo(\\+.+)?$"', schema: "^texinfo(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[58], data: l };
                                                                                                                          e === null ? e = [yd] : e.push(yd), r++;
                                                                                                                        }
                                                                                                                      } else {
                                                                                                                        let yd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/58/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[58] }, message: "must be string", schema: p.oneOf[0].oneOf[58].type, parentSchema: p.oneOf[0].oneOf[58], data: l };
                                                                                                                        e === null ? e = [yd] : e.push(yd), r++;
                                                                                                                      }
                                                                                                                      var m = hd === r;
                                                                                                                      if (m && g)
                                                                                                                        g = false, x = [x, 58];
                                                                                                                      else {
                                                                                                                        m && (g = true, x = 58);
                                                                                                                        let yd = r, cO = r;
                                                                                                                        if (typeof l == "string") {
                                                                                                                          if (!y1.test(l)) {
                                                                                                                            let bd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/59/pattern", keyword: "pattern", params: { pattern: "^textile(\\+.+)?$", schema: p.oneOf[0].oneOf[59] }, message: 'must match pattern "^textile(\\+.+)?$"', schema: "^textile(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[59], data: l };
                                                                                                                            e === null ? e = [bd] : e.push(bd), r++;
                                                                                                                          }
                                                                                                                        } else {
                                                                                                                          let bd = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/59/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[59] }, message: "must be string", schema: p.oneOf[0].oneOf[59].type, parentSchema: p.oneOf[0].oneOf[59], data: l };
                                                                                                                          e === null ? e = [bd] : e.push(bd), r++;
                                                                                                                        }
                                                                                                                        var m = yd === r;
                                                                                                                        if (m && g)
                                                                                                                          g = false, x = [x, 59];
                                                                                                                        else {
                                                                                                                          m && (g = true, x = 59);
                                                                                                                          let bd = r, pO = r;
                                                                                                                          if (typeof l == "string") {
                                                                                                                            if (!b1.test(l)) {
                                                                                                                              let _d = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/60/pattern", keyword: "pattern", params: { pattern: "^xwiki(\\+.+)?$", schema: p.oneOf[0].oneOf[60] }, message: 'must match pattern "^xwiki(\\+.+)?$"', schema: "^xwiki(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[60], data: l };
                                                                                                                              e === null ? e = [_d] : e.push(_d), r++;
                                                                                                                            }
                                                                                                                          } else {
                                                                                                                            let _d = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/60/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[60] }, message: "must be string", schema: p.oneOf[0].oneOf[60].type, parentSchema: p.oneOf[0].oneOf[60], data: l };
                                                                                                                            e === null ? e = [_d] : e.push(_d), r++;
                                                                                                                          }
                                                                                                                          var m = bd === r;
                                                                                                                          if (m && g)
                                                                                                                            g = false, x = [x, 60];
                                                                                                                          else {
                                                                                                                            m && (g = true, x = 60);
                                                                                                                            let _d = r, uO = r;
                                                                                                                            if (typeof l == "string") {
                                                                                                                              if (!_1.test(l)) {
                                                                                                                                let Ed = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/61/pattern", keyword: "pattern", params: { pattern: "^zimwiki(\\+.+)?$", schema: p.oneOf[0].oneOf[61] }, message: 'must match pattern "^zimwiki(\\+.+)?$"', schema: "^zimwiki(\\+.+)?$", parentSchema: p.oneOf[0].oneOf[61], data: l };
                                                                                                                                e === null ? e = [Ed] : e.push(Ed), r++;
                                                                                                                              }
                                                                                                                            } else {
                                                                                                                              let Ed = { instancePath: o, schemaPath: "#/oneOf/0/oneOf/61/type", keyword: "type", params: { type: "string", schema: p.oneOf[0].oneOf[61] }, message: "must be string", schema: p.oneOf[0].oneOf[61].type, parentSchema: p.oneOf[0].oneOf[61], data: l };
                                                                                                                              e === null ? e = [Ed] : e.push(Ed), r++;
                                                                                                                            }
                                                                                                                            var m = _d === r;
                                                                                                                            m && g ? (g = false, x = [x, 61]) : m && (g = true, x = 61);
                                                                                                                          }
                                                                                                                        }
                                                                                                                      }
                                                                                                                    }
                                                                                                                  }
                                                                                                                }
                                                                                                              }
                                                                                                            }
                                                                                                          }
                                                                                                        }
                                                                                                      }
                                                                                                    }
                                                                                                  }
                                                                                                }
                                                                                              }
                                                                                            }
                                                                                          }
                                                                                        }
                                                                                      }
                                                                                    }
                                                                                  }
                                                                                }
                                                                              }
                                                                            }
                                                                          }
                                                                        }
                                                                      }
                                                                    }
                                                                  }
                                                                }
                                                              }
                                                            }
                                                          }
                                                        }
                                                      }
                                                    }
                                                  }
                                                }
                                              }
                                            }
                                          }
                                        }
                                      }
                                    }
                                  }
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
    if (g)
      r = y, e !== null && (y ? e.length = y : e = null);
    else {
      let Ln = { instancePath: o, schemaPath: "#/oneOf/0/oneOf", keyword: "oneOf", params: { passingSchemas: x, schema: p.oneOf[0] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[0].oneOf, parentSchema: p.oneOf[0], data: l };
      e === null ? e = [Ln] : e.push(Ln), r++;
    }
    var V = u === r;
    V && (c = true, d = 0);
    let se = r, P = r;
    if (typeof l == "string") {
      if (!N0.test(l)) {
        let Ln = { instancePath: o, schemaPath: "#/oneOf/1/pattern", keyword: "pattern", params: { pattern: "^hugo(\\+.+)?$", schema: p.oneOf[1] }, message: 'must match pattern "^hugo(\\+.+)?$"', schema: "^hugo(\\+.+)?$", parentSchema: p.oneOf[1], data: l };
        e === null ? e = [Ln] : e.push(Ln), r++;
      }
    } else {
      let Ln = { instancePath: o, schemaPath: "#/oneOf/1/type", keyword: "type", params: { type: "string", schema: p.oneOf[1] }, message: "must be string", schema: p.oneOf[1].type, parentSchema: p.oneOf[1], data: l };
      e === null ? e = [Ln] : e.push(Ln), r++;
    }
    var V = se === r;
    if (V && c)
      c = false, d = [d, 1];
    else {
      V && (c = true, d = 1);
      let Ln = r, k1 = r, dq = r;
      if (l && typeof l == "object" && !Array.isArray(l)) {
        for (let i in l) {
          let t = r, mt = r;
          if (typeof i == "string") {
            if (!C0.test(i)) {
              let ne = { instancePath: o, schemaPath: "#/oneOf/2/allOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(commonmark-x|commonmarkX|jats-archiving|jatsArchiving|jats-articleauthoring|jatsArticleauthoring|jats-publishing|jatsPublishing|markdown-github|markdownGithub|markdown-mmd|markdownMmd|markdown-phpextra|markdownPhpextra|markdown-strict|markdownStrict))", schema: p.oneOf[2].allOf[0].propertyNames }, message: 'must match pattern "^(?!(commonmark-x|commonmarkX|jats-archiving|jatsArchiving|jats-articleauthoring|jatsArticleauthoring|jats-publishing|jatsPublishing|markdown-github|markdownGithub|markdown-mmd|markdownMmd|markdown-phpextra|markdownPhpextra|markdown-strict|markdownStrict))"', schema: "^(?!(commonmark-x|commonmarkX|jats-archiving|jatsArchiving|jats-articleauthoring|jatsArticleauthoring|jats-publishing|jatsPublishing|markdown-github|markdownGithub|markdown-mmd|markdownMmd|markdown-phpextra|markdownPhpextra|markdown-strict|markdownStrict))", parentSchema: p.oneOf[2].allOf[0].propertyNames, data: i, propertyName: i };
              e === null ? e = [ne] : e.push(ne), r++;
            }
          } else {
            let ne = { instancePath: o, schemaPath: "#/oneOf/2/allOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].propertyNames, data: i, propertyName: i };
            e === null ? e = [ne] : e.push(ne), r++;
          }
          var oe = t === r;
          if (!oe) {
            let ne = { instancePath: o, schemaPath: "#/oneOf/2/allOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: i, schema: p.oneOf[2].allOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0], data: l };
            e === null ? e = [ne] : e.push(ne), r++;
          }
        }
        let Gm = r;
        for (let i in l)
          if (!(dv.test(i) || fv.test(i) || mv.test(i) || gv.test(i) || hv.test(i) || yv.test(i) || bv.test(i) || _v.test(i) || vv.test(i) || qv.test(i) || Ov.test(i) || Pv.test(i) || xv.test(i) || kv.test(i) || Dv.test(i) || wv.test(i) || $v.test(i) || Sv.test(i) || Fv.test(i) || Av.test(i) || Bv.test(i) || jv.test(i) || Ev.test(i) || Nv.test(i) || Cv.test(i) || Iv.test(i) || Hv.test(i) || Lv.test(i) || Mv.test(i) || Tv.test(i) || zv.test(i) || Wv.test(i) || Rv.test(i) || Vv.test(i) || Uv.test(i) || Kv.test(i) || Jv.test(i) || Qv.test(i) || Gv.test(i) || Yv.test(i) || Xv.test(i) || Zv.test(i) || e1.test(i) || r1.test(i) || t1.test(i) || a1.test(i) || n1.test(i) || o1.test(i) || s1.test(i) || i1.test(i) || l1.test(i) || c1.test(i) || p1.test(i) || u1.test(i) || d1.test(i) || f1.test(i) || m1.test(i) || g1.test(i) || h1.test(i) || y1.test(i) || b1.test(i) || _1.test(i))) {
            let t = { instancePath: o, schemaPath: "#/oneOf/2/allOf/0/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: i, schema: p.oneOf[2].allOf[0] }, message: "must NOT have additional properties", schema: false, parentSchema: p.oneOf[2].allOf[0], data: l };
            e === null ? e = [t] : e.push(t), r++;
          }
        for (let i in l)
          if (dv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let ye in t) {
                let a = r, n = r;
                if (typeof ye == "string") {
                  if (!Xn.test(ye)) {
                    let oa = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Easciidoc(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf[0].propertyNames, data: ye, propertyName: ye };
                    e === null ? e = [oa] : e.push(oa), r++;
                  }
                } else {
                  let oa = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Easciidoc(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf[0].propertyNames, data: ye, propertyName: ye };
                  e === null ? e = [oa] : e.push(oa), r++;
                }
                var h = a === r;
                if (!h) {
                  let oa = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Easciidoc(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: ye, schema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [oa] : e.push(oa), r++;
                }
              }
              if (t.eval !== void 0) {
                let ye = t.eval, a = r;
                ue(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var f = a === r;
              }
              if (t.echo !== void 0) {
                let ye = t.echo, a = r;
                de(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var f = a === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let ye = t["tbl-colwidths"], a = r;
                be(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var f = a === r;
              }
              if (t.output !== void 0) {
                let ye = t.output, a = r;
                fe(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var f = a === r;
              }
              if (t.warning !== void 0) {
                let ye = t.warning, a = r;
                me(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var f = a === r;
              }
              if (t.error !== void 0) {
                let ye = t.error, a = r;
                ge(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var f = a === r;
              }
              if (t.include !== void 0) {
                let ye = t.include, a = r;
                he(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var f = a === r;
              }
              if (t.title !== void 0) {
                let ye = t.title, a = r;
                Ee(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var f = a === r;
              }
              if (t.date !== void 0) {
                let ye = t.date, a = r;
                Ne(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var f = a === r;
              }
              if (t.author !== void 0) {
                let ye = t.author, a = r;
                Ce(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var f = a === r;
              }
              if (t.abstract !== void 0) {
                let ye = t.abstract, a = r;
                ga(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var f = a === r;
              }
              if (t.crossref !== void 0) {
                let ye = t.crossref, a = r;
                Ie(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var f = a === r;
              }
              if (t.engine !== void 0) {
                let ye = t.engine, a = r;
                qe(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var f = a === r;
              }
              if (t.jupyter !== void 0) {
                let ye = t.jupyter, a = r;
                Oe(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var f = a === r;
              }
              if (t.knitr !== void 0) {
                let ye = t.knitr, a = r;
                Pe(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var f = a === r;
              }
              if (t.cache !== void 0) {
                let ye = t.cache, a = r;
                xe(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var f = a === r;
              }
              if (t.freeze !== void 0) {
                let ye = t.freeze, a = r;
                ke(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var f = a === r;
              }
              if (t.server !== void 0) {
                let ye = t.server, a = r;
                De(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var f = a === r;
              }
              if (t.daemon !== void 0) {
                let ye = t.daemon, a = r;
                we(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var f = a === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let ye = t["daemon-restart"], a = r;
                $e(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var f = a === r;
              }
              if (t.enabled !== void 0) {
                let ye = t.enabled, a = r;
                Se(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var f = a === r;
              }
              if (t.ipynb !== void 0) {
                let ye = t.ipynb, a = r;
                Fe(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var f = a === r;
              }
              if (t.debug !== void 0) {
                let ye = t.debug, a = r;
                Ae(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var f = a === r;
              }
              if (t["fig-width"] !== void 0) {
                let ye = t["fig-width"], a = r;
                He(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var f = a === r;
              }
              if (t["fig-height"] !== void 0) {
                let ye = t["fig-height"], a = r;
                Le(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var f = a === r;
              }
              if (t["fig-format"] !== void 0) {
                let ye = t["fig-format"], a = r;
                Me(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var f = a === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let ye = t["fig-dpi"], a = r;
                Te(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var f = a === r;
              }
              if (t.to !== void 0) {
                let ye = t.to, a = r;
                I(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var f = a === r;
              }
              if (t.writer !== void 0) {
                let ye = t.writer, a = r;
                I(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var f = a === r;
              }
              if (t["input-file"] !== void 0) {
                let ye = t["input-file"], a = r;
                ze(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var f = a === r;
              }
              if (t["input-files"] !== void 0) {
                let ye = t["input-files"], a = r;
                We(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var f = a === r;
              }
              if (t.defaults !== void 0) {
                let ye = t.defaults, a = r;
                Re(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var f = a === r;
              }
              if (t.variables !== void 0) {
                let ye = t.variables, a = r;
                Ve(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var f = a === r;
              }
              if (t.metadata !== void 0) {
                let ye = t.metadata, a = r;
                Ue(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var f = a === r;
              }
              if (t["request-headers"] !== void 0) {
                let ye = t["request-headers"], a = r;
                Ke(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var f = a === r;
              }
              if (t.trace !== void 0) {
                let ye = t.trace, a = r;
                Je(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var f = a === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let ye = t["fail-if-warnings"], a = r;
                Qe(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var f = a === r;
              }
              if (t["dump-args"] !== void 0) {
                let ye = t["dump-args"], a = r;
                Ge(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var f = a === r;
              }
              if (t["ignore-args"] !== void 0) {
                let ye = t["ignore-args"], a = r;
                Ye(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var f = a === r;
              }
              if (t["file-scope"] !== void 0) {
                let ye = t["file-scope"], a = r;
                Xe(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var f = a === r;
              }
              if (t["data-dir"] !== void 0) {
                let ye = t["data-dir"], a = r;
                Ze(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var f = a === r;
              }
              if (t.verbosity !== void 0) {
                let ye = t.verbosity, a = r;
                er(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var f = a === r;
              }
              if (t["log-file"] !== void 0) {
                let ye = t["log-file"], a = r;
                rr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var f = a === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let ye = t["strip-empty-paragraphs"], a = r;
                tr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var f = a === r;
              }
              if (t["output-divs"] !== void 0) {
                let ye = t["output-divs"], a = r;
                ar(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var f = a === r;
              }
              if (t["merge-includes"] !== void 0) {
                let ye = t["merge-includes"], a = r;
                nr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var f = a === r;
              }
              if (t["header-includes"] !== void 0) {
                let ye = t["header-includes"], a = r;
                Ir(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var f = a === r;
              }
              if (t["include-before"] !== void 0) {
                let ye = t["include-before"], a = r;
                Hr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var f = a === r;
              }
              if (t["include-after"] !== void 0) {
                let ye = t["include-after"], a = r;
                Lr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var f = a === r;
              }
              if (t["include-before-body"] !== void 0) {
                let ye = t["include-before-body"], a = r;
                Mr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var f = a === r;
              }
              if (t["include-after-body"] !== void 0) {
                let ye = t["include-after-body"], a = r;
                Tr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var f = a === r;
              }
              if (t["include-in-header"] !== void 0) {
                let ye = t["include-in-header"], a = r;
                zr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var f = a === r;
              }
              if (t["metadata-file"] !== void 0) {
                let ye = t["metadata-file"], a = r;
                or(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var f = a === r;
              }
              if (t["metadata-files"] !== void 0) {
                let ye = t["metadata-files"], a = r;
                sr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var f = a === r;
              }
              if (t.lang !== void 0) {
                let ye = t.lang, a = r;
                ir(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var f = a === r;
              }
              if (t.language !== void 0) {
                let ye = t.language, a = r;
                lr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var f = a === r;
              }
              if (t.dir !== void 0) {
                let ye = t.dir, a = r;
                cr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var f = a === r;
              }
              if (t.keywords !== void 0) {
                let ye = t.keywords, a = r;
                Ia(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keywords", parentData: t, parentDataProperty: "keywords", rootData: s }) || (e = e === null ? Ia.errors : e.concat(Ia.errors), r = e.length);
                var f = a === r;
              }
              if (t["number-sections"] !== void 0) {
                let ye = t["number-sections"], a = r;
                pr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var f = a === r;
              }
              if (t["number-offset"] !== void 0) {
                let ye = t["number-offset"], a = r;
                ur(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var f = a === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let ye = t["shift-heading-level-by"], a = r;
                dr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var f = a === r;
              }
              if (t.bibliography !== void 0) {
                let ye = t.bibliography, a = r;
                fr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var f = a === r;
              }
              if (t.csl !== void 0) {
                let ye = t.csl, a = r;
                mr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var f = a === r;
              }
              if (t.citeproc !== void 0) {
                let ye = t.citeproc, a = r;
                gr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var f = a === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let ye = t["citation-abbreviations"], a = r;
                hr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var f = a === r;
              }
              if (t.from !== void 0) {
                let ye = t.from, a = r;
                H(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var f = a === r;
              }
              if (t.reader !== void 0) {
                let ye = t.reader, a = r;
                H(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var f = a === r;
              }
              if (t["output-file"] !== void 0) {
                let ye = t["output-file"], a = r;
                yr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var f = a === r;
              }
              if (t["output-ext"] !== void 0) {
                let ye = t["output-ext"], a = r;
                br(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var f = a === r;
              }
              if (t.template !== void 0) {
                let ye = t.template, a = r;
                Sr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var f = a === r;
              }
              if (t.standalone !== void 0) {
                let ye = t.standalone, a = r;
                _r(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var f = a === r;
              }
              if (t.filters !== void 0) {
                let ye = t.filters, a = r;
                vr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var f = a === r;
              }
              if (t["keep-md"] !== void 0) {
                let ye = t["keep-md"], a = r;
                Be(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var f = a === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let ye = t["keep-ipynb"], a = r;
                je(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var f = a === r;
              }
              if (t["extract-media"] !== void 0) {
                let ye = t["extract-media"], a = r;
                qr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var f = a === r;
              }
              if (t["resource-path"] !== void 0) {
                let ye = t["resource-path"], a = r;
                Or(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var f = a === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let ye = t["default-image-extension"], a = r;
                Pr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var f = a === r;
              }
              if (t.abbreviations !== void 0) {
                let ye = t.abbreviations, a = r;
                xr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var f = a === r;
              }
              if (t.dpi !== void 0) {
                let ye = t.dpi, a = r;
                kr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var f = a === r;
              }
              if (t.wrap !== void 0) {
                let ye = t.wrap, a = r;
                it(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var f = a === r;
              }
              if (t.columns !== void 0) {
                let ye = t.columns, a = r;
                lt(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var f = a === r;
              }
              if (t["tab-stop"] !== void 0) {
                let ye = t["tab-stop"], a = r;
                ct(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var f = a === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let ye = t["preserve-tabs"], a = r;
                pt(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var f = a === r;
              }
              if (t.eol !== void 0) {
                let ye = t.eol, a = r;
                ut(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var f = a === r;
              }
              if (t.toc !== void 0) {
                let ye = t.toc, a = r;
                K(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var f = a === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let ye = t["table-of-contents"], a = r;
                K(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var f = a === r;
              }
              if (t["toc-depth"] !== void 0) {
                let ye = t["toc-depth"], a = r;
                Wr(ye, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var f = a === r;
              }
            } else {
              let ye = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Easciidoc(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [ye] : e.push(ye), r++;
            }
            var ft = ht === r;
            ft && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let ye = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Easciidoc(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [ye] : e.push(ye), r++;
            }
            var ft = gt === r;
            if (ft && L ? (L = false, M = [M, 1]) : ft && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let ye = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Easciidoc(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^asciidoc(\\+.+)?$"], data: t };
              e === null ? e = [ye] : e.push(ye), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (fv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Easciidoctor(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Easciidoctor(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var Dr = n === r;
                if (!Dr) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Easciidoctor(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var wr = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var wr = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var wr = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var wr = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var wr = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var wr = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var wr = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var wr = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var wr = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var wr = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var wr = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var wr = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var wr = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var wr = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var wr = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var wr = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var wr = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var wr = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var wr = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var wr = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var wr = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var wr = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var wr = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var wr = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var wr = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var wr = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var wr = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var wr = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var wr = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var wr = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var wr = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var wr = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var wr = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var wr = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var wr = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var wr = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var wr = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var wr = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var wr = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var wr = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var wr = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var wr = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var wr = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var wr = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var wr = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var wr = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var wr = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var wr = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var wr = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var wr = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var wr = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var wr = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var wr = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var wr = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var wr = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var wr = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var wr = n === r;
              }
              if (t.keywords !== void 0) {
                let a = t.keywords, n = r;
                Ia(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keywords", parentData: t, parentDataProperty: "keywords", rootData: s }) || (e = e === null ? Ia.errors : e.concat(Ia.errors), r = e.length);
                var wr = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var wr = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var wr = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var wr = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var wr = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var wr = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var wr = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var wr = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var wr = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var wr = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var wr = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var wr = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var wr = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var wr = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var wr = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var wr = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var wr = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var wr = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var wr = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var wr = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var wr = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var wr = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var wr = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var wr = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var wr = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var wr = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var wr = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var wr = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var wr = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var wr = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Easciidoctor(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var ta = ht === r;
            ta && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Easciidoctor(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var ta = gt === r;
            if (ta && L ? (L = false, M = [M, 1]) : ta && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Easciidoctor(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^asciidoctor(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (mv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!I0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebeamer(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|links_as_notes|linksAsNotes|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|latex_auto_mk|latexAutoMk|latex_auto_install|latexAutoInstall|latex_min_runs|latexMinRuns|latex_max_runs|latexMaxRuns|latex_clean|latexClean|latex_makeindex|latexMakeindex|latex_makeindex_opts|latexMakeindexOpts|latex_tlmgr_opts|latexTlmgrOpts|latex_output_dir|latexOutputDir|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|title_meta|titleMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|section_titles|sectionTitles|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|keep_tex|keepTex|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", schema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|links_as_notes|linksAsNotes|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|latex_auto_mk|latexAutoMk|latex_auto_install|latexAutoInstall|latex_min_runs|latexMinRuns|latex_max_runs|latexMaxRuns|latex_clean|latexClean|latex_makeindex|latexMakeindex|latex_makeindex_opts|latexMakeindexOpts|latex_tlmgr_opts|latexTlmgrOpts|latex_output_dir|latexOutputDir|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|title_meta|titleMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|section_titles|sectionTitles|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|keep_tex|keepTex|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))"', schema: "^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|links_as_notes|linksAsNotes|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|latex_auto_mk|latexAutoMk|latex_auto_install|latexAutoInstall|latex_min_runs|latexMinRuns|latex_max_runs|latexMaxRuns|latex_clean|latexClean|latex_makeindex|latexMakeindex|latex_makeindex_opts|latexMakeindexOpts|latex_tlmgr_opts|latexTlmgrOpts|latex_output_dir|latexOutputDir|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|title_meta|titleMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|section_titles|sectionTitles|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|keep_tex|keepTex|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebeamer(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var jt = n === r;
                if (!jt) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebeamer(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var ee = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var ee = n === r;
              }
              if (t["code-line-numbers"] !== void 0) {
                let a = t["code-line-numbers"], n = r;
                ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-line-numbers", parentData: t, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
                var ee = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var ee = n === r;
              }
              if (t["fig-env"] !== void 0) {
                let a = t["fig-env"], n = r;
                Zn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-env", parentData: t, parentDataProperty: "fig-env", rootData: s }) || (e = e === null ? Zn.errors : e.concat(Zn.errors), r = e.length);
                var ee = n === r;
              }
              if (t["fig-pos"] !== void 0) {
                let a = t["fig-pos"], n = r;
                eo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-pos", parentData: t, parentDataProperty: "fig-pos", rootData: s }) || (e = e === null ? eo.errors : e.concat(eo.errors), r = e.length);
                var ee = n === r;
              }
              if (t["cap-location"] !== void 0) {
                let a = t["cap-location"], n = r;
                ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cap-location", parentData: t, parentDataProperty: "cap-location", rootData: s }) || (e = e === null ? ja.errors : e.concat(ja.errors), r = e.length);
                var ee = n === r;
              }
              if (t["fig-cap-location"] !== void 0) {
                let a = t["fig-cap-location"], n = r;
                Ea(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-cap-location", parentData: t, parentDataProperty: "fig-cap-location", rootData: s }) || (e = e === null ? Ea.errors : e.concat(Ea.errors), r = e.length);
                var ee = n === r;
              }
              if (t["tbl-cap-location"] !== void 0) {
                let a = t["tbl-cap-location"], n = r;
                Na(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-cap-location", parentData: t, parentDataProperty: "tbl-cap-location", rootData: s }) || (e = e === null ? Na.errors : e.concat(Na.errors), r = e.length);
                var ee = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var ee = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var ee = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var ee = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var ee = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var ee = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var ee = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var ee = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var ee = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var ee = n === r;
              }
              if (t.institute !== void 0) {
                let a = t.institute, n = r;
                ro(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/institute", parentData: t, parentDataProperty: "institute", rootData: s }) || (e = e === null ? ro.errors : e.concat(ro.errors), r = e.length);
                var ee = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var ee = n === r;
              }
              if (t.thanks !== void 0) {
                let a = t.thanks, n = r;
                Qo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/thanks", parentData: t, parentDataProperty: "thanks", rootData: s }) || (e = e === null ? Qo.errors : e.concat(Qo.errors), r = e.length);
                var ee = n === r;
              }
              if (t["highlight-style"] !== void 0) {
                let a = t["highlight-style"], n = r;
                _a(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/highlight-style", parentData: t, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
                var ee = n === r;
              }
              if (t["syntax-definition"] !== void 0) {
                let a = t["syntax-definition"], n = r;
                va(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definition", parentData: t, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
                var ee = n === r;
              }
              if (t["syntax-definitions"] !== void 0) {
                let a = t["syntax-definitions"], n = r;
                qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definitions", parentData: t, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
                var ee = n === r;
              }
              if (t.listings !== void 0) {
                let a = t.listings, n = r;
                Yo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/listings", parentData: t, parentDataProperty: "listings", rootData: s }) || (e = e === null ? Yo.errors : e.concat(Yo.errors), r = e.length);
                var ee = n === r;
              }
              if (t["indented-code-classes"] !== void 0) {
                let a = t["indented-code-classes"], n = r;
                Oa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indented-code-classes", parentData: t, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
                var ee = n === r;
              }
              if (t.linkcolor !== void 0) {
                let a = t.linkcolor, n = r;
                Ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linkcolor", parentData: t, parentDataProperty: "linkcolor", rootData: s }) || (e = e === null ? Ua.errors : e.concat(Ua.errors), r = e.length);
                var ee = n === r;
              }
              if (t.filecolor !== void 0) {
                let a = t.filecolor, n = r;
                Xo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filecolor", parentData: t, parentDataProperty: "filecolor", rootData: s }) || (e = e === null ? Xo.errors : e.concat(Xo.errors), r = e.length);
                var ee = n === r;
              }
              if (t.citecolor !== void 0) {
                let a = t.citecolor, n = r;
                Zo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citecolor", parentData: t, parentDataProperty: "citecolor", rootData: s }) || (e = e === null ? Zo.errors : e.concat(Zo.errors), r = e.length);
                var ee = n === r;
              }
              if (t.urlcolor !== void 0) {
                let a = t.urlcolor, n = r;
                es(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/urlcolor", parentData: t, parentDataProperty: "urlcolor", rootData: s }) || (e = e === null ? es.errors : e.concat(es.errors), r = e.length);
                var ee = n === r;
              }
              if (t.toccolor !== void 0) {
                let a = t.toccolor, n = r;
                rs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toccolor", parentData: t, parentDataProperty: "toccolor", rootData: s }) || (e = e === null ? rs.errors : e.concat(rs.errors), r = e.length);
                var ee = n === r;
              }
              if (t.colorlinks !== void 0) {
                let a = t.colorlinks, n = r;
                ts(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/colorlinks", parentData: t, parentDataProperty: "colorlinks", rootData: s }) || (e = e === null ? ts.errors : e.concat(ts.errors), r = e.length);
                var ee = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var ee = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var ee = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var ee = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var ee = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var ee = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var ee = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var ee = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var ee = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var ee = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var ee = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var ee = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var ee = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var ee = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var ee = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var ee = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var ee = n === r;
              }
              if (t.mainfont !== void 0) {
                let a = t.mainfont, n = r;
                Ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mainfont", parentData: t, parentDataProperty: "mainfont", rootData: s }) || (e = e === null ? Ja.errors : e.concat(Ja.errors), r = e.length);
                var ee = n === r;
              }
              if (t.monofont !== void 0) {
                let a = t.monofont, n = r;
                Qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monofont", parentData: t, parentDataProperty: "monofont", rootData: s }) || (e = e === null ? Qa.errors : e.concat(Qa.errors), r = e.length);
                var ee = n === r;
              }
              if (t.fontsize !== void 0) {
                let a = t.fontsize, n = r;
                Ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontsize", parentData: t, parentDataProperty: "fontsize", rootData: s }) || (e = e === null ? Ga.errors : e.concat(Ga.errors), r = e.length);
                var ee = n === r;
              }
              if (t.fontenc !== void 0) {
                let a = t.fontenc, n = r;
                _s(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontenc", parentData: t, parentDataProperty: "fontenc", rootData: s }) || (e = e === null ? _s.errors : e.concat(_s.errors), r = e.length);
                var ee = n === r;
              }
              if (t.fontfamily !== void 0) {
                let a = t.fontfamily, n = r;
                Bo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontfamily", parentData: t, parentDataProperty: "fontfamily", rootData: s }) || (e = e === null ? Bo.errors : e.concat(Bo.errors), r = e.length);
                var ee = n === r;
              }
              if (t.fontfamilyoptions !== void 0) {
                let a = t.fontfamilyoptions, n = r;
                vs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontfamilyoptions", parentData: t, parentDataProperty: "fontfamilyoptions", rootData: s }) || (e = e === null ? vs.errors : e.concat(vs.errors), r = e.length);
                var ee = n === r;
              }
              if (t.sansfont !== void 0) {
                let a = t.sansfont, n = r;
                qs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/sansfont", parentData: t, parentDataProperty: "sansfont", rootData: s }) || (e = e === null ? qs.errors : e.concat(qs.errors), r = e.length);
                var ee = n === r;
              }
              if (t.mathfont !== void 0) {
                let a = t.mathfont, n = r;
                Os(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mathfont", parentData: t, parentDataProperty: "mathfont", rootData: s }) || (e = e === null ? Os.errors : e.concat(Os.errors), r = e.length);
                var ee = n === r;
              }
              if (t.CJKmainfont !== void 0) {
                let a = t.CJKmainfont, n = r;
                Ps(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/CJKmainfont", parentData: t, parentDataProperty: "CJKmainfont", rootData: s }) || (e = e === null ? Ps.errors : e.concat(Ps.errors), r = e.length);
                var ee = n === r;
              }
              if (t.mainfontoptions !== void 0) {
                let a = t.mainfontoptions, n = r;
                xs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mainfontoptions", parentData: t, parentDataProperty: "mainfontoptions", rootData: s }) || (e = e === null ? xs.errors : e.concat(xs.errors), r = e.length);
                var ee = n === r;
              }
              if (t.sansfontoptions !== void 0) {
                let a = t.sansfontoptions, n = r;
                ks(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/sansfontoptions", parentData: t, parentDataProperty: "sansfontoptions", rootData: s }) || (e = e === null ? ks.errors : e.concat(ks.errors), r = e.length);
                var ee = n === r;
              }
              if (t.monofontoptions !== void 0) {
                let a = t.monofontoptions, n = r;
                Ds(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monofontoptions", parentData: t, parentDataProperty: "monofontoptions", rootData: s }) || (e = e === null ? Ds.errors : e.concat(Ds.errors), r = e.length);
                var ee = n === r;
              }
              if (t.mathfontoptions !== void 0) {
                let a = t.mathfontoptions, n = r;
                ws(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mathfontoptions", parentData: t, parentDataProperty: "mathfontoptions", rootData: s }) || (e = e === null ? ws.errors : e.concat(ws.errors), r = e.length);
                var ee = n === r;
              }
              if (t.CJKoptions !== void 0) {
                let a = t.CJKoptions, n = r;
                $s(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/CJKoptions", parentData: t, parentDataProperty: "CJKoptions", rootData: s }) || (e = e === null ? $s.errors : e.concat($s.errors), r = e.length);
                var ee = n === r;
              }
              if (t.microtypeoptions !== void 0) {
                let a = t.microtypeoptions, n = r;
                Ss(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/microtypeoptions", parentData: t, parentDataProperty: "microtypeoptions", rootData: s }) || (e = e === null ? Ss.errors : e.concat(Ss.errors), r = e.length);
                var ee = n === r;
              }
              if (t.linestretch !== void 0) {
                let a = t.linestretch, n = r;
                Ya(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linestretch", parentData: t, parentDataProperty: "linestretch", rootData: s }) || (e = e === null ? Ya.errors : e.concat(Ya.errors), r = e.length);
                var ee = n === r;
              }
              if (t["links-as-notes"] !== void 0) {
                let a = t["links-as-notes"], n = r;
                Fs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/links-as-notes", parentData: t, parentDataProperty: "links-as-notes", rootData: s }) || (e = e === null ? Fs.errors : e.concat(Fs.errors), r = e.length);
                var ee = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var ee = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var ee = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var ee = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var ee = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var ee = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var ee = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var ee = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var ee = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var ee = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var ee = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var ee = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var ee = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var ee = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var ee = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var ee = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var ee = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var ee = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var ee = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var ee = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var ee = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var ee = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var ee = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var ee = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var ee = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var ee = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var ee = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var ee = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var ee = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var ee = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var ee = n === r;
              }
              if (t["latex-auto-mk"] !== void 0) {
                let a = t["latex-auto-mk"], n = r;
                El(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-auto-mk", parentData: t, parentDataProperty: "latex-auto-mk", rootData: s }) || (e = e === null ? El.errors : e.concat(El.errors), r = e.length);
                var ee = n === r;
              }
              if (t["latex-auto-install"] !== void 0) {
                let a = t["latex-auto-install"], n = r;
                Nl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-auto-install", parentData: t, parentDataProperty: "latex-auto-install", rootData: s }) || (e = e === null ? Nl.errors : e.concat(Nl.errors), r = e.length);
                var ee = n === r;
              }
              if (t["latex-min-runs"] !== void 0) {
                let a = t["latex-min-runs"], n = r;
                Cl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-min-runs", parentData: t, parentDataProperty: "latex-min-runs", rootData: s }) || (e = e === null ? Cl.errors : e.concat(Cl.errors), r = e.length);
                var ee = n === r;
              }
              if (t["latex-max-runs"] !== void 0) {
                let a = t["latex-max-runs"], n = r;
                Il(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-max-runs", parentData: t, parentDataProperty: "latex-max-runs", rootData: s }) || (e = e === null ? Il.errors : e.concat(Il.errors), r = e.length);
                var ee = n === r;
              }
              if (t["latex-clean"] !== void 0) {
                let a = t["latex-clean"], n = r;
                Hl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-clean", parentData: t, parentDataProperty: "latex-clean", rootData: s }) || (e = e === null ? Hl.errors : e.concat(Hl.errors), r = e.length);
                var ee = n === r;
              }
              if (t["latex-makeindex"] !== void 0) {
                let a = t["latex-makeindex"], n = r;
                Ll(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-makeindex", parentData: t, parentDataProperty: "latex-makeindex", rootData: s }) || (e = e === null ? Ll.errors : e.concat(Ll.errors), r = e.length);
                var ee = n === r;
              }
              if (t["latex-makeindex-opts"] !== void 0) {
                let a = t["latex-makeindex-opts"], n = r;
                Ml(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-makeindex-opts", parentData: t, parentDataProperty: "latex-makeindex-opts", rootData: s }) || (e = e === null ? Ml.errors : e.concat(Ml.errors), r = e.length);
                var ee = n === r;
              }
              if (t["latex-tlmgr-opts"] !== void 0) {
                let a = t["latex-tlmgr-opts"], n = r;
                Tl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-tlmgr-opts", parentData: t, parentDataProperty: "latex-tlmgr-opts", rootData: s }) || (e = e === null ? Tl.errors : e.concat(Tl.errors), r = e.length);
                var ee = n === r;
              }
              if (t["latex-output-dir"] !== void 0) {
                let a = t["latex-output-dir"], n = r;
                zl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-output-dir", parentData: t, parentDataProperty: "latex-output-dir", rootData: s }) || (e = e === null ? zl.errors : e.concat(zl.errors), r = e.length);
                var ee = n === r;
              }
              if (t.documentclass !== void 0) {
                let a = t.documentclass, n = r;
                js(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/documentclass", parentData: t, parentDataProperty: "documentclass", rootData: s }) || (e = e === null ? js.errors : e.concat(js.errors), r = e.length);
                var ee = n === r;
              }
              if (t.classoption !== void 0) {
                let a = t.classoption, n = r;
                un(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/classoption", parentData: t, parentDataProperty: "classoption", rootData: s }) || (e = e === null ? un.errors : e.concat(un.errors), r = e.length);
                var ee = n === r;
              }
              if (t.pagestyle !== void 0) {
                let a = t.pagestyle, n = r;
                Es(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pagestyle", parentData: t, parentDataProperty: "pagestyle", rootData: s }) || (e = e === null ? Es.errors : e.concat(Es.errors), r = e.length);
                var ee = n === r;
              }
              if (t.papersize !== void 0) {
                let a = t.papersize, n = r;
                Ns(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/papersize", parentData: t, parentDataProperty: "papersize", rootData: s }) || (e = e === null ? Ns.errors : e.concat(Ns.errors), r = e.length);
                var ee = n === r;
              }
              if (t["margin-left"] !== void 0) {
                let a = t["margin-left"], n = r;
                Xa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-left", parentData: t, parentDataProperty: "margin-left", rootData: s }) || (e = e === null ? Xa.errors : e.concat(Xa.errors), r = e.length);
                var ee = n === r;
              }
              if (t["margin-right"] !== void 0) {
                let a = t["margin-right"], n = r;
                Za(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-right", parentData: t, parentDataProperty: "margin-right", rootData: s }) || (e = e === null ? Za.errors : e.concat(Za.errors), r = e.length);
                var ee = n === r;
              }
              if (t["margin-top"] !== void 0) {
                let a = t["margin-top"], n = r;
                en(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-top", parentData: t, parentDataProperty: "margin-top", rootData: s }) || (e = e === null ? en.errors : e.concat(en.errors), r = e.length);
                var ee = n === r;
              }
              if (t["margin-bottom"] !== void 0) {
                let a = t["margin-bottom"], n = r;
                rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-bottom", parentData: t, parentDataProperty: "margin-bottom", rootData: s }) || (e = e === null ? rn.errors : e.concat(rn.errors), r = e.length);
                var ee = n === r;
              }
              if (t.geometry !== void 0) {
                let a = t.geometry, n = r;
                Hs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/geometry", parentData: t, parentDataProperty: "geometry", rootData: s }) || (e = e === null ? Hs.errors : e.concat(Hs.errors), r = e.length);
                var ee = n === r;
              }
              if (t.hyperrefoptions !== void 0) {
                let a = t.hyperrefoptions, n = r;
                Ls(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/hyperrefoptions", parentData: t, parentDataProperty: "hyperrefoptions", rootData: s }) || (e = e === null ? Ls.errors : e.concat(Ls.errors), r = e.length);
                var ee = n === r;
              }
              if (t.indent !== void 0) {
                let a = t.indent, n = r;
                Eo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indent", parentData: t, parentDataProperty: "indent", rootData: s }) || (e = e === null ? Eo.errors : e.concat(Eo.errors), r = e.length);
                var ee = n === r;
              }
              if (t["block-headings"] !== void 0) {
                let a = t["block-headings"], n = r;
                Ms(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/block-headings", parentData: t, parentDataProperty: "block-headings", rootData: s }) || (e = e === null ? Ms.errors : e.concat(Ms.errors), r = e.length);
                var ee = n === r;
              }
              if (t.keywords !== void 0) {
                let a = t.keywords, n = r;
                Ia(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keywords", parentData: t, parentDataProperty: "keywords", rootData: s }) || (e = e === null ? Ia.errors : e.concat(Ia.errors), r = e.length);
                var ee = n === r;
              }
              if (t.subject !== void 0) {
                let a = t.subject, n = r;
                fo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subject", parentData: t, parentDataProperty: "subject", rootData: s }) || (e = e === null ? fo.errors : e.concat(fo.errors), r = e.length);
                var ee = n === r;
              }
              if (t["title-meta"] !== void 0) {
                let a = t["title-meta"], n = r;
                Ts(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title-meta", parentData: t, parentDataProperty: "title-meta", rootData: s }) || (e = e === null ? Ts.errors : e.concat(Ts.errors), r = e.length);
                var ee = n === r;
              }
              if (t["author-meta"] !== void 0) {
                let a = t["author-meta"], n = r;
                dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author-meta", parentData: t, parentDataProperty: "author-meta", rootData: s }) || (e = e === null ? dn.errors : e.concat(dn.errors), r = e.length);
                var ee = n === r;
              }
              if (t["date-meta"] !== void 0) {
                let a = t["date-meta"], n = r;
                Ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date-meta", parentData: t, parentDataProperty: "date-meta", rootData: s }) || (e = e === null ? Ha.errors : e.concat(Ha.errors), r = e.length);
                var ee = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var ee = n === r;
              }
              if (t["number-depth"] !== void 0) {
                let a = t["number-depth"], n = r;
                Da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-depth", parentData: t, parentDataProperty: "number-depth", rootData: s }) || (e = e === null ? Da.errors : e.concat(Da.errors), r = e.length);
                var ee = n === r;
              }
              if (t.secnumdepth !== void 0) {
                let a = t.secnumdepth, n = r;
                zs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/secnumdepth", parentData: t, parentDataProperty: "secnumdepth", rootData: s }) || (e = e === null ? zs.errors : e.concat(zs.errors), r = e.length);
                var ee = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var ee = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var ee = n === r;
              }
              if (t["top-level-division"] !== void 0) {
                let a = t["top-level-division"], n = r;
                Bn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/top-level-division", parentData: t, parentDataProperty: "top-level-division", rootData: s }) || (e = e === null ? Bn.errors : e.concat(Bn.errors), r = e.length);
                var ee = n === r;
              }
              if (t["pdf-engine"] !== void 0) {
                let a = t["pdf-engine"], n = r;
                mo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pdf-engine", parentData: t, parentDataProperty: "pdf-engine", rootData: s }) || (e = e === null ? mo.errors : e.concat(mo.errors), r = e.length);
                var ee = n === r;
              }
              if (t["pdf-engine-opt"] !== void 0) {
                let a = t["pdf-engine-opt"], n = r;
                go(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pdf-engine-opt", parentData: t, parentDataProperty: "pdf-engine-opt", rootData: s }) || (e = e === null ? go.errors : e.concat(go.errors), r = e.length);
                var ee = n === r;
              }
              if (t.beamerarticle !== void 0) {
                let a = t.beamerarticle, n = r;
                Mc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/beamerarticle", parentData: t, parentDataProperty: "beamerarticle", rootData: s }) || (e = e === null ? Mc.errors : e.concat(Mc.errors), r = e.length);
                var ee = n === r;
              }
              if (t.beameroption !== void 0) {
                let a = t.beameroption, n = r;
                Tc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/beameroption", parentData: t, parentDataProperty: "beameroption", rootData: s }) || (e = e === null ? Tc.errors : e.concat(Tc.errors), r = e.length);
                var ee = n === r;
              }
              if (t.aspectratio !== void 0) {
                let a = t.aspectratio, n = r;
                zc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/aspectratio", parentData: t, parentDataProperty: "aspectratio", rootData: s }) || (e = e === null ? zc.errors : e.concat(zc.errors), r = e.length);
                var ee = n === r;
              }
              if (t.logo !== void 0) {
                let a = t.logo, n = r;
                em(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/logo", parentData: t, parentDataProperty: "logo", rootData: s }) || (e = e === null ? em.errors : e.concat(em.errors), r = e.length);
                var ee = n === r;
              }
              if (t.titlegraphic !== void 0) {
                let a = t.titlegraphic, n = r;
                Wc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/titlegraphic", parentData: t, parentDataProperty: "titlegraphic", rootData: s }) || (e = e === null ? Wc.errors : e.concat(Wc.errors), r = e.length);
                var ee = n === r;
              }
              if (t.navigation !== void 0) {
                let a = t.navigation, n = r;
                Rc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/navigation", parentData: t, parentDataProperty: "navigation", rootData: s }) || (e = e === null ? Rc.errors : e.concat(Rc.errors), r = e.length);
                var ee = n === r;
              }
              if (t["section-titles"] !== void 0) {
                let a = t["section-titles"], n = r;
                Vc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/section-titles", parentData: t, parentDataProperty: "section-titles", rootData: s }) || (e = e === null ? Vc.errors : e.concat(Vc.errors), r = e.length);
                var ee = n === r;
              }
              if (t.colortheme !== void 0) {
                let a = t.colortheme, n = r;
                Uc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/colortheme", parentData: t, parentDataProperty: "colortheme", rootData: s }) || (e = e === null ? Uc.errors : e.concat(Uc.errors), r = e.length);
                var ee = n === r;
              }
              if (t.fonttheme !== void 0) {
                let a = t.fonttheme, n = r;
                Kc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fonttheme", parentData: t, parentDataProperty: "fonttheme", rootData: s }) || (e = e === null ? Kc.errors : e.concat(Kc.errors), r = e.length);
                var ee = n === r;
              }
              if (t.innertheme !== void 0) {
                let a = t.innertheme, n = r;
                Jc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/innertheme", parentData: t, parentDataProperty: "innertheme", rootData: s }) || (e = e === null ? Jc.errors : e.concat(Jc.errors), r = e.length);
                var ee = n === r;
              }
              if (t.outertheme !== void 0) {
                let a = t.outertheme, n = r;
                Qc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/outertheme", parentData: t, parentDataProperty: "outertheme", rootData: s }) || (e = e === null ? Qc.errors : e.concat(Qc.errors), r = e.length);
                var ee = n === r;
              }
              if (t.themeoptions !== void 0) {
                let a = t.themeoptions, n = r;
                Gc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/themeoptions", parentData: t, parentDataProperty: "themeoptions", rootData: s }) || (e = e === null ? Gc.errors : e.concat(Gc.errors), r = e.length);
                var ee = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var ee = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var ee = n === r;
              }
              if (t["cite-method"] !== void 0) {
                let a = t["cite-method"], n = r;
                Ks(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cite-method", parentData: t, parentDataProperty: "cite-method", rootData: s }) || (e = e === null ? Ks.errors : e.concat(Ks.errors), r = e.length);
                var ee = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var ee = n === r;
              }
              if (t.biblatexoptions !== void 0) {
                let a = t.biblatexoptions, n = r;
                Js(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/biblatexoptions", parentData: t, parentDataProperty: "biblatexoptions", rootData: s }) || (e = e === null ? Js.errors : e.concat(Js.errors), r = e.length);
                var ee = n === r;
              }
              if (t.natbiboptions !== void 0) {
                let a = t.natbiboptions, n = r;
                Qs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/natbiboptions", parentData: t, parentDataProperty: "natbiboptions", rootData: s }) || (e = e === null ? Qs.errors : e.concat(Qs.errors), r = e.length);
                var ee = n === r;
              }
              if (t["biblio-style"] !== void 0) {
                let a = t["biblio-style"], n = r;
                Gs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/biblio-style", parentData: t, parentDataProperty: "biblio-style", rootData: s }) || (e = e === null ? Gs.errors : e.concat(Gs.errors), r = e.length);
                var ee = n === r;
              }
              if (t["biblio-title"] !== void 0) {
                let a = t["biblio-title"], n = r;
                Ys(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/biblio-title", parentData: t, parentDataProperty: "biblio-title", rootData: s }) || (e = e === null ? Ys.errors : e.concat(Ys.errors), r = e.length);
                var ee = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var ee = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var ee = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var ee = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var ee = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var ee = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var ee = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var ee = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var ee = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var ee = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var ee = n === r;
              }
              if (t["keep-tex"] !== void 0) {
                let a = t["keep-tex"], n = r;
                Vl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-tex", parentData: t, parentDataProperty: "keep-tex", rootData: s }) || (e = e === null ? Vl.errors : e.concat(Vl.errors), r = e.length);
                var ee = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var ee = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var ee = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var ee = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var ee = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var ee = n === r;
              }
              if (t.incremental !== void 0) {
                let a = t.incremental, n = r;
                Rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/incremental", parentData: t, parentDataProperty: "incremental", rootData: s }) || (e = e === null ? Rn.errors : e.concat(Rn.errors), r = e.length);
                var ee = n === r;
              }
              if (t["slide-level"] !== void 0) {
                let a = t["slide-level"], n = r;
                Vn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/slide-level", parentData: t, parentDataProperty: "slide-level", rootData: s }) || (e = e === null ? Vn.errors : e.concat(Vn.errors), r = e.length);
                var ee = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var ee = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var ee = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var ee = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var ee = n === r;
              }
              if (t["toc-title"] !== void 0) {
                let a = t["toc-title"], n = r;
                La(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-title", parentData: t, parentDataProperty: "toc-title", rootData: s }) || (e = e === null ? La.errors : e.concat(La.errors), r = e.length);
                var ee = n === r;
              }
              if (t.lof !== void 0) {
                let a = t.lof, n = r;
                Zs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lof", parentData: t, parentDataProperty: "lof", rootData: s }) || (e = e === null ? Zs.errors : e.concat(Zs.errors), r = e.length);
                var ee = n === r;
              }
              if (t.lot !== void 0) {
                let a = t.lot, n = r;
                ei(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lot", parentData: t, parentDataProperty: "lot", rootData: s }) || (e = e === null ? ei.errors : e.concat(ei.errors), r = e.length);
                var ee = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebeamer(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var za = ht === r;
            za && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebeamer(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var za = gt === r;
            if (za && L ? (L = false, M = [M, 1]) : za && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebeamer(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^beamer(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (gv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebiblatex(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebiblatex(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var sa = n === r;
                if (!sa) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebiblatex(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var _e = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var _e = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var _e = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var _e = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var _e = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var _e = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var _e = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var _e = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var _e = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var _e = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var _e = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var _e = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var _e = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var _e = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var _e = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var _e = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var _e = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var _e = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var _e = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var _e = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var _e = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var _e = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var _e = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var _e = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var _e = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var _e = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var _e = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var _e = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var _e = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var _e = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var _e = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var _e = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var _e = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var _e = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var _e = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var _e = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var _e = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var _e = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var _e = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var _e = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var _e = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var _e = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var _e = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var _e = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var _e = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var _e = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var _e = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var _e = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var _e = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var _e = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var _e = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var _e = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var _e = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var _e = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var _e = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var _e = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var _e = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var _e = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var _e = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var _e = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var _e = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var _e = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var _e = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var _e = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var _e = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var _e = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var _e = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var _e = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var _e = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var _e = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var _e = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var _e = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var _e = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var _e = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var _e = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var _e = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var _e = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var _e = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var _e = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var _e = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var _e = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var _e = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var _e = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var _e = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var _e = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebiblatex(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Ma = ht === r;
            Ma && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebiblatex(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Ma = gt === r;
            if (Ma && L ? (L = false, M = [M, 1]) : Ma && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebiblatex(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^biblatex(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (hv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebibtex(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebibtex(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var fa = n === r;
                if (!fa) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebibtex(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var ie = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var ie = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var ie = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var ie = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var ie = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var ie = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var ie = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var ie = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var ie = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var ie = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var ie = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var ie = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var ie = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var ie = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var ie = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var ie = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var ie = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var ie = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var ie = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var ie = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var ie = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var ie = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var ie = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var ie = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var ie = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var ie = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var ie = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var ie = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var ie = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var ie = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var ie = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var ie = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var ie = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var ie = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var ie = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var ie = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var ie = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var ie = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var ie = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var ie = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var ie = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var ie = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var ie = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var ie = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var ie = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var ie = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var ie = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var ie = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var ie = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var ie = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var ie = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var ie = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var ie = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var ie = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var ie = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var ie = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var ie = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var ie = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var ie = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var ie = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var ie = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var ie = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var ie = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var ie = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var ie = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var ie = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var ie = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var ie = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var ie = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var ie = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var ie = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var ie = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var ie = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var ie = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var ie = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var ie = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var ie = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var ie = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var ie = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var ie = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var ie = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var ie = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var ie = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var ie = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var ie = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebibtex(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var ka = ht === r;
            ka && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebibtex(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var ka = gt === r;
            if (ka && L ? (L = false, M = [M, 1]) : ka && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ebibtex(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^bibtex(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (yv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!v1.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecommonmark(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecommonmark(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var hn = n === r;
                if (!hn) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecommonmark(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var et = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var et = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var et = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var et = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var et = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var et = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var et = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var et = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var et = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var et = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var et = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var et = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var et = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var et = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var et = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var et = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var et = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var et = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var et = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var et = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var et = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var et = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var et = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var et = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var et = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var et = n === r;
              }
              if (t["reference-location"] !== void 0) {
                let a = t["reference-location"], n = r;
                Ca(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-location", parentData: t, parentDataProperty: "reference-location", rootData: s }) || (e = e === null ? Ca.errors : e.concat(Ca.errors), r = e.length);
                var et = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var et = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var et = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var et = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var et = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var et = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var et = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var et = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var et = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var et = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var et = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var et = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var et = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var et = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var et = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var et = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var et = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var et = n === r;
              }
              if (t["prefer-html"] !== void 0) {
                let a = t["prefer-html"], n = r;
                jo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/prefer-html", parentData: t, parentDataProperty: "prefer-html", rootData: s }) || (e = e === null ? jo.errors : e.concat(jo.errors), r = e.length);
                var et = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var et = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var et = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var et = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var et = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var et = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var et = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var et = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var et = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var et = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var et = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var et = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var et = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var et = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var et = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var et = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var et = n === r;
              }
              if (t["identifier-prefix"] !== void 0) {
                let a = t["identifier-prefix"], n = r;
                Pa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier-prefix", parentData: t, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
                var et = n === r;
              }
              if (t.variant !== void 0) {
                let a = t.variant, n = r;
                Lo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variant", parentData: t, parentDataProperty: "variant", rootData: s }) || (e = e === null ? Lo.errors : e.concat(Lo.errors), r = e.length);
                var et = n === r;
              }
              if (t["markdown-headings"] !== void 0) {
                let a = t["markdown-headings"], n = r;
                ho(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/markdown-headings", parentData: t, parentDataProperty: "markdown-headings", rootData: s }) || (e = e === null ? ho.errors : e.concat(ho.errors), r = e.length);
                var et = n === r;
              }
              if (t["keep-yaml"] !== void 0) {
                let a = t["keep-yaml"], n = r;
                Mo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-yaml", parentData: t, parentDataProperty: "keep-yaml", rootData: s }) || (e = e === null ? Mo.errors : e.concat(Mo.errors), r = e.length);
                var et = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var et = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var et = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var et = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var et = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var et = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var et = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var et = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var et = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var et = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var et = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var et = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var et = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var et = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var et = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var et = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var et = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var et = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var et = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var et = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var et = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var et = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var et = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var et = n === r;
              }
              if (t["strip-comments"] !== void 0) {
                let a = t["strip-comments"], n = r;
                Ra(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-comments", parentData: t, parentDataProperty: "strip-comments", rootData: s }) || (e = e === null ? Ra.errors : e.concat(Ra.errors), r = e.length);
                var et = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var et = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var et = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var et = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var et = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecommonmark(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var yt = ht === r;
            yt && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecommonmark(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var yt = gt === r;
            if (yt && L ? (L = false, M = [M, 1]) : yt && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecommonmark(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^commonmark(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (bv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecommonmark_x(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecommonmark_x(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var _n = n === r;
                if (!_n) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecommonmark_x(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var st = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var st = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var st = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var st = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var st = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var st = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var st = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var st = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var st = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var st = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var st = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var st = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var st = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var st = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var st = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var st = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var st = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var st = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var st = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var st = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var st = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var st = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var st = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var st = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var st = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var st = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var st = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var st = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var st = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var st = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var st = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var st = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var st = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var st = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var st = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var st = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var st = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var st = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var st = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var st = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var st = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var st = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var st = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var st = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var st = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var st = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var st = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var st = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var st = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var st = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var st = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var st = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var st = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var st = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var st = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var st = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var st = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var st = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var st = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var st = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var st = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var st = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var st = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var st = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var st = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var st = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var st = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var st = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var st = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var st = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var st = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var st = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var st = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var st = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var st = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var st = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var st = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var st = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var st = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var st = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var st = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var st = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var st = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var st = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var st = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecommonmark_x(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Et = ht === r;
            Et && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecommonmark_x(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Et = gt === r;
            if (Et && L ? (L = false, M = [M, 1]) : Et && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecommonmark_x(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^commonmark_x(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (_v.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!H0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Econtext(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Econtext(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var ca = n === r;
                if (!ca) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Econtext(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.linkcolor !== void 0) {
                let a = t.linkcolor, n = r;
                Ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linkcolor", parentData: t, parentDataProperty: "linkcolor", rootData: s }) || (e = e === null ? Ua.errors : e.concat(Ua.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.contrastcolor !== void 0) {
                let a = t.contrastcolor, n = r;
                vc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/contrastcolor", parentData: t, parentDataProperty: "contrastcolor", rootData: s }) || (e = e === null ? vc.errors : e.concat(vc.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.mainfont !== void 0) {
                let a = t.mainfont, n = r;
                Ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mainfont", parentData: t, parentDataProperty: "mainfont", rootData: s }) || (e = e === null ? Ja.errors : e.concat(Ja.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.monofont !== void 0) {
                let a = t.monofont, n = r;
                Qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monofont", parentData: t, parentDataProperty: "monofont", rootData: s }) || (e = e === null ? Qa.errors : e.concat(Qa.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.fontsize !== void 0) {
                let a = t.fontsize, n = r;
                Ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontsize", parentData: t, parentDataProperty: "fontsize", rootData: s }) || (e = e === null ? Ga.errors : e.concat(Ga.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.linestretch !== void 0) {
                let a = t.linestretch, n = r;
                Ya(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linestretch", parentData: t, parentDataProperty: "linestretch", rootData: s }) || (e = e === null ? Ya.errors : e.concat(Ya.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.interlinespace !== void 0) {
                let a = t.interlinespace, n = r;
                Pc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/interlinespace", parentData: t, parentDataProperty: "interlinespace", rootData: s }) || (e = e === null ? Pc.errors : e.concat(Pc.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.linkstyle !== void 0) {
                let a = t.linkstyle, n = r;
                xc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linkstyle", parentData: t, parentDataProperty: "linkstyle", rootData: s }) || (e = e === null ? xc.errors : e.concat(xc.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.whitespace !== void 0) {
                let a = t.whitespace, n = r;
                kc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/whitespace", parentData: t, parentDataProperty: "whitespace", rootData: s }) || (e = e === null ? kc.errors : e.concat(kc.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.indenting !== void 0) {
                let a = t.indenting, n = r;
                Dc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indenting", parentData: t, parentDataProperty: "indenting", rootData: s }) || (e = e === null ? Dc.errors : e.concat(Dc.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.headertext !== void 0) {
                let a = t.headertext, n = r;
                Fc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/headertext", parentData: t, parentDataProperty: "headertext", rootData: s }) || (e = e === null ? Fc.errors : e.concat(Fc.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.footertext !== void 0) {
                let a = t.footertext, n = r;
                Ac(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/footertext", parentData: t, parentDataProperty: "footertext", rootData: s }) || (e = e === null ? Ac.errors : e.concat(Ac.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.includesource !== void 0) {
                let a = t.includesource, n = r;
                Bc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/includesource", parentData: t, parentDataProperty: "includesource", rootData: s }) || (e = e === null ? Bc.errors : e.concat(Bc.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.layout !== void 0) {
                let a = t.layout, n = r;
                Ec(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/layout", parentData: t, parentDataProperty: "layout", rootData: s }) || (e = e === null ? Ec.errors : e.concat(Ec.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["margin-left"] !== void 0) {
                let a = t["margin-left"], n = r;
                Xa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-left", parentData: t, parentDataProperty: "margin-left", rootData: s }) || (e = e === null ? Xa.errors : e.concat(Xa.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["margin-right"] !== void 0) {
                let a = t["margin-right"], n = r;
                Za(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-right", parentData: t, parentDataProperty: "margin-right", rootData: s }) || (e = e === null ? Za.errors : e.concat(Za.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["margin-top"] !== void 0) {
                let a = t["margin-top"], n = r;
                en(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-top", parentData: t, parentDataProperty: "margin-top", rootData: s }) || (e = e === null ? en.errors : e.concat(en.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["margin-bottom"] !== void 0) {
                let a = t["margin-bottom"], n = r;
                rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-bottom", parentData: t, parentDataProperty: "margin-bottom", rootData: s }) || (e = e === null ? rn.errors : e.concat(rn.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.keywords !== void 0) {
                let a = t.keywords, n = r;
                Ia(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keywords", parentData: t, parentDataProperty: "keywords", rootData: s }) || (e = e === null ? Ia.errors : e.concat(Ia.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.pagenumbering !== void 0) {
                let a = t.pagenumbering, n = r;
                Lc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pagenumbering", parentData: t, parentDataProperty: "pagenumbering", rootData: s }) || (e = e === null ? Lc.errors : e.concat(Lc.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["top-level-division"] !== void 0) {
                let a = t["top-level-division"], n = r;
                Bn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/top-level-division", parentData: t, parentDataProperty: "top-level-division", rootData: s }) || (e = e === null ? Bn.errors : e.concat(Bn.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["pdf-engine"] !== void 0) {
                let a = t["pdf-engine"], n = r;
                mo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pdf-engine", parentData: t, parentDataProperty: "pdf-engine", rootData: s }) || (e = e === null ? mo.errors : e.concat(mo.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["pdf-engine-opt"] !== void 0) {
                let a = t["pdf-engine-opt"], n = r;
                go(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pdf-engine-opt", parentData: t, parentDataProperty: "pdf-engine-opt", rootData: s }) || (e = e === null ? go.errors : e.concat(go.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.pdfa !== void 0) {
                let a = t.pdfa, n = r;
                Zc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pdfa", parentData: t, parentDataProperty: "pdfa", rootData: s }) || (e = e === null ? Zc.errors : e.concat(Zc.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.pdfaiccprofile !== void 0) {
                let a = t.pdfaiccprofile, n = r;
                ep(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pdfaiccprofile", parentData: t, parentDataProperty: "pdfaiccprofile", rootData: s }) || (e = e === null ? ep.errors : e.concat(ep.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.pdfaintent !== void 0) {
                let a = t.pdfaintent, n = r;
                rp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pdfaintent", parentData: t, parentDataProperty: "pdfaintent", rootData: s }) || (e = e === null ? rp.errors : e.concat(rp.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Gr = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Gr = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Gr = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Econtext(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Tt = ht === r;
            Tt && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Econtext(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Tt = gt === r;
            if (Tt && L ? (L = false, M = [M, 1]) : Tt && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Econtext(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^context(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (vv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecsljson(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecsljson(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var bn = n === r;
                if (!bn) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecsljson(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var vt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var vt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var vt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var vt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var vt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var vt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var vt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var vt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var vt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var vt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var vt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var vt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var vt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var vt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var vt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var vt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var vt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var vt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var vt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var vt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var vt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var vt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var vt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var vt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var vt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var vt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var vt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var vt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var vt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var vt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var vt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var vt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var vt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var vt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var vt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var vt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var vt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var vt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var vt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var vt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var vt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var vt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var vt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var vt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var vt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var vt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var vt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var vt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var vt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var vt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var vt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var vt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var vt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var vt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var vt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var vt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var vt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var vt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var vt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var vt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var vt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var vt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var vt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var vt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var vt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var vt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var vt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var vt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var vt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var vt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var vt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var vt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var vt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var vt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var vt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var vt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var vt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var vt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var vt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var vt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var vt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var vt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var vt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var vt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var vt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecsljson(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var yn = ht === r;
            yn && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecsljson(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var yn = gt === r;
            if (yn && L ? (L = false, M = [M, 1]) : yn && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ecsljson(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^csljson(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (qv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!q1.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension))", schema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var Va = n === r;
                if (!Va) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["top-level-division"] !== void 0) {
                let a = t["top-level-division"], n = r;
                Bn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/top-level-division", parentData: t, parentDataProperty: "top-level-division", rootData: s }) || (e = e === null ? Bn.errors : e.concat(Bn.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["identifier-prefix"] !== void 0) {
                let a = t["identifier-prefix"], n = r;
                Pa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier-prefix", parentData: t, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Dt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Dt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Dt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var nl = ht === r;
            nl && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var nl = gt === r;
            if (nl && L ? (L = false, M = [M, 1]) : nl && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Ov.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!q1.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook4(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension))", schema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook4(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var ol = n === r;
                if (!ol) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook4(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var wt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var wt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var wt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var wt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var wt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var wt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var wt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var wt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var wt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var wt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var wt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var wt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var wt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var wt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var wt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var wt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var wt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var wt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var wt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var wt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var wt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var wt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var wt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var wt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var wt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var wt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var wt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var wt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var wt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var wt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var wt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var wt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var wt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var wt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var wt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var wt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var wt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var wt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var wt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var wt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var wt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var wt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var wt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var wt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var wt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var wt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var wt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var wt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var wt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var wt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var wt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var wt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var wt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var wt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var wt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var wt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var wt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var wt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var wt = n === r;
              }
              if (t["top-level-division"] !== void 0) {
                let a = t["top-level-division"], n = r;
                Bn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/top-level-division", parentData: t, parentDataProperty: "top-level-division", rootData: s }) || (e = e === null ? Bn.errors : e.concat(Bn.errors), r = e.length);
                var wt = n === r;
              }
              if (t["identifier-prefix"] !== void 0) {
                let a = t["identifier-prefix"], n = r;
                Pa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier-prefix", parentData: t, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
                var wt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var wt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var wt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var wt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var wt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var wt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var wt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var wt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var wt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var wt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var wt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var wt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var wt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var wt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var wt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var wt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var wt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var wt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var wt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook4(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var ii = ht === r;
            ii && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook4(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var ii = gt === r;
            if (ii && L ? (L = false, M = [M, 1]) : ii && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook4(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook4(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Pv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!q1.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook5(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension))", schema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook5(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var Jn = n === r;
                if (!Jn) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook5(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["top-level-division"] !== void 0) {
                let a = t["top-level-division"], n = r;
                Bn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/top-level-division", parentData: t, parentDataProperty: "top-level-division", rootData: s }) || (e = e === null ? Bn.errors : e.concat(Bn.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["identifier-prefix"] !== void 0) {
                let a = t["identifier-prefix"], n = r;
                Pa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier-prefix", parentData: t, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Bt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Bt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Bt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook5(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var oo = ht === r;
            oo && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook5(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var oo = gt === r;
            if (oo && L ? (L = false, M = [M, 1]) : oo && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocbook5(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docbook5(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (xv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!L0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocx(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|track_changes|trackChanges|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|page_width|pageWidth|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|reference_doc|referenceDoc|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", schema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|track_changes|trackChanges|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|page_width|pageWidth|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|reference_doc|referenceDoc|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))"', schema: "^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|track_changes|trackChanges|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|page_width|pageWidth|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|reference_doc|referenceDoc|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocx(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var qo = n === r;
                if (!qo) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocx(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["code-line-numbers"] !== void 0) {
                let a = t["code-line-numbers"], n = r;
                ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-line-numbers", parentData: t, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["highlight-style"] !== void 0) {
                let a = t["highlight-style"], n = r;
                _a(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/highlight-style", parentData: t, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["syntax-definition"] !== void 0) {
                let a = t["syntax-definition"], n = r;
                va(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definition", parentData: t, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["syntax-definitions"] !== void 0) {
                let a = t["syntax-definitions"], n = r;
                qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definitions", parentData: t, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["indented-code-classes"] !== void 0) {
                let a = t["indented-code-classes"], n = r;
                Oa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indented-code-classes", parentData: t, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["track-changes"] !== void 0) {
                let a = t["track-changes"], n = r;
                Sc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/track-changes", parentData: t, parentDataProperty: "track-changes", rootData: s }) || (e = e === null ? Sc.errors : e.concat(Sc.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["page-width"] !== void 0) {
                let a = t["page-width"], n = r;
                Is(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/page-width", parentData: t, parentDataProperty: "page-width", rootData: s }) || (e = e === null ? Is.errors : e.concat(Is.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.subject !== void 0) {
                let a = t.subject, n = r;
                fo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subject", parentData: t, parentDataProperty: "subject", rootData: s }) || (e = e === null ? fo.errors : e.concat(fo.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.description !== void 0) {
                let a = t.description, n = r;
                Wl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/description", parentData: t, parentDataProperty: "description", rootData: s }) || (e = e === null ? Wl.errors : e.concat(Wl.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.category !== void 0) {
                let a = t.category, n = r;
                Rl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/category", parentData: t, parentDataProperty: "category", rootData: s }) || (e = e === null ? Rl.errors : e.concat(Rl.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["number-depth"] !== void 0) {
                let a = t["number-depth"], n = r;
                Da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-depth", parentData: t, parentDataProperty: "number-depth", rootData: s }) || (e = e === null ? Da.errors : e.concat(Da.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["reference-doc"] !== void 0) {
                let a = t["reference-doc"], n = r;
                Ws(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-doc", parentData: t, parentDataProperty: "reference-doc", rootData: s }) || (e = e === null ? Ws.errors : e.concat(Ws.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Ot = n === r;
              }
              if (t["toc-title"] !== void 0) {
                let a = t["toc-title"], n = r;
                La(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-title", parentData: t, parentDataProperty: "toc-title", rootData: s }) || (e = e === null ? La.errors : e.concat(La.errors), r = e.length);
                var Ot = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocx(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var xo = ht === r;
            xo && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocx(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var xo = gt === r;
            if (xo && L ? (L = false, M = [M, 1]) : xo && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edocx(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^docx(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (kv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edokuwiki(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edokuwiki(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var li = n === r;
                if (!li) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edokuwiki(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Pt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Pt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Pt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edokuwiki(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var bo = ht === r;
            bo && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edokuwiki(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var bo = gt === r;
            if (bo && L ? (L = false, M = [M, 1]) : bo && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edokuwiki(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^dokuwiki(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Dv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!M0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edzslides(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edzslides(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var sl = n === r;
                if (!sl) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edzslides(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["code-fold"] !== void 0) {
                let a = t["code-fold"], n = r;
                Fa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-fold", parentData: t, parentDataProperty: "code-fold", rootData: s }) || (e = e === null ? Fa.errors : e.concat(Fa.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["code-summary"] !== void 0) {
                let a = t["code-summary"], n = r;
                Aa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-summary", parentData: t, parentDataProperty: "code-summary", rootData: s }) || (e = e === null ? Aa.errors : e.concat(Aa.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["code-overflow"] !== void 0) {
                let a = t["code-overflow"], n = r;
                Ba(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-overflow", parentData: t, parentDataProperty: "code-overflow", rootData: s }) || (e = e === null ? Ba.errors : e.concat(Ba.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["code-line-numbers"] !== void 0) {
                let a = t["code-line-numbers"], n = r;
                ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-line-numbers", parentData: t, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["cap-location"] !== void 0) {
                let a = t["cap-location"], n = r;
                ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cap-location", parentData: t, parentDataProperty: "cap-location", rootData: s }) || (e = e === null ? ja.errors : e.concat(ja.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["fig-cap-location"] !== void 0) {
                let a = t["fig-cap-location"], n = r;
                Ea(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-cap-location", parentData: t, parentDataProperty: "fig-cap-location", rootData: s }) || (e = e === null ? Ea.errors : e.concat(Ea.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["tbl-cap-location"] !== void 0) {
                let a = t["tbl-cap-location"], n = r;
                Na(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-cap-location", parentData: t, parentDataProperty: "tbl-cap-location", rootData: s }) || (e = e === null ? Na.errors : e.concat(Na.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.institute !== void 0) {
                let a = t.institute, n = r;
                ro(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/institute", parentData: t, parentDataProperty: "institute", rootData: s }) || (e = e === null ? ro.errors : e.concat(ro.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["code-copy"] !== void 0) {
                let a = t["code-copy"], n = r;
                ln(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-copy", parentData: t, parentDataProperty: "code-copy", rootData: s }) || (e = e === null ? ln.errors : e.concat(ln.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["code-link"] !== void 0) {
                let a = t["code-link"], n = r;
                On(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-link", parentData: t, parentDataProperty: "code-link", rootData: s }) || (e = e === null ? On.errors : e.concat(On.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["highlight-style"] !== void 0) {
                let a = t["highlight-style"], n = r;
                _a(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/highlight-style", parentData: t, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["syntax-definition"] !== void 0) {
                let a = t["syntax-definition"], n = r;
                va(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definition", parentData: t, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["syntax-definitions"] !== void 0) {
                let a = t["syntax-definitions"], n = r;
                qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definitions", parentData: t, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["indented-code-classes"] !== void 0) {
                let a = t["indented-code-classes"], n = r;
                Oa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indented-code-classes", parentData: t, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.fontcolor !== void 0) {
                let a = t.fontcolor, n = r;
                Pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontcolor", parentData: t, parentDataProperty: "fontcolor", rootData: s }) || (e = e === null ? Pn.errors : e.concat(Pn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.linkcolor !== void 0) {
                let a = t.linkcolor, n = r;
                Ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linkcolor", parentData: t, parentDataProperty: "linkcolor", rootData: s }) || (e = e === null ? Ua.errors : e.concat(Ua.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.monobackgroundcolor !== void 0) {
                let a = t.monobackgroundcolor, n = r;
                xn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monobackgroundcolor", parentData: t, parentDataProperty: "monobackgroundcolor", rootData: s }) || (e = e === null ? xn.errors : e.concat(xn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.backgroundcolor !== void 0) {
                let a = t.backgroundcolor, n = r;
                kn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/backgroundcolor", parentData: t, parentDataProperty: "backgroundcolor", rootData: s }) || (e = e === null ? kn.errors : e.concat(kn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.comments !== void 0) {
                let a = t.comments, n = r;
                Dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/comments", parentData: t, parentDataProperty: "comments", rootData: s }) || (e = e === null ? Dn.errors : e.concat(Dn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["fig-responsive"] !== void 0) {
                let a = t["fig-responsive"], n = r;
                cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-responsive", parentData: t, parentDataProperty: "fig-responsive", rootData: s }) || (e = e === null ? cn.errors : e.concat(cn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.mainfont !== void 0) {
                let a = t.mainfont, n = r;
                Ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mainfont", parentData: t, parentDataProperty: "mainfont", rootData: s }) || (e = e === null ? Ja.errors : e.concat(Ja.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.monofont !== void 0) {
                let a = t.monofont, n = r;
                Qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monofont", parentData: t, parentDataProperty: "monofont", rootData: s }) || (e = e === null ? Qa.errors : e.concat(Qa.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.fontsize !== void 0) {
                let a = t.fontsize, n = r;
                Ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontsize", parentData: t, parentDataProperty: "fontsize", rootData: s }) || (e = e === null ? Ga.errors : e.concat(Ga.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.linestretch !== void 0) {
                let a = t.linestretch, n = r;
                Ya(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linestretch", parentData: t, parentDataProperty: "linestretch", rootData: s }) || (e = e === null ? Ya.errors : e.concat(Ya.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["footnotes-hover"] !== void 0) {
                let a = t["footnotes-hover"], n = r;
                wn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/footnotes-hover", parentData: t, parentDataProperty: "footnotes-hover", rootData: s }) || (e = e === null ? wn.errors : e.concat(wn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["reference-location"] !== void 0) {
                let a = t["reference-location"], n = r;
                Ca(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-location", parentData: t, parentDataProperty: "reference-location", rootData: s }) || (e = e === null ? Ca.errors : e.concat(Ca.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.resources !== void 0) {
                let a = t.resources, n = r;
                pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resources", parentData: t, parentDataProperty: "resources", rootData: s }) || (e = e === null ? pn.errors : e.concat(pn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.classoption !== void 0) {
                let a = t.classoption, n = r;
                un(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/classoption", parentData: t, parentDataProperty: "classoption", rootData: s }) || (e = e === null ? un.errors : e.concat(un.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["max-width"] !== void 0) {
                let a = t["max-width"], n = r;
                $n(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/max-width", parentData: t, parentDataProperty: "max-width", rootData: s }) || (e = e === null ? $n.errors : e.concat($n.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["margin-left"] !== void 0) {
                let a = t["margin-left"], n = r;
                Xa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-left", parentData: t, parentDataProperty: "margin-left", rootData: s }) || (e = e === null ? Xa.errors : e.concat(Xa.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["margin-right"] !== void 0) {
                let a = t["margin-right"], n = r;
                Za(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-right", parentData: t, parentDataProperty: "margin-right", rootData: s }) || (e = e === null ? Za.errors : e.concat(Za.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["margin-top"] !== void 0) {
                let a = t["margin-top"], n = r;
                en(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-top", parentData: t, parentDataProperty: "margin-top", rootData: s }) || (e = e === null ? en.errors : e.concat(en.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["margin-bottom"] !== void 0) {
                let a = t["margin-bottom"], n = r;
                rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-bottom", parentData: t, parentDataProperty: "margin-bottom", rootData: s }) || (e = e === null ? rn.errors : e.concat(rn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.keywords !== void 0) {
                let a = t.keywords, n = r;
                Ia(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keywords", parentData: t, parentDataProperty: "keywords", rootData: s }) || (e = e === null ? Ia.errors : e.concat(Ia.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.pagetitle !== void 0) {
                let a = t.pagetitle, n = r;
                Sn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pagetitle", parentData: t, parentDataProperty: "pagetitle", rootData: s }) || (e = e === null ? Sn.errors : e.concat(Sn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["title-prefix"] !== void 0) {
                let a = t["title-prefix"], n = r;
                Fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title-prefix", parentData: t, parentDataProperty: "title-prefix", rootData: s }) || (e = e === null ? Fn.errors : e.concat(Fn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["description-meta"] !== void 0) {
                let a = t["description-meta"], n = r;
                An(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/description-meta", parentData: t, parentDataProperty: "description-meta", rootData: s }) || (e = e === null ? An.errors : e.concat(An.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["author-meta"] !== void 0) {
                let a = t["author-meta"], n = r;
                dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author-meta", parentData: t, parentDataProperty: "author-meta", rootData: s }) || (e = e === null ? dn.errors : e.concat(dn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["date-meta"] !== void 0) {
                let a = t["date-meta"], n = r;
                Ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date-meta", parentData: t, parentDataProperty: "date-meta", rootData: s }) || (e = e === null ? Ha.errors : e.concat(Ha.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["number-depth"] !== void 0) {
                let a = t["number-depth"], n = r;
                Da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-depth", parentData: t, parentDataProperty: "number-depth", rootData: s }) || (e = e === null ? Da.errors : e.concat(Da.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["document-css"] !== void 0) {
                let a = t["document-css"], n = r;
                jn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/document-css", parentData: t, parentDataProperty: "document-css", rootData: s }) || (e = e === null ? jn.errors : e.concat(jn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.css !== void 0) {
                let a = t.css, n = r;
                fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/css", parentData: t, parentDataProperty: "css", rootData: s }) || (e = e === null ? fn.errors : e.concat(fn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["section-divs"] !== void 0) {
                let a = t["section-divs"], n = r;
                En(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/section-divs", parentData: t, parentDataProperty: "section-divs", rootData: s }) || (e = e === null ? En.errors : e.concat(En.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["identifier-prefix"] !== void 0) {
                let a = t["identifier-prefix"], n = r;
                Pa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier-prefix", parentData: t, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["email-obfuscation"] !== void 0) {
                let a = t["email-obfuscation"], n = r;
                Nn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/email-obfuscation", parentData: t, parentDataProperty: "email-obfuscation", rootData: s }) || (e = e === null ? Nn.errors : e.concat(Nn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["html-q-tags"] !== void 0) {
                let a = t["html-q-tags"], n = r;
                mn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-q-tags", parentData: t, parentDataProperty: "html-q-tags", rootData: s }) || (e = e === null ? mn.errors : e.concat(mn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["citations-hover"] !== void 0) {
                let a = t["citations-hover"], n = r;
                Cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citations-hover", parentData: t, parentDataProperty: "citations-hover", rootData: s }) || (e = e === null ? Cn.errors : e.concat(Cn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["self-contained"] !== void 0) {
                let a = t["self-contained"], n = r;
                In(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/self-contained", parentData: t, parentDataProperty: "self-contained", rootData: s }) || (e = e === null ? In.errors : e.concat(In.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["self-contained-math"] !== void 0) {
                let a = t["self-contained-math"], n = r;
                Hn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/self-contained-math", parentData: t, parentDataProperty: "self-contained-math", rootData: s }) || (e = e === null ? Hn.errors : e.concat(Hn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.incremental !== void 0) {
                let a = t.incremental, n = r;
                Rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/incremental", parentData: t, parentDataProperty: "incremental", rootData: s }) || (e = e === null ? Rn.errors : e.concat(Rn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["slide-level"] !== void 0) {
                let a = t["slide-level"], n = r;
                Vn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/slide-level", parentData: t, parentDataProperty: "slide-level", rootData: s }) || (e = e === null ? Vn.errors : e.concat(Vn.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["strip-comments"] !== void 0) {
                let a = t["strip-comments"], n = r;
                Ra(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-comments", parentData: t, parentDataProperty: "strip-comments", rootData: s }) || (e = e === null ? Ra.errors : e.concat(Ra.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var Ur = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Ur = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Ur = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edzslides(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Q = ht === r;
            Q && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edzslides(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Q = gt === r;
            if (Q && L ? (L = false, M = [M, 1]) : Q && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Edzslides(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^dzslides(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (wv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!O1.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|html_q_tags|htmlQTags|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", schema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|html_q_tags|htmlQTags|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))"', schema: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|html_q_tags|htmlQTags|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var ba = n === r;
                if (!ba) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var $r = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var $r = n === r;
              }
              if (t["code-fold"] !== void 0) {
                let a = t["code-fold"], n = r;
                Fa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-fold", parentData: t, parentDataProperty: "code-fold", rootData: s }) || (e = e === null ? Fa.errors : e.concat(Fa.errors), r = e.length);
                var $r = n === r;
              }
              if (t["code-summary"] !== void 0) {
                let a = t["code-summary"], n = r;
                Aa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-summary", parentData: t, parentDataProperty: "code-summary", rootData: s }) || (e = e === null ? Aa.errors : e.concat(Aa.errors), r = e.length);
                var $r = n === r;
              }
              if (t["code-overflow"] !== void 0) {
                let a = t["code-overflow"], n = r;
                Ba(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-overflow", parentData: t, parentDataProperty: "code-overflow", rootData: s }) || (e = e === null ? Ba.errors : e.concat(Ba.errors), r = e.length);
                var $r = n === r;
              }
              if (t["code-line-numbers"] !== void 0) {
                let a = t["code-line-numbers"], n = r;
                ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-line-numbers", parentData: t, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
                var $r = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var $r = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var $r = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var $r = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var $r = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var $r = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var $r = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var $r = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var $r = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var $r = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var $r = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var $r = n === r;
              }
              if (t["abstract-title"] !== void 0) {
                let a = t["abstract-title"], n = r;
                to(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract-title", parentData: t, parentDataProperty: "abstract-title", rootData: s }) || (e = e === null ? to.errors : e.concat(to.errors), r = e.length);
                var $r = n === r;
              }
              if (t["code-copy"] !== void 0) {
                let a = t["code-copy"], n = r;
                ln(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-copy", parentData: t, parentDataProperty: "code-copy", rootData: s }) || (e = e === null ? ln.errors : e.concat(ln.errors), r = e.length);
                var $r = n === r;
              }
              if (t["highlight-style"] !== void 0) {
                let a = t["highlight-style"], n = r;
                _a(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/highlight-style", parentData: t, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
                var $r = n === r;
              }
              if (t["syntax-definition"] !== void 0) {
                let a = t["syntax-definition"], n = r;
                va(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definition", parentData: t, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
                var $r = n === r;
              }
              if (t["syntax-definitions"] !== void 0) {
                let a = t["syntax-definitions"], n = r;
                qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definitions", parentData: t, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
                var $r = n === r;
              }
              if (t["indented-code-classes"] !== void 0) {
                let a = t["indented-code-classes"], n = r;
                Oa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indented-code-classes", parentData: t, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
                var $r = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var $r = n === r;
              }
              if (t.identifier !== void 0) {
                let a = t.identifier, n = r;
                as(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier", parentData: t, parentDataProperty: "identifier", rootData: s }) || (e = e === null ? as.errors : e.concat(as.errors), r = e.length);
                var $r = n === r;
              }
              if (t.creator !== void 0) {
                let a = t.creator, n = r;
                ns(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/creator", parentData: t, parentDataProperty: "creator", rootData: s }) || (e = e === null ? ns.errors : e.concat(ns.errors), r = e.length);
                var $r = n === r;
              }
              if (t.contributor !== void 0) {
                let a = t.contributor, n = r;
                os(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/contributor", parentData: t, parentDataProperty: "contributor", rootData: s }) || (e = e === null ? os.errors : e.concat(os.errors), r = e.length);
                var $r = n === r;
              }
              if (t.subject !== void 0) {
                let a = t.subject, n = r;
                Bl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subject", parentData: t, parentDataProperty: "subject", rootData: s }) || (e = e === null ? Bl.errors : e.concat(Bl.errors), r = e.length);
                var $r = n === r;
              }
              if (t.type !== void 0) {
                let a = t.type, n = r;
                ss(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/type", parentData: t, parentDataProperty: "type", rootData: s }) || (e = e === null ? ss.errors : e.concat(ss.errors), r = e.length);
                var $r = n === r;
              }
              if (t.format !== void 0) {
                let a = t.format, n = r;
                jl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/format", parentData: t, parentDataProperty: "format", rootData: s }) || (e = e === null ? jl.errors : e.concat(jl.errors), r = e.length);
                var $r = n === r;
              }
              if (t.relation !== void 0) {
                let a = t.relation, n = r;
                is(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/relation", parentData: t, parentDataProperty: "relation", rootData: s }) || (e = e === null ? is.errors : e.concat(is.errors), r = e.length);
                var $r = n === r;
              }
              if (t.coverage !== void 0) {
                let a = t.coverage, n = r;
                ls(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/coverage", parentData: t, parentDataProperty: "coverage", rootData: s }) || (e = e === null ? ls.errors : e.concat(ls.errors), r = e.length);
                var $r = n === r;
              }
              if (t.rights !== void 0) {
                let a = t.rights, n = r;
                cs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/rights", parentData: t, parentDataProperty: "rights", rootData: s }) || (e = e === null ? cs.errors : e.concat(cs.errors), r = e.length);
                var $r = n === r;
              }
              if (t["belongs-to-collection"] !== void 0) {
                let a = t["belongs-to-collection"], n = r;
                ps(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/belongs-to-collection", parentData: t, parentDataProperty: "belongs-to-collection", rootData: s }) || (e = e === null ? ps.errors : e.concat(ps.errors), r = e.length);
                var $r = n === r;
              }
              if (t["group-position"] !== void 0) {
                let a = t["group-position"], n = r;
                us(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/group-position", parentData: t, parentDataProperty: "group-position", rootData: s }) || (e = e === null ? us.errors : e.concat(us.errors), r = e.length);
                var $r = n === r;
              }
              if (t["page-progression-direction"] !== void 0) {
                let a = t["page-progression-direction"], n = r;
                ds(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/page-progression-direction", parentData: t, parentDataProperty: "page-progression-direction", rootData: s }) || (e = e === null ? ds.errors : e.concat(ds.errors), r = e.length);
                var $r = n === r;
              }
              if (t.ibooks !== void 0) {
                let a = t.ibooks, n = r;
                fs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ibooks", parentData: t, parentDataProperty: "ibooks", rootData: s }) || (e = e === null ? fs.errors : e.concat(fs.errors), r = e.length);
                var $r = n === r;
              }
              if (t["epub-metadata"] !== void 0) {
                let a = t["epub-metadata"], n = r;
                ms(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/epub-metadata", parentData: t, parentDataProperty: "epub-metadata", rootData: s }) || (e = e === null ? ms.errors : e.concat(ms.errors), r = e.length);
                var $r = n === r;
              }
              if (t["epub-subdirectory"] !== void 0) {
                let a = t["epub-subdirectory"], n = r;
                gs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/epub-subdirectory", parentData: t, parentDataProperty: "epub-subdirectory", rootData: s }) || (e = e === null ? gs.errors : e.concat(gs.errors), r = e.length);
                var $r = n === r;
              }
              if (t["epub-fonts"] !== void 0) {
                let a = t["epub-fonts"], n = r;
                hs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/epub-fonts", parentData: t, parentDataProperty: "epub-fonts", rootData: s }) || (e = e === null ? hs.errors : e.concat(hs.errors), r = e.length);
                var $r = n === r;
              }
              if (t["epub-chapter-level"] !== void 0) {
                let a = t["epub-chapter-level"], n = r;
                ys(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/epub-chapter-level", parentData: t, parentDataProperty: "epub-chapter-level", rootData: s }) || (e = e === null ? ys.errors : e.concat(ys.errors), r = e.length);
                var $r = n === r;
              }
              if (t["epub-cover-image"] !== void 0) {
                let a = t["epub-cover-image"], n = r;
                bs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/epub-cover-image", parentData: t, parentDataProperty: "epub-cover-image", rootData: s }) || (e = e === null ? bs.errors : e.concat(bs.errors), r = e.length);
                var $r = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var $r = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var $r = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var $r = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var $r = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var $r = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var $r = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var $r = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var $r = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var $r = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var $r = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var $r = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var $r = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var $r = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var $r = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var $r = n === r;
              }
              if (t["fig-responsive"] !== void 0) {
                let a = t["fig-responsive"], n = r;
                cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-responsive", parentData: t, parentDataProperty: "fig-responsive", rootData: s }) || (e = e === null ? cn.errors : e.concat(cn.errors), r = e.length);
                var $r = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var $r = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var $r = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var $r = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var $r = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var $r = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var $r = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var $r = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var $r = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var $r = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var $r = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var $r = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var $r = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var $r = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var $r = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var $r = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var $r = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var $r = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var $r = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var $r = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var $r = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var $r = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var $r = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var $r = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var $r = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var $r = n === r;
              }
              if (t.resources !== void 0) {
                let a = t.resources, n = r;
                pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resources", parentData: t, parentDataProperty: "resources", rootData: s }) || (e = e === null ? pn.errors : e.concat(pn.errors), r = e.length);
                var $r = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var $r = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var $r = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var $r = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var $r = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var $r = n === r;
              }
              if (t["date-meta"] !== void 0) {
                let a = t["date-meta"], n = r;
                Ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date-meta", parentData: t, parentDataProperty: "date-meta", rootData: s }) || (e = e === null ? Ha.errors : e.concat(Ha.errors), r = e.length);
                var $r = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var $r = n === r;
              }
              if (t["number-depth"] !== void 0) {
                let a = t["number-depth"], n = r;
                Da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-depth", parentData: t, parentDataProperty: "number-depth", rootData: s }) || (e = e === null ? Da.errors : e.concat(Da.errors), r = e.length);
                var $r = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var $r = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var $r = n === r;
              }
              if (t.css !== void 0) {
                let a = t.css, n = r;
                fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/css", parentData: t, parentDataProperty: "css", rootData: s }) || (e = e === null ? fn.errors : e.concat(fn.errors), r = e.length);
                var $r = n === r;
              }
              if (t["html-math-method"] !== void 0) {
                let a = t["html-math-method"], n = r;
                Wn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-math-method", parentData: t, parentDataProperty: "html-math-method", rootData: s }) || (e = e === null ? Wn.errors : e.concat(Wn.errors), r = e.length);
                var $r = n === r;
              }
              if (t["html-q-tags"] !== void 0) {
                let a = t["html-q-tags"], n = r;
                mn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-q-tags", parentData: t, parentDataProperty: "html-q-tags", rootData: s }) || (e = e === null ? mn.errors : e.concat(mn.errors), r = e.length);
                var $r = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var $r = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var $r = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var $r = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var $r = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var $r = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var $r = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var $r = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var $r = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var $r = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var $r = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var $r = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var $r = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var $r = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var $r = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var $r = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var $r = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var $r = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var $r = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var $r = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var $r = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var $r = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var $r = n === r;
              }
              if (t["toc-title"] !== void 0) {
                let a = t["toc-title"], n = r;
                La(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-title", parentData: t, parentDataProperty: "toc-title", rootData: s }) || (e = e === null ? La.errors : e.concat(La.errors), r = e.length);
                var $r = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var kt = ht === r;
            kt && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var kt = gt === r;
            if (kt && L ? (L = false, M = [M, 1]) : kt && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if ($v.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!O1.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub2(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|html_q_tags|htmlQTags|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", schema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|html_q_tags|htmlQTags|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))"', schema: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|html_q_tags|htmlQTags|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub2(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var $t = n === r;
                if (!$t) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub2(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["code-fold"] !== void 0) {
                let a = t["code-fold"], n = r;
                Fa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-fold", parentData: t, parentDataProperty: "code-fold", rootData: s }) || (e = e === null ? Fa.errors : e.concat(Fa.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["code-summary"] !== void 0) {
                let a = t["code-summary"], n = r;
                Aa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-summary", parentData: t, parentDataProperty: "code-summary", rootData: s }) || (e = e === null ? Aa.errors : e.concat(Aa.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["code-overflow"] !== void 0) {
                let a = t["code-overflow"], n = r;
                Ba(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-overflow", parentData: t, parentDataProperty: "code-overflow", rootData: s }) || (e = e === null ? Ba.errors : e.concat(Ba.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["code-line-numbers"] !== void 0) {
                let a = t["code-line-numbers"], n = r;
                ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-line-numbers", parentData: t, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["abstract-title"] !== void 0) {
                let a = t["abstract-title"], n = r;
                to(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract-title", parentData: t, parentDataProperty: "abstract-title", rootData: s }) || (e = e === null ? to.errors : e.concat(to.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["code-copy"] !== void 0) {
                let a = t["code-copy"], n = r;
                ln(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-copy", parentData: t, parentDataProperty: "code-copy", rootData: s }) || (e = e === null ? ln.errors : e.concat(ln.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["highlight-style"] !== void 0) {
                let a = t["highlight-style"], n = r;
                _a(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/highlight-style", parentData: t, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["syntax-definition"] !== void 0) {
                let a = t["syntax-definition"], n = r;
                va(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definition", parentData: t, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["syntax-definitions"] !== void 0) {
                let a = t["syntax-definitions"], n = r;
                qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definitions", parentData: t, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["indented-code-classes"] !== void 0) {
                let a = t["indented-code-classes"], n = r;
                Oa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indented-code-classes", parentData: t, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.identifier !== void 0) {
                let a = t.identifier, n = r;
                as(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier", parentData: t, parentDataProperty: "identifier", rootData: s }) || (e = e === null ? as.errors : e.concat(as.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.creator !== void 0) {
                let a = t.creator, n = r;
                ns(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/creator", parentData: t, parentDataProperty: "creator", rootData: s }) || (e = e === null ? ns.errors : e.concat(ns.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.contributor !== void 0) {
                let a = t.contributor, n = r;
                os(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/contributor", parentData: t, parentDataProperty: "contributor", rootData: s }) || (e = e === null ? os.errors : e.concat(os.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.subject !== void 0) {
                let a = t.subject, n = r;
                Bl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subject", parentData: t, parentDataProperty: "subject", rootData: s }) || (e = e === null ? Bl.errors : e.concat(Bl.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.type !== void 0) {
                let a = t.type, n = r;
                ss(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/type", parentData: t, parentDataProperty: "type", rootData: s }) || (e = e === null ? ss.errors : e.concat(ss.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.format !== void 0) {
                let a = t.format, n = r;
                jl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/format", parentData: t, parentDataProperty: "format", rootData: s }) || (e = e === null ? jl.errors : e.concat(jl.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.relation !== void 0) {
                let a = t.relation, n = r;
                is(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/relation", parentData: t, parentDataProperty: "relation", rootData: s }) || (e = e === null ? is.errors : e.concat(is.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.coverage !== void 0) {
                let a = t.coverage, n = r;
                ls(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/coverage", parentData: t, parentDataProperty: "coverage", rootData: s }) || (e = e === null ? ls.errors : e.concat(ls.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.rights !== void 0) {
                let a = t.rights, n = r;
                cs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/rights", parentData: t, parentDataProperty: "rights", rootData: s }) || (e = e === null ? cs.errors : e.concat(cs.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["belongs-to-collection"] !== void 0) {
                let a = t["belongs-to-collection"], n = r;
                ps(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/belongs-to-collection", parentData: t, parentDataProperty: "belongs-to-collection", rootData: s }) || (e = e === null ? ps.errors : e.concat(ps.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["group-position"] !== void 0) {
                let a = t["group-position"], n = r;
                us(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/group-position", parentData: t, parentDataProperty: "group-position", rootData: s }) || (e = e === null ? us.errors : e.concat(us.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["page-progression-direction"] !== void 0) {
                let a = t["page-progression-direction"], n = r;
                ds(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/page-progression-direction", parentData: t, parentDataProperty: "page-progression-direction", rootData: s }) || (e = e === null ? ds.errors : e.concat(ds.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.ibooks !== void 0) {
                let a = t.ibooks, n = r;
                fs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ibooks", parentData: t, parentDataProperty: "ibooks", rootData: s }) || (e = e === null ? fs.errors : e.concat(fs.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["epub-metadata"] !== void 0) {
                let a = t["epub-metadata"], n = r;
                ms(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/epub-metadata", parentData: t, parentDataProperty: "epub-metadata", rootData: s }) || (e = e === null ? ms.errors : e.concat(ms.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["epub-subdirectory"] !== void 0) {
                let a = t["epub-subdirectory"], n = r;
                gs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/epub-subdirectory", parentData: t, parentDataProperty: "epub-subdirectory", rootData: s }) || (e = e === null ? gs.errors : e.concat(gs.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["epub-fonts"] !== void 0) {
                let a = t["epub-fonts"], n = r;
                hs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/epub-fonts", parentData: t, parentDataProperty: "epub-fonts", rootData: s }) || (e = e === null ? hs.errors : e.concat(hs.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["epub-chapter-level"] !== void 0) {
                let a = t["epub-chapter-level"], n = r;
                ys(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/epub-chapter-level", parentData: t, parentDataProperty: "epub-chapter-level", rootData: s }) || (e = e === null ? ys.errors : e.concat(ys.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["epub-cover-image"] !== void 0) {
                let a = t["epub-cover-image"], n = r;
                bs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/epub-cover-image", parentData: t, parentDataProperty: "epub-cover-image", rootData: s }) || (e = e === null ? bs.errors : e.concat(bs.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["fig-responsive"] !== void 0) {
                let a = t["fig-responsive"], n = r;
                cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-responsive", parentData: t, parentDataProperty: "fig-responsive", rootData: s }) || (e = e === null ? cn.errors : e.concat(cn.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.resources !== void 0) {
                let a = t.resources, n = r;
                pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resources", parentData: t, parentDataProperty: "resources", rootData: s }) || (e = e === null ? pn.errors : e.concat(pn.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["date-meta"] !== void 0) {
                let a = t["date-meta"], n = r;
                Ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date-meta", parentData: t, parentDataProperty: "date-meta", rootData: s }) || (e = e === null ? Ha.errors : e.concat(Ha.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["number-depth"] !== void 0) {
                let a = t["number-depth"], n = r;
                Da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-depth", parentData: t, parentDataProperty: "number-depth", rootData: s }) || (e = e === null ? Da.errors : e.concat(Da.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.css !== void 0) {
                let a = t.css, n = r;
                fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/css", parentData: t, parentDataProperty: "css", rootData: s }) || (e = e === null ? fn.errors : e.concat(fn.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["html-math-method"] !== void 0) {
                let a = t["html-math-method"], n = r;
                Wn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-math-method", parentData: t, parentDataProperty: "html-math-method", rootData: s }) || (e = e === null ? Wn.errors : e.concat(Wn.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["html-q-tags"] !== void 0) {
                let a = t["html-q-tags"], n = r;
                mn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-q-tags", parentData: t, parentDataProperty: "html-q-tags", rootData: s }) || (e = e === null ? mn.errors : e.concat(mn.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var Jr = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Jr = n === r;
              }
              if (t["toc-title"] !== void 0) {
                let a = t["toc-title"], n = r;
                La(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-title", parentData: t, parentDataProperty: "toc-title", rootData: s }) || (e = e === null ? La.errors : e.concat(La.errors), r = e.length);
                var Jr = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub2(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var io = ht === r;
            io && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub2(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var io = gt === r;
            if (io && L ? (L = false, M = [M, 1]) : io && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub2(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub2(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Sv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!O1.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub3(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|html_q_tags|htmlQTags|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", schema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|html_q_tags|htmlQTags|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))"', schema: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|html_q_tags|htmlQTags|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub3(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var an = n === r;
                if (!an) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub3(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var w = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var w = n === r;
              }
              if (t["code-fold"] !== void 0) {
                let a = t["code-fold"], n = r;
                Fa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-fold", parentData: t, parentDataProperty: "code-fold", rootData: s }) || (e = e === null ? Fa.errors : e.concat(Fa.errors), r = e.length);
                var w = n === r;
              }
              if (t["code-summary"] !== void 0) {
                let a = t["code-summary"], n = r;
                Aa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-summary", parentData: t, parentDataProperty: "code-summary", rootData: s }) || (e = e === null ? Aa.errors : e.concat(Aa.errors), r = e.length);
                var w = n === r;
              }
              if (t["code-overflow"] !== void 0) {
                let a = t["code-overflow"], n = r;
                Ba(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-overflow", parentData: t, parentDataProperty: "code-overflow", rootData: s }) || (e = e === null ? Ba.errors : e.concat(Ba.errors), r = e.length);
                var w = n === r;
              }
              if (t["code-line-numbers"] !== void 0) {
                let a = t["code-line-numbers"], n = r;
                ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-line-numbers", parentData: t, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
                var w = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var w = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var w = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var w = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var w = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var w = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var w = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var w = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var w = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var w = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var w = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var w = n === r;
              }
              if (t["abstract-title"] !== void 0) {
                let a = t["abstract-title"], n = r;
                to(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract-title", parentData: t, parentDataProperty: "abstract-title", rootData: s }) || (e = e === null ? to.errors : e.concat(to.errors), r = e.length);
                var w = n === r;
              }
              if (t["code-copy"] !== void 0) {
                let a = t["code-copy"], n = r;
                ln(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-copy", parentData: t, parentDataProperty: "code-copy", rootData: s }) || (e = e === null ? ln.errors : e.concat(ln.errors), r = e.length);
                var w = n === r;
              }
              if (t["highlight-style"] !== void 0) {
                let a = t["highlight-style"], n = r;
                _a(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/highlight-style", parentData: t, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
                var w = n === r;
              }
              if (t["syntax-definition"] !== void 0) {
                let a = t["syntax-definition"], n = r;
                va(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definition", parentData: t, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
                var w = n === r;
              }
              if (t["syntax-definitions"] !== void 0) {
                let a = t["syntax-definitions"], n = r;
                qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definitions", parentData: t, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
                var w = n === r;
              }
              if (t["indented-code-classes"] !== void 0) {
                let a = t["indented-code-classes"], n = r;
                Oa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indented-code-classes", parentData: t, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
                var w = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var w = n === r;
              }
              if (t.identifier !== void 0) {
                let a = t.identifier, n = r;
                as(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier", parentData: t, parentDataProperty: "identifier", rootData: s }) || (e = e === null ? as.errors : e.concat(as.errors), r = e.length);
                var w = n === r;
              }
              if (t.creator !== void 0) {
                let a = t.creator, n = r;
                ns(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/creator", parentData: t, parentDataProperty: "creator", rootData: s }) || (e = e === null ? ns.errors : e.concat(ns.errors), r = e.length);
                var w = n === r;
              }
              if (t.contributor !== void 0) {
                let a = t.contributor, n = r;
                os(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/contributor", parentData: t, parentDataProperty: "contributor", rootData: s }) || (e = e === null ? os.errors : e.concat(os.errors), r = e.length);
                var w = n === r;
              }
              if (t.subject !== void 0) {
                let a = t.subject, n = r;
                Bl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subject", parentData: t, parentDataProperty: "subject", rootData: s }) || (e = e === null ? Bl.errors : e.concat(Bl.errors), r = e.length);
                var w = n === r;
              }
              if (t.type !== void 0) {
                let a = t.type, n = r;
                ss(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/type", parentData: t, parentDataProperty: "type", rootData: s }) || (e = e === null ? ss.errors : e.concat(ss.errors), r = e.length);
                var w = n === r;
              }
              if (t.format !== void 0) {
                let a = t.format, n = r;
                jl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/format", parentData: t, parentDataProperty: "format", rootData: s }) || (e = e === null ? jl.errors : e.concat(jl.errors), r = e.length);
                var w = n === r;
              }
              if (t.relation !== void 0) {
                let a = t.relation, n = r;
                is(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/relation", parentData: t, parentDataProperty: "relation", rootData: s }) || (e = e === null ? is.errors : e.concat(is.errors), r = e.length);
                var w = n === r;
              }
              if (t.coverage !== void 0) {
                let a = t.coverage, n = r;
                ls(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/coverage", parentData: t, parentDataProperty: "coverage", rootData: s }) || (e = e === null ? ls.errors : e.concat(ls.errors), r = e.length);
                var w = n === r;
              }
              if (t.rights !== void 0) {
                let a = t.rights, n = r;
                cs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/rights", parentData: t, parentDataProperty: "rights", rootData: s }) || (e = e === null ? cs.errors : e.concat(cs.errors), r = e.length);
                var w = n === r;
              }
              if (t["belongs-to-collection"] !== void 0) {
                let a = t["belongs-to-collection"], n = r;
                ps(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/belongs-to-collection", parentData: t, parentDataProperty: "belongs-to-collection", rootData: s }) || (e = e === null ? ps.errors : e.concat(ps.errors), r = e.length);
                var w = n === r;
              }
              if (t["group-position"] !== void 0) {
                let a = t["group-position"], n = r;
                us(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/group-position", parentData: t, parentDataProperty: "group-position", rootData: s }) || (e = e === null ? us.errors : e.concat(us.errors), r = e.length);
                var w = n === r;
              }
              if (t["page-progression-direction"] !== void 0) {
                let a = t["page-progression-direction"], n = r;
                ds(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/page-progression-direction", parentData: t, parentDataProperty: "page-progression-direction", rootData: s }) || (e = e === null ? ds.errors : e.concat(ds.errors), r = e.length);
                var w = n === r;
              }
              if (t.ibooks !== void 0) {
                let a = t.ibooks, n = r;
                fs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ibooks", parentData: t, parentDataProperty: "ibooks", rootData: s }) || (e = e === null ? fs.errors : e.concat(fs.errors), r = e.length);
                var w = n === r;
              }
              if (t["epub-metadata"] !== void 0) {
                let a = t["epub-metadata"], n = r;
                ms(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/epub-metadata", parentData: t, parentDataProperty: "epub-metadata", rootData: s }) || (e = e === null ? ms.errors : e.concat(ms.errors), r = e.length);
                var w = n === r;
              }
              if (t["epub-subdirectory"] !== void 0) {
                let a = t["epub-subdirectory"], n = r;
                gs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/epub-subdirectory", parentData: t, parentDataProperty: "epub-subdirectory", rootData: s }) || (e = e === null ? gs.errors : e.concat(gs.errors), r = e.length);
                var w = n === r;
              }
              if (t["epub-fonts"] !== void 0) {
                let a = t["epub-fonts"], n = r;
                hs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/epub-fonts", parentData: t, parentDataProperty: "epub-fonts", rootData: s }) || (e = e === null ? hs.errors : e.concat(hs.errors), r = e.length);
                var w = n === r;
              }
              if (t["epub-chapter-level"] !== void 0) {
                let a = t["epub-chapter-level"], n = r;
                ys(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/epub-chapter-level", parentData: t, parentDataProperty: "epub-chapter-level", rootData: s }) || (e = e === null ? ys.errors : e.concat(ys.errors), r = e.length);
                var w = n === r;
              }
              if (t["epub-cover-image"] !== void 0) {
                let a = t["epub-cover-image"], n = r;
                bs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/epub-cover-image", parentData: t, parentDataProperty: "epub-cover-image", rootData: s }) || (e = e === null ? bs.errors : e.concat(bs.errors), r = e.length);
                var w = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var w = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var w = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var w = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var w = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var w = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var w = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var w = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var w = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var w = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var w = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var w = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var w = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var w = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var w = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var w = n === r;
              }
              if (t["fig-responsive"] !== void 0) {
                let a = t["fig-responsive"], n = r;
                cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-responsive", parentData: t, parentDataProperty: "fig-responsive", rootData: s }) || (e = e === null ? cn.errors : e.concat(cn.errors), r = e.length);
                var w = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var w = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var w = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var w = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var w = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var w = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var w = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var w = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var w = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var w = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var w = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var w = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var w = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var w = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var w = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var w = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var w = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var w = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var w = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var w = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var w = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var w = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var w = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var w = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var w = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var w = n === r;
              }
              if (t.resources !== void 0) {
                let a = t.resources, n = r;
                pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resources", parentData: t, parentDataProperty: "resources", rootData: s }) || (e = e === null ? pn.errors : e.concat(pn.errors), r = e.length);
                var w = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var w = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var w = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var w = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var w = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var w = n === r;
              }
              if (t["date-meta"] !== void 0) {
                let a = t["date-meta"], n = r;
                Ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date-meta", parentData: t, parentDataProperty: "date-meta", rootData: s }) || (e = e === null ? Ha.errors : e.concat(Ha.errors), r = e.length);
                var w = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var w = n === r;
              }
              if (t["number-depth"] !== void 0) {
                let a = t["number-depth"], n = r;
                Da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-depth", parentData: t, parentDataProperty: "number-depth", rootData: s }) || (e = e === null ? Da.errors : e.concat(Da.errors), r = e.length);
                var w = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var w = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var w = n === r;
              }
              if (t.css !== void 0) {
                let a = t.css, n = r;
                fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/css", parentData: t, parentDataProperty: "css", rootData: s }) || (e = e === null ? fn.errors : e.concat(fn.errors), r = e.length);
                var w = n === r;
              }
              if (t["html-math-method"] !== void 0) {
                let a = t["html-math-method"], n = r;
                Wn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-math-method", parentData: t, parentDataProperty: "html-math-method", rootData: s }) || (e = e === null ? Wn.errors : e.concat(Wn.errors), r = e.length);
                var w = n === r;
              }
              if (t["html-q-tags"] !== void 0) {
                let a = t["html-q-tags"], n = r;
                mn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-q-tags", parentData: t, parentDataProperty: "html-q-tags", rootData: s }) || (e = e === null ? mn.errors : e.concat(mn.errors), r = e.length);
                var w = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var w = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var w = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var w = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var w = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var w = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var w = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var w = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var w = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var w = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var w = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var w = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var w = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var w = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var w = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var w = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var w = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var w = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var w = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var w = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var w = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var w = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var w = n === r;
              }
              if (t["toc-title"] !== void 0) {
                let a = t["toc-title"], n = r;
                La(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-title", parentData: t, parentDataProperty: "toc-title", rootData: s }) || (e = e === null ? La.errors : e.concat(La.errors), r = e.length);
                var w = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub3(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Nt = ht === r;
            Nt && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub3(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Nt = gt === r;
            if (Nt && L ? (L = false, M = [M, 1]) : Nt && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eepub3(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^epub3(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Fv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Efb2(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Efb2(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var Y = n === r;
                if (!Y) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Efb2(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var N = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var N = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var N = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var N = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var N = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var N = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var N = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var N = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var N = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var N = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var N = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var N = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var N = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var N = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var N = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var N = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var N = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var N = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var N = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var N = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var N = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var N = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var N = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var N = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var N = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var N = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var N = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var N = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var N = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var N = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var N = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var N = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var N = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var N = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var N = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var N = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var N = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var N = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var N = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var N = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var N = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var N = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var N = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var N = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var N = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var N = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var N = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var N = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var N = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var N = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var N = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var N = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var N = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var N = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var N = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var N = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var N = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var N = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var N = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var N = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var N = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var N = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var N = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var N = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var N = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var N = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var N = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var N = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var N = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var N = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var N = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var N = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var N = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var N = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var N = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var N = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var N = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var N = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var N = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var N = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var N = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var N = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var N = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var N = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var N = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Efb2(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var ae = ht === r;
            ae && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Efb2(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var ae = gt === r;
            if (ae && L ? (L = false, M = [M, 1]) : ae && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Efb2(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^fb2(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Av.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!T0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Egfm(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|html_math_method|htmlMathMethod|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Egfm(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var nt = n === r;
                if (!nt) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Egfm(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var J = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var J = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var J = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var J = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var J = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var J = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var J = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var J = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var J = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var J = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var J = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var J = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var J = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var J = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var J = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var J = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var J = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var J = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var J = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var J = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var J = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var J = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var J = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var J = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var J = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var J = n === r;
              }
              if (t["reference-location"] !== void 0) {
                let a = t["reference-location"], n = r;
                Ca(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-location", parentData: t, parentDataProperty: "reference-location", rootData: s }) || (e = e === null ? Ca.errors : e.concat(Ca.errors), r = e.length);
                var J = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var J = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var J = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var J = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var J = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var J = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var J = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var J = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var J = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var J = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var J = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var J = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var J = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var J = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var J = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var J = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var J = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var J = n === r;
              }
              if (t["prefer-html"] !== void 0) {
                let a = t["prefer-html"], n = r;
                jo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/prefer-html", parentData: t, parentDataProperty: "prefer-html", rootData: s }) || (e = e === null ? jo.errors : e.concat(jo.errors), r = e.length);
                var J = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var J = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var J = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var J = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var J = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var J = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var J = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var J = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var J = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var J = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var J = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var J = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var J = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var J = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var J = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var J = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var J = n === r;
              }
              if (t["html-math-method"] !== void 0) {
                let a = t["html-math-method"], n = r;
                Wn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-math-method", parentData: t, parentDataProperty: "html-math-method", rootData: s }) || (e = e === null ? Wn.errors : e.concat(Wn.errors), r = e.length);
                var J = n === r;
              }
              if (t["identifier-prefix"] !== void 0) {
                let a = t["identifier-prefix"], n = r;
                Pa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier-prefix", parentData: t, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
                var J = n === r;
              }
              if (t.variant !== void 0) {
                let a = t.variant, n = r;
                Lo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variant", parentData: t, parentDataProperty: "variant", rootData: s }) || (e = e === null ? Lo.errors : e.concat(Lo.errors), r = e.length);
                var J = n === r;
              }
              if (t["markdown-headings"] !== void 0) {
                let a = t["markdown-headings"], n = r;
                ho(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/markdown-headings", parentData: t, parentDataProperty: "markdown-headings", rootData: s }) || (e = e === null ? ho.errors : e.concat(ho.errors), r = e.length);
                var J = n === r;
              }
              if (t["keep-yaml"] !== void 0) {
                let a = t["keep-yaml"], n = r;
                Mo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-yaml", parentData: t, parentDataProperty: "keep-yaml", rootData: s }) || (e = e === null ? Mo.errors : e.concat(Mo.errors), r = e.length);
                var J = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var J = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var J = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var J = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var J = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var J = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var J = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var J = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var J = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var J = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var J = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var J = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var J = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var J = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var J = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var J = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var J = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var J = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var J = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var J = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var J = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var J = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var J = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var J = n === r;
              }
              if (t["strip-comments"] !== void 0) {
                let a = t["strip-comments"], n = r;
                Ra(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-comments", parentData: t, parentDataProperty: "strip-comments", rootData: s }) || (e = e === null ? Ra.errors : e.concat(Ra.errors), r = e.length);
                var J = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var J = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var J = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var J = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var J = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Egfm(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var ot = ht === r;
            ot && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Egfm(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var ot = gt === r;
            if (ot && L ? (L = false, M = [M, 1]) : ot && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Egfm(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^gfm(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Bv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!z0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehaddock(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehaddock(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var le = n === r;
                if (!le) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehaddock(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var pe = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var pe = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var pe = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var pe = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var pe = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var pe = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var pe = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var pe = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var pe = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var pe = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var pe = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var pe = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var pe = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var pe = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var pe = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var pe = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var pe = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var pe = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var pe = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var pe = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var pe = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var pe = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var pe = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var pe = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var pe = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var pe = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var pe = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var pe = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var pe = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var pe = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var pe = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var pe = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var pe = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var pe = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var pe = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var pe = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var pe = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var pe = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var pe = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var pe = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var pe = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var pe = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var pe = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var pe = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var pe = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var pe = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var pe = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var pe = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var pe = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var pe = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var pe = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var pe = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var pe = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var pe = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var pe = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var pe = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var pe = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var pe = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var pe = n === r;
              }
              if (t["identifier-prefix"] !== void 0) {
                let a = t["identifier-prefix"], n = r;
                Pa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier-prefix", parentData: t, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
                var pe = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var pe = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var pe = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var pe = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var pe = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var pe = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var pe = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var pe = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var pe = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var pe = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var pe = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var pe = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var pe = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var pe = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var pe = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var pe = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var pe = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var pe = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var pe = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var pe = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var pe = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var pe = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var pe = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var pe = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var pe = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var pe = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var pe = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehaddock(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var v = ht === r;
            v && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehaddock(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var v = gt === r;
            if (v && L ? (L = false, M = [M, 1]) : v && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehaddock(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^haddock(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (jv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!P1.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_layout|pageLayout|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_location|citationLocation|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))", schema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_layout|pageLayout|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_location|citationLocation|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))"', schema: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_layout|pageLayout|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_location|citationLocation|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var E = n === r;
                if (!E) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var A = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var A = n === r;
              }
              if (t["code-fold"] !== void 0) {
                let a = t["code-fold"], n = r;
                Fa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-fold", parentData: t, parentDataProperty: "code-fold", rootData: s }) || (e = e === null ? Fa.errors : e.concat(Fa.errors), r = e.length);
                var A = n === r;
              }
              if (t["code-summary"] !== void 0) {
                let a = t["code-summary"], n = r;
                Aa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-summary", parentData: t, parentDataProperty: "code-summary", rootData: s }) || (e = e === null ? Aa.errors : e.concat(Aa.errors), r = e.length);
                var A = n === r;
              }
              if (t["code-overflow"] !== void 0) {
                let a = t["code-overflow"], n = r;
                Ba(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-overflow", parentData: t, parentDataProperty: "code-overflow", rootData: s }) || (e = e === null ? Ba.errors : e.concat(Ba.errors), r = e.length);
                var A = n === r;
              }
              if (t["code-line-numbers"] !== void 0) {
                let a = t["code-line-numbers"], n = r;
                ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-line-numbers", parentData: t, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
                var A = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var A = n === r;
              }
              if (t["cap-location"] !== void 0) {
                let a = t["cap-location"], n = r;
                ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cap-location", parentData: t, parentDataProperty: "cap-location", rootData: s }) || (e = e === null ? ja.errors : e.concat(ja.errors), r = e.length);
                var A = n === r;
              }
              if (t["fig-cap-location"] !== void 0) {
                let a = t["fig-cap-location"], n = r;
                Ea(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-cap-location", parentData: t, parentDataProperty: "fig-cap-location", rootData: s }) || (e = e === null ? Ea.errors : e.concat(Ea.errors), r = e.length);
                var A = n === r;
              }
              if (t["tbl-cap-location"] !== void 0) {
                let a = t["tbl-cap-location"], n = r;
                Na(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-cap-location", parentData: t, parentDataProperty: "tbl-cap-location", rootData: s }) || (e = e === null ? Na.errors : e.concat(Na.errors), r = e.length);
                var A = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var A = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var A = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var A = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var A = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var A = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var A = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var A = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var A = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var A = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var A = n === r;
              }
              if (t["abstract-title"] !== void 0) {
                let a = t["abstract-title"], n = r;
                to(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract-title", parentData: t, parentDataProperty: "abstract-title", rootData: s }) || (e = e === null ? to.errors : e.concat(to.errors), r = e.length);
                var A = n === r;
              }
              if (t.doi !== void 0) {
                let a = t.doi, n = r;
                Jo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/doi", parentData: t, parentDataProperty: "doi", rootData: s }) || (e = e === null ? Jo.errors : e.concat(Jo.errors), r = e.length);
                var A = n === r;
              }
              if (t["code-copy"] !== void 0) {
                let a = t["code-copy"], n = r;
                ln(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-copy", parentData: t, parentDataProperty: "code-copy", rootData: s }) || (e = e === null ? ln.errors : e.concat(ln.errors), r = e.length);
                var A = n === r;
              }
              if (t["code-link"] !== void 0) {
                let a = t["code-link"], n = r;
                On(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-link", parentData: t, parentDataProperty: "code-link", rootData: s }) || (e = e === null ? On.errors : e.concat(On.errors), r = e.length);
                var A = n === r;
              }
              if (t["code-tools"] !== void 0) {
                let a = t["code-tools"], n = r;
                Go(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-tools", parentData: t, parentDataProperty: "code-tools", rootData: s }) || (e = e === null ? Go.errors : e.concat(Go.errors), r = e.length);
                var A = n === r;
              }
              if (t["highlight-style"] !== void 0) {
                let a = t["highlight-style"], n = r;
                _a(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/highlight-style", parentData: t, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
                var A = n === r;
              }
              if (t["syntax-definition"] !== void 0) {
                let a = t["syntax-definition"], n = r;
                va(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definition", parentData: t, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
                var A = n === r;
              }
              if (t["syntax-definitions"] !== void 0) {
                let a = t["syntax-definitions"], n = r;
                qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definitions", parentData: t, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
                var A = n === r;
              }
              if (t["indented-code-classes"] !== void 0) {
                let a = t["indented-code-classes"], n = r;
                Oa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indented-code-classes", parentData: t, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
                var A = n === r;
              }
              if (t.fontcolor !== void 0) {
                let a = t.fontcolor, n = r;
                Pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontcolor", parentData: t, parentDataProperty: "fontcolor", rootData: s }) || (e = e === null ? Pn.errors : e.concat(Pn.errors), r = e.length);
                var A = n === r;
              }
              if (t.linkcolor !== void 0) {
                let a = t.linkcolor, n = r;
                Ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linkcolor", parentData: t, parentDataProperty: "linkcolor", rootData: s }) || (e = e === null ? Ua.errors : e.concat(Ua.errors), r = e.length);
                var A = n === r;
              }
              if (t.monobackgroundcolor !== void 0) {
                let a = t.monobackgroundcolor, n = r;
                xn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monobackgroundcolor", parentData: t, parentDataProperty: "monobackgroundcolor", rootData: s }) || (e = e === null ? xn.errors : e.concat(xn.errors), r = e.length);
                var A = n === r;
              }
              if (t.backgroundcolor !== void 0) {
                let a = t.backgroundcolor, n = r;
                kn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/backgroundcolor", parentData: t, parentDataProperty: "backgroundcolor", rootData: s }) || (e = e === null ? kn.errors : e.concat(kn.errors), r = e.length);
                var A = n === r;
              }
              if (t.comments !== void 0) {
                let a = t.comments, n = r;
                Dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/comments", parentData: t, parentDataProperty: "comments", rootData: s }) || (e = e === null ? Dn.errors : e.concat(Dn.errors), r = e.length);
                var A = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var A = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var A = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var A = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var A = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var A = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var A = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var A = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var A = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var A = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var A = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var A = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var A = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var A = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var A = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var A = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var A = n === r;
              }
              if (t["fig-responsive"] !== void 0) {
                let a = t["fig-responsive"], n = r;
                cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-responsive", parentData: t, parentDataProperty: "fig-responsive", rootData: s }) || (e = e === null ? cn.errors : e.concat(cn.errors), r = e.length);
                var A = n === r;
              }
              if (t.mainfont !== void 0) {
                let a = t.mainfont, n = r;
                Ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mainfont", parentData: t, parentDataProperty: "mainfont", rootData: s }) || (e = e === null ? Ja.errors : e.concat(Ja.errors), r = e.length);
                var A = n === r;
              }
              if (t.monofont !== void 0) {
                let a = t.monofont, n = r;
                Qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monofont", parentData: t, parentDataProperty: "monofont", rootData: s }) || (e = e === null ? Qa.errors : e.concat(Qa.errors), r = e.length);
                var A = n === r;
              }
              if (t.fontsize !== void 0) {
                let a = t.fontsize, n = r;
                Ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontsize", parentData: t, parentDataProperty: "fontsize", rootData: s }) || (e = e === null ? Ga.errors : e.concat(Ga.errors), r = e.length);
                var A = n === r;
              }
              if (t.linestretch !== void 0) {
                let a = t.linestretch, n = r;
                Ya(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linestretch", parentData: t, parentDataProperty: "linestretch", rootData: s }) || (e = e === null ? Ya.errors : e.concat(Ya.errors), r = e.length);
                var A = n === r;
              }
              if (t["footnotes-hover"] !== void 0) {
                let a = t["footnotes-hover"], n = r;
                wn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/footnotes-hover", parentData: t, parentDataProperty: "footnotes-hover", rootData: s }) || (e = e === null ? wn.errors : e.concat(wn.errors), r = e.length);
                var A = n === r;
              }
              if (t["reference-location"] !== void 0) {
                let a = t["reference-location"], n = r;
                Ca(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-location", parentData: t, parentDataProperty: "reference-location", rootData: s }) || (e = e === null ? Ca.errors : e.concat(Ca.errors), r = e.length);
                var A = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var A = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var A = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var A = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var A = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var A = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var A = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var A = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var A = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var A = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var A = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var A = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var A = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var A = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var A = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var A = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var A = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var A = n === r;
              }
              if (t["keep-source"] !== void 0) {
                let a = t["keep-source"], n = r;
                As(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-source", parentData: t, parentDataProperty: "keep-source", rootData: s }) || (e = e === null ? As.errors : e.concat(As.errors), r = e.length);
                var A = n === r;
              }
              if (t["keep-hidden"] !== void 0) {
                let a = t["keep-hidden"], n = r;
                Bs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-hidden", parentData: t, parentDataProperty: "keep-hidden", rootData: s }) || (e = e === null ? Bs.errors : e.concat(Bs.errors), r = e.length);
                var A = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var A = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var A = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var A = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var A = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var A = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var A = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var A = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var A = n === r;
              }
              if (t.resources !== void 0) {
                let a = t.resources, n = r;
                pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resources", parentData: t, parentDataProperty: "resources", rootData: s }) || (e = e === null ? pn.errors : e.concat(pn.errors), r = e.length);
                var A = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var A = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var A = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var A = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var A = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var A = n === r;
              }
              if (t.classoption !== void 0) {
                let a = t.classoption, n = r;
                un(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/classoption", parentData: t, parentDataProperty: "classoption", rootData: s }) || (e = e === null ? un.errors : e.concat(un.errors), r = e.length);
                var A = n === r;
              }
              if (t["page-layout"] !== void 0) {
                let a = t["page-layout"], n = r;
                Cs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/page-layout", parentData: t, parentDataProperty: "page-layout", rootData: s }) || (e = e === null ? Cs.errors : e.concat(Cs.errors), r = e.length);
                var A = n === r;
              }
              if (t["max-width"] !== void 0) {
                let a = t["max-width"], n = r;
                $n(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/max-width", parentData: t, parentDataProperty: "max-width", rootData: s }) || (e = e === null ? $n.errors : e.concat($n.errors), r = e.length);
                var A = n === r;
              }
              if (t["margin-left"] !== void 0) {
                let a = t["margin-left"], n = r;
                Xa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-left", parentData: t, parentDataProperty: "margin-left", rootData: s }) || (e = e === null ? Xa.errors : e.concat(Xa.errors), r = e.length);
                var A = n === r;
              }
              if (t["margin-right"] !== void 0) {
                let a = t["margin-right"], n = r;
                Za(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-right", parentData: t, parentDataProperty: "margin-right", rootData: s }) || (e = e === null ? Za.errors : e.concat(Za.errors), r = e.length);
                var A = n === r;
              }
              if (t["margin-top"] !== void 0) {
                let a = t["margin-top"], n = r;
                en(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-top", parentData: t, parentDataProperty: "margin-top", rootData: s }) || (e = e === null ? en.errors : e.concat(en.errors), r = e.length);
                var A = n === r;
              }
              if (t["margin-bottom"] !== void 0) {
                let a = t["margin-bottom"], n = r;
                rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-bottom", parentData: t, parentDataProperty: "margin-bottom", rootData: s }) || (e = e === null ? rn.errors : e.concat(rn.errors), r = e.length);
                var A = n === r;
              }
              if (t["link-external-icon"] !== void 0) {
                let a = t["link-external-icon"], n = r;
                No(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/link-external-icon", parentData: t, parentDataProperty: "link-external-icon", rootData: s }) || (e = e === null ? No.errors : e.concat(No.errors), r = e.length);
                var A = n === r;
              }
              if (t["link-external-newwindow"] !== void 0) {
                let a = t["link-external-newwindow"], n = r;
                Co(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/link-external-newwindow", parentData: t, parentDataProperty: "link-external-newwindow", rootData: s }) || (e = e === null ? Co.errors : e.concat(Co.errors), r = e.length);
                var A = n === r;
              }
              if (t["link-external-filter"] !== void 0) {
                let a = t["link-external-filter"], n = r;
                Io(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/link-external-filter", parentData: t, parentDataProperty: "link-external-filter", rootData: s }) || (e = e === null ? Io.errors : e.concat(Io.errors), r = e.length);
                var A = n === r;
              }
              if (t.keywords !== void 0) {
                let a = t.keywords, n = r;
                Ia(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keywords", parentData: t, parentDataProperty: "keywords", rootData: s }) || (e = e === null ? Ia.errors : e.concat(Ia.errors), r = e.length);
                var A = n === r;
              }
              if (t.pagetitle !== void 0) {
                let a = t.pagetitle, n = r;
                Sn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pagetitle", parentData: t, parentDataProperty: "pagetitle", rootData: s }) || (e = e === null ? Sn.errors : e.concat(Sn.errors), r = e.length);
                var A = n === r;
              }
              if (t["title-prefix"] !== void 0) {
                let a = t["title-prefix"], n = r;
                Fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title-prefix", parentData: t, parentDataProperty: "title-prefix", rootData: s }) || (e = e === null ? Fn.errors : e.concat(Fn.errors), r = e.length);
                var A = n === r;
              }
              if (t["description-meta"] !== void 0) {
                let a = t["description-meta"], n = r;
                An(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/description-meta", parentData: t, parentDataProperty: "description-meta", rootData: s }) || (e = e === null ? An.errors : e.concat(An.errors), r = e.length);
                var A = n === r;
              }
              if (t["author-meta"] !== void 0) {
                let a = t["author-meta"], n = r;
                dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author-meta", parentData: t, parentDataProperty: "author-meta", rootData: s }) || (e = e === null ? dn.errors : e.concat(dn.errors), r = e.length);
                var A = n === r;
              }
              if (t["date-meta"] !== void 0) {
                let a = t["date-meta"], n = r;
                Ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date-meta", parentData: t, parentDataProperty: "date-meta", rootData: s }) || (e = e === null ? Ha.errors : e.concat(Ha.errors), r = e.length);
                var A = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var A = n === r;
              }
              if (t["number-depth"] !== void 0) {
                let a = t["number-depth"], n = r;
                Da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-depth", parentData: t, parentDataProperty: "number-depth", rootData: s }) || (e = e === null ? Da.errors : e.concat(Da.errors), r = e.length);
                var A = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var A = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var A = n === r;
              }
              if (t.theme !== void 0) {
                let a = t.theme, n = r;
                Ho(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/theme", parentData: t, parentDataProperty: "theme", rootData: s }) || (e = e === null ? Ho.errors : e.concat(Ho.errors), r = e.length);
                var A = n === r;
              }
              if (t.minimal !== void 0) {
                let a = t.minimal, n = r;
                Rs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/minimal", parentData: t, parentDataProperty: "minimal", rootData: s }) || (e = e === null ? Rs.errors : e.concat(Rs.errors), r = e.length);
                var A = n === r;
              }
              if (t["document-css"] !== void 0) {
                let a = t["document-css"], n = r;
                jn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/document-css", parentData: t, parentDataProperty: "document-css", rootData: s }) || (e = e === null ? jn.errors : e.concat(jn.errors), r = e.length);
                var A = n === r;
              }
              if (t.css !== void 0) {
                let a = t.css, n = r;
                fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/css", parentData: t, parentDataProperty: "css", rootData: s }) || (e = e === null ? fn.errors : e.concat(fn.errors), r = e.length);
                var A = n === r;
              }
              if (t["anchor-sections"] !== void 0) {
                let a = t["anchor-sections"], n = r;
                Vs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/anchor-sections", parentData: t, parentDataProperty: "anchor-sections", rootData: s }) || (e = e === null ? Vs.errors : e.concat(Vs.errors), r = e.length);
                var A = n === r;
              }
              if (t["html-math-method"] !== void 0) {
                let a = t["html-math-method"], n = r;
                Wn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-math-method", parentData: t, parentDataProperty: "html-math-method", rootData: s }) || (e = e === null ? Wn.errors : e.concat(Wn.errors), r = e.length);
                var A = n === r;
              }
              if (t["section-divs"] !== void 0) {
                let a = t["section-divs"], n = r;
                En(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/section-divs", parentData: t, parentDataProperty: "section-divs", rootData: s }) || (e = e === null ? En.errors : e.concat(En.errors), r = e.length);
                var A = n === r;
              }
              if (t["identifier-prefix"] !== void 0) {
                let a = t["identifier-prefix"], n = r;
                Pa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier-prefix", parentData: t, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
                var A = n === r;
              }
              if (t["email-obfuscation"] !== void 0) {
                let a = t["email-obfuscation"], n = r;
                Nn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/email-obfuscation", parentData: t, parentDataProperty: "email-obfuscation", rootData: s }) || (e = e === null ? Nn.errors : e.concat(Nn.errors), r = e.length);
                var A = n === r;
              }
              if (t["html-q-tags"] !== void 0) {
                let a = t["html-q-tags"], n = r;
                mn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-q-tags", parentData: t, parentDataProperty: "html-q-tags", rootData: s }) || (e = e === null ? mn.errors : e.concat(mn.errors), r = e.length);
                var A = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var A = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var A = n === r;
              }
              if (t["citations-hover"] !== void 0) {
                let a = t["citations-hover"], n = r;
                Cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citations-hover", parentData: t, parentDataProperty: "citations-hover", rootData: s }) || (e = e === null ? Cn.errors : e.concat(Cn.errors), r = e.length);
                var A = n === r;
              }
              if (t["citation-location"] !== void 0) {
                let a = t["citation-location"], n = r;
                Us(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-location", parentData: t, parentDataProperty: "citation-location", rootData: s }) || (e = e === null ? Us.errors : e.concat(Us.errors), r = e.length);
                var A = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var A = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var A = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var A = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var A = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var A = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var A = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var A = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var A = n === r;
              }
              if (t["self-contained"] !== void 0) {
                let a = t["self-contained"], n = r;
                In(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/self-contained", parentData: t, parentDataProperty: "self-contained", rootData: s }) || (e = e === null ? In.errors : e.concat(In.errors), r = e.length);
                var A = n === r;
              }
              if (t["self-contained-math"] !== void 0) {
                let a = t["self-contained-math"], n = r;
                Hn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/self-contained-math", parentData: t, parentDataProperty: "self-contained-math", rootData: s }) || (e = e === null ? Hn.errors : e.concat(Hn.errors), r = e.length);
                var A = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var A = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var A = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var A = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var A = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var A = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var A = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var A = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var A = n === r;
              }
              if (t["strip-comments"] !== void 0) {
                let a = t["strip-comments"], n = r;
                Ra(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-comments", parentData: t, parentDataProperty: "strip-comments", rootData: s }) || (e = e === null ? Ra.errors : e.concat(Ra.errors), r = e.length);
                var A = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var A = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var A = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var A = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var A = n === r;
              }
              if (t["toc-location"] !== void 0) {
                let a = t["toc-location"], n = r;
                Xs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-location", parentData: t, parentDataProperty: "toc-location", rootData: s }) || (e = e === null ? Xs.errors : e.concat(Xs.errors), r = e.length);
                var A = n === r;
              }
              if (t["toc-title"] !== void 0) {
                let a = t["toc-title"], n = r;
                La(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-title", parentData: t, parentDataProperty: "toc-title", rootData: s }) || (e = e === null ? La.errors : e.concat(La.errors), r = e.length);
                var A = n === r;
              }
              if (t.search !== void 0) {
                let a = t.search, n = r;
                ri(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/search", parentData: t, parentDataProperty: "search", rootData: s }) || (e = e === null ? ri.errors : e.concat(ri.errors), r = e.length);
                var A = n === r;
              }
              if (t.aliases !== void 0) {
                let a = t.aliases, n = r;
                ti(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/aliases", parentData: t, parentDataProperty: "aliases", rootData: s }) || (e = e === null ? ti.errors : e.concat(ti.errors), r = e.length);
                var A = n === r;
              }
              if (t.image !== void 0) {
                let a = t.image, n = r;
                ai(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/image", parentData: t, parentDataProperty: "image", rootData: s }) || (e = e === null ? ai.errors : e.concat(ai.errors), r = e.length);
                var A = n === r;
              }
              if (t["image-height"] !== void 0) {
                let a = t["image-height"], n = r;
                ni(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/image-height", parentData: t, parentDataProperty: "image-height", rootData: s }) || (e = e === null ? ni.errors : e.concat(ni.errors), r = e.length);
                var A = n === r;
              }
              if (t["image-width"] !== void 0) {
                let a = t["image-width"], n = r;
                oi(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/image-width", parentData: t, parentDataProperty: "image-width", rootData: s }) || (e = e === null ? oi.errors : e.concat(oi.errors), r = e.length);
                var A = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Z = ht === r;
            Z && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Z = gt === r;
            if (Z && L ? (L = false, M = [M, 1]) : Z && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^html(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Ev.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!P1.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml4(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_layout|pageLayout|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_location|citationLocation|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))", schema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_layout|pageLayout|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_location|citationLocation|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))"', schema: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_layout|pageLayout|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_location|citationLocation|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml4(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var T = n === r;
                if (!T) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml4(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var U = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var U = n === r;
              }
              if (t["code-fold"] !== void 0) {
                let a = t["code-fold"], n = r;
                Fa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-fold", parentData: t, parentDataProperty: "code-fold", rootData: s }) || (e = e === null ? Fa.errors : e.concat(Fa.errors), r = e.length);
                var U = n === r;
              }
              if (t["code-summary"] !== void 0) {
                let a = t["code-summary"], n = r;
                Aa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-summary", parentData: t, parentDataProperty: "code-summary", rootData: s }) || (e = e === null ? Aa.errors : e.concat(Aa.errors), r = e.length);
                var U = n === r;
              }
              if (t["code-overflow"] !== void 0) {
                let a = t["code-overflow"], n = r;
                Ba(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-overflow", parentData: t, parentDataProperty: "code-overflow", rootData: s }) || (e = e === null ? Ba.errors : e.concat(Ba.errors), r = e.length);
                var U = n === r;
              }
              if (t["code-line-numbers"] !== void 0) {
                let a = t["code-line-numbers"], n = r;
                ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-line-numbers", parentData: t, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
                var U = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var U = n === r;
              }
              if (t["cap-location"] !== void 0) {
                let a = t["cap-location"], n = r;
                ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cap-location", parentData: t, parentDataProperty: "cap-location", rootData: s }) || (e = e === null ? ja.errors : e.concat(ja.errors), r = e.length);
                var U = n === r;
              }
              if (t["fig-cap-location"] !== void 0) {
                let a = t["fig-cap-location"], n = r;
                Ea(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-cap-location", parentData: t, parentDataProperty: "fig-cap-location", rootData: s }) || (e = e === null ? Ea.errors : e.concat(Ea.errors), r = e.length);
                var U = n === r;
              }
              if (t["tbl-cap-location"] !== void 0) {
                let a = t["tbl-cap-location"], n = r;
                Na(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-cap-location", parentData: t, parentDataProperty: "tbl-cap-location", rootData: s }) || (e = e === null ? Na.errors : e.concat(Na.errors), r = e.length);
                var U = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var U = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var U = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var U = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var U = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var U = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var U = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var U = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var U = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var U = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var U = n === r;
              }
              if (t["abstract-title"] !== void 0) {
                let a = t["abstract-title"], n = r;
                to(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract-title", parentData: t, parentDataProperty: "abstract-title", rootData: s }) || (e = e === null ? to.errors : e.concat(to.errors), r = e.length);
                var U = n === r;
              }
              if (t.doi !== void 0) {
                let a = t.doi, n = r;
                Jo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/doi", parentData: t, parentDataProperty: "doi", rootData: s }) || (e = e === null ? Jo.errors : e.concat(Jo.errors), r = e.length);
                var U = n === r;
              }
              if (t["code-copy"] !== void 0) {
                let a = t["code-copy"], n = r;
                ln(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-copy", parentData: t, parentDataProperty: "code-copy", rootData: s }) || (e = e === null ? ln.errors : e.concat(ln.errors), r = e.length);
                var U = n === r;
              }
              if (t["code-link"] !== void 0) {
                let a = t["code-link"], n = r;
                On(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-link", parentData: t, parentDataProperty: "code-link", rootData: s }) || (e = e === null ? On.errors : e.concat(On.errors), r = e.length);
                var U = n === r;
              }
              if (t["code-tools"] !== void 0) {
                let a = t["code-tools"], n = r;
                Go(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-tools", parentData: t, parentDataProperty: "code-tools", rootData: s }) || (e = e === null ? Go.errors : e.concat(Go.errors), r = e.length);
                var U = n === r;
              }
              if (t["highlight-style"] !== void 0) {
                let a = t["highlight-style"], n = r;
                _a(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/highlight-style", parentData: t, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
                var U = n === r;
              }
              if (t["syntax-definition"] !== void 0) {
                let a = t["syntax-definition"], n = r;
                va(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definition", parentData: t, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
                var U = n === r;
              }
              if (t["syntax-definitions"] !== void 0) {
                let a = t["syntax-definitions"], n = r;
                qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definitions", parentData: t, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
                var U = n === r;
              }
              if (t["indented-code-classes"] !== void 0) {
                let a = t["indented-code-classes"], n = r;
                Oa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indented-code-classes", parentData: t, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
                var U = n === r;
              }
              if (t.fontcolor !== void 0) {
                let a = t.fontcolor, n = r;
                Pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontcolor", parentData: t, parentDataProperty: "fontcolor", rootData: s }) || (e = e === null ? Pn.errors : e.concat(Pn.errors), r = e.length);
                var U = n === r;
              }
              if (t.linkcolor !== void 0) {
                let a = t.linkcolor, n = r;
                Ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linkcolor", parentData: t, parentDataProperty: "linkcolor", rootData: s }) || (e = e === null ? Ua.errors : e.concat(Ua.errors), r = e.length);
                var U = n === r;
              }
              if (t.monobackgroundcolor !== void 0) {
                let a = t.monobackgroundcolor, n = r;
                xn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monobackgroundcolor", parentData: t, parentDataProperty: "monobackgroundcolor", rootData: s }) || (e = e === null ? xn.errors : e.concat(xn.errors), r = e.length);
                var U = n === r;
              }
              if (t.backgroundcolor !== void 0) {
                let a = t.backgroundcolor, n = r;
                kn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/backgroundcolor", parentData: t, parentDataProperty: "backgroundcolor", rootData: s }) || (e = e === null ? kn.errors : e.concat(kn.errors), r = e.length);
                var U = n === r;
              }
              if (t.comments !== void 0) {
                let a = t.comments, n = r;
                Dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/comments", parentData: t, parentDataProperty: "comments", rootData: s }) || (e = e === null ? Dn.errors : e.concat(Dn.errors), r = e.length);
                var U = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var U = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var U = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var U = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var U = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var U = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var U = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var U = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var U = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var U = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var U = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var U = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var U = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var U = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var U = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var U = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var U = n === r;
              }
              if (t["fig-responsive"] !== void 0) {
                let a = t["fig-responsive"], n = r;
                cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-responsive", parentData: t, parentDataProperty: "fig-responsive", rootData: s }) || (e = e === null ? cn.errors : e.concat(cn.errors), r = e.length);
                var U = n === r;
              }
              if (t.mainfont !== void 0) {
                let a = t.mainfont, n = r;
                Ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mainfont", parentData: t, parentDataProperty: "mainfont", rootData: s }) || (e = e === null ? Ja.errors : e.concat(Ja.errors), r = e.length);
                var U = n === r;
              }
              if (t.monofont !== void 0) {
                let a = t.monofont, n = r;
                Qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monofont", parentData: t, parentDataProperty: "monofont", rootData: s }) || (e = e === null ? Qa.errors : e.concat(Qa.errors), r = e.length);
                var U = n === r;
              }
              if (t.fontsize !== void 0) {
                let a = t.fontsize, n = r;
                Ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontsize", parentData: t, parentDataProperty: "fontsize", rootData: s }) || (e = e === null ? Ga.errors : e.concat(Ga.errors), r = e.length);
                var U = n === r;
              }
              if (t.linestretch !== void 0) {
                let a = t.linestretch, n = r;
                Ya(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linestretch", parentData: t, parentDataProperty: "linestretch", rootData: s }) || (e = e === null ? Ya.errors : e.concat(Ya.errors), r = e.length);
                var U = n === r;
              }
              if (t["footnotes-hover"] !== void 0) {
                let a = t["footnotes-hover"], n = r;
                wn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/footnotes-hover", parentData: t, parentDataProperty: "footnotes-hover", rootData: s }) || (e = e === null ? wn.errors : e.concat(wn.errors), r = e.length);
                var U = n === r;
              }
              if (t["reference-location"] !== void 0) {
                let a = t["reference-location"], n = r;
                Ca(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-location", parentData: t, parentDataProperty: "reference-location", rootData: s }) || (e = e === null ? Ca.errors : e.concat(Ca.errors), r = e.length);
                var U = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var U = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var U = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var U = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var U = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var U = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var U = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var U = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var U = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var U = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var U = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var U = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var U = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var U = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var U = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var U = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var U = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var U = n === r;
              }
              if (t["keep-source"] !== void 0) {
                let a = t["keep-source"], n = r;
                As(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-source", parentData: t, parentDataProperty: "keep-source", rootData: s }) || (e = e === null ? As.errors : e.concat(As.errors), r = e.length);
                var U = n === r;
              }
              if (t["keep-hidden"] !== void 0) {
                let a = t["keep-hidden"], n = r;
                Bs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-hidden", parentData: t, parentDataProperty: "keep-hidden", rootData: s }) || (e = e === null ? Bs.errors : e.concat(Bs.errors), r = e.length);
                var U = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var U = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var U = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var U = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var U = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var U = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var U = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var U = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var U = n === r;
              }
              if (t.resources !== void 0) {
                let a = t.resources, n = r;
                pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resources", parentData: t, parentDataProperty: "resources", rootData: s }) || (e = e === null ? pn.errors : e.concat(pn.errors), r = e.length);
                var U = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var U = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var U = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var U = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var U = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var U = n === r;
              }
              if (t.classoption !== void 0) {
                let a = t.classoption, n = r;
                un(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/classoption", parentData: t, parentDataProperty: "classoption", rootData: s }) || (e = e === null ? un.errors : e.concat(un.errors), r = e.length);
                var U = n === r;
              }
              if (t["page-layout"] !== void 0) {
                let a = t["page-layout"], n = r;
                Cs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/page-layout", parentData: t, parentDataProperty: "page-layout", rootData: s }) || (e = e === null ? Cs.errors : e.concat(Cs.errors), r = e.length);
                var U = n === r;
              }
              if (t["max-width"] !== void 0) {
                let a = t["max-width"], n = r;
                $n(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/max-width", parentData: t, parentDataProperty: "max-width", rootData: s }) || (e = e === null ? $n.errors : e.concat($n.errors), r = e.length);
                var U = n === r;
              }
              if (t["margin-left"] !== void 0) {
                let a = t["margin-left"], n = r;
                Xa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-left", parentData: t, parentDataProperty: "margin-left", rootData: s }) || (e = e === null ? Xa.errors : e.concat(Xa.errors), r = e.length);
                var U = n === r;
              }
              if (t["margin-right"] !== void 0) {
                let a = t["margin-right"], n = r;
                Za(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-right", parentData: t, parentDataProperty: "margin-right", rootData: s }) || (e = e === null ? Za.errors : e.concat(Za.errors), r = e.length);
                var U = n === r;
              }
              if (t["margin-top"] !== void 0) {
                let a = t["margin-top"], n = r;
                en(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-top", parentData: t, parentDataProperty: "margin-top", rootData: s }) || (e = e === null ? en.errors : e.concat(en.errors), r = e.length);
                var U = n === r;
              }
              if (t["margin-bottom"] !== void 0) {
                let a = t["margin-bottom"], n = r;
                rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-bottom", parentData: t, parentDataProperty: "margin-bottom", rootData: s }) || (e = e === null ? rn.errors : e.concat(rn.errors), r = e.length);
                var U = n === r;
              }
              if (t["link-external-icon"] !== void 0) {
                let a = t["link-external-icon"], n = r;
                No(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/link-external-icon", parentData: t, parentDataProperty: "link-external-icon", rootData: s }) || (e = e === null ? No.errors : e.concat(No.errors), r = e.length);
                var U = n === r;
              }
              if (t["link-external-newwindow"] !== void 0) {
                let a = t["link-external-newwindow"], n = r;
                Co(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/link-external-newwindow", parentData: t, parentDataProperty: "link-external-newwindow", rootData: s }) || (e = e === null ? Co.errors : e.concat(Co.errors), r = e.length);
                var U = n === r;
              }
              if (t["link-external-filter"] !== void 0) {
                let a = t["link-external-filter"], n = r;
                Io(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/link-external-filter", parentData: t, parentDataProperty: "link-external-filter", rootData: s }) || (e = e === null ? Io.errors : e.concat(Io.errors), r = e.length);
                var U = n === r;
              }
              if (t.keywords !== void 0) {
                let a = t.keywords, n = r;
                Ia(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keywords", parentData: t, parentDataProperty: "keywords", rootData: s }) || (e = e === null ? Ia.errors : e.concat(Ia.errors), r = e.length);
                var U = n === r;
              }
              if (t.pagetitle !== void 0) {
                let a = t.pagetitle, n = r;
                Sn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pagetitle", parentData: t, parentDataProperty: "pagetitle", rootData: s }) || (e = e === null ? Sn.errors : e.concat(Sn.errors), r = e.length);
                var U = n === r;
              }
              if (t["title-prefix"] !== void 0) {
                let a = t["title-prefix"], n = r;
                Fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title-prefix", parentData: t, parentDataProperty: "title-prefix", rootData: s }) || (e = e === null ? Fn.errors : e.concat(Fn.errors), r = e.length);
                var U = n === r;
              }
              if (t["description-meta"] !== void 0) {
                let a = t["description-meta"], n = r;
                An(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/description-meta", parentData: t, parentDataProperty: "description-meta", rootData: s }) || (e = e === null ? An.errors : e.concat(An.errors), r = e.length);
                var U = n === r;
              }
              if (t["author-meta"] !== void 0) {
                let a = t["author-meta"], n = r;
                dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author-meta", parentData: t, parentDataProperty: "author-meta", rootData: s }) || (e = e === null ? dn.errors : e.concat(dn.errors), r = e.length);
                var U = n === r;
              }
              if (t["date-meta"] !== void 0) {
                let a = t["date-meta"], n = r;
                Ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date-meta", parentData: t, parentDataProperty: "date-meta", rootData: s }) || (e = e === null ? Ha.errors : e.concat(Ha.errors), r = e.length);
                var U = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var U = n === r;
              }
              if (t["number-depth"] !== void 0) {
                let a = t["number-depth"], n = r;
                Da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-depth", parentData: t, parentDataProperty: "number-depth", rootData: s }) || (e = e === null ? Da.errors : e.concat(Da.errors), r = e.length);
                var U = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var U = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var U = n === r;
              }
              if (t.theme !== void 0) {
                let a = t.theme, n = r;
                Ho(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/theme", parentData: t, parentDataProperty: "theme", rootData: s }) || (e = e === null ? Ho.errors : e.concat(Ho.errors), r = e.length);
                var U = n === r;
              }
              if (t.minimal !== void 0) {
                let a = t.minimal, n = r;
                Rs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/minimal", parentData: t, parentDataProperty: "minimal", rootData: s }) || (e = e === null ? Rs.errors : e.concat(Rs.errors), r = e.length);
                var U = n === r;
              }
              if (t["document-css"] !== void 0) {
                let a = t["document-css"], n = r;
                jn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/document-css", parentData: t, parentDataProperty: "document-css", rootData: s }) || (e = e === null ? jn.errors : e.concat(jn.errors), r = e.length);
                var U = n === r;
              }
              if (t.css !== void 0) {
                let a = t.css, n = r;
                fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/css", parentData: t, parentDataProperty: "css", rootData: s }) || (e = e === null ? fn.errors : e.concat(fn.errors), r = e.length);
                var U = n === r;
              }
              if (t["anchor-sections"] !== void 0) {
                let a = t["anchor-sections"], n = r;
                Vs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/anchor-sections", parentData: t, parentDataProperty: "anchor-sections", rootData: s }) || (e = e === null ? Vs.errors : e.concat(Vs.errors), r = e.length);
                var U = n === r;
              }
              if (t["html-math-method"] !== void 0) {
                let a = t["html-math-method"], n = r;
                Wn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-math-method", parentData: t, parentDataProperty: "html-math-method", rootData: s }) || (e = e === null ? Wn.errors : e.concat(Wn.errors), r = e.length);
                var U = n === r;
              }
              if (t["section-divs"] !== void 0) {
                let a = t["section-divs"], n = r;
                En(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/section-divs", parentData: t, parentDataProperty: "section-divs", rootData: s }) || (e = e === null ? En.errors : e.concat(En.errors), r = e.length);
                var U = n === r;
              }
              if (t["identifier-prefix"] !== void 0) {
                let a = t["identifier-prefix"], n = r;
                Pa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier-prefix", parentData: t, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
                var U = n === r;
              }
              if (t["email-obfuscation"] !== void 0) {
                let a = t["email-obfuscation"], n = r;
                Nn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/email-obfuscation", parentData: t, parentDataProperty: "email-obfuscation", rootData: s }) || (e = e === null ? Nn.errors : e.concat(Nn.errors), r = e.length);
                var U = n === r;
              }
              if (t["html-q-tags"] !== void 0) {
                let a = t["html-q-tags"], n = r;
                mn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-q-tags", parentData: t, parentDataProperty: "html-q-tags", rootData: s }) || (e = e === null ? mn.errors : e.concat(mn.errors), r = e.length);
                var U = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var U = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var U = n === r;
              }
              if (t["citations-hover"] !== void 0) {
                let a = t["citations-hover"], n = r;
                Cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citations-hover", parentData: t, parentDataProperty: "citations-hover", rootData: s }) || (e = e === null ? Cn.errors : e.concat(Cn.errors), r = e.length);
                var U = n === r;
              }
              if (t["citation-location"] !== void 0) {
                let a = t["citation-location"], n = r;
                Us(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-location", parentData: t, parentDataProperty: "citation-location", rootData: s }) || (e = e === null ? Us.errors : e.concat(Us.errors), r = e.length);
                var U = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var U = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var U = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var U = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var U = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var U = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var U = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var U = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var U = n === r;
              }
              if (t["self-contained"] !== void 0) {
                let a = t["self-contained"], n = r;
                In(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/self-contained", parentData: t, parentDataProperty: "self-contained", rootData: s }) || (e = e === null ? In.errors : e.concat(In.errors), r = e.length);
                var U = n === r;
              }
              if (t["self-contained-math"] !== void 0) {
                let a = t["self-contained-math"], n = r;
                Hn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/self-contained-math", parentData: t, parentDataProperty: "self-contained-math", rootData: s }) || (e = e === null ? Hn.errors : e.concat(Hn.errors), r = e.length);
                var U = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var U = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var U = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var U = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var U = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var U = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var U = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var U = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var U = n === r;
              }
              if (t["strip-comments"] !== void 0) {
                let a = t["strip-comments"], n = r;
                Ra(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-comments", parentData: t, parentDataProperty: "strip-comments", rootData: s }) || (e = e === null ? Ra.errors : e.concat(Ra.errors), r = e.length);
                var U = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var U = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var U = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var U = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var U = n === r;
              }
              if (t["toc-location"] !== void 0) {
                let a = t["toc-location"], n = r;
                Xs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-location", parentData: t, parentDataProperty: "toc-location", rootData: s }) || (e = e === null ? Xs.errors : e.concat(Xs.errors), r = e.length);
                var U = n === r;
              }
              if (t["toc-title"] !== void 0) {
                let a = t["toc-title"], n = r;
                La(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-title", parentData: t, parentDataProperty: "toc-title", rootData: s }) || (e = e === null ? La.errors : e.concat(La.errors), r = e.length);
                var U = n === r;
              }
              if (t.search !== void 0) {
                let a = t.search, n = r;
                ri(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/search", parentData: t, parentDataProperty: "search", rootData: s }) || (e = e === null ? ri.errors : e.concat(ri.errors), r = e.length);
                var U = n === r;
              }
              if (t.aliases !== void 0) {
                let a = t.aliases, n = r;
                ti(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/aliases", parentData: t, parentDataProperty: "aliases", rootData: s }) || (e = e === null ? ti.errors : e.concat(ti.errors), r = e.length);
                var U = n === r;
              }
              if (t.image !== void 0) {
                let a = t.image, n = r;
                ai(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/image", parentData: t, parentDataProperty: "image", rootData: s }) || (e = e === null ? ai.errors : e.concat(ai.errors), r = e.length);
                var U = n === r;
              }
              if (t["image-height"] !== void 0) {
                let a = t["image-height"], n = r;
                ni(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/image-height", parentData: t, parentDataProperty: "image-height", rootData: s }) || (e = e === null ? ni.errors : e.concat(ni.errors), r = e.length);
                var U = n === r;
              }
              if (t["image-width"] !== void 0) {
                let a = t["image-width"], n = r;
                oi(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/image-width", parentData: t, parentDataProperty: "image-width", rootData: s }) || (e = e === null ? oi.errors : e.concat(oi.errors), r = e.length);
                var U = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml4(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var re = ht === r;
            re && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml4(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var re = gt === r;
            if (re && L ? (L = false, M = [M, 1]) : re && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml4(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^html4(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Nv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!P1.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml5(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_layout|pageLayout|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_location|citationLocation|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))", schema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_layout|pageLayout|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_location|citationLocation|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))"', schema: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_layout|pageLayout|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_location|citationLocation|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml5(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var Fr = n === r;
                if (!Fr) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml5(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var z = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var z = n === r;
              }
              if (t["code-fold"] !== void 0) {
                let a = t["code-fold"], n = r;
                Fa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-fold", parentData: t, parentDataProperty: "code-fold", rootData: s }) || (e = e === null ? Fa.errors : e.concat(Fa.errors), r = e.length);
                var z = n === r;
              }
              if (t["code-summary"] !== void 0) {
                let a = t["code-summary"], n = r;
                Aa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-summary", parentData: t, parentDataProperty: "code-summary", rootData: s }) || (e = e === null ? Aa.errors : e.concat(Aa.errors), r = e.length);
                var z = n === r;
              }
              if (t["code-overflow"] !== void 0) {
                let a = t["code-overflow"], n = r;
                Ba(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-overflow", parentData: t, parentDataProperty: "code-overflow", rootData: s }) || (e = e === null ? Ba.errors : e.concat(Ba.errors), r = e.length);
                var z = n === r;
              }
              if (t["code-line-numbers"] !== void 0) {
                let a = t["code-line-numbers"], n = r;
                ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-line-numbers", parentData: t, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
                var z = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var z = n === r;
              }
              if (t["cap-location"] !== void 0) {
                let a = t["cap-location"], n = r;
                ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cap-location", parentData: t, parentDataProperty: "cap-location", rootData: s }) || (e = e === null ? ja.errors : e.concat(ja.errors), r = e.length);
                var z = n === r;
              }
              if (t["fig-cap-location"] !== void 0) {
                let a = t["fig-cap-location"], n = r;
                Ea(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-cap-location", parentData: t, parentDataProperty: "fig-cap-location", rootData: s }) || (e = e === null ? Ea.errors : e.concat(Ea.errors), r = e.length);
                var z = n === r;
              }
              if (t["tbl-cap-location"] !== void 0) {
                let a = t["tbl-cap-location"], n = r;
                Na(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-cap-location", parentData: t, parentDataProperty: "tbl-cap-location", rootData: s }) || (e = e === null ? Na.errors : e.concat(Na.errors), r = e.length);
                var z = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var z = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var z = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var z = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var z = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var z = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var z = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var z = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var z = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var z = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var z = n === r;
              }
              if (t["abstract-title"] !== void 0) {
                let a = t["abstract-title"], n = r;
                to(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract-title", parentData: t, parentDataProperty: "abstract-title", rootData: s }) || (e = e === null ? to.errors : e.concat(to.errors), r = e.length);
                var z = n === r;
              }
              if (t.doi !== void 0) {
                let a = t.doi, n = r;
                Jo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/doi", parentData: t, parentDataProperty: "doi", rootData: s }) || (e = e === null ? Jo.errors : e.concat(Jo.errors), r = e.length);
                var z = n === r;
              }
              if (t["code-copy"] !== void 0) {
                let a = t["code-copy"], n = r;
                ln(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-copy", parentData: t, parentDataProperty: "code-copy", rootData: s }) || (e = e === null ? ln.errors : e.concat(ln.errors), r = e.length);
                var z = n === r;
              }
              if (t["code-link"] !== void 0) {
                let a = t["code-link"], n = r;
                On(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-link", parentData: t, parentDataProperty: "code-link", rootData: s }) || (e = e === null ? On.errors : e.concat(On.errors), r = e.length);
                var z = n === r;
              }
              if (t["code-tools"] !== void 0) {
                let a = t["code-tools"], n = r;
                Go(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-tools", parentData: t, parentDataProperty: "code-tools", rootData: s }) || (e = e === null ? Go.errors : e.concat(Go.errors), r = e.length);
                var z = n === r;
              }
              if (t["highlight-style"] !== void 0) {
                let a = t["highlight-style"], n = r;
                _a(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/highlight-style", parentData: t, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
                var z = n === r;
              }
              if (t["syntax-definition"] !== void 0) {
                let a = t["syntax-definition"], n = r;
                va(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definition", parentData: t, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
                var z = n === r;
              }
              if (t["syntax-definitions"] !== void 0) {
                let a = t["syntax-definitions"], n = r;
                qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definitions", parentData: t, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
                var z = n === r;
              }
              if (t["indented-code-classes"] !== void 0) {
                let a = t["indented-code-classes"], n = r;
                Oa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indented-code-classes", parentData: t, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
                var z = n === r;
              }
              if (t.fontcolor !== void 0) {
                let a = t.fontcolor, n = r;
                Pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontcolor", parentData: t, parentDataProperty: "fontcolor", rootData: s }) || (e = e === null ? Pn.errors : e.concat(Pn.errors), r = e.length);
                var z = n === r;
              }
              if (t.linkcolor !== void 0) {
                let a = t.linkcolor, n = r;
                Ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linkcolor", parentData: t, parentDataProperty: "linkcolor", rootData: s }) || (e = e === null ? Ua.errors : e.concat(Ua.errors), r = e.length);
                var z = n === r;
              }
              if (t.monobackgroundcolor !== void 0) {
                let a = t.monobackgroundcolor, n = r;
                xn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monobackgroundcolor", parentData: t, parentDataProperty: "monobackgroundcolor", rootData: s }) || (e = e === null ? xn.errors : e.concat(xn.errors), r = e.length);
                var z = n === r;
              }
              if (t.backgroundcolor !== void 0) {
                let a = t.backgroundcolor, n = r;
                kn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/backgroundcolor", parentData: t, parentDataProperty: "backgroundcolor", rootData: s }) || (e = e === null ? kn.errors : e.concat(kn.errors), r = e.length);
                var z = n === r;
              }
              if (t.comments !== void 0) {
                let a = t.comments, n = r;
                Dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/comments", parentData: t, parentDataProperty: "comments", rootData: s }) || (e = e === null ? Dn.errors : e.concat(Dn.errors), r = e.length);
                var z = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var z = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var z = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var z = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var z = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var z = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var z = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var z = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var z = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var z = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var z = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var z = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var z = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var z = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var z = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var z = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var z = n === r;
              }
              if (t["fig-responsive"] !== void 0) {
                let a = t["fig-responsive"], n = r;
                cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-responsive", parentData: t, parentDataProperty: "fig-responsive", rootData: s }) || (e = e === null ? cn.errors : e.concat(cn.errors), r = e.length);
                var z = n === r;
              }
              if (t.mainfont !== void 0) {
                let a = t.mainfont, n = r;
                Ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mainfont", parentData: t, parentDataProperty: "mainfont", rootData: s }) || (e = e === null ? Ja.errors : e.concat(Ja.errors), r = e.length);
                var z = n === r;
              }
              if (t.monofont !== void 0) {
                let a = t.monofont, n = r;
                Qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monofont", parentData: t, parentDataProperty: "monofont", rootData: s }) || (e = e === null ? Qa.errors : e.concat(Qa.errors), r = e.length);
                var z = n === r;
              }
              if (t.fontsize !== void 0) {
                let a = t.fontsize, n = r;
                Ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontsize", parentData: t, parentDataProperty: "fontsize", rootData: s }) || (e = e === null ? Ga.errors : e.concat(Ga.errors), r = e.length);
                var z = n === r;
              }
              if (t.linestretch !== void 0) {
                let a = t.linestretch, n = r;
                Ya(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linestretch", parentData: t, parentDataProperty: "linestretch", rootData: s }) || (e = e === null ? Ya.errors : e.concat(Ya.errors), r = e.length);
                var z = n === r;
              }
              if (t["footnotes-hover"] !== void 0) {
                let a = t["footnotes-hover"], n = r;
                wn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/footnotes-hover", parentData: t, parentDataProperty: "footnotes-hover", rootData: s }) || (e = e === null ? wn.errors : e.concat(wn.errors), r = e.length);
                var z = n === r;
              }
              if (t["reference-location"] !== void 0) {
                let a = t["reference-location"], n = r;
                Ca(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-location", parentData: t, parentDataProperty: "reference-location", rootData: s }) || (e = e === null ? Ca.errors : e.concat(Ca.errors), r = e.length);
                var z = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var z = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var z = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var z = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var z = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var z = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var z = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var z = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var z = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var z = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var z = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var z = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var z = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var z = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var z = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var z = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var z = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var z = n === r;
              }
              if (t["keep-source"] !== void 0) {
                let a = t["keep-source"], n = r;
                As(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-source", parentData: t, parentDataProperty: "keep-source", rootData: s }) || (e = e === null ? As.errors : e.concat(As.errors), r = e.length);
                var z = n === r;
              }
              if (t["keep-hidden"] !== void 0) {
                let a = t["keep-hidden"], n = r;
                Bs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-hidden", parentData: t, parentDataProperty: "keep-hidden", rootData: s }) || (e = e === null ? Bs.errors : e.concat(Bs.errors), r = e.length);
                var z = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var z = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var z = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var z = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var z = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var z = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var z = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var z = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var z = n === r;
              }
              if (t.resources !== void 0) {
                let a = t.resources, n = r;
                pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resources", parentData: t, parentDataProperty: "resources", rootData: s }) || (e = e === null ? pn.errors : e.concat(pn.errors), r = e.length);
                var z = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var z = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var z = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var z = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var z = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var z = n === r;
              }
              if (t.classoption !== void 0) {
                let a = t.classoption, n = r;
                un(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/classoption", parentData: t, parentDataProperty: "classoption", rootData: s }) || (e = e === null ? un.errors : e.concat(un.errors), r = e.length);
                var z = n === r;
              }
              if (t["page-layout"] !== void 0) {
                let a = t["page-layout"], n = r;
                Cs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/page-layout", parentData: t, parentDataProperty: "page-layout", rootData: s }) || (e = e === null ? Cs.errors : e.concat(Cs.errors), r = e.length);
                var z = n === r;
              }
              if (t["max-width"] !== void 0) {
                let a = t["max-width"], n = r;
                $n(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/max-width", parentData: t, parentDataProperty: "max-width", rootData: s }) || (e = e === null ? $n.errors : e.concat($n.errors), r = e.length);
                var z = n === r;
              }
              if (t["margin-left"] !== void 0) {
                let a = t["margin-left"], n = r;
                Xa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-left", parentData: t, parentDataProperty: "margin-left", rootData: s }) || (e = e === null ? Xa.errors : e.concat(Xa.errors), r = e.length);
                var z = n === r;
              }
              if (t["margin-right"] !== void 0) {
                let a = t["margin-right"], n = r;
                Za(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-right", parentData: t, parentDataProperty: "margin-right", rootData: s }) || (e = e === null ? Za.errors : e.concat(Za.errors), r = e.length);
                var z = n === r;
              }
              if (t["margin-top"] !== void 0) {
                let a = t["margin-top"], n = r;
                en(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-top", parentData: t, parentDataProperty: "margin-top", rootData: s }) || (e = e === null ? en.errors : e.concat(en.errors), r = e.length);
                var z = n === r;
              }
              if (t["margin-bottom"] !== void 0) {
                let a = t["margin-bottom"], n = r;
                rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-bottom", parentData: t, parentDataProperty: "margin-bottom", rootData: s }) || (e = e === null ? rn.errors : e.concat(rn.errors), r = e.length);
                var z = n === r;
              }
              if (t["link-external-icon"] !== void 0) {
                let a = t["link-external-icon"], n = r;
                No(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/link-external-icon", parentData: t, parentDataProperty: "link-external-icon", rootData: s }) || (e = e === null ? No.errors : e.concat(No.errors), r = e.length);
                var z = n === r;
              }
              if (t["link-external-newwindow"] !== void 0) {
                let a = t["link-external-newwindow"], n = r;
                Co(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/link-external-newwindow", parentData: t, parentDataProperty: "link-external-newwindow", rootData: s }) || (e = e === null ? Co.errors : e.concat(Co.errors), r = e.length);
                var z = n === r;
              }
              if (t["link-external-filter"] !== void 0) {
                let a = t["link-external-filter"], n = r;
                Io(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/link-external-filter", parentData: t, parentDataProperty: "link-external-filter", rootData: s }) || (e = e === null ? Io.errors : e.concat(Io.errors), r = e.length);
                var z = n === r;
              }
              if (t.keywords !== void 0) {
                let a = t.keywords, n = r;
                Ia(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keywords", parentData: t, parentDataProperty: "keywords", rootData: s }) || (e = e === null ? Ia.errors : e.concat(Ia.errors), r = e.length);
                var z = n === r;
              }
              if (t.pagetitle !== void 0) {
                let a = t.pagetitle, n = r;
                Sn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pagetitle", parentData: t, parentDataProperty: "pagetitle", rootData: s }) || (e = e === null ? Sn.errors : e.concat(Sn.errors), r = e.length);
                var z = n === r;
              }
              if (t["title-prefix"] !== void 0) {
                let a = t["title-prefix"], n = r;
                Fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title-prefix", parentData: t, parentDataProperty: "title-prefix", rootData: s }) || (e = e === null ? Fn.errors : e.concat(Fn.errors), r = e.length);
                var z = n === r;
              }
              if (t["description-meta"] !== void 0) {
                let a = t["description-meta"], n = r;
                An(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/description-meta", parentData: t, parentDataProperty: "description-meta", rootData: s }) || (e = e === null ? An.errors : e.concat(An.errors), r = e.length);
                var z = n === r;
              }
              if (t["author-meta"] !== void 0) {
                let a = t["author-meta"], n = r;
                dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author-meta", parentData: t, parentDataProperty: "author-meta", rootData: s }) || (e = e === null ? dn.errors : e.concat(dn.errors), r = e.length);
                var z = n === r;
              }
              if (t["date-meta"] !== void 0) {
                let a = t["date-meta"], n = r;
                Ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date-meta", parentData: t, parentDataProperty: "date-meta", rootData: s }) || (e = e === null ? Ha.errors : e.concat(Ha.errors), r = e.length);
                var z = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var z = n === r;
              }
              if (t["number-depth"] !== void 0) {
                let a = t["number-depth"], n = r;
                Da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-depth", parentData: t, parentDataProperty: "number-depth", rootData: s }) || (e = e === null ? Da.errors : e.concat(Da.errors), r = e.length);
                var z = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var z = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var z = n === r;
              }
              if (t.theme !== void 0) {
                let a = t.theme, n = r;
                Ho(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/theme", parentData: t, parentDataProperty: "theme", rootData: s }) || (e = e === null ? Ho.errors : e.concat(Ho.errors), r = e.length);
                var z = n === r;
              }
              if (t.minimal !== void 0) {
                let a = t.minimal, n = r;
                Rs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/minimal", parentData: t, parentDataProperty: "minimal", rootData: s }) || (e = e === null ? Rs.errors : e.concat(Rs.errors), r = e.length);
                var z = n === r;
              }
              if (t["document-css"] !== void 0) {
                let a = t["document-css"], n = r;
                jn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/document-css", parentData: t, parentDataProperty: "document-css", rootData: s }) || (e = e === null ? jn.errors : e.concat(jn.errors), r = e.length);
                var z = n === r;
              }
              if (t.css !== void 0) {
                let a = t.css, n = r;
                fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/css", parentData: t, parentDataProperty: "css", rootData: s }) || (e = e === null ? fn.errors : e.concat(fn.errors), r = e.length);
                var z = n === r;
              }
              if (t["anchor-sections"] !== void 0) {
                let a = t["anchor-sections"], n = r;
                Vs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/anchor-sections", parentData: t, parentDataProperty: "anchor-sections", rootData: s }) || (e = e === null ? Vs.errors : e.concat(Vs.errors), r = e.length);
                var z = n === r;
              }
              if (t["html-math-method"] !== void 0) {
                let a = t["html-math-method"], n = r;
                Wn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-math-method", parentData: t, parentDataProperty: "html-math-method", rootData: s }) || (e = e === null ? Wn.errors : e.concat(Wn.errors), r = e.length);
                var z = n === r;
              }
              if (t["section-divs"] !== void 0) {
                let a = t["section-divs"], n = r;
                En(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/section-divs", parentData: t, parentDataProperty: "section-divs", rootData: s }) || (e = e === null ? En.errors : e.concat(En.errors), r = e.length);
                var z = n === r;
              }
              if (t["identifier-prefix"] !== void 0) {
                let a = t["identifier-prefix"], n = r;
                Pa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier-prefix", parentData: t, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
                var z = n === r;
              }
              if (t["email-obfuscation"] !== void 0) {
                let a = t["email-obfuscation"], n = r;
                Nn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/email-obfuscation", parentData: t, parentDataProperty: "email-obfuscation", rootData: s }) || (e = e === null ? Nn.errors : e.concat(Nn.errors), r = e.length);
                var z = n === r;
              }
              if (t["html-q-tags"] !== void 0) {
                let a = t["html-q-tags"], n = r;
                mn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-q-tags", parentData: t, parentDataProperty: "html-q-tags", rootData: s }) || (e = e === null ? mn.errors : e.concat(mn.errors), r = e.length);
                var z = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var z = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var z = n === r;
              }
              if (t["citations-hover"] !== void 0) {
                let a = t["citations-hover"], n = r;
                Cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citations-hover", parentData: t, parentDataProperty: "citations-hover", rootData: s }) || (e = e === null ? Cn.errors : e.concat(Cn.errors), r = e.length);
                var z = n === r;
              }
              if (t["citation-location"] !== void 0) {
                let a = t["citation-location"], n = r;
                Us(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-location", parentData: t, parentDataProperty: "citation-location", rootData: s }) || (e = e === null ? Us.errors : e.concat(Us.errors), r = e.length);
                var z = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var z = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var z = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var z = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var z = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var z = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var z = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var z = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var z = n === r;
              }
              if (t["self-contained"] !== void 0) {
                let a = t["self-contained"], n = r;
                In(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/self-contained", parentData: t, parentDataProperty: "self-contained", rootData: s }) || (e = e === null ? In.errors : e.concat(In.errors), r = e.length);
                var z = n === r;
              }
              if (t["self-contained-math"] !== void 0) {
                let a = t["self-contained-math"], n = r;
                Hn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/self-contained-math", parentData: t, parentDataProperty: "self-contained-math", rootData: s }) || (e = e === null ? Hn.errors : e.concat(Hn.errors), r = e.length);
                var z = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var z = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var z = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var z = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var z = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var z = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var z = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var z = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var z = n === r;
              }
              if (t["strip-comments"] !== void 0) {
                let a = t["strip-comments"], n = r;
                Ra(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-comments", parentData: t, parentDataProperty: "strip-comments", rootData: s }) || (e = e === null ? Ra.errors : e.concat(Ra.errors), r = e.length);
                var z = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var z = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var z = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var z = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var z = n === r;
              }
              if (t["toc-location"] !== void 0) {
                let a = t["toc-location"], n = r;
                Xs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-location", parentData: t, parentDataProperty: "toc-location", rootData: s }) || (e = e === null ? Xs.errors : e.concat(Xs.errors), r = e.length);
                var z = n === r;
              }
              if (t["toc-title"] !== void 0) {
                let a = t["toc-title"], n = r;
                La(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-title", parentData: t, parentDataProperty: "toc-title", rootData: s }) || (e = e === null ? La.errors : e.concat(La.errors), r = e.length);
                var z = n === r;
              }
              if (t.search !== void 0) {
                let a = t.search, n = r;
                ri(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/search", parentData: t, parentDataProperty: "search", rootData: s }) || (e = e === null ? ri.errors : e.concat(ri.errors), r = e.length);
                var z = n === r;
              }
              if (t.aliases !== void 0) {
                let a = t.aliases, n = r;
                ti(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/aliases", parentData: t, parentDataProperty: "aliases", rootData: s }) || (e = e === null ? ti.errors : e.concat(ti.errors), r = e.length);
                var z = n === r;
              }
              if (t.image !== void 0) {
                let a = t.image, n = r;
                ai(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/image", parentData: t, parentDataProperty: "image", rootData: s }) || (e = e === null ? ai.errors : e.concat(ai.errors), r = e.length);
                var z = n === r;
              }
              if (t["image-height"] !== void 0) {
                let a = t["image-height"], n = r;
                ni(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/image-height", parentData: t, parentDataProperty: "image-height", rootData: s }) || (e = e === null ? ni.errors : e.concat(ni.errors), r = e.length);
                var z = n === r;
              }
              if (t["image-width"] !== void 0) {
                let a = t["image-width"], n = r;
                oi(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/image-width", parentData: t, parentDataProperty: "image-width", rootData: s }) || (e = e === null ? oi.errors : e.concat(oi.errors), r = e.length);
                var z = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml5(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Xr = ht === r;
            Xr && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml5(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Xr = gt === r;
            if (Xr && L ? (L = false, M = [M, 1]) : Xr && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ehtml5(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^html5(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Cv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eicml(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eicml(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var W = n === r;
                if (!W) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eicml(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var C = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var C = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var C = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var C = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var C = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var C = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var C = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var C = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var C = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var C = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var C = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var C = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var C = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var C = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var C = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var C = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var C = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var C = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var C = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var C = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var C = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var C = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var C = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var C = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var C = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var C = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var C = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var C = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var C = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var C = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var C = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var C = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var C = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var C = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var C = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var C = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var C = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var C = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var C = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var C = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var C = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var C = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var C = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var C = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var C = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var C = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var C = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var C = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var C = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var C = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var C = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var C = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var C = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var C = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var C = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var C = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var C = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var C = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var C = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var C = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var C = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var C = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var C = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var C = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var C = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var C = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var C = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var C = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var C = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var C = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var C = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var C = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var C = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var C = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var C = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var C = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var C = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var C = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var C = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var C = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var C = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var C = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var C = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var C = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var C = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eicml(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var te = ht === r;
            te && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eicml(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var te = gt === r;
            if (te && L ? (L = false, M = [M, 1]) : te && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eicml(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^icml(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Iv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!W0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eipynb(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|markdown_headings|markdownHeadings|ipynb_output|ipynbOutput|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|markdown_headings|markdownHeadings|ipynb_output|ipynbOutput|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|markdown_headings|markdownHeadings|ipynb_output|ipynbOutput|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eipynb(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var Br = n === r;
                if (!Br) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eipynb(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var X = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var X = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var X = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var X = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var X = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var X = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var X = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var X = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var X = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var X = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var X = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var X = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var X = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var X = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var X = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var X = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var X = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var X = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var X = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var X = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var X = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var X = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var X = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var X = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var X = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var X = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var X = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var X = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var X = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var X = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var X = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var X = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var X = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var X = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var X = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var X = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var X = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var X = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var X = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var X = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var X = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var X = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var X = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var X = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var X = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var X = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var X = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var X = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var X = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var X = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var X = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var X = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var X = n === r;
              }
              if (t["markdown-headings"] !== void 0) {
                let a = t["markdown-headings"], n = r;
                ho(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/markdown-headings", parentData: t, parentDataProperty: "markdown-headings", rootData: s }) || (e = e === null ? ho.errors : e.concat(ho.errors), r = e.length);
                var X = n === r;
              }
              if (t["ipynb-output"] !== void 0) {
                let a = t["ipynb-output"], n = r;
                Xc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb-output", parentData: t, parentDataProperty: "ipynb-output", rootData: s }) || (e = e === null ? Xc.errors : e.concat(Xc.errors), r = e.length);
                var X = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var X = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var X = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var X = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var X = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var X = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var X = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var X = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var X = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var X = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var X = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var X = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var X = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var X = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var X = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var X = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var X = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var X = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var X = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var X = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var X = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var X = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var X = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var X = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var X = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var X = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eipynb(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Yr = ht === r;
            Yr && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eipynb(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Yr = gt === r;
            if (Yr && L ? (L = false, M = [M, 1]) : Yr && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eipynb(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^ipynb(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Hv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Ym.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))", schema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var G = n === r;
                if (!G) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.affiliation !== void 0) {
                let a = t.affiliation, n = r;
                ko(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/affiliation", parentData: t, parentDataProperty: "affiliation", rootData: s }) || (e = e === null ? ko.errors : e.concat(ko.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.copyright !== void 0) {
                let a = t.copyright, n = r;
                Do(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/copyright", parentData: t, parentDataProperty: "copyright", rootData: s }) || (e = e === null ? Do.errors : e.concat(Do.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.article !== void 0) {
                let a = t.article, n = r;
                wo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/article", parentData: t, parentDataProperty: "article", rootData: s }) || (e = e === null ? wo.errors : e.concat(wo.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.journal !== void 0) {
                let a = t.journal, n = r;
                $o(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/journal", parentData: t, parentDataProperty: "journal", rootData: s }) || (e = e === null ? $o.errors : e.concat($o.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.notes !== void 0) {
                let a = t.notes, n = r;
                So(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/notes", parentData: t, parentDataProperty: "notes", rootData: s }) || (e = e === null ? So.errors : e.concat(So.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.tags !== void 0) {
                let a = t.tags, n = r;
                Fo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tags", parentData: t, parentDataProperty: "tags", rootData: s }) || (e = e === null ? Fo.errors : e.concat(Fo.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Ar = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Ar = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Ar = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var _t = ht === r;
            _t && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var _t = gt === r;
            if (_t && L ? (L = false, M = [M, 1]) : _t && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Lv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Ym.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_archiving(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))", schema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_archiving(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var Zr = n === r;
                if (!Zr) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_archiving(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Er = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Er = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Er = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Er = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Er = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Er = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Er = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Er = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Er = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Er = n === r;
              }
              if (t.affiliation !== void 0) {
                let a = t.affiliation, n = r;
                ko(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/affiliation", parentData: t, parentDataProperty: "affiliation", rootData: s }) || (e = e === null ? ko.errors : e.concat(ko.errors), r = e.length);
                var Er = n === r;
              }
              if (t.copyright !== void 0) {
                let a = t.copyright, n = r;
                Do(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/copyright", parentData: t, parentDataProperty: "copyright", rootData: s }) || (e = e === null ? Do.errors : e.concat(Do.errors), r = e.length);
                var Er = n === r;
              }
              if (t.article !== void 0) {
                let a = t.article, n = r;
                wo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/article", parentData: t, parentDataProperty: "article", rootData: s }) || (e = e === null ? wo.errors : e.concat(wo.errors), r = e.length);
                var Er = n === r;
              }
              if (t.journal !== void 0) {
                let a = t.journal, n = r;
                $o(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/journal", parentData: t, parentDataProperty: "journal", rootData: s }) || (e = e === null ? $o.errors : e.concat($o.errors), r = e.length);
                var Er = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var Er = n === r;
              }
              if (t.notes !== void 0) {
                let a = t.notes, n = r;
                So(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/notes", parentData: t, parentDataProperty: "notes", rootData: s }) || (e = e === null ? So.errors : e.concat(So.errors), r = e.length);
                var Er = n === r;
              }
              if (t.tags !== void 0) {
                let a = t.tags, n = r;
                Fo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tags", parentData: t, parentDataProperty: "tags", rootData: s }) || (e = e === null ? Fo.errors : e.concat(Fo.errors), r = e.length);
                var Er = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Er = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Er = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Er = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Er = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Er = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Er = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Er = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Er = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Er = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Er = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Er = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Er = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Er = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Er = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Er = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Er = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Er = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Er = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Er = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Er = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Er = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Er = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Er = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Er = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Er = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Er = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Er = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Er = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Er = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Er = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Er = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Er = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Er = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Er = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Er = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Er = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Er = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Er = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Er = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Er = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Er = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Er = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Er = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Er = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Er = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Er = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Er = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Er = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Er = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Er = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Er = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Er = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Er = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Er = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Er = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Er = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Er = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Er = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Er = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Er = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Er = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Er = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Er = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Er = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Er = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Er = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_archiving(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Wa = ht === r;
            Wa && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_archiving(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Wa = gt === r;
            if (Wa && L ? (L = false, M = [M, 1]) : Wa && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_archiving(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_archiving(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Mv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Ym.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_articleauthoring(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))", schema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_articleauthoring(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var Rr = n === r;
                if (!Rr) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_articleauthoring(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.affiliation !== void 0) {
                let a = t.affiliation, n = r;
                ko(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/affiliation", parentData: t, parentDataProperty: "affiliation", rootData: s }) || (e = e === null ? ko.errors : e.concat(ko.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.copyright !== void 0) {
                let a = t.copyright, n = r;
                Do(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/copyright", parentData: t, parentDataProperty: "copyright", rootData: s }) || (e = e === null ? Do.errors : e.concat(Do.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.article !== void 0) {
                let a = t.article, n = r;
                wo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/article", parentData: t, parentDataProperty: "article", rootData: s }) || (e = e === null ? wo.errors : e.concat(wo.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.journal !== void 0) {
                let a = t.journal, n = r;
                $o(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/journal", parentData: t, parentDataProperty: "journal", rootData: s }) || (e = e === null ? $o.errors : e.concat($o.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.notes !== void 0) {
                let a = t.notes, n = r;
                So(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/notes", parentData: t, parentDataProperty: "notes", rootData: s }) || (e = e === null ? So.errors : e.concat(So.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.tags !== void 0) {
                let a = t.tags, n = r;
                Fo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tags", parentData: t, parentDataProperty: "tags", rootData: s }) || (e = e === null ? Fo.errors : e.concat(Fo.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Qr = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Qr = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Qr = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_articleauthoring(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var ya = ht === r;
            ya && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_articleauthoring(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var ya = gt === r;
            if (ya && L ? (L = false, M = [M, 1]) : ya && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_articleauthoring(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_articleauthoring(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Tv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Ym.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_publishing(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))", schema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_publishing(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var pa = n === r;
                if (!pa) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_publishing(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.affiliation !== void 0) {
                let a = t.affiliation, n = r;
                ko(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/affiliation", parentData: t, parentDataProperty: "affiliation", rootData: s }) || (e = e === null ? ko.errors : e.concat(ko.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.copyright !== void 0) {
                let a = t.copyright, n = r;
                Do(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/copyright", parentData: t, parentDataProperty: "copyright", rootData: s }) || (e = e === null ? Do.errors : e.concat(Do.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.article !== void 0) {
                let a = t.article, n = r;
                wo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/article", parentData: t, parentDataProperty: "article", rootData: s }) || (e = e === null ? wo.errors : e.concat(wo.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.journal !== void 0) {
                let a = t.journal, n = r;
                $o(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/journal", parentData: t, parentDataProperty: "journal", rootData: s }) || (e = e === null ? $o.errors : e.concat($o.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.notes !== void 0) {
                let a = t.notes, n = r;
                So(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/notes", parentData: t, parentDataProperty: "notes", rootData: s }) || (e = e === null ? So.errors : e.concat(So.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.tags !== void 0) {
                let a = t.tags, n = r;
                Fo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tags", parentData: t, parentDataProperty: "tags", rootData: s }) || (e = e === null ? Fo.errors : e.concat(Fo.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Nr = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Nr = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Nr = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_publishing(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var lo = ht === r;
            lo && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_publishing(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var lo = gt === r;
            if (lo && L ? (L = false, M = [M, 1]) : lo && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejats_publishing(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jats_publishing(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (zv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejira(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejira(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var Tn = n === r;
                if (!Tn) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejira(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var bt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var bt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var bt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var bt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var bt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var bt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var bt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var bt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var bt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var bt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var bt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var bt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var bt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var bt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var bt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var bt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var bt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var bt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var bt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var bt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var bt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var bt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var bt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var bt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var bt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var bt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var bt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var bt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var bt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var bt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var bt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var bt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var bt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var bt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var bt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var bt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var bt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var bt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var bt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var bt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var bt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var bt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var bt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var bt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var bt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var bt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var bt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var bt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var bt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var bt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var bt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var bt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var bt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var bt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var bt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var bt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var bt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var bt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var bt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var bt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var bt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var bt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var bt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var bt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var bt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var bt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var bt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var bt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var bt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var bt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var bt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var bt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var bt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var bt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var bt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var bt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var bt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var bt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var bt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var bt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var bt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var bt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var bt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var bt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var bt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejira(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Ro = ht === r;
            Ro && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejira(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Ro = gt === r;
            if (Ro && L ? (L = false, M = [M, 1]) : Ro && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejira(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^jira(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Wv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejson(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejson(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var Xm = n === r;
                if (!Xm) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejson(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var dt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var dt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var dt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var dt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var dt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var dt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var dt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var dt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var dt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var dt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var dt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var dt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var dt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var dt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var dt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var dt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var dt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var dt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var dt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var dt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var dt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var dt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var dt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var dt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var dt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var dt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var dt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var dt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var dt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var dt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var dt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var dt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var dt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var dt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var dt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var dt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var dt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var dt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var dt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var dt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var dt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var dt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var dt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var dt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var dt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var dt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var dt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var dt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var dt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var dt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var dt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var dt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var dt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var dt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var dt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var dt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var dt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var dt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var dt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var dt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var dt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var dt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var dt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var dt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var dt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var dt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var dt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var dt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var dt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var dt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var dt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var dt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var dt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var dt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var dt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var dt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var dt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var dt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var dt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var dt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var dt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var dt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var dt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var dt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var dt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejson(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var fc = ht === r;
            fc && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejson(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var fc = gt === r;
            if (fc && L ? (L = false, M = [M, 1]) : fc && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ejson(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^json(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Rv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!R0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Elatex(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|links_as_notes|linksAsNotes|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|title_meta|titleMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", schema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|links_as_notes|linksAsNotes|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|title_meta|titleMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))"', schema: "^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|links_as_notes|linksAsNotes|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|title_meta|titleMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Elatex(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var il = n === r;
                if (!il) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Elatex(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["code-line-numbers"] !== void 0) {
                let a = t["code-line-numbers"], n = r;
                ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-line-numbers", parentData: t, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["fig-env"] !== void 0) {
                let a = t["fig-env"], n = r;
                Zn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-env", parentData: t, parentDataProperty: "fig-env", rootData: s }) || (e = e === null ? Zn.errors : e.concat(Zn.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["fig-pos"] !== void 0) {
                let a = t["fig-pos"], n = r;
                eo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-pos", parentData: t, parentDataProperty: "fig-pos", rootData: s }) || (e = e === null ? eo.errors : e.concat(eo.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["cap-location"] !== void 0) {
                let a = t["cap-location"], n = r;
                ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cap-location", parentData: t, parentDataProperty: "cap-location", rootData: s }) || (e = e === null ? ja.errors : e.concat(ja.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["fig-cap-location"] !== void 0) {
                let a = t["fig-cap-location"], n = r;
                Ea(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-cap-location", parentData: t, parentDataProperty: "fig-cap-location", rootData: s }) || (e = e === null ? Ea.errors : e.concat(Ea.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["tbl-cap-location"] !== void 0) {
                let a = t["tbl-cap-location"], n = r;
                Na(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-cap-location", parentData: t, parentDataProperty: "tbl-cap-location", rootData: s }) || (e = e === null ? Na.errors : e.concat(Na.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.thanks !== void 0) {
                let a = t.thanks, n = r;
                Qo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/thanks", parentData: t, parentDataProperty: "thanks", rootData: s }) || (e = e === null ? Qo.errors : e.concat(Qo.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["highlight-style"] !== void 0) {
                let a = t["highlight-style"], n = r;
                _a(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/highlight-style", parentData: t, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["syntax-definition"] !== void 0) {
                let a = t["syntax-definition"], n = r;
                va(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definition", parentData: t, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["syntax-definitions"] !== void 0) {
                let a = t["syntax-definitions"], n = r;
                qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definitions", parentData: t, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.listings !== void 0) {
                let a = t.listings, n = r;
                Yo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/listings", parentData: t, parentDataProperty: "listings", rootData: s }) || (e = e === null ? Yo.errors : e.concat(Yo.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["indented-code-classes"] !== void 0) {
                let a = t["indented-code-classes"], n = r;
                Oa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indented-code-classes", parentData: t, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.linkcolor !== void 0) {
                let a = t.linkcolor, n = r;
                Ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linkcolor", parentData: t, parentDataProperty: "linkcolor", rootData: s }) || (e = e === null ? Ua.errors : e.concat(Ua.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.filecolor !== void 0) {
                let a = t.filecolor, n = r;
                Xo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filecolor", parentData: t, parentDataProperty: "filecolor", rootData: s }) || (e = e === null ? Xo.errors : e.concat(Xo.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.citecolor !== void 0) {
                let a = t.citecolor, n = r;
                Zo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citecolor", parentData: t, parentDataProperty: "citecolor", rootData: s }) || (e = e === null ? Zo.errors : e.concat(Zo.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.urlcolor !== void 0) {
                let a = t.urlcolor, n = r;
                es(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/urlcolor", parentData: t, parentDataProperty: "urlcolor", rootData: s }) || (e = e === null ? es.errors : e.concat(es.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.toccolor !== void 0) {
                let a = t.toccolor, n = r;
                rs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toccolor", parentData: t, parentDataProperty: "toccolor", rootData: s }) || (e = e === null ? rs.errors : e.concat(rs.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.colorlinks !== void 0) {
                let a = t.colorlinks, n = r;
                ts(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/colorlinks", parentData: t, parentDataProperty: "colorlinks", rootData: s }) || (e = e === null ? ts.errors : e.concat(ts.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.mainfont !== void 0) {
                let a = t.mainfont, n = r;
                Ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mainfont", parentData: t, parentDataProperty: "mainfont", rootData: s }) || (e = e === null ? Ja.errors : e.concat(Ja.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.monofont !== void 0) {
                let a = t.monofont, n = r;
                Qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monofont", parentData: t, parentDataProperty: "monofont", rootData: s }) || (e = e === null ? Qa.errors : e.concat(Qa.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.fontsize !== void 0) {
                let a = t.fontsize, n = r;
                Ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontsize", parentData: t, parentDataProperty: "fontsize", rootData: s }) || (e = e === null ? Ga.errors : e.concat(Ga.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.fontenc !== void 0) {
                let a = t.fontenc, n = r;
                _s(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontenc", parentData: t, parentDataProperty: "fontenc", rootData: s }) || (e = e === null ? _s.errors : e.concat(_s.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.fontfamily !== void 0) {
                let a = t.fontfamily, n = r;
                Bo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontfamily", parentData: t, parentDataProperty: "fontfamily", rootData: s }) || (e = e === null ? Bo.errors : e.concat(Bo.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.fontfamilyoptions !== void 0) {
                let a = t.fontfamilyoptions, n = r;
                vs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontfamilyoptions", parentData: t, parentDataProperty: "fontfamilyoptions", rootData: s }) || (e = e === null ? vs.errors : e.concat(vs.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.sansfont !== void 0) {
                let a = t.sansfont, n = r;
                qs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/sansfont", parentData: t, parentDataProperty: "sansfont", rootData: s }) || (e = e === null ? qs.errors : e.concat(qs.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.mathfont !== void 0) {
                let a = t.mathfont, n = r;
                Os(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mathfont", parentData: t, parentDataProperty: "mathfont", rootData: s }) || (e = e === null ? Os.errors : e.concat(Os.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.CJKmainfont !== void 0) {
                let a = t.CJKmainfont, n = r;
                Ps(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/CJKmainfont", parentData: t, parentDataProperty: "CJKmainfont", rootData: s }) || (e = e === null ? Ps.errors : e.concat(Ps.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.mainfontoptions !== void 0) {
                let a = t.mainfontoptions, n = r;
                xs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mainfontoptions", parentData: t, parentDataProperty: "mainfontoptions", rootData: s }) || (e = e === null ? xs.errors : e.concat(xs.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.sansfontoptions !== void 0) {
                let a = t.sansfontoptions, n = r;
                ks(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/sansfontoptions", parentData: t, parentDataProperty: "sansfontoptions", rootData: s }) || (e = e === null ? ks.errors : e.concat(ks.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.monofontoptions !== void 0) {
                let a = t.monofontoptions, n = r;
                Ds(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monofontoptions", parentData: t, parentDataProperty: "monofontoptions", rootData: s }) || (e = e === null ? Ds.errors : e.concat(Ds.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.mathfontoptions !== void 0) {
                let a = t.mathfontoptions, n = r;
                ws(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mathfontoptions", parentData: t, parentDataProperty: "mathfontoptions", rootData: s }) || (e = e === null ? ws.errors : e.concat(ws.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.CJKoptions !== void 0) {
                let a = t.CJKoptions, n = r;
                $s(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/CJKoptions", parentData: t, parentDataProperty: "CJKoptions", rootData: s }) || (e = e === null ? $s.errors : e.concat($s.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.microtypeoptions !== void 0) {
                let a = t.microtypeoptions, n = r;
                Ss(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/microtypeoptions", parentData: t, parentDataProperty: "microtypeoptions", rootData: s }) || (e = e === null ? Ss.errors : e.concat(Ss.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.linestretch !== void 0) {
                let a = t.linestretch, n = r;
                Ya(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linestretch", parentData: t, parentDataProperty: "linestretch", rootData: s }) || (e = e === null ? Ya.errors : e.concat(Ya.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["links-as-notes"] !== void 0) {
                let a = t["links-as-notes"], n = r;
                Fs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/links-as-notes", parentData: t, parentDataProperty: "links-as-notes", rootData: s }) || (e = e === null ? Fs.errors : e.concat(Fs.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.documentclass !== void 0) {
                let a = t.documentclass, n = r;
                js(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/documentclass", parentData: t, parentDataProperty: "documentclass", rootData: s }) || (e = e === null ? js.errors : e.concat(js.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.classoption !== void 0) {
                let a = t.classoption, n = r;
                un(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/classoption", parentData: t, parentDataProperty: "classoption", rootData: s }) || (e = e === null ? un.errors : e.concat(un.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.pagestyle !== void 0) {
                let a = t.pagestyle, n = r;
                Es(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pagestyle", parentData: t, parentDataProperty: "pagestyle", rootData: s }) || (e = e === null ? Es.errors : e.concat(Es.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.papersize !== void 0) {
                let a = t.papersize, n = r;
                Ns(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/papersize", parentData: t, parentDataProperty: "papersize", rootData: s }) || (e = e === null ? Ns.errors : e.concat(Ns.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["margin-left"] !== void 0) {
                let a = t["margin-left"], n = r;
                Xa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-left", parentData: t, parentDataProperty: "margin-left", rootData: s }) || (e = e === null ? Xa.errors : e.concat(Xa.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["margin-right"] !== void 0) {
                let a = t["margin-right"], n = r;
                Za(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-right", parentData: t, parentDataProperty: "margin-right", rootData: s }) || (e = e === null ? Za.errors : e.concat(Za.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["margin-top"] !== void 0) {
                let a = t["margin-top"], n = r;
                en(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-top", parentData: t, parentDataProperty: "margin-top", rootData: s }) || (e = e === null ? en.errors : e.concat(en.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["margin-bottom"] !== void 0) {
                let a = t["margin-bottom"], n = r;
                rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-bottom", parentData: t, parentDataProperty: "margin-bottom", rootData: s }) || (e = e === null ? rn.errors : e.concat(rn.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.geometry !== void 0) {
                let a = t.geometry, n = r;
                Hs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/geometry", parentData: t, parentDataProperty: "geometry", rootData: s }) || (e = e === null ? Hs.errors : e.concat(Hs.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.hyperrefoptions !== void 0) {
                let a = t.hyperrefoptions, n = r;
                Ls(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/hyperrefoptions", parentData: t, parentDataProperty: "hyperrefoptions", rootData: s }) || (e = e === null ? Ls.errors : e.concat(Ls.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.indent !== void 0) {
                let a = t.indent, n = r;
                Eo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indent", parentData: t, parentDataProperty: "indent", rootData: s }) || (e = e === null ? Eo.errors : e.concat(Eo.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["block-headings"] !== void 0) {
                let a = t["block-headings"], n = r;
                Ms(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/block-headings", parentData: t, parentDataProperty: "block-headings", rootData: s }) || (e = e === null ? Ms.errors : e.concat(Ms.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.keywords !== void 0) {
                let a = t.keywords, n = r;
                Ia(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keywords", parentData: t, parentDataProperty: "keywords", rootData: s }) || (e = e === null ? Ia.errors : e.concat(Ia.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.subject !== void 0) {
                let a = t.subject, n = r;
                fo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subject", parentData: t, parentDataProperty: "subject", rootData: s }) || (e = e === null ? fo.errors : e.concat(fo.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["title-meta"] !== void 0) {
                let a = t["title-meta"], n = r;
                Ts(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title-meta", parentData: t, parentDataProperty: "title-meta", rootData: s }) || (e = e === null ? Ts.errors : e.concat(Ts.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["author-meta"] !== void 0) {
                let a = t["author-meta"], n = r;
                dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author-meta", parentData: t, parentDataProperty: "author-meta", rootData: s }) || (e = e === null ? dn.errors : e.concat(dn.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["date-meta"] !== void 0) {
                let a = t["date-meta"], n = r;
                Ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date-meta", parentData: t, parentDataProperty: "date-meta", rootData: s }) || (e = e === null ? Ha.errors : e.concat(Ha.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["number-depth"] !== void 0) {
                let a = t["number-depth"], n = r;
                Da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-depth", parentData: t, parentDataProperty: "number-depth", rootData: s }) || (e = e === null ? Da.errors : e.concat(Da.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.secnumdepth !== void 0) {
                let a = t.secnumdepth, n = r;
                zs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/secnumdepth", parentData: t, parentDataProperty: "secnumdepth", rootData: s }) || (e = e === null ? zs.errors : e.concat(zs.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["top-level-division"] !== void 0) {
                let a = t["top-level-division"], n = r;
                Bn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/top-level-division", parentData: t, parentDataProperty: "top-level-division", rootData: s }) || (e = e === null ? Bn.errors : e.concat(Bn.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["pdf-engine"] !== void 0) {
                let a = t["pdf-engine"], n = r;
                mo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pdf-engine", parentData: t, parentDataProperty: "pdf-engine", rootData: s }) || (e = e === null ? mo.errors : e.concat(mo.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["pdf-engine-opt"] !== void 0) {
                let a = t["pdf-engine-opt"], n = r;
                go(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pdf-engine-opt", parentData: t, parentDataProperty: "pdf-engine-opt", rootData: s }) || (e = e === null ? go.errors : e.concat(go.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["cite-method"] !== void 0) {
                let a = t["cite-method"], n = r;
                Ks(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cite-method", parentData: t, parentDataProperty: "cite-method", rootData: s }) || (e = e === null ? Ks.errors : e.concat(Ks.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.biblatexoptions !== void 0) {
                let a = t.biblatexoptions, n = r;
                Js(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/biblatexoptions", parentData: t, parentDataProperty: "biblatexoptions", rootData: s }) || (e = e === null ? Js.errors : e.concat(Js.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.natbiboptions !== void 0) {
                let a = t.natbiboptions, n = r;
                Qs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/natbiboptions", parentData: t, parentDataProperty: "natbiboptions", rootData: s }) || (e = e === null ? Qs.errors : e.concat(Qs.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["biblio-style"] !== void 0) {
                let a = t["biblio-style"], n = r;
                Gs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/biblio-style", parentData: t, parentDataProperty: "biblio-style", rootData: s }) || (e = e === null ? Gs.errors : e.concat(Gs.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["biblio-title"] !== void 0) {
                let a = t["biblio-title"], n = r;
                Ys(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/biblio-title", parentData: t, parentDataProperty: "biblio-title", rootData: s }) || (e = e === null ? Ys.errors : e.concat(Ys.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Kr = n === r;
              }
              if (t["toc-title"] !== void 0) {
                let a = t["toc-title"], n = r;
                La(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-title", parentData: t, parentDataProperty: "toc-title", rootData: s }) || (e = e === null ? La.errors : e.concat(La.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.lof !== void 0) {
                let a = t.lof, n = r;
                Zs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lof", parentData: t, parentDataProperty: "lof", rootData: s }) || (e = e === null ? Zs.errors : e.concat(Zs.errors), r = e.length);
                var Kr = n === r;
              }
              if (t.lot !== void 0) {
                let a = t.lot, n = r;
                ei(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lot", parentData: t, parentDataProperty: "lot", rootData: s }) || (e = e === null ? ei.errors : e.concat(ei.errors), r = e.length);
                var Kr = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Elatex(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var mc = ht === r;
            mc && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Elatex(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var mc = gt === r;
            if (mc && L ? (L = false, M = [M, 1]) : mc && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Elatex(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^latex(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Vv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!V0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eman(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))", schema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eman(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var Zm = n === r;
                if (!Zm) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eman(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var St = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var St = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var St = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var St = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var St = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var St = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var St = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var St = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var St = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var St = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var St = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var St = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var St = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var St = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var St = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var St = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var St = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var St = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var St = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var St = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var St = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var St = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var St = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var St = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var St = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var St = n === r;
              }
              if (t.adjusting !== void 0) {
                let a = t.adjusting, n = r;
                wc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/adjusting", parentData: t, parentDataProperty: "adjusting", rootData: s }) || (e = e === null ? wc.errors : e.concat(wc.errors), r = e.length);
                var St = n === r;
              }
              if (t.hyphenate !== void 0) {
                let a = t.hyphenate, n = r;
                $c(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/hyphenate", parentData: t, parentDataProperty: "hyphenate", rootData: s }) || (e = e === null ? $c.errors : e.concat($c.errors), r = e.length);
                var St = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var St = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var St = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var St = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var St = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var St = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var St = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var St = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var St = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var St = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var St = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var St = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var St = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var St = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var St = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var St = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var St = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var St = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var St = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var St = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var St = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var St = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var St = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var St = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var St = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var St = n === r;
              }
              if (t.footer !== void 0) {
                let a = t.footer, n = r;
                Qf(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/footer", parentData: t, parentDataProperty: "footer", rootData: s }) || (e = e === null ? Qf.errors : e.concat(Qf.errors), r = e.length);
                var St = n === r;
              }
              if (t.header !== void 0) {
                let a = t.header, n = r;
                jc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header", parentData: t, parentDataProperty: "header", rootData: s }) || (e = e === null ? jc.errors : e.concat(jc.errors), r = e.length);
                var St = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var St = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var St = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var St = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var St = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var St = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var St = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var St = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var St = n === r;
              }
              if (t.section !== void 0) {
                let a = t.section, n = r;
                Yc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/section", parentData: t, parentDataProperty: "section", rootData: s }) || (e = e === null ? Yc.errors : e.concat(Yc.errors), r = e.length);
                var St = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var St = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var St = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var St = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var St = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var St = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var St = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var St = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var St = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var St = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var St = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var St = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var St = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var St = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var St = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var St = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var St = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var St = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var St = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var St = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var St = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var St = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var St = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var St = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eman(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var qm = ht === r;
            qm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eman(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var qm = gt === r;
            if (qm && L ? (L = false, M = [M, 1]) : qm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eman(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^man(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Uv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!v1.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var z1 = n === r;
                if (!z1) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["reference-location"] !== void 0) {
                let a = t["reference-location"], n = r;
                Ca(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-location", parentData: t, parentDataProperty: "reference-location", rootData: s }) || (e = e === null ? Ca.errors : e.concat(Ca.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["prefer-html"] !== void 0) {
                let a = t["prefer-html"], n = r;
                jo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/prefer-html", parentData: t, parentDataProperty: "prefer-html", rootData: s }) || (e = e === null ? jo.errors : e.concat(jo.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["identifier-prefix"] !== void 0) {
                let a = t["identifier-prefix"], n = r;
                Pa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier-prefix", parentData: t, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.variant !== void 0) {
                let a = t.variant, n = r;
                Lo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variant", parentData: t, parentDataProperty: "variant", rootData: s }) || (e = e === null ? Lo.errors : e.concat(Lo.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["markdown-headings"] !== void 0) {
                let a = t["markdown-headings"], n = r;
                ho(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/markdown-headings", parentData: t, parentDataProperty: "markdown-headings", rootData: s }) || (e = e === null ? ho.errors : e.concat(ho.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["keep-yaml"] !== void 0) {
                let a = t["keep-yaml"], n = r;
                Mo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-yaml", parentData: t, parentDataProperty: "keep-yaml", rootData: s }) || (e = e === null ? Mo.errors : e.concat(Mo.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["strip-comments"] !== void 0) {
                let a = t["strip-comments"], n = r;
                Ra(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-comments", parentData: t, parentDataProperty: "strip-comments", rootData: s }) || (e = e === null ? Ra.errors : e.concat(Ra.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var Ft = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Ft = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Ft = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Om = ht === r;
            Om && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Om = gt === r;
            if (Om && L ? (L = false, M = [M, 1]) : Om && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Kv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_github(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_github(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var W1 = n === r;
                if (!W1) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_github(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var zt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var zt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var zt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var zt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var zt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var zt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var zt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var zt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var zt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var zt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var zt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var zt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var zt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var zt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var zt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var zt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var zt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var zt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var zt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var zt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var zt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var zt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var zt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var zt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var zt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var zt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var zt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var zt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var zt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var zt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var zt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var zt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var zt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var zt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var zt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var zt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var zt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var zt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var zt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var zt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var zt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var zt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var zt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var zt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var zt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var zt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var zt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var zt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var zt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var zt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var zt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var zt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var zt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var zt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var zt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var zt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var zt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var zt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var zt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var zt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var zt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var zt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var zt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var zt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var zt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var zt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var zt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var zt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var zt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var zt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var zt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var zt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var zt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var zt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var zt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var zt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var zt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var zt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var zt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var zt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var zt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var zt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var zt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var zt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var zt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_github(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Pm = ht === r;
            Pm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_github(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Pm = gt === r;
            if (Pm && L ? (L = false, M = [M, 1]) : Pm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_github(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_github(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Jv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_mmd(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_mmd(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var R1 = n === r;
                if (!R1) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_mmd(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Wt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Wt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Wt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_mmd(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var xm = ht === r;
            xm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_mmd(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var xm = gt === r;
            if (xm && L ? (L = false, M = [M, 1]) : xm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_mmd(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_mmd(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Qv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_phpextra(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_phpextra(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var V1 = n === r;
                if (!V1) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_phpextra(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Rt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Rt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Rt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_phpextra(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var km = ht === r;
            km && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_phpextra(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var km = gt === r;
            if (km && L ? (L = false, M = [M, 1]) : km && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_phpextra(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_phpextra(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Gv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_strict(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_strict(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var U1 = n === r;
                if (!U1) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_strict(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Vt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Vt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Vt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_strict(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Dm = ht === r;
            Dm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_strict(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Dm = gt === r;
            if (Dm && L ? (L = false, M = [M, 1]) : Dm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkdown_strict(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markdown_strict(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Yv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!v1.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkua(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|identifier_prefix|identifierPrefix|markdown_headings|markdownHeadings|keep_yaml|keepYaml|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkua(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var K1 = n === r;
                if (!K1) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkua(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var At = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var At = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var At = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var At = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var At = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var At = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var At = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var At = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var At = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var At = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var At = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var At = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var At = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var At = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var At = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var At = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var At = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var At = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var At = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var At = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var At = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var At = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var At = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var At = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var At = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var At = n === r;
              }
              if (t["reference-location"] !== void 0) {
                let a = t["reference-location"], n = r;
                Ca(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-location", parentData: t, parentDataProperty: "reference-location", rootData: s }) || (e = e === null ? Ca.errors : e.concat(Ca.errors), r = e.length);
                var At = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var At = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var At = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var At = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var At = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var At = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var At = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var At = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var At = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var At = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var At = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var At = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var At = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var At = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var At = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var At = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var At = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var At = n === r;
              }
              if (t["prefer-html"] !== void 0) {
                let a = t["prefer-html"], n = r;
                jo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/prefer-html", parentData: t, parentDataProperty: "prefer-html", rootData: s }) || (e = e === null ? jo.errors : e.concat(jo.errors), r = e.length);
                var At = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var At = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var At = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var At = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var At = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var At = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var At = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var At = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var At = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var At = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var At = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var At = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var At = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var At = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var At = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var At = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var At = n === r;
              }
              if (t["identifier-prefix"] !== void 0) {
                let a = t["identifier-prefix"], n = r;
                Pa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier-prefix", parentData: t, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
                var At = n === r;
              }
              if (t.variant !== void 0) {
                let a = t.variant, n = r;
                Lo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variant", parentData: t, parentDataProperty: "variant", rootData: s }) || (e = e === null ? Lo.errors : e.concat(Lo.errors), r = e.length);
                var At = n === r;
              }
              if (t["markdown-headings"] !== void 0) {
                let a = t["markdown-headings"], n = r;
                ho(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/markdown-headings", parentData: t, parentDataProperty: "markdown-headings", rootData: s }) || (e = e === null ? ho.errors : e.concat(ho.errors), r = e.length);
                var At = n === r;
              }
              if (t["keep-yaml"] !== void 0) {
                let a = t["keep-yaml"], n = r;
                Mo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-yaml", parentData: t, parentDataProperty: "keep-yaml", rootData: s }) || (e = e === null ? Mo.errors : e.concat(Mo.errors), r = e.length);
                var At = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var At = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var At = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var At = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var At = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var At = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var At = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var At = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var At = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var At = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var At = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var At = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var At = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var At = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var At = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var At = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var At = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var At = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var At = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var At = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var At = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var At = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var At = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var At = n === r;
              }
              if (t["strip-comments"] !== void 0) {
                let a = t["strip-comments"], n = r;
                Ra(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-comments", parentData: t, parentDataProperty: "strip-comments", rootData: s }) || (e = e === null ? Ra.errors : e.concat(Ra.errors), r = e.length);
                var At = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var At = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var At = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var At = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var At = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkua(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var wm = ht === r;
            wm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkua(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var wm = gt === r;
            if (wm && L ? (L = false, M = [M, 1]) : wm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emarkua(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^markua(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Xv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emediawiki(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emediawiki(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var J1 = n === r;
                if (!J1) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emediawiki(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Ut = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Ut = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Ut = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emediawiki(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var $m = ht === r;
            $m && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emediawiki(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var $m = gt === r;
            if ($m && L ? (L = false, M = [M, 1]) : $m && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emediawiki(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^mediawiki(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (Zv.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!U0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ems(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_line_numbers|codeLineNumbers|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(code_line_numbers|codeLineNumbers|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(code_line_numbers|codeLineNumbers|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ems(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var Q1 = n === r;
                if (!Q1) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ems(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var xt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var xt = n === r;
              }
              if (t["code-line-numbers"] !== void 0) {
                let a = t["code-line-numbers"], n = r;
                ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-line-numbers", parentData: t, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
                var xt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var xt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var xt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var xt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var xt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var xt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var xt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var xt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var xt = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var xt = n === r;
              }
              if (t["highlight-style"] !== void 0) {
                let a = t["highlight-style"], n = r;
                _a(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/highlight-style", parentData: t, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
                var xt = n === r;
              }
              if (t["syntax-definition"] !== void 0) {
                let a = t["syntax-definition"], n = r;
                va(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definition", parentData: t, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
                var xt = n === r;
              }
              if (t["syntax-definitions"] !== void 0) {
                let a = t["syntax-definitions"], n = r;
                qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definitions", parentData: t, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
                var xt = n === r;
              }
              if (t["indented-code-classes"] !== void 0) {
                let a = t["indented-code-classes"], n = r;
                Oa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indented-code-classes", parentData: t, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
                var xt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var xt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var xt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var xt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var xt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var xt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var xt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var xt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var xt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var xt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var xt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var xt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var xt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var xt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var xt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var xt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var xt = n === r;
              }
              if (t.fontfamily !== void 0) {
                let a = t.fontfamily, n = r;
                Bo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontfamily", parentData: t, parentDataProperty: "fontfamily", rootData: s }) || (e = e === null ? Bo.errors : e.concat(Bo.errors), r = e.length);
                var xt = n === r;
              }
              if (t.pointsize !== void 0) {
                let a = t.pointsize, n = r;
                qc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pointsize", parentData: t, parentDataProperty: "pointsize", rootData: s }) || (e = e === null ? qc.errors : e.concat(qc.errors), r = e.length);
                var xt = n === r;
              }
              if (t.lineheight !== void 0) {
                let a = t.lineheight, n = r;
                Oc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lineheight", parentData: t, parentDataProperty: "lineheight", rootData: s }) || (e = e === null ? Oc.errors : e.concat(Oc.errors), r = e.length);
                var xt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var xt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var xt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var xt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var xt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var xt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var xt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var xt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var xt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var xt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var xt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var xt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var xt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var xt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var xt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var xt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var xt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var xt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var xt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var xt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var xt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var xt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var xt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var xt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var xt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var xt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var xt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var xt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var xt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var xt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var xt = n === r;
              }
              if (t.indent !== void 0) {
                let a = t.indent, n = r;
                Eo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indent", parentData: t, parentDataProperty: "indent", rootData: s }) || (e = e === null ? Eo.errors : e.concat(Eo.errors), r = e.length);
                var xt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var xt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var xt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var xt = n === r;
              }
              if (t["pdf-engine"] !== void 0) {
                let a = t["pdf-engine"], n = r;
                mo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pdf-engine", parentData: t, parentDataProperty: "pdf-engine", rootData: s }) || (e = e === null ? mo.errors : e.concat(mo.errors), r = e.length);
                var xt = n === r;
              }
              if (t["pdf-engine-opt"] !== void 0) {
                let a = t["pdf-engine-opt"], n = r;
                go(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pdf-engine-opt", parentData: t, parentDataProperty: "pdf-engine-opt", rootData: s }) || (e = e === null ? go.errors : e.concat(go.errors), r = e.length);
                var xt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var xt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var xt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var xt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var xt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var xt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var xt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var xt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var xt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var xt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var xt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var xt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var xt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var xt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var xt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var xt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var xt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var xt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var xt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var xt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var xt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var xt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var xt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var xt = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var xt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var xt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var xt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var xt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ems(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Sm = ht === r;
            Sm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ems(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Sm = gt === r;
            if (Sm && L ? (L = false, M = [M, 1]) : Sm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ems(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^ms(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (e1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!K0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emuse(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emuse(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var G1 = n === r;
                if (!G1) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emuse(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["reference-location"] !== void 0) {
                let a = t["reference-location"], n = r;
                Ca(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-location", parentData: t, parentDataProperty: "reference-location", rootData: s }) || (e = e === null ? Ca.errors : e.concat(Ca.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Ct = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Ct = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Ct = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emuse(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Fm = ht === r;
            Fm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emuse(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Fm = gt === r;
            if (Fm && L ? (L = false, M = [M, 1]) : Fm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Emuse(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^muse(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (r1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Enative(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Enative(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var Y1 = n === r;
                if (!Y1) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Enative(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Kt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Kt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Kt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Enative(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Am = ht === r;
            Am && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Enative(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Am = gt === r;
            if (Am && L ? (L = false, M = [M, 1]) : Am && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Enative(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^native(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (t1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!J0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eodt(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(fig_align|figAlign|tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_width|pageWidth|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|reference_doc|referenceDoc|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", schema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(fig_align|figAlign|tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_width|pageWidth|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|reference_doc|referenceDoc|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))"', schema: "^(?!(fig_align|figAlign|tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_width|pageWidth|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|reference_doc|referenceDoc|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eodt(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var X1 = n === r;
                if (!X1) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eodt(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var It = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var It = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var It = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var It = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var It = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var It = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var It = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var It = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var It = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var It = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var It = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var It = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var It = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var It = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var It = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var It = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var It = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var It = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var It = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var It = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var It = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var It = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var It = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var It = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var It = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var It = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var It = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var It = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var It = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var It = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var It = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var It = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var It = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var It = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var It = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var It = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var It = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var It = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var It = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var It = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var It = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var It = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var It = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var It = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var It = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var It = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var It = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var It = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var It = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var It = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var It = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var It = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var It = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var It = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var It = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var It = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var It = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var It = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var It = n === r;
              }
              if (t["page-width"] !== void 0) {
                let a = t["page-width"], n = r;
                Is(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/page-width", parentData: t, parentDataProperty: "page-width", rootData: s }) || (e = e === null ? Is.errors : e.concat(Is.errors), r = e.length);
                var It = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var It = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var It = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var It = n === r;
              }
              if (t["reference-doc"] !== void 0) {
                let a = t["reference-doc"], n = r;
                Ws(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-doc", parentData: t, parentDataProperty: "reference-doc", rootData: s }) || (e = e === null ? Ws.errors : e.concat(Ws.errors), r = e.length);
                var It = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var It = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var It = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var It = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var It = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var It = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var It = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var It = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var It = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var It = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var It = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var It = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var It = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var It = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var It = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var It = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var It = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var It = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var It = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var It = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var It = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var It = n === r;
              }
              if (t["toc-title"] !== void 0) {
                let a = t["toc-title"], n = r;
                La(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-title", parentData: t, parentDataProperty: "toc-title", rootData: s }) || (e = e === null ? La.errors : e.concat(La.errors), r = e.length);
                var It = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eodt(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Bm = ht === r;
            Bm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eodt(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Bm = gt === r;
            if (Bm && L ? (L = false, M = [M, 1]) : Bm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eodt(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^odt(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (a1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Q0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eopendocument(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(fig_align|figAlign|tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_width|pageWidth|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", schema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(fig_align|figAlign|tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_width|pageWidth|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))"', schema: "^(?!(fig_align|figAlign|tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|page_width|pageWidth|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eopendocument(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var Z1 = n === r;
                if (!Z1) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eopendocument(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var ra = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var ra = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var ra = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var ra = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var ra = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var ra = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var ra = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var ra = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var ra = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var ra = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var ra = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var ra = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var ra = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var ra = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var ra = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var ra = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var ra = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var ra = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var ra = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var ra = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var ra = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var ra = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var ra = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var ra = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var ra = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var ra = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var ra = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var ra = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var ra = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var ra = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var ra = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var ra = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var ra = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var ra = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var ra = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var ra = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var ra = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var ra = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var ra = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var ra = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var ra = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var ra = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var ra = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var ra = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var ra = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var ra = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var ra = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var ra = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var ra = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var ra = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var ra = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var ra = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var ra = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var ra = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var ra = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var ra = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var ra = n === r;
              }
              if (t["page-width"] !== void 0) {
                let a = t["page-width"], n = r;
                Is(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/page-width", parentData: t, parentDataProperty: "page-width", rootData: s }) || (e = e === null ? Is.errors : e.concat(Is.errors), r = e.length);
                var ra = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var ra = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var ra = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var ra = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var ra = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var ra = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var ra = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var ra = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var ra = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var ra = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var ra = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var ra = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var ra = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var ra = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var ra = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var ra = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var ra = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var ra = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var ra = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var ra = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var ra = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var ra = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var ra = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var ra = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var ra = n === r;
              }
              if (t["toc-title"] !== void 0) {
                let a = t["toc-title"], n = r;
                La(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-title", parentData: t, parentDataProperty: "toc-title", rootData: s }) || (e = e === null ? La.errors : e.concat(La.errors), r = e.length);
                var ra = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eopendocument(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var jm = ht === r;
            jm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eopendocument(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var jm = gt === r;
            if (jm && L ? (L = false, M = [M, 1]) : jm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eopendocument(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^opendocument(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (n1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eopml(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eopml(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var e0 = n === r;
                if (!e0) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eopml(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Jt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Jt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Jt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eopml(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Em = ht === r;
            Em && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eopml(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Em = gt === r;
            if (Em && L ? (L = false, M = [M, 1]) : Em && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eopml(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^opml(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (o1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eorg(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eorg(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var r0 = n === r;
                if (!r0) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eorg(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Qt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Qt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Qt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eorg(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Nm = ht === r;
            Nm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eorg(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Nm = gt === r;
            if (Nm && L ? (L = false, M = [M, 1]) : Nm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eorg(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^org(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (s1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!G0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Epdf(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|links_as_notes|linksAsNotes|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|latex_auto_mk|latexAutoMk|latex_auto_install|latexAutoInstall|latex_min_runs|latexMinRuns|latex_max_runs|latexMaxRuns|latex_clean|latexClean|latex_makeindex|latexMakeindex|latex_makeindex_opts|latexMakeindexOpts|latex_tlmgr_opts|latexTlmgrOpts|latex_output_dir|latexOutputDir|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|title_meta|titleMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|keep_tex|keepTex|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", schema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|links_as_notes|linksAsNotes|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|latex_auto_mk|latexAutoMk|latex_auto_install|latexAutoInstall|latex_min_runs|latexMinRuns|latex_max_runs|latexMaxRuns|latex_clean|latexClean|latex_makeindex|latexMakeindex|latex_makeindex_opts|latexMakeindexOpts|latex_tlmgr_opts|latexTlmgrOpts|latex_output_dir|latexOutputDir|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|title_meta|titleMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|keep_tex|keepTex|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))"', schema: "^(?!(code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|links_as_notes|linksAsNotes|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|latex_auto_mk|latexAutoMk|latex_auto_install|latexAutoInstall|latex_min_runs|latexMinRuns|latex_max_runs|latexMaxRuns|latex_clean|latexClean|latex_makeindex|latexMakeindex|latex_makeindex_opts|latexMakeindexOpts|latex_tlmgr_opts|latexTlmgrOpts|latex_output_dir|latexOutputDir|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|title_meta|titleMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|keep_tex|keepTex|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Epdf(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var t0 = n === r;
                if (!t0) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Epdf(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["code-line-numbers"] !== void 0) {
                let a = t["code-line-numbers"], n = r;
                ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-line-numbers", parentData: t, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["fig-env"] !== void 0) {
                let a = t["fig-env"], n = r;
                Zn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-env", parentData: t, parentDataProperty: "fig-env", rootData: s }) || (e = e === null ? Zn.errors : e.concat(Zn.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["fig-pos"] !== void 0) {
                let a = t["fig-pos"], n = r;
                eo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-pos", parentData: t, parentDataProperty: "fig-pos", rootData: s }) || (e = e === null ? eo.errors : e.concat(eo.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["cap-location"] !== void 0) {
                let a = t["cap-location"], n = r;
                ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cap-location", parentData: t, parentDataProperty: "cap-location", rootData: s }) || (e = e === null ? ja.errors : e.concat(ja.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["fig-cap-location"] !== void 0) {
                let a = t["fig-cap-location"], n = r;
                Ea(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-cap-location", parentData: t, parentDataProperty: "fig-cap-location", rootData: s }) || (e = e === null ? Ea.errors : e.concat(Ea.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["tbl-cap-location"] !== void 0) {
                let a = t["tbl-cap-location"], n = r;
                Na(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-cap-location", parentData: t, parentDataProperty: "tbl-cap-location", rootData: s }) || (e = e === null ? Na.errors : e.concat(Na.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.abstract !== void 0) {
                let a = t.abstract, n = r;
                ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abstract", parentData: t, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.thanks !== void 0) {
                let a = t.thanks, n = r;
                Qo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/thanks", parentData: t, parentDataProperty: "thanks", rootData: s }) || (e = e === null ? Qo.errors : e.concat(Qo.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["highlight-style"] !== void 0) {
                let a = t["highlight-style"], n = r;
                _a(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/highlight-style", parentData: t, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["syntax-definition"] !== void 0) {
                let a = t["syntax-definition"], n = r;
                va(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definition", parentData: t, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["syntax-definitions"] !== void 0) {
                let a = t["syntax-definitions"], n = r;
                qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definitions", parentData: t, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.listings !== void 0) {
                let a = t.listings, n = r;
                Yo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/listings", parentData: t, parentDataProperty: "listings", rootData: s }) || (e = e === null ? Yo.errors : e.concat(Yo.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["indented-code-classes"] !== void 0) {
                let a = t["indented-code-classes"], n = r;
                Oa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indented-code-classes", parentData: t, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.linkcolor !== void 0) {
                let a = t.linkcolor, n = r;
                Ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linkcolor", parentData: t, parentDataProperty: "linkcolor", rootData: s }) || (e = e === null ? Ua.errors : e.concat(Ua.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.filecolor !== void 0) {
                let a = t.filecolor, n = r;
                Xo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filecolor", parentData: t, parentDataProperty: "filecolor", rootData: s }) || (e = e === null ? Xo.errors : e.concat(Xo.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.citecolor !== void 0) {
                let a = t.citecolor, n = r;
                Zo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citecolor", parentData: t, parentDataProperty: "citecolor", rootData: s }) || (e = e === null ? Zo.errors : e.concat(Zo.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.urlcolor !== void 0) {
                let a = t.urlcolor, n = r;
                es(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/urlcolor", parentData: t, parentDataProperty: "urlcolor", rootData: s }) || (e = e === null ? es.errors : e.concat(es.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.toccolor !== void 0) {
                let a = t.toccolor, n = r;
                rs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toccolor", parentData: t, parentDataProperty: "toccolor", rootData: s }) || (e = e === null ? rs.errors : e.concat(rs.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.colorlinks !== void 0) {
                let a = t.colorlinks, n = r;
                ts(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/colorlinks", parentData: t, parentDataProperty: "colorlinks", rootData: s }) || (e = e === null ? ts.errors : e.concat(ts.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.mainfont !== void 0) {
                let a = t.mainfont, n = r;
                Ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mainfont", parentData: t, parentDataProperty: "mainfont", rootData: s }) || (e = e === null ? Ja.errors : e.concat(Ja.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.monofont !== void 0) {
                let a = t.monofont, n = r;
                Qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monofont", parentData: t, parentDataProperty: "monofont", rootData: s }) || (e = e === null ? Qa.errors : e.concat(Qa.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.fontsize !== void 0) {
                let a = t.fontsize, n = r;
                Ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontsize", parentData: t, parentDataProperty: "fontsize", rootData: s }) || (e = e === null ? Ga.errors : e.concat(Ga.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.fontenc !== void 0) {
                let a = t.fontenc, n = r;
                _s(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontenc", parentData: t, parentDataProperty: "fontenc", rootData: s }) || (e = e === null ? _s.errors : e.concat(_s.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.fontfamily !== void 0) {
                let a = t.fontfamily, n = r;
                Bo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontfamily", parentData: t, parentDataProperty: "fontfamily", rootData: s }) || (e = e === null ? Bo.errors : e.concat(Bo.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.fontfamilyoptions !== void 0) {
                let a = t.fontfamilyoptions, n = r;
                vs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontfamilyoptions", parentData: t, parentDataProperty: "fontfamilyoptions", rootData: s }) || (e = e === null ? vs.errors : e.concat(vs.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.sansfont !== void 0) {
                let a = t.sansfont, n = r;
                qs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/sansfont", parentData: t, parentDataProperty: "sansfont", rootData: s }) || (e = e === null ? qs.errors : e.concat(qs.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.mathfont !== void 0) {
                let a = t.mathfont, n = r;
                Os(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mathfont", parentData: t, parentDataProperty: "mathfont", rootData: s }) || (e = e === null ? Os.errors : e.concat(Os.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.CJKmainfont !== void 0) {
                let a = t.CJKmainfont, n = r;
                Ps(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/CJKmainfont", parentData: t, parentDataProperty: "CJKmainfont", rootData: s }) || (e = e === null ? Ps.errors : e.concat(Ps.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.mainfontoptions !== void 0) {
                let a = t.mainfontoptions, n = r;
                xs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mainfontoptions", parentData: t, parentDataProperty: "mainfontoptions", rootData: s }) || (e = e === null ? xs.errors : e.concat(xs.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.sansfontoptions !== void 0) {
                let a = t.sansfontoptions, n = r;
                ks(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/sansfontoptions", parentData: t, parentDataProperty: "sansfontoptions", rootData: s }) || (e = e === null ? ks.errors : e.concat(ks.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.monofontoptions !== void 0) {
                let a = t.monofontoptions, n = r;
                Ds(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monofontoptions", parentData: t, parentDataProperty: "monofontoptions", rootData: s }) || (e = e === null ? Ds.errors : e.concat(Ds.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.mathfontoptions !== void 0) {
                let a = t.mathfontoptions, n = r;
                ws(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mathfontoptions", parentData: t, parentDataProperty: "mathfontoptions", rootData: s }) || (e = e === null ? ws.errors : e.concat(ws.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.CJKoptions !== void 0) {
                let a = t.CJKoptions, n = r;
                $s(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/CJKoptions", parentData: t, parentDataProperty: "CJKoptions", rootData: s }) || (e = e === null ? $s.errors : e.concat($s.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.microtypeoptions !== void 0) {
                let a = t.microtypeoptions, n = r;
                Ss(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/microtypeoptions", parentData: t, parentDataProperty: "microtypeoptions", rootData: s }) || (e = e === null ? Ss.errors : e.concat(Ss.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.linestretch !== void 0) {
                let a = t.linestretch, n = r;
                Ya(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linestretch", parentData: t, parentDataProperty: "linestretch", rootData: s }) || (e = e === null ? Ya.errors : e.concat(Ya.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["links-as-notes"] !== void 0) {
                let a = t["links-as-notes"], n = r;
                Fs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/links-as-notes", parentData: t, parentDataProperty: "links-as-notes", rootData: s }) || (e = e === null ? Fs.errors : e.concat(Fs.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["reference-location"] !== void 0) {
                let a = t["reference-location"], n = r;
                Ca(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-location", parentData: t, parentDataProperty: "reference-location", rootData: s }) || (e = e === null ? Ca.errors : e.concat(Ca.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["latex-auto-mk"] !== void 0) {
                let a = t["latex-auto-mk"], n = r;
                El(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-auto-mk", parentData: t, parentDataProperty: "latex-auto-mk", rootData: s }) || (e = e === null ? El.errors : e.concat(El.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["latex-auto-install"] !== void 0) {
                let a = t["latex-auto-install"], n = r;
                Nl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-auto-install", parentData: t, parentDataProperty: "latex-auto-install", rootData: s }) || (e = e === null ? Nl.errors : e.concat(Nl.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["latex-min-runs"] !== void 0) {
                let a = t["latex-min-runs"], n = r;
                Cl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-min-runs", parentData: t, parentDataProperty: "latex-min-runs", rootData: s }) || (e = e === null ? Cl.errors : e.concat(Cl.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["latex-max-runs"] !== void 0) {
                let a = t["latex-max-runs"], n = r;
                Il(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-max-runs", parentData: t, parentDataProperty: "latex-max-runs", rootData: s }) || (e = e === null ? Il.errors : e.concat(Il.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["latex-clean"] !== void 0) {
                let a = t["latex-clean"], n = r;
                Hl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-clean", parentData: t, parentDataProperty: "latex-clean", rootData: s }) || (e = e === null ? Hl.errors : e.concat(Hl.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["latex-makeindex"] !== void 0) {
                let a = t["latex-makeindex"], n = r;
                Ll(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-makeindex", parentData: t, parentDataProperty: "latex-makeindex", rootData: s }) || (e = e === null ? Ll.errors : e.concat(Ll.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["latex-makeindex-opts"] !== void 0) {
                let a = t["latex-makeindex-opts"], n = r;
                Ml(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-makeindex-opts", parentData: t, parentDataProperty: "latex-makeindex-opts", rootData: s }) || (e = e === null ? Ml.errors : e.concat(Ml.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["latex-tlmgr-opts"] !== void 0) {
                let a = t["latex-tlmgr-opts"], n = r;
                Tl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-tlmgr-opts", parentData: t, parentDataProperty: "latex-tlmgr-opts", rootData: s }) || (e = e === null ? Tl.errors : e.concat(Tl.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["latex-output-dir"] !== void 0) {
                let a = t["latex-output-dir"], n = r;
                zl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/latex-output-dir", parentData: t, parentDataProperty: "latex-output-dir", rootData: s }) || (e = e === null ? zl.errors : e.concat(zl.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.documentclass !== void 0) {
                let a = t.documentclass, n = r;
                js(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/documentclass", parentData: t, parentDataProperty: "documentclass", rootData: s }) || (e = e === null ? js.errors : e.concat(js.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.classoption !== void 0) {
                let a = t.classoption, n = r;
                un(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/classoption", parentData: t, parentDataProperty: "classoption", rootData: s }) || (e = e === null ? un.errors : e.concat(un.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.pagestyle !== void 0) {
                let a = t.pagestyle, n = r;
                Es(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pagestyle", parentData: t, parentDataProperty: "pagestyle", rootData: s }) || (e = e === null ? Es.errors : e.concat(Es.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.papersize !== void 0) {
                let a = t.papersize, n = r;
                Ns(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/papersize", parentData: t, parentDataProperty: "papersize", rootData: s }) || (e = e === null ? Ns.errors : e.concat(Ns.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["margin-left"] !== void 0) {
                let a = t["margin-left"], n = r;
                Xa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-left", parentData: t, parentDataProperty: "margin-left", rootData: s }) || (e = e === null ? Xa.errors : e.concat(Xa.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["margin-right"] !== void 0) {
                let a = t["margin-right"], n = r;
                Za(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-right", parentData: t, parentDataProperty: "margin-right", rootData: s }) || (e = e === null ? Za.errors : e.concat(Za.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["margin-top"] !== void 0) {
                let a = t["margin-top"], n = r;
                en(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-top", parentData: t, parentDataProperty: "margin-top", rootData: s }) || (e = e === null ? en.errors : e.concat(en.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["margin-bottom"] !== void 0) {
                let a = t["margin-bottom"], n = r;
                rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-bottom", parentData: t, parentDataProperty: "margin-bottom", rootData: s }) || (e = e === null ? rn.errors : e.concat(rn.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.geometry !== void 0) {
                let a = t.geometry, n = r;
                Hs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/geometry", parentData: t, parentDataProperty: "geometry", rootData: s }) || (e = e === null ? Hs.errors : e.concat(Hs.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.hyperrefoptions !== void 0) {
                let a = t.hyperrefoptions, n = r;
                Ls(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/hyperrefoptions", parentData: t, parentDataProperty: "hyperrefoptions", rootData: s }) || (e = e === null ? Ls.errors : e.concat(Ls.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.indent !== void 0) {
                let a = t.indent, n = r;
                Eo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indent", parentData: t, parentDataProperty: "indent", rootData: s }) || (e = e === null ? Eo.errors : e.concat(Eo.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["block-headings"] !== void 0) {
                let a = t["block-headings"], n = r;
                Ms(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/block-headings", parentData: t, parentDataProperty: "block-headings", rootData: s }) || (e = e === null ? Ms.errors : e.concat(Ms.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.keywords !== void 0) {
                let a = t.keywords, n = r;
                Ia(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keywords", parentData: t, parentDataProperty: "keywords", rootData: s }) || (e = e === null ? Ia.errors : e.concat(Ia.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.subject !== void 0) {
                let a = t.subject, n = r;
                fo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subject", parentData: t, parentDataProperty: "subject", rootData: s }) || (e = e === null ? fo.errors : e.concat(fo.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["title-meta"] !== void 0) {
                let a = t["title-meta"], n = r;
                Ts(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title-meta", parentData: t, parentDataProperty: "title-meta", rootData: s }) || (e = e === null ? Ts.errors : e.concat(Ts.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["author-meta"] !== void 0) {
                let a = t["author-meta"], n = r;
                dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author-meta", parentData: t, parentDataProperty: "author-meta", rootData: s }) || (e = e === null ? dn.errors : e.concat(dn.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["date-meta"] !== void 0) {
                let a = t["date-meta"], n = r;
                Ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date-meta", parentData: t, parentDataProperty: "date-meta", rootData: s }) || (e = e === null ? Ha.errors : e.concat(Ha.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["number-depth"] !== void 0) {
                let a = t["number-depth"], n = r;
                Da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-depth", parentData: t, parentDataProperty: "number-depth", rootData: s }) || (e = e === null ? Da.errors : e.concat(Da.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.secnumdepth !== void 0) {
                let a = t.secnumdepth, n = r;
                zs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/secnumdepth", parentData: t, parentDataProperty: "secnumdepth", rootData: s }) || (e = e === null ? zs.errors : e.concat(zs.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["top-level-division"] !== void 0) {
                let a = t["top-level-division"], n = r;
                Bn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/top-level-division", parentData: t, parentDataProperty: "top-level-division", rootData: s }) || (e = e === null ? Bn.errors : e.concat(Bn.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["pdf-engine"] !== void 0) {
                let a = t["pdf-engine"], n = r;
                mo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pdf-engine", parentData: t, parentDataProperty: "pdf-engine", rootData: s }) || (e = e === null ? mo.errors : e.concat(mo.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["pdf-engine-opt"] !== void 0) {
                let a = t["pdf-engine-opt"], n = r;
                go(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pdf-engine-opt", parentData: t, parentDataProperty: "pdf-engine-opt", rootData: s }) || (e = e === null ? go.errors : e.concat(go.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["cite-method"] !== void 0) {
                let a = t["cite-method"], n = r;
                Ks(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cite-method", parentData: t, parentDataProperty: "cite-method", rootData: s }) || (e = e === null ? Ks.errors : e.concat(Ks.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.biblatexoptions !== void 0) {
                let a = t.biblatexoptions, n = r;
                Js(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/biblatexoptions", parentData: t, parentDataProperty: "biblatexoptions", rootData: s }) || (e = e === null ? Js.errors : e.concat(Js.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.natbiboptions !== void 0) {
                let a = t.natbiboptions, n = r;
                Qs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/natbiboptions", parentData: t, parentDataProperty: "natbiboptions", rootData: s }) || (e = e === null ? Qs.errors : e.concat(Qs.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["biblio-style"] !== void 0) {
                let a = t["biblio-style"], n = r;
                Gs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/biblio-style", parentData: t, parentDataProperty: "biblio-style", rootData: s }) || (e = e === null ? Gs.errors : e.concat(Gs.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["biblio-title"] !== void 0) {
                let a = t["biblio-title"], n = r;
                Ys(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/biblio-title", parentData: t, parentDataProperty: "biblio-title", rootData: s }) || (e = e === null ? Ys.errors : e.concat(Ys.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["keep-tex"] !== void 0) {
                let a = t["keep-tex"], n = r;
                Vl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-tex", parentData: t, parentDataProperty: "keep-tex", rootData: s }) || (e = e === null ? Vl.errors : e.concat(Vl.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Vr = n === r;
              }
              if (t["toc-title"] !== void 0) {
                let a = t["toc-title"], n = r;
                La(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-title", parentData: t, parentDataProperty: "toc-title", rootData: s }) || (e = e === null ? La.errors : e.concat(La.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.lof !== void 0) {
                let a = t.lof, n = r;
                Zs(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lof", parentData: t, parentDataProperty: "lof", rootData: s }) || (e = e === null ? Zs.errors : e.concat(Zs.errors), r = e.length);
                var Vr = n === r;
              }
              if (t.lot !== void 0) {
                let a = t.lot, n = r;
                ei(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lot", parentData: t, parentDataProperty: "lot", rootData: s }) || (e = e === null ? ei.errors : e.concat(ei.errors), r = e.length);
                var Vr = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Epdf(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cm = ht === r;
            Cm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Epdf(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cm = gt === r;
            if (Cm && L ? (L = false, M = [M, 1]) : Cm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Epdf(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^pdf(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (i1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eplain(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eplain(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var a0 = n === r;
                if (!a0) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eplain(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Gt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Gt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Gt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eplain(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Im = ht === r;
            Im && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eplain(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Im = gt === r;
            if (Im && L ? (L = false, M = [M, 1]) : Im && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eplain(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^plain(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (l1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Y0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Epptx(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|reference_doc|referenceDoc|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", schema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|reference_doc|referenceDoc|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|reference_doc|referenceDoc|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Epptx(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var n0 = n === r;
                if (!n0) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Epptx(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var na = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var na = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var na = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var na = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var na = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var na = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var na = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var na = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var na = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var na = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var na = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var na = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var na = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var na = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var na = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var na = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var na = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var na = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var na = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var na = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var na = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var na = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var na = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var na = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var na = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var na = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var na = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var na = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var na = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var na = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var na = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var na = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var na = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var na = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var na = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var na = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var na = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var na = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var na = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var na = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var na = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var na = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var na = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var na = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var na = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var na = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var na = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var na = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var na = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var na = n === r;
              }
              if (t.subject !== void 0) {
                let a = t.subject, n = r;
                fo(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subject", parentData: t, parentDataProperty: "subject", rootData: s }) || (e = e === null ? fo.errors : e.concat(fo.errors), r = e.length);
                var na = n === r;
              }
              if (t.description !== void 0) {
                let a = t.description, n = r;
                Wl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/description", parentData: t, parentDataProperty: "description", rootData: s }) || (e = e === null ? Wl.errors : e.concat(Wl.errors), r = e.length);
                var na = n === r;
              }
              if (t.category !== void 0) {
                let a = t.category, n = r;
                Rl(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/category", parentData: t, parentDataProperty: "category", rootData: s }) || (e = e === null ? Rl.errors : e.concat(Rl.errors), r = e.length);
                var na = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var na = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var na = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var na = n === r;
              }
              if (t["reference-doc"] !== void 0) {
                let a = t["reference-doc"], n = r;
                Ws(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-doc", parentData: t, parentDataProperty: "reference-doc", rootData: s }) || (e = e === null ? Ws.errors : e.concat(Ws.errors), r = e.length);
                var na = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var na = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var na = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var na = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var na = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var na = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var na = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var na = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var na = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var na = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var na = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var na = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var na = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var na = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var na = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var na = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var na = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var na = n === r;
              }
              if (t.incremental !== void 0) {
                let a = t.incremental, n = r;
                Rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/incremental", parentData: t, parentDataProperty: "incremental", rootData: s }) || (e = e === null ? Rn.errors : e.concat(Rn.errors), r = e.length);
                var na = n === r;
              }
              if (t["slide-level"] !== void 0) {
                let a = t["slide-level"], n = r;
                Vn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/slide-level", parentData: t, parentDataProperty: "slide-level", rootData: s }) || (e = e === null ? Vn.errors : e.concat(Vn.errors), r = e.length);
                var na = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var na = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var na = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var na = n === r;
              }
              if (t["toc-title"] !== void 0) {
                let a = t["toc-title"], n = r;
                La(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-title", parentData: t, parentDataProperty: "toc-title", rootData: s }) || (e = e === null ? La.errors : e.concat(La.errors), r = e.length);
                var na = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Epptx(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Hm = ht === r;
            Hm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Epptx(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Hm = gt === r;
            if (Hm && L ? (L = false, M = [M, 1]) : Hm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Epptx(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^pptx(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (c1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!X0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Erevealjs(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|reveal_js_url|revealJsUrl|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|auto_stretch|autoStretch|min_scale|minScale|max_scale|maxScale|disable_layout|disableLayout|preview_links|previewLinks|auto_play_media|autoPlayMedia|preload_iframes|preloadIframes|view_distance|viewDistance|mobile_view_distance|mobileViewDistance|parallax_background_image|parallaxBackgroundImage|parallax_background_size|parallaxBackgroundSize|parallax_background_horizontal|parallaxBackgroundHorizontal|parallax_background_vertical|parallaxBackgroundVertical|navigation_mode|navigationMode|mouse_wheel|mouseWheel|hide_inactive_cursor|hideInactiveCursor|hide_cursor_time|hideCursorTime|controls_layout|controlsLayout|controls_tutorial|controlsTutorial|controls_back_arrows|controlsBackArrows|auto_slide|autoSlide|auto_slide_stoppable|autoSlideStoppable|auto_slide_method|autoSlideMethod|default_timing|defaultTiming|hash_type|hashType|hash_one_based_index|hashOneBasedIndex|respond_to_hash_changes|respondToHashChanges|fragment_in_url|fragmentInUrl|slide_tone|slideTone|pdf_separate_fragments|pdfSeparateFragments|pdf_page_height_offset|pdfPageHeightOffset|transition_speed|transitionSpeed|background_transition|backgroundTransition|auto_animate|autoAnimate|auto_animate_easing|autoAnimateEasing|auto_animate_duration|autoAnimateDuration|auto_animate_unmatched|autoAnimateUnmatched|auto_animate_styles|autoAnimateStyles|slide_level|slideLevel|slide_number|slideNumber|show_slide_number|showSlideNumber|title_slide_attributes|titleSlideAttributes|center_title_slide|centerTitleSlide|show_notes|showNotes|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", schema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|reveal_js_url|revealJsUrl|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|auto_stretch|autoStretch|min_scale|minScale|max_scale|maxScale|disable_layout|disableLayout|preview_links|previewLinks|auto_play_media|autoPlayMedia|preload_iframes|preloadIframes|view_distance|viewDistance|mobile_view_distance|mobileViewDistance|parallax_background_image|parallaxBackgroundImage|parallax_background_size|parallaxBackgroundSize|parallax_background_horizontal|parallaxBackgroundHorizontal|parallax_background_vertical|parallaxBackgroundVertical|navigation_mode|navigationMode|mouse_wheel|mouseWheel|hide_inactive_cursor|hideInactiveCursor|hide_cursor_time|hideCursorTime|controls_layout|controlsLayout|controls_tutorial|controlsTutorial|controls_back_arrows|controlsBackArrows|auto_slide|autoSlide|auto_slide_stoppable|autoSlideStoppable|auto_slide_method|autoSlideMethod|default_timing|defaultTiming|hash_type|hashType|hash_one_based_index|hashOneBasedIndex|respond_to_hash_changes|respondToHashChanges|fragment_in_url|fragmentInUrl|slide_tone|slideTone|pdf_separate_fragments|pdfSeparateFragments|pdf_page_height_offset|pdfPageHeightOffset|transition_speed|transitionSpeed|background_transition|backgroundTransition|auto_animate|autoAnimate|auto_animate_easing|autoAnimateEasing|auto_animate_duration|autoAnimateDuration|auto_animate_unmatched|autoAnimateUnmatched|auto_animate_styles|autoAnimateStyles|slide_level|slideLevel|slide_number|slideNumber|show_slide_number|showSlideNumber|title_slide_attributes|titleSlideAttributes|center_title_slide|centerTitleSlide|show_notes|showNotes|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))"', schema: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|reveal_js_url|revealJsUrl|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|auto_stretch|autoStretch|min_scale|minScale|max_scale|maxScale|disable_layout|disableLayout|preview_links|previewLinks|auto_play_media|autoPlayMedia|preload_iframes|preloadIframes|view_distance|viewDistance|mobile_view_distance|mobileViewDistance|parallax_background_image|parallaxBackgroundImage|parallax_background_size|parallaxBackgroundSize|parallax_background_horizontal|parallaxBackgroundHorizontal|parallax_background_vertical|parallaxBackgroundVertical|navigation_mode|navigationMode|mouse_wheel|mouseWheel|hide_inactive_cursor|hideInactiveCursor|hide_cursor_time|hideCursorTime|controls_layout|controlsLayout|controls_tutorial|controlsTutorial|controls_back_arrows|controlsBackArrows|auto_slide|autoSlide|auto_slide_stoppable|autoSlideStoppable|auto_slide_method|autoSlideMethod|default_timing|defaultTiming|hash_type|hashType|hash_one_based_index|hashOneBasedIndex|respond_to_hash_changes|respondToHashChanges|fragment_in_url|fragmentInUrl|slide_tone|slideTone|pdf_separate_fragments|pdfSeparateFragments|pdf_page_height_offset|pdfPageHeightOffset|transition_speed|transitionSpeed|background_transition|backgroundTransition|auto_animate|autoAnimate|auto_animate_easing|autoAnimateEasing|auto_animate_duration|autoAnimateDuration|auto_animate_unmatched|autoAnimateUnmatched|auto_animate_styles|autoAnimateStyles|slide_level|slideLevel|slide_number|slideNumber|show_slide_number|showSlideNumber|title_slide_attributes|titleSlideAttributes|center_title_slide|centerTitleSlide|show_notes|showNotes|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_title|tocTitle))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Erevealjs(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var o0 = n === r;
                if (!o0) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Erevealjs(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var ce = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var ce = n === r;
              }
              if (t["code-fold"] !== void 0) {
                let a = t["code-fold"], n = r;
                Fa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-fold", parentData: t, parentDataProperty: "code-fold", rootData: s }) || (e = e === null ? Fa.errors : e.concat(Fa.errors), r = e.length);
                var ce = n === r;
              }
              if (t["code-summary"] !== void 0) {
                let a = t["code-summary"], n = r;
                Aa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-summary", parentData: t, parentDataProperty: "code-summary", rootData: s }) || (e = e === null ? Aa.errors : e.concat(Aa.errors), r = e.length);
                var ce = n === r;
              }
              if (t["code-overflow"] !== void 0) {
                let a = t["code-overflow"], n = r;
                Ba(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-overflow", parentData: t, parentDataProperty: "code-overflow", rootData: s }) || (e = e === null ? Ba.errors : e.concat(Ba.errors), r = e.length);
                var ce = n === r;
              }
              if (t["code-line-numbers"] !== void 0) {
                let a = t["code-line-numbers"], n = r;
                ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-line-numbers", parentData: t, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
                var ce = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var ce = n === r;
              }
              if (t["cap-location"] !== void 0) {
                let a = t["cap-location"], n = r;
                ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cap-location", parentData: t, parentDataProperty: "cap-location", rootData: s }) || (e = e === null ? ja.errors : e.concat(ja.errors), r = e.length);
                var ce = n === r;
              }
              if (t["fig-cap-location"] !== void 0) {
                let a = t["fig-cap-location"], n = r;
                Ea(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-cap-location", parentData: t, parentDataProperty: "fig-cap-location", rootData: s }) || (e = e === null ? Ea.errors : e.concat(Ea.errors), r = e.length);
                var ce = n === r;
              }
              if (t["tbl-cap-location"] !== void 0) {
                let a = t["tbl-cap-location"], n = r;
                Na(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-cap-location", parentData: t, parentDataProperty: "tbl-cap-location", rootData: s }) || (e = e === null ? Na.errors : e.concat(Na.errors), r = e.length);
                var ce = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var ce = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var ce = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var ce = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var ce = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var ce = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var ce = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var ce = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var ce = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var ce = n === r;
              }
              if (t.institute !== void 0) {
                let a = t.institute, n = r;
                ro(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/institute", parentData: t, parentDataProperty: "institute", rootData: s }) || (e = e === null ? ro.errors : e.concat(ro.errors), r = e.length);
                var ce = n === r;
              }
              if (t["code-copy"] !== void 0) {
                let a = t["code-copy"], n = r;
                ln(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-copy", parentData: t, parentDataProperty: "code-copy", rootData: s }) || (e = e === null ? ln.errors : e.concat(ln.errors), r = e.length);
                var ce = n === r;
              }
              if (t["code-link"] !== void 0) {
                let a = t["code-link"], n = r;
                On(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-link", parentData: t, parentDataProperty: "code-link", rootData: s }) || (e = e === null ? On.errors : e.concat(On.errors), r = e.length);
                var ce = n === r;
              }
              if (t["highlight-style"] !== void 0) {
                let a = t["highlight-style"], n = r;
                _a(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/highlight-style", parentData: t, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
                var ce = n === r;
              }
              if (t["syntax-definition"] !== void 0) {
                let a = t["syntax-definition"], n = r;
                va(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definition", parentData: t, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
                var ce = n === r;
              }
              if (t["syntax-definitions"] !== void 0) {
                let a = t["syntax-definitions"], n = r;
                qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definitions", parentData: t, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
                var ce = n === r;
              }
              if (t["indented-code-classes"] !== void 0) {
                let a = t["indented-code-classes"], n = r;
                Oa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indented-code-classes", parentData: t, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
                var ce = n === r;
              }
              if (t.fontcolor !== void 0) {
                let a = t.fontcolor, n = r;
                Pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontcolor", parentData: t, parentDataProperty: "fontcolor", rootData: s }) || (e = e === null ? Pn.errors : e.concat(Pn.errors), r = e.length);
                var ce = n === r;
              }
              if (t.linkcolor !== void 0) {
                let a = t.linkcolor, n = r;
                Ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linkcolor", parentData: t, parentDataProperty: "linkcolor", rootData: s }) || (e = e === null ? Ua.errors : e.concat(Ua.errors), r = e.length);
                var ce = n === r;
              }
              if (t.monobackgroundcolor !== void 0) {
                let a = t.monobackgroundcolor, n = r;
                xn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monobackgroundcolor", parentData: t, parentDataProperty: "monobackgroundcolor", rootData: s }) || (e = e === null ? xn.errors : e.concat(xn.errors), r = e.length);
                var ce = n === r;
              }
              if (t.backgroundcolor !== void 0) {
                let a = t.backgroundcolor, n = r;
                kn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/backgroundcolor", parentData: t, parentDataProperty: "backgroundcolor", rootData: s }) || (e = e === null ? kn.errors : e.concat(kn.errors), r = e.length);
                var ce = n === r;
              }
              if (t.comments !== void 0) {
                let a = t.comments, n = r;
                Dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/comments", parentData: t, parentDataProperty: "comments", rootData: s }) || (e = e === null ? Dn.errors : e.concat(Dn.errors), r = e.length);
                var ce = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var ce = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var ce = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var ce = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var ce = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var ce = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var ce = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var ce = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var ce = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var ce = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var ce = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var ce = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var ce = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var ce = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var ce = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var ce = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var ce = n === r;
              }
              if (t["fig-responsive"] !== void 0) {
                let a = t["fig-responsive"], n = r;
                cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-responsive", parentData: t, parentDataProperty: "fig-responsive", rootData: s }) || (e = e === null ? cn.errors : e.concat(cn.errors), r = e.length);
                var ce = n === r;
              }
              if (t.mainfont !== void 0) {
                let a = t.mainfont, n = r;
                Ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mainfont", parentData: t, parentDataProperty: "mainfont", rootData: s }) || (e = e === null ? Ja.errors : e.concat(Ja.errors), r = e.length);
                var ce = n === r;
              }
              if (t.monofont !== void 0) {
                let a = t.monofont, n = r;
                Qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monofont", parentData: t, parentDataProperty: "monofont", rootData: s }) || (e = e === null ? Qa.errors : e.concat(Qa.errors), r = e.length);
                var ce = n === r;
              }
              if (t.fontsize !== void 0) {
                let a = t.fontsize, n = r;
                Ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontsize", parentData: t, parentDataProperty: "fontsize", rootData: s }) || (e = e === null ? Ga.errors : e.concat(Ga.errors), r = e.length);
                var ce = n === r;
              }
              if (t.linestretch !== void 0) {
                let a = t.linestretch, n = r;
                Ya(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linestretch", parentData: t, parentDataProperty: "linestretch", rootData: s }) || (e = e === null ? Ya.errors : e.concat(Ya.errors), r = e.length);
                var ce = n === r;
              }
              if (t["footnotes-hover"] !== void 0) {
                let a = t["footnotes-hover"], n = r;
                wn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/footnotes-hover", parentData: t, parentDataProperty: "footnotes-hover", rootData: s }) || (e = e === null ? wn.errors : e.concat(wn.errors), r = e.length);
                var ce = n === r;
              }
              if (t["reference-location"] !== void 0) {
                let a = t["reference-location"], n = r;
                Ca(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-location", parentData: t, parentDataProperty: "reference-location", rootData: s }) || (e = e === null ? Ca.errors : e.concat(Ca.errors), r = e.length);
                var ce = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var ce = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var ce = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var ce = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var ce = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var ce = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var ce = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var ce = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var ce = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var ce = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var ce = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var ce = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var ce = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var ce = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var ce = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var ce = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var ce = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var ce = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var ce = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var ce = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var ce = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var ce = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var ce = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var ce = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var ce = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var ce = n === r;
              }
              if (t.resources !== void 0) {
                let a = t.resources, n = r;
                pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resources", parentData: t, parentDataProperty: "resources", rootData: s }) || (e = e === null ? pn.errors : e.concat(pn.errors), r = e.length);
                var ce = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var ce = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var ce = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var ce = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var ce = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var ce = n === r;
              }
              if (t.classoption !== void 0) {
                let a = t.classoption, n = r;
                un(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/classoption", parentData: t, parentDataProperty: "classoption", rootData: s }) || (e = e === null ? un.errors : e.concat(un.errors), r = e.length);
                var ce = n === r;
              }
              if (t["max-width"] !== void 0) {
                let a = t["max-width"], n = r;
                $n(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/max-width", parentData: t, parentDataProperty: "max-width", rootData: s }) || (e = e === null ? $n.errors : e.concat($n.errors), r = e.length);
                var ce = n === r;
              }
              if (t["margin-left"] !== void 0) {
                let a = t["margin-left"], n = r;
                Xa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-left", parentData: t, parentDataProperty: "margin-left", rootData: s }) || (e = e === null ? Xa.errors : e.concat(Xa.errors), r = e.length);
                var ce = n === r;
              }
              if (t["margin-right"] !== void 0) {
                let a = t["margin-right"], n = r;
                Za(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-right", parentData: t, parentDataProperty: "margin-right", rootData: s }) || (e = e === null ? Za.errors : e.concat(Za.errors), r = e.length);
                var ce = n === r;
              }
              if (t["margin-top"] !== void 0) {
                let a = t["margin-top"], n = r;
                en(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-top", parentData: t, parentDataProperty: "margin-top", rootData: s }) || (e = e === null ? en.errors : e.concat(en.errors), r = e.length);
                var ce = n === r;
              }
              if (t["margin-bottom"] !== void 0) {
                let a = t["margin-bottom"], n = r;
                rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-bottom", parentData: t, parentDataProperty: "margin-bottom", rootData: s }) || (e = e === null ? rn.errors : e.concat(rn.errors), r = e.length);
                var ce = n === r;
              }
              if (t["reveal-js-url"] !== void 0) {
                let a = t["reveal-js-url"], n = r;
                Nc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reveal-js-url", parentData: t, parentDataProperty: "reveal-js-url", rootData: s }) || (e = e === null ? Nc.errors : e.concat(Nc.errors), r = e.length);
                var ce = n === r;
              }
              if (t["link-external-icon"] !== void 0) {
                let a = t["link-external-icon"], n = r;
                No(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/link-external-icon", parentData: t, parentDataProperty: "link-external-icon", rootData: s }) || (e = e === null ? No.errors : e.concat(No.errors), r = e.length);
                var ce = n === r;
              }
              if (t["link-external-newwindow"] !== void 0) {
                let a = t["link-external-newwindow"], n = r;
                Co(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/link-external-newwindow", parentData: t, parentDataProperty: "link-external-newwindow", rootData: s }) || (e = e === null ? Co.errors : e.concat(Co.errors), r = e.length);
                var ce = n === r;
              }
              if (t["link-external-filter"] !== void 0) {
                let a = t["link-external-filter"], n = r;
                Io(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/link-external-filter", parentData: t, parentDataProperty: "link-external-filter", rootData: s }) || (e = e === null ? Io.errors : e.concat(Io.errors), r = e.length);
                var ce = n === r;
              }
              if (t.keywords !== void 0) {
                let a = t.keywords, n = r;
                Ia(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keywords", parentData: t, parentDataProperty: "keywords", rootData: s }) || (e = e === null ? Ia.errors : e.concat(Ia.errors), r = e.length);
                var ce = n === r;
              }
              if (t.pagetitle !== void 0) {
                let a = t.pagetitle, n = r;
                Sn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pagetitle", parentData: t, parentDataProperty: "pagetitle", rootData: s }) || (e = e === null ? Sn.errors : e.concat(Sn.errors), r = e.length);
                var ce = n === r;
              }
              if (t["title-prefix"] !== void 0) {
                let a = t["title-prefix"], n = r;
                Fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title-prefix", parentData: t, parentDataProperty: "title-prefix", rootData: s }) || (e = e === null ? Fn.errors : e.concat(Fn.errors), r = e.length);
                var ce = n === r;
              }
              if (t["description-meta"] !== void 0) {
                let a = t["description-meta"], n = r;
                An(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/description-meta", parentData: t, parentDataProperty: "description-meta", rootData: s }) || (e = e === null ? An.errors : e.concat(An.errors), r = e.length);
                var ce = n === r;
              }
              if (t["author-meta"] !== void 0) {
                let a = t["author-meta"], n = r;
                dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author-meta", parentData: t, parentDataProperty: "author-meta", rootData: s }) || (e = e === null ? dn.errors : e.concat(dn.errors), r = e.length);
                var ce = n === r;
              }
              if (t["date-meta"] !== void 0) {
                let a = t["date-meta"], n = r;
                Ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date-meta", parentData: t, parentDataProperty: "date-meta", rootData: s }) || (e = e === null ? Ha.errors : e.concat(Ha.errors), r = e.length);
                var ce = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var ce = n === r;
              }
              if (t["number-depth"] !== void 0) {
                let a = t["number-depth"], n = r;
                Da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-depth", parentData: t, parentDataProperty: "number-depth", rootData: s }) || (e = e === null ? Da.errors : e.concat(Da.errors), r = e.length);
                var ce = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var ce = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var ce = n === r;
              }
              if (t.theme !== void 0) {
                let a = t.theme, n = r;
                Ho(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/theme", parentData: t, parentDataProperty: "theme", rootData: s }) || (e = e === null ? Ho.errors : e.concat(Ho.errors), r = e.length);
                var ce = n === r;
              }
              if (t["document-css"] !== void 0) {
                let a = t["document-css"], n = r;
                jn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/document-css", parentData: t, parentDataProperty: "document-css", rootData: s }) || (e = e === null ? jn.errors : e.concat(jn.errors), r = e.length);
                var ce = n === r;
              }
              if (t.css !== void 0) {
                let a = t.css, n = r;
                fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/css", parentData: t, parentDataProperty: "css", rootData: s }) || (e = e === null ? fn.errors : e.concat(fn.errors), r = e.length);
                var ce = n === r;
              }
              if (t["section-divs"] !== void 0) {
                let a = t["section-divs"], n = r;
                En(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/section-divs", parentData: t, parentDataProperty: "section-divs", rootData: s }) || (e = e === null ? En.errors : e.concat(En.errors), r = e.length);
                var ce = n === r;
              }
              if (t["identifier-prefix"] !== void 0) {
                let a = t["identifier-prefix"], n = r;
                Pa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier-prefix", parentData: t, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
                var ce = n === r;
              }
              if (t["email-obfuscation"] !== void 0) {
                let a = t["email-obfuscation"], n = r;
                Nn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/email-obfuscation", parentData: t, parentDataProperty: "email-obfuscation", rootData: s }) || (e = e === null ? Nn.errors : e.concat(Nn.errors), r = e.length);
                var ce = n === r;
              }
              if (t["html-q-tags"] !== void 0) {
                let a = t["html-q-tags"], n = r;
                mn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-q-tags", parentData: t, parentDataProperty: "html-q-tags", rootData: s }) || (e = e === null ? mn.errors : e.concat(mn.errors), r = e.length);
                var ce = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var ce = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var ce = n === r;
              }
              if (t["citations-hover"] !== void 0) {
                let a = t["citations-hover"], n = r;
                Cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citations-hover", parentData: t, parentDataProperty: "citations-hover", rootData: s }) || (e = e === null ? Cn.errors : e.concat(Cn.errors), r = e.length);
                var ce = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var ce = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var ce = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var ce = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var ce = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var ce = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var ce = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var ce = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var ce = n === r;
              }
              if (t["self-contained"] !== void 0) {
                let a = t["self-contained"], n = r;
                In(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/self-contained", parentData: t, parentDataProperty: "self-contained", rootData: s }) || (e = e === null ? In.errors : e.concat(In.errors), r = e.length);
                var ce = n === r;
              }
              if (t["self-contained-math"] !== void 0) {
                let a = t["self-contained-math"], n = r;
                Hn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/self-contained-math", parentData: t, parentDataProperty: "self-contained-math", rootData: s }) || (e = e === null ? Hn.errors : e.concat(Hn.errors), r = e.length);
                var ce = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var ce = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var ce = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var ce = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var ce = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var ce = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var ce = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var ce = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var ce = n === r;
              }
              if (t.logo !== void 0) {
                let a = t.logo, n = r;
                tp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/logo", parentData: t, parentDataProperty: "logo", rootData: s }) || (e = e === null ? tp.errors : e.concat(tp.errors), r = e.length);
                var ce = n === r;
              }
              if (t.footer !== void 0) {
                let a = t.footer, n = r;
                ap(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/footer", parentData: t, parentDataProperty: "footer", rootData: s }) || (e = e === null ? ap.errors : e.concat(ap.errors), r = e.length);
                var ce = n === r;
              }
              if (t.scrollable !== void 0) {
                let a = t.scrollable, n = r;
                np(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/scrollable", parentData: t, parentDataProperty: "scrollable", rootData: s }) || (e = e === null ? np.errors : e.concat(np.errors), r = e.length);
                var ce = n === r;
              }
              if (t.smaller !== void 0) {
                let a = t.smaller, n = r;
                op(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/smaller", parentData: t, parentDataProperty: "smaller", rootData: s }) || (e = e === null ? op.errors : e.concat(op.errors), r = e.length);
                var ce = n === r;
              }
              if (t.embedded !== void 0) {
                let a = t.embedded, n = r;
                sp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/embedded", parentData: t, parentDataProperty: "embedded", rootData: s }) || (e = e === null ? sp.errors : e.concat(sp.errors), r = e.length);
                var ce = n === r;
              }
              if (t.display !== void 0) {
                let a = t.display, n = r;
                ip(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/display", parentData: t, parentDataProperty: "display", rootData: s }) || (e = e === null ? ip.errors : e.concat(ip.errors), r = e.length);
                var ce = n === r;
              }
              if (t["auto-stretch"] !== void 0) {
                let a = t["auto-stretch"], n = r;
                lp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/auto-stretch", parentData: t, parentDataProperty: "auto-stretch", rootData: s }) || (e = e === null ? lp.errors : e.concat(lp.errors), r = e.length);
                var ce = n === r;
              }
              if (t.width !== void 0) {
                let a = t.width, n = r;
                cp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/width", parentData: t, parentDataProperty: "width", rootData: s }) || (e = e === null ? cp.errors : e.concat(cp.errors), r = e.length);
                var ce = n === r;
              }
              if (t.height !== void 0) {
                let a = t.height, n = r;
                pp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/height", parentData: t, parentDataProperty: "height", rootData: s }) || (e = e === null ? pp.errors : e.concat(pp.errors), r = e.length);
                var ce = n === r;
              }
              if (t.margin !== void 0) {
                let a = t.margin, n = r;
                up(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin", parentData: t, parentDataProperty: "margin", rootData: s }) || (e = e === null ? up.errors : e.concat(up.errors), r = e.length);
                var ce = n === r;
              }
              if (t["min-scale"] !== void 0) {
                let a = t["min-scale"], n = r;
                dp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/min-scale", parentData: t, parentDataProperty: "min-scale", rootData: s }) || (e = e === null ? dp.errors : e.concat(dp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["max-scale"] !== void 0) {
                let a = t["max-scale"], n = r;
                fp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/max-scale", parentData: t, parentDataProperty: "max-scale", rootData: s }) || (e = e === null ? fp.errors : e.concat(fp.errors), r = e.length);
                var ce = n === r;
              }
              if (t.center !== void 0) {
                let a = t.center, n = r;
                mp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/center", parentData: t, parentDataProperty: "center", rootData: s }) || (e = e === null ? mp.errors : e.concat(mp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["disable-layout"] !== void 0) {
                let a = t["disable-layout"], n = r;
                gp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/disable-layout", parentData: t, parentDataProperty: "disable-layout", rootData: s }) || (e = e === null ? gp.errors : e.concat(gp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["preview-links"] !== void 0) {
                let a = t["preview-links"], n = r;
                hp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preview-links", parentData: t, parentDataProperty: "preview-links", rootData: s }) || (e = e === null ? hp.errors : e.concat(hp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["auto-play-media"] !== void 0) {
                let a = t["auto-play-media"], n = r;
                yp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/auto-play-media", parentData: t, parentDataProperty: "auto-play-media", rootData: s }) || (e = e === null ? yp.errors : e.concat(yp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["preload-iframes"] !== void 0) {
                let a = t["preload-iframes"], n = r;
                bp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preload-iframes", parentData: t, parentDataProperty: "preload-iframes", rootData: s }) || (e = e === null ? bp.errors : e.concat(bp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["view-distance"] !== void 0) {
                let a = t["view-distance"], n = r;
                _p(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/view-distance", parentData: t, parentDataProperty: "view-distance", rootData: s }) || (e = e === null ? _p.errors : e.concat(_p.errors), r = e.length);
                var ce = n === r;
              }
              if (t["mobile-view-distance"] !== void 0) {
                let a = t["mobile-view-distance"], n = r;
                vp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mobile-view-distance", parentData: t, parentDataProperty: "mobile-view-distance", rootData: s }) || (e = e === null ? vp.errors : e.concat(vp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["parallax-background-image"] !== void 0) {
                let a = t["parallax-background-image"], n = r;
                qp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/parallax-background-image", parentData: t, parentDataProperty: "parallax-background-image", rootData: s }) || (e = e === null ? qp.errors : e.concat(qp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["parallax-background-size"] !== void 0) {
                let a = t["parallax-background-size"], n = r;
                Op(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/parallax-background-size", parentData: t, parentDataProperty: "parallax-background-size", rootData: s }) || (e = e === null ? Op.errors : e.concat(Op.errors), r = e.length);
                var ce = n === r;
              }
              if (t["parallax-background-horizontal"] !== void 0) {
                let a = t["parallax-background-horizontal"], n = r;
                Pp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/parallax-background-horizontal", parentData: t, parentDataProperty: "parallax-background-horizontal", rootData: s }) || (e = e === null ? Pp.errors : e.concat(Pp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["parallax-background-vertical"] !== void 0) {
                let a = t["parallax-background-vertical"], n = r;
                xp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/parallax-background-vertical", parentData: t, parentDataProperty: "parallax-background-vertical", rootData: s }) || (e = e === null ? xp.errors : e.concat(xp.errors), r = e.length);
                var ce = n === r;
              }
              if (t.progress !== void 0) {
                let a = t.progress, n = r;
                kp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/progress", parentData: t, parentDataProperty: "progress", rootData: s }) || (e = e === null ? kp.errors : e.concat(kp.errors), r = e.length);
                var ce = n === r;
              }
              if (t.history !== void 0) {
                let a = t.history, n = r;
                Dp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/history", parentData: t, parentDataProperty: "history", rootData: s }) || (e = e === null ? Dp.errors : e.concat(Dp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["navigation-mode"] !== void 0) {
                let a = t["navigation-mode"], n = r;
                wp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/navigation-mode", parentData: t, parentDataProperty: "navigation-mode", rootData: s }) || (e = e === null ? wp.errors : e.concat(wp.errors), r = e.length);
                var ce = n === r;
              }
              if (t.touch !== void 0) {
                let a = t.touch, n = r;
                $p(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/touch", parentData: t, parentDataProperty: "touch", rootData: s }) || (e = e === null ? $p.errors : e.concat($p.errors), r = e.length);
                var ce = n === r;
              }
              if (t.keyboard !== void 0) {
                let a = t.keyboard, n = r;
                Sp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keyboard", parentData: t, parentDataProperty: "keyboard", rootData: s }) || (e = e === null ? Sp.errors : e.concat(Sp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["mouse-wheel"] !== void 0) {
                let a = t["mouse-wheel"], n = r;
                Fp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mouse-wheel", parentData: t, parentDataProperty: "mouse-wheel", rootData: s }) || (e = e === null ? Fp.errors : e.concat(Fp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["hide-inactive-cursor"] !== void 0) {
                let a = t["hide-inactive-cursor"], n = r;
                Ap(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/hide-inactive-cursor", parentData: t, parentDataProperty: "hide-inactive-cursor", rootData: s }) || (e = e === null ? Ap.errors : e.concat(Ap.errors), r = e.length);
                var ce = n === r;
              }
              if (t["hide-cursor-time"] !== void 0) {
                let a = t["hide-cursor-time"], n = r;
                Bp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/hide-cursor-time", parentData: t, parentDataProperty: "hide-cursor-time", rootData: s }) || (e = e === null ? Bp.errors : e.concat(Bp.errors), r = e.length);
                var ce = n === r;
              }
              if (t.loop !== void 0) {
                let a = t.loop, n = r;
                jp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/loop", parentData: t, parentDataProperty: "loop", rootData: s }) || (e = e === null ? jp.errors : e.concat(jp.errors), r = e.length);
                var ce = n === r;
              }
              if (t.shuffle !== void 0) {
                let a = t.shuffle, n = r;
                Ep(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shuffle", parentData: t, parentDataProperty: "shuffle", rootData: s }) || (e = e === null ? Ep.errors : e.concat(Ep.errors), r = e.length);
                var ce = n === r;
              }
              if (t.controls !== void 0) {
                let a = t.controls, n = r;
                Np(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/controls", parentData: t, parentDataProperty: "controls", rootData: s }) || (e = e === null ? Np.errors : e.concat(Np.errors), r = e.length);
                var ce = n === r;
              }
              if (t["controls-layout"] !== void 0) {
                let a = t["controls-layout"], n = r;
                Cp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/controls-layout", parentData: t, parentDataProperty: "controls-layout", rootData: s }) || (e = e === null ? Cp.errors : e.concat(Cp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["controls-tutorial"] !== void 0) {
                let a = t["controls-tutorial"], n = r;
                Ip(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/controls-tutorial", parentData: t, parentDataProperty: "controls-tutorial", rootData: s }) || (e = e === null ? Ip.errors : e.concat(Ip.errors), r = e.length);
                var ce = n === r;
              }
              if (t["controls-back-arrows"] !== void 0) {
                let a = t["controls-back-arrows"], n = r;
                Hp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/controls-back-arrows", parentData: t, parentDataProperty: "controls-back-arrows", rootData: s }) || (e = e === null ? Hp.errors : e.concat(Hp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["auto-slide"] !== void 0) {
                let a = t["auto-slide"], n = r;
                Lp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/auto-slide", parentData: t, parentDataProperty: "auto-slide", rootData: s }) || (e = e === null ? Lp.errors : e.concat(Lp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["auto-slide-stoppable"] !== void 0) {
                let a = t["auto-slide-stoppable"], n = r;
                Mp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/auto-slide-stoppable", parentData: t, parentDataProperty: "auto-slide-stoppable", rootData: s }) || (e = e === null ? Mp.errors : e.concat(Mp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["auto-slide-method"] !== void 0) {
                let a = t["auto-slide-method"], n = r;
                Tp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/auto-slide-method", parentData: t, parentDataProperty: "auto-slide-method", rootData: s }) || (e = e === null ? Tp.errors : e.concat(Tp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["default-timing"] !== void 0) {
                let a = t["default-timing"], n = r;
                zp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-timing", parentData: t, parentDataProperty: "default-timing", rootData: s }) || (e = e === null ? zp.errors : e.concat(zp.errors), r = e.length);
                var ce = n === r;
              }
              if (t.pause !== void 0) {
                let a = t.pause, n = r;
                Wp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pause", parentData: t, parentDataProperty: "pause", rootData: s }) || (e = e === null ? Wp.errors : e.concat(Wp.errors), r = e.length);
                var ce = n === r;
              }
              if (t.help !== void 0) {
                let a = t.help, n = r;
                Rp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/help", parentData: t, parentDataProperty: "help", rootData: s }) || (e = e === null ? Rp.errors : e.concat(Rp.errors), r = e.length);
                var ce = n === r;
              }
              if (t.hash !== void 0) {
                let a = t.hash, n = r;
                Vp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/hash", parentData: t, parentDataProperty: "hash", rootData: s }) || (e = e === null ? Vp.errors : e.concat(Vp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["hash-type"] !== void 0) {
                let a = t["hash-type"], n = r;
                Up(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/hash-type", parentData: t, parentDataProperty: "hash-type", rootData: s }) || (e = e === null ? Up.errors : e.concat(Up.errors), r = e.length);
                var ce = n === r;
              }
              if (t["hash-one-based-index"] !== void 0) {
                let a = t["hash-one-based-index"], n = r;
                Kp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/hash-one-based-index", parentData: t, parentDataProperty: "hash-one-based-index", rootData: s }) || (e = e === null ? Kp.errors : e.concat(Kp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["respond-to-hash-changes"] !== void 0) {
                let a = t["respond-to-hash-changes"], n = r;
                Jp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/respond-to-hash-changes", parentData: t, parentDataProperty: "respond-to-hash-changes", rootData: s }) || (e = e === null ? Jp.errors : e.concat(Jp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["fragment-in-url"] !== void 0) {
                let a = t["fragment-in-url"], n = r;
                Qp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fragment-in-url", parentData: t, parentDataProperty: "fragment-in-url", rootData: s }) || (e = e === null ? Qp.errors : e.concat(Qp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["slide-tone"] !== void 0) {
                let a = t["slide-tone"], n = r;
                Gp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/slide-tone", parentData: t, parentDataProperty: "slide-tone", rootData: s }) || (e = e === null ? Gp.errors : e.concat(Gp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["pdf-separate-fragments"] !== void 0) {
                let a = t["pdf-separate-fragments"], n = r;
                Yp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pdf-separate-fragments", parentData: t, parentDataProperty: "pdf-separate-fragments", rootData: s }) || (e = e === null ? Yp.errors : e.concat(Yp.errors), r = e.length);
                var ce = n === r;
              }
              if (t["pdf-page-height-offset"] !== void 0) {
                let a = t["pdf-page-height-offset"], n = r;
                Xp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pdf-page-height-offset", parentData: t, parentDataProperty: "pdf-page-height-offset", rootData: s }) || (e = e === null ? Xp.errors : e.concat(Xp.errors), r = e.length);
                var ce = n === r;
              }
              if (t.overview !== void 0) {
                let a = t.overview, n = r;
                Zp(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/overview", parentData: t, parentDataProperty: "overview", rootData: s }) || (e = e === null ? Zp.errors : e.concat(Zp.errors), r = e.length);
                var ce = n === r;
              }
              if (t.menu !== void 0) {
                let a = t.menu, n = r;
                eu(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/menu", parentData: t, parentDataProperty: "menu", rootData: s }) || (e = e === null ? eu.errors : e.concat(eu.errors), r = e.length);
                var ce = n === r;
              }
              if (t.chalkboard !== void 0) {
                let a = t.chalkboard, n = r;
                ru(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/chalkboard", parentData: t, parentDataProperty: "chalkboard", rootData: s }) || (e = e === null ? ru.errors : e.concat(ru.errors), r = e.length);
                var ce = n === r;
              }
              if (t.multiplex !== void 0) {
                let a = t.multiplex, n = r;
                tu(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/multiplex", parentData: t, parentDataProperty: "multiplex", rootData: s }) || (e = e === null ? tu.errors : e.concat(tu.errors), r = e.length);
                var ce = n === r;
              }
              if (t.transition !== void 0) {
                let a = t.transition, n = r;
                au(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/transition", parentData: t, parentDataProperty: "transition", rootData: s }) || (e = e === null ? au.errors : e.concat(au.errors), r = e.length);
                var ce = n === r;
              }
              if (t["transition-speed"] !== void 0) {
                let a = t["transition-speed"], n = r;
                nu(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/transition-speed", parentData: t, parentDataProperty: "transition-speed", rootData: s }) || (e = e === null ? nu.errors : e.concat(nu.errors), r = e.length);
                var ce = n === r;
              }
              if (t["background-transition"] !== void 0) {
                let a = t["background-transition"], n = r;
                ou(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/background-transition", parentData: t, parentDataProperty: "background-transition", rootData: s }) || (e = e === null ? ou.errors : e.concat(ou.errors), r = e.length);
                var ce = n === r;
              }
              if (t.fragments !== void 0) {
                let a = t.fragments, n = r;
                su(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fragments", parentData: t, parentDataProperty: "fragments", rootData: s }) || (e = e === null ? su.errors : e.concat(su.errors), r = e.length);
                var ce = n === r;
              }
              if (t["auto-animate"] !== void 0) {
                let a = t["auto-animate"], n = r;
                iu(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/auto-animate", parentData: t, parentDataProperty: "auto-animate", rootData: s }) || (e = e === null ? iu.errors : e.concat(iu.errors), r = e.length);
                var ce = n === r;
              }
              if (t["auto-animate-easing"] !== void 0) {
                let a = t["auto-animate-easing"], n = r;
                lu(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/auto-animate-easing", parentData: t, parentDataProperty: "auto-animate-easing", rootData: s }) || (e = e === null ? lu.errors : e.concat(lu.errors), r = e.length);
                var ce = n === r;
              }
              if (t["auto-animate-duration"] !== void 0) {
                let a = t["auto-animate-duration"], n = r;
                cu(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/auto-animate-duration", parentData: t, parentDataProperty: "auto-animate-duration", rootData: s }) || (e = e === null ? cu.errors : e.concat(cu.errors), r = e.length);
                var ce = n === r;
              }
              if (t["auto-animate-unmatched"] !== void 0) {
                let a = t["auto-animate-unmatched"], n = r;
                pu(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/auto-animate-unmatched", parentData: t, parentDataProperty: "auto-animate-unmatched", rootData: s }) || (e = e === null ? pu.errors : e.concat(pu.errors), r = e.length);
                var ce = n === r;
              }
              if (t["auto-animate-styles"] !== void 0) {
                let a = t["auto-animate-styles"], n = r;
                du(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/auto-animate-styles", parentData: t, parentDataProperty: "auto-animate-styles", rootData: s }) || (e = e === null ? du.errors : e.concat(du.errors), r = e.length);
                var ce = n === r;
              }
              if (t.incremental !== void 0) {
                let a = t.incremental, n = r;
                Rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/incremental", parentData: t, parentDataProperty: "incremental", rootData: s }) || (e = e === null ? Rn.errors : e.concat(Rn.errors), r = e.length);
                var ce = n === r;
              }
              if (t["slide-level"] !== void 0) {
                let a = t["slide-level"], n = r;
                Vn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/slide-level", parentData: t, parentDataProperty: "slide-level", rootData: s }) || (e = e === null ? Vn.errors : e.concat(Vn.errors), r = e.length);
                var ce = n === r;
              }
              if (t["slide-number"] !== void 0) {
                let a = t["slide-number"], n = r;
                fu(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/slide-number", parentData: t, parentDataProperty: "slide-number", rootData: s }) || (e = e === null ? fu.errors : e.concat(fu.errors), r = e.length);
                var ce = n === r;
              }
              if (t["show-slide-number"] !== void 0) {
                let a = t["show-slide-number"], n = r;
                mu(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/show-slide-number", parentData: t, parentDataProperty: "show-slide-number", rootData: s }) || (e = e === null ? mu.errors : e.concat(mu.errors), r = e.length);
                var ce = n === r;
              }
              if (t["title-slide-attributes"] !== void 0) {
                let a = t["title-slide-attributes"], n = r;
                gu(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title-slide-attributes", parentData: t, parentDataProperty: "title-slide-attributes", rootData: s }) || (e = e === null ? gu.errors : e.concat(gu.errors), r = e.length);
                var ce = n === r;
              }
              if (t["center-title-slide"] !== void 0) {
                let a = t["center-title-slide"], n = r;
                hu(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/center-title-slide", parentData: t, parentDataProperty: "center-title-slide", rootData: s }) || (e = e === null ? hu.errors : e.concat(hu.errors), r = e.length);
                var ce = n === r;
              }
              if (t["show-notes"] !== void 0) {
                let a = t["show-notes"], n = r;
                yu(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/show-notes", parentData: t, parentDataProperty: "show-notes", rootData: s }) || (e = e === null ? yu.errors : e.concat(yu.errors), r = e.length);
                var ce = n === r;
              }
              if (t.rtl !== void 0) {
                let a = t.rtl, n = r;
                bu(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/rtl", parentData: t, parentDataProperty: "rtl", rootData: s }) || (e = e === null ? bu.errors : e.concat(bu.errors), r = e.length);
                var ce = n === r;
              }
              if (t["strip-comments"] !== void 0) {
                let a = t["strip-comments"], n = r;
                Ra(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-comments", parentData: t, parentDataProperty: "strip-comments", rootData: s }) || (e = e === null ? Ra.errors : e.concat(Ra.errors), r = e.length);
                var ce = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var ce = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var ce = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var ce = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var ce = n === r;
              }
              if (t["toc-title"] !== void 0) {
                let a = t["toc-title"], n = r;
                La(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-title", parentData: t, parentDataProperty: "toc-title", rootData: s }) || (e = e === null ? La.errors : e.concat(La.errors), r = e.length);
                var ce = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Erevealjs(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Lm = ht === r;
            Lm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Erevealjs(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Lm = gt === r;
            if (Lm && L ? (L = false, M = [M, 1]) : Lm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Erevealjs(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^revealjs(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (p1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Erst(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Erst(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var s0 = n === r;
                if (!s0) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Erst(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Yt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Yt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Yt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Erst(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Mm = ht === r;
            Mm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Erst(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Mm = gt === r;
            if (Mm && L ? (L = false, M = [M, 1]) : Mm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Erst(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^rst(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (u1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Z0.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ertf(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(fig_align|figAlign|tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(fig_align|figAlign|tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(fig_align|figAlign|tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ertf(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var i0 = n === r;
                if (!i0) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ertf(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Ht = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Ht = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Ht = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ertf(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Tm = ht === r;
            Tm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ertf(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Tm = gt === r;
            if (Tm && L ? (L = false, M = [M, 1]) : Tm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ertf(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^rtf(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (d1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!eq.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Es5(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|s5_url|s5Url|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|s5_url|s5Url|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|s5_url|s5Url|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Es5(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var l0 = n === r;
                if (!l0) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Es5(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var rt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var rt = n === r;
              }
              if (t["code-fold"] !== void 0) {
                let a = t["code-fold"], n = r;
                Fa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-fold", parentData: t, parentDataProperty: "code-fold", rootData: s }) || (e = e === null ? Fa.errors : e.concat(Fa.errors), r = e.length);
                var rt = n === r;
              }
              if (t["code-summary"] !== void 0) {
                let a = t["code-summary"], n = r;
                Aa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-summary", parentData: t, parentDataProperty: "code-summary", rootData: s }) || (e = e === null ? Aa.errors : e.concat(Aa.errors), r = e.length);
                var rt = n === r;
              }
              if (t["code-overflow"] !== void 0) {
                let a = t["code-overflow"], n = r;
                Ba(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-overflow", parentData: t, parentDataProperty: "code-overflow", rootData: s }) || (e = e === null ? Ba.errors : e.concat(Ba.errors), r = e.length);
                var rt = n === r;
              }
              if (t["code-line-numbers"] !== void 0) {
                let a = t["code-line-numbers"], n = r;
                ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-line-numbers", parentData: t, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
                var rt = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var rt = n === r;
              }
              if (t["cap-location"] !== void 0) {
                let a = t["cap-location"], n = r;
                ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cap-location", parentData: t, parentDataProperty: "cap-location", rootData: s }) || (e = e === null ? ja.errors : e.concat(ja.errors), r = e.length);
                var rt = n === r;
              }
              if (t["fig-cap-location"] !== void 0) {
                let a = t["fig-cap-location"], n = r;
                Ea(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-cap-location", parentData: t, parentDataProperty: "fig-cap-location", rootData: s }) || (e = e === null ? Ea.errors : e.concat(Ea.errors), r = e.length);
                var rt = n === r;
              }
              if (t["tbl-cap-location"] !== void 0) {
                let a = t["tbl-cap-location"], n = r;
                Na(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-cap-location", parentData: t, parentDataProperty: "tbl-cap-location", rootData: s }) || (e = e === null ? Na.errors : e.concat(Na.errors), r = e.length);
                var rt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var rt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var rt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var rt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var rt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var rt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var rt = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var rt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var rt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var rt = n === r;
              }
              if (t.institute !== void 0) {
                let a = t.institute, n = r;
                ro(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/institute", parentData: t, parentDataProperty: "institute", rootData: s }) || (e = e === null ? ro.errors : e.concat(ro.errors), r = e.length);
                var rt = n === r;
              }
              if (t["code-copy"] !== void 0) {
                let a = t["code-copy"], n = r;
                ln(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-copy", parentData: t, parentDataProperty: "code-copy", rootData: s }) || (e = e === null ? ln.errors : e.concat(ln.errors), r = e.length);
                var rt = n === r;
              }
              if (t["code-link"] !== void 0) {
                let a = t["code-link"], n = r;
                On(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-link", parentData: t, parentDataProperty: "code-link", rootData: s }) || (e = e === null ? On.errors : e.concat(On.errors), r = e.length);
                var rt = n === r;
              }
              if (t["highlight-style"] !== void 0) {
                let a = t["highlight-style"], n = r;
                _a(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/highlight-style", parentData: t, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
                var rt = n === r;
              }
              if (t["syntax-definition"] !== void 0) {
                let a = t["syntax-definition"], n = r;
                va(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definition", parentData: t, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
                var rt = n === r;
              }
              if (t["syntax-definitions"] !== void 0) {
                let a = t["syntax-definitions"], n = r;
                qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definitions", parentData: t, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
                var rt = n === r;
              }
              if (t["indented-code-classes"] !== void 0) {
                let a = t["indented-code-classes"], n = r;
                Oa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indented-code-classes", parentData: t, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
                var rt = n === r;
              }
              if (t.fontcolor !== void 0) {
                let a = t.fontcolor, n = r;
                Pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontcolor", parentData: t, parentDataProperty: "fontcolor", rootData: s }) || (e = e === null ? Pn.errors : e.concat(Pn.errors), r = e.length);
                var rt = n === r;
              }
              if (t.linkcolor !== void 0) {
                let a = t.linkcolor, n = r;
                Ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linkcolor", parentData: t, parentDataProperty: "linkcolor", rootData: s }) || (e = e === null ? Ua.errors : e.concat(Ua.errors), r = e.length);
                var rt = n === r;
              }
              if (t.monobackgroundcolor !== void 0) {
                let a = t.monobackgroundcolor, n = r;
                xn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monobackgroundcolor", parentData: t, parentDataProperty: "monobackgroundcolor", rootData: s }) || (e = e === null ? xn.errors : e.concat(xn.errors), r = e.length);
                var rt = n === r;
              }
              if (t.backgroundcolor !== void 0) {
                let a = t.backgroundcolor, n = r;
                kn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/backgroundcolor", parentData: t, parentDataProperty: "backgroundcolor", rootData: s }) || (e = e === null ? kn.errors : e.concat(kn.errors), r = e.length);
                var rt = n === r;
              }
              if (t.comments !== void 0) {
                let a = t.comments, n = r;
                Dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/comments", parentData: t, parentDataProperty: "comments", rootData: s }) || (e = e === null ? Dn.errors : e.concat(Dn.errors), r = e.length);
                var rt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var rt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var rt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var rt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var rt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var rt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var rt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var rt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var rt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var rt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var rt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var rt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var rt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var rt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var rt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var rt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var rt = n === r;
              }
              if (t["fig-responsive"] !== void 0) {
                let a = t["fig-responsive"], n = r;
                cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-responsive", parentData: t, parentDataProperty: "fig-responsive", rootData: s }) || (e = e === null ? cn.errors : e.concat(cn.errors), r = e.length);
                var rt = n === r;
              }
              if (t.mainfont !== void 0) {
                let a = t.mainfont, n = r;
                Ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mainfont", parentData: t, parentDataProperty: "mainfont", rootData: s }) || (e = e === null ? Ja.errors : e.concat(Ja.errors), r = e.length);
                var rt = n === r;
              }
              if (t.monofont !== void 0) {
                let a = t.monofont, n = r;
                Qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monofont", parentData: t, parentDataProperty: "monofont", rootData: s }) || (e = e === null ? Qa.errors : e.concat(Qa.errors), r = e.length);
                var rt = n === r;
              }
              if (t.fontsize !== void 0) {
                let a = t.fontsize, n = r;
                Ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontsize", parentData: t, parentDataProperty: "fontsize", rootData: s }) || (e = e === null ? Ga.errors : e.concat(Ga.errors), r = e.length);
                var rt = n === r;
              }
              if (t.linestretch !== void 0) {
                let a = t.linestretch, n = r;
                Ya(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linestretch", parentData: t, parentDataProperty: "linestretch", rootData: s }) || (e = e === null ? Ya.errors : e.concat(Ya.errors), r = e.length);
                var rt = n === r;
              }
              if (t["footnotes-hover"] !== void 0) {
                let a = t["footnotes-hover"], n = r;
                wn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/footnotes-hover", parentData: t, parentDataProperty: "footnotes-hover", rootData: s }) || (e = e === null ? wn.errors : e.concat(wn.errors), r = e.length);
                var rt = n === r;
              }
              if (t["reference-location"] !== void 0) {
                let a = t["reference-location"], n = r;
                Ca(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-location", parentData: t, parentDataProperty: "reference-location", rootData: s }) || (e = e === null ? Ca.errors : e.concat(Ca.errors), r = e.length);
                var rt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var rt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var rt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var rt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var rt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var rt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var rt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var rt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var rt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var rt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var rt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var rt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var rt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var rt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var rt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var rt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var rt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var rt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var rt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var rt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var rt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var rt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var rt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var rt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var rt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var rt = n === r;
              }
              if (t.resources !== void 0) {
                let a = t.resources, n = r;
                pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resources", parentData: t, parentDataProperty: "resources", rootData: s }) || (e = e === null ? pn.errors : e.concat(pn.errors), r = e.length);
                var rt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var rt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var rt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var rt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var rt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var rt = n === r;
              }
              if (t.classoption !== void 0) {
                let a = t.classoption, n = r;
                un(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/classoption", parentData: t, parentDataProperty: "classoption", rootData: s }) || (e = e === null ? un.errors : e.concat(un.errors), r = e.length);
                var rt = n === r;
              }
              if (t["max-width"] !== void 0) {
                let a = t["max-width"], n = r;
                $n(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/max-width", parentData: t, parentDataProperty: "max-width", rootData: s }) || (e = e === null ? $n.errors : e.concat($n.errors), r = e.length);
                var rt = n === r;
              }
              if (t["margin-left"] !== void 0) {
                let a = t["margin-left"], n = r;
                Xa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-left", parentData: t, parentDataProperty: "margin-left", rootData: s }) || (e = e === null ? Xa.errors : e.concat(Xa.errors), r = e.length);
                var rt = n === r;
              }
              if (t["margin-right"] !== void 0) {
                let a = t["margin-right"], n = r;
                Za(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-right", parentData: t, parentDataProperty: "margin-right", rootData: s }) || (e = e === null ? Za.errors : e.concat(Za.errors), r = e.length);
                var rt = n === r;
              }
              if (t["margin-top"] !== void 0) {
                let a = t["margin-top"], n = r;
                en(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-top", parentData: t, parentDataProperty: "margin-top", rootData: s }) || (e = e === null ? en.errors : e.concat(en.errors), r = e.length);
                var rt = n === r;
              }
              if (t["margin-bottom"] !== void 0) {
                let a = t["margin-bottom"], n = r;
                rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-bottom", parentData: t, parentDataProperty: "margin-bottom", rootData: s }) || (e = e === null ? rn.errors : e.concat(rn.errors), r = e.length);
                var rt = n === r;
              }
              if (t["s5-url"] !== void 0) {
                let a = t["s5-url"], n = r;
                Cc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/s5-url", parentData: t, parentDataProperty: "s5-url", rootData: s }) || (e = e === null ? Cc.errors : e.concat(Cc.errors), r = e.length);
                var rt = n === r;
              }
              if (t.keywords !== void 0) {
                let a = t.keywords, n = r;
                Ia(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keywords", parentData: t, parentDataProperty: "keywords", rootData: s }) || (e = e === null ? Ia.errors : e.concat(Ia.errors), r = e.length);
                var rt = n === r;
              }
              if (t.pagetitle !== void 0) {
                let a = t.pagetitle, n = r;
                Sn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pagetitle", parentData: t, parentDataProperty: "pagetitle", rootData: s }) || (e = e === null ? Sn.errors : e.concat(Sn.errors), r = e.length);
                var rt = n === r;
              }
              if (t["title-prefix"] !== void 0) {
                let a = t["title-prefix"], n = r;
                Fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title-prefix", parentData: t, parentDataProperty: "title-prefix", rootData: s }) || (e = e === null ? Fn.errors : e.concat(Fn.errors), r = e.length);
                var rt = n === r;
              }
              if (t["description-meta"] !== void 0) {
                let a = t["description-meta"], n = r;
                An(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/description-meta", parentData: t, parentDataProperty: "description-meta", rootData: s }) || (e = e === null ? An.errors : e.concat(An.errors), r = e.length);
                var rt = n === r;
              }
              if (t["author-meta"] !== void 0) {
                let a = t["author-meta"], n = r;
                dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author-meta", parentData: t, parentDataProperty: "author-meta", rootData: s }) || (e = e === null ? dn.errors : e.concat(dn.errors), r = e.length);
                var rt = n === r;
              }
              if (t["date-meta"] !== void 0) {
                let a = t["date-meta"], n = r;
                Ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date-meta", parentData: t, parentDataProperty: "date-meta", rootData: s }) || (e = e === null ? Ha.errors : e.concat(Ha.errors), r = e.length);
                var rt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var rt = n === r;
              }
              if (t["number-depth"] !== void 0) {
                let a = t["number-depth"], n = r;
                Da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-depth", parentData: t, parentDataProperty: "number-depth", rootData: s }) || (e = e === null ? Da.errors : e.concat(Da.errors), r = e.length);
                var rt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var rt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var rt = n === r;
              }
              if (t["document-css"] !== void 0) {
                let a = t["document-css"], n = r;
                jn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/document-css", parentData: t, parentDataProperty: "document-css", rootData: s }) || (e = e === null ? jn.errors : e.concat(jn.errors), r = e.length);
                var rt = n === r;
              }
              if (t.css !== void 0) {
                let a = t.css, n = r;
                fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/css", parentData: t, parentDataProperty: "css", rootData: s }) || (e = e === null ? fn.errors : e.concat(fn.errors), r = e.length);
                var rt = n === r;
              }
              if (t["section-divs"] !== void 0) {
                let a = t["section-divs"], n = r;
                En(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/section-divs", parentData: t, parentDataProperty: "section-divs", rootData: s }) || (e = e === null ? En.errors : e.concat(En.errors), r = e.length);
                var rt = n === r;
              }
              if (t["identifier-prefix"] !== void 0) {
                let a = t["identifier-prefix"], n = r;
                Pa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier-prefix", parentData: t, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
                var rt = n === r;
              }
              if (t["email-obfuscation"] !== void 0) {
                let a = t["email-obfuscation"], n = r;
                Nn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/email-obfuscation", parentData: t, parentDataProperty: "email-obfuscation", rootData: s }) || (e = e === null ? Nn.errors : e.concat(Nn.errors), r = e.length);
                var rt = n === r;
              }
              if (t["html-q-tags"] !== void 0) {
                let a = t["html-q-tags"], n = r;
                mn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-q-tags", parentData: t, parentDataProperty: "html-q-tags", rootData: s }) || (e = e === null ? mn.errors : e.concat(mn.errors), r = e.length);
                var rt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var rt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var rt = n === r;
              }
              if (t["citations-hover"] !== void 0) {
                let a = t["citations-hover"], n = r;
                Cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citations-hover", parentData: t, parentDataProperty: "citations-hover", rootData: s }) || (e = e === null ? Cn.errors : e.concat(Cn.errors), r = e.length);
                var rt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var rt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var rt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var rt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var rt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var rt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var rt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var rt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var rt = n === r;
              }
              if (t["self-contained"] !== void 0) {
                let a = t["self-contained"], n = r;
                In(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/self-contained", parentData: t, parentDataProperty: "self-contained", rootData: s }) || (e = e === null ? In.errors : e.concat(In.errors), r = e.length);
                var rt = n === r;
              }
              if (t["self-contained-math"] !== void 0) {
                let a = t["self-contained-math"], n = r;
                Hn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/self-contained-math", parentData: t, parentDataProperty: "self-contained-math", rootData: s }) || (e = e === null ? Hn.errors : e.concat(Hn.errors), r = e.length);
                var rt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var rt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var rt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var rt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var rt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var rt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var rt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var rt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var rt = n === r;
              }
              if (t.incremental !== void 0) {
                let a = t.incremental, n = r;
                Rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/incremental", parentData: t, parentDataProperty: "incremental", rootData: s }) || (e = e === null ? Rn.errors : e.concat(Rn.errors), r = e.length);
                var rt = n === r;
              }
              if (t["slide-level"] !== void 0) {
                let a = t["slide-level"], n = r;
                Vn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/slide-level", parentData: t, parentDataProperty: "slide-level", rootData: s }) || (e = e === null ? Vn.errors : e.concat(Vn.errors), r = e.length);
                var rt = n === r;
              }
              if (t["strip-comments"] !== void 0) {
                let a = t["strip-comments"], n = r;
                Ra(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-comments", parentData: t, parentDataProperty: "strip-comments", rootData: s }) || (e = e === null ? Ra.errors : e.concat(Ra.errors), r = e.length);
                var rt = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var rt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var rt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var rt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var rt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Es5(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var zm = ht === r;
            zm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Es5(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var zm = gt === r;
            if (zm && L ? (L = false, M = [M, 1]) : zm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Es5(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^s5(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (f1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!rq.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eslideous(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|slideous_url|slideousUrl|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|slideous_url|slideousUrl|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|slideous_url|slideousUrl|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eslideous(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var c0 = n === r;
                if (!c0) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eslideous(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var tt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var tt = n === r;
              }
              if (t["code-fold"] !== void 0) {
                let a = t["code-fold"], n = r;
                Fa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-fold", parentData: t, parentDataProperty: "code-fold", rootData: s }) || (e = e === null ? Fa.errors : e.concat(Fa.errors), r = e.length);
                var tt = n === r;
              }
              if (t["code-summary"] !== void 0) {
                let a = t["code-summary"], n = r;
                Aa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-summary", parentData: t, parentDataProperty: "code-summary", rootData: s }) || (e = e === null ? Aa.errors : e.concat(Aa.errors), r = e.length);
                var tt = n === r;
              }
              if (t["code-overflow"] !== void 0) {
                let a = t["code-overflow"], n = r;
                Ba(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-overflow", parentData: t, parentDataProperty: "code-overflow", rootData: s }) || (e = e === null ? Ba.errors : e.concat(Ba.errors), r = e.length);
                var tt = n === r;
              }
              if (t["code-line-numbers"] !== void 0) {
                let a = t["code-line-numbers"], n = r;
                ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-line-numbers", parentData: t, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
                var tt = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var tt = n === r;
              }
              if (t["cap-location"] !== void 0) {
                let a = t["cap-location"], n = r;
                ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cap-location", parentData: t, parentDataProperty: "cap-location", rootData: s }) || (e = e === null ? ja.errors : e.concat(ja.errors), r = e.length);
                var tt = n === r;
              }
              if (t["fig-cap-location"] !== void 0) {
                let a = t["fig-cap-location"], n = r;
                Ea(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-cap-location", parentData: t, parentDataProperty: "fig-cap-location", rootData: s }) || (e = e === null ? Ea.errors : e.concat(Ea.errors), r = e.length);
                var tt = n === r;
              }
              if (t["tbl-cap-location"] !== void 0) {
                let a = t["tbl-cap-location"], n = r;
                Na(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-cap-location", parentData: t, parentDataProperty: "tbl-cap-location", rootData: s }) || (e = e === null ? Na.errors : e.concat(Na.errors), r = e.length);
                var tt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var tt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var tt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var tt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var tt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var tt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var tt = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var tt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var tt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var tt = n === r;
              }
              if (t.institute !== void 0) {
                let a = t.institute, n = r;
                ro(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/institute", parentData: t, parentDataProperty: "institute", rootData: s }) || (e = e === null ? ro.errors : e.concat(ro.errors), r = e.length);
                var tt = n === r;
              }
              if (t["code-copy"] !== void 0) {
                let a = t["code-copy"], n = r;
                ln(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-copy", parentData: t, parentDataProperty: "code-copy", rootData: s }) || (e = e === null ? ln.errors : e.concat(ln.errors), r = e.length);
                var tt = n === r;
              }
              if (t["code-link"] !== void 0) {
                let a = t["code-link"], n = r;
                On(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-link", parentData: t, parentDataProperty: "code-link", rootData: s }) || (e = e === null ? On.errors : e.concat(On.errors), r = e.length);
                var tt = n === r;
              }
              if (t["highlight-style"] !== void 0) {
                let a = t["highlight-style"], n = r;
                _a(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/highlight-style", parentData: t, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
                var tt = n === r;
              }
              if (t["syntax-definition"] !== void 0) {
                let a = t["syntax-definition"], n = r;
                va(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definition", parentData: t, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
                var tt = n === r;
              }
              if (t["syntax-definitions"] !== void 0) {
                let a = t["syntax-definitions"], n = r;
                qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definitions", parentData: t, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
                var tt = n === r;
              }
              if (t["indented-code-classes"] !== void 0) {
                let a = t["indented-code-classes"], n = r;
                Oa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indented-code-classes", parentData: t, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
                var tt = n === r;
              }
              if (t.fontcolor !== void 0) {
                let a = t.fontcolor, n = r;
                Pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontcolor", parentData: t, parentDataProperty: "fontcolor", rootData: s }) || (e = e === null ? Pn.errors : e.concat(Pn.errors), r = e.length);
                var tt = n === r;
              }
              if (t.linkcolor !== void 0) {
                let a = t.linkcolor, n = r;
                Ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linkcolor", parentData: t, parentDataProperty: "linkcolor", rootData: s }) || (e = e === null ? Ua.errors : e.concat(Ua.errors), r = e.length);
                var tt = n === r;
              }
              if (t.monobackgroundcolor !== void 0) {
                let a = t.monobackgroundcolor, n = r;
                xn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monobackgroundcolor", parentData: t, parentDataProperty: "monobackgroundcolor", rootData: s }) || (e = e === null ? xn.errors : e.concat(xn.errors), r = e.length);
                var tt = n === r;
              }
              if (t.backgroundcolor !== void 0) {
                let a = t.backgroundcolor, n = r;
                kn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/backgroundcolor", parentData: t, parentDataProperty: "backgroundcolor", rootData: s }) || (e = e === null ? kn.errors : e.concat(kn.errors), r = e.length);
                var tt = n === r;
              }
              if (t.comments !== void 0) {
                let a = t.comments, n = r;
                Dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/comments", parentData: t, parentDataProperty: "comments", rootData: s }) || (e = e === null ? Dn.errors : e.concat(Dn.errors), r = e.length);
                var tt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var tt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var tt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var tt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var tt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var tt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var tt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var tt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var tt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var tt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var tt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var tt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var tt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var tt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var tt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var tt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var tt = n === r;
              }
              if (t["fig-responsive"] !== void 0) {
                let a = t["fig-responsive"], n = r;
                cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-responsive", parentData: t, parentDataProperty: "fig-responsive", rootData: s }) || (e = e === null ? cn.errors : e.concat(cn.errors), r = e.length);
                var tt = n === r;
              }
              if (t.mainfont !== void 0) {
                let a = t.mainfont, n = r;
                Ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mainfont", parentData: t, parentDataProperty: "mainfont", rootData: s }) || (e = e === null ? Ja.errors : e.concat(Ja.errors), r = e.length);
                var tt = n === r;
              }
              if (t.monofont !== void 0) {
                let a = t.monofont, n = r;
                Qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monofont", parentData: t, parentDataProperty: "monofont", rootData: s }) || (e = e === null ? Qa.errors : e.concat(Qa.errors), r = e.length);
                var tt = n === r;
              }
              if (t.fontsize !== void 0) {
                let a = t.fontsize, n = r;
                Ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontsize", parentData: t, parentDataProperty: "fontsize", rootData: s }) || (e = e === null ? Ga.errors : e.concat(Ga.errors), r = e.length);
                var tt = n === r;
              }
              if (t.linestretch !== void 0) {
                let a = t.linestretch, n = r;
                Ya(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linestretch", parentData: t, parentDataProperty: "linestretch", rootData: s }) || (e = e === null ? Ya.errors : e.concat(Ya.errors), r = e.length);
                var tt = n === r;
              }
              if (t["footnotes-hover"] !== void 0) {
                let a = t["footnotes-hover"], n = r;
                wn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/footnotes-hover", parentData: t, parentDataProperty: "footnotes-hover", rootData: s }) || (e = e === null ? wn.errors : e.concat(wn.errors), r = e.length);
                var tt = n === r;
              }
              if (t["reference-location"] !== void 0) {
                let a = t["reference-location"], n = r;
                Ca(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-location", parentData: t, parentDataProperty: "reference-location", rootData: s }) || (e = e === null ? Ca.errors : e.concat(Ca.errors), r = e.length);
                var tt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var tt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var tt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var tt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var tt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var tt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var tt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var tt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var tt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var tt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var tt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var tt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var tt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var tt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var tt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var tt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var tt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var tt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var tt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var tt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var tt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var tt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var tt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var tt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var tt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var tt = n === r;
              }
              if (t.resources !== void 0) {
                let a = t.resources, n = r;
                pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resources", parentData: t, parentDataProperty: "resources", rootData: s }) || (e = e === null ? pn.errors : e.concat(pn.errors), r = e.length);
                var tt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var tt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var tt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var tt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var tt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var tt = n === r;
              }
              if (t.classoption !== void 0) {
                let a = t.classoption, n = r;
                un(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/classoption", parentData: t, parentDataProperty: "classoption", rootData: s }) || (e = e === null ? un.errors : e.concat(un.errors), r = e.length);
                var tt = n === r;
              }
              if (t["max-width"] !== void 0) {
                let a = t["max-width"], n = r;
                $n(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/max-width", parentData: t, parentDataProperty: "max-width", rootData: s }) || (e = e === null ? $n.errors : e.concat($n.errors), r = e.length);
                var tt = n === r;
              }
              if (t["margin-left"] !== void 0) {
                let a = t["margin-left"], n = r;
                Xa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-left", parentData: t, parentDataProperty: "margin-left", rootData: s }) || (e = e === null ? Xa.errors : e.concat(Xa.errors), r = e.length);
                var tt = n === r;
              }
              if (t["margin-right"] !== void 0) {
                let a = t["margin-right"], n = r;
                Za(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-right", parentData: t, parentDataProperty: "margin-right", rootData: s }) || (e = e === null ? Za.errors : e.concat(Za.errors), r = e.length);
                var tt = n === r;
              }
              if (t["margin-top"] !== void 0) {
                let a = t["margin-top"], n = r;
                en(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-top", parentData: t, parentDataProperty: "margin-top", rootData: s }) || (e = e === null ? en.errors : e.concat(en.errors), r = e.length);
                var tt = n === r;
              }
              if (t["margin-bottom"] !== void 0) {
                let a = t["margin-bottom"], n = r;
                rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-bottom", parentData: t, parentDataProperty: "margin-bottom", rootData: s }) || (e = e === null ? rn.errors : e.concat(rn.errors), r = e.length);
                var tt = n === r;
              }
              if (t["slideous-url"] !== void 0) {
                let a = t["slideous-url"], n = r;
                Hc(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/slideous-url", parentData: t, parentDataProperty: "slideous-url", rootData: s }) || (e = e === null ? Hc.errors : e.concat(Hc.errors), r = e.length);
                var tt = n === r;
              }
              if (t.keywords !== void 0) {
                let a = t.keywords, n = r;
                Ia(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keywords", parentData: t, parentDataProperty: "keywords", rootData: s }) || (e = e === null ? Ia.errors : e.concat(Ia.errors), r = e.length);
                var tt = n === r;
              }
              if (t.pagetitle !== void 0) {
                let a = t.pagetitle, n = r;
                Sn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pagetitle", parentData: t, parentDataProperty: "pagetitle", rootData: s }) || (e = e === null ? Sn.errors : e.concat(Sn.errors), r = e.length);
                var tt = n === r;
              }
              if (t["title-prefix"] !== void 0) {
                let a = t["title-prefix"], n = r;
                Fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title-prefix", parentData: t, parentDataProperty: "title-prefix", rootData: s }) || (e = e === null ? Fn.errors : e.concat(Fn.errors), r = e.length);
                var tt = n === r;
              }
              if (t["description-meta"] !== void 0) {
                let a = t["description-meta"], n = r;
                An(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/description-meta", parentData: t, parentDataProperty: "description-meta", rootData: s }) || (e = e === null ? An.errors : e.concat(An.errors), r = e.length);
                var tt = n === r;
              }
              if (t["author-meta"] !== void 0) {
                let a = t["author-meta"], n = r;
                dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author-meta", parentData: t, parentDataProperty: "author-meta", rootData: s }) || (e = e === null ? dn.errors : e.concat(dn.errors), r = e.length);
                var tt = n === r;
              }
              if (t["date-meta"] !== void 0) {
                let a = t["date-meta"], n = r;
                Ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date-meta", parentData: t, parentDataProperty: "date-meta", rootData: s }) || (e = e === null ? Ha.errors : e.concat(Ha.errors), r = e.length);
                var tt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var tt = n === r;
              }
              if (t["number-depth"] !== void 0) {
                let a = t["number-depth"], n = r;
                Da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-depth", parentData: t, parentDataProperty: "number-depth", rootData: s }) || (e = e === null ? Da.errors : e.concat(Da.errors), r = e.length);
                var tt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var tt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var tt = n === r;
              }
              if (t["document-css"] !== void 0) {
                let a = t["document-css"], n = r;
                jn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/document-css", parentData: t, parentDataProperty: "document-css", rootData: s }) || (e = e === null ? jn.errors : e.concat(jn.errors), r = e.length);
                var tt = n === r;
              }
              if (t.css !== void 0) {
                let a = t.css, n = r;
                fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/css", parentData: t, parentDataProperty: "css", rootData: s }) || (e = e === null ? fn.errors : e.concat(fn.errors), r = e.length);
                var tt = n === r;
              }
              if (t["section-divs"] !== void 0) {
                let a = t["section-divs"], n = r;
                En(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/section-divs", parentData: t, parentDataProperty: "section-divs", rootData: s }) || (e = e === null ? En.errors : e.concat(En.errors), r = e.length);
                var tt = n === r;
              }
              if (t["identifier-prefix"] !== void 0) {
                let a = t["identifier-prefix"], n = r;
                Pa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier-prefix", parentData: t, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
                var tt = n === r;
              }
              if (t["email-obfuscation"] !== void 0) {
                let a = t["email-obfuscation"], n = r;
                Nn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/email-obfuscation", parentData: t, parentDataProperty: "email-obfuscation", rootData: s }) || (e = e === null ? Nn.errors : e.concat(Nn.errors), r = e.length);
                var tt = n === r;
              }
              if (t["html-q-tags"] !== void 0) {
                let a = t["html-q-tags"], n = r;
                mn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-q-tags", parentData: t, parentDataProperty: "html-q-tags", rootData: s }) || (e = e === null ? mn.errors : e.concat(mn.errors), r = e.length);
                var tt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var tt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var tt = n === r;
              }
              if (t["citations-hover"] !== void 0) {
                let a = t["citations-hover"], n = r;
                Cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citations-hover", parentData: t, parentDataProperty: "citations-hover", rootData: s }) || (e = e === null ? Cn.errors : e.concat(Cn.errors), r = e.length);
                var tt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var tt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var tt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var tt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var tt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var tt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var tt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var tt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var tt = n === r;
              }
              if (t["self-contained"] !== void 0) {
                let a = t["self-contained"], n = r;
                In(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/self-contained", parentData: t, parentDataProperty: "self-contained", rootData: s }) || (e = e === null ? In.errors : e.concat(In.errors), r = e.length);
                var tt = n === r;
              }
              if (t["self-contained-math"] !== void 0) {
                let a = t["self-contained-math"], n = r;
                Hn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/self-contained-math", parentData: t, parentDataProperty: "self-contained-math", rootData: s }) || (e = e === null ? Hn.errors : e.concat(Hn.errors), r = e.length);
                var tt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var tt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var tt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var tt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var tt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var tt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var tt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var tt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var tt = n === r;
              }
              if (t.incremental !== void 0) {
                let a = t.incremental, n = r;
                Rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/incremental", parentData: t, parentDataProperty: "incremental", rootData: s }) || (e = e === null ? Rn.errors : e.concat(Rn.errors), r = e.length);
                var tt = n === r;
              }
              if (t["slide-level"] !== void 0) {
                let a = t["slide-level"], n = r;
                Vn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/slide-level", parentData: t, parentDataProperty: "slide-level", rootData: s }) || (e = e === null ? Vn.errors : e.concat(Vn.errors), r = e.length);
                var tt = n === r;
              }
              if (t["strip-comments"] !== void 0) {
                let a = t["strip-comments"], n = r;
                Ra(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-comments", parentData: t, parentDataProperty: "strip-comments", rootData: s }) || (e = e === null ? Ra.errors : e.concat(Ra.errors), r = e.length);
                var tt = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var tt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var tt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var tt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var tt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eslideous(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Wm = ht === r;
            Wm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eslideous(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Wm = gt === r;
            if (Wm && L ? (L = false, M = [M, 1]) : Wm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eslideous(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^slideous(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (m1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!tq.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eslidy(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|slidy_url|slidyUrl|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|slidy_url|slidyUrl|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|code_copy|codeCopy|code_link|codeLink|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|footnotes_hover|footnotesHover|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|slidy_url|slidyUrl|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|document_css|documentCss|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|citations_hover|citationsHover|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|slide_level|slideLevel|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eslidy(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var p0 = n === r;
                if (!p0) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eslidy(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var at = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var at = n === r;
              }
              if (t["code-fold"] !== void 0) {
                let a = t["code-fold"], n = r;
                Fa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-fold", parentData: t, parentDataProperty: "code-fold", rootData: s }) || (e = e === null ? Fa.errors : e.concat(Fa.errors), r = e.length);
                var at = n === r;
              }
              if (t["code-summary"] !== void 0) {
                let a = t["code-summary"], n = r;
                Aa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-summary", parentData: t, parentDataProperty: "code-summary", rootData: s }) || (e = e === null ? Aa.errors : e.concat(Aa.errors), r = e.length);
                var at = n === r;
              }
              if (t["code-overflow"] !== void 0) {
                let a = t["code-overflow"], n = r;
                Ba(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-overflow", parentData: t, parentDataProperty: "code-overflow", rootData: s }) || (e = e === null ? Ba.errors : e.concat(Ba.errors), r = e.length);
                var at = n === r;
              }
              if (t["code-line-numbers"] !== void 0) {
                let a = t["code-line-numbers"], n = r;
                ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-line-numbers", parentData: t, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
                var at = n === r;
              }
              if (t["fig-align"] !== void 0) {
                let a = t["fig-align"], n = r;
                la(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-align", parentData: t, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
                var at = n === r;
              }
              if (t["cap-location"] !== void 0) {
                let a = t["cap-location"], n = r;
                ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cap-location", parentData: t, parentDataProperty: "cap-location", rootData: s }) || (e = e === null ? ja.errors : e.concat(ja.errors), r = e.length);
                var at = n === r;
              }
              if (t["fig-cap-location"] !== void 0) {
                let a = t["fig-cap-location"], n = r;
                Ea(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-cap-location", parentData: t, parentDataProperty: "fig-cap-location", rootData: s }) || (e = e === null ? Ea.errors : e.concat(Ea.errors), r = e.length);
                var at = n === r;
              }
              if (t["tbl-cap-location"] !== void 0) {
                let a = t["tbl-cap-location"], n = r;
                Na(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-cap-location", parentData: t, parentDataProperty: "tbl-cap-location", rootData: s }) || (e = e === null ? Na.errors : e.concat(Na.errors), r = e.length);
                var at = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var at = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var at = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var at = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var at = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var at = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var at = n === r;
              }
              if (t.subtitle !== void 0) {
                let a = t.subtitle, n = r;
                ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/subtitle", parentData: t, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
                var at = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var at = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var at = n === r;
              }
              if (t.institute !== void 0) {
                let a = t.institute, n = r;
                ro(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/institute", parentData: t, parentDataProperty: "institute", rootData: s }) || (e = e === null ? ro.errors : e.concat(ro.errors), r = e.length);
                var at = n === r;
              }
              if (t["code-copy"] !== void 0) {
                let a = t["code-copy"], n = r;
                ln(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-copy", parentData: t, parentDataProperty: "code-copy", rootData: s }) || (e = e === null ? ln.errors : e.concat(ln.errors), r = e.length);
                var at = n === r;
              }
              if (t["code-link"] !== void 0) {
                let a = t["code-link"], n = r;
                On(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/code-link", parentData: t, parentDataProperty: "code-link", rootData: s }) || (e = e === null ? On.errors : e.concat(On.errors), r = e.length);
                var at = n === r;
              }
              if (t["highlight-style"] !== void 0) {
                let a = t["highlight-style"], n = r;
                _a(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/highlight-style", parentData: t, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
                var at = n === r;
              }
              if (t["syntax-definition"] !== void 0) {
                let a = t["syntax-definition"], n = r;
                va(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definition", parentData: t, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
                var at = n === r;
              }
              if (t["syntax-definitions"] !== void 0) {
                let a = t["syntax-definitions"], n = r;
                qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/syntax-definitions", parentData: t, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
                var at = n === r;
              }
              if (t["indented-code-classes"] !== void 0) {
                let a = t["indented-code-classes"], n = r;
                Oa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/indented-code-classes", parentData: t, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
                var at = n === r;
              }
              if (t.fontcolor !== void 0) {
                let a = t.fontcolor, n = r;
                Pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontcolor", parentData: t, parentDataProperty: "fontcolor", rootData: s }) || (e = e === null ? Pn.errors : e.concat(Pn.errors), r = e.length);
                var at = n === r;
              }
              if (t.linkcolor !== void 0) {
                let a = t.linkcolor, n = r;
                Ua(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linkcolor", parentData: t, parentDataProperty: "linkcolor", rootData: s }) || (e = e === null ? Ua.errors : e.concat(Ua.errors), r = e.length);
                var at = n === r;
              }
              if (t.monobackgroundcolor !== void 0) {
                let a = t.monobackgroundcolor, n = r;
                xn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monobackgroundcolor", parentData: t, parentDataProperty: "monobackgroundcolor", rootData: s }) || (e = e === null ? xn.errors : e.concat(xn.errors), r = e.length);
                var at = n === r;
              }
              if (t.backgroundcolor !== void 0) {
                let a = t.backgroundcolor, n = r;
                kn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/backgroundcolor", parentData: t, parentDataProperty: "backgroundcolor", rootData: s }) || (e = e === null ? kn.errors : e.concat(kn.errors), r = e.length);
                var at = n === r;
              }
              if (t.comments !== void 0) {
                let a = t.comments, n = r;
                Dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/comments", parentData: t, parentDataProperty: "comments", rootData: s }) || (e = e === null ? Dn.errors : e.concat(Dn.errors), r = e.length);
                var at = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var at = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var at = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var at = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var at = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var at = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var at = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var at = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var at = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var at = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var at = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var at = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var at = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var at = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var at = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var at = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var at = n === r;
              }
              if (t["fig-responsive"] !== void 0) {
                let a = t["fig-responsive"], n = r;
                cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-responsive", parentData: t, parentDataProperty: "fig-responsive", rootData: s }) || (e = e === null ? cn.errors : e.concat(cn.errors), r = e.length);
                var at = n === r;
              }
              if (t.mainfont !== void 0) {
                let a = t.mainfont, n = r;
                Ja(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/mainfont", parentData: t, parentDataProperty: "mainfont", rootData: s }) || (e = e === null ? Ja.errors : e.concat(Ja.errors), r = e.length);
                var at = n === r;
              }
              if (t.monofont !== void 0) {
                let a = t.monofont, n = r;
                Qa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/monofont", parentData: t, parentDataProperty: "monofont", rootData: s }) || (e = e === null ? Qa.errors : e.concat(Qa.errors), r = e.length);
                var at = n === r;
              }
              if (t.fontsize !== void 0) {
                let a = t.fontsize, n = r;
                Ga(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fontsize", parentData: t, parentDataProperty: "fontsize", rootData: s }) || (e = e === null ? Ga.errors : e.concat(Ga.errors), r = e.length);
                var at = n === r;
              }
              if (t.linestretch !== void 0) {
                let a = t.linestretch, n = r;
                Ya(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/linestretch", parentData: t, parentDataProperty: "linestretch", rootData: s }) || (e = e === null ? Ya.errors : e.concat(Ya.errors), r = e.length);
                var at = n === r;
              }
              if (t["footnotes-hover"] !== void 0) {
                let a = t["footnotes-hover"], n = r;
                wn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/footnotes-hover", parentData: t, parentDataProperty: "footnotes-hover", rootData: s }) || (e = e === null ? wn.errors : e.concat(wn.errors), r = e.length);
                var at = n === r;
              }
              if (t["reference-location"] !== void 0) {
                let a = t["reference-location"], n = r;
                Ca(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reference-location", parentData: t, parentDataProperty: "reference-location", rootData: s }) || (e = e === null ? Ca.errors : e.concat(Ca.errors), r = e.length);
                var at = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var at = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var at = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var at = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var at = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var at = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var at = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var at = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var at = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var at = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var at = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var at = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var at = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var at = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var at = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var at = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var at = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var at = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var at = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var at = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var at = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var at = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var at = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var at = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var at = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var at = n === r;
              }
              if (t.resources !== void 0) {
                let a = t.resources, n = r;
                pn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resources", parentData: t, parentDataProperty: "resources", rootData: s }) || (e = e === null ? pn.errors : e.concat(pn.errors), r = e.length);
                var at = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var at = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var at = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var at = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var at = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var at = n === r;
              }
              if (t.classoption !== void 0) {
                let a = t.classoption, n = r;
                un(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/classoption", parentData: t, parentDataProperty: "classoption", rootData: s }) || (e = e === null ? un.errors : e.concat(un.errors), r = e.length);
                var at = n === r;
              }
              if (t["max-width"] !== void 0) {
                let a = t["max-width"], n = r;
                $n(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/max-width", parentData: t, parentDataProperty: "max-width", rootData: s }) || (e = e === null ? $n.errors : e.concat($n.errors), r = e.length);
                var at = n === r;
              }
              if (t["margin-left"] !== void 0) {
                let a = t["margin-left"], n = r;
                Xa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-left", parentData: t, parentDataProperty: "margin-left", rootData: s }) || (e = e === null ? Xa.errors : e.concat(Xa.errors), r = e.length);
                var at = n === r;
              }
              if (t["margin-right"] !== void 0) {
                let a = t["margin-right"], n = r;
                Za(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-right", parentData: t, parentDataProperty: "margin-right", rootData: s }) || (e = e === null ? Za.errors : e.concat(Za.errors), r = e.length);
                var at = n === r;
              }
              if (t["margin-top"] !== void 0) {
                let a = t["margin-top"], n = r;
                en(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-top", parentData: t, parentDataProperty: "margin-top", rootData: s }) || (e = e === null ? en.errors : e.concat(en.errors), r = e.length);
                var at = n === r;
              }
              if (t["margin-bottom"] !== void 0) {
                let a = t["margin-bottom"], n = r;
                rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/margin-bottom", parentData: t, parentDataProperty: "margin-bottom", rootData: s }) || (e = e === null ? rn.errors : e.concat(rn.errors), r = e.length);
                var at = n === r;
              }
              if (t["slidy-url"] !== void 0) {
                let a = t["slidy-url"], n = r;
                Ic(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/slidy-url", parentData: t, parentDataProperty: "slidy-url", rootData: s }) || (e = e === null ? Ic.errors : e.concat(Ic.errors), r = e.length);
                var at = n === r;
              }
              if (t.keywords !== void 0) {
                let a = t.keywords, n = r;
                Ia(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keywords", parentData: t, parentDataProperty: "keywords", rootData: s }) || (e = e === null ? Ia.errors : e.concat(Ia.errors), r = e.length);
                var at = n === r;
              }
              if (t.pagetitle !== void 0) {
                let a = t.pagetitle, n = r;
                Sn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/pagetitle", parentData: t, parentDataProperty: "pagetitle", rootData: s }) || (e = e === null ? Sn.errors : e.concat(Sn.errors), r = e.length);
                var at = n === r;
              }
              if (t["title-prefix"] !== void 0) {
                let a = t["title-prefix"], n = r;
                Fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title-prefix", parentData: t, parentDataProperty: "title-prefix", rootData: s }) || (e = e === null ? Fn.errors : e.concat(Fn.errors), r = e.length);
                var at = n === r;
              }
              if (t["description-meta"] !== void 0) {
                let a = t["description-meta"], n = r;
                An(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/description-meta", parentData: t, parentDataProperty: "description-meta", rootData: s }) || (e = e === null ? An.errors : e.concat(An.errors), r = e.length);
                var at = n === r;
              }
              if (t["author-meta"] !== void 0) {
                let a = t["author-meta"], n = r;
                dn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author-meta", parentData: t, parentDataProperty: "author-meta", rootData: s }) || (e = e === null ? dn.errors : e.concat(dn.errors), r = e.length);
                var at = n === r;
              }
              if (t["date-meta"] !== void 0) {
                let a = t["date-meta"], n = r;
                Ha(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date-meta", parentData: t, parentDataProperty: "date-meta", rootData: s }) || (e = e === null ? Ha.errors : e.concat(Ha.errors), r = e.length);
                var at = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var at = n === r;
              }
              if (t["number-depth"] !== void 0) {
                let a = t["number-depth"], n = r;
                Da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-depth", parentData: t, parentDataProperty: "number-depth", rootData: s }) || (e = e === null ? Da.errors : e.concat(Da.errors), r = e.length);
                var at = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var at = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var at = n === r;
              }
              if (t["document-css"] !== void 0) {
                let a = t["document-css"], n = r;
                jn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/document-css", parentData: t, parentDataProperty: "document-css", rootData: s }) || (e = e === null ? jn.errors : e.concat(jn.errors), r = e.length);
                var at = n === r;
              }
              if (t.css !== void 0) {
                let a = t.css, n = r;
                fn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/css", parentData: t, parentDataProperty: "css", rootData: s }) || (e = e === null ? fn.errors : e.concat(fn.errors), r = e.length);
                var at = n === r;
              }
              if (t["section-divs"] !== void 0) {
                let a = t["section-divs"], n = r;
                En(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/section-divs", parentData: t, parentDataProperty: "section-divs", rootData: s }) || (e = e === null ? En.errors : e.concat(En.errors), r = e.length);
                var at = n === r;
              }
              if (t["identifier-prefix"] !== void 0) {
                let a = t["identifier-prefix"], n = r;
                Pa(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/identifier-prefix", parentData: t, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
                var at = n === r;
              }
              if (t["email-obfuscation"] !== void 0) {
                let a = t["email-obfuscation"], n = r;
                Nn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/email-obfuscation", parentData: t, parentDataProperty: "email-obfuscation", rootData: s }) || (e = e === null ? Nn.errors : e.concat(Nn.errors), r = e.length);
                var at = n === r;
              }
              if (t["html-q-tags"] !== void 0) {
                let a = t["html-q-tags"], n = r;
                mn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/html-q-tags", parentData: t, parentDataProperty: "html-q-tags", rootData: s }) || (e = e === null ? mn.errors : e.concat(mn.errors), r = e.length);
                var at = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var at = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var at = n === r;
              }
              if (t["citations-hover"] !== void 0) {
                let a = t["citations-hover"], n = r;
                Cn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citations-hover", parentData: t, parentDataProperty: "citations-hover", rootData: s }) || (e = e === null ? Cn.errors : e.concat(Cn.errors), r = e.length);
                var at = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var at = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var at = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var at = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var at = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var at = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var at = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var at = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var at = n === r;
              }
              if (t["self-contained"] !== void 0) {
                let a = t["self-contained"], n = r;
                In(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/self-contained", parentData: t, parentDataProperty: "self-contained", rootData: s }) || (e = e === null ? In.errors : e.concat(In.errors), r = e.length);
                var at = n === r;
              }
              if (t["self-contained-math"] !== void 0) {
                let a = t["self-contained-math"], n = r;
                Hn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/self-contained-math", parentData: t, parentDataProperty: "self-contained-math", rootData: s }) || (e = e === null ? Hn.errors : e.concat(Hn.errors), r = e.length);
                var at = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var at = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var at = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var at = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var at = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var at = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var at = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var at = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var at = n === r;
              }
              if (t.incremental !== void 0) {
                let a = t.incremental, n = r;
                Rn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/incremental", parentData: t, parentDataProperty: "incremental", rootData: s }) || (e = e === null ? Rn.errors : e.concat(Rn.errors), r = e.length);
                var at = n === r;
              }
              if (t["slide-level"] !== void 0) {
                let a = t["slide-level"], n = r;
                Vn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/slide-level", parentData: t, parentDataProperty: "slide-level", rootData: s }) || (e = e === null ? Vn.errors : e.concat(Vn.errors), r = e.length);
                var at = n === r;
              }
              if (t["strip-comments"] !== void 0) {
                let a = t["strip-comments"], n = r;
                Ra(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-comments", parentData: t, parentDataProperty: "strip-comments", rootData: s }) || (e = e === null ? Ra.errors : e.concat(Ra.errors), r = e.length);
                var at = n === r;
              }
              if (t.ascii !== void 0) {
                let a = t.ascii, n = r;
                da(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ascii", parentData: t, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
                var at = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var at = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var at = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var at = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eslidy(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Rm = ht === r;
            Rm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eslidy(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Rm = gt === r;
            if (Rm && L ? (L = false, M = [M, 1]) : Rm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Eslidy(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^slidy(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (g1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!aq.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etei(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etei(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var u0 = n === r;
                if (!u0) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etei(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["top-level-division"] !== void 0) {
                let a = t["top-level-division"], n = r;
                Bn(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/top-level-division", parentData: t, parentDataProperty: "top-level-division", rootData: s }) || (e = e === null ? Bn.errors : e.concat(Bn.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Lt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Lt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Lt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etei(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Vm = ht === r;
            Vm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etei(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Vm = gt === r;
            if (Vm && L ? (L = false, M = [M, 1]) : Vm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etei(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^tei(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (h1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etexinfo(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etexinfo(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var d0 = n === r;
                if (!d0) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etexinfo(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Xt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Xt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Xt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etexinfo(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Um = ht === r;
            Um && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etexinfo(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Um = gt === r;
            if (Um && L ? (L = false, M = [M, 1]) : Um && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etexinfo(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^texinfo(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (y1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!nq.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etextile(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etextile(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var f0 = n === r;
                if (!f0) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etextile(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["strip-comments"] !== void 0) {
                let a = t["strip-comments"], n = r;
                Ra(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-comments", parentData: t, parentDataProperty: "strip-comments", rootData: s }) || (e = e === null ? Ra.errors : e.concat(Ra.errors), r = e.length);
                var Mt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Mt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Mt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etextile(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Km = ht === r;
            Km && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etextile(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Km = gt === r;
            if (Km && L ? (L = false, M = [M, 1]) : Km && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Etextile(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^textile(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (b1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Exwiki(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Exwiki(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var m0 = n === r;
                if (!m0) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Exwiki(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var Zt = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var Zt = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var Zt = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Exwiki(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Jm = ht === r;
            Jm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Exwiki(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Jm = gt === r;
            if (Jm && L ? (L = false, M = [M, 1]) : Jm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Exwiki(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^xwiki(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
        for (let i in l)
          if (_1.test(i)) {
            let t = l[i], mt = r, ne = r, L = false, M = null, ht = r, ia = r;
            if (t && typeof t == "object" && !Array.isArray(t)) {
              for (let a in t) {
                let n = r, oa = r;
                if (typeof a == "string") {
                  if (!Xn.test(a)) {
                    let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ezimwiki(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", schema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))"', schema: "^(?!(tbl_colwidths|tblColwidths|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|strip_empty_paragraphs|stripEmptyParagraphs|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|number_sections|numberSections|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|keep_md|keepMd|keep_ipynb|keepIpynb|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|tab_stop|tabStop|preserve_tabs|preserveTabs|table_of_contents|tableOfContents|toc_depth|tocDepth))", parentSchema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                    e === null ? e = [k] : e.push(k), r++;
                  }
                } else {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ezimwiki(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf[0].propertyNames }, message: "must be string", schema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf[0].propertyNames.type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf[0].propertyNames, data: a, propertyName: a };
                  e === null ? e = [k] : e.push(k), r++;
                }
                var g0 = n === r;
                if (!g0) {
                  let k = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ezimwiki(%5C%2B.%2B)%3F%24/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: a, schema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf[0] }, message: "property name must be valid", schema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf[0].propertyNames, parentSchema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf[0], data: t };
                  e === null ? e = [k] : e.push(k), r++;
                }
              }
              if (t.eval !== void 0) {
                let a = t.eval, n = r;
                ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eval", parentData: t, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
                var ea = n === r;
              }
              if (t.echo !== void 0) {
                let a = t.echo, n = r;
                de(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/echo", parentData: t, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
                var ea = n === r;
              }
              if (t["tbl-colwidths"] !== void 0) {
                let a = t["tbl-colwidths"], n = r;
                be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tbl-colwidths", parentData: t, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
                var ea = n === r;
              }
              if (t.output !== void 0) {
                let a = t.output, n = r;
                fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output", parentData: t, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
                var ea = n === r;
              }
              if (t.warning !== void 0) {
                let a = t.warning, n = r;
                me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/warning", parentData: t, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
                var ea = n === r;
              }
              if (t.error !== void 0) {
                let a = t.error, n = r;
                ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/error", parentData: t, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
                var ea = n === r;
              }
              if (t.include !== void 0) {
                let a = t.include, n = r;
                he(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include", parentData: t, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
                var ea = n === r;
              }
              if (t.title !== void 0) {
                let a = t.title, n = r;
                Ee(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/title", parentData: t, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
                var ea = n === r;
              }
              if (t.date !== void 0) {
                let a = t.date, n = r;
                Ne(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/date", parentData: t, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
                var ea = n === r;
              }
              if (t.author !== void 0) {
                let a = t.author, n = r;
                Ce(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/author", parentData: t, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
                var ea = n === r;
              }
              if (t.crossref !== void 0) {
                let a = t.crossref, n = r;
                Ie(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/crossref", parentData: t, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
                var ea = n === r;
              }
              if (t.engine !== void 0) {
                let a = t.engine, n = r;
                qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/engine", parentData: t, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
                var ea = n === r;
              }
              if (t.jupyter !== void 0) {
                let a = t.jupyter, n = r;
                Oe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/jupyter", parentData: t, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
                var ea = n === r;
              }
              if (t.knitr !== void 0) {
                let a = t.knitr, n = r;
                Pe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/knitr", parentData: t, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
                var ea = n === r;
              }
              if (t.cache !== void 0) {
                let a = t.cache, n = r;
                xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/cache", parentData: t, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
                var ea = n === r;
              }
              if (t.freeze !== void 0) {
                let a = t.freeze, n = r;
                ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/freeze", parentData: t, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
                var ea = n === r;
              }
              if (t.server !== void 0) {
                let a = t.server, n = r;
                De(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/server", parentData: t, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
                var ea = n === r;
              }
              if (t.daemon !== void 0) {
                let a = t.daemon, n = r;
                we(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon", parentData: t, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
                var ea = n === r;
              }
              if (t["daemon-restart"] !== void 0) {
                let a = t["daemon-restart"], n = r;
                $e(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/daemon-restart", parentData: t, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
                var ea = n === r;
              }
              if (t.enabled !== void 0) {
                let a = t.enabled, n = r;
                Se(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/enabled", parentData: t, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
                var ea = n === r;
              }
              if (t.ipynb !== void 0) {
                let a = t.ipynb, n = r;
                Fe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ipynb", parentData: t, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
                var ea = n === r;
              }
              if (t.debug !== void 0) {
                let a = t.debug, n = r;
                Ae(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/debug", parentData: t, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
                var ea = n === r;
              }
              if (t["fig-width"] !== void 0) {
                let a = t["fig-width"], n = r;
                He(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-width", parentData: t, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
                var ea = n === r;
              }
              if (t["fig-height"] !== void 0) {
                let a = t["fig-height"], n = r;
                Le(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-height", parentData: t, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
                var ea = n === r;
              }
              if (t["fig-format"] !== void 0) {
                let a = t["fig-format"], n = r;
                Me(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-format", parentData: t, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
                var ea = n === r;
              }
              if (t["fig-dpi"] !== void 0) {
                let a = t["fig-dpi"], n = r;
                Te(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fig-dpi", parentData: t, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
                var ea = n === r;
              }
              if (t.to !== void 0) {
                let a = t.to, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/to", parentData: t, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var ea = n === r;
              }
              if (t.writer !== void 0) {
                let a = t.writer, n = r;
                I(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/writer", parentData: t, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
                var ea = n === r;
              }
              if (t["input-file"] !== void 0) {
                let a = t["input-file"], n = r;
                ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-file", parentData: t, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
                var ea = n === r;
              }
              if (t["input-files"] !== void 0) {
                let a = t["input-files"], n = r;
                We(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/input-files", parentData: t, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
                var ea = n === r;
              }
              if (t.defaults !== void 0) {
                let a = t.defaults, n = r;
                Re(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/defaults", parentData: t, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
                var ea = n === r;
              }
              if (t.variables !== void 0) {
                let a = t.variables, n = r;
                Ve(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/variables", parentData: t, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
                var ea = n === r;
              }
              if (t.metadata !== void 0) {
                let a = t.metadata, n = r;
                Ue(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata", parentData: t, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
                var ea = n === r;
              }
              if (t["request-headers"] !== void 0) {
                let a = t["request-headers"], n = r;
                Ke(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/request-headers", parentData: t, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
                var ea = n === r;
              }
              if (t.trace !== void 0) {
                let a = t.trace, n = r;
                Je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/trace", parentData: t, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
                var ea = n === r;
              }
              if (t["fail-if-warnings"] !== void 0) {
                let a = t["fail-if-warnings"], n = r;
                Qe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/fail-if-warnings", parentData: t, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
                var ea = n === r;
              }
              if (t["dump-args"] !== void 0) {
                let a = t["dump-args"], n = r;
                Ge(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dump-args", parentData: t, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
                var ea = n === r;
              }
              if (t["ignore-args"] !== void 0) {
                let a = t["ignore-args"], n = r;
                Ye(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/ignore-args", parentData: t, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
                var ea = n === r;
              }
              if (t["file-scope"] !== void 0) {
                let a = t["file-scope"], n = r;
                Xe(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/file-scope", parentData: t, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
                var ea = n === r;
              }
              if (t["data-dir"] !== void 0) {
                let a = t["data-dir"], n = r;
                Ze(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/data-dir", parentData: t, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
                var ea = n === r;
              }
              if (t.verbosity !== void 0) {
                let a = t.verbosity, n = r;
                er(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/verbosity", parentData: t, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
                var ea = n === r;
              }
              if (t["log-file"] !== void 0) {
                let a = t["log-file"], n = r;
                rr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/log-file", parentData: t, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
                var ea = n === r;
              }
              if (t["strip-empty-paragraphs"] !== void 0) {
                let a = t["strip-empty-paragraphs"], n = r;
                tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/strip-empty-paragraphs", parentData: t, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
                var ea = n === r;
              }
              if (t["output-divs"] !== void 0) {
                let a = t["output-divs"], n = r;
                ar(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-divs", parentData: t, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
                var ea = n === r;
              }
              if (t["merge-includes"] !== void 0) {
                let a = t["merge-includes"], n = r;
                nr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/merge-includes", parentData: t, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
                var ea = n === r;
              }
              if (t["header-includes"] !== void 0) {
                let a = t["header-includes"], n = r;
                Ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/header-includes", parentData: t, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
                var ea = n === r;
              }
              if (t["include-before"] !== void 0) {
                let a = t["include-before"], n = r;
                Hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before", parentData: t, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
                var ea = n === r;
              }
              if (t["include-after"] !== void 0) {
                let a = t["include-after"], n = r;
                Lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after", parentData: t, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
                var ea = n === r;
              }
              if (t["include-before-body"] !== void 0) {
                let a = t["include-before-body"], n = r;
                Mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-before-body", parentData: t, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
                var ea = n === r;
              }
              if (t["include-after-body"] !== void 0) {
                let a = t["include-after-body"], n = r;
                Tr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-after-body", parentData: t, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
                var ea = n === r;
              }
              if (t["include-in-header"] !== void 0) {
                let a = t["include-in-header"], n = r;
                zr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/include-in-header", parentData: t, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
                var ea = n === r;
              }
              if (t["metadata-file"] !== void 0) {
                let a = t["metadata-file"], n = r;
                or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-file", parentData: t, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
                var ea = n === r;
              }
              if (t["metadata-files"] !== void 0) {
                let a = t["metadata-files"], n = r;
                sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/metadata-files", parentData: t, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
                var ea = n === r;
              }
              if (t.lang !== void 0) {
                let a = t.lang, n = r;
                ir(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/lang", parentData: t, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
                var ea = n === r;
              }
              if (t.language !== void 0) {
                let a = t.language, n = r;
                lr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/language", parentData: t, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
                var ea = n === r;
              }
              if (t.dir !== void 0) {
                let a = t.dir, n = r;
                cr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dir", parentData: t, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
                var ea = n === r;
              }
              if (t["number-sections"] !== void 0) {
                let a = t["number-sections"], n = r;
                pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-sections", parentData: t, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
                var ea = n === r;
              }
              if (t["number-offset"] !== void 0) {
                let a = t["number-offset"], n = r;
                ur(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/number-offset", parentData: t, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
                var ea = n === r;
              }
              if (t["shift-heading-level-by"] !== void 0) {
                let a = t["shift-heading-level-by"], n = r;
                dr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/shift-heading-level-by", parentData: t, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
                var ea = n === r;
              }
              if (t.bibliography !== void 0) {
                let a = t.bibliography, n = r;
                fr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/bibliography", parentData: t, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
                var ea = n === r;
              }
              if (t.csl !== void 0) {
                let a = t.csl, n = r;
                mr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/csl", parentData: t, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
                var ea = n === r;
              }
              if (t.citeproc !== void 0) {
                let a = t.citeproc, n = r;
                gr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citeproc", parentData: t, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
                var ea = n === r;
              }
              if (t["citation-abbreviations"] !== void 0) {
                let a = t["citation-abbreviations"], n = r;
                hr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/citation-abbreviations", parentData: t, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
                var ea = n === r;
              }
              if (t.from !== void 0) {
                let a = t.from, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/from", parentData: t, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var ea = n === r;
              }
              if (t.reader !== void 0) {
                let a = t.reader, n = r;
                H(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/reader", parentData: t, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
                var ea = n === r;
              }
              if (t["output-file"] !== void 0) {
                let a = t["output-file"], n = r;
                yr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-file", parentData: t, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
                var ea = n === r;
              }
              if (t["output-ext"] !== void 0) {
                let a = t["output-ext"], n = r;
                br(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/output-ext", parentData: t, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
                var ea = n === r;
              }
              if (t.template !== void 0) {
                let a = t.template, n = r;
                Sr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/template", parentData: t, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
                var ea = n === r;
              }
              if (t.standalone !== void 0) {
                let a = t.standalone, n = r;
                _r(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/standalone", parentData: t, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
                var ea = n === r;
              }
              if (t.filters !== void 0) {
                let a = t.filters, n = r;
                vr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/filters", parentData: t, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
                var ea = n === r;
              }
              if (t["keep-md"] !== void 0) {
                let a = t["keep-md"], n = r;
                Be(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-md", parentData: t, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
                var ea = n === r;
              }
              if (t["keep-ipynb"] !== void 0) {
                let a = t["keep-ipynb"], n = r;
                je(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/keep-ipynb", parentData: t, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
                var ea = n === r;
              }
              if (t["extract-media"] !== void 0) {
                let a = t["extract-media"], n = r;
                qr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/extract-media", parentData: t, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
                var ea = n === r;
              }
              if (t["resource-path"] !== void 0) {
                let a = t["resource-path"], n = r;
                Or(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/resource-path", parentData: t, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
                var ea = n === r;
              }
              if (t["default-image-extension"] !== void 0) {
                let a = t["default-image-extension"], n = r;
                Pr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/default-image-extension", parentData: t, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
                var ea = n === r;
              }
              if (t.abbreviations !== void 0) {
                let a = t.abbreviations, n = r;
                xr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/abbreviations", parentData: t, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
                var ea = n === r;
              }
              if (t.dpi !== void 0) {
                let a = t.dpi, n = r;
                kr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/dpi", parentData: t, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
                var ea = n === r;
              }
              if (t.wrap !== void 0) {
                let a = t.wrap, n = r;
                it(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/wrap", parentData: t, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
                var ea = n === r;
              }
              if (t.columns !== void 0) {
                let a = t.columns, n = r;
                lt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/columns", parentData: t, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
                var ea = n === r;
              }
              if (t["tab-stop"] !== void 0) {
                let a = t["tab-stop"], n = r;
                ct(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/tab-stop", parentData: t, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
                var ea = n === r;
              }
              if (t["preserve-tabs"] !== void 0) {
                let a = t["preserve-tabs"], n = r;
                pt(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/preserve-tabs", parentData: t, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
                var ea = n === r;
              }
              if (t.eol !== void 0) {
                let a = t.eol, n = r;
                ut(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/eol", parentData: t, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
                var ea = n === r;
              }
              if (t.toc !== void 0) {
                let a = t.toc, n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc", parentData: t, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var ea = n === r;
              }
              if (t["table-of-contents"] !== void 0) {
                let a = t["table-of-contents"], n = r;
                K(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/table-of-contents", parentData: t, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
                var ea = n === r;
              }
              if (t["toc-depth"] !== void 0) {
                let a = t["toc-depth"], n = r;
                Wr(a, { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1") + "/toc-depth", parentData: t, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
                var ea = n === r;
              }
            } else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ezimwiki(%5C%2B.%2B)%3F%24/oneOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf[0].type, parentSchema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf[0], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Qm = ht === r;
            Qm && (L = true, M = 0);
            let gt = r, ma = p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf[1].enum;
            if (t !== "default") {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ezimwiki(%5C%2B.%2B)%3F%24/oneOf/1/enum", keyword: "enum", params: { allowedValues: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf[1].enum, schema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf[1] }, message: "must be equal to one of the allowed values", schema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf[1].enum, parentSchema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf[1], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Qm = gt === r;
            if (Qm && L ? (L = false, M = [M, 1]) : Qm && (L = true, M = 1), L)
              r = ne, e !== null && (ne ? e.length = ne : e = null);
            else {
              let a = { instancePath: o + "/" + i.replace(/~/g, "~0").replace(/\//g, "~1"), schemaPath: "#/oneOf/2/allOf/0/patternProperties/%5Ezimwiki(%5C%2B.%2B)%3F%24/oneOf", keyword: "oneOf", params: { passingSchemas: M, schema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"] }, message: "must match exactly one schema in oneOf", schema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"].oneOf, parentSchema: p.oneOf[2].allOf[0].patternProperties["^zimwiki(\\+.+)?$"], data: t };
              e === null ? e = [a] : e.push(a), r++;
            }
            var Cr = mt === r;
          }
      } else {
        let Gm = { instancePath: o, schemaPath: "#/oneOf/2/allOf/0/type", keyword: "type", params: { type: "object", schema: p.oneOf[2].allOf[0] }, message: "must be object", schema: p.oneOf[2].allOf[0].type, parentSchema: p.oneOf[2].allOf[0], data: l };
        e === null ? e = [Gm] : e.push(Gm), r++;
      }
      var uq = k1 === r, V = Ln === r;
      V && c ? (c = false, d = [d, 2]) : V && (c = true, d = 2);
    }
    if (c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let Ln = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: p }, message: "must match exactly one schema in oneOf", schema: p.oneOf, parentSchema: p, data: l };
      e === null ? e = [Ln] : e.push(Ln), r++;
    }
    return _u.errors = e, r === 0;
  }
  B["front-matter"] = _m;
  var yo = { oneOf: [{ type: "null", description: "be the null value" }, { allOf: [{ type: "object", description: "be a Quarto YAML front matter object", properties: { execute: { $ref: "front-matter-execute", description: "be a front-matter-execute object" }, format: { $ref: "front-matter-format", description: "be exactly one of: the name of a pandoc-supported output format, 'hugo', all of: an object" } }, patternProperties: {} }, { type: "object", description: "be an object", properties: { eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-fold": { $ref: "quarto-resource-cell-codeoutput-code-fold", description: "quarto-resource-cell-codeoutput-code-fold" }, "code-summary": { $ref: "quarto-resource-cell-codeoutput-code-summary", description: "quarto-resource-cell-codeoutput-code-summary" }, "code-overflow": { $ref: "quarto-resource-cell-codeoutput-code-overflow", description: "quarto-resource-cell-codeoutput-code-overflow" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "fig-env": { $ref: "quarto-resource-cell-figure-fig-env", description: "quarto-resource-cell-figure-fig-env" }, "fig-pos": { $ref: "quarto-resource-cell-figure-fig-pos", description: "quarto-resource-cell-figure-fig-pos" }, "cap-location": { $ref: "quarto-resource-cell-pagelayout-cap-location", description: "quarto-resource-cell-pagelayout-cap-location" }, "fig-cap-location": { $ref: "quarto-resource-cell-pagelayout-fig-cap-location", description: "quarto-resource-cell-pagelayout-fig-cap-location" }, "tbl-cap-location": { $ref: "quarto-resource-cell-pagelayout-tbl-cap-location", description: "quarto-resource-cell-pagelayout-tbl-cap-location" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, title: { $ref: "quarto-resource-document-attributes-title", description: "quarto-resource-document-attributes-title" }, subtitle: { $ref: "quarto-resource-document-attributes-subtitle", description: "quarto-resource-document-attributes-subtitle" }, date: { $ref: "quarto-resource-document-attributes-date", description: "quarto-resource-document-attributes-date" }, author: { $ref: "quarto-resource-document-attributes-author", description: "quarto-resource-document-attributes-author" }, affiliation: { $ref: "quarto-resource-document-attributes-affiliation", description: "quarto-resource-document-attributes-affiliation" }, copyright: { $ref: "quarto-resource-document-attributes-copyright", description: "quarto-resource-document-attributes-copyright" }, article: { $ref: "quarto-resource-document-attributes-article", description: "quarto-resource-document-attributes-article" }, journal: { $ref: "quarto-resource-document-attributes-journal", description: "quarto-resource-document-attributes-journal" }, institute: { $ref: "quarto-resource-document-attributes-institute", description: "quarto-resource-document-attributes-institute" }, abstract: { $ref: "quarto-resource-document-attributes-abstract", description: "quarto-resource-document-attributes-abstract" }, "abstract-title": { $ref: "quarto-resource-document-attributes-abstract-title", description: "quarto-resource-document-attributes-abstract-title" }, notes: { $ref: "quarto-resource-document-attributes-notes", description: "quarto-resource-document-attributes-notes" }, tags: { $ref: "quarto-resource-document-attributes-tags", description: "quarto-resource-document-attributes-tags" }, doi: { $ref: "quarto-resource-document-attributes-doi", description: "quarto-resource-document-attributes-doi" }, thanks: { $ref: "quarto-resource-document-attributes-thanks", description: "quarto-resource-document-attributes-thanks" }, "code-copy": { $ref: "quarto-resource-document-code-code-copy", description: "quarto-resource-document-code-code-copy" }, "code-link": { $ref: "quarto-resource-document-code-code-link", description: "quarto-resource-document-code-code-link" }, "code-tools": { $ref: "quarto-resource-document-code-code-tools", description: "quarto-resource-document-code-code-tools" }, "highlight-style": { $ref: "quarto-resource-document-code-highlight-style", description: "quarto-resource-document-code-highlight-style" }, "syntax-definition": { $ref: "quarto-resource-document-code-syntax-definition", description: "quarto-resource-document-code-syntax-definition" }, "syntax-definitions": { $ref: "quarto-resource-document-code-syntax-definitions", description: "quarto-resource-document-code-syntax-definitions" }, listings: { $ref: "quarto-resource-document-code-listings", description: "quarto-resource-document-code-listings" }, "indented-code-classes": { $ref: "quarto-resource-document-code-indented-code-classes", description: "quarto-resource-document-code-indented-code-classes" }, fontcolor: { $ref: "quarto-resource-document-colors-fontcolor", description: "quarto-resource-document-colors-fontcolor" }, linkcolor: { $ref: "quarto-resource-document-colors-linkcolor", description: "quarto-resource-document-colors-linkcolor" }, monobackgroundcolor: { $ref: "quarto-resource-document-colors-monobackgroundcolor", description: "quarto-resource-document-colors-monobackgroundcolor" }, backgroundcolor: { $ref: "quarto-resource-document-colors-backgroundcolor", description: "quarto-resource-document-colors-backgroundcolor" }, filecolor: { $ref: "quarto-resource-document-colors-filecolor", description: "quarto-resource-document-colors-filecolor" }, citecolor: { $ref: "quarto-resource-document-colors-citecolor", description: "quarto-resource-document-colors-citecolor" }, urlcolor: { $ref: "quarto-resource-document-colors-urlcolor", description: "quarto-resource-document-colors-urlcolor" }, toccolor: { $ref: "quarto-resource-document-colors-toccolor", description: "quarto-resource-document-colors-toccolor" }, colorlinks: { $ref: "quarto-resource-document-colors-colorlinks", description: "quarto-resource-document-colors-colorlinks" }, contrastcolor: { $ref: "quarto-resource-document-colors-contrastcolor", description: "quarto-resource-document-colors-contrastcolor" }, comments: { $ref: "quarto-resource-document-comments-comments", description: "quarto-resource-document-comments-comments" }, crossref: { $ref: "quarto-resource-document-crossref-crossref", description: "quarto-resource-document-crossref-crossref" }, identifier: { $ref: "quarto-resource-document-epub-identifier", description: "quarto-resource-document-epub-identifier" }, creator: { $ref: "quarto-resource-document-epub-creator", description: "quarto-resource-document-epub-creator" }, contributor: { $ref: "quarto-resource-document-epub-contributor", description: "quarto-resource-document-epub-contributor" }, subject: { $ref: "quarto-resource-document-metadata-subject", description: "quarto-resource-document-metadata-subject" }, type: { $ref: "quarto-resource-document-epub-type", description: "quarto-resource-document-epub-type" }, relation: { $ref: "quarto-resource-document-epub-relation", description: "quarto-resource-document-epub-relation" }, coverage: { $ref: "quarto-resource-document-epub-coverage", description: "quarto-resource-document-epub-coverage" }, rights: { $ref: "quarto-resource-document-epub-rights", description: "quarto-resource-document-epub-rights" }, "belongs-to-collection": { $ref: "quarto-resource-document-epub-belongs-to-collection", description: "quarto-resource-document-epub-belongs-to-collection" }, "group-position": { $ref: "quarto-resource-document-epub-group-position", description: "quarto-resource-document-epub-group-position" }, "page-progression-direction": { $ref: "quarto-resource-document-epub-page-progression-direction", description: "quarto-resource-document-epub-page-progression-direction" }, ibooks: { $ref: "quarto-resource-document-epub-ibooks", description: "quarto-resource-document-epub-ibooks" }, "epub-metadata": { $ref: "quarto-resource-document-epub-epub-metadata", description: "quarto-resource-document-epub-epub-metadata" }, "epub-subdirectory": { $ref: "quarto-resource-document-epub-epub-subdirectory", description: "quarto-resource-document-epub-epub-subdirectory" }, "epub-fonts": { $ref: "quarto-resource-document-epub-epub-fonts", description: "quarto-resource-document-epub-epub-fonts" }, "epub-chapter-level": { $ref: "quarto-resource-document-epub-epub-chapter-level", description: "quarto-resource-document-epub-epub-chapter-level" }, "epub-cover-image": { $ref: "quarto-resource-document-epub-epub-cover-image", description: "quarto-resource-document-epub-epub-cover-image" }, engine: { $ref: "quarto-resource-document-execute-engine", description: "quarto-resource-document-execute-engine" }, jupyter: { $ref: "quarto-resource-document-execute-jupyter", description: "quarto-resource-document-execute-jupyter" }, knitr: { $ref: "quarto-resource-document-execute-knitr", description: "quarto-resource-document-execute-knitr" }, cache: { $ref: "quarto-resource-document-execute-cache", description: "quarto-resource-document-execute-cache" }, freeze: { $ref: "quarto-resource-document-execute-freeze", description: "quarto-resource-document-execute-freeze" }, server: { $ref: "quarto-resource-document-execute-server", description: "quarto-resource-document-execute-server" }, daemon: { $ref: "quarto-resource-document-execute-daemon", description: "quarto-resource-document-execute-daemon" }, "daemon-restart": { $ref: "quarto-resource-document-execute-daemon-restart", description: "quarto-resource-document-execute-daemon-restart" }, enabled: { $ref: "quarto-resource-document-execute-enabled", description: "quarto-resource-document-execute-enabled" }, ipynb: { $ref: "quarto-resource-document-execute-ipynb", description: "quarto-resource-document-execute-ipynb" }, debug: { $ref: "quarto-resource-document-execute-debug", description: "quarto-resource-document-execute-debug" }, "fig-width": { $ref: "quarto-resource-document-figures-fig-width", description: "quarto-resource-document-figures-fig-width" }, "fig-height": { $ref: "quarto-resource-document-figures-fig-height", description: "quarto-resource-document-figures-fig-height" }, "fig-format": { $ref: "quarto-resource-document-figures-fig-format", description: "quarto-resource-document-figures-fig-format" }, "fig-dpi": { $ref: "quarto-resource-document-figures-fig-dpi", description: "quarto-resource-document-figures-fig-dpi" }, "fig-responsive": { $ref: "quarto-resource-document-figures-fig-responsive", description: "quarto-resource-document-figures-fig-responsive" }, mainfont: { $ref: "quarto-resource-document-fonts-mainfont", description: "quarto-resource-document-fonts-mainfont" }, monofont: { $ref: "quarto-resource-document-fonts-monofont", description: "quarto-resource-document-fonts-monofont" }, fontsize: { $ref: "quarto-resource-document-fonts-fontsize", description: "quarto-resource-document-fonts-fontsize" }, fontenc: { $ref: "quarto-resource-document-fonts-fontenc", description: "quarto-resource-document-fonts-fontenc" }, fontfamily: { $ref: "quarto-resource-document-fonts-fontfamily", description: "quarto-resource-document-fonts-fontfamily" }, fontfamilyoptions: { $ref: "quarto-resource-document-fonts-fontfamilyoptions", description: "quarto-resource-document-fonts-fontfamilyoptions" }, sansfont: { $ref: "quarto-resource-document-fonts-sansfont", description: "quarto-resource-document-fonts-sansfont" }, mathfont: { $ref: "quarto-resource-document-fonts-mathfont", description: "quarto-resource-document-fonts-mathfont" }, CJKmainfont: { $ref: "quarto-resource-document-fonts-CJKmainfont", description: "quarto-resource-document-fonts-CJKmainfont" }, mainfontoptions: { $ref: "quarto-resource-document-fonts-mainfontoptions", description: "quarto-resource-document-fonts-mainfontoptions" }, sansfontoptions: { $ref: "quarto-resource-document-fonts-sansfontoptions", description: "quarto-resource-document-fonts-sansfontoptions" }, monofontoptions: { $ref: "quarto-resource-document-fonts-monofontoptions", description: "quarto-resource-document-fonts-monofontoptions" }, mathfontoptions: { $ref: "quarto-resource-document-fonts-mathfontoptions", description: "quarto-resource-document-fonts-mathfontoptions" }, CJKoptions: { $ref: "quarto-resource-document-fonts-CJKoptions", description: "quarto-resource-document-fonts-CJKoptions" }, microtypeoptions: { $ref: "quarto-resource-document-fonts-microtypeoptions", description: "quarto-resource-document-fonts-microtypeoptions" }, pointsize: { $ref: "quarto-resource-document-fonts-pointsize", description: "quarto-resource-document-fonts-pointsize" }, lineheight: { $ref: "quarto-resource-document-fonts-lineheight", description: "quarto-resource-document-fonts-lineheight" }, linestretch: { $ref: "quarto-resource-document-fonts-linestretch", description: "quarto-resource-document-fonts-linestretch" }, interlinespace: { $ref: "quarto-resource-document-fonts-interlinespace", description: "quarto-resource-document-fonts-interlinespace" }, linkstyle: { $ref: "quarto-resource-document-fonts-linkstyle", description: "quarto-resource-document-fonts-linkstyle" }, whitespace: { $ref: "quarto-resource-document-fonts-whitespace", description: "quarto-resource-document-fonts-whitespace" }, "footnotes-hover": { $ref: "quarto-resource-document-footnotes-footnotes-hover", description: "quarto-resource-document-footnotes-footnotes-hover" }, "links-as-notes": { $ref: "quarto-resource-document-footnotes-links-as-notes", description: "quarto-resource-document-footnotes-links-as-notes" }, "reference-location": { $ref: "quarto-resource-document-footnotes-reference-location", description: "quarto-resource-document-footnotes-reference-location" }, indenting: { $ref: "quarto-resource-document-formatting-indenting", description: "quarto-resource-document-formatting-indenting" }, adjusting: { $ref: "quarto-resource-document-formatting-adjusting", description: "quarto-resource-document-formatting-adjusting" }, hyphenate: { $ref: "quarto-resource-document-formatting-hyphenate", description: "quarto-resource-document-formatting-hyphenate" }, to: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, writer: { $ref: "quarto-resource-document-hidden-to", description: "quarto-resource-document-hidden-to" }, "input-file": { $ref: "quarto-resource-document-hidden-input-file", description: "quarto-resource-document-hidden-input-file" }, "input-files": { $ref: "quarto-resource-document-hidden-input-files", description: "quarto-resource-document-hidden-input-files" }, defaults: { $ref: "quarto-resource-document-hidden-defaults", description: "quarto-resource-document-hidden-defaults" }, variables: { $ref: "quarto-resource-document-hidden-variables", description: "quarto-resource-document-hidden-variables" }, metadata: { $ref: "quarto-resource-document-hidden-metadata", description: "quarto-resource-document-hidden-metadata" }, "request-headers": { $ref: "quarto-resource-document-hidden-request-headers", description: "quarto-resource-document-hidden-request-headers" }, trace: { $ref: "quarto-resource-document-hidden-trace", description: "quarto-resource-document-hidden-trace" }, "fail-if-warnings": { $ref: "quarto-resource-document-hidden-fail-if-warnings", description: "quarto-resource-document-hidden-fail-if-warnings" }, "dump-args": { $ref: "quarto-resource-document-hidden-dump-args", description: "quarto-resource-document-hidden-dump-args" }, "ignore-args": { $ref: "quarto-resource-document-hidden-ignore-args", description: "quarto-resource-document-hidden-ignore-args" }, "file-scope": { $ref: "quarto-resource-document-hidden-file-scope", description: "quarto-resource-document-hidden-file-scope" }, "data-dir": { $ref: "quarto-resource-document-hidden-data-dir", description: "quarto-resource-document-hidden-data-dir" }, verbosity: { $ref: "quarto-resource-document-hidden-verbosity", description: "quarto-resource-document-hidden-verbosity" }, "log-file": { $ref: "quarto-resource-document-hidden-log-file", description: "quarto-resource-document-hidden-log-file" }, "track-changes": { $ref: "quarto-resource-document-hidden-track-changes", description: "quarto-resource-document-hidden-track-changes" }, "strip-empty-paragraphs": { $ref: "quarto-resource-document-hidden-strip-empty-paragraphs", description: "quarto-resource-document-hidden-strip-empty-paragraphs" }, "keep-source": { $ref: "quarto-resource-document-hidden-keep-source", description: "quarto-resource-document-hidden-keep-source" }, "keep-hidden": { $ref: "quarto-resource-document-hidden-keep-hidden", description: "quarto-resource-document-hidden-keep-hidden" }, "prefer-html": { $ref: "quarto-resource-document-hidden-prefer-html", description: "quarto-resource-document-hidden-prefer-html" }, "output-divs": { $ref: "quarto-resource-document-hidden-output-divs", description: "quarto-resource-document-hidden-output-divs" }, "merge-includes": { $ref: "quarto-resource-document-hidden-merge-includes", description: "quarto-resource-document-hidden-merge-includes" }, "header-includes": { $ref: "quarto-resource-document-includes-header-includes", description: "quarto-resource-document-includes-header-includes" }, "include-before": { $ref: "quarto-resource-document-includes-include-before", description: "quarto-resource-document-includes-include-before" }, "include-after": { $ref: "quarto-resource-document-includes-include-after", description: "quarto-resource-document-includes-include-after" }, "include-before-body": { $ref: "quarto-resource-document-includes-include-before-body", description: "quarto-resource-document-includes-include-before-body" }, "include-after-body": { $ref: "quarto-resource-document-includes-include-after-body", description: "quarto-resource-document-includes-include-after-body" }, "include-in-header": { $ref: "quarto-resource-document-includes-include-in-header", description: "quarto-resource-document-includes-include-in-header" }, resources: { $ref: "quarto-resource-document-includes-resources", description: "quarto-resource-document-includes-resources" }, headertext: { $ref: "quarto-resource-document-includes-headertext", description: "quarto-resource-document-includes-headertext" }, footertext: { $ref: "quarto-resource-document-includes-footertext", description: "quarto-resource-document-includes-footertext" }, includesource: { $ref: "quarto-resource-document-includes-includesource", description: "quarto-resource-document-includes-includesource" }, footer: { $ref: "quarto-resource-document-reveal-content-footer", description: "quarto-resource-document-reveal-content-footer" }, header: { $ref: "quarto-resource-document-includes-header", description: "quarto-resource-document-includes-header" }, "metadata-file": { $ref: "quarto-resource-document-includes-metadata-file", description: "quarto-resource-document-includes-metadata-file" }, "metadata-files": { $ref: "quarto-resource-document-includes-metadata-files", description: "quarto-resource-document-includes-metadata-files" }, lang: { $ref: "quarto-resource-document-language-lang", description: "quarto-resource-document-language-lang" }, language: { $ref: "quarto-resource-document-language-language", description: "quarto-resource-document-language-language" }, dir: { $ref: "quarto-resource-document-language-dir", description: "quarto-resource-document-language-dir" }, "latex-auto-mk": { $ref: "quarto-resource-document-latexmk-latex-auto-mk", description: "quarto-resource-document-latexmk-latex-auto-mk" }, "latex-auto-install": { $ref: "quarto-resource-document-latexmk-latex-auto-install", description: "quarto-resource-document-latexmk-latex-auto-install" }, "latex-min-runs": { $ref: "quarto-resource-document-latexmk-latex-min-runs", description: "quarto-resource-document-latexmk-latex-min-runs" }, "latex-max-runs": { $ref: "quarto-resource-document-latexmk-latex-max-runs", description: "quarto-resource-document-latexmk-latex-max-runs" }, "latex-clean": { $ref: "quarto-resource-document-latexmk-latex-clean", description: "quarto-resource-document-latexmk-latex-clean" }, "latex-makeindex": { $ref: "quarto-resource-document-latexmk-latex-makeindex", description: "quarto-resource-document-latexmk-latex-makeindex" }, "latex-makeindex-opts": { $ref: "quarto-resource-document-latexmk-latex-makeindex-opts", description: "quarto-resource-document-latexmk-latex-makeindex-opts" }, "latex-tlmgr-opts": { $ref: "quarto-resource-document-latexmk-latex-tlmgr-opts", description: "quarto-resource-document-latexmk-latex-tlmgr-opts" }, "latex-output-dir": { $ref: "quarto-resource-document-latexmk-latex-output-dir", description: "quarto-resource-document-latexmk-latex-output-dir" }, documentclass: { $ref: "quarto-resource-document-layout-documentclass", description: "quarto-resource-document-layout-documentclass" }, classoption: { $ref: "quarto-resource-document-layout-classoption", description: "quarto-resource-document-layout-classoption" }, pagestyle: { $ref: "quarto-resource-document-layout-pagestyle", description: "quarto-resource-document-layout-pagestyle" }, papersize: { $ref: "quarto-resource-document-layout-papersize", description: "quarto-resource-document-layout-papersize" }, layout: { $ref: "quarto-resource-document-layout-layout", description: "quarto-resource-document-layout-layout" }, "page-layout": { $ref: "quarto-resource-document-layout-page-layout", description: "quarto-resource-document-layout-page-layout" }, "page-width": { $ref: "quarto-resource-document-layout-page-width", description: "quarto-resource-document-layout-page-width" }, "max-width": { $ref: "quarto-resource-document-layout-max-width", description: "quarto-resource-document-layout-max-width" }, "margin-left": { $ref: "quarto-resource-document-layout-margin-left", description: "quarto-resource-document-layout-margin-left" }, "margin-right": { $ref: "quarto-resource-document-layout-margin-right", description: "quarto-resource-document-layout-margin-right" }, "margin-top": { $ref: "quarto-resource-document-layout-margin-top", description: "quarto-resource-document-layout-margin-top" }, "margin-bottom": { $ref: "quarto-resource-document-layout-margin-bottom", description: "quarto-resource-document-layout-margin-bottom" }, geometry: { $ref: "quarto-resource-document-layout-geometry", description: "quarto-resource-document-layout-geometry" }, hyperrefoptions: { $ref: "quarto-resource-document-layout-hyperrefoptions", description: "quarto-resource-document-layout-hyperrefoptions" }, indent: { $ref: "quarto-resource-document-layout-indent", description: "quarto-resource-document-layout-indent" }, "block-headings": { $ref: "quarto-resource-document-layout-block-headings", description: "quarto-resource-document-layout-block-headings" }, "reveal-js-url": { $ref: "quarto-resource-document-library-reveal-js-url", description: "quarto-resource-document-library-reveal-js-url" }, "s5-url": { $ref: "quarto-resource-document-library-s5-url", description: "quarto-resource-document-library-s5-url" }, "slidy-url": { $ref: "quarto-resource-document-library-slidy-url", description: "quarto-resource-document-library-slidy-url" }, "slideous-url": { $ref: "quarto-resource-document-library-slideous-url", description: "quarto-resource-document-library-slideous-url" }, "link-external-icon": { $ref: "quarto-resource-document-links-link-external-icon", description: "quarto-resource-document-links-link-external-icon" }, "link-external-newwindow": { $ref: "quarto-resource-document-links-link-external-newwindow", description: "quarto-resource-document-links-link-external-newwindow" }, "link-external-filter": { $ref: "quarto-resource-document-links-link-external-filter", description: "quarto-resource-document-links-link-external-filter" }, keywords: { $ref: "quarto-resource-document-metadata-keywords", description: "quarto-resource-document-metadata-keywords" }, description: { $ref: "quarto-resource-document-metadata-description", description: "quarto-resource-document-metadata-description" }, category: { $ref: "quarto-resource-document-metadata-category", description: "quarto-resource-document-metadata-category" }, "title-meta": { $ref: "quarto-resource-document-metadata-title-meta", description: "quarto-resource-document-metadata-title-meta" }, pagetitle: { $ref: "quarto-resource-document-metadata-pagetitle", description: "quarto-resource-document-metadata-pagetitle" }, "title-prefix": { $ref: "quarto-resource-document-metadata-title-prefix", description: "quarto-resource-document-metadata-title-prefix" }, "description-meta": { $ref: "quarto-resource-document-metadata-description-meta", description: "quarto-resource-document-metadata-description-meta" }, "author-meta": { $ref: "quarto-resource-document-metadata-author-meta", description: "quarto-resource-document-metadata-author-meta" }, "date-meta": { $ref: "quarto-resource-document-metadata-date-meta", description: "quarto-resource-document-metadata-date-meta" }, "number-sections": { $ref: "quarto-resource-document-numbering-number-sections", description: "quarto-resource-document-numbering-number-sections" }, "number-depth": { $ref: "quarto-resource-document-numbering-number-depth", description: "quarto-resource-document-numbering-number-depth" }, secnumdepth: { $ref: "quarto-resource-document-numbering-secnumdepth", description: "quarto-resource-document-numbering-secnumdepth" }, "number-offset": { $ref: "quarto-resource-document-numbering-number-offset", description: "quarto-resource-document-numbering-number-offset" }, "shift-heading-level-by": { $ref: "quarto-resource-document-numbering-shift-heading-level-by", description: "quarto-resource-document-numbering-shift-heading-level-by" }, pagenumbering: { $ref: "quarto-resource-document-numbering-pagenumbering", description: "quarto-resource-document-numbering-pagenumbering" }, "top-level-division": { $ref: "quarto-resource-document-numbering-top-level-division", description: "quarto-resource-document-numbering-top-level-division" }, "reference-doc": { $ref: "quarto-resource-document-options-reference-doc", description: "quarto-resource-document-options-reference-doc" }, theme: { $ref: "quarto-resource-document-options-theme", description: "quarto-resource-document-options-theme" }, minimal: { $ref: "quarto-resource-document-options-minimal", description: "quarto-resource-document-options-minimal" }, "document-css": { $ref: "quarto-resource-document-options-document-css", description: "quarto-resource-document-options-document-css" }, css: { $ref: "quarto-resource-document-options-css", description: "quarto-resource-document-options-css" }, "anchor-sections": { $ref: "quarto-resource-document-options-anchor-sections", description: "quarto-resource-document-options-anchor-sections" }, "html-math-method": { $ref: "quarto-resource-document-options-html-math-method", description: "quarto-resource-document-options-html-math-method" }, "section-divs": { $ref: "quarto-resource-document-options-section-divs", description: "quarto-resource-document-options-section-divs" }, "identifier-prefix": { $ref: "quarto-resource-document-options-identifier-prefix", description: "quarto-resource-document-options-identifier-prefix" }, "email-obfuscation": { $ref: "quarto-resource-document-options-email-obfuscation", description: "quarto-resource-document-options-email-obfuscation" }, "html-q-tags": { $ref: "quarto-resource-document-options-html-q-tags", description: "quarto-resource-document-options-html-q-tags" }, "pdf-engine": { $ref: "quarto-resource-document-options-pdf-engine", description: "quarto-resource-document-options-pdf-engine" }, "pdf-engine-opt": { $ref: "quarto-resource-document-options-pdf-engine-opt", description: "quarto-resource-document-options-pdf-engine-opt" }, beamerarticle: { $ref: "quarto-resource-document-options-beamerarticle", description: "quarto-resource-document-options-beamerarticle" }, beameroption: { $ref: "quarto-resource-document-options-beameroption", description: "quarto-resource-document-options-beameroption" }, aspectratio: { $ref: "quarto-resource-document-options-aspectratio", description: "quarto-resource-document-options-aspectratio" }, logo: { $ref: "quarto-resource-document-reveal-content-logo", description: "quarto-resource-document-reveal-content-logo" }, titlegraphic: { $ref: "quarto-resource-document-options-titlegraphic", description: "quarto-resource-document-options-titlegraphic" }, navigation: { $ref: "quarto-resource-document-options-navigation", description: "quarto-resource-document-options-navigation" }, "section-titles": { $ref: "quarto-resource-document-options-section-titles", description: "quarto-resource-document-options-section-titles" }, colortheme: { $ref: "quarto-resource-document-options-colortheme", description: "quarto-resource-document-options-colortheme" }, fonttheme: { $ref: "quarto-resource-document-options-fonttheme", description: "quarto-resource-document-options-fonttheme" }, innertheme: { $ref: "quarto-resource-document-options-innertheme", description: "quarto-resource-document-options-innertheme" }, outertheme: { $ref: "quarto-resource-document-options-outertheme", description: "quarto-resource-document-options-outertheme" }, themeoptions: { $ref: "quarto-resource-document-options-themeoptions", description: "quarto-resource-document-options-themeoptions" }, section: { $ref: "quarto-resource-document-options-section", description: "quarto-resource-document-options-section" }, variant: { $ref: "quarto-resource-document-options-variant", description: "quarto-resource-document-options-variant" }, "markdown-headings": { $ref: "quarto-resource-document-options-markdown-headings", description: "quarto-resource-document-options-markdown-headings" }, "keep-yaml": { $ref: "quarto-resource-document-options-keep-yaml", description: "quarto-resource-document-options-keep-yaml" }, "ipynb-output": { $ref: "quarto-resource-document-options-ipynb-output", description: "quarto-resource-document-options-ipynb-output" }, pdfa: { $ref: "quarto-resource-document-pdfa-pdfa", description: "quarto-resource-document-pdfa-pdfa" }, pdfaiccprofile: { $ref: "quarto-resource-document-pdfa-pdfaiccprofile", description: "quarto-resource-document-pdfa-pdfaiccprofile" }, pdfaintent: { $ref: "quarto-resource-document-pdfa-pdfaintent", description: "quarto-resource-document-pdfa-pdfaintent" }, bibliography: { $ref: "quarto-resource-document-references-bibliography", description: "quarto-resource-document-references-bibliography" }, csl: { $ref: "quarto-resource-document-references-csl", description: "quarto-resource-document-references-csl" }, "citations-hover": { $ref: "quarto-resource-document-references-citations-hover", description: "quarto-resource-document-references-citations-hover" }, "citation-location": { $ref: "quarto-resource-document-references-citation-location", description: "quarto-resource-document-references-citation-location" }, "cite-method": { $ref: "quarto-resource-document-references-cite-method", description: "quarto-resource-document-references-cite-method" }, citeproc: { $ref: "quarto-resource-document-references-citeproc", description: "quarto-resource-document-references-citeproc" }, biblatexoptions: { $ref: "quarto-resource-document-references-biblatexoptions", description: "quarto-resource-document-references-biblatexoptions" }, natbiboptions: { $ref: "quarto-resource-document-references-natbiboptions", description: "quarto-resource-document-references-natbiboptions" }, "biblio-style": { $ref: "quarto-resource-document-references-biblio-style", description: "quarto-resource-document-references-biblio-style" }, "biblio-title": { $ref: "quarto-resource-document-references-biblio-title", description: "quarto-resource-document-references-biblio-title" }, "citation-abbreviations": { $ref: "quarto-resource-document-references-citation-abbreviations", description: "quarto-resource-document-references-citation-abbreviations" }, from: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, reader: { $ref: "quarto-resource-document-render-from", description: "quarto-resource-document-render-from" }, "output-file": { $ref: "quarto-resource-document-render-output-file", description: "quarto-resource-document-render-output-file" }, "output-ext": { $ref: "quarto-resource-document-render-output-ext", description: "quarto-resource-document-render-output-ext" }, template: { $ref: "quarto-resource-document-render-template", description: "quarto-resource-document-render-template" }, standalone: { $ref: "quarto-resource-document-render-standalone", description: "quarto-resource-document-render-standalone" }, "self-contained": { $ref: "quarto-resource-document-render-self-contained", description: "quarto-resource-document-render-self-contained" }, "self-contained-math": { $ref: "quarto-resource-document-render-self-contained-math", description: "quarto-resource-document-render-self-contained-math" }, filters: { $ref: "quarto-resource-document-render-filters", description: "quarto-resource-document-render-filters" }, "keep-md": { $ref: "quarto-resource-document-render-keep-md", description: "quarto-resource-document-render-keep-md" }, "keep-ipynb": { $ref: "quarto-resource-document-render-keep-ipynb", description: "quarto-resource-document-render-keep-ipynb" }, "keep-tex": { $ref: "quarto-resource-document-render-keep-tex", description: "quarto-resource-document-render-keep-tex" }, "extract-media": { $ref: "quarto-resource-document-render-extract-media", description: "quarto-resource-document-render-extract-media" }, "resource-path": { $ref: "quarto-resource-document-render-resource-path", description: "quarto-resource-document-render-resource-path" }, "default-image-extension": { $ref: "quarto-resource-document-render-default-image-extension", description: "quarto-resource-document-render-default-image-extension" }, abbreviations: { $ref: "quarto-resource-document-render-abbreviations", description: "quarto-resource-document-render-abbreviations" }, dpi: { $ref: "quarto-resource-document-render-dpi", description: "quarto-resource-document-render-dpi" }, scrollable: { $ref: "quarto-resource-document-reveal-content-scrollable", description: "quarto-resource-document-reveal-content-scrollable" }, smaller: { $ref: "quarto-resource-document-reveal-content-smaller", description: "quarto-resource-document-reveal-content-smaller" }, embedded: { $ref: "quarto-resource-document-reveal-hidden-embedded", description: "quarto-resource-document-reveal-hidden-embedded" }, display: { $ref: "quarto-resource-document-reveal-hidden-display", description: "quarto-resource-document-reveal-hidden-display" }, "auto-stretch": { $ref: "quarto-resource-document-reveal-layout-auto-stretch", description: "quarto-resource-document-reveal-layout-auto-stretch" }, width: { $ref: "quarto-resource-document-reveal-layout-width", description: "quarto-resource-document-reveal-layout-width" }, height: { $ref: "quarto-resource-document-reveal-layout-height", description: "quarto-resource-document-reveal-layout-height" }, margin: { $ref: "quarto-resource-document-reveal-layout-margin", description: "quarto-resource-document-reveal-layout-margin" }, "min-scale": { $ref: "quarto-resource-document-reveal-layout-min-scale", description: "quarto-resource-document-reveal-layout-min-scale" }, "max-scale": { $ref: "quarto-resource-document-reveal-layout-max-scale", description: "quarto-resource-document-reveal-layout-max-scale" }, center: { $ref: "quarto-resource-document-reveal-layout-center", description: "quarto-resource-document-reveal-layout-center" }, "disable-layout": { $ref: "quarto-resource-document-reveal-layout-disable-layout", description: "quarto-resource-document-reveal-layout-disable-layout" }, "preview-links": { $ref: "quarto-resource-document-reveal-media-preview-links", description: "quarto-resource-document-reveal-media-preview-links" }, "auto-play-media": { $ref: "quarto-resource-document-reveal-media-auto-play-media", description: "quarto-resource-document-reveal-media-auto-play-media" }, "preload-iframes": { $ref: "quarto-resource-document-reveal-media-preload-iframes", description: "quarto-resource-document-reveal-media-preload-iframes" }, "view-distance": { $ref: "quarto-resource-document-reveal-media-view-distance", description: "quarto-resource-document-reveal-media-view-distance" }, "mobile-view-distance": { $ref: "quarto-resource-document-reveal-media-mobile-view-distance", description: "quarto-resource-document-reveal-media-mobile-view-distance" }, "parallax-background-image": { $ref: "quarto-resource-document-reveal-media-parallax-background-image", description: "quarto-resource-document-reveal-media-parallax-background-image" }, "parallax-background-size": { $ref: "quarto-resource-document-reveal-media-parallax-background-size", description: "quarto-resource-document-reveal-media-parallax-background-size" }, "parallax-background-horizontal": { $ref: "quarto-resource-document-reveal-media-parallax-background-horizontal", description: "quarto-resource-document-reveal-media-parallax-background-horizontal" }, "parallax-background-vertical": { $ref: "quarto-resource-document-reveal-media-parallax-background-vertical", description: "quarto-resource-document-reveal-media-parallax-background-vertical" }, progress: { $ref: "quarto-resource-document-reveal-navigation-progress", description: "quarto-resource-document-reveal-navigation-progress" }, history: { $ref: "quarto-resource-document-reveal-navigation-history", description: "quarto-resource-document-reveal-navigation-history" }, "navigation-mode": { $ref: "quarto-resource-document-reveal-navigation-navigation-mode", description: "quarto-resource-document-reveal-navigation-navigation-mode" }, touch: { $ref: "quarto-resource-document-reveal-navigation-touch", description: "quarto-resource-document-reveal-navigation-touch" }, keyboard: { $ref: "quarto-resource-document-reveal-navigation-keyboard", description: "quarto-resource-document-reveal-navigation-keyboard" }, "mouse-wheel": { $ref: "quarto-resource-document-reveal-navigation-mouse-wheel", description: "quarto-resource-document-reveal-navigation-mouse-wheel" }, "hide-inactive-cursor": { $ref: "quarto-resource-document-reveal-navigation-hide-inactive-cursor", description: "quarto-resource-document-reveal-navigation-hide-inactive-cursor" }, "hide-cursor-time": { $ref: "quarto-resource-document-reveal-navigation-hide-cursor-time", description: "quarto-resource-document-reveal-navigation-hide-cursor-time" }, loop: { $ref: "quarto-resource-document-reveal-navigation-loop", description: "quarto-resource-document-reveal-navigation-loop" }, shuffle: { $ref: "quarto-resource-document-reveal-navigation-shuffle", description: "quarto-resource-document-reveal-navigation-shuffle" }, controls: { $ref: "quarto-resource-document-reveal-navigation-controls", description: "quarto-resource-document-reveal-navigation-controls" }, "controls-layout": { $ref: "quarto-resource-document-reveal-navigation-controls-layout", description: "quarto-resource-document-reveal-navigation-controls-layout" }, "controls-tutorial": { $ref: "quarto-resource-document-reveal-navigation-controls-tutorial", description: "quarto-resource-document-reveal-navigation-controls-tutorial" }, "controls-back-arrows": { $ref: "quarto-resource-document-reveal-navigation-controls-back-arrows", description: "quarto-resource-document-reveal-navigation-controls-back-arrows" }, "auto-slide": { $ref: "quarto-resource-document-reveal-navigation-auto-slide", description: "quarto-resource-document-reveal-navigation-auto-slide" }, "auto-slide-stoppable": { $ref: "quarto-resource-document-reveal-navigation-auto-slide-stoppable", description: "quarto-resource-document-reveal-navigation-auto-slide-stoppable" }, "auto-slide-method": { $ref: "quarto-resource-document-reveal-navigation-auto-slide-method", description: "quarto-resource-document-reveal-navigation-auto-slide-method" }, "default-timing": { $ref: "quarto-resource-document-reveal-navigation-default-timing", description: "quarto-resource-document-reveal-navigation-default-timing" }, pause: { $ref: "quarto-resource-document-reveal-navigation-pause", description: "quarto-resource-document-reveal-navigation-pause" }, help: { $ref: "quarto-resource-document-reveal-navigation-help", description: "quarto-resource-document-reveal-navigation-help" }, hash: { $ref: "quarto-resource-document-reveal-navigation-hash", description: "quarto-resource-document-reveal-navigation-hash" }, "hash-type": { $ref: "quarto-resource-document-reveal-navigation-hash-type", description: "quarto-resource-document-reveal-navigation-hash-type" }, "hash-one-based-index": { $ref: "quarto-resource-document-reveal-navigation-hash-one-based-index", description: "quarto-resource-document-reveal-navigation-hash-one-based-index" }, "respond-to-hash-changes": { $ref: "quarto-resource-document-reveal-navigation-respond-to-hash-changes", description: "quarto-resource-document-reveal-navigation-respond-to-hash-changes" }, "fragment-in-url": { $ref: "quarto-resource-document-reveal-navigation-fragment-in-url", description: "quarto-resource-document-reveal-navigation-fragment-in-url" }, "slide-tone": { $ref: "quarto-resource-document-reveal-navigation-slide-tone", description: "quarto-resource-document-reveal-navigation-slide-tone" }, "pdf-separate-fragments": { $ref: "quarto-resource-document-reveal-print-pdf-separate-fragments", description: "quarto-resource-document-reveal-print-pdf-separate-fragments" }, "pdf-page-height-offset": { $ref: "quarto-resource-document-reveal-print-pdf-page-height-offset", description: "quarto-resource-document-reveal-print-pdf-page-height-offset" }, overview: { $ref: "quarto-resource-document-reveal-tools-overview", description: "quarto-resource-document-reveal-tools-overview" }, menu: { $ref: "quarto-resource-document-reveal-tools-menu", description: "quarto-resource-document-reveal-tools-menu" }, chalkboard: { $ref: "quarto-resource-document-reveal-tools-chalkboard", description: "quarto-resource-document-reveal-tools-chalkboard" }, multiplex: { $ref: "quarto-resource-document-reveal-tools-multiplex", description: "quarto-resource-document-reveal-tools-multiplex" }, transition: { $ref: "quarto-resource-document-reveal-transitions-transition", description: "quarto-resource-document-reveal-transitions-transition" }, "transition-speed": { $ref: "quarto-resource-document-reveal-transitions-transition-speed", description: "quarto-resource-document-reveal-transitions-transition-speed" }, "background-transition": { $ref: "quarto-resource-document-reveal-transitions-background-transition", description: "quarto-resource-document-reveal-transitions-background-transition" }, fragments: { $ref: "quarto-resource-document-reveal-transitions-fragments", description: "quarto-resource-document-reveal-transitions-fragments" }, "auto-animate": { $ref: "quarto-resource-document-reveal-transitions-auto-animate", description: "quarto-resource-document-reveal-transitions-auto-animate" }, "auto-animate-easing": { $ref: "quarto-resource-document-reveal-transitions-auto-animate-easing", description: "quarto-resource-document-reveal-transitions-auto-animate-easing" }, "auto-animate-duration": { $ref: "quarto-resource-document-reveal-transitions-auto-animate-duration", description: "quarto-resource-document-reveal-transitions-auto-animate-duration" }, "auto-animate-unmatched": { $ref: "quarto-resource-document-reveal-transitions-auto-animate-unmatched", description: "quarto-resource-document-reveal-transitions-auto-animate-unmatched" }, "auto-animate-styles": { $ref: "quarto-resource-document-reveal-transitions-auto-animate-styles", description: "quarto-resource-document-reveal-transitions-auto-animate-styles" }, incremental: { $ref: "quarto-resource-document-slides-incremental", description: "quarto-resource-document-slides-incremental" }, "slide-level": { $ref: "quarto-resource-document-slides-slide-level", description: "quarto-resource-document-slides-slide-level" }, "slide-number": { $ref: "quarto-resource-document-slides-slide-number", description: "quarto-resource-document-slides-slide-number" }, "show-slide-number": { $ref: "quarto-resource-document-slides-show-slide-number", description: "quarto-resource-document-slides-show-slide-number" }, "title-slide-attributes": { $ref: "quarto-resource-document-slides-title-slide-attributes", description: "quarto-resource-document-slides-title-slide-attributes" }, "center-title-slide": { $ref: "quarto-resource-document-slides-center-title-slide", description: "quarto-resource-document-slides-center-title-slide" }, "show-notes": { $ref: "quarto-resource-document-slides-show-notes", description: "quarto-resource-document-slides-show-notes" }, rtl: { $ref: "quarto-resource-document-slides-rtl", description: "quarto-resource-document-slides-rtl" }, wrap: { $ref: "quarto-resource-document-text-wrap", description: "quarto-resource-document-text-wrap" }, columns: { $ref: "quarto-resource-document-text-columns", description: "quarto-resource-document-text-columns" }, "tab-stop": { $ref: "quarto-resource-document-text-tab-stop", description: "quarto-resource-document-text-tab-stop" }, "preserve-tabs": { $ref: "quarto-resource-document-text-preserve-tabs", description: "quarto-resource-document-text-preserve-tabs" }, eol: { $ref: "quarto-resource-document-text-eol", description: "quarto-resource-document-text-eol" }, "strip-comments": { $ref: "quarto-resource-document-text-strip-comments", description: "quarto-resource-document-text-strip-comments" }, ascii: { $ref: "quarto-resource-document-text-ascii", description: "quarto-resource-document-text-ascii" }, toc: { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "table-of-contents": { $ref: "quarto-resource-document-toc-toc", description: "quarto-resource-document-toc-toc" }, "toc-depth": { $ref: "quarto-resource-document-toc-toc-depth", description: "quarto-resource-document-toc-toc-depth" }, "toc-location": { $ref: "quarto-resource-document-toc-toc-location", description: "quarto-resource-document-toc-toc-location" }, "toc-title": { $ref: "quarto-resource-document-toc-toc-title", description: "quarto-resource-document-toc-toc-title" }, lof: { $ref: "quarto-resource-document-toc-lof", description: "quarto-resource-document-toc-lof" }, lot: { $ref: "quarto-resource-document-toc-lot", description: "quarto-resource-document-toc-lot" }, search: { $ref: "quarto-resource-document-website-search", description: "quarto-resource-document-website-search" }, aliases: { $ref: "quarto-resource-document-website-aliases", description: "quarto-resource-document-website-aliases" }, image: { $ref: "quarto-resource-document-website-image", description: "quarto-resource-document-website-image" }, "image-height": { $ref: "quarto-resource-document-website-image-height", description: "quarto-resource-document-website-image-height" }, "image-width": { $ref: "quarto-resource-document-website-image-width", description: "quarto-resource-document-website-image-width" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|footnotes_hover|footnotesHover|links_as_notes|linksAsNotes|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|track_changes|trackChanges|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|latex_auto_mk|latexAutoMk|latex_auto_install|latexAutoInstall|latex_min_runs|latexMinRuns|latex_max_runs|latexMaxRuns|latex_clean|latexClean|latex_makeindex|latexMakeindex|latex_makeindex_opts|latexMakeindexOpts|latex_tlmgr_opts|latexTlmgrOpts|latex_output_dir|latexOutputDir|page_layout|pageLayout|page_width|pageWidth|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|reveal_js_url|revealJsUrl|s5_url|s5Url|slidy_url|slidyUrl|slideous_url|slideousUrl|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_meta|titleMeta|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|reference_doc|referenceDoc|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|section_titles|sectionTitles|markdown_headings|markdownHeadings|keep_yaml|keepYaml|ipynb_output|ipynbOutput|citations_hover|citationsHover|citation_location|citationLocation|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|keep_tex|keepTex|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|auto_stretch|autoStretch|min_scale|minScale|max_scale|maxScale|disable_layout|disableLayout|preview_links|previewLinks|auto_play_media|autoPlayMedia|preload_iframes|preloadIframes|view_distance|viewDistance|mobile_view_distance|mobileViewDistance|parallax_background_image|parallaxBackgroundImage|parallax_background_size|parallaxBackgroundSize|parallax_background_horizontal|parallaxBackgroundHorizontal|parallax_background_vertical|parallaxBackgroundVertical|navigation_mode|navigationMode|mouse_wheel|mouseWheel|hide_inactive_cursor|hideInactiveCursor|hide_cursor_time|hideCursorTime|controls_layout|controlsLayout|controls_tutorial|controlsTutorial|controls_back_arrows|controlsBackArrows|auto_slide|autoSlide|auto_slide_stoppable|autoSlideStoppable|auto_slide_method|autoSlideMethod|default_timing|defaultTiming|hash_type|hashType|hash_one_based_index|hashOneBasedIndex|respond_to_hash_changes|respondToHashChanges|fragment_in_url|fragmentInUrl|slide_tone|slideTone|pdf_separate_fragments|pdfSeparateFragments|pdf_page_height_offset|pdfPageHeightOffset|transition_speed|transitionSpeed|background_transition|backgroundTransition|auto_animate|autoAnimate|auto_animate_easing|autoAnimateEasing|auto_animate_duration|autoAnimateDuration|auto_animate_unmatched|autoAnimateUnmatched|auto_animate_styles|autoAnimateStyles|slide_level|slideLevel|slide_number|slideNumber|show_slide_number|showSlideNumber|title_slide_attributes|titleSlideAttributes|center_title_slide|centerTitleSlide|show_notes|showNotes|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))" } }, { $ref: "front-matter-execute", description: "be a front-matter-execute object" }], description: "be all of: a Quarto YAML front matter object, an object, a front-matter-execute object" }], description: "be exactly one of: the null value, all of: a Quarto YAML front matter object, an object, a front-matter-execute object", $id: "front-matter" }, oq = new RegExp("^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|footnotes_hover|footnotesHover|links_as_notes|linksAsNotes|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|track_changes|trackChanges|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|latex_auto_mk|latexAutoMk|latex_auto_install|latexAutoInstall|latex_min_runs|latexMinRuns|latex_max_runs|latexMaxRuns|latex_clean|latexClean|latex_makeindex|latexMakeindex|latex_makeindex_opts|latexMakeindexOpts|latex_tlmgr_opts|latexTlmgrOpts|latex_output_dir|latexOutputDir|page_layout|pageLayout|page_width|pageWidth|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|reveal_js_url|revealJsUrl|s5_url|s5Url|slidy_url|slidyUrl|slideous_url|slideousUrl|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_meta|titleMeta|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|reference_doc|referenceDoc|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|section_titles|sectionTitles|markdown_headings|markdownHeadings|keep_yaml|keepYaml|ipynb_output|ipynbOutput|citations_hover|citationsHover|citation_location|citationLocation|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|keep_tex|keepTex|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|auto_stretch|autoStretch|min_scale|minScale|max_scale|maxScale|disable_layout|disableLayout|preview_links|previewLinks|auto_play_media|autoPlayMedia|preload_iframes|preloadIframes|view_distance|viewDistance|mobile_view_distance|mobileViewDistance|parallax_background_image|parallaxBackgroundImage|parallax_background_size|parallaxBackgroundSize|parallax_background_horizontal|parallaxBackgroundHorizontal|parallax_background_vertical|parallaxBackgroundVertical|navigation_mode|navigationMode|mouse_wheel|mouseWheel|hide_inactive_cursor|hideInactiveCursor|hide_cursor_time|hideCursorTime|controls_layout|controlsLayout|controls_tutorial|controlsTutorial|controls_back_arrows|controlsBackArrows|auto_slide|autoSlide|auto_slide_stoppable|autoSlideStoppable|auto_slide_method|autoSlideMethod|default_timing|defaultTiming|hash_type|hashType|hash_one_based_index|hashOneBasedIndex|respond_to_hash_changes|respondToHashChanges|fragment_in_url|fragmentInUrl|slide_tone|slideTone|pdf_separate_fragments|pdfSeparateFragments|pdf_page_height_offset|pdfPageHeightOffset|transition_speed|transitionSpeed|background_transition|backgroundTransition|auto_animate|autoAnimate|auto_animate_easing|autoAnimateEasing|auto_animate_duration|autoAnimateDuration|auto_animate_unmatched|autoAnimateUnmatched|auto_animate_styles|autoAnimateStyles|slide_level|slideLevel|slide_number|slideNumber|show_slide_number|showSlideNumber|title_slide_attributes|titleSlideAttributes|center_title_slide|centerTitleSlide|show_notes|showNotes|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))", "u");
  function _m(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = false, d = null, u = r;
    if (l !== null) {
      let h = { instancePath: o, schemaPath: "#/oneOf/0/type", keyword: "type", params: { type: "null", schema: yo.oneOf[0] }, message: "must be null", schema: yo.oneOf[0].type, parentSchema: yo.oneOf[0], data: l };
      e === null ? e = [h] : e.push(h), r++;
    }
    let y = r;
    var g = u === r;
    g && (c = true, d = 0);
    let x = r, D = r, $ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      if (l.execute !== void 0) {
        let h = l.execute, f = r;
        si(h, { instancePath: o + "/execute", parentData: l, parentDataProperty: "execute", rootData: s }) || (e = e === null ? si.errors : e.concat(si.errors), r = e.length);
        var m = f === r;
      }
      if (l.format !== void 0) {
        let h = l.format, f = r;
        _u(h, { instancePath: o + "/format", parentData: l, parentDataProperty: "format", rootData: s }) || (e = e === null ? _u.errors : e.concat(_u.errors), r = e.length);
        var m = f === r;
      }
    } else {
      let h = { instancePath: o, schemaPath: "#/oneOf/1/allOf/0/type", keyword: "type", params: { type: "object", schema: yo.oneOf[1].allOf[0] }, message: "must be object", schema: yo.oneOf[1].allOf[0].type, parentSchema: yo.oneOf[1].allOf[0], data: l };
      e === null ? e = [h] : e.push(h), r++;
    }
    var R = D === r;
    let j = r, V = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let h in l) {
        let f = r, ft = r;
        if (typeof h == "string") {
          if (!oq.test(h)) {
            let Cr = { instancePath: o, schemaPath: "#/oneOf/1/allOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|footnotes_hover|footnotesHover|links_as_notes|linksAsNotes|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|track_changes|trackChanges|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|latex_auto_mk|latexAutoMk|latex_auto_install|latexAutoInstall|latex_min_runs|latexMinRuns|latex_max_runs|latexMaxRuns|latex_clean|latexClean|latex_makeindex|latexMakeindex|latex_makeindex_opts|latexMakeindexOpts|latex_tlmgr_opts|latexTlmgrOpts|latex_output_dir|latexOutputDir|page_layout|pageLayout|page_width|pageWidth|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|reveal_js_url|revealJsUrl|s5_url|s5Url|slidy_url|slidyUrl|slideous_url|slideousUrl|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_meta|titleMeta|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|reference_doc|referenceDoc|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|section_titles|sectionTitles|markdown_headings|markdownHeadings|keep_yaml|keepYaml|ipynb_output|ipynbOutput|citations_hover|citationsHover|citation_location|citationLocation|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|keep_tex|keepTex|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|auto_stretch|autoStretch|min_scale|minScale|max_scale|maxScale|disable_layout|disableLayout|preview_links|previewLinks|auto_play_media|autoPlayMedia|preload_iframes|preloadIframes|view_distance|viewDistance|mobile_view_distance|mobileViewDistance|parallax_background_image|parallaxBackgroundImage|parallax_background_size|parallaxBackgroundSize|parallax_background_horizontal|parallaxBackgroundHorizontal|parallax_background_vertical|parallaxBackgroundVertical|navigation_mode|navigationMode|mouse_wheel|mouseWheel|hide_inactive_cursor|hideInactiveCursor|hide_cursor_time|hideCursorTime|controls_layout|controlsLayout|controls_tutorial|controlsTutorial|controls_back_arrows|controlsBackArrows|auto_slide|autoSlide|auto_slide_stoppable|autoSlideStoppable|auto_slide_method|autoSlideMethod|default_timing|defaultTiming|hash_type|hashType|hash_one_based_index|hashOneBasedIndex|respond_to_hash_changes|respondToHashChanges|fragment_in_url|fragmentInUrl|slide_tone|slideTone|pdf_separate_fragments|pdfSeparateFragments|pdf_page_height_offset|pdfPageHeightOffset|transition_speed|transitionSpeed|background_transition|backgroundTransition|auto_animate|autoAnimate|auto_animate_easing|autoAnimateEasing|auto_animate_duration|autoAnimateDuration|auto_animate_unmatched|autoAnimateUnmatched|auto_animate_styles|autoAnimateStyles|slide_level|slideLevel|slide_number|slideNumber|show_slide_number|showSlideNumber|title_slide_attributes|titleSlideAttributes|center_title_slide|centerTitleSlide|show_notes|showNotes|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))", schema: yo.oneOf[1].allOf[1].propertyNames }, message: 'must match pattern "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|footnotes_hover|footnotesHover|links_as_notes|linksAsNotes|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|track_changes|trackChanges|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|latex_auto_mk|latexAutoMk|latex_auto_install|latexAutoInstall|latex_min_runs|latexMinRuns|latex_max_runs|latexMaxRuns|latex_clean|latexClean|latex_makeindex|latexMakeindex|latex_makeindex_opts|latexMakeindexOpts|latex_tlmgr_opts|latexTlmgrOpts|latex_output_dir|latexOutputDir|page_layout|pageLayout|page_width|pageWidth|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|reveal_js_url|revealJsUrl|s5_url|s5Url|slidy_url|slidyUrl|slideous_url|slideousUrl|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_meta|titleMeta|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|reference_doc|referenceDoc|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|section_titles|sectionTitles|markdown_headings|markdownHeadings|keep_yaml|keepYaml|ipynb_output|ipynbOutput|citations_hover|citationsHover|citation_location|citationLocation|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|keep_tex|keepTex|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|auto_stretch|autoStretch|min_scale|minScale|max_scale|maxScale|disable_layout|disableLayout|preview_links|previewLinks|auto_play_media|autoPlayMedia|preload_iframes|preloadIframes|view_distance|viewDistance|mobile_view_distance|mobileViewDistance|parallax_background_image|parallaxBackgroundImage|parallax_background_size|parallaxBackgroundSize|parallax_background_horizontal|parallaxBackgroundHorizontal|parallax_background_vertical|parallaxBackgroundVertical|navigation_mode|navigationMode|mouse_wheel|mouseWheel|hide_inactive_cursor|hideInactiveCursor|hide_cursor_time|hideCursorTime|controls_layout|controlsLayout|controls_tutorial|controlsTutorial|controls_back_arrows|controlsBackArrows|auto_slide|autoSlide|auto_slide_stoppable|autoSlideStoppable|auto_slide_method|autoSlideMethod|default_timing|defaultTiming|hash_type|hashType|hash_one_based_index|hashOneBasedIndex|respond_to_hash_changes|respondToHashChanges|fragment_in_url|fragmentInUrl|slide_tone|slideTone|pdf_separate_fragments|pdfSeparateFragments|pdf_page_height_offset|pdfPageHeightOffset|transition_speed|transitionSpeed|background_transition|backgroundTransition|auto_animate|autoAnimate|auto_animate_easing|autoAnimateEasing|auto_animate_duration|autoAnimateDuration|auto_animate_unmatched|autoAnimateUnmatched|auto_animate_styles|autoAnimateStyles|slide_level|slideLevel|slide_number|slideNumber|show_slide_number|showSlideNumber|title_slide_attributes|titleSlideAttributes|center_title_slide|centerTitleSlide|show_notes|showNotes|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))"', schema: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|fig_align|figAlign|fig_env|figEnv|fig_pos|figPos|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_colwidths|tblColwidths|abstract_title|abstractTitle|code_copy|codeCopy|code_link|codeLink|code_tools|codeTools|highlight_style|highlightStyle|syntax_definition|syntaxDefinition|syntax_definitions|syntaxDefinitions|indented_code_classes|indentedCodeClasses|belongs_to_collection|belongsToCollection|group_position|groupPosition|page_progression_direction|pageProgressionDirection|epub_metadata|epubMetadata|epub_subdirectory|epubSubdirectory|epub_fonts|epubFonts|epub_chapter_level|epubChapterLevel|epub_cover_image|epubCoverImage|daemon_restart|daemonRestart|fig_width|figWidth|fig_height|figHeight|fig_format|figFormat|fig_dpi|figDpi|fig_responsive|figResponsive|cjkmainfont|cjkmainfont|cjkoptions|cjkoptions|footnotes_hover|footnotesHover|links_as_notes|linksAsNotes|reference_location|referenceLocation|input_file|inputFile|input_files|inputFiles|request_headers|requestHeaders|fail_if_warnings|failIfWarnings|dump_args|dumpArgs|ignore_args|ignoreArgs|file_scope|fileScope|data_dir|dataDir|log_file|logFile|track_changes|trackChanges|strip_empty_paragraphs|stripEmptyParagraphs|keep_source|keepSource|keep_hidden|keepHidden|prefer_html|preferHtml|output_divs|outputDivs|merge_includes|mergeIncludes|header_includes|headerIncludes|include_before|includeBefore|include_after|includeAfter|include_before_body|includeBeforeBody|include_after_body|includeAfterBody|include_in_header|includeInHeader|metadata_file|metadataFile|metadata_files|metadataFiles|latex_auto_mk|latexAutoMk|latex_auto_install|latexAutoInstall|latex_min_runs|latexMinRuns|latex_max_runs|latexMaxRuns|latex_clean|latexClean|latex_makeindex|latexMakeindex|latex_makeindex_opts|latexMakeindexOpts|latex_tlmgr_opts|latexTlmgrOpts|latex_output_dir|latexOutputDir|page_layout|pageLayout|page_width|pageWidth|max_width|maxWidth|margin_left|marginLeft|margin_right|marginRight|margin_top|marginTop|margin_bottom|marginBottom|block_headings|blockHeadings|reveal_js_url|revealJsUrl|s5_url|s5Url|slidy_url|slidyUrl|slideous_url|slideousUrl|link_external_icon|linkExternalIcon|link_external_newwindow|linkExternalNewwindow|link_external_filter|linkExternalFilter|title_meta|titleMeta|title_prefix|titlePrefix|description_meta|descriptionMeta|author_meta|authorMeta|date_meta|dateMeta|number_sections|numberSections|number_depth|numberDepth|number_offset|numberOffset|shift_heading_level_by|shiftHeadingLevelBy|top_level_division|topLevelDivision|reference_doc|referenceDoc|document_css|documentCss|anchor_sections|anchorSections|html_math_method|htmlMathMethod|section_divs|sectionDivs|identifier_prefix|identifierPrefix|email_obfuscation|emailObfuscation|html_q_tags|htmlQTags|pdf_engine|pdfEngine|pdf_engine_opt|pdfEngineOpt|section_titles|sectionTitles|markdown_headings|markdownHeadings|keep_yaml|keepYaml|ipynb_output|ipynbOutput|citations_hover|citationsHover|citation_location|citationLocation|cite_method|citeMethod|biblio_style|biblioStyle|biblio_title|biblioTitle|citation_abbreviations|citationAbbreviations|output_file|outputFile|output_ext|outputExt|self_contained|selfContained|self_contained_math|selfContainedMath|keep_md|keepMd|keep_ipynb|keepIpynb|keep_tex|keepTex|extract_media|extractMedia|resource_path|resourcePath|default_image_extension|defaultImageExtension|auto_stretch|autoStretch|min_scale|minScale|max_scale|maxScale|disable_layout|disableLayout|preview_links|previewLinks|auto_play_media|autoPlayMedia|preload_iframes|preloadIframes|view_distance|viewDistance|mobile_view_distance|mobileViewDistance|parallax_background_image|parallaxBackgroundImage|parallax_background_size|parallaxBackgroundSize|parallax_background_horizontal|parallaxBackgroundHorizontal|parallax_background_vertical|parallaxBackgroundVertical|navigation_mode|navigationMode|mouse_wheel|mouseWheel|hide_inactive_cursor|hideInactiveCursor|hide_cursor_time|hideCursorTime|controls_layout|controlsLayout|controls_tutorial|controlsTutorial|controls_back_arrows|controlsBackArrows|auto_slide|autoSlide|auto_slide_stoppable|autoSlideStoppable|auto_slide_method|autoSlideMethod|default_timing|defaultTiming|hash_type|hashType|hash_one_based_index|hashOneBasedIndex|respond_to_hash_changes|respondToHashChanges|fragment_in_url|fragmentInUrl|slide_tone|slideTone|pdf_separate_fragments|pdfSeparateFragments|pdf_page_height_offset|pdfPageHeightOffset|transition_speed|transitionSpeed|background_transition|backgroundTransition|auto_animate|autoAnimate|auto_animate_easing|autoAnimateEasing|auto_animate_duration|autoAnimateDuration|auto_animate_unmatched|autoAnimateUnmatched|auto_animate_styles|autoAnimateStyles|slide_level|slideLevel|slide_number|slideNumber|show_slide_number|showSlideNumber|title_slide_attributes|titleSlideAttributes|center_title_slide|centerTitleSlide|show_notes|showNotes|tab_stop|tabStop|preserve_tabs|preserveTabs|strip_comments|stripComments|table_of_contents|tableOfContents|toc_depth|tocDepth|toc_location|tocLocation|toc_title|tocTitle|image_height|imageHeight|image_width|imageWidth))", parentSchema: yo.oneOf[1].allOf[1].propertyNames, data: h, propertyName: h };
            e === null ? e = [Cr] : e.push(Cr), r++;
          }
        } else {
          let Cr = { instancePath: o, schemaPath: "#/oneOf/1/allOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: yo.oneOf[1].allOf[1].propertyNames }, message: "must be string", schema: yo.oneOf[1].allOf[1].propertyNames.type, parentSchema: yo.oneOf[1].allOf[1].propertyNames, data: h, propertyName: h };
          e === null ? e = [Cr] : e.push(Cr), r++;
        }
        var se = f === r;
        if (!se) {
          let Cr = { instancePath: o, schemaPath: "#/oneOf/1/allOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: h, schema: yo.oneOf[1].allOf[1] }, message: "property name must be valid", schema: yo.oneOf[1].allOf[1].propertyNames, parentSchema: yo.oneOf[1].allOf[1], data: l };
          e === null ? e = [Cr] : e.push(Cr), r++;
        }
      }
      if (l.eval !== void 0) {
        let h = l.eval, f = r;
        ue(h, { instancePath: o + "/eval", parentData: l, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
        var P = f === r;
      }
      if (l.echo !== void 0) {
        let h = l.echo, f = r;
        de(h, { instancePath: o + "/echo", parentData: l, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
        var P = f === r;
      }
      if (l["code-fold"] !== void 0) {
        let h = l["code-fold"], f = r;
        Fa(h, { instancePath: o + "/code-fold", parentData: l, parentDataProperty: "code-fold", rootData: s }) || (e = e === null ? Fa.errors : e.concat(Fa.errors), r = e.length);
        var P = f === r;
      }
      if (l["code-summary"] !== void 0) {
        let h = l["code-summary"], f = r;
        Aa(h, { instancePath: o + "/code-summary", parentData: l, parentDataProperty: "code-summary", rootData: s }) || (e = e === null ? Aa.errors : e.concat(Aa.errors), r = e.length);
        var P = f === r;
      }
      if (l["code-overflow"] !== void 0) {
        let h = l["code-overflow"], f = r;
        Ba(h, { instancePath: o + "/code-overflow", parentData: l, parentDataProperty: "code-overflow", rootData: s }) || (e = e === null ? Ba.errors : e.concat(Ba.errors), r = e.length);
        var P = f === r;
      }
      if (l["code-line-numbers"] !== void 0) {
        let h = l["code-line-numbers"], f = r;
        ua(h, { instancePath: o + "/code-line-numbers", parentData: l, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
        var P = f === r;
      }
      if (l["fig-align"] !== void 0) {
        let h = l["fig-align"], f = r;
        la(h, { instancePath: o + "/fig-align", parentData: l, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
        var P = f === r;
      }
      if (l["fig-env"] !== void 0) {
        let h = l["fig-env"], f = r;
        Zn(h, { instancePath: o + "/fig-env", parentData: l, parentDataProperty: "fig-env", rootData: s }) || (e = e === null ? Zn.errors : e.concat(Zn.errors), r = e.length);
        var P = f === r;
      }
      if (l["fig-pos"] !== void 0) {
        let h = l["fig-pos"], f = r;
        eo(h, { instancePath: o + "/fig-pos", parentData: l, parentDataProperty: "fig-pos", rootData: s }) || (e = e === null ? eo.errors : e.concat(eo.errors), r = e.length);
        var P = f === r;
      }
      if (l["cap-location"] !== void 0) {
        let h = l["cap-location"], f = r;
        ja(h, { instancePath: o + "/cap-location", parentData: l, parentDataProperty: "cap-location", rootData: s }) || (e = e === null ? ja.errors : e.concat(ja.errors), r = e.length);
        var P = f === r;
      }
      if (l["fig-cap-location"] !== void 0) {
        let h = l["fig-cap-location"], f = r;
        Ea(h, { instancePath: o + "/fig-cap-location", parentData: l, parentDataProperty: "fig-cap-location", rootData: s }) || (e = e === null ? Ea.errors : e.concat(Ea.errors), r = e.length);
        var P = f === r;
      }
      if (l["tbl-cap-location"] !== void 0) {
        let h = l["tbl-cap-location"], f = r;
        Na(h, { instancePath: o + "/tbl-cap-location", parentData: l, parentDataProperty: "tbl-cap-location", rootData: s }) || (e = e === null ? Na.errors : e.concat(Na.errors), r = e.length);
        var P = f === r;
      }
      if (l["tbl-colwidths"] !== void 0) {
        let h = l["tbl-colwidths"], f = r;
        be(h, { instancePath: o + "/tbl-colwidths", parentData: l, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
        var P = f === r;
      }
      if (l.output !== void 0) {
        let h = l.output, f = r;
        fe(h, { instancePath: o + "/output", parentData: l, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
        var P = f === r;
      }
      if (l.warning !== void 0) {
        let h = l.warning, f = r;
        me(h, { instancePath: o + "/warning", parentData: l, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
        var P = f === r;
      }
      if (l.error !== void 0) {
        let h = l.error, f = r;
        ge(h, { instancePath: o + "/error", parentData: l, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
        var P = f === r;
      }
      if (l.include !== void 0) {
        let h = l.include, f = r;
        he(h, { instancePath: o + "/include", parentData: l, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
        var P = f === r;
      }
      if (l.title !== void 0) {
        let h = l.title, f = r;
        Ee(h, { instancePath: o + "/title", parentData: l, parentDataProperty: "title", rootData: s }) || (e = e === null ? Ee.errors : e.concat(Ee.errors), r = e.length);
        var P = f === r;
      }
      if (l.subtitle !== void 0) {
        let h = l.subtitle, f = r;
        ha(h, { instancePath: o + "/subtitle", parentData: l, parentDataProperty: "subtitle", rootData: s }) || (e = e === null ? ha.errors : e.concat(ha.errors), r = e.length);
        var P = f === r;
      }
      if (l.date !== void 0) {
        let h = l.date, f = r;
        Ne(h, { instancePath: o + "/date", parentData: l, parentDataProperty: "date", rootData: s }) || (e = e === null ? Ne.errors : e.concat(Ne.errors), r = e.length);
        var P = f === r;
      }
      if (l.author !== void 0) {
        let h = l.author, f = r;
        Ce(h, { instancePath: o + "/author", parentData: l, parentDataProperty: "author", rootData: s }) || (e = e === null ? Ce.errors : e.concat(Ce.errors), r = e.length);
        var P = f === r;
      }
      if (l.affiliation !== void 0) {
        let h = l.affiliation, f = r;
        ko(h, { instancePath: o + "/affiliation", parentData: l, parentDataProperty: "affiliation", rootData: s }) || (e = e === null ? ko.errors : e.concat(ko.errors), r = e.length);
        var P = f === r;
      }
      if (l.copyright !== void 0) {
        let h = l.copyright, f = r;
        Do(h, { instancePath: o + "/copyright", parentData: l, parentDataProperty: "copyright", rootData: s }) || (e = e === null ? Do.errors : e.concat(Do.errors), r = e.length);
        var P = f === r;
      }
      if (l.article !== void 0) {
        let h = l.article, f = r;
        wo(h, { instancePath: o + "/article", parentData: l, parentDataProperty: "article", rootData: s }) || (e = e === null ? wo.errors : e.concat(wo.errors), r = e.length);
        var P = f === r;
      }
      if (l.journal !== void 0) {
        let h = l.journal, f = r;
        $o(h, { instancePath: o + "/journal", parentData: l, parentDataProperty: "journal", rootData: s }) || (e = e === null ? $o.errors : e.concat($o.errors), r = e.length);
        var P = f === r;
      }
      if (l.institute !== void 0) {
        let h = l.institute, f = r;
        ro(h, { instancePath: o + "/institute", parentData: l, parentDataProperty: "institute", rootData: s }) || (e = e === null ? ro.errors : e.concat(ro.errors), r = e.length);
        var P = f === r;
      }
      if (l.abstract !== void 0) {
        let h = l.abstract, f = r;
        ga(h, { instancePath: o + "/abstract", parentData: l, parentDataProperty: "abstract", rootData: s }) || (e = e === null ? ga.errors : e.concat(ga.errors), r = e.length);
        var P = f === r;
      }
      if (l["abstract-title"] !== void 0) {
        let h = l["abstract-title"], f = r;
        to(h, { instancePath: o + "/abstract-title", parentData: l, parentDataProperty: "abstract-title", rootData: s }) || (e = e === null ? to.errors : e.concat(to.errors), r = e.length);
        var P = f === r;
      }
      if (l.notes !== void 0) {
        let h = l.notes, f = r;
        So(h, { instancePath: o + "/notes", parentData: l, parentDataProperty: "notes", rootData: s }) || (e = e === null ? So.errors : e.concat(So.errors), r = e.length);
        var P = f === r;
      }
      if (l.tags !== void 0) {
        let h = l.tags, f = r;
        Fo(h, { instancePath: o + "/tags", parentData: l, parentDataProperty: "tags", rootData: s }) || (e = e === null ? Fo.errors : e.concat(Fo.errors), r = e.length);
        var P = f === r;
      }
      if (l.doi !== void 0) {
        let h = l.doi, f = r;
        Jo(h, { instancePath: o + "/doi", parentData: l, parentDataProperty: "doi", rootData: s }) || (e = e === null ? Jo.errors : e.concat(Jo.errors), r = e.length);
        var P = f === r;
      }
      if (l.thanks !== void 0) {
        let h = l.thanks, f = r;
        Qo(h, { instancePath: o + "/thanks", parentData: l, parentDataProperty: "thanks", rootData: s }) || (e = e === null ? Qo.errors : e.concat(Qo.errors), r = e.length);
        var P = f === r;
      }
      if (l["code-copy"] !== void 0) {
        let h = l["code-copy"], f = r;
        ln(h, { instancePath: o + "/code-copy", parentData: l, parentDataProperty: "code-copy", rootData: s }) || (e = e === null ? ln.errors : e.concat(ln.errors), r = e.length);
        var P = f === r;
      }
      if (l["code-link"] !== void 0) {
        let h = l["code-link"], f = r;
        On(h, { instancePath: o + "/code-link", parentData: l, parentDataProperty: "code-link", rootData: s }) || (e = e === null ? On.errors : e.concat(On.errors), r = e.length);
        var P = f === r;
      }
      if (l["code-tools"] !== void 0) {
        let h = l["code-tools"], f = r;
        Go(h, { instancePath: o + "/code-tools", parentData: l, parentDataProperty: "code-tools", rootData: s }) || (e = e === null ? Go.errors : e.concat(Go.errors), r = e.length);
        var P = f === r;
      }
      if (l["highlight-style"] !== void 0) {
        let h = l["highlight-style"], f = r;
        _a(h, { instancePath: o + "/highlight-style", parentData: l, parentDataProperty: "highlight-style", rootData: s }) || (e = e === null ? _a.errors : e.concat(_a.errors), r = e.length);
        var P = f === r;
      }
      if (l["syntax-definition"] !== void 0) {
        let h = l["syntax-definition"], f = r;
        va(h, { instancePath: o + "/syntax-definition", parentData: l, parentDataProperty: "syntax-definition", rootData: s }) || (e = e === null ? va.errors : e.concat(va.errors), r = e.length);
        var P = f === r;
      }
      if (l["syntax-definitions"] !== void 0) {
        let h = l["syntax-definitions"], f = r;
        qa(h, { instancePath: o + "/syntax-definitions", parentData: l, parentDataProperty: "syntax-definitions", rootData: s }) || (e = e === null ? qa.errors : e.concat(qa.errors), r = e.length);
        var P = f === r;
      }
      if (l.listings !== void 0) {
        let h = l.listings, f = r;
        Yo(h, { instancePath: o + "/listings", parentData: l, parentDataProperty: "listings", rootData: s }) || (e = e === null ? Yo.errors : e.concat(Yo.errors), r = e.length);
        var P = f === r;
      }
      if (l["indented-code-classes"] !== void 0) {
        let h = l["indented-code-classes"], f = r;
        Oa(h, { instancePath: o + "/indented-code-classes", parentData: l, parentDataProperty: "indented-code-classes", rootData: s }) || (e = e === null ? Oa.errors : e.concat(Oa.errors), r = e.length);
        var P = f === r;
      }
      if (l.fontcolor !== void 0) {
        let h = l.fontcolor, f = r;
        Pn(h, { instancePath: o + "/fontcolor", parentData: l, parentDataProperty: "fontcolor", rootData: s }) || (e = e === null ? Pn.errors : e.concat(Pn.errors), r = e.length);
        var P = f === r;
      }
      if (l.linkcolor !== void 0) {
        let h = l.linkcolor, f = r;
        Ua(h, { instancePath: o + "/linkcolor", parentData: l, parentDataProperty: "linkcolor", rootData: s }) || (e = e === null ? Ua.errors : e.concat(Ua.errors), r = e.length);
        var P = f === r;
      }
      if (l.monobackgroundcolor !== void 0) {
        let h = l.monobackgroundcolor, f = r;
        xn(h, { instancePath: o + "/monobackgroundcolor", parentData: l, parentDataProperty: "monobackgroundcolor", rootData: s }) || (e = e === null ? xn.errors : e.concat(xn.errors), r = e.length);
        var P = f === r;
      }
      if (l.backgroundcolor !== void 0) {
        let h = l.backgroundcolor, f = r;
        kn(h, { instancePath: o + "/backgroundcolor", parentData: l, parentDataProperty: "backgroundcolor", rootData: s }) || (e = e === null ? kn.errors : e.concat(kn.errors), r = e.length);
        var P = f === r;
      }
      if (l.filecolor !== void 0) {
        let h = l.filecolor, f = r;
        Xo(h, { instancePath: o + "/filecolor", parentData: l, parentDataProperty: "filecolor", rootData: s }) || (e = e === null ? Xo.errors : e.concat(Xo.errors), r = e.length);
        var P = f === r;
      }
      if (l.citecolor !== void 0) {
        let h = l.citecolor, f = r;
        Zo(h, { instancePath: o + "/citecolor", parentData: l, parentDataProperty: "citecolor", rootData: s }) || (e = e === null ? Zo.errors : e.concat(Zo.errors), r = e.length);
        var P = f === r;
      }
      if (l.urlcolor !== void 0) {
        let h = l.urlcolor, f = r;
        es(h, { instancePath: o + "/urlcolor", parentData: l, parentDataProperty: "urlcolor", rootData: s }) || (e = e === null ? es.errors : e.concat(es.errors), r = e.length);
        var P = f === r;
      }
      if (l.toccolor !== void 0) {
        let h = l.toccolor, f = r;
        rs(h, { instancePath: o + "/toccolor", parentData: l, parentDataProperty: "toccolor", rootData: s }) || (e = e === null ? rs.errors : e.concat(rs.errors), r = e.length);
        var P = f === r;
      }
      if (l.colorlinks !== void 0) {
        let h = l.colorlinks, f = r;
        ts(h, { instancePath: o + "/colorlinks", parentData: l, parentDataProperty: "colorlinks", rootData: s }) || (e = e === null ? ts.errors : e.concat(ts.errors), r = e.length);
        var P = f === r;
      }
      if (l.contrastcolor !== void 0) {
        let h = l.contrastcolor, f = r;
        vc(h, { instancePath: o + "/contrastcolor", parentData: l, parentDataProperty: "contrastcolor", rootData: s }) || (e = e === null ? vc.errors : e.concat(vc.errors), r = e.length);
        var P = f === r;
      }
      if (l.comments !== void 0) {
        let h = l.comments, f = r;
        Dn(h, { instancePath: o + "/comments", parentData: l, parentDataProperty: "comments", rootData: s }) || (e = e === null ? Dn.errors : e.concat(Dn.errors), r = e.length);
        var P = f === r;
      }
      if (l.crossref !== void 0) {
        let h = l.crossref, f = r;
        Ie(h, { instancePath: o + "/crossref", parentData: l, parentDataProperty: "crossref", rootData: s }) || (e = e === null ? Ie.errors : e.concat(Ie.errors), r = e.length);
        var P = f === r;
      }
      if (l.identifier !== void 0) {
        let h = l.identifier, f = r;
        as(h, { instancePath: o + "/identifier", parentData: l, parentDataProperty: "identifier", rootData: s }) || (e = e === null ? as.errors : e.concat(as.errors), r = e.length);
        var P = f === r;
      }
      if (l.creator !== void 0) {
        let h = l.creator, f = r;
        ns(h, { instancePath: o + "/creator", parentData: l, parentDataProperty: "creator", rootData: s }) || (e = e === null ? ns.errors : e.concat(ns.errors), r = e.length);
        var P = f === r;
      }
      if (l.contributor !== void 0) {
        let h = l.contributor, f = r;
        os(h, { instancePath: o + "/contributor", parentData: l, parentDataProperty: "contributor", rootData: s }) || (e = e === null ? os.errors : e.concat(os.errors), r = e.length);
        var P = f === r;
      }
      if (l.subject !== void 0) {
        let h = l.subject, f = r;
        fo(h, { instancePath: o + "/subject", parentData: l, parentDataProperty: "subject", rootData: s }) || (e = e === null ? fo.errors : e.concat(fo.errors), r = e.length);
        var P = f === r;
      }
      if (l.type !== void 0) {
        let h = l.type, f = r;
        ss(h, { instancePath: o + "/type", parentData: l, parentDataProperty: "type", rootData: s }) || (e = e === null ? ss.errors : e.concat(ss.errors), r = e.length);
        var P = f === r;
      }
      if (l.relation !== void 0) {
        let h = l.relation, f = r;
        is(h, { instancePath: o + "/relation", parentData: l, parentDataProperty: "relation", rootData: s }) || (e = e === null ? is.errors : e.concat(is.errors), r = e.length);
        var P = f === r;
      }
      if (l.coverage !== void 0) {
        let h = l.coverage, f = r;
        ls(h, { instancePath: o + "/coverage", parentData: l, parentDataProperty: "coverage", rootData: s }) || (e = e === null ? ls.errors : e.concat(ls.errors), r = e.length);
        var P = f === r;
      }
      if (l.rights !== void 0) {
        let h = l.rights, f = r;
        cs(h, { instancePath: o + "/rights", parentData: l, parentDataProperty: "rights", rootData: s }) || (e = e === null ? cs.errors : e.concat(cs.errors), r = e.length);
        var P = f === r;
      }
      if (l["belongs-to-collection"] !== void 0) {
        let h = l["belongs-to-collection"], f = r;
        ps(h, { instancePath: o + "/belongs-to-collection", parentData: l, parentDataProperty: "belongs-to-collection", rootData: s }) || (e = e === null ? ps.errors : e.concat(ps.errors), r = e.length);
        var P = f === r;
      }
      if (l["group-position"] !== void 0) {
        let h = l["group-position"], f = r;
        us(h, { instancePath: o + "/group-position", parentData: l, parentDataProperty: "group-position", rootData: s }) || (e = e === null ? us.errors : e.concat(us.errors), r = e.length);
        var P = f === r;
      }
      if (l["page-progression-direction"] !== void 0) {
        let h = l["page-progression-direction"], f = r;
        ds(h, { instancePath: o + "/page-progression-direction", parentData: l, parentDataProperty: "page-progression-direction", rootData: s }) || (e = e === null ? ds.errors : e.concat(ds.errors), r = e.length);
        var P = f === r;
      }
      if (l.ibooks !== void 0) {
        let h = l.ibooks, f = r;
        fs(h, { instancePath: o + "/ibooks", parentData: l, parentDataProperty: "ibooks", rootData: s }) || (e = e === null ? fs.errors : e.concat(fs.errors), r = e.length);
        var P = f === r;
      }
      if (l["epub-metadata"] !== void 0) {
        let h = l["epub-metadata"], f = r;
        ms(h, { instancePath: o + "/epub-metadata", parentData: l, parentDataProperty: "epub-metadata", rootData: s }) || (e = e === null ? ms.errors : e.concat(ms.errors), r = e.length);
        var P = f === r;
      }
      if (l["epub-subdirectory"] !== void 0) {
        let h = l["epub-subdirectory"], f = r;
        gs(h, { instancePath: o + "/epub-subdirectory", parentData: l, parentDataProperty: "epub-subdirectory", rootData: s }) || (e = e === null ? gs.errors : e.concat(gs.errors), r = e.length);
        var P = f === r;
      }
      if (l["epub-fonts"] !== void 0) {
        let h = l["epub-fonts"], f = r;
        hs(h, { instancePath: o + "/epub-fonts", parentData: l, parentDataProperty: "epub-fonts", rootData: s }) || (e = e === null ? hs.errors : e.concat(hs.errors), r = e.length);
        var P = f === r;
      }
      if (l["epub-chapter-level"] !== void 0) {
        let h = l["epub-chapter-level"], f = r;
        ys(h, { instancePath: o + "/epub-chapter-level", parentData: l, parentDataProperty: "epub-chapter-level", rootData: s }) || (e = e === null ? ys.errors : e.concat(ys.errors), r = e.length);
        var P = f === r;
      }
      if (l["epub-cover-image"] !== void 0) {
        let h = l["epub-cover-image"], f = r;
        bs(h, { instancePath: o + "/epub-cover-image", parentData: l, parentDataProperty: "epub-cover-image", rootData: s }) || (e = e === null ? bs.errors : e.concat(bs.errors), r = e.length);
        var P = f === r;
      }
      if (l.engine !== void 0) {
        let h = l.engine, f = r;
        qe(h, { instancePath: o + "/engine", parentData: l, parentDataProperty: "engine", rootData: s }) || (e = e === null ? qe.errors : e.concat(qe.errors), r = e.length);
        var P = f === r;
      }
      if (l.jupyter !== void 0) {
        let h = l.jupyter, f = r;
        Oe(h, { instancePath: o + "/jupyter", parentData: l, parentDataProperty: "jupyter", rootData: s }) || (e = e === null ? Oe.errors : e.concat(Oe.errors), r = e.length);
        var P = f === r;
      }
      if (l.knitr !== void 0) {
        let h = l.knitr, f = r;
        Pe(h, { instancePath: o + "/knitr", parentData: l, parentDataProperty: "knitr", rootData: s }) || (e = e === null ? Pe.errors : e.concat(Pe.errors), r = e.length);
        var P = f === r;
      }
      if (l.cache !== void 0) {
        let h = l.cache, f = r;
        xe(h, { instancePath: o + "/cache", parentData: l, parentDataProperty: "cache", rootData: s }) || (e = e === null ? xe.errors : e.concat(xe.errors), r = e.length);
        var P = f === r;
      }
      if (l.freeze !== void 0) {
        let h = l.freeze, f = r;
        ke(h, { instancePath: o + "/freeze", parentData: l, parentDataProperty: "freeze", rootData: s }) || (e = e === null ? ke.errors : e.concat(ke.errors), r = e.length);
        var P = f === r;
      }
      if (l.server !== void 0) {
        let h = l.server, f = r;
        De(h, { instancePath: o + "/server", parentData: l, parentDataProperty: "server", rootData: s }) || (e = e === null ? De.errors : e.concat(De.errors), r = e.length);
        var P = f === r;
      }
      if (l.daemon !== void 0) {
        let h = l.daemon, f = r;
        we(h, { instancePath: o + "/daemon", parentData: l, parentDataProperty: "daemon", rootData: s }) || (e = e === null ? we.errors : e.concat(we.errors), r = e.length);
        var P = f === r;
      }
      if (l["daemon-restart"] !== void 0) {
        let h = l["daemon-restart"], f = r;
        $e(h, { instancePath: o + "/daemon-restart", parentData: l, parentDataProperty: "daemon-restart", rootData: s }) || (e = e === null ? $e.errors : e.concat($e.errors), r = e.length);
        var P = f === r;
      }
      if (l.enabled !== void 0) {
        let h = l.enabled, f = r;
        Se(h, { instancePath: o + "/enabled", parentData: l, parentDataProperty: "enabled", rootData: s }) || (e = e === null ? Se.errors : e.concat(Se.errors), r = e.length);
        var P = f === r;
      }
      if (l.ipynb !== void 0) {
        let h = l.ipynb, f = r;
        Fe(h, { instancePath: o + "/ipynb", parentData: l, parentDataProperty: "ipynb", rootData: s }) || (e = e === null ? Fe.errors : e.concat(Fe.errors), r = e.length);
        var P = f === r;
      }
      if (l.debug !== void 0) {
        let h = l.debug, f = r;
        Ae(h, { instancePath: o + "/debug", parentData: l, parentDataProperty: "debug", rootData: s }) || (e = e === null ? Ae.errors : e.concat(Ae.errors), r = e.length);
        var P = f === r;
      }
      if (l["fig-width"] !== void 0) {
        let h = l["fig-width"], f = r;
        He(h, { instancePath: o + "/fig-width", parentData: l, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? He.errors : e.concat(He.errors), r = e.length);
        var P = f === r;
      }
      if (l["fig-height"] !== void 0) {
        let h = l["fig-height"], f = r;
        Le(h, { instancePath: o + "/fig-height", parentData: l, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? Le.errors : e.concat(Le.errors), r = e.length);
        var P = f === r;
      }
      if (l["fig-format"] !== void 0) {
        let h = l["fig-format"], f = r;
        Me(h, { instancePath: o + "/fig-format", parentData: l, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? Me.errors : e.concat(Me.errors), r = e.length);
        var P = f === r;
      }
      if (l["fig-dpi"] !== void 0) {
        let h = l["fig-dpi"], f = r;
        Te(h, { instancePath: o + "/fig-dpi", parentData: l, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? Te.errors : e.concat(Te.errors), r = e.length);
        var P = f === r;
      }
      if (l["fig-responsive"] !== void 0) {
        let h = l["fig-responsive"], f = r;
        cn(h, { instancePath: o + "/fig-responsive", parentData: l, parentDataProperty: "fig-responsive", rootData: s }) || (e = e === null ? cn.errors : e.concat(cn.errors), r = e.length);
        var P = f === r;
      }
      if (l.mainfont !== void 0) {
        let h = l.mainfont, f = r;
        Ja(h, { instancePath: o + "/mainfont", parentData: l, parentDataProperty: "mainfont", rootData: s }) || (e = e === null ? Ja.errors : e.concat(Ja.errors), r = e.length);
        var P = f === r;
      }
      if (l.monofont !== void 0) {
        let h = l.monofont, f = r;
        Qa(h, { instancePath: o + "/monofont", parentData: l, parentDataProperty: "monofont", rootData: s }) || (e = e === null ? Qa.errors : e.concat(Qa.errors), r = e.length);
        var P = f === r;
      }
      if (l.fontsize !== void 0) {
        let h = l.fontsize, f = r;
        Ga(h, { instancePath: o + "/fontsize", parentData: l, parentDataProperty: "fontsize", rootData: s }) || (e = e === null ? Ga.errors : e.concat(Ga.errors), r = e.length);
        var P = f === r;
      }
      if (l.fontenc !== void 0) {
        let h = l.fontenc, f = r;
        _s(h, { instancePath: o + "/fontenc", parentData: l, parentDataProperty: "fontenc", rootData: s }) || (e = e === null ? _s.errors : e.concat(_s.errors), r = e.length);
        var P = f === r;
      }
      if (l.fontfamily !== void 0) {
        let h = l.fontfamily, f = r;
        Bo(h, { instancePath: o + "/fontfamily", parentData: l, parentDataProperty: "fontfamily", rootData: s }) || (e = e === null ? Bo.errors : e.concat(Bo.errors), r = e.length);
        var P = f === r;
      }
      if (l.fontfamilyoptions !== void 0) {
        let h = l.fontfamilyoptions, f = r;
        vs(h, { instancePath: o + "/fontfamilyoptions", parentData: l, parentDataProperty: "fontfamilyoptions", rootData: s }) || (e = e === null ? vs.errors : e.concat(vs.errors), r = e.length);
        var P = f === r;
      }
      if (l.sansfont !== void 0) {
        let h = l.sansfont, f = r;
        qs(h, { instancePath: o + "/sansfont", parentData: l, parentDataProperty: "sansfont", rootData: s }) || (e = e === null ? qs.errors : e.concat(qs.errors), r = e.length);
        var P = f === r;
      }
      if (l.mathfont !== void 0) {
        let h = l.mathfont, f = r;
        Os(h, { instancePath: o + "/mathfont", parentData: l, parentDataProperty: "mathfont", rootData: s }) || (e = e === null ? Os.errors : e.concat(Os.errors), r = e.length);
        var P = f === r;
      }
      if (l.CJKmainfont !== void 0) {
        let h = l.CJKmainfont, f = r;
        Ps(h, { instancePath: o + "/CJKmainfont", parentData: l, parentDataProperty: "CJKmainfont", rootData: s }) || (e = e === null ? Ps.errors : e.concat(Ps.errors), r = e.length);
        var P = f === r;
      }
      if (l.mainfontoptions !== void 0) {
        let h = l.mainfontoptions, f = r;
        xs(h, { instancePath: o + "/mainfontoptions", parentData: l, parentDataProperty: "mainfontoptions", rootData: s }) || (e = e === null ? xs.errors : e.concat(xs.errors), r = e.length);
        var P = f === r;
      }
      if (l.sansfontoptions !== void 0) {
        let h = l.sansfontoptions, f = r;
        ks(h, { instancePath: o + "/sansfontoptions", parentData: l, parentDataProperty: "sansfontoptions", rootData: s }) || (e = e === null ? ks.errors : e.concat(ks.errors), r = e.length);
        var P = f === r;
      }
      if (l.monofontoptions !== void 0) {
        let h = l.monofontoptions, f = r;
        Ds(h, { instancePath: o + "/monofontoptions", parentData: l, parentDataProperty: "monofontoptions", rootData: s }) || (e = e === null ? Ds.errors : e.concat(Ds.errors), r = e.length);
        var P = f === r;
      }
      if (l.mathfontoptions !== void 0) {
        let h = l.mathfontoptions, f = r;
        ws(h, { instancePath: o + "/mathfontoptions", parentData: l, parentDataProperty: "mathfontoptions", rootData: s }) || (e = e === null ? ws.errors : e.concat(ws.errors), r = e.length);
        var P = f === r;
      }
      if (l.CJKoptions !== void 0) {
        let h = l.CJKoptions, f = r;
        $s(h, { instancePath: o + "/CJKoptions", parentData: l, parentDataProperty: "CJKoptions", rootData: s }) || (e = e === null ? $s.errors : e.concat($s.errors), r = e.length);
        var P = f === r;
      }
      if (l.microtypeoptions !== void 0) {
        let h = l.microtypeoptions, f = r;
        Ss(h, { instancePath: o + "/microtypeoptions", parentData: l, parentDataProperty: "microtypeoptions", rootData: s }) || (e = e === null ? Ss.errors : e.concat(Ss.errors), r = e.length);
        var P = f === r;
      }
      if (l.pointsize !== void 0) {
        let h = l.pointsize, f = r;
        qc(h, { instancePath: o + "/pointsize", parentData: l, parentDataProperty: "pointsize", rootData: s }) || (e = e === null ? qc.errors : e.concat(qc.errors), r = e.length);
        var P = f === r;
      }
      if (l.lineheight !== void 0) {
        let h = l.lineheight, f = r;
        Oc(h, { instancePath: o + "/lineheight", parentData: l, parentDataProperty: "lineheight", rootData: s }) || (e = e === null ? Oc.errors : e.concat(Oc.errors), r = e.length);
        var P = f === r;
      }
      if (l.linestretch !== void 0) {
        let h = l.linestretch, f = r;
        Ya(h, { instancePath: o + "/linestretch", parentData: l, parentDataProperty: "linestretch", rootData: s }) || (e = e === null ? Ya.errors : e.concat(Ya.errors), r = e.length);
        var P = f === r;
      }
      if (l.interlinespace !== void 0) {
        let h = l.interlinespace, f = r;
        Pc(h, { instancePath: o + "/interlinespace", parentData: l, parentDataProperty: "interlinespace", rootData: s }) || (e = e === null ? Pc.errors : e.concat(Pc.errors), r = e.length);
        var P = f === r;
      }
      if (l.linkstyle !== void 0) {
        let h = l.linkstyle, f = r;
        xc(h, { instancePath: o + "/linkstyle", parentData: l, parentDataProperty: "linkstyle", rootData: s }) || (e = e === null ? xc.errors : e.concat(xc.errors), r = e.length);
        var P = f === r;
      }
      if (l.whitespace !== void 0) {
        let h = l.whitespace, f = r;
        kc(h, { instancePath: o + "/whitespace", parentData: l, parentDataProperty: "whitespace", rootData: s }) || (e = e === null ? kc.errors : e.concat(kc.errors), r = e.length);
        var P = f === r;
      }
      if (l["footnotes-hover"] !== void 0) {
        let h = l["footnotes-hover"], f = r;
        wn(h, { instancePath: o + "/footnotes-hover", parentData: l, parentDataProperty: "footnotes-hover", rootData: s }) || (e = e === null ? wn.errors : e.concat(wn.errors), r = e.length);
        var P = f === r;
      }
      if (l["links-as-notes"] !== void 0) {
        let h = l["links-as-notes"], f = r;
        Fs(h, { instancePath: o + "/links-as-notes", parentData: l, parentDataProperty: "links-as-notes", rootData: s }) || (e = e === null ? Fs.errors : e.concat(Fs.errors), r = e.length);
        var P = f === r;
      }
      if (l["reference-location"] !== void 0) {
        let h = l["reference-location"], f = r;
        Ca(h, { instancePath: o + "/reference-location", parentData: l, parentDataProperty: "reference-location", rootData: s }) || (e = e === null ? Ca.errors : e.concat(Ca.errors), r = e.length);
        var P = f === r;
      }
      if (l.indenting !== void 0) {
        let h = l.indenting, f = r;
        Dc(h, { instancePath: o + "/indenting", parentData: l, parentDataProperty: "indenting", rootData: s }) || (e = e === null ? Dc.errors : e.concat(Dc.errors), r = e.length);
        var P = f === r;
      }
      if (l.adjusting !== void 0) {
        let h = l.adjusting, f = r;
        wc(h, { instancePath: o + "/adjusting", parentData: l, parentDataProperty: "adjusting", rootData: s }) || (e = e === null ? wc.errors : e.concat(wc.errors), r = e.length);
        var P = f === r;
      }
      if (l.hyphenate !== void 0) {
        let h = l.hyphenate, f = r;
        $c(h, { instancePath: o + "/hyphenate", parentData: l, parentDataProperty: "hyphenate", rootData: s }) || (e = e === null ? $c.errors : e.concat($c.errors), r = e.length);
        var P = f === r;
      }
      if (l.to !== void 0) {
        let h = l.to, f = r;
        I(h, { instancePath: o + "/to", parentData: l, parentDataProperty: "to", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
        var P = f === r;
      }
      if (l.writer !== void 0) {
        let h = l.writer, f = r;
        I(h, { instancePath: o + "/writer", parentData: l, parentDataProperty: "writer", rootData: s }) || (e = e === null ? I.errors : e.concat(I.errors), r = e.length);
        var P = f === r;
      }
      if (l["input-file"] !== void 0) {
        let h = l["input-file"], f = r;
        ze(h, { instancePath: o + "/input-file", parentData: l, parentDataProperty: "input-file", rootData: s }) || (e = e === null ? ze.errors : e.concat(ze.errors), r = e.length);
        var P = f === r;
      }
      if (l["input-files"] !== void 0) {
        let h = l["input-files"], f = r;
        We(h, { instancePath: o + "/input-files", parentData: l, parentDataProperty: "input-files", rootData: s }) || (e = e === null ? We.errors : e.concat(We.errors), r = e.length);
        var P = f === r;
      }
      if (l.defaults !== void 0) {
        let h = l.defaults, f = r;
        Re(h, { instancePath: o + "/defaults", parentData: l, parentDataProperty: "defaults", rootData: s }) || (e = e === null ? Re.errors : e.concat(Re.errors), r = e.length);
        var P = f === r;
      }
      if (l.variables !== void 0) {
        let h = l.variables, f = r;
        Ve(h, { instancePath: o + "/variables", parentData: l, parentDataProperty: "variables", rootData: s }) || (e = e === null ? Ve.errors : e.concat(Ve.errors), r = e.length);
        var P = f === r;
      }
      if (l.metadata !== void 0) {
        let h = l.metadata, f = r;
        Ue(h, { instancePath: o + "/metadata", parentData: l, parentDataProperty: "metadata", rootData: s }) || (e = e === null ? Ue.errors : e.concat(Ue.errors), r = e.length);
        var P = f === r;
      }
      if (l["request-headers"] !== void 0) {
        let h = l["request-headers"], f = r;
        Ke(h, { instancePath: o + "/request-headers", parentData: l, parentDataProperty: "request-headers", rootData: s }) || (e = e === null ? Ke.errors : e.concat(Ke.errors), r = e.length);
        var P = f === r;
      }
      if (l.trace !== void 0) {
        let h = l.trace, f = r;
        Je(h, { instancePath: o + "/trace", parentData: l, parentDataProperty: "trace", rootData: s }) || (e = e === null ? Je.errors : e.concat(Je.errors), r = e.length);
        var P = f === r;
      }
      if (l["fail-if-warnings"] !== void 0) {
        let h = l["fail-if-warnings"], f = r;
        Qe(h, { instancePath: o + "/fail-if-warnings", parentData: l, parentDataProperty: "fail-if-warnings", rootData: s }) || (e = e === null ? Qe.errors : e.concat(Qe.errors), r = e.length);
        var P = f === r;
      }
      if (l["dump-args"] !== void 0) {
        let h = l["dump-args"], f = r;
        Ge(h, { instancePath: o + "/dump-args", parentData: l, parentDataProperty: "dump-args", rootData: s }) || (e = e === null ? Ge.errors : e.concat(Ge.errors), r = e.length);
        var P = f === r;
      }
      if (l["ignore-args"] !== void 0) {
        let h = l["ignore-args"], f = r;
        Ye(h, { instancePath: o + "/ignore-args", parentData: l, parentDataProperty: "ignore-args", rootData: s }) || (e = e === null ? Ye.errors : e.concat(Ye.errors), r = e.length);
        var P = f === r;
      }
      if (l["file-scope"] !== void 0) {
        let h = l["file-scope"], f = r;
        Xe(h, { instancePath: o + "/file-scope", parentData: l, parentDataProperty: "file-scope", rootData: s }) || (e = e === null ? Xe.errors : e.concat(Xe.errors), r = e.length);
        var P = f === r;
      }
      if (l["data-dir"] !== void 0) {
        let h = l["data-dir"], f = r;
        Ze(h, { instancePath: o + "/data-dir", parentData: l, parentDataProperty: "data-dir", rootData: s }) || (e = e === null ? Ze.errors : e.concat(Ze.errors), r = e.length);
        var P = f === r;
      }
      if (l.verbosity !== void 0) {
        let h = l.verbosity, f = r;
        er(h, { instancePath: o + "/verbosity", parentData: l, parentDataProperty: "verbosity", rootData: s }) || (e = e === null ? er.errors : e.concat(er.errors), r = e.length);
        var P = f === r;
      }
      if (l["log-file"] !== void 0) {
        let h = l["log-file"], f = r;
        rr(h, { instancePath: o + "/log-file", parentData: l, parentDataProperty: "log-file", rootData: s }) || (e = e === null ? rr.errors : e.concat(rr.errors), r = e.length);
        var P = f === r;
      }
      if (l["track-changes"] !== void 0) {
        let h = l["track-changes"], f = r;
        Sc(h, { instancePath: o + "/track-changes", parentData: l, parentDataProperty: "track-changes", rootData: s }) || (e = e === null ? Sc.errors : e.concat(Sc.errors), r = e.length);
        var P = f === r;
      }
      if (l["strip-empty-paragraphs"] !== void 0) {
        let h = l["strip-empty-paragraphs"], f = r;
        tr(h, { instancePath: o + "/strip-empty-paragraphs", parentData: l, parentDataProperty: "strip-empty-paragraphs", rootData: s }) || (e = e === null ? tr.errors : e.concat(tr.errors), r = e.length);
        var P = f === r;
      }
      if (l["keep-source"] !== void 0) {
        let h = l["keep-source"], f = r;
        As(h, { instancePath: o + "/keep-source", parentData: l, parentDataProperty: "keep-source", rootData: s }) || (e = e === null ? As.errors : e.concat(As.errors), r = e.length);
        var P = f === r;
      }
      if (l["keep-hidden"] !== void 0) {
        let h = l["keep-hidden"], f = r;
        Bs(h, { instancePath: o + "/keep-hidden", parentData: l, parentDataProperty: "keep-hidden", rootData: s }) || (e = e === null ? Bs.errors : e.concat(Bs.errors), r = e.length);
        var P = f === r;
      }
      if (l["prefer-html"] !== void 0) {
        let h = l["prefer-html"], f = r;
        jo(h, { instancePath: o + "/prefer-html", parentData: l, parentDataProperty: "prefer-html", rootData: s }) || (e = e === null ? jo.errors : e.concat(jo.errors), r = e.length);
        var P = f === r;
      }
      if (l["output-divs"] !== void 0) {
        let h = l["output-divs"], f = r;
        ar(h, { instancePath: o + "/output-divs", parentData: l, parentDataProperty: "output-divs", rootData: s }) || (e = e === null ? ar.errors : e.concat(ar.errors), r = e.length);
        var P = f === r;
      }
      if (l["merge-includes"] !== void 0) {
        let h = l["merge-includes"], f = r;
        nr(h, { instancePath: o + "/merge-includes", parentData: l, parentDataProperty: "merge-includes", rootData: s }) || (e = e === null ? nr.errors : e.concat(nr.errors), r = e.length);
        var P = f === r;
      }
      if (l["header-includes"] !== void 0) {
        let h = l["header-includes"], f = r;
        Ir(h, { instancePath: o + "/header-includes", parentData: l, parentDataProperty: "header-includes", rootData: s }) || (e = e === null ? Ir.errors : e.concat(Ir.errors), r = e.length);
        var P = f === r;
      }
      if (l["include-before"] !== void 0) {
        let h = l["include-before"], f = r;
        Hr(h, { instancePath: o + "/include-before", parentData: l, parentDataProperty: "include-before", rootData: s }) || (e = e === null ? Hr.errors : e.concat(Hr.errors), r = e.length);
        var P = f === r;
      }
      if (l["include-after"] !== void 0) {
        let h = l["include-after"], f = r;
        Lr(h, { instancePath: o + "/include-after", parentData: l, parentDataProperty: "include-after", rootData: s }) || (e = e === null ? Lr.errors : e.concat(Lr.errors), r = e.length);
        var P = f === r;
      }
      if (l["include-before-body"] !== void 0) {
        let h = l["include-before-body"], f = r;
        Mr(h, { instancePath: o + "/include-before-body", parentData: l, parentDataProperty: "include-before-body", rootData: s }) || (e = e === null ? Mr.errors : e.concat(Mr.errors), r = e.length);
        var P = f === r;
      }
      if (l["include-after-body"] !== void 0) {
        let h = l["include-after-body"], f = r;
        Tr(h, { instancePath: o + "/include-after-body", parentData: l, parentDataProperty: "include-after-body", rootData: s }) || (e = e === null ? Tr.errors : e.concat(Tr.errors), r = e.length);
        var P = f === r;
      }
      if (l["include-in-header"] !== void 0) {
        let h = l["include-in-header"], f = r;
        zr(h, { instancePath: o + "/include-in-header", parentData: l, parentDataProperty: "include-in-header", rootData: s }) || (e = e === null ? zr.errors : e.concat(zr.errors), r = e.length);
        var P = f === r;
      }
      if (l.resources !== void 0) {
        let h = l.resources, f = r;
        pn(h, { instancePath: o + "/resources", parentData: l, parentDataProperty: "resources", rootData: s }) || (e = e === null ? pn.errors : e.concat(pn.errors), r = e.length);
        var P = f === r;
      }
      if (l.headertext !== void 0) {
        let h = l.headertext, f = r;
        Fc(h, { instancePath: o + "/headertext", parentData: l, parentDataProperty: "headertext", rootData: s }) || (e = e === null ? Fc.errors : e.concat(Fc.errors), r = e.length);
        var P = f === r;
      }
      if (l.footertext !== void 0) {
        let h = l.footertext, f = r;
        Ac(h, { instancePath: o + "/footertext", parentData: l, parentDataProperty: "footertext", rootData: s }) || (e = e === null ? Ac.errors : e.concat(Ac.errors), r = e.length);
        var P = f === r;
      }
      if (l.includesource !== void 0) {
        let h = l.includesource, f = r;
        Bc(h, { instancePath: o + "/includesource", parentData: l, parentDataProperty: "includesource", rootData: s }) || (e = e === null ? Bc.errors : e.concat(Bc.errors), r = e.length);
        var P = f === r;
      }
      if (l.footer !== void 0) {
        let h = l.footer, f = r;
        ap(h, { instancePath: o + "/footer", parentData: l, parentDataProperty: "footer", rootData: s }) || (e = e === null ? ap.errors : e.concat(ap.errors), r = e.length);
        var P = f === r;
      }
      if (l.header !== void 0) {
        let h = l.header, f = r;
        jc(h, { instancePath: o + "/header", parentData: l, parentDataProperty: "header", rootData: s }) || (e = e === null ? jc.errors : e.concat(jc.errors), r = e.length);
        var P = f === r;
      }
      if (l["metadata-file"] !== void 0) {
        let h = l["metadata-file"], f = r;
        or(h, { instancePath: o + "/metadata-file", parentData: l, parentDataProperty: "metadata-file", rootData: s }) || (e = e === null ? or.errors : e.concat(or.errors), r = e.length);
        var P = f === r;
      }
      if (l["metadata-files"] !== void 0) {
        let h = l["metadata-files"], f = r;
        sr(h, { instancePath: o + "/metadata-files", parentData: l, parentDataProperty: "metadata-files", rootData: s }) || (e = e === null ? sr.errors : e.concat(sr.errors), r = e.length);
        var P = f === r;
      }
      if (l.lang !== void 0) {
        let h = l.lang, f = r;
        ir(h, { instancePath: o + "/lang", parentData: l, parentDataProperty: "lang", rootData: s }) || (e = e === null ? ir.errors : e.concat(ir.errors), r = e.length);
        var P = f === r;
      }
      if (l.language !== void 0) {
        let h = l.language, f = r;
        lr(h, { instancePath: o + "/language", parentData: l, parentDataProperty: "language", rootData: s }) || (e = e === null ? lr.errors : e.concat(lr.errors), r = e.length);
        var P = f === r;
      }
      if (l.dir !== void 0) {
        let h = l.dir, f = r;
        cr(h, { instancePath: o + "/dir", parentData: l, parentDataProperty: "dir", rootData: s }) || (e = e === null ? cr.errors : e.concat(cr.errors), r = e.length);
        var P = f === r;
      }
      if (l["latex-auto-mk"] !== void 0) {
        let h = l["latex-auto-mk"], f = r;
        El(h, { instancePath: o + "/latex-auto-mk", parentData: l, parentDataProperty: "latex-auto-mk", rootData: s }) || (e = e === null ? El.errors : e.concat(El.errors), r = e.length);
        var P = f === r;
      }
      if (l["latex-auto-install"] !== void 0) {
        let h = l["latex-auto-install"], f = r;
        Nl(h, { instancePath: o + "/latex-auto-install", parentData: l, parentDataProperty: "latex-auto-install", rootData: s }) || (e = e === null ? Nl.errors : e.concat(Nl.errors), r = e.length);
        var P = f === r;
      }
      if (l["latex-min-runs"] !== void 0) {
        let h = l["latex-min-runs"], f = r;
        Cl(h, { instancePath: o + "/latex-min-runs", parentData: l, parentDataProperty: "latex-min-runs", rootData: s }) || (e = e === null ? Cl.errors : e.concat(Cl.errors), r = e.length);
        var P = f === r;
      }
      if (l["latex-max-runs"] !== void 0) {
        let h = l["latex-max-runs"], f = r;
        Il(h, { instancePath: o + "/latex-max-runs", parentData: l, parentDataProperty: "latex-max-runs", rootData: s }) || (e = e === null ? Il.errors : e.concat(Il.errors), r = e.length);
        var P = f === r;
      }
      if (l["latex-clean"] !== void 0) {
        let h = l["latex-clean"], f = r;
        Hl(h, { instancePath: o + "/latex-clean", parentData: l, parentDataProperty: "latex-clean", rootData: s }) || (e = e === null ? Hl.errors : e.concat(Hl.errors), r = e.length);
        var P = f === r;
      }
      if (l["latex-makeindex"] !== void 0) {
        let h = l["latex-makeindex"], f = r;
        Ll(h, { instancePath: o + "/latex-makeindex", parentData: l, parentDataProperty: "latex-makeindex", rootData: s }) || (e = e === null ? Ll.errors : e.concat(Ll.errors), r = e.length);
        var P = f === r;
      }
      if (l["latex-makeindex-opts"] !== void 0) {
        let h = l["latex-makeindex-opts"], f = r;
        Ml(h, { instancePath: o + "/latex-makeindex-opts", parentData: l, parentDataProperty: "latex-makeindex-opts", rootData: s }) || (e = e === null ? Ml.errors : e.concat(Ml.errors), r = e.length);
        var P = f === r;
      }
      if (l["latex-tlmgr-opts"] !== void 0) {
        let h = l["latex-tlmgr-opts"], f = r;
        Tl(h, { instancePath: o + "/latex-tlmgr-opts", parentData: l, parentDataProperty: "latex-tlmgr-opts", rootData: s }) || (e = e === null ? Tl.errors : e.concat(Tl.errors), r = e.length);
        var P = f === r;
      }
      if (l["latex-output-dir"] !== void 0) {
        let h = l["latex-output-dir"], f = r;
        zl(h, { instancePath: o + "/latex-output-dir", parentData: l, parentDataProperty: "latex-output-dir", rootData: s }) || (e = e === null ? zl.errors : e.concat(zl.errors), r = e.length);
        var P = f === r;
      }
      if (l.documentclass !== void 0) {
        let h = l.documentclass, f = r;
        js(h, { instancePath: o + "/documentclass", parentData: l, parentDataProperty: "documentclass", rootData: s }) || (e = e === null ? js.errors : e.concat(js.errors), r = e.length);
        var P = f === r;
      }
      if (l.classoption !== void 0) {
        let h = l.classoption, f = r;
        un(h, { instancePath: o + "/classoption", parentData: l, parentDataProperty: "classoption", rootData: s }) || (e = e === null ? un.errors : e.concat(un.errors), r = e.length);
        var P = f === r;
      }
      if (l.pagestyle !== void 0) {
        let h = l.pagestyle, f = r;
        Es(h, { instancePath: o + "/pagestyle", parentData: l, parentDataProperty: "pagestyle", rootData: s }) || (e = e === null ? Es.errors : e.concat(Es.errors), r = e.length);
        var P = f === r;
      }
      if (l.papersize !== void 0) {
        let h = l.papersize, f = r;
        Ns(h, { instancePath: o + "/papersize", parentData: l, parentDataProperty: "papersize", rootData: s }) || (e = e === null ? Ns.errors : e.concat(Ns.errors), r = e.length);
        var P = f === r;
      }
      if (l.layout !== void 0) {
        let h = l.layout, f = r;
        Ec(h, { instancePath: o + "/layout", parentData: l, parentDataProperty: "layout", rootData: s }) || (e = e === null ? Ec.errors : e.concat(Ec.errors), r = e.length);
        var P = f === r;
      }
      if (l["page-layout"] !== void 0) {
        let h = l["page-layout"], f = r;
        Cs(h, { instancePath: o + "/page-layout", parentData: l, parentDataProperty: "page-layout", rootData: s }) || (e = e === null ? Cs.errors : e.concat(Cs.errors), r = e.length);
        var P = f === r;
      }
      if (l["page-width"] !== void 0) {
        let h = l["page-width"], f = r;
        Is(h, { instancePath: o + "/page-width", parentData: l, parentDataProperty: "page-width", rootData: s }) || (e = e === null ? Is.errors : e.concat(Is.errors), r = e.length);
        var P = f === r;
      }
      if (l["max-width"] !== void 0) {
        let h = l["max-width"], f = r;
        $n(h, { instancePath: o + "/max-width", parentData: l, parentDataProperty: "max-width", rootData: s }) || (e = e === null ? $n.errors : e.concat($n.errors), r = e.length);
        var P = f === r;
      }
      if (l["margin-left"] !== void 0) {
        let h = l["margin-left"], f = r;
        Xa(h, { instancePath: o + "/margin-left", parentData: l, parentDataProperty: "margin-left", rootData: s }) || (e = e === null ? Xa.errors : e.concat(Xa.errors), r = e.length);
        var P = f === r;
      }
      if (l["margin-right"] !== void 0) {
        let h = l["margin-right"], f = r;
        Za(h, { instancePath: o + "/margin-right", parentData: l, parentDataProperty: "margin-right", rootData: s }) || (e = e === null ? Za.errors : e.concat(Za.errors), r = e.length);
        var P = f === r;
      }
      if (l["margin-top"] !== void 0) {
        let h = l["margin-top"], f = r;
        en(h, { instancePath: o + "/margin-top", parentData: l, parentDataProperty: "margin-top", rootData: s }) || (e = e === null ? en.errors : e.concat(en.errors), r = e.length);
        var P = f === r;
      }
      if (l["margin-bottom"] !== void 0) {
        let h = l["margin-bottom"], f = r;
        rn(h, { instancePath: o + "/margin-bottom", parentData: l, parentDataProperty: "margin-bottom", rootData: s }) || (e = e === null ? rn.errors : e.concat(rn.errors), r = e.length);
        var P = f === r;
      }
      if (l.geometry !== void 0) {
        let h = l.geometry, f = r;
        Hs(h, { instancePath: o + "/geometry", parentData: l, parentDataProperty: "geometry", rootData: s }) || (e = e === null ? Hs.errors : e.concat(Hs.errors), r = e.length);
        var P = f === r;
      }
      if (l.hyperrefoptions !== void 0) {
        let h = l.hyperrefoptions, f = r;
        Ls(h, { instancePath: o + "/hyperrefoptions", parentData: l, parentDataProperty: "hyperrefoptions", rootData: s }) || (e = e === null ? Ls.errors : e.concat(Ls.errors), r = e.length);
        var P = f === r;
      }
      if (l.indent !== void 0) {
        let h = l.indent, f = r;
        Eo(h, { instancePath: o + "/indent", parentData: l, parentDataProperty: "indent", rootData: s }) || (e = e === null ? Eo.errors : e.concat(Eo.errors), r = e.length);
        var P = f === r;
      }
      if (l["block-headings"] !== void 0) {
        let h = l["block-headings"], f = r;
        Ms(h, { instancePath: o + "/block-headings", parentData: l, parentDataProperty: "block-headings", rootData: s }) || (e = e === null ? Ms.errors : e.concat(Ms.errors), r = e.length);
        var P = f === r;
      }
      if (l["reveal-js-url"] !== void 0) {
        let h = l["reveal-js-url"], f = r;
        Nc(h, { instancePath: o + "/reveal-js-url", parentData: l, parentDataProperty: "reveal-js-url", rootData: s }) || (e = e === null ? Nc.errors : e.concat(Nc.errors), r = e.length);
        var P = f === r;
      }
      if (l["s5-url"] !== void 0) {
        let h = l["s5-url"], f = r;
        Cc(h, { instancePath: o + "/s5-url", parentData: l, parentDataProperty: "s5-url", rootData: s }) || (e = e === null ? Cc.errors : e.concat(Cc.errors), r = e.length);
        var P = f === r;
      }
      if (l["slidy-url"] !== void 0) {
        let h = l["slidy-url"], f = r;
        Ic(h, { instancePath: o + "/slidy-url", parentData: l, parentDataProperty: "slidy-url", rootData: s }) || (e = e === null ? Ic.errors : e.concat(Ic.errors), r = e.length);
        var P = f === r;
      }
      if (l["slideous-url"] !== void 0) {
        let h = l["slideous-url"], f = r;
        Hc(h, { instancePath: o + "/slideous-url", parentData: l, parentDataProperty: "slideous-url", rootData: s }) || (e = e === null ? Hc.errors : e.concat(Hc.errors), r = e.length);
        var P = f === r;
      }
      if (l["link-external-icon"] !== void 0) {
        let h = l["link-external-icon"], f = r;
        No(h, { instancePath: o + "/link-external-icon", parentData: l, parentDataProperty: "link-external-icon", rootData: s }) || (e = e === null ? No.errors : e.concat(No.errors), r = e.length);
        var P = f === r;
      }
      if (l["link-external-newwindow"] !== void 0) {
        let h = l["link-external-newwindow"], f = r;
        Co(h, { instancePath: o + "/link-external-newwindow", parentData: l, parentDataProperty: "link-external-newwindow", rootData: s }) || (e = e === null ? Co.errors : e.concat(Co.errors), r = e.length);
        var P = f === r;
      }
      if (l["link-external-filter"] !== void 0) {
        let h = l["link-external-filter"], f = r;
        Io(h, { instancePath: o + "/link-external-filter", parentData: l, parentDataProperty: "link-external-filter", rootData: s }) || (e = e === null ? Io.errors : e.concat(Io.errors), r = e.length);
        var P = f === r;
      }
      if (l.keywords !== void 0) {
        let h = l.keywords, f = r;
        Ia(h, { instancePath: o + "/keywords", parentData: l, parentDataProperty: "keywords", rootData: s }) || (e = e === null ? Ia.errors : e.concat(Ia.errors), r = e.length);
        var P = f === r;
      }
      if (l.description !== void 0) {
        let h = l.description, f = r;
        Wl(h, { instancePath: o + "/description", parentData: l, parentDataProperty: "description", rootData: s }) || (e = e === null ? Wl.errors : e.concat(Wl.errors), r = e.length);
        var P = f === r;
      }
      if (l.category !== void 0) {
        let h = l.category, f = r;
        Rl(h, { instancePath: o + "/category", parentData: l, parentDataProperty: "category", rootData: s }) || (e = e === null ? Rl.errors : e.concat(Rl.errors), r = e.length);
        var P = f === r;
      }
      if (l["title-meta"] !== void 0) {
        let h = l["title-meta"], f = r;
        Ts(h, { instancePath: o + "/title-meta", parentData: l, parentDataProperty: "title-meta", rootData: s }) || (e = e === null ? Ts.errors : e.concat(Ts.errors), r = e.length);
        var P = f === r;
      }
      if (l.pagetitle !== void 0) {
        let h = l.pagetitle, f = r;
        Sn(h, { instancePath: o + "/pagetitle", parentData: l, parentDataProperty: "pagetitle", rootData: s }) || (e = e === null ? Sn.errors : e.concat(Sn.errors), r = e.length);
        var P = f === r;
      }
      if (l["title-prefix"] !== void 0) {
        let h = l["title-prefix"], f = r;
        Fn(h, { instancePath: o + "/title-prefix", parentData: l, parentDataProperty: "title-prefix", rootData: s }) || (e = e === null ? Fn.errors : e.concat(Fn.errors), r = e.length);
        var P = f === r;
      }
      if (l["description-meta"] !== void 0) {
        let h = l["description-meta"], f = r;
        An(h, { instancePath: o + "/description-meta", parentData: l, parentDataProperty: "description-meta", rootData: s }) || (e = e === null ? An.errors : e.concat(An.errors), r = e.length);
        var P = f === r;
      }
      if (l["author-meta"] !== void 0) {
        let h = l["author-meta"], f = r;
        dn(h, { instancePath: o + "/author-meta", parentData: l, parentDataProperty: "author-meta", rootData: s }) || (e = e === null ? dn.errors : e.concat(dn.errors), r = e.length);
        var P = f === r;
      }
      if (l["date-meta"] !== void 0) {
        let h = l["date-meta"], f = r;
        Ha(h, { instancePath: o + "/date-meta", parentData: l, parentDataProperty: "date-meta", rootData: s }) || (e = e === null ? Ha.errors : e.concat(Ha.errors), r = e.length);
        var P = f === r;
      }
      if (l["number-sections"] !== void 0) {
        let h = l["number-sections"], f = r;
        pr(h, { instancePath: o + "/number-sections", parentData: l, parentDataProperty: "number-sections", rootData: s }) || (e = e === null ? pr.errors : e.concat(pr.errors), r = e.length);
        var P = f === r;
      }
      if (l["number-depth"] !== void 0) {
        let h = l["number-depth"], f = r;
        Da(h, { instancePath: o + "/number-depth", parentData: l, parentDataProperty: "number-depth", rootData: s }) || (e = e === null ? Da.errors : e.concat(Da.errors), r = e.length);
        var P = f === r;
      }
      if (l.secnumdepth !== void 0) {
        let h = l.secnumdepth, f = r;
        zs(h, { instancePath: o + "/secnumdepth", parentData: l, parentDataProperty: "secnumdepth", rootData: s }) || (e = e === null ? zs.errors : e.concat(zs.errors), r = e.length);
        var P = f === r;
      }
      if (l["number-offset"] !== void 0) {
        let h = l["number-offset"], f = r;
        ur(h, { instancePath: o + "/number-offset", parentData: l, parentDataProperty: "number-offset", rootData: s }) || (e = e === null ? ur.errors : e.concat(ur.errors), r = e.length);
        var P = f === r;
      }
      if (l["shift-heading-level-by"] !== void 0) {
        let h = l["shift-heading-level-by"], f = r;
        dr(h, { instancePath: o + "/shift-heading-level-by", parentData: l, parentDataProperty: "shift-heading-level-by", rootData: s }) || (e = e === null ? dr.errors : e.concat(dr.errors), r = e.length);
        var P = f === r;
      }
      if (l.pagenumbering !== void 0) {
        let h = l.pagenumbering, f = r;
        Lc(h, { instancePath: o + "/pagenumbering", parentData: l, parentDataProperty: "pagenumbering", rootData: s }) || (e = e === null ? Lc.errors : e.concat(Lc.errors), r = e.length);
        var P = f === r;
      }
      if (l["top-level-division"] !== void 0) {
        let h = l["top-level-division"], f = r;
        Bn(h, { instancePath: o + "/top-level-division", parentData: l, parentDataProperty: "top-level-division", rootData: s }) || (e = e === null ? Bn.errors : e.concat(Bn.errors), r = e.length);
        var P = f === r;
      }
      if (l["reference-doc"] !== void 0) {
        let h = l["reference-doc"], f = r;
        Ws(h, { instancePath: o + "/reference-doc", parentData: l, parentDataProperty: "reference-doc", rootData: s }) || (e = e === null ? Ws.errors : e.concat(Ws.errors), r = e.length);
        var P = f === r;
      }
      if (l.theme !== void 0) {
        let h = l.theme, f = r;
        Ho(h, { instancePath: o + "/theme", parentData: l, parentDataProperty: "theme", rootData: s }) || (e = e === null ? Ho.errors : e.concat(Ho.errors), r = e.length);
        var P = f === r;
      }
      if (l.minimal !== void 0) {
        let h = l.minimal, f = r;
        Rs(h, { instancePath: o + "/minimal", parentData: l, parentDataProperty: "minimal", rootData: s }) || (e = e === null ? Rs.errors : e.concat(Rs.errors), r = e.length);
        var P = f === r;
      }
      if (l["document-css"] !== void 0) {
        let h = l["document-css"], f = r;
        jn(h, { instancePath: o + "/document-css", parentData: l, parentDataProperty: "document-css", rootData: s }) || (e = e === null ? jn.errors : e.concat(jn.errors), r = e.length);
        var P = f === r;
      }
      if (l.css !== void 0) {
        let h = l.css, f = r;
        fn(h, { instancePath: o + "/css", parentData: l, parentDataProperty: "css", rootData: s }) || (e = e === null ? fn.errors : e.concat(fn.errors), r = e.length);
        var P = f === r;
      }
      if (l["anchor-sections"] !== void 0) {
        let h = l["anchor-sections"], f = r;
        Vs(h, { instancePath: o + "/anchor-sections", parentData: l, parentDataProperty: "anchor-sections", rootData: s }) || (e = e === null ? Vs.errors : e.concat(Vs.errors), r = e.length);
        var P = f === r;
      }
      if (l["html-math-method"] !== void 0) {
        let h = l["html-math-method"], f = r;
        Wn(h, { instancePath: o + "/html-math-method", parentData: l, parentDataProperty: "html-math-method", rootData: s }) || (e = e === null ? Wn.errors : e.concat(Wn.errors), r = e.length);
        var P = f === r;
      }
      if (l["section-divs"] !== void 0) {
        let h = l["section-divs"], f = r;
        En(h, { instancePath: o + "/section-divs", parentData: l, parentDataProperty: "section-divs", rootData: s }) || (e = e === null ? En.errors : e.concat(En.errors), r = e.length);
        var P = f === r;
      }
      if (l["identifier-prefix"] !== void 0) {
        let h = l["identifier-prefix"], f = r;
        Pa(h, { instancePath: o + "/identifier-prefix", parentData: l, parentDataProperty: "identifier-prefix", rootData: s }) || (e = e === null ? Pa.errors : e.concat(Pa.errors), r = e.length);
        var P = f === r;
      }
      if (l["email-obfuscation"] !== void 0) {
        let h = l["email-obfuscation"], f = r;
        Nn(h, { instancePath: o + "/email-obfuscation", parentData: l, parentDataProperty: "email-obfuscation", rootData: s }) || (e = e === null ? Nn.errors : e.concat(Nn.errors), r = e.length);
        var P = f === r;
      }
      if (l["html-q-tags"] !== void 0) {
        let h = l["html-q-tags"], f = r;
        mn(h, { instancePath: o + "/html-q-tags", parentData: l, parentDataProperty: "html-q-tags", rootData: s }) || (e = e === null ? mn.errors : e.concat(mn.errors), r = e.length);
        var P = f === r;
      }
      if (l["pdf-engine"] !== void 0) {
        let h = l["pdf-engine"], f = r;
        mo(h, { instancePath: o + "/pdf-engine", parentData: l, parentDataProperty: "pdf-engine", rootData: s }) || (e = e === null ? mo.errors : e.concat(mo.errors), r = e.length);
        var P = f === r;
      }
      if (l["pdf-engine-opt"] !== void 0) {
        let h = l["pdf-engine-opt"], f = r;
        go(h, { instancePath: o + "/pdf-engine-opt", parentData: l, parentDataProperty: "pdf-engine-opt", rootData: s }) || (e = e === null ? go.errors : e.concat(go.errors), r = e.length);
        var P = f === r;
      }
      if (l.beamerarticle !== void 0) {
        let h = l.beamerarticle, f = r;
        Mc(h, { instancePath: o + "/beamerarticle", parentData: l, parentDataProperty: "beamerarticle", rootData: s }) || (e = e === null ? Mc.errors : e.concat(Mc.errors), r = e.length);
        var P = f === r;
      }
      if (l.beameroption !== void 0) {
        let h = l.beameroption, f = r;
        Tc(h, { instancePath: o + "/beameroption", parentData: l, parentDataProperty: "beameroption", rootData: s }) || (e = e === null ? Tc.errors : e.concat(Tc.errors), r = e.length);
        var P = f === r;
      }
      if (l.aspectratio !== void 0) {
        let h = l.aspectratio, f = r;
        zc(h, { instancePath: o + "/aspectratio", parentData: l, parentDataProperty: "aspectratio", rootData: s }) || (e = e === null ? zc.errors : e.concat(zc.errors), r = e.length);
        var P = f === r;
      }
      if (l.logo !== void 0) {
        let h = l.logo, f = r;
        tp(h, { instancePath: o + "/logo", parentData: l, parentDataProperty: "logo", rootData: s }) || (e = e === null ? tp.errors : e.concat(tp.errors), r = e.length);
        var P = f === r;
      }
      if (l.titlegraphic !== void 0) {
        let h = l.titlegraphic, f = r;
        Wc(h, { instancePath: o + "/titlegraphic", parentData: l, parentDataProperty: "titlegraphic", rootData: s }) || (e = e === null ? Wc.errors : e.concat(Wc.errors), r = e.length);
        var P = f === r;
      }
      if (l.navigation !== void 0) {
        let h = l.navigation, f = r;
        Rc(h, { instancePath: o + "/navigation", parentData: l, parentDataProperty: "navigation", rootData: s }) || (e = e === null ? Rc.errors : e.concat(Rc.errors), r = e.length);
        var P = f === r;
      }
      if (l["section-titles"] !== void 0) {
        let h = l["section-titles"], f = r;
        Vc(h, { instancePath: o + "/section-titles", parentData: l, parentDataProperty: "section-titles", rootData: s }) || (e = e === null ? Vc.errors : e.concat(Vc.errors), r = e.length);
        var P = f === r;
      }
      if (l.colortheme !== void 0) {
        let h = l.colortheme, f = r;
        Uc(h, { instancePath: o + "/colortheme", parentData: l, parentDataProperty: "colortheme", rootData: s }) || (e = e === null ? Uc.errors : e.concat(Uc.errors), r = e.length);
        var P = f === r;
      }
      if (l.fonttheme !== void 0) {
        let h = l.fonttheme, f = r;
        Kc(h, { instancePath: o + "/fonttheme", parentData: l, parentDataProperty: "fonttheme", rootData: s }) || (e = e === null ? Kc.errors : e.concat(Kc.errors), r = e.length);
        var P = f === r;
      }
      if (l.innertheme !== void 0) {
        let h = l.innertheme, f = r;
        Jc(h, { instancePath: o + "/innertheme", parentData: l, parentDataProperty: "innertheme", rootData: s }) || (e = e === null ? Jc.errors : e.concat(Jc.errors), r = e.length);
        var P = f === r;
      }
      if (l.outertheme !== void 0) {
        let h = l.outertheme, f = r;
        Qc(h, { instancePath: o + "/outertheme", parentData: l, parentDataProperty: "outertheme", rootData: s }) || (e = e === null ? Qc.errors : e.concat(Qc.errors), r = e.length);
        var P = f === r;
      }
      if (l.themeoptions !== void 0) {
        let h = l.themeoptions, f = r;
        Gc(h, { instancePath: o + "/themeoptions", parentData: l, parentDataProperty: "themeoptions", rootData: s }) || (e = e === null ? Gc.errors : e.concat(Gc.errors), r = e.length);
        var P = f === r;
      }
      if (l.section !== void 0) {
        let h = l.section, f = r;
        Yc(h, { instancePath: o + "/section", parentData: l, parentDataProperty: "section", rootData: s }) || (e = e === null ? Yc.errors : e.concat(Yc.errors), r = e.length);
        var P = f === r;
      }
      if (l.variant !== void 0) {
        let h = l.variant, f = r;
        Lo(h, { instancePath: o + "/variant", parentData: l, parentDataProperty: "variant", rootData: s }) || (e = e === null ? Lo.errors : e.concat(Lo.errors), r = e.length);
        var P = f === r;
      }
      if (l["markdown-headings"] !== void 0) {
        let h = l["markdown-headings"], f = r;
        ho(h, { instancePath: o + "/markdown-headings", parentData: l, parentDataProperty: "markdown-headings", rootData: s }) || (e = e === null ? ho.errors : e.concat(ho.errors), r = e.length);
        var P = f === r;
      }
      if (l["keep-yaml"] !== void 0) {
        let h = l["keep-yaml"], f = r;
        Mo(h, { instancePath: o + "/keep-yaml", parentData: l, parentDataProperty: "keep-yaml", rootData: s }) || (e = e === null ? Mo.errors : e.concat(Mo.errors), r = e.length);
        var P = f === r;
      }
      if (l["ipynb-output"] !== void 0) {
        let h = l["ipynb-output"], f = r;
        Xc(h, { instancePath: o + "/ipynb-output", parentData: l, parentDataProperty: "ipynb-output", rootData: s }) || (e = e === null ? Xc.errors : e.concat(Xc.errors), r = e.length);
        var P = f === r;
      }
      if (l.pdfa !== void 0) {
        let h = l.pdfa, f = r;
        Zc(h, { instancePath: o + "/pdfa", parentData: l, parentDataProperty: "pdfa", rootData: s }) || (e = e === null ? Zc.errors : e.concat(Zc.errors), r = e.length);
        var P = f === r;
      }
      if (l.pdfaiccprofile !== void 0) {
        let h = l.pdfaiccprofile, f = r;
        ep(h, { instancePath: o + "/pdfaiccprofile", parentData: l, parentDataProperty: "pdfaiccprofile", rootData: s }) || (e = e === null ? ep.errors : e.concat(ep.errors), r = e.length);
        var P = f === r;
      }
      if (l.pdfaintent !== void 0) {
        let h = l.pdfaintent, f = r;
        rp(h, { instancePath: o + "/pdfaintent", parentData: l, parentDataProperty: "pdfaintent", rootData: s }) || (e = e === null ? rp.errors : e.concat(rp.errors), r = e.length);
        var P = f === r;
      }
      if (l.bibliography !== void 0) {
        let h = l.bibliography, f = r;
        fr(h, { instancePath: o + "/bibliography", parentData: l, parentDataProperty: "bibliography", rootData: s }) || (e = e === null ? fr.errors : e.concat(fr.errors), r = e.length);
        var P = f === r;
      }
      if (l.csl !== void 0) {
        let h = l.csl, f = r;
        mr(h, { instancePath: o + "/csl", parentData: l, parentDataProperty: "csl", rootData: s }) || (e = e === null ? mr.errors : e.concat(mr.errors), r = e.length);
        var P = f === r;
      }
      if (l["citations-hover"] !== void 0) {
        let h = l["citations-hover"], f = r;
        Cn(h, { instancePath: o + "/citations-hover", parentData: l, parentDataProperty: "citations-hover", rootData: s }) || (e = e === null ? Cn.errors : e.concat(Cn.errors), r = e.length);
        var P = f === r;
      }
      if (l["citation-location"] !== void 0) {
        let h = l["citation-location"], f = r;
        Us(h, { instancePath: o + "/citation-location", parentData: l, parentDataProperty: "citation-location", rootData: s }) || (e = e === null ? Us.errors : e.concat(Us.errors), r = e.length);
        var P = f === r;
      }
      if (l["cite-method"] !== void 0) {
        let h = l["cite-method"], f = r;
        Ks(h, { instancePath: o + "/cite-method", parentData: l, parentDataProperty: "cite-method", rootData: s }) || (e = e === null ? Ks.errors : e.concat(Ks.errors), r = e.length);
        var P = f === r;
      }
      if (l.citeproc !== void 0) {
        let h = l.citeproc, f = r;
        gr(h, { instancePath: o + "/citeproc", parentData: l, parentDataProperty: "citeproc", rootData: s }) || (e = e === null ? gr.errors : e.concat(gr.errors), r = e.length);
        var P = f === r;
      }
      if (l.biblatexoptions !== void 0) {
        let h = l.biblatexoptions, f = r;
        Js(h, { instancePath: o + "/biblatexoptions", parentData: l, parentDataProperty: "biblatexoptions", rootData: s }) || (e = e === null ? Js.errors : e.concat(Js.errors), r = e.length);
        var P = f === r;
      }
      if (l.natbiboptions !== void 0) {
        let h = l.natbiboptions, f = r;
        Qs(h, { instancePath: o + "/natbiboptions", parentData: l, parentDataProperty: "natbiboptions", rootData: s }) || (e = e === null ? Qs.errors : e.concat(Qs.errors), r = e.length);
        var P = f === r;
      }
      if (l["biblio-style"] !== void 0) {
        let h = l["biblio-style"], f = r;
        Gs(h, { instancePath: o + "/biblio-style", parentData: l, parentDataProperty: "biblio-style", rootData: s }) || (e = e === null ? Gs.errors : e.concat(Gs.errors), r = e.length);
        var P = f === r;
      }
      if (l["biblio-title"] !== void 0) {
        let h = l["biblio-title"], f = r;
        Ys(h, { instancePath: o + "/biblio-title", parentData: l, parentDataProperty: "biblio-title", rootData: s }) || (e = e === null ? Ys.errors : e.concat(Ys.errors), r = e.length);
        var P = f === r;
      }
      if (l["citation-abbreviations"] !== void 0) {
        let h = l["citation-abbreviations"], f = r;
        hr(h, { instancePath: o + "/citation-abbreviations", parentData: l, parentDataProperty: "citation-abbreviations", rootData: s }) || (e = e === null ? hr.errors : e.concat(hr.errors), r = e.length);
        var P = f === r;
      }
      if (l.from !== void 0) {
        let h = l.from, f = r;
        H(h, { instancePath: o + "/from", parentData: l, parentDataProperty: "from", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
        var P = f === r;
      }
      if (l.reader !== void 0) {
        let h = l.reader, f = r;
        H(h, { instancePath: o + "/reader", parentData: l, parentDataProperty: "reader", rootData: s }) || (e = e === null ? H.errors : e.concat(H.errors), r = e.length);
        var P = f === r;
      }
      if (l["output-file"] !== void 0) {
        let h = l["output-file"], f = r;
        yr(h, { instancePath: o + "/output-file", parentData: l, parentDataProperty: "output-file", rootData: s }) || (e = e === null ? yr.errors : e.concat(yr.errors), r = e.length);
        var P = f === r;
      }
      if (l["output-ext"] !== void 0) {
        let h = l["output-ext"], f = r;
        br(h, { instancePath: o + "/output-ext", parentData: l, parentDataProperty: "output-ext", rootData: s }) || (e = e === null ? br.errors : e.concat(br.errors), r = e.length);
        var P = f === r;
      }
      if (l.template !== void 0) {
        let h = l.template, f = r;
        Sr(h, { instancePath: o + "/template", parentData: l, parentDataProperty: "template", rootData: s }) || (e = e === null ? Sr.errors : e.concat(Sr.errors), r = e.length);
        var P = f === r;
      }
      if (l.standalone !== void 0) {
        let h = l.standalone, f = r;
        _r(h, { instancePath: o + "/standalone", parentData: l, parentDataProperty: "standalone", rootData: s }) || (e = e === null ? _r.errors : e.concat(_r.errors), r = e.length);
        var P = f === r;
      }
      if (l["self-contained"] !== void 0) {
        let h = l["self-contained"], f = r;
        In(h, { instancePath: o + "/self-contained", parentData: l, parentDataProperty: "self-contained", rootData: s }) || (e = e === null ? In.errors : e.concat(In.errors), r = e.length);
        var P = f === r;
      }
      if (l["self-contained-math"] !== void 0) {
        let h = l["self-contained-math"], f = r;
        Hn(h, { instancePath: o + "/self-contained-math", parentData: l, parentDataProperty: "self-contained-math", rootData: s }) || (e = e === null ? Hn.errors : e.concat(Hn.errors), r = e.length);
        var P = f === r;
      }
      if (l.filters !== void 0) {
        let h = l.filters, f = r;
        vr(h, { instancePath: o + "/filters", parentData: l, parentDataProperty: "filters", rootData: s }) || (e = e === null ? vr.errors : e.concat(vr.errors), r = e.length);
        var P = f === r;
      }
      if (l["keep-md"] !== void 0) {
        let h = l["keep-md"], f = r;
        Be(h, { instancePath: o + "/keep-md", parentData: l, parentDataProperty: "keep-md", rootData: s }) || (e = e === null ? Be.errors : e.concat(Be.errors), r = e.length);
        var P = f === r;
      }
      if (l["keep-ipynb"] !== void 0) {
        let h = l["keep-ipynb"], f = r;
        je(h, { instancePath: o + "/keep-ipynb", parentData: l, parentDataProperty: "keep-ipynb", rootData: s }) || (e = e === null ? je.errors : e.concat(je.errors), r = e.length);
        var P = f === r;
      }
      if (l["keep-tex"] !== void 0) {
        let h = l["keep-tex"], f = r;
        Vl(h, { instancePath: o + "/keep-tex", parentData: l, parentDataProperty: "keep-tex", rootData: s }) || (e = e === null ? Vl.errors : e.concat(Vl.errors), r = e.length);
        var P = f === r;
      }
      if (l["extract-media"] !== void 0) {
        let h = l["extract-media"], f = r;
        qr(h, { instancePath: o + "/extract-media", parentData: l, parentDataProperty: "extract-media", rootData: s }) || (e = e === null ? qr.errors : e.concat(qr.errors), r = e.length);
        var P = f === r;
      }
      if (l["resource-path"] !== void 0) {
        let h = l["resource-path"], f = r;
        Or(h, { instancePath: o + "/resource-path", parentData: l, parentDataProperty: "resource-path", rootData: s }) || (e = e === null ? Or.errors : e.concat(Or.errors), r = e.length);
        var P = f === r;
      }
      if (l["default-image-extension"] !== void 0) {
        let h = l["default-image-extension"], f = r;
        Pr(h, { instancePath: o + "/default-image-extension", parentData: l, parentDataProperty: "default-image-extension", rootData: s }) || (e = e === null ? Pr.errors : e.concat(Pr.errors), r = e.length);
        var P = f === r;
      }
      if (l.abbreviations !== void 0) {
        let h = l.abbreviations, f = r;
        xr(h, { instancePath: o + "/abbreviations", parentData: l, parentDataProperty: "abbreviations", rootData: s }) || (e = e === null ? xr.errors : e.concat(xr.errors), r = e.length);
        var P = f === r;
      }
      if (l.dpi !== void 0) {
        let h = l.dpi, f = r;
        kr(h, { instancePath: o + "/dpi", parentData: l, parentDataProperty: "dpi", rootData: s }) || (e = e === null ? kr.errors : e.concat(kr.errors), r = e.length);
        var P = f === r;
      }
      if (l.scrollable !== void 0) {
        let h = l.scrollable, f = r;
        np(h, { instancePath: o + "/scrollable", parentData: l, parentDataProperty: "scrollable", rootData: s }) || (e = e === null ? np.errors : e.concat(np.errors), r = e.length);
        var P = f === r;
      }
      if (l.smaller !== void 0) {
        let h = l.smaller, f = r;
        op(h, { instancePath: o + "/smaller", parentData: l, parentDataProperty: "smaller", rootData: s }) || (e = e === null ? op.errors : e.concat(op.errors), r = e.length);
        var P = f === r;
      }
      if (l.embedded !== void 0) {
        let h = l.embedded, f = r;
        sp(h, { instancePath: o + "/embedded", parentData: l, parentDataProperty: "embedded", rootData: s }) || (e = e === null ? sp.errors : e.concat(sp.errors), r = e.length);
        var P = f === r;
      }
      if (l.display !== void 0) {
        let h = l.display, f = r;
        ip(h, { instancePath: o + "/display", parentData: l, parentDataProperty: "display", rootData: s }) || (e = e === null ? ip.errors : e.concat(ip.errors), r = e.length);
        var P = f === r;
      }
      if (l["auto-stretch"] !== void 0) {
        let h = l["auto-stretch"], f = r;
        lp(h, { instancePath: o + "/auto-stretch", parentData: l, parentDataProperty: "auto-stretch", rootData: s }) || (e = e === null ? lp.errors : e.concat(lp.errors), r = e.length);
        var P = f === r;
      }
      if (l.width !== void 0) {
        let h = l.width, f = r;
        cp(h, { instancePath: o + "/width", parentData: l, parentDataProperty: "width", rootData: s }) || (e = e === null ? cp.errors : e.concat(cp.errors), r = e.length);
        var P = f === r;
      }
      if (l.height !== void 0) {
        let h = l.height, f = r;
        pp(h, { instancePath: o + "/height", parentData: l, parentDataProperty: "height", rootData: s }) || (e = e === null ? pp.errors : e.concat(pp.errors), r = e.length);
        var P = f === r;
      }
      if (l.margin !== void 0) {
        let h = l.margin, f = r;
        up(h, { instancePath: o + "/margin", parentData: l, parentDataProperty: "margin", rootData: s }) || (e = e === null ? up.errors : e.concat(up.errors), r = e.length);
        var P = f === r;
      }
      if (l["min-scale"] !== void 0) {
        let h = l["min-scale"], f = r;
        dp(h, { instancePath: o + "/min-scale", parentData: l, parentDataProperty: "min-scale", rootData: s }) || (e = e === null ? dp.errors : e.concat(dp.errors), r = e.length);
        var P = f === r;
      }
      if (l["max-scale"] !== void 0) {
        let h = l["max-scale"], f = r;
        fp(h, { instancePath: o + "/max-scale", parentData: l, parentDataProperty: "max-scale", rootData: s }) || (e = e === null ? fp.errors : e.concat(fp.errors), r = e.length);
        var P = f === r;
      }
      if (l.center !== void 0) {
        let h = l.center, f = r;
        mp(h, { instancePath: o + "/center", parentData: l, parentDataProperty: "center", rootData: s }) || (e = e === null ? mp.errors : e.concat(mp.errors), r = e.length);
        var P = f === r;
      }
      if (l["disable-layout"] !== void 0) {
        let h = l["disable-layout"], f = r;
        gp(h, { instancePath: o + "/disable-layout", parentData: l, parentDataProperty: "disable-layout", rootData: s }) || (e = e === null ? gp.errors : e.concat(gp.errors), r = e.length);
        var P = f === r;
      }
      if (l["preview-links"] !== void 0) {
        let h = l["preview-links"], f = r;
        hp(h, { instancePath: o + "/preview-links", parentData: l, parentDataProperty: "preview-links", rootData: s }) || (e = e === null ? hp.errors : e.concat(hp.errors), r = e.length);
        var P = f === r;
      }
      if (l["auto-play-media"] !== void 0) {
        let h = l["auto-play-media"], f = r;
        yp(h, { instancePath: o + "/auto-play-media", parentData: l, parentDataProperty: "auto-play-media", rootData: s }) || (e = e === null ? yp.errors : e.concat(yp.errors), r = e.length);
        var P = f === r;
      }
      if (l["preload-iframes"] !== void 0) {
        let h = l["preload-iframes"], f = r;
        bp(h, { instancePath: o + "/preload-iframes", parentData: l, parentDataProperty: "preload-iframes", rootData: s }) || (e = e === null ? bp.errors : e.concat(bp.errors), r = e.length);
        var P = f === r;
      }
      if (l["view-distance"] !== void 0) {
        let h = l["view-distance"], f = r;
        _p(h, { instancePath: o + "/view-distance", parentData: l, parentDataProperty: "view-distance", rootData: s }) || (e = e === null ? _p.errors : e.concat(_p.errors), r = e.length);
        var P = f === r;
      }
      if (l["mobile-view-distance"] !== void 0) {
        let h = l["mobile-view-distance"], f = r;
        vp(h, { instancePath: o + "/mobile-view-distance", parentData: l, parentDataProperty: "mobile-view-distance", rootData: s }) || (e = e === null ? vp.errors : e.concat(vp.errors), r = e.length);
        var P = f === r;
      }
      if (l["parallax-background-image"] !== void 0) {
        let h = l["parallax-background-image"], f = r;
        qp(h, { instancePath: o + "/parallax-background-image", parentData: l, parentDataProperty: "parallax-background-image", rootData: s }) || (e = e === null ? qp.errors : e.concat(qp.errors), r = e.length);
        var P = f === r;
      }
      if (l["parallax-background-size"] !== void 0) {
        let h = l["parallax-background-size"], f = r;
        Op(h, { instancePath: o + "/parallax-background-size", parentData: l, parentDataProperty: "parallax-background-size", rootData: s }) || (e = e === null ? Op.errors : e.concat(Op.errors), r = e.length);
        var P = f === r;
      }
      if (l["parallax-background-horizontal"] !== void 0) {
        let h = l["parallax-background-horizontal"], f = r;
        Pp(h, { instancePath: o + "/parallax-background-horizontal", parentData: l, parentDataProperty: "parallax-background-horizontal", rootData: s }) || (e = e === null ? Pp.errors : e.concat(Pp.errors), r = e.length);
        var P = f === r;
      }
      if (l["parallax-background-vertical"] !== void 0) {
        let h = l["parallax-background-vertical"], f = r;
        xp(h, { instancePath: o + "/parallax-background-vertical", parentData: l, parentDataProperty: "parallax-background-vertical", rootData: s }) || (e = e === null ? xp.errors : e.concat(xp.errors), r = e.length);
        var P = f === r;
      }
      if (l.progress !== void 0) {
        let h = l.progress, f = r;
        kp(h, { instancePath: o + "/progress", parentData: l, parentDataProperty: "progress", rootData: s }) || (e = e === null ? kp.errors : e.concat(kp.errors), r = e.length);
        var P = f === r;
      }
      if (l.history !== void 0) {
        let h = l.history, f = r;
        Dp(h, { instancePath: o + "/history", parentData: l, parentDataProperty: "history", rootData: s }) || (e = e === null ? Dp.errors : e.concat(Dp.errors), r = e.length);
        var P = f === r;
      }
      if (l["navigation-mode"] !== void 0) {
        let h = l["navigation-mode"], f = r;
        wp(h, { instancePath: o + "/navigation-mode", parentData: l, parentDataProperty: "navigation-mode", rootData: s }) || (e = e === null ? wp.errors : e.concat(wp.errors), r = e.length);
        var P = f === r;
      }
      if (l.touch !== void 0) {
        let h = l.touch, f = r;
        $p(h, { instancePath: o + "/touch", parentData: l, parentDataProperty: "touch", rootData: s }) || (e = e === null ? $p.errors : e.concat($p.errors), r = e.length);
        var P = f === r;
      }
      if (l.keyboard !== void 0) {
        let h = l.keyboard, f = r;
        Sp(h, { instancePath: o + "/keyboard", parentData: l, parentDataProperty: "keyboard", rootData: s }) || (e = e === null ? Sp.errors : e.concat(Sp.errors), r = e.length);
        var P = f === r;
      }
      if (l["mouse-wheel"] !== void 0) {
        let h = l["mouse-wheel"], f = r;
        Fp(h, { instancePath: o + "/mouse-wheel", parentData: l, parentDataProperty: "mouse-wheel", rootData: s }) || (e = e === null ? Fp.errors : e.concat(Fp.errors), r = e.length);
        var P = f === r;
      }
      if (l["hide-inactive-cursor"] !== void 0) {
        let h = l["hide-inactive-cursor"], f = r;
        Ap(h, { instancePath: o + "/hide-inactive-cursor", parentData: l, parentDataProperty: "hide-inactive-cursor", rootData: s }) || (e = e === null ? Ap.errors : e.concat(Ap.errors), r = e.length);
        var P = f === r;
      }
      if (l["hide-cursor-time"] !== void 0) {
        let h = l["hide-cursor-time"], f = r;
        Bp(h, { instancePath: o + "/hide-cursor-time", parentData: l, parentDataProperty: "hide-cursor-time", rootData: s }) || (e = e === null ? Bp.errors : e.concat(Bp.errors), r = e.length);
        var P = f === r;
      }
      if (l.loop !== void 0) {
        let h = l.loop, f = r;
        jp(h, { instancePath: o + "/loop", parentData: l, parentDataProperty: "loop", rootData: s }) || (e = e === null ? jp.errors : e.concat(jp.errors), r = e.length);
        var P = f === r;
      }
      if (l.shuffle !== void 0) {
        let h = l.shuffle, f = r;
        Ep(h, { instancePath: o + "/shuffle", parentData: l, parentDataProperty: "shuffle", rootData: s }) || (e = e === null ? Ep.errors : e.concat(Ep.errors), r = e.length);
        var P = f === r;
      }
      if (l.controls !== void 0) {
        let h = l.controls, f = r;
        Np(h, { instancePath: o + "/controls", parentData: l, parentDataProperty: "controls", rootData: s }) || (e = e === null ? Np.errors : e.concat(Np.errors), r = e.length);
        var P = f === r;
      }
      if (l["controls-layout"] !== void 0) {
        let h = l["controls-layout"], f = r;
        Cp(h, { instancePath: o + "/controls-layout", parentData: l, parentDataProperty: "controls-layout", rootData: s }) || (e = e === null ? Cp.errors : e.concat(Cp.errors), r = e.length);
        var P = f === r;
      }
      if (l["controls-tutorial"] !== void 0) {
        let h = l["controls-tutorial"], f = r;
        Ip(h, { instancePath: o + "/controls-tutorial", parentData: l, parentDataProperty: "controls-tutorial", rootData: s }) || (e = e === null ? Ip.errors : e.concat(Ip.errors), r = e.length);
        var P = f === r;
      }
      if (l["controls-back-arrows"] !== void 0) {
        let h = l["controls-back-arrows"], f = r;
        Hp(h, { instancePath: o + "/controls-back-arrows", parentData: l, parentDataProperty: "controls-back-arrows", rootData: s }) || (e = e === null ? Hp.errors : e.concat(Hp.errors), r = e.length);
        var P = f === r;
      }
      if (l["auto-slide"] !== void 0) {
        let h = l["auto-slide"], f = r;
        Lp(h, { instancePath: o + "/auto-slide", parentData: l, parentDataProperty: "auto-slide", rootData: s }) || (e = e === null ? Lp.errors : e.concat(Lp.errors), r = e.length);
        var P = f === r;
      }
      if (l["auto-slide-stoppable"] !== void 0) {
        let h = l["auto-slide-stoppable"], f = r;
        Mp(h, { instancePath: o + "/auto-slide-stoppable", parentData: l, parentDataProperty: "auto-slide-stoppable", rootData: s }) || (e = e === null ? Mp.errors : e.concat(Mp.errors), r = e.length);
        var P = f === r;
      }
      if (l["auto-slide-method"] !== void 0) {
        let h = l["auto-slide-method"], f = r;
        Tp(h, { instancePath: o + "/auto-slide-method", parentData: l, parentDataProperty: "auto-slide-method", rootData: s }) || (e = e === null ? Tp.errors : e.concat(Tp.errors), r = e.length);
        var P = f === r;
      }
      if (l["default-timing"] !== void 0) {
        let h = l["default-timing"], f = r;
        zp(h, { instancePath: o + "/default-timing", parentData: l, parentDataProperty: "default-timing", rootData: s }) || (e = e === null ? zp.errors : e.concat(zp.errors), r = e.length);
        var P = f === r;
      }
      if (l.pause !== void 0) {
        let h = l.pause, f = r;
        Wp(h, { instancePath: o + "/pause", parentData: l, parentDataProperty: "pause", rootData: s }) || (e = e === null ? Wp.errors : e.concat(Wp.errors), r = e.length);
        var P = f === r;
      }
      if (l.help !== void 0) {
        let h = l.help, f = r;
        Rp(h, { instancePath: o + "/help", parentData: l, parentDataProperty: "help", rootData: s }) || (e = e === null ? Rp.errors : e.concat(Rp.errors), r = e.length);
        var P = f === r;
      }
      if (l.hash !== void 0) {
        let h = l.hash, f = r;
        Vp(h, { instancePath: o + "/hash", parentData: l, parentDataProperty: "hash", rootData: s }) || (e = e === null ? Vp.errors : e.concat(Vp.errors), r = e.length);
        var P = f === r;
      }
      if (l["hash-type"] !== void 0) {
        let h = l["hash-type"], f = r;
        Up(h, { instancePath: o + "/hash-type", parentData: l, parentDataProperty: "hash-type", rootData: s }) || (e = e === null ? Up.errors : e.concat(Up.errors), r = e.length);
        var P = f === r;
      }
      if (l["hash-one-based-index"] !== void 0) {
        let h = l["hash-one-based-index"], f = r;
        Kp(h, { instancePath: o + "/hash-one-based-index", parentData: l, parentDataProperty: "hash-one-based-index", rootData: s }) || (e = e === null ? Kp.errors : e.concat(Kp.errors), r = e.length);
        var P = f === r;
      }
      if (l["respond-to-hash-changes"] !== void 0) {
        let h = l["respond-to-hash-changes"], f = r;
        Jp(h, { instancePath: o + "/respond-to-hash-changes", parentData: l, parentDataProperty: "respond-to-hash-changes", rootData: s }) || (e = e === null ? Jp.errors : e.concat(Jp.errors), r = e.length);
        var P = f === r;
      }
      if (l["fragment-in-url"] !== void 0) {
        let h = l["fragment-in-url"], f = r;
        Qp(h, { instancePath: o + "/fragment-in-url", parentData: l, parentDataProperty: "fragment-in-url", rootData: s }) || (e = e === null ? Qp.errors : e.concat(Qp.errors), r = e.length);
        var P = f === r;
      }
      if (l["slide-tone"] !== void 0) {
        let h = l["slide-tone"], f = r;
        Gp(h, { instancePath: o + "/slide-tone", parentData: l, parentDataProperty: "slide-tone", rootData: s }) || (e = e === null ? Gp.errors : e.concat(Gp.errors), r = e.length);
        var P = f === r;
      }
      if (l["pdf-separate-fragments"] !== void 0) {
        let h = l["pdf-separate-fragments"], f = r;
        Yp(h, { instancePath: o + "/pdf-separate-fragments", parentData: l, parentDataProperty: "pdf-separate-fragments", rootData: s }) || (e = e === null ? Yp.errors : e.concat(Yp.errors), r = e.length);
        var P = f === r;
      }
      if (l["pdf-page-height-offset"] !== void 0) {
        let h = l["pdf-page-height-offset"], f = r;
        Xp(h, { instancePath: o + "/pdf-page-height-offset", parentData: l, parentDataProperty: "pdf-page-height-offset", rootData: s }) || (e = e === null ? Xp.errors : e.concat(Xp.errors), r = e.length);
        var P = f === r;
      }
      if (l.overview !== void 0) {
        let h = l.overview, f = r;
        Zp(h, { instancePath: o + "/overview", parentData: l, parentDataProperty: "overview", rootData: s }) || (e = e === null ? Zp.errors : e.concat(Zp.errors), r = e.length);
        var P = f === r;
      }
      if (l.menu !== void 0) {
        let h = l.menu, f = r;
        eu(h, { instancePath: o + "/menu", parentData: l, parentDataProperty: "menu", rootData: s }) || (e = e === null ? eu.errors : e.concat(eu.errors), r = e.length);
        var P = f === r;
      }
      if (l.chalkboard !== void 0) {
        let h = l.chalkboard, f = r;
        ru(h, { instancePath: o + "/chalkboard", parentData: l, parentDataProperty: "chalkboard", rootData: s }) || (e = e === null ? ru.errors : e.concat(ru.errors), r = e.length);
        var P = f === r;
      }
      if (l.multiplex !== void 0) {
        let h = l.multiplex, f = r;
        tu(h, { instancePath: o + "/multiplex", parentData: l, parentDataProperty: "multiplex", rootData: s }) || (e = e === null ? tu.errors : e.concat(tu.errors), r = e.length);
        var P = f === r;
      }
      if (l.transition !== void 0) {
        let h = l.transition, f = r;
        au(h, { instancePath: o + "/transition", parentData: l, parentDataProperty: "transition", rootData: s }) || (e = e === null ? au.errors : e.concat(au.errors), r = e.length);
        var P = f === r;
      }
      if (l["transition-speed"] !== void 0) {
        let h = l["transition-speed"], f = r;
        nu(h, { instancePath: o + "/transition-speed", parentData: l, parentDataProperty: "transition-speed", rootData: s }) || (e = e === null ? nu.errors : e.concat(nu.errors), r = e.length);
        var P = f === r;
      }
      if (l["background-transition"] !== void 0) {
        let h = l["background-transition"], f = r;
        ou(h, { instancePath: o + "/background-transition", parentData: l, parentDataProperty: "background-transition", rootData: s }) || (e = e === null ? ou.errors : e.concat(ou.errors), r = e.length);
        var P = f === r;
      }
      if (l.fragments !== void 0) {
        let h = l.fragments, f = r;
        su(h, { instancePath: o + "/fragments", parentData: l, parentDataProperty: "fragments", rootData: s }) || (e = e === null ? su.errors : e.concat(su.errors), r = e.length);
        var P = f === r;
      }
      if (l["auto-animate"] !== void 0) {
        let h = l["auto-animate"], f = r;
        iu(h, { instancePath: o + "/auto-animate", parentData: l, parentDataProperty: "auto-animate", rootData: s }) || (e = e === null ? iu.errors : e.concat(iu.errors), r = e.length);
        var P = f === r;
      }
      if (l["auto-animate-easing"] !== void 0) {
        let h = l["auto-animate-easing"], f = r;
        lu(h, { instancePath: o + "/auto-animate-easing", parentData: l, parentDataProperty: "auto-animate-easing", rootData: s }) || (e = e === null ? lu.errors : e.concat(lu.errors), r = e.length);
        var P = f === r;
      }
      if (l["auto-animate-duration"] !== void 0) {
        let h = l["auto-animate-duration"], f = r;
        cu(h, { instancePath: o + "/auto-animate-duration", parentData: l, parentDataProperty: "auto-animate-duration", rootData: s }) || (e = e === null ? cu.errors : e.concat(cu.errors), r = e.length);
        var P = f === r;
      }
      if (l["auto-animate-unmatched"] !== void 0) {
        let h = l["auto-animate-unmatched"], f = r;
        pu(h, { instancePath: o + "/auto-animate-unmatched", parentData: l, parentDataProperty: "auto-animate-unmatched", rootData: s }) || (e = e === null ? pu.errors : e.concat(pu.errors), r = e.length);
        var P = f === r;
      }
      if (l["auto-animate-styles"] !== void 0) {
        let h = l["auto-animate-styles"], f = r;
        du(h, { instancePath: o + "/auto-animate-styles", parentData: l, parentDataProperty: "auto-animate-styles", rootData: s }) || (e = e === null ? du.errors : e.concat(du.errors), r = e.length);
        var P = f === r;
      }
      if (l.incremental !== void 0) {
        let h = l.incremental, f = r;
        Rn(h, { instancePath: o + "/incremental", parentData: l, parentDataProperty: "incremental", rootData: s }) || (e = e === null ? Rn.errors : e.concat(Rn.errors), r = e.length);
        var P = f === r;
      }
      if (l["slide-level"] !== void 0) {
        let h = l["slide-level"], f = r;
        Vn(h, { instancePath: o + "/slide-level", parentData: l, parentDataProperty: "slide-level", rootData: s }) || (e = e === null ? Vn.errors : e.concat(Vn.errors), r = e.length);
        var P = f === r;
      }
      if (l["slide-number"] !== void 0) {
        let h = l["slide-number"], f = r;
        fu(h, { instancePath: o + "/slide-number", parentData: l, parentDataProperty: "slide-number", rootData: s }) || (e = e === null ? fu.errors : e.concat(fu.errors), r = e.length);
        var P = f === r;
      }
      if (l["show-slide-number"] !== void 0) {
        let h = l["show-slide-number"], f = r;
        mu(h, { instancePath: o + "/show-slide-number", parentData: l, parentDataProperty: "show-slide-number", rootData: s }) || (e = e === null ? mu.errors : e.concat(mu.errors), r = e.length);
        var P = f === r;
      }
      if (l["title-slide-attributes"] !== void 0) {
        let h = l["title-slide-attributes"], f = r;
        gu(h, { instancePath: o + "/title-slide-attributes", parentData: l, parentDataProperty: "title-slide-attributes", rootData: s }) || (e = e === null ? gu.errors : e.concat(gu.errors), r = e.length);
        var P = f === r;
      }
      if (l["center-title-slide"] !== void 0) {
        let h = l["center-title-slide"], f = r;
        hu(h, { instancePath: o + "/center-title-slide", parentData: l, parentDataProperty: "center-title-slide", rootData: s }) || (e = e === null ? hu.errors : e.concat(hu.errors), r = e.length);
        var P = f === r;
      }
      if (l["show-notes"] !== void 0) {
        let h = l["show-notes"], f = r;
        yu(h, { instancePath: o + "/show-notes", parentData: l, parentDataProperty: "show-notes", rootData: s }) || (e = e === null ? yu.errors : e.concat(yu.errors), r = e.length);
        var P = f === r;
      }
      if (l.rtl !== void 0) {
        let h = l.rtl, f = r;
        bu(h, { instancePath: o + "/rtl", parentData: l, parentDataProperty: "rtl", rootData: s }) || (e = e === null ? bu.errors : e.concat(bu.errors), r = e.length);
        var P = f === r;
      }
      if (l.wrap !== void 0) {
        let h = l.wrap, f = r;
        it(h, { instancePath: o + "/wrap", parentData: l, parentDataProperty: "wrap", rootData: s }) || (e = e === null ? it.errors : e.concat(it.errors), r = e.length);
        var P = f === r;
      }
      if (l.columns !== void 0) {
        let h = l.columns, f = r;
        lt(h, { instancePath: o + "/columns", parentData: l, parentDataProperty: "columns", rootData: s }) || (e = e === null ? lt.errors : e.concat(lt.errors), r = e.length);
        var P = f === r;
      }
      if (l["tab-stop"] !== void 0) {
        let h = l["tab-stop"], f = r;
        ct(h, { instancePath: o + "/tab-stop", parentData: l, parentDataProperty: "tab-stop", rootData: s }) || (e = e === null ? ct.errors : e.concat(ct.errors), r = e.length);
        var P = f === r;
      }
      if (l["preserve-tabs"] !== void 0) {
        let h = l["preserve-tabs"], f = r;
        pt(h, { instancePath: o + "/preserve-tabs", parentData: l, parentDataProperty: "preserve-tabs", rootData: s }) || (e = e === null ? pt.errors : e.concat(pt.errors), r = e.length);
        var P = f === r;
      }
      if (l.eol !== void 0) {
        let h = l.eol, f = r;
        ut(h, { instancePath: o + "/eol", parentData: l, parentDataProperty: "eol", rootData: s }) || (e = e === null ? ut.errors : e.concat(ut.errors), r = e.length);
        var P = f === r;
      }
      if (l["strip-comments"] !== void 0) {
        let h = l["strip-comments"], f = r;
        Ra(h, { instancePath: o + "/strip-comments", parentData: l, parentDataProperty: "strip-comments", rootData: s }) || (e = e === null ? Ra.errors : e.concat(Ra.errors), r = e.length);
        var P = f === r;
      }
      if (l.ascii !== void 0) {
        let h = l.ascii, f = r;
        da(h, { instancePath: o + "/ascii", parentData: l, parentDataProperty: "ascii", rootData: s }) || (e = e === null ? da.errors : e.concat(da.errors), r = e.length);
        var P = f === r;
      }
      if (l.toc !== void 0) {
        let h = l.toc, f = r;
        K(h, { instancePath: o + "/toc", parentData: l, parentDataProperty: "toc", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
        var P = f === r;
      }
      if (l["table-of-contents"] !== void 0) {
        let h = l["table-of-contents"], f = r;
        K(h, { instancePath: o + "/table-of-contents", parentData: l, parentDataProperty: "table-of-contents", rootData: s }) || (e = e === null ? K.errors : e.concat(K.errors), r = e.length);
        var P = f === r;
      }
      if (l["toc-depth"] !== void 0) {
        let h = l["toc-depth"], f = r;
        Wr(h, { instancePath: o + "/toc-depth", parentData: l, parentDataProperty: "toc-depth", rootData: s }) || (e = e === null ? Wr.errors : e.concat(Wr.errors), r = e.length);
        var P = f === r;
      }
      if (l["toc-location"] !== void 0) {
        let h = l["toc-location"], f = r;
        Xs(h, { instancePath: o + "/toc-location", parentData: l, parentDataProperty: "toc-location", rootData: s }) || (e = e === null ? Xs.errors : e.concat(Xs.errors), r = e.length);
        var P = f === r;
      }
      if (l["toc-title"] !== void 0) {
        let h = l["toc-title"], f = r;
        La(h, { instancePath: o + "/toc-title", parentData: l, parentDataProperty: "toc-title", rootData: s }) || (e = e === null ? La.errors : e.concat(La.errors), r = e.length);
        var P = f === r;
      }
      if (l.lof !== void 0) {
        let h = l.lof, f = r;
        Zs(h, { instancePath: o + "/lof", parentData: l, parentDataProperty: "lof", rootData: s }) || (e = e === null ? Zs.errors : e.concat(Zs.errors), r = e.length);
        var P = f === r;
      }
      if (l.lot !== void 0) {
        let h = l.lot, f = r;
        ei(h, { instancePath: o + "/lot", parentData: l, parentDataProperty: "lot", rootData: s }) || (e = e === null ? ei.errors : e.concat(ei.errors), r = e.length);
        var P = f === r;
      }
      if (l.search !== void 0) {
        let h = l.search, f = r;
        ri(h, { instancePath: o + "/search", parentData: l, parentDataProperty: "search", rootData: s }) || (e = e === null ? ri.errors : e.concat(ri.errors), r = e.length);
        var P = f === r;
      }
      if (l.aliases !== void 0) {
        let h = l.aliases, f = r;
        ti(h, { instancePath: o + "/aliases", parentData: l, parentDataProperty: "aliases", rootData: s }) || (e = e === null ? ti.errors : e.concat(ti.errors), r = e.length);
        var P = f === r;
      }
      if (l.image !== void 0) {
        let h = l.image, f = r;
        ai(h, { instancePath: o + "/image", parentData: l, parentDataProperty: "image", rootData: s }) || (e = e === null ? ai.errors : e.concat(ai.errors), r = e.length);
        var P = f === r;
      }
      if (l["image-height"] !== void 0) {
        let h = l["image-height"], f = r;
        ni(h, { instancePath: o + "/image-height", parentData: l, parentDataProperty: "image-height", rootData: s }) || (e = e === null ? ni.errors : e.concat(ni.errors), r = e.length);
        var P = f === r;
      }
      if (l["image-width"] !== void 0) {
        let h = l["image-width"], f = r;
        oi(h, { instancePath: o + "/image-width", parentData: l, parentDataProperty: "image-width", rootData: s }) || (e = e === null ? oi.errors : e.concat(oi.errors), r = e.length);
        var P = f === r;
      }
    } else {
      let h = { instancePath: o, schemaPath: "#/oneOf/1/allOf/1/type", keyword: "type", params: { type: "object", schema: yo.oneOf[1].allOf[1] }, message: "must be object", schema: yo.oneOf[1].allOf[1].type, parentSchema: yo.oneOf[1].allOf[1], data: l };
      e === null ? e = [h] : e.push(h), r++;
    }
    var R = j === r;
    let oe = r;
    si(l, { instancePath: o, parentData: S, parentDataProperty: F, rootData: s }) || (e = e === null ? si.errors : e.concat(si.errors), r = e.length);
    var R = oe === r, g = x === r;
    if (g && c ? (c = false, d = [d, 1]) : g && (c = true, d = 1), c)
      r = _, e !== null && (_ ? e.length = _ : e = null);
    else {
      let h = { instancePath: o, schemaPath: "#/oneOf", keyword: "oneOf", params: { passingSchemas: d, schema: yo }, message: "must match exactly one schema in oneOf", schema: yo.oneOf, parentSchema: yo, data: l };
      e === null ? e = [h] : e.push(h), r++;
    }
    return _m.errors = e, r === 0;
  }
  B["project-config-fields"] = vm;
  var b = { type: "object", description: "be an object", properties: { project: { type: "object", description: "be an object", properties: { type: { enum: ["website", "book", "site"], description: "be one of: `website`, `book`, `site`" }, render: { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }, "execute-dir": { enum: ["file", "project"], description: "be one of: `file`, `project`" }, "output-dir": { type: "string", description: "be a string" }, "lib-dir": { type: "string", description: "be a string" }, resources: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(execute_dir|executeDir|output_dir|outputDir|lib_dir|libDir))" } }, website: { $ref: "base-website", description: "be base-website" }, book: { type: "object", description: "be an object", properties: { title: { type: "string", description: "be a string" }, description: { type: "string", description: "be a string" }, "fav-icon": { type: "string", description: "be a string" }, "site-url": { type: "string", description: "be a string" }, "site-path": { type: "string", description: "be a string" }, "repo-url": { type: "string", description: "be a string" }, "repo-branch": { type: "string", description: "be a string" }, "repo-actions": { oneOf: [{ enum: ["none", "edit", "source", "issue"], description: "be one of: `none`, `edit`, `source`, `issue`" }, { type: "array", description: "be an array of values, where each element should be one of: `none`, `edit`, `source`, `issue`", items: { enum: ["none", "edit", "source", "issue"], description: "be one of: `none`, `edit`, `source`, `issue`" } }], description: "be exactly one of: one of: `none`, `edit`, `source`, `issue`, an array of values, where each element should be one of: `none`, `edit`, `source`, `issue`" }, "google-analytics": { anyOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: { "tracking-id": { type: "string", description: "be a string" }, storage: { enum: ["cookies", "none"], description: "be one of: `cookies`, `none`" }, "anonymize-ip": { type: "boolean", description: "be `true` or `false`" }, version: { enum: [3, 4], description: "be one of: `3`, `4`" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(tracking_id|trackingId|anonymize_ip|anonymizeIp))" } }], description: "be at least one of: a string, an object" }, "cookie-consent": { anyOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { type: { enum: ["implied", "express"], description: "be one of: `implied`, `express`" }, style: { enum: ["simple", "headline", "interstitial", "standalone"], description: "be one of: `simple`, `headline`, `interstitial`, `standalone`" }, palette: { enum: ["light", "dark"], description: "be one of: `light`, `dark`" }, "policy-url": { type: "string", description: "be a string" }, "prefs-text": { type: "string", description: "be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(policy_url|policyUrl|prefs_text|prefsText))" } }], description: "be at least one of: `true` or `false`, an object" }, search: { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { location: { enum: ["navbar", "sidebar"], description: "be one of: `navbar`, `sidebar`" }, type: { enum: ["overlay", "textbox"], description: "be one of: `overlay`, `textbox`" }, limit: { type: "number", description: "be a number" }, "collapse-after": { type: "number", description: "be a number" }, "copy-button": { type: "boolean", description: "be `true` or `false`" }, algolia: { type: "object", description: "be an object", properties: { "index-name": { type: "string", description: "be a string" }, "application-id": { type: "string", description: "be a string" }, "search-only-api-key": { type: "string", description: "be a string" }, "analytics-events": { type: "boolean", description: "be `true` or `false`" }, "show-logo": { type: "boolean", description: "be `true` or `false`" }, "index-fields": { type: "object", description: "be an object", properties: { href: { type: "string", description: "be a string" }, title: { type: "string", description: "be a string" }, text: { type: "string", description: "be a string" }, section: { type: "string", description: "be a string" } }, patternProperties: {}, additionalProperties: false }, params: { type: "object", description: "be an object", properties: {}, patternProperties: {} } }, patternProperties: {}, additionalProperties: false, propertyNames: { type: "string", pattern: "^(?!(index_name|indexName|application_id|applicationId|search_only_api_key|searchOnlyApiKey|analytics_events|analyticsEvents|show_logo|showLogo|index_fields|indexFields))" } } }, patternProperties: {}, additionalProperties: false, propertyNames: { type: "string", pattern: "^(?!(collapse_after|collapseAfter|copy_button|copyButton))" } }], description: "be exactly one of: `true` or `false`, an object" }, navbar: { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { title: { oneOf: [{ type: "string", description: "be a string" }, { type: "boolean", description: "be `true` or `false`" }], description: "be exactly one of: a string, `true` or `false`" }, logo: { type: "string", description: "be a string" }, background: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, foreground: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, search: { type: "boolean", description: "be `true` or `false`" }, pinned: { type: "boolean", description: "be `true` or `false`" }, collapse: { type: "boolean", description: "be `true` or `false`" }, "collapse-below": { enum: ["sm", "md", "lg", "xl", "xxl"], description: "be one of: `sm`, `md`, `lg`, `xl`, `xxl`" }, left: { type: "array", description: "be an array of values, where each element should be navigation-item", items: { $ref: "navigation-item", description: "be navigation-item" } }, right: { type: "array", description: "be an array of values, where each element should be navigation-item", items: { $ref: "navigation-item", description: "be navigation-item" } } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(collapse_below|collapseBelow))" } }], description: "be exactly one of: `true` or `false`, an object" }, sidebar: { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { oneOf: [{ type: "object", description: "be an object", properties: { id: { type: "string", description: "be a string" }, title: { oneOf: [{ type: "string", description: "be a string" }, { type: "boolean", description: "be `true` or `false`" }], description: "be exactly one of: a string, `true` or `false`" }, subtitle: { type: "string", description: "be a string" }, logo: { type: "string", description: "be a string" }, search: { type: "boolean", description: "be `true` or `false`" }, tools: { type: "array", description: "be an array of values, where each element should be tool-item", items: { $ref: "tool-item", description: "be tool-item" } }, contents: { $ref: "sidebar-contents", description: "be sidebar-contents" }, style: { enum: ["docked", "floating"], description: "be one of: `docked`, `floating`" }, background: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, foreground: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, border: { type: "boolean", description: "be `true` or `false`" }, alignment: { enum: ["left", "right", "center"], description: "be one of: `left`, `right`, `center`" }, "collapse-level": { type: "number", description: "be a number" }, pinned: { type: "boolean", description: "be `true` or `false`" }, header: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, footer: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(collapse_level|collapseLevel))" } }, { type: "array", description: "be an array of values, where each element should be an object", items: { type: "object", description: "be an object", properties: { id: { type: "string", description: "be a string" }, title: { oneOf: [{ type: "string", description: "be a string" }, { type: "boolean", description: "be `true` or `false`" }], description: "be exactly one of: a string, `true` or `false`" }, subtitle: { type: "string", description: "be a string" }, logo: { type: "string", description: "be a string" }, search: { type: "boolean", description: "be `true` or `false`" }, tools: { type: "array", description: "be an array of values, where each element should be tool-item", items: { $ref: "tool-item", description: "be tool-item" } }, contents: { $ref: "sidebar-contents", description: "be sidebar-contents" }, style: { enum: ["docked", "floating"], description: "be one of: `docked`, `floating`" }, background: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, foreground: { anyOf: [{ enum: ["primary", "secondary", "success", "danger", "warning", "info", "light", "dark"], description: "be one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`" }, { type: "string", description: "be a string" }], description: "be at least one of: one of: `primary`, `secondary`, `success`, `danger`, `warning`, `info`, `light`, `dark`, a string" }, border: { type: "boolean", description: "be `true` or `false`" }, alignment: { enum: ["left", "right", "center"], description: "be one of: `left`, `right`, `center`" }, "collapse-level": { type: "number", description: "be a number" }, pinned: { type: "boolean", description: "be `true` or `false`" }, header: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, footer: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(collapse_level|collapseLevel))" } } }], description: "be exactly one of: an object, an array of values, where each element should be an object" }], description: "be exactly one of: `true` or `false`, exactly one of: an object, an array of values, where each element should be an object" }, "body-header": { type: "string", description: "be a string" }, "body-footer": { type: "string", description: "be a string" }, "margin-header": { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, "margin-footer": { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, "page-navigation": { type: "boolean", description: "be `true` or `false`" }, "page-footer": { oneOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: { left: { $ref: "page-footer-region", description: "be page-footer-region" }, right: { $ref: "page-footer-region", description: "be page-footer-region" }, center: { $ref: "page-footer-region", description: "be page-footer-region" }, border: { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "string", description: "be a string" }], description: "be exactly one of: `true` or `false`, a string" }, background: { type: "string", description: "be a string" }, foreground: { type: "string", description: "be a string" } }, patternProperties: {}, additionalProperties: false }], description: "be exactly one of: a string, an object" }, image: { type: "string", description: "be a string" }, comments: { $ref: "comments", description: "be comments" }, "open-graph": { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { title: { type: "string", description: "be a string" }, description: { type: "string", description: "be a string" }, image: { type: "string", description: "be a string" }, "image-width": { type: "number", description: "be a number" }, "image-height": { type: "number", description: "be a number" }, locale: { type: "string", description: "be a string" }, "site-name": { type: "string", description: "be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(^(?!^(?!(site_name|siteName)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))" }, required: [], additionalProperties: false }], description: "be exactly one of: `true` or `false`, an object" }, "twitter-card": { oneOf: [{ type: "boolean", description: "be `true` or `false`" }, { type: "object", description: "be an object", properties: { title: { type: "string", description: "be a string" }, description: { type: "string", description: "be a string" }, image: { type: "string", description: "be a string" }, "image-width": { type: "number", description: "be a number" }, "image-height": { type: "number", description: "be a number" }, "card-style": { enum: ["summary", "summary_large_image"], description: "be one of: `summary`, `summary_large_image`" }, creator: { type: "string", description: "be a string" }, site: { type: "string", description: "be a string" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(^(?!^(?!(card_style|cardStyle)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))" }, required: [], additionalProperties: false }], description: "be exactly one of: `true` or `false`, an object" }, subtitle: { type: "string", description: "be a string" }, author: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, date: { type: "string", description: "be a string" }, abstract: { type: "string", description: "be a string" }, chapters: { $ref: "chapter-list", description: "be chapter-list" }, references: { type: "string", description: "be a string" }, appendices: { type: "array", description: "be an array of values, where each element should be chapter-item", items: { $ref: "chapter-item", description: "be chapter-item" } }, "output-file": { type: "string", description: "be a string" }, "cover-image": { type: "string", description: "be a string" }, sharing: { oneOf: [{ enum: ["twitter", "facebook", "linkedin"], description: "be one of: `twitter`, `facebook`, `linkedin`" }, { type: "array", description: "be an array of values, where each element should be one of: `twitter`, `facebook`, `linkedin`", items: { enum: ["twitter", "facebook", "linkedin"], description: "be one of: `twitter`, `facebook`, `linkedin`" } }], description: "be exactly one of: one of: `twitter`, `facebook`, `linkedin`, an array of values, where each element should be one of: `twitter`, `facebook`, `linkedin`" }, downloads: { oneOf: [{ enum: ["pdf", "epub", "docx"], description: "be one of: `pdf`, `epub`, `docx`" }, { type: "array", description: "be an array of values, where each element should be one of: `pdf`, `epub`, `docx`", items: { enum: ["pdf", "epub", "docx"], description: "be one of: `pdf`, `epub`, `docx`" } }], description: "be exactly one of: one of: `pdf`, `epub`, `docx`, an array of values, where each element should be one of: `pdf`, `epub`, `docx`" }, tools: { type: "array", description: "be an array of values, where each element should be navigation-item", items: { $ref: "navigation-item", description: "be navigation-item" } } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(^(?!^(?!(output_file|outputFile|cover_image|coverImage)))|^(?!^(?!(fav_icon|favIcon|site_url|siteUrl|site_path|sitePath|repo_url|repoUrl|repo_branch|repoBranch|repo_actions|repoActions|google_analytics|googleAnalytics|cookie_consent|cookieConsent|body_header|bodyHeader|body_footer|bodyFooter|margin_header|marginHeader|margin_footer|marginFooter|page_navigation|pageNavigation|page_footer|pageFooter|open_graph|openGraph|twitter_card|twitterCard)))))" }, required: [] } }, patternProperties: {}, $id: "project-config-fields" };
  function vm(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      if (l.project !== void 0) {
        let J = l.project, ot = r, le = r;
        if (J && typeof J == "object" && !Array.isArray(J)) {
          for (let pe in J) {
            let v = r, E = r;
            if (typeof pe == "string") {
              if (!B1.test(pe)) {
                let A = { instancePath: o + "/project", schemaPath: "#/properties/project/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(execute_dir|executeDir|output_dir|outputDir|lib_dir|libDir))", schema: b.properties.project.propertyNames }, message: 'must match pattern "^(?!(execute_dir|executeDir|output_dir|outputDir|lib_dir|libDir))"', schema: "^(?!(execute_dir|executeDir|output_dir|outputDir|lib_dir|libDir))", parentSchema: b.properties.project.propertyNames, data: pe, propertyName: pe };
                e === null ? e = [A] : e.push(A), r++;
              }
            } else {
              let A = { instancePath: o + "/project", schemaPath: "#/properties/project/propertyNames/type", keyword: "type", params: { type: "string", schema: b.properties.project.propertyNames }, message: "must be string", schema: b.properties.project.propertyNames.type, parentSchema: b.properties.project.propertyNames, data: pe, propertyName: pe };
              e === null ? e = [A] : e.push(A), r++;
            }
            var c = v === r;
            if (!c) {
              let A = { instancePath: o + "/project", schemaPath: "#/properties/project/propertyNames", keyword: "propertyNames", params: { propertyName: pe, schema: b.properties.project }, message: "property name must be valid", schema: b.properties.project.propertyNames, parentSchema: b.properties.project, data: J };
              e === null ? e = [A] : e.push(A), r++;
            }
          }
          if (J.type !== void 0) {
            let pe = J.type, v = r, E = b.properties.project.properties.type.enum;
            if (!(pe === "website" || pe === "book" || pe === "site")) {
              let A = { instancePath: o + "/project/type", schemaPath: "#/properties/project/properties/type/enum", keyword: "enum", params: { allowedValues: b.properties.project.properties.type.enum, schema: b.properties.project.properties.type }, message: "must be equal to one of the allowed values", schema: b.properties.project.properties.type.enum, parentSchema: b.properties.project.properties.type, data: pe };
              e === null ? e = [A] : e.push(A), r++;
            }
            var d = v === r;
          }
          if (J.render !== void 0) {
            let pe = J.render, v = r, E = r;
            if (Array.isArray(pe)) {
              let Z = true, T = pe.length;
              for (let U = 0; U < T; U++) {
                let re = pe[U], Fr = r;
                if (typeof re != "string") {
                  let Xr = { instancePath: o + "/project/render/" + U, schemaPath: "#/properties/project/properties/render/items/type", keyword: "type", params: { type: "string", schema: b.properties.project.properties.render.items }, message: "must be string", schema: b.properties.project.properties.render.items.type, parentSchema: b.properties.project.properties.render.items, data: re };
                  e === null ? e = [Xr] : e.push(Xr), r++;
                }
                let z = r;
                var u = Fr === r;
                u || (Z = false);
              }
            } else {
              let Z = { instancePath: o + "/project/render", schemaPath: "#/properties/project/properties/render/type", keyword: "type", params: { type: "array", schema: b.properties.project.properties.render }, message: "must be array", schema: b.properties.project.properties.render.type, parentSchema: b.properties.project.properties.render, data: pe };
              e === null ? e = [Z] : e.push(Z), r++;
            }
            var d = v === r;
          }
          if (J["execute-dir"] !== void 0) {
            let pe = J["execute-dir"], v = r, E = b.properties.project.properties["execute-dir"].enum;
            if (!(pe === "file" || pe === "project")) {
              let Z = { instancePath: o + "/project/execute-dir", schemaPath: "#/properties/project/properties/execute-dir/enum", keyword: "enum", params: { allowedValues: b.properties.project.properties["execute-dir"].enum, schema: b.properties.project.properties["execute-dir"] }, message: "must be equal to one of the allowed values", schema: b.properties.project.properties["execute-dir"].enum, parentSchema: b.properties.project.properties["execute-dir"], data: pe };
              e === null ? e = [Z] : e.push(Z), r++;
            }
            var d = v === r;
          }
          if (J["output-dir"] !== void 0) {
            let pe = J["output-dir"], v = r;
            if (typeof pe != "string") {
              let Z = { instancePath: o + "/project/output-dir", schemaPath: "#/properties/project/properties/output-dir/type", keyword: "type", params: { type: "string", schema: b.properties.project.properties["output-dir"] }, message: "must be string", schema: b.properties.project.properties["output-dir"].type, parentSchema: b.properties.project.properties["output-dir"], data: pe };
              e === null ? e = [Z] : e.push(Z), r++;
            }
            let E = r;
            var d = v === r;
          }
          if (J["lib-dir"] !== void 0) {
            let pe = J["lib-dir"], v = r;
            if (typeof pe != "string") {
              let Z = { instancePath: o + "/project/lib-dir", schemaPath: "#/properties/project/properties/lib-dir/type", keyword: "type", params: { type: "string", schema: b.properties.project.properties["lib-dir"] }, message: "must be string", schema: b.properties.project.properties["lib-dir"].type, parentSchema: b.properties.project.properties["lib-dir"], data: pe };
              e === null ? e = [Z] : e.push(Z), r++;
            }
            let E = r;
            var d = v === r;
          }
          if (J.resources !== void 0) {
            let pe = J.resources, v = r, E = r, A = false, Z = null, T = r;
            if (typeof pe != "string") {
              let Xr = { instancePath: o + "/project/resources", schemaPath: "#/properties/project/properties/resources/oneOf/0/type", keyword: "type", params: { type: "string", schema: b.properties.project.properties.resources.oneOf[0] }, message: "must be string", schema: b.properties.project.properties.resources.oneOf[0].type, parentSchema: b.properties.project.properties.resources.oneOf[0], data: pe };
              e === null ? e = [Xr] : e.push(Xr), r++;
            }
            let U = r;
            var y = T === r;
            y && (A = true, Z = 0);
            let re = r, Fr = r;
            if (Array.isArray(pe)) {
              let Xr = true, W = pe.length;
              for (let C = 0; C < W; C++) {
                let te = pe[C], Br = r;
                if (typeof te != "string") {
                  let Yr = { instancePath: o + "/project/resources/" + C, schemaPath: "#/properties/project/properties/resources/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: b.properties.project.properties.resources.oneOf[1].items }, message: "must be string", schema: b.properties.project.properties.resources.oneOf[1].items.type, parentSchema: b.properties.project.properties.resources.oneOf[1].items, data: te };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let X = r;
                var g = Br === r;
                g || (Xr = false);
              }
            } else {
              let Xr = { instancePath: o + "/project/resources", schemaPath: "#/properties/project/properties/resources/oneOf/1/type", keyword: "type", params: { type: "array", schema: b.properties.project.properties.resources.oneOf[1] }, message: "must be array", schema: b.properties.project.properties.resources.oneOf[1].type, parentSchema: b.properties.project.properties.resources.oneOf[1], data: pe };
              e === null ? e = [Xr] : e.push(Xr), r++;
            }
            var y = re === r;
            if (y && A ? (A = false, Z = [Z, 1]) : y && (A = true, Z = 1), A)
              r = E, e !== null && (E ? e.length = E : e = null);
            else {
              let Xr = { instancePath: o + "/project/resources", schemaPath: "#/properties/project/properties/resources/oneOf", keyword: "oneOf", params: { passingSchemas: Z, schema: b.properties.project.properties.resources }, message: "must match exactly one schema in oneOf", schema: b.properties.project.properties.resources.oneOf, parentSchema: b.properties.project.properties.resources, data: pe };
              e === null ? e = [Xr] : e.push(Xr), r++;
            }
            var d = v === r;
          }
        } else {
          let pe = { instancePath: o + "/project", schemaPath: "#/properties/project/type", keyword: "type", params: { type: "object", schema: b.properties.project }, message: "must be object", schema: b.properties.project.type, parentSchema: b.properties.project, data: J };
          e === null ? e = [pe] : e.push(pe), r++;
        }
        var x = ot === r;
      }
      if (l.website !== void 0) {
        let J = l.website, ot = r;
        hc(J, { instancePath: o + "/website", parentData: l, parentDataProperty: "website", rootData: s }) || (e = e === null ? hc.errors : e.concat(hc.errors), r = e.length);
        var x = ot === r;
      }
      if (l.book !== void 0) {
        let J = l.book, ot = r, le = r;
        if (J && typeof J == "object" && !Array.isArray(J)) {
          for (let v in J) {
            let E = r, A = r;
            if (typeof v == "string") {
              if (!N1.test(v)) {
                let Z = { instancePath: o + "/book", schemaPath: "#/properties/book/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(^(?!^(?!(output_file|outputFile|cover_image|coverImage)))|^(?!^(?!(fav_icon|favIcon|site_url|siteUrl|site_path|sitePath|repo_url|repoUrl|repo_branch|repoBranch|repo_actions|repoActions|google_analytics|googleAnalytics|cookie_consent|cookieConsent|body_header|bodyHeader|body_footer|bodyFooter|margin_header|marginHeader|margin_footer|marginFooter|page_navigation|pageNavigation|page_footer|pageFooter|open_graph|openGraph|twitter_card|twitterCard)))))", schema: b.properties.book.propertyNames }, message: 'must match pattern "^(?!(^(?!^(?!(output_file|outputFile|cover_image|coverImage)))|^(?!^(?!(fav_icon|favIcon|site_url|siteUrl|site_path|sitePath|repo_url|repoUrl|repo_branch|repoBranch|repo_actions|repoActions|google_analytics|googleAnalytics|cookie_consent|cookieConsent|body_header|bodyHeader|body_footer|bodyFooter|margin_header|marginHeader|margin_footer|marginFooter|page_navigation|pageNavigation|page_footer|pageFooter|open_graph|openGraph|twitter_card|twitterCard)))))"', schema: "^(?!(^(?!^(?!(output_file|outputFile|cover_image|coverImage)))|^(?!^(?!(fav_icon|favIcon|site_url|siteUrl|site_path|sitePath|repo_url|repoUrl|repo_branch|repoBranch|repo_actions|repoActions|google_analytics|googleAnalytics|cookie_consent|cookieConsent|body_header|bodyHeader|body_footer|bodyFooter|margin_header|marginHeader|margin_footer|marginFooter|page_navigation|pageNavigation|page_footer|pageFooter|open_graph|openGraph|twitter_card|twitterCard)))))", parentSchema: b.properties.book.propertyNames, data: v, propertyName: v };
                e === null ? e = [Z] : e.push(Z), r++;
              }
            } else {
              let Z = { instancePath: o + "/book", schemaPath: "#/properties/book/propertyNames/type", keyword: "type", params: { type: "string", schema: b.properties.book.propertyNames }, message: "must be string", schema: b.properties.book.propertyNames.type, parentSchema: b.properties.book.propertyNames, data: v, propertyName: v };
              e === null ? e = [Z] : e.push(Z), r++;
            }
            var D = E === r;
            if (!D) {
              let Z = { instancePath: o + "/book", schemaPath: "#/properties/book/propertyNames", keyword: "propertyNames", params: { propertyName: v, schema: b.properties.book }, message: "property name must be valid", schema: b.properties.book.propertyNames, parentSchema: b.properties.book, data: J };
              e === null ? e = [Z] : e.push(Z), r++;
            }
          }
          if (J.title !== void 0) {
            let v = J.title, E = r;
            if (typeof v != "string") {
              let Z = { instancePath: o + "/book/title", schemaPath: "#/properties/book/properties/title/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.title }, message: "must be string", schema: b.properties.book.properties.title.type, parentSchema: b.properties.book.properties.title, data: v };
              e === null ? e = [Z] : e.push(Z), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J.description !== void 0) {
            let v = J.description, E = r;
            if (typeof v != "string") {
              let T = { instancePath: o + "/book/description", schemaPath: "#/properties/book/properties/description/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.description }, message: "must be string", schema: b.properties.book.properties.description.type, parentSchema: b.properties.book.properties.description, data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J["fav-icon"] !== void 0) {
            let v = J["fav-icon"], E = r;
            if (typeof v != "string") {
              let T = { instancePath: o + "/book/fav-icon", schemaPath: "#/properties/book/properties/fav-icon/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["fav-icon"] }, message: "must be string", schema: b.properties.book.properties["fav-icon"].type, parentSchema: b.properties.book.properties["fav-icon"], data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J["site-url"] !== void 0) {
            let v = J["site-url"], E = r;
            if (typeof v != "string") {
              let T = { instancePath: o + "/book/site-url", schemaPath: "#/properties/book/properties/site-url/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["site-url"] }, message: "must be string", schema: b.properties.book.properties["site-url"].type, parentSchema: b.properties.book.properties["site-url"], data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J["site-path"] !== void 0) {
            let v = J["site-path"], E = r;
            if (typeof v != "string") {
              let T = { instancePath: o + "/book/site-path", schemaPath: "#/properties/book/properties/site-path/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["site-path"] }, message: "must be string", schema: b.properties.book.properties["site-path"].type, parentSchema: b.properties.book.properties["site-path"], data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J["repo-url"] !== void 0) {
            let v = J["repo-url"], E = r;
            if (typeof v != "string") {
              let T = { instancePath: o + "/book/repo-url", schemaPath: "#/properties/book/properties/repo-url/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["repo-url"] }, message: "must be string", schema: b.properties.book.properties["repo-url"].type, parentSchema: b.properties.book.properties["repo-url"], data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J["repo-branch"] !== void 0) {
            let v = J["repo-branch"], E = r;
            if (typeof v != "string") {
              let T = { instancePath: o + "/book/repo-branch", schemaPath: "#/properties/book/properties/repo-branch/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["repo-branch"] }, message: "must be string", schema: b.properties.book.properties["repo-branch"].type, parentSchema: b.properties.book.properties["repo-branch"], data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J["repo-actions"] !== void 0) {
            let v = J["repo-actions"], E = r, A = r, Z = false, T = null, U = r, re = b.properties.book.properties["repo-actions"].oneOf[0].enum;
            if (!(v === "none" || v === "edit" || v === "source" || v === "issue")) {
              let W = { instancePath: o + "/book/repo-actions", schemaPath: "#/properties/book/properties/repo-actions/oneOf/0/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties["repo-actions"].oneOf[0].enum, schema: b.properties.book.properties["repo-actions"].oneOf[0] }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties["repo-actions"].oneOf[0].enum, parentSchema: b.properties.book.properties["repo-actions"].oneOf[0], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var m = U === r;
            m && (Z = true, T = 0);
            let Fr = r, z = r;
            if (Array.isArray(v)) {
              let W = true, C = v.length;
              for (let te = 0; te < C; te++) {
                let Br = v[te], X = r, Yr = b.properties.book.properties["repo-actions"].oneOf[1].items.enum;
                if (!(Br === "none" || Br === "edit" || Br === "source" || Br === "issue")) {
                  let G = { instancePath: o + "/book/repo-actions/" + te, schemaPath: "#/properties/book/properties/repo-actions/oneOf/1/items/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties["repo-actions"].oneOf[1].items.enum, schema: b.properties.book.properties["repo-actions"].oneOf[1].items }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties["repo-actions"].oneOf[1].items.enum, parentSchema: b.properties.book.properties["repo-actions"].oneOf[1].items, data: Br };
                  e === null ? e = [G] : e.push(G), r++;
                }
                var R = X === r;
                R || (W = false);
              }
            } else {
              let W = { instancePath: o + "/book/repo-actions", schemaPath: "#/properties/book/properties/repo-actions/oneOf/1/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties["repo-actions"].oneOf[1] }, message: "must be array", schema: b.properties.book.properties["repo-actions"].oneOf[1].type, parentSchema: b.properties.book.properties["repo-actions"].oneOf[1], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var m = Fr === r;
            if (m && Z ? (Z = false, T = [T, 1]) : m && (Z = true, T = 1), Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let W = { instancePath: o + "/book/repo-actions", schemaPath: "#/properties/book/properties/repo-actions/oneOf", keyword: "oneOf", params: { passingSchemas: T, schema: b.properties.book.properties["repo-actions"] }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties["repo-actions"].oneOf, parentSchema: b.properties.book.properties["repo-actions"], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var $ = E === r;
          }
          if (J["google-analytics"] !== void 0) {
            let v = J["google-analytics"], E = r, A = r, Z = false, T = r;
            if (typeof v != "string") {
              let Fr = { instancePath: o + "/book/google-analytics", schemaPath: "#/properties/book/properties/google-analytics/anyOf/0/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["google-analytics"].anyOf[0] }, message: "must be string", schema: b.properties.book.properties["google-analytics"].anyOf[0].type, parentSchema: b.properties.book.properties["google-analytics"].anyOf[0], data: v };
              e === null ? e = [Fr] : e.push(Fr), r++;
            }
            let U = r;
            var j = T === r;
            if (Z = Z || j, !Z) {
              let Fr = r, z = r;
              if (v && typeof v == "object" && !Array.isArray(v)) {
                for (let W in v) {
                  let C = r, te = r;
                  if (typeof W == "string") {
                    if (!tg.test(W)) {
                      let Br = { instancePath: o + "/book/google-analytics", schemaPath: "#/properties/book/properties/google-analytics/anyOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(tracking_id|trackingId|anonymize_ip|anonymizeIp))", schema: b.properties.book.properties["google-analytics"].anyOf[1].propertyNames }, message: 'must match pattern "^(?!(tracking_id|trackingId|anonymize_ip|anonymizeIp))"', schema: "^(?!(tracking_id|trackingId|anonymize_ip|anonymizeIp))", parentSchema: b.properties.book.properties["google-analytics"].anyOf[1].propertyNames, data: W, propertyName: W };
                      e === null ? e = [Br] : e.push(Br), r++;
                    }
                  } else {
                    let Br = { instancePath: o + "/book/google-analytics", schemaPath: "#/properties/book/properties/google-analytics/anyOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["google-analytics"].anyOf[1].propertyNames }, message: "must be string", schema: b.properties.book.properties["google-analytics"].anyOf[1].propertyNames.type, parentSchema: b.properties.book.properties["google-analytics"].anyOf[1].propertyNames, data: W, propertyName: W };
                    e === null ? e = [Br] : e.push(Br), r++;
                  }
                  var V = C === r;
                  if (!V) {
                    let Br = { instancePath: o + "/book/google-analytics", schemaPath: "#/properties/book/properties/google-analytics/anyOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: W, schema: b.properties.book.properties["google-analytics"].anyOf[1] }, message: "property name must be valid", schema: b.properties.book.properties["google-analytics"].anyOf[1].propertyNames, parentSchema: b.properties.book.properties["google-analytics"].anyOf[1], data: v };
                    e === null ? e = [Br] : e.push(Br), r++;
                  }
                }
                if (v["tracking-id"] !== void 0) {
                  let W = v["tracking-id"], C = r;
                  if (typeof W != "string") {
                    let Br = { instancePath: o + "/book/google-analytics/tracking-id", schemaPath: "#/properties/book/properties/google-analytics/anyOf/1/properties/tracking-id/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["google-analytics"].anyOf[1].properties["tracking-id"] }, message: "must be string", schema: b.properties.book.properties["google-analytics"].anyOf[1].properties["tracking-id"].type, parentSchema: b.properties.book.properties["google-analytics"].anyOf[1].properties["tracking-id"], data: W };
                    e === null ? e = [Br] : e.push(Br), r++;
                  }
                  let te = r;
                  var se = C === r;
                }
                if (v.storage !== void 0) {
                  let W = v.storage, C = r, te = b.properties.book.properties["google-analytics"].anyOf[1].properties.storage.enum;
                  if (!(W === "cookies" || W === "none")) {
                    let X = { instancePath: o + "/book/google-analytics/storage", schemaPath: "#/properties/book/properties/google-analytics/anyOf/1/properties/storage/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties["google-analytics"].anyOf[1].properties.storage.enum, schema: b.properties.book.properties["google-analytics"].anyOf[1].properties.storage }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties["google-analytics"].anyOf[1].properties.storage.enum, parentSchema: b.properties.book.properties["google-analytics"].anyOf[1].properties.storage, data: W };
                    e === null ? e = [X] : e.push(X), r++;
                  }
                  var se = C === r;
                }
                if (v["anonymize-ip"] !== void 0) {
                  let W = v["anonymize-ip"], C = r;
                  if (typeof W != "boolean") {
                    let X = { instancePath: o + "/book/google-analytics/anonymize-ip", schemaPath: "#/properties/book/properties/google-analytics/anyOf/1/properties/anonymize-ip/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties["google-analytics"].anyOf[1].properties["anonymize-ip"] }, message: "must be boolean", schema: b.properties.book.properties["google-analytics"].anyOf[1].properties["anonymize-ip"].type, parentSchema: b.properties.book.properties["google-analytics"].anyOf[1].properties["anonymize-ip"], data: W };
                    e === null ? e = [X] : e.push(X), r++;
                  }
                  let te = r;
                  var se = C === r;
                }
                if (v.version !== void 0) {
                  let W = v.version, C = r, te = b.properties.book.properties["google-analytics"].anyOf[1].properties.version.enum;
                  if (!(W === 3 || W === 4)) {
                    let X = { instancePath: o + "/book/google-analytics/version", schemaPath: "#/properties/book/properties/google-analytics/anyOf/1/properties/version/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties["google-analytics"].anyOf[1].properties.version.enum, schema: b.properties.book.properties["google-analytics"].anyOf[1].properties.version }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties["google-analytics"].anyOf[1].properties.version.enum, parentSchema: b.properties.book.properties["google-analytics"].anyOf[1].properties.version, data: W };
                    e === null ? e = [X] : e.push(X), r++;
                  }
                  var se = C === r;
                }
              } else {
                let W = { instancePath: o + "/book/google-analytics", schemaPath: "#/properties/book/properties/google-analytics/anyOf/1/type", keyword: "type", params: { type: "object", schema: b.properties.book.properties["google-analytics"].anyOf[1] }, message: "must be object", schema: b.properties.book.properties["google-analytics"].anyOf[1].type, parentSchema: b.properties.book.properties["google-analytics"].anyOf[1], data: v };
                e === null ? e = [W] : e.push(W), r++;
              }
              var j = Fr === r;
              Z = Z || j;
            }
            if (Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let Fr = { instancePath: o + "/book/google-analytics", schemaPath: "#/properties/book/properties/google-analytics/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: b.properties.book.properties["google-analytics"].anyOf, parentSchema: b.properties.book.properties["google-analytics"], data: v };
              e === null ? e = [Fr] : e.push(Fr), r++;
            }
            var $ = E === r;
          }
          if (J["cookie-consent"] !== void 0) {
            let v = J["cookie-consent"], E = r, A = r, Z = false, T = r;
            if (typeof v != "boolean") {
              let Fr = { instancePath: o + "/book/cookie-consent", schemaPath: "#/properties/book/properties/cookie-consent/anyOf/0/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties["cookie-consent"].anyOf[0] }, message: "must be boolean", schema: b.properties.book.properties["cookie-consent"].anyOf[0].type, parentSchema: b.properties.book.properties["cookie-consent"].anyOf[0], data: v };
              e === null ? e = [Fr] : e.push(Fr), r++;
            }
            let U = r;
            var P = T === r;
            if (Z = Z || P, !Z) {
              let Fr = r, z = r;
              if (v && typeof v == "object" && !Array.isArray(v)) {
                for (let W in v) {
                  let C = r, te = r;
                  if (typeof W == "string") {
                    if (!ag.test(W)) {
                      let Br = { instancePath: o + "/book/cookie-consent", schemaPath: "#/properties/book/properties/cookie-consent/anyOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(policy_url|policyUrl|prefs_text|prefsText))", schema: b.properties.book.properties["cookie-consent"].anyOf[1].propertyNames }, message: 'must match pattern "^(?!(policy_url|policyUrl|prefs_text|prefsText))"', schema: "^(?!(policy_url|policyUrl|prefs_text|prefsText))", parentSchema: b.properties.book.properties["cookie-consent"].anyOf[1].propertyNames, data: W, propertyName: W };
                      e === null ? e = [Br] : e.push(Br), r++;
                    }
                  } else {
                    let Br = { instancePath: o + "/book/cookie-consent", schemaPath: "#/properties/book/properties/cookie-consent/anyOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["cookie-consent"].anyOf[1].propertyNames }, message: "must be string", schema: b.properties.book.properties["cookie-consent"].anyOf[1].propertyNames.type, parentSchema: b.properties.book.properties["cookie-consent"].anyOf[1].propertyNames, data: W, propertyName: W };
                    e === null ? e = [Br] : e.push(Br), r++;
                  }
                  var oe = C === r;
                  if (!oe) {
                    let Br = { instancePath: o + "/book/cookie-consent", schemaPath: "#/properties/book/properties/cookie-consent/anyOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: W, schema: b.properties.book.properties["cookie-consent"].anyOf[1] }, message: "property name must be valid", schema: b.properties.book.properties["cookie-consent"].anyOf[1].propertyNames, parentSchema: b.properties.book.properties["cookie-consent"].anyOf[1], data: v };
                    e === null ? e = [Br] : e.push(Br), r++;
                  }
                }
                if (v.type !== void 0) {
                  let W = v.type, C = r, te = b.properties.book.properties["cookie-consent"].anyOf[1].properties.type.enum;
                  if (!(W === "implied" || W === "express")) {
                    let Br = { instancePath: o + "/book/cookie-consent/type", schemaPath: "#/properties/book/properties/cookie-consent/anyOf/1/properties/type/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties["cookie-consent"].anyOf[1].properties.type.enum, schema: b.properties.book.properties["cookie-consent"].anyOf[1].properties.type }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties["cookie-consent"].anyOf[1].properties.type.enum, parentSchema: b.properties.book.properties["cookie-consent"].anyOf[1].properties.type, data: W };
                    e === null ? e = [Br] : e.push(Br), r++;
                  }
                  var h = C === r;
                }
                if (v.style !== void 0) {
                  let W = v.style, C = r, te = b.properties.book.properties["cookie-consent"].anyOf[1].properties.style.enum;
                  if (!(W === "simple" || W === "headline" || W === "interstitial" || W === "standalone")) {
                    let X = { instancePath: o + "/book/cookie-consent/style", schemaPath: "#/properties/book/properties/cookie-consent/anyOf/1/properties/style/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties["cookie-consent"].anyOf[1].properties.style.enum, schema: b.properties.book.properties["cookie-consent"].anyOf[1].properties.style }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties["cookie-consent"].anyOf[1].properties.style.enum, parentSchema: b.properties.book.properties["cookie-consent"].anyOf[1].properties.style, data: W };
                    e === null ? e = [X] : e.push(X), r++;
                  }
                  var h = C === r;
                }
                if (v.palette !== void 0) {
                  let W = v.palette, C = r, te = b.properties.book.properties["cookie-consent"].anyOf[1].properties.palette.enum;
                  if (!(W === "light" || W === "dark")) {
                    let X = { instancePath: o + "/book/cookie-consent/palette", schemaPath: "#/properties/book/properties/cookie-consent/anyOf/1/properties/palette/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties["cookie-consent"].anyOf[1].properties.palette.enum, schema: b.properties.book.properties["cookie-consent"].anyOf[1].properties.palette }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties["cookie-consent"].anyOf[1].properties.palette.enum, parentSchema: b.properties.book.properties["cookie-consent"].anyOf[1].properties.palette, data: W };
                    e === null ? e = [X] : e.push(X), r++;
                  }
                  var h = C === r;
                }
                if (v["policy-url"] !== void 0) {
                  let W = v["policy-url"], C = r;
                  if (typeof W != "string") {
                    let X = { instancePath: o + "/book/cookie-consent/policy-url", schemaPath: "#/properties/book/properties/cookie-consent/anyOf/1/properties/policy-url/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["cookie-consent"].anyOf[1].properties["policy-url"] }, message: "must be string", schema: b.properties.book.properties["cookie-consent"].anyOf[1].properties["policy-url"].type, parentSchema: b.properties.book.properties["cookie-consent"].anyOf[1].properties["policy-url"], data: W };
                    e === null ? e = [X] : e.push(X), r++;
                  }
                  let te = r;
                  var h = C === r;
                }
                if (v["prefs-text"] !== void 0) {
                  let W = v["prefs-text"], C = r;
                  if (typeof W != "string") {
                    let X = { instancePath: o + "/book/cookie-consent/prefs-text", schemaPath: "#/properties/book/properties/cookie-consent/anyOf/1/properties/prefs-text/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["cookie-consent"].anyOf[1].properties["prefs-text"] }, message: "must be string", schema: b.properties.book.properties["cookie-consent"].anyOf[1].properties["prefs-text"].type, parentSchema: b.properties.book.properties["cookie-consent"].anyOf[1].properties["prefs-text"], data: W };
                    e === null ? e = [X] : e.push(X), r++;
                  }
                  let te = r;
                  var h = C === r;
                }
              } else {
                let W = { instancePath: o + "/book/cookie-consent", schemaPath: "#/properties/book/properties/cookie-consent/anyOf/1/type", keyword: "type", params: { type: "object", schema: b.properties.book.properties["cookie-consent"].anyOf[1] }, message: "must be object", schema: b.properties.book.properties["cookie-consent"].anyOf[1].type, parentSchema: b.properties.book.properties["cookie-consent"].anyOf[1], data: v };
                e === null ? e = [W] : e.push(W), r++;
              }
              var P = Fr === r;
              Z = Z || P;
            }
            if (Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let Fr = { instancePath: o + "/book/cookie-consent", schemaPath: "#/properties/book/properties/cookie-consent/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: b.properties.book.properties["cookie-consent"].anyOf, parentSchema: b.properties.book.properties["cookie-consent"], data: v };
              e === null ? e = [Fr] : e.push(Fr), r++;
            }
            var $ = E === r;
          }
          if (J.search !== void 0) {
            let v = J.search, E = r, A = r, Z = false, T = null, U = r;
            if (typeof v != "boolean") {
              let W = { instancePath: o + "/book/search", schemaPath: "#/properties/book/properties/search/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.search.oneOf[0] }, message: "must be boolean", schema: b.properties.book.properties.search.oneOf[0].type, parentSchema: b.properties.book.properties.search.oneOf[0], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            let re = r;
            var f = U === r;
            f && (Z = true, T = 0);
            let Fr = r, z = r;
            if (v && typeof v == "object" && !Array.isArray(v)) {
              for (let C in v) {
                let te = r, Br = r;
                if (typeof C == "string") {
                  if (!ng.test(C)) {
                    let X = { instancePath: o + "/book/search", schemaPath: "#/properties/book/properties/search/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(collapse_after|collapseAfter|copy_button|copyButton))", schema: b.properties.book.properties.search.oneOf[1].propertyNames }, message: 'must match pattern "^(?!(collapse_after|collapseAfter|copy_button|copyButton))"', schema: "^(?!(collapse_after|collapseAfter|copy_button|copyButton))", parentSchema: b.properties.book.properties.search.oneOf[1].propertyNames, data: C, propertyName: C };
                    e === null ? e = [X] : e.push(X), r++;
                  }
                } else {
                  let X = { instancePath: o + "/book/search", schemaPath: "#/properties/book/properties/search/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.search.oneOf[1].propertyNames }, message: "must be string", schema: b.properties.book.properties.search.oneOf[1].propertyNames.type, parentSchema: b.properties.book.properties.search.oneOf[1].propertyNames, data: C, propertyName: C };
                  e === null ? e = [X] : e.push(X), r++;
                }
                var ft = te === r;
                if (!ft) {
                  let X = { instancePath: o + "/book/search", schemaPath: "#/properties/book/properties/search/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: C, schema: b.properties.book.properties.search.oneOf[1] }, message: "property name must be valid", schema: b.properties.book.properties.search.oneOf[1].propertyNames, parentSchema: b.properties.book.properties.search.oneOf[1], data: v };
                  e === null ? e = [X] : e.push(X), r++;
                }
              }
              let W = r;
              for (let C in v)
                if (!(C === "location" || C === "type" || C === "limit" || C === "collapse-after" || C === "copy-button" || C === "algolia")) {
                  let te = { instancePath: o + "/book/search", schemaPath: "#/properties/book/properties/search/oneOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: C, schema: b.properties.book.properties.search.oneOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: b.properties.book.properties.search.oneOf[1], data: v };
                  e === null ? e = [te] : e.push(te), r++;
                }
              if (v.location !== void 0) {
                let C = v.location, te = r, Br = b.properties.book.properties.search.oneOf[1].properties.location.enum;
                if (!(C === "navbar" || C === "sidebar")) {
                  let X = { instancePath: o + "/book/search/location", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/location/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.search.oneOf[1].properties.location.enum, schema: b.properties.book.properties.search.oneOf[1].properties.location }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.search.oneOf[1].properties.location.enum, parentSchema: b.properties.book.properties.search.oneOf[1].properties.location, data: C };
                  e === null ? e = [X] : e.push(X), r++;
                }
                var Cr = te === r;
              }
              if (v.type !== void 0) {
                let C = v.type, te = r, Br = b.properties.book.properties.search.oneOf[1].properties.type.enum;
                if (!(C === "overlay" || C === "textbox")) {
                  let Yr = { instancePath: o + "/book/search/type", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/type/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.search.oneOf[1].properties.type.enum, schema: b.properties.book.properties.search.oneOf[1].properties.type }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.search.oneOf[1].properties.type.enum, parentSchema: b.properties.book.properties.search.oneOf[1].properties.type, data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                var Cr = te === r;
              }
              if (v.limit !== void 0) {
                let C = v.limit, te = r;
                if (!(typeof C == "number" && isFinite(C))) {
                  let Yr = { instancePath: o + "/book/search/limit", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/limit/type", keyword: "type", params: { type: "number", schema: b.properties.book.properties.search.oneOf[1].properties.limit }, message: "must be number", schema: b.properties.book.properties.search.oneOf[1].properties.limit.type, parentSchema: b.properties.book.properties.search.oneOf[1].properties.limit, data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var Cr = te === r;
              }
              if (v["collapse-after"] !== void 0) {
                let C = v["collapse-after"], te = r;
                if (!(typeof C == "number" && isFinite(C))) {
                  let Yr = { instancePath: o + "/book/search/collapse-after", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/collapse-after/type", keyword: "type", params: { type: "number", schema: b.properties.book.properties.search.oneOf[1].properties["collapse-after"] }, message: "must be number", schema: b.properties.book.properties.search.oneOf[1].properties["collapse-after"].type, parentSchema: b.properties.book.properties.search.oneOf[1].properties["collapse-after"], data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var Cr = te === r;
              }
              if (v["copy-button"] !== void 0) {
                let C = v["copy-button"], te = r;
                if (typeof C != "boolean") {
                  let Yr = { instancePath: o + "/book/search/copy-button", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/copy-button/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.search.oneOf[1].properties["copy-button"] }, message: "must be boolean", schema: b.properties.book.properties.search.oneOf[1].properties["copy-button"].type, parentSchema: b.properties.book.properties.search.oneOf[1].properties["copy-button"], data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var Cr = te === r;
              }
              if (v.algolia !== void 0) {
                let C = v.algolia, te = r, Br = r;
                if (C && typeof C == "object" && !Array.isArray(C)) {
                  for (let G in C) {
                    let Ar = r, _t = r;
                    if (typeof G == "string") {
                      if (!og.test(G)) {
                        let Zr = { instancePath: o + "/book/search/algolia", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(index_name|indexName|application_id|applicationId|search_only_api_key|searchOnlyApiKey|analytics_events|analyticsEvents|show_logo|showLogo|index_fields|indexFields))", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.propertyNames }, message: 'must match pattern "^(?!(index_name|indexName|application_id|applicationId|search_only_api_key|searchOnlyApiKey|analytics_events|analyticsEvents|show_logo|showLogo|index_fields|indexFields))"', schema: "^(?!(index_name|indexName|application_id|applicationId|search_only_api_key|searchOnlyApiKey|analytics_events|analyticsEvents|show_logo|showLogo|index_fields|indexFields))", parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia.propertyNames, data: G, propertyName: G };
                        e === null ? e = [Zr] : e.push(Zr), r++;
                      }
                    } else {
                      let Zr = { instancePath: o + "/book/search/algolia", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/propertyNames/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.propertyNames }, message: "must be string", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.propertyNames.type, parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia.propertyNames, data: G, propertyName: G };
                      e === null ? e = [Zr] : e.push(Zr), r++;
                    }
                    var Dr = Ar === r;
                    if (!Dr) {
                      let Zr = { instancePath: o + "/book/search/algolia", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/propertyNames", keyword: "propertyNames", params: { propertyName: G, schema: b.properties.book.properties.search.oneOf[1].properties.algolia }, message: "property name must be valid", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.propertyNames, parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia, data: C };
                      e === null ? e = [Zr] : e.push(Zr), r++;
                    }
                  }
                  let Yr = r;
                  for (let G in C)
                    if (!(G === "index-name" || G === "application-id" || G === "search-only-api-key" || G === "analytics-events" || G === "show-logo" || G === "index-fields" || G === "params")) {
                      let Ar = { instancePath: o + "/book/search/algolia", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: G, schema: b.properties.book.properties.search.oneOf[1].properties.algolia }, message: "must NOT have additional properties", schema: false, parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia, data: C };
                      e === null ? e = [Ar] : e.push(Ar), r++;
                    }
                  if (C["index-name"] !== void 0) {
                    let G = C["index-name"], Ar = r;
                    if (typeof G != "string") {
                      let Zr = { instancePath: o + "/book/search/algolia/index-name", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/properties/index-name/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-name"] }, message: "must be string", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-name"].type, parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-name"], data: G };
                      e === null ? e = [Zr] : e.push(Zr), r++;
                    }
                    let _t = r;
                    var wr = Ar === r;
                  }
                  if (C["application-id"] !== void 0) {
                    let G = C["application-id"], Ar = r;
                    if (typeof G != "string") {
                      let Er = { instancePath: o + "/book/search/algolia/application-id", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/properties/application-id/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["application-id"] }, message: "must be string", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["application-id"].type, parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["application-id"], data: G };
                      e === null ? e = [Er] : e.push(Er), r++;
                    }
                    let _t = r;
                    var wr = Ar === r;
                  }
                  if (C["search-only-api-key"] !== void 0) {
                    let G = C["search-only-api-key"], Ar = r;
                    if (typeof G != "string") {
                      let Er = { instancePath: o + "/book/search/algolia/search-only-api-key", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/properties/search-only-api-key/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["search-only-api-key"] }, message: "must be string", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["search-only-api-key"].type, parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["search-only-api-key"], data: G };
                      e === null ? e = [Er] : e.push(Er), r++;
                    }
                    let _t = r;
                    var wr = Ar === r;
                  }
                  if (C["analytics-events"] !== void 0) {
                    let G = C["analytics-events"], Ar = r;
                    if (typeof G != "boolean") {
                      let Er = { instancePath: o + "/book/search/algolia/analytics-events", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/properties/analytics-events/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["analytics-events"] }, message: "must be boolean", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["analytics-events"].type, parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["analytics-events"], data: G };
                      e === null ? e = [Er] : e.push(Er), r++;
                    }
                    let _t = r;
                    var wr = Ar === r;
                  }
                  if (C["show-logo"] !== void 0) {
                    let G = C["show-logo"], Ar = r;
                    if (typeof G != "boolean") {
                      let Er = { instancePath: o + "/book/search/algolia/show-logo", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/properties/show-logo/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["show-logo"] }, message: "must be boolean", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["show-logo"].type, parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["show-logo"], data: G };
                      e === null ? e = [Er] : e.push(Er), r++;
                    }
                    let _t = r;
                    var wr = Ar === r;
                  }
                  if (C["index-fields"] !== void 0) {
                    let G = C["index-fields"], Ar = r, _t = r;
                    if (G && typeof G == "object" && !Array.isArray(G)) {
                      let Er = r;
                      for (let Wa in G)
                        if (!(Wa === "href" || Wa === "title" || Wa === "text" || Wa === "section")) {
                          let Rr = { instancePath: o + "/book/search/algolia/index-fields", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/properties/index-fields/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: Wa, schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"] }, message: "must NOT have additional properties", schema: false, parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"], data: G };
                          e === null ? e = [Rr] : e.push(Rr), r++;
                        }
                      if (G.href !== void 0) {
                        let Wa = G.href, Rr = r;
                        if (typeof Wa != "string") {
                          let ya = { instancePath: o + "/book/search/algolia/index-fields/href", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/properties/index-fields/properties/href/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.href }, message: "must be string", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.href.type, parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.href, data: Wa };
                          e === null ? e = [ya] : e.push(ya), r++;
                        }
                        let Qr = r;
                        var ta = Rr === r;
                      }
                      if (G.title !== void 0) {
                        let Wa = G.title, Rr = r;
                        if (typeof Wa != "string") {
                          let pa = { instancePath: o + "/book/search/algolia/index-fields/title", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/properties/index-fields/properties/title/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.title }, message: "must be string", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.title.type, parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.title, data: Wa };
                          e === null ? e = [pa] : e.push(pa), r++;
                        }
                        let Qr = r;
                        var ta = Rr === r;
                      }
                      if (G.text !== void 0) {
                        let Wa = G.text, Rr = r;
                        if (typeof Wa != "string") {
                          let pa = { instancePath: o + "/book/search/algolia/index-fields/text", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/properties/index-fields/properties/text/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.text }, message: "must be string", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.text.type, parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.text, data: Wa };
                          e === null ? e = [pa] : e.push(pa), r++;
                        }
                        let Qr = r;
                        var ta = Rr === r;
                      }
                      if (G.section !== void 0) {
                        let Wa = G.section, Rr = r;
                        if (typeof Wa != "string") {
                          let pa = { instancePath: o + "/book/search/algolia/index-fields/section", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/properties/index-fields/properties/section/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.section }, message: "must be string", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.section.type, parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"].properties.section, data: Wa };
                          e === null ? e = [pa] : e.push(pa), r++;
                        }
                        let Qr = r;
                        var ta = Rr === r;
                      }
                    } else {
                      let Er = { instancePath: o + "/book/search/algolia/index-fields", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/properties/index-fields/type", keyword: "type", params: { type: "object", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"] }, message: "must be object", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"].type, parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties["index-fields"], data: G };
                      e === null ? e = [Er] : e.push(Er), r++;
                    }
                    var wr = Ar === r;
                  }
                  if (C.params !== void 0) {
                    let G = C.params, Ar = r, _t = r;
                    if (!(G && typeof G == "object" && !Array.isArray(G))) {
                      let Er = { instancePath: o + "/book/search/algolia/params", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/properties/params/type", keyword: "type", params: { type: "object", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties.params }, message: "must be object", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties.params.type, parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia.properties.params, data: G };
                      e === null ? e = [Er] : e.push(Er), r++;
                    }
                    var wr = Ar === r;
                  }
                } else {
                  let Yr = { instancePath: o + "/book/search/algolia", schemaPath: "#/properties/book/properties/search/oneOf/1/properties/algolia/type", keyword: "type", params: { type: "object", schema: b.properties.book.properties.search.oneOf[1].properties.algolia }, message: "must be object", schema: b.properties.book.properties.search.oneOf[1].properties.algolia.type, parentSchema: b.properties.book.properties.search.oneOf[1].properties.algolia, data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                var Cr = te === r;
              }
            } else {
              let W = { instancePath: o + "/book/search", schemaPath: "#/properties/book/properties/search/oneOf/1/type", keyword: "type", params: { type: "object", schema: b.properties.book.properties.search.oneOf[1] }, message: "must be object", schema: b.properties.book.properties.search.oneOf[1].type, parentSchema: b.properties.book.properties.search.oneOf[1], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var f = Fr === r;
            if (f && Z ? (Z = false, T = [T, 1]) : f && (Z = true, T = 1), Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let W = { instancePath: o + "/book/search", schemaPath: "#/properties/book/properties/search/oneOf", keyword: "oneOf", params: { passingSchemas: T, schema: b.properties.book.properties.search }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties.search.oneOf, parentSchema: b.properties.book.properties.search, data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var $ = E === r;
          }
          if (J.navbar !== void 0) {
            let v = J.navbar, E = r, A = r, Z = false, T = null, U = r;
            if (typeof v != "boolean") {
              let W = { instancePath: o + "/book/navbar", schemaPath: "#/properties/book/properties/navbar/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.navbar.oneOf[0] }, message: "must be boolean", schema: b.properties.book.properties.navbar.oneOf[0].type, parentSchema: b.properties.book.properties.navbar.oneOf[0], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            let re = r;
            var jt = U === r;
            jt && (Z = true, T = 0);
            let Fr = r, z = r;
            if (v && typeof v == "object" && !Array.isArray(v)) {
              for (let W in v) {
                let C = r, te = r;
                if (typeof W == "string") {
                  if (!sg.test(W)) {
                    let Br = { instancePath: o + "/book/navbar", schemaPath: "#/properties/book/properties/navbar/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(collapse_below|collapseBelow))", schema: b.properties.book.properties.navbar.oneOf[1].propertyNames }, message: 'must match pattern "^(?!(collapse_below|collapseBelow))"', schema: "^(?!(collapse_below|collapseBelow))", parentSchema: b.properties.book.properties.navbar.oneOf[1].propertyNames, data: W, propertyName: W };
                    e === null ? e = [Br] : e.push(Br), r++;
                  }
                } else {
                  let Br = { instancePath: o + "/book/navbar", schemaPath: "#/properties/book/properties/navbar/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.navbar.oneOf[1].propertyNames }, message: "must be string", schema: b.properties.book.properties.navbar.oneOf[1].propertyNames.type, parentSchema: b.properties.book.properties.navbar.oneOf[1].propertyNames, data: W, propertyName: W };
                  e === null ? e = [Br] : e.push(Br), r++;
                }
                var ee = C === r;
                if (!ee) {
                  let Br = { instancePath: o + "/book/navbar", schemaPath: "#/properties/book/properties/navbar/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: W, schema: b.properties.book.properties.navbar.oneOf[1] }, message: "property name must be valid", schema: b.properties.book.properties.navbar.oneOf[1].propertyNames, parentSchema: b.properties.book.properties.navbar.oneOf[1], data: v };
                  e === null ? e = [Br] : e.push(Br), r++;
                }
              }
              if (v.title !== void 0) {
                let W = v.title, C = r, te = r, Br = false, X = null, Yr = r;
                if (typeof W != "string") {
                  let Zr = { instancePath: o + "/book/navbar/title", schemaPath: "#/properties/book/properties/navbar/oneOf/1/properties/title/oneOf/0/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.navbar.oneOf[1].properties.title.oneOf[0] }, message: "must be string", schema: b.properties.book.properties.navbar.oneOf[1].properties.title.oneOf[0].type, parentSchema: b.properties.book.properties.navbar.oneOf[1].properties.title.oneOf[0], data: W };
                  e === null ? e = [Zr] : e.push(Zr), r++;
                }
                let G = r;
                var za = Yr === r;
                za && (Br = true, X = 0);
                let Ar = r;
                if (typeof W != "boolean") {
                  let Zr = { instancePath: o + "/book/navbar/title", schemaPath: "#/properties/book/properties/navbar/oneOf/1/properties/title/oneOf/1/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.navbar.oneOf[1].properties.title.oneOf[1] }, message: "must be boolean", schema: b.properties.book.properties.navbar.oneOf[1].properties.title.oneOf[1].type, parentSchema: b.properties.book.properties.navbar.oneOf[1].properties.title.oneOf[1], data: W };
                  e === null ? e = [Zr] : e.push(Zr), r++;
                }
                let _t = r;
                var za = Ar === r;
                if (za && Br ? (Br = false, X = [X, 1]) : za && (Br = true, X = 1), Br)
                  r = te, e !== null && (te ? e.length = te : e = null);
                else {
                  let Zr = { instancePath: o + "/book/navbar/title", schemaPath: "#/properties/book/properties/navbar/oneOf/1/properties/title/oneOf", keyword: "oneOf", params: { passingSchemas: X, schema: b.properties.book.properties.navbar.oneOf[1].properties.title }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties.navbar.oneOf[1].properties.title.oneOf, parentSchema: b.properties.book.properties.navbar.oneOf[1].properties.title, data: W };
                  e === null ? e = [Zr] : e.push(Zr), r++;
                }
                var sa = C === r;
              }
              if (v.logo !== void 0) {
                let W = v.logo, C = r;
                if (typeof W != "string") {
                  let X = { instancePath: o + "/book/navbar/logo", schemaPath: "#/properties/book/properties/navbar/oneOf/1/properties/logo/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.navbar.oneOf[1].properties.logo }, message: "must be string", schema: b.properties.book.properties.navbar.oneOf[1].properties.logo.type, parentSchema: b.properties.book.properties.navbar.oneOf[1].properties.logo, data: W };
                  e === null ? e = [X] : e.push(X), r++;
                }
                let te = r;
                var sa = C === r;
              }
              if (v.background !== void 0) {
                let W = v.background, C = r, te = r, Br = false, X = r, Yr = b.properties.book.properties.navbar.oneOf[1].properties.background.anyOf[0].enum;
                if (!(W === "primary" || W === "secondary" || W === "success" || W === "danger" || W === "warning" || W === "info" || W === "light" || W === "dark")) {
                  let Ar = { instancePath: o + "/book/navbar/background", schemaPath: "#/properties/book/properties/navbar/oneOf/1/properties/background/anyOf/0/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.navbar.oneOf[1].properties.background.anyOf[0].enum, schema: b.properties.book.properties.navbar.oneOf[1].properties.background.anyOf[0] }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.navbar.oneOf[1].properties.background.anyOf[0].enum, parentSchema: b.properties.book.properties.navbar.oneOf[1].properties.background.anyOf[0], data: W };
                  e === null ? e = [Ar] : e.push(Ar), r++;
                }
                var _e = X === r;
                if (Br = Br || _e, !Br) {
                  let Ar = r;
                  if (typeof W != "string") {
                    let Er = { instancePath: o + "/book/navbar/background", schemaPath: "#/properties/book/properties/navbar/oneOf/1/properties/background/anyOf/1/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.navbar.oneOf[1].properties.background.anyOf[1] }, message: "must be string", schema: b.properties.book.properties.navbar.oneOf[1].properties.background.anyOf[1].type, parentSchema: b.properties.book.properties.navbar.oneOf[1].properties.background.anyOf[1], data: W };
                    e === null ? e = [Er] : e.push(Er), r++;
                  }
                  let _t = r;
                  var _e = Ar === r;
                  Br = Br || _e;
                }
                if (Br)
                  r = te, e !== null && (te ? e.length = te : e = null);
                else {
                  let Ar = { instancePath: o + "/book/navbar/background", schemaPath: "#/properties/book/properties/navbar/oneOf/1/properties/background/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: b.properties.book.properties.navbar.oneOf[1].properties.background.anyOf, parentSchema: b.properties.book.properties.navbar.oneOf[1].properties.background, data: W };
                  e === null ? e = [Ar] : e.push(Ar), r++;
                }
                var sa = C === r;
              }
              if (v.foreground !== void 0) {
                let W = v.foreground, C = r, te = r, Br = false, X = r, Yr = b.properties.book.properties.navbar.oneOf[1].properties.foreground.anyOf[0].enum;
                if (!(W === "primary" || W === "secondary" || W === "success" || W === "danger" || W === "warning" || W === "info" || W === "light" || W === "dark")) {
                  let Ar = { instancePath: o + "/book/navbar/foreground", schemaPath: "#/properties/book/properties/navbar/oneOf/1/properties/foreground/anyOf/0/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.navbar.oneOf[1].properties.foreground.anyOf[0].enum, schema: b.properties.book.properties.navbar.oneOf[1].properties.foreground.anyOf[0] }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.navbar.oneOf[1].properties.foreground.anyOf[0].enum, parentSchema: b.properties.book.properties.navbar.oneOf[1].properties.foreground.anyOf[0], data: W };
                  e === null ? e = [Ar] : e.push(Ar), r++;
                }
                var Ma = X === r;
                if (Br = Br || Ma, !Br) {
                  let Ar = r;
                  if (typeof W != "string") {
                    let Er = { instancePath: o + "/book/navbar/foreground", schemaPath: "#/properties/book/properties/navbar/oneOf/1/properties/foreground/anyOf/1/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.navbar.oneOf[1].properties.foreground.anyOf[1] }, message: "must be string", schema: b.properties.book.properties.navbar.oneOf[1].properties.foreground.anyOf[1].type, parentSchema: b.properties.book.properties.navbar.oneOf[1].properties.foreground.anyOf[1], data: W };
                    e === null ? e = [Er] : e.push(Er), r++;
                  }
                  let _t = r;
                  var Ma = Ar === r;
                  Br = Br || Ma;
                }
                if (Br)
                  r = te, e !== null && (te ? e.length = te : e = null);
                else {
                  let Ar = { instancePath: o + "/book/navbar/foreground", schemaPath: "#/properties/book/properties/navbar/oneOf/1/properties/foreground/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: b.properties.book.properties.navbar.oneOf[1].properties.foreground.anyOf, parentSchema: b.properties.book.properties.navbar.oneOf[1].properties.foreground, data: W };
                  e === null ? e = [Ar] : e.push(Ar), r++;
                }
                var sa = C === r;
              }
              if (v.search !== void 0) {
                let W = v.search, C = r;
                if (typeof W != "boolean") {
                  let X = { instancePath: o + "/book/navbar/search", schemaPath: "#/properties/book/properties/navbar/oneOf/1/properties/search/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.navbar.oneOf[1].properties.search }, message: "must be boolean", schema: b.properties.book.properties.navbar.oneOf[1].properties.search.type, parentSchema: b.properties.book.properties.navbar.oneOf[1].properties.search, data: W };
                  e === null ? e = [X] : e.push(X), r++;
                }
                let te = r;
                var sa = C === r;
              }
              if (v.pinned !== void 0) {
                let W = v.pinned, C = r;
                if (typeof W != "boolean") {
                  let X = { instancePath: o + "/book/navbar/pinned", schemaPath: "#/properties/book/properties/navbar/oneOf/1/properties/pinned/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.navbar.oneOf[1].properties.pinned }, message: "must be boolean", schema: b.properties.book.properties.navbar.oneOf[1].properties.pinned.type, parentSchema: b.properties.book.properties.navbar.oneOf[1].properties.pinned, data: W };
                  e === null ? e = [X] : e.push(X), r++;
                }
                let te = r;
                var sa = C === r;
              }
              if (v.collapse !== void 0) {
                let W = v.collapse, C = r;
                if (typeof W != "boolean") {
                  let X = { instancePath: o + "/book/navbar/collapse", schemaPath: "#/properties/book/properties/navbar/oneOf/1/properties/collapse/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.navbar.oneOf[1].properties.collapse }, message: "must be boolean", schema: b.properties.book.properties.navbar.oneOf[1].properties.collapse.type, parentSchema: b.properties.book.properties.navbar.oneOf[1].properties.collapse, data: W };
                  e === null ? e = [X] : e.push(X), r++;
                }
                let te = r;
                var sa = C === r;
              }
              if (v["collapse-below"] !== void 0) {
                let W = v["collapse-below"], C = r, te = b.properties.book.properties.navbar.oneOf[1].properties["collapse-below"].enum;
                if (!(W === "sm" || W === "md" || W === "lg" || W === "xl" || W === "xxl")) {
                  let X = { instancePath: o + "/book/navbar/collapse-below", schemaPath: "#/properties/book/properties/navbar/oneOf/1/properties/collapse-below/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.navbar.oneOf[1].properties["collapse-below"].enum, schema: b.properties.book.properties.navbar.oneOf[1].properties["collapse-below"] }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.navbar.oneOf[1].properties["collapse-below"].enum, parentSchema: b.properties.book.properties.navbar.oneOf[1].properties["collapse-below"], data: W };
                  e === null ? e = [X] : e.push(X), r++;
                }
                var sa = C === r;
              }
              if (v.left !== void 0) {
                let W = v.left, C = r, te = r;
                if (Array.isArray(W)) {
                  let X = true, Yr = W.length;
                  for (let G = 0; G < Yr; G++) {
                    let Ar = W[G], _t = r;
                    wa(Ar, { instancePath: o + "/book/navbar/left/" + G, parentData: W, parentDataProperty: G, rootData: s }) || (e = e === null ? wa.errors : e.concat(wa.errors), r = e.length);
                    var fa = _t === r;
                    fa || (X = false);
                  }
                } else {
                  let X = { instancePath: o + "/book/navbar/left", schemaPath: "#/properties/book/properties/navbar/oneOf/1/properties/left/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties.navbar.oneOf[1].properties.left }, message: "must be array", schema: b.properties.book.properties.navbar.oneOf[1].properties.left.type, parentSchema: b.properties.book.properties.navbar.oneOf[1].properties.left, data: W };
                  e === null ? e = [X] : e.push(X), r++;
                }
                var sa = C === r;
              }
              if (v.right !== void 0) {
                let W = v.right, C = r, te = r;
                if (Array.isArray(W)) {
                  let X = true, Yr = W.length;
                  for (let G = 0; G < Yr; G++) {
                    let Ar = W[G], _t = r;
                    wa(Ar, { instancePath: o + "/book/navbar/right/" + G, parentData: W, parentDataProperty: G, rootData: s }) || (e = e === null ? wa.errors : e.concat(wa.errors), r = e.length);
                    var ie = _t === r;
                    ie || (X = false);
                  }
                } else {
                  let X = { instancePath: o + "/book/navbar/right", schemaPath: "#/properties/book/properties/navbar/oneOf/1/properties/right/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties.navbar.oneOf[1].properties.right }, message: "must be array", schema: b.properties.book.properties.navbar.oneOf[1].properties.right.type, parentSchema: b.properties.book.properties.navbar.oneOf[1].properties.right, data: W };
                  e === null ? e = [X] : e.push(X), r++;
                }
                var sa = C === r;
              }
            } else {
              let W = { instancePath: o + "/book/navbar", schemaPath: "#/properties/book/properties/navbar/oneOf/1/type", keyword: "type", params: { type: "object", schema: b.properties.book.properties.navbar.oneOf[1] }, message: "must be object", schema: b.properties.book.properties.navbar.oneOf[1].type, parentSchema: b.properties.book.properties.navbar.oneOf[1], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var jt = Fr === r;
            if (jt && Z ? (Z = false, T = [T, 1]) : jt && (Z = true, T = 1), Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let W = { instancePath: o + "/book/navbar", schemaPath: "#/properties/book/properties/navbar/oneOf", keyword: "oneOf", params: { passingSchemas: T, schema: b.properties.book.properties.navbar }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties.navbar.oneOf, parentSchema: b.properties.book.properties.navbar, data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var $ = E === r;
          }
          if (J.sidebar !== void 0) {
            let v = J.sidebar, E = r, A = r, Z = false, T = null, U = r;
            if (typeof v != "boolean") {
              let G = { instancePath: o + "/book/sidebar", schemaPath: "#/properties/book/properties/sidebar/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.sidebar.oneOf[0] }, message: "must be boolean", schema: b.properties.book.properties.sidebar.oneOf[0].type, parentSchema: b.properties.book.properties.sidebar.oneOf[0], data: v };
              e === null ? e = [G] : e.push(G), r++;
            }
            let re = r;
            var ka = U === r;
            ka && (Z = true, T = 0);
            let Fr = r, z = r, Xr = false, W = null, C = r, te = r;
            if (v && typeof v == "object" && !Array.isArray(v)) {
              for (let G in v) {
                let Ar = r, _t = r;
                if (typeof G == "string") {
                  if (!vd.test(G)) {
                    let Zr = { instancePath: o + "/book/sidebar", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(collapse_level|collapseLevel))", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].propertyNames }, message: 'must match pattern "^(?!(collapse_level|collapseLevel))"', schema: "^(?!(collapse_level|collapseLevel))", parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].propertyNames, data: G, propertyName: G };
                    e === null ? e = [Zr] : e.push(Zr), r++;
                  }
                } else {
                  let Zr = { instancePath: o + "/book/sidebar", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/propertyNames/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].propertyNames }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].propertyNames.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].propertyNames, data: G, propertyName: G };
                  e === null ? e = [Zr] : e.push(Zr), r++;
                }
                var hn = Ar === r;
                if (!hn) {
                  let Zr = { instancePath: o + "/book/sidebar", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/propertyNames", keyword: "propertyNames", params: { propertyName: G, schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0] }, message: "property name must be valid", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].propertyNames, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0], data: v };
                  e === null ? e = [Zr] : e.push(Zr), r++;
                }
              }
              if (v.id !== void 0) {
                let G = v.id, Ar = r;
                if (typeof G != "string") {
                  let Zr = { instancePath: o + "/book/sidebar/id", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/id/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.id }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.id.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.id, data: G };
                  e === null ? e = [Zr] : e.push(Zr), r++;
                }
                let _t = r;
                var et = Ar === r;
              }
              if (v.title !== void 0) {
                let G = v.title, Ar = r, _t = r, Zr = false, Er = null, Wa = r;
                if (typeof G != "string") {
                  let Nr = { instancePath: o + "/book/sidebar/title", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/title/oneOf/0/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[0] }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[0].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[0], data: G };
                  e === null ? e = [Nr] : e.push(Nr), r++;
                }
                let Rr = r;
                var yt = Wa === r;
                yt && (Zr = true, Er = 0);
                let Qr = r;
                if (typeof G != "boolean") {
                  let Nr = { instancePath: o + "/book/sidebar/title", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/title/oneOf/1/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[1] }, message: "must be boolean", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[1].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf[1], data: G };
                  e === null ? e = [Nr] : e.push(Nr), r++;
                }
                let ya = r;
                var yt = Qr === r;
                if (yt && Zr ? (Zr = false, Er = [Er, 1]) : yt && (Zr = true, Er = 1), Zr)
                  r = _t, e !== null && (_t ? e.length = _t : e = null);
                else {
                  let Nr = { instancePath: o + "/book/sidebar/title", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/title/oneOf", keyword: "oneOf", params: { passingSchemas: Er, schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.title }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.title.oneOf, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.title, data: G };
                  e === null ? e = [Nr] : e.push(Nr), r++;
                }
                var et = Ar === r;
              }
              if (v.subtitle !== void 0) {
                let G = v.subtitle, Ar = r;
                if (typeof G != "string") {
                  let Er = { instancePath: o + "/book/sidebar/subtitle", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/subtitle/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.subtitle }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.subtitle.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.subtitle, data: G };
                  e === null ? e = [Er] : e.push(Er), r++;
                }
                let _t = r;
                var et = Ar === r;
              }
              if (v.logo !== void 0) {
                let G = v.logo, Ar = r;
                if (typeof G != "string") {
                  let Er = { instancePath: o + "/book/sidebar/logo", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/logo/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.logo }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.logo.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.logo, data: G };
                  e === null ? e = [Er] : e.push(Er), r++;
                }
                let _t = r;
                var et = Ar === r;
              }
              if (v.search !== void 0) {
                let G = v.search, Ar = r;
                if (typeof G != "boolean") {
                  let Er = { instancePath: o + "/book/sidebar/search", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/search/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.search }, message: "must be boolean", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.search.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.search, data: G };
                  e === null ? e = [Er] : e.push(Er), r++;
                }
                let _t = r;
                var et = Ar === r;
              }
              if (v.tools !== void 0) {
                let G = v.tools, Ar = r, _t = r;
                if (Array.isArray(G)) {
                  let Er = true, Wa = G.length;
                  for (let Rr = 0; Rr < Wa; Rr++) {
                    let Qr = G[Rr], ya = r;
                    co(Qr, { instancePath: o + "/book/sidebar/tools/" + Rr, parentData: G, parentDataProperty: Rr, rootData: s }) || (e = e === null ? co.errors : e.concat(co.errors), r = e.length);
                    var _n = ya === r;
                    _n || (Er = false);
                  }
                } else {
                  let Er = { instancePath: o + "/book/sidebar/tools", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/tools/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.tools }, message: "must be array", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.tools.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.tools, data: G };
                  e === null ? e = [Er] : e.push(Er), r++;
                }
                var et = Ar === r;
              }
              if (v.contents !== void 0) {
                let G = v.contents, Ar = r;
                so(G, { instancePath: o + "/book/sidebar/contents", parentData: v, parentDataProperty: "contents", rootData: s }) || (e = e === null ? so.errors : e.concat(so.errors), r = e.length);
                var et = Ar === r;
              }
              if (v.style !== void 0) {
                let G = v.style, Ar = r, _t = b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.style.enum;
                if (!(G === "docked" || G === "floating")) {
                  let Er = { instancePath: o + "/book/sidebar/style", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/style/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.style.enum, schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.style }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.style.enum, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.style, data: G };
                  e === null ? e = [Er] : e.push(Er), r++;
                }
                var et = Ar === r;
              }
              if (v.background !== void 0) {
                let G = v.background, Ar = r, _t = r, Zr = false, Er = r, Wa = b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[0].enum;
                if (!(G === "primary" || G === "secondary" || G === "success" || G === "danger" || G === "warning" || G === "info" || G === "light" || G === "dark")) {
                  let Qr = { instancePath: o + "/book/sidebar/background", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/background/anyOf/0/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[0].enum, schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[0] }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[0].enum, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[0], data: G };
                  e === null ? e = [Qr] : e.push(Qr), r++;
                }
                var st = Er === r;
                if (Zr = Zr || st, !Zr) {
                  let Qr = r;
                  if (typeof G != "string") {
                    let Nr = { instancePath: o + "/book/sidebar/background", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/background/anyOf/1/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[1] }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[1].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf[1], data: G };
                    e === null ? e = [Nr] : e.push(Nr), r++;
                  }
                  let ya = r;
                  var st = Qr === r;
                  Zr = Zr || st;
                }
                if (Zr)
                  r = _t, e !== null && (_t ? e.length = _t : e = null);
                else {
                  let Qr = { instancePath: o + "/book/sidebar/background", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/background/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.background.anyOf, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.background, data: G };
                  e === null ? e = [Qr] : e.push(Qr), r++;
                }
                var et = Ar === r;
              }
              if (v.foreground !== void 0) {
                let G = v.foreground, Ar = r, _t = r, Zr = false, Er = r, Wa = b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[0].enum;
                if (!(G === "primary" || G === "secondary" || G === "success" || G === "danger" || G === "warning" || G === "info" || G === "light" || G === "dark")) {
                  let Qr = { instancePath: o + "/book/sidebar/foreground", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/foreground/anyOf/0/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[0].enum, schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[0] }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[0].enum, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[0], data: G };
                  e === null ? e = [Qr] : e.push(Qr), r++;
                }
                var Et = Er === r;
                if (Zr = Zr || Et, !Zr) {
                  let Qr = r;
                  if (typeof G != "string") {
                    let Nr = { instancePath: o + "/book/sidebar/foreground", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/foreground/anyOf/1/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[1] }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[1].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf[1], data: G };
                    e === null ? e = [Nr] : e.push(Nr), r++;
                  }
                  let ya = r;
                  var Et = Qr === r;
                  Zr = Zr || Et;
                }
                if (Zr)
                  r = _t, e !== null && (_t ? e.length = _t : e = null);
                else {
                  let Qr = { instancePath: o + "/book/sidebar/foreground", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/foreground/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.foreground.anyOf, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.foreground, data: G };
                  e === null ? e = [Qr] : e.push(Qr), r++;
                }
                var et = Ar === r;
              }
              if (v.border !== void 0) {
                let G = v.border, Ar = r;
                if (typeof G != "boolean") {
                  let Er = { instancePath: o + "/book/sidebar/border", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/border/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.border }, message: "must be boolean", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.border.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.border, data: G };
                  e === null ? e = [Er] : e.push(Er), r++;
                }
                let _t = r;
                var et = Ar === r;
              }
              if (v.alignment !== void 0) {
                let G = v.alignment, Ar = r, _t = b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.alignment.enum;
                if (!(G === "left" || G === "right" || G === "center")) {
                  let Er = { instancePath: o + "/book/sidebar/alignment", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/alignment/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.alignment.enum, schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.alignment }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.alignment.enum, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.alignment, data: G };
                  e === null ? e = [Er] : e.push(Er), r++;
                }
                var et = Ar === r;
              }
              if (v["collapse-level"] !== void 0) {
                let G = v["collapse-level"], Ar = r;
                if (!(typeof G == "number" && isFinite(G))) {
                  let Er = { instancePath: o + "/book/sidebar/collapse-level", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/collapse-level/type", keyword: "type", params: { type: "number", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties["collapse-level"] }, message: "must be number", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties["collapse-level"].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties["collapse-level"], data: G };
                  e === null ? e = [Er] : e.push(Er), r++;
                }
                let _t = r;
                var et = Ar === r;
              }
              if (v.pinned !== void 0) {
                let G = v.pinned, Ar = r;
                if (typeof G != "boolean") {
                  let Er = { instancePath: o + "/book/sidebar/pinned", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/pinned/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.pinned }, message: "must be boolean", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.pinned.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.pinned, data: G };
                  e === null ? e = [Er] : e.push(Er), r++;
                }
                let _t = r;
                var et = Ar === r;
              }
              if (v.header !== void 0) {
                let G = v.header, Ar = r, _t = r, Zr = false, Er = null, Wa = r;
                if (typeof G != "string") {
                  let Nr = { instancePath: o + "/book/sidebar/header", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/header/oneOf/0/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[0] }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[0].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[0], data: G };
                  e === null ? e = [Nr] : e.push(Nr), r++;
                }
                let Rr = r;
                var ca = Wa === r;
                ca && (Zr = true, Er = 0);
                let Qr = r, ya = r;
                if (Array.isArray(G)) {
                  let Nr = true, lo = G.length;
                  for (let Tn = 0; Tn < lo; Tn++) {
                    let bt = G[Tn], Ro = r;
                    if (typeof bt != "string") {
                      let dt = { instancePath: o + "/book/sidebar/header/" + Tn, schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/header/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1].items }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1].items.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1].items, data: bt };
                      e === null ? e = [dt] : e.push(dt), r++;
                    }
                    let Xm = r;
                    var Gr = Ro === r;
                    Gr || (Nr = false);
                  }
                } else {
                  let Nr = { instancePath: o + "/book/sidebar/header", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/header/oneOf/1/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1] }, message: "must be array", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf[1], data: G };
                  e === null ? e = [Nr] : e.push(Nr), r++;
                }
                var ca = Qr === r;
                if (ca && Zr ? (Zr = false, Er = [Er, 1]) : ca && (Zr = true, Er = 1), Zr)
                  r = _t, e !== null && (_t ? e.length = _t : e = null);
                else {
                  let Nr = { instancePath: o + "/book/sidebar/header", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/header/oneOf", keyword: "oneOf", params: { passingSchemas: Er, schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.header }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.header.oneOf, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.header, data: G };
                  e === null ? e = [Nr] : e.push(Nr), r++;
                }
                var et = Ar === r;
              }
              if (v.footer !== void 0) {
                let G = v.footer, Ar = r, _t = r, Zr = false, Er = null, Wa = r;
                if (typeof G != "string") {
                  let Nr = { instancePath: o + "/book/sidebar/footer", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/footer/oneOf/0/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[0] }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[0].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[0], data: G };
                  e === null ? e = [Nr] : e.push(Nr), r++;
                }
                let Rr = r;
                var Tt = Wa === r;
                Tt && (Zr = true, Er = 0);
                let Qr = r, ya = r;
                if (Array.isArray(G)) {
                  let Nr = true, lo = G.length;
                  for (let Tn = 0; Tn < lo; Tn++) {
                    let bt = G[Tn], Ro = r;
                    if (typeof bt != "string") {
                      let dt = { instancePath: o + "/book/sidebar/footer/" + Tn, schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/footer/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1].items }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1].items.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1].items, data: bt };
                      e === null ? e = [dt] : e.push(dt), r++;
                    }
                    let Xm = r;
                    var bn = Ro === r;
                    bn || (Nr = false);
                  }
                } else {
                  let Nr = { instancePath: o + "/book/sidebar/footer", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/footer/oneOf/1/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1] }, message: "must be array", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf[1], data: G };
                  e === null ? e = [Nr] : e.push(Nr), r++;
                }
                var Tt = Qr === r;
                if (Tt && Zr ? (Zr = false, Er = [Er, 1]) : Tt && (Zr = true, Er = 1), Zr)
                  r = _t, e !== null && (_t ? e.length = _t : e = null);
                else {
                  let Nr = { instancePath: o + "/book/sidebar/footer", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/properties/footer/oneOf", keyword: "oneOf", params: { passingSchemas: Er, schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.footer }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.footer.oneOf, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].properties.footer, data: G };
                  e === null ? e = [Nr] : e.push(Nr), r++;
                }
                var et = Ar === r;
              }
            } else {
              let G = { instancePath: o + "/book/sidebar", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/0/type", keyword: "type", params: { type: "object", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0] }, message: "must be object", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[0], data: v };
              e === null ? e = [G] : e.push(G), r++;
            }
            var vt = C === r;
            vt && (Xr = true, W = 0);
            let Br = r, X = r;
            if (Array.isArray(v)) {
              let G = true, Ar = v.length;
              for (let _t = 0; _t < Ar; _t++) {
                let Zr = v[_t], Er = r, Wa = r;
                if (Zr && typeof Zr == "object" && !Array.isArray(Zr)) {
                  for (let Rr in Zr) {
                    let Qr = r, ya = r;
                    if (typeof Rr == "string") {
                      if (!vd.test(Rr)) {
                        let pa = { instancePath: o + "/book/sidebar/" + _t, schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(collapse_level|collapseLevel))", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames }, message: 'must match pattern "^(?!(collapse_level|collapseLevel))"', schema: "^(?!(collapse_level|collapseLevel))", parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames, data: Rr, propertyName: Rr };
                        e === null ? e = [pa] : e.push(pa), r++;
                      }
                    } else {
                      let pa = { instancePath: o + "/book/sidebar/" + _t, schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/propertyNames/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames, data: Rr, propertyName: Rr };
                      e === null ? e = [pa] : e.push(pa), r++;
                    }
                    var yn = Qr === r;
                    if (!yn) {
                      let pa = { instancePath: o + "/book/sidebar/" + _t, schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/propertyNames", keyword: "propertyNames", params: { propertyName: Rr, schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items }, message: "property name must be valid", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.propertyNames, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items, data: Zr };
                      e === null ? e = [pa] : e.push(pa), r++;
                    }
                  }
                  if (Zr.id !== void 0) {
                    let Rr = Zr.id, Qr = r;
                    if (typeof Rr != "string") {
                      let pa = { instancePath: o + "/book/sidebar/" + _t + "/id", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/id/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.id }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.id.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.id, data: Rr };
                      e === null ? e = [pa] : e.push(pa), r++;
                    }
                    let ya = r;
                    var Va = Qr === r;
                  }
                  if (Zr.title !== void 0) {
                    let Rr = Zr.title, Qr = r, ya = r, pa = false, Nr = null, lo = r;
                    if (typeof Rr != "string") {
                      let dt = { instancePath: o + "/book/sidebar/" + _t + "/title", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/title/oneOf/0/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[0] }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[0].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[0], data: Rr };
                      e === null ? e = [dt] : e.push(dt), r++;
                    }
                    let Tn = r;
                    var Dt = lo === r;
                    Dt && (pa = true, Nr = 0);
                    let bt = r;
                    if (typeof Rr != "boolean") {
                      let dt = { instancePath: o + "/book/sidebar/" + _t + "/title", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/title/oneOf/1/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[1] }, message: "must be boolean", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[1].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf[1], data: Rr };
                      e === null ? e = [dt] : e.push(dt), r++;
                    }
                    let Ro = r;
                    var Dt = bt === r;
                    if (Dt && pa ? (pa = false, Nr = [Nr, 1]) : Dt && (pa = true, Nr = 1), pa)
                      r = ya, e !== null && (ya ? e.length = ya : e = null);
                    else {
                      let dt = { instancePath: o + "/book/sidebar/" + _t + "/title", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/title/oneOf", keyword: "oneOf", params: { passingSchemas: Nr, schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.title }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.title.oneOf, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.title, data: Rr };
                      e === null ? e = [dt] : e.push(dt), r++;
                    }
                    var Va = Qr === r;
                  }
                  if (Zr.subtitle !== void 0) {
                    let Rr = Zr.subtitle, Qr = r;
                    if (typeof Rr != "string") {
                      let Nr = { instancePath: o + "/book/sidebar/" + _t + "/subtitle", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/subtitle/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.subtitle }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.subtitle.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.subtitle, data: Rr };
                      e === null ? e = [Nr] : e.push(Nr), r++;
                    }
                    let ya = r;
                    var Va = Qr === r;
                  }
                  if (Zr.logo !== void 0) {
                    let Rr = Zr.logo, Qr = r;
                    if (typeof Rr != "string") {
                      let Nr = { instancePath: o + "/book/sidebar/" + _t + "/logo", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/logo/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.logo }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.logo.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.logo, data: Rr };
                      e === null ? e = [Nr] : e.push(Nr), r++;
                    }
                    let ya = r;
                    var Va = Qr === r;
                  }
                  if (Zr.search !== void 0) {
                    let Rr = Zr.search, Qr = r;
                    if (typeof Rr != "boolean") {
                      let Nr = { instancePath: o + "/book/sidebar/" + _t + "/search", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/search/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.search }, message: "must be boolean", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.search.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.search, data: Rr };
                      e === null ? e = [Nr] : e.push(Nr), r++;
                    }
                    let ya = r;
                    var Va = Qr === r;
                  }
                  if (Zr.tools !== void 0) {
                    let Rr = Zr.tools, Qr = r, ya = r;
                    if (Array.isArray(Rr)) {
                      let Nr = true, lo = Rr.length;
                      for (let Tn = 0; Tn < lo; Tn++) {
                        let bt = Rr[Tn], Ro = r;
                        co(bt, { instancePath: o + "/book/sidebar/" + _t + "/tools/" + Tn, parentData: Rr, parentDataProperty: Tn, rootData: s }) || (e = e === null ? co.errors : e.concat(co.errors), r = e.length);
                        var nl = Ro === r;
                        nl || (Nr = false);
                      }
                    } else {
                      let Nr = { instancePath: o + "/book/sidebar/" + _t + "/tools", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/tools/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.tools }, message: "must be array", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.tools.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.tools, data: Rr };
                      e === null ? e = [Nr] : e.push(Nr), r++;
                    }
                    var Va = Qr === r;
                  }
                  if (Zr.contents !== void 0) {
                    let Rr = Zr.contents, Qr = r;
                    so(Rr, { instancePath: o + "/book/sidebar/" + _t + "/contents", parentData: Zr, parentDataProperty: "contents", rootData: s }) || (e = e === null ? so.errors : e.concat(so.errors), r = e.length);
                    var Va = Qr === r;
                  }
                  if (Zr.style !== void 0) {
                    let Rr = Zr.style, Qr = r, ya = b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.style.enum;
                    if (!(Rr === "docked" || Rr === "floating")) {
                      let Nr = { instancePath: o + "/book/sidebar/" + _t + "/style", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/style/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.style.enum, schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.style }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.style.enum, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.style, data: Rr };
                      e === null ? e = [Nr] : e.push(Nr), r++;
                    }
                    var Va = Qr === r;
                  }
                  if (Zr.background !== void 0) {
                    let Rr = Zr.background, Qr = r, ya = r, pa = false, Nr = r, lo = b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[0].enum;
                    if (!(Rr === "primary" || Rr === "secondary" || Rr === "success" || Rr === "danger" || Rr === "warning" || Rr === "info" || Rr === "light" || Rr === "dark")) {
                      let bt = { instancePath: o + "/book/sidebar/" + _t + "/background", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/background/anyOf/0/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[0].enum, schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[0] }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[0].enum, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[0], data: Rr };
                      e === null ? e = [bt] : e.push(bt), r++;
                    }
                    var ol = Nr === r;
                    if (pa = pa || ol, !pa) {
                      let bt = r;
                      if (typeof Rr != "string") {
                        let dt = { instancePath: o + "/book/sidebar/" + _t + "/background", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/background/anyOf/1/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[1] }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[1].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf[1], data: Rr };
                        e === null ? e = [dt] : e.push(dt), r++;
                      }
                      let Ro = r;
                      var ol = bt === r;
                      pa = pa || ol;
                    }
                    if (pa)
                      r = ya, e !== null && (ya ? e.length = ya : e = null);
                    else {
                      let bt = { instancePath: o + "/book/sidebar/" + _t + "/background", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/background/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.background.anyOf, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.background, data: Rr };
                      e === null ? e = [bt] : e.push(bt), r++;
                    }
                    var Va = Qr === r;
                  }
                  if (Zr.foreground !== void 0) {
                    let Rr = Zr.foreground, Qr = r, ya = r, pa = false, Nr = r, lo = b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[0].enum;
                    if (!(Rr === "primary" || Rr === "secondary" || Rr === "success" || Rr === "danger" || Rr === "warning" || Rr === "info" || Rr === "light" || Rr === "dark")) {
                      let bt = { instancePath: o + "/book/sidebar/" + _t + "/foreground", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/foreground/anyOf/0/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[0].enum, schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[0] }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[0].enum, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[0], data: Rr };
                      e === null ? e = [bt] : e.push(bt), r++;
                    }
                    var wt = Nr === r;
                    if (pa = pa || wt, !pa) {
                      let bt = r;
                      if (typeof Rr != "string") {
                        let dt = { instancePath: o + "/book/sidebar/" + _t + "/foreground", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/foreground/anyOf/1/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[1] }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[1].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf[1], data: Rr };
                        e === null ? e = [dt] : e.push(dt), r++;
                      }
                      let Ro = r;
                      var wt = bt === r;
                      pa = pa || wt;
                    }
                    if (pa)
                      r = ya, e !== null && (ya ? e.length = ya : e = null);
                    else {
                      let bt = { instancePath: o + "/book/sidebar/" + _t + "/foreground", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/foreground/anyOf", keyword: "anyOf", params: {}, message: "must match a schema in anyOf", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground.anyOf, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.foreground, data: Rr };
                      e === null ? e = [bt] : e.push(bt), r++;
                    }
                    var Va = Qr === r;
                  }
                  if (Zr.border !== void 0) {
                    let Rr = Zr.border, Qr = r;
                    if (typeof Rr != "boolean") {
                      let Nr = { instancePath: o + "/book/sidebar/" + _t + "/border", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/border/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.border }, message: "must be boolean", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.border.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.border, data: Rr };
                      e === null ? e = [Nr] : e.push(Nr), r++;
                    }
                    let ya = r;
                    var Va = Qr === r;
                  }
                  if (Zr.alignment !== void 0) {
                    let Rr = Zr.alignment, Qr = r, ya = b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.alignment.enum;
                    if (!(Rr === "left" || Rr === "right" || Rr === "center")) {
                      let Nr = { instancePath: o + "/book/sidebar/" + _t + "/alignment", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/alignment/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.alignment.enum, schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.alignment }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.alignment.enum, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.alignment, data: Rr };
                      e === null ? e = [Nr] : e.push(Nr), r++;
                    }
                    var Va = Qr === r;
                  }
                  if (Zr["collapse-level"] !== void 0) {
                    let Rr = Zr["collapse-level"], Qr = r;
                    if (!(typeof Rr == "number" && isFinite(Rr))) {
                      let Nr = { instancePath: o + "/book/sidebar/" + _t + "/collapse-level", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/collapse-level/type", keyword: "type", params: { type: "number", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties["collapse-level"] }, message: "must be number", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties["collapse-level"].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties["collapse-level"], data: Rr };
                      e === null ? e = [Nr] : e.push(Nr), r++;
                    }
                    let ya = r;
                    var Va = Qr === r;
                  }
                  if (Zr.pinned !== void 0) {
                    let Rr = Zr.pinned, Qr = r;
                    if (typeof Rr != "boolean") {
                      let Nr = { instancePath: o + "/book/sidebar/" + _t + "/pinned", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/pinned/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.pinned }, message: "must be boolean", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.pinned.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.pinned, data: Rr };
                      e === null ? e = [Nr] : e.push(Nr), r++;
                    }
                    let ya = r;
                    var Va = Qr === r;
                  }
                  if (Zr.header !== void 0) {
                    let Rr = Zr.header, Qr = r, ya = r, pa = false, Nr = null, lo = r;
                    if (typeof Rr != "string") {
                      let dt = { instancePath: o + "/book/sidebar/" + _t + "/header", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/header/oneOf/0/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[0] }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[0].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[0], data: Rr };
                      e === null ? e = [dt] : e.push(dt), r++;
                    }
                    let Tn = r;
                    var ii = lo === r;
                    ii && (pa = true, Nr = 0);
                    let bt = r, Ro = r;
                    if (Array.isArray(Rr)) {
                      let dt = true, fc = Rr.length;
                      for (let il = 0; il < fc; il++) {
                        let Kr = Rr[il], mc = r;
                        if (typeof Kr != "string") {
                          let St = { instancePath: o + "/book/sidebar/" + _t + "/header/" + il, schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/header/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1].items }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1].items.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1].items, data: Kr };
                          e === null ? e = [St] : e.push(St), r++;
                        }
                        let Zm = r;
                        var Jn = mc === r;
                        Jn || (dt = false);
                      }
                    } else {
                      let dt = { instancePath: o + "/book/sidebar/" + _t + "/header", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/header/oneOf/1/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1] }, message: "must be array", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf[1], data: Rr };
                      e === null ? e = [dt] : e.push(dt), r++;
                    }
                    var ii = bt === r;
                    if (ii && pa ? (pa = false, Nr = [Nr, 1]) : ii && (pa = true, Nr = 1), pa)
                      r = ya, e !== null && (ya ? e.length = ya : e = null);
                    else {
                      let dt = { instancePath: o + "/book/sidebar/" + _t + "/header", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/header/oneOf", keyword: "oneOf", params: { passingSchemas: Nr, schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.header }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.header.oneOf, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.header, data: Rr };
                      e === null ? e = [dt] : e.push(dt), r++;
                    }
                    var Va = Qr === r;
                  }
                  if (Zr.footer !== void 0) {
                    let Rr = Zr.footer, Qr = r, ya = r, pa = false, Nr = null, lo = r;
                    if (typeof Rr != "string") {
                      let dt = { instancePath: o + "/book/sidebar/" + _t + "/footer", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/footer/oneOf/0/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[0] }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[0].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[0], data: Rr };
                      e === null ? e = [dt] : e.push(dt), r++;
                    }
                    let Tn = r;
                    var Bt = lo === r;
                    Bt && (pa = true, Nr = 0);
                    let bt = r, Ro = r;
                    if (Array.isArray(Rr)) {
                      let dt = true, fc = Rr.length;
                      for (let il = 0; il < fc; il++) {
                        let Kr = Rr[il], mc = r;
                        if (typeof Kr != "string") {
                          let St = { instancePath: o + "/book/sidebar/" + _t + "/footer/" + il, schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/footer/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1].items }, message: "must be string", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1].items.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1].items, data: Kr };
                          e === null ? e = [St] : e.push(St), r++;
                        }
                        let Zm = r;
                        var oo = mc === r;
                        oo || (dt = false);
                      }
                    } else {
                      let dt = { instancePath: o + "/book/sidebar/" + _t + "/footer", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/footer/oneOf/1/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1] }, message: "must be array", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf[1], data: Rr };
                      e === null ? e = [dt] : e.push(dt), r++;
                    }
                    var Bt = bt === r;
                    if (Bt && pa ? (pa = false, Nr = [Nr, 1]) : Bt && (pa = true, Nr = 1), pa)
                      r = ya, e !== null && (ya ? e.length = ya : e = null);
                    else {
                      let dt = { instancePath: o + "/book/sidebar/" + _t + "/footer", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/properties/footer/oneOf", keyword: "oneOf", params: { passingSchemas: Nr, schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer.oneOf, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.properties.footer, data: Rr };
                      e === null ? e = [dt] : e.push(dt), r++;
                    }
                    var Va = Qr === r;
                  }
                } else {
                  let Rr = { instancePath: o + "/book/sidebar/" + _t, schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/items/type", keyword: "type", params: { type: "object", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items }, message: "must be object", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items.type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].items, data: Zr };
                  e === null ? e = [Rr] : e.push(Rr), r++;
                }
                var qo = Er === r;
                qo || (G = false);
              }
            } else {
              let G = { instancePath: o + "/book/sidebar", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf/1/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1] }, message: "must be array", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1].type, parentSchema: b.properties.book.properties.sidebar.oneOf[1].oneOf[1], data: v };
              e === null ? e = [G] : e.push(G), r++;
            }
            var vt = Br === r;
            if (vt && Xr ? (Xr = false, W = [W, 1]) : vt && (Xr = true, W = 1), Xr)
              r = z, e !== null && (z ? e.length = z : e = null);
            else {
              let G = { instancePath: o + "/book/sidebar", schemaPath: "#/properties/book/properties/sidebar/oneOf/1/oneOf", keyword: "oneOf", params: { passingSchemas: W, schema: b.properties.book.properties.sidebar.oneOf[1] }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties.sidebar.oneOf[1].oneOf, parentSchema: b.properties.book.properties.sidebar.oneOf[1], data: v };
              e === null ? e = [G] : e.push(G), r++;
            }
            var ka = Fr === r;
            if (ka && Z ? (Z = false, T = [T, 1]) : ka && (Z = true, T = 1), Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let G = { instancePath: o + "/book/sidebar", schemaPath: "#/properties/book/properties/sidebar/oneOf", keyword: "oneOf", params: { passingSchemas: T, schema: b.properties.book.properties.sidebar }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties.sidebar.oneOf, parentSchema: b.properties.book.properties.sidebar, data: v };
              e === null ? e = [G] : e.push(G), r++;
            }
            var $ = E === r;
          }
          if (J["body-header"] !== void 0) {
            let v = J["body-header"], E = r;
            if (typeof v != "string") {
              let T = { instancePath: o + "/book/body-header", schemaPath: "#/properties/book/properties/body-header/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["body-header"] }, message: "must be string", schema: b.properties.book.properties["body-header"].type, parentSchema: b.properties.book.properties["body-header"], data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J["body-footer"] !== void 0) {
            let v = J["body-footer"], E = r;
            if (typeof v != "string") {
              let T = { instancePath: o + "/book/body-footer", schemaPath: "#/properties/book/properties/body-footer/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["body-footer"] }, message: "must be string", schema: b.properties.book.properties["body-footer"].type, parentSchema: b.properties.book.properties["body-footer"], data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J["margin-header"] !== void 0) {
            let v = J["margin-header"], E = r, A = r, Z = false, T = null, U = r;
            if (typeof v != "string") {
              let W = { instancePath: o + "/book/margin-header", schemaPath: "#/properties/book/properties/margin-header/oneOf/0/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["margin-header"].oneOf[0] }, message: "must be string", schema: b.properties.book.properties["margin-header"].oneOf[0].type, parentSchema: b.properties.book.properties["margin-header"].oneOf[0], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            let re = r;
            var Ot = U === r;
            Ot && (Z = true, T = 0);
            let Fr = r, z = r;
            if (Array.isArray(v)) {
              let W = true, C = v.length;
              for (let te = 0; te < C; te++) {
                let Br = v[te], X = r;
                if (typeof Br != "string") {
                  let G = { instancePath: o + "/book/margin-header/" + te, schemaPath: "#/properties/book/properties/margin-header/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["margin-header"].oneOf[1].items }, message: "must be string", schema: b.properties.book.properties["margin-header"].oneOf[1].items.type, parentSchema: b.properties.book.properties["margin-header"].oneOf[1].items, data: Br };
                  e === null ? e = [G] : e.push(G), r++;
                }
                let Yr = r;
                var xo = X === r;
                xo || (W = false);
              }
            } else {
              let W = { instancePath: o + "/book/margin-header", schemaPath: "#/properties/book/properties/margin-header/oneOf/1/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties["margin-header"].oneOf[1] }, message: "must be array", schema: b.properties.book.properties["margin-header"].oneOf[1].type, parentSchema: b.properties.book.properties["margin-header"].oneOf[1], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var Ot = Fr === r;
            if (Ot && Z ? (Z = false, T = [T, 1]) : Ot && (Z = true, T = 1), Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let W = { instancePath: o + "/book/margin-header", schemaPath: "#/properties/book/properties/margin-header/oneOf", keyword: "oneOf", params: { passingSchemas: T, schema: b.properties.book.properties["margin-header"] }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties["margin-header"].oneOf, parentSchema: b.properties.book.properties["margin-header"], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var $ = E === r;
          }
          if (J["margin-footer"] !== void 0) {
            let v = J["margin-footer"], E = r, A = r, Z = false, T = null, U = r;
            if (typeof v != "string") {
              let W = { instancePath: o + "/book/margin-footer", schemaPath: "#/properties/book/properties/margin-footer/oneOf/0/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["margin-footer"].oneOf[0] }, message: "must be string", schema: b.properties.book.properties["margin-footer"].oneOf[0].type, parentSchema: b.properties.book.properties["margin-footer"].oneOf[0], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            let re = r;
            var li = U === r;
            li && (Z = true, T = 0);
            let Fr = r, z = r;
            if (Array.isArray(v)) {
              let W = true, C = v.length;
              for (let te = 0; te < C; te++) {
                let Br = v[te], X = r;
                if (typeof Br != "string") {
                  let G = { instancePath: o + "/book/margin-footer/" + te, schemaPath: "#/properties/book/properties/margin-footer/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["margin-footer"].oneOf[1].items }, message: "must be string", schema: b.properties.book.properties["margin-footer"].oneOf[1].items.type, parentSchema: b.properties.book.properties["margin-footer"].oneOf[1].items, data: Br };
                  e === null ? e = [G] : e.push(G), r++;
                }
                let Yr = r;
                var Pt = X === r;
                Pt || (W = false);
              }
            } else {
              let W = { instancePath: o + "/book/margin-footer", schemaPath: "#/properties/book/properties/margin-footer/oneOf/1/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties["margin-footer"].oneOf[1] }, message: "must be array", schema: b.properties.book.properties["margin-footer"].oneOf[1].type, parentSchema: b.properties.book.properties["margin-footer"].oneOf[1], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var li = Fr === r;
            if (li && Z ? (Z = false, T = [T, 1]) : li && (Z = true, T = 1), Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let W = { instancePath: o + "/book/margin-footer", schemaPath: "#/properties/book/properties/margin-footer/oneOf", keyword: "oneOf", params: { passingSchemas: T, schema: b.properties.book.properties["margin-footer"] }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties["margin-footer"].oneOf, parentSchema: b.properties.book.properties["margin-footer"], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var $ = E === r;
          }
          if (J["page-navigation"] !== void 0) {
            let v = J["page-navigation"], E = r;
            if (typeof v != "boolean") {
              let T = { instancePath: o + "/book/page-navigation", schemaPath: "#/properties/book/properties/page-navigation/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties["page-navigation"] }, message: "must be boolean", schema: b.properties.book.properties["page-navigation"].type, parentSchema: b.properties.book.properties["page-navigation"], data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J["page-footer"] !== void 0) {
            let v = J["page-footer"], E = r, A = r, Z = false, T = null, U = r;
            if (typeof v != "string") {
              let W = { instancePath: o + "/book/page-footer", schemaPath: "#/properties/book/properties/page-footer/oneOf/0/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["page-footer"].oneOf[0] }, message: "must be string", schema: b.properties.book.properties["page-footer"].oneOf[0].type, parentSchema: b.properties.book.properties["page-footer"].oneOf[0], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            let re = r;
            var bo = U === r;
            bo && (Z = true, T = 0);
            let Fr = r, z = r;
            if (v && typeof v == "object" && !Array.isArray(v)) {
              let W = r;
              for (let C in v)
                if (!(C === "left" || C === "right" || C === "center" || C === "border" || C === "background" || C === "foreground")) {
                  let te = { instancePath: o + "/book/page-footer", schemaPath: "#/properties/book/properties/page-footer/oneOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: C, schema: b.properties.book.properties["page-footer"].oneOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: b.properties.book.properties["page-footer"].oneOf[1], data: v };
                  e === null ? e = [te] : e.push(te), r++;
                }
              if (v.left !== void 0) {
                let C = v.left, te = r;
                qn(C, { instancePath: o + "/book/page-footer/left", parentData: v, parentDataProperty: "left", rootData: s }) || (e = e === null ? qn.errors : e.concat(qn.errors), r = e.length);
                var sl = te === r;
              }
              if (v.right !== void 0) {
                let C = v.right, te = r;
                qn(C, { instancePath: o + "/book/page-footer/right", parentData: v, parentDataProperty: "right", rootData: s }) || (e = e === null ? qn.errors : e.concat(qn.errors), r = e.length);
                var sl = te === r;
              }
              if (v.center !== void 0) {
                let C = v.center, te = r;
                qn(C, { instancePath: o + "/book/page-footer/center", parentData: v, parentDataProperty: "center", rootData: s }) || (e = e === null ? qn.errors : e.concat(qn.errors), r = e.length);
                var sl = te === r;
              }
              if (v.border !== void 0) {
                let C = v.border, te = r, Br = r, X = false, Yr = null, G = r;
                if (typeof C != "boolean") {
                  let Wa = { instancePath: o + "/book/page-footer/border", schemaPath: "#/properties/book/properties/page-footer/oneOf/1/properties/border/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties["page-footer"].oneOf[1].properties.border.oneOf[0] }, message: "must be boolean", schema: b.properties.book.properties["page-footer"].oneOf[1].properties.border.oneOf[0].type, parentSchema: b.properties.book.properties["page-footer"].oneOf[1].properties.border.oneOf[0], data: C };
                  e === null ? e = [Wa] : e.push(Wa), r++;
                }
                let Ar = r;
                var Ur = G === r;
                Ur && (X = true, Yr = 0);
                let _t = r;
                if (typeof C != "string") {
                  let Wa = { instancePath: o + "/book/page-footer/border", schemaPath: "#/properties/book/properties/page-footer/oneOf/1/properties/border/oneOf/1/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["page-footer"].oneOf[1].properties.border.oneOf[1] }, message: "must be string", schema: b.properties.book.properties["page-footer"].oneOf[1].properties.border.oneOf[1].type, parentSchema: b.properties.book.properties["page-footer"].oneOf[1].properties.border.oneOf[1], data: C };
                  e === null ? e = [Wa] : e.push(Wa), r++;
                }
                let Zr = r;
                var Ur = _t === r;
                if (Ur && X ? (X = false, Yr = [Yr, 1]) : Ur && (X = true, Yr = 1), X)
                  r = Br, e !== null && (Br ? e.length = Br : e = null);
                else {
                  let Wa = { instancePath: o + "/book/page-footer/border", schemaPath: "#/properties/book/properties/page-footer/oneOf/1/properties/border/oneOf", keyword: "oneOf", params: { passingSchemas: Yr, schema: b.properties.book.properties["page-footer"].oneOf[1].properties.border }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties["page-footer"].oneOf[1].properties.border.oneOf, parentSchema: b.properties.book.properties["page-footer"].oneOf[1].properties.border, data: C };
                  e === null ? e = [Wa] : e.push(Wa), r++;
                }
                var sl = te === r;
              }
              if (v.background !== void 0) {
                let C = v.background, te = r;
                if (typeof C != "string") {
                  let Yr = { instancePath: o + "/book/page-footer/background", schemaPath: "#/properties/book/properties/page-footer/oneOf/1/properties/background/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["page-footer"].oneOf[1].properties.background }, message: "must be string", schema: b.properties.book.properties["page-footer"].oneOf[1].properties.background.type, parentSchema: b.properties.book.properties["page-footer"].oneOf[1].properties.background, data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var sl = te === r;
              }
              if (v.foreground !== void 0) {
                let C = v.foreground, te = r;
                if (typeof C != "string") {
                  let Yr = { instancePath: o + "/book/page-footer/foreground", schemaPath: "#/properties/book/properties/page-footer/oneOf/1/properties/foreground/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["page-footer"].oneOf[1].properties.foreground }, message: "must be string", schema: b.properties.book.properties["page-footer"].oneOf[1].properties.foreground.type, parentSchema: b.properties.book.properties["page-footer"].oneOf[1].properties.foreground, data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var sl = te === r;
              }
            } else {
              let W = { instancePath: o + "/book/page-footer", schemaPath: "#/properties/book/properties/page-footer/oneOf/1/type", keyword: "type", params: { type: "object", schema: b.properties.book.properties["page-footer"].oneOf[1] }, message: "must be object", schema: b.properties.book.properties["page-footer"].oneOf[1].type, parentSchema: b.properties.book.properties["page-footer"].oneOf[1], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var bo = Fr === r;
            if (bo && Z ? (Z = false, T = [T, 1]) : bo && (Z = true, T = 1), Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let W = { instancePath: o + "/book/page-footer", schemaPath: "#/properties/book/properties/page-footer/oneOf", keyword: "oneOf", params: { passingSchemas: T, schema: b.properties.book.properties["page-footer"] }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties["page-footer"].oneOf, parentSchema: b.properties.book.properties["page-footer"], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var $ = E === r;
          }
          if (J.image !== void 0) {
            let v = J.image, E = r;
            if (typeof v != "string") {
              let T = { instancePath: o + "/book/image", schemaPath: "#/properties/book/properties/image/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.image }, message: "must be string", schema: b.properties.book.properties.image.type, parentSchema: b.properties.book.properties.image, data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J.comments !== void 0) {
            let v = J.comments, E = r;
            Uo(v, { instancePath: o + "/book/comments", parentData: J, parentDataProperty: "comments", rootData: s }) || (e = e === null ? Uo.errors : e.concat(Uo.errors), r = e.length);
            var $ = E === r;
          }
          if (J["open-graph"] !== void 0) {
            let v = J["open-graph"], E = r, A = r, Z = false, T = null, U = r;
            if (typeof v != "boolean") {
              let W = { instancePath: o + "/book/open-graph", schemaPath: "#/properties/book/properties/open-graph/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties["open-graph"].oneOf[0] }, message: "must be boolean", schema: b.properties.book.properties["open-graph"].oneOf[0].type, parentSchema: b.properties.book.properties["open-graph"].oneOf[0], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            let re = r;
            var Q = U === r;
            Q && (Z = true, T = 0);
            let Fr = r, z = r;
            if (v && typeof v == "object" && !Array.isArray(v)) {
              for (let C in v) {
                let te = r, Br = r;
                if (typeof C == "string") {
                  if (!ig.test(C)) {
                    let X = { instancePath: o + "/book/open-graph", schemaPath: "#/properties/book/properties/open-graph/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(^(?!^(?!(site_name|siteName)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))", schema: b.properties.book.properties["open-graph"].oneOf[1].propertyNames }, message: 'must match pattern "^(?!(^(?!^(?!(site_name|siteName)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))"', schema: "^(?!(^(?!^(?!(site_name|siteName)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))", parentSchema: b.properties.book.properties["open-graph"].oneOf[1].propertyNames, data: C, propertyName: C };
                    e === null ? e = [X] : e.push(X), r++;
                  }
                } else {
                  let X = { instancePath: o + "/book/open-graph", schemaPath: "#/properties/book/properties/open-graph/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["open-graph"].oneOf[1].propertyNames }, message: "must be string", schema: b.properties.book.properties["open-graph"].oneOf[1].propertyNames.type, parentSchema: b.properties.book.properties["open-graph"].oneOf[1].propertyNames, data: C, propertyName: C };
                  e === null ? e = [X] : e.push(X), r++;
                }
                var ba = te === r;
                if (!ba) {
                  let X = { instancePath: o + "/book/open-graph", schemaPath: "#/properties/book/properties/open-graph/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: C, schema: b.properties.book.properties["open-graph"].oneOf[1] }, message: "property name must be valid", schema: b.properties.book.properties["open-graph"].oneOf[1].propertyNames, parentSchema: b.properties.book.properties["open-graph"].oneOf[1], data: v };
                  e === null ? e = [X] : e.push(X), r++;
                }
              }
              let W = r;
              for (let C in v)
                if (!(C === "title" || C === "description" || C === "image" || C === "image-width" || C === "image-height" || C === "locale" || C === "site-name")) {
                  let te = { instancePath: o + "/book/open-graph", schemaPath: "#/properties/book/properties/open-graph/oneOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: C, schema: b.properties.book.properties["open-graph"].oneOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: b.properties.book.properties["open-graph"].oneOf[1], data: v };
                  e === null ? e = [te] : e.push(te), r++;
                }
              if (v.title !== void 0) {
                let C = v.title, te = r;
                if (typeof C != "string") {
                  let X = { instancePath: o + "/book/open-graph/title", schemaPath: "#/properties/book/properties/open-graph/oneOf/1/properties/title/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["open-graph"].oneOf[1].properties.title }, message: "must be string", schema: b.properties.book.properties["open-graph"].oneOf[1].properties.title.type, parentSchema: b.properties.book.properties["open-graph"].oneOf[1].properties.title, data: C };
                  e === null ? e = [X] : e.push(X), r++;
                }
                let Br = r;
                var $r = te === r;
              }
              if (v.description !== void 0) {
                let C = v.description, te = r;
                if (typeof C != "string") {
                  let Yr = { instancePath: o + "/book/open-graph/description", schemaPath: "#/properties/book/properties/open-graph/oneOf/1/properties/description/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["open-graph"].oneOf[1].properties.description }, message: "must be string", schema: b.properties.book.properties["open-graph"].oneOf[1].properties.description.type, parentSchema: b.properties.book.properties["open-graph"].oneOf[1].properties.description, data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var $r = te === r;
              }
              if (v.image !== void 0) {
                let C = v.image, te = r;
                if (typeof C != "string") {
                  let Yr = { instancePath: o + "/book/open-graph/image", schemaPath: "#/properties/book/properties/open-graph/oneOf/1/properties/image/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["open-graph"].oneOf[1].properties.image }, message: "must be string", schema: b.properties.book.properties["open-graph"].oneOf[1].properties.image.type, parentSchema: b.properties.book.properties["open-graph"].oneOf[1].properties.image, data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var $r = te === r;
              }
              if (v["image-width"] !== void 0) {
                let C = v["image-width"], te = r;
                if (!(typeof C == "number" && isFinite(C))) {
                  let Yr = { instancePath: o + "/book/open-graph/image-width", schemaPath: "#/properties/book/properties/open-graph/oneOf/1/properties/image-width/type", keyword: "type", params: { type: "number", schema: b.properties.book.properties["open-graph"].oneOf[1].properties["image-width"] }, message: "must be number", schema: b.properties.book.properties["open-graph"].oneOf[1].properties["image-width"].type, parentSchema: b.properties.book.properties["open-graph"].oneOf[1].properties["image-width"], data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var $r = te === r;
              }
              if (v["image-height"] !== void 0) {
                let C = v["image-height"], te = r;
                if (!(typeof C == "number" && isFinite(C))) {
                  let Yr = { instancePath: o + "/book/open-graph/image-height", schemaPath: "#/properties/book/properties/open-graph/oneOf/1/properties/image-height/type", keyword: "type", params: { type: "number", schema: b.properties.book.properties["open-graph"].oneOf[1].properties["image-height"] }, message: "must be number", schema: b.properties.book.properties["open-graph"].oneOf[1].properties["image-height"].type, parentSchema: b.properties.book.properties["open-graph"].oneOf[1].properties["image-height"], data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var $r = te === r;
              }
              if (v.locale !== void 0) {
                let C = v.locale, te = r;
                if (typeof C != "string") {
                  let Yr = { instancePath: o + "/book/open-graph/locale", schemaPath: "#/properties/book/properties/open-graph/oneOf/1/properties/locale/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["open-graph"].oneOf[1].properties.locale }, message: "must be string", schema: b.properties.book.properties["open-graph"].oneOf[1].properties.locale.type, parentSchema: b.properties.book.properties["open-graph"].oneOf[1].properties.locale, data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var $r = te === r;
              }
              if (v["site-name"] !== void 0) {
                let C = v["site-name"], te = r;
                if (typeof C != "string") {
                  let Yr = { instancePath: o + "/book/open-graph/site-name", schemaPath: "#/properties/book/properties/open-graph/oneOf/1/properties/site-name/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["open-graph"].oneOf[1].properties["site-name"] }, message: "must be string", schema: b.properties.book.properties["open-graph"].oneOf[1].properties["site-name"].type, parentSchema: b.properties.book.properties["open-graph"].oneOf[1].properties["site-name"], data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var $r = te === r;
              }
            } else {
              let W = { instancePath: o + "/book/open-graph", schemaPath: "#/properties/book/properties/open-graph/oneOf/1/type", keyword: "type", params: { type: "object", schema: b.properties.book.properties["open-graph"].oneOf[1] }, message: "must be object", schema: b.properties.book.properties["open-graph"].oneOf[1].type, parentSchema: b.properties.book.properties["open-graph"].oneOf[1], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var Q = Fr === r;
            if (Q && Z ? (Z = false, T = [T, 1]) : Q && (Z = true, T = 1), Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let W = { instancePath: o + "/book/open-graph", schemaPath: "#/properties/book/properties/open-graph/oneOf", keyword: "oneOf", params: { passingSchemas: T, schema: b.properties.book.properties["open-graph"] }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties["open-graph"].oneOf, parentSchema: b.properties.book.properties["open-graph"], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var $ = E === r;
          }
          if (J["twitter-card"] !== void 0) {
            let v = J["twitter-card"], E = r, A = r, Z = false, T = null, U = r;
            if (typeof v != "boolean") {
              let W = { instancePath: o + "/book/twitter-card", schemaPath: "#/properties/book/properties/twitter-card/oneOf/0/type", keyword: "type", params: { type: "boolean", schema: b.properties.book.properties["twitter-card"].oneOf[0] }, message: "must be boolean", schema: b.properties.book.properties["twitter-card"].oneOf[0].type, parentSchema: b.properties.book.properties["twitter-card"].oneOf[0], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            let re = r;
            var kt = U === r;
            kt && (Z = true, T = 0);
            let Fr = r, z = r;
            if (v && typeof v == "object" && !Array.isArray(v)) {
              for (let C in v) {
                let te = r, Br = r;
                if (typeof C == "string") {
                  if (!lg.test(C)) {
                    let X = { instancePath: o + "/book/twitter-card", schemaPath: "#/properties/book/properties/twitter-card/oneOf/1/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(^(?!^(?!(card_style|cardStyle)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))", schema: b.properties.book.properties["twitter-card"].oneOf[1].propertyNames }, message: 'must match pattern "^(?!(^(?!^(?!(card_style|cardStyle)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))"', schema: "^(?!(^(?!^(?!(card_style|cardStyle)))|^(?!^(?!(image_width|imageWidth|image_height|imageHeight)))))", parentSchema: b.properties.book.properties["twitter-card"].oneOf[1].propertyNames, data: C, propertyName: C };
                    e === null ? e = [X] : e.push(X), r++;
                  }
                } else {
                  let X = { instancePath: o + "/book/twitter-card", schemaPath: "#/properties/book/properties/twitter-card/oneOf/1/propertyNames/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["twitter-card"].oneOf[1].propertyNames }, message: "must be string", schema: b.properties.book.properties["twitter-card"].oneOf[1].propertyNames.type, parentSchema: b.properties.book.properties["twitter-card"].oneOf[1].propertyNames, data: C, propertyName: C };
                  e === null ? e = [X] : e.push(X), r++;
                }
                var $t = te === r;
                if (!$t) {
                  let X = { instancePath: o + "/book/twitter-card", schemaPath: "#/properties/book/properties/twitter-card/oneOf/1/propertyNames", keyword: "propertyNames", params: { propertyName: C, schema: b.properties.book.properties["twitter-card"].oneOf[1] }, message: "property name must be valid", schema: b.properties.book.properties["twitter-card"].oneOf[1].propertyNames, parentSchema: b.properties.book.properties["twitter-card"].oneOf[1], data: v };
                  e === null ? e = [X] : e.push(X), r++;
                }
              }
              let W = r;
              for (let C in v)
                if (!(C === "title" || C === "description" || C === "image" || C === "image-width" || C === "image-height" || C === "card-style" || C === "creator" || C === "site")) {
                  let te = { instancePath: o + "/book/twitter-card", schemaPath: "#/properties/book/properties/twitter-card/oneOf/1/additionalProperties", keyword: "additionalProperties", params: { additionalProperty: C, schema: b.properties.book.properties["twitter-card"].oneOf[1] }, message: "must NOT have additional properties", schema: false, parentSchema: b.properties.book.properties["twitter-card"].oneOf[1], data: v };
                  e === null ? e = [te] : e.push(te), r++;
                }
              if (v.title !== void 0) {
                let C = v.title, te = r;
                if (typeof C != "string") {
                  let X = { instancePath: o + "/book/twitter-card/title", schemaPath: "#/properties/book/properties/twitter-card/oneOf/1/properties/title/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["twitter-card"].oneOf[1].properties.title }, message: "must be string", schema: b.properties.book.properties["twitter-card"].oneOf[1].properties.title.type, parentSchema: b.properties.book.properties["twitter-card"].oneOf[1].properties.title, data: C };
                  e === null ? e = [X] : e.push(X), r++;
                }
                let Br = r;
                var Jr = te === r;
              }
              if (v.description !== void 0) {
                let C = v.description, te = r;
                if (typeof C != "string") {
                  let Yr = { instancePath: o + "/book/twitter-card/description", schemaPath: "#/properties/book/properties/twitter-card/oneOf/1/properties/description/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["twitter-card"].oneOf[1].properties.description }, message: "must be string", schema: b.properties.book.properties["twitter-card"].oneOf[1].properties.description.type, parentSchema: b.properties.book.properties["twitter-card"].oneOf[1].properties.description, data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var Jr = te === r;
              }
              if (v.image !== void 0) {
                let C = v.image, te = r;
                if (typeof C != "string") {
                  let Yr = { instancePath: o + "/book/twitter-card/image", schemaPath: "#/properties/book/properties/twitter-card/oneOf/1/properties/image/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["twitter-card"].oneOf[1].properties.image }, message: "must be string", schema: b.properties.book.properties["twitter-card"].oneOf[1].properties.image.type, parentSchema: b.properties.book.properties["twitter-card"].oneOf[1].properties.image, data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var Jr = te === r;
              }
              if (v["image-width"] !== void 0) {
                let C = v["image-width"], te = r;
                if (!(typeof C == "number" && isFinite(C))) {
                  let Yr = { instancePath: o + "/book/twitter-card/image-width", schemaPath: "#/properties/book/properties/twitter-card/oneOf/1/properties/image-width/type", keyword: "type", params: { type: "number", schema: b.properties.book.properties["twitter-card"].oneOf[1].properties["image-width"] }, message: "must be number", schema: b.properties.book.properties["twitter-card"].oneOf[1].properties["image-width"].type, parentSchema: b.properties.book.properties["twitter-card"].oneOf[1].properties["image-width"], data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var Jr = te === r;
              }
              if (v["image-height"] !== void 0) {
                let C = v["image-height"], te = r;
                if (!(typeof C == "number" && isFinite(C))) {
                  let Yr = { instancePath: o + "/book/twitter-card/image-height", schemaPath: "#/properties/book/properties/twitter-card/oneOf/1/properties/image-height/type", keyword: "type", params: { type: "number", schema: b.properties.book.properties["twitter-card"].oneOf[1].properties["image-height"] }, message: "must be number", schema: b.properties.book.properties["twitter-card"].oneOf[1].properties["image-height"].type, parentSchema: b.properties.book.properties["twitter-card"].oneOf[1].properties["image-height"], data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var Jr = te === r;
              }
              if (v["card-style"] !== void 0) {
                let C = v["card-style"], te = r, Br = b.properties.book.properties["twitter-card"].oneOf[1].properties["card-style"].enum;
                if (!(C === "summary" || C === "summary_large_image")) {
                  let Yr = { instancePath: o + "/book/twitter-card/card-style", schemaPath: "#/properties/book/properties/twitter-card/oneOf/1/properties/card-style/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties["twitter-card"].oneOf[1].properties["card-style"].enum, schema: b.properties.book.properties["twitter-card"].oneOf[1].properties["card-style"] }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties["twitter-card"].oneOf[1].properties["card-style"].enum, parentSchema: b.properties.book.properties["twitter-card"].oneOf[1].properties["card-style"], data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                var Jr = te === r;
              }
              if (v.creator !== void 0) {
                let C = v.creator, te = r;
                if (typeof C != "string") {
                  let Yr = { instancePath: o + "/book/twitter-card/creator", schemaPath: "#/properties/book/properties/twitter-card/oneOf/1/properties/creator/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["twitter-card"].oneOf[1].properties.creator }, message: "must be string", schema: b.properties.book.properties["twitter-card"].oneOf[1].properties.creator.type, parentSchema: b.properties.book.properties["twitter-card"].oneOf[1].properties.creator, data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var Jr = te === r;
              }
              if (v.site !== void 0) {
                let C = v.site, te = r;
                if (typeof C != "string") {
                  let Yr = { instancePath: o + "/book/twitter-card/site", schemaPath: "#/properties/book/properties/twitter-card/oneOf/1/properties/site/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["twitter-card"].oneOf[1].properties.site }, message: "must be string", schema: b.properties.book.properties["twitter-card"].oneOf[1].properties.site.type, parentSchema: b.properties.book.properties["twitter-card"].oneOf[1].properties.site, data: C };
                  e === null ? e = [Yr] : e.push(Yr), r++;
                }
                let Br = r;
                var Jr = te === r;
              }
            } else {
              let W = { instancePath: o + "/book/twitter-card", schemaPath: "#/properties/book/properties/twitter-card/oneOf/1/type", keyword: "type", params: { type: "object", schema: b.properties.book.properties["twitter-card"].oneOf[1] }, message: "must be object", schema: b.properties.book.properties["twitter-card"].oneOf[1].type, parentSchema: b.properties.book.properties["twitter-card"].oneOf[1], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var kt = Fr === r;
            if (kt && Z ? (Z = false, T = [T, 1]) : kt && (Z = true, T = 1), Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let W = { instancePath: o + "/book/twitter-card", schemaPath: "#/properties/book/properties/twitter-card/oneOf", keyword: "oneOf", params: { passingSchemas: T, schema: b.properties.book.properties["twitter-card"] }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties["twitter-card"].oneOf, parentSchema: b.properties.book.properties["twitter-card"], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var $ = E === r;
          }
          if (J.subtitle !== void 0) {
            let v = J.subtitle, E = r;
            if (typeof v != "string") {
              let T = { instancePath: o + "/book/subtitle", schemaPath: "#/properties/book/properties/subtitle/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.subtitle }, message: "must be string", schema: b.properties.book.properties.subtitle.type, parentSchema: b.properties.book.properties.subtitle, data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J.author !== void 0) {
            let v = J.author, E = r, A = r, Z = false, T = null, U = r;
            if (typeof v != "string") {
              let W = { instancePath: o + "/book/author", schemaPath: "#/properties/book/properties/author/oneOf/0/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.author.oneOf[0] }, message: "must be string", schema: b.properties.book.properties.author.oneOf[0].type, parentSchema: b.properties.book.properties.author.oneOf[0], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            let re = r;
            var io = U === r;
            io && (Z = true, T = 0);
            let Fr = r, z = r;
            if (Array.isArray(v)) {
              let W = true, C = v.length;
              for (let te = 0; te < C; te++) {
                let Br = v[te], X = r;
                if (typeof Br != "string") {
                  let G = { instancePath: o + "/book/author/" + te, schemaPath: "#/properties/book/properties/author/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.author.oneOf[1].items }, message: "must be string", schema: b.properties.book.properties.author.oneOf[1].items.type, parentSchema: b.properties.book.properties.author.oneOf[1].items, data: Br };
                  e === null ? e = [G] : e.push(G), r++;
                }
                let Yr = r;
                var an = X === r;
                an || (W = false);
              }
            } else {
              let W = { instancePath: o + "/book/author", schemaPath: "#/properties/book/properties/author/oneOf/1/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties.author.oneOf[1] }, message: "must be array", schema: b.properties.book.properties.author.oneOf[1].type, parentSchema: b.properties.book.properties.author.oneOf[1], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var io = Fr === r;
            if (io && Z ? (Z = false, T = [T, 1]) : io && (Z = true, T = 1), Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let W = { instancePath: o + "/book/author", schemaPath: "#/properties/book/properties/author/oneOf", keyword: "oneOf", params: { passingSchemas: T, schema: b.properties.book.properties.author }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties.author.oneOf, parentSchema: b.properties.book.properties.author, data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var $ = E === r;
          }
          if (J.date !== void 0) {
            let v = J.date, E = r;
            if (typeof v != "string") {
              let T = { instancePath: o + "/book/date", schemaPath: "#/properties/book/properties/date/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.date }, message: "must be string", schema: b.properties.book.properties.date.type, parentSchema: b.properties.book.properties.date, data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J.abstract !== void 0) {
            let v = J.abstract, E = r;
            if (typeof v != "string") {
              let T = { instancePath: o + "/book/abstract", schemaPath: "#/properties/book/properties/abstract/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.abstract }, message: "must be string", schema: b.properties.book.properties.abstract.type, parentSchema: b.properties.book.properties.abstract, data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J.chapters !== void 0) {
            let v = J.chapters, E = r;
            bc(v, { instancePath: o + "/book/chapters", parentData: J, parentDataProperty: "chapters", rootData: s }) || (e = e === null ? bc.errors : e.concat(bc.errors), r = e.length);
            var $ = E === r;
          }
          if (J.references !== void 0) {
            let v = J.references, E = r;
            if (typeof v != "string") {
              let T = { instancePath: o + "/book/references", schemaPath: "#/properties/book/properties/references/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties.references }, message: "must be string", schema: b.properties.book.properties.references.type, parentSchema: b.properties.book.properties.references, data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J.appendices !== void 0) {
            let v = J.appendices, E = r, A = r;
            if (Array.isArray(v)) {
              let T = true, U = v.length;
              for (let re = 0; re < U; re++) {
                let Fr = v[re], z = r;
                yc(Fr, { instancePath: o + "/book/appendices/" + re, parentData: v, parentDataProperty: re, rootData: s }) || (e = e === null ? yc.errors : e.concat(yc.errors), r = e.length);
                var w = z === r;
                w || (T = false);
              }
            } else {
              let T = { instancePath: o + "/book/appendices", schemaPath: "#/properties/book/properties/appendices/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties.appendices }, message: "must be array", schema: b.properties.book.properties.appendices.type, parentSchema: b.properties.book.properties.appendices, data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            var $ = E === r;
          }
          if (J["output-file"] !== void 0) {
            let v = J["output-file"], E = r;
            if (typeof v != "string") {
              let T = { instancePath: o + "/book/output-file", schemaPath: "#/properties/book/properties/output-file/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["output-file"] }, message: "must be string", schema: b.properties.book.properties["output-file"].type, parentSchema: b.properties.book.properties["output-file"], data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J["cover-image"] !== void 0) {
            let v = J["cover-image"], E = r;
            if (typeof v != "string") {
              let T = { instancePath: o + "/book/cover-image", schemaPath: "#/properties/book/properties/cover-image/type", keyword: "type", params: { type: "string", schema: b.properties.book.properties["cover-image"] }, message: "must be string", schema: b.properties.book.properties["cover-image"].type, parentSchema: b.properties.book.properties["cover-image"], data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            let A = r;
            var $ = E === r;
          }
          if (J.sharing !== void 0) {
            let v = J.sharing, E = r, A = r, Z = false, T = null, U = r, re = b.properties.book.properties.sharing.oneOf[0].enum;
            if (!(v === "twitter" || v === "facebook" || v === "linkedin")) {
              let W = { instancePath: o + "/book/sharing", schemaPath: "#/properties/book/properties/sharing/oneOf/0/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.sharing.oneOf[0].enum, schema: b.properties.book.properties.sharing.oneOf[0] }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.sharing.oneOf[0].enum, parentSchema: b.properties.book.properties.sharing.oneOf[0], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var Nt = U === r;
            Nt && (Z = true, T = 0);
            let Fr = r, z = r;
            if (Array.isArray(v)) {
              let W = true, C = v.length;
              for (let te = 0; te < C; te++) {
                let Br = v[te], X = r, Yr = b.properties.book.properties.sharing.oneOf[1].items.enum;
                if (!(Br === "twitter" || Br === "facebook" || Br === "linkedin")) {
                  let G = { instancePath: o + "/book/sharing/" + te, schemaPath: "#/properties/book/properties/sharing/oneOf/1/items/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.sharing.oneOf[1].items.enum, schema: b.properties.book.properties.sharing.oneOf[1].items }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.sharing.oneOf[1].items.enum, parentSchema: b.properties.book.properties.sharing.oneOf[1].items, data: Br };
                  e === null ? e = [G] : e.push(G), r++;
                }
                var Y = X === r;
                Y || (W = false);
              }
            } else {
              let W = { instancePath: o + "/book/sharing", schemaPath: "#/properties/book/properties/sharing/oneOf/1/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties.sharing.oneOf[1] }, message: "must be array", schema: b.properties.book.properties.sharing.oneOf[1].type, parentSchema: b.properties.book.properties.sharing.oneOf[1], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var Nt = Fr === r;
            if (Nt && Z ? (Z = false, T = [T, 1]) : Nt && (Z = true, T = 1), Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let W = { instancePath: o + "/book/sharing", schemaPath: "#/properties/book/properties/sharing/oneOf", keyword: "oneOf", params: { passingSchemas: T, schema: b.properties.book.properties.sharing }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties.sharing.oneOf, parentSchema: b.properties.book.properties.sharing, data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var $ = E === r;
          }
          if (J.downloads !== void 0) {
            let v = J.downloads, E = r, A = r, Z = false, T = null, U = r, re = b.properties.book.properties.downloads.oneOf[0].enum;
            if (!(v === "pdf" || v === "epub" || v === "docx")) {
              let W = { instancePath: o + "/book/downloads", schemaPath: "#/properties/book/properties/downloads/oneOf/0/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.downloads.oneOf[0].enum, schema: b.properties.book.properties.downloads.oneOf[0] }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.downloads.oneOf[0].enum, parentSchema: b.properties.book.properties.downloads.oneOf[0], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var N = U === r;
            N && (Z = true, T = 0);
            let Fr = r, z = r;
            if (Array.isArray(v)) {
              let W = true, C = v.length;
              for (let te = 0; te < C; te++) {
                let Br = v[te], X = r, Yr = b.properties.book.properties.downloads.oneOf[1].items.enum;
                if (!(Br === "pdf" || Br === "epub" || Br === "docx")) {
                  let G = { instancePath: o + "/book/downloads/" + te, schemaPath: "#/properties/book/properties/downloads/oneOf/1/items/enum", keyword: "enum", params: { allowedValues: b.properties.book.properties.downloads.oneOf[1].items.enum, schema: b.properties.book.properties.downloads.oneOf[1].items }, message: "must be equal to one of the allowed values", schema: b.properties.book.properties.downloads.oneOf[1].items.enum, parentSchema: b.properties.book.properties.downloads.oneOf[1].items, data: Br };
                  e === null ? e = [G] : e.push(G), r++;
                }
                var ae = X === r;
                ae || (W = false);
              }
            } else {
              let W = { instancePath: o + "/book/downloads", schemaPath: "#/properties/book/properties/downloads/oneOf/1/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties.downloads.oneOf[1] }, message: "must be array", schema: b.properties.book.properties.downloads.oneOf[1].type, parentSchema: b.properties.book.properties.downloads.oneOf[1], data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var N = Fr === r;
            if (N && Z ? (Z = false, T = [T, 1]) : N && (Z = true, T = 1), Z)
              r = A, e !== null && (A ? e.length = A : e = null);
            else {
              let W = { instancePath: o + "/book/downloads", schemaPath: "#/properties/book/properties/downloads/oneOf", keyword: "oneOf", params: { passingSchemas: T, schema: b.properties.book.properties.downloads }, message: "must match exactly one schema in oneOf", schema: b.properties.book.properties.downloads.oneOf, parentSchema: b.properties.book.properties.downloads, data: v };
              e === null ? e = [W] : e.push(W), r++;
            }
            var $ = E === r;
          }
          if (J.tools !== void 0) {
            let v = J.tools, E = r, A = r;
            if (Array.isArray(v)) {
              let T = true, U = v.length;
              for (let re = 0; re < U; re++) {
                let Fr = v[re], z = r;
                wa(Fr, { instancePath: o + "/book/tools/" + re, parentData: v, parentDataProperty: re, rootData: s }) || (e = e === null ? wa.errors : e.concat(wa.errors), r = e.length);
                var nt = z === r;
                nt || (T = false);
              }
            } else {
              let T = { instancePath: o + "/book/tools", schemaPath: "#/properties/book/properties/tools/type", keyword: "type", params: { type: "array", schema: b.properties.book.properties.tools }, message: "must be array", schema: b.properties.book.properties.tools.type, parentSchema: b.properties.book.properties.tools, data: v };
              e === null ? e = [T] : e.push(T), r++;
            }
            var $ = E === r;
          }
        } else {
          let v = { instancePath: o + "/book", schemaPath: "#/properties/book/type", keyword: "type", params: { type: "object", schema: b.properties.book }, message: "must be object", schema: b.properties.book.type, parentSchema: b.properties.book, data: J };
          e === null ? e = [v] : e.push(v), r++;
        }
        var x = ot === r;
      }
    } else {
      let J = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: b }, message: "must be object", schema: b.type, parentSchema: b, data: l };
      e === null ? e = [J] : e.push(J), r++;
    }
    return vm.errors = e, r === 0;
  }
  B["project-config"] = I1;
  var x1 = { allOf: [{ type: "object", description: "be a Quarto YAML front matter object", properties: { execute: { $ref: "front-matter-execute", description: "be a front-matter-execute object" }, format: { $ref: "front-matter-format", description: "be exactly one of: the name of a pandoc-supported output format, 'hugo', all of: an object" } }, patternProperties: {} }, { $ref: "front-matter-execute", description: "be a front-matter-execute object" }, { $ref: "front-matter", description: "be exactly one of: the null value, all of: a Quarto YAML front matter object, an object, a front-matter-execute object" }, { $ref: "project-config-fields", description: "be an object" }], description: "be a project configuration object", $id: "project-config" };
  function I1(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r, c = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      if (l.execute !== void 0) {
        let D = l.execute, $ = r;
        si(D, { instancePath: o + "/execute", parentData: l, parentDataProperty: "execute", rootData: s }) || (e = e === null ? si.errors : e.concat(si.errors), r = e.length);
        var d = $ === r;
      }
      if (l.format !== void 0) {
        let D = l.format, $ = r;
        _u(D, { instancePath: o + "/format", parentData: l, parentDataProperty: "format", rootData: s }) || (e = e === null ? _u.errors : e.concat(_u.errors), r = e.length);
        var d = $ === r;
      }
    } else {
      let D = { instancePath: o, schemaPath: "#/allOf/0/type", keyword: "type", params: { type: "object", schema: x1.allOf[0] }, message: "must be object", schema: x1.allOf[0].type, parentSchema: x1.allOf[0], data: l };
      e === null ? e = [D] : e.push(D), r++;
    }
    var u = _ === r;
    let y = r;
    si(l, { instancePath: o, parentData: S, parentDataProperty: F, rootData: s }) || (e = e === null ? si.errors : e.concat(si.errors), r = e.length);
    var u = y === r;
    let g = r;
    _m(l, { instancePath: o, parentData: S, parentDataProperty: F, rootData: s }) || (e = e === null ? _m.errors : e.concat(_m.errors), r = e.length);
    var u = g === r;
    let x = r;
    vm(l, { instancePath: o, parentData: S, parentDataProperty: F, rootData: s }) || (e = e === null ? vm.errors : e.concat(vm.errors), r = e.length);
    var u = x === r;
    return I1.errors = e, r === 0;
  }
  B["engine-markdown"] = H1;
  var Ql = { type: "object", description: "be an object", properties: { label: { $ref: "quarto-resource-cell-attributes-label", description: "quarto-resource-cell-attributes-label" }, classes: { $ref: "quarto-resource-cell-attributes-classes", description: "quarto-resource-cell-attributes-classes" }, eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-fold": { $ref: "quarto-resource-cell-codeoutput-code-fold", description: "quarto-resource-cell-codeoutput-code-fold" }, "code-summary": { $ref: "quarto-resource-cell-codeoutput-code-summary", description: "quarto-resource-cell-codeoutput-code-summary" }, "code-overflow": { $ref: "quarto-resource-cell-codeoutput-code-overflow", description: "quarto-resource-cell-codeoutput-code-overflow" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "lst-label": { $ref: "quarto-resource-cell-codeoutput-lst-label", description: "quarto-resource-cell-codeoutput-lst-label" }, "lst-cap": { $ref: "quarto-resource-cell-codeoutput-lst-cap", description: "quarto-resource-cell-codeoutput-lst-cap" }, "fig-cap": { $ref: "quarto-resource-cell-figure-fig-cap", description: "quarto-resource-cell-figure-fig-cap" }, "fig-subcap": { $ref: "quarto-resource-cell-figure-fig-subcap", description: "quarto-resource-cell-figure-fig-subcap" }, "fig-link": { $ref: "quarto-resource-cell-figure-fig-link", description: "quarto-resource-cell-figure-fig-link" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "fig-alt": { $ref: "quarto-resource-cell-figure-fig-alt", description: "quarto-resource-cell-figure-fig-alt" }, "fig-env": { $ref: "quarto-resource-cell-figure-fig-env", description: "quarto-resource-cell-figure-fig-env" }, "fig-pos": { $ref: "quarto-resource-cell-figure-fig-pos", description: "quarto-resource-cell-figure-fig-pos" }, "fig-scap": { $ref: "quarto-resource-cell-figure-fig-scap", description: "quarto-resource-cell-figure-fig-scap" }, layout: { $ref: "quarto-resource-cell-layout-layout", description: "quarto-resource-cell-layout-layout" }, "layout-ncol": { $ref: "quarto-resource-cell-layout-layout-ncol", description: "quarto-resource-cell-layout-layout-ncol" }, "layout-nrow": { $ref: "quarto-resource-cell-layout-layout-nrow", description: "quarto-resource-cell-layout-layout-nrow" }, "layout-align": { $ref: "quarto-resource-cell-layout-layout-align", description: "quarto-resource-cell-layout-layout-align" }, "layout-valign": { $ref: "quarto-resource-cell-layout-layout-valign", description: "quarto-resource-cell-layout-layout-valign" }, column: { $ref: "quarto-resource-cell-pagelayout-column", description: "quarto-resource-cell-pagelayout-column" }, "fig-column": { $ref: "quarto-resource-cell-pagelayout-fig-column", description: "quarto-resource-cell-pagelayout-fig-column" }, "tbl-column": { $ref: "quarto-resource-cell-pagelayout-tbl-column", description: "quarto-resource-cell-pagelayout-tbl-column" }, "cap-location": { $ref: "quarto-resource-cell-pagelayout-cap-location", description: "quarto-resource-cell-pagelayout-cap-location" }, "fig-cap-location": { $ref: "quarto-resource-cell-pagelayout-fig-cap-location", description: "quarto-resource-cell-pagelayout-fig-cap-location" }, "tbl-cap-location": { $ref: "quarto-resource-cell-pagelayout-tbl-cap-location", description: "quarto-resource-cell-pagelayout-tbl-cap-location" }, "tbl-cap": { $ref: "quarto-resource-cell-table-tbl-cap", description: "quarto-resource-cell-table-tbl-cap" }, "tbl-subcap": { $ref: "quarto-resource-cell-table-tbl-subcap", description: "quarto-resource-cell-table-tbl-subcap" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, panel: { $ref: "quarto-resource-cell-textoutput-panel", description: "quarto-resource-cell-textoutput-panel" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|lst_label|lstLabel|lst_cap|lstCap|fig_cap|figCap|fig_subcap|figSubcap|fig_link|figLink|fig_align|figAlign|fig_alt|figAlt|fig_env|figEnv|fig_pos|figPos|fig_scap|figScap|layout_ncol|layoutNcol|layout_nrow|layoutNrow|layout_align|layoutAlign|layout_valign|layoutValign|fig_column|figColumn|tbl_column|tblColumn|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_cap|tblCap|tbl_subcap|tblSubcap))" }, $id: "engine-markdown" }, sq = new RegExp("^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|lst_label|lstLabel|lst_cap|lstCap|fig_cap|figCap|fig_subcap|figSubcap|fig_link|figLink|fig_align|figAlign|fig_alt|figAlt|fig_env|figEnv|fig_pos|figPos|fig_scap|figScap|layout_ncol|layoutNcol|layout_nrow|layoutNrow|layout_align|layoutAlign|layout_valign|layoutValign|fig_column|figColumn|tbl_column|tblColumn|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_cap|tblCap|tbl_subcap|tblSubcap))", "u");
  function H1(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let u in l) {
        let y = r, g = r;
        if (typeof u == "string") {
          if (!sq.test(u)) {
            let x = { instancePath: o, schemaPath: "#/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|lst_label|lstLabel|lst_cap|lstCap|fig_cap|figCap|fig_subcap|figSubcap|fig_link|figLink|fig_align|figAlign|fig_alt|figAlt|fig_env|figEnv|fig_pos|figPos|fig_scap|figScap|layout_ncol|layoutNcol|layout_nrow|layoutNrow|layout_align|layoutAlign|layout_valign|layoutValign|fig_column|figColumn|tbl_column|tblColumn|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_cap|tblCap|tbl_subcap|tblSubcap))", schema: Ql.propertyNames }, message: 'must match pattern "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|lst_label|lstLabel|lst_cap|lstCap|fig_cap|figCap|fig_subcap|figSubcap|fig_link|figLink|fig_align|figAlign|fig_alt|figAlt|fig_env|figEnv|fig_pos|figPos|fig_scap|figScap|layout_ncol|layoutNcol|layout_nrow|layoutNrow|layout_align|layoutAlign|layout_valign|layoutValign|fig_column|figColumn|tbl_column|tblColumn|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_cap|tblCap|tbl_subcap|tblSubcap))"', schema: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|lst_label|lstLabel|lst_cap|lstCap|fig_cap|figCap|fig_subcap|figSubcap|fig_link|figLink|fig_align|figAlign|fig_alt|figAlt|fig_env|figEnv|fig_pos|figPos|fig_scap|figScap|layout_ncol|layoutNcol|layout_nrow|layoutNrow|layout_align|layoutAlign|layout_valign|layoutValign|fig_column|figColumn|tbl_column|tblColumn|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_cap|tblCap|tbl_subcap|tblSubcap))", parentSchema: Ql.propertyNames, data: u, propertyName: u };
            e === null ? e = [x] : e.push(x), r++;
          }
        } else {
          let x = { instancePath: o, schemaPath: "#/propertyNames/type", keyword: "type", params: { type: "string", schema: Ql.propertyNames }, message: "must be string", schema: Ql.propertyNames.type, parentSchema: Ql.propertyNames, data: u, propertyName: u };
          e === null ? e = [x] : e.push(x), r++;
        }
        var c = y === r;
        if (!c) {
          let x = { instancePath: o, schemaPath: "#/propertyNames", keyword: "propertyNames", params: { propertyName: u, schema: Ql }, message: "property name must be valid", schema: Ql.propertyNames, parentSchema: Ql, data: l };
          e === null ? e = [x] : e.push(x), r++;
        }
      }
      if (l.label !== void 0) {
        let u = l.label, y = r;
        pl(u, { instancePath: o + "/label", parentData: l, parentDataProperty: "label", rootData: s }) || (e = e === null ? pl.errors : e.concat(pl.errors), r = e.length);
        var d = y === r;
      }
      if (l.classes !== void 0) {
        let u = l.classes, y = r;
        ul(u, { instancePath: o + "/classes", parentData: l, parentDataProperty: "classes", rootData: s }) || (e = e === null ? ul.errors : e.concat(ul.errors), r = e.length);
        var d = y === r;
      }
      if (l.eval !== void 0) {
        let u = l.eval, y = r;
        ue(u, { instancePath: o + "/eval", parentData: l, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
        var d = y === r;
      }
      if (l.echo !== void 0) {
        let u = l.echo, y = r;
        de(u, { instancePath: o + "/echo", parentData: l, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
        var d = y === r;
      }
      if (l["code-fold"] !== void 0) {
        let u = l["code-fold"], y = r;
        Fa(u, { instancePath: o + "/code-fold", parentData: l, parentDataProperty: "code-fold", rootData: s }) || (e = e === null ? Fa.errors : e.concat(Fa.errors), r = e.length);
        var d = y === r;
      }
      if (l["code-summary"] !== void 0) {
        let u = l["code-summary"], y = r;
        Aa(u, { instancePath: o + "/code-summary", parentData: l, parentDataProperty: "code-summary", rootData: s }) || (e = e === null ? Aa.errors : e.concat(Aa.errors), r = e.length);
        var d = y === r;
      }
      if (l["code-overflow"] !== void 0) {
        let u = l["code-overflow"], y = r;
        Ba(u, { instancePath: o + "/code-overflow", parentData: l, parentDataProperty: "code-overflow", rootData: s }) || (e = e === null ? Ba.errors : e.concat(Ba.errors), r = e.length);
        var d = y === r;
      }
      if (l["code-line-numbers"] !== void 0) {
        let u = l["code-line-numbers"], y = r;
        ua(u, { instancePath: o + "/code-line-numbers", parentData: l, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
        var d = y === r;
      }
      if (l["lst-label"] !== void 0) {
        let u = l["lst-label"], y = r;
        fl(u, { instancePath: o + "/lst-label", parentData: l, parentDataProperty: "lst-label", rootData: s }) || (e = e === null ? fl.errors : e.concat(fl.errors), r = e.length);
        var d = y === r;
      }
      if (l["lst-cap"] !== void 0) {
        let u = l["lst-cap"], y = r;
        ml(u, { instancePath: o + "/lst-cap", parentData: l, parentDataProperty: "lst-cap", rootData: s }) || (e = e === null ? ml.errors : e.concat(ml.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-cap"] !== void 0) {
        let u = l["fig-cap"], y = r;
        gl(u, { instancePath: o + "/fig-cap", parentData: l, parentDataProperty: "fig-cap", rootData: s }) || (e = e === null ? gl.errors : e.concat(gl.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-subcap"] !== void 0) {
        let u = l["fig-subcap"], y = r;
        hl(u, { instancePath: o + "/fig-subcap", parentData: l, parentDataProperty: "fig-subcap", rootData: s }) || (e = e === null ? hl.errors : e.concat(hl.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-link"] !== void 0) {
        let u = l["fig-link"], y = r;
        yl(u, { instancePath: o + "/fig-link", parentData: l, parentDataProperty: "fig-link", rootData: s }) || (e = e === null ? yl.errors : e.concat(yl.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-align"] !== void 0) {
        let u = l["fig-align"], y = r;
        la(u, { instancePath: o + "/fig-align", parentData: l, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-alt"] !== void 0) {
        let u = l["fig-alt"], y = r;
        bl(u, { instancePath: o + "/fig-alt", parentData: l, parentDataProperty: "fig-alt", rootData: s }) || (e = e === null ? bl.errors : e.concat(bl.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-env"] !== void 0) {
        let u = l["fig-env"], y = r;
        Zn(u, { instancePath: o + "/fig-env", parentData: l, parentDataProperty: "fig-env", rootData: s }) || (e = e === null ? Zn.errors : e.concat(Zn.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-pos"] !== void 0) {
        let u = l["fig-pos"], y = r;
        eo(u, { instancePath: o + "/fig-pos", parentData: l, parentDataProperty: "fig-pos", rootData: s }) || (e = e === null ? eo.errors : e.concat(eo.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-scap"] !== void 0) {
        let u = l["fig-scap"], y = r;
        _l(u, { instancePath: o + "/fig-scap", parentData: l, parentDataProperty: "fig-scap", rootData: s }) || (e = e === null ? _l.errors : e.concat(_l.errors), r = e.length);
        var d = y === r;
      }
      if (l.layout !== void 0) {
        let u = l.layout, y = r;
        vl(u, { instancePath: o + "/layout", parentData: l, parentDataProperty: "layout", rootData: s }) || (e = e === null ? vl.errors : e.concat(vl.errors), r = e.length);
        var d = y === r;
      }
      if (l["layout-ncol"] !== void 0) {
        let u = l["layout-ncol"], y = r;
        ql(u, { instancePath: o + "/layout-ncol", parentData: l, parentDataProperty: "layout-ncol", rootData: s }) || (e = e === null ? ql.errors : e.concat(ql.errors), r = e.length);
        var d = y === r;
      }
      if (l["layout-nrow"] !== void 0) {
        let u = l["layout-nrow"], y = r;
        Ol(u, { instancePath: o + "/layout-nrow", parentData: l, parentDataProperty: "layout-nrow", rootData: s }) || (e = e === null ? Ol.errors : e.concat(Ol.errors), r = e.length);
        var d = y === r;
      }
      if (l["layout-align"] !== void 0) {
        let u = l["layout-align"], y = r;
        Pl(u, { instancePath: o + "/layout-align", parentData: l, parentDataProperty: "layout-align", rootData: s }) || (e = e === null ? Pl.errors : e.concat(Pl.errors), r = e.length);
        var d = y === r;
      }
      if (l["layout-valign"] !== void 0) {
        let u = l["layout-valign"], y = r;
        xl(u, { instancePath: o + "/layout-valign", parentData: l, parentDataProperty: "layout-valign", rootData: s }) || (e = e === null ? xl.errors : e.concat(xl.errors), r = e.length);
        var d = y === r;
      }
      if (l.column !== void 0) {
        let u = l.column, y = r;
        kl(u, { instancePath: o + "/column", parentData: l, parentDataProperty: "column", rootData: s }) || (e = e === null ? kl.errors : e.concat(kl.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-column"] !== void 0) {
        let u = l["fig-column"], y = r;
        Dl(u, { instancePath: o + "/fig-column", parentData: l, parentDataProperty: "fig-column", rootData: s }) || (e = e === null ? Dl.errors : e.concat(Dl.errors), r = e.length);
        var d = y === r;
      }
      if (l["tbl-column"] !== void 0) {
        let u = l["tbl-column"], y = r;
        wl(u, { instancePath: o + "/tbl-column", parentData: l, parentDataProperty: "tbl-column", rootData: s }) || (e = e === null ? wl.errors : e.concat(wl.errors), r = e.length);
        var d = y === r;
      }
      if (l["cap-location"] !== void 0) {
        let u = l["cap-location"], y = r;
        ja(u, { instancePath: o + "/cap-location", parentData: l, parentDataProperty: "cap-location", rootData: s }) || (e = e === null ? ja.errors : e.concat(ja.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-cap-location"] !== void 0) {
        let u = l["fig-cap-location"], y = r;
        Ea(u, { instancePath: o + "/fig-cap-location", parentData: l, parentDataProperty: "fig-cap-location", rootData: s }) || (e = e === null ? Ea.errors : e.concat(Ea.errors), r = e.length);
        var d = y === r;
      }
      if (l["tbl-cap-location"] !== void 0) {
        let u = l["tbl-cap-location"], y = r;
        Na(u, { instancePath: o + "/tbl-cap-location", parentData: l, parentDataProperty: "tbl-cap-location", rootData: s }) || (e = e === null ? Na.errors : e.concat(Na.errors), r = e.length);
        var d = y === r;
      }
      if (l["tbl-cap"] !== void 0) {
        let u = l["tbl-cap"], y = r;
        $l(u, { instancePath: o + "/tbl-cap", parentData: l, parentDataProperty: "tbl-cap", rootData: s }) || (e = e === null ? $l.errors : e.concat($l.errors), r = e.length);
        var d = y === r;
      }
      if (l["tbl-subcap"] !== void 0) {
        let u = l["tbl-subcap"], y = r;
        Sl(u, { instancePath: o + "/tbl-subcap", parentData: l, parentDataProperty: "tbl-subcap", rootData: s }) || (e = e === null ? Sl.errors : e.concat(Sl.errors), r = e.length);
        var d = y === r;
      }
      if (l.output !== void 0) {
        let u = l.output, y = r;
        fe(u, { instancePath: o + "/output", parentData: l, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
        var d = y === r;
      }
      if (l.warning !== void 0) {
        let u = l.warning, y = r;
        me(u, { instancePath: o + "/warning", parentData: l, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
        var d = y === r;
      }
      if (l.error !== void 0) {
        let u = l.error, y = r;
        ge(u, { instancePath: o + "/error", parentData: l, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
        var d = y === r;
      }
      if (l.include !== void 0) {
        let u = l.include, y = r;
        he(u, { instancePath: o + "/include", parentData: l, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
        var d = y === r;
      }
      if (l.panel !== void 0) {
        let u = l.panel, y = r;
        Fl(u, { instancePath: o + "/panel", parentData: l, parentDataProperty: "panel", rootData: s }) || (e = e === null ? Fl.errors : e.concat(Fl.errors), r = e.length);
        var d = y === r;
      }
    } else {
      let u = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: Ql }, message: "must be object", schema: Ql.type, parentSchema: Ql, data: l };
      e === null ? e = [u] : e.push(u), r++;
    }
    return H1.errors = e, r === 0;
  }
  B["engine-knitr"] = L1;
  var Gl = { type: "object", description: "be an object", properties: { label: { $ref: "quarto-resource-cell-attributes-label", description: "quarto-resource-cell-attributes-label" }, classes: { $ref: "quarto-resource-cell-attributes-classes", description: "quarto-resource-cell-attributes-classes" }, cache: { $ref: "quarto-resource-cell-cache-cache", description: "quarto-resource-cell-cache-cache" }, "cache-path": { $ref: "quarto-resource-cell-cache-cache-path", description: "quarto-resource-cell-cache-cache-path" }, "cache-vars": { $ref: "quarto-resource-cell-cache-cache-vars", description: "quarto-resource-cell-cache-cache-vars" }, "cache-globals": { $ref: "quarto-resource-cell-cache-cache-globals", description: "quarto-resource-cell-cache-cache-globals" }, "cache-lazy": { $ref: "quarto-resource-cell-cache-cache-lazy", description: "quarto-resource-cell-cache-cache-lazy" }, "cache-rebuild": { $ref: "quarto-resource-cell-cache-cache-rebuild", description: "quarto-resource-cell-cache-cache-rebuild" }, "cache-comments": { $ref: "quarto-resource-cell-cache-cache-comments", description: "quarto-resource-cell-cache-cache-comments" }, dependson: { $ref: "quarto-resource-cell-cache-dependson", description: "quarto-resource-cell-cache-dependson" }, autodep: { $ref: "quarto-resource-cell-cache-autodep", description: "quarto-resource-cell-cache-autodep" }, eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-fold": { $ref: "quarto-resource-cell-codeoutput-code-fold", description: "quarto-resource-cell-codeoutput-code-fold" }, "code-summary": { $ref: "quarto-resource-cell-codeoutput-code-summary", description: "quarto-resource-cell-codeoutput-code-summary" }, "code-overflow": { $ref: "quarto-resource-cell-codeoutput-code-overflow", description: "quarto-resource-cell-codeoutput-code-overflow" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "lst-label": { $ref: "quarto-resource-cell-codeoutput-lst-label", description: "quarto-resource-cell-codeoutput-lst-label" }, "lst-cap": { $ref: "quarto-resource-cell-codeoutput-lst-cap", description: "quarto-resource-cell-codeoutput-lst-cap" }, tidy: { $ref: "quarto-resource-cell-codeoutput-tidy", description: "quarto-resource-cell-codeoutput-tidy" }, "tidy-opts": { $ref: "quarto-resource-cell-codeoutput-tidy-opts", description: "quarto-resource-cell-codeoutput-tidy-opts" }, collapse: { $ref: "quarto-resource-cell-codeoutput-collapse", description: "quarto-resource-cell-codeoutput-collapse" }, prompt: { $ref: "quarto-resource-cell-codeoutput-prompt", description: "quarto-resource-cell-codeoutput-prompt" }, highlight: { $ref: "quarto-resource-cell-codeoutput-highlight", description: "quarto-resource-cell-codeoutput-highlight" }, "class-source": { $ref: "quarto-resource-cell-codeoutput-class-source", description: "quarto-resource-cell-codeoutput-class-source" }, "attr-source": { $ref: "quarto-resource-cell-codeoutput-attr-source", description: "quarto-resource-cell-codeoutput-attr-source" }, "fig-width": { $ref: "quarto-resource-cell-figure-fig-width", description: "quarto-resource-cell-figure-fig-width" }, "fig-height": { $ref: "quarto-resource-cell-figure-fig-height", description: "quarto-resource-cell-figure-fig-height" }, "fig-cap": { $ref: "quarto-resource-cell-figure-fig-cap", description: "quarto-resource-cell-figure-fig-cap" }, "fig-subcap": { $ref: "quarto-resource-cell-figure-fig-subcap", description: "quarto-resource-cell-figure-fig-subcap" }, "fig-link": { $ref: "quarto-resource-cell-figure-fig-link", description: "quarto-resource-cell-figure-fig-link" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "fig-alt": { $ref: "quarto-resource-cell-figure-fig-alt", description: "quarto-resource-cell-figure-fig-alt" }, "fig-env": { $ref: "quarto-resource-cell-figure-fig-env", description: "quarto-resource-cell-figure-fig-env" }, "fig-pos": { $ref: "quarto-resource-cell-figure-fig-pos", description: "quarto-resource-cell-figure-fig-pos" }, "fig-scap": { $ref: "quarto-resource-cell-figure-fig-scap", description: "quarto-resource-cell-figure-fig-scap" }, "fig-format": { $ref: "quarto-resource-cell-figure-fig-format", description: "quarto-resource-cell-figure-fig-format" }, "fig-dpi": { $ref: "quarto-resource-cell-figure-fig-dpi", description: "quarto-resource-cell-figure-fig-dpi" }, "out-width": { $ref: "quarto-resource-cell-figure-out-width", description: "quarto-resource-cell-figure-out-width" }, "out-height": { $ref: "quarto-resource-cell-figure-out-height", description: "quarto-resource-cell-figure-out-height" }, "fig-keep": { $ref: "quarto-resource-cell-figure-fig-keep", description: "quarto-resource-cell-figure-fig-keep" }, "fig-show": { $ref: "quarto-resource-cell-figure-fig-show", description: "quarto-resource-cell-figure-fig-show" }, "out-extra": { $ref: "quarto-resource-cell-figure-out-extra", description: "quarto-resource-cell-figure-out-extra" }, external: { $ref: "quarto-resource-cell-figure-external", description: "quarto-resource-cell-figure-external" }, sanitize: { $ref: "quarto-resource-cell-figure-sanitize", description: "quarto-resource-cell-figure-sanitize" }, interval: { $ref: "quarto-resource-cell-figure-interval", description: "quarto-resource-cell-figure-interval" }, aniopts: { $ref: "quarto-resource-cell-figure-aniopts", description: "quarto-resource-cell-figure-aniopts" }, "animation-hook": { $ref: "quarto-resource-cell-figure-animation-hook", description: "quarto-resource-cell-figure-animation-hook" }, child: { $ref: "quarto-resource-cell-include-child", description: "quarto-resource-cell-include-child" }, file: { $ref: "quarto-resource-cell-include-file", description: "quarto-resource-cell-include-file" }, code: { $ref: "quarto-resource-cell-include-code", description: "quarto-resource-cell-include-code" }, purl: { $ref: "quarto-resource-cell-include-purl", description: "quarto-resource-cell-include-purl" }, layout: { $ref: "quarto-resource-cell-layout-layout", description: "quarto-resource-cell-layout-layout" }, "layout-ncol": { $ref: "quarto-resource-cell-layout-layout-ncol", description: "quarto-resource-cell-layout-layout-ncol" }, "layout-nrow": { $ref: "quarto-resource-cell-layout-layout-nrow", description: "quarto-resource-cell-layout-layout-nrow" }, "layout-align": { $ref: "quarto-resource-cell-layout-layout-align", description: "quarto-resource-cell-layout-layout-align" }, "layout-valign": { $ref: "quarto-resource-cell-layout-layout-valign", description: "quarto-resource-cell-layout-layout-valign" }, column: { $ref: "quarto-resource-cell-pagelayout-column", description: "quarto-resource-cell-pagelayout-column" }, "fig-column": { $ref: "quarto-resource-cell-pagelayout-fig-column", description: "quarto-resource-cell-pagelayout-fig-column" }, "tbl-column": { $ref: "quarto-resource-cell-pagelayout-tbl-column", description: "quarto-resource-cell-pagelayout-tbl-column" }, "cap-location": { $ref: "quarto-resource-cell-pagelayout-cap-location", description: "quarto-resource-cell-pagelayout-cap-location" }, "fig-cap-location": { $ref: "quarto-resource-cell-pagelayout-fig-cap-location", description: "quarto-resource-cell-pagelayout-fig-cap-location" }, "tbl-cap-location": { $ref: "quarto-resource-cell-pagelayout-tbl-cap-location", description: "quarto-resource-cell-pagelayout-tbl-cap-location" }, "tbl-cap": { $ref: "quarto-resource-cell-table-tbl-cap", description: "quarto-resource-cell-table-tbl-cap" }, "tbl-subcap": { $ref: "quarto-resource-cell-table-tbl-subcap", description: "quarto-resource-cell-table-tbl-subcap" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, panel: { $ref: "quarto-resource-cell-textoutput-panel", description: "quarto-resource-cell-textoutput-panel" }, message: { $ref: "quarto-resource-cell-textoutput-message", description: "quarto-resource-cell-textoutput-message" }, results: { $ref: "quarto-resource-cell-textoutput-results", description: "quarto-resource-cell-textoutput-results" }, comment: { $ref: "quarto-resource-cell-textoutput-comment", description: "quarto-resource-cell-textoutput-comment" }, "class-output": { $ref: "quarto-resource-cell-textoutput-class-output", description: "quarto-resource-cell-textoutput-class-output" }, "attr-output": { $ref: "quarto-resource-cell-textoutput-attr-output", description: "quarto-resource-cell-textoutput-attr-output" }, "class-warning": { $ref: "quarto-resource-cell-textoutput-class-warning", description: "quarto-resource-cell-textoutput-class-warning" }, "attr-warning": { $ref: "quarto-resource-cell-textoutput-attr-warning", description: "quarto-resource-cell-textoutput-attr-warning" }, "class-message": { $ref: "quarto-resource-cell-textoutput-class-message", description: "quarto-resource-cell-textoutput-class-message" }, "attr-message": { $ref: "quarto-resource-cell-textoutput-attr-message", description: "quarto-resource-cell-textoutput-attr-message" }, "class-error": { $ref: "quarto-resource-cell-textoutput-class-error", description: "quarto-resource-cell-textoutput-class-error" }, "attr-error": { $ref: "quarto-resource-cell-textoutput-attr-error", description: "quarto-resource-cell-textoutput-attr-error" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(cache_path|cachePath|cache_vars|cacheVars|cache_globals|cacheGlobals|cache_lazy|cacheLazy|cache_rebuild|cacheRebuild|cache_comments|cacheComments|code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|lst_label|lstLabel|lst_cap|lstCap|tidy_opts|tidyOpts|class_source|classSource|attr_source|attrSource|fig_width|figWidth|fig_height|figHeight|fig_cap|figCap|fig_subcap|figSubcap|fig_link|figLink|fig_align|figAlign|fig_alt|figAlt|fig_env|figEnv|fig_pos|figPos|fig_scap|figScap|fig_format|figFormat|fig_dpi|figDpi|out_width|outWidth|out_height|outHeight|fig_keep|figKeep|fig_show|figShow|out_extra|outExtra|animation_hook|animationHook|layout_ncol|layoutNcol|layout_nrow|layoutNrow|layout_align|layoutAlign|layout_valign|layoutValign|fig_column|figColumn|tbl_column|tblColumn|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_cap|tblCap|tbl_subcap|tblSubcap|tbl_colwidths|tblColwidths|class_output|classOutput|attr_output|attrOutput|class_warning|classWarning|attr_warning|attrWarning|class_message|classMessage|attr_message|attrMessage|class_error|classError|attr_error|attrError))" }, $id: "engine-knitr" }, iq = new RegExp("^(?!(cache_path|cachePath|cache_vars|cacheVars|cache_globals|cacheGlobals|cache_lazy|cacheLazy|cache_rebuild|cacheRebuild|cache_comments|cacheComments|code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|lst_label|lstLabel|lst_cap|lstCap|tidy_opts|tidyOpts|class_source|classSource|attr_source|attrSource|fig_width|figWidth|fig_height|figHeight|fig_cap|figCap|fig_subcap|figSubcap|fig_link|figLink|fig_align|figAlign|fig_alt|figAlt|fig_env|figEnv|fig_pos|figPos|fig_scap|figScap|fig_format|figFormat|fig_dpi|figDpi|out_width|outWidth|out_height|outHeight|fig_keep|figKeep|fig_show|figShow|out_extra|outExtra|animation_hook|animationHook|layout_ncol|layoutNcol|layout_nrow|layoutNrow|layout_align|layoutAlign|layout_valign|layoutValign|fig_column|figColumn|tbl_column|tblColumn|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_cap|tblCap|tbl_subcap|tblSubcap|tbl_colwidths|tblColwidths|class_output|classOutput|attr_output|attrOutput|class_warning|classWarning|attr_warning|attrWarning|class_message|classMessage|attr_message|attrMessage|class_error|classError|attr_error|attrError))", "u");
  function L1(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let u in l) {
        let y = r, g = r;
        if (typeof u == "string") {
          if (!iq.test(u)) {
            let x = { instancePath: o, schemaPath: "#/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(cache_path|cachePath|cache_vars|cacheVars|cache_globals|cacheGlobals|cache_lazy|cacheLazy|cache_rebuild|cacheRebuild|cache_comments|cacheComments|code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|lst_label|lstLabel|lst_cap|lstCap|tidy_opts|tidyOpts|class_source|classSource|attr_source|attrSource|fig_width|figWidth|fig_height|figHeight|fig_cap|figCap|fig_subcap|figSubcap|fig_link|figLink|fig_align|figAlign|fig_alt|figAlt|fig_env|figEnv|fig_pos|figPos|fig_scap|figScap|fig_format|figFormat|fig_dpi|figDpi|out_width|outWidth|out_height|outHeight|fig_keep|figKeep|fig_show|figShow|out_extra|outExtra|animation_hook|animationHook|layout_ncol|layoutNcol|layout_nrow|layoutNrow|layout_align|layoutAlign|layout_valign|layoutValign|fig_column|figColumn|tbl_column|tblColumn|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_cap|tblCap|tbl_subcap|tblSubcap|tbl_colwidths|tblColwidths|class_output|classOutput|attr_output|attrOutput|class_warning|classWarning|attr_warning|attrWarning|class_message|classMessage|attr_message|attrMessage|class_error|classError|attr_error|attrError))", schema: Gl.propertyNames }, message: 'must match pattern "^(?!(cache_path|cachePath|cache_vars|cacheVars|cache_globals|cacheGlobals|cache_lazy|cacheLazy|cache_rebuild|cacheRebuild|cache_comments|cacheComments|code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|lst_label|lstLabel|lst_cap|lstCap|tidy_opts|tidyOpts|class_source|classSource|attr_source|attrSource|fig_width|figWidth|fig_height|figHeight|fig_cap|figCap|fig_subcap|figSubcap|fig_link|figLink|fig_align|figAlign|fig_alt|figAlt|fig_env|figEnv|fig_pos|figPos|fig_scap|figScap|fig_format|figFormat|fig_dpi|figDpi|out_width|outWidth|out_height|outHeight|fig_keep|figKeep|fig_show|figShow|out_extra|outExtra|animation_hook|animationHook|layout_ncol|layoutNcol|layout_nrow|layoutNrow|layout_align|layoutAlign|layout_valign|layoutValign|fig_column|figColumn|tbl_column|tblColumn|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_cap|tblCap|tbl_subcap|tblSubcap|tbl_colwidths|tblColwidths|class_output|classOutput|attr_output|attrOutput|class_warning|classWarning|attr_warning|attrWarning|class_message|classMessage|attr_message|attrMessage|class_error|classError|attr_error|attrError))"', schema: "^(?!(cache_path|cachePath|cache_vars|cacheVars|cache_globals|cacheGlobals|cache_lazy|cacheLazy|cache_rebuild|cacheRebuild|cache_comments|cacheComments|code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|lst_label|lstLabel|lst_cap|lstCap|tidy_opts|tidyOpts|class_source|classSource|attr_source|attrSource|fig_width|figWidth|fig_height|figHeight|fig_cap|figCap|fig_subcap|figSubcap|fig_link|figLink|fig_align|figAlign|fig_alt|figAlt|fig_env|figEnv|fig_pos|figPos|fig_scap|figScap|fig_format|figFormat|fig_dpi|figDpi|out_width|outWidth|out_height|outHeight|fig_keep|figKeep|fig_show|figShow|out_extra|outExtra|animation_hook|animationHook|layout_ncol|layoutNcol|layout_nrow|layoutNrow|layout_align|layoutAlign|layout_valign|layoutValign|fig_column|figColumn|tbl_column|tblColumn|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_cap|tblCap|tbl_subcap|tblSubcap|tbl_colwidths|tblColwidths|class_output|classOutput|attr_output|attrOutput|class_warning|classWarning|attr_warning|attrWarning|class_message|classMessage|attr_message|attrMessage|class_error|classError|attr_error|attrError))", parentSchema: Gl.propertyNames, data: u, propertyName: u };
            e === null ? e = [x] : e.push(x), r++;
          }
        } else {
          let x = { instancePath: o, schemaPath: "#/propertyNames/type", keyword: "type", params: { type: "string", schema: Gl.propertyNames }, message: "must be string", schema: Gl.propertyNames.type, parentSchema: Gl.propertyNames, data: u, propertyName: u };
          e === null ? e = [x] : e.push(x), r++;
        }
        var c = y === r;
        if (!c) {
          let x = { instancePath: o, schemaPath: "#/propertyNames", keyword: "propertyNames", params: { propertyName: u, schema: Gl }, message: "property name must be valid", schema: Gl.propertyNames, parentSchema: Gl, data: l };
          e === null ? e = [x] : e.push(x), r++;
        }
      }
      if (l.label !== void 0) {
        let u = l.label, y = r;
        pl(u, { instancePath: o + "/label", parentData: l, parentDataProperty: "label", rootData: s }) || (e = e === null ? pl.errors : e.concat(pl.errors), r = e.length);
        var d = y === r;
      }
      if (l.classes !== void 0) {
        let u = l.classes, y = r;
        ul(u, { instancePath: o + "/classes", parentData: l, parentDataProperty: "classes", rootData: s }) || (e = e === null ? ul.errors : e.concat(ul.errors), r = e.length);
        var d = y === r;
      }
      if (l.cache !== void 0) {
        let u = l.cache, y = r;
        zd(u, { instancePath: o + "/cache", parentData: l, parentDataProperty: "cache", rootData: s }) || (e = e === null ? zd.errors : e.concat(zd.errors), r = e.length);
        var d = y === r;
      }
      if (l["cache-path"] !== void 0) {
        let u = l["cache-path"], y = r;
        Wd(u, { instancePath: o + "/cache-path", parentData: l, parentDataProperty: "cache-path", rootData: s }) || (e = e === null ? Wd.errors : e.concat(Wd.errors), r = e.length);
        var d = y === r;
      }
      if (l["cache-vars"] !== void 0) {
        let u = l["cache-vars"], y = r;
        Rd(u, { instancePath: o + "/cache-vars", parentData: l, parentDataProperty: "cache-vars", rootData: s }) || (e = e === null ? Rd.errors : e.concat(Rd.errors), r = e.length);
        var d = y === r;
      }
      if (l["cache-globals"] !== void 0) {
        let u = l["cache-globals"], y = r;
        Vd(u, { instancePath: o + "/cache-globals", parentData: l, parentDataProperty: "cache-globals", rootData: s }) || (e = e === null ? Vd.errors : e.concat(Vd.errors), r = e.length);
        var d = y === r;
      }
      if (l["cache-lazy"] !== void 0) {
        let u = l["cache-lazy"], y = r;
        Ud(u, { instancePath: o + "/cache-lazy", parentData: l, parentDataProperty: "cache-lazy", rootData: s }) || (e = e === null ? Ud.errors : e.concat(Ud.errors), r = e.length);
        var d = y === r;
      }
      if (l["cache-rebuild"] !== void 0) {
        let u = l["cache-rebuild"], y = r;
        Kd(u, { instancePath: o + "/cache-rebuild", parentData: l, parentDataProperty: "cache-rebuild", rootData: s }) || (e = e === null ? Kd.errors : e.concat(Kd.errors), r = e.length);
        var d = y === r;
      }
      if (l["cache-comments"] !== void 0) {
        let u = l["cache-comments"], y = r;
        Jd(u, { instancePath: o + "/cache-comments", parentData: l, parentDataProperty: "cache-comments", rootData: s }) || (e = e === null ? Jd.errors : e.concat(Jd.errors), r = e.length);
        var d = y === r;
      }
      if (l.dependson !== void 0) {
        let u = l.dependson, y = r;
        Qd(u, { instancePath: o + "/dependson", parentData: l, parentDataProperty: "dependson", rootData: s }) || (e = e === null ? Qd.errors : e.concat(Qd.errors), r = e.length);
        var d = y === r;
      }
      if (l.autodep !== void 0) {
        let u = l.autodep, y = r;
        Gd(u, { instancePath: o + "/autodep", parentData: l, parentDataProperty: "autodep", rootData: s }) || (e = e === null ? Gd.errors : e.concat(Gd.errors), r = e.length);
        var d = y === r;
      }
      if (l.eval !== void 0) {
        let u = l.eval, y = r;
        ue(u, { instancePath: o + "/eval", parentData: l, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
        var d = y === r;
      }
      if (l.echo !== void 0) {
        let u = l.echo, y = r;
        de(u, { instancePath: o + "/echo", parentData: l, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
        var d = y === r;
      }
      if (l["code-fold"] !== void 0) {
        let u = l["code-fold"], y = r;
        Fa(u, { instancePath: o + "/code-fold", parentData: l, parentDataProperty: "code-fold", rootData: s }) || (e = e === null ? Fa.errors : e.concat(Fa.errors), r = e.length);
        var d = y === r;
      }
      if (l["code-summary"] !== void 0) {
        let u = l["code-summary"], y = r;
        Aa(u, { instancePath: o + "/code-summary", parentData: l, parentDataProperty: "code-summary", rootData: s }) || (e = e === null ? Aa.errors : e.concat(Aa.errors), r = e.length);
        var d = y === r;
      }
      if (l["code-overflow"] !== void 0) {
        let u = l["code-overflow"], y = r;
        Ba(u, { instancePath: o + "/code-overflow", parentData: l, parentDataProperty: "code-overflow", rootData: s }) || (e = e === null ? Ba.errors : e.concat(Ba.errors), r = e.length);
        var d = y === r;
      }
      if (l["code-line-numbers"] !== void 0) {
        let u = l["code-line-numbers"], y = r;
        ua(u, { instancePath: o + "/code-line-numbers", parentData: l, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
        var d = y === r;
      }
      if (l["lst-label"] !== void 0) {
        let u = l["lst-label"], y = r;
        fl(u, { instancePath: o + "/lst-label", parentData: l, parentDataProperty: "lst-label", rootData: s }) || (e = e === null ? fl.errors : e.concat(fl.errors), r = e.length);
        var d = y === r;
      }
      if (l["lst-cap"] !== void 0) {
        let u = l["lst-cap"], y = r;
        ml(u, { instancePath: o + "/lst-cap", parentData: l, parentDataProperty: "lst-cap", rootData: s }) || (e = e === null ? ml.errors : e.concat(ml.errors), r = e.length);
        var d = y === r;
      }
      if (l.tidy !== void 0) {
        let u = l.tidy, y = r;
        Xd(u, { instancePath: o + "/tidy", parentData: l, parentDataProperty: "tidy", rootData: s }) || (e = e === null ? Xd.errors : e.concat(Xd.errors), r = e.length);
        var d = y === r;
      }
      if (l["tidy-opts"] !== void 0) {
        let u = l["tidy-opts"], y = r;
        Zd(u, { instancePath: o + "/tidy-opts", parentData: l, parentDataProperty: "tidy-opts", rootData: s }) || (e = e === null ? Zd.errors : e.concat(Zd.errors), r = e.length);
        var d = y === r;
      }
      if (l.collapse !== void 0) {
        let u = l.collapse, y = r;
        ef(u, { instancePath: o + "/collapse", parentData: l, parentDataProperty: "collapse", rootData: s }) || (e = e === null ? ef.errors : e.concat(ef.errors), r = e.length);
        var d = y === r;
      }
      if (l.prompt !== void 0) {
        let u = l.prompt, y = r;
        rf(u, { instancePath: o + "/prompt", parentData: l, parentDataProperty: "prompt", rootData: s }) || (e = e === null ? rf.errors : e.concat(rf.errors), r = e.length);
        var d = y === r;
      }
      if (l.highlight !== void 0) {
        let u = l.highlight, y = r;
        tf(u, { instancePath: o + "/highlight", parentData: l, parentDataProperty: "highlight", rootData: s }) || (e = e === null ? tf.errors : e.concat(tf.errors), r = e.length);
        var d = y === r;
      }
      if (l["class-source"] !== void 0) {
        let u = l["class-source"], y = r;
        af(u, { instancePath: o + "/class-source", parentData: l, parentDataProperty: "class-source", rootData: s }) || (e = e === null ? af.errors : e.concat(af.errors), r = e.length);
        var d = y === r;
      }
      if (l["attr-source"] !== void 0) {
        let u = l["attr-source"], y = r;
        nf(u, { instancePath: o + "/attr-source", parentData: l, parentDataProperty: "attr-source", rootData: s }) || (e = e === null ? nf.errors : e.concat(nf.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-width"] !== void 0) {
        let u = l["fig-width"], y = r;
        of(u, { instancePath: o + "/fig-width", parentData: l, parentDataProperty: "fig-width", rootData: s }) || (e = e === null ? of.errors : e.concat(of.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-height"] !== void 0) {
        let u = l["fig-height"], y = r;
        sf(u, { instancePath: o + "/fig-height", parentData: l, parentDataProperty: "fig-height", rootData: s }) || (e = e === null ? sf.errors : e.concat(sf.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-cap"] !== void 0) {
        let u = l["fig-cap"], y = r;
        gl(u, { instancePath: o + "/fig-cap", parentData: l, parentDataProperty: "fig-cap", rootData: s }) || (e = e === null ? gl.errors : e.concat(gl.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-subcap"] !== void 0) {
        let u = l["fig-subcap"], y = r;
        hl(u, { instancePath: o + "/fig-subcap", parentData: l, parentDataProperty: "fig-subcap", rootData: s }) || (e = e === null ? hl.errors : e.concat(hl.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-link"] !== void 0) {
        let u = l["fig-link"], y = r;
        yl(u, { instancePath: o + "/fig-link", parentData: l, parentDataProperty: "fig-link", rootData: s }) || (e = e === null ? yl.errors : e.concat(yl.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-align"] !== void 0) {
        let u = l["fig-align"], y = r;
        la(u, { instancePath: o + "/fig-align", parentData: l, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-alt"] !== void 0) {
        let u = l["fig-alt"], y = r;
        bl(u, { instancePath: o + "/fig-alt", parentData: l, parentDataProperty: "fig-alt", rootData: s }) || (e = e === null ? bl.errors : e.concat(bl.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-env"] !== void 0) {
        let u = l["fig-env"], y = r;
        Zn(u, { instancePath: o + "/fig-env", parentData: l, parentDataProperty: "fig-env", rootData: s }) || (e = e === null ? Zn.errors : e.concat(Zn.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-pos"] !== void 0) {
        let u = l["fig-pos"], y = r;
        eo(u, { instancePath: o + "/fig-pos", parentData: l, parentDataProperty: "fig-pos", rootData: s }) || (e = e === null ? eo.errors : e.concat(eo.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-scap"] !== void 0) {
        let u = l["fig-scap"], y = r;
        _l(u, { instancePath: o + "/fig-scap", parentData: l, parentDataProperty: "fig-scap", rootData: s }) || (e = e === null ? _l.errors : e.concat(_l.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-format"] !== void 0) {
        let u = l["fig-format"], y = r;
        cf(u, { instancePath: o + "/fig-format", parentData: l, parentDataProperty: "fig-format", rootData: s }) || (e = e === null ? cf.errors : e.concat(cf.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-dpi"] !== void 0) {
        let u = l["fig-dpi"], y = r;
        pf(u, { instancePath: o + "/fig-dpi", parentData: l, parentDataProperty: "fig-dpi", rootData: s }) || (e = e === null ? pf.errors : e.concat(pf.errors), r = e.length);
        var d = y === r;
      }
      if (l["out-width"] !== void 0) {
        let u = l["out-width"], y = r;
        uf(u, { instancePath: o + "/out-width", parentData: l, parentDataProperty: "out-width", rootData: s }) || (e = e === null ? uf.errors : e.concat(uf.errors), r = e.length);
        var d = y === r;
      }
      if (l["out-height"] !== void 0) {
        let u = l["out-height"], y = r;
        df(u, { instancePath: o + "/out-height", parentData: l, parentDataProperty: "out-height", rootData: s }) || (e = e === null ? df.errors : e.concat(df.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-keep"] !== void 0) {
        let u = l["fig-keep"], y = r;
        ff(u, { instancePath: o + "/fig-keep", parentData: l, parentDataProperty: "fig-keep", rootData: s }) || (e = e === null ? ff.errors : e.concat(ff.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-show"] !== void 0) {
        let u = l["fig-show"], y = r;
        gf(u, { instancePath: o + "/fig-show", parentData: l, parentDataProperty: "fig-show", rootData: s }) || (e = e === null ? gf.errors : e.concat(gf.errors), r = e.length);
        var d = y === r;
      }
      if (l["out-extra"] !== void 0) {
        let u = l["out-extra"], y = r;
        hf(u, { instancePath: o + "/out-extra", parentData: l, parentDataProperty: "out-extra", rootData: s }) || (e = e === null ? hf.errors : e.concat(hf.errors), r = e.length);
        var d = y === r;
      }
      if (l.external !== void 0) {
        let u = l.external, y = r;
        yf(u, { instancePath: o + "/external", parentData: l, parentDataProperty: "external", rootData: s }) || (e = e === null ? yf.errors : e.concat(yf.errors), r = e.length);
        var d = y === r;
      }
      if (l.sanitize !== void 0) {
        let u = l.sanitize, y = r;
        bf(u, { instancePath: o + "/sanitize", parentData: l, parentDataProperty: "sanitize", rootData: s }) || (e = e === null ? bf.errors : e.concat(bf.errors), r = e.length);
        var d = y === r;
      }
      if (l.interval !== void 0) {
        let u = l.interval, y = r;
        _f(u, { instancePath: o + "/interval", parentData: l, parentDataProperty: "interval", rootData: s }) || (e = e === null ? _f.errors : e.concat(_f.errors), r = e.length);
        var d = y === r;
      }
      if (l.aniopts !== void 0) {
        let u = l.aniopts, y = r;
        vf(u, { instancePath: o + "/aniopts", parentData: l, parentDataProperty: "aniopts", rootData: s }) || (e = e === null ? vf.errors : e.concat(vf.errors), r = e.length);
        var d = y === r;
      }
      if (l["animation-hook"] !== void 0) {
        let u = l["animation-hook"], y = r;
        qf(u, { instancePath: o + "/animation-hook", parentData: l, parentDataProperty: "animation-hook", rootData: s }) || (e = e === null ? qf.errors : e.concat(qf.errors), r = e.length);
        var d = y === r;
      }
      if (l.child !== void 0) {
        let u = l.child, y = r;
        Of(u, { instancePath: o + "/child", parentData: l, parentDataProperty: "child", rootData: s }) || (e = e === null ? Of.errors : e.concat(Of.errors), r = e.length);
        var d = y === r;
      }
      if (l.file !== void 0) {
        let u = l.file, y = r;
        Pf(u, { instancePath: o + "/file", parentData: l, parentDataProperty: "file", rootData: s }) || (e = e === null ? Pf.errors : e.concat(Pf.errors), r = e.length);
        var d = y === r;
      }
      if (l.code !== void 0) {
        let u = l.code, y = r;
        xf(u, { instancePath: o + "/code", parentData: l, parentDataProperty: "code", rootData: s }) || (e = e === null ? xf.errors : e.concat(xf.errors), r = e.length);
        var d = y === r;
      }
      if (l.purl !== void 0) {
        let u = l.purl, y = r;
        kf(u, { instancePath: o + "/purl", parentData: l, parentDataProperty: "purl", rootData: s }) || (e = e === null ? kf.errors : e.concat(kf.errors), r = e.length);
        var d = y === r;
      }
      if (l.layout !== void 0) {
        let u = l.layout, y = r;
        vl(u, { instancePath: o + "/layout", parentData: l, parentDataProperty: "layout", rootData: s }) || (e = e === null ? vl.errors : e.concat(vl.errors), r = e.length);
        var d = y === r;
      }
      if (l["layout-ncol"] !== void 0) {
        let u = l["layout-ncol"], y = r;
        ql(u, { instancePath: o + "/layout-ncol", parentData: l, parentDataProperty: "layout-ncol", rootData: s }) || (e = e === null ? ql.errors : e.concat(ql.errors), r = e.length);
        var d = y === r;
      }
      if (l["layout-nrow"] !== void 0) {
        let u = l["layout-nrow"], y = r;
        Ol(u, { instancePath: o + "/layout-nrow", parentData: l, parentDataProperty: "layout-nrow", rootData: s }) || (e = e === null ? Ol.errors : e.concat(Ol.errors), r = e.length);
        var d = y === r;
      }
      if (l["layout-align"] !== void 0) {
        let u = l["layout-align"], y = r;
        Pl(u, { instancePath: o + "/layout-align", parentData: l, parentDataProperty: "layout-align", rootData: s }) || (e = e === null ? Pl.errors : e.concat(Pl.errors), r = e.length);
        var d = y === r;
      }
      if (l["layout-valign"] !== void 0) {
        let u = l["layout-valign"], y = r;
        xl(u, { instancePath: o + "/layout-valign", parentData: l, parentDataProperty: "layout-valign", rootData: s }) || (e = e === null ? xl.errors : e.concat(xl.errors), r = e.length);
        var d = y === r;
      }
      if (l.column !== void 0) {
        let u = l.column, y = r;
        kl(u, { instancePath: o + "/column", parentData: l, parentDataProperty: "column", rootData: s }) || (e = e === null ? kl.errors : e.concat(kl.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-column"] !== void 0) {
        let u = l["fig-column"], y = r;
        Dl(u, { instancePath: o + "/fig-column", parentData: l, parentDataProperty: "fig-column", rootData: s }) || (e = e === null ? Dl.errors : e.concat(Dl.errors), r = e.length);
        var d = y === r;
      }
      if (l["tbl-column"] !== void 0) {
        let u = l["tbl-column"], y = r;
        wl(u, { instancePath: o + "/tbl-column", parentData: l, parentDataProperty: "tbl-column", rootData: s }) || (e = e === null ? wl.errors : e.concat(wl.errors), r = e.length);
        var d = y === r;
      }
      if (l["cap-location"] !== void 0) {
        let u = l["cap-location"], y = r;
        ja(u, { instancePath: o + "/cap-location", parentData: l, parentDataProperty: "cap-location", rootData: s }) || (e = e === null ? ja.errors : e.concat(ja.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-cap-location"] !== void 0) {
        let u = l["fig-cap-location"], y = r;
        Ea(u, { instancePath: o + "/fig-cap-location", parentData: l, parentDataProperty: "fig-cap-location", rootData: s }) || (e = e === null ? Ea.errors : e.concat(Ea.errors), r = e.length);
        var d = y === r;
      }
      if (l["tbl-cap-location"] !== void 0) {
        let u = l["tbl-cap-location"], y = r;
        Na(u, { instancePath: o + "/tbl-cap-location", parentData: l, parentDataProperty: "tbl-cap-location", rootData: s }) || (e = e === null ? Na.errors : e.concat(Na.errors), r = e.length);
        var d = y === r;
      }
      if (l["tbl-cap"] !== void 0) {
        let u = l["tbl-cap"], y = r;
        $l(u, { instancePath: o + "/tbl-cap", parentData: l, parentDataProperty: "tbl-cap", rootData: s }) || (e = e === null ? $l.errors : e.concat($l.errors), r = e.length);
        var d = y === r;
      }
      if (l["tbl-subcap"] !== void 0) {
        let u = l["tbl-subcap"], y = r;
        Sl(u, { instancePath: o + "/tbl-subcap", parentData: l, parentDataProperty: "tbl-subcap", rootData: s }) || (e = e === null ? Sl.errors : e.concat(Sl.errors), r = e.length);
        var d = y === r;
      }
      if (l["tbl-colwidths"] !== void 0) {
        let u = l["tbl-colwidths"], y = r;
        be(u, { instancePath: o + "/tbl-colwidths", parentData: l, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
        var d = y === r;
      }
      if (l.output !== void 0) {
        let u = l.output, y = r;
        fe(u, { instancePath: o + "/output", parentData: l, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
        var d = y === r;
      }
      if (l.warning !== void 0) {
        let u = l.warning, y = r;
        me(u, { instancePath: o + "/warning", parentData: l, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
        var d = y === r;
      }
      if (l.error !== void 0) {
        let u = l.error, y = r;
        ge(u, { instancePath: o + "/error", parentData: l, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
        var d = y === r;
      }
      if (l.include !== void 0) {
        let u = l.include, y = r;
        he(u, { instancePath: o + "/include", parentData: l, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
        var d = y === r;
      }
      if (l.panel !== void 0) {
        let u = l.panel, y = r;
        Fl(u, { instancePath: o + "/panel", parentData: l, parentDataProperty: "panel", rootData: s }) || (e = e === null ? Fl.errors : e.concat(Fl.errors), r = e.length);
        var d = y === r;
      }
      if (l.message !== void 0) {
        let u = l.message, y = r;
        Bf(u, { instancePath: o + "/message", parentData: l, parentDataProperty: "message", rootData: s }) || (e = e === null ? Bf.errors : e.concat(Bf.errors), r = e.length);
        var d = y === r;
      }
      if (l.results !== void 0) {
        let u = l.results, y = r;
        jf(u, { instancePath: o + "/results", parentData: l, parentDataProperty: "results", rootData: s }) || (e = e === null ? jf.errors : e.concat(jf.errors), r = e.length);
        var d = y === r;
      }
      if (l.comment !== void 0) {
        let u = l.comment, y = r;
        Ef(u, { instancePath: o + "/comment", parentData: l, parentDataProperty: "comment", rootData: s }) || (e = e === null ? Ef.errors : e.concat(Ef.errors), r = e.length);
        var d = y === r;
      }
      if (l["class-output"] !== void 0) {
        let u = l["class-output"], y = r;
        Nf(u, { instancePath: o + "/class-output", parentData: l, parentDataProperty: "class-output", rootData: s }) || (e = e === null ? Nf.errors : e.concat(Nf.errors), r = e.length);
        var d = y === r;
      }
      if (l["attr-output"] !== void 0) {
        let u = l["attr-output"], y = r;
        Cf(u, { instancePath: o + "/attr-output", parentData: l, parentDataProperty: "attr-output", rootData: s }) || (e = e === null ? Cf.errors : e.concat(Cf.errors), r = e.length);
        var d = y === r;
      }
      if (l["class-warning"] !== void 0) {
        let u = l["class-warning"], y = r;
        If(u, { instancePath: o + "/class-warning", parentData: l, parentDataProperty: "class-warning", rootData: s }) || (e = e === null ? If.errors : e.concat(If.errors), r = e.length);
        var d = y === r;
      }
      if (l["attr-warning"] !== void 0) {
        let u = l["attr-warning"], y = r;
        Hf(u, { instancePath: o + "/attr-warning", parentData: l, parentDataProperty: "attr-warning", rootData: s }) || (e = e === null ? Hf.errors : e.concat(Hf.errors), r = e.length);
        var d = y === r;
      }
      if (l["class-message"] !== void 0) {
        let u = l["class-message"], y = r;
        Lf(u, { instancePath: o + "/class-message", parentData: l, parentDataProperty: "class-message", rootData: s }) || (e = e === null ? Lf.errors : e.concat(Lf.errors), r = e.length);
        var d = y === r;
      }
      if (l["attr-message"] !== void 0) {
        let u = l["attr-message"], y = r;
        Mf(u, { instancePath: o + "/attr-message", parentData: l, parentDataProperty: "attr-message", rootData: s }) || (e = e === null ? Mf.errors : e.concat(Mf.errors), r = e.length);
        var d = y === r;
      }
      if (l["class-error"] !== void 0) {
        let u = l["class-error"], y = r;
        Tf(u, { instancePath: o + "/class-error", parentData: l, parentDataProperty: "class-error", rootData: s }) || (e = e === null ? Tf.errors : e.concat(Tf.errors), r = e.length);
        var d = y === r;
      }
      if (l["attr-error"] !== void 0) {
        let u = l["attr-error"], y = r;
        zf(u, { instancePath: o + "/attr-error", parentData: l, parentDataProperty: "attr-error", rootData: s }) || (e = e === null ? zf.errors : e.concat(zf.errors), r = e.length);
        var d = y === r;
      }
    } else {
      let u = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: Gl }, message: "must be object", schema: Gl.type, parentSchema: Gl, data: l };
      e === null ? e = [u] : e.push(u), r++;
    }
    return L1.errors = e, r === 0;
  }
  B["engine-jupyter"] = M1;
  var Yl = { type: "object", description: "be an object", properties: { label: { $ref: "quarto-resource-cell-attributes-label", description: "quarto-resource-cell-attributes-label" }, classes: { $ref: "quarto-resource-cell-attributes-classes", description: "quarto-resource-cell-attributes-classes" }, tags: { $ref: "quarto-resource-cell-attributes-tags", description: "quarto-resource-cell-attributes-tags" }, id: { $ref: "quarto-resource-cell-attributes-id", description: "quarto-resource-cell-attributes-id" }, eval: { $ref: "quarto-resource-cell-codeoutput-eval", description: "quarto-resource-cell-codeoutput-eval" }, echo: { $ref: "quarto-resource-cell-codeoutput-echo", description: "quarto-resource-cell-codeoutput-echo" }, "code-fold": { $ref: "quarto-resource-cell-codeoutput-code-fold", description: "quarto-resource-cell-codeoutput-code-fold" }, "code-summary": { $ref: "quarto-resource-cell-codeoutput-code-summary", description: "quarto-resource-cell-codeoutput-code-summary" }, "code-overflow": { $ref: "quarto-resource-cell-codeoutput-code-overflow", description: "quarto-resource-cell-codeoutput-code-overflow" }, "code-line-numbers": { $ref: "quarto-resource-cell-codeoutput-code-line-numbers", description: "quarto-resource-cell-codeoutput-code-line-numbers" }, "lst-label": { $ref: "quarto-resource-cell-codeoutput-lst-label", description: "quarto-resource-cell-codeoutput-lst-label" }, "lst-cap": { $ref: "quarto-resource-cell-codeoutput-lst-cap", description: "quarto-resource-cell-codeoutput-lst-cap" }, "fig-cap": { $ref: "quarto-resource-cell-figure-fig-cap", description: "quarto-resource-cell-figure-fig-cap" }, "fig-subcap": { $ref: "quarto-resource-cell-figure-fig-subcap", description: "quarto-resource-cell-figure-fig-subcap" }, "fig-link": { $ref: "quarto-resource-cell-figure-fig-link", description: "quarto-resource-cell-figure-fig-link" }, "fig-align": { $ref: "quarto-resource-cell-figure-fig-align", description: "quarto-resource-cell-figure-fig-align" }, "fig-alt": { $ref: "quarto-resource-cell-figure-fig-alt", description: "quarto-resource-cell-figure-fig-alt" }, "fig-env": { $ref: "quarto-resource-cell-figure-fig-env", description: "quarto-resource-cell-figure-fig-env" }, "fig-pos": { $ref: "quarto-resource-cell-figure-fig-pos", description: "quarto-resource-cell-figure-fig-pos" }, "fig-scap": { $ref: "quarto-resource-cell-figure-fig-scap", description: "quarto-resource-cell-figure-fig-scap" }, layout: { $ref: "quarto-resource-cell-layout-layout", description: "quarto-resource-cell-layout-layout" }, "layout-ncol": { $ref: "quarto-resource-cell-layout-layout-ncol", description: "quarto-resource-cell-layout-layout-ncol" }, "layout-nrow": { $ref: "quarto-resource-cell-layout-layout-nrow", description: "quarto-resource-cell-layout-layout-nrow" }, "layout-align": { $ref: "quarto-resource-cell-layout-layout-align", description: "quarto-resource-cell-layout-layout-align" }, "layout-valign": { $ref: "quarto-resource-cell-layout-layout-valign", description: "quarto-resource-cell-layout-layout-valign" }, column: { $ref: "quarto-resource-cell-pagelayout-column", description: "quarto-resource-cell-pagelayout-column" }, "fig-column": { $ref: "quarto-resource-cell-pagelayout-fig-column", description: "quarto-resource-cell-pagelayout-fig-column" }, "tbl-column": { $ref: "quarto-resource-cell-pagelayout-tbl-column", description: "quarto-resource-cell-pagelayout-tbl-column" }, "cap-location": { $ref: "quarto-resource-cell-pagelayout-cap-location", description: "quarto-resource-cell-pagelayout-cap-location" }, "fig-cap-location": { $ref: "quarto-resource-cell-pagelayout-fig-cap-location", description: "quarto-resource-cell-pagelayout-fig-cap-location" }, "tbl-cap-location": { $ref: "quarto-resource-cell-pagelayout-tbl-cap-location", description: "quarto-resource-cell-pagelayout-tbl-cap-location" }, "tbl-cap": { $ref: "quarto-resource-cell-table-tbl-cap", description: "quarto-resource-cell-table-tbl-cap" }, "tbl-subcap": { $ref: "quarto-resource-cell-table-tbl-subcap", description: "quarto-resource-cell-table-tbl-subcap" }, "tbl-colwidths": { $ref: "quarto-resource-cell-table-tbl-colwidths", description: "quarto-resource-cell-table-tbl-colwidths" }, output: { $ref: "quarto-resource-cell-textoutput-output", description: "quarto-resource-cell-textoutput-output" }, warning: { $ref: "quarto-resource-cell-textoutput-warning", description: "quarto-resource-cell-textoutput-warning" }, error: { $ref: "quarto-resource-cell-textoutput-error", description: "quarto-resource-cell-textoutput-error" }, include: { $ref: "quarto-resource-cell-textoutput-include", description: "quarto-resource-cell-textoutput-include" }, panel: { $ref: "quarto-resource-cell-textoutput-panel", description: "quarto-resource-cell-textoutput-panel" } }, patternProperties: {}, propertyNames: { type: "string", pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|lst_label|lstLabel|lst_cap|lstCap|fig_cap|figCap|fig_subcap|figSubcap|fig_link|figLink|fig_align|figAlign|fig_alt|figAlt|fig_env|figEnv|fig_pos|figPos|fig_scap|figScap|layout_ncol|layoutNcol|layout_nrow|layoutNrow|layout_align|layoutAlign|layout_valign|layoutValign|fig_column|figColumn|tbl_column|tblColumn|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_cap|tblCap|tbl_subcap|tblSubcap|tbl_colwidths|tblColwidths))" }, $id: "engine-jupyter" }, lq = new RegExp("^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|lst_label|lstLabel|lst_cap|lstCap|fig_cap|figCap|fig_subcap|figSubcap|fig_link|figLink|fig_align|figAlign|fig_alt|figAlt|fig_env|figEnv|fig_pos|figPos|fig_scap|figScap|layout_ncol|layoutNcol|layout_nrow|layoutNrow|layout_align|layoutAlign|layout_valign|layoutValign|fig_column|figColumn|tbl_column|tblColumn|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_cap|tblCap|tbl_subcap|tblSubcap|tbl_colwidths|tblColwidths))", "u");
  function M1(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      for (let u in l) {
        let y = r, g = r;
        if (typeof u == "string") {
          if (!lq.test(u)) {
            let x = { instancePath: o, schemaPath: "#/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|lst_label|lstLabel|lst_cap|lstCap|fig_cap|figCap|fig_subcap|figSubcap|fig_link|figLink|fig_align|figAlign|fig_alt|figAlt|fig_env|figEnv|fig_pos|figPos|fig_scap|figScap|layout_ncol|layoutNcol|layout_nrow|layoutNrow|layout_align|layoutAlign|layout_valign|layoutValign|fig_column|figColumn|tbl_column|tblColumn|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_cap|tblCap|tbl_subcap|tblSubcap|tbl_colwidths|tblColwidths))", schema: Yl.propertyNames }, message: 'must match pattern "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|lst_label|lstLabel|lst_cap|lstCap|fig_cap|figCap|fig_subcap|figSubcap|fig_link|figLink|fig_align|figAlign|fig_alt|figAlt|fig_env|figEnv|fig_pos|figPos|fig_scap|figScap|layout_ncol|layoutNcol|layout_nrow|layoutNrow|layout_align|layoutAlign|layout_valign|layoutValign|fig_column|figColumn|tbl_column|tblColumn|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_cap|tblCap|tbl_subcap|tblSubcap|tbl_colwidths|tblColwidths))"', schema: "^(?!(code_fold|codeFold|code_summary|codeSummary|code_overflow|codeOverflow|code_line_numbers|codeLineNumbers|lst_label|lstLabel|lst_cap|lstCap|fig_cap|figCap|fig_subcap|figSubcap|fig_link|figLink|fig_align|figAlign|fig_alt|figAlt|fig_env|figEnv|fig_pos|figPos|fig_scap|figScap|layout_ncol|layoutNcol|layout_nrow|layoutNrow|layout_align|layoutAlign|layout_valign|layoutValign|fig_column|figColumn|tbl_column|tblColumn|cap_location|capLocation|fig_cap_location|figCapLocation|tbl_cap_location|tblCapLocation|tbl_cap|tblCap|tbl_subcap|tblSubcap|tbl_colwidths|tblColwidths))", parentSchema: Yl.propertyNames, data: u, propertyName: u };
            e === null ? e = [x] : e.push(x), r++;
          }
        } else {
          let x = { instancePath: o, schemaPath: "#/propertyNames/type", keyword: "type", params: { type: "string", schema: Yl.propertyNames }, message: "must be string", schema: Yl.propertyNames.type, parentSchema: Yl.propertyNames, data: u, propertyName: u };
          e === null ? e = [x] : e.push(x), r++;
        }
        var c = y === r;
        if (!c) {
          let x = { instancePath: o, schemaPath: "#/propertyNames", keyword: "propertyNames", params: { propertyName: u, schema: Yl }, message: "property name must be valid", schema: Yl.propertyNames, parentSchema: Yl, data: l };
          e === null ? e = [x] : e.push(x), r++;
        }
      }
      if (l.label !== void 0) {
        let u = l.label, y = r;
        pl(u, { instancePath: o + "/label", parentData: l, parentDataProperty: "label", rootData: s }) || (e = e === null ? pl.errors : e.concat(pl.errors), r = e.length);
        var d = y === r;
      }
      if (l.classes !== void 0) {
        let u = l.classes, y = r;
        ul(u, { instancePath: o + "/classes", parentData: l, parentDataProperty: "classes", rootData: s }) || (e = e === null ? ul.errors : e.concat(ul.errors), r = e.length);
        var d = y === r;
      }
      if (l.tags !== void 0) {
        let u = l.tags, y = r;
        Md(u, { instancePath: o + "/tags", parentData: l, parentDataProperty: "tags", rootData: s }) || (e = e === null ? Md.errors : e.concat(Md.errors), r = e.length);
        var d = y === r;
      }
      if (l.id !== void 0) {
        let u = l.id, y = r;
        Td(u, { instancePath: o + "/id", parentData: l, parentDataProperty: "id", rootData: s }) || (e = e === null ? Td.errors : e.concat(Td.errors), r = e.length);
        var d = y === r;
      }
      if (l.eval !== void 0) {
        let u = l.eval, y = r;
        ue(u, { instancePath: o + "/eval", parentData: l, parentDataProperty: "eval", rootData: s }) || (e = e === null ? ue.errors : e.concat(ue.errors), r = e.length);
        var d = y === r;
      }
      if (l.echo !== void 0) {
        let u = l.echo, y = r;
        de(u, { instancePath: o + "/echo", parentData: l, parentDataProperty: "echo", rootData: s }) || (e = e === null ? de.errors : e.concat(de.errors), r = e.length);
        var d = y === r;
      }
      if (l["code-fold"] !== void 0) {
        let u = l["code-fold"], y = r;
        Fa(u, { instancePath: o + "/code-fold", parentData: l, parentDataProperty: "code-fold", rootData: s }) || (e = e === null ? Fa.errors : e.concat(Fa.errors), r = e.length);
        var d = y === r;
      }
      if (l["code-summary"] !== void 0) {
        let u = l["code-summary"], y = r;
        Aa(u, { instancePath: o + "/code-summary", parentData: l, parentDataProperty: "code-summary", rootData: s }) || (e = e === null ? Aa.errors : e.concat(Aa.errors), r = e.length);
        var d = y === r;
      }
      if (l["code-overflow"] !== void 0) {
        let u = l["code-overflow"], y = r;
        Ba(u, { instancePath: o + "/code-overflow", parentData: l, parentDataProperty: "code-overflow", rootData: s }) || (e = e === null ? Ba.errors : e.concat(Ba.errors), r = e.length);
        var d = y === r;
      }
      if (l["code-line-numbers"] !== void 0) {
        let u = l["code-line-numbers"], y = r;
        ua(u, { instancePath: o + "/code-line-numbers", parentData: l, parentDataProperty: "code-line-numbers", rootData: s }) || (e = e === null ? ua.errors : e.concat(ua.errors), r = e.length);
        var d = y === r;
      }
      if (l["lst-label"] !== void 0) {
        let u = l["lst-label"], y = r;
        fl(u, { instancePath: o + "/lst-label", parentData: l, parentDataProperty: "lst-label", rootData: s }) || (e = e === null ? fl.errors : e.concat(fl.errors), r = e.length);
        var d = y === r;
      }
      if (l["lst-cap"] !== void 0) {
        let u = l["lst-cap"], y = r;
        ml(u, { instancePath: o + "/lst-cap", parentData: l, parentDataProperty: "lst-cap", rootData: s }) || (e = e === null ? ml.errors : e.concat(ml.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-cap"] !== void 0) {
        let u = l["fig-cap"], y = r;
        gl(u, { instancePath: o + "/fig-cap", parentData: l, parentDataProperty: "fig-cap", rootData: s }) || (e = e === null ? gl.errors : e.concat(gl.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-subcap"] !== void 0) {
        let u = l["fig-subcap"], y = r;
        hl(u, { instancePath: o + "/fig-subcap", parentData: l, parentDataProperty: "fig-subcap", rootData: s }) || (e = e === null ? hl.errors : e.concat(hl.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-link"] !== void 0) {
        let u = l["fig-link"], y = r;
        yl(u, { instancePath: o + "/fig-link", parentData: l, parentDataProperty: "fig-link", rootData: s }) || (e = e === null ? yl.errors : e.concat(yl.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-align"] !== void 0) {
        let u = l["fig-align"], y = r;
        la(u, { instancePath: o + "/fig-align", parentData: l, parentDataProperty: "fig-align", rootData: s }) || (e = e === null ? la.errors : e.concat(la.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-alt"] !== void 0) {
        let u = l["fig-alt"], y = r;
        bl(u, { instancePath: o + "/fig-alt", parentData: l, parentDataProperty: "fig-alt", rootData: s }) || (e = e === null ? bl.errors : e.concat(bl.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-env"] !== void 0) {
        let u = l["fig-env"], y = r;
        Zn(u, { instancePath: o + "/fig-env", parentData: l, parentDataProperty: "fig-env", rootData: s }) || (e = e === null ? Zn.errors : e.concat(Zn.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-pos"] !== void 0) {
        let u = l["fig-pos"], y = r;
        eo(u, { instancePath: o + "/fig-pos", parentData: l, parentDataProperty: "fig-pos", rootData: s }) || (e = e === null ? eo.errors : e.concat(eo.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-scap"] !== void 0) {
        let u = l["fig-scap"], y = r;
        _l(u, { instancePath: o + "/fig-scap", parentData: l, parentDataProperty: "fig-scap", rootData: s }) || (e = e === null ? _l.errors : e.concat(_l.errors), r = e.length);
        var d = y === r;
      }
      if (l.layout !== void 0) {
        let u = l.layout, y = r;
        vl(u, { instancePath: o + "/layout", parentData: l, parentDataProperty: "layout", rootData: s }) || (e = e === null ? vl.errors : e.concat(vl.errors), r = e.length);
        var d = y === r;
      }
      if (l["layout-ncol"] !== void 0) {
        let u = l["layout-ncol"], y = r;
        ql(u, { instancePath: o + "/layout-ncol", parentData: l, parentDataProperty: "layout-ncol", rootData: s }) || (e = e === null ? ql.errors : e.concat(ql.errors), r = e.length);
        var d = y === r;
      }
      if (l["layout-nrow"] !== void 0) {
        let u = l["layout-nrow"], y = r;
        Ol(u, { instancePath: o + "/layout-nrow", parentData: l, parentDataProperty: "layout-nrow", rootData: s }) || (e = e === null ? Ol.errors : e.concat(Ol.errors), r = e.length);
        var d = y === r;
      }
      if (l["layout-align"] !== void 0) {
        let u = l["layout-align"], y = r;
        Pl(u, { instancePath: o + "/layout-align", parentData: l, parentDataProperty: "layout-align", rootData: s }) || (e = e === null ? Pl.errors : e.concat(Pl.errors), r = e.length);
        var d = y === r;
      }
      if (l["layout-valign"] !== void 0) {
        let u = l["layout-valign"], y = r;
        xl(u, { instancePath: o + "/layout-valign", parentData: l, parentDataProperty: "layout-valign", rootData: s }) || (e = e === null ? xl.errors : e.concat(xl.errors), r = e.length);
        var d = y === r;
      }
      if (l.column !== void 0) {
        let u = l.column, y = r;
        kl(u, { instancePath: o + "/column", parentData: l, parentDataProperty: "column", rootData: s }) || (e = e === null ? kl.errors : e.concat(kl.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-column"] !== void 0) {
        let u = l["fig-column"], y = r;
        Dl(u, { instancePath: o + "/fig-column", parentData: l, parentDataProperty: "fig-column", rootData: s }) || (e = e === null ? Dl.errors : e.concat(Dl.errors), r = e.length);
        var d = y === r;
      }
      if (l["tbl-column"] !== void 0) {
        let u = l["tbl-column"], y = r;
        wl(u, { instancePath: o + "/tbl-column", parentData: l, parentDataProperty: "tbl-column", rootData: s }) || (e = e === null ? wl.errors : e.concat(wl.errors), r = e.length);
        var d = y === r;
      }
      if (l["cap-location"] !== void 0) {
        let u = l["cap-location"], y = r;
        ja(u, { instancePath: o + "/cap-location", parentData: l, parentDataProperty: "cap-location", rootData: s }) || (e = e === null ? ja.errors : e.concat(ja.errors), r = e.length);
        var d = y === r;
      }
      if (l["fig-cap-location"] !== void 0) {
        let u = l["fig-cap-location"], y = r;
        Ea(u, { instancePath: o + "/fig-cap-location", parentData: l, parentDataProperty: "fig-cap-location", rootData: s }) || (e = e === null ? Ea.errors : e.concat(Ea.errors), r = e.length);
        var d = y === r;
      }
      if (l["tbl-cap-location"] !== void 0) {
        let u = l["tbl-cap-location"], y = r;
        Na(u, { instancePath: o + "/tbl-cap-location", parentData: l, parentDataProperty: "tbl-cap-location", rootData: s }) || (e = e === null ? Na.errors : e.concat(Na.errors), r = e.length);
        var d = y === r;
      }
      if (l["tbl-cap"] !== void 0) {
        let u = l["tbl-cap"], y = r;
        $l(u, { instancePath: o + "/tbl-cap", parentData: l, parentDataProperty: "tbl-cap", rootData: s }) || (e = e === null ? $l.errors : e.concat($l.errors), r = e.length);
        var d = y === r;
      }
      if (l["tbl-subcap"] !== void 0) {
        let u = l["tbl-subcap"], y = r;
        Sl(u, { instancePath: o + "/tbl-subcap", parentData: l, parentDataProperty: "tbl-subcap", rootData: s }) || (e = e === null ? Sl.errors : e.concat(Sl.errors), r = e.length);
        var d = y === r;
      }
      if (l["tbl-colwidths"] !== void 0) {
        let u = l["tbl-colwidths"], y = r;
        be(u, { instancePath: o + "/tbl-colwidths", parentData: l, parentDataProperty: "tbl-colwidths", rootData: s }) || (e = e === null ? be.errors : e.concat(be.errors), r = e.length);
        var d = y === r;
      }
      if (l.output !== void 0) {
        let u = l.output, y = r;
        fe(u, { instancePath: o + "/output", parentData: l, parentDataProperty: "output", rootData: s }) || (e = e === null ? fe.errors : e.concat(fe.errors), r = e.length);
        var d = y === r;
      }
      if (l.warning !== void 0) {
        let u = l.warning, y = r;
        me(u, { instancePath: o + "/warning", parentData: l, parentDataProperty: "warning", rootData: s }) || (e = e === null ? me.errors : e.concat(me.errors), r = e.length);
        var d = y === r;
      }
      if (l.error !== void 0) {
        let u = l.error, y = r;
        ge(u, { instancePath: o + "/error", parentData: l, parentDataProperty: "error", rootData: s }) || (e = e === null ? ge.errors : e.concat(ge.errors), r = e.length);
        var d = y === r;
      }
      if (l.include !== void 0) {
        let u = l.include, y = r;
        he(u, { instancePath: o + "/include", parentData: l, parentDataProperty: "include", rootData: s }) || (e = e === null ? he.errors : e.concat(he.errors), r = e.length);
        var d = y === r;
      }
      if (l.panel !== void 0) {
        let u = l.panel, y = r;
        Fl(u, { instancePath: o + "/panel", parentData: l, parentDataProperty: "panel", rootData: s }) || (e = e === null ? Fl.errors : e.concat(Fl.errors), r = e.length);
        var d = y === r;
      }
    } else {
      let u = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: Yl }, message: "must be object", schema: Yl.type, parentSchema: Yl, data: l };
      e === null ? e = [u] : e.push(u), r++;
    }
    return M1.errors = e, r === 0;
  }
  B["plugin-reveal"] = T1;
  var aa = { type: "object", description: "be an object", properties: { path: { type: "string", description: "be a string" }, name: { type: "string", description: "be a string" }, register: { type: "boolean", description: "be `true` or `false`" }, script: { oneOf: [{ oneOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: { path: { type: "string", description: "be a string" }, async: { type: "boolean", description: "be `true` or `false`" } }, patternProperties: {}, required: ["path"] }], description: "be exactly one of: a string, an object" }, { type: "array", description: "be an array of values, where each element should be exactly one of: a string, an object", items: { oneOf: [{ type: "string", description: "be a string" }, { type: "object", description: "be an object", properties: { path: { type: "string", description: "be a string" }, async: { type: "boolean", description: "be `true` or `false`" } }, patternProperties: {}, required: ["path"] }], description: "be exactly one of: a string, an object" } }], description: "be exactly one of: exactly one of: a string, an object, an array of values, where each element should be exactly one of: a string, an object" }, stylesheet: { oneOf: [{ type: "string", description: "be a string" }, { type: "array", description: "be an array of values, where each element should be a string", items: { type: "string", description: "be a string" } }], description: "be exactly one of: a string, an array of values, where each element should be a string" }, "self-contained": { type: "boolean", description: "be `true` or `false`" } }, patternProperties: {}, required: ["name"], propertyNames: { type: "string", pattern: "^(?!(self_contained|selfContained))" }, $id: "plugin-reveal" }, cq = new RegExp("^(?!(self_contained|selfContained))", "u");
  function T1(l, { instancePath: o = "", parentData: S, parentDataProperty: F, rootData: s = l } = {}) {
    let e = null, r = 0, _ = r;
    if (l && typeof l == "object" && !Array.isArray(l)) {
      if (l.name === void 0) {
        let j = { instancePath: o, schemaPath: "#/required", keyword: "required", params: { missingProperty: "name", schema: aa }, message: "must have required property 'name'", schema: aa.required, parentSchema: aa, data: l };
        e === null ? e = [j] : e.push(j), r++;
      }
      for (let j in l) {
        let V = r, se = r;
        if (typeof j == "string") {
          if (!cq.test(j)) {
            let P = { instancePath: o, schemaPath: "#/propertyNames/pattern", keyword: "pattern", params: { pattern: "^(?!(self_contained|selfContained))", schema: aa.propertyNames }, message: 'must match pattern "^(?!(self_contained|selfContained))"', schema: "^(?!(self_contained|selfContained))", parentSchema: aa.propertyNames, data: j, propertyName: j };
            e === null ? e = [P] : e.push(P), r++;
          }
        } else {
          let P = { instancePath: o, schemaPath: "#/propertyNames/type", keyword: "type", params: { type: "string", schema: aa.propertyNames }, message: "must be string", schema: aa.propertyNames.type, parentSchema: aa.propertyNames, data: j, propertyName: j };
          e === null ? e = [P] : e.push(P), r++;
        }
        var c = V === r;
        if (!c) {
          let P = { instancePath: o, schemaPath: "#/propertyNames", keyword: "propertyNames", params: { propertyName: j, schema: aa }, message: "property name must be valid", schema: aa.propertyNames, parentSchema: aa, data: l };
          e === null ? e = [P] : e.push(P), r++;
        }
      }
      if (l.path !== void 0) {
        let j = l.path, V = r;
        if (typeof j != "string") {
          let P = { instancePath: o + "/path", schemaPath: "#/properties/path/type", keyword: "type", params: { type: "string", schema: aa.properties.path }, message: "must be string", schema: aa.properties.path.type, parentSchema: aa.properties.path, data: j };
          e === null ? e = [P] : e.push(P), r++;
        }
        let se = r;
        var d = V === r;
      }
      if (l.name !== void 0) {
        let j = l.name, V = r;
        if (typeof j != "string") {
          let oe = { instancePath: o + "/name", schemaPath: "#/properties/name/type", keyword: "type", params: { type: "string", schema: aa.properties.name }, message: "must be string", schema: aa.properties.name.type, parentSchema: aa.properties.name, data: j };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let se = r;
        var d = V === r;
      }
      if (l.register !== void 0) {
        let j = l.register, V = r;
        if (typeof j != "boolean") {
          let oe = { instancePath: o + "/register", schemaPath: "#/properties/register/type", keyword: "type", params: { type: "boolean", schema: aa.properties.register }, message: "must be boolean", schema: aa.properties.register.type, parentSchema: aa.properties.register, data: j };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let se = r;
        var d = V === r;
      }
      if (l.script !== void 0) {
        let j = l.script, V = r, se = r, P = false, oe = null, h = r, f = r, ft = false, Cr = null, Dr = r;
        if (typeof j != "string") {
          let _e = { instancePath: o + "/script", schemaPath: "#/properties/script/oneOf/0/oneOf/0/type", keyword: "type", params: { type: "string", schema: aa.properties.script.oneOf[0].oneOf[0] }, message: "must be string", schema: aa.properties.script.oneOf[0].oneOf[0].type, parentSchema: aa.properties.script.oneOf[0].oneOf[0], data: j };
          e === null ? e = [_e] : e.push(_e), r++;
        }
        let wr = r;
        var u = Dr === r;
        u && (ft = true, Cr = 0);
        let ta = r, jt = r;
        if (j && typeof j == "object" && !Array.isArray(j)) {
          if (j.path === void 0) {
            let _e = { instancePath: o + "/script", schemaPath: "#/properties/script/oneOf/0/oneOf/1/required", keyword: "required", params: { missingProperty: "path", schema: aa.properties.script.oneOf[0].oneOf[1] }, message: "must have required property 'path'", schema: aa.properties.script.oneOf[0].oneOf[1].required, parentSchema: aa.properties.script.oneOf[0].oneOf[1], data: j };
            e === null ? e = [_e] : e.push(_e), r++;
          }
          if (j.path !== void 0) {
            let _e = j.path, Ma = r;
            if (typeof _e != "string") {
              let ie = { instancePath: o + "/script/path", schemaPath: "#/properties/script/oneOf/0/oneOf/1/properties/path/type", keyword: "type", params: { type: "string", schema: aa.properties.script.oneOf[0].oneOf[1].properties.path }, message: "must be string", schema: aa.properties.script.oneOf[0].oneOf[1].properties.path.type, parentSchema: aa.properties.script.oneOf[0].oneOf[1].properties.path, data: _e };
              e === null ? e = [ie] : e.push(ie), r++;
            }
            let fa = r;
            var y = Ma === r;
          }
          if (j.async !== void 0) {
            let _e = j.async, Ma = r;
            if (typeof _e != "boolean") {
              let ka = { instancePath: o + "/script/async", schemaPath: "#/properties/script/oneOf/0/oneOf/1/properties/async/type", keyword: "type", params: { type: "boolean", schema: aa.properties.script.oneOf[0].oneOf[1].properties.async }, message: "must be boolean", schema: aa.properties.script.oneOf[0].oneOf[1].properties.async.type, parentSchema: aa.properties.script.oneOf[0].oneOf[1].properties.async, data: _e };
              e === null ? e = [ka] : e.push(ka), r++;
            }
            let fa = r;
            var y = Ma === r;
          }
        } else {
          let _e = { instancePath: o + "/script", schemaPath: "#/properties/script/oneOf/0/oneOf/1/type", keyword: "type", params: { type: "object", schema: aa.properties.script.oneOf[0].oneOf[1] }, message: "must be object", schema: aa.properties.script.oneOf[0].oneOf[1].type, parentSchema: aa.properties.script.oneOf[0].oneOf[1], data: j };
          e === null ? e = [_e] : e.push(_e), r++;
        }
        var u = ta === r;
        if (u && ft ? (ft = false, Cr = [Cr, 1]) : u && (ft = true, Cr = 1), ft)
          r = f, e !== null && (f ? e.length = f : e = null);
        else {
          let _e = { instancePath: o + "/script", schemaPath: "#/properties/script/oneOf/0/oneOf", keyword: "oneOf", params: { passingSchemas: Cr, schema: aa.properties.script.oneOf[0] }, message: "must match exactly one schema in oneOf", schema: aa.properties.script.oneOf[0].oneOf, parentSchema: aa.properties.script.oneOf[0], data: j };
          e === null ? e = [_e] : e.push(_e), r++;
        }
        var g = h === r;
        g && (P = true, oe = 0);
        let ee = r, za = r;
        if (Array.isArray(j)) {
          let _e = true, Ma = j.length;
          for (let fa = 0; fa < Ma; fa++) {
            let ie = j[fa], ka = r, hn = r, et = false, yt = null, _n = r;
            if (typeof ie != "string") {
              let Gr = { instancePath: o + "/script/" + fa, schemaPath: "#/properties/script/oneOf/1/items/oneOf/0/type", keyword: "type", params: { type: "string", schema: aa.properties.script.oneOf[1].items.oneOf[0] }, message: "must be string", schema: aa.properties.script.oneOf[1].items.oneOf[0].type, parentSchema: aa.properties.script.oneOf[1].items.oneOf[0], data: ie };
              e === null ? e = [Gr] : e.push(Gr), r++;
            }
            let st = r;
            var x = _n === r;
            x && (et = true, yt = 0);
            let Et = r, ca = r;
            if (ie && typeof ie == "object" && !Array.isArray(ie)) {
              if (ie.path === void 0) {
                let Gr = { instancePath: o + "/script/" + fa, schemaPath: "#/properties/script/oneOf/1/items/oneOf/1/required", keyword: "required", params: { missingProperty: "path", schema: aa.properties.script.oneOf[1].items.oneOf[1] }, message: "must have required property 'path'", schema: aa.properties.script.oneOf[1].items.oneOf[1].required, parentSchema: aa.properties.script.oneOf[1].items.oneOf[1], data: ie };
                e === null ? e = [Gr] : e.push(Gr), r++;
              }
              if (ie.path !== void 0) {
                let Gr = ie.path, Tt = r;
                if (typeof Gr != "string") {
                  let vt = { instancePath: o + "/script/" + fa + "/path", schemaPath: "#/properties/script/oneOf/1/items/oneOf/1/properties/path/type", keyword: "type", params: { type: "string", schema: aa.properties.script.oneOf[1].items.oneOf[1].properties.path }, message: "must be string", schema: aa.properties.script.oneOf[1].items.oneOf[1].properties.path.type, parentSchema: aa.properties.script.oneOf[1].items.oneOf[1].properties.path, data: Gr };
                  e === null ? e = [vt] : e.push(vt), r++;
                }
                let bn = r;
                var D = Tt === r;
              }
              if (ie.async !== void 0) {
                let Gr = ie.async, Tt = r;
                if (typeof Gr != "boolean") {
                  let yn = { instancePath: o + "/script/" + fa + "/async", schemaPath: "#/properties/script/oneOf/1/items/oneOf/1/properties/async/type", keyword: "type", params: { type: "boolean", schema: aa.properties.script.oneOf[1].items.oneOf[1].properties.async }, message: "must be boolean", schema: aa.properties.script.oneOf[1].items.oneOf[1].properties.async.type, parentSchema: aa.properties.script.oneOf[1].items.oneOf[1].properties.async, data: Gr };
                  e === null ? e = [yn] : e.push(yn), r++;
                }
                let bn = r;
                var D = Tt === r;
              }
            } else {
              let Gr = { instancePath: o + "/script/" + fa, schemaPath: "#/properties/script/oneOf/1/items/oneOf/1/type", keyword: "type", params: { type: "object", schema: aa.properties.script.oneOf[1].items.oneOf[1] }, message: "must be object", schema: aa.properties.script.oneOf[1].items.oneOf[1].type, parentSchema: aa.properties.script.oneOf[1].items.oneOf[1], data: ie };
              e === null ? e = [Gr] : e.push(Gr), r++;
            }
            var x = Et === r;
            if (x && et ? (et = false, yt = [yt, 1]) : x && (et = true, yt = 1), et)
              r = hn, e !== null && (hn ? e.length = hn : e = null);
            else {
              let Gr = { instancePath: o + "/script/" + fa, schemaPath: "#/properties/script/oneOf/1/items/oneOf", keyword: "oneOf", params: { passingSchemas: yt, schema: aa.properties.script.oneOf[1].items }, message: "must match exactly one schema in oneOf", schema: aa.properties.script.oneOf[1].items.oneOf, parentSchema: aa.properties.script.oneOf[1].items, data: ie };
              e === null ? e = [Gr] : e.push(Gr), r++;
            }
            var $ = ka === r;
            $ || (_e = false);
          }
        } else {
          let _e = { instancePath: o + "/script", schemaPath: "#/properties/script/oneOf/1/type", keyword: "type", params: { type: "array", schema: aa.properties.script.oneOf[1] }, message: "must be array", schema: aa.properties.script.oneOf[1].type, parentSchema: aa.properties.script.oneOf[1], data: j };
          e === null ? e = [_e] : e.push(_e), r++;
        }
        var g = ee === r;
        if (g && P ? (P = false, oe = [oe, 1]) : g && (P = true, oe = 1), P)
          r = se, e !== null && (se ? e.length = se : e = null);
        else {
          let _e = { instancePath: o + "/script", schemaPath: "#/properties/script/oneOf", keyword: "oneOf", params: { passingSchemas: oe, schema: aa.properties.script }, message: "must match exactly one schema in oneOf", schema: aa.properties.script.oneOf, parentSchema: aa.properties.script, data: j };
          e === null ? e = [_e] : e.push(_e), r++;
        }
        var d = V === r;
      }
      if (l.stylesheet !== void 0) {
        let j = l.stylesheet, V = r, se = r, P = false, oe = null, h = r;
        if (typeof j != "string") {
          let wr = { instancePath: o + "/stylesheet", schemaPath: "#/properties/stylesheet/oneOf/0/type", keyword: "type", params: { type: "string", schema: aa.properties.stylesheet.oneOf[0] }, message: "must be string", schema: aa.properties.stylesheet.oneOf[0].type, parentSchema: aa.properties.stylesheet.oneOf[0], data: j };
          e === null ? e = [wr] : e.push(wr), r++;
        }
        let f = r;
        var m = h === r;
        m && (P = true, oe = 0);
        let ft = r, Cr = r;
        if (Array.isArray(j)) {
          let wr = true, ta = j.length;
          for (let jt = 0; jt < ta; jt++) {
            let ee = j[jt], za = r;
            if (typeof ee != "string") {
              let _e = { instancePath: o + "/stylesheet/" + jt, schemaPath: "#/properties/stylesheet/oneOf/1/items/type", keyword: "type", params: { type: "string", schema: aa.properties.stylesheet.oneOf[1].items }, message: "must be string", schema: aa.properties.stylesheet.oneOf[1].items.type, parentSchema: aa.properties.stylesheet.oneOf[1].items, data: ee };
              e === null ? e = [_e] : e.push(_e), r++;
            }
            let sa = r;
            var R = za === r;
            R || (wr = false);
          }
        } else {
          let wr = { instancePath: o + "/stylesheet", schemaPath: "#/properties/stylesheet/oneOf/1/type", keyword: "type", params: { type: "array", schema: aa.properties.stylesheet.oneOf[1] }, message: "must be array", schema: aa.properties.stylesheet.oneOf[1].type, parentSchema: aa.properties.stylesheet.oneOf[1], data: j };
          e === null ? e = [wr] : e.push(wr), r++;
        }
        var m = ft === r;
        if (m && P ? (P = false, oe = [oe, 1]) : m && (P = true, oe = 1), P)
          r = se, e !== null && (se ? e.length = se : e = null);
        else {
          let wr = { instancePath: o + "/stylesheet", schemaPath: "#/properties/stylesheet/oneOf", keyword: "oneOf", params: { passingSchemas: oe, schema: aa.properties.stylesheet }, message: "must match exactly one schema in oneOf", schema: aa.properties.stylesheet.oneOf, parentSchema: aa.properties.stylesheet, data: j };
          e === null ? e = [wr] : e.push(wr), r++;
        }
        var d = V === r;
      }
      if (l["self-contained"] !== void 0) {
        let j = l["self-contained"], V = r;
        if (typeof j != "boolean") {
          let oe = { instancePath: o + "/self-contained", schemaPath: "#/properties/self-contained/type", keyword: "type", params: { type: "boolean", schema: aa.properties["self-contained"] }, message: "must be boolean", schema: aa.properties["self-contained"].type, parentSchema: aa.properties["self-contained"], data: j };
          e === null ? e = [oe] : e.push(oe), r++;
        }
        let se = r;
        var d = V === r;
      }
    } else {
      let j = { instancePath: o, schemaPath: "#/type", keyword: "type", params: { type: "object", schema: aa }, message: "must be object", schema: aa.type, parentSchema: aa, data: l };
      e === null ? e = [j] : e.push(j), r++;
    }
    return T1.errors = e, r === 0;
  }
});
var standalone_schema_validators_default = pq();
